/**
 * @license almond 0.3.1 Copyright (c) 2011-2014, The Dojo Foundation All Rights Reserved.
 * Available via the MIT or new BSD license.
 * see: http://github.com/jrburke/almond for details
 */
//Going sloppy to avoid 'use strict' string cost, but strict practices should
//be followed.
/*jslint sloppy: true */
/*global setTimeout: false */

var requirejs, require, define;
(function (undef) {
    var main, req, makeMap, handlers,
        defined = {},
        waiting = {},
        config = {},
        defining = {},
        hasOwn = Object.prototype.hasOwnProperty,
        aps = [].slice,
        jsSuffixRegExp = /\.js$/;

    function hasProp(obj, prop) {
        return hasOwn.call(obj, prop);
    }

    /**
     * Given a relative module name, like ./something, normalize it to
     * a real name that can be mapped to a path.
     * @param {String} name the relative name
     * @param {String} baseName a real name that the name arg is relative
     * to.
     * @returns {String} normalized name
     */
    function normalize(name, baseName) {
        var nameParts, nameSegment, mapValue, foundMap, lastIndex,
            foundI, foundStarMap, starI, i, j, part,
            baseParts = baseName && baseName.split("/"),
            map = config.map,
            starMap = (map && map['*']) || {};

        //Adjust any relative paths.
        if (name && name.charAt(0) === ".") {
            //If have a base name, try to normalize against it,
            //otherwise, assume it is a top-level require that will
            //be relative to baseUrl in the end.
            if (baseName) {
                name = name.split('/');
                lastIndex = name.length - 1;

                // Node .js allowance:
                if (config.nodeIdCompat && jsSuffixRegExp.test(name[lastIndex])) {
                    name[lastIndex] = name[lastIndex].replace(jsSuffixRegExp, '');
                }

                //Lop off the last part of baseParts, so that . matches the
                //"directory" and not name of the baseName's module. For instance,
                //baseName of "one/two/three", maps to "one/two/three.js", but we
                //want the directory, "one/two" for this normalization.
                name = baseParts.slice(0, baseParts.length - 1).concat(name);

                //start trimDots
                for (i = 0; i < name.length; i += 1) {
                    part = name[i];
                    if (part === ".") {
                        name.splice(i, 1);
                        i -= 1;
                    } else if (part === "..") {
                        if (i === 1 && (name[2] === '..' || name[0] === '..')) {
                            //End of the line. Keep at least one non-dot
                            //path segment at the front so it can be mapped
                            //correctly to disk. Otherwise, there is likely
                            //no path mapping for a path starting with '..'.
                            //This can still fail, but catches the most reasonable
                            //uses of ..
                            break;
                        } else if (i > 0) {
                            name.splice(i - 1, 2);
                            i -= 2;
                        }
                    }
                }
                //end trimDots

                name = name.join("/");
            } else if (name.indexOf('./') === 0) {
                // No baseName, so this is ID is resolved relative
                // to baseUrl, pull off the leading dot.
                name = name.substring(2);
            }
        }

        //Apply map config if available.
        if ((baseParts || starMap) && map) {
            nameParts = name.split('/');

            for (i = nameParts.length; i > 0; i -= 1) {
                nameSegment = nameParts.slice(0, i).join("/");

                if (baseParts) {
                    //Find the longest baseName segment match in the config.
                    //So, do joins on the biggest to smallest lengths of baseParts.
                    for (j = baseParts.length; j > 0; j -= 1) {
                        mapValue = map[baseParts.slice(0, j).join('/')];

                        //baseName segment has  config, find if it has one for
                        //this name.
                        if (mapValue) {
                            mapValue = mapValue[nameSegment];
                            if (mapValue) {
                                //Match, update name to the new value.
                                foundMap = mapValue;
                                foundI = i;
                                break;
                            }
                        }
                    }
                }

                if (foundMap) {
                    break;
                }

                //Check for a star map match, but just hold on to it,
                //if there is a shorter segment match later in a matching
                //config, then favor over this star map.
                if (!foundStarMap && starMap && starMap[nameSegment]) {
                    foundStarMap = starMap[nameSegment];
                    starI = i;
                }
            }

            if (!foundMap && foundStarMap) {
                foundMap = foundStarMap;
                foundI = starI;
            }

            if (foundMap) {
                nameParts.splice(0, foundI, foundMap);
                name = nameParts.join('/');
            }
        }

        return name;
    }

    function makeRequire(relName, forceSync) {
        return function () {
            //A version of a require function that passes a moduleName
            //value for items that may need to
            //look up paths relative to the moduleName
            var args = aps.call(arguments, 0);

            //If first arg is not require('string'), and there is only
            //one arg, it is the array form without a callback. Insert
            //a null so that the following concat is correct.
            if (typeof args[0] !== 'string' && args.length === 1) {
                args.push(null);
            }
            return req.apply(undef, args.concat([relName, forceSync]));
        };
    }

    function makeNormalize(relName) {
        return function (name) {
            return normalize(name, relName);
        };
    }

    function makeLoad(depName) {
        return function (value) {
            defined[depName] = value;
        };
    }

    function callDep(name) {
        if (hasProp(waiting, name)) {
            var args = waiting[name];
            delete waiting[name];
            defining[name] = true;
            main.apply(undef, args);
        }

        if (!hasProp(defined, name) && !hasProp(defining, name)) {
            throw new Error('No ' + name);
        }
        return defined[name];
    }

    //Turns a plugin!resource to [plugin, resource]
    //with the plugin being undefined if the name
    //did not have a plugin prefix.
    function splitPrefix(name) {
        var prefix,
            index = name ? name.indexOf('!') : -1;
        if (index > -1) {
            prefix = name.substring(0, index);
            name = name.substring(index + 1, name.length);
        }
        return [prefix, name];
    }

    /**
     * Makes a name map, normalizing the name, and using a plugin
     * for normalization if necessary. Grabs a ref to plugin
     * too, as an optimization.
     */
    makeMap = function (name, relName) {
        var plugin,
            parts = splitPrefix(name),
            prefix = parts[0];

        name = parts[1];

        if (prefix) {
            prefix = normalize(prefix, relName);
            plugin = callDep(prefix);
        }

        //Normalize according
        if (prefix) {
            if (plugin && plugin.normalize) {
                name = plugin.normalize(name, makeNormalize(relName));
            } else {
                name = normalize(name, relName);
            }
        } else {
            name = normalize(name, relName);
            parts = splitPrefix(name);
            prefix = parts[0];
            name = parts[1];
            if (prefix) {
                plugin = callDep(prefix);
            }
        }

        //Using ridiculous property names for space reasons
        return {
            f: prefix ? prefix + '!' + name : name, //fullName
            n: name,
            pr: prefix,
            p: plugin
        };
    };

    function makeConfig(name) {
        return function () {
            return (config && config.config && config.config[name]) || {};
        };
    }

    handlers = {
        require: function (name) {
            return makeRequire(name);
        },
        exports: function (name) {
            var e = defined[name];
            if (typeof e !== 'undefined') {
                return e;
            } else {
                return (defined[name] = {});
            }
        },
        module: function (name) {
            return {
                id: name,
                uri: '',
                exports: defined[name],
                config: makeConfig(name)
            };
        }
    };

    main = function (name, deps, callback, relName) {
        var cjsModule, depName, ret, map, i,
            args = [],
            callbackType = typeof callback,
            usingExports;

        //Use name if no relName
        relName = relName || name;

        //Call the callback to define the module, if necessary.
        if (callbackType === 'undefined' || callbackType === 'function') {
            //Pull out the defined dependencies and pass the ordered
            //values to the callback.
            //Default to [require, exports, module] if no deps
            deps = !deps.length && callback.length ? ['require', 'exports', 'module'] : deps;
            for (i = 0; i < deps.length; i += 1) {
                map = makeMap(deps[i], relName);
                depName = map.f;

                //Fast path CommonJS standard dependencies.
                if (depName === "require") {
                    args[i] = handlers.require(name);
                } else if (depName === "exports") {
                    //CommonJS module spec 1.1
                    args[i] = handlers.exports(name);
                    usingExports = true;
                } else if (depName === "module") {
                    //CommonJS module spec 1.1
                    cjsModule = args[i] = handlers.module(name);
                } else if (hasProp(defined, depName) ||
                           hasProp(waiting, depName) ||
                           hasProp(defining, depName)) {
                    args[i] = callDep(depName);
                } else if (map.p) {
                    map.p.load(map.n, makeRequire(relName, true), makeLoad(depName), {});
                    args[i] = defined[depName];
                } else {
                    throw new Error(name + ' missing ' + depName);
                }
            }

            ret = callback ? callback.apply(defined[name], args) : undefined;

            if (name) {
                //If setting exports via "module" is in play,
                //favor that over return value and exports. After that,
                //favor a non-undefined return value over exports use.
                if (cjsModule && cjsModule.exports !== undef &&
                        cjsModule.exports !== defined[name]) {
                    defined[name] = cjsModule.exports;
                } else if (ret !== undef || !usingExports) {
                    //Use the return value from the function.
                    defined[name] = ret;
                }
            }
        } else if (name) {
            //May just be an object definition for the module. Only
            //worry about defining if have a module name.
            defined[name] = callback;
        }
    };

    requirejs = require = req = function (deps, callback, relName, forceSync, alt) {
        if (typeof deps === "string") {
            if (handlers[deps]) {
                //callback in this case is really relName
                return handlers[deps](callback);
            }
            //Just return the module wanted. In this scenario, the
            //deps arg is the module name, and second arg (if passed)
            //is just the relName.
            //Normalize module name, if it contains . or ..
            return callDep(makeMap(deps, callback).f);
        } else if (!deps.splice) {
            //deps is a config object, not an array.
            config = deps;
            if (config.deps) {
                req(config.deps, config.callback);
            }
            if (!callback) {
                return;
            }

            if (callback.splice) {
                //callback is an array, which means it is a dependency list.
                //Adjust args if there are dependencies
                deps = callback;
                callback = relName;
                relName = null;
            } else {
                deps = undef;
            }
        }

        //Support require(['a'])
        callback = callback || function () {};

        //If relName is a function, it is an errback handler,
        //so remove it.
        if (typeof relName === 'function') {
            relName = forceSync;
            forceSync = alt;
        }

        //Simulate async callback;
        if (forceSync) {
            main(undef, deps, callback, relName);
        } else {
            //Using a non-zero value because of concern for what old browsers
            //do, and latest browsers "upgrade" to 4 if lower value is used:
            //http://www.whatwg.org/specs/web-apps/current-work/multipage/timers.html#dom-windowtimers-settimeout:
            //If want a value immediately, use require('id') instead -- something
            //that works in almond on the global level, but not guaranteed and
            //unlikely to work in other AMD implementations.
            setTimeout(function () {
                main(undef, deps, callback, relName);
            }, 4);
        }

        return req;
    };

    /**
     * Just drops the config on the floor, but returns req in case
     * the config return value is used.
     */
    req.config = function (cfg) {
        return req(cfg);
    };

    /**
     * Expose module registry for debugging and tooling
     */
    requirejs._defined = defined;

    define = function (name, deps, callback) {
        if (typeof name !== 'string') {
            throw new Error('See almond README: incorrect module build, no module name');
        }

        //This module may not have dependencies
        if (!deps.splice) {
            //deps is not an array, so probably means
            //an object literal or factory function for
            //the value. Adjust args.
            callback = deps;
            deps = [];
        }

        if (!hasProp(defined, name) && !hasProp(waiting, name)) {
            waiting[name] = [name, deps, callback];
        }
    };

    define.amd = {
        jQuery: true
    };
}());

define("almond", function(){});

/*!
 * jQuery JavaScript Library v2.1.4
 * http://jquery.com/
 *
 * Includes Sizzle.js
 * http://sizzlejs.com/
 *
 * Copyright 2005, 2014 jQuery Foundation, Inc. and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2015-04-28T16:01Z
 */

(function( global, factory ) {

	if ( typeof module === "object" && typeof module.exports === "object" ) {
		// For CommonJS and CommonJS-like environments where a proper `window`
		// is present, execute the factory and get jQuery.
		// For environments that do not have a `window` with a `document`
		// (such as Node.js), expose a factory as module.exports.
		// This accentuates the need for the creation of a real `window`.
		// e.g. var jQuery = require("jquery")(window);
		// See ticket #14549 for more info.
		module.exports = global.document ?
			factory( global, true ) :
			function( w ) {
				if ( !w.document ) {
					throw new Error( "jQuery requires a window with a document" );
				}
				return factory( w );
			};
	} else {
		factory( global );
	}

// Pass this if window is not defined yet
}(typeof window !== "undefined" ? window : this, function( window, noGlobal ) {

// Support: Firefox 18+
// Can't be in strict mode, several libs including ASP.NET trace
// the stack via arguments.caller.callee and Firefox dies if
// you try to trace through "use strict" call chains. (#13335)
//

var arr = [];

var slice = arr.slice;

var concat = arr.concat;

var push = arr.push;

var indexOf = arr.indexOf;

var class2type = {};

var toString = class2type.toString;

var hasOwn = class2type.hasOwnProperty;

var support = {};



var
	// Use the correct document accordingly with window argument (sandbox)
	document = window.document,

	version = "2.1.4",

	// Define a local copy of jQuery
	jQuery = function( selector, context ) {
		// The jQuery object is actually just the init constructor 'enhanced'
		// Need init if jQuery is called (just allow error to be thrown if not included)
		return new jQuery.fn.init( selector, context );
	},

	// Support: Android<4.1
	// Make sure we trim BOM and NBSP
	rtrim = /^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,

	// Matches dashed string for camelizing
	rmsPrefix = /^-ms-/,
	rdashAlpha = /-([\da-z])/gi,

	// Used by jQuery.camelCase as callback to replace()
	fcamelCase = function( all, letter ) {
		return letter.toUpperCase();
	};

jQuery.fn = jQuery.prototype = {
	// The current version of jQuery being used
	jquery: version,

	constructor: jQuery,

	// Start with an empty selector
	selector: "",

	// The default length of a jQuery object is 0
	length: 0,

	toArray: function() {
		return slice.call( this );
	},

	// Get the Nth element in the matched element set OR
	// Get the whole matched element set as a clean array
	get: function( num ) {
		return num != null ?

			// Return just the one element from the set
			( num < 0 ? this[ num + this.length ] : this[ num ] ) :

			// Return all the elements in a clean array
			slice.call( this );
	},

	// Take an array of elements and push it onto the stack
	// (returning the new matched element set)
	pushStack: function( elems ) {

		// Build a new jQuery matched element set
		var ret = jQuery.merge( this.constructor(), elems );

		// Add the old object onto the stack (as a reference)
		ret.prevObject = this;
		ret.context = this.context;

		// Return the newly-formed element set
		return ret;
	},

	// Execute a callback for every element in the matched set.
	// (You can seed the arguments with an array of args, but this is
	// only used internally.)
	each: function( callback, args ) {
		return jQuery.each( this, callback, args );
	},

	map: function( callback ) {
		return this.pushStack( jQuery.map(this, function( elem, i ) {
			return callback.call( elem, i, elem );
		}));
	},

	slice: function() {
		return this.pushStack( slice.apply( this, arguments ) );
	},

	first: function() {
		return this.eq( 0 );
	},

	last: function() {
		return this.eq( -1 );
	},

	eq: function( i ) {
		var len = this.length,
			j = +i + ( i < 0 ? len : 0 );
		return this.pushStack( j >= 0 && j < len ? [ this[j] ] : [] );
	},

	end: function() {
		return this.prevObject || this.constructor(null);
	},

	// For internal use only.
	// Behaves like an Array's method, not like a jQuery method.
	push: push,
	sort: arr.sort,
	splice: arr.splice
};

jQuery.extend = jQuery.fn.extend = function() {
	var options, name, src, copy, copyIsArray, clone,
		target = arguments[0] || {},
		i = 1,
		length = arguments.length,
		deep = false;

	// Handle a deep copy situation
	if ( typeof target === "boolean" ) {
		deep = target;

		// Skip the boolean and the target
		target = arguments[ i ] || {};
		i++;
	}

	// Handle case when target is a string or something (possible in deep copy)
	if ( typeof target !== "object" && !jQuery.isFunction(target) ) {
		target = {};
	}

	// Extend jQuery itself if only one argument is passed
	if ( i === length ) {
		target = this;
		i--;
	}

	for ( ; i < length; i++ ) {
		// Only deal with non-null/undefined values
		if ( (options = arguments[ i ]) != null ) {
			// Extend the base object
			for ( name in options ) {
				src = target[ name ];
				copy = options[ name ];

				// Prevent never-ending loop
				if ( target === copy ) {
					continue;
				}

				// Recurse if we're merging plain objects or arrays
				if ( deep && copy && ( jQuery.isPlainObject(copy) || (copyIsArray = jQuery.isArray(copy)) ) ) {
					if ( copyIsArray ) {
						copyIsArray = false;
						clone = src && jQuery.isArray(src) ? src : [];

					} else {
						clone = src && jQuery.isPlainObject(src) ? src : {};
					}

					// Never move original objects, clone them
					target[ name ] = jQuery.extend( deep, clone, copy );

				// Don't bring in undefined values
				} else if ( copy !== undefined ) {
					target[ name ] = copy;
				}
			}
		}
	}

	// Return the modified object
	return target;
};

jQuery.extend({
	// Unique for each copy of jQuery on the page
	expando: "jQuery" + ( version + Math.random() ).replace( /\D/g, "" ),

	// Assume jQuery is ready without the ready module
	isReady: true,

	error: function( msg ) {
		throw new Error( msg );
	},

	noop: function() {},

	isFunction: function( obj ) {
		return jQuery.type(obj) === "function";
	},

	isArray: Array.isArray,

	isWindow: function( obj ) {
		return obj != null && obj === obj.window;
	},

	isNumeric: function( obj ) {
		// parseFloat NaNs numeric-cast false positives (null|true|false|"")
		// ...but misinterprets leading-number strings, particularly hex literals ("0x...")
		// subtraction forces infinities to NaN
		// adding 1 corrects loss of precision from parseFloat (#15100)
		return !jQuery.isArray( obj ) && (obj - parseFloat( obj ) + 1) >= 0;
	},

	isPlainObject: function( obj ) {
		// Not plain objects:
		// - Any object or value whose internal [[Class]] property is not "[object Object]"
		// - DOM nodes
		// - window
		if ( jQuery.type( obj ) !== "object" || obj.nodeType || jQuery.isWindow( obj ) ) {
			return false;
		}

		if ( obj.constructor &&
				!hasOwn.call( obj.constructor.prototype, "isPrototypeOf" ) ) {
			return false;
		}

		// If the function hasn't returned already, we're confident that
		// |obj| is a plain object, created by {} or constructed with new Object
		return true;
	},

	isEmptyObject: function( obj ) {
		var name;
		for ( name in obj ) {
			return false;
		}
		return true;
	},

	type: function( obj ) {
		if ( obj == null ) {
			return obj + "";
		}
		// Support: Android<4.0, iOS<6 (functionish RegExp)
		return typeof obj === "object" || typeof obj === "function" ?
			class2type[ toString.call(obj) ] || "object" :
			typeof obj;
	},

	// Evaluates a script in a global context
	globalEval: function( code ) {
		var script,
			indirect = eval;

		code = jQuery.trim( code );

		if ( code ) {
			// If the code includes a valid, prologue position
			// strict mode pragma, execute code by injecting a
			// script tag into the document.
			if ( code.indexOf("use strict") === 1 ) {
				script = document.createElement("script");
				script.text = code;
				document.head.appendChild( script ).parentNode.removeChild( script );
			} else {
			// Otherwise, avoid the DOM node creation, insertion
			// and removal by using an indirect global eval
				indirect( code );
			}
		}
	},

	// Convert dashed to camelCase; used by the css and data modules
	// Support: IE9-11+
	// Microsoft forgot to hump their vendor prefix (#9572)
	camelCase: function( string ) {
		return string.replace( rmsPrefix, "ms-" ).replace( rdashAlpha, fcamelCase );
	},

	nodeName: function( elem, name ) {
		return elem.nodeName && elem.nodeName.toLowerCase() === name.toLowerCase();
	},

	// args is for internal usage only
	each: function( obj, callback, args ) {
		var value,
			i = 0,
			length = obj.length,
			isArray = isArraylike( obj );

		if ( args ) {
			if ( isArray ) {
				for ( ; i < length; i++ ) {
					value = callback.apply( obj[ i ], args );

					if ( value === false ) {
						break;
					}
				}
			} else {
				for ( i in obj ) {
					value = callback.apply( obj[ i ], args );

					if ( value === false ) {
						break;
					}
				}
			}

		// A special, fast, case for the most common use of each
		} else {
			if ( isArray ) {
				for ( ; i < length; i++ ) {
					value = callback.call( obj[ i ], i, obj[ i ] );

					if ( value === false ) {
						break;
					}
				}
			} else {
				for ( i in obj ) {
					value = callback.call( obj[ i ], i, obj[ i ] );

					if ( value === false ) {
						break;
					}
				}
			}
		}

		return obj;
	},

	// Support: Android<4.1
	trim: function( text ) {
		return text == null ?
			"" :
			( text + "" ).replace( rtrim, "" );
	},

	// results is for internal usage only
	makeArray: function( arr, results ) {
		var ret = results || [];

		if ( arr != null ) {
			if ( isArraylike( Object(arr) ) ) {
				jQuery.merge( ret,
					typeof arr === "string" ?
					[ arr ] : arr
				);
			} else {
				push.call( ret, arr );
			}
		}

		return ret;
	},

	inArray: function( elem, arr, i ) {
		return arr == null ? -1 : indexOf.call( arr, elem, i );
	},

	merge: function( first, second ) {
		var len = +second.length,
			j = 0,
			i = first.length;

		for ( ; j < len; j++ ) {
			first[ i++ ] = second[ j ];
		}

		first.length = i;

		return first;
	},

	grep: function( elems, callback, invert ) {
		var callbackInverse,
			matches = [],
			i = 0,
			length = elems.length,
			callbackExpect = !invert;

		// Go through the array, only saving the items
		// that pass the validator function
		for ( ; i < length; i++ ) {
			callbackInverse = !callback( elems[ i ], i );
			if ( callbackInverse !== callbackExpect ) {
				matches.push( elems[ i ] );
			}
		}

		return matches;
	},

	// arg is for internal usage only
	map: function( elems, callback, arg ) {
		var value,
			i = 0,
			length = elems.length,
			isArray = isArraylike( elems ),
			ret = [];

		// Go through the array, translating each of the items to their new values
		if ( isArray ) {
			for ( ; i < length; i++ ) {
				value = callback( elems[ i ], i, arg );

				if ( value != null ) {
					ret.push( value );
				}
			}

		// Go through every key on the object,
		} else {
			for ( i in elems ) {
				value = callback( elems[ i ], i, arg );

				if ( value != null ) {
					ret.push( value );
				}
			}
		}

		// Flatten any nested arrays
		return concat.apply( [], ret );
	},

	// A global GUID counter for objects
	guid: 1,

	// Bind a function to a context, optionally partially applying any
	// arguments.
	proxy: function( fn, context ) {
		var tmp, args, proxy;

		if ( typeof context === "string" ) {
			tmp = fn[ context ];
			context = fn;
			fn = tmp;
		}

		// Quick check to determine if target is callable, in the spec
		// this throws a TypeError, but we will just return undefined.
		if ( !jQuery.isFunction( fn ) ) {
			return undefined;
		}

		// Simulated bind
		args = slice.call( arguments, 2 );
		proxy = function() {
			return fn.apply( context || this, args.concat( slice.call( arguments ) ) );
		};

		// Set the guid of unique handler to the same of original handler, so it can be removed
		proxy.guid = fn.guid = fn.guid || jQuery.guid++;

		return proxy;
	},

	now: Date.now,

	// jQuery.support is not used in Core but other projects attach their
	// properties to it so it needs to exist.
	support: support
});

// Populate the class2type map
jQuery.each("Boolean Number String Function Array Date RegExp Object Error".split(" "), function(i, name) {
	class2type[ "[object " + name + "]" ] = name.toLowerCase();
});

function isArraylike( obj ) {

	// Support: iOS 8.2 (not reproducible in simulator)
	// `in` check used to prevent JIT error (gh-2145)
	// hasOwn isn't used here due to false negatives
	// regarding Nodelist length in IE
	var length = "length" in obj && obj.length,
		type = jQuery.type( obj );

	if ( type === "function" || jQuery.isWindow( obj ) ) {
		return false;
	}

	if ( obj.nodeType === 1 && length ) {
		return true;
	}

	return type === "array" || length === 0 ||
		typeof length === "number" && length > 0 && ( length - 1 ) in obj;
}
var Sizzle =
/*!
 * Sizzle CSS Selector Engine v2.2.0-pre
 * http://sizzlejs.com/
 *
 * Copyright 2008, 2014 jQuery Foundation, Inc. and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2014-12-16
 */
(function( window ) {

var i,
	support,
	Expr,
	getText,
	isXML,
	tokenize,
	compile,
	select,
	outermostContext,
	sortInput,
	hasDuplicate,

	// Local document vars
	setDocument,
	document,
	docElem,
	documentIsHTML,
	rbuggyQSA,
	rbuggyMatches,
	matches,
	contains,

	// Instance-specific data
	expando = "sizzle" + 1 * new Date(),
	preferredDoc = window.document,
	dirruns = 0,
	done = 0,
	classCache = createCache(),
	tokenCache = createCache(),
	compilerCache = createCache(),
	sortOrder = function( a, b ) {
		if ( a === b ) {
			hasDuplicate = true;
		}
		return 0;
	},

	// General-purpose constants
	MAX_NEGATIVE = 1 << 31,

	// Instance methods
	hasOwn = ({}).hasOwnProperty,
	arr = [],
	pop = arr.pop,
	push_native = arr.push,
	push = arr.push,
	slice = arr.slice,
	// Use a stripped-down indexOf as it's faster than native
	// http://jsperf.com/thor-indexof-vs-for/5
	indexOf = function( list, elem ) {
		var i = 0,
			len = list.length;
		for ( ; i < len; i++ ) {
			if ( list[i] === elem ) {
				return i;
			}
		}
		return -1;
	},

	booleans = "checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",

	// Regular expressions

	// Whitespace characters http://www.w3.org/TR/css3-selectors/#whitespace
	whitespace = "[\\x20\\t\\r\\n\\f]",
	// http://www.w3.org/TR/css3-syntax/#characters
	characterEncoding = "(?:\\\\.|[\\w-]|[^\\x00-\\xa0])+",

	// Loosely modeled on CSS identifier characters
	// An unquoted value should be a CSS identifier http://www.w3.org/TR/css3-selectors/#attribute-selectors
	// Proper syntax: http://www.w3.org/TR/CSS21/syndata.html#value-def-identifier
	identifier = characterEncoding.replace( "w", "w#" ),

	// Attribute selectors: http://www.w3.org/TR/selectors/#attribute-selectors
	attributes = "\\[" + whitespace + "*(" + characterEncoding + ")(?:" + whitespace +
		// Operator (capture 2)
		"*([*^$|!~]?=)" + whitespace +
		// "Attribute values must be CSS identifiers [capture 5] or strings [capture 3 or capture 4]"
		"*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|(" + identifier + "))|)" + whitespace +
		"*\\]",

	pseudos = ":(" + characterEncoding + ")(?:\\((" +
		// To reduce the number of selectors needing tokenize in the preFilter, prefer arguments:
		// 1. quoted (capture 3; capture 4 or capture 5)
		"('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|" +
		// 2. simple (capture 6)
		"((?:\\\\.|[^\\\\()[\\]]|" + attributes + ")*)|" +
		// 3. anything else (capture 2)
		".*" +
		")\\)|)",

	// Leading and non-escaped trailing whitespace, capturing some non-whitespace characters preceding the latter
	rwhitespace = new RegExp( whitespace + "+", "g" ),
	rtrim = new RegExp( "^" + whitespace + "+|((?:^|[^\\\\])(?:\\\\.)*)" + whitespace + "+$", "g" ),

	rcomma = new RegExp( "^" + whitespace + "*," + whitespace + "*" ),
	rcombinators = new RegExp( "^" + whitespace + "*([>+~]|" + whitespace + ")" + whitespace + "*" ),

	rattributeQuotes = new RegExp( "=" + whitespace + "*([^\\]'\"]*?)" + whitespace + "*\\]", "g" ),

	rpseudo = new RegExp( pseudos ),
	ridentifier = new RegExp( "^" + identifier + "$" ),

	matchExpr = {
		"ID": new RegExp( "^#(" + characterEncoding + ")" ),
		"CLASS": new RegExp( "^\\.(" + characterEncoding + ")" ),
		"TAG": new RegExp( "^(" + characterEncoding.replace( "w", "w*" ) + ")" ),
		"ATTR": new RegExp( "^" + attributes ),
		"PSEUDO": new RegExp( "^" + pseudos ),
		"CHILD": new RegExp( "^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\(" + whitespace +
			"*(even|odd|(([+-]|)(\\d*)n|)" + whitespace + "*(?:([+-]|)" + whitespace +
			"*(\\d+)|))" + whitespace + "*\\)|)", "i" ),
		"bool": new RegExp( "^(?:" + booleans + ")$", "i" ),
		// For use in libraries implementing .is()
		// We use this for POS matching in `select`
		"needsContext": new RegExp( "^" + whitespace + "*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\(" +
			whitespace + "*((?:-\\d)?\\d*)" + whitespace + "*\\)|)(?=[^-]|$)", "i" )
	},

	rinputs = /^(?:input|select|textarea|button)$/i,
	rheader = /^h\d$/i,

	rnative = /^[^{]+\{\s*\[native \w/,

	// Easily-parseable/retrievable ID or TAG or CLASS selectors
	rquickExpr = /^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,

	rsibling = /[+~]/,
	rescape = /'|\\/g,

	// CSS escapes http://www.w3.org/TR/CSS21/syndata.html#escaped-characters
	runescape = new RegExp( "\\\\([\\da-f]{1,6}" + whitespace + "?|(" + whitespace + ")|.)", "ig" ),
	funescape = function( _, escaped, escapedWhitespace ) {
		var high = "0x" + escaped - 0x10000;
		// NaN means non-codepoint
		// Support: Firefox<24
		// Workaround erroneous numeric interpretation of +"0x"
		return high !== high || escapedWhitespace ?
			escaped :
			high < 0 ?
				// BMP codepoint
				String.fromCharCode( high + 0x10000 ) :
				// Supplemental Plane codepoint (surrogate pair)
				String.fromCharCode( high >> 10 | 0xD800, high & 0x3FF | 0xDC00 );
	},

	// Used for iframes
	// See setDocument()
	// Removing the function wrapper causes a "Permission Denied"
	// error in IE
	unloadHandler = function() {
		setDocument();
	};

// Optimize for push.apply( _, NodeList )
try {
	push.apply(
		(arr = slice.call( preferredDoc.childNodes )),
		preferredDoc.childNodes
	);
	// Support: Android<4.0
	// Detect silently failing push.apply
	arr[ preferredDoc.childNodes.length ].nodeType;
} catch ( e ) {
	push = { apply: arr.length ?

		// Leverage slice if possible
		function( target, els ) {
			push_native.apply( target, slice.call(els) );
		} :

		// Support: IE<9
		// Otherwise append directly
		function( target, els ) {
			var j = target.length,
				i = 0;
			// Can't trust NodeList.length
			while ( (target[j++] = els[i++]) ) {}
			target.length = j - 1;
		}
	};
}

function Sizzle( selector, context, results, seed ) {
	var match, elem, m, nodeType,
		// QSA vars
		i, groups, old, nid, newContext, newSelector;

	if ( ( context ? context.ownerDocument || context : preferredDoc ) !== document ) {
		setDocument( context );
	}

	context = context || document;
	results = results || [];
	nodeType = context.nodeType;

	if ( typeof selector !== "string" || !selector ||
		nodeType !== 1 && nodeType !== 9 && nodeType !== 11 ) {

		return results;
	}

	if ( !seed && documentIsHTML ) {

		// Try to shortcut find operations when possible (e.g., not under DocumentFragment)
		if ( nodeType !== 11 && (match = rquickExpr.exec( selector )) ) {
			// Speed-up: Sizzle("#ID")
			if ( (m = match[1]) ) {
				if ( nodeType === 9 ) {
					elem = context.getElementById( m );
					// Check parentNode to catch when Blackberry 4.6 returns
					// nodes that are no longer in the document (jQuery #6963)
					if ( elem && elem.parentNode ) {
						// Handle the case where IE, Opera, and Webkit return items
						// by name instead of ID
						if ( elem.id === m ) {
							results.push( elem );
							return results;
						}
					} else {
						return results;
					}
				} else {
					// Context is not a document
					if ( context.ownerDocument && (elem = context.ownerDocument.getElementById( m )) &&
						contains( context, elem ) && elem.id === m ) {
						results.push( elem );
						return results;
					}
				}

			// Speed-up: Sizzle("TAG")
			} else if ( match[2] ) {
				push.apply( results, context.getElementsByTagName( selector ) );
				return results;

			// Speed-up: Sizzle(".CLASS")
			} else if ( (m = match[3]) && support.getElementsByClassName ) {
				push.apply( results, context.getElementsByClassName( m ) );
				return results;
			}
		}

		// QSA path
		if ( support.qsa && (!rbuggyQSA || !rbuggyQSA.test( selector )) ) {
			nid = old = expando;
			newContext = context;
			newSelector = nodeType !== 1 && selector;

			// qSA works strangely on Element-rooted queries
			// We can work around this by specifying an extra ID on the root
			// and working up from there (Thanks to Andrew Dupont for the technique)
			// IE 8 doesn't work on object elements
			if ( nodeType === 1 && context.nodeName.toLowerCase() !== "object" ) {
				groups = tokenize( selector );

				if ( (old = context.getAttribute("id")) ) {
					nid = old.replace( rescape, "\\$&" );
				} else {
					context.setAttribute( "id", nid );
				}
				nid = "[id='" + nid + "'] ";

				i = groups.length;
				while ( i-- ) {
					groups[i] = nid + toSelector( groups[i] );
				}
				newContext = rsibling.test( selector ) && testContext( context.parentNode ) || context;
				newSelector = groups.join(",");
			}

			if ( newSelector ) {
				try {
					push.apply( results,
						newContext.querySelectorAll( newSelector )
					);
					return results;
				} catch(qsaError) {
				} finally {
					if ( !old ) {
						context.removeAttribute("id");
					}
				}
			}
		}
	}

	// All others
	return select( selector.replace( rtrim, "$1" ), context, results, seed );
}

/**
 * Create key-value caches of limited size
 * @returns {Function(string, Object)} Returns the Object data after storing it on itself with
 *	property name the (space-suffixed) string and (if the cache is larger than Expr.cacheLength)
 *	deleting the oldest entry
 */
function createCache() {
	var keys = [];

	function cache( key, value ) {
		// Use (key + " ") to avoid collision with native prototype properties (see Issue #157)
		if ( keys.push( key + " " ) > Expr.cacheLength ) {
			// Only keep the most recent entries
			delete cache[ keys.shift() ];
		}
		return (cache[ key + " " ] = value);
	}
	return cache;
}

/**
 * Mark a function for special use by Sizzle
 * @param {Function} fn The function to mark
 */
function markFunction( fn ) {
	fn[ expando ] = true;
	return fn;
}

/**
 * Support testing using an element
 * @param {Function} fn Passed the created div and expects a boolean result
 */
function assert( fn ) {
	var div = document.createElement("div");

	try {
		return !!fn( div );
	} catch (e) {
		return false;
	} finally {
		// Remove from its parent by default
		if ( div.parentNode ) {
			div.parentNode.removeChild( div );
		}
		// release memory in IE
		div = null;
	}
}

/**
 * Adds the same handler for all of the specified attrs
 * @param {String} attrs Pipe-separated list of attributes
 * @param {Function} handler The method that will be applied
 */
function addHandle( attrs, handler ) {
	var arr = attrs.split("|"),
		i = attrs.length;

	while ( i-- ) {
		Expr.attrHandle[ arr[i] ] = handler;
	}
}

/**
 * Checks document order of two siblings
 * @param {Element} a
 * @param {Element} b
 * @returns {Number} Returns less than 0 if a precedes b, greater than 0 if a follows b
 */
function siblingCheck( a, b ) {
	var cur = b && a,
		diff = cur && a.nodeType === 1 && b.nodeType === 1 &&
			( ~b.sourceIndex || MAX_NEGATIVE ) -
			( ~a.sourceIndex || MAX_NEGATIVE );

	// Use IE sourceIndex if available on both nodes
	if ( diff ) {
		return diff;
	}

	// Check if b follows a
	if ( cur ) {
		while ( (cur = cur.nextSibling) ) {
			if ( cur === b ) {
				return -1;
			}
		}
	}

	return a ? 1 : -1;
}

/**
 * Returns a function to use in pseudos for input types
 * @param {String} type
 */
function createInputPseudo( type ) {
	return function( elem ) {
		var name = elem.nodeName.toLowerCase();
		return name === "input" && elem.type === type;
	};
}

/**
 * Returns a function to use in pseudos for buttons
 * @param {String} type
 */
function createButtonPseudo( type ) {
	return function( elem ) {
		var name = elem.nodeName.toLowerCase();
		return (name === "input" || name === "button") && elem.type === type;
	};
}

/**
 * Returns a function to use in pseudos for positionals
 * @param {Function} fn
 */
function createPositionalPseudo( fn ) {
	return markFunction(function( argument ) {
		argument = +argument;
		return markFunction(function( seed, matches ) {
			var j,
				matchIndexes = fn( [], seed.length, argument ),
				i = matchIndexes.length;

			// Match elements found at the specified indexes
			while ( i-- ) {
				if ( seed[ (j = matchIndexes[i]) ] ) {
					seed[j] = !(matches[j] = seed[j]);
				}
			}
		});
	});
}

/**
 * Checks a node for validity as a Sizzle context
 * @param {Element|Object=} context
 * @returns {Element|Object|Boolean} The input node if acceptable, otherwise a falsy value
 */
function testContext( context ) {
	return context && typeof context.getElementsByTagName !== "undefined" && context;
}

// Expose support vars for convenience
support = Sizzle.support = {};

/**
 * Detects XML nodes
 * @param {Element|Object} elem An element or a document
 * @returns {Boolean} True iff elem is a non-HTML XML node
 */
isXML = Sizzle.isXML = function( elem ) {
	// documentElement is verified for cases where it doesn't yet exist
	// (such as loading iframes in IE - #4833)
	var documentElement = elem && (elem.ownerDocument || elem).documentElement;
	return documentElement ? documentElement.nodeName !== "HTML" : false;
};

/**
 * Sets document-related variables once based on the current document
 * @param {Element|Object} [doc] An element or document object to use to set the document
 * @returns {Object} Returns the current document
 */
setDocument = Sizzle.setDocument = function( node ) {
	var hasCompare, parent,
		doc = node ? node.ownerDocument || node : preferredDoc;

	// If no document and documentElement is available, return
	if ( doc === document || doc.nodeType !== 9 || !doc.documentElement ) {
		return document;
	}

	// Set our document
	document = doc;
	docElem = doc.documentElement;
	parent = doc.defaultView;

	// Support: IE>8
	// If iframe document is assigned to "document" variable and if iframe has been reloaded,
	// IE will throw "permission denied" error when accessing "document" variable, see jQuery #13936
	// IE6-8 do not support the defaultView property so parent will be undefined
	if ( parent && parent !== parent.top ) {
		// IE11 does not have attachEvent, so all must suffer
		if ( parent.addEventListener ) {
			parent.addEventListener( "unload", unloadHandler, false );
		} else if ( parent.attachEvent ) {
			parent.attachEvent( "onunload", unloadHandler );
		}
	}

	/* Support tests
	---------------------------------------------------------------------- */
	documentIsHTML = !isXML( doc );

	/* Attributes
	---------------------------------------------------------------------- */

	// Support: IE<8
	// Verify that getAttribute really returns attributes and not properties
	// (excepting IE8 booleans)
	support.attributes = assert(function( div ) {
		div.className = "i";
		return !div.getAttribute("className");
	});

	/* getElement(s)By*
	---------------------------------------------------------------------- */

	// Check if getElementsByTagName("*") returns only elements
	support.getElementsByTagName = assert(function( div ) {
		div.appendChild( doc.createComment("") );
		return !div.getElementsByTagName("*").length;
	});

	// Support: IE<9
	support.getElementsByClassName = rnative.test( doc.getElementsByClassName );

	// Support: IE<10
	// Check if getElementById returns elements by name
	// The broken getElementById methods don't pick up programatically-set names,
	// so use a roundabout getElementsByName test
	support.getById = assert(function( div ) {
		docElem.appendChild( div ).id = expando;
		return !doc.getElementsByName || !doc.getElementsByName( expando ).length;
	});

	// ID find and filter
	if ( support.getById ) {
		Expr.find["ID"] = function( id, context ) {
			if ( typeof context.getElementById !== "undefined" && documentIsHTML ) {
				var m = context.getElementById( id );
				// Check parentNode to catch when Blackberry 4.6 returns
				// nodes that are no longer in the document #6963
				return m && m.parentNode ? [ m ] : [];
			}
		};
		Expr.filter["ID"] = function( id ) {
			var attrId = id.replace( runescape, funescape );
			return function( elem ) {
				return elem.getAttribute("id") === attrId;
			};
		};
	} else {
		// Support: IE6/7
		// getElementById is not reliable as a find shortcut
		delete Expr.find["ID"];

		Expr.filter["ID"] =  function( id ) {
			var attrId = id.replace( runescape, funescape );
			return function( elem ) {
				var node = typeof elem.getAttributeNode !== "undefined" && elem.getAttributeNode("id");
				return node && node.value === attrId;
			};
		};
	}

	// Tag
	Expr.find["TAG"] = support.getElementsByTagName ?
		function( tag, context ) {
			if ( typeof context.getElementsByTagName !== "undefined" ) {
				return context.getElementsByTagName( tag );

			// DocumentFragment nodes don't have gEBTN
			} else if ( support.qsa ) {
				return context.querySelectorAll( tag );
			}
		} :

		function( tag, context ) {
			var elem,
				tmp = [],
				i = 0,
				// By happy coincidence, a (broken) gEBTN appears on DocumentFragment nodes too
				results = context.getElementsByTagName( tag );

			// Filter out possible comments
			if ( tag === "*" ) {
				while ( (elem = results[i++]) ) {
					if ( elem.nodeType === 1 ) {
						tmp.push( elem );
					}
				}

				return tmp;
			}
			return results;
		};

	// Class
	Expr.find["CLASS"] = support.getElementsByClassName && function( className, context ) {
		if ( documentIsHTML ) {
			return context.getElementsByClassName( className );
		}
	};

	/* QSA/matchesSelector
	---------------------------------------------------------------------- */

	// QSA and matchesSelector support

	// matchesSelector(:active) reports false when true (IE9/Opera 11.5)
	rbuggyMatches = [];

	// qSa(:focus) reports false when true (Chrome 21)
	// We allow this because of a bug in IE8/9 that throws an error
	// whenever `document.activeElement` is accessed on an iframe
	// So, we allow :focus to pass through QSA all the time to avoid the IE error
	// See http://bugs.jquery.com/ticket/13378
	rbuggyQSA = [];

	if ( (support.qsa = rnative.test( doc.querySelectorAll )) ) {
		// Build QSA regex
		// Regex strategy adopted from Diego Perini
		assert(function( div ) {
			// Select is set to empty string on purpose
			// This is to test IE's treatment of not explicitly
			// setting a boolean content attribute,
			// since its presence should be enough
			// http://bugs.jquery.com/ticket/12359
			docElem.appendChild( div ).innerHTML = "<a id='" + expando + "'></a>" +
				"<select id='" + expando + "-\f]' msallowcapture=''>" +
				"<option selected=''></option></select>";

			// Support: IE8, Opera 11-12.16
			// Nothing should be selected when empty strings follow ^= or $= or *=
			// The test attribute must be unknown in Opera but "safe" for WinRT
			// http://msdn.microsoft.com/en-us/library/ie/hh465388.aspx#attribute_section
			if ( div.querySelectorAll("[msallowcapture^='']").length ) {
				rbuggyQSA.push( "[*^$]=" + whitespace + "*(?:''|\"\")" );
			}

			// Support: IE8
			// Boolean attributes and "value" are not treated correctly
			if ( !div.querySelectorAll("[selected]").length ) {
				rbuggyQSA.push( "\\[" + whitespace + "*(?:value|" + booleans + ")" );
			}

			// Support: Chrome<29, Android<4.2+, Safari<7.0+, iOS<7.0+, PhantomJS<1.9.7+
			if ( !div.querySelectorAll( "[id~=" + expando + "-]" ).length ) {
				rbuggyQSA.push("~=");
			}

			// Webkit/Opera - :checked should return selected option elements
			// http://www.w3.org/TR/2011/REC-css3-selectors-20110929/#checked
			// IE8 throws error here and will not see later tests
			if ( !div.querySelectorAll(":checked").length ) {
				rbuggyQSA.push(":checked");
			}

			// Support: Safari 8+, iOS 8+
			// https://bugs.webkit.org/show_bug.cgi?id=136851
			// In-page `selector#id sibing-combinator selector` fails
			if ( !div.querySelectorAll( "a#" + expando + "+*" ).length ) {
				rbuggyQSA.push(".#.+[+~]");
			}
		});

		assert(function( div ) {
			// Support: Windows 8 Native Apps
			// The type and name attributes are restricted during .innerHTML assignment
			var input = doc.createElement("input");
			input.setAttribute( "type", "hidden" );
			div.appendChild( input ).setAttribute( "name", "D" );

			// Support: IE8
			// Enforce case-sensitivity of name attribute
			if ( div.querySelectorAll("[name=d]").length ) {
				rbuggyQSA.push( "name" + whitespace + "*[*^$|!~]?=" );
			}

			// FF 3.5 - :enabled/:disabled and hidden elements (hidden elements are still enabled)
			// IE8 throws error here and will not see later tests
			if ( !div.querySelectorAll(":enabled").length ) {
				rbuggyQSA.push( ":enabled", ":disabled" );
			}

			// Opera 10-11 does not throw on post-comma invalid pseudos
			div.querySelectorAll("*,:x");
			rbuggyQSA.push(",.*:");
		});
	}

	if ( (support.matchesSelector = rnative.test( (matches = docElem.matches ||
		docElem.webkitMatchesSelector ||
		docElem.mozMatchesSelector ||
		docElem.oMatchesSelector ||
		docElem.msMatchesSelector) )) ) {

		assert(function( div ) {
			// Check to see if it's possible to do matchesSelector
			// on a disconnected node (IE 9)
			support.disconnectedMatch = matches.call( div, "div" );

			// This should fail with an exception
			// Gecko does not error, returns false instead
			matches.call( div, "[s!='']:x" );
			rbuggyMatches.push( "!=", pseudos );
		});
	}

	rbuggyQSA = rbuggyQSA.length && new RegExp( rbuggyQSA.join("|") );
	rbuggyMatches = rbuggyMatches.length && new RegExp( rbuggyMatches.join("|") );

	/* Contains
	---------------------------------------------------------------------- */
	hasCompare = rnative.test( docElem.compareDocumentPosition );

	// Element contains another
	// Purposefully does not implement inclusive descendent
	// As in, an element does not contain itself
	contains = hasCompare || rnative.test( docElem.contains ) ?
		function( a, b ) {
			var adown = a.nodeType === 9 ? a.documentElement : a,
				bup = b && b.parentNode;
			return a === bup || !!( bup && bup.nodeType === 1 && (
				adown.contains ?
					adown.contains( bup ) :
					a.compareDocumentPosition && a.compareDocumentPosition( bup ) & 16
			));
		} :
		function( a, b ) {
			if ( b ) {
				while ( (b = b.parentNode) ) {
					if ( b === a ) {
						return true;
					}
				}
			}
			return false;
		};

	/* Sorting
	---------------------------------------------------------------------- */

	// Document order sorting
	sortOrder = hasCompare ?
	function( a, b ) {

		// Flag for duplicate removal
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}

		// Sort on method existence if only one input has compareDocumentPosition
		var compare = !a.compareDocumentPosition - !b.compareDocumentPosition;
		if ( compare ) {
			return compare;
		}

		// Calculate position if both inputs belong to the same document
		compare = ( a.ownerDocument || a ) === ( b.ownerDocument || b ) ?
			a.compareDocumentPosition( b ) :

			// Otherwise we know they are disconnected
			1;

		// Disconnected nodes
		if ( compare & 1 ||
			(!support.sortDetached && b.compareDocumentPosition( a ) === compare) ) {

			// Choose the first element that is related to our preferred document
			if ( a === doc || a.ownerDocument === preferredDoc && contains(preferredDoc, a) ) {
				return -1;
			}
			if ( b === doc || b.ownerDocument === preferredDoc && contains(preferredDoc, b) ) {
				return 1;
			}

			// Maintain original order
			return sortInput ?
				( indexOf( sortInput, a ) - indexOf( sortInput, b ) ) :
				0;
		}

		return compare & 4 ? -1 : 1;
	} :
	function( a, b ) {
		// Exit early if the nodes are identical
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}

		var cur,
			i = 0,
			aup = a.parentNode,
			bup = b.parentNode,
			ap = [ a ],
			bp = [ b ];

		// Parentless nodes are either documents or disconnected
		if ( !aup || !bup ) {
			return a === doc ? -1 :
				b === doc ? 1 :
				aup ? -1 :
				bup ? 1 :
				sortInput ?
				( indexOf( sortInput, a ) - indexOf( sortInput, b ) ) :
				0;

		// If the nodes are siblings, we can do a quick check
		} else if ( aup === bup ) {
			return siblingCheck( a, b );
		}

		// Otherwise we need full lists of their ancestors for comparison
		cur = a;
		while ( (cur = cur.parentNode) ) {
			ap.unshift( cur );
		}
		cur = b;
		while ( (cur = cur.parentNode) ) {
			bp.unshift( cur );
		}

		// Walk down the tree looking for a discrepancy
		while ( ap[i] === bp[i] ) {
			i++;
		}

		return i ?
			// Do a sibling check if the nodes have a common ancestor
			siblingCheck( ap[i], bp[i] ) :

			// Otherwise nodes in our document sort first
			ap[i] === preferredDoc ? -1 :
			bp[i] === preferredDoc ? 1 :
			0;
	};

	return doc;
};

Sizzle.matches = function( expr, elements ) {
	return Sizzle( expr, null, null, elements );
};

Sizzle.matchesSelector = function( elem, expr ) {
	// Set document vars if needed
	if ( ( elem.ownerDocument || elem ) !== document ) {
		setDocument( elem );
	}

	// Make sure that attribute selectors are quoted
	expr = expr.replace( rattributeQuotes, "='$1']" );

	if ( support.matchesSelector && documentIsHTML &&
		( !rbuggyMatches || !rbuggyMatches.test( expr ) ) &&
		( !rbuggyQSA     || !rbuggyQSA.test( expr ) ) ) {

		try {
			var ret = matches.call( elem, expr );

			// IE 9's matchesSelector returns false on disconnected nodes
			if ( ret || support.disconnectedMatch ||
					// As well, disconnected nodes are said to be in a document
					// fragment in IE 9
					elem.document && elem.document.nodeType !== 11 ) {
				return ret;
			}
		} catch (e) {}
	}

	return Sizzle( expr, document, null, [ elem ] ).length > 0;
};

Sizzle.contains = function( context, elem ) {
	// Set document vars if needed
	if ( ( context.ownerDocument || context ) !== document ) {
		setDocument( context );
	}
	return contains( context, elem );
};

Sizzle.attr = function( elem, name ) {
	// Set document vars if needed
	if ( ( elem.ownerDocument || elem ) !== document ) {
		setDocument( elem );
	}

	var fn = Expr.attrHandle[ name.toLowerCase() ],
		// Don't get fooled by Object.prototype properties (jQuery #13807)
		val = fn && hasOwn.call( Expr.attrHandle, name.toLowerCase() ) ?
			fn( elem, name, !documentIsHTML ) :
			undefined;

	return val !== undefined ?
		val :
		support.attributes || !documentIsHTML ?
			elem.getAttribute( name ) :
			(val = elem.getAttributeNode(name)) && val.specified ?
				val.value :
				null;
};

Sizzle.error = function( msg ) {
	throw new Error( "Syntax error, unrecognized expression: " + msg );
};

/**
 * Document sorting and removing duplicates
 * @param {ArrayLike} results
 */
Sizzle.uniqueSort = function( results ) {
	var elem,
		duplicates = [],
		j = 0,
		i = 0;

	// Unless we *know* we can detect duplicates, assume their presence
	hasDuplicate = !support.detectDuplicates;
	sortInput = !support.sortStable && results.slice( 0 );
	results.sort( sortOrder );

	if ( hasDuplicate ) {
		while ( (elem = results[i++]) ) {
			if ( elem === results[ i ] ) {
				j = duplicates.push( i );
			}
		}
		while ( j-- ) {
			results.splice( duplicates[ j ], 1 );
		}
	}

	// Clear input after sorting to release objects
	// See https://github.com/jquery/sizzle/pull/225
	sortInput = null;

	return results;
};

/**
 * Utility function for retrieving the text value of an array of DOM nodes
 * @param {Array|Element} elem
 */
getText = Sizzle.getText = function( elem ) {
	var node,
		ret = "",
		i = 0,
		nodeType = elem.nodeType;

	if ( !nodeType ) {
		// If no nodeType, this is expected to be an array
		while ( (node = elem[i++]) ) {
			// Do not traverse comment nodes
			ret += getText( node );
		}
	} else if ( nodeType === 1 || nodeType === 9 || nodeType === 11 ) {
		// Use textContent for elements
		// innerText usage removed for consistency of new lines (jQuery #11153)
		if ( typeof elem.textContent === "string" ) {
			return elem.textContent;
		} else {
			// Traverse its children
			for ( elem = elem.firstChild; elem; elem = elem.nextSibling ) {
				ret += getText( elem );
			}
		}
	} else if ( nodeType === 3 || nodeType === 4 ) {
		return elem.nodeValue;
	}
	// Do not include comment or processing instruction nodes

	return ret;
};

Expr = Sizzle.selectors = {

	// Can be adjusted by the user
	cacheLength: 50,

	createPseudo: markFunction,

	match: matchExpr,

	attrHandle: {},

	find: {},

	relative: {
		">": { dir: "parentNode", first: true },
		" ": { dir: "parentNode" },
		"+": { dir: "previousSibling", first: true },
		"~": { dir: "previousSibling" }
	},

	preFilter: {
		"ATTR": function( match ) {
			match[1] = match[1].replace( runescape, funescape );

			// Move the given value to match[3] whether quoted or unquoted
			match[3] = ( match[3] || match[4] || match[5] || "" ).replace( runescape, funescape );

			if ( match[2] === "~=" ) {
				match[3] = " " + match[3] + " ";
			}

			return match.slice( 0, 4 );
		},

		"CHILD": function( match ) {
			/* matches from matchExpr["CHILD"]
				1 type (only|nth|...)
				2 what (child|of-type)
				3 argument (even|odd|\d*|\d*n([+-]\d+)?|...)
				4 xn-component of xn+y argument ([+-]?\d*n|)
				5 sign of xn-component
				6 x of xn-component
				7 sign of y-component
				8 y of y-component
			*/
			match[1] = match[1].toLowerCase();

			if ( match[1].slice( 0, 3 ) === "nth" ) {
				// nth-* requires argument
				if ( !match[3] ) {
					Sizzle.error( match[0] );
				}

				// numeric x and y parameters for Expr.filter.CHILD
				// remember that false/true cast respectively to 0/1
				match[4] = +( match[4] ? match[5] + (match[6] || 1) : 2 * ( match[3] === "even" || match[3] === "odd" ) );
				match[5] = +( ( match[7] + match[8] ) || match[3] === "odd" );

			// other types prohibit arguments
			} else if ( match[3] ) {
				Sizzle.error( match[0] );
			}

			return match;
		},

		"PSEUDO": function( match ) {
			var excess,
				unquoted = !match[6] && match[2];

			if ( matchExpr["CHILD"].test( match[0] ) ) {
				return null;
			}

			// Accept quoted arguments as-is
			if ( match[3] ) {
				match[2] = match[4] || match[5] || "";

			// Strip excess characters from unquoted arguments
			} else if ( unquoted && rpseudo.test( unquoted ) &&
				// Get excess from tokenize (recursively)
				(excess = tokenize( unquoted, true )) &&
				// advance to the next closing parenthesis
				(excess = unquoted.indexOf( ")", unquoted.length - excess ) - unquoted.length) ) {

				// excess is a negative index
				match[0] = match[0].slice( 0, excess );
				match[2] = unquoted.slice( 0, excess );
			}

			// Return only captures needed by the pseudo filter method (type and argument)
			return match.slice( 0, 3 );
		}
	},

	filter: {

		"TAG": function( nodeNameSelector ) {
			var nodeName = nodeNameSelector.replace( runescape, funescape ).toLowerCase();
			return nodeNameSelector === "*" ?
				function() { return true; } :
				function( elem ) {
					return elem.nodeName && elem.nodeName.toLowerCase() === nodeName;
				};
		},

		"CLASS": function( className ) {
			var pattern = classCache[ className + " " ];

			return pattern ||
				(pattern = new RegExp( "(^|" + whitespace + ")" + className + "(" + whitespace + "|$)" )) &&
				classCache( className, function( elem ) {
					return pattern.test( typeof elem.className === "string" && elem.className || typeof elem.getAttribute !== "undefined" && elem.getAttribute("class") || "" );
				});
		},

		"ATTR": function( name, operator, check ) {
			return function( elem ) {
				var result = Sizzle.attr( elem, name );

				if ( result == null ) {
					return operator === "!=";
				}
				if ( !operator ) {
					return true;
				}

				result += "";

				return operator === "=" ? result === check :
					operator === "!=" ? result !== check :
					operator === "^=" ? check && result.indexOf( check ) === 0 :
					operator === "*=" ? check && result.indexOf( check ) > -1 :
					operator === "$=" ? check && result.slice( -check.length ) === check :
					operator === "~=" ? ( " " + result.replace( rwhitespace, " " ) + " " ).indexOf( check ) > -1 :
					operator === "|=" ? result === check || result.slice( 0, check.length + 1 ) === check + "-" :
					false;
			};
		},

		"CHILD": function( type, what, argument, first, last ) {
			var simple = type.slice( 0, 3 ) !== "nth",
				forward = type.slice( -4 ) !== "last",
				ofType = what === "of-type";

			return first === 1 && last === 0 ?

				// Shortcut for :nth-*(n)
				function( elem ) {
					return !!elem.parentNode;
				} :

				function( elem, context, xml ) {
					var cache, outerCache, node, diff, nodeIndex, start,
						dir = simple !== forward ? "nextSibling" : "previousSibling",
						parent = elem.parentNode,
						name = ofType && elem.nodeName.toLowerCase(),
						useCache = !xml && !ofType;

					if ( parent ) {

						// :(first|last|only)-(child|of-type)
						if ( simple ) {
							while ( dir ) {
								node = elem;
								while ( (node = node[ dir ]) ) {
									if ( ofType ? node.nodeName.toLowerCase() === name : node.nodeType === 1 ) {
										return false;
									}
								}
								// Reverse direction for :only-* (if we haven't yet done so)
								start = dir = type === "only" && !start && "nextSibling";
							}
							return true;
						}

						start = [ forward ? parent.firstChild : parent.lastChild ];

						// non-xml :nth-child(...) stores cache data on `parent`
						if ( forward && useCache ) {
							// Seek `elem` from a previously-cached index
							outerCache = parent[ expando ] || (parent[ expando ] = {});
							cache = outerCache[ type ] || [];
							nodeIndex = cache[0] === dirruns && cache[1];
							diff = cache[0] === dirruns && cache[2];
							node = nodeIndex && parent.childNodes[ nodeIndex ];

							while ( (node = ++nodeIndex && node && node[ dir ] ||

								// Fallback to seeking `elem` from the start
								(diff = nodeIndex = 0) || start.pop()) ) {

								// When found, cache indexes on `parent` and break
								if ( node.nodeType === 1 && ++diff && node === elem ) {
									outerCache[ type ] = [ dirruns, nodeIndex, diff ];
									break;
								}
							}

						// Use previously-cached element index if available
						} else if ( useCache && (cache = (elem[ expando ] || (elem[ expando ] = {}))[ type ]) && cache[0] === dirruns ) {
							diff = cache[1];

						// xml :nth-child(...) or :nth-last-child(...) or :nth(-last)?-of-type(...)
						} else {
							// Use the same loop as above to seek `elem` from the start
							while ( (node = ++nodeIndex && node && node[ dir ] ||
								(diff = nodeIndex = 0) || start.pop()) ) {

								if ( ( ofType ? node.nodeName.toLowerCase() === name : node.nodeType === 1 ) && ++diff ) {
									// Cache the index of each encountered element
									if ( useCache ) {
										(node[ expando ] || (node[ expando ] = {}))[ type ] = [ dirruns, diff ];
									}

									if ( node === elem ) {
										break;
									}
								}
							}
						}

						// Incorporate the offset, then check against cycle size
						diff -= last;
						return diff === first || ( diff % first === 0 && diff / first >= 0 );
					}
				};
		},

		"PSEUDO": function( pseudo, argument ) {
			// pseudo-class names are case-insensitive
			// http://www.w3.org/TR/selectors/#pseudo-classes
			// Prioritize by case sensitivity in case custom pseudos are added with uppercase letters
			// Remember that setFilters inherits from pseudos
			var args,
				fn = Expr.pseudos[ pseudo ] || Expr.setFilters[ pseudo.toLowerCase() ] ||
					Sizzle.error( "unsupported pseudo: " + pseudo );

			// The user may use createPseudo to indicate that
			// arguments are needed to create the filter function
			// just as Sizzle does
			if ( fn[ expando ] ) {
				return fn( argument );
			}

			// But maintain support for old signatures
			if ( fn.length > 1 ) {
				args = [ pseudo, pseudo, "", argument ];
				return Expr.setFilters.hasOwnProperty( pseudo.toLowerCase() ) ?
					markFunction(function( seed, matches ) {
						var idx,
							matched = fn( seed, argument ),
							i = matched.length;
						while ( i-- ) {
							idx = indexOf( seed, matched[i] );
							seed[ idx ] = !( matches[ idx ] = matched[i] );
						}
					}) :
					function( elem ) {
						return fn( elem, 0, args );
					};
			}

			return fn;
		}
	},

	pseudos: {
		// Potentially complex pseudos
		"not": markFunction(function( selector ) {
			// Trim the selector passed to compile
			// to avoid treating leading and trailing
			// spaces as combinators
			var input = [],
				results = [],
				matcher = compile( selector.replace( rtrim, "$1" ) );

			return matcher[ expando ] ?
				markFunction(function( seed, matches, context, xml ) {
					var elem,
						unmatched = matcher( seed, null, xml, [] ),
						i = seed.length;

					// Match elements unmatched by `matcher`
					while ( i-- ) {
						if ( (elem = unmatched[i]) ) {
							seed[i] = !(matches[i] = elem);
						}
					}
				}) :
				function( elem, context, xml ) {
					input[0] = elem;
					matcher( input, null, xml, results );
					// Don't keep the element (issue #299)
					input[0] = null;
					return !results.pop();
				};
		}),

		"has": markFunction(function( selector ) {
			return function( elem ) {
				return Sizzle( selector, elem ).length > 0;
			};
		}),

		"contains": markFunction(function( text ) {
			text = text.replace( runescape, funescape );
			return function( elem ) {
				return ( elem.textContent || elem.innerText || getText( elem ) ).indexOf( text ) > -1;
			};
		}),

		// "Whether an element is represented by a :lang() selector
		// is based solely on the element's language value
		// being equal to the identifier C,
		// or beginning with the identifier C immediately followed by "-".
		// The matching of C against the element's language value is performed case-insensitively.
		// The identifier C does not have to be a valid language name."
		// http://www.w3.org/TR/selectors/#lang-pseudo
		"lang": markFunction( function( lang ) {
			// lang value must be a valid identifier
			if ( !ridentifier.test(lang || "") ) {
				Sizzle.error( "unsupported lang: " + lang );
			}
			lang = lang.replace( runescape, funescape ).toLowerCase();
			return function( elem ) {
				var elemLang;
				do {
					if ( (elemLang = documentIsHTML ?
						elem.lang :
						elem.getAttribute("xml:lang") || elem.getAttribute("lang")) ) {

						elemLang = elemLang.toLowerCase();
						return elemLang === lang || elemLang.indexOf( lang + "-" ) === 0;
					}
				} while ( (elem = elem.parentNode) && elem.nodeType === 1 );
				return false;
			};
		}),

		// Miscellaneous
		"target": function( elem ) {
			var hash = window.location && window.location.hash;
			return hash && hash.slice( 1 ) === elem.id;
		},

		"root": function( elem ) {
			return elem === docElem;
		},

		"focus": function( elem ) {
			return elem === document.activeElement && (!document.hasFocus || document.hasFocus()) && !!(elem.type || elem.href || ~elem.tabIndex);
		},

		// Boolean properties
		"enabled": function( elem ) {
			return elem.disabled === false;
		},

		"disabled": function( elem ) {
			return elem.disabled === true;
		},

		"checked": function( elem ) {
			// In CSS3, :checked should return both checked and selected elements
			// http://www.w3.org/TR/2011/REC-css3-selectors-20110929/#checked
			var nodeName = elem.nodeName.toLowerCase();
			return (nodeName === "input" && !!elem.checked) || (nodeName === "option" && !!elem.selected);
		},

		"selected": function( elem ) {
			// Accessing this property makes selected-by-default
			// options in Safari work properly
			if ( elem.parentNode ) {
				elem.parentNode.selectedIndex;
			}

			return elem.selected === true;
		},

		// Contents
		"empty": function( elem ) {
			// http://www.w3.org/TR/selectors/#empty-pseudo
			// :empty is negated by element (1) or content nodes (text: 3; cdata: 4; entity ref: 5),
			//   but not by others (comment: 8; processing instruction: 7; etc.)
			// nodeType < 6 works because attributes (2) do not appear as children
			for ( elem = elem.firstChild; elem; elem = elem.nextSibling ) {
				if ( elem.nodeType < 6 ) {
					return false;
				}
			}
			return true;
		},

		"parent": function( elem ) {
			return !Expr.pseudos["empty"]( elem );
		},

		// Element/input types
		"header": function( elem ) {
			return rheader.test( elem.nodeName );
		},

		"input": function( elem ) {
			return rinputs.test( elem.nodeName );
		},

		"button": function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return name === "input" && elem.type === "button" || name === "button";
		},

		"text": function( elem ) {
			var attr;
			return elem.nodeName.toLowerCase() === "input" &&
				elem.type === "text" &&

				// Support: IE<8
				// New HTML5 attribute values (e.g., "search") appear with elem.type === "text"
				( (attr = elem.getAttribute("type")) == null || attr.toLowerCase() === "text" );
		},

		// Position-in-collection
		"first": createPositionalPseudo(function() {
			return [ 0 ];
		}),

		"last": createPositionalPseudo(function( matchIndexes, length ) {
			return [ length - 1 ];
		}),

		"eq": createPositionalPseudo(function( matchIndexes, length, argument ) {
			return [ argument < 0 ? argument + length : argument ];
		}),

		"even": createPositionalPseudo(function( matchIndexes, length ) {
			var i = 0;
			for ( ; i < length; i += 2 ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"odd": createPositionalPseudo(function( matchIndexes, length ) {
			var i = 1;
			for ( ; i < length; i += 2 ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"lt": createPositionalPseudo(function( matchIndexes, length, argument ) {
			var i = argument < 0 ? argument + length : argument;
			for ( ; --i >= 0; ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"gt": createPositionalPseudo(function( matchIndexes, length, argument ) {
			var i = argument < 0 ? argument + length : argument;
			for ( ; ++i < length; ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		})
	}
};

Expr.pseudos["nth"] = Expr.pseudos["eq"];

// Add button/input type pseudos
for ( i in { radio: true, checkbox: true, file: true, password: true, image: true } ) {
	Expr.pseudos[ i ] = createInputPseudo( i );
}
for ( i in { submit: true, reset: true } ) {
	Expr.pseudos[ i ] = createButtonPseudo( i );
}

// Easy API for creating new setFilters
function setFilters() {}
setFilters.prototype = Expr.filters = Expr.pseudos;
Expr.setFilters = new setFilters();

tokenize = Sizzle.tokenize = function( selector, parseOnly ) {
	var matched, match, tokens, type,
		soFar, groups, preFilters,
		cached = tokenCache[ selector + " " ];

	if ( cached ) {
		return parseOnly ? 0 : cached.slice( 0 );
	}

	soFar = selector;
	groups = [];
	preFilters = Expr.preFilter;

	while ( soFar ) {

		// Comma and first run
		if ( !matched || (match = rcomma.exec( soFar )) ) {
			if ( match ) {
				// Don't consume trailing commas as valid
				soFar = soFar.slice( match[0].length ) || soFar;
			}
			groups.push( (tokens = []) );
		}

		matched = false;

		// Combinators
		if ( (match = rcombinators.exec( soFar )) ) {
			matched = match.shift();
			tokens.push({
				value: matched,
				// Cast descendant combinators to space
				type: match[0].replace( rtrim, " " )
			});
			soFar = soFar.slice( matched.length );
		}

		// Filters
		for ( type in Expr.filter ) {
			if ( (match = matchExpr[ type ].exec( soFar )) && (!preFilters[ type ] ||
				(match = preFilters[ type ]( match ))) ) {
				matched = match.shift();
				tokens.push({
					value: matched,
					type: type,
					matches: match
				});
				soFar = soFar.slice( matched.length );
			}
		}

		if ( !matched ) {
			break;
		}
	}

	// Return the length of the invalid excess
	// if we're just parsing
	// Otherwise, throw an error or return tokens
	return parseOnly ?
		soFar.length :
		soFar ?
			Sizzle.error( selector ) :
			// Cache the tokens
			tokenCache( selector, groups ).slice( 0 );
};

function toSelector( tokens ) {
	var i = 0,
		len = tokens.length,
		selector = "";
	for ( ; i < len; i++ ) {
		selector += tokens[i].value;
	}
	return selector;
}

function addCombinator( matcher, combinator, base ) {
	var dir = combinator.dir,
		checkNonElements = base && dir === "parentNode",
		doneName = done++;

	return combinator.first ?
		// Check against closest ancestor/preceding element
		function( elem, context, xml ) {
			while ( (elem = elem[ dir ]) ) {
				if ( elem.nodeType === 1 || checkNonElements ) {
					return matcher( elem, context, xml );
				}
			}
		} :

		// Check against all ancestor/preceding elements
		function( elem, context, xml ) {
			var oldCache, outerCache,
				newCache = [ dirruns, doneName ];

			// We can't set arbitrary data on XML nodes, so they don't benefit from dir caching
			if ( xml ) {
				while ( (elem = elem[ dir ]) ) {
					if ( elem.nodeType === 1 || checkNonElements ) {
						if ( matcher( elem, context, xml ) ) {
							return true;
						}
					}
				}
			} else {
				while ( (elem = elem[ dir ]) ) {
					if ( elem.nodeType === 1 || checkNonElements ) {
						outerCache = elem[ expando ] || (elem[ expando ] = {});
						if ( (oldCache = outerCache[ dir ]) &&
							oldCache[ 0 ] === dirruns && oldCache[ 1 ] === doneName ) {

							// Assign to newCache so results back-propagate to previous elements
							return (newCache[ 2 ] = oldCache[ 2 ]);
						} else {
							// Reuse newcache so results back-propagate to previous elements
							outerCache[ dir ] = newCache;

							// A match means we're done; a fail means we have to keep checking
							if ( (newCache[ 2 ] = matcher( elem, context, xml )) ) {
								return true;
							}
						}
					}
				}
			}
		};
}

function elementMatcher( matchers ) {
	return matchers.length > 1 ?
		function( elem, context, xml ) {
			var i = matchers.length;
			while ( i-- ) {
				if ( !matchers[i]( elem, context, xml ) ) {
					return false;
				}
			}
			return true;
		} :
		matchers[0];
}

function multipleContexts( selector, contexts, results ) {
	var i = 0,
		len = contexts.length;
	for ( ; i < len; i++ ) {
		Sizzle( selector, contexts[i], results );
	}
	return results;
}

function condense( unmatched, map, filter, context, xml ) {
	var elem,
		newUnmatched = [],
		i = 0,
		len = unmatched.length,
		mapped = map != null;

	for ( ; i < len; i++ ) {
		if ( (elem = unmatched[i]) ) {
			if ( !filter || filter( elem, context, xml ) ) {
				newUnmatched.push( elem );
				if ( mapped ) {
					map.push( i );
				}
			}
		}
	}

	return newUnmatched;
}

function setMatcher( preFilter, selector, matcher, postFilter, postFinder, postSelector ) {
	if ( postFilter && !postFilter[ expando ] ) {
		postFilter = setMatcher( postFilter );
	}
	if ( postFinder && !postFinder[ expando ] ) {
		postFinder = setMatcher( postFinder, postSelector );
	}
	return markFunction(function( seed, results, context, xml ) {
		var temp, i, elem,
			preMap = [],
			postMap = [],
			preexisting = results.length,

			// Get initial elements from seed or context
			elems = seed || multipleContexts( selector || "*", context.nodeType ? [ context ] : context, [] ),

			// Prefilter to get matcher input, preserving a map for seed-results synchronization
			matcherIn = preFilter && ( seed || !selector ) ?
				condense( elems, preMap, preFilter, context, xml ) :
				elems,

			matcherOut = matcher ?
				// If we have a postFinder, or filtered seed, or non-seed postFilter or preexisting results,
				postFinder || ( seed ? preFilter : preexisting || postFilter ) ?

					// ...intermediate processing is necessary
					[] :

					// ...otherwise use results directly
					results :
				matcherIn;

		// Find primary matches
		if ( matcher ) {
			matcher( matcherIn, matcherOut, context, xml );
		}

		// Apply postFilter
		if ( postFilter ) {
			temp = condense( matcherOut, postMap );
			postFilter( temp, [], context, xml );

			// Un-match failing elements by moving them back to matcherIn
			i = temp.length;
			while ( i-- ) {
				if ( (elem = temp[i]) ) {
					matcherOut[ postMap[i] ] = !(matcherIn[ postMap[i] ] = elem);
				}
			}
		}

		if ( seed ) {
			if ( postFinder || preFilter ) {
				if ( postFinder ) {
					// Get the final matcherOut by condensing this intermediate into postFinder contexts
					temp = [];
					i = matcherOut.length;
					while ( i-- ) {
						if ( (elem = matcherOut[i]) ) {
							// Restore matcherIn since elem is not yet a final match
							temp.push( (matcherIn[i] = elem) );
						}
					}
					postFinder( null, (matcherOut = []), temp, xml );
				}

				// Move matched elements from seed to results to keep them synchronized
				i = matcherOut.length;
				while ( i-- ) {
					if ( (elem = matcherOut[i]) &&
						(temp = postFinder ? indexOf( seed, elem ) : preMap[i]) > -1 ) {

						seed[temp] = !(results[temp] = elem);
					}
				}
			}

		// Add elements to results, through postFinder if defined
		} else {
			matcherOut = condense(
				matcherOut === results ?
					matcherOut.splice( preexisting, matcherOut.length ) :
					matcherOut
			);
			if ( postFinder ) {
				postFinder( null, results, matcherOut, xml );
			} else {
				push.apply( results, matcherOut );
			}
		}
	});
}

function matcherFromTokens( tokens ) {
	var checkContext, matcher, j,
		len = tokens.length,
		leadingRelative = Expr.relative[ tokens[0].type ],
		implicitRelative = leadingRelative || Expr.relative[" "],
		i = leadingRelative ? 1 : 0,

		// The foundational matcher ensures that elements are reachable from top-level context(s)
		matchContext = addCombinator( function( elem ) {
			return elem === checkContext;
		}, implicitRelative, true ),
		matchAnyContext = addCombinator( function( elem ) {
			return indexOf( checkContext, elem ) > -1;
		}, implicitRelative, true ),
		matchers = [ function( elem, context, xml ) {
			var ret = ( !leadingRelative && ( xml || context !== outermostContext ) ) || (
				(checkContext = context).nodeType ?
					matchContext( elem, context, xml ) :
					matchAnyContext( elem, context, xml ) );
			// Avoid hanging onto element (issue #299)
			checkContext = null;
			return ret;
		} ];

	for ( ; i < len; i++ ) {
		if ( (matcher = Expr.relative[ tokens[i].type ]) ) {
			matchers = [ addCombinator(elementMatcher( matchers ), matcher) ];
		} else {
			matcher = Expr.filter[ tokens[i].type ].apply( null, tokens[i].matches );

			// Return special upon seeing a positional matcher
			if ( matcher[ expando ] ) {
				// Find the next relative operator (if any) for proper handling
				j = ++i;
				for ( ; j < len; j++ ) {
					if ( Expr.relative[ tokens[j].type ] ) {
						break;
					}
				}
				return setMatcher(
					i > 1 && elementMatcher( matchers ),
					i > 1 && toSelector(
						// If the preceding token was a descendant combinator, insert an implicit any-element `*`
						tokens.slice( 0, i - 1 ).concat({ value: tokens[ i - 2 ].type === " " ? "*" : "" })
					).replace( rtrim, "$1" ),
					matcher,
					i < j && matcherFromTokens( tokens.slice( i, j ) ),
					j < len && matcherFromTokens( (tokens = tokens.slice( j )) ),
					j < len && toSelector( tokens )
				);
			}
			matchers.push( matcher );
		}
	}

	return elementMatcher( matchers );
}

function matcherFromGroupMatchers( elementMatchers, setMatchers ) {
	var bySet = setMatchers.length > 0,
		byElement = elementMatchers.length > 0,
		superMatcher = function( seed, context, xml, results, outermost ) {
			var elem, j, matcher,
				matchedCount = 0,
				i = "0",
				unmatched = seed && [],
				setMatched = [],
				contextBackup = outermostContext,
				// We must always have either seed elements or outermost context
				elems = seed || byElement && Expr.find["TAG"]( "*", outermost ),
				// Use integer dirruns iff this is the outermost matcher
				dirrunsUnique = (dirruns += contextBackup == null ? 1 : Math.random() || 0.1),
				len = elems.length;

			if ( outermost ) {
				outermostContext = context !== document && context;
			}

			// Add elements passing elementMatchers directly to results
			// Keep `i` a string if there are no elements so `matchedCount` will be "00" below
			// Support: IE<9, Safari
			// Tolerate NodeList properties (IE: "length"; Safari: <number>) matching elements by id
			for ( ; i !== len && (elem = elems[i]) != null; i++ ) {
				if ( byElement && elem ) {
					j = 0;
					while ( (matcher = elementMatchers[j++]) ) {
						if ( matcher( elem, context, xml ) ) {
							results.push( elem );
							break;
						}
					}
					if ( outermost ) {
						dirruns = dirrunsUnique;
					}
				}

				// Track unmatched elements for set filters
				if ( bySet ) {
					// They will have gone through all possible matchers
					if ( (elem = !matcher && elem) ) {
						matchedCount--;
					}

					// Lengthen the array for every element, matched or not
					if ( seed ) {
						unmatched.push( elem );
					}
				}
			}

			// Apply set filters to unmatched elements
			matchedCount += i;
			if ( bySet && i !== matchedCount ) {
				j = 0;
				while ( (matcher = setMatchers[j++]) ) {
					matcher( unmatched, setMatched, context, xml );
				}

				if ( seed ) {
					// Reintegrate element matches to eliminate the need for sorting
					if ( matchedCount > 0 ) {
						while ( i-- ) {
							if ( !(unmatched[i] || setMatched[i]) ) {
								setMatched[i] = pop.call( results );
							}
						}
					}

					// Discard index placeholder values to get only actual matches
					setMatched = condense( setMatched );
				}

				// Add matches to results
				push.apply( results, setMatched );

				// Seedless set matches succeeding multiple successful matchers stipulate sorting
				if ( outermost && !seed && setMatched.length > 0 &&
					( matchedCount + setMatchers.length ) > 1 ) {

					Sizzle.uniqueSort( results );
				}
			}

			// Override manipulation of globals by nested matchers
			if ( outermost ) {
				dirruns = dirrunsUnique;
				outermostContext = contextBackup;
			}

			return unmatched;
		};

	return bySet ?
		markFunction( superMatcher ) :
		superMatcher;
}

compile = Sizzle.compile = function( selector, match /* Internal Use Only */ ) {
	var i,
		setMatchers = [],
		elementMatchers = [],
		cached = compilerCache[ selector + " " ];

	if ( !cached ) {
		// Generate a function of recursive functions that can be used to check each element
		if ( !match ) {
			match = tokenize( selector );
		}
		i = match.length;
		while ( i-- ) {
			cached = matcherFromTokens( match[i] );
			if ( cached[ expando ] ) {
				setMatchers.push( cached );
			} else {
				elementMatchers.push( cached );
			}
		}

		// Cache the compiled function
		cached = compilerCache( selector, matcherFromGroupMatchers( elementMatchers, setMatchers ) );

		// Save selector and tokenization
		cached.selector = selector;
	}
	return cached;
};

/**
 * A low-level selection function that works with Sizzle's compiled
 *  selector functions
 * @param {String|Function} selector A selector or a pre-compiled
 *  selector function built with Sizzle.compile
 * @param {Element} context
 * @param {Array} [results]
 * @param {Array} [seed] A set of elements to match against
 */
select = Sizzle.select = function( selector, context, results, seed ) {
	var i, tokens, token, type, find,
		compiled = typeof selector === "function" && selector,
		match = !seed && tokenize( (selector = compiled.selector || selector) );

	results = results || [];

	// Try to minimize operations if there is no seed and only one group
	if ( match.length === 1 ) {

		// Take a shortcut and set the context if the root selector is an ID
		tokens = match[0] = match[0].slice( 0 );
		if ( tokens.length > 2 && (token = tokens[0]).type === "ID" &&
				support.getById && context.nodeType === 9 && documentIsHTML &&
				Expr.relative[ tokens[1].type ] ) {

			context = ( Expr.find["ID"]( token.matches[0].replace(runescape, funescape), context ) || [] )[0];
			if ( !context ) {
				return results;

			// Precompiled matchers will still verify ancestry, so step up a level
			} else if ( compiled ) {
				context = context.parentNode;
			}

			selector = selector.slice( tokens.shift().value.length );
		}

		// Fetch a seed set for right-to-left matching
		i = matchExpr["needsContext"].test( selector ) ? 0 : tokens.length;
		while ( i-- ) {
			token = tokens[i];

			// Abort if we hit a combinator
			if ( Expr.relative[ (type = token.type) ] ) {
				break;
			}
			if ( (find = Expr.find[ type ]) ) {
				// Search, expanding context for leading sibling combinators
				if ( (seed = find(
					token.matches[0].replace( runescape, funescape ),
					rsibling.test( tokens[0].type ) && testContext( context.parentNode ) || context
				)) ) {

					// If seed is empty or no tokens remain, we can return early
					tokens.splice( i, 1 );
					selector = seed.length && toSelector( tokens );
					if ( !selector ) {
						push.apply( results, seed );
						return results;
					}

					break;
				}
			}
		}
	}

	// Compile and execute a filtering function if one is not provided
	// Provide `match` to avoid retokenization if we modified the selector above
	( compiled || compile( selector, match ) )(
		seed,
		context,
		!documentIsHTML,
		results,
		rsibling.test( selector ) && testContext( context.parentNode ) || context
	);
	return results;
};

// One-time assignments

// Sort stability
support.sortStable = expando.split("").sort( sortOrder ).join("") === expando;

// Support: Chrome 14-35+
// Always assume duplicates if they aren't passed to the comparison function
support.detectDuplicates = !!hasDuplicate;

// Initialize against the default document
setDocument();

// Support: Webkit<537.32 - Safari 6.0.3/Chrome 25 (fixed in Chrome 27)
// Detached nodes confoundingly follow *each other*
support.sortDetached = assert(function( div1 ) {
	// Should return 1, but returns 4 (following)
	return div1.compareDocumentPosition( document.createElement("div") ) & 1;
});

// Support: IE<8
// Prevent attribute/property "interpolation"
// http://msdn.microsoft.com/en-us/library/ms536429%28VS.85%29.aspx
if ( !assert(function( div ) {
	div.innerHTML = "<a href='#'></a>";
	return div.firstChild.getAttribute("href") === "#" ;
}) ) {
	addHandle( "type|href|height|width", function( elem, name, isXML ) {
		if ( !isXML ) {
			return elem.getAttribute( name, name.toLowerCase() === "type" ? 1 : 2 );
		}
	});
}

// Support: IE<9
// Use defaultValue in place of getAttribute("value")
if ( !support.attributes || !assert(function( div ) {
	div.innerHTML = "<input/>";
	div.firstChild.setAttribute( "value", "" );
	return div.firstChild.getAttribute( "value" ) === "";
}) ) {
	addHandle( "value", function( elem, name, isXML ) {
		if ( !isXML && elem.nodeName.toLowerCase() === "input" ) {
			return elem.defaultValue;
		}
	});
}

// Support: IE<9
// Use getAttributeNode to fetch booleans when getAttribute lies
if ( !assert(function( div ) {
	return div.getAttribute("disabled") == null;
}) ) {
	addHandle( booleans, function( elem, name, isXML ) {
		var val;
		if ( !isXML ) {
			return elem[ name ] === true ? name.toLowerCase() :
					(val = elem.getAttributeNode( name )) && val.specified ?
					val.value :
				null;
		}
	});
}

return Sizzle;

})( window );



jQuery.find = Sizzle;
jQuery.expr = Sizzle.selectors;
jQuery.expr[":"] = jQuery.expr.pseudos;
jQuery.unique = Sizzle.uniqueSort;
jQuery.text = Sizzle.getText;
jQuery.isXMLDoc = Sizzle.isXML;
jQuery.contains = Sizzle.contains;



var rneedsContext = jQuery.expr.match.needsContext;

var rsingleTag = (/^<(\w+)\s*\/?>(?:<\/\1>|)$/);



var risSimple = /^.[^:#\[\.,]*$/;

// Implement the identical functionality for filter and not
function winnow( elements, qualifier, not ) {
	if ( jQuery.isFunction( qualifier ) ) {
		return jQuery.grep( elements, function( elem, i ) {
			/* jshint -W018 */
			return !!qualifier.call( elem, i, elem ) !== not;
		});

	}

	if ( qualifier.nodeType ) {
		return jQuery.grep( elements, function( elem ) {
			return ( elem === qualifier ) !== not;
		});

	}

	if ( typeof qualifier === "string" ) {
		if ( risSimple.test( qualifier ) ) {
			return jQuery.filter( qualifier, elements, not );
		}

		qualifier = jQuery.filter( qualifier, elements );
	}

	return jQuery.grep( elements, function( elem ) {
		return ( indexOf.call( qualifier, elem ) >= 0 ) !== not;
	});
}

jQuery.filter = function( expr, elems, not ) {
	var elem = elems[ 0 ];

	if ( not ) {
		expr = ":not(" + expr + ")";
	}

	return elems.length === 1 && elem.nodeType === 1 ?
		jQuery.find.matchesSelector( elem, expr ) ? [ elem ] : [] :
		jQuery.find.matches( expr, jQuery.grep( elems, function( elem ) {
			return elem.nodeType === 1;
		}));
};

jQuery.fn.extend({
	find: function( selector ) {
		var i,
			len = this.length,
			ret = [],
			self = this;

		if ( typeof selector !== "string" ) {
			return this.pushStack( jQuery( selector ).filter(function() {
				for ( i = 0; i < len; i++ ) {
					if ( jQuery.contains( self[ i ], this ) ) {
						return true;
					}
				}
			}) );
		}

		for ( i = 0; i < len; i++ ) {
			jQuery.find( selector, self[ i ], ret );
		}

		// Needed because $( selector, context ) becomes $( context ).find( selector )
		ret = this.pushStack( len > 1 ? jQuery.unique( ret ) : ret );
		ret.selector = this.selector ? this.selector + " " + selector : selector;
		return ret;
	},
	filter: function( selector ) {
		return this.pushStack( winnow(this, selector || [], false) );
	},
	not: function( selector ) {
		return this.pushStack( winnow(this, selector || [], true) );
	},
	is: function( selector ) {
		return !!winnow(
			this,

			// If this is a positional/relative selector, check membership in the returned set
			// so $("p:first").is("p:last") won't return true for a doc with two "p".
			typeof selector === "string" && rneedsContext.test( selector ) ?
				jQuery( selector ) :
				selector || [],
			false
		).length;
	}
});


// Initialize a jQuery object


// A central reference to the root jQuery(document)
var rootjQuery,

	// A simple way to check for HTML strings
	// Prioritize #id over <tag> to avoid XSS via location.hash (#9521)
	// Strict HTML recognition (#11290: must start with <)
	rquickExpr = /^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]*))$/,

	init = jQuery.fn.init = function( selector, context ) {
		var match, elem;

		// HANDLE: $(""), $(null), $(undefined), $(false)
		if ( !selector ) {
			return this;
		}

		// Handle HTML strings
		if ( typeof selector === "string" ) {
			if ( selector[0] === "<" && selector[ selector.length - 1 ] === ">" && selector.length >= 3 ) {
				// Assume that strings that start and end with <> are HTML and skip the regex check
				match = [ null, selector, null ];

			} else {
				match = rquickExpr.exec( selector );
			}

			// Match html or make sure no context is specified for #id
			if ( match && (match[1] || !context) ) {

				// HANDLE: $(html) -> $(array)
				if ( match[1] ) {
					context = context instanceof jQuery ? context[0] : context;

					// Option to run scripts is true for back-compat
					// Intentionally let the error be thrown if parseHTML is not present
					jQuery.merge( this, jQuery.parseHTML(
						match[1],
						context && context.nodeType ? context.ownerDocument || context : document,
						true
					) );

					// HANDLE: $(html, props)
					if ( rsingleTag.test( match[1] ) && jQuery.isPlainObject( context ) ) {
						for ( match in context ) {
							// Properties of context are called as methods if possible
							if ( jQuery.isFunction( this[ match ] ) ) {
								this[ match ]( context[ match ] );

							// ...and otherwise set as attributes
							} else {
								this.attr( match, context[ match ] );
							}
						}
					}

					return this;

				// HANDLE: $(#id)
				} else {
					elem = document.getElementById( match[2] );

					// Support: Blackberry 4.6
					// gEBID returns nodes no longer in the document (#6963)
					if ( elem && elem.parentNode ) {
						// Inject the element directly into the jQuery object
						this.length = 1;
						this[0] = elem;
					}

					this.context = document;
					this.selector = selector;
					return this;
				}

			// HANDLE: $(expr, $(...))
			} else if ( !context || context.jquery ) {
				return ( context || rootjQuery ).find( selector );

			// HANDLE: $(expr, context)
			// (which is just equivalent to: $(context).find(expr)
			} else {
				return this.constructor( context ).find( selector );
			}

		// HANDLE: $(DOMElement)
		} else if ( selector.nodeType ) {
			this.context = this[0] = selector;
			this.length = 1;
			return this;

		// HANDLE: $(function)
		// Shortcut for document ready
		} else if ( jQuery.isFunction( selector ) ) {
			return typeof rootjQuery.ready !== "undefined" ?
				rootjQuery.ready( selector ) :
				// Execute immediately if ready is not present
				selector( jQuery );
		}

		if ( selector.selector !== undefined ) {
			this.selector = selector.selector;
			this.context = selector.context;
		}

		return jQuery.makeArray( selector, this );
	};

// Give the init function the jQuery prototype for later instantiation
init.prototype = jQuery.fn;

// Initialize central reference
rootjQuery = jQuery( document );


var rparentsprev = /^(?:parents|prev(?:Until|All))/,
	// Methods guaranteed to produce a unique set when starting from a unique set
	guaranteedUnique = {
		children: true,
		contents: true,
		next: true,
		prev: true
	};

jQuery.extend({
	dir: function( elem, dir, until ) {
		var matched = [],
			truncate = until !== undefined;

		while ( (elem = elem[ dir ]) && elem.nodeType !== 9 ) {
			if ( elem.nodeType === 1 ) {
				if ( truncate && jQuery( elem ).is( until ) ) {
					break;
				}
				matched.push( elem );
			}
		}
		return matched;
	},

	sibling: function( n, elem ) {
		var matched = [];

		for ( ; n; n = n.nextSibling ) {
			if ( n.nodeType === 1 && n !== elem ) {
				matched.push( n );
			}
		}

		return matched;
	}
});

jQuery.fn.extend({
	has: function( target ) {
		var targets = jQuery( target, this ),
			l = targets.length;

		return this.filter(function() {
			var i = 0;
			for ( ; i < l; i++ ) {
				if ( jQuery.contains( this, targets[i] ) ) {
					return true;
				}
			}
		});
	},

	closest: function( selectors, context ) {
		var cur,
			i = 0,
			l = this.length,
			matched = [],
			pos = rneedsContext.test( selectors ) || typeof selectors !== "string" ?
				jQuery( selectors, context || this.context ) :
				0;

		for ( ; i < l; i++ ) {
			for ( cur = this[i]; cur && cur !== context; cur = cur.parentNode ) {
				// Always skip document fragments
				if ( cur.nodeType < 11 && (pos ?
					pos.index(cur) > -1 :

					// Don't pass non-elements to Sizzle
					cur.nodeType === 1 &&
						jQuery.find.matchesSelector(cur, selectors)) ) {

					matched.push( cur );
					break;
				}
			}
		}

		return this.pushStack( matched.length > 1 ? jQuery.unique( matched ) : matched );
	},

	// Determine the position of an element within the set
	index: function( elem ) {

		// No argument, return index in parent
		if ( !elem ) {
			return ( this[ 0 ] && this[ 0 ].parentNode ) ? this.first().prevAll().length : -1;
		}

		// Index in selector
		if ( typeof elem === "string" ) {
			return indexOf.call( jQuery( elem ), this[ 0 ] );
		}

		// Locate the position of the desired element
		return indexOf.call( this,

			// If it receives a jQuery object, the first element is used
			elem.jquery ? elem[ 0 ] : elem
		);
	},

	add: function( selector, context ) {
		return this.pushStack(
			jQuery.unique(
				jQuery.merge( this.get(), jQuery( selector, context ) )
			)
		);
	},

	addBack: function( selector ) {
		return this.add( selector == null ?
			this.prevObject : this.prevObject.filter(selector)
		);
	}
});

function sibling( cur, dir ) {
	while ( (cur = cur[dir]) && cur.nodeType !== 1 ) {}
	return cur;
}

jQuery.each({
	parent: function( elem ) {
		var parent = elem.parentNode;
		return parent && parent.nodeType !== 11 ? parent : null;
	},
	parents: function( elem ) {
		return jQuery.dir( elem, "parentNode" );
	},
	parentsUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "parentNode", until );
	},
	next: function( elem ) {
		return sibling( elem, "nextSibling" );
	},
	prev: function( elem ) {
		return sibling( elem, "previousSibling" );
	},
	nextAll: function( elem ) {
		return jQuery.dir( elem, "nextSibling" );
	},
	prevAll: function( elem ) {
		return jQuery.dir( elem, "previousSibling" );
	},
	nextUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "nextSibling", until );
	},
	prevUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "previousSibling", until );
	},
	siblings: function( elem ) {
		return jQuery.sibling( ( elem.parentNode || {} ).firstChild, elem );
	},
	children: function( elem ) {
		return jQuery.sibling( elem.firstChild );
	},
	contents: function( elem ) {
		return elem.contentDocument || jQuery.merge( [], elem.childNodes );
	}
}, function( name, fn ) {
	jQuery.fn[ name ] = function( until, selector ) {
		var matched = jQuery.map( this, fn, until );

		if ( name.slice( -5 ) !== "Until" ) {
			selector = until;
		}

		if ( selector && typeof selector === "string" ) {
			matched = jQuery.filter( selector, matched );
		}

		if ( this.length > 1 ) {
			// Remove duplicates
			if ( !guaranteedUnique[ name ] ) {
				jQuery.unique( matched );
			}

			// Reverse order for parents* and prev-derivatives
			if ( rparentsprev.test( name ) ) {
				matched.reverse();
			}
		}

		return this.pushStack( matched );
	};
});
var rnotwhite = (/\S+/g);



// String to Object options format cache
var optionsCache = {};

// Convert String-formatted options into Object-formatted ones and store in cache
function createOptions( options ) {
	var object = optionsCache[ options ] = {};
	jQuery.each( options.match( rnotwhite ) || [], function( _, flag ) {
		object[ flag ] = true;
	});
	return object;
}

/*
 * Create a callback list using the following parameters:
 *
 *	options: an optional list of space-separated options that will change how
 *			the callback list behaves or a more traditional option object
 *
 * By default a callback list will act like an event callback list and can be
 * "fired" multiple times.
 *
 * Possible options:
 *
 *	once:			will ensure the callback list can only be fired once (like a Deferred)
 *
 *	memory:			will keep track of previous values and will call any callback added
 *					after the list has been fired right away with the latest "memorized"
 *					values (like a Deferred)
 *
 *	unique:			will ensure a callback can only be added once (no duplicate in the list)
 *
 *	stopOnFalse:	interrupt callings when a callback returns false
 *
 */
jQuery.Callbacks = function( options ) {

	// Convert options from String-formatted to Object-formatted if needed
	// (we check in cache first)
	options = typeof options === "string" ?
		( optionsCache[ options ] || createOptions( options ) ) :
		jQuery.extend( {}, options );

	var // Last fire value (for non-forgettable lists)
		memory,
		// Flag to know if list was already fired
		fired,
		// Flag to know if list is currently firing
		firing,
		// First callback to fire (used internally by add and fireWith)
		firingStart,
		// End of the loop when firing
		firingLength,
		// Index of currently firing callback (modified by remove if needed)
		firingIndex,
		// Actual callback list
		list = [],
		// Stack of fire calls for repeatable lists
		stack = !options.once && [],
		// Fire callbacks
		fire = function( data ) {
			memory = options.memory && data;
			fired = true;
			firingIndex = firingStart || 0;
			firingStart = 0;
			firingLength = list.length;
			firing = true;
			for ( ; list && firingIndex < firingLength; firingIndex++ ) {
				if ( list[ firingIndex ].apply( data[ 0 ], data[ 1 ] ) === false && options.stopOnFalse ) {
					memory = false; // To prevent further calls using add
					break;
				}
			}
			firing = false;
			if ( list ) {
				if ( stack ) {
					if ( stack.length ) {
						fire( stack.shift() );
					}
				} else if ( memory ) {
					list = [];
				} else {
					self.disable();
				}
			}
		},
		// Actual Callbacks object
		self = {
			// Add a callback or a collection of callbacks to the list
			add: function() {
				if ( list ) {
					// First, we save the current length
					var start = list.length;
					(function add( args ) {
						jQuery.each( args, function( _, arg ) {
							var type = jQuery.type( arg );
							if ( type === "function" ) {
								if ( !options.unique || !self.has( arg ) ) {
									list.push( arg );
								}
							} else if ( arg && arg.length && type !== "string" ) {
								// Inspect recursively
								add( arg );
							}
						});
					})( arguments );
					// Do we need to add the callbacks to the
					// current firing batch?
					if ( firing ) {
						firingLength = list.length;
					// With memory, if we're not firing then
					// we should call right away
					} else if ( memory ) {
						firingStart = start;
						fire( memory );
					}
				}
				return this;
			},
			// Remove a callback from the list
			remove: function() {
				if ( list ) {
					jQuery.each( arguments, function( _, arg ) {
						var index;
						while ( ( index = jQuery.inArray( arg, list, index ) ) > -1 ) {
							list.splice( index, 1 );
							// Handle firing indexes
							if ( firing ) {
								if ( index <= firingLength ) {
									firingLength--;
								}
								if ( index <= firingIndex ) {
									firingIndex--;
								}
							}
						}
					});
				}
				return this;
			},
			// Check if a given callback is in the list.
			// If no argument is given, return whether or not list has callbacks attached.
			has: function( fn ) {
				return fn ? jQuery.inArray( fn, list ) > -1 : !!( list && list.length );
			},
			// Remove all callbacks from the list
			empty: function() {
				list = [];
				firingLength = 0;
				return this;
			},
			// Have the list do nothing anymore
			disable: function() {
				list = stack = memory = undefined;
				return this;
			},
			// Is it disabled?
			disabled: function() {
				return !list;
			},
			// Lock the list in its current state
			lock: function() {
				stack = undefined;
				if ( !memory ) {
					self.disable();
				}
				return this;
			},
			// Is it locked?
			locked: function() {
				return !stack;
			},
			// Call all callbacks with the given context and arguments
			fireWith: function( context, args ) {
				if ( list && ( !fired || stack ) ) {
					args = args || [];
					args = [ context, args.slice ? args.slice() : args ];
					if ( firing ) {
						stack.push( args );
					} else {
						fire( args );
					}
				}
				return this;
			},
			// Call all the callbacks with the given arguments
			fire: function() {
				self.fireWith( this, arguments );
				return this;
			},
			// To know if the callbacks have already been called at least once
			fired: function() {
				return !!fired;
			}
		};

	return self;
};


jQuery.extend({

	Deferred: function( func ) {
		var tuples = [
				// action, add listener, listener list, final state
				[ "resolve", "done", jQuery.Callbacks("once memory"), "resolved" ],
				[ "reject", "fail", jQuery.Callbacks("once memory"), "rejected" ],
				[ "notify", "progress", jQuery.Callbacks("memory") ]
			],
			state = "pending",
			promise = {
				state: function() {
					return state;
				},
				always: function() {
					deferred.done( arguments ).fail( arguments );
					return this;
				},
				then: function( /* fnDone, fnFail, fnProgress */ ) {
					var fns = arguments;
					return jQuery.Deferred(function( newDefer ) {
						jQuery.each( tuples, function( i, tuple ) {
							var fn = jQuery.isFunction( fns[ i ] ) && fns[ i ];
							// deferred[ done | fail | progress ] for forwarding actions to newDefer
							deferred[ tuple[1] ](function() {
								var returned = fn && fn.apply( this, arguments );
								if ( returned && jQuery.isFunction( returned.promise ) ) {
									returned.promise()
										.done( newDefer.resolve )
										.fail( newDefer.reject )
										.progress( newDefer.notify );
								} else {
									newDefer[ tuple[ 0 ] + "With" ]( this === promise ? newDefer.promise() : this, fn ? [ returned ] : arguments );
								}
							});
						});
						fns = null;
					}).promise();
				},
				// Get a promise for this deferred
				// If obj is provided, the promise aspect is added to the object
				promise: function( obj ) {
					return obj != null ? jQuery.extend( obj, promise ) : promise;
				}
			},
			deferred = {};

		// Keep pipe for back-compat
		promise.pipe = promise.then;

		// Add list-specific methods
		jQuery.each( tuples, function( i, tuple ) {
			var list = tuple[ 2 ],
				stateString = tuple[ 3 ];

			// promise[ done | fail | progress ] = list.add
			promise[ tuple[1] ] = list.add;

			// Handle state
			if ( stateString ) {
				list.add(function() {
					// state = [ resolved | rejected ]
					state = stateString;

				// [ reject_list | resolve_list ].disable; progress_list.lock
				}, tuples[ i ^ 1 ][ 2 ].disable, tuples[ 2 ][ 2 ].lock );
			}

			// deferred[ resolve | reject | notify ]
			deferred[ tuple[0] ] = function() {
				deferred[ tuple[0] + "With" ]( this === deferred ? promise : this, arguments );
				return this;
			};
			deferred[ tuple[0] + "With" ] = list.fireWith;
		});

		// Make the deferred a promise
		promise.promise( deferred );

		// Call given func if any
		if ( func ) {
			func.call( deferred, deferred );
		}

		// All done!
		return deferred;
	},

	// Deferred helper
	when: function( subordinate /* , ..., subordinateN */ ) {
		var i = 0,
			resolveValues = slice.call( arguments ),
			length = resolveValues.length,

			// the count of uncompleted subordinates
			remaining = length !== 1 || ( subordinate && jQuery.isFunction( subordinate.promise ) ) ? length : 0,

			// the master Deferred. If resolveValues consist of only a single Deferred, just use that.
			deferred = remaining === 1 ? subordinate : jQuery.Deferred(),

			// Update function for both resolve and progress values
			updateFunc = function( i, contexts, values ) {
				return function( value ) {
					contexts[ i ] = this;
					values[ i ] = arguments.length > 1 ? slice.call( arguments ) : value;
					if ( values === progressValues ) {
						deferred.notifyWith( contexts, values );
					} else if ( !( --remaining ) ) {
						deferred.resolveWith( contexts, values );
					}
				};
			},

			progressValues, progressContexts, resolveContexts;

		// Add listeners to Deferred subordinates; treat others as resolved
		if ( length > 1 ) {
			progressValues = new Array( length );
			progressContexts = new Array( length );
			resolveContexts = new Array( length );
			for ( ; i < length; i++ ) {
				if ( resolveValues[ i ] && jQuery.isFunction( resolveValues[ i ].promise ) ) {
					resolveValues[ i ].promise()
						.done( updateFunc( i, resolveContexts, resolveValues ) )
						.fail( deferred.reject )
						.progress( updateFunc( i, progressContexts, progressValues ) );
				} else {
					--remaining;
				}
			}
		}

		// If we're not waiting on anything, resolve the master
		if ( !remaining ) {
			deferred.resolveWith( resolveContexts, resolveValues );
		}

		return deferred.promise();
	}
});


// The deferred used on DOM ready
var readyList;

jQuery.fn.ready = function( fn ) {
	// Add the callback
	jQuery.ready.promise().done( fn );

	return this;
};

jQuery.extend({
	// Is the DOM ready to be used? Set to true once it occurs.
	isReady: false,

	// A counter to track how many items to wait for before
	// the ready event fires. See #6781
	readyWait: 1,

	// Hold (or release) the ready event
	holdReady: function( hold ) {
		if ( hold ) {
			jQuery.readyWait++;
		} else {
			jQuery.ready( true );
		}
	},

	// Handle when the DOM is ready
	ready: function( wait ) {

		// Abort if there are pending holds or we're already ready
		if ( wait === true ? --jQuery.readyWait : jQuery.isReady ) {
			return;
		}

		// Remember that the DOM is ready
		jQuery.isReady = true;

		// If a normal DOM Ready event fired, decrement, and wait if need be
		if ( wait !== true && --jQuery.readyWait > 0 ) {
			return;
		}

		// If there are functions bound, to execute
		readyList.resolveWith( document, [ jQuery ] );

		// Trigger any bound ready events
		if ( jQuery.fn.triggerHandler ) {
			jQuery( document ).triggerHandler( "ready" );
			jQuery( document ).off( "ready" );
		}
	}
});

/**
 * The ready event handler and self cleanup method
 */
function completed() {
	document.removeEventListener( "DOMContentLoaded", completed, false );
	window.removeEventListener( "load", completed, false );
	jQuery.ready();
}

jQuery.ready.promise = function( obj ) {
	if ( !readyList ) {

		readyList = jQuery.Deferred();

		// Catch cases where $(document).ready() is called after the browser event has already occurred.
		// We once tried to use readyState "interactive" here, but it caused issues like the one
		// discovered by ChrisS here: http://bugs.jquery.com/ticket/12282#comment:15
		if ( document.readyState === "complete" ) {
			// Handle it asynchronously to allow scripts the opportunity to delay ready
			setTimeout( jQuery.ready );

		} else {

			// Use the handy event callback
			document.addEventListener( "DOMContentLoaded", completed, false );

			// A fallback to window.onload, that will always work
			window.addEventListener( "load", completed, false );
		}
	}
	return readyList.promise( obj );
};

// Kick off the DOM ready check even if the user does not
jQuery.ready.promise();




// Multifunctional method to get and set values of a collection
// The value/s can optionally be executed if it's a function
var access = jQuery.access = function( elems, fn, key, value, chainable, emptyGet, raw ) {
	var i = 0,
		len = elems.length,
		bulk = key == null;

	// Sets many values
	if ( jQuery.type( key ) === "object" ) {
		chainable = true;
		for ( i in key ) {
			jQuery.access( elems, fn, i, key[i], true, emptyGet, raw );
		}

	// Sets one value
	} else if ( value !== undefined ) {
		chainable = true;

		if ( !jQuery.isFunction( value ) ) {
			raw = true;
		}

		if ( bulk ) {
			// Bulk operations run against the entire set
			if ( raw ) {
				fn.call( elems, value );
				fn = null;

			// ...except when executing function values
			} else {
				bulk = fn;
				fn = function( elem, key, value ) {
					return bulk.call( jQuery( elem ), value );
				};
			}
		}

		if ( fn ) {
			for ( ; i < len; i++ ) {
				fn( elems[i], key, raw ? value : value.call( elems[i], i, fn( elems[i], key ) ) );
			}
		}
	}

	return chainable ?
		elems :

		// Gets
		bulk ?
			fn.call( elems ) :
			len ? fn( elems[0], key ) : emptyGet;
};


/**
 * Determines whether an object can have data
 */
jQuery.acceptData = function( owner ) {
	// Accepts only:
	//  - Node
	//    - Node.ELEMENT_NODE
	//    - Node.DOCUMENT_NODE
	//  - Object
	//    - Any
	/* jshint -W018 */
	return owner.nodeType === 1 || owner.nodeType === 9 || !( +owner.nodeType );
};


function Data() {
	// Support: Android<4,
	// Old WebKit does not have Object.preventExtensions/freeze method,
	// return new empty object instead with no [[set]] accessor
	Object.defineProperty( this.cache = {}, 0, {
		get: function() {
			return {};
		}
	});

	this.expando = jQuery.expando + Data.uid++;
}

Data.uid = 1;
Data.accepts = jQuery.acceptData;

Data.prototype = {
	key: function( owner ) {
		// We can accept data for non-element nodes in modern browsers,
		// but we should not, see #8335.
		// Always return the key for a frozen object.
		if ( !Data.accepts( owner ) ) {
			return 0;
		}

		var descriptor = {},
			// Check if the owner object already has a cache key
			unlock = owner[ this.expando ];

		// If not, create one
		if ( !unlock ) {
			unlock = Data.uid++;

			// Secure it in a non-enumerable, non-writable property
			try {
				descriptor[ this.expando ] = { value: unlock };
				Object.defineProperties( owner, descriptor );

			// Support: Android<4
			// Fallback to a less secure definition
			} catch ( e ) {
				descriptor[ this.expando ] = unlock;
				jQuery.extend( owner, descriptor );
			}
		}

		// Ensure the cache object
		if ( !this.cache[ unlock ] ) {
			this.cache[ unlock ] = {};
		}

		return unlock;
	},
	set: function( owner, data, value ) {
		var prop,
			// There may be an unlock assigned to this node,
			// if there is no entry for this "owner", create one inline
			// and set the unlock as though an owner entry had always existed
			unlock = this.key( owner ),
			cache = this.cache[ unlock ];

		// Handle: [ owner, key, value ] args
		if ( typeof data === "string" ) {
			cache[ data ] = value;

		// Handle: [ owner, { properties } ] args
		} else {
			// Fresh assignments by object are shallow copied
			if ( jQuery.isEmptyObject( cache ) ) {
				jQuery.extend( this.cache[ unlock ], data );
			// Otherwise, copy the properties one-by-one to the cache object
			} else {
				for ( prop in data ) {
					cache[ prop ] = data[ prop ];
				}
			}
		}
		return cache;
	},
	get: function( owner, key ) {
		// Either a valid cache is found, or will be created.
		// New caches will be created and the unlock returned,
		// allowing direct access to the newly created
		// empty data object. A valid owner object must be provided.
		var cache = this.cache[ this.key( owner ) ];

		return key === undefined ?
			cache : cache[ key ];
	},
	access: function( owner, key, value ) {
		var stored;
		// In cases where either:
		//
		//   1. No key was specified
		//   2. A string key was specified, but no value provided
		//
		// Take the "read" path and allow the get method to determine
		// which value to return, respectively either:
		//
		//   1. The entire cache object
		//   2. The data stored at the key
		//
		if ( key === undefined ||
				((key && typeof key === "string") && value === undefined) ) {

			stored = this.get( owner, key );

			return stored !== undefined ?
				stored : this.get( owner, jQuery.camelCase(key) );
		}

		// [*]When the key is not a string, or both a key and value
		// are specified, set or extend (existing objects) with either:
		//
		//   1. An object of properties
		//   2. A key and value
		//
		this.set( owner, key, value );

		// Since the "set" path can have two possible entry points
		// return the expected data based on which path was taken[*]
		return value !== undefined ? value : key;
	},
	remove: function( owner, key ) {
		var i, name, camel,
			unlock = this.key( owner ),
			cache = this.cache[ unlock ];

		if ( key === undefined ) {
			this.cache[ unlock ] = {};

		} else {
			// Support array or space separated string of keys
			if ( jQuery.isArray( key ) ) {
				// If "name" is an array of keys...
				// When data is initially created, via ("key", "val") signature,
				// keys will be converted to camelCase.
				// Since there is no way to tell _how_ a key was added, remove
				// both plain key and camelCase key. #12786
				// This will only penalize the array argument path.
				name = key.concat( key.map( jQuery.camelCase ) );
			} else {
				camel = jQuery.camelCase( key );
				// Try the string as a key before any manipulation
				if ( key in cache ) {
					name = [ key, camel ];
				} else {
					// If a key with the spaces exists, use it.
					// Otherwise, create an array by matching non-whitespace
					name = camel;
					name = name in cache ?
						[ name ] : ( name.match( rnotwhite ) || [] );
				}
			}

			i = name.length;
			while ( i-- ) {
				delete cache[ name[ i ] ];
			}
		}
	},
	hasData: function( owner ) {
		return !jQuery.isEmptyObject(
			this.cache[ owner[ this.expando ] ] || {}
		);
	},
	discard: function( owner ) {
		if ( owner[ this.expando ] ) {
			delete this.cache[ owner[ this.expando ] ];
		}
	}
};
var data_priv = new Data();

var data_user = new Data();



//	Implementation Summary
//
//	1. Enforce API surface and semantic compatibility with 1.9.x branch
//	2. Improve the module's maintainability by reducing the storage
//		paths to a single mechanism.
//	3. Use the same single mechanism to support "private" and "user" data.
//	4. _Never_ expose "private" data to user code (TODO: Drop _data, _removeData)
//	5. Avoid exposing implementation details on user objects (eg. expando properties)
//	6. Provide a clear path for implementation upgrade to WeakMap in 2014

var rbrace = /^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,
	rmultiDash = /([A-Z])/g;

function dataAttr( elem, key, data ) {
	var name;

	// If nothing was found internally, try to fetch any
	// data from the HTML5 data-* attribute
	if ( data === undefined && elem.nodeType === 1 ) {
		name = "data-" + key.replace( rmultiDash, "-$1" ).toLowerCase();
		data = elem.getAttribute( name );

		if ( typeof data === "string" ) {
			try {
				data = data === "true" ? true :
					data === "false" ? false :
					data === "null" ? null :
					// Only convert to a number if it doesn't change the string
					+data + "" === data ? +data :
					rbrace.test( data ) ? jQuery.parseJSON( data ) :
					data;
			} catch( e ) {}

			// Make sure we set the data so it isn't changed later
			data_user.set( elem, key, data );
		} else {
			data = undefined;
		}
	}
	return data;
}

jQuery.extend({
	hasData: function( elem ) {
		return data_user.hasData( elem ) || data_priv.hasData( elem );
	},

	data: function( elem, name, data ) {
		return data_user.access( elem, name, data );
	},

	removeData: function( elem, name ) {
		data_user.remove( elem, name );
	},

	// TODO: Now that all calls to _data and _removeData have been replaced
	// with direct calls to data_priv methods, these can be deprecated.
	_data: function( elem, name, data ) {
		return data_priv.access( elem, name, data );
	},

	_removeData: function( elem, name ) {
		data_priv.remove( elem, name );
	}
});

jQuery.fn.extend({
	data: function( key, value ) {
		var i, name, data,
			elem = this[ 0 ],
			attrs = elem && elem.attributes;

		// Gets all values
		if ( key === undefined ) {
			if ( this.length ) {
				data = data_user.get( elem );

				if ( elem.nodeType === 1 && !data_priv.get( elem, "hasDataAttrs" ) ) {
					i = attrs.length;
					while ( i-- ) {

						// Support: IE11+
						// The attrs elements can be null (#14894)
						if ( attrs[ i ] ) {
							name = attrs[ i ].name;
							if ( name.indexOf( "data-" ) === 0 ) {
								name = jQuery.camelCase( name.slice(5) );
								dataAttr( elem, name, data[ name ] );
							}
						}
					}
					data_priv.set( elem, "hasDataAttrs", true );
				}
			}

			return data;
		}

		// Sets multiple values
		if ( typeof key === "object" ) {
			return this.each(function() {
				data_user.set( this, key );
			});
		}

		return access( this, function( value ) {
			var data,
				camelKey = jQuery.camelCase( key );

			// The calling jQuery object (element matches) is not empty
			// (and therefore has an element appears at this[ 0 ]) and the
			// `value` parameter was not undefined. An empty jQuery object
			// will result in `undefined` for elem = this[ 0 ] which will
			// throw an exception if an attempt to read a data cache is made.
			if ( elem && value === undefined ) {
				// Attempt to get data from the cache
				// with the key as-is
				data = data_user.get( elem, key );
				if ( data !== undefined ) {
					return data;
				}

				// Attempt to get data from the cache
				// with the key camelized
				data = data_user.get( elem, camelKey );
				if ( data !== undefined ) {
					return data;
				}

				// Attempt to "discover" the data in
				// HTML5 custom data-* attrs
				data = dataAttr( elem, camelKey, undefined );
				if ( data !== undefined ) {
					return data;
				}

				// We tried really hard, but the data doesn't exist.
				return;
			}

			// Set the data...
			this.each(function() {
				// First, attempt to store a copy or reference of any
				// data that might've been store with a camelCased key.
				var data = data_user.get( this, camelKey );

				// For HTML5 data-* attribute interop, we have to
				// store property names with dashes in a camelCase form.
				// This might not apply to all properties...*
				data_user.set( this, camelKey, value );

				// *... In the case of properties that might _actually_
				// have dashes, we need to also store a copy of that
				// unchanged property.
				if ( key.indexOf("-") !== -1 && data !== undefined ) {
					data_user.set( this, key, value );
				}
			});
		}, null, value, arguments.length > 1, null, true );
	},

	removeData: function( key ) {
		return this.each(function() {
			data_user.remove( this, key );
		});
	}
});


jQuery.extend({
	queue: function( elem, type, data ) {
		var queue;

		if ( elem ) {
			type = ( type || "fx" ) + "queue";
			queue = data_priv.get( elem, type );

			// Speed up dequeue by getting out quickly if this is just a lookup
			if ( data ) {
				if ( !queue || jQuery.isArray( data ) ) {
					queue = data_priv.access( elem, type, jQuery.makeArray(data) );
				} else {
					queue.push( data );
				}
			}
			return queue || [];
		}
	},

	dequeue: function( elem, type ) {
		type = type || "fx";

		var queue = jQuery.queue( elem, type ),
			startLength = queue.length,
			fn = queue.shift(),
			hooks = jQuery._queueHooks( elem, type ),
			next = function() {
				jQuery.dequeue( elem, type );
			};

		// If the fx queue is dequeued, always remove the progress sentinel
		if ( fn === "inprogress" ) {
			fn = queue.shift();
			startLength--;
		}

		if ( fn ) {

			// Add a progress sentinel to prevent the fx queue from being
			// automatically dequeued
			if ( type === "fx" ) {
				queue.unshift( "inprogress" );
			}

			// Clear up the last queue stop function
			delete hooks.stop;
			fn.call( elem, next, hooks );
		}

		if ( !startLength && hooks ) {
			hooks.empty.fire();
		}
	},

	// Not public - generate a queueHooks object, or return the current one
	_queueHooks: function( elem, type ) {
		var key = type + "queueHooks";
		return data_priv.get( elem, key ) || data_priv.access( elem, key, {
			empty: jQuery.Callbacks("once memory").add(function() {
				data_priv.remove( elem, [ type + "queue", key ] );
			})
		});
	}
});

jQuery.fn.extend({
	queue: function( type, data ) {
		var setter = 2;

		if ( typeof type !== "string" ) {
			data = type;
			type = "fx";
			setter--;
		}

		if ( arguments.length < setter ) {
			return jQuery.queue( this[0], type );
		}

		return data === undefined ?
			this :
			this.each(function() {
				var queue = jQuery.queue( this, type, data );

				// Ensure a hooks for this queue
				jQuery._queueHooks( this, type );

				if ( type === "fx" && queue[0] !== "inprogress" ) {
					jQuery.dequeue( this, type );
				}
			});
	},
	dequeue: function( type ) {
		return this.each(function() {
			jQuery.dequeue( this, type );
		});
	},
	clearQueue: function( type ) {
		return this.queue( type || "fx", [] );
	},
	// Get a promise resolved when queues of a certain type
	// are emptied (fx is the type by default)
	promise: function( type, obj ) {
		var tmp,
			count = 1,
			defer = jQuery.Deferred(),
			elements = this,
			i = this.length,
			resolve = function() {
				if ( !( --count ) ) {
					defer.resolveWith( elements, [ elements ] );
				}
			};

		if ( typeof type !== "string" ) {
			obj = type;
			type = undefined;
		}
		type = type || "fx";

		while ( i-- ) {
			tmp = data_priv.get( elements[ i ], type + "queueHooks" );
			if ( tmp && tmp.empty ) {
				count++;
				tmp.empty.add( resolve );
			}
		}
		resolve();
		return defer.promise( obj );
	}
});
var pnum = (/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/).source;

var cssExpand = [ "Top", "Right", "Bottom", "Left" ];

var isHidden = function( elem, el ) {
		// isHidden might be called from jQuery#filter function;
		// in that case, element will be second argument
		elem = el || elem;
		return jQuery.css( elem, "display" ) === "none" || !jQuery.contains( elem.ownerDocument, elem );
	};

var rcheckableType = (/^(?:checkbox|radio)$/i);



(function() {
	var fragment = document.createDocumentFragment(),
		div = fragment.appendChild( document.createElement( "div" ) ),
		input = document.createElement( "input" );

	// Support: Safari<=5.1
	// Check state lost if the name is set (#11217)
	// Support: Windows Web Apps (WWA)
	// `name` and `type` must use .setAttribute for WWA (#14901)
	input.setAttribute( "type", "radio" );
	input.setAttribute( "checked", "checked" );
	input.setAttribute( "name", "t" );

	div.appendChild( input );

	// Support: Safari<=5.1, Android<4.2
	// Older WebKit doesn't clone checked state correctly in fragments
	support.checkClone = div.cloneNode( true ).cloneNode( true ).lastChild.checked;

	// Support: IE<=11+
	// Make sure textarea (and checkbox) defaultValue is properly cloned
	div.innerHTML = "<textarea>x</textarea>";
	support.noCloneChecked = !!div.cloneNode( true ).lastChild.defaultValue;
})();
var strundefined = typeof undefined;



support.focusinBubbles = "onfocusin" in window;


var
	rkeyEvent = /^key/,
	rmouseEvent = /^(?:mouse|pointer|contextmenu)|click/,
	rfocusMorph = /^(?:focusinfocus|focusoutblur)$/,
	rtypenamespace = /^([^.]*)(?:\.(.+)|)$/;

function returnTrue() {
	return true;
}

function returnFalse() {
	return false;
}

function safeActiveElement() {
	try {
		return document.activeElement;
	} catch ( err ) { }
}

/*
 * Helper functions for managing events -- not part of the public interface.
 * Props to Dean Edwards' addEvent library for many of the ideas.
 */
jQuery.event = {

	global: {},

	add: function( elem, types, handler, data, selector ) {

		var handleObjIn, eventHandle, tmp,
			events, t, handleObj,
			special, handlers, type, namespaces, origType,
			elemData = data_priv.get( elem );

		// Don't attach events to noData or text/comment nodes (but allow plain objects)
		if ( !elemData ) {
			return;
		}

		// Caller can pass in an object of custom data in lieu of the handler
		if ( handler.handler ) {
			handleObjIn = handler;
			handler = handleObjIn.handler;
			selector = handleObjIn.selector;
		}

		// Make sure that the handler has a unique ID, used to find/remove it later
		if ( !handler.guid ) {
			handler.guid = jQuery.guid++;
		}

		// Init the element's event structure and main handler, if this is the first
		if ( !(events = elemData.events) ) {
			events = elemData.events = {};
		}
		if ( !(eventHandle = elemData.handle) ) {
			eventHandle = elemData.handle = function( e ) {
				// Discard the second event of a jQuery.event.trigger() and
				// when an event is called after a page has unloaded
				return typeof jQuery !== strundefined && jQuery.event.triggered !== e.type ?
					jQuery.event.dispatch.apply( elem, arguments ) : undefined;
			};
		}

		// Handle multiple events separated by a space
		types = ( types || "" ).match( rnotwhite ) || [ "" ];
		t = types.length;
		while ( t-- ) {
			tmp = rtypenamespace.exec( types[t] ) || [];
			type = origType = tmp[1];
			namespaces = ( tmp[2] || "" ).split( "." ).sort();

			// There *must* be a type, no attaching namespace-only handlers
			if ( !type ) {
				continue;
			}

			// If event changes its type, use the special event handlers for the changed type
			special = jQuery.event.special[ type ] || {};

			// If selector defined, determine special event api type, otherwise given type
			type = ( selector ? special.delegateType : special.bindType ) || type;

			// Update special based on newly reset type
			special = jQuery.event.special[ type ] || {};

			// handleObj is passed to all event handlers
			handleObj = jQuery.extend({
				type: type,
				origType: origType,
				data: data,
				handler: handler,
				guid: handler.guid,
				selector: selector,
				needsContext: selector && jQuery.expr.match.needsContext.test( selector ),
				namespace: namespaces.join(".")
			}, handleObjIn );

			// Init the event handler queue if we're the first
			if ( !(handlers = events[ type ]) ) {
				handlers = events[ type ] = [];
				handlers.delegateCount = 0;

				// Only use addEventListener if the special events handler returns false
				if ( !special.setup || special.setup.call( elem, data, namespaces, eventHandle ) === false ) {
					if ( elem.addEventListener ) {
						elem.addEventListener( type, eventHandle, false );
					}
				}
			}

			if ( special.add ) {
				special.add.call( elem, handleObj );

				if ( !handleObj.handler.guid ) {
					handleObj.handler.guid = handler.guid;
				}
			}

			// Add to the element's handler list, delegates in front
			if ( selector ) {
				handlers.splice( handlers.delegateCount++, 0, handleObj );
			} else {
				handlers.push( handleObj );
			}

			// Keep track of which events have ever been used, for event optimization
			jQuery.event.global[ type ] = true;
		}

	},

	// Detach an event or set of events from an element
	remove: function( elem, types, handler, selector, mappedTypes ) {

		var j, origCount, tmp,
			events, t, handleObj,
			special, handlers, type, namespaces, origType,
			elemData = data_priv.hasData( elem ) && data_priv.get( elem );

		if ( !elemData || !(events = elemData.events) ) {
			return;
		}

		// Once for each type.namespace in types; type may be omitted
		types = ( types || "" ).match( rnotwhite ) || [ "" ];
		t = types.length;
		while ( t-- ) {
			tmp = rtypenamespace.exec( types[t] ) || [];
			type = origType = tmp[1];
			namespaces = ( tmp[2] || "" ).split( "." ).sort();

			// Unbind all events (on this namespace, if provided) for the element
			if ( !type ) {
				for ( type in events ) {
					jQuery.event.remove( elem, type + types[ t ], handler, selector, true );
				}
				continue;
			}

			special = jQuery.event.special[ type ] || {};
			type = ( selector ? special.delegateType : special.bindType ) || type;
			handlers = events[ type ] || [];
			tmp = tmp[2] && new RegExp( "(^|\\.)" + namespaces.join("\\.(?:.*\\.|)") + "(\\.|$)" );

			// Remove matching events
			origCount = j = handlers.length;
			while ( j-- ) {
				handleObj = handlers[ j ];

				if ( ( mappedTypes || origType === handleObj.origType ) &&
					( !handler || handler.guid === handleObj.guid ) &&
					( !tmp || tmp.test( handleObj.namespace ) ) &&
					( !selector || selector === handleObj.selector || selector === "**" && handleObj.selector ) ) {
					handlers.splice( j, 1 );

					if ( handleObj.selector ) {
						handlers.delegateCount--;
					}
					if ( special.remove ) {
						special.remove.call( elem, handleObj );
					}
				}
			}

			// Remove generic event handler if we removed something and no more handlers exist
			// (avoids potential for endless recursion during removal of special event handlers)
			if ( origCount && !handlers.length ) {
				if ( !special.teardown || special.teardown.call( elem, namespaces, elemData.handle ) === false ) {
					jQuery.removeEvent( elem, type, elemData.handle );
				}

				delete events[ type ];
			}
		}

		// Remove the expando if it's no longer used
		if ( jQuery.isEmptyObject( events ) ) {
			delete elemData.handle;
			data_priv.remove( elem, "events" );
		}
	},

	trigger: function( event, data, elem, onlyHandlers ) {

		var i, cur, tmp, bubbleType, ontype, handle, special,
			eventPath = [ elem || document ],
			type = hasOwn.call( event, "type" ) ? event.type : event,
			namespaces = hasOwn.call( event, "namespace" ) ? event.namespace.split(".") : [];

		cur = tmp = elem = elem || document;

		// Don't do events on text and comment nodes
		if ( elem.nodeType === 3 || elem.nodeType === 8 ) {
			return;
		}

		// focus/blur morphs to focusin/out; ensure we're not firing them right now
		if ( rfocusMorph.test( type + jQuery.event.triggered ) ) {
			return;
		}

		if ( type.indexOf(".") >= 0 ) {
			// Namespaced trigger; create a regexp to match event type in handle()
			namespaces = type.split(".");
			type = namespaces.shift();
			namespaces.sort();
		}
		ontype = type.indexOf(":") < 0 && "on" + type;

		// Caller can pass in a jQuery.Event object, Object, or just an event type string
		event = event[ jQuery.expando ] ?
			event :
			new jQuery.Event( type, typeof event === "object" && event );

		// Trigger bitmask: & 1 for native handlers; & 2 for jQuery (always true)
		event.isTrigger = onlyHandlers ? 2 : 3;
		event.namespace = namespaces.join(".");
		event.namespace_re = event.namespace ?
			new RegExp( "(^|\\.)" + namespaces.join("\\.(?:.*\\.|)") + "(\\.|$)" ) :
			null;

		// Clean up the event in case it is being reused
		event.result = undefined;
		if ( !event.target ) {
			event.target = elem;
		}

		// Clone any incoming data and prepend the event, creating the handler arg list
		data = data == null ?
			[ event ] :
			jQuery.makeArray( data, [ event ] );

		// Allow special events to draw outside the lines
		special = jQuery.event.special[ type ] || {};
		if ( !onlyHandlers && special.trigger && special.trigger.apply( elem, data ) === false ) {
			return;
		}

		// Determine event propagation path in advance, per W3C events spec (#9951)
		// Bubble up to document, then to window; watch for a global ownerDocument var (#9724)
		if ( !onlyHandlers && !special.noBubble && !jQuery.isWindow( elem ) ) {

			bubbleType = special.delegateType || type;
			if ( !rfocusMorph.test( bubbleType + type ) ) {
				cur = cur.parentNode;
			}
			for ( ; cur; cur = cur.parentNode ) {
				eventPath.push( cur );
				tmp = cur;
			}

			// Only add window if we got to document (e.g., not plain obj or detached DOM)
			if ( tmp === (elem.ownerDocument || document) ) {
				eventPath.push( tmp.defaultView || tmp.parentWindow || window );
			}
		}

		// Fire handlers on the event path
		i = 0;
		while ( (cur = eventPath[i++]) && !event.isPropagationStopped() ) {

			event.type = i > 1 ?
				bubbleType :
				special.bindType || type;

			// jQuery handler
			handle = ( data_priv.get( cur, "events" ) || {} )[ event.type ] && data_priv.get( cur, "handle" );
			if ( handle ) {
				handle.apply( cur, data );
			}

			// Native handler
			handle = ontype && cur[ ontype ];
			if ( handle && handle.apply && jQuery.acceptData( cur ) ) {
				event.result = handle.apply( cur, data );
				if ( event.result === false ) {
					event.preventDefault();
				}
			}
		}
		event.type = type;

		// If nobody prevented the default action, do it now
		if ( !onlyHandlers && !event.isDefaultPrevented() ) {

			if ( (!special._default || special._default.apply( eventPath.pop(), data ) === false) &&
				jQuery.acceptData( elem ) ) {

				// Call a native DOM method on the target with the same name name as the event.
				// Don't do default actions on window, that's where global variables be (#6170)
				if ( ontype && jQuery.isFunction( elem[ type ] ) && !jQuery.isWindow( elem ) ) {

					// Don't re-trigger an onFOO event when we call its FOO() method
					tmp = elem[ ontype ];

					if ( tmp ) {
						elem[ ontype ] = null;
					}

					// Prevent re-triggering of the same event, since we already bubbled it above
					jQuery.event.triggered = type;
					elem[ type ]();
					jQuery.event.triggered = undefined;

					if ( tmp ) {
						elem[ ontype ] = tmp;
					}
				}
			}
		}

		return event.result;
	},

	dispatch: function( event ) {

		// Make a writable jQuery.Event from the native event object
		event = jQuery.event.fix( event );

		var i, j, ret, matched, handleObj,
			handlerQueue = [],
			args = slice.call( arguments ),
			handlers = ( data_priv.get( this, "events" ) || {} )[ event.type ] || [],
			special = jQuery.event.special[ event.type ] || {};

		// Use the fix-ed jQuery.Event rather than the (read-only) native event
		args[0] = event;
		event.delegateTarget = this;

		// Call the preDispatch hook for the mapped type, and let it bail if desired
		if ( special.preDispatch && special.preDispatch.call( this, event ) === false ) {
			return;
		}

		// Determine handlers
		handlerQueue = jQuery.event.handlers.call( this, event, handlers );

		// Run delegates first; they may want to stop propagation beneath us
		i = 0;
		while ( (matched = handlerQueue[ i++ ]) && !event.isPropagationStopped() ) {
			event.currentTarget = matched.elem;

			j = 0;
			while ( (handleObj = matched.handlers[ j++ ]) && !event.isImmediatePropagationStopped() ) {

				// Triggered event must either 1) have no namespace, or 2) have namespace(s)
				// a subset or equal to those in the bound event (both can have no namespace).
				if ( !event.namespace_re || event.namespace_re.test( handleObj.namespace ) ) {

					event.handleObj = handleObj;
					event.data = handleObj.data;

					ret = ( (jQuery.event.special[ handleObj.origType ] || {}).handle || handleObj.handler )
							.apply( matched.elem, args );

					if ( ret !== undefined ) {
						if ( (event.result = ret) === false ) {
							event.preventDefault();
							event.stopPropagation();
						}
					}
				}
			}
		}

		// Call the postDispatch hook for the mapped type
		if ( special.postDispatch ) {
			special.postDispatch.call( this, event );
		}

		return event.result;
	},

	handlers: function( event, handlers ) {
		var i, matches, sel, handleObj,
			handlerQueue = [],
			delegateCount = handlers.delegateCount,
			cur = event.target;

		// Find delegate handlers
		// Black-hole SVG <use> instance trees (#13180)
		// Avoid non-left-click bubbling in Firefox (#3861)
		if ( delegateCount && cur.nodeType && (!event.button || event.type !== "click") ) {

			for ( ; cur !== this; cur = cur.parentNode || this ) {

				// Don't process clicks on disabled elements (#6911, #8165, #11382, #11764)
				if ( cur.disabled !== true || event.type !== "click" ) {
					matches = [];
					for ( i = 0; i < delegateCount; i++ ) {
						handleObj = handlers[ i ];

						// Don't conflict with Object.prototype properties (#13203)
						sel = handleObj.selector + " ";

						if ( matches[ sel ] === undefined ) {
							matches[ sel ] = handleObj.needsContext ?
								jQuery( sel, this ).index( cur ) >= 0 :
								jQuery.find( sel, this, null, [ cur ] ).length;
						}
						if ( matches[ sel ] ) {
							matches.push( handleObj );
						}
					}
					if ( matches.length ) {
						handlerQueue.push({ elem: cur, handlers: matches });
					}
				}
			}
		}

		// Add the remaining (directly-bound) handlers
		if ( delegateCount < handlers.length ) {
			handlerQueue.push({ elem: this, handlers: handlers.slice( delegateCount ) });
		}

		return handlerQueue;
	},

	// Includes some event props shared by KeyEvent and MouseEvent
	props: "altKey bubbles cancelable ctrlKey currentTarget eventPhase metaKey relatedTarget shiftKey target timeStamp view which".split(" "),

	fixHooks: {},

	keyHooks: {
		props: "char charCode key keyCode".split(" "),
		filter: function( event, original ) {

			// Add which for key events
			if ( event.which == null ) {
				event.which = original.charCode != null ? original.charCode : original.keyCode;
			}

			return event;
		}
	},

	mouseHooks: {
		props: "button buttons clientX clientY offsetX offsetY pageX pageY screenX screenY toElement".split(" "),
		filter: function( event, original ) {
			var eventDoc, doc, body,
				button = original.button;

			// Calculate pageX/Y if missing and clientX/Y available
			if ( event.pageX == null && original.clientX != null ) {
				eventDoc = event.target.ownerDocument || document;
				doc = eventDoc.documentElement;
				body = eventDoc.body;

				event.pageX = original.clientX + ( doc && doc.scrollLeft || body && body.scrollLeft || 0 ) - ( doc && doc.clientLeft || body && body.clientLeft || 0 );
				event.pageY = original.clientY + ( doc && doc.scrollTop  || body && body.scrollTop  || 0 ) - ( doc && doc.clientTop  || body && body.clientTop  || 0 );
			}

			// Add which for click: 1 === left; 2 === middle; 3 === right
			// Note: button is not normalized, so don't use it
			if ( !event.which && button !== undefined ) {
				event.which = ( button & 1 ? 1 : ( button & 2 ? 3 : ( button & 4 ? 2 : 0 ) ) );
			}

			return event;
		}
	},

	fix: function( event ) {
		if ( event[ jQuery.expando ] ) {
			return event;
		}

		// Create a writable copy of the event object and normalize some properties
		var i, prop, copy,
			type = event.type,
			originalEvent = event,
			fixHook = this.fixHooks[ type ];

		if ( !fixHook ) {
			this.fixHooks[ type ] = fixHook =
				rmouseEvent.test( type ) ? this.mouseHooks :
				rkeyEvent.test( type ) ? this.keyHooks :
				{};
		}
		copy = fixHook.props ? this.props.concat( fixHook.props ) : this.props;

		event = new jQuery.Event( originalEvent );

		i = copy.length;
		while ( i-- ) {
			prop = copy[ i ];
			event[ prop ] = originalEvent[ prop ];
		}

		// Support: Cordova 2.5 (WebKit) (#13255)
		// All events should have a target; Cordova deviceready doesn't
		if ( !event.target ) {
			event.target = document;
		}

		// Support: Safari 6.0+, Chrome<28
		// Target should not be a text node (#504, #13143)
		if ( event.target.nodeType === 3 ) {
			event.target = event.target.parentNode;
		}

		return fixHook.filter ? fixHook.filter( event, originalEvent ) : event;
	},

	special: {
		load: {
			// Prevent triggered image.load events from bubbling to window.load
			noBubble: true
		},
		focus: {
			// Fire native event if possible so blur/focus sequence is correct
			trigger: function() {
				if ( this !== safeActiveElement() && this.focus ) {
					this.focus();
					return false;
				}
			},
			delegateType: "focusin"
		},
		blur: {
			trigger: function() {
				if ( this === safeActiveElement() && this.blur ) {
					this.blur();
					return false;
				}
			},
			delegateType: "focusout"
		},
		click: {
			// For checkbox, fire native event so checked state will be right
			trigger: function() {
				if ( this.type === "checkbox" && this.click && jQuery.nodeName( this, "input" ) ) {
					this.click();
					return false;
				}
			},

			// For cross-browser consistency, don't fire native .click() on links
			_default: function( event ) {
				return jQuery.nodeName( event.target, "a" );
			}
		},

		beforeunload: {
			postDispatch: function( event ) {

				// Support: Firefox 20+
				// Firefox doesn't alert if the returnValue field is not set.
				if ( event.result !== undefined && event.originalEvent ) {
					event.originalEvent.returnValue = event.result;
				}
			}
		}
	},

	simulate: function( type, elem, event, bubble ) {
		// Piggyback on a donor event to simulate a different one.
		// Fake originalEvent to avoid donor's stopPropagation, but if the
		// simulated event prevents default then we do the same on the donor.
		var e = jQuery.extend(
			new jQuery.Event(),
			event,
			{
				type: type,
				isSimulated: true,
				originalEvent: {}
			}
		);
		if ( bubble ) {
			jQuery.event.trigger( e, null, elem );
		} else {
			jQuery.event.dispatch.call( elem, e );
		}
		if ( e.isDefaultPrevented() ) {
			event.preventDefault();
		}
	}
};

jQuery.removeEvent = function( elem, type, handle ) {
	if ( elem.removeEventListener ) {
		elem.removeEventListener( type, handle, false );
	}
};

jQuery.Event = function( src, props ) {
	// Allow instantiation without the 'new' keyword
	if ( !(this instanceof jQuery.Event) ) {
		return new jQuery.Event( src, props );
	}

	// Event object
	if ( src && src.type ) {
		this.originalEvent = src;
		this.type = src.type;

		// Events bubbling up the document may have been marked as prevented
		// by a handler lower down the tree; reflect the correct value.
		this.isDefaultPrevented = src.defaultPrevented ||
				src.defaultPrevented === undefined &&
				// Support: Android<4.0
				src.returnValue === false ?
			returnTrue :
			returnFalse;

	// Event type
	} else {
		this.type = src;
	}

	// Put explicitly provided properties onto the event object
	if ( props ) {
		jQuery.extend( this, props );
	}

	// Create a timestamp if incoming event doesn't have one
	this.timeStamp = src && src.timeStamp || jQuery.now();

	// Mark it as fixed
	this[ jQuery.expando ] = true;
};

// jQuery.Event is based on DOM3 Events as specified by the ECMAScript Language Binding
// http://www.w3.org/TR/2003/WD-DOM-Level-3-Events-20030331/ecma-script-binding.html
jQuery.Event.prototype = {
	isDefaultPrevented: returnFalse,
	isPropagationStopped: returnFalse,
	isImmediatePropagationStopped: returnFalse,

	preventDefault: function() {
		var e = this.originalEvent;

		this.isDefaultPrevented = returnTrue;

		if ( e && e.preventDefault ) {
			e.preventDefault();
		}
	},
	stopPropagation: function() {
		var e = this.originalEvent;

		this.isPropagationStopped = returnTrue;

		if ( e && e.stopPropagation ) {
			e.stopPropagation();
		}
	},
	stopImmediatePropagation: function() {
		var e = this.originalEvent;

		this.isImmediatePropagationStopped = returnTrue;

		if ( e && e.stopImmediatePropagation ) {
			e.stopImmediatePropagation();
		}

		this.stopPropagation();
	}
};

// Create mouseenter/leave events using mouseover/out and event-time checks
// Support: Chrome 15+
jQuery.each({
	mouseenter: "mouseover",
	mouseleave: "mouseout",
	pointerenter: "pointerover",
	pointerleave: "pointerout"
}, function( orig, fix ) {
	jQuery.event.special[ orig ] = {
		delegateType: fix,
		bindType: fix,

		handle: function( event ) {
			var ret,
				target = this,
				related = event.relatedTarget,
				handleObj = event.handleObj;

			// For mousenter/leave call the handler if related is outside the target.
			// NB: No relatedTarget if the mouse left/entered the browser window
			if ( !related || (related !== target && !jQuery.contains( target, related )) ) {
				event.type = handleObj.origType;
				ret = handleObj.handler.apply( this, arguments );
				event.type = fix;
			}
			return ret;
		}
	};
});

// Support: Firefox, Chrome, Safari
// Create "bubbling" focus and blur events
if ( !support.focusinBubbles ) {
	jQuery.each({ focus: "focusin", blur: "focusout" }, function( orig, fix ) {

		// Attach a single capturing handler on the document while someone wants focusin/focusout
		var handler = function( event ) {
				jQuery.event.simulate( fix, event.target, jQuery.event.fix( event ), true );
			};

		jQuery.event.special[ fix ] = {
			setup: function() {
				var doc = this.ownerDocument || this,
					attaches = data_priv.access( doc, fix );

				if ( !attaches ) {
					doc.addEventListener( orig, handler, true );
				}
				data_priv.access( doc, fix, ( attaches || 0 ) + 1 );
			},
			teardown: function() {
				var doc = this.ownerDocument || this,
					attaches = data_priv.access( doc, fix ) - 1;

				if ( !attaches ) {
					doc.removeEventListener( orig, handler, true );
					data_priv.remove( doc, fix );

				} else {
					data_priv.access( doc, fix, attaches );
				}
			}
		};
	});
}

jQuery.fn.extend({

	on: function( types, selector, data, fn, /*INTERNAL*/ one ) {
		var origFn, type;

		// Types can be a map of types/handlers
		if ( typeof types === "object" ) {
			// ( types-Object, selector, data )
			if ( typeof selector !== "string" ) {
				// ( types-Object, data )
				data = data || selector;
				selector = undefined;
			}
			for ( type in types ) {
				this.on( type, selector, data, types[ type ], one );
			}
			return this;
		}

		if ( data == null && fn == null ) {
			// ( types, fn )
			fn = selector;
			data = selector = undefined;
		} else if ( fn == null ) {
			if ( typeof selector === "string" ) {
				// ( types, selector, fn )
				fn = data;
				data = undefined;
			} else {
				// ( types, data, fn )
				fn = data;
				data = selector;
				selector = undefined;
			}
		}
		if ( fn === false ) {
			fn = returnFalse;
		} else if ( !fn ) {
			return this;
		}

		if ( one === 1 ) {
			origFn = fn;
			fn = function( event ) {
				// Can use an empty set, since event contains the info
				jQuery().off( event );
				return origFn.apply( this, arguments );
			};
			// Use same guid so caller can remove using origFn
			fn.guid = origFn.guid || ( origFn.guid = jQuery.guid++ );
		}
		return this.each( function() {
			jQuery.event.add( this, types, fn, data, selector );
		});
	},
	one: function( types, selector, data, fn ) {
		return this.on( types, selector, data, fn, 1 );
	},
	off: function( types, selector, fn ) {
		var handleObj, type;
		if ( types && types.preventDefault && types.handleObj ) {
			// ( event )  dispatched jQuery.Event
			handleObj = types.handleObj;
			jQuery( types.delegateTarget ).off(
				handleObj.namespace ? handleObj.origType + "." + handleObj.namespace : handleObj.origType,
				handleObj.selector,
				handleObj.handler
			);
			return this;
		}
		if ( typeof types === "object" ) {
			// ( types-object [, selector] )
			for ( type in types ) {
				this.off( type, selector, types[ type ] );
			}
			return this;
		}
		if ( selector === false || typeof selector === "function" ) {
			// ( types [, fn] )
			fn = selector;
			selector = undefined;
		}
		if ( fn === false ) {
			fn = returnFalse;
		}
		return this.each(function() {
			jQuery.event.remove( this, types, fn, selector );
		});
	},

	trigger: function( type, data ) {
		return this.each(function() {
			jQuery.event.trigger( type, data, this );
		});
	},
	triggerHandler: function( type, data ) {
		var elem = this[0];
		if ( elem ) {
			return jQuery.event.trigger( type, data, elem, true );
		}
	}
});


var
	rxhtmlTag = /<(?!area|br|col|embed|hr|img|input|link|meta|param)(([\w:]+)[^>]*)\/>/gi,
	rtagName = /<([\w:]+)/,
	rhtml = /<|&#?\w+;/,
	rnoInnerhtml = /<(?:script|style|link)/i,
	// checked="checked" or checked
	rchecked = /checked\s*(?:[^=]|=\s*.checked.)/i,
	rscriptType = /^$|\/(?:java|ecma)script/i,
	rscriptTypeMasked = /^true\/(.*)/,
	rcleanScript = /^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g,

	// We have to close these tags to support XHTML (#13200)
	wrapMap = {

		// Support: IE9
		option: [ 1, "<select multiple='multiple'>", "</select>" ],

		thead: [ 1, "<table>", "</table>" ],
		col: [ 2, "<table><colgroup>", "</colgroup></table>" ],
		tr: [ 2, "<table><tbody>", "</tbody></table>" ],
		td: [ 3, "<table><tbody><tr>", "</tr></tbody></table>" ],

		_default: [ 0, "", "" ]
	};

// Support: IE9
wrapMap.optgroup = wrapMap.option;

wrapMap.tbody = wrapMap.tfoot = wrapMap.colgroup = wrapMap.caption = wrapMap.thead;
wrapMap.th = wrapMap.td;

// Support: 1.x compatibility
// Manipulating tables requires a tbody
function manipulationTarget( elem, content ) {
	return jQuery.nodeName( elem, "table" ) &&
		jQuery.nodeName( content.nodeType !== 11 ? content : content.firstChild, "tr" ) ?

		elem.getElementsByTagName("tbody")[0] ||
			elem.appendChild( elem.ownerDocument.createElement("tbody") ) :
		elem;
}

// Replace/restore the type attribute of script elements for safe DOM manipulation
function disableScript( elem ) {
	elem.type = (elem.getAttribute("type") !== null) + "/" + elem.type;
	return elem;
}
function restoreScript( elem ) {
	var match = rscriptTypeMasked.exec( elem.type );

	if ( match ) {
		elem.type = match[ 1 ];
	} else {
		elem.removeAttribute("type");
	}

	return elem;
}

// Mark scripts as having already been evaluated
function setGlobalEval( elems, refElements ) {
	var i = 0,
		l = elems.length;

	for ( ; i < l; i++ ) {
		data_priv.set(
			elems[ i ], "globalEval", !refElements || data_priv.get( refElements[ i ], "globalEval" )
		);
	}
}

function cloneCopyEvent( src, dest ) {
	var i, l, type, pdataOld, pdataCur, udataOld, udataCur, events;

	if ( dest.nodeType !== 1 ) {
		return;
	}

	// 1. Copy private data: events, handlers, etc.
	if ( data_priv.hasData( src ) ) {
		pdataOld = data_priv.access( src );
		pdataCur = data_priv.set( dest, pdataOld );
		events = pdataOld.events;

		if ( events ) {
			delete pdataCur.handle;
			pdataCur.events = {};

			for ( type in events ) {
				for ( i = 0, l = events[ type ].length; i < l; i++ ) {
					jQuery.event.add( dest, type, events[ type ][ i ] );
				}
			}
		}
	}

	// 2. Copy user data
	if ( data_user.hasData( src ) ) {
		udataOld = data_user.access( src );
		udataCur = jQuery.extend( {}, udataOld );

		data_user.set( dest, udataCur );
	}
}

function getAll( context, tag ) {
	var ret = context.getElementsByTagName ? context.getElementsByTagName( tag || "*" ) :
			context.querySelectorAll ? context.querySelectorAll( tag || "*" ) :
			[];

	return tag === undefined || tag && jQuery.nodeName( context, tag ) ?
		jQuery.merge( [ context ], ret ) :
		ret;
}

// Fix IE bugs, see support tests
function fixInput( src, dest ) {
	var nodeName = dest.nodeName.toLowerCase();

	// Fails to persist the checked state of a cloned checkbox or radio button.
	if ( nodeName === "input" && rcheckableType.test( src.type ) ) {
		dest.checked = src.checked;

	// Fails to return the selected option to the default selected state when cloning options
	} else if ( nodeName === "input" || nodeName === "textarea" ) {
		dest.defaultValue = src.defaultValue;
	}
}

jQuery.extend({
	clone: function( elem, dataAndEvents, deepDataAndEvents ) {
		var i, l, srcElements, destElements,
			clone = elem.cloneNode( true ),
			inPage = jQuery.contains( elem.ownerDocument, elem );

		// Fix IE cloning issues
		if ( !support.noCloneChecked && ( elem.nodeType === 1 || elem.nodeType === 11 ) &&
				!jQuery.isXMLDoc( elem ) ) {

			// We eschew Sizzle here for performance reasons: http://jsperf.com/getall-vs-sizzle/2
			destElements = getAll( clone );
			srcElements = getAll( elem );

			for ( i = 0, l = srcElements.length; i < l; i++ ) {
				fixInput( srcElements[ i ], destElements[ i ] );
			}
		}

		// Copy the events from the original to the clone
		if ( dataAndEvents ) {
			if ( deepDataAndEvents ) {
				srcElements = srcElements || getAll( elem );
				destElements = destElements || getAll( clone );

				for ( i = 0, l = srcElements.length; i < l; i++ ) {
					cloneCopyEvent( srcElements[ i ], destElements[ i ] );
				}
			} else {
				cloneCopyEvent( elem, clone );
			}
		}

		// Preserve script evaluation history
		destElements = getAll( clone, "script" );
		if ( destElements.length > 0 ) {
			setGlobalEval( destElements, !inPage && getAll( elem, "script" ) );
		}

		// Return the cloned set
		return clone;
	},

	buildFragment: function( elems, context, scripts, selection ) {
		var elem, tmp, tag, wrap, contains, j,
			fragment = context.createDocumentFragment(),
			nodes = [],
			i = 0,
			l = elems.length;

		for ( ; i < l; i++ ) {
			elem = elems[ i ];

			if ( elem || elem === 0 ) {

				// Add nodes directly
				if ( jQuery.type( elem ) === "object" ) {
					// Support: QtWebKit, PhantomJS
					// push.apply(_, arraylike) throws on ancient WebKit
					jQuery.merge( nodes, elem.nodeType ? [ elem ] : elem );

				// Convert non-html into a text node
				} else if ( !rhtml.test( elem ) ) {
					nodes.push( context.createTextNode( elem ) );

				// Convert html into DOM nodes
				} else {
					tmp = tmp || fragment.appendChild( context.createElement("div") );

					// Deserialize a standard representation
					tag = ( rtagName.exec( elem ) || [ "", "" ] )[ 1 ].toLowerCase();
					wrap = wrapMap[ tag ] || wrapMap._default;
					tmp.innerHTML = wrap[ 1 ] + elem.replace( rxhtmlTag, "<$1></$2>" ) + wrap[ 2 ];

					// Descend through wrappers to the right content
					j = wrap[ 0 ];
					while ( j-- ) {
						tmp = tmp.lastChild;
					}

					// Support: QtWebKit, PhantomJS
					// push.apply(_, arraylike) throws on ancient WebKit
					jQuery.merge( nodes, tmp.childNodes );

					// Remember the top-level container
					tmp = fragment.firstChild;

					// Ensure the created nodes are orphaned (#12392)
					tmp.textContent = "";
				}
			}
		}

		// Remove wrapper from fragment
		fragment.textContent = "";

		i = 0;
		while ( (elem = nodes[ i++ ]) ) {

			// #4087 - If origin and destination elements are the same, and this is
			// that element, do not do anything
			if ( selection && jQuery.inArray( elem, selection ) !== -1 ) {
				continue;
			}

			contains = jQuery.contains( elem.ownerDocument, elem );

			// Append to fragment
			tmp = getAll( fragment.appendChild( elem ), "script" );

			// Preserve script evaluation history
			if ( contains ) {
				setGlobalEval( tmp );
			}

			// Capture executables
			if ( scripts ) {
				j = 0;
				while ( (elem = tmp[ j++ ]) ) {
					if ( rscriptType.test( elem.type || "" ) ) {
						scripts.push( elem );
					}
				}
			}
		}

		return fragment;
	},

	cleanData: function( elems ) {
		var data, elem, type, key,
			special = jQuery.event.special,
			i = 0;

		for ( ; (elem = elems[ i ]) !== undefined; i++ ) {
			if ( jQuery.acceptData( elem ) ) {
				key = elem[ data_priv.expando ];

				if ( key && (data = data_priv.cache[ key ]) ) {
					if ( data.events ) {
						for ( type in data.events ) {
							if ( special[ type ] ) {
								jQuery.event.remove( elem, type );

							// This is a shortcut to avoid jQuery.event.remove's overhead
							} else {
								jQuery.removeEvent( elem, type, data.handle );
							}
						}
					}
					if ( data_priv.cache[ key ] ) {
						// Discard any remaining `private` data
						delete data_priv.cache[ key ];
					}
				}
			}
			// Discard any remaining `user` data
			delete data_user.cache[ elem[ data_user.expando ] ];
		}
	}
});

jQuery.fn.extend({
	text: function( value ) {
		return access( this, function( value ) {
			return value === undefined ?
				jQuery.text( this ) :
				this.empty().each(function() {
					if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
						this.textContent = value;
					}
				});
		}, null, value, arguments.length );
	},

	append: function() {
		return this.domManip( arguments, function( elem ) {
			if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
				var target = manipulationTarget( this, elem );
				target.appendChild( elem );
			}
		});
	},

	prepend: function() {
		return this.domManip( arguments, function( elem ) {
			if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
				var target = manipulationTarget( this, elem );
				target.insertBefore( elem, target.firstChild );
			}
		});
	},

	before: function() {
		return this.domManip( arguments, function( elem ) {
			if ( this.parentNode ) {
				this.parentNode.insertBefore( elem, this );
			}
		});
	},

	after: function() {
		return this.domManip( arguments, function( elem ) {
			if ( this.parentNode ) {
				this.parentNode.insertBefore( elem, this.nextSibling );
			}
		});
	},

	remove: function( selector, keepData /* Internal Use Only */ ) {
		var elem,
			elems = selector ? jQuery.filter( selector, this ) : this,
			i = 0;

		for ( ; (elem = elems[i]) != null; i++ ) {
			if ( !keepData && elem.nodeType === 1 ) {
				jQuery.cleanData( getAll( elem ) );
			}

			if ( elem.parentNode ) {
				if ( keepData && jQuery.contains( elem.ownerDocument, elem ) ) {
					setGlobalEval( getAll( elem, "script" ) );
				}
				elem.parentNode.removeChild( elem );
			}
		}

		return this;
	},

	empty: function() {
		var elem,
			i = 0;

		for ( ; (elem = this[i]) != null; i++ ) {
			if ( elem.nodeType === 1 ) {

				// Prevent memory leaks
				jQuery.cleanData( getAll( elem, false ) );

				// Remove any remaining nodes
				elem.textContent = "";
			}
		}

		return this;
	},

	clone: function( dataAndEvents, deepDataAndEvents ) {
		dataAndEvents = dataAndEvents == null ? false : dataAndEvents;
		deepDataAndEvents = deepDataAndEvents == null ? dataAndEvents : deepDataAndEvents;

		return this.map(function() {
			return jQuery.clone( this, dataAndEvents, deepDataAndEvents );
		});
	},

	html: function( value ) {
		return access( this, function( value ) {
			var elem = this[ 0 ] || {},
				i = 0,
				l = this.length;

			if ( value === undefined && elem.nodeType === 1 ) {
				return elem.innerHTML;
			}

			// See if we can take a shortcut and just use innerHTML
			if ( typeof value === "string" && !rnoInnerhtml.test( value ) &&
				!wrapMap[ ( rtagName.exec( value ) || [ "", "" ] )[ 1 ].toLowerCase() ] ) {

				value = value.replace( rxhtmlTag, "<$1></$2>" );

				try {
					for ( ; i < l; i++ ) {
						elem = this[ i ] || {};

						// Remove element nodes and prevent memory leaks
						if ( elem.nodeType === 1 ) {
							jQuery.cleanData( getAll( elem, false ) );
							elem.innerHTML = value;
						}
					}

					elem = 0;

				// If using innerHTML throws an exception, use the fallback method
				} catch( e ) {}
			}

			if ( elem ) {
				this.empty().append( value );
			}
		}, null, value, arguments.length );
	},

	replaceWith: function() {
		var arg = arguments[ 0 ];

		// Make the changes, replacing each context element with the new content
		this.domManip( arguments, function( elem ) {
			arg = this.parentNode;

			jQuery.cleanData( getAll( this ) );

			if ( arg ) {
				arg.replaceChild( elem, this );
			}
		});

		// Force removal if there was no new content (e.g., from empty arguments)
		return arg && (arg.length || arg.nodeType) ? this : this.remove();
	},

	detach: function( selector ) {
		return this.remove( selector, true );
	},

	domManip: function( args, callback ) {

		// Flatten any nested arrays
		args = concat.apply( [], args );

		var fragment, first, scripts, hasScripts, node, doc,
			i = 0,
			l = this.length,
			set = this,
			iNoClone = l - 1,
			value = args[ 0 ],
			isFunction = jQuery.isFunction( value );

		// We can't cloneNode fragments that contain checked, in WebKit
		if ( isFunction ||
				( l > 1 && typeof value === "string" &&
					!support.checkClone && rchecked.test( value ) ) ) {
			return this.each(function( index ) {
				var self = set.eq( index );
				if ( isFunction ) {
					args[ 0 ] = value.call( this, index, self.html() );
				}
				self.domManip( args, callback );
			});
		}

		if ( l ) {
			fragment = jQuery.buildFragment( args, this[ 0 ].ownerDocument, false, this );
			first = fragment.firstChild;

			if ( fragment.childNodes.length === 1 ) {
				fragment = first;
			}

			if ( first ) {
				scripts = jQuery.map( getAll( fragment, "script" ), disableScript );
				hasScripts = scripts.length;

				// Use the original fragment for the last item instead of the first because it can end up
				// being emptied incorrectly in certain situations (#8070).
				for ( ; i < l; i++ ) {
					node = fragment;

					if ( i !== iNoClone ) {
						node = jQuery.clone( node, true, true );

						// Keep references to cloned scripts for later restoration
						if ( hasScripts ) {
							// Support: QtWebKit
							// jQuery.merge because push.apply(_, arraylike) throws
							jQuery.merge( scripts, getAll( node, "script" ) );
						}
					}

					callback.call( this[ i ], node, i );
				}

				if ( hasScripts ) {
					doc = scripts[ scripts.length - 1 ].ownerDocument;

					// Reenable scripts
					jQuery.map( scripts, restoreScript );

					// Evaluate executable scripts on first document insertion
					for ( i = 0; i < hasScripts; i++ ) {
						node = scripts[ i ];
						if ( rscriptType.test( node.type || "" ) &&
							!data_priv.access( node, "globalEval" ) && jQuery.contains( doc, node ) ) {

							if ( node.src ) {
								// Optional AJAX dependency, but won't run scripts if not present
								if ( jQuery._evalUrl ) {
									jQuery._evalUrl( node.src );
								}
							} else {
								jQuery.globalEval( node.textContent.replace( rcleanScript, "" ) );
							}
						}
					}
				}
			}
		}

		return this;
	}
});

jQuery.each({
	appendTo: "append",
	prependTo: "prepend",
	insertBefore: "before",
	insertAfter: "after",
	replaceAll: "replaceWith"
}, function( name, original ) {
	jQuery.fn[ name ] = function( selector ) {
		var elems,
			ret = [],
			insert = jQuery( selector ),
			last = insert.length - 1,
			i = 0;

		for ( ; i <= last; i++ ) {
			elems = i === last ? this : this.clone( true );
			jQuery( insert[ i ] )[ original ]( elems );

			// Support: QtWebKit
			// .get() because push.apply(_, arraylike) throws
			push.apply( ret, elems.get() );
		}

		return this.pushStack( ret );
	};
});


var iframe,
	elemdisplay = {};

/**
 * Retrieve the actual display of a element
 * @param {String} name nodeName of the element
 * @param {Object} doc Document object
 */
// Called only from within defaultDisplay
function actualDisplay( name, doc ) {
	var style,
		elem = jQuery( doc.createElement( name ) ).appendTo( doc.body ),

		// getDefaultComputedStyle might be reliably used only on attached element
		display = window.getDefaultComputedStyle && ( style = window.getDefaultComputedStyle( elem[ 0 ] ) ) ?

			// Use of this method is a temporary fix (more like optimization) until something better comes along,
			// since it was removed from specification and supported only in FF
			style.display : jQuery.css( elem[ 0 ], "display" );

	// We don't have any data stored on the element,
	// so use "detach" method as fast way to get rid of the element
	elem.detach();

	return display;
}

/**
 * Try to determine the default display value of an element
 * @param {String} nodeName
 */
function defaultDisplay( nodeName ) {
	var doc = document,
		display = elemdisplay[ nodeName ];

	if ( !display ) {
		display = actualDisplay( nodeName, doc );

		// If the simple way fails, read from inside an iframe
		if ( display === "none" || !display ) {

			// Use the already-created iframe if possible
			iframe = (iframe || jQuery( "<iframe frameborder='0' width='0' height='0'/>" )).appendTo( doc.documentElement );

			// Always write a new HTML skeleton so Webkit and Firefox don't choke on reuse
			doc = iframe[ 0 ].contentDocument;

			// Support: IE
			doc.write();
			doc.close();

			display = actualDisplay( nodeName, doc );
			iframe.detach();
		}

		// Store the correct default display
		elemdisplay[ nodeName ] = display;
	}

	return display;
}
var rmargin = (/^margin/);

var rnumnonpx = new RegExp( "^(" + pnum + ")(?!px)[a-z%]+$", "i" );

var getStyles = function( elem ) {
		// Support: IE<=11+, Firefox<=30+ (#15098, #14150)
		// IE throws on elements created in popups
		// FF meanwhile throws on frame elements through "defaultView.getComputedStyle"
		if ( elem.ownerDocument.defaultView.opener ) {
			return elem.ownerDocument.defaultView.getComputedStyle( elem, null );
		}

		return window.getComputedStyle( elem, null );
	};



function curCSS( elem, name, computed ) {
	var width, minWidth, maxWidth, ret,
		style = elem.style;

	computed = computed || getStyles( elem );

	// Support: IE9
	// getPropertyValue is only needed for .css('filter') (#12537)
	if ( computed ) {
		ret = computed.getPropertyValue( name ) || computed[ name ];
	}

	if ( computed ) {

		if ( ret === "" && !jQuery.contains( elem.ownerDocument, elem ) ) {
			ret = jQuery.style( elem, name );
		}

		// Support: iOS < 6
		// A tribute to the "awesome hack by Dean Edwards"
		// iOS < 6 (at least) returns percentage for a larger set of values, but width seems to be reliably pixels
		// this is against the CSSOM draft spec: http://dev.w3.org/csswg/cssom/#resolved-values
		if ( rnumnonpx.test( ret ) && rmargin.test( name ) ) {

			// Remember the original values
			width = style.width;
			minWidth = style.minWidth;
			maxWidth = style.maxWidth;

			// Put in the new values to get a computed value out
			style.minWidth = style.maxWidth = style.width = ret;
			ret = computed.width;

			// Revert the changed values
			style.width = width;
			style.minWidth = minWidth;
			style.maxWidth = maxWidth;
		}
	}

	return ret !== undefined ?
		// Support: IE
		// IE returns zIndex value as an integer.
		ret + "" :
		ret;
}


function addGetHookIf( conditionFn, hookFn ) {
	// Define the hook, we'll check on the first run if it's really needed.
	return {
		get: function() {
			if ( conditionFn() ) {
				// Hook not needed (or it's not possible to use it due
				// to missing dependency), remove it.
				delete this.get;
				return;
			}

			// Hook needed; redefine it so that the support test is not executed again.
			return (this.get = hookFn).apply( this, arguments );
		}
	};
}


(function() {
	var pixelPositionVal, boxSizingReliableVal,
		docElem = document.documentElement,
		container = document.createElement( "div" ),
		div = document.createElement( "div" );

	if ( !div.style ) {
		return;
	}

	// Support: IE9-11+
	// Style of cloned element affects source element cloned (#8908)
	div.style.backgroundClip = "content-box";
	div.cloneNode( true ).style.backgroundClip = "";
	support.clearCloneStyle = div.style.backgroundClip === "content-box";

	container.style.cssText = "border:0;width:0;height:0;top:0;left:-9999px;margin-top:1px;" +
		"position:absolute";
	container.appendChild( div );

	// Executing both pixelPosition & boxSizingReliable tests require only one layout
	// so they're executed at the same time to save the second computation.
	function computePixelPositionAndBoxSizingReliable() {
		div.style.cssText =
			// Support: Firefox<29, Android 2.3
			// Vendor-prefix box-sizing
			"-webkit-box-sizing:border-box;-moz-box-sizing:border-box;" +
			"box-sizing:border-box;display:block;margin-top:1%;top:1%;" +
			"border:1px;padding:1px;width:4px;position:absolute";
		div.innerHTML = "";
		docElem.appendChild( container );

		var divStyle = window.getComputedStyle( div, null );
		pixelPositionVal = divStyle.top !== "1%";
		boxSizingReliableVal = divStyle.width === "4px";

		docElem.removeChild( container );
	}

	// Support: node.js jsdom
	// Don't assume that getComputedStyle is a property of the global object
	if ( window.getComputedStyle ) {
		jQuery.extend( support, {
			pixelPosition: function() {

				// This test is executed only once but we still do memoizing
				// since we can use the boxSizingReliable pre-computing.
				// No need to check if the test was already performed, though.
				computePixelPositionAndBoxSizingReliable();
				return pixelPositionVal;
			},
			boxSizingReliable: function() {
				if ( boxSizingReliableVal == null ) {
					computePixelPositionAndBoxSizingReliable();
				}
				return boxSizingReliableVal;
			},
			reliableMarginRight: function() {

				// Support: Android 2.3
				// Check if div with explicit width and no margin-right incorrectly
				// gets computed margin-right based on width of container. (#3333)
				// WebKit Bug 13343 - getComputedStyle returns wrong value for margin-right
				// This support function is only executed once so no memoizing is needed.
				var ret,
					marginDiv = div.appendChild( document.createElement( "div" ) );

				// Reset CSS: box-sizing; display; margin; border; padding
				marginDiv.style.cssText = div.style.cssText =
					// Support: Firefox<29, Android 2.3
					// Vendor-prefix box-sizing
					"-webkit-box-sizing:content-box;-moz-box-sizing:content-box;" +
					"box-sizing:content-box;display:block;margin:0;border:0;padding:0";
				marginDiv.style.marginRight = marginDiv.style.width = "0";
				div.style.width = "1px";
				docElem.appendChild( container );

				ret = !parseFloat( window.getComputedStyle( marginDiv, null ).marginRight );

				docElem.removeChild( container );
				div.removeChild( marginDiv );

				return ret;
			}
		});
	}
})();


// A method for quickly swapping in/out CSS properties to get correct calculations.
jQuery.swap = function( elem, options, callback, args ) {
	var ret, name,
		old = {};

	// Remember the old values, and insert the new ones
	for ( name in options ) {
		old[ name ] = elem.style[ name ];
		elem.style[ name ] = options[ name ];
	}

	ret = callback.apply( elem, args || [] );

	// Revert the old values
	for ( name in options ) {
		elem.style[ name ] = old[ name ];
	}

	return ret;
};


var
	// Swappable if display is none or starts with table except "table", "table-cell", or "table-caption"
	// See here for display values: https://developer.mozilla.org/en-US/docs/CSS/display
	rdisplayswap = /^(none|table(?!-c[ea]).+)/,
	rnumsplit = new RegExp( "^(" + pnum + ")(.*)$", "i" ),
	rrelNum = new RegExp( "^([+-])=(" + pnum + ")", "i" ),

	cssShow = { position: "absolute", visibility: "hidden", display: "block" },
	cssNormalTransform = {
		letterSpacing: "0",
		fontWeight: "400"
	},

	cssPrefixes = [ "Webkit", "O", "Moz", "ms" ];

// Return a css property mapped to a potentially vendor prefixed property
function vendorPropName( style, name ) {

	// Shortcut for names that are not vendor prefixed
	if ( name in style ) {
		return name;
	}

	// Check for vendor prefixed names
	var capName = name[0].toUpperCase() + name.slice(1),
		origName = name,
		i = cssPrefixes.length;

	while ( i-- ) {
		name = cssPrefixes[ i ] + capName;
		if ( name in style ) {
			return name;
		}
	}

	return origName;
}

function setPositiveNumber( elem, value, subtract ) {
	var matches = rnumsplit.exec( value );
	return matches ?
		// Guard against undefined "subtract", e.g., when used as in cssHooks
		Math.max( 0, matches[ 1 ] - ( subtract || 0 ) ) + ( matches[ 2 ] || "px" ) :
		value;
}

function augmentWidthOrHeight( elem, name, extra, isBorderBox, styles ) {
	var i = extra === ( isBorderBox ? "border" : "content" ) ?
		// If we already have the right measurement, avoid augmentation
		4 :
		// Otherwise initialize for horizontal or vertical properties
		name === "width" ? 1 : 0,

		val = 0;

	for ( ; i < 4; i += 2 ) {
		// Both box models exclude margin, so add it if we want it
		if ( extra === "margin" ) {
			val += jQuery.css( elem, extra + cssExpand[ i ], true, styles );
		}

		if ( isBorderBox ) {
			// border-box includes padding, so remove it if we want content
			if ( extra === "content" ) {
				val -= jQuery.css( elem, "padding" + cssExpand[ i ], true, styles );
			}

			// At this point, extra isn't border nor margin, so remove border
			if ( extra !== "margin" ) {
				val -= jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );
			}
		} else {
			// At this point, extra isn't content, so add padding
			val += jQuery.css( elem, "padding" + cssExpand[ i ], true, styles );

			// At this point, extra isn't content nor padding, so add border
			if ( extra !== "padding" ) {
				val += jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );
			}
		}
	}

	return val;
}

function getWidthOrHeight( elem, name, extra ) {

	// Start with offset property, which is equivalent to the border-box value
	var valueIsBorderBox = true,
		val = name === "width" ? elem.offsetWidth : elem.offsetHeight,
		styles = getStyles( elem ),
		isBorderBox = jQuery.css( elem, "boxSizing", false, styles ) === "border-box";

	// Some non-html elements return undefined for offsetWidth, so check for null/undefined
	// svg - https://bugzilla.mozilla.org/show_bug.cgi?id=649285
	// MathML - https://bugzilla.mozilla.org/show_bug.cgi?id=491668
	if ( val <= 0 || val == null ) {
		// Fall back to computed then uncomputed css if necessary
		val = curCSS( elem, name, styles );
		if ( val < 0 || val == null ) {
			val = elem.style[ name ];
		}

		// Computed unit is not pixels. Stop here and return.
		if ( rnumnonpx.test(val) ) {
			return val;
		}

		// Check for style in case a browser which returns unreliable values
		// for getComputedStyle silently falls back to the reliable elem.style
		valueIsBorderBox = isBorderBox &&
			( support.boxSizingReliable() || val === elem.style[ name ] );

		// Normalize "", auto, and prepare for extra
		val = parseFloat( val ) || 0;
	}

	// Use the active box-sizing model to add/subtract irrelevant styles
	return ( val +
		augmentWidthOrHeight(
			elem,
			name,
			extra || ( isBorderBox ? "border" : "content" ),
			valueIsBorderBox,
			styles
		)
	) + "px";
}

function showHide( elements, show ) {
	var display, elem, hidden,
		values = [],
		index = 0,
		length = elements.length;

	for ( ; index < length; index++ ) {
		elem = elements[ index ];
		if ( !elem.style ) {
			continue;
		}

		values[ index ] = data_priv.get( elem, "olddisplay" );
		display = elem.style.display;
		if ( show ) {
			// Reset the inline display of this element to learn if it is
			// being hidden by cascaded rules or not
			if ( !values[ index ] && display === "none" ) {
				elem.style.display = "";
			}

			// Set elements which have been overridden with display: none
			// in a stylesheet to whatever the default browser style is
			// for such an element
			if ( elem.style.display === "" && isHidden( elem ) ) {
				values[ index ] = data_priv.access( elem, "olddisplay", defaultDisplay(elem.nodeName) );
			}
		} else {
			hidden = isHidden( elem );

			if ( display !== "none" || !hidden ) {
				data_priv.set( elem, "olddisplay", hidden ? display : jQuery.css( elem, "display" ) );
			}
		}
	}

	// Set the display of most of the elements in a second loop
	// to avoid the constant reflow
	for ( index = 0; index < length; index++ ) {
		elem = elements[ index ];
		if ( !elem.style ) {
			continue;
		}
		if ( !show || elem.style.display === "none" || elem.style.display === "" ) {
			elem.style.display = show ? values[ index ] || "" : "none";
		}
	}

	return elements;
}

jQuery.extend({

	// Add in style property hooks for overriding the default
	// behavior of getting and setting a style property
	cssHooks: {
		opacity: {
			get: function( elem, computed ) {
				if ( computed ) {

					// We should always get a number back from opacity
					var ret = curCSS( elem, "opacity" );
					return ret === "" ? "1" : ret;
				}
			}
		}
	},

	// Don't automatically add "px" to these possibly-unitless properties
	cssNumber: {
		"columnCount": true,
		"fillOpacity": true,
		"flexGrow": true,
		"flexShrink": true,
		"fontWeight": true,
		"lineHeight": true,
		"opacity": true,
		"order": true,
		"orphans": true,
		"widows": true,
		"zIndex": true,
		"zoom": true
	},

	// Add in properties whose names you wish to fix before
	// setting or getting the value
	cssProps: {
		"float": "cssFloat"
	},

	// Get and set the style property on a DOM Node
	style: function( elem, name, value, extra ) {

		// Don't set styles on text and comment nodes
		if ( !elem || elem.nodeType === 3 || elem.nodeType === 8 || !elem.style ) {
			return;
		}

		// Make sure that we're working with the right name
		var ret, type, hooks,
			origName = jQuery.camelCase( name ),
			style = elem.style;

		name = jQuery.cssProps[ origName ] || ( jQuery.cssProps[ origName ] = vendorPropName( style, origName ) );

		// Gets hook for the prefixed version, then unprefixed version
		hooks = jQuery.cssHooks[ name ] || jQuery.cssHooks[ origName ];

		// Check if we're setting a value
		if ( value !== undefined ) {
			type = typeof value;

			// Convert "+=" or "-=" to relative numbers (#7345)
			if ( type === "string" && (ret = rrelNum.exec( value )) ) {
				value = ( ret[1] + 1 ) * ret[2] + parseFloat( jQuery.css( elem, name ) );
				// Fixes bug #9237
				type = "number";
			}

			// Make sure that null and NaN values aren't set (#7116)
			if ( value == null || value !== value ) {
				return;
			}

			// If a number, add 'px' to the (except for certain CSS properties)
			if ( type === "number" && !jQuery.cssNumber[ origName ] ) {
				value += "px";
			}

			// Support: IE9-11+
			// background-* props affect original clone's values
			if ( !support.clearCloneStyle && value === "" && name.indexOf( "background" ) === 0 ) {
				style[ name ] = "inherit";
			}

			// If a hook was provided, use that value, otherwise just set the specified value
			if ( !hooks || !("set" in hooks) || (value = hooks.set( elem, value, extra )) !== undefined ) {
				style[ name ] = value;
			}

		} else {
			// If a hook was provided get the non-computed value from there
			if ( hooks && "get" in hooks && (ret = hooks.get( elem, false, extra )) !== undefined ) {
				return ret;
			}

			// Otherwise just get the value from the style object
			return style[ name ];
		}
	},

	css: function( elem, name, extra, styles ) {
		var val, num, hooks,
			origName = jQuery.camelCase( name );

		// Make sure that we're working with the right name
		name = jQuery.cssProps[ origName ] || ( jQuery.cssProps[ origName ] = vendorPropName( elem.style, origName ) );

		// Try prefixed name followed by the unprefixed name
		hooks = jQuery.cssHooks[ name ] || jQuery.cssHooks[ origName ];

		// If a hook was provided get the computed value from there
		if ( hooks && "get" in hooks ) {
			val = hooks.get( elem, true, extra );
		}

		// Otherwise, if a way to get the computed value exists, use that
		if ( val === undefined ) {
			val = curCSS( elem, name, styles );
		}

		// Convert "normal" to computed value
		if ( val === "normal" && name in cssNormalTransform ) {
			val = cssNormalTransform[ name ];
		}

		// Make numeric if forced or a qualifier was provided and val looks numeric
		if ( extra === "" || extra ) {
			num = parseFloat( val );
			return extra === true || jQuery.isNumeric( num ) ? num || 0 : val;
		}
		return val;
	}
});

jQuery.each([ "height", "width" ], function( i, name ) {
	jQuery.cssHooks[ name ] = {
		get: function( elem, computed, extra ) {
			if ( computed ) {

				// Certain elements can have dimension info if we invisibly show them
				// but it must have a current display style that would benefit
				return rdisplayswap.test( jQuery.css( elem, "display" ) ) && elem.offsetWidth === 0 ?
					jQuery.swap( elem, cssShow, function() {
						return getWidthOrHeight( elem, name, extra );
					}) :
					getWidthOrHeight( elem, name, extra );
			}
		},

		set: function( elem, value, extra ) {
			var styles = extra && getStyles( elem );
			return setPositiveNumber( elem, value, extra ?
				augmentWidthOrHeight(
					elem,
					name,
					extra,
					jQuery.css( elem, "boxSizing", false, styles ) === "border-box",
					styles
				) : 0
			);
		}
	};
});

// Support: Android 2.3
jQuery.cssHooks.marginRight = addGetHookIf( support.reliableMarginRight,
	function( elem, computed ) {
		if ( computed ) {
			return jQuery.swap( elem, { "display": "inline-block" },
				curCSS, [ elem, "marginRight" ] );
		}
	}
);

// These hooks are used by animate to expand properties
jQuery.each({
	margin: "",
	padding: "",
	border: "Width"
}, function( prefix, suffix ) {
	jQuery.cssHooks[ prefix + suffix ] = {
		expand: function( value ) {
			var i = 0,
				expanded = {},

				// Assumes a single number if not a string
				parts = typeof value === "string" ? value.split(" ") : [ value ];

			for ( ; i < 4; i++ ) {
				expanded[ prefix + cssExpand[ i ] + suffix ] =
					parts[ i ] || parts[ i - 2 ] || parts[ 0 ];
			}

			return expanded;
		}
	};

	if ( !rmargin.test( prefix ) ) {
		jQuery.cssHooks[ prefix + suffix ].set = setPositiveNumber;
	}
});

jQuery.fn.extend({
	css: function( name, value ) {
		return access( this, function( elem, name, value ) {
			var styles, len,
				map = {},
				i = 0;

			if ( jQuery.isArray( name ) ) {
				styles = getStyles( elem );
				len = name.length;

				for ( ; i < len; i++ ) {
					map[ name[ i ] ] = jQuery.css( elem, name[ i ], false, styles );
				}

				return map;
			}

			return value !== undefined ?
				jQuery.style( elem, name, value ) :
				jQuery.css( elem, name );
		}, name, value, arguments.length > 1 );
	},
	show: function() {
		return showHide( this, true );
	},
	hide: function() {
		return showHide( this );
	},
	toggle: function( state ) {
		if ( typeof state === "boolean" ) {
			return state ? this.show() : this.hide();
		}

		return this.each(function() {
			if ( isHidden( this ) ) {
				jQuery( this ).show();
			} else {
				jQuery( this ).hide();
			}
		});
	}
});


function Tween( elem, options, prop, end, easing ) {
	return new Tween.prototype.init( elem, options, prop, end, easing );
}
jQuery.Tween = Tween;

Tween.prototype = {
	constructor: Tween,
	init: function( elem, options, prop, end, easing, unit ) {
		this.elem = elem;
		this.prop = prop;
		this.easing = easing || "swing";
		this.options = options;
		this.start = this.now = this.cur();
		this.end = end;
		this.unit = unit || ( jQuery.cssNumber[ prop ] ? "" : "px" );
	},
	cur: function() {
		var hooks = Tween.propHooks[ this.prop ];

		return hooks && hooks.get ?
			hooks.get( this ) :
			Tween.propHooks._default.get( this );
	},
	run: function( percent ) {
		var eased,
			hooks = Tween.propHooks[ this.prop ];

		if ( this.options.duration ) {
			this.pos = eased = jQuery.easing[ this.easing ](
				percent, this.options.duration * percent, 0, 1, this.options.duration
			);
		} else {
			this.pos = eased = percent;
		}
		this.now = ( this.end - this.start ) * eased + this.start;

		if ( this.options.step ) {
			this.options.step.call( this.elem, this.now, this );
		}

		if ( hooks && hooks.set ) {
			hooks.set( this );
		} else {
			Tween.propHooks._default.set( this );
		}
		return this;
	}
};

Tween.prototype.init.prototype = Tween.prototype;

Tween.propHooks = {
	_default: {
		get: function( tween ) {
			var result;

			if ( tween.elem[ tween.prop ] != null &&
				(!tween.elem.style || tween.elem.style[ tween.prop ] == null) ) {
				return tween.elem[ tween.prop ];
			}

			// Passing an empty string as a 3rd parameter to .css will automatically
			// attempt a parseFloat and fallback to a string if the parse fails.
			// Simple values such as "10px" are parsed to Float;
			// complex values such as "rotate(1rad)" are returned as-is.
			result = jQuery.css( tween.elem, tween.prop, "" );
			// Empty strings, null, undefined and "auto" are converted to 0.
			return !result || result === "auto" ? 0 : result;
		},
		set: function( tween ) {
			// Use step hook for back compat.
			// Use cssHook if its there.
			// Use .style if available and use plain properties where available.
			if ( jQuery.fx.step[ tween.prop ] ) {
				jQuery.fx.step[ tween.prop ]( tween );
			} else if ( tween.elem.style && ( tween.elem.style[ jQuery.cssProps[ tween.prop ] ] != null || jQuery.cssHooks[ tween.prop ] ) ) {
				jQuery.style( tween.elem, tween.prop, tween.now + tween.unit );
			} else {
				tween.elem[ tween.prop ] = tween.now;
			}
		}
	}
};

// Support: IE9
// Panic based approach to setting things on disconnected nodes
Tween.propHooks.scrollTop = Tween.propHooks.scrollLeft = {
	set: function( tween ) {
		if ( tween.elem.nodeType && tween.elem.parentNode ) {
			tween.elem[ tween.prop ] = tween.now;
		}
	}
};

jQuery.easing = {
	linear: function( p ) {
		return p;
	},
	swing: function( p ) {
		return 0.5 - Math.cos( p * Math.PI ) / 2;
	}
};

jQuery.fx = Tween.prototype.init;

// Back Compat <1.8 extension point
jQuery.fx.step = {};




var
	fxNow, timerId,
	rfxtypes = /^(?:toggle|show|hide)$/,
	rfxnum = new RegExp( "^(?:([+-])=|)(" + pnum + ")([a-z%]*)$", "i" ),
	rrun = /queueHooks$/,
	animationPrefilters = [ defaultPrefilter ],
	tweeners = {
		"*": [ function( prop, value ) {
			var tween = this.createTween( prop, value ),
				target = tween.cur(),
				parts = rfxnum.exec( value ),
				unit = parts && parts[ 3 ] || ( jQuery.cssNumber[ prop ] ? "" : "px" ),

				// Starting value computation is required for potential unit mismatches
				start = ( jQuery.cssNumber[ prop ] || unit !== "px" && +target ) &&
					rfxnum.exec( jQuery.css( tween.elem, prop ) ),
				scale = 1,
				maxIterations = 20;

			if ( start && start[ 3 ] !== unit ) {
				// Trust units reported by jQuery.css
				unit = unit || start[ 3 ];

				// Make sure we update the tween properties later on
				parts = parts || [];

				// Iteratively approximate from a nonzero starting point
				start = +target || 1;

				do {
					// If previous iteration zeroed out, double until we get *something*.
					// Use string for doubling so we don't accidentally see scale as unchanged below
					scale = scale || ".5";

					// Adjust and apply
					start = start / scale;
					jQuery.style( tween.elem, prop, start + unit );

				// Update scale, tolerating zero or NaN from tween.cur(),
				// break the loop if scale is unchanged or perfect, or if we've just had enough
				} while ( scale !== (scale = tween.cur() / target) && scale !== 1 && --maxIterations );
			}

			// Update tween properties
			if ( parts ) {
				start = tween.start = +start || +target || 0;
				tween.unit = unit;
				// If a +=/-= token was provided, we're doing a relative animation
				tween.end = parts[ 1 ] ?
					start + ( parts[ 1 ] + 1 ) * parts[ 2 ] :
					+parts[ 2 ];
			}

			return tween;
		} ]
	};

// Animations created synchronously will run synchronously
function createFxNow() {
	setTimeout(function() {
		fxNow = undefined;
	});
	return ( fxNow = jQuery.now() );
}

// Generate parameters to create a standard animation
function genFx( type, includeWidth ) {
	var which,
		i = 0,
		attrs = { height: type };

	// If we include width, step value is 1 to do all cssExpand values,
	// otherwise step value is 2 to skip over Left and Right
	includeWidth = includeWidth ? 1 : 0;
	for ( ; i < 4 ; i += 2 - includeWidth ) {
		which = cssExpand[ i ];
		attrs[ "margin" + which ] = attrs[ "padding" + which ] = type;
	}

	if ( includeWidth ) {
		attrs.opacity = attrs.width = type;
	}

	return attrs;
}

function createTween( value, prop, animation ) {
	var tween,
		collection = ( tweeners[ prop ] || [] ).concat( tweeners[ "*" ] ),
		index = 0,
		length = collection.length;
	for ( ; index < length; index++ ) {
		if ( (tween = collection[ index ].call( animation, prop, value )) ) {

			// We're done with this property
			return tween;
		}
	}
}

function defaultPrefilter( elem, props, opts ) {
	/* jshint validthis: true */
	var prop, value, toggle, tween, hooks, oldfire, display, checkDisplay,
		anim = this,
		orig = {},
		style = elem.style,
		hidden = elem.nodeType && isHidden( elem ),
		dataShow = data_priv.get( elem, "fxshow" );

	// Handle queue: false promises
	if ( !opts.queue ) {
		hooks = jQuery._queueHooks( elem, "fx" );
		if ( hooks.unqueued == null ) {
			hooks.unqueued = 0;
			oldfire = hooks.empty.fire;
			hooks.empty.fire = function() {
				if ( !hooks.unqueued ) {
					oldfire();
				}
			};
		}
		hooks.unqueued++;

		anim.always(function() {
			// Ensure the complete handler is called before this completes
			anim.always(function() {
				hooks.unqueued--;
				if ( !jQuery.queue( elem, "fx" ).length ) {
					hooks.empty.fire();
				}
			});
		});
	}

	// Height/width overflow pass
	if ( elem.nodeType === 1 && ( "height" in props || "width" in props ) ) {
		// Make sure that nothing sneaks out
		// Record all 3 overflow attributes because IE9-10 do not
		// change the overflow attribute when overflowX and
		// overflowY are set to the same value
		opts.overflow = [ style.overflow, style.overflowX, style.overflowY ];

		// Set display property to inline-block for height/width
		// animations on inline elements that are having width/height animated
		display = jQuery.css( elem, "display" );

		// Test default display if display is currently "none"
		checkDisplay = display === "none" ?
			data_priv.get( elem, "olddisplay" ) || defaultDisplay( elem.nodeName ) : display;

		if ( checkDisplay === "inline" && jQuery.css( elem, "float" ) === "none" ) {
			style.display = "inline-block";
		}
	}

	if ( opts.overflow ) {
		style.overflow = "hidden";
		anim.always(function() {
			style.overflow = opts.overflow[ 0 ];
			style.overflowX = opts.overflow[ 1 ];
			style.overflowY = opts.overflow[ 2 ];
		});
	}

	// show/hide pass
	for ( prop in props ) {
		value = props[ prop ];
		if ( rfxtypes.exec( value ) ) {
			delete props[ prop ];
			toggle = toggle || value === "toggle";
			if ( value === ( hidden ? "hide" : "show" ) ) {

				// If there is dataShow left over from a stopped hide or show and we are going to proceed with show, we should pretend to be hidden
				if ( value === "show" && dataShow && dataShow[ prop ] !== undefined ) {
					hidden = true;
				} else {
					continue;
				}
			}
			orig[ prop ] = dataShow && dataShow[ prop ] || jQuery.style( elem, prop );

		// Any non-fx value stops us from restoring the original display value
		} else {
			display = undefined;
		}
	}

	if ( !jQuery.isEmptyObject( orig ) ) {
		if ( dataShow ) {
			if ( "hidden" in dataShow ) {
				hidden = dataShow.hidden;
			}
		} else {
			dataShow = data_priv.access( elem, "fxshow", {} );
		}

		// Store state if its toggle - enables .stop().toggle() to "reverse"
		if ( toggle ) {
			dataShow.hidden = !hidden;
		}
		if ( hidden ) {
			jQuery( elem ).show();
		} else {
			anim.done(function() {
				jQuery( elem ).hide();
			});
		}
		anim.done(function() {
			var prop;

			data_priv.remove( elem, "fxshow" );
			for ( prop in orig ) {
				jQuery.style( elem, prop, orig[ prop ] );
			}
		});
		for ( prop in orig ) {
			tween = createTween( hidden ? dataShow[ prop ] : 0, prop, anim );

			if ( !( prop in dataShow ) ) {
				dataShow[ prop ] = tween.start;
				if ( hidden ) {
					tween.end = tween.start;
					tween.start = prop === "width" || prop === "height" ? 1 : 0;
				}
			}
		}

	// If this is a noop like .hide().hide(), restore an overwritten display value
	} else if ( (display === "none" ? defaultDisplay( elem.nodeName ) : display) === "inline" ) {
		style.display = display;
	}
}

function propFilter( props, specialEasing ) {
	var index, name, easing, value, hooks;

	// camelCase, specialEasing and expand cssHook pass
	for ( index in props ) {
		name = jQuery.camelCase( index );
		easing = specialEasing[ name ];
		value = props[ index ];
		if ( jQuery.isArray( value ) ) {
			easing = value[ 1 ];
			value = props[ index ] = value[ 0 ];
		}

		if ( index !== name ) {
			props[ name ] = value;
			delete props[ index ];
		}

		hooks = jQuery.cssHooks[ name ];
		if ( hooks && "expand" in hooks ) {
			value = hooks.expand( value );
			delete props[ name ];

			// Not quite $.extend, this won't overwrite existing keys.
			// Reusing 'index' because we have the correct "name"
			for ( index in value ) {
				if ( !( index in props ) ) {
					props[ index ] = value[ index ];
					specialEasing[ index ] = easing;
				}
			}
		} else {
			specialEasing[ name ] = easing;
		}
	}
}

function Animation( elem, properties, options ) {
	var result,
		stopped,
		index = 0,
		length = animationPrefilters.length,
		deferred = jQuery.Deferred().always( function() {
			// Don't match elem in the :animated selector
			delete tick.elem;
		}),
		tick = function() {
			if ( stopped ) {
				return false;
			}
			var currentTime = fxNow || createFxNow(),
				remaining = Math.max( 0, animation.startTime + animation.duration - currentTime ),
				// Support: Android 2.3
				// Archaic crash bug won't allow us to use `1 - ( 0.5 || 0 )` (#12497)
				temp = remaining / animation.duration || 0,
				percent = 1 - temp,
				index = 0,
				length = animation.tweens.length;

			for ( ; index < length ; index++ ) {
				animation.tweens[ index ].run( percent );
			}

			deferred.notifyWith( elem, [ animation, percent, remaining ]);

			if ( percent < 1 && length ) {
				return remaining;
			} else {
				deferred.resolveWith( elem, [ animation ] );
				return false;
			}
		},
		animation = deferred.promise({
			elem: elem,
			props: jQuery.extend( {}, properties ),
			opts: jQuery.extend( true, { specialEasing: {} }, options ),
			originalProperties: properties,
			originalOptions: options,
			startTime: fxNow || createFxNow(),
			duration: options.duration,
			tweens: [],
			createTween: function( prop, end ) {
				var tween = jQuery.Tween( elem, animation.opts, prop, end,
						animation.opts.specialEasing[ prop ] || animation.opts.easing );
				animation.tweens.push( tween );
				return tween;
			},
			stop: function( gotoEnd ) {
				var index = 0,
					// If we are going to the end, we want to run all the tweens
					// otherwise we skip this part
					length = gotoEnd ? animation.tweens.length : 0;
				if ( stopped ) {
					return this;
				}
				stopped = true;
				for ( ; index < length ; index++ ) {
					animation.tweens[ index ].run( 1 );
				}

				// Resolve when we played the last frame; otherwise, reject
				if ( gotoEnd ) {
					deferred.resolveWith( elem, [ animation, gotoEnd ] );
				} else {
					deferred.rejectWith( elem, [ animation, gotoEnd ] );
				}
				return this;
			}
		}),
		props = animation.props;

	propFilter( props, animation.opts.specialEasing );

	for ( ; index < length ; index++ ) {
		result = animationPrefilters[ index ].call( animation, elem, props, animation.opts );
		if ( result ) {
			return result;
		}
	}

	jQuery.map( props, createTween, animation );

	if ( jQuery.isFunction( animation.opts.start ) ) {
		animation.opts.start.call( elem, animation );
	}

	jQuery.fx.timer(
		jQuery.extend( tick, {
			elem: elem,
			anim: animation,
			queue: animation.opts.queue
		})
	);

	// attach callbacks from options
	return animation.progress( animation.opts.progress )
		.done( animation.opts.done, animation.opts.complete )
		.fail( animation.opts.fail )
		.always( animation.opts.always );
}

jQuery.Animation = jQuery.extend( Animation, {

	tweener: function( props, callback ) {
		if ( jQuery.isFunction( props ) ) {
			callback = props;
			props = [ "*" ];
		} else {
			props = props.split(" ");
		}

		var prop,
			index = 0,
			length = props.length;

		for ( ; index < length ; index++ ) {
			prop = props[ index ];
			tweeners[ prop ] = tweeners[ prop ] || [];
			tweeners[ prop ].unshift( callback );
		}
	},

	prefilter: function( callback, prepend ) {
		if ( prepend ) {
			animationPrefilters.unshift( callback );
		} else {
			animationPrefilters.push( callback );
		}
	}
});

jQuery.speed = function( speed, easing, fn ) {
	var opt = speed && typeof speed === "object" ? jQuery.extend( {}, speed ) : {
		complete: fn || !fn && easing ||
			jQuery.isFunction( speed ) && speed,
		duration: speed,
		easing: fn && easing || easing && !jQuery.isFunction( easing ) && easing
	};

	opt.duration = jQuery.fx.off ? 0 : typeof opt.duration === "number" ? opt.duration :
		opt.duration in jQuery.fx.speeds ? jQuery.fx.speeds[ opt.duration ] : jQuery.fx.speeds._default;

	// Normalize opt.queue - true/undefined/null -> "fx"
	if ( opt.queue == null || opt.queue === true ) {
		opt.queue = "fx";
	}

	// Queueing
	opt.old = opt.complete;

	opt.complete = function() {
		if ( jQuery.isFunction( opt.old ) ) {
			opt.old.call( this );
		}

		if ( opt.queue ) {
			jQuery.dequeue( this, opt.queue );
		}
	};

	return opt;
};

jQuery.fn.extend({
	fadeTo: function( speed, to, easing, callback ) {

		// Show any hidden elements after setting opacity to 0
		return this.filter( isHidden ).css( "opacity", 0 ).show()

			// Animate to the value specified
			.end().animate({ opacity: to }, speed, easing, callback );
	},
	animate: function( prop, speed, easing, callback ) {
		var empty = jQuery.isEmptyObject( prop ),
			optall = jQuery.speed( speed, easing, callback ),
			doAnimation = function() {
				// Operate on a copy of prop so per-property easing won't be lost
				var anim = Animation( this, jQuery.extend( {}, prop ), optall );

				// Empty animations, or finishing resolves immediately
				if ( empty || data_priv.get( this, "finish" ) ) {
					anim.stop( true );
				}
			};
			doAnimation.finish = doAnimation;

		return empty || optall.queue === false ?
			this.each( doAnimation ) :
			this.queue( optall.queue, doAnimation );
	},
	stop: function( type, clearQueue, gotoEnd ) {
		var stopQueue = function( hooks ) {
			var stop = hooks.stop;
			delete hooks.stop;
			stop( gotoEnd );
		};

		if ( typeof type !== "string" ) {
			gotoEnd = clearQueue;
			clearQueue = type;
			type = undefined;
		}
		if ( clearQueue && type !== false ) {
			this.queue( type || "fx", [] );
		}

		return this.each(function() {
			var dequeue = true,
				index = type != null && type + "queueHooks",
				timers = jQuery.timers,
				data = data_priv.get( this );

			if ( index ) {
				if ( data[ index ] && data[ index ].stop ) {
					stopQueue( data[ index ] );
				}
			} else {
				for ( index in data ) {
					if ( data[ index ] && data[ index ].stop && rrun.test( index ) ) {
						stopQueue( data[ index ] );
					}
				}
			}

			for ( index = timers.length; index--; ) {
				if ( timers[ index ].elem === this && (type == null || timers[ index ].queue === type) ) {
					timers[ index ].anim.stop( gotoEnd );
					dequeue = false;
					timers.splice( index, 1 );
				}
			}

			// Start the next in the queue if the last step wasn't forced.
			// Timers currently will call their complete callbacks, which
			// will dequeue but only if they were gotoEnd.
			if ( dequeue || !gotoEnd ) {
				jQuery.dequeue( this, type );
			}
		});
	},
	finish: function( type ) {
		if ( type !== false ) {
			type = type || "fx";
		}
		return this.each(function() {
			var index,
				data = data_priv.get( this ),
				queue = data[ type + "queue" ],
				hooks = data[ type + "queueHooks" ],
				timers = jQuery.timers,
				length = queue ? queue.length : 0;

			// Enable finishing flag on private data
			data.finish = true;

			// Empty the queue first
			jQuery.queue( this, type, [] );

			if ( hooks && hooks.stop ) {
				hooks.stop.call( this, true );
			}

			// Look for any active animations, and finish them
			for ( index = timers.length; index--; ) {
				if ( timers[ index ].elem === this && timers[ index ].queue === type ) {
					timers[ index ].anim.stop( true );
					timers.splice( index, 1 );
				}
			}

			// Look for any animations in the old queue and finish them
			for ( index = 0; index < length; index++ ) {
				if ( queue[ index ] && queue[ index ].finish ) {
					queue[ index ].finish.call( this );
				}
			}

			// Turn off finishing flag
			delete data.finish;
		});
	}
});

jQuery.each([ "toggle", "show", "hide" ], function( i, name ) {
	var cssFn = jQuery.fn[ name ];
	jQuery.fn[ name ] = function( speed, easing, callback ) {
		return speed == null || typeof speed === "boolean" ?
			cssFn.apply( this, arguments ) :
			this.animate( genFx( name, true ), speed, easing, callback );
	};
});

// Generate shortcuts for custom animations
jQuery.each({
	slideDown: genFx("show"),
	slideUp: genFx("hide"),
	slideToggle: genFx("toggle"),
	fadeIn: { opacity: "show" },
	fadeOut: { opacity: "hide" },
	fadeToggle: { opacity: "toggle" }
}, function( name, props ) {
	jQuery.fn[ name ] = function( speed, easing, callback ) {
		return this.animate( props, speed, easing, callback );
	};
});

jQuery.timers = [];
jQuery.fx.tick = function() {
	var timer,
		i = 0,
		timers = jQuery.timers;

	fxNow = jQuery.now();

	for ( ; i < timers.length; i++ ) {
		timer = timers[ i ];
		// Checks the timer has not already been removed
		if ( !timer() && timers[ i ] === timer ) {
			timers.splice( i--, 1 );
		}
	}

	if ( !timers.length ) {
		jQuery.fx.stop();
	}
	fxNow = undefined;
};

jQuery.fx.timer = function( timer ) {
	jQuery.timers.push( timer );
	if ( timer() ) {
		jQuery.fx.start();
	} else {
		jQuery.timers.pop();
	}
};

jQuery.fx.interval = 13;

jQuery.fx.start = function() {
	if ( !timerId ) {
		timerId = setInterval( jQuery.fx.tick, jQuery.fx.interval );
	}
};

jQuery.fx.stop = function() {
	clearInterval( timerId );
	timerId = null;
};

jQuery.fx.speeds = {
	slow: 600,
	fast: 200,
	// Default speed
	_default: 400
};


// Based off of the plugin by Clint Helfers, with permission.
// http://blindsignals.com/index.php/2009/07/jquery-delay/
jQuery.fn.delay = function( time, type ) {
	time = jQuery.fx ? jQuery.fx.speeds[ time ] || time : time;
	type = type || "fx";

	return this.queue( type, function( next, hooks ) {
		var timeout = setTimeout( next, time );
		hooks.stop = function() {
			clearTimeout( timeout );
		};
	});
};


(function() {
	var input = document.createElement( "input" ),
		select = document.createElement( "select" ),
		opt = select.appendChild( document.createElement( "option" ) );

	input.type = "checkbox";

	// Support: iOS<=5.1, Android<=4.2+
	// Default value for a checkbox should be "on"
	support.checkOn = input.value !== "";

	// Support: IE<=11+
	// Must access selectedIndex to make default options select
	support.optSelected = opt.selected;

	// Support: Android<=2.3
	// Options inside disabled selects are incorrectly marked as disabled
	select.disabled = true;
	support.optDisabled = !opt.disabled;

	// Support: IE<=11+
	// An input loses its value after becoming a radio
	input = document.createElement( "input" );
	input.value = "t";
	input.type = "radio";
	support.radioValue = input.value === "t";
})();


var nodeHook, boolHook,
	attrHandle = jQuery.expr.attrHandle;

jQuery.fn.extend({
	attr: function( name, value ) {
		return access( this, jQuery.attr, name, value, arguments.length > 1 );
	},

	removeAttr: function( name ) {
		return this.each(function() {
			jQuery.removeAttr( this, name );
		});
	}
});

jQuery.extend({
	attr: function( elem, name, value ) {
		var hooks, ret,
			nType = elem.nodeType;

		// don't get/set attributes on text, comment and attribute nodes
		if ( !elem || nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		// Fallback to prop when attributes are not supported
		if ( typeof elem.getAttribute === strundefined ) {
			return jQuery.prop( elem, name, value );
		}

		// All attributes are lowercase
		// Grab necessary hook if one is defined
		if ( nType !== 1 || !jQuery.isXMLDoc( elem ) ) {
			name = name.toLowerCase();
			hooks = jQuery.attrHooks[ name ] ||
				( jQuery.expr.match.bool.test( name ) ? boolHook : nodeHook );
		}

		if ( value !== undefined ) {

			if ( value === null ) {
				jQuery.removeAttr( elem, name );

			} else if ( hooks && "set" in hooks && (ret = hooks.set( elem, value, name )) !== undefined ) {
				return ret;

			} else {
				elem.setAttribute( name, value + "" );
				return value;
			}

		} else if ( hooks && "get" in hooks && (ret = hooks.get( elem, name )) !== null ) {
			return ret;

		} else {
			ret = jQuery.find.attr( elem, name );

			// Non-existent attributes return null, we normalize to undefined
			return ret == null ?
				undefined :
				ret;
		}
	},

	removeAttr: function( elem, value ) {
		var name, propName,
			i = 0,
			attrNames = value && value.match( rnotwhite );

		if ( attrNames && elem.nodeType === 1 ) {
			while ( (name = attrNames[i++]) ) {
				propName = jQuery.propFix[ name ] || name;

				// Boolean attributes get special treatment (#10870)
				if ( jQuery.expr.match.bool.test( name ) ) {
					// Set corresponding property to false
					elem[ propName ] = false;
				}

				elem.removeAttribute( name );
			}
		}
	},

	attrHooks: {
		type: {
			set: function( elem, value ) {
				if ( !support.radioValue && value === "radio" &&
					jQuery.nodeName( elem, "input" ) ) {
					var val = elem.value;
					elem.setAttribute( "type", value );
					if ( val ) {
						elem.value = val;
					}
					return value;
				}
			}
		}
	}
});

// Hooks for boolean attributes
boolHook = {
	set: function( elem, value, name ) {
		if ( value === false ) {
			// Remove boolean attributes when set to false
			jQuery.removeAttr( elem, name );
		} else {
			elem.setAttribute( name, name );
		}
		return name;
	}
};
jQuery.each( jQuery.expr.match.bool.source.match( /\w+/g ), function( i, name ) {
	var getter = attrHandle[ name ] || jQuery.find.attr;

	attrHandle[ name ] = function( elem, name, isXML ) {
		var ret, handle;
		if ( !isXML ) {
			// Avoid an infinite loop by temporarily removing this function from the getter
			handle = attrHandle[ name ];
			attrHandle[ name ] = ret;
			ret = getter( elem, name, isXML ) != null ?
				name.toLowerCase() :
				null;
			attrHandle[ name ] = handle;
		}
		return ret;
	};
});




var rfocusable = /^(?:input|select|textarea|button)$/i;

jQuery.fn.extend({
	prop: function( name, value ) {
		return access( this, jQuery.prop, name, value, arguments.length > 1 );
	},

	removeProp: function( name ) {
		return this.each(function() {
			delete this[ jQuery.propFix[ name ] || name ];
		});
	}
});

jQuery.extend({
	propFix: {
		"for": "htmlFor",
		"class": "className"
	},

	prop: function( elem, name, value ) {
		var ret, hooks, notxml,
			nType = elem.nodeType;

		// Don't get/set properties on text, comment and attribute nodes
		if ( !elem || nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		notxml = nType !== 1 || !jQuery.isXMLDoc( elem );

		if ( notxml ) {
			// Fix name and attach hooks
			name = jQuery.propFix[ name ] || name;
			hooks = jQuery.propHooks[ name ];
		}

		if ( value !== undefined ) {
			return hooks && "set" in hooks && (ret = hooks.set( elem, value, name )) !== undefined ?
				ret :
				( elem[ name ] = value );

		} else {
			return hooks && "get" in hooks && (ret = hooks.get( elem, name )) !== null ?
				ret :
				elem[ name ];
		}
	},

	propHooks: {
		tabIndex: {
			get: function( elem ) {
				return elem.hasAttribute( "tabindex" ) || rfocusable.test( elem.nodeName ) || elem.href ?
					elem.tabIndex :
					-1;
			}
		}
	}
});

if ( !support.optSelected ) {
	jQuery.propHooks.selected = {
		get: function( elem ) {
			var parent = elem.parentNode;
			if ( parent && parent.parentNode ) {
				parent.parentNode.selectedIndex;
			}
			return null;
		}
	};
}

jQuery.each([
	"tabIndex",
	"readOnly",
	"maxLength",
	"cellSpacing",
	"cellPadding",
	"rowSpan",
	"colSpan",
	"useMap",
	"frameBorder",
	"contentEditable"
], function() {
	jQuery.propFix[ this.toLowerCase() ] = this;
});




var rclass = /[\t\r\n\f]/g;

jQuery.fn.extend({
	addClass: function( value ) {
		var classes, elem, cur, clazz, j, finalValue,
			proceed = typeof value === "string" && value,
			i = 0,
			len = this.length;

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( j ) {
				jQuery( this ).addClass( value.call( this, j, this.className ) );
			});
		}

		if ( proceed ) {
			// The disjunction here is for better compressibility (see removeClass)
			classes = ( value || "" ).match( rnotwhite ) || [];

			for ( ; i < len; i++ ) {
				elem = this[ i ];
				cur = elem.nodeType === 1 && ( elem.className ?
					( " " + elem.className + " " ).replace( rclass, " " ) :
					" "
				);

				if ( cur ) {
					j = 0;
					while ( (clazz = classes[j++]) ) {
						if ( cur.indexOf( " " + clazz + " " ) < 0 ) {
							cur += clazz + " ";
						}
					}

					// only assign if different to avoid unneeded rendering.
					finalValue = jQuery.trim( cur );
					if ( elem.className !== finalValue ) {
						elem.className = finalValue;
					}
				}
			}
		}

		return this;
	},

	removeClass: function( value ) {
		var classes, elem, cur, clazz, j, finalValue,
			proceed = arguments.length === 0 || typeof value === "string" && value,
			i = 0,
			len = this.length;

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( j ) {
				jQuery( this ).removeClass( value.call( this, j, this.className ) );
			});
		}
		if ( proceed ) {
			classes = ( value || "" ).match( rnotwhite ) || [];

			for ( ; i < len; i++ ) {
				elem = this[ i ];
				// This expression is here for better compressibility (see addClass)
				cur = elem.nodeType === 1 && ( elem.className ?
					( " " + elem.className + " " ).replace( rclass, " " ) :
					""
				);

				if ( cur ) {
					j = 0;
					while ( (clazz = classes[j++]) ) {
						// Remove *all* instances
						while ( cur.indexOf( " " + clazz + " " ) >= 0 ) {
							cur = cur.replace( " " + clazz + " ", " " );
						}
					}

					// Only assign if different to avoid unneeded rendering.
					finalValue = value ? jQuery.trim( cur ) : "";
					if ( elem.className !== finalValue ) {
						elem.className = finalValue;
					}
				}
			}
		}

		return this;
	},

	toggleClass: function( value, stateVal ) {
		var type = typeof value;

		if ( typeof stateVal === "boolean" && type === "string" ) {
			return stateVal ? this.addClass( value ) : this.removeClass( value );
		}

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( i ) {
				jQuery( this ).toggleClass( value.call(this, i, this.className, stateVal), stateVal );
			});
		}

		return this.each(function() {
			if ( type === "string" ) {
				// Toggle individual class names
				var className,
					i = 0,
					self = jQuery( this ),
					classNames = value.match( rnotwhite ) || [];

				while ( (className = classNames[ i++ ]) ) {
					// Check each className given, space separated list
					if ( self.hasClass( className ) ) {
						self.removeClass( className );
					} else {
						self.addClass( className );
					}
				}

			// Toggle whole class name
			} else if ( type === strundefined || type === "boolean" ) {
				if ( this.className ) {
					// store className if set
					data_priv.set( this, "__className__", this.className );
				}

				// If the element has a class name or if we're passed `false`,
				// then remove the whole classname (if there was one, the above saved it).
				// Otherwise bring back whatever was previously saved (if anything),
				// falling back to the empty string if nothing was stored.
				this.className = this.className || value === false ? "" : data_priv.get( this, "__className__" ) || "";
			}
		});
	},

	hasClass: function( selector ) {
		var className = " " + selector + " ",
			i = 0,
			l = this.length;
		for ( ; i < l; i++ ) {
			if ( this[i].nodeType === 1 && (" " + this[i].className + " ").replace(rclass, " ").indexOf( className ) >= 0 ) {
				return true;
			}
		}

		return false;
	}
});




var rreturn = /\r/g;

jQuery.fn.extend({
	val: function( value ) {
		var hooks, ret, isFunction,
			elem = this[0];

		if ( !arguments.length ) {
			if ( elem ) {
				hooks = jQuery.valHooks[ elem.type ] || jQuery.valHooks[ elem.nodeName.toLowerCase() ];

				if ( hooks && "get" in hooks && (ret = hooks.get( elem, "value" )) !== undefined ) {
					return ret;
				}

				ret = elem.value;

				return typeof ret === "string" ?
					// Handle most common string cases
					ret.replace(rreturn, "") :
					// Handle cases where value is null/undef or number
					ret == null ? "" : ret;
			}

			return;
		}

		isFunction = jQuery.isFunction( value );

		return this.each(function( i ) {
			var val;

			if ( this.nodeType !== 1 ) {
				return;
			}

			if ( isFunction ) {
				val = value.call( this, i, jQuery( this ).val() );
			} else {
				val = value;
			}

			// Treat null/undefined as ""; convert numbers to string
			if ( val == null ) {
				val = "";

			} else if ( typeof val === "number" ) {
				val += "";

			} else if ( jQuery.isArray( val ) ) {
				val = jQuery.map( val, function( value ) {
					return value == null ? "" : value + "";
				});
			}

			hooks = jQuery.valHooks[ this.type ] || jQuery.valHooks[ this.nodeName.toLowerCase() ];

			// If set returns undefined, fall back to normal setting
			if ( !hooks || !("set" in hooks) || hooks.set( this, val, "value" ) === undefined ) {
				this.value = val;
			}
		});
	}
});

jQuery.extend({
	valHooks: {
		option: {
			get: function( elem ) {
				var val = jQuery.find.attr( elem, "value" );
				return val != null ?
					val :
					// Support: IE10-11+
					// option.text throws exceptions (#14686, #14858)
					jQuery.trim( jQuery.text( elem ) );
			}
		},
		select: {
			get: function( elem ) {
				var value, option,
					options = elem.options,
					index = elem.selectedIndex,
					one = elem.type === "select-one" || index < 0,
					values = one ? null : [],
					max = one ? index + 1 : options.length,
					i = index < 0 ?
						max :
						one ? index : 0;

				// Loop through all the selected options
				for ( ; i < max; i++ ) {
					option = options[ i ];

					// IE6-9 doesn't update selected after form reset (#2551)
					if ( ( option.selected || i === index ) &&
							// Don't return options that are disabled or in a disabled optgroup
							( support.optDisabled ? !option.disabled : option.getAttribute( "disabled" ) === null ) &&
							( !option.parentNode.disabled || !jQuery.nodeName( option.parentNode, "optgroup" ) ) ) {

						// Get the specific value for the option
						value = jQuery( option ).val();

						// We don't need an array for one selects
						if ( one ) {
							return value;
						}

						// Multi-Selects return an array
						values.push( value );
					}
				}

				return values;
			},

			set: function( elem, value ) {
				var optionSet, option,
					options = elem.options,
					values = jQuery.makeArray( value ),
					i = options.length;

				while ( i-- ) {
					option = options[ i ];
					if ( (option.selected = jQuery.inArray( option.value, values ) >= 0) ) {
						optionSet = true;
					}
				}

				// Force browsers to behave consistently when non-matching value is set
				if ( !optionSet ) {
					elem.selectedIndex = -1;
				}
				return values;
			}
		}
	}
});

// Radios and checkboxes getter/setter
jQuery.each([ "radio", "checkbox" ], function() {
	jQuery.valHooks[ this ] = {
		set: function( elem, value ) {
			if ( jQuery.isArray( value ) ) {
				return ( elem.checked = jQuery.inArray( jQuery(elem).val(), value ) >= 0 );
			}
		}
	};
	if ( !support.checkOn ) {
		jQuery.valHooks[ this ].get = function( elem ) {
			return elem.getAttribute("value") === null ? "on" : elem.value;
		};
	}
});




// Return jQuery for attributes-only inclusion


jQuery.each( ("blur focus focusin focusout load resize scroll unload click dblclick " +
	"mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave " +
	"change select submit keydown keypress keyup error contextmenu").split(" "), function( i, name ) {

	// Handle event binding
	jQuery.fn[ name ] = function( data, fn ) {
		return arguments.length > 0 ?
			this.on( name, null, data, fn ) :
			this.trigger( name );
	};
});

jQuery.fn.extend({
	hover: function( fnOver, fnOut ) {
		return this.mouseenter( fnOver ).mouseleave( fnOut || fnOver );
	},

	bind: function( types, data, fn ) {
		return this.on( types, null, data, fn );
	},
	unbind: function( types, fn ) {
		return this.off( types, null, fn );
	},

	delegate: function( selector, types, data, fn ) {
		return this.on( types, selector, data, fn );
	},
	undelegate: function( selector, types, fn ) {
		// ( namespace ) or ( selector, types [, fn] )
		return arguments.length === 1 ? this.off( selector, "**" ) : this.off( types, selector || "**", fn );
	}
});


var nonce = jQuery.now();

var rquery = (/\?/);



// Support: Android 2.3
// Workaround failure to string-cast null input
jQuery.parseJSON = function( data ) {
	return JSON.parse( data + "" );
};


// Cross-browser xml parsing
jQuery.parseXML = function( data ) {
	var xml, tmp;
	if ( !data || typeof data !== "string" ) {
		return null;
	}

	// Support: IE9
	try {
		tmp = new DOMParser();
		xml = tmp.parseFromString( data, "text/xml" );
	} catch ( e ) {
		xml = undefined;
	}

	if ( !xml || xml.getElementsByTagName( "parsererror" ).length ) {
		jQuery.error( "Invalid XML: " + data );
	}
	return xml;
};


var
	rhash = /#.*$/,
	rts = /([?&])_=[^&]*/,
	rheaders = /^(.*?):[ \t]*([^\r\n]*)$/mg,
	// #7653, #8125, #8152: local protocol detection
	rlocalProtocol = /^(?:about|app|app-storage|.+-extension|file|res|widget):$/,
	rnoContent = /^(?:GET|HEAD)$/,
	rprotocol = /^\/\//,
	rurl = /^([\w.+-]+:)(?:\/\/(?:[^\/?#]*@|)([^\/?#:]*)(?::(\d+)|)|)/,

	/* Prefilters
	 * 1) They are useful to introduce custom dataTypes (see ajax/jsonp.js for an example)
	 * 2) These are called:
	 *    - BEFORE asking for a transport
	 *    - AFTER param serialization (s.data is a string if s.processData is true)
	 * 3) key is the dataType
	 * 4) the catchall symbol "*" can be used
	 * 5) execution will start with transport dataType and THEN continue down to "*" if needed
	 */
	prefilters = {},

	/* Transports bindings
	 * 1) key is the dataType
	 * 2) the catchall symbol "*" can be used
	 * 3) selection will start with transport dataType and THEN go to "*" if needed
	 */
	transports = {},

	// Avoid comment-prolog char sequence (#10098); must appease lint and evade compression
	allTypes = "*/".concat( "*" ),

	// Document location
	ajaxLocation = window.location.href,

	// Segment location into parts
	ajaxLocParts = rurl.exec( ajaxLocation.toLowerCase() ) || [];

// Base "constructor" for jQuery.ajaxPrefilter and jQuery.ajaxTransport
function addToPrefiltersOrTransports( structure ) {

	// dataTypeExpression is optional and defaults to "*"
	return function( dataTypeExpression, func ) {

		if ( typeof dataTypeExpression !== "string" ) {
			func = dataTypeExpression;
			dataTypeExpression = "*";
		}

		var dataType,
			i = 0,
			dataTypes = dataTypeExpression.toLowerCase().match( rnotwhite ) || [];

		if ( jQuery.isFunction( func ) ) {
			// For each dataType in the dataTypeExpression
			while ( (dataType = dataTypes[i++]) ) {
				// Prepend if requested
				if ( dataType[0] === "+" ) {
					dataType = dataType.slice( 1 ) || "*";
					(structure[ dataType ] = structure[ dataType ] || []).unshift( func );

				// Otherwise append
				} else {
					(structure[ dataType ] = structure[ dataType ] || []).push( func );
				}
			}
		}
	};
}

// Base inspection function for prefilters and transports
function inspectPrefiltersOrTransports( structure, options, originalOptions, jqXHR ) {

	var inspected = {},
		seekingTransport = ( structure === transports );

	function inspect( dataType ) {
		var selected;
		inspected[ dataType ] = true;
		jQuery.each( structure[ dataType ] || [], function( _, prefilterOrFactory ) {
			var dataTypeOrTransport = prefilterOrFactory( options, originalOptions, jqXHR );
			if ( typeof dataTypeOrTransport === "string" && !seekingTransport && !inspected[ dataTypeOrTransport ] ) {
				options.dataTypes.unshift( dataTypeOrTransport );
				inspect( dataTypeOrTransport );
				return false;
			} else if ( seekingTransport ) {
				return !( selected = dataTypeOrTransport );
			}
		});
		return selected;
	}

	return inspect( options.dataTypes[ 0 ] ) || !inspected[ "*" ] && inspect( "*" );
}

// A special extend for ajax options
// that takes "flat" options (not to be deep extended)
// Fixes #9887
function ajaxExtend( target, src ) {
	var key, deep,
		flatOptions = jQuery.ajaxSettings.flatOptions || {};

	for ( key in src ) {
		if ( src[ key ] !== undefined ) {
			( flatOptions[ key ] ? target : ( deep || (deep = {}) ) )[ key ] = src[ key ];
		}
	}
	if ( deep ) {
		jQuery.extend( true, target, deep );
	}

	return target;
}

/* Handles responses to an ajax request:
 * - finds the right dataType (mediates between content-type and expected dataType)
 * - returns the corresponding response
 */
function ajaxHandleResponses( s, jqXHR, responses ) {

	var ct, type, finalDataType, firstDataType,
		contents = s.contents,
		dataTypes = s.dataTypes;

	// Remove auto dataType and get content-type in the process
	while ( dataTypes[ 0 ] === "*" ) {
		dataTypes.shift();
		if ( ct === undefined ) {
			ct = s.mimeType || jqXHR.getResponseHeader("Content-Type");
		}
	}

	// Check if we're dealing with a known content-type
	if ( ct ) {
		for ( type in contents ) {
			if ( contents[ type ] && contents[ type ].test( ct ) ) {
				dataTypes.unshift( type );
				break;
			}
		}
	}

	// Check to see if we have a response for the expected dataType
	if ( dataTypes[ 0 ] in responses ) {
		finalDataType = dataTypes[ 0 ];
	} else {
		// Try convertible dataTypes
		for ( type in responses ) {
			if ( !dataTypes[ 0 ] || s.converters[ type + " " + dataTypes[0] ] ) {
				finalDataType = type;
				break;
			}
			if ( !firstDataType ) {
				firstDataType = type;
			}
		}
		// Or just use first one
		finalDataType = finalDataType || firstDataType;
	}

	// If we found a dataType
	// We add the dataType to the list if needed
	// and return the corresponding response
	if ( finalDataType ) {
		if ( finalDataType !== dataTypes[ 0 ] ) {
			dataTypes.unshift( finalDataType );
		}
		return responses[ finalDataType ];
	}
}

/* Chain conversions given the request and the original response
 * Also sets the responseXXX fields on the jqXHR instance
 */
function ajaxConvert( s, response, jqXHR, isSuccess ) {
	var conv2, current, conv, tmp, prev,
		converters = {},
		// Work with a copy of dataTypes in case we need to modify it for conversion
		dataTypes = s.dataTypes.slice();

	// Create converters map with lowercased keys
	if ( dataTypes[ 1 ] ) {
		for ( conv in s.converters ) {
			converters[ conv.toLowerCase() ] = s.converters[ conv ];
		}
	}

	current = dataTypes.shift();

	// Convert to each sequential dataType
	while ( current ) {

		if ( s.responseFields[ current ] ) {
			jqXHR[ s.responseFields[ current ] ] = response;
		}

		// Apply the dataFilter if provided
		if ( !prev && isSuccess && s.dataFilter ) {
			response = s.dataFilter( response, s.dataType );
		}

		prev = current;
		current = dataTypes.shift();

		if ( current ) {

		// There's only work to do if current dataType is non-auto
			if ( current === "*" ) {

				current = prev;

			// Convert response if prev dataType is non-auto and differs from current
			} else if ( prev !== "*" && prev !== current ) {

				// Seek a direct converter
				conv = converters[ prev + " " + current ] || converters[ "* " + current ];

				// If none found, seek a pair
				if ( !conv ) {
					for ( conv2 in converters ) {

						// If conv2 outputs current
						tmp = conv2.split( " " );
						if ( tmp[ 1 ] === current ) {

							// If prev can be converted to accepted input
							conv = converters[ prev + " " + tmp[ 0 ] ] ||
								converters[ "* " + tmp[ 0 ] ];
							if ( conv ) {
								// Condense equivalence converters
								if ( conv === true ) {
									conv = converters[ conv2 ];

								// Otherwise, insert the intermediate dataType
								} else if ( converters[ conv2 ] !== true ) {
									current = tmp[ 0 ];
									dataTypes.unshift( tmp[ 1 ] );
								}
								break;
							}
						}
					}
				}

				// Apply converter (if not an equivalence)
				if ( conv !== true ) {

					// Unless errors are allowed to bubble, catch and return them
					if ( conv && s[ "throws" ] ) {
						response = conv( response );
					} else {
						try {
							response = conv( response );
						} catch ( e ) {
							return { state: "parsererror", error: conv ? e : "No conversion from " + prev + " to " + current };
						}
					}
				}
			}
		}
	}

	return { state: "success", data: response };
}

jQuery.extend({

	// Counter for holding the number of active queries
	active: 0,

	// Last-Modified header cache for next request
	lastModified: {},
	etag: {},

	ajaxSettings: {
		url: ajaxLocation,
		type: "GET",
		isLocal: rlocalProtocol.test( ajaxLocParts[ 1 ] ),
		global: true,
		processData: true,
		async: true,
		contentType: "application/x-www-form-urlencoded; charset=UTF-8",
		/*
		timeout: 0,
		data: null,
		dataType: null,
		username: null,
		password: null,
		cache: null,
		throws: false,
		traditional: false,
		headers: {},
		*/

		accepts: {
			"*": allTypes,
			text: "text/plain",
			html: "text/html",
			xml: "application/xml, text/xml",
			json: "application/json, text/javascript"
		},

		contents: {
			xml: /xml/,
			html: /html/,
			json: /json/
		},

		responseFields: {
			xml: "responseXML",
			text: "responseText",
			json: "responseJSON"
		},

		// Data converters
		// Keys separate source (or catchall "*") and destination types with a single space
		converters: {

			// Convert anything to text
			"* text": String,

			// Text to html (true = no transformation)
			"text html": true,

			// Evaluate text as a json expression
			"text json": jQuery.parseJSON,

			// Parse text as xml
			"text xml": jQuery.parseXML
		},

		// For options that shouldn't be deep extended:
		// you can add your own custom options here if
		// and when you create one that shouldn't be
		// deep extended (see ajaxExtend)
		flatOptions: {
			url: true,
			context: true
		}
	},

	// Creates a full fledged settings object into target
	// with both ajaxSettings and settings fields.
	// If target is omitted, writes into ajaxSettings.
	ajaxSetup: function( target, settings ) {
		return settings ?

			// Building a settings object
			ajaxExtend( ajaxExtend( target, jQuery.ajaxSettings ), settings ) :

			// Extending ajaxSettings
			ajaxExtend( jQuery.ajaxSettings, target );
	},

	ajaxPrefilter: addToPrefiltersOrTransports( prefilters ),
	ajaxTransport: addToPrefiltersOrTransports( transports ),

	// Main method
	ajax: function( url, options ) {

		// If url is an object, simulate pre-1.5 signature
		if ( typeof url === "object" ) {
			options = url;
			url = undefined;
		}

		// Force options to be an object
		options = options || {};

		var transport,
			// URL without anti-cache param
			cacheURL,
			// Response headers
			responseHeadersString,
			responseHeaders,
			// timeout handle
			timeoutTimer,
			// Cross-domain detection vars
			parts,
			// To know if global events are to be dispatched
			fireGlobals,
			// Loop variable
			i,
			// Create the final options object
			s = jQuery.ajaxSetup( {}, options ),
			// Callbacks context
			callbackContext = s.context || s,
			// Context for global events is callbackContext if it is a DOM node or jQuery collection
			globalEventContext = s.context && ( callbackContext.nodeType || callbackContext.jquery ) ?
				jQuery( callbackContext ) :
				jQuery.event,
			// Deferreds
			deferred = jQuery.Deferred(),
			completeDeferred = jQuery.Callbacks("once memory"),
			// Status-dependent callbacks
			statusCode = s.statusCode || {},
			// Headers (they are sent all at once)
			requestHeaders = {},
			requestHeadersNames = {},
			// The jqXHR state
			state = 0,
			// Default abort message
			strAbort = "canceled",
			// Fake xhr
			jqXHR = {
				readyState: 0,

				// Builds headers hashtable if needed
				getResponseHeader: function( key ) {
					var match;
					if ( state === 2 ) {
						if ( !responseHeaders ) {
							responseHeaders = {};
							while ( (match = rheaders.exec( responseHeadersString )) ) {
								responseHeaders[ match[1].toLowerCase() ] = match[ 2 ];
							}
						}
						match = responseHeaders[ key.toLowerCase() ];
					}
					return match == null ? null : match;
				},

				// Raw string
				getAllResponseHeaders: function() {
					return state === 2 ? responseHeadersString : null;
				},

				// Caches the header
				setRequestHeader: function( name, value ) {
					var lname = name.toLowerCase();
					if ( !state ) {
						name = requestHeadersNames[ lname ] = requestHeadersNames[ lname ] || name;
						requestHeaders[ name ] = value;
					}
					return this;
				},

				// Overrides response content-type header
				overrideMimeType: function( type ) {
					if ( !state ) {
						s.mimeType = type;
					}
					return this;
				},

				// Status-dependent callbacks
				statusCode: function( map ) {
					var code;
					if ( map ) {
						if ( state < 2 ) {
							for ( code in map ) {
								// Lazy-add the new callback in a way that preserves old ones
								statusCode[ code ] = [ statusCode[ code ], map[ code ] ];
							}
						} else {
							// Execute the appropriate callbacks
							jqXHR.always( map[ jqXHR.status ] );
						}
					}
					return this;
				},

				// Cancel the request
				abort: function( statusText ) {
					var finalText = statusText || strAbort;
					if ( transport ) {
						transport.abort( finalText );
					}
					done( 0, finalText );
					return this;
				}
			};

		// Attach deferreds
		deferred.promise( jqXHR ).complete = completeDeferred.add;
		jqXHR.success = jqXHR.done;
		jqXHR.error = jqXHR.fail;

		// Remove hash character (#7531: and string promotion)
		// Add protocol if not provided (prefilters might expect it)
		// Handle falsy url in the settings object (#10093: consistency with old signature)
		// We also use the url parameter if available
		s.url = ( ( url || s.url || ajaxLocation ) + "" ).replace( rhash, "" )
			.replace( rprotocol, ajaxLocParts[ 1 ] + "//" );

		// Alias method option to type as per ticket #12004
		s.type = options.method || options.type || s.method || s.type;

		// Extract dataTypes list
		s.dataTypes = jQuery.trim( s.dataType || "*" ).toLowerCase().match( rnotwhite ) || [ "" ];

		// A cross-domain request is in order when we have a protocol:host:port mismatch
		if ( s.crossDomain == null ) {
			parts = rurl.exec( s.url.toLowerCase() );
			s.crossDomain = !!( parts &&
				( parts[ 1 ] !== ajaxLocParts[ 1 ] || parts[ 2 ] !== ajaxLocParts[ 2 ] ||
					( parts[ 3 ] || ( parts[ 1 ] === "http:" ? "80" : "443" ) ) !==
						( ajaxLocParts[ 3 ] || ( ajaxLocParts[ 1 ] === "http:" ? "80" : "443" ) ) )
			);
		}

		// Convert data if not already a string
		if ( s.data && s.processData && typeof s.data !== "string" ) {
			s.data = jQuery.param( s.data, s.traditional );
		}

		// Apply prefilters
		inspectPrefiltersOrTransports( prefilters, s, options, jqXHR );

		// If request was aborted inside a prefilter, stop there
		if ( state === 2 ) {
			return jqXHR;
		}

		// We can fire global events as of now if asked to
		// Don't fire events if jQuery.event is undefined in an AMD-usage scenario (#15118)
		fireGlobals = jQuery.event && s.global;

		// Watch for a new set of requests
		if ( fireGlobals && jQuery.active++ === 0 ) {
			jQuery.event.trigger("ajaxStart");
		}

		// Uppercase the type
		s.type = s.type.toUpperCase();

		// Determine if request has content
		s.hasContent = !rnoContent.test( s.type );

		// Save the URL in case we're toying with the If-Modified-Since
		// and/or If-None-Match header later on
		cacheURL = s.url;

		// More options handling for requests with no content
		if ( !s.hasContent ) {

			// If data is available, append data to url
			if ( s.data ) {
				cacheURL = ( s.url += ( rquery.test( cacheURL ) ? "&" : "?" ) + s.data );
				// #9682: remove data so that it's not used in an eventual retry
				delete s.data;
			}

			// Add anti-cache in url if needed
			if ( s.cache === false ) {
				s.url = rts.test( cacheURL ) ?

					// If there is already a '_' parameter, set its value
					cacheURL.replace( rts, "$1_=" + nonce++ ) :

					// Otherwise add one to the end
					cacheURL + ( rquery.test( cacheURL ) ? "&" : "?" ) + "_=" + nonce++;
			}
		}

		// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
		if ( s.ifModified ) {
			if ( jQuery.lastModified[ cacheURL ] ) {
				jqXHR.setRequestHeader( "If-Modified-Since", jQuery.lastModified[ cacheURL ] );
			}
			if ( jQuery.etag[ cacheURL ] ) {
				jqXHR.setRequestHeader( "If-None-Match", jQuery.etag[ cacheURL ] );
			}
		}

		// Set the correct header, if data is being sent
		if ( s.data && s.hasContent && s.contentType !== false || options.contentType ) {
			jqXHR.setRequestHeader( "Content-Type", s.contentType );
		}

		// Set the Accepts header for the server, depending on the dataType
		jqXHR.setRequestHeader(
			"Accept",
			s.dataTypes[ 0 ] && s.accepts[ s.dataTypes[0] ] ?
				s.accepts[ s.dataTypes[0] ] + ( s.dataTypes[ 0 ] !== "*" ? ", " + allTypes + "; q=0.01" : "" ) :
				s.accepts[ "*" ]
		);

		// Check for headers option
		for ( i in s.headers ) {
			jqXHR.setRequestHeader( i, s.headers[ i ] );
		}

		// Allow custom headers/mimetypes and early abort
		if ( s.beforeSend && ( s.beforeSend.call( callbackContext, jqXHR, s ) === false || state === 2 ) ) {
			// Abort if not done already and return
			return jqXHR.abort();
		}

		// Aborting is no longer a cancellation
		strAbort = "abort";

		// Install callbacks on deferreds
		for ( i in { success: 1, error: 1, complete: 1 } ) {
			jqXHR[ i ]( s[ i ] );
		}

		// Get transport
		transport = inspectPrefiltersOrTransports( transports, s, options, jqXHR );

		// If no transport, we auto-abort
		if ( !transport ) {
			done( -1, "No Transport" );
		} else {
			jqXHR.readyState = 1;

			// Send global event
			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxSend", [ jqXHR, s ] );
			}
			// Timeout
			if ( s.async && s.timeout > 0 ) {
				timeoutTimer = setTimeout(function() {
					jqXHR.abort("timeout");
				}, s.timeout );
			}

			try {
				state = 1;
				transport.send( requestHeaders, done );
			} catch ( e ) {
				// Propagate exception as error if not done
				if ( state < 2 ) {
					done( -1, e );
				// Simply rethrow otherwise
				} else {
					throw e;
				}
			}
		}

		// Callback for when everything is done
		function done( status, nativeStatusText, responses, headers ) {
			var isSuccess, success, error, response, modified,
				statusText = nativeStatusText;

			// Called once
			if ( state === 2 ) {
				return;
			}

			// State is "done" now
			state = 2;

			// Clear timeout if it exists
			if ( timeoutTimer ) {
				clearTimeout( timeoutTimer );
			}

			// Dereference transport for early garbage collection
			// (no matter how long the jqXHR object will be used)
			transport = undefined;

			// Cache response headers
			responseHeadersString = headers || "";

			// Set readyState
			jqXHR.readyState = status > 0 ? 4 : 0;

			// Determine if successful
			isSuccess = status >= 200 && status < 300 || status === 304;

			// Get response data
			if ( responses ) {
				response = ajaxHandleResponses( s, jqXHR, responses );
			}

			// Convert no matter what (that way responseXXX fields are always set)
			response = ajaxConvert( s, response, jqXHR, isSuccess );

			// If successful, handle type chaining
			if ( isSuccess ) {

				// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
				if ( s.ifModified ) {
					modified = jqXHR.getResponseHeader("Last-Modified");
					if ( modified ) {
						jQuery.lastModified[ cacheURL ] = modified;
					}
					modified = jqXHR.getResponseHeader("etag");
					if ( modified ) {
						jQuery.etag[ cacheURL ] = modified;
					}
				}

				// if no content
				if ( status === 204 || s.type === "HEAD" ) {
					statusText = "nocontent";

				// if not modified
				} else if ( status === 304 ) {
					statusText = "notmodified";

				// If we have data, let's convert it
				} else {
					statusText = response.state;
					success = response.data;
					error = response.error;
					isSuccess = !error;
				}
			} else {
				// Extract error from statusText and normalize for non-aborts
				error = statusText;
				if ( status || !statusText ) {
					statusText = "error";
					if ( status < 0 ) {
						status = 0;
					}
				}
			}

			// Set data for the fake xhr object
			jqXHR.status = status;
			jqXHR.statusText = ( nativeStatusText || statusText ) + "";

			// Success/Error
			if ( isSuccess ) {
				deferred.resolveWith( callbackContext, [ success, statusText, jqXHR ] );
			} else {
				deferred.rejectWith( callbackContext, [ jqXHR, statusText, error ] );
			}

			// Status-dependent callbacks
			jqXHR.statusCode( statusCode );
			statusCode = undefined;

			if ( fireGlobals ) {
				globalEventContext.trigger( isSuccess ? "ajaxSuccess" : "ajaxError",
					[ jqXHR, s, isSuccess ? success : error ] );
			}

			// Complete
			completeDeferred.fireWith( callbackContext, [ jqXHR, statusText ] );

			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxComplete", [ jqXHR, s ] );
				// Handle the global AJAX counter
				if ( !( --jQuery.active ) ) {
					jQuery.event.trigger("ajaxStop");
				}
			}
		}

		return jqXHR;
	},

	getJSON: function( url, data, callback ) {
		return jQuery.get( url, data, callback, "json" );
	},

	getScript: function( url, callback ) {
		return jQuery.get( url, undefined, callback, "script" );
	}
});

jQuery.each( [ "get", "post" ], function( i, method ) {
	jQuery[ method ] = function( url, data, callback, type ) {
		// Shift arguments if data argument was omitted
		if ( jQuery.isFunction( data ) ) {
			type = type || callback;
			callback = data;
			data = undefined;
		}

		return jQuery.ajax({
			url: url,
			type: method,
			dataType: type,
			data: data,
			success: callback
		});
	};
});


jQuery._evalUrl = function( url ) {
	return jQuery.ajax({
		url: url,
		type: "GET",
		dataType: "script",
		async: false,
		global: false,
		"throws": true
	});
};


jQuery.fn.extend({
	wrapAll: function( html ) {
		var wrap;

		if ( jQuery.isFunction( html ) ) {
			return this.each(function( i ) {
				jQuery( this ).wrapAll( html.call(this, i) );
			});
		}

		if ( this[ 0 ] ) {

			// The elements to wrap the target around
			wrap = jQuery( html, this[ 0 ].ownerDocument ).eq( 0 ).clone( true );

			if ( this[ 0 ].parentNode ) {
				wrap.insertBefore( this[ 0 ] );
			}

			wrap.map(function() {
				var elem = this;

				while ( elem.firstElementChild ) {
					elem = elem.firstElementChild;
				}

				return elem;
			}).append( this );
		}

		return this;
	},

	wrapInner: function( html ) {
		if ( jQuery.isFunction( html ) ) {
			return this.each(function( i ) {
				jQuery( this ).wrapInner( html.call(this, i) );
			});
		}

		return this.each(function() {
			var self = jQuery( this ),
				contents = self.contents();

			if ( contents.length ) {
				contents.wrapAll( html );

			} else {
				self.append( html );
			}
		});
	},

	wrap: function( html ) {
		var isFunction = jQuery.isFunction( html );

		return this.each(function( i ) {
			jQuery( this ).wrapAll( isFunction ? html.call(this, i) : html );
		});
	},

	unwrap: function() {
		return this.parent().each(function() {
			if ( !jQuery.nodeName( this, "body" ) ) {
				jQuery( this ).replaceWith( this.childNodes );
			}
		}).end();
	}
});


jQuery.expr.filters.hidden = function( elem ) {
	// Support: Opera <= 12.12
	// Opera reports offsetWidths and offsetHeights less than zero on some elements
	return elem.offsetWidth <= 0 && elem.offsetHeight <= 0;
};
jQuery.expr.filters.visible = function( elem ) {
	return !jQuery.expr.filters.hidden( elem );
};




var r20 = /%20/g,
	rbracket = /\[\]$/,
	rCRLF = /\r?\n/g,
	rsubmitterTypes = /^(?:submit|button|image|reset|file)$/i,
	rsubmittable = /^(?:input|select|textarea|keygen)/i;

function buildParams( prefix, obj, traditional, add ) {
	var name;

	if ( jQuery.isArray( obj ) ) {
		// Serialize array item.
		jQuery.each( obj, function( i, v ) {
			if ( traditional || rbracket.test( prefix ) ) {
				// Treat each array item as a scalar.
				add( prefix, v );

			} else {
				// Item is non-scalar (array or object), encode its numeric index.
				buildParams( prefix + "[" + ( typeof v === "object" ? i : "" ) + "]", v, traditional, add );
			}
		});

	} else if ( !traditional && jQuery.type( obj ) === "object" ) {
		// Serialize object item.
		for ( name in obj ) {
			buildParams( prefix + "[" + name + "]", obj[ name ], traditional, add );
		}

	} else {
		// Serialize scalar item.
		add( prefix, obj );
	}
}

// Serialize an array of form elements or a set of
// key/values into a query string
jQuery.param = function( a, traditional ) {
	var prefix,
		s = [],
		add = function( key, value ) {
			// If value is a function, invoke it and return its value
			value = jQuery.isFunction( value ) ? value() : ( value == null ? "" : value );
			s[ s.length ] = encodeURIComponent( key ) + "=" + encodeURIComponent( value );
		};

	// Set traditional to true for jQuery <= 1.3.2 behavior.
	if ( traditional === undefined ) {
		traditional = jQuery.ajaxSettings && jQuery.ajaxSettings.traditional;
	}

	// If an array was passed in, assume that it is an array of form elements.
	if ( jQuery.isArray( a ) || ( a.jquery && !jQuery.isPlainObject( a ) ) ) {
		// Serialize the form elements
		jQuery.each( a, function() {
			add( this.name, this.value );
		});

	} else {
		// If traditional, encode the "old" way (the way 1.3.2 or older
		// did it), otherwise encode params recursively.
		for ( prefix in a ) {
			buildParams( prefix, a[ prefix ], traditional, add );
		}
	}

	// Return the resulting serialization
	return s.join( "&" ).replace( r20, "+" );
};

jQuery.fn.extend({
	serialize: function() {
		return jQuery.param( this.serializeArray() );
	},
	serializeArray: function() {
		return this.map(function() {
			// Can add propHook for "elements" to filter or add form elements
			var elements = jQuery.prop( this, "elements" );
			return elements ? jQuery.makeArray( elements ) : this;
		})
		.filter(function() {
			var type = this.type;

			// Use .is( ":disabled" ) so that fieldset[disabled] works
			return this.name && !jQuery( this ).is( ":disabled" ) &&
				rsubmittable.test( this.nodeName ) && !rsubmitterTypes.test( type ) &&
				( this.checked || !rcheckableType.test( type ) );
		})
		.map(function( i, elem ) {
			var val = jQuery( this ).val();

			return val == null ?
				null :
				jQuery.isArray( val ) ?
					jQuery.map( val, function( val ) {
						return { name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
					}) :
					{ name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
		}).get();
	}
});


jQuery.ajaxSettings.xhr = function() {
	try {
		return new XMLHttpRequest();
	} catch( e ) {}
};

var xhrId = 0,
	xhrCallbacks = {},
	xhrSuccessStatus = {
		// file protocol always yields status code 0, assume 200
		0: 200,
		// Support: IE9
		// #1450: sometimes IE returns 1223 when it should be 204
		1223: 204
	},
	xhrSupported = jQuery.ajaxSettings.xhr();

// Support: IE9
// Open requests must be manually aborted on unload (#5280)
// See https://support.microsoft.com/kb/2856746 for more info
if ( window.attachEvent ) {
	window.attachEvent( "onunload", function() {
		for ( var key in xhrCallbacks ) {
			xhrCallbacks[ key ]();
		}
	});
}

support.cors = !!xhrSupported && ( "withCredentials" in xhrSupported );
support.ajax = xhrSupported = !!xhrSupported;

jQuery.ajaxTransport(function( options ) {
	var callback;

	// Cross domain only allowed if supported through XMLHttpRequest
	if ( support.cors || xhrSupported && !options.crossDomain ) {
		return {
			send: function( headers, complete ) {
				var i,
					xhr = options.xhr(),
					id = ++xhrId;

				xhr.open( options.type, options.url, options.async, options.username, options.password );

				// Apply custom fields if provided
				if ( options.xhrFields ) {
					for ( i in options.xhrFields ) {
						xhr[ i ] = options.xhrFields[ i ];
					}
				}

				// Override mime type if needed
				if ( options.mimeType && xhr.overrideMimeType ) {
					xhr.overrideMimeType( options.mimeType );
				}

				// X-Requested-With header
				// For cross-domain requests, seeing as conditions for a preflight are
				// akin to a jigsaw puzzle, we simply never set it to be sure.
				// (it can always be set on a per-request basis or even using ajaxSetup)
				// For same-domain requests, won't change header if already provided.
				if ( !options.crossDomain && !headers["X-Requested-With"] ) {
					headers["X-Requested-With"] = "XMLHttpRequest";
				}

				// Set headers
				for ( i in headers ) {
					xhr.setRequestHeader( i, headers[ i ] );
				}

				// Callback
				callback = function( type ) {
					return function() {
						if ( callback ) {
							delete xhrCallbacks[ id ];
							callback = xhr.onload = xhr.onerror = null;

							if ( type === "abort" ) {
								xhr.abort();
							} else if ( type === "error" ) {
								complete(
									// file: protocol always yields status 0; see #8605, #14207
									xhr.status,
									xhr.statusText
								);
							} else {
								complete(
									xhrSuccessStatus[ xhr.status ] || xhr.status,
									xhr.statusText,
									// Support: IE9
									// Accessing binary-data responseText throws an exception
									// (#11426)
									typeof xhr.responseText === "string" ? {
										text: xhr.responseText
									} : undefined,
									xhr.getAllResponseHeaders()
								);
							}
						}
					};
				};

				// Listen to events
				xhr.onload = callback();
				xhr.onerror = callback("error");

				// Create the abort callback
				callback = xhrCallbacks[ id ] = callback("abort");

				try {
					// Do send the request (this may raise an exception)
					xhr.send( options.hasContent && options.data || null );
				} catch ( e ) {
					// #14683: Only rethrow if this hasn't been notified as an error yet
					if ( callback ) {
						throw e;
					}
				}
			},

			abort: function() {
				if ( callback ) {
					callback();
				}
			}
		};
	}
});




// Install script dataType
jQuery.ajaxSetup({
	accepts: {
		script: "text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"
	},
	contents: {
		script: /(?:java|ecma)script/
	},
	converters: {
		"text script": function( text ) {
			jQuery.globalEval( text );
			return text;
		}
	}
});

// Handle cache's special case and crossDomain
jQuery.ajaxPrefilter( "script", function( s ) {
	if ( s.cache === undefined ) {
		s.cache = false;
	}
	if ( s.crossDomain ) {
		s.type = "GET";
	}
});

// Bind script tag hack transport
jQuery.ajaxTransport( "script", function( s ) {
	// This transport only deals with cross domain requests
	if ( s.crossDomain ) {
		var script, callback;
		return {
			send: function( _, complete ) {
				script = jQuery("<script>").prop({
					async: true,
					charset: s.scriptCharset,
					src: s.url
				}).on(
					"load error",
					callback = function( evt ) {
						script.remove();
						callback = null;
						if ( evt ) {
							complete( evt.type === "error" ? 404 : 200, evt.type );
						}
					}
				);
				document.head.appendChild( script[ 0 ] );
			},
			abort: function() {
				if ( callback ) {
					callback();
				}
			}
		};
	}
});




var oldCallbacks = [],
	rjsonp = /(=)\?(?=&|$)|\?\?/;

// Default jsonp settings
jQuery.ajaxSetup({
	jsonp: "callback",
	jsonpCallback: function() {
		var callback = oldCallbacks.pop() || ( jQuery.expando + "_" + ( nonce++ ) );
		this[ callback ] = true;
		return callback;
	}
});

// Detect, normalize options and install callbacks for jsonp requests
jQuery.ajaxPrefilter( "json jsonp", function( s, originalSettings, jqXHR ) {

	var callbackName, overwritten, responseContainer,
		jsonProp = s.jsonp !== false && ( rjsonp.test( s.url ) ?
			"url" :
			typeof s.data === "string" && !( s.contentType || "" ).indexOf("application/x-www-form-urlencoded") && rjsonp.test( s.data ) && "data"
		);

	// Handle iff the expected data type is "jsonp" or we have a parameter to set
	if ( jsonProp || s.dataTypes[ 0 ] === "jsonp" ) {

		// Get callback name, remembering preexisting value associated with it
		callbackName = s.jsonpCallback = jQuery.isFunction( s.jsonpCallback ) ?
			s.jsonpCallback() :
			s.jsonpCallback;

		// Insert callback into url or form data
		if ( jsonProp ) {
			s[ jsonProp ] = s[ jsonProp ].replace( rjsonp, "$1" + callbackName );
		} else if ( s.jsonp !== false ) {
			s.url += ( rquery.test( s.url ) ? "&" : "?" ) + s.jsonp + "=" + callbackName;
		}

		// Use data converter to retrieve json after script execution
		s.converters["script json"] = function() {
			if ( !responseContainer ) {
				jQuery.error( callbackName + " was not called" );
			}
			return responseContainer[ 0 ];
		};

		// force json dataType
		s.dataTypes[ 0 ] = "json";

		// Install callback
		overwritten = window[ callbackName ];
		window[ callbackName ] = function() {
			responseContainer = arguments;
		};

		// Clean-up function (fires after converters)
		jqXHR.always(function() {
			// Restore preexisting value
			window[ callbackName ] = overwritten;

			// Save back as free
			if ( s[ callbackName ] ) {
				// make sure that re-using the options doesn't screw things around
				s.jsonpCallback = originalSettings.jsonpCallback;

				// save the callback name for future use
				oldCallbacks.push( callbackName );
			}

			// Call if it was a function and we have a response
			if ( responseContainer && jQuery.isFunction( overwritten ) ) {
				overwritten( responseContainer[ 0 ] );
			}

			responseContainer = overwritten = undefined;
		});

		// Delegate to script
		return "script";
	}
});




// data: string of html
// context (optional): If specified, the fragment will be created in this context, defaults to document
// keepScripts (optional): If true, will include scripts passed in the html string
jQuery.parseHTML = function( data, context, keepScripts ) {
	if ( !data || typeof data !== "string" ) {
		return null;
	}
	if ( typeof context === "boolean" ) {
		keepScripts = context;
		context = false;
	}
	context = context || document;

	var parsed = rsingleTag.exec( data ),
		scripts = !keepScripts && [];

	// Single tag
	if ( parsed ) {
		return [ context.createElement( parsed[1] ) ];
	}

	parsed = jQuery.buildFragment( [ data ], context, scripts );

	if ( scripts && scripts.length ) {
		jQuery( scripts ).remove();
	}

	return jQuery.merge( [], parsed.childNodes );
};


// Keep a copy of the old load method
var _load = jQuery.fn.load;

/**
 * Load a url into a page
 */
jQuery.fn.load = function( url, params, callback ) {
	if ( typeof url !== "string" && _load ) {
		return _load.apply( this, arguments );
	}

	var selector, type, response,
		self = this,
		off = url.indexOf(" ");

	if ( off >= 0 ) {
		selector = jQuery.trim( url.slice( off ) );
		url = url.slice( 0, off );
	}

	// If it's a function
	if ( jQuery.isFunction( params ) ) {

		// We assume that it's the callback
		callback = params;
		params = undefined;

	// Otherwise, build a param string
	} else if ( params && typeof params === "object" ) {
		type = "POST";
	}

	// If we have elements to modify, make the request
	if ( self.length > 0 ) {
		jQuery.ajax({
			url: url,

			// if "type" variable is undefined, then "GET" method will be used
			type: type,
			dataType: "html",
			data: params
		}).done(function( responseText ) {

			// Save response for use in complete callback
			response = arguments;

			self.html( selector ?

				// If a selector was specified, locate the right elements in a dummy div
				// Exclude scripts to avoid IE 'Permission Denied' errors
				jQuery("<div>").append( jQuery.parseHTML( responseText ) ).find( selector ) :

				// Otherwise use the full result
				responseText );

		}).complete( callback && function( jqXHR, status ) {
			self.each( callback, response || [ jqXHR.responseText, status, jqXHR ] );
		});
	}

	return this;
};




// Attach a bunch of functions for handling common AJAX events
jQuery.each( [ "ajaxStart", "ajaxStop", "ajaxComplete", "ajaxError", "ajaxSuccess", "ajaxSend" ], function( i, type ) {
	jQuery.fn[ type ] = function( fn ) {
		return this.on( type, fn );
	};
});




jQuery.expr.filters.animated = function( elem ) {
	return jQuery.grep(jQuery.timers, function( fn ) {
		return elem === fn.elem;
	}).length;
};




var docElem = window.document.documentElement;

/**
 * Gets a window from an element
 */
function getWindow( elem ) {
	return jQuery.isWindow( elem ) ? elem : elem.nodeType === 9 && elem.defaultView;
}

jQuery.offset = {
	setOffset: function( elem, options, i ) {
		var curPosition, curLeft, curCSSTop, curTop, curOffset, curCSSLeft, calculatePosition,
			position = jQuery.css( elem, "position" ),
			curElem = jQuery( elem ),
			props = {};

		// Set position first, in-case top/left are set even on static elem
		if ( position === "static" ) {
			elem.style.position = "relative";
		}

		curOffset = curElem.offset();
		curCSSTop = jQuery.css( elem, "top" );
		curCSSLeft = jQuery.css( elem, "left" );
		calculatePosition = ( position === "absolute" || position === "fixed" ) &&
			( curCSSTop + curCSSLeft ).indexOf("auto") > -1;

		// Need to be able to calculate position if either
		// top or left is auto and position is either absolute or fixed
		if ( calculatePosition ) {
			curPosition = curElem.position();
			curTop = curPosition.top;
			curLeft = curPosition.left;

		} else {
			curTop = parseFloat( curCSSTop ) || 0;
			curLeft = parseFloat( curCSSLeft ) || 0;
		}

		if ( jQuery.isFunction( options ) ) {
			options = options.call( elem, i, curOffset );
		}

		if ( options.top != null ) {
			props.top = ( options.top - curOffset.top ) + curTop;
		}
		if ( options.left != null ) {
			props.left = ( options.left - curOffset.left ) + curLeft;
		}

		if ( "using" in options ) {
			options.using.call( elem, props );

		} else {
			curElem.css( props );
		}
	}
};

jQuery.fn.extend({
	offset: function( options ) {
		if ( arguments.length ) {
			return options === undefined ?
				this :
				this.each(function( i ) {
					jQuery.offset.setOffset( this, options, i );
				});
		}

		var docElem, win,
			elem = this[ 0 ],
			box = { top: 0, left: 0 },
			doc = elem && elem.ownerDocument;

		if ( !doc ) {
			return;
		}

		docElem = doc.documentElement;

		// Make sure it's not a disconnected DOM node
		if ( !jQuery.contains( docElem, elem ) ) {
			return box;
		}

		// Support: BlackBerry 5, iOS 3 (original iPhone)
		// If we don't have gBCR, just use 0,0 rather than error
		if ( typeof elem.getBoundingClientRect !== strundefined ) {
			box = elem.getBoundingClientRect();
		}
		win = getWindow( doc );
		return {
			top: box.top + win.pageYOffset - docElem.clientTop,
			left: box.left + win.pageXOffset - docElem.clientLeft
		};
	},

	position: function() {
		if ( !this[ 0 ] ) {
			return;
		}

		var offsetParent, offset,
			elem = this[ 0 ],
			parentOffset = { top: 0, left: 0 };

		// Fixed elements are offset from window (parentOffset = {top:0, left: 0}, because it is its only offset parent
		if ( jQuery.css( elem, "position" ) === "fixed" ) {
			// Assume getBoundingClientRect is there when computed position is fixed
			offset = elem.getBoundingClientRect();

		} else {
			// Get *real* offsetParent
			offsetParent = this.offsetParent();

			// Get correct offsets
			offset = this.offset();
			if ( !jQuery.nodeName( offsetParent[ 0 ], "html" ) ) {
				parentOffset = offsetParent.offset();
			}

			// Add offsetParent borders
			parentOffset.top += jQuery.css( offsetParent[ 0 ], "borderTopWidth", true );
			parentOffset.left += jQuery.css( offsetParent[ 0 ], "borderLeftWidth", true );
		}

		// Subtract parent offsets and element margins
		return {
			top: offset.top - parentOffset.top - jQuery.css( elem, "marginTop", true ),
			left: offset.left - parentOffset.left - jQuery.css( elem, "marginLeft", true )
		};
	},

	offsetParent: function() {
		return this.map(function() {
			var offsetParent = this.offsetParent || docElem;

			while ( offsetParent && ( !jQuery.nodeName( offsetParent, "html" ) && jQuery.css( offsetParent, "position" ) === "static" ) ) {
				offsetParent = offsetParent.offsetParent;
			}

			return offsetParent || docElem;
		});
	}
});

// Create scrollLeft and scrollTop methods
jQuery.each( { scrollLeft: "pageXOffset", scrollTop: "pageYOffset" }, function( method, prop ) {
	var top = "pageYOffset" === prop;

	jQuery.fn[ method ] = function( val ) {
		return access( this, function( elem, method, val ) {
			var win = getWindow( elem );

			if ( val === undefined ) {
				return win ? win[ prop ] : elem[ method ];
			}

			if ( win ) {
				win.scrollTo(
					!top ? val : window.pageXOffset,
					top ? val : window.pageYOffset
				);

			} else {
				elem[ method ] = val;
			}
		}, method, val, arguments.length, null );
	};
});

// Support: Safari<7+, Chrome<37+
// Add the top/left cssHooks using jQuery.fn.position
// Webkit bug: https://bugs.webkit.org/show_bug.cgi?id=29084
// Blink bug: https://code.google.com/p/chromium/issues/detail?id=229280
// getComputedStyle returns percent when specified for top/left/bottom/right;
// rather than make the css module depend on the offset module, just check for it here
jQuery.each( [ "top", "left" ], function( i, prop ) {
	jQuery.cssHooks[ prop ] = addGetHookIf( support.pixelPosition,
		function( elem, computed ) {
			if ( computed ) {
				computed = curCSS( elem, prop );
				// If curCSS returns percentage, fallback to offset
				return rnumnonpx.test( computed ) ?
					jQuery( elem ).position()[ prop ] + "px" :
					computed;
			}
		}
	);
});


// Create innerHeight, innerWidth, height, width, outerHeight and outerWidth methods
jQuery.each( { Height: "height", Width: "width" }, function( name, type ) {
	jQuery.each( { padding: "inner" + name, content: type, "": "outer" + name }, function( defaultExtra, funcName ) {
		// Margin is only for outerHeight, outerWidth
		jQuery.fn[ funcName ] = function( margin, value ) {
			var chainable = arguments.length && ( defaultExtra || typeof margin !== "boolean" ),
				extra = defaultExtra || ( margin === true || value === true ? "margin" : "border" );

			return access( this, function( elem, type, value ) {
				var doc;

				if ( jQuery.isWindow( elem ) ) {
					// As of 5/8/2012 this will yield incorrect results for Mobile Safari, but there
					// isn't a whole lot we can do. See pull request at this URL for discussion:
					// https://github.com/jquery/jquery/pull/764
					return elem.document.documentElement[ "client" + name ];
				}

				// Get document width or height
				if ( elem.nodeType === 9 ) {
					doc = elem.documentElement;

					// Either scroll[Width/Height] or offset[Width/Height] or client[Width/Height],
					// whichever is greatest
					return Math.max(
						elem.body[ "scroll" + name ], doc[ "scroll" + name ],
						elem.body[ "offset" + name ], doc[ "offset" + name ],
						doc[ "client" + name ]
					);
				}

				return value === undefined ?
					// Get width or height on the element, requesting but not forcing parseFloat
					jQuery.css( elem, type, extra ) :

					// Set width or height on the element
					jQuery.style( elem, type, value, extra );
			}, type, chainable ? margin : undefined, chainable, null );
		};
	});
});


// The number of elements contained in the matched element set
jQuery.fn.size = function() {
	return this.length;
};

jQuery.fn.andSelf = jQuery.fn.addBack;




// Register as a named AMD module, since jQuery can be concatenated with other
// files that may use define, but not via a proper concatenation script that
// understands anonymous AMD modules. A named AMD is safest and most robust
// way to register. Lowercase jquery is used because AMD module names are
// derived from file names, and jQuery is normally delivered in a lowercase
// file name. Do this after creating the global so that if an AMD module wants
// to call noConflict to hide this version of jQuery, it will work.

// Note that for maximum portability, libraries that are not jQuery should
// declare themselves as anonymous modules, and avoid setting a global if an
// AMD loader is present. jQuery is a special case. For more information, see
// https://github.com/jrburke/requirejs/wiki/Updating-existing-libraries#wiki-anon

if ( typeof define === "function" && define.amd ) {
	define( "jquery", [], function() {
		return jQuery;
	});
}




var
	// Map over jQuery in case of overwrite
	_jQuery = window.jQuery,

	// Map over the $ in case of overwrite
	_$ = window.$;

jQuery.noConflict = function( deep ) {
	if ( window.$ === jQuery ) {
		window.$ = _$;
	}

	if ( deep && window.jQuery === jQuery ) {
		window.jQuery = _jQuery;
	}

	return jQuery;
};

// Expose jQuery and $ identifiers, even in AMD
// (#7102#comment:10, https://github.com/jquery/jquery/pull/557)
// and CommonJS for browser emulators (#13566)
if ( typeof noGlobal === strundefined ) {
	window.jQuery = window.$ = jQuery;
}




return jQuery;

}));

/*!
 * Modernizr v2.8.3
 * www.modernizr.com
 *
 * Copyright (c) Faruk Ates, Paul Irish, Alex Sexton
 * Available under the BSD and MIT licenses: www.modernizr.com/license/
 */
window.Modernizr=function(a,b,c){function d(a){t.cssText=a}function e(a,b){return d(x.join(a+";")+(b||""))}function f(a,b){return typeof a===b}function g(a,b){return!!~(""+a).indexOf(b)}function h(a,b){for(var d in a){var e=a[d];if(!g(e,"-")&&t[e]!==c)return"pfx"==b?e:!0}return!1}function i(a,b,d){for(var e in a){var g=b[a[e]];if(g!==c)return d===!1?a[e]:f(g,"function")?g.bind(d||b):g}return!1}function j(a,b,c){var d=a.charAt(0).toUpperCase()+a.slice(1),e=(a+" "+z.join(d+" ")+d).split(" ");return f(b,"string")||f(b,"undefined")?h(e,b):(e=(a+" "+A.join(d+" ")+d).split(" "),i(e,b,c))}function k(){o.input=function(c){for(var d=0,e=c.length;e>d;d++)E[c[d]]=!!(c[d]in u);return E.list&&(E.list=!(!b.createElement("datalist")||!a.HTMLDataListElement)),E}("autocomplete autofocus list placeholder max min multiple pattern required step".split(" ")),o.inputtypes=function(a){for(var d,e,f,g=0,h=a.length;h>g;g++)u.setAttribute("type",e=a[g]),d="text"!==u.type,d&&(u.value=v,u.style.cssText="position:absolute;visibility:hidden;",/^range$/.test(e)&&u.style.WebkitAppearance!==c?(q.appendChild(u),f=b.defaultView,d=f.getComputedStyle&&"textfield"!==f.getComputedStyle(u,null).WebkitAppearance&&0!==u.offsetHeight,q.removeChild(u)):/^(search|tel)$/.test(e)||(d=/^(url|email)$/.test(e)?u.checkValidity&&u.checkValidity()===!1:u.value!=v)),D[a[g]]=!!d;return D}("search tel url email datetime date month week time datetime-local number range color".split(" "))}var l,m,n="2.8.3",o={},p=!0,q=b.documentElement,r="modernizr",s=b.createElement(r),t=s.style,u=b.createElement("input"),v=":)",w={}.toString,x=" -webkit- -moz- -o- -ms- ".split(" "),y="Webkit Moz O ms",z=y.split(" "),A=y.toLowerCase().split(" "),B={svg:"http://www.w3.org/2000/svg"},C={},D={},E={},F=[],G=F.slice,H=function(a,c,d,e){var f,g,h,i,j=b.createElement("div"),k=b.body,l=k||b.createElement("body");if(parseInt(d,10))for(;d--;)h=b.createElement("div"),h.id=e?e[d]:r+(d+1),j.appendChild(h);return f=["&#173;",'<style id="s',r,'">',a,"</style>"].join(""),j.id=r,(k?j:l).innerHTML+=f,l.appendChild(j),k||(l.style.background="",l.style.overflow="hidden",i=q.style.overflow,q.style.overflow="hidden",q.appendChild(l)),g=c(j,a),k?j.parentNode.removeChild(j):(l.parentNode.removeChild(l),q.style.overflow=i),!!g},I=function(b){var c=a.matchMedia||a.msMatchMedia;if(c)return c(b)&&c(b).matches||!1;var d;return H("@media "+b+" { #"+r+" { position: absolute; } }",function(b){d="absolute"==(a.getComputedStyle?getComputedStyle(b,null):b.currentStyle).position}),d},J=function(){function a(a,e){e=e||b.createElement(d[a]||"div"),a="on"+a;var g=a in e;return g||(e.setAttribute||(e=b.createElement("div")),e.setAttribute&&e.removeAttribute&&(e.setAttribute(a,""),g=f(e[a],"function"),f(e[a],"undefined")||(e[a]=c),e.removeAttribute(a))),e=null,g}var d={select:"input",change:"input",submit:"form",reset:"form",error:"img",load:"img",abort:"img"};return a}(),K={}.hasOwnProperty;m=f(K,"undefined")||f(K.call,"undefined")?function(a,b){return b in a&&f(a.constructor.prototype[b],"undefined")}:function(a,b){return K.call(a,b)},Function.prototype.bind||(Function.prototype.bind=function(a){var b=this;if("function"!=typeof b)throw new TypeError;var c=G.call(arguments,1),d=function(){if(this instanceof d){var e=function(){};e.prototype=b.prototype;var f=new e,g=b.apply(f,c.concat(G.call(arguments)));return Object(g)===g?g:f}return b.apply(a,c.concat(G.call(arguments)))};return d}),C.flexbox=function(){return j("flexWrap")},C.flexboxlegacy=function(){return j("boxDirection")},C.canvas=function(){var a=b.createElement("canvas");return!(!a.getContext||!a.getContext("2d"))},C.canvastext=function(){return!(!o.canvas||!f(b.createElement("canvas").getContext("2d").fillText,"function"))},C.webgl=function(){return!!a.WebGLRenderingContext},C.touch=function(){var c;return"ontouchstart"in a||a.DocumentTouch&&b instanceof DocumentTouch?c=!0:H(["@media (",x.join("touch-enabled),("),r,")","{#modernizr{top:9px;position:absolute}}"].join(""),function(a){c=9===a.offsetTop}),c},C.geolocation=function(){return"geolocation"in navigator},C.postmessage=function(){return!!a.postMessage},C.websqldatabase=function(){return!!a.openDatabase},C.indexedDB=function(){return!!j("indexedDB",a)},C.hashchange=function(){return J("hashchange",a)&&(b.documentMode===c||b.documentMode>7)},C.history=function(){return!(!a.history||!history.pushState)},C.draganddrop=function(){var a=b.createElement("div");return"draggable"in a||"ondragstart"in a&&"ondrop"in a},C.websockets=function(){return"WebSocket"in a||"MozWebSocket"in a},C.rgba=function(){return d("background-color:rgba(150,255,150,.5)"),g(t.backgroundColor,"rgba")},C.hsla=function(){return d("background-color:hsla(120,40%,100%,.5)"),g(t.backgroundColor,"rgba")||g(t.backgroundColor,"hsla")},C.multiplebgs=function(){return d("background:url(https://),url(https://),red url(https://)"),/(url\s*\(.*?){3}/.test(t.background)},C.backgroundsize=function(){return j("backgroundSize")},C.borderimage=function(){return j("borderImage")},C.borderradius=function(){return j("borderRadius")},C.boxshadow=function(){return j("boxShadow")},C.textshadow=function(){return""===b.createElement("div").style.textShadow},C.opacity=function(){return e("opacity:.55"),/^0.55$/.test(t.opacity)},C.cssanimations=function(){return j("animationName")},C.csscolumns=function(){return j("columnCount")},C.cssgradients=function(){var a="background-image:",b="gradient(linear,left top,right bottom,from(#9f9),to(white));",c="linear-gradient(left top,#9f9, white);";return d((a+"-webkit- ".split(" ").join(b+a)+x.join(c+a)).slice(0,-a.length)),g(t.backgroundImage,"gradient")},C.cssreflections=function(){return j("boxReflect")},C.csstransforms=function(){return!!j("transform")},C.csstransforms3d=function(){var a=!!j("perspective");return a&&"webkitPerspective"in q.style&&H("@media (transform-3d),(-webkit-transform-3d){#modernizr{left:9px;position:absolute;height:3px;}}",function(b,c){a=9===b.offsetLeft&&3===b.offsetHeight}),a},C.csstransitions=function(){return j("transition")},C.fontface=function(){var a;return H('@font-face {font-family:"font";src:url("https://")}',function(c,d){var e=b.getElementById("smodernizr"),f=e.sheet||e.styleSheet,g=f?f.cssRules&&f.cssRules[0]?f.cssRules[0].cssText:f.cssText||"":"";a=/src/i.test(g)&&0===g.indexOf(d.split(" ")[0])}),a},C.generatedcontent=function(){var a;return H(["#",r,"{font:0/0 a}#",r,':after{content:"',v,'";visibility:hidden;font:3px/1 a}'].join(""),function(b){a=b.offsetHeight>=3}),a},C.video=function(){var a=b.createElement("video"),c=!1;try{(c=!!a.canPlayType)&&(c=new Boolean(c),c.ogg=a.canPlayType('video/ogg; codecs="theora"').replace(/^no$/,""),c.h264=a.canPlayType('video/mp4; codecs="avc1.42E01E"').replace(/^no$/,""),c.webm=a.canPlayType('video/webm; codecs="vp8, vorbis"').replace(/^no$/,""))}catch(d){}return c},C.audio=function(){var a=b.createElement("audio"),c=!1;try{(c=!!a.canPlayType)&&(c=new Boolean(c),c.ogg=a.canPlayType('audio/ogg; codecs="vorbis"').replace(/^no$/,""),c.mp3=a.canPlayType("audio/mpeg;").replace(/^no$/,""),c.wav=a.canPlayType('audio/wav; codecs="1"').replace(/^no$/,""),c.m4a=(a.canPlayType("audio/x-m4a;")||a.canPlayType("audio/aac;")).replace(/^no$/,""))}catch(d){}return c},C.localstorage=function(){try{return localStorage.setItem(r,r),localStorage.removeItem(r),!0}catch(a){return!1}},C.sessionstorage=function(){try{return sessionStorage.setItem(r,r),sessionStorage.removeItem(r),!0}catch(a){return!1}},C.webworkers=function(){return!!a.Worker},C.applicationcache=function(){return!!a.applicationCache},C.svg=function(){return!!b.createElementNS&&!!b.createElementNS(B.svg,"svg").createSVGRect},C.inlinesvg=function(){var a=b.createElement("div");return a.innerHTML="<svg/>",(a.firstChild&&a.firstChild.namespaceURI)==B.svg},C.smil=function(){return!!b.createElementNS&&/SVGAnimate/.test(w.call(b.createElementNS(B.svg,"animate")))},C.svgclippaths=function(){return!!b.createElementNS&&/SVGClipPath/.test(w.call(b.createElementNS(B.svg,"clipPath")))};for(var L in C)m(C,L)&&(l=L.toLowerCase(),o[l]=C[L](),F.push((o[l]?"":"no-")+l));return o.input||k(),o.addTest=function(a,b){if("object"==typeof a)for(var d in a)m(a,d)&&o.addTest(d,a[d]);else{if(a=a.toLowerCase(),o[a]!==c)return o;b="function"==typeof b?b():b,"undefined"!=typeof p&&p&&(q.className+=" "+(b?"":"no-")+a),o[a]=b}return o},d(""),s=u=null,function(a,b){function c(a,b){var c=a.createElement("p"),d=a.getElementsByTagName("head")[0]||a.documentElement;return c.innerHTML="x<style>"+b+"</style>",d.insertBefore(c.lastChild,d.firstChild)}function d(){var a=s.elements;return"string"==typeof a?a.split(" "):a}function e(a){var b=r[a[p]];return b||(b={},q++,a[p]=q,r[q]=b),b}function f(a,c,d){if(c||(c=b),k)return c.createElement(a);d||(d=e(c));var f;return f=d.cache[a]?d.cache[a].cloneNode():o.test(a)?(d.cache[a]=d.createElem(a)).cloneNode():d.createElem(a),!f.canHaveChildren||n.test(a)||f.tagUrn?f:d.frag.appendChild(f)}function g(a,c){if(a||(a=b),k)return a.createDocumentFragment();c=c||e(a);for(var f=c.frag.cloneNode(),g=0,h=d(),i=h.length;i>g;g++)f.createElement(h[g]);return f}function h(a,b){b.cache||(b.cache={},b.createElem=a.createElement,b.createFrag=a.createDocumentFragment,b.frag=b.createFrag()),a.createElement=function(c){return s.shivMethods?f(c,a,b):b.createElem(c)},a.createDocumentFragment=Function("h,f","return function(){var n=f.cloneNode(),c=n.createElement;h.shivMethods&&("+d().join().replace(/[\w\-]+/g,function(a){return b.createElem(a),b.frag.createElement(a),'c("'+a+'")'})+");return n}")(s,b.frag)}function i(a){a||(a=b);var d=e(a);return!s.shivCSS||j||d.hasCSS||(d.hasCSS=!!c(a,"article,aside,dialog,figcaption,figure,footer,header,hgroup,main,nav,section{display:block}mark{background:#FF0;color:#000}template{display:none}")),k||h(a,d),a}var j,k,l="3.7.0",m=a.html5||{},n=/^<|^(?:button|map|select|textarea|object|iframe|option|optgroup)$/i,o=/^(?:a|b|code|div|fieldset|h1|h2|h3|h4|h5|h6|i|label|li|ol|p|q|span|strong|style|table|tbody|td|th|tr|ul)$/i,p="_html5shiv",q=0,r={};!function(){try{var a=b.createElement("a");a.innerHTML="<xyz></xyz>",j="hidden"in a,k=1==a.childNodes.length||function(){b.createElement("a");var a=b.createDocumentFragment();return"undefined"==typeof a.cloneNode||"undefined"==typeof a.createDocumentFragment||"undefined"==typeof a.createElement}()}catch(c){j=!0,k=!0}}();var s={elements:m.elements||"abbr article aside audio bdi canvas data datalist details dialog figcaption figure footer header hgroup main mark meter nav output progress section summary template time video",version:l,shivCSS:m.shivCSS!==!1,supportsUnknownElements:k,shivMethods:m.shivMethods!==!1,type:"default",shivDocument:i,createElement:f,createDocumentFragment:g};a.html5=s,i(b)}(this,b),o._version=n,o._prefixes=x,o._domPrefixes=A,o._cssomPrefixes=z,o.mq=I,o.hasEvent=J,o.testProp=function(a){return h([a])},o.testAllProps=j,o.testStyles=H,o.prefixed=function(a,b,c){return b?j(a,b,c):j(a,"pfx")},q.className=q.className.replace(/(^|\s)no-js(\s|$)/,"$1$2")+(p?" js "+F.join(" "):""),o}(this,this.document);
define("modernizr", (function (global) {
    return function () {
        var ret, fn;
        return ret || global.Modernizr;
    };
}(this)));

/*
 * Foundation Responsive Library
 * http://foundation.zurb.com
 * Copyright 2014, ZURB
 * Free to use under the MIT license.
 * http://www.opensource.org/licenses/mit-license.php
*/

(function ($, window, document, undefined) {
  'use strict';

  var header_helpers = function (class_array) {
    var i = class_array.length;
    var head = $('head');

    while (i--) {
      if (head.has('.' + class_array[i]).length === 0) {
        head.append('<meta class="' + class_array[i] + '" />');
      }
    }
  };

  header_helpers([
    'foundation-mq-small',
    'foundation-mq-small-only',
    'foundation-mq-medium',
    'foundation-mq-medium-only',
    'foundation-mq-large',
    'foundation-mq-large-only',
    'foundation-mq-xlarge',
    'foundation-mq-xlarge-only',
    'foundation-mq-xxlarge',
    'foundation-data-attribute-namespace']);

  // Enable FastClick if present

  $(function () {
    if (typeof FastClick !== 'undefined') {
      // Don't attach to body if undefined
      if (typeof document.body !== 'undefined') {
        FastClick.attach(document.body);
      }
    }
  });

  // private Fast Selector wrapper,
  // returns jQuery object. Only use where
  // getElementById is not available.
  var S = function (selector, context) {
    if (typeof selector === 'string') {
      if (context) {
        var cont;
        if (context.jquery) {
          cont = context[0];
          if (!cont) {
            return context;
          }
        } else {
          cont = context;
        }
        return $(cont.querySelectorAll(selector));
      }

      return $(document.querySelectorAll(selector));
    }

    return $(selector, context);
  };

  // Namespace functions.

  var attr_name = function (init) {
    var arr = [];
    if (!init) {
      arr.push('data');
    }
    if (this.namespace.length > 0) {
      arr.push(this.namespace);
    }
    arr.push(this.name);

    return arr.join('-');
  };

  var add_namespace = function (str) {
    var parts = str.split('-'),
        i = parts.length,
        arr = [];

    while (i--) {
      if (i !== 0) {
        arr.push(parts[i]);
      } else {
        if (this.namespace.length > 0) {
          arr.push(this.namespace, parts[i]);
        } else {
          arr.push(parts[i]);
        }
      }
    }

    return arr.reverse().join('-');
  };

  // Event binding and data-options updating.

  var bindings = function (method, options) {
    var self = this,
        bind = function(){
          var $this = S(this),
              should_bind_events = !$this.data(self.attr_name(true) + '-init');
          $this.data(self.attr_name(true) + '-init', $.extend({}, self.settings, (options || method), self.data_options($this)));

          if (should_bind_events) {
            self.events(this);
          }
        };

    if (S(this.scope).is('[' + this.attr_name() +']')) {
      bind.call(this.scope);
    } else {
      S('[' + this.attr_name() +']', this.scope).each(bind);
    }
    // # Patch to fix #5043 to move this *after* the if/else clause in order for Backbone and similar frameworks to have improved control over event binding and data-options updating.
    if (typeof method === 'string') {
      return this[method].call(this, options);
    }

  };

  var single_image_loaded = function (image, callback) {
    function loaded () {
      callback(image[0]);
    }

    function bindLoad () {
      this.one('load', loaded);

      if (/MSIE (\d+\.\d+);/.test(navigator.userAgent)) {
        var src = this.attr( 'src' ),
            param = src.match( /\?/ ) ? '&' : '?';

        param += 'random=' + (new Date()).getTime();
        this.attr('src', src + param);
      }
    }

    if (!image.attr('src')) {
      loaded();
      return;
    }

    if (image[0].complete || image[0].readyState === 4) {
      loaded();
    } else {
      bindLoad.call(image);
    }
  };

  /*! matchMedia() polyfill - Test a CSS media type/query in JS. Authors & copyright (c) 2012: Scott Jehl, Paul Irish, Nicholas Zakas, David Knight. Dual MIT/BSD license */

  window.matchMedia || (window.matchMedia = function() {
      "use strict";

      // For browsers that support matchMedium api such as IE 9 and webkit
      var styleMedia = (window.styleMedia || window.media);

      // For those that don't support matchMedium
      if (!styleMedia) {
          var style       = document.createElement('style'),
              script      = document.getElementsByTagName('script')[0],
              info        = null;

          style.type  = 'text/css';
          style.id    = 'matchmediajs-test';

          script.parentNode.insertBefore(style, script);

          // 'style.currentStyle' is used by IE <= 8 and 'window.getComputedStyle' for all other browsers
          info = ('getComputedStyle' in window) && window.getComputedStyle(style, null) || style.currentStyle;

          styleMedia = {
              matchMedium: function(media) {
                  var text = '@media ' + media + '{ #matchmediajs-test { width: 1px; } }';

                  // 'style.styleSheet' is used by IE <= 8 and 'style.textContent' for all other browsers
                  if (style.styleSheet) {
                      style.styleSheet.cssText = text;
                  } else {
                      style.textContent = text;
                  }

                  // Test if media query is true or false
                  return info.width === '1px';
              }
          };
      }

      return function(media) {
          return {
              matches: styleMedia.matchMedium(media || 'all'),
              media: media || 'all'
          };
      };
  }());

  /*
   * jquery.requestAnimationFrame
   * https://github.com/gnarf37/jquery-requestAnimationFrame
   * Requires jQuery 1.8+
   *
   * Copyright (c) 2012 Corey Frang
   * Licensed under the MIT license.
   */

  (function(jQuery) {


  // requestAnimationFrame polyfill adapted from Erik Möller
  // fixes from Paul Irish and Tino Zijdel
  // http://paulirish.com/2011/requestanimationframe-for-smart-animating/
  // http://my.opera.com/emoller/blog/2011/12/20/requestanimationframe-for-smart-er-animating

  var animating,
      lastTime = 0,
      vendors = ['webkit', 'moz'],
      requestAnimationFrame = window.requestAnimationFrame,
      cancelAnimationFrame = window.cancelAnimationFrame,
      jqueryFxAvailable = 'undefined' !== typeof jQuery.fx;

  for (; lastTime < vendors.length && !requestAnimationFrame; lastTime++) {
    requestAnimationFrame = window[ vendors[lastTime] + 'RequestAnimationFrame' ];
    cancelAnimationFrame = cancelAnimationFrame ||
      window[ vendors[lastTime] + 'CancelAnimationFrame' ] ||
      window[ vendors[lastTime] + 'CancelRequestAnimationFrame' ];
  }

  function raf() {
    if (animating) {
      requestAnimationFrame(raf);

      if (jqueryFxAvailable) {
        jQuery.fx.tick();
      }
    }
  }

  if (requestAnimationFrame) {
    // use rAF
    window.requestAnimationFrame = requestAnimationFrame;
    window.cancelAnimationFrame = cancelAnimationFrame;

    if (jqueryFxAvailable) {
      jQuery.fx.timer = function (timer) {
        if (timer() && jQuery.timers.push(timer) && !animating) {
          animating = true;
          raf();
        }
      };

      jQuery.fx.stop = function () {
        animating = false;
      };
    }
  } else {
    // polyfill
    window.requestAnimationFrame = function (callback) {
      var currTime = new Date().getTime(),
        timeToCall = Math.max(0, 16 - (currTime - lastTime)),
        id = window.setTimeout(function () {
          callback(currTime + timeToCall);
        }, timeToCall);
      lastTime = currTime + timeToCall;
      return id;
    };

    window.cancelAnimationFrame = function (id) {
      clearTimeout(id);
    };

  }

  }( $ ));

  function removeQuotes (string) {
    if (typeof string === 'string' || string instanceof String) {
      string = string.replace(/^['\\/"]+|(;\s?})+|['\\/"]+$/g, '');
    }

    return string;
  }

  window.Foundation = {
    name : 'Foundation',

    version : '5.5.2',

    media_queries : {
      'small'       : S('.foundation-mq-small').css('font-family').replace(/^[\/\\'"]+|(;\s?})+|[\/\\'"]+$/g, ''),
      'small-only'  : S('.foundation-mq-small-only').css('font-family').replace(/^[\/\\'"]+|(;\s?})+|[\/\\'"]+$/g, ''),
      'medium'      : S('.foundation-mq-medium').css('font-family').replace(/^[\/\\'"]+|(;\s?})+|[\/\\'"]+$/g, ''),
      'medium-only' : S('.foundation-mq-medium-only').css('font-family').replace(/^[\/\\'"]+|(;\s?})+|[\/\\'"]+$/g, ''),
      'large'       : S('.foundation-mq-large').css('font-family').replace(/^[\/\\'"]+|(;\s?})+|[\/\\'"]+$/g, ''),
      'large-only'  : S('.foundation-mq-large-only').css('font-family').replace(/^[\/\\'"]+|(;\s?})+|[\/\\'"]+$/g, ''),
      'xlarge'      : S('.foundation-mq-xlarge').css('font-family').replace(/^[\/\\'"]+|(;\s?})+|[\/\\'"]+$/g, ''),
      'xlarge-only' : S('.foundation-mq-xlarge-only').css('font-family').replace(/^[\/\\'"]+|(;\s?})+|[\/\\'"]+$/g, ''),
      'xxlarge'     : S('.foundation-mq-xxlarge').css('font-family').replace(/^[\/\\'"]+|(;\s?})+|[\/\\'"]+$/g, '')
    },

    stylesheet : $('<style></style>').appendTo('head')[0].sheet,

    global : {
      namespace : undefined
    },

    init : function (scope, libraries, method, options, response) {
      var args = [scope, method, options, response],
          responses = [];

      // check RTL
      this.rtl = /rtl/i.test(S('html').attr('dir'));

      // set foundation global scope
      this.scope = scope || this.scope;

      this.set_namespace();

      if (libraries && typeof libraries === 'string' && !/reflow/i.test(libraries)) {
        if (this.libs.hasOwnProperty(libraries)) {
          responses.push(this.init_lib(libraries, args));
        }
      } else {
        for (var lib in this.libs) {
          responses.push(this.init_lib(lib, libraries));
        }
      }

      S(window).load(function () {
        S(window)
          .trigger('resize.fndtn.clearing')
          .trigger('resize.fndtn.dropdown')
          .trigger('resize.fndtn.equalizer')
          .trigger('resize.fndtn.interchange')
          .trigger('resize.fndtn.joyride')
          .trigger('resize.fndtn.magellan')
          .trigger('resize.fndtn.topbar')
          .trigger('resize.fndtn.slider');
      });

      return scope;
    },

    init_lib : function (lib, args) {
      if (this.libs.hasOwnProperty(lib)) {
        this.patch(this.libs[lib]);

        if (args && args.hasOwnProperty(lib)) {
            if (typeof this.libs[lib].settings !== 'undefined') {
              $.extend(true, this.libs[lib].settings, args[lib]);
            } else if (typeof this.libs[lib].defaults !== 'undefined') {
              $.extend(true, this.libs[lib].defaults, args[lib]);
            }
          return this.libs[lib].init.apply(this.libs[lib], [this.scope, args[lib]]);
        }

        args = args instanceof Array ? args : new Array(args);
        return this.libs[lib].init.apply(this.libs[lib], args);
      }

      return function () {};
    },

    patch : function (lib) {
      lib.scope = this.scope;
      lib.namespace = this.global.namespace;
      lib.rtl = this.rtl;
      lib['data_options'] = this.utils.data_options;
      lib['attr_name'] = attr_name;
      lib['add_namespace'] = add_namespace;
      lib['bindings'] = bindings;
      lib['S'] = this.utils.S;
    },

    inherit : function (scope, methods) {
      var methods_arr = methods.split(' '),
          i = methods_arr.length;

      while (i--) {
        if (this.utils.hasOwnProperty(methods_arr[i])) {
          scope[methods_arr[i]] = this.utils[methods_arr[i]];
        }
      }
    },

    set_namespace : function () {

      // Description:
      //    Don't bother reading the namespace out of the meta tag
      //    if the namespace has been set globally in javascript
      //
      // Example:
      //    Foundation.global.namespace = 'my-namespace';
      // or make it an empty string:
      //    Foundation.global.namespace = '';
      //
      //

      // If the namespace has not been set (is undefined), try to read it out of the meta element.
      // Otherwise use the globally defined namespace, even if it's empty ('')
      var namespace = ( this.global.namespace === undefined ) ? $('.foundation-data-attribute-namespace').css('font-family') : this.global.namespace;

      // Finally, if the namsepace is either undefined or false, set it to an empty string.
      // Otherwise use the namespace value.
      this.global.namespace = ( namespace === undefined || /false/i.test(namespace) ) ? '' : namespace;
    },

    libs : {},

    // methods that can be inherited in libraries
    utils : {

      // Description:
      //    Fast Selector wrapper returns jQuery object. Only use where getElementById
      //    is not available.
      //
      // Arguments:
      //    Selector (String): CSS selector describing the element(s) to be
      //    returned as a jQuery object.
      //
      //    Scope (String): CSS selector describing the area to be searched. Default
      //    is document.
      //
      // Returns:
      //    Element (jQuery Object): jQuery object containing elements matching the
      //    selector within the scope.
      S : S,

      // Description:
      //    Executes a function a max of once every n milliseconds
      //
      // Arguments:
      //    Func (Function): Function to be throttled.
      //
      //    Delay (Integer): Function execution threshold in milliseconds.
      //
      // Returns:
      //    Lazy_function (Function): Function with throttling applied.
      throttle : function (func, delay) {
        var timer = null;

        return function () {
          var context = this, args = arguments;

          if (timer == null) {
            timer = setTimeout(function () {
              func.apply(context, args);
              timer = null;
            }, delay);
          }
        };
      },

      // Description:
      //    Executes a function when it stops being invoked for n seconds
      //    Modified version of _.debounce() http://underscorejs.org
      //
      // Arguments:
      //    Func (Function): Function to be debounced.
      //
      //    Delay (Integer): Function execution threshold in milliseconds.
      //
      //    Immediate (Bool): Whether the function should be called at the beginning
      //    of the delay instead of the end. Default is false.
      //
      // Returns:
      //    Lazy_function (Function): Function with debouncing applied.
      debounce : function (func, delay, immediate) {
        var timeout, result;
        return function () {
          var context = this, args = arguments;
          var later = function () {
            timeout = null;
            if (!immediate) {
              result = func.apply(context, args);
            }
          };
          var callNow = immediate && !timeout;
          clearTimeout(timeout);
          timeout = setTimeout(later, delay);
          if (callNow) {
            result = func.apply(context, args);
          }
          return result;
        };
      },

      // Description:
      //    Parses data-options attribute
      //
      // Arguments:
      //    El (jQuery Object): Element to be parsed.
      //
      // Returns:
      //    Options (Javascript Object): Contents of the element's data-options
      //    attribute.
      data_options : function (el, data_attr_name) {
        data_attr_name = data_attr_name || 'options';
        var opts = {}, ii, p, opts_arr,
            data_options = function (el) {
              var namespace = Foundation.global.namespace;

              if (namespace.length > 0) {
                return el.data(namespace + '-' + data_attr_name);
              }

              return el.data(data_attr_name);
            };

        var cached_options = data_options(el);

        if (typeof cached_options === 'object') {
          return cached_options;
        }

        opts_arr = (cached_options || ':').split(';');
        ii = opts_arr.length;

        function isNumber (o) {
          return !isNaN (o - 0) && o !== null && o !== '' && o !== false && o !== true;
        }

        function trim (str) {
          if (typeof str === 'string') {
            return $.trim(str);
          }
          return str;
        }

        while (ii--) {
          p = opts_arr[ii].split(':');
          p = [p[0], p.slice(1).join(':')];

          if (/true/i.test(p[1])) {
            p[1] = true;
          }
          if (/false/i.test(p[1])) {
            p[1] = false;
          }
          if (isNumber(p[1])) {
            if (p[1].indexOf('.') === -1) {
              p[1] = parseInt(p[1], 10);
            } else {
              p[1] = parseFloat(p[1]);
            }
          }

          if (p.length === 2 && p[0].length > 0) {
            opts[trim(p[0])] = trim(p[1]);
          }
        }

        return opts;
      },

      // Description:
      //    Adds JS-recognizable media queries
      //
      // Arguments:
      //    Media (String): Key string for the media query to be stored as in
      //    Foundation.media_queries
      //
      //    Class (String): Class name for the generated <meta> tag
      register_media : function (media, media_class) {
        if (Foundation.media_queries[media] === undefined) {
          $('head').append('<meta class="' + media_class + '"/>');
          Foundation.media_queries[media] = removeQuotes($('.' + media_class).css('font-family'));
        }
      },

      // Description:
      //    Add custom CSS within a JS-defined media query
      //
      // Arguments:
      //    Rule (String): CSS rule to be appended to the document.
      //
      //    Media (String): Optional media query string for the CSS rule to be
      //    nested under.
      add_custom_rule : function (rule, media) {
        if (media === undefined && Foundation.stylesheet) {
          Foundation.stylesheet.insertRule(rule, Foundation.stylesheet.cssRules.length);
        } else {
          var query = Foundation.media_queries[media];

          if (query !== undefined) {
            Foundation.stylesheet.insertRule('@media ' +
              Foundation.media_queries[media] + '{ ' + rule + ' }', Foundation.stylesheet.cssRules.length);
          }
        }
      },

      // Description:
      //    Performs a callback function when an image is fully loaded
      //
      // Arguments:
      //    Image (jQuery Object): Image(s) to check if loaded.
      //
      //    Callback (Function): Function to execute when image is fully loaded.
      image_loaded : function (images, callback) {
        var self = this,
            unloaded = images.length;

        function pictures_has_height(images) {
          var pictures_number = images.length;

          for (var i = pictures_number - 1; i >= 0; i--) {
            if(images.attr('height') === undefined) {
              return false;
            };
          };

          return true;
        }

        if (unloaded === 0 || pictures_has_height(images)) {
          callback(images);
        }

        images.each(function () {
          single_image_loaded(self.S(this), function () {
            unloaded -= 1;
            if (unloaded === 0) {
              callback(images);
            }
          });
        });
      },

      // Description:
      //    Returns a random, alphanumeric string
      //
      // Arguments:
      //    Length (Integer): Length of string to be generated. Defaults to random
      //    integer.
      //
      // Returns:
      //    Rand (String): Pseudo-random, alphanumeric string.
      random_str : function () {
        if (!this.fidx) {
          this.fidx = 0;
        }
        this.prefix = this.prefix || [(this.name || 'F'), (+new Date).toString(36)].join('-');

        return this.prefix + (this.fidx++).toString(36);
      },

      // Description:
      //    Helper for window.matchMedia
      //
      // Arguments:
      //    mq (String): Media query
      //
      // Returns:
      //    (Boolean): Whether the media query passes or not
      match : function (mq) {
        return window.matchMedia(mq).matches;
      },

      // Description:
      //    Helpers for checking Foundation default media queries with JS
      //
      // Returns:
      //    (Boolean): Whether the media query passes or not

      is_small_up : function () {
        return this.match(Foundation.media_queries.small);
      },

      is_medium_up : function () {
        return this.match(Foundation.media_queries.medium);
      },

      is_large_up : function () {
        return this.match(Foundation.media_queries.large);
      },

      is_xlarge_up : function () {
        return this.match(Foundation.media_queries.xlarge);
      },

      is_xxlarge_up : function () {
        return this.match(Foundation.media_queries.xxlarge);
      },

      is_small_only : function () {
        return !this.is_medium_up() && !this.is_large_up() && !this.is_xlarge_up() && !this.is_xxlarge_up();
      },

      is_medium_only : function () {
        return this.is_medium_up() && !this.is_large_up() && !this.is_xlarge_up() && !this.is_xxlarge_up();
      },

      is_large_only : function () {
        return this.is_medium_up() && this.is_large_up() && !this.is_xlarge_up() && !this.is_xxlarge_up();
      },

      is_xlarge_only : function () {
        return this.is_medium_up() && this.is_large_up() && this.is_xlarge_up() && !this.is_xxlarge_up();
      },

      is_xxlarge_only : function () {
        return this.is_medium_up() && this.is_large_up() && this.is_xlarge_up() && this.is_xxlarge_up();
      }
    }
  };

  $.fn.foundation = function () {
    var args = Array.prototype.slice.call(arguments, 0);

    return this.each(function () {
      Foundation.init.apply(Foundation, [this].concat(args));
      return this;
    });
  };

}(jQuery, window, window.document));

define("foundation.core", ["jquery","modernizr"], (function (global) {
    return function () {
        var ret, fn;
        return ret || global.Foundation;
    };
}(this)));

;(function ($, window, document, undefined) {
  'use strict';

  Foundation.libs.dropdown = {
    name : 'dropdown',

    version : '5.5.2',

    settings : {
      active_class : 'open',
      disabled_class : 'disabled',
      mega_class : 'mega',
      align : 'bottom',
      is_hover : false,
      hover_timeout : 150,
      opened : function () {},
      closed : function () {}
    },

    init : function (scope, method, options) {
      Foundation.inherit(this, 'throttle');

      $.extend(true, this.settings, method, options);
      this.bindings(method, options);
    },

    events : function (scope) {
      var self = this,
          S = self.S;

      S(this.scope)
        .off('.dropdown')
        .on('click.fndtn.dropdown', '[' + this.attr_name() + ']', function (e) {
          var settings = S(this).data(self.attr_name(true) + '-init') || self.settings;
          if (!settings.is_hover || Modernizr.touch) {
            e.preventDefault();
            if (S(this).parent('[data-reveal-id]').length) {
              e.stopPropagation();
            }
            self.toggle($(this));
          }
        })
        .on('mouseenter.fndtn.dropdown', '[' + this.attr_name() + '], [' + this.attr_name() + '-content]', function (e) {
          var $this = S(this),
              dropdown,
              target;

          clearTimeout(self.timeout);

          if ($this.data(self.data_attr())) {
            dropdown = S('#' + $this.data(self.data_attr()));
            target = $this;
          } else {
            dropdown = $this;
            target = S('[' + self.attr_name() + '="' + dropdown.attr('id') + '"]');
          }

          var settings = target.data(self.attr_name(true) + '-init') || self.settings;

          if (S(e.currentTarget).data(self.data_attr()) && settings.is_hover) {
            self.closeall.call(self);
          }

          if (settings.is_hover) {
            self.open.apply(self, [dropdown, target]);
          }
        })
        .on('mouseleave.fndtn.dropdown', '[' + this.attr_name() + '], [' + this.attr_name() + '-content]', function (e) {
          var $this = S(this);
          var settings;

          if ($this.data(self.data_attr())) {
              settings = $this.data(self.data_attr(true) + '-init') || self.settings;
          } else {
              var target   = S('[' + self.attr_name() + '="' + S(this).attr('id') + '"]'),
                  settings = target.data(self.attr_name(true) + '-init') || self.settings;
          }

          self.timeout = setTimeout(function () {
            if ($this.data(self.data_attr())) {
              if (settings.is_hover) {
                self.close.call(self, S('#' + $this.data(self.data_attr())));
              }
            } else {
              if (settings.is_hover) {
                self.close.call(self, $this);
              }
            }
          }.bind(this), settings.hover_timeout);
        })
        .on('click.fndtn.dropdown', function (e) {
          var parent = S(e.target).closest('[' + self.attr_name() + '-content]');
          var links  = parent.find('a');

          if (links.length > 0 && parent.attr('aria-autoclose') !== 'false') {
              self.close.call(self, S('[' + self.attr_name() + '-content]'));
          }

          if (e.target !== document && !$.contains(document.documentElement, e.target)) {
            return;
          }

          if (S(e.target).closest('[' + self.attr_name() + ']').length > 0) {
            return;
          }

          if (!(S(e.target).data('revealId')) &&
            (parent.length > 0 && (S(e.target).is('[' + self.attr_name() + '-content]') ||
              $.contains(parent.first()[0], e.target)))) {
            e.stopPropagation();
            return;
          }

          self.close.call(self, S('[' + self.attr_name() + '-content]'));
        })
        .on('opened.fndtn.dropdown', '[' + self.attr_name() + '-content]', function () {
          self.settings.opened.call(this);
        })
        .on('closed.fndtn.dropdown', '[' + self.attr_name() + '-content]', function () {
          self.settings.closed.call(this);
        });

      S(window)
        .off('.dropdown')
        .on('resize.fndtn.dropdown', self.throttle(function () {
          self.resize.call(self);
        }, 50));

      this.resize();
    },

    close : function (dropdown) {
      var self = this;
      dropdown.each(function (idx) {
        var original_target = $('[' + self.attr_name() + '=' + dropdown[idx].id + ']') || $('aria-controls=' + dropdown[idx].id + ']');
        original_target.attr('aria-expanded', 'false');
        if (self.S(this).hasClass(self.settings.active_class)) {
          self.S(this)
            .css(Foundation.rtl ? 'right' : 'left', '-99999px')
            .attr('aria-hidden', 'true')
            .removeClass(self.settings.active_class)
            .prev('[' + self.attr_name() + ']')
            .removeClass(self.settings.active_class)
            .removeData('target');

          self.S(this).trigger('closed.fndtn.dropdown', [dropdown]);
        }
      });
      dropdown.removeClass('f-open-' + this.attr_name(true));
    },

    closeall : function () {
      var self = this;
      $.each(self.S('.f-open-' + this.attr_name(true)), function () {
        self.close.call(self, self.S(this));
      });
    },

    open : function (dropdown, target) {
      this
        .css(dropdown
        .addClass(this.settings.active_class), target);
      dropdown.prev('[' + this.attr_name() + ']').addClass(this.settings.active_class);
      dropdown.data('target', target.get(0)).trigger('opened.fndtn.dropdown', [dropdown, target]);
      dropdown.attr('aria-hidden', 'false');
      target.attr('aria-expanded', 'true');
      dropdown.focus();
      dropdown.addClass('f-open-' + this.attr_name(true));
    },

    data_attr : function () {
      if (this.namespace.length > 0) {
        return this.namespace + '-' + this.name;
      }

      return this.name;
    },

    toggle : function (target) {
      if (target.hasClass(this.settings.disabled_class)) {
        return;
      }
      var dropdown = this.S('#' + target.data(this.data_attr()));
      if (dropdown.length === 0) {
        // No dropdown found, not continuing
        return;
      }

      this.close.call(this, this.S('[' + this.attr_name() + '-content]').not(dropdown));

      if (dropdown.hasClass(this.settings.active_class)) {
        this.close.call(this, dropdown);
        if (dropdown.data('target') !== target.get(0)) {
          this.open.call(this, dropdown, target);
        }
      } else {
        this.open.call(this, dropdown, target);
      }
    },

    resize : function () {
      var dropdown = this.S('[' + this.attr_name() + '-content].open');
      var target = $(dropdown.data("target"));

      if (dropdown.length && target.length) {
        this.css(dropdown, target);
      }
    },

    css : function (dropdown, target) {
      var left_offset = Math.max((target.width() - dropdown.width()) / 2, 8),
          settings = target.data(this.attr_name(true) + '-init') || this.settings,
          parentOverflow = dropdown.parent().css('overflow-y') || dropdown.parent().css('overflow');

      this.clear_idx();



      if (this.small()) {
        var p = this.dirs.bottom.call(dropdown, target, settings);

        dropdown.attr('style', '').removeClass('drop-left drop-right drop-top').css({
          position : 'absolute',
          width : '95%',
          'max-width' : 'none',
          top : p.top
        });

        dropdown.css(Foundation.rtl ? 'right' : 'left', left_offset);
      }
      // detect if dropdown is in an overflow container
      else if (parentOverflow !== 'visible') {
        var offset = target[0].offsetTop + target[0].offsetHeight;

        dropdown.attr('style', '').css({
          position : 'absolute',
          top : offset
        });

        dropdown.css(Foundation.rtl ? 'right' : 'left', left_offset);
      }
      else {

        this.style(dropdown, target, settings);
      }

      return dropdown;
    },

    style : function (dropdown, target, settings) {
      var css = $.extend({position : 'absolute'},
        this.dirs[settings.align].call(dropdown, target, settings));

      dropdown.attr('style', '').css(css);
    },

    // return CSS property object
    // `this` is the dropdown
    dirs : {
      // Calculate target offset
      _base : function (t) {
        var o_p = this.offsetParent(),
            o = o_p.offset(),
            p = t.offset();

        p.top -= o.top;
        p.left -= o.left;

        //set some flags on the p object to pass along
        p.missRight = false;
        p.missTop = false;
        p.missLeft = false;
        p.leftRightFlag = false;

        //lets see if the panel will be off the screen
        //get the actual width of the page and store it
        var actualBodyWidth;
        if (document.getElementsByClassName('row')[0]) {
          actualBodyWidth = document.getElementsByClassName('row')[0].clientWidth;
        } else {
          actualBodyWidth = window.innerWidth;
        }

        var actualMarginWidth = (window.innerWidth - actualBodyWidth) / 2;
        var actualBoundary = actualBodyWidth;

        if (!this.hasClass('mega')) {
          //miss top
          if (t.offset().top <= this.outerHeight()) {
            p.missTop = true;
            actualBoundary = window.innerWidth - actualMarginWidth;
            p.leftRightFlag = true;
          }

          //miss right
          if (t.offset().left + this.outerWidth() > t.offset().left + actualMarginWidth && t.offset().left - actualMarginWidth > this.outerWidth()) {
            p.missRight = true;
            p.missLeft = false;
          }

          //miss left
          if (t.offset().left - this.outerWidth() <= 0) {
            p.missLeft = true;
            p.missRight = false;
          }
        }

        return p;
      },

      top : function (t, s) {
        var self = Foundation.libs.dropdown,
            p = self.dirs._base.call(this, t);

        this.addClass('drop-top');

        if (p.missTop == true) {
          p.top = p.top + t.outerHeight() + this.outerHeight();
          this.removeClass('drop-top');
        }

        if (p.missRight == true) {
          p.left = p.left - this.outerWidth() + t.outerWidth();
        }

        if (t.outerWidth() < this.outerWidth() || self.small() || this.hasClass(s.mega_menu)) {
          self.adjust_pip(this, t, s, p);
        }

        if (Foundation.rtl) {
          return {left : p.left - this.outerWidth() + t.outerWidth(),
            top : p.top - this.outerHeight()};
        }

        return {left : p.left, top : p.top - this.outerHeight()};
      },

      bottom : function (t, s) {
        var self = Foundation.libs.dropdown,
            p = self.dirs._base.call(this, t);

        if (p.missRight == true) {
          p.left = p.left - this.outerWidth() + t.outerWidth();
        }

        if (t.outerWidth() < this.outerWidth() || self.small() || this.hasClass(s.mega_menu)) {
          self.adjust_pip(this, t, s, p);
        }

        if (self.rtl) {
          return {left : p.left - this.outerWidth() + t.outerWidth(), top : p.top + t.outerHeight()};
        }

        return {left : p.left, top : p.top + t.outerHeight()};
      },

      left : function (t, s) {
        var p = Foundation.libs.dropdown.dirs._base.call(this, t);

        this.addClass('drop-left');

        if (p.missLeft == true) {
          p.left =  p.left + this.outerWidth();
          p.top = p.top + t.outerHeight();
          this.removeClass('drop-left');
        }

        return {left : p.left - this.outerWidth(), top : p.top};
      },

      right : function (t, s) {
        var p = Foundation.libs.dropdown.dirs._base.call(this, t);

        this.addClass('drop-right');

        if (p.missRight == true) {
          p.left = p.left - this.outerWidth();
          p.top = p.top + t.outerHeight();
          this.removeClass('drop-right');
        } else {
          p.triggeredRight = true;
        }

        var self = Foundation.libs.dropdown;

        if (t.outerWidth() < this.outerWidth() || self.small() || this.hasClass(s.mega_menu)) {
          self.adjust_pip(this, t, s, p);
        }

        return {left : p.left + t.outerWidth(), top : p.top};
      }
    },

    // Insert rule to style psuedo elements
    adjust_pip : function (dropdown, target, settings, position) {
      var sheet = Foundation.stylesheet,
          pip_offset_base = 8;

      if (dropdown.hasClass(settings.mega_class)) {
        pip_offset_base = position.left + (target.outerWidth() / 2) - 8;
      } else if (this.small()) {
        pip_offset_base += position.left - 8;
      }

      this.rule_idx = sheet.cssRules.length;

      //default
      var sel_before = '.f-dropdown.open:before',
          sel_after  = '.f-dropdown.open:after',
          css_before = 'left: ' + pip_offset_base + 'px;',
          css_after  = 'left: ' + (pip_offset_base - 1) + 'px;';

      if (position.missRight == true) {
        pip_offset_base = dropdown.outerWidth() - 23;
        sel_before = '.f-dropdown.open:before',
        sel_after  = '.f-dropdown.open:after',
        css_before = 'left: ' + pip_offset_base + 'px;',
        css_after  = 'left: ' + (pip_offset_base - 1) + 'px;';
      }

      //just a case where right is fired, but its not missing right
      if (position.triggeredRight == true) {
        sel_before = '.f-dropdown.open:before',
        sel_after  = '.f-dropdown.open:after',
        css_before = 'left:-12px;',
        css_after  = 'left:-14px;';
      }

      if (sheet.insertRule) {
        sheet.insertRule([sel_before, '{', css_before, '}'].join(' '), this.rule_idx);
        sheet.insertRule([sel_after, '{', css_after, '}'].join(' '), this.rule_idx + 1);
      } else {
        sheet.addRule(sel_before, css_before, this.rule_idx);
        sheet.addRule(sel_after, css_after, this.rule_idx + 1);
      }
    },

    // Remove old dropdown rule index
    clear_idx : function () {
      var sheet = Foundation.stylesheet;

      if (typeof this.rule_idx !== 'undefined') {
        sheet.deleteRule(this.rule_idx);
        sheet.deleteRule(this.rule_idx);
        delete this.rule_idx;
      }
    },

    small : function () {
      return matchMedia(Foundation.media_queries.small).matches &&
        !matchMedia(Foundation.media_queries.medium).matches;
    },

    off : function () {
      this.S(this.scope).off('.fndtn.dropdown');
      this.S('html, body').off('.fndtn.dropdown');
      this.S(window).off('.fndtn.dropdown');
      this.S('[data-dropdown-content]').off('.fndtn.dropdown');
    },

    reflow : function () {}
  };
}(jQuery, window, window.document));

define("foundation.dropdown", ["foundation.core"], function(){});

define('tabs/dashboard-tab/tabId',['require'],function(require) {
  return 'dashboard-tab';
});

define('tabs/settings-tab/tabId',['require'],function(require){
  return 'settings-tab';
});
define('tabs/provision-tab/tabId',['require'],function(require) {
  return 'provision-tab';
});

;(function ($, window, document, undefined) {
  'use strict';

  Foundation.libs.reveal = {
    name : 'reveal',

    version : '5.5.2',

    locked : false,

    settings : {
      animation : 'fadeAndPop',
      animation_speed : 250,
      close_on_background_click : true,
      close_on_esc : true,
      dismiss_modal_class : 'close-reveal-modal',
      multiple_opened : false,
      bg_class : 'reveal-modal-bg',
      root_element : 'body',
      open : function(){},
      opened : function(){},
      close : function(){},
      closed : function(){},
      on_ajax_error: $.noop,
      bg : $('.reveal-modal-bg'),
      css : {
        open : {
          'opacity' : 0,
          'visibility' : 'visible',
          'display' : 'block'
        },
        close : {
          'opacity' : 1,
          'visibility' : 'hidden',
          'display' : 'none'
        }
      }
    },

    init : function (scope, method, options) {
      $.extend(true, this.settings, method, options);
      this.bindings(method, options);
    },

    events : function (scope) {
      var self = this,
          S = self.S;

      S(this.scope)
        .off('.reveal')
        .on('click.fndtn.reveal', '[' + this.add_namespace('data-reveal-id') + ']:not([disabled])', function (e) {
          e.preventDefault();

          if (!self.locked) {
            var element = S(this),
                ajax = element.data(self.data_attr('reveal-ajax')),
                replaceContentSel = element.data(self.data_attr('reveal-replace-content'));

            self.locked = true;

            if (typeof ajax === 'undefined') {
              self.open.call(self, element);
            } else {
              var url = ajax === true ? element.attr('href') : ajax;
              self.open.call(self, element, {url : url}, { replaceContentSel : replaceContentSel });
            }
          }
        });

      S(document)
        .on('click.fndtn.reveal', this.close_targets(), function (e) {
          e.preventDefault();
          if (!self.locked) {
            var settings = S('[' + self.attr_name() + '].open').data(self.attr_name(true) + '-init') || self.settings,
                bg_clicked = S(e.target)[0] === S('.' + settings.bg_class)[0];

            if (bg_clicked) {
              if (settings.close_on_background_click) {
                e.stopPropagation();
              } else {
                return;
              }
            }

            self.locked = true;
            self.close.call(self, bg_clicked ? S('[' + self.attr_name() + '].open:not(.toback)') : S(this).closest('[' + self.attr_name() + ']'));
          }
        });

      if (S('[' + self.attr_name() + ']', this.scope).length > 0) {
        S(this.scope)
          // .off('.reveal')
          .on('open.fndtn.reveal', this.settings.open)
          .on('opened.fndtn.reveal', this.settings.opened)
          .on('opened.fndtn.reveal', this.open_video)
          .on('close.fndtn.reveal', this.settings.close)
          .on('closed.fndtn.reveal', this.settings.closed)
          .on('closed.fndtn.reveal', this.close_video);
      } else {
        S(this.scope)
          // .off('.reveal')
          .on('open.fndtn.reveal', '[' + self.attr_name() + ']', this.settings.open)
          .on('opened.fndtn.reveal', '[' + self.attr_name() + ']', this.settings.opened)
          .on('opened.fndtn.reveal', '[' + self.attr_name() + ']', this.open_video)
          .on('close.fndtn.reveal', '[' + self.attr_name() + ']', this.settings.close)
          .on('closed.fndtn.reveal', '[' + self.attr_name() + ']', this.settings.closed)
          .on('closed.fndtn.reveal', '[' + self.attr_name() + ']', this.close_video);
      }

      return true;
    },

    // PATCH #3: turning on key up capture only when a reveal window is open
    key_up_on : function (scope) {
      var self = this;

      // PATCH #1: fixing multiple keyup event trigger from single key press
      self.S('body').off('keyup.fndtn.reveal').on('keyup.fndtn.reveal', function ( event ) {
        var open_modal = self.S('[' + self.attr_name() + '].open'),
            settings = open_modal.data(self.attr_name(true) + '-init') || self.settings ;
        // PATCH #2: making sure that the close event can be called only while unlocked,
        //           so that multiple keyup.fndtn.reveal events don't prevent clean closing of the reveal window.
        if ( settings && event.which === 27  && settings.close_on_esc && !self.locked) { // 27 is the keycode for the Escape key
          self.close.call(self, open_modal);
        }
      });

      return true;
    },

    // PATCH #3: turning on key up capture only when a reveal window is open
    key_up_off : function (scope) {
      this.S('body').off('keyup.fndtn.reveal');
      return true;
    },

    open : function (target, ajax_settings) {
      var self = this,
          modal;

      if (target) {
        if (typeof target.selector !== 'undefined') {
          // Find the named node; only use the first one found, since the rest of the code assumes there's only one node
          modal = self.S('#' + target.data(self.data_attr('reveal-id'))).first();
        } else {
          modal = self.S(this.scope);

          ajax_settings = target;
        }
      } else {
        modal = self.S(this.scope);
      }

      var settings = modal.data(self.attr_name(true) + '-init');
      settings = settings || this.settings;


      if (modal.hasClass('open') && target.attr('data-reveal-id') == modal.attr('id')) {
        return self.close(modal);
      }

      if (!modal.hasClass('open')) {
        var open_modal = self.S('[' + self.attr_name() + '].open');

        if (typeof modal.data('css-top') === 'undefined') {
          modal.data('css-top', parseInt(modal.css('top'), 10))
            .data('offset', this.cache_offset(modal));
        }

        modal.attr('tabindex','0').attr('aria-hidden','false');

        this.key_up_on(modal);    // PATCH #3: turning on key up capture only when a reveal window is open

        // Prevent namespace event from triggering twice
        modal.on('open.fndtn.reveal', function(e) {
          if (e.namespace !== 'fndtn.reveal') return;
        });

        modal.on('open.fndtn.reveal').trigger('open.fndtn.reveal');

        if (open_modal.length < 1) {
          this.toggle_bg(modal, true);
        }

        if (typeof ajax_settings === 'string') {
          ajax_settings = {
            url : ajax_settings
          };
        }

        if (typeof ajax_settings === 'undefined' || !ajax_settings.url) {
          if (open_modal.length > 0) {
            if (settings.multiple_opened) {
              self.to_back(open_modal);
            } else {
              self.hide(open_modal, settings.css.close);
            }
          }

          this.show(modal, settings.css.open);
        } else {
          var old_success = typeof ajax_settings.success !== 'undefined' ? ajax_settings.success : null;
          $.extend(ajax_settings, {
            success : function (data, textStatus, jqXHR) {
              if ( $.isFunction(old_success) ) {
                var result = old_success(data, textStatus, jqXHR);
                if (typeof result == 'string') {
                  data = result;
                }
              }

              if (typeof options !== 'undefined' && typeof options.replaceContentSel !== 'undefined') {
                modal.find(options.replaceContentSel).html(data);
              } else {
                modal.html(data);
              }

              self.S(modal).foundation('section', 'reflow');
              self.S(modal).children().foundation();

              if (open_modal.length > 0) {
                if (settings.multiple_opened) {
                  self.to_back(open_modal);
                } else {
                  self.hide(open_modal, settings.css.close);
                }
              }
              self.show(modal, settings.css.open);
            }
          });

          // check for if user initalized with error callback
          if (settings.on_ajax_error !== $.noop) {
            $.extend(ajax_settings, {
              error : settings.on_ajax_error
            });
          }

          $.ajax(ajax_settings);
        }
      }
      self.S(window).trigger('resize');
    },

    close : function (modal) {
      var modal = modal && modal.length ? modal : this.S(this.scope),
          open_modals = this.S('[' + this.attr_name() + '].open'),
          settings = modal.data(this.attr_name(true) + '-init') || this.settings,
          self = this;

      if (open_modals.length > 0) {

        modal.removeAttr('tabindex','0').attr('aria-hidden','true');

        this.locked = true;
        this.key_up_off(modal);   // PATCH #3: turning on key up capture only when a reveal window is open

        modal.trigger('close.fndtn.reveal');

        if ((settings.multiple_opened && open_modals.length === 1) || !settings.multiple_opened || modal.length > 1) {
          self.toggle_bg(modal, false);
          self.to_front(modal);
        }

        if (settings.multiple_opened) {
          self.hide(modal, settings.css.close, settings);
          self.to_front($($.makeArray(open_modals).reverse()[1]));
        } else {
          self.hide(open_modals, settings.css.close, settings);
        }
      }
    },

    close_targets : function () {
      var base = '.' + this.settings.dismiss_modal_class;

      if (this.settings.close_on_background_click) {
        return base + ', .' + this.settings.bg_class;
      }

      return base;
    },

    toggle_bg : function (modal, state) {
      if (this.S('.' + this.settings.bg_class).length === 0) {
        this.settings.bg = $('<div />', {'class': this.settings.bg_class})
          .appendTo('body').hide();
      }

      var visible = this.settings.bg.filter(':visible').length > 0;
      if ( state != visible ) {
        if ( state == undefined ? visible : !state ) {
          this.hide(this.settings.bg);
        } else {
          this.show(this.settings.bg);
        }
      }
    },

    show : function (el, css) {
      // is modal
      if (css) {
        var settings = el.data(this.attr_name(true) + '-init') || this.settings,
            root_element = settings.root_element,
            context = this;

        if (el.parent(root_element).length === 0) {
          var placeholder = el.wrap('<div style="display: none;" />').parent();

          el.on('closed.fndtn.reveal.wrapped', function () {
            el.detach().appendTo(placeholder);
            el.unwrap().unbind('closed.fndtn.reveal.wrapped');
          });

          el.detach().appendTo(root_element);
        }

        var animData = getAnimationData(settings.animation);
        if (!animData.animate) {
          this.locked = false;
        }
        if (animData.pop) {
          css.top = $(window).scrollTop() - el.data('offset') + 'px';
          var end_css = {
            top: $(window).scrollTop() + el.data('css-top') + 'px',
            opacity: 1
          };

          return setTimeout(function () {
            return el
              .css(css)
              .animate(end_css, settings.animation_speed, 'linear', function () {
                context.locked = false;
                el.trigger('opened.fndtn.reveal');
              })
              .addClass('open');
          }, settings.animation_speed / 2);
        }

        if (animData.fade) {
          css.top = $(window).scrollTop() + el.data('css-top') + 'px';
          var end_css = {opacity: 1};

          return setTimeout(function () {
            return el
              .css(css)
              .animate(end_css, settings.animation_speed, 'linear', function () {
                context.locked = false;
                el.trigger('opened.fndtn.reveal');
              })
              .addClass('open');
          }, settings.animation_speed / 2);
        }

        return el.css(css).show().css({opacity : 1}).addClass('open').trigger('opened.fndtn.reveal');
      }

      var settings = this.settings;

      // should we animate the background?
      if (getAnimationData(settings.animation).fade) {
        return el.fadeIn(settings.animation_speed / 2);
      }

      this.locked = false;

      return el.show();
    },

    to_back : function(el) {
      el.addClass('toback');
    },

    to_front : function(el) {
      el.removeClass('toback');
    },

    hide : function (el, css) {
      // is modal
      if (css) {
        var settings = el.data(this.attr_name(true) + '-init'),
            context = this;
        settings = settings || this.settings;

        var animData = getAnimationData(settings.animation);
        if (!animData.animate) {
          this.locked = false;
        }
        if (animData.pop) {
          var end_css = {
            top: - $(window).scrollTop() - el.data('offset') + 'px',
            opacity: 0
          };

          return setTimeout(function () {
            return el
              .animate(end_css, settings.animation_speed, 'linear', function () {
                context.locked = false;
                el.css(css).trigger('closed.fndtn.reveal');
              })
              .removeClass('open');
          }, settings.animation_speed / 2);
        }

        if (animData.fade) {
          var end_css = {opacity : 0};

          return setTimeout(function () {
            return el
              .animate(end_css, settings.animation_speed, 'linear', function () {
                context.locked = false;
                el.css(css).trigger('closed.fndtn.reveal');
              })
              .removeClass('open');
          }, settings.animation_speed / 2);
        }

        return el.hide().css(css).removeClass('open').trigger('closed.fndtn.reveal');
      }

      var settings = this.settings;

      // should we animate the background?
      if (getAnimationData(settings.animation).fade) {
        return el.fadeOut(settings.animation_speed / 2);
      }

      return el.hide();
    },

    close_video : function (e) {
      var video = $('.flex-video', e.target),
          iframe = $('iframe', video);

      if (iframe.length > 0) {
        iframe.attr('data-src', iframe[0].src);
        iframe.attr('src', iframe.attr('src'));
        video.hide();
      }
    },

    open_video : function (e) {
      var video = $('.flex-video', e.target),
          iframe = video.find('iframe');

      if (iframe.length > 0) {
        var data_src = iframe.attr('data-src');
        if (typeof data_src === 'string') {
          iframe[0].src = iframe.attr('data-src');
        } else {
          var src = iframe[0].src;
          iframe[0].src = undefined;
          iframe[0].src = src;
        }
        video.show();
      }
    },

    data_attr : function (str) {
      if (this.namespace.length > 0) {
        return this.namespace + '-' + str;
      }

      return str;
    },

    cache_offset : function (modal) {
      var offset = modal.show().height() + parseInt(modal.css('top'), 10) + modal.scrollY;

      modal.hide();

      return offset;
    },

    off : function () {
      $(this.scope).off('.fndtn.reveal');
    },

    reflow : function () {}
  };

  /*
   * getAnimationData('popAndFade') // {animate: true,  pop: true,  fade: true}
   * getAnimationData('fade')       // {animate: true,  pop: false, fade: true}
   * getAnimationData('pop')        // {animate: true,  pop: true,  fade: false}
   * getAnimationData('foo')        // {animate: false, pop: false, fade: false}
   * getAnimationData(null)         // {animate: false, pop: false, fade: false}
   */
  function getAnimationData(str) {
    var fade = /fade/i.test(str);
    var pop = /pop/i.test(str);
    return {
      animate : fade || pop,
      pop : pop,
      fade : fade
    };
  }
}(jQuery, window, window.document));

define("foundation.reveal", ["foundation.core"], function(){});

;(function ($, window, document, undefined) {
  'use strict';

  Foundation.libs.tab = {
    name : 'tab',

    version : '5.5.2',

    settings : {
      active_class : 'active',
      callback : function () {},
      deep_linking : false,
      scroll_to_content : true,
      is_hover : false
    },

    default_tab_hashes : [],

    init : function (scope, method, options) {
      var self = this,
          S = this.S;

	  // Store the default active tabs which will be referenced when the
	  // location hash is absent, as in the case of navigating the tabs and
	  // returning to the first viewing via the browser Back button.
	  S('[' + this.attr_name() + '] > .active > a', this.scope).each(function () {
	    self.default_tab_hashes.push(this.hash);
	  });

      // store the initial href, which is used to allow correct behaviour of the
      // browser back button when deep linking is turned on.
      self.entry_location = window.location.href;

      this.bindings(method, options);
      this.handle_location_hash_change();
    },

    events : function () {
      var self = this,
          S = this.S;

      var usual_tab_behavior =  function (e, target) {
          var settings = S(target).closest('[' + self.attr_name() + ']').data(self.attr_name(true) + '-init');
          if (!settings.is_hover || Modernizr.touch) {
            e.preventDefault();
            e.stopPropagation();
            self.toggle_active_tab(S(target).parent());
          }
        };

      S(this.scope)
        .off('.tab')
        // Key event: focus/tab key
        .on('keydown.fndtn.tab', '[' + this.attr_name() + '] > * > a', function(e) {
          var el = this;
          var keyCode = e.keyCode || e.which;
            // if user pressed tab key
            if (keyCode == 9) { 
              e.preventDefault();
              // TODO: Change usual_tab_behavior into accessibility function?
              usual_tab_behavior(e, el);
            } 
        })
        // Click event: tab title
        .on('click.fndtn.tab', '[' + this.attr_name() + '] > * > a', function(e) {
          var el = this;
          usual_tab_behavior(e, el);
        })
        // Hover event: tab title
        .on('mouseenter.fndtn.tab', '[' + this.attr_name() + '] > * > a', function (e) {
          var settings = S(this).closest('[' + self.attr_name() + ']').data(self.attr_name(true) + '-init');
          if (settings.is_hover) {
            self.toggle_active_tab(S(this).parent());
          }
        });

      // Location hash change event
      S(window).on('hashchange.fndtn.tab', function (e) {
        e.preventDefault();
        self.handle_location_hash_change();
      });
    },

    handle_location_hash_change : function () {

      var self = this,
          S = this.S;

      S('[' + this.attr_name() + ']', this.scope).each(function () {
        var settings = S(this).data(self.attr_name(true) + '-init');
        if (settings.deep_linking) {
          // Match the location hash to a label
          var hash;
          if (settings.scroll_to_content) {
            hash = self.scope.location.hash;
          } else {
            // prefix the hash to prevent anchor scrolling
            hash = self.scope.location.hash.replace('fndtn-', '');
          }
          if (hash != '') {
            // Check whether the location hash references a tab content div or
            // another element on the page (inside or outside the tab content div)
            var hash_element = S(hash);
            if (hash_element.hasClass('content') && hash_element.parent().hasClass('tabs-content')) {
              // Tab content div
              self.toggle_active_tab($('[' + self.attr_name() + '] > * > a[href=' + hash + ']').parent());
            } else {
              // Not the tab content div. If inside the tab content, find the
              // containing tab and toggle it as active.
              var hash_tab_container_id = hash_element.closest('.content').attr('id');
              if (hash_tab_container_id != undefined) {
                self.toggle_active_tab($('[' + self.attr_name() + '] > * > a[href=#' + hash_tab_container_id + ']').parent(), hash);
              }
            }
          } else {
            // Reference the default tab hashes which were initialized in the init function
            for (var ind = 0; ind < self.default_tab_hashes.length; ind++) {
              self.toggle_active_tab($('[' + self.attr_name() + '] > * > a[href=' + self.default_tab_hashes[ind] + ']').parent());
            }
          }
        }
       });
     },

    toggle_active_tab : function (tab, location_hash) {
      var self = this,
          S = self.S,
          tabs = tab.closest('[' + this.attr_name() + ']'),
          tab_link = tab.find('a'),
          anchor = tab.children('a').first(),
          target_hash = '#' + anchor.attr('href').split('#')[1],
          target = S(target_hash),
          siblings = tab.siblings(),
          settings = tabs.data(this.attr_name(true) + '-init'),
          interpret_keyup_action = function (e) {
            // Light modification of Heydon Pickering's Practical ARIA Examples: http://heydonworks.com/practical_aria_examples/js/a11y.js

            // define current, previous and next (possible) tabs

            var $original = $(this);
            var $prev = $(this).parents('li').prev().children('[role="tab"]');
            var $next = $(this).parents('li').next().children('[role="tab"]');
            var $target;

            // find the direction (prev or next)

            switch (e.keyCode) {
              case 37:
                $target = $prev;
                break;
              case 39:
                $target = $next;
                break;
              default:
                $target = false
                  break;
            }

            if ($target.length) {
              $original.attr({
                'tabindex' : '-1',
                'aria-selected' : null
              });
              $target.attr({
                'tabindex' : '0',
                'aria-selected' : true
              }).focus();
            }

            // Hide panels

            $('[role="tabpanel"]')
              .attr('aria-hidden', 'true');

            // Show panel which corresponds to target

            $('#' + $(document.activeElement).attr('href').substring(1))
              .attr('aria-hidden', null);

          },
          go_to_hash = function(hash) {
            // This function allows correct behaviour of the browser's back button when deep linking is enabled. Without it
            // the user would get continually redirected to the default hash.
            var is_entry_location = window.location.href === self.entry_location,
                default_hash = settings.scroll_to_content ? self.default_tab_hashes[0] : is_entry_location ? window.location.hash :'fndtn-' + self.default_tab_hashes[0].replace('#', '')

            if (!(is_entry_location && hash === default_hash)) {
              window.location.hash = hash;
            }
          };

      // allow usage of data-tab-content attribute instead of href
      if (anchor.data('tab-content')) {
        target_hash = '#' + anchor.data('tab-content').split('#')[1];
        target = S(target_hash);
      }

      if (settings.deep_linking) {

        if (settings.scroll_to_content) {

          // retain current hash to scroll to content
          go_to_hash(location_hash || target_hash);

          if (location_hash == undefined || location_hash == target_hash) {
            tab.parent()[0].scrollIntoView();
          } else {
            S(target_hash)[0].scrollIntoView();
          }
        } else {
          // prefix the hashes so that the browser doesn't scroll down
          if (location_hash != undefined) {
            go_to_hash('fndtn-' + location_hash.replace('#', ''));
          } else {
            go_to_hash('fndtn-' + target_hash.replace('#', ''));
          }
        }
      }

      // WARNING: The activation and deactivation of the tab content must
      // occur after the deep linking in order to properly refresh the browser
      // window (notably in Chrome).
      // Clean up multiple attr instances to done once
      tab.addClass(settings.active_class).triggerHandler('opened');
      tab_link.attr({'aria-selected' : 'true',  tabindex : 0});
      siblings.removeClass(settings.active_class)
      siblings.find('a').attr({'aria-selected' : 'false',  tabindex : -1});
      target.siblings().removeClass(settings.active_class).attr({'aria-hidden' : 'true',  tabindex : -1});
      target.addClass(settings.active_class).attr('aria-hidden', 'false').removeAttr('tabindex');
      settings.callback(tab);
      target.triggerHandler('toggled', [target]);
      tabs.triggerHandler('toggled', [tab]);

      tab_link.off('keydown').on('keydown', interpret_keyup_action );
    },

    data_attr : function (str) {
      if (this.namespace.length > 0) {
        return this.namespace + '-' + str;
      }

      return str;
    },

    off : function () {},

    reflow : function () {}
  };
}(jQuery, window, window.document));

define("foundation.tab", ["foundation.core"], function(){});

define('sunstone-config',['require','jquery'],function(require) {
  require('jquery');

  // Clone the local config object in a private var
  var _config = $.extend(true, {}, config);

  var Config = {
    'isTabEnabled': function(tabName) {
      var enabled = _config['view']['enabled_tabs'][tabName];
      return enabled;
    },

    "isTabActionEnabled": function(tabName, actionName, panelName) {
      var enabled = false;
      var configTab = _config['view']['tabs'][tabName];

      if (configTab != undefined) {
        if (panelName) {
          enabled = configTab['panel_tabs_actions'][panelName][actionName];
        } else {
          enabled = configTab['actions'][actionName];
        }
      }

      return enabled;
    },

    "isTabPanelEnabled": function(tabName, panelTabName) {
      if (_config['view']['tabs'][tabName]) {
        var enabled = _config['view']['tabs'][tabName]['panel_tabs'][panelTabName];
        return enabled;
      } else {
        return false;
      }
    },

    "isFeatureEnabled": function(featureName) {
      if (_config['view']['features'] && _config['view']['features'][featureName]) {
        return true;
      } else {
        return false;
      }
    },

    "tabTableColumns": function(tabName) {
      if (!_config['view']['tabs'][tabName]) {
        return [];
      }

      var columns = _config['view']['tabs'][tabName]['table_columns'];

      if (columns) {
        return columns;
      } else {
        return [];
      }
    },

    "isTemplateCreationTabEnabled": function(templateTabName) {
      if (_config['view']['tabs']['templates-tab']) {
        var enabled = _config['view']['tabs']['templates-tab']['template_creation_tabs'][templateTabName];
        return enabled;
      } else {
        return false;
      }
    },

    "dashboardWidgets": function(perRow) {
      if (!_config['view']['tabs']['dashboard-tab']) {
        return []
      }

      var widgets = _config['view']['tabs']['dashboard-tab'][perRow];

      if (widgets) {
        return widgets;
      } else {
        return [];
      }
    },

    "tableOrder": function() {
      return _config['user_config']["table_order"];
    },

    "provision": {
      "dashboard": {
        "isEnabled": function(widget) {
          if (_config['view']['tabs']['provision-tab']) {
            var enabled = _config['view']['tabs']['provision-tab']['dashboard'][widget];
            return enabled;
          } else {
            return false;
          }
        }
      },
      "create_vm": {
        "isEnabled": function(widget) {
          if (_config['view']['tabs']['provision-tab'] && _config['view']['tabs']['provision-tab']["create_vm"]) {
            return _config['view']['tabs']['provision-tab']['create_vm'][widget];
          } else {
            return false;
          }
        }
      },
      "logo": (_config['view']["provision_logo"] || "images/one_small_logo.png")
    },

    'autoRefresh' : _config['view']['autorefresh'],
    'tableOrder': _config['user_config']['table_order'],
    'vncProxyPort': _config['system_config']['vnc_proxy_port'],
    'vncWSS': _config['user_config']['vnc_wss'],
    'logo': (_config['view']["small_logo"] || "images/one_small_logo.png")
  }

  return Config;
});

define('utils/locale',['require'],function(require) {
  var LANGUAGE_OPTIONS =
    '<option value="en_US">English (en_US)</option>\
    <option value="ca">Catalan (ca)</option>\
    <option value="cs_CZ">Czech (cs_CZ)</option>\
    <option value="nl_NL">Dutch (nl_NL)</option>\
    <option value="da">Danish (da)</option>\
    <option value="fr_FR">French (fr_FR)</option>\
    <option value="de">German (de)</option>\
    <option value="el_GR">Greek (el_GR)</option>\
    <option value="it_IT">Italian (it_IT)</option>\
    <option value="ja">Japanese (ja)</option>\
    <option value="lt_LT">Lithuanian (lt_LT)</option>\
    <option value="fa_IR">Persian (fa_IR)</option>\
    <option value="pl">Polish (pl)</option>\
    <option value="pt_BR">Portuguese (pt_BR)</option>\
    <option value="pt_PT">Portuguese (pt_PT)</option>\
    <option value="ru_RU">Russian (ru_RU)</option>\
    <option value="zh_CN">Simplified Chinese (zh_CN)</option>\
    <option value="sk_SK">Slovak (sk_SK)</option>\
    <option value="es_ES">Spanish (es_ES)</option>\
    <option value="zh_TW">Traditional Chinese (zh_TW)</option>';

  var MONTHS = new Array(
        tr("January"), tr("February"), tr("March"), tr("April"), tr("May"),
        tr("June"), tr("July"), tr("August"), tr("September"), tr("October"),
        tr("November"), tr("December"));

  function tr(str) {
    var tmp = locale[str];
    if (tmp == null || tmp == "") {
      tmp = str;
    }
    return tmp;
  };

  return {
    'tr': tr,
    'months': MONTHS,
    'language_options': LANGUAGE_OPTIONS
  }
});

/**
 * jGrowl 1.4.2
 *
 * Dual licensed under the MIT (http://www.opensource.org/licenses/mit-license.php)
 * and GPL (http://www.opensource.org/licenses/gpl-license.php) licenses.
 *
 * Written by Stan Lemon <stosh1985@gmail.com>
 * Last updated: 2015.02.01
 *
 * jGrowl is a jQuery plugin implementing unobtrusive userland notifications.  These
 * notifications function similarly to the Growl Framework available for
 * Mac OS X (http://growl.info).
 *
 * To Do:
 * - Move library settings to containers and allow them to be changed per container
 *
 * Changes in 1.4.2
 * - Added word-break to less/css
 *
 * Changes in 1.4.1
 * - Added appendTo option
 * - jQuery compatibility updates
 * - Add check for closing a notification before it opens
 *
 * Changes in 1.4.0
 * - Removed IE6 support
 * - Added LESS support
 *
 * Changes in 1.3.0
 * - Added non-vendor border-radius to stylesheet
 * - Added grunt for generating minified js and css
 * - Added npm package info
 * - Added bower package info
 * - Updates for jshint
 *
 * Changes in 1.2.13
 * - Fixed clearing interval when the container shuts down
 *
 * Changes in 1.2.12
 * - Added compressed versions using UglifyJS and Sqwish
 * - Improved README with configuration options explanation
 * - Added a source map
 *
 * Changes in 1.2.11
 * - Fix artifacts left behind by the shutdown method and text-cleanup
 *
 * Changes in 1.2.10
 * - Fix beforeClose to be called in click event
 *
 * Changes in 1.2.9
 * - Fixed BC break in jQuery 2.0 beta
 *
 * Changes in 1.2.8
 * - Fixes for jQuery 1.9 and the MSIE6 check, note that with jQuery 2.0 support
 *   jGrowl intends to drop support for IE6 altogether
 *
 * Changes in 1.2.6
 * - Fixed js error when a notification is opening and closing at the same time
 *
 * Changes in 1.2.5
 * - Changed wrapper jGrowl's options usage to "o" instead of $.jGrowl.defaults
 * - Added themeState option to control 'highlight' or 'error' for jQuery UI
 * - Ammended some CSS to provide default positioning for nested usage.
 * - Changed some CSS to be prefixed with jGrowl- to prevent namespacing issues
 * - Added two new options - openDuration and closeDuration to allow
 *   better control of notification open and close speeds, respectively
 *   Patch contributed by Jesse Vincet.
 * - Added afterOpen callback.  Patch contributed by Russel Branca.
 *
 * Changes in 1.2.4
 * - Fixed IE bug with the close-all button
 * - Fixed IE bug with the filter CSS attribute (special thanks to gotwic)
 * - Update IE opacity CSS
 * - Changed font sizes to use "em", and only set the base style
 *
 * Changes in 1.2.3
 * - The callbacks no longer use the container as context, instead they use the actual notification
 * - The callbacks now receive the container as a parameter after the options parameter
 * - beforeOpen and beforeClose now check the return value, if it's false - the notification does
 *   not continue.  The open callback will also halt execution if it returns false.
 * - Fixed bug where containers would get confused
 * - Expanded the pause functionality to pause an entire container.
 *
 * Changes in 1.2.2
 * - Notification can now be theme rolled for jQuery UI, special thanks to Jeff Chan!
 *
 * Changes in 1.2.1
 * - Fixed instance where the interval would fire the close method multiple times.
 * - Added CSS to hide from print media
 * - Fixed issue with closer button when div { position: relative } is set
 * - Fixed leaking issue with multiple containers.  Special thanks to Matthew Hanlon!
 *
 * Changes in 1.2.0
 * - Added message pooling to limit the number of messages appearing at a given time.
 * - Closing a notification is now bound to the notification object and triggered by the close button.
 *
 * Changes in 1.1.2
 * - Added iPhone styled example
 * - Fixed possible IE7 bug when determining if the ie6 class shoudl be applied.
 * - Added template for the close button, so that it's content could be customized.
 *
 * Changes in 1.1.1
 * - Fixed CSS styling bug for ie6 caused by a mispelling
 * - Changes height restriction on default notifications to min-height
 * - Added skinned examples using a variety of images
 * - Added the ability to customize the content of the [close all] box
 * - Added jTweet, an example of using jGrowl + Twitter
 *
 * Changes in 1.1.0
 * - Multiple container and instances.
 * - Standard $.jGrowl() now wraps $.fn.jGrowl() by first establishing a generic jGrowl container.
 * - Instance methods of a jGrowl container can be called by $.fn.jGrowl(methodName)
 * - Added glue preferenced, which allows notifications to be inserted before or after nodes in the container
 * - Added new log callback which is called before anything is done for the notification
 * - Corner's attribute are now applied on an individual notification basis.
 *
 * Changes in 1.0.4
 * - Various CSS fixes so that jGrowl renders correctly in IE6.
 *
 * Changes in 1.0.3
 * - Fixed bug with options persisting across notifications
 * - Fixed theme application bug
 * - Simplified some selectors and manipulations.
 * - Added beforeOpen and beforeClose callbacks
 * - Reorganized some lines of code to be more readable
 * - Removed unnecessary this.defaults context
 * - If corners plugin is present, it's now customizable.
 * - Customizable open animation.
 * - Customizable close animation.
 * - Customizable animation easing.
 * - Added customizable positioning (top-left, top-right, bottom-left, bottom-right, center)
 *
 * Changes in 1.0.2
 * - All CSS styling is now external.
 * - Added a theme parameter which specifies a secondary class for styling, such
 *   that notifications can be customized in appearance on a per message basis.
 * - Notification life span is now customizable on a per message basis.
 * - Added the ability to disable the global closer, enabled by default.
 * - Added callbacks for when a notification is opened or closed.
 * - Added callback for the global closer.
 * - Customizable animation speed.
 * - jGrowl now set itself up and tears itself down.
 *
 * Changes in 1.0.1:
 * - Removed dependency on metadata plugin in favor of .data()
 * - Namespaced all events
 */
(function($) {
	/** jGrowl Wrapper - Establish a base jGrowl Container for compatibility with older releases. **/
	$.jGrowl = function( m , o ) {
		// To maintain compatibility with older version that only supported one instance we'll create the base container.
		if ( $('#jGrowl').length === 0 )
			$('<div id="jGrowl"></div>').addClass( (o && o.position) ? o.position : $.jGrowl.defaults.position ).appendTo( (o && o.appendTo) ? o.appendTo : $.jGrowl.defaults.appendTo );

		// Create a notification on the container.
		$('#jGrowl').jGrowl(m,o);
	};


	/** Raise jGrowl Notification on a jGrowl Container **/
	$.fn.jGrowl = function( m , o ) {
		// Short hand for passing in just an object to this method
		if ( o === undefined && $.isPlainObject(m) ) {
			o = m;
			m = o.message;
		}

		if ( $.isFunction(this.each) ) {
			var args = arguments;

			return this.each(function() {
				/** Create a jGrowl Instance on the Container if it does not exist **/
				if ( $(this).data('jGrowl.instance') === undefined ) {
					$(this).data('jGrowl.instance', $.extend( new $.fn.jGrowl(), { notifications: [], element: null, interval: null } ));
					$(this).data('jGrowl.instance').startup( this );
				}

				/** Optionally call jGrowl instance methods, or just raise a normal notification **/
				if ( $.isFunction($(this).data('jGrowl.instance')[m]) ) {
					$(this).data('jGrowl.instance')[m].apply( $(this).data('jGrowl.instance') , $.makeArray(args).slice(1) );
				} else {
					$(this).data('jGrowl.instance').create( m , o );
				}
			});
		}
	};

	$.extend( $.fn.jGrowl.prototype , {

		/** Default JGrowl Settings **/
		defaults: {
			pool:				0,
			header:				'',
			group:				'',
			sticky:				false,
			position:			'top-right',
			appendTo:			'body',
			glue:				'after',
			theme:				'default',
			themeState:			'highlight',
			corners:			'10px',
			check:				250,
			life:				3000,
			closeDuration:		'normal',
			openDuration:		'normal',
			easing:				'swing',
			closer:				true,
			closeTemplate:		'&times;',
			closerTemplate:		'<div>[ close all ]</div>',
			log:				function() {},
			beforeOpen:			function() {},
			afterOpen:			function() {},
			open:				function() {},
			beforeClose:		function() {},
			close:				function() {},
			click:				function() {},
			animateOpen:		{
				opacity:		'show'
			},
			animateClose:		{
				opacity:		'hide'
			}
		},

		notifications: [],

		/** jGrowl Container Node **/
		element:				null,

		/** Interval Function **/
		interval:				null,

		/** Create a Notification **/
		create: function( message , options ) {
			var o = $.extend({}, this.defaults, options);

			/* To keep backward compatibility with 1.24 and earlier, honor 'speed' if the user has set it */
			if (typeof o.speed !== 'undefined') {
				o.openDuration = o.speed;
				o.closeDuration = o.speed;
			}

			this.notifications.push({ message: message , options: o });

			o.log.apply( this.element , [this.element,message,o] );
		},

		render: function( n ) {
			var self = this;
			var message = n.message;
			var o = n.options;

			// Support for jQuery theme-states, if this is not used it displays a widget header
			o.themeState = (o.themeState === '') ? '' : 'ui-state-' + o.themeState;

			var notification = $('<div/>')
				.addClass('jGrowl-notification alert ' + o.themeState + ' ui-corner-all' + ((o.group !== undefined && o.group !== '') ? ' ' + o.group : ''))
				.append($('<button/>').addClass('jGrowl-close').html(o.closeTemplate))
				.append($('<div/>').addClass('jGrowl-header').html(o.header))
				.append($('<div/>').addClass('jGrowl-message').html(message))
				.data("jGrowl", o).addClass(o.theme).children('.jGrowl-close').bind("click.jGrowl", function() {
					$(this).parent().trigger('jGrowl.beforeClose');
					return false;
				})
				.parent();


			/** Notification Actions **/
			$(notification).bind("mouseover.jGrowl", function() {
				$('.jGrowl-notification', self.element).data("jGrowl.pause", true);
			}).bind("mouseout.jGrowl", function() {
				$('.jGrowl-notification', self.element).data("jGrowl.pause", false);
			}).bind('jGrowl.beforeOpen', function() {
				if ( o.beforeOpen.apply( notification , [notification,message,o,self.element] ) !== false ) {
					$(this).trigger('jGrowl.open');
				}
			}).bind('jGrowl.open', function() {
				if ( o.open.apply( notification , [notification,message,o,self.element] ) !== false ) {
					if ( o.glue == 'after' ) {
						$('.jGrowl-notification:last', self.element).after(notification);
					} else {
						$('.jGrowl-notification:first', self.element).before(notification);
					}

					$(this).animate(o.animateOpen, o.openDuration, o.easing, function() {
						// Fixes some anti-aliasing issues with IE filters.
						if ($.support.opacity === false)
							this.style.removeAttribute('filter');

						if ( $(this).data("jGrowl") !== null && typeof $(this).data("jGrowl") !== 'undefined') // Happens when a notification is closing before it's open.
							$(this).data("jGrowl").created = new Date();

						$(this).trigger('jGrowl.afterOpen');
					});
				}
			}).bind('jGrowl.afterOpen', function() {
				o.afterOpen.apply( notification , [notification,message,o,self.element] );
			}).bind('click', function() {
				o.click.apply( notification, [notification.message,o,self.element] );
			}).bind('jGrowl.beforeClose', function() {
				if ( o.beforeClose.apply( notification , [notification,message,o,self.element] ) !== false )
					$(this).trigger('jGrowl.close');
			}).bind('jGrowl.close', function() {
				// Pause the notification, lest during the course of animation another close event gets called.
				$(this).data('jGrowl.pause', true);
				$(this).animate(o.animateClose, o.closeDuration, o.easing, function() {
					if ( $.isFunction(o.close) ) {
						if ( o.close.apply( notification , [notification,message,o,self.element] ) !== false )
							$(this).remove();
					} else {
						$(this).remove();
					}
				});
			}).trigger('jGrowl.beforeOpen');

			/** Optional Corners Plugin **/
			if ( o.corners !== '' && $.fn.corner !== undefined ) $(notification).corner( o.corners );

			/** Add a Global Closer if more than one notification exists **/
			if ($('.jGrowl-notification:parent', self.element).length > 1 &&
				$('.jGrowl-closer', self.element).length === 0 && this.defaults.closer !== false ) {
				$(this.defaults.closerTemplate).addClass('jGrowl-closer ' + this.defaults.themeState + ' ui-corner-all').addClass(this.defaults.theme)
					.appendTo(self.element).animate(this.defaults.animateOpen, this.defaults.speed, this.defaults.easing)
					.bind("click.jGrowl", function() {
						$(this).siblings().trigger("jGrowl.beforeClose");

						if ( $.isFunction( self.defaults.closer ) ) {
							self.defaults.closer.apply( $(this).parent()[0] , [$(this).parent()[0]] );
						}
					});
			}
		},

		/** Update the jGrowl Container, removing old jGrowl notifications **/
		update: function() {
			$(this.element).find('.jGrowl-notification:parent').each( function() {
				if ($(this).data("jGrowl") !== undefined && $(this).data("jGrowl").created !== undefined &&
					($(this).data("jGrowl").created.getTime() + parseInt($(this).data("jGrowl").life, 10))  < (new Date()).getTime() &&
					$(this).data("jGrowl").sticky !== true &&
					($(this).data("jGrowl.pause") === undefined || $(this).data("jGrowl.pause") !== true) ) {

					// Pause the notification, lest during the course of animation another close event gets called.
					$(this).trigger('jGrowl.beforeClose');
				}
			});

			if (this.notifications.length > 0 &&
				(this.defaults.pool === 0 || $(this.element).find('.jGrowl-notification:parent').length < this.defaults.pool) )
				this.render( this.notifications.shift() );

			if ($(this.element).find('.jGrowl-notification:parent').length < 2 ) {
				$(this.element).find('.jGrowl-closer').animate(this.defaults.animateClose, this.defaults.speed, this.defaults.easing, function() {
					$(this).remove();
				});
			}
		},

		/** Setup the jGrowl Notification Container **/
		startup: function(e) {
			this.element = $(e).addClass('jGrowl').append('<div class="jGrowl-notification"></div>');
			this.interval = setInterval( function() {
				$(e).data('jGrowl.instance').update();
			}, parseInt(this.defaults.check, 10));
		},

		/** Shutdown jGrowl, removing it and clearing the interval **/
		shutdown: function() {
			$(this.element).removeClass('jGrowl')
				.find('.jGrowl-notification').trigger('jGrowl.close')
				.parent().empty()
			;

			clearInterval(this.interval);
		},

		close: function() {
			$(this.element).find('.jGrowl-notification').each(function(){
				$(this).trigger('jGrowl.beforeClose');
			});
		}
	});

	/** Reference the Defaults Object for compatibility with older versions of jGrowl **/
	$.jGrowl.defaults = $.fn.jGrowl.prototype.defaults;

})(jQuery);

define("jgrowl", ["jquery"], function(){});

define('utils/notifier',['require','jgrowl','utils/locale'],function(require) {
  require('jgrowl');

  var Locale = require('utils/locale');

  //Notification of submission of action
  var _notifySubmit = function(action, args, extra_param) {
    var action_text = action.replace(/OpenNebula\./, '').replace(/\./, ' ');

    var msg = "";
    if (!args || (typeof args == 'object' && args.constructor != Array)) {

      msg += action_text;
    } else {

      msg += action_text + ": " + args;
    };
    if (extra_param && extra_param.constructor != Object) {
      msg += " >> " + extra_param;
    };

    $.jGrowl(msg, {theme: "jGrowl-notify-submit", position: "bottom-right"});
  }

  //Notification on error
  var _notifyError = function(msg) {
    $.jGrowl(msg, {theme: "jGrowl-notify-error", position: "bottom-right", sticky: true});
  }

  //Standard notification
  var _notifyMessage = function(msg) {
    $.jGrowl(msg, {theme: "jGrowl-notify-submit", position: "bottom-right"});
  }

  var _notifyCustom = function(title, msg, sticky) {
    msg = (title ? title : "") + msg;
    $.jGrowl(msg, {theme: "jGrowl-notify-submit", position: "bottom-right", sticky: sticky});
  }

  //standard handling for the server errors on ajax requests.
  //Pops up a message with the information.
  var _onError = function(request, error_json, container) {
    var method;
    var action;
    var object;
    var id;
    var reason;
    var m;
    var message = error_json.error.message;

    //redirect to login if unauthenticated
    if (error_json.error.http_status == "401") {
      window.location.href = "login";
      return false;
    };

    if (!message) {
      _notifyError(Locale.tr("Cannot contact server: is it running and reachable?"));
      return false;
    };

    if (error_json.error.http_status == "404") {
      _notifyError(message);
      return false;
    }

    if (container) {
      container.show();
      return false;
    }

    if (message.match(/^Network is unreachable .+$/)) {
      _notifyError(Locale.tr("Network is unreachable: is OpenNebula running?"));
      return false;
    };

    //Parse known errors:
    var get_error = /^\[(\w+)\] Error getting ([\w ]+) \[(\d+)\]\.$/;
    var auth_error = /^\[(\w+)\] User \[(\d+)\] not authorized to perform action on ([\w ]+).$/;

    if (m = message.match(get_error)) {
      method  = m[1];
      action  = "Show";
      object  = m[2];
      id      = m[3];
    } else if (m = message.match(auth_error)) {
      method = m[1];
      object     = m[3];
      reason = Locale.tr("Unauthorized");
    };

    if (m) {
      var rows;
      var i;
      var value;
      rows = ["method", "action", "object", "id", "reason"];
      message = "";
      for (i = 0; i < rows.length; i++) {
        key = rows[i];
        value = eval(key);
        if (value)
            message += "<tr><td class=\"key_error\">" + key + "</td><td>" + value + "</td></tr>";
      }
      message = "<table>" + message + "</table>";
    };

    _notifyError(message);
    return true;
  }

  return {
    'notifySubmit': _notifySubmit,
    'notifyError': _notifyError,
    'notifyMessage': _notifyMessage,
    'notifyCustom': _notifyCustom,
    'onError': _onError
  }
});

/*!

 handlebars v2.0.0

Copyright (C) 2011-2014 by Yehuda Katz

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

@license
*/
/* exported Handlebars */
(function (root, factory) {
  if (typeof define === 'function' && define.amd) {
    define('hbs/handlebars',[], factory);
  } else if (typeof exports === 'object') {
    module.exports = factory();
  } else {
    root.Handlebars = root.Handlebars || factory();
  }
}(this, function () {
// handlebars/safe-string.js
var __module4__ = (function() {
  "use strict";
  var __exports__;
  // Build out our basic SafeString type
  function SafeString(string) {
    this.string = string;
  }

  SafeString.prototype.toString = function() {
    return "" + this.string;
  };

  __exports__ = SafeString;
  return __exports__;
})();

// handlebars/utils.js
var __module3__ = (function(__dependency1__) {
  "use strict";
  var __exports__ = {};
  /*jshint -W004 */
  var SafeString = __dependency1__;

  var escape = {
    "&": "&amp;",
    "<": "&lt;",
    ">": "&gt;",
    '"': "&quot;",
    "'": "&#x27;",
    "`": "&#x60;"
  };

  var badChars = /[&<>"'`]/g;
  var possible = /[&<>"'`]/;

  function escapeChar(chr) {
    return escape[chr];
  }

  function extend(obj /* , ...source */) {
    for (var i = 1; i < arguments.length; i++) {
      for (var key in arguments[i]) {
        if (Object.prototype.hasOwnProperty.call(arguments[i], key)) {
          obj[key] = arguments[i][key];
        }
      }
    }

    return obj;
  }

  __exports__.extend = extend;var toString = Object.prototype.toString;
  __exports__.toString = toString;
  // Sourced from lodash
  // https://github.com/bestiejs/lodash/blob/master/LICENSE.txt
  var isFunction = function(value) {
    return typeof value === 'function';
  };
  // fallback for older versions of Chrome and Safari
  /* istanbul ignore next */
  if (isFunction(/x/)) {
    isFunction = function(value) {
      return typeof value === 'function' && toString.call(value) === '[object Function]';
    };
  }
  var isFunction;
  __exports__.isFunction = isFunction;
  /* istanbul ignore next */
  var isArray = Array.isArray || function(value) {
    return (value && typeof value === 'object') ? toString.call(value) === '[object Array]' : false;
  };
  __exports__.isArray = isArray;

  function escapeExpression(string) {
    // don't escape SafeStrings, since they're already safe
    if (string instanceof SafeString) {
      return string.toString();
    } else if (string == null) {
      return "";
    } else if (!string) {
      return string + '';
    }

    // Force a string conversion as this will be done by the append regardless and
    // the regex test will do this transparently behind the scenes, causing issues if
    // an object's to string has escaped characters in it.
    string = "" + string;

    if(!possible.test(string)) { return string; }
    return string.replace(badChars, escapeChar);
  }

  __exports__.escapeExpression = escapeExpression;function isEmpty(value) {
    if (!value && value !== 0) {
      return true;
    } else if (isArray(value) && value.length === 0) {
      return true;
    } else {
      return false;
    }
  }

  __exports__.isEmpty = isEmpty;function appendContextPath(contextPath, id) {
    return (contextPath ? contextPath + '.' : '') + id;
  }

  __exports__.appendContextPath = appendContextPath;
  return __exports__;
})(__module4__);

// handlebars/exception.js
var __module5__ = (function() {
  "use strict";
  var __exports__;

  var errorProps = ['description', 'fileName', 'lineNumber', 'message', 'name', 'number', 'stack'];

  function Exception(message, node) {
    var line;
    if (node && node.firstLine) {
      line = node.firstLine;

      message += ' - ' + line + ':' + node.firstColumn;
    }

    var tmp = Error.prototype.constructor.call(this, message);

    // Unfortunately errors are not enumerable in Chrome (at least), so `for prop in tmp` doesn't work.
    for (var idx = 0; idx < errorProps.length; idx++) {
      this[errorProps[idx]] = tmp[errorProps[idx]];
    }

    if (line) {
      this.lineNumber = line;
      this.column = node.firstColumn;
    }
  }

  Exception.prototype = new Error();

  __exports__ = Exception;
  return __exports__;
})();

// handlebars/base.js
var __module2__ = (function(__dependency1__, __dependency2__) {
  "use strict";
  var __exports__ = {};
  var Utils = __dependency1__;
  var Exception = __dependency2__;

  var VERSION = "2.0.0";
  __exports__.VERSION = VERSION;var COMPILER_REVISION = 6;
  __exports__.COMPILER_REVISION = COMPILER_REVISION;
  var REVISION_CHANGES = {
    1: '<= 1.0.rc.2', // 1.0.rc.2 is actually rev2 but doesn't report it
    2: '== 1.0.0-rc.3',
    3: '== 1.0.0-rc.4',
    4: '== 1.x.x',
    5: '== 2.0.0-alpha.x',
    6: '>= 2.0.0-beta.1'
  };
  __exports__.REVISION_CHANGES = REVISION_CHANGES;
  var isArray = Utils.isArray,
      isFunction = Utils.isFunction,
      toString = Utils.toString,
      objectType = '[object Object]';

  function HandlebarsEnvironment(helpers, partials) {
    this.helpers = helpers || {};
    this.partials = partials || {};

    registerDefaultHelpers(this);
  }

  __exports__.HandlebarsEnvironment = HandlebarsEnvironment;HandlebarsEnvironment.prototype = {
    constructor: HandlebarsEnvironment,

    logger: logger,
    log: log,

    registerHelper: function(name, fn) {
      if (toString.call(name) === objectType) {
        if (fn) { throw new Exception('Arg not supported with multiple helpers'); }
        Utils.extend(this.helpers, name);
      } else {
        this.helpers[name] = fn;
      }
    },
    unregisterHelper: function(name) {
      delete this.helpers[name];
    },

    registerPartial: function(name, partial) {
      if (toString.call(name) === objectType) {
        Utils.extend(this.partials,  name);
      } else {
        this.partials[name] = partial;
      }
    },
    unregisterPartial: function(name) {
      delete this.partials[name];
    }
  };

  function registerDefaultHelpers(instance) {
    instance.registerHelper('helperMissing', function(/* [args, ]options */) {
      if(arguments.length === 1) {
        // A missing field in a {{foo}} constuct.
        return undefined;
      } else {
        // Someone is actually trying to call something, blow up.
        throw new Exception("Missing helper: '" + arguments[arguments.length-1].name + "'");
      }
    });

    instance.registerHelper('blockHelperMissing', function(context, options) {
      var inverse = options.inverse,
          fn = options.fn;

      if(context === true) {
        return fn(this);
      } else if(context === false || context == null) {
        return inverse(this);
      } else if (isArray(context)) {
        if(context.length > 0) {
          if (options.ids) {
            options.ids = [options.name];
          }

          return instance.helpers.each(context, options);
        } else {
          return inverse(this);
        }
      } else {
        if (options.data && options.ids) {
          var data = createFrame(options.data);
          data.contextPath = Utils.appendContextPath(options.data.contextPath, options.name);
          options = {data: data};
        }

        return fn(context, options);
      }
    });

    instance.registerHelper('each', function(context, options) {
      if (!options) {
        throw new Exception('Must pass iterator to #each');
      }

      var fn = options.fn, inverse = options.inverse;
      var i = 0, ret = "", data;

      var contextPath;
      if (options.data && options.ids) {
        contextPath = Utils.appendContextPath(options.data.contextPath, options.ids[0]) + '.';
      }

      if (isFunction(context)) { context = context.call(this); }

      if (options.data) {
        data = createFrame(options.data);
      }

      if(context && typeof context === 'object') {
        if (isArray(context)) {
          for(var j = context.length; i<j; i++) {
            if (data) {
              data.index = i;
              data.first = (i === 0);
              data.last  = (i === (context.length-1));

              if (contextPath) {
                data.contextPath = contextPath + i;
              }
            }
            ret = ret + fn(context[i], { data: data });
          }
        } else {
          for(var key in context) {
            if(context.hasOwnProperty(key)) {
              if(data) {
                data.key = key;
                data.index = i;
                data.first = (i === 0);

                if (contextPath) {
                  data.contextPath = contextPath + key;
                }
              }
              ret = ret + fn(context[key], {data: data});
              i++;
            }
          }
        }
      }

      if(i === 0){
        ret = inverse(this);
      }

      return ret;
    });

    instance.registerHelper('if', function(conditional, options) {
      if (isFunction(conditional)) { conditional = conditional.call(this); }

      // Default behavior is to render the positive path if the value is truthy and not empty.
      // The `includeZero` option may be set to treat the condtional as purely not empty based on the
      // behavior of isEmpty. Effectively this determines if 0 is handled by the positive path or negative.
      if ((!options.hash.includeZero && !conditional) || Utils.isEmpty(conditional)) {
        return options.inverse(this);
      } else {
        return options.fn(this);
      }
    });

    instance.registerHelper('unless', function(conditional, options) {
      return instance.helpers['if'].call(this, conditional, {fn: options.inverse, inverse: options.fn, hash: options.hash});
    });

    instance.registerHelper('with', function(context, options) {
      if (isFunction(context)) { context = context.call(this); }

      var fn = options.fn;

      if (!Utils.isEmpty(context)) {
        if (options.data && options.ids) {
          var data = createFrame(options.data);
          data.contextPath = Utils.appendContextPath(options.data.contextPath, options.ids[0]);
          options = {data:data};
        }

        return fn(context, options);
      } else {
        return options.inverse(this);
      }
    });

    instance.registerHelper('log', function(message, options) {
      var level = options.data && options.data.level != null ? parseInt(options.data.level, 10) : 1;
      instance.log(level, message);
    });

    instance.registerHelper('lookup', function(obj, field) {
      return obj && obj[field];
    });
  }

  var logger = {
    methodMap: { 0: 'debug', 1: 'info', 2: 'warn', 3: 'error' },

    // State enum
    DEBUG: 0,
    INFO: 1,
    WARN: 2,
    ERROR: 3,
    level: 3,

    // can be overridden in the host environment
    log: function(level, message) {
      if (logger.level <= level) {
        var method = logger.methodMap[level];
        if (typeof console !== 'undefined' && console[method]) {
          console[method].call(console, message);
        }
      }
    }
  };
  __exports__.logger = logger;
  var log = logger.log;
  __exports__.log = log;
  var createFrame = function(object) {
    var frame = Utils.extend({}, object);
    frame._parent = object;
    return frame;
  };
  __exports__.createFrame = createFrame;
  return __exports__;
})(__module3__, __module5__);

// handlebars/runtime.js
var __module6__ = (function(__dependency1__, __dependency2__, __dependency3__) {
  "use strict";
  var __exports__ = {};
  var Utils = __dependency1__;
  var Exception = __dependency2__;
  var COMPILER_REVISION = __dependency3__.COMPILER_REVISION;
  var REVISION_CHANGES = __dependency3__.REVISION_CHANGES;
  var createFrame = __dependency3__.createFrame;

  function checkRevision(compilerInfo) {
    var compilerRevision = compilerInfo && compilerInfo[0] || 1,
        currentRevision = COMPILER_REVISION;

    if (compilerRevision !== currentRevision) {
      if (compilerRevision < currentRevision) {
        var runtimeVersions = REVISION_CHANGES[currentRevision],
            compilerVersions = REVISION_CHANGES[compilerRevision];
        throw new Exception("Template was precompiled with an older version of Handlebars than the current runtime. "+
              "Please update your precompiler to a newer version ("+runtimeVersions+") or downgrade your runtime to an older version ("+compilerVersions+").");
      } else {
        // Use the embedded version info since the runtime doesn't know about this revision yet
        throw new Exception("Template was precompiled with a newer version of Handlebars than the current runtime. "+
              "Please update your runtime to a newer version ("+compilerInfo[1]+").");
      }
    }
  }

  __exports__.checkRevision = checkRevision;// TODO: Remove this line and break up compilePartial

  function template(templateSpec, env) {
    /* istanbul ignore next */
    if (!env) {
      throw new Exception("No environment passed to template");
    }
    if (!templateSpec || !templateSpec.main) {
      throw new Exception('Unknown template object: ' + typeof templateSpec);
    }

    // Note: Using env.VM references rather than local var references throughout this section to allow
    // for external users to override these as psuedo-supported APIs.
    env.VM.checkRevision(templateSpec.compiler);

    var invokePartialWrapper = function(partial, indent, name, context, hash, helpers, partials, data, depths) {
      if (hash) {
        context = Utils.extend({}, context, hash);
      }

      var result = env.VM.invokePartial.call(this, partial, name, context, helpers, partials, data, depths);

      if (result == null && env.compile) {
        var options = { helpers: helpers, partials: partials, data: data, depths: depths };
        partials[name] = env.compile(partial, { data: data !== undefined, compat: templateSpec.compat }, env);
        result = partials[name](context, options);
      }
      if (result != null) {
        if (indent) {
          var lines = result.split('\n');
          for (var i = 0, l = lines.length; i < l; i++) {
            if (!lines[i] && i + 1 === l) {
              break;
            }

            lines[i] = indent + lines[i];
          }
          result = lines.join('\n');
        }
        return result;
      } else {
        throw new Exception("The partial " + name + " could not be compiled when running in runtime-only mode");
      }
    };

    // Just add water
    var container = {
      lookup: function(depths, name) {
        var len = depths.length;
        for (var i = 0; i < len; i++) {
          if (depths[i] && depths[i][name] != null) {
            return depths[i][name];
          }
        }
      },
      lambda: function(current, context) {
        return typeof current === 'function' ? current.call(context) : current;
      },

      escapeExpression: Utils.escapeExpression,
      invokePartial: invokePartialWrapper,

      fn: function(i) {
        return templateSpec[i];
      },

      programs: [],
      program: function(i, data, depths) {
        var programWrapper = this.programs[i],
            fn = this.fn(i);
        if (data || depths) {
          programWrapper = program(this, i, fn, data, depths);
        } else if (!programWrapper) {
          programWrapper = this.programs[i] = program(this, i, fn);
        }
        return programWrapper;
      },

      data: function(data, depth) {
        while (data && depth--) {
          data = data._parent;
        }
        return data;
      },
      merge: function(param, common) {
        var ret = param || common;

        if (param && common && (param !== common)) {
          ret = Utils.extend({}, common, param);
        }

        return ret;
      },

      noop: env.VM.noop,
      compilerInfo: templateSpec.compiler
    };

    var ret = function(context, options) {
      options = options || {};
      var data = options.data;

      ret._setup(options);
      if (!options.partial && templateSpec.useData) {
        data = initData(context, data);
      }
      var depths;
      if (templateSpec.useDepths) {
        depths = options.depths ? [context].concat(options.depths) : [context];
      }

      return templateSpec.main.call(container, context, container.helpers, container.partials, data, depths);
    };
    ret.isTop = true;

    ret._setup = function(options) {
      if (!options.partial) {
        container.helpers = container.merge(options.helpers, env.helpers);

        if (templateSpec.usePartial) {
          container.partials = container.merge(options.partials, env.partials);
        }
      } else {
        container.helpers = options.helpers;
        container.partials = options.partials;
      }
    };

    ret._child = function(i, data, depths) {
      if (templateSpec.useDepths && !depths) {
        throw new Exception('must pass parent depths');
      }

      return program(container, i, templateSpec[i], data, depths);
    };
    return ret;
  }

  __exports__.template = template;function program(container, i, fn, data, depths) {
    var prog = function(context, options) {
      options = options || {};

      return fn.call(container, context, container.helpers, container.partials, options.data || data, depths && [context].concat(depths));
    };
    prog.program = i;
    prog.depth = depths ? depths.length : 0;
    return prog;
  }

  __exports__.program = program;function invokePartial(partial, name, context, helpers, partials, data, depths) {
    var options = { partial: true, helpers: helpers, partials: partials, data: data, depths: depths };

    if(partial === undefined) {
      throw new Exception("The partial " + name + " could not be found");
    } else if(partial instanceof Function) {
      return partial(context, options);
    }
  }

  __exports__.invokePartial = invokePartial;function noop() { return ""; }

  __exports__.noop = noop;function initData(context, data) {
    if (!data || !('root' in data)) {
      data = data ? createFrame(data) : {};
      data.root = context;
    }
    return data;
  }
  return __exports__;
})(__module3__, __module5__, __module2__);

// handlebars.runtime.js
var __module1__ = (function(__dependency1__, __dependency2__, __dependency3__, __dependency4__, __dependency5__) {
  "use strict";
  var __exports__;
  /*globals Handlebars: true */
  var base = __dependency1__;

  // Each of these augment the Handlebars object. No need to setup here.
  // (This is done to easily share code between commonjs and browse envs)
  var SafeString = __dependency2__;
  var Exception = __dependency3__;
  var Utils = __dependency4__;
  var runtime = __dependency5__;

  // For compatibility and usage outside of module systems, make the Handlebars object a namespace
  var create = function() {
    var hb = new base.HandlebarsEnvironment();

    Utils.extend(hb, base);
    hb.SafeString = SafeString;
    hb.Exception = Exception;
    hb.Utils = Utils;
    hb.escapeExpression = Utils.escapeExpression;

    hb.VM = runtime;
    hb.template = function(spec) {
      return runtime.template(spec, hb);
    };

    return hb;
  };

  var Handlebars = create();
  Handlebars.create = create;

  Handlebars['default'] = Handlebars;

  __exports__ = Handlebars;
  return __exports__;
})(__module2__, __module4__, __module5__, __module3__, __module6__);

// handlebars/compiler/ast.js
var __module7__ = (function(__dependency1__) {
  "use strict";
  var __exports__;
  var Exception = __dependency1__;

  function LocationInfo(locInfo) {
    locInfo = locInfo || {};
    this.firstLine   = locInfo.first_line;
    this.firstColumn = locInfo.first_column;
    this.lastColumn  = locInfo.last_column;
    this.lastLine    = locInfo.last_line;
  }

  var AST = {
    ProgramNode: function(statements, strip, locInfo) {
      LocationInfo.call(this, locInfo);
      this.type = "program";
      this.statements = statements;
      this.strip = strip;
    },

    MustacheNode: function(rawParams, hash, open, strip, locInfo) {
      LocationInfo.call(this, locInfo);
      this.type = "mustache";
      this.strip = strip;

      // Open may be a string parsed from the parser or a passed boolean flag
      if (open != null && open.charAt) {
        // Must use charAt to support IE pre-10
        var escapeFlag = open.charAt(3) || open.charAt(2);
        this.escaped = escapeFlag !== '{' && escapeFlag !== '&';
      } else {
        this.escaped = !!open;
      }

      if (rawParams instanceof AST.SexprNode) {
        this.sexpr = rawParams;
      } else {
        // Support old AST API
        this.sexpr = new AST.SexprNode(rawParams, hash);
      }

      // Support old AST API that stored this info in MustacheNode
      this.id = this.sexpr.id;
      this.params = this.sexpr.params;
      this.hash = this.sexpr.hash;
      this.eligibleHelper = this.sexpr.eligibleHelper;
      this.isHelper = this.sexpr.isHelper;
    },

    SexprNode: function(rawParams, hash, locInfo) {
      LocationInfo.call(this, locInfo);

      this.type = "sexpr";
      this.hash = hash;

      var id = this.id = rawParams[0];
      var params = this.params = rawParams.slice(1);

      // a mustache is definitely a helper if:
      // * it is an eligible helper, and
      // * it has at least one parameter or hash segment
      this.isHelper = !!(params.length || hash);

      // a mustache is an eligible helper if:
      // * its id is simple (a single part, not `this` or `..`)
      this.eligibleHelper = this.isHelper || id.isSimple;

      // if a mustache is an eligible helper but not a definite
      // helper, it is ambiguous, and will be resolved in a later
      // pass or at runtime.
    },

    PartialNode: function(partialName, context, hash, strip, locInfo) {
      LocationInfo.call(this, locInfo);
      this.type         = "partial";
      this.partialName  = partialName;
      this.context      = context;
      this.hash = hash;
      this.strip = strip;

      this.strip.inlineStandalone = true;
    },

    BlockNode: function(mustache, program, inverse, strip, locInfo) {
      LocationInfo.call(this, locInfo);

      this.type = 'block';
      this.mustache = mustache;
      this.program  = program;
      this.inverse  = inverse;
      this.strip = strip;

      if (inverse && !program) {
        this.isInverse = true;
      }
    },

    RawBlockNode: function(mustache, content, close, locInfo) {
      LocationInfo.call(this, locInfo);

      if (mustache.sexpr.id.original !== close) {
        throw new Exception(mustache.sexpr.id.original + " doesn't match " + close, this);
      }

      content = new AST.ContentNode(content, locInfo);

      this.type = 'block';
      this.mustache = mustache;
      this.program = new AST.ProgramNode([content], {}, locInfo);
    },

    ContentNode: function(string, locInfo) {
      LocationInfo.call(this, locInfo);
      this.type = "content";
      this.original = this.string = string;
    },

    HashNode: function(pairs, locInfo) {
      LocationInfo.call(this, locInfo);
      this.type = "hash";
      this.pairs = pairs;
    },

    IdNode: function(parts, locInfo) {
      LocationInfo.call(this, locInfo);
      this.type = "ID";

      var original = "",
          dig = [],
          depth = 0,
          depthString = '';

      for(var i=0,l=parts.length; i<l; i++) {
        var part = parts[i].part;
        original += (parts[i].separator || '') + part;

        if (part === ".." || part === "." || part === "this") {
          if (dig.length > 0) {
            throw new Exception("Invalid path: " + original, this);
          } else if (part === "..") {
            depth++;
            depthString += '../';
          } else {
            this.isScoped = true;
          }
        } else {
          dig.push(part);
        }
      }

      this.original = original;
      this.parts    = dig;
      this.string   = dig.join('.');
      this.depth    = depth;
      this.idName   = depthString + this.string;

      // an ID is simple if it only has one part, and that part is not
      // `..` or `this`.
      this.isSimple = parts.length === 1 && !this.isScoped && depth === 0;

      this.stringModeValue = this.string;
    },

    PartialNameNode: function(name, locInfo) {
      LocationInfo.call(this, locInfo);
      this.type = "PARTIAL_NAME";
      this.name = name.original;
    },

    DataNode: function(id, locInfo) {
      LocationInfo.call(this, locInfo);
      this.type = "DATA";
      this.id = id;
      this.stringModeValue = id.stringModeValue;
      this.idName = '@' + id.stringModeValue;
    },

    StringNode: function(string, locInfo) {
      LocationInfo.call(this, locInfo);
      this.type = "STRING";
      this.original =
        this.string =
        this.stringModeValue = string;
    },

    NumberNode: function(number, locInfo) {
      LocationInfo.call(this, locInfo);
      this.type = "NUMBER";
      this.original =
        this.number = number;
      this.stringModeValue = Number(number);
    },

    BooleanNode: function(bool, locInfo) {
      LocationInfo.call(this, locInfo);
      this.type = "BOOLEAN";
      this.bool = bool;
      this.stringModeValue = bool === "true";
    },

    CommentNode: function(comment, locInfo) {
      LocationInfo.call(this, locInfo);
      this.type = "comment";
      this.comment = comment;

      this.strip = {
        inlineStandalone: true
      };
    }
  };


  // Must be exported as an object rather than the root of the module as the jison lexer
  // most modify the object to operate properly.
  __exports__ = AST;
  return __exports__;
})(__module5__);

// handlebars/compiler/parser.js
var __module9__ = (function() {
  "use strict";
  var __exports__;
  /* jshint ignore:start */
  /* istanbul ignore next */
  /* Jison generated parser */
  var handlebars = (function(){
  var parser = {trace: function trace() { },
  yy: {},
  symbols_: {"error":2,"root":3,"program":4,"EOF":5,"program_repetition0":6,"statement":7,"mustache":8,"block":9,"rawBlock":10,"partial":11,"CONTENT":12,"COMMENT":13,"openRawBlock":14,"END_RAW_BLOCK":15,"OPEN_RAW_BLOCK":16,"sexpr":17,"CLOSE_RAW_BLOCK":18,"openBlock":19,"block_option0":20,"closeBlock":21,"openInverse":22,"block_option1":23,"OPEN_BLOCK":24,"CLOSE":25,"OPEN_INVERSE":26,"inverseAndProgram":27,"INVERSE":28,"OPEN_ENDBLOCK":29,"path":30,"OPEN":31,"OPEN_UNESCAPED":32,"CLOSE_UNESCAPED":33,"OPEN_PARTIAL":34,"partialName":35,"param":36,"partial_option0":37,"partial_option1":38,"sexpr_repetition0":39,"sexpr_option0":40,"dataName":41,"STRING":42,"NUMBER":43,"BOOLEAN":44,"OPEN_SEXPR":45,"CLOSE_SEXPR":46,"hash":47,"hash_repetition_plus0":48,"hashSegment":49,"ID":50,"EQUALS":51,"DATA":52,"pathSegments":53,"SEP":54,"$accept":0,"$end":1},
  terminals_: {2:"error",5:"EOF",12:"CONTENT",13:"COMMENT",15:"END_RAW_BLOCK",16:"OPEN_RAW_BLOCK",18:"CLOSE_RAW_BLOCK",24:"OPEN_BLOCK",25:"CLOSE",26:"OPEN_INVERSE",28:"INVERSE",29:"OPEN_ENDBLOCK",31:"OPEN",32:"OPEN_UNESCAPED",33:"CLOSE_UNESCAPED",34:"OPEN_PARTIAL",42:"STRING",43:"NUMBER",44:"BOOLEAN",45:"OPEN_SEXPR",46:"CLOSE_SEXPR",50:"ID",51:"EQUALS",52:"DATA",54:"SEP"},
  productions_: [0,[3,2],[4,1],[7,1],[7,1],[7,1],[7,1],[7,1],[7,1],[10,3],[14,3],[9,4],[9,4],[19,3],[22,3],[27,2],[21,3],[8,3],[8,3],[11,5],[11,4],[17,3],[17,1],[36,1],[36,1],[36,1],[36,1],[36,1],[36,3],[47,1],[49,3],[35,1],[35,1],[35,1],[41,2],[30,1],[53,3],[53,1],[6,0],[6,2],[20,0],[20,1],[23,0],[23,1],[37,0],[37,1],[38,0],[38,1],[39,0],[39,2],[40,0],[40,1],[48,1],[48,2]],
  performAction: function anonymous(yytext,yyleng,yylineno,yy,yystate,$$,_$) {

  var $0 = $$.length - 1;
  switch (yystate) {
  case 1: yy.prepareProgram($$[$0-1].statements, true); return $$[$0-1];
  break;
  case 2:this.$ = new yy.ProgramNode(yy.prepareProgram($$[$0]), {}, this._$);
  break;
  case 3:this.$ = $$[$0];
  break;
  case 4:this.$ = $$[$0];
  break;
  case 5:this.$ = $$[$0];
  break;
  case 6:this.$ = $$[$0];
  break;
  case 7:this.$ = new yy.ContentNode($$[$0], this._$);
  break;
  case 8:this.$ = new yy.CommentNode($$[$0], this._$);
  break;
  case 9:this.$ = new yy.RawBlockNode($$[$0-2], $$[$0-1], $$[$0], this._$);
  break;
  case 10:this.$ = new yy.MustacheNode($$[$0-1], null, '', '', this._$);
  break;
  case 11:this.$ = yy.prepareBlock($$[$0-3], $$[$0-2], $$[$0-1], $$[$0], false, this._$);
  break;
  case 12:this.$ = yy.prepareBlock($$[$0-3], $$[$0-2], $$[$0-1], $$[$0], true, this._$);
  break;
  case 13:this.$ = new yy.MustacheNode($$[$0-1], null, $$[$0-2], yy.stripFlags($$[$0-2], $$[$0]), this._$);
  break;
  case 14:this.$ = new yy.MustacheNode($$[$0-1], null, $$[$0-2], yy.stripFlags($$[$0-2], $$[$0]), this._$);
  break;
  case 15:this.$ = { strip: yy.stripFlags($$[$0-1], $$[$0-1]), program: $$[$0] };
  break;
  case 16:this.$ = {path: $$[$0-1], strip: yy.stripFlags($$[$0-2], $$[$0])};
  break;
  case 17:this.$ = new yy.MustacheNode($$[$0-1], null, $$[$0-2], yy.stripFlags($$[$0-2], $$[$0]), this._$);
  break;
  case 18:this.$ = new yy.MustacheNode($$[$0-1], null, $$[$0-2], yy.stripFlags($$[$0-2], $$[$0]), this._$);
  break;
  case 19:this.$ = new yy.PartialNode($$[$0-3], $$[$0-2], $$[$0-1], yy.stripFlags($$[$0-4], $$[$0]), this._$);
  break;
  case 20:this.$ = new yy.PartialNode($$[$0-2], undefined, $$[$0-1], yy.stripFlags($$[$0-3], $$[$0]), this._$);
  break;
  case 21:this.$ = new yy.SexprNode([$$[$0-2]].concat($$[$0-1]), $$[$0], this._$);
  break;
  case 22:this.$ = new yy.SexprNode([$$[$0]], null, this._$);
  break;
  case 23:this.$ = $$[$0];
  break;
  case 24:this.$ = new yy.StringNode($$[$0], this._$);
  break;
  case 25:this.$ = new yy.NumberNode($$[$0], this._$);
  break;
  case 26:this.$ = new yy.BooleanNode($$[$0], this._$);
  break;
  case 27:this.$ = $$[$0];
  break;
  case 28:$$[$0-1].isHelper = true; this.$ = $$[$0-1];
  break;
  case 29:this.$ = new yy.HashNode($$[$0], this._$);
  break;
  case 30:this.$ = [$$[$0-2], $$[$0]];
  break;
  case 31:this.$ = new yy.PartialNameNode($$[$0], this._$);
  break;
  case 32:this.$ = new yy.PartialNameNode(new yy.StringNode($$[$0], this._$), this._$);
  break;
  case 33:this.$ = new yy.PartialNameNode(new yy.NumberNode($$[$0], this._$));
  break;
  case 34:this.$ = new yy.DataNode($$[$0], this._$);
  break;
  case 35:this.$ = new yy.IdNode($$[$0], this._$);
  break;
  case 36: $$[$0-2].push({part: $$[$0], separator: $$[$0-1]}); this.$ = $$[$0-2];
  break;
  case 37:this.$ = [{part: $$[$0]}];
  break;
  case 38:this.$ = [];
  break;
  case 39:$$[$0-1].push($$[$0]);
  break;
  case 48:this.$ = [];
  break;
  case 49:$$[$0-1].push($$[$0]);
  break;
  case 52:this.$ = [$$[$0]];
  break;
  case 53:$$[$0-1].push($$[$0]);
  break;
  }
  },
  table: [{3:1,4:2,5:[2,38],6:3,12:[2,38],13:[2,38],16:[2,38],24:[2,38],26:[2,38],31:[2,38],32:[2,38],34:[2,38]},{1:[3]},{5:[1,4]},{5:[2,2],7:5,8:6,9:7,10:8,11:9,12:[1,10],13:[1,11],14:16,16:[1,20],19:14,22:15,24:[1,18],26:[1,19],28:[2,2],29:[2,2],31:[1,12],32:[1,13],34:[1,17]},{1:[2,1]},{5:[2,39],12:[2,39],13:[2,39],16:[2,39],24:[2,39],26:[2,39],28:[2,39],29:[2,39],31:[2,39],32:[2,39],34:[2,39]},{5:[2,3],12:[2,3],13:[2,3],16:[2,3],24:[2,3],26:[2,3],28:[2,3],29:[2,3],31:[2,3],32:[2,3],34:[2,3]},{5:[2,4],12:[2,4],13:[2,4],16:[2,4],24:[2,4],26:[2,4],28:[2,4],29:[2,4],31:[2,4],32:[2,4],34:[2,4]},{5:[2,5],12:[2,5],13:[2,5],16:[2,5],24:[2,5],26:[2,5],28:[2,5],29:[2,5],31:[2,5],32:[2,5],34:[2,5]},{5:[2,6],12:[2,6],13:[2,6],16:[2,6],24:[2,6],26:[2,6],28:[2,6],29:[2,6],31:[2,6],32:[2,6],34:[2,6]},{5:[2,7],12:[2,7],13:[2,7],16:[2,7],24:[2,7],26:[2,7],28:[2,7],29:[2,7],31:[2,7],32:[2,7],34:[2,7]},{5:[2,8],12:[2,8],13:[2,8],16:[2,8],24:[2,8],26:[2,8],28:[2,8],29:[2,8],31:[2,8],32:[2,8],34:[2,8]},{17:21,30:22,41:23,50:[1,26],52:[1,25],53:24},{17:27,30:22,41:23,50:[1,26],52:[1,25],53:24},{4:28,6:3,12:[2,38],13:[2,38],16:[2,38],24:[2,38],26:[2,38],28:[2,38],29:[2,38],31:[2,38],32:[2,38],34:[2,38]},{4:29,6:3,12:[2,38],13:[2,38],16:[2,38],24:[2,38],26:[2,38],28:[2,38],29:[2,38],31:[2,38],32:[2,38],34:[2,38]},{12:[1,30]},{30:32,35:31,42:[1,33],43:[1,34],50:[1,26],53:24},{17:35,30:22,41:23,50:[1,26],52:[1,25],53:24},{17:36,30:22,41:23,50:[1,26],52:[1,25],53:24},{17:37,30:22,41:23,50:[1,26],52:[1,25],53:24},{25:[1,38]},{18:[2,48],25:[2,48],33:[2,48],39:39,42:[2,48],43:[2,48],44:[2,48],45:[2,48],46:[2,48],50:[2,48],52:[2,48]},{18:[2,22],25:[2,22],33:[2,22],46:[2,22]},{18:[2,35],25:[2,35],33:[2,35],42:[2,35],43:[2,35],44:[2,35],45:[2,35],46:[2,35],50:[2,35],52:[2,35],54:[1,40]},{30:41,50:[1,26],53:24},{18:[2,37],25:[2,37],33:[2,37],42:[2,37],43:[2,37],44:[2,37],45:[2,37],46:[2,37],50:[2,37],52:[2,37],54:[2,37]},{33:[1,42]},{20:43,27:44,28:[1,45],29:[2,40]},{23:46,27:47,28:[1,45],29:[2,42]},{15:[1,48]},{25:[2,46],30:51,36:49,38:50,41:55,42:[1,52],43:[1,53],44:[1,54],45:[1,56],47:57,48:58,49:60,50:[1,59],52:[1,25],53:24},{25:[2,31],42:[2,31],43:[2,31],44:[2,31],45:[2,31],50:[2,31],52:[2,31]},{25:[2,32],42:[2,32],43:[2,32],44:[2,32],45:[2,32],50:[2,32],52:[2,32]},{25:[2,33],42:[2,33],43:[2,33],44:[2,33],45:[2,33],50:[2,33],52:[2,33]},{25:[1,61]},{25:[1,62]},{18:[1,63]},{5:[2,17],12:[2,17],13:[2,17],16:[2,17],24:[2,17],26:[2,17],28:[2,17],29:[2,17],31:[2,17],32:[2,17],34:[2,17]},{18:[2,50],25:[2,50],30:51,33:[2,50],36:65,40:64,41:55,42:[1,52],43:[1,53],44:[1,54],45:[1,56],46:[2,50],47:66,48:58,49:60,50:[1,59],52:[1,25],53:24},{50:[1,67]},{18:[2,34],25:[2,34],33:[2,34],42:[2,34],43:[2,34],44:[2,34],45:[2,34],46:[2,34],50:[2,34],52:[2,34]},{5:[2,18],12:[2,18],13:[2,18],16:[2,18],24:[2,18],26:[2,18],28:[2,18],29:[2,18],31:[2,18],32:[2,18],34:[2,18]},{21:68,29:[1,69]},{29:[2,41]},{4:70,6:3,12:[2,38],13:[2,38],16:[2,38],24:[2,38],26:[2,38],29:[2,38],31:[2,38],32:[2,38],34:[2,38]},{21:71,29:[1,69]},{29:[2,43]},{5:[2,9],12:[2,9],13:[2,9],16:[2,9],24:[2,9],26:[2,9],28:[2,9],29:[2,9],31:[2,9],32:[2,9],34:[2,9]},{25:[2,44],37:72,47:73,48:58,49:60,50:[1,74]},{25:[1,75]},{18:[2,23],25:[2,23],33:[2,23],42:[2,23],43:[2,23],44:[2,23],45:[2,23],46:[2,23],50:[2,23],52:[2,23]},{18:[2,24],25:[2,24],33:[2,24],42:[2,24],43:[2,24],44:[2,24],45:[2,24],46:[2,24],50:[2,24],52:[2,24]},{18:[2,25],25:[2,25],33:[2,25],42:[2,25],43:[2,25],44:[2,25],45:[2,25],46:[2,25],50:[2,25],52:[2,25]},{18:[2,26],25:[2,26],33:[2,26],42:[2,26],43:[2,26],44:[2,26],45:[2,26],46:[2,26],50:[2,26],52:[2,26]},{18:[2,27],25:[2,27],33:[2,27],42:[2,27],43:[2,27],44:[2,27],45:[2,27],46:[2,27],50:[2,27],52:[2,27]},{17:76,30:22,41:23,50:[1,26],52:[1,25],53:24},{25:[2,47]},{18:[2,29],25:[2,29],33:[2,29],46:[2,29],49:77,50:[1,74]},{18:[2,37],25:[2,37],33:[2,37],42:[2,37],43:[2,37],44:[2,37],45:[2,37],46:[2,37],50:[2,37],51:[1,78],52:[2,37],54:[2,37]},{18:[2,52],25:[2,52],33:[2,52],46:[2,52],50:[2,52]},{12:[2,13],13:[2,13],16:[2,13],24:[2,13],26:[2,13],28:[2,13],29:[2,13],31:[2,13],32:[2,13],34:[2,13]},{12:[2,14],13:[2,14],16:[2,14],24:[2,14],26:[2,14],28:[2,14],29:[2,14],31:[2,14],32:[2,14],34:[2,14]},{12:[2,10]},{18:[2,21],25:[2,21],33:[2,21],46:[2,21]},{18:[2,49],25:[2,49],33:[2,49],42:[2,49],43:[2,49],44:[2,49],45:[2,49],46:[2,49],50:[2,49],52:[2,49]},{18:[2,51],25:[2,51],33:[2,51],46:[2,51]},{18:[2,36],25:[2,36],33:[2,36],42:[2,36],43:[2,36],44:[2,36],45:[2,36],46:[2,36],50:[2,36],52:[2,36],54:[2,36]},{5:[2,11],12:[2,11],13:[2,11],16:[2,11],24:[2,11],26:[2,11],28:[2,11],29:[2,11],31:[2,11],32:[2,11],34:[2,11]},{30:79,50:[1,26],53:24},{29:[2,15]},{5:[2,12],12:[2,12],13:[2,12],16:[2,12],24:[2,12],26:[2,12],28:[2,12],29:[2,12],31:[2,12],32:[2,12],34:[2,12]},{25:[1,80]},{25:[2,45]},{51:[1,78]},{5:[2,20],12:[2,20],13:[2,20],16:[2,20],24:[2,20],26:[2,20],28:[2,20],29:[2,20],31:[2,20],32:[2,20],34:[2,20]},{46:[1,81]},{18:[2,53],25:[2,53],33:[2,53],46:[2,53],50:[2,53]},{30:51,36:82,41:55,42:[1,52],43:[1,53],44:[1,54],45:[1,56],50:[1,26],52:[1,25],53:24},{25:[1,83]},{5:[2,19],12:[2,19],13:[2,19],16:[2,19],24:[2,19],26:[2,19],28:[2,19],29:[2,19],31:[2,19],32:[2,19],34:[2,19]},{18:[2,28],25:[2,28],33:[2,28],42:[2,28],43:[2,28],44:[2,28],45:[2,28],46:[2,28],50:[2,28],52:[2,28]},{18:[2,30],25:[2,30],33:[2,30],46:[2,30],50:[2,30]},{5:[2,16],12:[2,16],13:[2,16],16:[2,16],24:[2,16],26:[2,16],28:[2,16],29:[2,16],31:[2,16],32:[2,16],34:[2,16]}],
  defaultActions: {4:[2,1],44:[2,41],47:[2,43],57:[2,47],63:[2,10],70:[2,15],73:[2,45]},
  parseError: function parseError(str, hash) {
      throw new Error(str);
  },
  parse: function parse(input) {
      var self = this, stack = [0], vstack = [null], lstack = [], table = this.table, yytext = "", yylineno = 0, yyleng = 0, recovering = 0, TERROR = 2, EOF = 1;
      this.lexer.setInput(input);
      this.lexer.yy = this.yy;
      this.yy.lexer = this.lexer;
      this.yy.parser = this;
      if (typeof this.lexer.yylloc == "undefined")
          this.lexer.yylloc = {};
      var yyloc = this.lexer.yylloc;
      lstack.push(yyloc);
      var ranges = this.lexer.options && this.lexer.options.ranges;
      if (typeof this.yy.parseError === "function")
          this.parseError = this.yy.parseError;
      function popStack(n) {
          stack.length = stack.length - 2 * n;
          vstack.length = vstack.length - n;
          lstack.length = lstack.length - n;
      }
      function lex() {
          var token;
          token = self.lexer.lex() || 1;
          if (typeof token !== "number") {
              token = self.symbols_[token] || token;
          }
          return token;
      }
      var symbol, preErrorSymbol, state, action, a, r, yyval = {}, p, len, newState, expected;
      while (true) {
          state = stack[stack.length - 1];
          if (this.defaultActions[state]) {
              action = this.defaultActions[state];
          } else {
              if (symbol === null || typeof symbol == "undefined") {
                  symbol = lex();
              }
              action = table[state] && table[state][symbol];
          }
          if (typeof action === "undefined" || !action.length || !action[0]) {
              var errStr = "";
              if (!recovering) {
                  expected = [];
                  for (p in table[state])
                      if (this.terminals_[p] && p > 2) {
                          expected.push("'" + this.terminals_[p] + "'");
                      }
                  if (this.lexer.showPosition) {
                      errStr = "Parse error on line " + (yylineno + 1) + ":\n" + this.lexer.showPosition() + "\nExpecting " + expected.join(", ") + ", got '" + (this.terminals_[symbol] || symbol) + "'";
                  } else {
                      errStr = "Parse error on line " + (yylineno + 1) + ": Unexpected " + (symbol == 1?"end of input":"'" + (this.terminals_[symbol] || symbol) + "'");
                  }
                  this.parseError(errStr, {text: this.lexer.match, token: this.terminals_[symbol] || symbol, line: this.lexer.yylineno, loc: yyloc, expected: expected});
              }
          }
          if (action[0] instanceof Array && action.length > 1) {
              throw new Error("Parse Error: multiple actions possible at state: " + state + ", token: " + symbol);
          }
          switch (action[0]) {
          case 1:
              stack.push(symbol);
              vstack.push(this.lexer.yytext);
              lstack.push(this.lexer.yylloc);
              stack.push(action[1]);
              symbol = null;
              if (!preErrorSymbol) {
                  yyleng = this.lexer.yyleng;
                  yytext = this.lexer.yytext;
                  yylineno = this.lexer.yylineno;
                  yyloc = this.lexer.yylloc;
                  if (recovering > 0)
                      recovering--;
              } else {
                  symbol = preErrorSymbol;
                  preErrorSymbol = null;
              }
              break;
          case 2:
              len = this.productions_[action[1]][1];
              yyval.$ = vstack[vstack.length - len];
              yyval._$ = {first_line: lstack[lstack.length - (len || 1)].first_line, last_line: lstack[lstack.length - 1].last_line, first_column: lstack[lstack.length - (len || 1)].first_column, last_column: lstack[lstack.length - 1].last_column};
              if (ranges) {
                  yyval._$.range = [lstack[lstack.length - (len || 1)].range[0], lstack[lstack.length - 1].range[1]];
              }
              r = this.performAction.call(yyval, yytext, yyleng, yylineno, this.yy, action[1], vstack, lstack);
              if (typeof r !== "undefined") {
                  return r;
              }
              if (len) {
                  stack = stack.slice(0, -1 * len * 2);
                  vstack = vstack.slice(0, -1 * len);
                  lstack = lstack.slice(0, -1 * len);
              }
              stack.push(this.productions_[action[1]][0]);
              vstack.push(yyval.$);
              lstack.push(yyval._$);
              newState = table[stack[stack.length - 2]][stack[stack.length - 1]];
              stack.push(newState);
              break;
          case 3:
              return true;
          }
      }
      return true;
  }
  };
  /* Jison generated lexer */
  var lexer = (function(){
  var lexer = ({EOF:1,
  parseError:function parseError(str, hash) {
          if (this.yy.parser) {
              this.yy.parser.parseError(str, hash);
          } else {
              throw new Error(str);
          }
      },
  setInput:function (input) {
          this._input = input;
          this._more = this._less = this.done = false;
          this.yylineno = this.yyleng = 0;
          this.yytext = this.matched = this.match = '';
          this.conditionStack = ['INITIAL'];
          this.yylloc = {first_line:1,first_column:0,last_line:1,last_column:0};
          if (this.options.ranges) this.yylloc.range = [0,0];
          this.offset = 0;
          return this;
      },
  input:function () {
          var ch = this._input[0];
          this.yytext += ch;
          this.yyleng++;
          this.offset++;
          this.match += ch;
          this.matched += ch;
          var lines = ch.match(/(?:\r\n?|\n).*/g);
          if (lines) {
              this.yylineno++;
              this.yylloc.last_line++;
          } else {
              this.yylloc.last_column++;
          }
          if (this.options.ranges) this.yylloc.range[1]++;

          this._input = this._input.slice(1);
          return ch;
      },
  unput:function (ch) {
          var len = ch.length;
          var lines = ch.split(/(?:\r\n?|\n)/g);

          this._input = ch + this._input;
          this.yytext = this.yytext.substr(0, this.yytext.length-len-1);
          //this.yyleng -= len;
          this.offset -= len;
          var oldLines = this.match.split(/(?:\r\n?|\n)/g);
          this.match = this.match.substr(0, this.match.length-1);
          this.matched = this.matched.substr(0, this.matched.length-1);

          if (lines.length-1) this.yylineno -= lines.length-1;
          var r = this.yylloc.range;

          this.yylloc = {first_line: this.yylloc.first_line,
            last_line: this.yylineno+1,
            first_column: this.yylloc.first_column,
            last_column: lines ?
                (lines.length === oldLines.length ? this.yylloc.first_column : 0) + oldLines[oldLines.length - lines.length].length - lines[0].length:
                this.yylloc.first_column - len
            };

          if (this.options.ranges) {
              this.yylloc.range = [r[0], r[0] + this.yyleng - len];
          }
          return this;
      },
  more:function () {
          this._more = true;
          return this;
      },
  less:function (n) {
          this.unput(this.match.slice(n));
      },
  pastInput:function () {
          var past = this.matched.substr(0, this.matched.length - this.match.length);
          return (past.length > 20 ? '...':'') + past.substr(-20).replace(/\n/g, "");
      },
  upcomingInput:function () {
          var next = this.match;
          if (next.length < 20) {
              next += this._input.substr(0, 20-next.length);
          }
          return (next.substr(0,20)+(next.length > 20 ? '...':'')).replace(/\n/g, "");
      },
  showPosition:function () {
          var pre = this.pastInput();
          var c = new Array(pre.length + 1).join("-");
          return pre + this.upcomingInput() + "\n" + c+"^";
      },
  next:function () {
          if (this.done) {
              return this.EOF;
          }
          if (!this._input) this.done = true;

          var token,
              match,
              tempMatch,
              index,
              col,
              lines;
          if (!this._more) {
              this.yytext = '';
              this.match = '';
          }
          var rules = this._currentRules();
          for (var i=0;i < rules.length; i++) {
              tempMatch = this._input.match(this.rules[rules[i]]);
              if (tempMatch && (!match || tempMatch[0].length > match[0].length)) {
                  match = tempMatch;
                  index = i;
                  if (!this.options.flex) break;
              }
          }
          if (match) {
              lines = match[0].match(/(?:\r\n?|\n).*/g);
              if (lines) this.yylineno += lines.length;
              this.yylloc = {first_line: this.yylloc.last_line,
                             last_line: this.yylineno+1,
                             first_column: this.yylloc.last_column,
                             last_column: lines ? lines[lines.length-1].length-lines[lines.length-1].match(/\r?\n?/)[0].length : this.yylloc.last_column + match[0].length};
              this.yytext += match[0];
              this.match += match[0];
              this.matches = match;
              this.yyleng = this.yytext.length;
              if (this.options.ranges) {
                  this.yylloc.range = [this.offset, this.offset += this.yyleng];
              }
              this._more = false;
              this._input = this._input.slice(match[0].length);
              this.matched += match[0];
              token = this.performAction.call(this, this.yy, this, rules[index],this.conditionStack[this.conditionStack.length-1]);
              if (this.done && this._input) this.done = false;
              if (token) return token;
              else return;
          }
          if (this._input === "") {
              return this.EOF;
          } else {
              return this.parseError('Lexical error on line '+(this.yylineno+1)+'. Unrecognized text.\n'+this.showPosition(),
                      {text: "", token: null, line: this.yylineno});
          }
      },
  lex:function lex() {
          var r = this.next();
          if (typeof r !== 'undefined') {
              return r;
          } else {
              return this.lex();
          }
      },
  begin:function begin(condition) {
          this.conditionStack.push(condition);
      },
  popState:function popState() {
          return this.conditionStack.pop();
      },
  _currentRules:function _currentRules() {
          return this.conditions[this.conditionStack[this.conditionStack.length-1]].rules;
      },
  topState:function () {
          return this.conditionStack[this.conditionStack.length-2];
      },
  pushState:function begin(condition) {
          this.begin(condition);
      }});
  lexer.options = {};
  lexer.performAction = function anonymous(yy,yy_,$avoiding_name_collisions,YY_START) {


  function strip(start, end) {
    return yy_.yytext = yy_.yytext.substr(start, yy_.yyleng-end);
  }


  var YYSTATE=YY_START
  switch($avoiding_name_collisions) {
  case 0:
                                     if(yy_.yytext.slice(-2) === "\\\\") {
                                       strip(0,1);
                                       this.begin("mu");
                                     } else if(yy_.yytext.slice(-1) === "\\") {
                                       strip(0,1);
                                       this.begin("emu");
                                     } else {
                                       this.begin("mu");
                                     }
                                     if(yy_.yytext) return 12;

  break;
  case 1:return 12;
  break;
  case 2:
                                     this.popState();
                                     return 12;

  break;
  case 3:
                                    yy_.yytext = yy_.yytext.substr(5, yy_.yyleng-9);
                                    this.popState();
                                    return 15;

  break;
  case 4: return 12;
  break;
  case 5:strip(0,4); this.popState(); return 13;
  break;
  case 6:return 45;
  break;
  case 7:return 46;
  break;
  case 8: return 16;
  break;
  case 9:
                                    this.popState();
                                    this.begin('raw');
                                    return 18;

  break;
  case 10:return 34;
  break;
  case 11:return 24;
  break;
  case 12:return 29;
  break;
  case 13:this.popState(); return 28;
  break;
  case 14:this.popState(); return 28;
  break;
  case 15:return 26;
  break;
  case 16:return 26;
  break;
  case 17:return 32;
  break;
  case 18:return 31;
  break;
  case 19:this.popState(); this.begin('com');
  break;
  case 20:strip(3,5); this.popState(); return 13;
  break;
  case 21:return 31;
  break;
  case 22:return 51;
  break;
  case 23:return 50;
  break;
  case 24:return 50;
  break;
  case 25:return 54;
  break;
  case 26:// ignore whitespace
  break;
  case 27:this.popState(); return 33;
  break;
  case 28:this.popState(); return 25;
  break;
  case 29:yy_.yytext = strip(1,2).replace(/\\"/g,'"'); return 42;
  break;
  case 30:yy_.yytext = strip(1,2).replace(/\\'/g,"'"); return 42;
  break;
  case 31:return 52;
  break;
  case 32:return 44;
  break;
  case 33:return 44;
  break;
  case 34:return 43;
  break;
  case 35:return 50;
  break;
  case 36:yy_.yytext = strip(1,2); return 50;
  break;
  case 37:return 'INVALID';
  break;
  case 38:return 5;
  break;
  }
  };
  lexer.rules = [/^(?:[^\x00]*?(?=(\{\{)))/,/^(?:[^\x00]+)/,/^(?:[^\x00]{2,}?(?=(\{\{|\\\{\{|\\\\\{\{|$)))/,/^(?:\{\{\{\{\/[^\s!"#%-,\.\/;->@\[-\^`\{-~]+(?=[=}\s\/.])\}\}\}\})/,/^(?:[^\x00]*?(?=(\{\{\{\{\/)))/,/^(?:[\s\S]*?--\}\})/,/^(?:\()/,/^(?:\))/,/^(?:\{\{\{\{)/,/^(?:\}\}\}\})/,/^(?:\{\{(~)?>)/,/^(?:\{\{(~)?#)/,/^(?:\{\{(~)?\/)/,/^(?:\{\{(~)?\^\s*(~)?\}\})/,/^(?:\{\{(~)?\s*else\s*(~)?\}\})/,/^(?:\{\{(~)?\^)/,/^(?:\{\{(~)?\s*else\b)/,/^(?:\{\{(~)?\{)/,/^(?:\{\{(~)?&)/,/^(?:\{\{!--)/,/^(?:\{\{![\s\S]*?\}\})/,/^(?:\{\{(~)?)/,/^(?:=)/,/^(?:\.\.)/,/^(?:\.(?=([=~}\s\/.)])))/,/^(?:[\/.])/,/^(?:\s+)/,/^(?:\}(~)?\}\})/,/^(?:(~)?\}\})/,/^(?:"(\\["]|[^"])*")/,/^(?:'(\\[']|[^'])*')/,/^(?:@)/,/^(?:true(?=([~}\s)])))/,/^(?:false(?=([~}\s)])))/,/^(?:-?[0-9]+(?:\.[0-9]+)?(?=([~}\s)])))/,/^(?:([^\s!"#%-,\.\/;->@\[-\^`\{-~]+(?=([=~}\s\/.)]))))/,/^(?:\[[^\]]*\])/,/^(?:.)/,/^(?:$)/];
  lexer.conditions = {"mu":{"rules":[6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38],"inclusive":false},"emu":{"rules":[2],"inclusive":false},"com":{"rules":[5],"inclusive":false},"raw":{"rules":[3,4],"inclusive":false},"INITIAL":{"rules":[0,1,38],"inclusive":true}};
  return lexer;})()
  parser.lexer = lexer;
  function Parser () { this.yy = {}; }Parser.prototype = parser;parser.Parser = Parser;
  return new Parser;
  })();__exports__ = handlebars;
  /* jshint ignore:end */
  return __exports__;
})();

// handlebars/compiler/helpers.js
var __module10__ = (function(__dependency1__) {
  "use strict";
  var __exports__ = {};
  var Exception = __dependency1__;

  function stripFlags(open, close) {
    return {
      left: open.charAt(2) === '~',
      right: close.charAt(close.length-3) === '~'
    };
  }

  __exports__.stripFlags = stripFlags;
  function prepareBlock(mustache, program, inverseAndProgram, close, inverted, locInfo) {
    /*jshint -W040 */
    if (mustache.sexpr.id.original !== close.path.original) {
      throw new Exception(mustache.sexpr.id.original + ' doesn\'t match ' + close.path.original, mustache);
    }

    var inverse = inverseAndProgram && inverseAndProgram.program;

    var strip = {
      left: mustache.strip.left,
      right: close.strip.right,

      // Determine the standalone candiacy. Basically flag our content as being possibly standalone
      // so our parent can determine if we actually are standalone
      openStandalone: isNextWhitespace(program.statements),
      closeStandalone: isPrevWhitespace((inverse || program).statements)
    };

    if (mustache.strip.right) {
      omitRight(program.statements, null, true);
    }

    if (inverse) {
      var inverseStrip = inverseAndProgram.strip;

      if (inverseStrip.left) {
        omitLeft(program.statements, null, true);
      }
      if (inverseStrip.right) {
        omitRight(inverse.statements, null, true);
      }
      if (close.strip.left) {
        omitLeft(inverse.statements, null, true);
      }

      // Find standalone else statments
      if (isPrevWhitespace(program.statements)
          && isNextWhitespace(inverse.statements)) {

        omitLeft(program.statements);
        omitRight(inverse.statements);
      }
    } else {
      if (close.strip.left) {
        omitLeft(program.statements, null, true);
      }
    }

    if (inverted) {
      return new this.BlockNode(mustache, inverse, program, strip, locInfo);
    } else {
      return new this.BlockNode(mustache, program, inverse, strip, locInfo);
    }
  }

  __exports__.prepareBlock = prepareBlock;
  function prepareProgram(statements, isRoot) {
    for (var i = 0, l = statements.length; i < l; i++) {
      var current = statements[i],
          strip = current.strip;

      if (!strip) {
        continue;
      }

      var _isPrevWhitespace = isPrevWhitespace(statements, i, isRoot, current.type === 'partial'),
          _isNextWhitespace = isNextWhitespace(statements, i, isRoot),

          openStandalone = strip.openStandalone && _isPrevWhitespace,
          closeStandalone = strip.closeStandalone && _isNextWhitespace,
          inlineStandalone = strip.inlineStandalone && _isPrevWhitespace && _isNextWhitespace;

      if (strip.right) {
        omitRight(statements, i, true);
      }
      if (strip.left) {
        omitLeft(statements, i, true);
      }

      if (inlineStandalone) {
        omitRight(statements, i);

        if (omitLeft(statements, i)) {
          // If we are on a standalone node, save the indent info for partials
          if (current.type === 'partial') {
            current.indent = (/([ \t]+$)/).exec(statements[i-1].original) ? RegExp.$1 : '';
          }
        }
      }
      if (openStandalone) {
        omitRight((current.program || current.inverse).statements);

        // Strip out the previous content node if it's whitespace only
        omitLeft(statements, i);
      }
      if (closeStandalone) {
        // Always strip the next node
        omitRight(statements, i);

        omitLeft((current.inverse || current.program).statements);
      }
    }

    return statements;
  }

  __exports__.prepareProgram = prepareProgram;function isPrevWhitespace(statements, i, isRoot) {
    if (i === undefined) {
      i = statements.length;
    }

    // Nodes that end with newlines are considered whitespace (but are special
    // cased for strip operations)
    var prev = statements[i-1],
        sibling = statements[i-2];
    if (!prev) {
      return isRoot;
    }

    if (prev.type === 'content') {
      return (sibling || !isRoot ? (/\r?\n\s*?$/) : (/(^|\r?\n)\s*?$/)).test(prev.original);
    }
  }
  function isNextWhitespace(statements, i, isRoot) {
    if (i === undefined) {
      i = -1;
    }

    var next = statements[i+1],
        sibling = statements[i+2];
    if (!next) {
      return isRoot;
    }

    if (next.type === 'content') {
      return (sibling || !isRoot ? (/^\s*?\r?\n/) : (/^\s*?(\r?\n|$)/)).test(next.original);
    }
  }

  // Marks the node to the right of the position as omitted.
  // I.e. {{foo}}' ' will mark the ' ' node as omitted.
  //
  // If i is undefined, then the first child will be marked as such.
  //
  // If mulitple is truthy then all whitespace will be stripped out until non-whitespace
  // content is met.
  function omitRight(statements, i, multiple) {
    var current = statements[i == null ? 0 : i + 1];
    if (!current || current.type !== 'content' || (!multiple && current.rightStripped)) {
      return;
    }

    var original = current.string;
    current.string = current.string.replace(multiple ? (/^\s+/) : (/^[ \t]*\r?\n?/), '');
    current.rightStripped = current.string !== original;
  }

  // Marks the node to the left of the position as omitted.
  // I.e. ' '{{foo}} will mark the ' ' node as omitted.
  //
  // If i is undefined then the last child will be marked as such.
  //
  // If mulitple is truthy then all whitespace will be stripped out until non-whitespace
  // content is met.
  function omitLeft(statements, i, multiple) {
    var current = statements[i == null ? statements.length - 1 : i - 1];
    if (!current || current.type !== 'content' || (!multiple && current.leftStripped)) {
      return;
    }

    // We omit the last node if it's whitespace only and not preceeded by a non-content node.
    var original = current.string;
    current.string = current.string.replace(multiple ? (/\s+$/) : (/[ \t]+$/), '');
    current.leftStripped = current.string !== original;
    return current.leftStripped;
  }
  return __exports__;
})(__module5__);

// handlebars/compiler/base.js
var __module8__ = (function(__dependency1__, __dependency2__, __dependency3__, __dependency4__) {
  "use strict";
  var __exports__ = {};
  var parser = __dependency1__;
  var AST = __dependency2__;
  var Helpers = __dependency3__;
  var extend = __dependency4__.extend;

  __exports__.parser = parser;

  var yy = {};
  extend(yy, Helpers, AST);

  function parse(input) {
    // Just return if an already-compile AST was passed in.
    if (input.constructor === AST.ProgramNode) { return input; }

    parser.yy = yy;

    return parser.parse(input);
  }

  __exports__.parse = parse;
  return __exports__;
})(__module9__, __module7__, __module10__, __module3__);

// handlebars/compiler/compiler.js
var __module11__ = (function(__dependency1__, __dependency2__) {
  "use strict";
  var __exports__ = {};
  var Exception = __dependency1__;
  var isArray = __dependency2__.isArray;

  var slice = [].slice;

  function Compiler() {}

  __exports__.Compiler = Compiler;// the foundHelper register will disambiguate helper lookup from finding a
  // function in a context. This is necessary for mustache compatibility, which
  // requires that context functions in blocks are evaluated by blockHelperMissing,
  // and then proceed as if the resulting value was provided to blockHelperMissing.

  Compiler.prototype = {
    compiler: Compiler,

    equals: function(other) {
      var len = this.opcodes.length;
      if (other.opcodes.length !== len) {
        return false;
      }

      for (var i = 0; i < len; i++) {
        var opcode = this.opcodes[i],
            otherOpcode = other.opcodes[i];
        if (opcode.opcode !== otherOpcode.opcode || !argEquals(opcode.args, otherOpcode.args)) {
          return false;
        }
      }

      // We know that length is the same between the two arrays because they are directly tied
      // to the opcode behavior above.
      len = this.children.length;
      for (i = 0; i < len; i++) {
        if (!this.children[i].equals(other.children[i])) {
          return false;
        }
      }

      return true;
    },

    guid: 0,

    compile: function(program, options) {
      this.opcodes = [];
      this.children = [];
      this.depths = {list: []};
      this.options = options;
      this.stringParams = options.stringParams;
      this.trackIds = options.trackIds;

      // These changes will propagate to the other compiler components
      var knownHelpers = this.options.knownHelpers;
      this.options.knownHelpers = {
        'helperMissing': true,
        'blockHelperMissing': true,
        'each': true,
        'if': true,
        'unless': true,
        'with': true,
        'log': true,
        'lookup': true
      };
      if (knownHelpers) {
        for (var name in knownHelpers) {
          this.options.knownHelpers[name] = knownHelpers[name];
        }
      }

      return this.accept(program);
    },

    accept: function(node) {
      return this[node.type](node);
    },

    program: function(program) {
      var statements = program.statements;

      for(var i=0, l=statements.length; i<l; i++) {
        this.accept(statements[i]);
      }
      this.isSimple = l === 1;

      this.depths.list = this.depths.list.sort(function(a, b) {
        return a - b;
      });

      return this;
    },

    compileProgram: function(program) {
      var result = new this.compiler().compile(program, this.options);
      var guid = this.guid++, depth;

      this.usePartial = this.usePartial || result.usePartial;

      this.children[guid] = result;

      for(var i=0, l=result.depths.list.length; i<l; i++) {
        depth = result.depths.list[i];

        if(depth < 2) { continue; }
        else { this.addDepth(depth - 1); }
      }

      return guid;
    },

    block: function(block) {
      var mustache = block.mustache,
          program = block.program,
          inverse = block.inverse;

      if (program) {
        program = this.compileProgram(program);
      }

      if (inverse) {
        inverse = this.compileProgram(inverse);
      }

      var sexpr = mustache.sexpr;
      var type = this.classifySexpr(sexpr);

      if (type === "helper") {
        this.helperSexpr(sexpr, program, inverse);
      } else if (type === "simple") {
        this.simpleSexpr(sexpr);

        // now that the simple mustache is resolved, we need to
        // evaluate it by executing `blockHelperMissing`
        this.opcode('pushProgram', program);
        this.opcode('pushProgram', inverse);
        this.opcode('emptyHash');
        this.opcode('blockValue', sexpr.id.original);
      } else {
        this.ambiguousSexpr(sexpr, program, inverse);

        // now that the simple mustache is resolved, we need to
        // evaluate it by executing `blockHelperMissing`
        this.opcode('pushProgram', program);
        this.opcode('pushProgram', inverse);
        this.opcode('emptyHash');
        this.opcode('ambiguousBlockValue');
      }

      this.opcode('append');
    },

    hash: function(hash) {
      var pairs = hash.pairs, i, l;

      this.opcode('pushHash');

      for(i=0, l=pairs.length; i<l; i++) {
        this.pushParam(pairs[i][1]);
      }
      while(i--) {
        this.opcode('assignToHash', pairs[i][0]);
      }
      this.opcode('popHash');
    },

    partial: function(partial) {
      var partialName = partial.partialName;
      this.usePartial = true;

      if (partial.hash) {
        this.accept(partial.hash);
      } else {
        this.opcode('push', 'undefined');
      }

      if (partial.context) {
        this.accept(partial.context);
      } else {
        this.opcode('getContext', 0);
        this.opcode('pushContext');
      }

      this.opcode('invokePartial', partialName.name, partial.indent || '');
      this.opcode('append');
    },

    content: function(content) {
      if (content.string) {
        this.opcode('appendContent', content.string);
      }
    },

    mustache: function(mustache) {
      this.sexpr(mustache.sexpr);

      if(mustache.escaped && !this.options.noEscape) {
        this.opcode('appendEscaped');
      } else {
        this.opcode('append');
      }
    },

    ambiguousSexpr: function(sexpr, program, inverse) {
      var id = sexpr.id,
          name = id.parts[0],
          isBlock = program != null || inverse != null;

      this.opcode('getContext', id.depth);

      this.opcode('pushProgram', program);
      this.opcode('pushProgram', inverse);

      this.ID(id);

      this.opcode('invokeAmbiguous', name, isBlock);
    },

    simpleSexpr: function(sexpr) {
      var id = sexpr.id;

      if (id.type === 'DATA') {
        this.DATA(id);
      } else if (id.parts.length) {
        this.ID(id);
      } else {
        // Simplified ID for `this`
        this.addDepth(id.depth);
        this.opcode('getContext', id.depth);
        this.opcode('pushContext');
      }

      this.opcode('resolvePossibleLambda');
    },

    helperSexpr: function(sexpr, program, inverse) {
      var params = this.setupFullMustacheParams(sexpr, program, inverse),
          id = sexpr.id,
          name = id.parts[0];

      if (this.options.knownHelpers[name]) {
        this.opcode('invokeKnownHelper', params.length, name);
      } else if (this.options.knownHelpersOnly) {
        throw new Exception("You specified knownHelpersOnly, but used the unknown helper " + name, sexpr);
      } else {
        id.falsy = true;

        this.ID(id);
        this.opcode('invokeHelper', params.length, id.original, id.isSimple);
      }
    },

    sexpr: function(sexpr) {
      var type = this.classifySexpr(sexpr);

      if (type === "simple") {
        this.simpleSexpr(sexpr);
      } else if (type === "helper") {
        this.helperSexpr(sexpr);
      } else {
        this.ambiguousSexpr(sexpr);
      }
    },

    ID: function(id) {
      this.addDepth(id.depth);
      this.opcode('getContext', id.depth);

      var name = id.parts[0];
      if (!name) {
        // Context reference, i.e. `{{foo .}}` or `{{foo ..}}`
        this.opcode('pushContext');
      } else {
        this.opcode('lookupOnContext', id.parts, id.falsy, id.isScoped);
      }
    },

    DATA: function(data) {
      this.options.data = true;
      this.opcode('lookupData', data.id.depth, data.id.parts);
    },

    STRING: function(string) {
      this.opcode('pushString', string.string);
    },

    NUMBER: function(number) {
      this.opcode('pushLiteral', number.number);
    },

    BOOLEAN: function(bool) {
      this.opcode('pushLiteral', bool.bool);
    },

    comment: function() {},

    // HELPERS
    opcode: function(name) {
      this.opcodes.push({ opcode: name, args: slice.call(arguments, 1) });
    },

    addDepth: function(depth) {
      if(depth === 0) { return; }

      if(!this.depths[depth]) {
        this.depths[depth] = true;
        this.depths.list.push(depth);
      }
    },

    classifySexpr: function(sexpr) {
      var isHelper   = sexpr.isHelper;
      var isEligible = sexpr.eligibleHelper;
      var options    = this.options;

      // if ambiguous, we can possibly resolve the ambiguity now
      // An eligible helper is one that does not have a complex path, i.e. `this.foo`, `../foo` etc.
      if (isEligible && !isHelper) {
        var name = sexpr.id.parts[0];

        if (options.knownHelpers[name]) {
          isHelper = true;
        } else if (options.knownHelpersOnly) {
          isEligible = false;
        }
      }

      if (isHelper) { return "helper"; }
      else if (isEligible) { return "ambiguous"; }
      else { return "simple"; }
    },

    pushParams: function(params) {
      for(var i=0, l=params.length; i<l; i++) {
        this.pushParam(params[i]);
      }
    },

    pushParam: function(val) {
      if (this.stringParams) {
        if(val.depth) {
          this.addDepth(val.depth);
        }
        this.opcode('getContext', val.depth || 0);
        this.opcode('pushStringParam', val.stringModeValue, val.type);

        if (val.type === 'sexpr') {
          // Subexpressions get evaluated and passed in
          // in string params mode.
          this.sexpr(val);
        }
      } else {
        if (this.trackIds) {
          this.opcode('pushId', val.type, val.idName || val.stringModeValue);
        }
        this.accept(val);
      }
    },

    setupFullMustacheParams: function(sexpr, program, inverse) {
      var params = sexpr.params;
      this.pushParams(params);

      this.opcode('pushProgram', program);
      this.opcode('pushProgram', inverse);

      if (sexpr.hash) {
        this.hash(sexpr.hash);
      } else {
        this.opcode('emptyHash');
      }

      return params;
    }
  };

  function precompile(input, options, env) {
    if (input == null || (typeof input !== 'string' && input.constructor !== env.AST.ProgramNode)) {
      throw new Exception("You must pass a string or Handlebars AST to Handlebars.precompile. You passed " + input);
    }

    options = options || {};
    if (!('data' in options)) {
      options.data = true;
    }
    if (options.compat) {
      options.useDepths = true;
    }

    var ast = env.parse(input);
    var environment = new env.Compiler().compile(ast, options);
    return new env.JavaScriptCompiler().compile(environment, options);
  }

  __exports__.precompile = precompile;function compile(input, options, env) {
    if (input == null || (typeof input !== 'string' && input.constructor !== env.AST.ProgramNode)) {
      throw new Exception("You must pass a string or Handlebars AST to Handlebars.compile. You passed " + input);
    }

    options = options || {};

    if (!('data' in options)) {
      options.data = true;
    }
    if (options.compat) {
      options.useDepths = true;
    }

    var compiled;

    function compileInput() {
      var ast = env.parse(input);
      var environment = new env.Compiler().compile(ast, options);
      var templateSpec = new env.JavaScriptCompiler().compile(environment, options, undefined, true);
      return env.template(templateSpec);
    }

    // Template is only compiled on first use and cached after that point.
    var ret = function(context, options) {
      if (!compiled) {
        compiled = compileInput();
      }
      return compiled.call(this, context, options);
    };
    ret._setup = function(options) {
      if (!compiled) {
        compiled = compileInput();
      }
      return compiled._setup(options);
    };
    ret._child = function(i, data, depths) {
      if (!compiled) {
        compiled = compileInput();
      }
      return compiled._child(i, data, depths);
    };
    return ret;
  }

  __exports__.compile = compile;function argEquals(a, b) {
    if (a === b) {
      return true;
    }

    if (isArray(a) && isArray(b) && a.length === b.length) {
      for (var i = 0; i < a.length; i++) {
        if (!argEquals(a[i], b[i])) {
          return false;
        }
      }
      return true;
    }
  }
  return __exports__;
})(__module5__, __module3__);

// handlebars/compiler/javascript-compiler.js
var __module12__ = (function(__dependency1__, __dependency2__) {
  "use strict";
  var __exports__;
  var COMPILER_REVISION = __dependency1__.COMPILER_REVISION;
  var REVISION_CHANGES = __dependency1__.REVISION_CHANGES;
  var Exception = __dependency2__;

  function Literal(value) {
    this.value = value;
  }

  function JavaScriptCompiler() {}

  JavaScriptCompiler.prototype = {
    // PUBLIC API: You can override these methods in a subclass to provide
    // alternative compiled forms for name lookup and buffering semantics
    nameLookup: function(parent, name /* , type*/) {
      if (JavaScriptCompiler.isValidJavaScriptVariableName(name)) {
        return parent + "." + name;
      } else {
        return parent + "['" + name + "']";
      }
    },
    depthedLookup: function(name) {
      this.aliases.lookup = 'this.lookup';

      return 'lookup(depths, "' + name + '")';
    },

    compilerInfo: function() {
      var revision = COMPILER_REVISION,
          versions = REVISION_CHANGES[revision];
      return [revision, versions];
    },

    appendToBuffer: function(string) {
      if (this.environment.isSimple) {
        return "return " + string + ";";
      } else {
        return {
          appendToBuffer: true,
          content: string,
          toString: function() { return "buffer += " + string + ";"; }
        };
      }
    },

    initializeBuffer: function() {
      return this.quotedString("");
    },

    namespace: "Handlebars",
    // END PUBLIC API

    compile: function(environment, options, context, asObject) {
      this.environment = environment;
      this.options = options;
      this.stringParams = this.options.stringParams;
      this.trackIds = this.options.trackIds;
      this.precompile = !asObject;

      this.name = this.environment.name;
      this.isChild = !!context;
      this.context = context || {
        programs: [],
        environments: []
      };

      this.preamble();

      this.stackSlot = 0;
      this.stackVars = [];
      this.aliases = {};
      this.registers = { list: [] };
      this.hashes = [];
      this.compileStack = [];
      this.inlineStack = [];

      this.compileChildren(environment, options);

      this.useDepths = this.useDepths || environment.depths.list.length || this.options.compat;

      var opcodes = environment.opcodes,
          opcode,
          i,
          l;

      for (i = 0, l = opcodes.length; i < l; i++) {
        opcode = opcodes[i];

        this[opcode.opcode].apply(this, opcode.args);
      }

      // Flush any trailing content that might be pending.
      this.pushSource('');

      /* istanbul ignore next */
      if (this.stackSlot || this.inlineStack.length || this.compileStack.length) {
        throw new Exception('Compile completed with content left on stack');
      }

      var fn = this.createFunctionContext(asObject);
      if (!this.isChild) {
        var ret = {
          compiler: this.compilerInfo(),
          main: fn
        };
        var programs = this.context.programs;
        for (i = 0, l = programs.length; i < l; i++) {
          if (programs[i]) {
            ret[i] = programs[i];
          }
        }

        if (this.environment.usePartial) {
          ret.usePartial = true;
        }
        if (this.options.data) {
          ret.useData = true;
        }
        if (this.useDepths) {
          ret.useDepths = true;
        }
        if (this.options.compat) {
          ret.compat = true;
        }

        if (!asObject) {
          ret.compiler = JSON.stringify(ret.compiler);
          ret = this.objectLiteral(ret);
        }

        return ret;
      } else {
        return fn;
      }
    },

    preamble: function() {
      // track the last context pushed into place to allow skipping the
      // getContext opcode when it would be a noop
      this.lastContext = 0;
      this.source = [];
    },

    createFunctionContext: function(asObject) {
      var varDeclarations = '';

      var locals = this.stackVars.concat(this.registers.list);
      if(locals.length > 0) {
        varDeclarations += ", " + locals.join(", ");
      }

      // Generate minimizer alias mappings
      for (var alias in this.aliases) {
        if (this.aliases.hasOwnProperty(alias)) {
          varDeclarations += ', ' + alias + '=' + this.aliases[alias];
        }
      }

      var params = ["depth0", "helpers", "partials", "data"];

      if (this.useDepths) {
        params.push('depths');
      }

      // Perform a second pass over the output to merge content when possible
      var source = this.mergeSource(varDeclarations);

      if (asObject) {
        params.push(source);

        return Function.apply(this, params);
      } else {
        return 'function(' + params.join(',') + ') {\n  ' + source + '}';
      }
    },
    mergeSource: function(varDeclarations) {
      var source = '',
          buffer,
          appendOnly = !this.forceBuffer,
          appendFirst;

      for (var i = 0, len = this.source.length; i < len; i++) {
        var line = this.source[i];
        if (line.appendToBuffer) {
          if (buffer) {
            buffer = buffer + '\n    + ' + line.content;
          } else {
            buffer = line.content;
          }
        } else {
          if (buffer) {
            if (!source) {
              appendFirst = true;
              source = buffer + ';\n  ';
            } else {
              source += 'buffer += ' + buffer + ';\n  ';
            }
            buffer = undefined;
          }
          source += line + '\n  ';

          if (!this.environment.isSimple) {
            appendOnly = false;
          }
        }
      }

      if (appendOnly) {
        if (buffer || !source) {
          source += 'return ' + (buffer || '""') + ';\n';
        }
      } else {
        varDeclarations += ", buffer = " + (appendFirst ? '' : this.initializeBuffer());
        if (buffer) {
          source += 'return buffer + ' + buffer + ';\n';
        } else {
          source += 'return buffer;\n';
        }
      }

      if (varDeclarations) {
        source = 'var ' + varDeclarations.substring(2) + (appendFirst ? '' : ';\n  ') + source;
      }

      return source;
    },

    // [blockValue]
    //
    // On stack, before: hash, inverse, program, value
    // On stack, after: return value of blockHelperMissing
    //
    // The purpose of this opcode is to take a block of the form
    // `{{#this.foo}}...{{/this.foo}}`, resolve the value of `foo`, and
    // replace it on the stack with the result of properly
    // invoking blockHelperMissing.
    blockValue: function(name) {
      this.aliases.blockHelperMissing = 'helpers.blockHelperMissing';

      var params = [this.contextName(0)];
      this.setupParams(name, 0, params);

      var blockName = this.popStack();
      params.splice(1, 0, blockName);

      this.push('blockHelperMissing.call(' + params.join(', ') + ')');
    },

    // [ambiguousBlockValue]
    //
    // On stack, before: hash, inverse, program, value
    // Compiler value, before: lastHelper=value of last found helper, if any
    // On stack, after, if no lastHelper: same as [blockValue]
    // On stack, after, if lastHelper: value
    ambiguousBlockValue: function() {
      this.aliases.blockHelperMissing = 'helpers.blockHelperMissing';

      // We're being a bit cheeky and reusing the options value from the prior exec
      var params = [this.contextName(0)];
      this.setupParams('', 0, params, true);

      this.flushInline();

      var current = this.topStack();
      params.splice(1, 0, current);

      this.pushSource("if (!" + this.lastHelper + ") { " + current + " = blockHelperMissing.call(" + params.join(", ") + "); }");
    },

    // [appendContent]
    //
    // On stack, before: ...
    // On stack, after: ...
    //
    // Appends the string value of `content` to the current buffer
    appendContent: function(content) {
      if (this.pendingContent) {
        content = this.pendingContent + content;
      }

      this.pendingContent = content;
    },

    // [append]
    //
    // On stack, before: value, ...
    // On stack, after: ...
    //
    // Coerces `value` to a String and appends it to the current buffer.
    //
    // If `value` is truthy, or 0, it is coerced into a string and appended
    // Otherwise, the empty string is appended
    append: function() {
      // Force anything that is inlined onto the stack so we don't have duplication
      // when we examine local
      this.flushInline();
      var local = this.popStack();
      this.pushSource('if (' + local + ' != null) { ' + this.appendToBuffer(local) + ' }');
      if (this.environment.isSimple) {
        this.pushSource("else { " + this.appendToBuffer("''") + " }");
      }
    },

    // [appendEscaped]
    //
    // On stack, before: value, ...
    // On stack, after: ...
    //
    // Escape `value` and append it to the buffer
    appendEscaped: function() {
      this.aliases.escapeExpression = 'this.escapeExpression';

      this.pushSource(this.appendToBuffer("escapeExpression(" + this.popStack() + ")"));
    },

    // [getContext]
    //
    // On stack, before: ...
    // On stack, after: ...
    // Compiler value, after: lastContext=depth
    //
    // Set the value of the `lastContext` compiler value to the depth
    getContext: function(depth) {
      this.lastContext = depth;
    },

    // [pushContext]
    //
    // On stack, before: ...
    // On stack, after: currentContext, ...
    //
    // Pushes the value of the current context onto the stack.
    pushContext: function() {
      this.pushStackLiteral(this.contextName(this.lastContext));
    },

    // [lookupOnContext]
    //
    // On stack, before: ...
    // On stack, after: currentContext[name], ...
    //
    // Looks up the value of `name` on the current context and pushes
    // it onto the stack.
    lookupOnContext: function(parts, falsy, scoped) {
      /*jshint -W083 */
      var i = 0,
          len = parts.length;

      if (!scoped && this.options.compat && !this.lastContext) {
        // The depthed query is expected to handle the undefined logic for the root level that
        // is implemented below, so we evaluate that directly in compat mode
        this.push(this.depthedLookup(parts[i++]));
      } else {
        this.pushContext();
      }

      for (; i < len; i++) {
        this.replaceStack(function(current) {
          var lookup = this.nameLookup(current, parts[i], 'context');
          // We want to ensure that zero and false are handled properly if the context (falsy flag)
          // needs to have the special handling for these values.
          if (!falsy) {
            return ' != null ? ' + lookup + ' : ' + current;
          } else {
            // Otherwise we can use generic falsy handling
            return ' && ' + lookup;
          }
        });
      }
    },

    // [lookupData]
    //
    // On stack, before: ...
    // On stack, after: data, ...
    //
    // Push the data lookup operator
    lookupData: function(depth, parts) {
      /*jshint -W083 */
      if (!depth) {
        this.pushStackLiteral('data');
      } else {
        this.pushStackLiteral('this.data(data, ' + depth + ')');
      }

      var len = parts.length;
      for (var i = 0; i < len; i++) {
        this.replaceStack(function(current) {
          return ' && ' + this.nameLookup(current, parts[i], 'data');
        });
      }
    },

    // [resolvePossibleLambda]
    //
    // On stack, before: value, ...
    // On stack, after: resolved value, ...
    //
    // If the `value` is a lambda, replace it on the stack by
    // the return value of the lambda
    resolvePossibleLambda: function() {
      this.aliases.lambda = 'this.lambda';

      this.push('lambda(' + this.popStack() + ', ' + this.contextName(0) + ')');
    },

    // [pushStringParam]
    //
    // On stack, before: ...
    // On stack, after: string, currentContext, ...
    //
    // This opcode is designed for use in string mode, which
    // provides the string value of a parameter along with its
    // depth rather than resolving it immediately.
    pushStringParam: function(string, type) {
      this.pushContext();
      this.pushString(type);

      // If it's a subexpression, the string result
      // will be pushed after this opcode.
      if (type !== 'sexpr') {
        if (typeof string === 'string') {
          this.pushString(string);
        } else {
          this.pushStackLiteral(string);
        }
      }
    },

    emptyHash: function() {
      this.pushStackLiteral('{}');

      if (this.trackIds) {
        this.push('{}'); // hashIds
      }
      if (this.stringParams) {
        this.push('{}'); // hashContexts
        this.push('{}'); // hashTypes
      }
    },
    pushHash: function() {
      if (this.hash) {
        this.hashes.push(this.hash);
      }
      this.hash = {values: [], types: [], contexts: [], ids: []};
    },
    popHash: function() {
      var hash = this.hash;
      this.hash = this.hashes.pop();

      if (this.trackIds) {
        this.push('{' + hash.ids.join(',') + '}');
      }
      if (this.stringParams) {
        this.push('{' + hash.contexts.join(',') + '}');
        this.push('{' + hash.types.join(',') + '}');
      }

      this.push('{\n    ' + hash.values.join(',\n    ') + '\n  }');
    },

    // [pushString]
    //
    // On stack, before: ...
    // On stack, after: quotedString(string), ...
    //
    // Push a quoted version of `string` onto the stack
    pushString: function(string) {
      this.pushStackLiteral(this.quotedString(string));
    },

    // [push]
    //
    // On stack, before: ...
    // On stack, after: expr, ...
    //
    // Push an expression onto the stack
    push: function(expr) {
      this.inlineStack.push(expr);
      return expr;
    },

    // [pushLiteral]
    //
    // On stack, before: ...
    // On stack, after: value, ...
    //
    // Pushes a value onto the stack. This operation prevents
    // the compiler from creating a temporary variable to hold
    // it.
    pushLiteral: function(value) {
      this.pushStackLiteral(value);
    },

    // [pushProgram]
    //
    // On stack, before: ...
    // On stack, after: program(guid), ...
    //
    // Push a program expression onto the stack. This takes
    // a compile-time guid and converts it into a runtime-accessible
    // expression.
    pushProgram: function(guid) {
      if (guid != null) {
        this.pushStackLiteral(this.programExpression(guid));
      } else {
        this.pushStackLiteral(null);
      }
    },

    // [invokeHelper]
    //
    // On stack, before: hash, inverse, program, params..., ...
    // On stack, after: result of helper invocation
    //
    // Pops off the helper's parameters, invokes the helper,
    // and pushes the helper's return value onto the stack.
    //
    // If the helper is not found, `helperMissing` is called.
    invokeHelper: function(paramSize, name, isSimple) {
      this.aliases.helperMissing = 'helpers.helperMissing';

      var nonHelper = this.popStack();
      var helper = this.setupHelper(paramSize, name);

      var lookup = (isSimple ? helper.name + ' || ' : '') + nonHelper + ' || helperMissing';
      this.push('((' + lookup + ').call(' + helper.callParams + '))');
    },

    // [invokeKnownHelper]
    //
    // On stack, before: hash, inverse, program, params..., ...
    // On stack, after: result of helper invocation
    //
    // This operation is used when the helper is known to exist,
    // so a `helperMissing` fallback is not required.
    invokeKnownHelper: function(paramSize, name) {
      var helper = this.setupHelper(paramSize, name);
      this.push(helper.name + ".call(" + helper.callParams + ")");
    },

    // [invokeAmbiguous]
    //
    // On stack, before: hash, inverse, program, params..., ...
    // On stack, after: result of disambiguation
    //
    // This operation is used when an expression like `{{foo}}`
    // is provided, but we don't know at compile-time whether it
    // is a helper or a path.
    //
    // This operation emits more code than the other options,
    // and can be avoided by passing the `knownHelpers` and
    // `knownHelpersOnly` flags at compile-time.
    invokeAmbiguous: function(name, helperCall) {
      this.aliases.functionType = '"function"';
      this.aliases.helperMissing = 'helpers.helperMissing';
      this.useRegister('helper');

      var nonHelper = this.popStack();

      this.emptyHash();
      var helper = this.setupHelper(0, name, helperCall);

      var helperName = this.lastHelper = this.nameLookup('helpers', name, 'helper');

      this.push(
        '((helper = (helper = ' + helperName + ' || ' + nonHelper + ') != null ? helper : helperMissing'
          + (helper.paramsInit ? '),(' + helper.paramsInit : '') + '),'
        + '(typeof helper === functionType ? helper.call(' + helper.callParams + ') : helper))');
    },

    // [invokePartial]
    //
    // On stack, before: context, ...
    // On stack after: result of partial invocation
    //
    // This operation pops off a context, invokes a partial with that context,
    // and pushes the result of the invocation back.
    invokePartial: function(name, indent) {
      var params = [this.nameLookup('partials', name, 'partial'), "'" + indent + "'", "'" + name + "'", this.popStack(), this.popStack(), "helpers", "partials"];

      if (this.options.data) {
        params.push("data");
      } else if (this.options.compat) {
        params.push('undefined');
      }
      if (this.options.compat) {
        params.push('depths');
      }

      this.push("this.invokePartial(" + params.join(", ") + ")");
    },

    // [assignToHash]
    //
    // On stack, before: value, ..., hash, ...
    // On stack, after: ..., hash, ...
    //
    // Pops a value off the stack and assigns it to the current hash
    assignToHash: function(key) {
      var value = this.popStack(),
          context,
          type,
          id;

      if (this.trackIds) {
        id = this.popStack();
      }
      if (this.stringParams) {
        type = this.popStack();
        context = this.popStack();
      }

      var hash = this.hash;
      if (context) {
        hash.contexts.push("'" + key + "': " + context);
      }
      if (type) {
        hash.types.push("'" + key + "': " + type);
      }
      if (id) {
        hash.ids.push("'" + key + "': " + id);
      }
      hash.values.push("'" + key + "': (" + value + ")");
    },

    pushId: function(type, name) {
      if (type === 'ID' || type === 'DATA') {
        this.pushString(name);
      } else if (type === 'sexpr') {
        this.pushStackLiteral('true');
      } else {
        this.pushStackLiteral('null');
      }
    },

    // HELPERS

    compiler: JavaScriptCompiler,

    compileChildren: function(environment, options) {
      var children = environment.children, child, compiler;

      for(var i=0, l=children.length; i<l; i++) {
        child = children[i];
        compiler = new this.compiler();

        var index = this.matchExistingProgram(child);

        if (index == null) {
          this.context.programs.push('');     // Placeholder to prevent name conflicts for nested children
          index = this.context.programs.length;
          child.index = index;
          child.name = 'program' + index;
          this.context.programs[index] = compiler.compile(child, options, this.context, !this.precompile);
          this.context.environments[index] = child;

          this.useDepths = this.useDepths || compiler.useDepths;
        } else {
          child.index = index;
          child.name = 'program' + index;
        }
      }
    },
    matchExistingProgram: function(child) {
      for (var i = 0, len = this.context.environments.length; i < len; i++) {
        var environment = this.context.environments[i];
        if (environment && environment.equals(child)) {
          return i;
        }
      }
    },

    programExpression: function(guid) {
      var child = this.environment.children[guid],
          depths = child.depths.list,
          useDepths = this.useDepths,
          depth;

      var programParams = [child.index, 'data'];

      if (useDepths) {
        programParams.push('depths');
      }

      return 'this.program(' + programParams.join(', ') + ')';
    },

    useRegister: function(name) {
      if(!this.registers[name]) {
        this.registers[name] = true;
        this.registers.list.push(name);
      }
    },

    pushStackLiteral: function(item) {
      return this.push(new Literal(item));
    },

    pushSource: function(source) {
      if (this.pendingContent) {
        this.source.push(this.appendToBuffer(this.quotedString(this.pendingContent)));
        this.pendingContent = undefined;
      }

      if (source) {
        this.source.push(source);
      }
    },

    pushStack: function(item) {
      this.flushInline();

      var stack = this.incrStack();
      this.pushSource(stack + " = " + item + ";");
      this.compileStack.push(stack);
      return stack;
    },

    replaceStack: function(callback) {
      var prefix = '',
          inline = this.isInline(),
          stack,
          createdStack,
          usedLiteral;

      /* istanbul ignore next */
      if (!this.isInline()) {
        throw new Exception('replaceStack on non-inline');
      }

      // We want to merge the inline statement into the replacement statement via ','
      var top = this.popStack(true);

      if (top instanceof Literal) {
        // Literals do not need to be inlined
        prefix = stack = top.value;
        usedLiteral = true;
      } else {
        // Get or create the current stack name for use by the inline
        createdStack = !this.stackSlot;
        var name = !createdStack ? this.topStackName() : this.incrStack();

        prefix = '(' + this.push(name) + ' = ' + top + ')';
        stack = this.topStack();
      }

      var item = callback.call(this, stack);

      if (!usedLiteral) {
        this.popStack();
      }
      if (createdStack) {
        this.stackSlot--;
      }
      this.push('(' + prefix + item + ')');
    },

    incrStack: function() {
      this.stackSlot++;
      if(this.stackSlot > this.stackVars.length) { this.stackVars.push("stack" + this.stackSlot); }
      return this.topStackName();
    },
    topStackName: function() {
      return "stack" + this.stackSlot;
    },
    flushInline: function() {
      var inlineStack = this.inlineStack;
      if (inlineStack.length) {
        this.inlineStack = [];
        for (var i = 0, len = inlineStack.length; i < len; i++) {
          var entry = inlineStack[i];
          if (entry instanceof Literal) {
            this.compileStack.push(entry);
          } else {
            this.pushStack(entry);
          }
        }
      }
    },
    isInline: function() {
      return this.inlineStack.length;
    },

    popStack: function(wrapped) {
      var inline = this.isInline(),
          item = (inline ? this.inlineStack : this.compileStack).pop();

      if (!wrapped && (item instanceof Literal)) {
        return item.value;
      } else {
        if (!inline) {
          /* istanbul ignore next */
          if (!this.stackSlot) {
            throw new Exception('Invalid stack pop');
          }
          this.stackSlot--;
        }
        return item;
      }
    },

    topStack: function() {
      var stack = (this.isInline() ? this.inlineStack : this.compileStack),
          item = stack[stack.length - 1];

      if (item instanceof Literal) {
        return item.value;
      } else {
        return item;
      }
    },

    contextName: function(context) {
      if (this.useDepths && context) {
        return 'depths[' + context + ']';
      } else {
        return 'depth' + context;
      }
    },

    quotedString: function(str) {
      return '"' + str
        .replace(/\\/g, '\\\\')
        .replace(/"/g, '\\"')
        .replace(/\n/g, '\\n')
        .replace(/\r/g, '\\r')
        .replace(/\u2028/g, '\\u2028')   // Per Ecma-262 7.3 + 7.8.4
        .replace(/\u2029/g, '\\u2029') + '"';
    },

    objectLiteral: function(obj) {
      var pairs = [];

      for (var key in obj) {
        if (obj.hasOwnProperty(key)) {
          pairs.push(this.quotedString(key) + ':' + obj[key]);
        }
      }

      return '{' + pairs.join(',') + '}';
    },

    setupHelper: function(paramSize, name, blockHelper) {
      var params = [],
          paramsInit = this.setupParams(name, paramSize, params, blockHelper);
      var foundHelper = this.nameLookup('helpers', name, 'helper');

      return {
        params: params,
        paramsInit: paramsInit,
        name: foundHelper,
        callParams: [this.contextName(0)].concat(params).join(", ")
      };
    },

    setupOptions: function(helper, paramSize, params) {
      var options = {}, contexts = [], types = [], ids = [], param, inverse, program;

      options.name = this.quotedString(helper);
      options.hash = this.popStack();

      if (this.trackIds) {
        options.hashIds = this.popStack();
      }
      if (this.stringParams) {
        options.hashTypes = this.popStack();
        options.hashContexts = this.popStack();
      }

      inverse = this.popStack();
      program = this.popStack();

      // Avoid setting fn and inverse if neither are set. This allows
      // helpers to do a check for `if (options.fn)`
      if (program || inverse) {
        if (!program) {
          program = 'this.noop';
        }

        if (!inverse) {
          inverse = 'this.noop';
        }

        options.fn = program;
        options.inverse = inverse;
      }

      // The parameters go on to the stack in order (making sure that they are evaluated in order)
      // so we need to pop them off the stack in reverse order
      var i = paramSize;
      while (i--) {
        param = this.popStack();
        params[i] = param;

        if (this.trackIds) {
          ids[i] = this.popStack();
        }
        if (this.stringParams) {
          types[i] = this.popStack();
          contexts[i] = this.popStack();
        }
      }

      if (this.trackIds) {
        options.ids = "[" + ids.join(",") + "]";
      }
      if (this.stringParams) {
        options.types = "[" + types.join(",") + "]";
        options.contexts = "[" + contexts.join(",") + "]";
      }

      if (this.options.data) {
        options.data = "data";
      }

      return options;
    },

    // the params and contexts arguments are passed in arrays
    // to fill in
    setupParams: function(helperName, paramSize, params, useRegister) {
      var options = this.objectLiteral(this.setupOptions(helperName, paramSize, params));

      if (useRegister) {
        this.useRegister('options');
        params.push('options');
        return 'options=' + options;
      } else {
        params.push(options);
        return '';
      }
    }
  };

  var reservedWords = (
    "break else new var" +
    " case finally return void" +
    " catch for switch while" +
    " continue function this with" +
    " default if throw" +
    " delete in try" +
    " do instanceof typeof" +
    " abstract enum int short" +
    " boolean export interface static" +
    " byte extends long super" +
    " char final native synchronized" +
    " class float package throws" +
    " const goto private transient" +
    " debugger implements protected volatile" +
    " double import public let yield"
  ).split(" ");

  var compilerWords = JavaScriptCompiler.RESERVED_WORDS = {};

  for(var i=0, l=reservedWords.length; i<l; i++) {
    compilerWords[reservedWords[i]] = true;
  }

  JavaScriptCompiler.isValidJavaScriptVariableName = function(name) {
    return !JavaScriptCompiler.RESERVED_WORDS[name] && /^[a-zA-Z_$][0-9a-zA-Z_$]*$/.test(name);
  };

  __exports__ = JavaScriptCompiler;
  return __exports__;
})(__module2__, __module5__);

// handlebars.js
var __module0__ = (function(__dependency1__, __dependency2__, __dependency3__, __dependency4__, __dependency5__) {
  "use strict";
  var __exports__;
  /*globals Handlebars: true */
  var Handlebars = __dependency1__;

  // Compiler imports
  var AST = __dependency2__;
  var Parser = __dependency3__.parser;
  var parse = __dependency3__.parse;
  var Compiler = __dependency4__.Compiler;
  var compile = __dependency4__.compile;
  var precompile = __dependency4__.precompile;
  var JavaScriptCompiler = __dependency5__;

  var _create = Handlebars.create;
  var create = function() {
    var hb = _create();

    hb.compile = function(input, options) {
      return compile(input, options, hb);
    };
    hb.precompile = function (input, options) {
      return precompile(input, options, hb);
    };

    hb.AST = AST;
    hb.Compiler = Compiler;
    hb.JavaScriptCompiler = JavaScriptCompiler;
    hb.Parser = Parser;
    hb.parse = parse;

    return hb;
  };

  Handlebars = create();
  Handlebars.create = create;

  Handlebars['default'] = Handlebars;

  __exports__ = Handlebars;
  return __exports__;
})(__module1__, __module7__, __module8__, __module11__, __module12__);

  return __module0__;
}));


//     Underscore.js 1.3.3
//     (c) 2009-2012 Jeremy Ashkenas, DocumentCloud Inc.
//     Underscore is freely distributable under the MIT license.
//     Portions of Underscore are inspired or borrowed from Prototype,
//     Oliver Steele's Functional, and John Resig's Micro-Templating.
//     For all details and documentation:
//     http://documentcloud.github.com/underscore

define('hbs/underscore',[],function() {

  // Baseline setup
  // --------------

  // Establish the object that gets returned to break out of a loop iteration.
  var breaker = {};

  // Save bytes in the minified (but not gzipped) version:
  var ArrayProto = Array.prototype, ObjProto = Object.prototype, FuncProto = Function.prototype;

  // Create quick reference variables for speed access to core prototypes.
  var slice            = ArrayProto.slice,
      unshift          = ArrayProto.unshift,
      toString         = ObjProto.toString,
      hasOwnProperty   = ObjProto.hasOwnProperty;

  // All **ECMAScript 5** native function implementations that we hope to use
  // are declared here.
  var
    nativeForEach      = ArrayProto.forEach,
    nativeMap          = ArrayProto.map,
    nativeReduce       = ArrayProto.reduce,
    nativeReduceRight  = ArrayProto.reduceRight,
    nativeFilter       = ArrayProto.filter,
    nativeEvery        = ArrayProto.every,
    nativeSome         = ArrayProto.some,
    nativeIndexOf      = ArrayProto.indexOf,
    nativeLastIndexOf  = ArrayProto.lastIndexOf,
    nativeIsArray      = Array.isArray,
    nativeKeys         = Object.keys,
    nativeBind         = FuncProto.bind;

  // Create a safe reference to the Underscore object for use below.
  var _ = function(obj) { return new wrapper(obj); };

  // Current version.
  _.VERSION = '1.3.3';

  // Collection Functions
  // --------------------

  // The cornerstone, an `each` implementation, aka `forEach`.
  // Handles objects with the built-in `forEach`, arrays, and raw objects.
  // Delegates to **ECMAScript 5**'s native `forEach` if available.
  var each = _.each = _.forEach = function(obj, iterator, context) {
    if (obj == null) return;
    if (nativeForEach && obj.forEach === nativeForEach) {
      obj.forEach(iterator, context);
    } else if (obj.length === +obj.length) {
      for (var i = 0, l = obj.length; i < l; i++) {
        if (i in obj && iterator.call(context, obj[i], i, obj) === breaker) return;
      }
    } else {
      for (var key in obj) {
        if (_.has(obj, key)) {
          if (iterator.call(context, obj[key], key, obj) === breaker) return;
        }
      }
    }
  };

  // Return the results of applying the iterator to each element.
  // Delegates to **ECMAScript 5**'s native `map` if available.
  _.map = _.collect = function(obj, iterator, context) {
    var results = [];
    if (obj == null) return results;
    if (nativeMap && obj.map === nativeMap) return obj.map(iterator, context);
    each(obj, function(value, index, list) {
      results[results.length] = iterator.call(context, value, index, list);
    });
    if (obj.length === +obj.length) results.length = obj.length;
    return results;
  };

  // **Reduce** builds up a single result from a list of values, aka `inject`,
  // or `foldl`. Delegates to **ECMAScript 5**'s native `reduce` if available.
  _.reduce = _.foldl = _.inject = function(obj, iterator, memo, context) {
    var initial = arguments.length > 2;
    if (obj == null) obj = [];
    if (nativeReduce && obj.reduce === nativeReduce) {
      if (context) iterator = _.bind(iterator, context);
      return initial ? obj.reduce(iterator, memo) : obj.reduce(iterator);
    }
    each(obj, function(value, index, list) {
      if (!initial) {
        memo = value;
        initial = true;
      } else {
        memo = iterator.call(context, memo, value, index, list);
      }
    });
    if (!initial) throw new TypeError('Reduce of empty array with no initial value');
    return memo;
  };

  // The right-associative version of reduce, also known as `foldr`.
  // Delegates to **ECMAScript 5**'s native `reduceRight` if available.
  _.reduceRight = _.foldr = function(obj, iterator, memo, context) {
    var initial = arguments.length > 2;
    if (obj == null) obj = [];
    if (nativeReduceRight && obj.reduceRight === nativeReduceRight) {
      if (context) iterator = _.bind(iterator, context);
      return initial ? obj.reduceRight(iterator, memo) : obj.reduceRight(iterator);
    }
    var reversed = _.toArray(obj).reverse();
    if (context && !initial) iterator = _.bind(iterator, context);
    return initial ? _.reduce(reversed, iterator, memo, context) : _.reduce(reversed, iterator);
  };

  // Return the first value which passes a truth test. Aliased as `detect`.
  _.find = _.detect = function(obj, iterator, context) {
    var result;
    any(obj, function(value, index, list) {
      if (iterator.call(context, value, index, list)) {
        result = value;
        return true;
      }
    });
    return result;
  };

  // Return all the elements that pass a truth test.
  // Delegates to **ECMAScript 5**'s native `filter` if available.
  // Aliased as `select`.
  _.filter = _.select = function(obj, iterator, context) {
    var results = [];
    if (obj == null) return results;
    if (nativeFilter && obj.filter === nativeFilter) return obj.filter(iterator, context);
    each(obj, function(value, index, list) {
      if (iterator.call(context, value, index, list)) results[results.length] = value;
    });
    return results;
  };

  // Return all the elements for which a truth test fails.
  _.reject = function(obj, iterator, context) {
    var results = [];
    if (obj == null) return results;
    each(obj, function(value, index, list) {
      if (!iterator.call(context, value, index, list)) results[results.length] = value;
    });
    return results;
  };

  // Determine whether all of the elements match a truth test.
  // Delegates to **ECMAScript 5**'s native `every` if available.
  // Aliased as `all`.
  _.every = _.all = function(obj, iterator, context) {
    var result = true;
    if (obj == null) return result;
    if (nativeEvery && obj.every === nativeEvery) return obj.every(iterator, context);
    each(obj, function(value, index, list) {
      if (!(result = result && iterator.call(context, value, index, list))) return breaker;
    });
    return !!result;
  };

  // Determine if at least one element in the object matches a truth test.
  // Delegates to **ECMAScript 5**'s native `some` if available.
  // Aliased as `any`.
  var any = _.some = _.any = function(obj, iterator, context) {
    iterator || (iterator = _.identity);
    var result = false;
    if (obj == null) return result;
    if (nativeSome && obj.some === nativeSome) return obj.some(iterator, context);
    each(obj, function(value, index, list) {
      if (result || (result = iterator.call(context, value, index, list))) return breaker;
    });
    return !!result;
  };

  // Determine if a given value is included in the array or object using `===`.
  // Aliased as `contains`.
  _.include = _.contains = function(obj, target) {
    var found = false;
    if (obj == null) return found;
    if (nativeIndexOf && obj.indexOf === nativeIndexOf) return obj.indexOf(target) != -1;
    found = any(obj, function(value) {
      return value === target;
    });
    return found;
  };

  // Invoke a method (with arguments) on every item in a collection.
  _.invoke = function(obj, method) {
    var args = slice.call(arguments, 2);
    return _.map(obj, function(value) {
      return (_.isFunction(method) ? method || value : value[method]).apply(value, args);
    });
  };

  // Convenience version of a common use case of `map`: fetching a property.
  _.pluck = function(obj, key) {
    return _.map(obj, function(value){ return value[key]; });
  };

  // Return the maximum element or (element-based computation).
  _.max = function(obj, iterator, context) {
    if (!iterator && _.isArray(obj) && obj[0] === +obj[0]) return Math.max.apply(Math, obj);
    if (!iterator && _.isEmpty(obj)) return -Infinity;
    var result = {computed : -Infinity};
    each(obj, function(value, index, list) {
      var computed = iterator ? iterator.call(context, value, index, list) : value;
      computed >= result.computed && (result = {value : value, computed : computed});
    });
    return result.value;
  };

  // Return the minimum element (or element-based computation).
  _.min = function(obj, iterator, context) {
    if (!iterator && _.isArray(obj) && obj[0] === +obj[0]) return Math.min.apply(Math, obj);
    if (!iterator && _.isEmpty(obj)) return Infinity;
    var result = {computed : Infinity};
    each(obj, function(value, index, list) {
      var computed = iterator ? iterator.call(context, value, index, list) : value;
      computed < result.computed && (result = {value : value, computed : computed});
    });
    return result.value;
  };

  // Shuffle an array.
  _.shuffle = function(obj) {
    var shuffled = [], rand;
    each(obj, function(value, index, list) {
      rand = Math.floor(Math.random() * (index + 1));
      shuffled[index] = shuffled[rand];
      shuffled[rand] = value;
    });
    return shuffled;
  };

  // Sort the object's values by a criterion produced by an iterator.
  _.sortBy = function(obj, val, context) {
    var iterator = _.isFunction(val) ? val : function(obj) { return obj[val]; };
    return _.pluck(_.map(obj, function(value, index, list) {
      return {
        value : value,
        criteria : iterator.call(context, value, index, list)
      };
    }).sort(function(left, right) {
      var a = left.criteria, b = right.criteria;
      if (a === void 0) return 1;
      if (b === void 0) return -1;
      return a < b ? -1 : a > b ? 1 : 0;
    }), 'value');
  };

  // Groups the object's values by a criterion. Pass either a string attribute
  // to group by, or a function that returns the criterion.
  _.groupBy = function(obj, val) {
    var result = {};
    var iterator = _.isFunction(val) ? val : function(obj) { return obj[val]; };
    each(obj, function(value, index) {
      var key = iterator(value, index);
      (result[key] || (result[key] = [])).push(value);
    });
    return result;
  };

  // Use a comparator function to figure out at what index an object should
  // be inserted so as to maintain order. Uses binary search.
  _.sortedIndex = function(array, obj, iterator) {
    iterator || (iterator = _.identity);
    var low = 0, high = array.length;
    while (low < high) {
      var mid = (low + high) >> 1;
      iterator(array[mid]) < iterator(obj) ? low = mid + 1 : high = mid;
    }
    return low;
  };

  // Safely convert anything iterable into a real, live array.
  _.toArray = function(obj) {
    if (!obj)                                     return [];
    if (_.isArray(obj))                           return slice.call(obj);
    if (_.isArguments(obj))                       return slice.call(obj);
    if (obj.toArray && _.isFunction(obj.toArray)) return obj.toArray();
    return _.values(obj);
  };

  // Return the number of elements in an object.
  _.size = function(obj) {
    return _.isArray(obj) ? obj.length : _.keys(obj).length;
  };

  // Array Functions
  // ---------------

  // Get the first element of an array. Passing **n** will return the first N
  // values in the array. Aliased as `head` and `take`. The **guard** check
  // allows it to work with `_.map`.
  _.first = _.head = _.take = function(array, n, guard) {
    return (n != null) && !guard ? slice.call(array, 0, n) : array[0];
  };

  // Returns everything but the last entry of the array. Especcialy useful on
  // the arguments object. Passing **n** will return all the values in
  // the array, excluding the last N. The **guard** check allows it to work with
  // `_.map`.
  _.initial = function(array, n, guard) {
    return slice.call(array, 0, array.length - ((n == null) || guard ? 1 : n));
  };

  // Get the last element of an array. Passing **n** will return the last N
  // values in the array. The **guard** check allows it to work with `_.map`.
  _.last = function(array, n, guard) {
    if ((n != null) && !guard) {
      return slice.call(array, Math.max(array.length - n, 0));
    } else {
      return array[array.length - 1];
    }
  };

  // Returns everything but the first entry of the array. Aliased as `tail`.
  // Especially useful on the arguments object. Passing an **index** will return
  // the rest of the values in the array from that index onward. The **guard**
  // check allows it to work with `_.map`.
  _.rest = _.tail = function(array, index, guard) {
    return slice.call(array, (index == null) || guard ? 1 : index);
  };

  // Trim out all falsy values from an array.
  _.compact = function(array) {
    return _.filter(array, function(value){ return !!value; });
  };

  // Return a completely flattened version of an array.
  _.flatten = function(array, shallow) {
    return _.reduce(array, function(memo, value) {
      if (_.isArray(value)) return memo.concat(shallow ? value : _.flatten(value));
      memo[memo.length] = value;
      return memo;
    }, []);
  };

  // Return a version of the array that does not contain the specified value(s).
  _.without = function(array) {
    return _.difference(array, slice.call(arguments, 1));
  };

  // Produce a duplicate-free version of the array. If the array has already
  // been sorted, you have the option of using a faster algorithm.
  // Aliased as `unique`.
  _.uniq = _.unique = function(array, isSorted, iterator) {
    var initial = iterator ? _.map(array, iterator) : array;
    var results = [];
    // The `isSorted` flag is irrelevant if the array only contains two elements.
    if (array.length < 3) isSorted = true;
    _.reduce(initial, function (memo, value, index) {
      if (isSorted ? _.last(memo) !== value || !memo.length : !_.include(memo, value)) {
        memo.push(value);
        results.push(array[index]);
      }
      return memo;
    }, []);
    return results;
  };

  // Produce an array that contains the union: each distinct element from all of
  // the passed-in arrays.
  _.union = function() {
    return _.uniq(_.flatten(arguments, true));
  };

  // Produce an array that contains every item shared between all the
  // passed-in arrays. (Aliased as "intersect" for back-compat.)
  _.intersection = _.intersect = function(array) {
    var rest = slice.call(arguments, 1);
    return _.filter(_.uniq(array), function(item) {
      return _.every(rest, function(other) {
        return _.indexOf(other, item) >= 0;
      });
    });
  };

  // Take the difference between one array and a number of other arrays.
  // Only the elements present in just the first array will remain.
  _.difference = function(array) {
    var rest = _.flatten(slice.call(arguments, 1), true);
    return _.filter(array, function(value){ return !_.include(rest, value); });
  };

  // Zip together multiple lists into a single array -- elements that share
  // an index go together.
  _.zip = function() {
    var args = slice.call(arguments);
    var length = _.max(_.pluck(args, 'length'));
    var results = new Array(length);
    for (var i = 0; i < length; i++) results[i] = _.pluck(args, "" + i);
    return results;
  };

  // If the browser doesn't supply us with indexOf (I'm looking at you, **MSIE**),
  // we need this function. Return the position of the first occurrence of an
  // item in an array, or -1 if the item is not included in the array.
  // Delegates to **ECMAScript 5**'s native `indexOf` if available.
  // If the array is large and already in sort order, pass `true`
  // for **isSorted** to use binary search.
  _.indexOf = function(array, item, isSorted) {
    if (array == null) return -1;
    var i, l;
    if (isSorted) {
      i = _.sortedIndex(array, item);
      return array[i] === item ? i : -1;
    }
    if (nativeIndexOf && array.indexOf === nativeIndexOf) return array.indexOf(item);
    for (i = 0, l = array.length; i < l; i++) if (i in array && array[i] === item) return i;
    return -1;
  };

  // Delegates to **ECMAScript 5**'s native `lastIndexOf` if available.
  _.lastIndexOf = function(array, item) {
    if (array == null) return -1;
    if (nativeLastIndexOf && array.lastIndexOf === nativeLastIndexOf) return array.lastIndexOf(item);
    var i = array.length;
    while (i--) if (i in array && array[i] === item) return i;
    return -1;
  };

  // Generate an integer Array containing an arithmetic progression. A port of
  // the native Python `range()` function. See
  // [the Python documentation](http://docs.python.org/library/functions.html#range).
  _.range = function(start, stop, step) {
    if (arguments.length <= 1) {
      stop = start || 0;
      start = 0;
    }
    step = arguments[2] || 1;

    var len = Math.max(Math.ceil((stop - start) / step), 0);
    var idx = 0;
    var range = new Array(len);

    while(idx < len) {
      range[idx++] = start;
      start += step;
    }

    return range;
  };

  // Function (ahem) Functions
  // ------------------

  // Reusable constructor function for prototype setting.
  var ctor = function(){};

  // Create a function bound to a given object (assigning `this`, and arguments,
  // optionally). Binding with arguments is also known as `curry`.
  // Delegates to **ECMAScript 5**'s native `Function.bind` if available.
  // We check for `func.bind` first, to fail fast when `func` is undefined.
  _.bind = function bind(func, context) {
    var bound, args;
    if (func.bind === nativeBind && nativeBind) return nativeBind.apply(func, slice.call(arguments, 1));
    if (!_.isFunction(func)) throw new TypeError;
    args = slice.call(arguments, 2);
    return bound = function() {
      if (!(this instanceof bound)) return func.apply(context, args.concat(slice.call(arguments)));
      ctor.prototype = func.prototype;
      var self = new ctor;
      var result = func.apply(self, args.concat(slice.call(arguments)));
      if (Object(result) === result) return result;
      return self;
    };
  };

  // Bind all of an object's methods to that object. Useful for ensuring that
  // all callbacks defined on an object belong to it.
  _.bindAll = function(obj) {
    var funcs = slice.call(arguments, 1);
    if (funcs.length == 0) funcs = _.functions(obj);
    each(funcs, function(f) { obj[f] = _.bind(obj[f], obj); });
    return obj;
  };

  // Memoize an expensive function by storing its results.
  _.memoize = function(func, hasher) {
    var memo = {};
    hasher || (hasher = _.identity);
    return function() {
      var key = hasher.apply(this, arguments);
      return _.has(memo, key) ? memo[key] : (memo[key] = func.apply(this, arguments));
    };
  };

  // Delays a function for the given number of milliseconds, and then calls
  // it with the arguments supplied.
  _.delay = function(func, wait) {
    var args = slice.call(arguments, 2);
    return setTimeout(function(){ return func.apply(null, args); }, wait);
  };

  // Defers a function, scheduling it to run after the current call stack has
  // cleared.
  _.defer = function(func) {
    return _.delay.apply(_, [func, 1].concat(slice.call(arguments, 1)));
  };

  // Returns a function, that, when invoked, will only be triggered at most once
  // during a given window of time.
  _.throttle = function(func, wait) {
    var context, args, timeout, throttling, more, result;
    var whenDone = _.debounce(function(){ more = throttling = false; }, wait);
    return function() {
      context = this; args = arguments;
      var later = function() {
        timeout = null;
        if (more) func.apply(context, args);
        whenDone();
      };
      if (!timeout) timeout = setTimeout(later, wait);
      if (throttling) {
        more = true;
      } else {
        result = func.apply(context, args);
      }
      whenDone();
      throttling = true;
      return result;
    };
  };

  // Returns a function, that, as long as it continues to be invoked, will not
  // be triggered. The function will be called after it stops being called for
  // N milliseconds. If `immediate` is passed, trigger the function on the
  // leading edge, instead of the trailing.
  _.debounce = function(func, wait, immediate) {
    var timeout;
    return function() {
      var context = this, args = arguments;
      var later = function() {
        timeout = null;
        if (!immediate) func.apply(context, args);
      };
      if (immediate && !timeout) func.apply(context, args);
      clearTimeout(timeout);
      timeout = setTimeout(later, wait);
    };
  };

  // Returns a function that will be executed at most one time, no matter how
  // often you call it. Useful for lazy initialization.
  _.once = function(func) {
    var ran = false, memo;
    return function() {
      if (ran) return memo;
      ran = true;
      return memo = func.apply(this, arguments);
    };
  };

  // Returns the first function passed as an argument to the second,
  // allowing you to adjust arguments, run code before and after, and
  // conditionally execute the original function.
  _.wrap = function(func, wrapper) {
    return function() {
      var args = [func].concat(slice.call(arguments, 0));
      return wrapper.apply(this, args);
    };
  };

  // Returns a function that is the composition of a list of functions, each
  // consuming the return value of the function that follows.
  _.compose = function() {
    var funcs = arguments;
    return function() {
      var args = arguments;
      for (var i = funcs.length - 1; i >= 0; i--) {
        args = [funcs[i].apply(this, args)];
      }
      return args[0];
    };
  };

  // Returns a function that will only be executed after being called N times.
  _.after = function(times, func) {
    if (times <= 0) return func();
    return function() {
      if (--times < 1) { return func.apply(this, arguments); }
    };
  };

  // Object Functions
  // ----------------

  // Retrieve the names of an object's properties.
  // Delegates to **ECMAScript 5**'s native `Object.keys`
  _.keys = nativeKeys || function(obj) {
    if (obj !== Object(obj)) throw new TypeError('Invalid object');
    var keys = [];
    for (var key in obj) if (_.has(obj, key)) keys[keys.length] = key;
    return keys;
  };

  // Retrieve the values of an object's properties.
  _.values = function(obj) {
    return _.map(obj, _.identity);
  };

  // Return a sorted list of the function names available on the object.
  // Aliased as `methods`
  _.functions = _.methods = function(obj) {
    var names = [];
    for (var key in obj) {
      if (_.isFunction(obj[key])) names.push(key);
    }
    return names.sort();
  };

  // Extend a given object with all the properties in passed-in object(s).
  _.extend = function(obj) {
    each(slice.call(arguments, 1), function(source) {
      for (var prop in source) {
        obj[prop] = source[prop];
      }
    });
    return obj;
  };

  // Return a copy of the object only containing the whitelisted properties.
  _.pick = function(obj) {
    var result = {};
    each(_.flatten(slice.call(arguments, 1)), function(key) {
      if (key in obj) result[key] = obj[key];
    });
    return result;
  };

  // Fill in a given object with default properties.
  _.defaults = function(obj) {
    each(slice.call(arguments, 1), function(source) {
      for (var prop in source) {
        if (obj[prop] == null) obj[prop] = source[prop];
      }
    });
    return obj;
  };

  // Create a (shallow-cloned) duplicate of an object.
  _.clone = function(obj) {
    if (!_.isObject(obj)) return obj;
    return _.isArray(obj) ? obj.slice() : _.extend({}, obj);
  };

  // Invokes interceptor with the obj, and then returns obj.
  // The primary purpose of this method is to "tap into" a method chain, in
  // order to perform operations on intermediate results within the chain.
  _.tap = function(obj, interceptor) {
    interceptor(obj);
    return obj;
  };

  // Internal recursive comparison function.
  function eq(a, b, stack) {
    // Identical objects are equal. `0 === -0`, but they aren't identical.
    // See the Harmony `egal` proposal: http://wiki.ecmascript.org/doku.php?id=harmony:egal.
    if (a === b) return a !== 0 || 1 / a == 1 / b;
    // A strict comparison is necessary because `null == undefined`.
    if (a == null || b == null) return a === b;
    // Unwrap any wrapped objects.
    if (a._chain) a = a._wrapped;
    if (b._chain) b = b._wrapped;
    // Invoke a custom `isEqual` method if one is provided.
    if (a.isEqual && _.isFunction(a.isEqual)) return a.isEqual(b);
    if (b.isEqual && _.isFunction(b.isEqual)) return b.isEqual(a);
    // Compare `[[Class]]` names.
    var className = toString.call(a);
    if (className != toString.call(b)) return false;
    switch (className) {
      // Strings, numbers, dates, and booleans are compared by value.
      case '[object String]':
        // Primitives and their corresponding object wrappers are equivalent; thus, `"5"` is
        // equivalent to `new String("5")`.
        return a == String(b);
      case '[object Number]':
        // `NaN`s are equivalent, but non-reflexive. An `egal` comparison is performed for
        // other numeric values.
        return a != +a ? b != +b : (a == 0 ? 1 / a == 1 / b : a == +b);
      case '[object Date]':
      case '[object Boolean]':
        // Coerce dates and booleans to numeric primitive values. Dates are compared by their
        // millisecond representations. Note that invalid dates with millisecond representations
        // of `NaN` are not equivalent.
        return +a == +b;
      // RegExps are compared by their source patterns and flags.
      case '[object RegExp]':
        return a.source == b.source &&
               a.global == b.global &&
               a.multiline == b.multiline &&
               a.ignoreCase == b.ignoreCase;
    }
    if (typeof a != 'object' || typeof b != 'object') return false;
    // Assume equality for cyclic structures. The algorithm for detecting cyclic
    // structures is adapted from ES 5.1 section 15.12.3, abstract operation `JO`.
    var length = stack.length;
    while (length--) {
      // Linear search. Performance is inversely proportional to the number of
      // unique nested structures.
      if (stack[length] == a) return true;
    }
    // Add the first object to the stack of traversed objects.
    stack.push(a);
    var size = 0, result = true;
    // Recursively compare objects and arrays.
    if (className == '[object Array]') {
      // Compare array lengths to determine if a deep comparison is necessary.
      size = a.length;
      result = size == b.length;
      if (result) {
        // Deep compare the contents, ignoring non-numeric properties.
        while (size--) {
          // Ensure commutative equality for sparse arrays.
          if (!(result = size in a == size in b && eq(a[size], b[size], stack))) break;
        }
      }
    } else {
      // Objects with different constructors are not equivalent.
      if ('constructor' in a != 'constructor' in b || a.constructor != b.constructor) return false;
      // Deep compare objects.
      for (var key in a) {
        if (_.has(a, key)) {
          // Count the expected number of properties.
          size++;
          // Deep compare each member.
          if (!(result = _.has(b, key) && eq(a[key], b[key], stack))) break;
        }
      }
      // Ensure that both objects contain the same number of properties.
      if (result) {
        for (key in b) {
          if (_.has(b, key) && !(size--)) break;
        }
        result = !size;
      }
    }
    // Remove the first object from the stack of traversed objects.
    stack.pop();
    return result;
  }

  // Perform a deep comparison to check if two objects are equal.
  _.isEqual = function(a, b) {
    return eq(a, b, []);
  };

  // Is a given array, string, or object empty?
  // An "empty" object has no enumerable own-properties.
  _.isEmpty = function(obj) {
    if (obj == null) return true;
    if (_.isArray(obj) || _.isString(obj)) return obj.length === 0;
    for (var key in obj) if (_.has(obj, key)) return false;
    return true;
  };

  // Is a given value a DOM element?
  _.isElement = function(obj) {
    return !!(obj && obj.nodeType == 1);
  };

  // Is a given value an array?
  // Delegates to ECMA5's native Array.isArray
  _.isArray = nativeIsArray || function(obj) {
    return toString.call(obj) == '[object Array]';
  };

  // Is a given variable an object?
  _.isObject = function(obj) {
    return obj === Object(obj);
  };

  // Is a given variable an arguments object?
  _.isArguments = function(obj) {
    return toString.call(obj) == '[object Arguments]';
  };
  if (!_.isArguments(arguments)) {
    _.isArguments = function(obj) {
      return !!(obj && _.has(obj, 'callee'));
    };
  }

  // Is a given value a function?
  _.isFunction = function(obj) {
    return toString.call(obj) == '[object Function]';
  };

  // Is a given value a string?
  _.isString = function(obj) {
    return toString.call(obj) == '[object String]';
  };

  // Is a given value a number?
  _.isNumber = function(obj) {
    return toString.call(obj) == '[object Number]';
  };

  // Is a given object a finite number?
  _.isFinite = function(obj) {
    return _.isNumber(obj) && isFinite(obj);
  };

  // Is the given value `NaN`?
  _.isNaN = function(obj) {
    // `NaN` is the only value for which `===` is not reflexive.
    return obj !== obj;
  };

  // Is a given value a boolean?
  _.isBoolean = function(obj) {
    return obj === true || obj === false || toString.call(obj) == '[object Boolean]';
  };

  // Is a given value a date?
  _.isDate = function(obj) {
    return toString.call(obj) == '[object Date]';
  };

  // Is the given value a regular expression?
  _.isRegExp = function(obj) {
    return toString.call(obj) == '[object RegExp]';
  };

  // Is a given value equal to null?
  _.isNull = function(obj) {
    return obj === null;
  };

  // Is a given variable undefined?
  _.isUndefined = function(obj) {
    return obj === void 0;
  };

  // Has own property?
  _.has = function(obj, key) {
    return hasOwnProperty.call(obj, key);
  };

  // Utility Functions
  // -----------------

  // Run Underscore.js in *noConflict* mode, returning the `_` variable to its
  // previous owner. Returns a reference to the Underscore object.
  _.noConflict = function() {
    root._ = previousUnderscore;
    return this;
  };

  // Keep the identity function around for default iterators.
  _.identity = function(value) {
    return value;
  };

  // Run a function **n** times.
  _.times = function (n, iterator, context) {
    for (var i = 0; i < n; i++) iterator.call(context, i);
  };

  // Escape a string for HTML interpolation.
  _.escape = function(string) {
    return (''+string).replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;').replace(/'/g, '&#x27;').replace(/\//g,'&#x2F;');
  };

  // If the value of the named property is a function then invoke it;
  // otherwise, return it.
  _.result = function(object, property) {
    if (object == null) return null;
    var value = object[property];
    return _.isFunction(value) ? value.call(object) : value;
  };

  // Add your own custom functions to the Underscore object, ensuring that
  // they're correctly added to the OOP wrapper as well.
  _.mixin = function(obj) {
    each(_.functions(obj), function(name){
      addToWrapper(name, _[name] = obj[name]);
    });
  };

  // Generate a unique integer id (unique within the entire client session).
  // Useful for temporary DOM ids.
  var idCounter = 0;
  _.uniqueId = function(prefix) {
    var id = idCounter++;
    return prefix ? prefix + id : id;
  };

  // By default, Underscore uses ERB-style template delimiters, change the
  // following template settings to use alternative delimiters.
  _.templateSettings = {
    evaluate    : /<%([\s\S]+?)%>/g,
    interpolate : /<%=([\s\S]+?)%>/g,
    escape      : /<%-([\s\S]+?)%>/g
  };

  // When customizing `templateSettings`, if you don't want to define an
  // interpolation, evaluation or escaping regex, we need one that is
  // guaranteed not to match.
  var noMatch = /.^/;

  // Certain characters need to be escaped so that they can be put into a
  // string literal.
  var escapes = {
    '\\': '\\',
    "'": "'",
    'r': '\r',
    'n': '\n',
    't': '\t',
    'u2028': '\u2028',
    'u2029': '\u2029'
  };

  for (var p in escapes) escapes[escapes[p]] = p;
  var escaper = /\\|'|\r|\n|\t|\u2028|\u2029/g;
  var unescaper = /\\(\\|'|r|n|t|u2028|u2029)/g;

  // Within an interpolation, evaluation, or escaping, remove HTML escaping
  // that had been previously added.
  var unescape = function(code) {
    return code.replace(unescaper, function(match, escape) {
      return escapes[escape];
    });
  };

  // JavaScript micro-templating, similar to John Resig's implementation.
  // Underscore templating handles arbitrary delimiters, preserves whitespace,
  // and correctly escapes quotes within interpolated code.
  _.template = function(text, data, settings) {
    settings = _.defaults(settings || {}, _.templateSettings);

    // Compile the template source, taking care to escape characters that
    // cannot be included in a string literal and then unescape them in code
    // blocks.
    var source = "__p+='" + text
      .replace(escaper, function(match) {
        return '\\' + escapes[match];
      })
      .replace(settings.escape || noMatch, function(match, code) {
        return "'+\n_.escape(" + unescape(code) + ")+\n'";
      })
      .replace(settings.interpolate || noMatch, function(match, code) {
        return "'+\n(" + unescape(code) + ")+\n'";
      })
      .replace(settings.evaluate || noMatch, function(match, code) {
        return "';\n" + unescape(code) + "\n;__p+='";
      }) + "';\n";

    // If a variable is not specified, place data values in local scope.
    if (!settings.variable) source = 'with(obj||{}){\n' + source + '}\n';

    source = "var __p='';" +
      "var print=function(){__p+=Array.prototype.join.call(arguments, '')};\n" +
      source + "return __p;\n";

    var render = new Function(settings.variable || 'obj', '_', source);
    if (data) return render(data, _);
    var template = function(data) {
      return render.call(this, data, _);
    };

    // Provide the compiled function source as a convenience for build time
    // precompilation.
    template.source = 'function(' + (settings.variable || 'obj') + '){\n' +
      source + '}';

    return template;
  };

  // Add a "chain" function, which will delegate to the wrapper.
  _.chain = function(obj) {
    return _(obj).chain();
  };

  // The OOP Wrapper
  // ---------------

  // If Underscore is called as a function, it returns a wrapped object that
  // can be used OO-style. This wrapper holds altered versions of all the
  // underscore functions. Wrapped objects may be chained.
  var wrapper = function(obj) { this._wrapped = obj; };

  // Expose `wrapper.prototype` as `_.prototype`
  _.prototype = wrapper.prototype;

  // Helper function to continue chaining intermediate results.
  var result = function(obj, chain) {
    return chain ? _(obj).chain() : obj;
  };

  // A method to easily add functions to the OOP wrapper.
  var addToWrapper = function(name, func) {
    wrapper.prototype[name] = function() {
      var args = slice.call(arguments);
      unshift.call(args, this._wrapped);
      return result(func.apply(_, args), this._chain);
    };
  };

  // Add all of the Underscore functions to the wrapper object.
  _.mixin(_);

  // Add all mutator Array functions to the wrapper.
  each(['pop', 'push', 'reverse', 'shift', 'sort', 'splice', 'unshift'], function(name) {
    var method = ArrayProto[name];
    wrapper.prototype[name] = function() {
      var wrapped = this._wrapped;
      method.apply(wrapped, arguments);
      var length = wrapped.length;
      if ((name == 'shift' || name == 'splice') && length === 0) delete wrapped[0];
      return result(wrapped, this._chain);
    };
  });

  // Add all accessor Array functions to the wrapper.
  each(['concat', 'join', 'slice'], function(name) {
    var method = ArrayProto[name];
    wrapper.prototype[name] = function() {
      return result(method.apply(this._wrapped, arguments), this._chain);
    };
  });

  // Start chaining a wrapped Underscore object.
  wrapper.prototype.chain = function() {
    this._chain = true;
    return this;
  };

  // Extracts the result from a wrapped and chained object.
  wrapper.prototype.value = function() {
    return this._wrapped;
  };

    return _;

});
;
define('hbs/i18nprecompile',['hbs/handlebars', "hbs/underscore"], function ( Handlebars, _ ) {

  function replaceLocaleStrings ( ast, mapping, options ) {
    options = options || {};
    mapping = mapping || {};
    // Base set of things
    if ( ast && ast.type === "program" && ast.statements ) {
      _(ast.statements).forEach(function(statement, i){
        var newString = "<!-- i18n error -->";
        // If it's a translation node
        if ( statement.type === "mustache" && statement.id && statement.id.original === "$" ) {

          if ( statement.params.length && statement.params[0].string ) {
            var key = statement.params[0].string;
            newString = mapping[ key ] || (options.originalKeyFallback ? key : newString);
          }
          ast.statements[i] = new Handlebars.AST.ContentNode(newString);
        }
        // If we need to recurse
        else if ( statement.program ) {
          statement.program = replaceLocaleStrings( statement.program, mapping, options );
        }
      });
      // Also cover the else blocks
      if (ast.inverse) {
        replaceLocaleStrings(ast.inverse, mapping, options);
      }
    }
    return ast;
  }

  return function precompile (string, mapping, options) {
    var ast, environment;

    options = options || {};

    if (!('data' in options)) {
      options.data = true;
    }

    if (options.compat) {
      options.useDepths = true;
    }

    ast = Handlebars.parse(string);

    // avoid replacing locale if mapping is `false`
    if (mapping !== false) {
        ast = replaceLocaleStrings(ast, mapping, options);
    }

    environment = new Handlebars.Compiler().compile(ast, options);
    return new Handlebars.JavaScriptCompiler().compile(environment, options);
  };
});
;
/*
    http://www.JSON.org/json2.js
    2011-10-19

    Public Domain.

    NO WARRANTY EXPRESSED OR IMPLIED. USE AT YOUR OWN RISK.

    See http://www.JSON.org/js.html


    This code should be minified before deployment.
    See http://javascript.crockford.com/jsmin.html

    USE YOUR OWN COPY. IT IS EXTREMELY UNWISE TO LOAD CODE FROM SERVERS YOU DO
    NOT CONTROL.
*/

/*jslint evil: true, regexp: true */

/*members "", "\b", "\t", "\n", "\f", "\r", "\"", JSON, "\\", apply,
    call, charCodeAt, getUTCDate, getUTCFullYear, getUTCHours,
    getUTCMinutes, getUTCMonth, getUTCSeconds, hasOwnProperty, join,
    lastIndex, length, parse, prototype, push, replace, slice, stringify,
    test, toJSON, toString, valueOf
*/

(function (window){

// Create a JSON object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

// Return the window JSON element if it exists;
var JSON = window.JSON || {};

(function () {
    'use strict';

    function f(n) {
        // Format integers to have at least two digits.
        return n < 10 ? '0' + n : n;
    }

    if (typeof Date.prototype.toJSON !== 'function') {

        Date.prototype.toJSON = function (key) {

            return isFinite(this.valueOf())
                ? this.getUTCFullYear()     + '-' +
                    f(this.getUTCMonth() + 1) + '-' +
                    f(this.getUTCDate())      + 'T' +
                    f(this.getUTCHours())     + ':' +
                    f(this.getUTCMinutes())   + ':' +
                    f(this.getUTCSeconds())   + 'Z'
                : null;
        };

        String.prototype.toJSON      =
            Number.prototype.toJSON  =
            Boolean.prototype.toJSON = function (key) {
                return this.valueOf();
            };
    }

    var cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        escapable = /[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        gap,
        indent,
        meta = {    // table of character substitutions
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '"' : '\\"',
            '\\': '\\\\'
        },
        rep;


    function quote(string) {

// If the string contains no control characters, no quote characters, and no
// backslash characters, then we can safely slap some quotes around it.
// Otherwise we must also replace the offending characters with safe escape
// sequences.

        escapable.lastIndex = 0;
        return escapable.test(string) ? '"' + string.replace(escapable, function (a) {
            var c = meta[a];
            return typeof c === 'string'
                ? c
                : '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
        }) + '"' : '"' + string + '"';
    }


    function str(key, holder) {

// Produce a string from holder[key].

        var i,          // The loop counter.
            k,          // The member key.
            v,          // The member value.
            length,
            mind = gap,
            partial,
            value = holder[key];

// If the value has a toJSON method, call it to obtain a replacement value.

        if (value && typeof value === 'object' &&
                typeof value.toJSON === 'function') {
            value = value.toJSON(key);
        }

// If we were called with a replacer function, then call the replacer to
// obtain a replacement value.

        if (typeof rep === 'function') {
            value = rep.call(holder, key, value);
        }

// What happens next depends on the value's type.

        switch (typeof value) {
        case 'string':
            return quote(value);

        case 'number':

// JSON numbers must be finite. Encode non-finite numbers as null.

            return isFinite(value) ? String(value) : 'null';

        case 'boolean':
        case 'null':

// If the value is a boolean or null, convert it to a string. Note:
// typeof null does not produce 'null'. The case is included here in
// the remote chance that this gets fixed someday.

            return String(value);

// If the type is 'object', we might be dealing with an object or an array or
// null.

        case 'object':

// Due to a specification blunder in ECMAScript, typeof null is 'object',
// so watch out for that case.

            if (!value) {
                return 'null';
            }

// Make an array to hold the partial results of stringifying this object value.

            gap += indent;
            partial = [];

// Is the value an array?

            if (Object.prototype.toString.apply(value) === '[object Array]') {

// The value is an array. Stringify every element. Use null as a placeholder
// for non-JSON values.

                length = value.length;
                for (i = 0; i < length; i += 1) {
                    partial[i] = str(i, value) || 'null';
                }

// Join all of the elements together, separated with commas, and wrap them in
// brackets.

                v = partial.length === 0
                    ? '[]'
                    : gap
                    ? '[\n' + gap + partial.join(',\n' + gap) + '\n' + mind + ']'
                    : '[' + partial.join(',') + ']';
                gap = mind;
                return v;
            }

// If the replacer is an array, use it to select the members to be stringified.

            if (rep && typeof rep === 'object') {
                length = rep.length;
                for (i = 0; i < length; i += 1) {
                    if (typeof rep[i] === 'string') {
                        k = rep[i];
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? ': ' : ':') + v);
                        }
                    }
                }
            } else {

// Otherwise, iterate through all of the keys in the object.

                for (k in value) {
                    if (Object.prototype.hasOwnProperty.call(value, k)) {
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? ': ' : ':') + v);
                        }
                    }
                }
            }

// Join all of the member texts together, separated with commas,
// and wrap them in braces.

            v = partial.length === 0
                ? '{}'
                : gap
                ? '{\n' + gap + partial.join(',\n' + gap) + '\n' + mind + '}'
                : '{' + partial.join(',') + '}';
            gap = mind;
            return v;
        }
    }

// If the JSON object does not yet have a stringify method, give it one.

    if (typeof JSON.stringify !== 'function') {
        JSON.stringify = function (value, replacer, space) {

// The stringify method takes a value and an optional replacer, and an optional
// space parameter, and returns a JSON text. The replacer can be a function
// that can replace values, or an array of strings that will select the keys.
// A default replacer method can be provided. Use of the space parameter can
// produce text that is more easily readable.

            var i;
            gap = '';
            indent = '';

// If the space parameter is a number, make an indent string containing that
// many spaces.

            if (typeof space === 'number') {
                for (i = 0; i < space; i += 1) {
                    indent += ' ';
                }

// If the space parameter is a string, it will be used as the indent string.

            } else if (typeof space === 'string') {
                indent = space;
            }

// If there is a replacer, it must be a function or an array.
// Otherwise, throw an error.

            rep = replacer;
            if (replacer && typeof replacer !== 'function' &&
                    (typeof replacer !== 'object' ||
                    typeof replacer.length !== 'number')) {
                throw new Error('JSON.stringify');
            }

// Make a fake root object containing our value under the key of ''.
// Return the result of stringifying the value.

            return str('', {'': value});
        };
    }


// If the JSON object does not yet have a parse method, give it one.

    if (typeof JSON.parse !== 'function') {
        JSON.parse = function (text, reviver) {

// The parse method takes a text and an optional reviver function, and returns
// a JavaScript value if the text is a valid JSON text.

            var j;

            function walk(holder, key) {

// The walk method is used to recursively walk the resulting structure so
// that modifications can be made.

                var k, v, value = holder[key];
                if (value && typeof value === 'object') {
                    for (k in value) {
                        if (Object.prototype.hasOwnProperty.call(value, k)) {
                            v = walk(value, k);
                            if (v !== undefined) {
                                value[k] = v;
                            } else {
                                delete value[k];
                            }
                        }
                    }
                }
                return reviver.call(holder, key, value);
            }


// Parsing happens in four stages. In the first stage, we replace certain
// Unicode characters with escape sequences. JavaScript handles many characters
// incorrectly, either silently deleting them, or treating them as line endings.

            text = String(text);
            cx.lastIndex = 0;
            if (cx.test(text)) {
                text = text.replace(cx, function (a) {
                    return '\\u' +
                        ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
                });
            }

// In the second stage, we run the text against regular expressions that look
// for non-JSON patterns. We are especially concerned with '()' and 'new'
// because they can cause invocation, and '=' because it can cause mutation.
// But just to be safe, we want to reject all unexpected forms.

// We split the second stage into 4 regexp operations in order to work around
// crippling inefficiencies in IE's and Safari's regexp engines. First we
// replace the JSON backslash pairs with '@' (a non-JSON character). Second, we
// replace all simple value tokens with ']' characters. Third, we delete all
// open brackets that follow a colon or comma or that begin the text. Finally,
// we look to see that the remaining characters are only whitespace or ']' or
// ',' or ':' or '{' or '}'. If that is so, then the text is safe for eval.

            if (/^[\],:{}\s]*$/
                    .test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@')
                        .replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']')
                        .replace(/(?:^|:|,)(?:\s*\[)+/g, ''))) {

// In the third stage we use the eval function to compile the text into a
// JavaScript structure. The '{' operator is subject to a syntactic ambiguity
// in JavaScript: it can begin a block or an object literal. We wrap the text
// in parens to eliminate the ambiguity.

                j = eval('(' + text + ')');

// In the optional fourth stage, we recursively walk the new structure, passing
// each name/value pair to a reviver function for possible transformation.

                return typeof reviver === 'function'
                    ? walk({'': j}, '')
                    : j;
            }

// If the text is not JSON parseable, then a SyntaxError is thrown.

            throw new SyntaxError('JSON.parse');
        };
    }
}());

define('hbs/json2',[],function(){
    return JSON;
});
// otherwise just leave it alone
    
}).call(this, this);
;
/**
 * @license Handlebars hbs 2.0.0 - Alex Sexton, but Handlebars has its own licensing junk
 *
 * Available via the MIT or new BSD license.
 * see: http://github.com/jrburke/require-cs for details on the plugin this was based off of
 */

/* Yes, deliciously evil. */
/*jslint evil: true, strict: false, plusplus: false, regexp: false */
/*global require: false, XMLHttpRequest: false, ActiveXObject: false,
define: false, process: false, window: false */
define('hbs',[
  'hbs/handlebars', 'hbs/underscore', 'hbs/i18nprecompile', 'hbs/json2'
], function (
  Handlebars, _, precompile, JSON
) {
    var fs;
  var getXhr;
  var progIds = ['Msxml2.XMLHTTP', 'Microsoft.XMLHTTP', 'Msxml2.XMLHTTP.4.0'];
  var fetchText = function () {
      throw new Error('Environment unsupported.');
  };
  var buildMap = [];
  var filecode = 'w+';
  var templateExtension = 'hbs';
  var customNameExtension = '@hbs';
  var devStyleDirectory = '/styles/';
  var buildStyleDirectory = '/demo-build/styles/';
  var helperDirectory = 'templates/helpers/';
  var i18nDirectory = 'templates/i18n/';
  var buildCSSFileName = 'screen.build.css';
  var onHbsReadMethod = "onHbsRead";

  Handlebars.registerHelper('$', function() {
    //placeholder for translation helper
  });

  if (typeof window !== 'undefined' && window.navigator && window.document && !window.navigator.userAgent.match(/Node.js/)) {
    // Browser action
    getXhr = function () {
      // Would love to dump the ActiveX crap in here. Need IE 6 to die first.
      var xhr;
      var i;
      var progId;
      if (typeof XMLHttpRequest !== 'undefined') {
        return ((arguments[0] === true)) ? new XDomainRequest() : new XMLHttpRequest();
      }
      else {
        for (i = 0; i < 3; i++) {
          progId = progIds[i];
          try {
            xhr = new ActiveXObject(progId);
          }
          catch (e) {}

          if (xhr) {
            // Faster next time
            progIds = [progId];
            break;
          }
        }
      }

      if (!xhr) {
          throw new Error('getXhr(): XMLHttpRequest not available');
      }

      return xhr;
    };

    // Returns the version of Windows Internet Explorer or a -1
    // (indicating the use of another browser).
    // Note: this is only for development mode. Does not run in production.
    getIEVersion = function(){
      // Return value assumes failure.
      var rv = -1;
      if (navigator.appName == 'Microsoft Internet Explorer') {
        var ua = navigator.userAgent;
        var re = new RegExp('MSIE ([0-9]{1,}[\.0-9]{0,})');
        if (re.exec(ua) != null) {
          rv = parseFloat( RegExp.$1 );
        }
      }
      return rv;
    };

    fetchText = function (url, callback) {
      var xdm = false;
      // If url is a fully qualified URL, it might be a cross domain request. Check for that.
      // IF url is a relative url, it cannot be cross domain.
      if (url.indexOf('http') != 0 ){
          xdm = false;
      }else{
          var uidx = (url.substr(0,5) === 'https') ? 8 : 7;
          var hidx = (window.location.href.substr(0,5) === 'https') ? 8 : 7;
          var dom = url.substr(uidx).split('/').shift();
          var msie = getIEVersion();
              xdm = ( dom != window.location.href.substr(hidx).split('/').shift() ) && (msie >= 7);
      }

      if ( xdm ) {
         var xdr = getXhr(true);
        xdr.open('GET', url);
        xdr.onload = function() {
          callback(xdr.responseText, url);
        };
        xdr.onprogress = function(){};
        xdr.ontimeout = function(){};
        xdr.onerror = function(){};
        setTimeout(function(){
          xdr.send();
        }, 0);
      }
      else {
        var xhr = getXhr();
        xhr.open('GET', url, true);
        xhr.onreadystatechange = function (evt) {
          //Do not explicitly handle errors, those should be
          //visible via console output in the browser.
          if (xhr.readyState === 4) {
            callback(xhr.responseText, url);
          }
        };
        xhr.send(null);
      }
    };

  }
  else if (
    typeof process !== 'undefined' &&
    process.versions &&
    !!process.versions.node
  ) {
    //Using special require.nodeRequire, something added by r.js.
    fs = require.nodeRequire('fs');
    fetchText = function ( path, callback ) {
      var body = fs.readFileSync(path, 'utf8') || '';
      // we need to remove BOM stuff from the file content
      body = body.replace(/^\uFEFF/, '');
      callback(body, path);
    };
  }
  else if (typeof java !== 'undefined' && typeof java.io !== 'undefined') {
    fetchText = function(path, callback) {
      var fis = new java.io.FileInputStream(path);
      var streamReader = new java.io.InputStreamReader(fis, "UTF-8");
      var reader = new java.io.BufferedReader(streamReader);
      var line;
      var text = '';
      while ((line = reader.readLine()) !== null) {
        text += new String(line) + '\n';
      }
      reader.close();
      callback(text, path);
    };
  }

  var cache = {};
  var fetchOrGetCached = function ( path, callback ){
    if ( cache[path] ){
      callback(cache[path]);
    }
    else {
      fetchText(path, function(data, path){
        cache[path] = data;
        callback.call(this, data);
      });
    }
  };
  var styleList = [];
  var styleMap = {};
  
  var config;
  var filesToRemove = [];

  return {

    get: function () {
      return Handlebars;
    },

    write: function (pluginName, name, write) {
      if ( (name + customNameExtension ) in buildMap) {
        var text = buildMap[name + customNameExtension];
        write.asModule(pluginName + '!' + name, text);
      }
    },

    version: '2.0.0',

    load: function (name, parentRequire, load, _config) {
            config = config || _config;

      var compiledName = name + customNameExtension;
      config.hbs = config.hbs || {};
      var disableI18n = !(config.hbs.i18n == true); // by default we disable i18n unless config.hbs.i18n is true
      var disableHelpers = (config.hbs.helpers == false); // be default we enable helpers unless config.hbs.helpers is false
      var partialsUrl = '';
      if(config.hbs.partialsUrl) {
        partialsUrl = config.hbs.partialsUrl;
        if(!partialsUrl.match(/\/$/)) partialsUrl += '/';
      }

      // Let redefine default fetchText
      if(config.hbs.fetchText) {
          fetchText = config.hbs.fetchText;
      }

      var partialDeps = [];

      function recursiveNodeSearch( statements, res ) {
        _(statements).forEach(function ( statement ) {
          if ( statement && statement.type && statement.type === 'partial' ) {
            res.push(statement.partialName.name);
          }
          if ( statement && statement.program && statement.program.statements ) {
            recursiveNodeSearch( statement.program.statements, res );
          }
          if ( statement && statement.inverse && statement.inverse.statements ) {
            recursiveNodeSearch( statement.inverse.statements, res );
          }
        });
        return res;
      }

      // TODO :: use the parser to do this!
      function findPartialDeps( nodes ) {
        var res = [];
        if ( nodes && nodes.statements ) {
          res = recursiveNodeSearch( nodes.statements, [] );
        }
        return _.unique(res);
      }

      // See if the first item is a comment that's json
      function getMetaData( nodes ) {
        var statement, res, test;
        if ( nodes && nodes.statements ) {
          statement = nodes.statements[0];
          if ( statement && statement.type === 'comment' ) {
            try {
              res = ( statement.comment ).replace(new RegExp('^[\\s]+|[\\s]+$', 'g'), '');
              test = JSON.parse(res);
              return res;
            }
            catch (e) {
              return JSON.stringify({
                description: res
              });
            }
          }
        }
        return '{}';
      }

      function composeParts ( parts ) {
        if ( !parts ) {
          return [];
        }
        var res = [parts[0]];
        var cur = parts[0];
        var i;

        for (i = 1; i < parts.length; ++i) {
          if ( parts.hasOwnProperty(i) ) {
            cur += '.' + parts[i];
            res.push( cur );
          }
        }
        return res;
      }

      function recursiveVarSearch( statements, res, prefix, helpersres ) {
        prefix = prefix ? prefix + '.' : '';

        var  newprefix = '';
        var flag = false;

        // loop through each statement
        _(statements).forEach(function(statement) {
          var parts;
          var part;
          var sideways;

          // if it's a mustache block
          if ( statement && statement.type && statement.type === 'mustache' ) {

            // If it has params, the first part is a helper or something
            if ( !statement.params || ! statement.params.length ) {
              parts = composeParts( statement.id.parts );
              for( part in parts ) {
                if ( parts[ part ] ) {
                  newprefix = parts[ part ] || newprefix;
                  res.push( prefix + parts[ part ] );
                }
              }
              res.push(prefix + statement.id.string);
            }

            var paramsWithoutParts = ['this', '.', '..', './..', '../..', '../../..'];

            // grab the params
            if ( statement.params && typeof Handlebars.helpers[statement.id.string] === 'undefined') {
              _(statement.params).forEach(function(param) {
                if ( _(paramsWithoutParts).contains(param.original)
                  || param instanceof Handlebars.AST.StringNode
                  || param instanceof Handlebars.AST.NumberNode
                  || param instanceof Handlebars.AST.BooleanNode
                  || param instanceof Handlebars.AST.DataNode
                  || param instanceof Handlebars.AST.SexprNode
                ) {
                  helpersres.push(statement.id.string);

                  // Look into the params to find subexpressions
                  if (typeof statement.params !== 'undefined') {
                      _(statement.params).forEach(function(param) {
                        if (param.type === 'sexpr' && param.isHelper === true) {
                          // Found subexpression in params
                          helpersres.push(param.id.string);
                        }
                      });
                  }

                  // Look in the hash to find sub expressions
                  if ((statement.hash != null) && (typeof statement.hash !== 'undefined') && (typeof statement.hash.pairs !== 'undefined')) {
                    _(statement.hash.pairs).forEach(function(pair) {
                      var pairName = pair[0],
                          pairValue = pair[1];
                      if (pairValue.type === 'sexpr' && pairValue.isHelper === true) {
                        // Found subexpression in hash params
                        helpersres.push(pairValue.id.string);
                      }
                    });
                  }
                }

                parts = composeParts( param.parts );

                for(var part in parts ) {
                  if ( parts[ part ] ) {
                    newprefix = parts[part] || newprefix;
                    helpersres.push(statement.id.string);
                    res.push( prefix + parts[ part ] );
                  }
                }
              });
              if ((statement.hash != null) && (typeof statement.hash !== 'undefined') && (typeof statement.hash.pairs !== 'undefined')) {
                //Even if it has no regular params, it may be a helper with hash params
                _(statement.hash.pairs).forEach(function(pair) {
                  var pairValue = pair[1];
                  if (pairValue instanceof Handlebars.AST.StringNode
                    || pairValue instanceof Handlebars.AST.NumberNode
                    || pairValue instanceof Handlebars.AST.BooleanNode
                    || pairValue instanceof Handlebars.AST.IdNode
                    //TODO: Add support for subexpressions here?
                  ) {
                    helpersres.push(statement.id.string);
                  }
                });
              }
            }
          }

          // If it's a meta block
          if ( statement && statement.mustache  ) {
            recursiveVarSearch( [statement.mustache], res, prefix + newprefix, helpersres );
          }

          // if it's a whole new program
          if ( statement && statement.program && statement.program.statements ) {
            sideways = recursiveVarSearch([statement.mustache],[], '', helpersres)[0] || '';
            if ( statement.inverse && statement.inverse.statements ) {
              recursiveVarSearch( statement.inverse.statements, res, prefix + newprefix + (sideways ? (prefix+newprefix) ? '.'+sideways : sideways : ''), helpersres);
            }
            recursiveVarSearch( statement.program.statements, res, prefix + newprefix + (sideways ? (prefix+newprefix) ? '.'+sideways : sideways : ''), helpersres);
          }
        });
        return res;
      }

      // This finds the Helper dependencies since it's soooo similar
      function getExternalDeps( nodes ) {
        var res   = [];
        var helpersres = [];

        if ( nodes && nodes.statements ) {
          res = recursiveVarSearch( nodes.statements, [], undefined, helpersres );
        }

        var defaultHelpers = [
          'helperMissing',
          'blockHelperMissing',
          'each',
          'if',
          'unless',
          'with',
          'log',
          'lookup'
        ];

        return {
          vars: _(res).chain().unique().map(function(e) {
            if ( e === '' ) {
              return '.';
            }
            if ( e.length && e[e.length-1] === '.' ) {
              return e.substr(0,e.length-1) + '[]';
            }
            return e;
          }).value(),

          helpers: _(helpersres).chain().unique().map(function(e){
            if ( _(defaultHelpers).contains(e) ) {
              return undefined;
            }
            return e;
          }).compact().value()
        };
      }

      function cleanPath(path) {
        var tokens = path.split('/');
        for(var i=0;i<tokens.length; i++) {
          if(tokens[i] === '..') {
            delete tokens[i-1];
            delete tokens[i];
          } else if (tokens[i] === '.') {
            delete tokens[i];
          }
        }
        return tokens.join('/').replace(/\/\/+/g,'/');
      }

      function fetchAndRegister(langMap) {
          fetchText(path, function(text, path) {

          var readCallback = (config.isBuild && config[onHbsReadMethod]) ? config[onHbsReadMethod]:  function(name,path,text){return text} ;
          // for some reason it doesn't include hbs _first_ when i don't add it here...
          var nodes = Handlebars.parse( readCallback(name, path, text));
          var partials = findPartialDeps( nodes );
          var meta = getMetaData( nodes );
          var extDeps = getExternalDeps( nodes );
          var vars = extDeps.vars;
          var helps = (extDeps.helpers || []);
          var debugOutputStart = '';
          var debugOutputEnd   = '';
          var debugProperties = '';
          var deps = [];
          var depStr, helpDepStr, metaObj, head, linkElem;
          var baseDir = name.substr(0,name.lastIndexOf('/')+1);

          require.config.hbs = require.config.hbs || {};
          require.config.hbs._partials = require.config.hbs._partials || {};

          if(meta !== '{}') {
            try {
              metaObj = JSON.parse(meta);
            } catch(e) {
              console.log('couldn\'t parse meta for %s', path);
            }
          }

          for ( var i in partials ) {
            if ( partials.hasOwnProperty(i) && typeof partials[i] === 'string') {  // make sure string, because we're iterating over all props
              var partialReference = partials[i];

              var path;
              if(partialReference.match(/^(\.|\/)+/)) {
                // relative path
                path = cleanPath(baseDir + partialReference)
              }
              else {
                // absolute path relative to config.hbs.partialsUrl if defined
                path = cleanPath(partialsUrl + partialReference);
              }

              require.config.hbs._partials[path] = require.config.hbs._partials[path] || [];

              // we can reference a same template with different paths (with absolute or relative)
              require.config.hbs._partials[path].references = require.config.hbs._partials[path].references || [];
              require.config.hbs._partials[path].references.push(partialReference);

              require.config.hbs._loadedDeps = require.config.hbs._loadedDeps || {};

              deps[i] = "hbs!"+path;
            }
          }

          depStr = deps.join("', '");

          helps = helps.concat((metaObj && metaObj.helpers) ? metaObj.helpers : []);
          helpDepStr = disableHelpers ?
            '' : (function (){
              var i;
              var paths = [];
              var pathGetter = config.hbs && config.hbs.helperPathCallback
                ? config.hbs.helperPathCallback
                : function (name){
                  return (config.hbs && config.hbs.helperDirectory ? config.hbs.helperDirectory : helperDirectory) + name;
                };

              for ( i = 0; i < helps.length; i++ ) {
                paths[i] = "'" + pathGetter(helps[i], path) + "'"
              }
              return paths;
            })().join(',');

          if ( helpDepStr ) {
            helpDepStr = ',' + helpDepStr;
          }

          if (metaObj) {
            try {
              if (metaObj.styles) {
                styleList = _.union(styleList, metaObj.styles);

                // In dev mode in the browser
                if ( require.isBrowser && ! config.isBuild ) {
                  head = document.head || document.getElementsByTagName('head')[0];
                  _(metaObj.styles).forEach(function (style) {
                    if ( !styleMap[style] ) {
                      linkElem = document.createElement('link');
                      linkElem.href = config.baseUrl + devStyleDirectory + style + '.css';
                      linkElem.media = 'all';
                      linkElem.rel = 'stylesheet';
                      linkElem.type = 'text/css';
                      head.appendChild(linkElem);
                      styleMap[style] = linkElem;
                    }
                  });
                }
                else if ( config.isBuild ) {
                  (function(){
                    var fs  = require.nodeRequire('fs');
                    var str = _(metaObj.styles).map(function (style) {
                      if (!styleMap[style]) {
                        styleMap[style] = true;
                        return '@import url('+style+'.css);\n';
                      }
                      return '';
                    }).join('\n');

                    // I write out my import statements to a file in order to help me build stuff.
                    // Then I use a tool to inline my import statements afterwards. (you can run r.js on it too)
                    fs.open(__dirname + buildStyleDirectory + buildCSSFileName, filecode, '0666', function( e, id ) {
                      fs.writeSync(id, str, null, encoding='utf8');
                      fs.close(id);
                    });
                    filecode = 'a';
                  })();
                }
              }
            }
            catch(e){
              console.log('error injecting styles');
            }
          }

          if ( ! config.isBuild && ! config.serverRender ) {
            debugOutputStart = '<!-- START - ' + name + ' -->';
            debugOutputEnd = '<!-- END - ' + name + ' -->';
            debugProperties = 't.meta = ' + meta + ';\n' +
                              't.helpers = ' + JSON.stringify(helps) + ';\n' +
                              't.deps = ' + JSON.stringify(deps) + ';\n' +
                              't.vars = ' + JSON.stringify(vars) + ';\n';
          }

          var mapping = disableI18n? false : _.extend( langMap, config.localeMapping );
          var configHbs = config.hbs || {};
          var options = _.extend(configHbs.compileOptions || {}, { originalKeyFallback: configHbs.originalKeyFallback });
          var prec = precompile( text, mapping, options);
          var tmplName = "'hbs!" + name + "',";

          if(depStr) depStr = ", '"+depStr+"'";

          var partialReferences = [];
          if(require.config.hbs._partials[name])
            partialReferences = require.config.hbs._partials[name].references;

          text = '/* START_TEMPLATE */\n' +
                 'define('+tmplName+"['hbs','hbs/handlebars'"+depStr+helpDepStr+'], function( hbs, Handlebars ){ \n' +
                   'var t = Handlebars.template(' + prec + ');\n' +
                   "Handlebars.registerPartial('" + name + "', t);\n";

          for(var i=0; i<partialReferences.length;i++)
            text += "Handlebars.registerPartial('" + partialReferences[i] + "', t);\n";

          text += debugProperties +
                   'return t;\n' +
                 '});\n' +
                 '/* END_TEMPLATE */\n';

          //Hold on to the transformed text if a build.
          if (config.isBuild) {
            buildMap[compiledName] = text;
          }

          //IE with conditional comments on cannot handle the
          //sourceURL trick, so skip it if enabled.
          /*@if (@_jscript) @else @*/
          if (!config.isBuild) {
            text += '\r\n//# sourceURL=' + path;
          }
          /*@end@*/

          if ( !config.isBuild ) {
            require( deps, function (){
              load.fromText(text);

              //Give result to load. Need to wait until the module
              //is fully parse, which will happen after this
              //execution.
              parentRequire([name], function (value) {
                load(value);
              });
            });
          }
          else {
            load.fromText(name, text);

            //Give result to load. Need to wait until the module
            //is fully parse, which will happen after this
            //execution.
            parentRequire([name], function (value) {
              load(value);
            });
          }

          if ( config.removeCombined && path ) {
            filesToRemove.push(path);
          }

        });
      }

      var path;
      var omitExtension = config.hbs && config.hbs.templateExtension === false;

      if (omitExtension) {
        path = parentRequire.toUrl(name);
      }
      else {
        path = parentRequire.toUrl(name +'.'+ (config.hbs && config.hbs.templateExtension ? config.hbs.templateExtension : templateExtension));
      }

      if (disableI18n){
        fetchAndRegister(false);
      }
      else {
        // Workaround until jam is able to pass config info or we move i18n to a separate module.
        // This logs a warning and disables i18n if there's an error loading the language file
        var langMapPath = (config.hbs && config.hbs.i18nDirectory ? config.hbs.i18nDirectory : i18nDirectory) + (config.locale || 'en_us') + '.json';
        try {
          fetchOrGetCached(parentRequire.toUrl(langMapPath), function (langMap) {
            fetchAndRegister(JSON.parse(langMap));
          });
        }
        catch(er) {
          // if there's no configuration at all, log a warning and disable i18n for this and subsequent templates
          if(!config.hbs) {
            console.warn('hbs: Error reading ' + langMapPath + ', disabling i18n. Ignore this if you\'re using jam, otherwise check your i18n configuration.\n');
            config.hbs = {i18n: false, helpers: true};
            fetchAndRegister(false);
          }
          else {
            throw er;
          }
        }
      }
          },

    onLayerEnd: function () {
      if (config.removeCombined && fs) {
        filesToRemove.forEach(function (path) {
          if (fs.existsSync(path)) {
            fs.unlinkSync(path);
          }
        });
      }
    }
  };
});
/* END_hbs_PLUGIN */
;

/* START_TEMPLATE */
define('hbs!utils/menu/top-row',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"small-12 columns\" style=\"padding-top: 15px\">\n  <div id=\"\" class=\"row left-body left-header\">\n    <div class=\"logo-header small-12 medium-4 columns\">\n      <button id=\"menu-toggle\" class=\"button medium radius secondary white_button\">\n        <i class=\"fa fa-lg fa-fw fa-bars\"></i>\n      </button>\n      <img src=\""
    + escapeExpression(((helper = (helper = helpers.logo || (depth0 != null ? depth0.logo : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"logo","hash":{},"data":data}) : helper)))
    + "\" style=\"height: 20px;vertical-align: middle;\"/>\n    </div>\n    <div class=\"small-12 medium-8 columns\">\n      <div class=\"user-zone-info right\">\n      </div>\n    </div>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('utils/menu/top-row', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!utils/menu/provision-top-row',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"right-content small-12 large-10 columns\">\n  <div class=\"row\">\n    <div class=\"small-12 medium-4 columns\">\n      <a href=\"#\" class=\"provision_image_header\" >\n        <img src=\""
    + escapeExpression(((helper = (helper = helpers.logo || (depth0 != null ? depth0.logo : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"logo","hash":{},"data":data}) : helper)))
    + "\" style=\"height:40px; vertical-align:top;\">\n      </a>\n    </div>\n    <div class=\"small-12 medium-8 columns\">\n      <div class=\"user-zone-info right\">\n      </div>\n    </div>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('utils/menu/provision-top-row', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/menu',['require','hbs!./menu/top-row','hbs!./menu/provision-top-row','sunstone-config'],function(require) {

  var TopRowHTML = require('hbs!./menu/top-row');
  var ProvisionTopRowHTML = require('hbs!./menu/provision-top-row');
  var Config = require('sunstone-config');

  return {
    'insert': _insert,
    'insertProvision': _insertProvision,
    'setup': _setup,
    'hide': _hide,
    'show': _show,
    'entryClick': _entryClick,
  };

  function _insert(){
    $('#top-row').html(TopRowHTML({logo: Config.logo}));

    _setup();
  }

  function _insertProvision(){
    $('#top-row').html(ProvisionTopRowHTML({logo: Config.provision.logo}));

    $("#menu-wrapper").remove();
  }

  function _setup(){
    $('#menu-toggle').on('click', function(){
      var hiding = $('.right-content').hasClass("large-10");

      if(!hiding){
        $('.right-content').toggleClass('large-10');
        $('.right-content').toggleClass('large-12');
      }

      $('#menu-wrapper').toggle(200, function(){
        if(hiding){
          $('.right-content').toggleClass('large-10');
          $('.right-content').toggleClass('large-12');
        }
      });
    });

    var prevWindowLarge = Foundation.utils.is_large_up();

    $(window).resize(function() {
      if(Foundation.utils.is_large_up()){
        $('#menu-wrapper').removeClass("menu-small");

        if(!prevWindowLarge){ // resizing from small to large, show menu
          _show();
        }

        prevWindowLarge = true;
      } else {
        $('#menu-wrapper').addClass("menu-small");

        if(prevWindowLarge){ // resizing from large to small, hide menu
          _hide();
        }

        prevWindowLarge = false;
      }
    });

    $(window).resize();
  }

  function _hide(){
    if($('#menu-wrapper').is(':visible')){
      $('#menu-toggle').click();
    }
  }

  function _show(){
    if(!$('#menu-wrapper').is(':visible')){
      $('#menu-toggle').click();
    }
  }

  function _entryClick(){
    if(!Foundation.utils.is_large_up()){
      _hide();
    }
  }
});

define('utils/dialogs/confirm/dialogId',['require'],function(require){
  return 'confirmDialog';
});
define('utils/dialogs/confirm-with-select/dialogId',['require'],function(require){
  return 'confirmWithSelectDialog';
});

/* START_TEMPLATE */
define('hbs!sunstone/tab',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, buffer = "  <div class=\"row header-row\">\n    <div class=\"large-12 columns\">\n      <h3 class=\"subheader header-title only-right-list\" style=\"border-bottom: 1px solid #efefef; padding-bottom: 10px;\">\n        <span class=\"header-resource\" style=\"border-bottom: 2px solid #0098c3; padding: 0px 50px 10px 0px;\">";
  stack1 = ((helper = (helper = helpers.listHeader || (depth0 != null ? depth0.listHeader : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"listHeader","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "</span>\n      </h3>\n      <h3 class=\"subheader header-title only-right-info\" style=\"border-bottom: 1px solid #efefef; padding-bottom: 10px;\" hidden>\n        <span class=\"header-resource\" style=\"border-bottom: 2px solid #0098c3; padding: 0px 50px 10px 0px;\">";
  stack1 = ((helper = (helper = helpers.infoHeader || (depth0 != null ? depth0.infoHeader : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"infoHeader","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n        &emsp;\n        <span class=\"resource-id\"></span>\n        &emsp;\n        <span class=\"resource-info-header\"></span>\n        &emsp;\n        <small class=\"resource-info-header-small\"></small>\n        </span>\n      </h3>\n      <h3 class=\"subheader header-title only-right-form\" style=\"border-bottom: 1px solid #efefef; padding-bottom: 10px;\" hidden>\n        <span class=\"right-form-title\" style=\"border-bottom: 2px solid #0098c3; padding: 0px 50px 10px 0px;\"></span>\n      </h3>\n    </div>\n  </div>\n";
},"3":function(depth0,helpers,partials,data) {
  var stack1, buffer = "  <div class=\"row actions_row\">\n    <div class=\"small-12 large-12 columns\">\n      <div class=\"action_blocks\"></div>\n      <div class=\"small-3 large-3 columns only-right-list\">\n";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.dataTable : depth0)) != null ? stack1.searchInputHTML : stack1), {"name":"if","hash":{},"fn":this.program(4, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "      </div>\n    </div>\n  </div>\n";
},"4":function(depth0,helpers,partials,data) {
  var stack1, lambda=this.lambda, buffer = "          ";
  stack1 = lambda(((stack1 = (depth0 != null ? depth0.dataTable : depth0)) != null ? stack1.searchInputHTML : stack1), depth0);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n";
},"6":function(depth0,helpers,partials,data) {
  var stack1, lambda=this.lambda, buffer = "    <div class=\"row\">\n      <div class=\"large-12 columns\">";
  stack1 = lambda(((stack1 = (depth0 != null ? depth0.dataTable : depth0)) != null ? stack1.dataTableHTML : stack1), depth0);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n    </div>\n";
},"8":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, buffer = "    ";
  stack1 = ((helper = (helper = helpers.content || (depth0 != null ? depth0.content : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"content","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n";
},"10":function(depth0,helpers,partials,data) {
  var stack1, buffer = "    <div class=\"row header-info\">\n      <div class=\"large-12 columns text-center totals-info\">\n        <h3 class=\"subheader\">\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.subheader : depth0), {"name":"if","hash":{},"fn":this.program(11, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "        </h3>\n      </div>\n    </div>\n";
},"11":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, buffer = "            ";
  stack1 = ((helper = (helper = helpers.subheader || (depth0 != null ? depth0.subheader : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"subheader","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.tabName || (depth0 != null ? depth0.tabName : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"tabName","hash":{},"data":data}) : helper)))
    + "\" class=\"tab\" style=\"display:none;\">\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.listHeader : depth0), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.buttons : depth0), {"name":"if","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "  <div class=\"right-list\">\n";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.dataTable : depth0)) != null ? stack1.dataTableHTML : stack1), {"name":"if","hash":{},"fn":this.program(6, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.content : depth0), {"name":"if","hash":{},"fn":this.program(8, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.subheader : depth0), {"name":"if","hash":{},"fn":this.program(10, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  </div>\n\n  <div class=\"right-info\" hidden></div>\n\n  <div class=\"large-12 small-12 right-form\" hidden>\n    <div class=\"loadingForm\">\n      <br>\n      <br>\n      <span class=\"fa-stack fa-5x\" style=\"color: #dfdfdf\"> <i class=\"fa fa-cloud fa-stack-2x\"/><i class=\"fa  fa-spinner fa-spin fa-stack-1x fa-inverse\"/>\n      </span>\n      <br>\n      <br>\n    </div>\n    <div class=\"contentForm\">\n    </div>\n  </div>\n</div>\n";
},"useData":true});
Handlebars.registerPartial('sunstone/tab', t);
return t;
});
/* END_TEMPLATE */
;
define('templates/helpers/tr',['require','hbs/handlebars','utils/locale'],function(require) {
  var Handlebars = require('hbs/handlebars');
  var Locale = require('utils/locale');

  var tr = function(context, options) {
    return Locale.tr(context);
  };

  Handlebars.registerHelper('tr', tr);
  return tr;
})
;

/* START_TEMPLATE */
define('hbs!sunstone/buttons',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "    <span id=\""
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "back_button\" class=\"only-right-info only-right-form\">\n      <a class=\"button small radius\" href=\"back\"> <i class=\"fa fa-arrow-left\"></i>\n        &emsp;&emsp; <i class=\"fa fa-list\"></i>\n      </a>\n    </span>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"text-center\">\n  <span class=\"left\">\n\n    <span id=\""
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "refresh_buttons\" class=\"only-right-info only-right-list\"></span>\n";
  stack1 = helpers.unless.call(depth0, (depth0 != null ? depth0.customContext : depth0), {"name":"unless","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "    <span id=\""
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "create_buttons\" class=\"only-right-list\"></span>\n  </span>\n\n  <span class=\"right\" style=\"margin-left: 20px\">\n    <button href='#' data-dropdown='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "user_buttons' class='only-right-info only-right-list top_button small  secondary button dropdown radius'>\n      <i class='fa fa-user'/></button>\n    <ul id='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "user_buttons' class='only-right-info only-right-list f-dropdown' data-dropdown-content></ul>\n\n    <button href='#' data-dropdown='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "vmsdelete_buttons' class='only-right-info only-right-list top_button small  button alert dropdown radius'>\n      <i class='fa fa-trash-o'/></button>\n    <ul id='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "vmsdelete_buttons' class='only-right-info only-right-list f-dropdown' data-dropdown-content></ul>\n\n    <span id='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "delete_buttons' class='only-right-info only-right-list'></span>\n  </span>\n\n  <span class=\"right\">\n    <span id=\""
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "vmsplay_buttons\" class='only-right-info only-right-list'></span>\n\n    <button href='#' data-dropdown='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "vmspause_buttons' class='only-right-info only-right-list top_button small  button secondary dropdown radius'>\n      <i class='fa fa-pause'/>\n    </button>\n    <ul id='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "vmspause_buttons' class='only-right-info only-right-list f-dropdown' data-dropdown-content></ul>\n\n    <button href='#' data-dropdown='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "vmsstop_buttons' class='only-right-info only-right-list top_button small  button secondary dropdown radius'>\n      <i class='fa fa-stop'/>\n    </button>\n    <ul id='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "vmsstop_buttons' class='only-right-info only-right-list f-dropdown' data-dropdown-content></ul>\n\n    <button href='#' data-dropdown='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "vmsrepeat_buttons' class='only-right-info only-right-list top_button small  button secondary dropdown radius'>\n      <i class='fa fa-repeat'/>\n    </button>\n    <ul id='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "vmsrepeat_buttons' class='only-right-info only-right-list f-dropdown' data-dropdown-content></ul>\n\n    <button href='#' data-dropdown='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "vmsplanification_buttons' class='only-right-info only-right-list top_button small  button secondary dropdown radius'>\n      <i class='fa fa-th-list'/>\n    </button>\n    <ul id='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "vmsplanification_buttons' class='only-right-info only-right-list f-dropdown' data-dropdown-content></ul>\n\n    <span id=\""
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "main_buttons\" class=\"only-right-info only-right-list\"></span>\n\n    <button href='#' data-dropdown='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "more_buttons' class='only-right-info only-right-list top_button small  button secondary dropdown radius'>\n      <i class='fa fa-ellipsis-v'/>\n    </button>\n    <ul id='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "more_buttons' class='only-right-info only-right-list f-dropdown' data-dropdown-content></ul>\n  </span>\n\n  <span id='"
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "form_buttons' class='only-right-form' style='display: none'>\n    <span id=\""
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "reset_button\" class=\"left\" style=\"margin-left: 10px;\">\n      <a class=\"button small secondary radius reset_button\" href=\"submit\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Reset", {"name":"tr","hash":{},"data":data})))
    + "</a>\n    </span>\n    <span id=\""
    + escapeExpression(((helper = (helper = helpers.customId || (depth0 != null ? depth0.customId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customId","hash":{},"data":data}) : helper)))
    + "submit_button\" class=\"left\" style=\"margin-left: 10px;\">\n      <a class=\"button small success radius submit_button\" href=\"submit\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Create", {"name":"tr","hash":{},"data":data})))
    + "</a>\n    </span>\n    <dl class=\"tabs right wizard_tabs\" data-tab style=\"margin-left: 10px;\">\n      <dd id=\"wizard_mode\" class=\"active\">\n        <a style=\"padding: 0.3rem 1rem;\" href=\"#wizardForms\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Wizard", {"name":"tr","hash":{},"data":data})))
    + "</a>\n      </dd>\n      <dd id=\"advanced_mode\">\n        <a style=\"padding: 0.3rem 1rem;\" id=\"advanced_mode_a\" href=\"#advancedForms\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced", {"name":"tr","hash":{},"data":data})))
    + "</a>\n      </dd>\n    </dl>\n  </span>\n</div>\n\"";
},"useData":true});
Handlebars.registerPartial('sunstone/buttons', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!sunstone/panels',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "    <dd ";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.active : depth0), {"name":"if","hash":{},"fn":this.program(2, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += ">\n      <a href=\"#"
    + escapeExpression(((helper = (helper = helpers.panelName || (depth0 != null ? depth0.panelName : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"panelName","hash":{},"data":data}) : helper)))
    + "\">\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.icon : depth0), {"name":"if","hash":{},"fn":this.program(4, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "        "
    + escapeExpression(((helper = (helper = helpers.title || (depth0 != null ? depth0.title : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"title","hash":{},"data":data}) : helper)))
    + "\n      </a>\n    </dd>\n";
},"2":function(depth0,helpers,partials,data) {
  return "class=\"active\"";
  },"4":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "          <i class=\"fa "
    + escapeExpression(((helper = (helper = helpers.icon || (depth0 != null ? depth0.icon : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"icon","hash":{},"data":data}) : helper)))
    + "\"></i>\n          <br>\n";
},"6":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "    <div id=\""
    + escapeExpression(((helper = (helper = helpers.panelName || (depth0 != null ? depth0.panelName : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"panelName","hash":{},"data":data}) : helper)))
    + "\" class=\"content ";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.active : depth0), {"name":"if","hash":{},"fn":this.program(7, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "\">\n      ";
  stack1 = ((helper = (helper = helpers.html || (depth0 != null ? depth0.html : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"html","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </div>\n";
},"7":function(depth0,helpers,partials,data) {
  return "active";
  },"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.containerId || (depth0 != null ? depth0.containerId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"containerId","hash":{},"data":data}) : helper)))
    + "\" class=\"bordered-tabs\">\n  <dl class=\"tabs right-info-tabs text-center\" data-tab>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.panels : depth0), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "  </dl>\n  <div class='tabs-content'>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.panels : depth0), {"name":"each","hash":{},"fn":this.program(6, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('sunstone/panels', t);
return t;
});
/* END_TEMPLATE */
;
define('sunstone',['require','jquery','foundation.reveal','foundation.tab','foundation.dropdown','sunstone-config','utils/locale','utils/notifier','utils/menu','utils/dialogs/confirm/dialogId','utils/dialogs/confirm-with-select/dialogId','hbs!sunstone/tab','hbs!./sunstone/buttons','hbs!./sunstone/panels'],function(require) {
  require('jquery');
  require('foundation.reveal');
  require('foundation.tab');
  require('foundation.dropdown');

  var Config = require('sunstone-config');
  var Locale = require('utils/locale');
  var Notifier = require('utils/notifier');
  var Menu = require('utils/menu');

  var TOP_INTERVAL = 10000; //ms
  var CONFIRM_DIALOG_ID = require('utils/dialogs/confirm/dialogId');
  var CONFIRM_WITH_SELECT_DIALOG_ID = require('utils/dialogs/confirm-with-select/dialogId');

  var SunstoneCfg = {
    "actions" : {},
    "dialogs" : {},
    "dialogInstances" : {},
    "tabs" : {}
  };

  var _addMainTab = function(tabObj) {
    var _tabId = tabObj.tabId;
    if (Config.isTabEnabled(_tabId)) {
      SunstoneCfg["tabs"][_tabId] = tabObj;

      var actions = tabObj.actions;
      if (actions) {
        _addActions(actions)
      }

      var panels = tabObj.panels;
      if (panels) {
        _addPanels(_tabId, panels)
      }

      var dialogs = tabObj.dialogs;
      if (dialogs) {
        _addDialogs(dialogs)
      }

      var formPanels = tabObj.formPanels;
      if (formPanels) {
        _addFormPanels(_tabId, formPanels)
      }
    }
  }

  var _addActions = function(actions) {
    $.each(actions, function(actionName, action) {
      SunstoneCfg["actions"][actionName] = action;
    })
    return false;
  }

  var _addDialogs = function(dialogs) {
    $.each(dialogs, function(index, dialog) {
      SunstoneCfg['dialogs'][dialog.DIALOG_ID] = dialog
    })
    return false;
  }

  var _addPanels = function(tabId, panels) {
    var indexedPanels = {}
    $.each(panels, function(index, panel) {
      indexedPanels[panel.PANEL_ID] = panel
    })
    SunstoneCfg["tabs"][tabId]['panels'] = indexedPanels;
    return false;
  }

  var _addFormPanels = function(tabId, formPanels) {
    var indexedFormPanels = {}
    $.each(formPanels, function(index, formPanel) {
      indexedFormPanels[formPanel.FORM_PANEL_ID] = formPanel
    })
    SunstoneCfg["tabs"][tabId]['formPanels'] = indexedFormPanels;
    SunstoneCfg["tabs"][tabId]['formPanelInstances'] = {};
    return false;
  }

  //Inserts all main tabs in the DOM
  var _insertTabs = function() {
    for (tabName in SunstoneCfg["tabs"]) {
      _insertTab(tabName);
      _insertButtonsInTab(tabName);
      _setupDataTable(tabName);

      // TODO Add openenbula actions
      /*if (config['view']['autorefresh']) {
        var tabContext = $("#" + tabName);
        var refreshButton = $(".fa-refresh", $(".action_blocks", tabContext).first());
        setInterval(function() {
          if (Sunstone.rightListVisible(tabContext)) {
            refreshButton.click();
          }
        }, TOP_INTERVAL);
      }*/
    }

    _setupTabs();
  }

  var _setupDataTable = function(tabName) {
    if (SunstoneCfg['tabs'][tabName].dataTable) {
      SunstoneCfg['tabs'][tabName].dataTable.initialize();
    }
  }

  //Inserts a main tab in the DOM. This is done by
  //adding the content to the proper div and by adding a list item
  //link to the navigation menu
  var _insertTab = function(tabName) {
    var tabInfo = SunstoneCfg['tabs'][tabName];
    var condition = tabInfo['condition'];
    var tabClass = tabInfo['tabClass'] ? tabInfo['tabClass'] : 'topTab';
    var parent = tabInfo['parentTab'] ? tabInfo['parentTab'] : '';

    //skip this tab if we do not meet the condition
    if (condition && !condition()) {return;}

    if (tabInfo.no_content === true) {
      tabClass += " tab_with_no_content"
    } else {
      tabInfo['tabName'] = tabName;
      var TabTemplate = require('hbs!sunstone/tab')
      $('div.right-content').append(TabTemplate(tabInfo));
    }

    var liItem = '<li id="li_' + tabName + '" class="' + tabClass + ' ' + parent + '"><a href="#">' + tabInfo.title + '</a></li>';

    $('div#menu ul#navigation').append(liItem);

    //if this is a submenu...
    if (parent.length) {
      var children = $('div#menu ul#navigation #li_' + parent);
      //if there are other submenus, insert after last of them
      if (children.length) {
        $('div#menu li#li_' + tabName).hide();//hide by default
        $('div#menu li#li_' + parent + ' span').css("display", "inline-block");
      }
    };

    if (tabInfo.forms) {
      $.each(tabInfo.forms, function(key, value) {
        Sunstone.addFormPanel(tabName, key, value)
      })
    }

    if (tabInfo.setup) {
      var context = $('div#' + tabName, $('div.right-content'));

      tabInfo.setup(context);
    }
  };

  //If we have defined a block of action buttons in a tab,
  //this function takes care of inserting them in the DOM.
  var _insertButtonsInTab = function(tabName, panelName, panelButtons, customContext) {
    var buttons = panelButtons ? panelButtons : SunstoneCfg["tabs"][tabName]["buttons"];
    var buttonCode = "";
    var condition = null;

    var context, customId;
    if (customContext) {
      customId = customContext.attr("id");
      context = customContext;
    } else {
      customId = tabName;
      context = $('div#' + tabName, $('div.right-content'));
    }

    var actionBlock = $('div.action_blocks', context)

    if (actionBlock.length) {

      var ButtonsTemplate = require('hbs!./sunstone/buttons')
      var buttonsRow = $(ButtonsTemplate({customId: customId, customContext: customContext}));

      //for every button defined for this tab...
      for (buttonName in buttons) {
        buttonCode = "";
        button = buttons[buttonName];

        //if we meet the condition we proceed. Otherwise we skip it.
        if (Config.isTabActionEnabled(tabName, buttonName, panelName) == false) {
          continue;
        }

        var type = button.type + '_button';
        var strClass = [type]
        switch (button.type) {
        case "select":
          break;
        case "image":
          strClass.push("action_button")
          break;
        case "create_dialog":
          strClass.push("action_button")
          strClass.push("top_button")
          break;
        default:
          strClass.push("top_button")
        }

        if (button.alwaysActive) {
          strClass.push("alwaysActive");
        }

        if (button.custom_classes) {
          strClass.push(button.custom_classes);
        }

        var buttonContext;
        var text;
        switch (button.layout) {
        case "create":
          buttonContext = $("#" + customId + "create_buttons", buttonsRow);
          icon = button.icon ? button.icon : '<i class="fa fa-plus"/>';
          text = button.text ? icon + ' ' + button.text : icon;
          strClass.push("success", "button", "small", "radius");
          buttonCode = '<button class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</button>';
          break;
        case "refresh":
          buttonContext = $("#" + customId + "refresh_buttons", buttonsRow);
          icon = button.icon ? button.icon : '<i class="fa fa-refresh fa-lg"/>';
          text = button.text ? icon + ' ' + button.text : icon;
          strClass.push("white_button", "refresh", "secondary", "button", "small", "radius");
          buttonCode = '<button class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</button>';
          break;
        case "top":
          buttonContext = $("#" + customId + "refresh_buttons", buttonsRow);
          text = '<span class="fa-stack">' +
              '<i class="fa fa-refresh fa-stack-2x" style="color: #dfdfdf"></i>' +
              '<i class="fa fa-play fa-stack-1x"></i>' +
            '</span>';
          strClass.push("white_button", "toggle_top_button", "only-right-list", "secondary", "button", "small", "radius");
          buttonCode = '<a class="' + strClass.join(' ') + '" style="padding-left:0px; margin-right: 20px">' + text + '</a>';
          break;
        case "main":
          buttonContext = $("#" + customId + "main_buttons", buttonsRow);
          text = button.text;
          strClass.push("secondary", "button", "small", "radius");
          buttonCode = '<button class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</button>';
          break;
        case "vmsplay_buttons":
          buttonContext = $("#" + customId + "vmsplay_buttons", buttonsRow);
          text = button.text;
          strClass.push("secondary", "button", "small", "radius");
          buttonCode = '<button class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</button>';
          break;
        case "vmspause_buttons":
          buttonContext = $("#" + customId + "vmspause_buttons", buttonsRow);
          text = button.text;
          buttonCode = '<li><a class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</a></li>';
          break;
        case "vmsstop_buttons":
          buttonContext = $("#" + customId + "vmsstop_buttons", buttonsRow);
          text = button.text;
          buttonCode = '<li><a class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</a></li>';
          break;
        case "vmsrepeat_buttons":
          buttonContext = $("#" + customId + "vmsrepeat_buttons", buttonsRow);
          text = button.text;
          buttonCode = '<li><a class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</a></li>';
          break;
        case "vmsdelete_buttons":
          buttonContext = $("#" + customId + "vmsdelete_buttons", buttonsRow);
          text = button.text;
          buttonCode = '<li><a class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</a></li>';
          break;
        case "vmsplanification_buttons":
          buttonContext = $("#" + customId + "vmsplanification_buttons", buttonsRow);
          text = button.text;
          buttonCode = '<li><a class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</a></li>';
          break;
        case "more_select":
          buttonContext = $("#" + customId + "more_buttons", buttonsRow);
          text = button.text;
          buttonCode = '<li><a class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</a></li>';
          break;
        case "user_select":
          buttonContext = $("#" + customId + "user_buttons", buttonsRow);
          text = button.text;
          buttonCode = '<li><a class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</a></li>';
          break;
        case "del":
          buttonContext = $("#" + customId + "delete_buttons", buttonsRow);
          text = '<i class=" fa fa-trash-o"/> ';
          strClass.push("alert", "button", "small", "radius");
          buttonCode = '<button class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</button>';
          break;
        default:
          buttonContext = $("#" + customId + "main_buttons", buttonsRow);
          text = button.text;
          strClass.push("secondary", "button", "small", "radius");
          buttonCode = '<button class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</button>';
        }

        buttonContext.append(buttonCode);
      }//for each button in tab
      //$('.top_button',actionBlock).button();
      //$('.top_button',actionBlock).addClass("secondary small button")

      actionBlock.append(buttonsRow);

      if ($("#" + customId + "more_buttons li", actionBlock).length == 0) {
        $("button[data-dropdown=" + customId + "more_buttons]", actionBlock).remove()
      }

      if ($("#" + customId + "user_buttons li", actionBlock).length == 0) {
        $("button[data-dropdown=" + customId + "user_buttons]", actionBlock).remove()
      }

      if ($("#" + customId + "vmsplanification_buttons li", actionBlock).length == 0) {
        $("button[data-dropdown=" + customId + "vmsplanification_buttons]", actionBlock).remove()
      }

      if ($("#" + customId + "vmsdelete_buttons li", actionBlock).length == 0) {
        $("button[data-dropdown=" + customId + "vmsdelete_buttons]", actionBlock).remove()
      }

      if ($("#" + customId + "vmsstop_buttons li", actionBlock).length == 0) {
        $("button[data-dropdown=" + customId + "vmsstop_buttons]", actionBlock).remove()
      }

      if ($("#" + customId + "vmspause_buttons li", actionBlock).length == 0) {
        $("button[data-dropdown=" + customId + "vmspause_buttons]", actionBlock).remove()
      }

      if ($("#" + customId + "vmsrepeat_buttons li", actionBlock).length == 0) {
        $("button[data-dropdown=" + customId + "vmsrepeat_buttons]", actionBlock).remove()
      }

      if ($("#" + customId + "user_buttons li", actionBlock).length == 0) {
        $("button[data-dropdown=" + customId + "user_buttons]", actionBlock).remove()
      }
      //actionBlock.foundationButtons();
      $('.top_button, .list_button', actionBlock).prop('disabled', false);
      $('.top_button, .list_button', actionBlock).prop('disabled', true);
      $('.create_dialog_button', actionBlock).prop('disabled', false).removeAttr('disabled');
      $('.alwaysActive', actionBlock).prop('disabled', false).removeAttr('disabled');

      $('#' + customId + 'reset_button', actionBlock).on("click", function() {
        _resetFormPanel(tabName);
        return false;
      })

      $('#' + customId + 'submit_button', actionBlock).on("click", function() {
        _submitFormPanel(tabName);
        return false;
      })

      $(document).foundation();
    }//if tab exists
  }

  var _setupButtons = function() {
    //Listen for .action_buttons
    //An action buttons runs a predefined action. If it has type
    //"multiple" it runs that action on the elements of a datatable.
    $(document).on("click", '.action_button', function() {
      var error = 0;
      var value = $(this).val()
      if ($.isEmptyObject(value)) {
        value = $(this).attr('href');
      }

      /*if (!$(this).hasClass("refresh")) {
        $(document).foundation('dropdown', 'closeall');
      }*/

      var action = SunstoneCfg["actions"][value];
      if (!action) {
        Notifier.notifyError("Action " + value + " not defined.");
        return false;
      };
      switch (action.type){
      case "multiple": //find the datatable
        var context = $(this).parents(".tab");
        var nodes = action.elements();
        error = _runAction(value, nodes);
        break;
      default:
        error = _runAction(value);
      }

      if (!error && !$(this).hasClass("refresh")) {
        //proceed to close confirm dialog in
        //case it was open
        _getDialogInstance(CONFIRM_DIALOG_ID).hide();
      };

      return false;
    });

    //Listen .confirm_buttons. These buttons show a confirmation dialog
    //before running the action.
    $(document).on("click", '.confirm_button', function() {
      var dialogInstance = _getDialogInstance(CONFIRM_DIALOG_ID)
      $('#' + CONFIRM_DIALOG_ID).data('buttonAction', $(this).attr('href'));
      $('#' + CONFIRM_DIALOG_ID).data('buttonTab', $(this).parents('.tab').attr('id'));
      dialogInstance.show();
      return false;
    });

    //Listen .confirm_buttons. These buttons show a confirmation dialog
    //with a select box before running the action.
    $(document).on("click", '.confirm_with_select_button', function() {
      var dialogInstance = _getDialogInstance(CONFIRM_WITH_SELECT_DIALOG_ID);
      $('#' + CONFIRM_WITH_SELECT_DIALOG_ID).data('buttonAction', $(this).attr('href'));
      $('#' + CONFIRM_WITH_SELECT_DIALOG_ID).data('buttonTab', $(this).parents('.tab').attr('id'));
      dialogInstance.show();
      return false;
    });

    $(document).foundation('dropdown', 'reflow');

    // Button to return to the list view from the detailed view
    $(document).on("click", "a[href='back']", function(e) {
      $(".navigation-active-li a", $("#navigation")).click();
      e.preventDefault();
    });
  }

  var _setupTabs = function() {
    var topTabs = $(".left-content ul li.topTab");
    var subTabs = $(".left-content ul li.subTab");

    subTabs.on("click", function() {
      if ($(this).hasClass('topTab')) {
        return false;
      } else {
        var tabName = $(this).attr('id').substring(3);
        _showTab(tabName);
        return false;
      }
    });

    topTabs.on("click", function(e) {
      var tabName = $(this).attr('id').substring(3);

      if ($(this).hasClass("tab_with_no_content")) {
        //Subtabs have a class with the name of  this tab
        var subtabs = $('div#menu li.' + tabName);
        subtabs.fadeToggle('fast');
        return false;
      } else {
        _showTab(tabName);
        return false;
      }
    });

    _setupButtons();
  };

  var _showRighList = function(tabName) {
    var tab = $('#' + tabName);
    $(".tab").hide();
    tab.show();

    $(".right-info", tab).hide();
    $(".right-form", tab).hide();
    $(".right-list", tab).show();
    $(".only-right-info", tab).hide();
    $(".only-right-form", tab).hide();
    $(".only-right-list", tab).show();
  };

  var _showRighInfo = function(tabName) {
    var tab = $('#' + tabName);
    $(".tab").hide();
    tab.show();

    $(".right-list", tab).hide();
    $(".right-form", tab).hide();
    $(".right-info", tab).show();
    $(".only-right-list", tab).hide();
    $(".only-right-form", tab).hide();
    $(".only-right-info", tab).show();
  }

  var _showTab = function(tabName) {
    if (!SunstoneCfg['tabs'][tabName]) {
      return false;
    }

    // Hide the menu in small windows
    Menu.entryClick();

    // TODO check if necessary
    // last_selected_row = null;

    if (tabName.indexOf('#') == 0) {
      tabName = tabName.substring(1);
    }

    //clean selected menu
    $("#navigation li").removeClass("navigation-active-li");
    $("#navigation li#li_" + tabName).addClass("navigation-active-li");

    var tab = $('#' + tabName);
    //show tab
    _showRighList(tabName);

    var dataTable = SunstoneCfg['tabs'][tabName]['dataTable'];
    if (dataTable) {
      dataTable.recountCheckboxes();
    }

    var res = SunstoneCfg['tabs'][tabName]['resource']
    if (res) {
      Sunstone.runAction(res + ".refresh");
    }
  }

  var _showElement = function(tabName, infoAction, elementId) {
    _showTab(tabName);

    var context = $('#' + tabName);

    $(".resource-id", context).html(elementId);
    $(".resource-info-header", context).text("");

    var loading = '<div style="margin-top: 20px; text-align: center; width: 100%"><img src="images/pbar.gif" alt="loading..." /></div>';
    $(".right-info", context).html(loading);
    _showRighInfo(tabName);

    Sunstone.runAction(infoAction, elementId);
    //enable action buttons
    $('.top_button, .list_button', context).attr('disabled', false);
  }

  var _insertPanels = function(tabName, info, contextTabId, context) {
    var context = context || $(".right-info", $("#" + tabName));
    var containerId = tabName + '-panels';
    var activaTab = $("dd.active a", $("#" + containerId));
    if (activaTab) {
      var activaTabHref = activaTab.attr('href');
    }

    var isRefresh = (activaTabHref != undefined);
    var prevPanelInstances = SunstoneCfg['tabs'][tabName]["panelInstances"];
    var prevPanelStates = {};

    if(isRefresh && prevPanelInstances != undefined){
      $.each(prevPanelInstances, function(panelName, panel) {
        if(panel.getState){
          prevPanelStates[panelName] = panel.getState(context);
        }
      });
    }

    var panels = SunstoneCfg['tabs'][tabName].panels;
    var active = false;
    var templatePanelsParams = []
    SunstoneCfg['tabs'][tabName]["panelInstances"] = {};

    $.each(panels, function(panelName, Panel) {
      if (Config.isTabPanelEnabled((contextTabId||tabName), panelName)) {
        if (activaTabHref) {
          if (activaTabHref == "#" + panelName) {
            active = true;
          }
        } else {
          activaTabHref = "#" + panelName
          active = true;
        }

        try {
          var panelInstance = new Panel(info, contextTabId);
          SunstoneCfg['tabs'][tabName]["panelInstances"][panelName] = panelInstance;
          templatePanelsParams.push({
            'panelName': panelName,
            'icon': panelInstance.icon,
            'title': panelInstance.title,
            'html': panelInstance.html(),
            'active': active
          })
        } catch (err) {
          console.log(err);
        }

        active = false;
      }
    });

    var TemplatePanels = require('hbs!./sunstone/panels');
    var html = TemplatePanels({
      'containerId': containerId,
      'panels': templatePanelsParams
    })

    context.html(html);

    $.each(SunstoneCfg['tabs'][tabName]["panelInstances"], function(panelName, panel) {
      panel.setup(context);

      if(isRefresh && prevPanelStates[panelName] && panel.setState){
        panel.setState( prevPanelStates[panelName], context );
      }

      if (panel.onShow) {
        context.off('click', '[href="#' + panel.panelId + '"]');
        context.on('click', '[href="#' + panel.panelId + '"]', function(){
          panel.onShow(context);
        });
      }
    });

    context.foundation('tab', 'reflow');
    $('[href=' + activaTabHref + ']', context).trigger("click");
  }

  //Runs a predefined action. Wraps the calls to opennebula.js and
  //can be use to run action depending on conditions and notify them
  //if desired. Returns 1 if some problem has been detected: i.e
  //the condition to run the action is not met, the action is not found
  var _runAction = function(action, dataArg, extraParam) {
    var actions = SunstoneCfg["actions"];
    if (!actions[action]) {
      Notifier.notifyError("Action " + action + " not defined");
      return 1;
    }

    var actionCfg = actions[action];
    var notify = actionCfg.notify;

    var condition = actionCfg["condition"];

    //Is the condition to run the action met?
    //Should we inform if it is not met?
    if (condition && !condition()) {
      if (notify) {
        Notifier.notifyError("This action cannot be run");
      }
      return 1;
    }

    var call = actionCfg["call"];
    var callback = actionCfg["callback"];
    var err = actionCfg["error"];

    switch (actionCfg.type){
    case "create":
    case "register":
      call({data:dataArg, success: callback, error:err});
      break;
    case "single":
      if (extraParam) {
        call({
          data:{
            id:dataArg,
            extra_param: extraParam
          },
          success: callback, error:err
        });
      } else {
        call({data:{id:dataArg}, success: callback, error:err});
      };
      break;
    case "list":
      call({success: callback, error:err, options:dataArg});
      break;
    case "monitor_global":
      call({
          timeout: true,
          success: callback,
          error:err,
          data: {monitor: dataArg}});
      break;
    case "monitor":
    case "monitor_single":
      call({
          timeout: true,
          success: callback,
          error:err,
          data: {id:dataArg, monitor: extraParam}});
      break;
    case "multiple":
      $.each(dataArg, function() {
        if (extraParam) {
          call({
                        data:{
                          id:this,
                          extra_param:extraParam
                        },
                        success: callback,
                        error: err});
        } else {
          call({
              data:{id:this},
              success: callback,
              error:err});
        }
      });
      break;
    default:
      if (dataArg && extraParam) {
        call(dataArg, extraParam);
      } else if (dataArg) {
        call(dataArg);
      } else {
        call();
      }
    }

    if (notify) {
      Notifier.notifySubmit(action, dataArg, extraParam);
    }

    return 0;
  }

  function _showFormPanel(tabId, formPanelId, action, onShow2) {
    var context = $("#" + tabId);
    _popFormPanelLoading(tabId);
    _enableFormPanelSubmit(tabId);
    _showFormPanelSubmit(tabId);

    setTimeout(function() {
      var tab = SunstoneCfg["tabs"][tabId];
      var formPanelInstance = tab["formPanelInstances"][formPanelId];

      var formContext = $("#" + tabId+" div[form-panel-id="+formPanelId+"]");

      if (!formPanelInstance) {
        formContext =
        $('<div class="tabs-content tabs-contentForm" form-panel-id="'+formPanelId+'">\
          <div class="content active" id="wizardForms"></div>\
          <div class="content" id="advancedForms"></div>\
        </div>').appendTo( $(".contentForm", context) );

        // Create panelInstance, insert in the DOM and setup
        var formPanel = tab["formPanels"][formPanelId];
        if (!formPanel) {
          console.log("Form Panel not defined");
          return false;
        } // Panel not defined

        formPanelInstance = new formPanel();
        formPanelInstance.setAction(formContext, action);
        tab["formPanelInstances"][formPanelId] = formPanelInstance;
        formPanelInstance.insert(formContext);
      }

      formPanelInstance.setAction(formContext, action);
      tab["activeFormPanel"] = formPanelInstance;

      // Hide wizard/advanced selector if advanced not defined
      if (formPanelInstance.htmlAdvanced) {
        $(".wizard_tabs", context).show();
      } else {
        $(".wizard_tabs", context).hide();
      }

      // Hide reset button if not defined
      if (formPanelInstance.resetButton()) {
        $(".reset_button", context).show();
      } else {
        $(".reset_button", context).hide();
      }

      _hideFormPanelLoading(tabId);

      formPanelInstance.onShow(formContext);
      if (onShow2) {
        onShow2(formPanelInstance, formContext);
      }
    }, 13)
  }

  var _submitFormPanel = function(tabId) {
    var context = $("#" + tabId);
    //_popFormPanelLoading(tabId);
    // Workaround until Foundation.abide support hidden forms
    
    var context = $("#" + tabId);
    $(".right-form-title", context).text(Locale.tr("Submitting..."));
    $(".submit_button", context).text(Locale.tr("Submitting..."));

    setTimeout(function() {
      var formPanelInstance = SunstoneCfg["tabs"][tabId].activeFormPanel

      if ($("#wizardForms.active", context).length > 0) {
        $('#' + formPanelInstance.formPanelId + 'Wizard').submit();
      } else if ($("#advancedForms.active", context).length > 0) {
        $('#' + formPanelInstance.formPanelId + 'Advanced').submit();
      }
    }, 13)
  }

  var _resetFormPanel = function(tabId, formPanelId) {
    _popFormPanelLoading(tabId);
    _enableFormPanelSubmit(tabId);

    setTimeout(function() {
      var formPanelInstance;
      if (formPanelId) {
        formPanelInstance = SunstoneCfg["tabs"][tabId]['formPanelInstances'][formPanelId];
      } else {
        formPanelInstance = SunstoneCfg["tabs"][tabId].activeFormPanel;
      }

      if (formPanelInstance) {
        var context = $("#" + tabId+" div[form-panel-id="+formPanelInstance.formPanelId+"]");

        formPanelId = formPanelInstance.formPanelId;

        formPanelInstance.reset(context);
      }

      _hideFormPanelLoading(tabId);

      if (formPanelInstance) {
        formPanelInstance.onShow(context);
      }
    }, 13)
  }

  function _hideFormPanelLoading(tabId) {
    var context = $("#" + tabId);
    //$(".right-form", context).html(content);

    $(".loadingForm", context).hide();
    $(".tabs-contentForm", context).hide();

    var formPanelInstance = SunstoneCfg["tabs"][tabId].activeFormPanel;
    if (formPanelInstance) {
      // Set title and button strings
      $(".right-form-title", context).text(formPanelInstance.title());
      $(".submit_button", context).text(formPanelInstance.buttonText());

      $("div[form-panel-id="+formPanelInstance.formPanelId+"]", context).show();
    }

  }

  function _hideFormPanel(tabId) {
    var context = $("#" + tabId);
    $("a[href=back]", context).trigger("click");
  }

  function _popFormPanelLoading(tabId) {
    var context = $("#" + tabId);
    $(".right-list", context).hide();
    $(".right-info", context).hide();
    $(".right-form", context).show();
    $(".only-right-list", context).hide();
    $(".only-right-info", context).hide();
    $(".only-right-form", context).show();

    $(".right-form-title", context).text(Locale.tr("Loading..."));
    $(".submit_button", context).text(Locale.tr("Loading..."));

    $(".tabs-contentForm", context).hide();
    $(".loadingForm", context).show();
  }

  function _disableFormPanelSubmit(tabId) {
    var context = $("#" + tabId);
    $(".submit_button", context).
        attr("disabled", "disabled").
        on("click.disable", function(e) { return false; });
  }

  function _enableFormPanelSubmit(tabId) {
    var context = $("#" + tabId);
    $(".submit_button", context).
        removeAttr("disabled").
        off("click.disable");
  }

  function _hideFormPanelSubmit(tabId) {
    var context = $("#" + tabId);
    $(".submit_button", context).hide();
  }

  function _showFormPanelSubmit(tabId) {
    var context = $("#" + tabId);
    $(".submit_button", context).show();
  }

  var _getButton = function(tadId, buttonName) {
    var button = null;
    var buttons = SunstoneCfg["tabs"][tadId]["buttons"];
    button = buttons[buttonName];
    //not found, is it in the list then?
    if (!button && buttons["action_list"]) {
      button = buttons["action_list"]["actions"][buttonName];
    }
    return button;
  }

  var _rightInfoVisible = function(context) {
    return $(".right-info", context).is(':visible');
  }

  var _rightListVisible = function(context) {
    return $(".right-list", context).is(':visible');
  }

  var _rightInfoResourceId = function(context) {
    return $(".resource-id", context).text();
  }

  var _getAction = function(actionId) {
    return SunstoneCfg["actions"][actionId];
  }

  var _getDataTable = function(tabName) {
    if (SunstoneCfg['tabs'][tabName]) {
      return SunstoneCfg['tabs'][tabName].dataTable;
    }
  }

  var _getDialogInstance = function(dialogId) {
    var dialogInstance = SunstoneCfg['dialogInstances'][dialogId];
    if (dialogInstance == undefined) {
      var Dialog = SunstoneCfg['dialogs'][dialogId]
      var dialogInstance = new Dialog();
      dialogInstance.insert();
      SunstoneCfg['dialogInstances'][dialogId] = dialogInstance;
    }

    return dialogInstance;
  }

  var Sunstone = {
    "addMainTab": _addMainTab,
    "addDialogs": _addDialogs,

    "insertTabs": _insertTabs,
    "insertPanels": _insertPanels,

    'showTab': _showTab,
    "showElement" : _showElement,

    "showFormPanel": _showFormPanel,
    "resetFormPanel": _resetFormPanel,
    "hideFormPanel": _hideFormPanel,
    "hideFormPanelLoading": _hideFormPanelLoading,
    "disableFormPanelSubmit": _disableFormPanelSubmit,
    "enableFormPanelSubmit": _enableFormPanelSubmit,
    "hideFormPanelSubmit": _hideFormPanelSubmit,
    "showFormPanelSubmit": _showFormPanelSubmit,


    "rightInfoVisible": _rightInfoVisible,
    "rightListVisible": _rightListVisible,
    "rightInfoResourceId": _rightInfoResourceId,

    "runAction" : _runAction,
    "getAction": _getAction,
    "getButton": _getButton,
    "getDataTable": _getDataTable,
    "getDialog": _getDialogInstance,

    "insertButtonsInTab": _insertButtonsInTab,

    "TOP_INTERVAL": TOP_INTERVAL
  }

  return Sunstone;
});

define('opennebula/helper',['require'],function(require) {
  var Helper = {
    "action": function(action, params) {
      obj = {
        "action": {
          "perform": action
        }
      }
      if (params) {
        obj.action.params = params;
      }
      return obj;
    },

    "request": function(resource, method, data) {
      var r = {
        "request": {
          "resource"  : resource,
          "method"    : method
        }
      }
      if (data) {
        if (typeof(data) != "array") {
          data = [data];
        }
        r.request.data = data;
      }
      return r;
    },

    "pool": function(resource, response) {
      var pool_name = resource + "_POOL";
      var type = resource;
      var pool;

      if (typeof(pool_name) == "undefined") {
        return Error('Incorrect Pool');
      }

      var p_pool = [];

      if (response[pool_name]) {
        pool = response[pool_name][type];
      } else {
        pool = null;
      }

      if (pool == null) {
        return p_pool;
      } else if (pool.length) {
        for (i = 0; i < pool.length; i++) {
          p_pool[i] = {};
          p_pool[i][type] = pool[i];
        }
        return (p_pool);
      } else {
        p_pool[0] = {};
        p_pool[0][type] = pool;
        return (p_pool);
      }
    },

    "pool_hash_processing": function(pool_name, resource_name, response) {
      var pool;

      if (typeof(pool_name) == "undefined") {
        return Error('Incorrect Pool');
      }

      var p_pool = {};

      if (response[pool_name]) {
        pool = response[pool_name][resource_name];
      } else {
        pool = null;
      }

      if (pool == null) {
        return p_pool;
      } else if (pool.length) {
        for (i = 0; i < pool.length; i++) {
          var res = {};
          res[resource_name] = pool[i];

          p_pool[res[resource_name]['ID']] = res;
        }
        return (p_pool);
      } else {
        var res = {};
        res[resource_name] = pool;

        p_pool[res[resource_name]['ID']] = res;

        return (p_pool);
      }
    },

    "pool_name_processing": function(pool_name, resource_name, response) {
      var pool;

      if (typeof(pool_name) == "undefined") {
        return Error('Incorrect Pool');
      }

      var p_pool = {};

      if (response[pool_name]) {
        pool = response[pool_name][resource_name];
      } else {
        pool = null;
      }

      if (pool == null) {
        return p_pool;
      } else if (pool.length) {
        for (i = 0; i < pool.length; i++) {
          var res = pool[i];

          p_pool[res['ID']] = res['NAME'];
        }
        return (p_pool);
      } else {
        var res = pool;

        p_pool[res['ID']] = res['NAME'];

        return (p_pool);
      }
    }
  };

  return Helper;
});

define('opennebula/error',['require'],function(require) {
  var Error = function(resp) {
    var error = {};
    if (resp.responseText) {
      try {
        error = JSON.parse(resp.responseText);
      }
      catch (e) {
        error.error = {message: "It appears there was a server exception. Please check server's log."};
      };
    } else {
      error.error = {};
    }
    error.error.http_status = resp.status;
    return error;
  }

  return Error;
})
;
define('opennebula/action',['require','./helper','./error'],function(require) {
  var OpenNebulaHelper = require('./helper');
  var OpenNebulaError = require('./error');

  var listCache = {};
  var listWaiting = {};
  var listCallbacks = {};

  var nameIndex = {};

  var CACHE_EXPIRE = 60000; //ms

  var _clearCache = function(cache_name) {
    listCache[cache_name] = null;
    //console.log(cache_name+" cache cleaned");
  }

  //Example: Simple action: publish. Simple action with action obj: deploy
  var _simple_action = function(params, resource, method, action_obj, path) {
    var callback = params.success;
    var callbackError = params.error;
    var id = params.data.id;
    var action, request;
    if (action_obj) {
      action = OpenNebulaHelper.action(method, action_obj);
      request = OpenNebulaHelper.request(resource, method, [id, action_obj]);
    } else {
      action = OpenNebulaHelper.action(method);
      request = OpenNebulaHelper.request(resource, method, id);
    };

    var reqPath = path ? path : resource.toLowerCase();
    var cache_name = params.cache_name ? params.cache_name : resource;

    $.ajax({
      url: reqPath + "/" + id + "/action",
      type: "POST",
      contentType: "application/json; charset=utf-8",
      data: JSON.stringify(action),
      success: function() {
        _clearCache(cache_name);

        return callback ? callback(request) : null;
      },
      error: function(response) {
        return callbackError ?
            callbackError(request, OpenNebulaError(response)) : null;
      }
    });
  }

  var Action = {
    "create": function(params, resource, path) {
      var callback = params.success;
      var callbackError = params.error;
      var data = params.data;
      var request = OpenNebulaHelper.request(resource, "create", data);
      var reqPath = path ? path : resource.toLowerCase();
      var cache_name = params.cache_name ? params.cache_name : resource;

      $.ajax({
        url: reqPath,
        type: "POST",
        dataType: "json",
        data: JSON.stringify(data),
        contentType: "application/json; charset=utf-8",
        success: function(response) {
          _clearCache(cache_name);

          return callback ? callback(request, response) : null;
        },
        error: function(response) {
          return callbackError ?
              callbackError(request, OpenNebulaError(response)) : null;
        }
      });
    },

    "del": function(params, resource, path) {
      var callback = params.success;
      var callbackError = params.error;
      var id = params.data.id;
      var request = OpenNebulaHelper.request(resource, "delete", id);
      var reqPath = path ? path : resource.toLowerCase();
      var cache_name = params.cache_name ? params.cache_name : resource;

      $.ajax({
        url: reqPath + "/" + id,
        type: "DELETE",
        success: function() {
          _clearCache(cache_name);

          return callback ? callback(request) : null;
        },
        error: function(response) {
          return callbackError ?
              callbackError(request, OpenNebulaError(response)) : null;
        }
      });
    },

    "list": function(params, resource, path, process) {
      var callback = params.success;
      var callbackError = params.error;
      var timeout = params.timeout || false;
      var request = OpenNebulaHelper.request(resource, "list");
      var reqPath = path ? path : resource.toLowerCase();
      var cache_name = params.cache_name ? params.cache_name : resource;

      var options = params.options;
      var force = false;
      if (options) {
        force = options.force;
      }

      if (force) {
        _clearCache(cache_name);
      }

      if (!force &&
          listCache[cache_name] &&
          listCache[cache_name]["timestamp"] + CACHE_EXPIRE > new Date().getTime()) {

        //console.log(cache_name+" list. Cache used");

        return callback ?
            callback(request, listCache[cache_name]["data"]) : null;
      }

      // TODO: Because callbacks are queued, we may need to force a
      // timeout. Otherwise a blocked call cannot be retried.

      if (!listCallbacks[cache_name]) {
        listCallbacks[cache_name] = [];
      }

      listCallbacks[cache_name].push({
        success : callback,
        error : callbackError
      });

      //console.log(cache_name+" list. Callback queued");

      if (listWaiting[cache_name]) {
        return;
      }

      listWaiting[cache_name] = true;

      //console.log(cache_name+" list. NO cache, calling ajax");

      $.ajax({
        url: reqPath,
        type: "GET",
        data: {timeout: timeout},
        dataType: "json",
        success: function(response) {
          var list;

          if (process){
            list = process(response);
          } else {
            list = OpenNebulaHelper.pool(resource, response);
          }

          nameIndex[cache_name] = OpenNebulaHelper.pool_name_processing(
                                        resource+"_POOL", resource, response);

          listCache[cache_name] = {
            timestamp   : new Date().getTime(),
            data        : list
          };

          listWaiting[cache_name] = false;

          for (var i = 0; i < listCallbacks[cache_name].length; i++) {
            var callback = listCallbacks[cache_name][i].success;

            if (callback) {
              //console.log(cache_name+" list. Callback called");
              callback(request, list, response);
            }
          }

          listCallbacks[cache_name] = [];

          return;
        },
        error: function(response) {
          listWaiting[cache_name] = false;

          for (var i = 0; i < listCallbacks[cache_name].length; i++) {
            var callback = listCallbacks[cache_name][i].error;

            if (callback) {
              //console.log(cache_name+" list. ERROR Callback called");
              callback(request, OpenNebulaError(response));
            }
          }

          listCallbacks[cache_name] = [];

          return;
        }
      });
    },

    "list_in_zone": function(params, resource, path) {
      var callback = params.success;
      var callbackError = params.error;
      var timeout = params.timeout || false;
      var request = OpenNebulaHelper.request(resource, "list");
      var reqPath = path ? path : resource.toLowerCase();

      $.ajax({
        url: reqPath,
        type: "GET",
        data: {timeout: timeout, zone_id: params.data.zone_id},
        dataType: "json",
        success: function(response) {
          var list = OpenNebulaHelper.pool(resource, response)
          return callback ?
              callback(request, list) : null;
        },
        error: function(response) {
          return callbackError ?
              callbackError(request, OpenNebulaError(response)) : null;
        }
      });
    },

    //Subresource examples: "log"...
    "show": function(params, resource, subresource, path) {
      var callback = params.success;
      var callbackError = params.error;
      var id = params.data.id;
      var data = params.data;
      var request = subresource ?
          OpenNebulaHelper.request(resource, subresource, id) :
          OpenNebulaHelper.request(resource, "show", id);

      var reqPath = path ? path : resource.toLowerCase();
      var url = reqPath + "/" + id;
      url = subresource ? url + "/" + subresource : url;

      $.ajax({
        url: url,
        type: "GET",
        dataType: "json",
        data: data,
        success: function(response) {
          return callback ? callback(request, response) : null;
        },
        error: function(response) {
          return callbackError ?
              callbackError(request, OpenNebulaError(response)) : null;
        }
      });
    },

    "chown": function(params, resource, path) {
      var id = params.data.extra_param;
      var action_obj = {"owner_id": id,
                        "group_id": "-1"};

      _simple_action(params, resource, "chown", action_obj, path);
    },

    "chgrp": function(params, resource, path) {
      var id = params.data.extra_param;
      var action_obj = {"owner_id": "-1",
                        "group_id": id};

      _simple_action(params, resource, "chown", action_obj, path);
    },

    "simple_action": _simple_action,

    "monitor": function(params, resource, all, path) {
      var callback = params.success;
      var callbackError = params.error;
      var data = params.data;

      var method = "monitor";
      var request = OpenNebulaHelper.request(resource, method, data);

      var url = path ? path : resource.toLowerCase();
      url = all ? url + "/monitor" : url + "/" + params.data.id + "/monitor";

      $.ajax({
        url: url,
        type: "GET",
        data: data['monitor'],
        dataType: "json",
        success: function(response) {
          return callback ? callback(request, response) : null;
        },
        error: function(response) {
          return callbackError ?
              callbackError(request, OpenNebulaError(response)) : null;
        }
      });
    },

    "accounting": function(params, resource, path) {
      var callback = params.success;
      var callbackError = params.error;
      var data = params.data;

      var method = "accounting";
      var request = OpenNebulaHelper.request(resource, method, data);

      var url = path ? path : resource.toLowerCase() + "/accounting";

      $.ajax({
        url: url,
        type: "GET",
        data: data,
        dataType: "json",
        success: function(response) {
          return callback ? callback(request, response) : null;
        },
        error: function(response) {
          return callbackError ?
              callbackError(request, OpenNebulaError(response)) : null;
        }
      });
    },

    "showback": function(params, resource, path) {
      var callback = params.success;
      var callbackError = params.error;
      var data = params.data;

      var method = "showback";
      var request = OpenNebulaHelper.request(resource, method, data);

      var url = path ? path : resource.toLowerCase() + "/showback";

      $.ajax({
        url: url,
        type: "GET",
        data: data,
        dataType: "json",
        success: function(response) {
          return callback ? callback(request, response) : null;
        },
        error: function(response) {
          return callbackError ?
              callbackError(request, OpenNebulaError(response)) : null;
        }
      });
    },

    "getName": function(id, cache_name){
      if(nameIndex[cache_name] != undefined){
        var name = nameIndex[cache_name][id];
        if (name != undefined){
          return name;
        }

        // TODO: if name is not found, perform a .list or .show to at least
        // get it ready for the next call?
      }

      return ""+id;
    },

    "clear_cache": _clearCache
  };

  return Action;
});

define('opennebula/auth',['require','./helper','./error'],function(require) {
  var OpenNebulaHelper = require('./helper');
  var OpenNebulaError = require('./error');

  var RESOURCE = "AUTH";

  var Auth = {
    "login": function(params) {
      var callback = params.success;
      var callbackError = params.error;
      var username = params.data.username;
      var password = params.data.password;
      var remember = params.remember;

      var request = OpenNebulaHelper.request(RESOURCE, "login");

      $.ajax({
        url: "login",
        type: "POST",
        data: {remember: remember},
        beforeSend : function(req) {
          var token = username + ':' + password;
          var authString = 'Basic ';
          if (typeof(btoa) === 'function')
              authString += btoa(unescape(encodeURIComponent(token)))
          else {
            token = CryptoJS.enc.Utf8.parse(token);
            authString += CryptoJS.enc.Base64.stringify(token)
          }

          req.setRequestHeader("Authorization", authString);
        },
        success: function(response) {
          return callback ? callback(request, response) : null;
        },
        error: function(response) {
          return callbackError ?
              callbackError(request, OpenNebulaError(response)) : null;
        }
      });
    },

    "logout": function(params) {
      var callback = params.success;
      var callbackError = params.error;

      var request = OpenNebulaHelper.request(RESOURCE, "logout");

      $.ajax({
        url: "logout",
        type: "POST",
        success: function(response) {
          // TODO $.cookie("one-user", null);
          return callback ? callback(request, response) : null;
        },
        error: function(response) {
          return callbackError ?
              callbackError(request, OpenNebulaError(response)) : null;
        }
      });
    }
  }

  return Auth;
})
;
define('opennebula/acl',['require','./action'],function(require) {
  var OpenNebulaAction = require('./action');

  var RESOURCE = "ACL";

  var Acl = {
    "resource": RESOURCE,
    "create" : function(params) {
      OpenNebulaAction.create(params, RESOURCE);
    },
    "del" : function(params) {
      OpenNebulaAction.del(params, RESOURCE);
    },
    "list" : function(params) {
      OpenNebulaAction.list(params, RESOURCE);
    }
  }

  return Acl;
})
;
define('opennebula/cluster',['require','./action'],function(require) {
  var OpenNebulaAction = require('./action');

  var RESOURCE = "CLUSTER";

  var Cluster = {
    "resource": RESOURCE,
    "create" : function(params) {
      OpenNebulaAction.create(params, RESOURCE);
    },
    "del" : function(params) {
      OpenNebulaAction.del(params, RESOURCE);
    },
    "list" : function(params) {
      OpenNebulaAction.list(params, RESOURCE);
    },
    "list_in_zone" : function(params) {
      OpenNebulaAction.list_in_zone(params, RESOURCE);
    },
    "show" : function(params) {
      OpenNebulaAction.show(params, RESOURCE);
    },
    "addhost" : function(params) {
      var action_obj = {"host_id": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "addhost", action_obj);
    },
    "delhost" : function(params) {
      var action_obj = {"host_id": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "delhost", action_obj);
    },
    "adddatastore" : function(params) {
      var action_obj = {"ds_id": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "adddatastore", action_obj);
    },
    "deldatastore" : function(params) {
      var action_obj = {"ds_id": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "deldatastore", action_obj);
    },
    "addvnet" : function(params) {
      var action_obj = {"vnet_id": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "addvnet", action_obj);
    },
    "delvnet" : function(params) {
      var action_obj = {"vnet_id": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "delvnet", action_obj);
    },
    "update" : function(params) {
      var action_obj = {"template_raw" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
    },
    "rename" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "rename", action_obj);
    },
    "getName": function(id){
      return OpenNebulaAction.getName(id, RESOURCE);
    }
  }

  return Cluster;
})
;
define('opennebula/datastore',['require','./action','utils/locale'],function(require) {
  var OpenNebulaAction = require('./action');
  var Locale = require('utils/locale');

  var RESOURCE = "DATASTORE";
  var STATES_STR = [
    Locale.tr("ON"),
    Locale.tr("OFF")];

  var TYPES_STR = [
    Locale.tr("IMAGE"),
    Locale.tr("SYSTEM"),
    Locale.tr("FILE")
  ];

  var STATES = {
    READY     : 0,
    DISABLED  : 1
  };

  var TYPES = {
    IMAGE_DS  : 0,
    SYSTEM_DS : 1,
    FILE_DS   : 2
  };

  var Datastore = {
    "resource": RESOURCE,
    "stateStr": function(stateId) {
      return STATES_STR[stateId];
    },
    "STATES": STATES,
    "typeStr": function(typeId) {
      return TYPES_STR[typeId];
    },
    "TYPES": TYPES,
    "create" : function(params) {
      OpenNebulaAction.create(params, RESOURCE);
    },
    "del" : function(params) {
      OpenNebulaAction.del(params, RESOURCE);
    },
    "list" : function(params) {
      OpenNebulaAction.list(params, RESOURCE);
    },
    "list_in_zone" : function(params) {
      OpenNebulaAction.list_in_zone(params, RESOURCE);
    },
    "show" : function(params) {
      OpenNebulaAction.show(params, RESOURCE);
    },
    "chown" : function(params) {
      OpenNebulaAction.chown(params, RESOURCE);
    },
    "chgrp" : function(params) {
      OpenNebulaAction.chgrp(params, RESOURCE);
    },
    "chmod" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "chmod", action_obj);
    },
    "update" : function(params) {
      var action_obj = {"template_raw" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
    },
    "rename" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "rename", action_obj);
    },
    "enable": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "enable");
    },
    "disable": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "disable");
    },
    "getName": function(id){
      return OpenNebulaAction.getName(id, RESOURCE);
    }
  }

  return Datastore;
})
;
define('utils/quotas/quota-limits',['require'],function(require) {
  // Constants
  var QUOTA_LIMIT_DEFAULT   = "-1";
  var QUOTA_LIMIT_UNLIMITED = "-2";

  return {
    'QUOTA_LIMIT_DEFAULT': QUOTA_LIMIT_DEFAULT,
    'QUOTA_LIMIT_UNLIMITED': QUOTA_LIMIT_UNLIMITED
  };
});

define('utils/quotas/quota-defaults',['require','./quota-limits'],function(require) {
  // The default quotas returned by the pool.list method are stored here
  var _defaultUserQuotas = {
    "VM_QUOTA": {
      "VM": {
        "CPU":      QUOTA_LIMIT_UNLIMITED,
        "MEMORY":   QUOTA_LIMIT_UNLIMITED,
        "VMS":      QUOTA_LIMIT_UNLIMITED,
        "SYSTEM_DISK_SIZE": QUOTA_LIMIT_UNLIMITED,
      }
    },
    "DATASTORE_QUOTA": {},
    "IMAGE_QUOTA": {},
    "NETWORK_QUOTA": {}
  };

  var _defaultGroupQuotas = {
    "VM_QUOTA": {
      "VM": {
        "CPU":      QUOTA_LIMIT_UNLIMITED,
        "MEMORY":   QUOTA_LIMIT_UNLIMITED,
        "VMS":      QUOTA_LIMIT_UNLIMITED,
        "SYSTEM_DISK_SIZE": QUOTA_LIMIT_UNLIMITED,
      }
    },
    "DATASTORE_QUOTA": {},
    "IMAGE_QUOTA": {},
    "NETWORK_QUOTA": {}
  };

  var QuotaLimits = require('./quota-limits');


  // Constants
  var QUOTA_LIMIT_DEFAULT   = QuotaLimits.QUOTA_LIMIT_DEFAULT;
  var QUOTA_LIMIT_UNLIMITED = QuotaLimits.QUOTA_LIMIT_UNLIMITED;


  function _setDefaultUserQuotas(defaultUserQuotas){
    _defaultUserQuotas = defaultUserQuotas;
  }

  function _setDefaultGroupQuotas(defaultGroupQuotas){
    _defaultGroupQuotas = defaultGroupQuotas;
  }

  function _getDefaultUserQuotas(){
    return _defaultUserQuotas;
  }

  function _getDefaultGroupQuotas(){
    return _defaultGroupQuotas;
  }

  /**
   * @param  {string} resource_name User or Group
   * @return {Object}
   */
  function _getDefaultQuotas(resource_name){
    if(resource_name == "User") {
      return _getDefaultUserQuotas();
    } else {
      return _getDefaultGroupQuotas();
    }
  }

  // Processes the default quotas as returned by OpenNebula, to be easier to
  // use in Sunstone
  function _default_quotas(default_quotas){
    // Initialize the VM_QUOTA to unlimited if it does not exist
    if ($.isEmptyObject(default_quotas.VM_QUOTA)){
      default_quotas.VM_QUOTA = {
        "VM" : {
          "VMS"           : QUOTA_LIMIT_UNLIMITED,
          "MEMORY"        : QUOTA_LIMIT_UNLIMITED,
          "CPU"           : QUOTA_LIMIT_UNLIMITED,
          "SYSTEM_DISK_SIZE" : QUOTA_LIMIT_UNLIMITED
        }
      };
    }

    // Replace the DATASTORE array with a map

    var ds_quotas = [];

    if ($.isArray(default_quotas.DATASTORE_QUOTA.DATASTORE))
      ds_quotas = default_quotas.DATASTORE_QUOTA.DATASTORE;
    else if (default_quotas.DATASTORE_QUOTA.DATASTORE)
      ds_quotas = [default_quotas.DATASTORE_QUOTA.DATASTORE];

    delete default_quotas.DATASTORE_QUOTA;

    default_quotas.DATASTORE_QUOTA = {};

    for (var i=0; i < ds_quotas.length; i++){
      default_quotas.DATASTORE_QUOTA[ds_quotas[i].ID] = ds_quotas[i];
    }

    // Replace the IMAGE array with a map

    var img_quotas = [];

    if ($.isArray(default_quotas.IMAGE_QUOTA.IMAGE))
      img_quotas = default_quotas.IMAGE_QUOTA.IMAGE;
    else if (default_quotas.IMAGE_QUOTA.IMAGE)
      img_quotas = [default_quotas.IMAGE_QUOTA.IMAGE];

    delete default_quotas.IMAGE_QUOTA;

    default_quotas.IMAGE_QUOTA = {};

    for (var i=0; i < img_quotas.length; i++){
      default_quotas.IMAGE_QUOTA[img_quotas[i].ID] = img_quotas[i];
    }

    // Replace the NETWORK array with a map

    var net_quotas = [];

    if ($.isArray(default_quotas.NETWORK_QUOTA.NETWORK))
      net_quotas = default_quotas.NETWORK_QUOTA.NETWORK;
    else if (default_quotas.NETWORK_QUOTA.NETWORK)
      net_quotas = [default_quotas.NETWORK_QUOTA.NETWORK];

    delete default_quotas.NETWORK_QUOTA;

    default_quotas.NETWORK_QUOTA = {};

    for (var i=0; i < net_quotas.length; i++){
      default_quotas.NETWORK_QUOTA[net_quotas[i].ID] = net_quotas[i];
    }

    return default_quotas;
  }


  return {
    'setDefaultUserQuotas': _setDefaultUserQuotas,
    'getDefaultUserQuotas': _getDefaultUserQuotas,
    'setDefaultGroupQuotas': _setDefaultGroupQuotas,
    'getDefaultGroupQuotas': _getDefaultGroupQuotas,
    'getDefaultQuotas': _getDefaultQuotas,
    'default_quotas': _default_quotas
  };
});

define('opennebula/group',['require','utils/quotas/quota-defaults','./action','./helper','./error'],function(require) {

  var QuotaDefaults = require('utils/quotas/quota-defaults');
  var OpenNebulaAction = require('./action');
  var OpenNebulaHelper = require('./helper');
  var OpenNebulaError  = require('./error');

  var RESOURCE = "GROUP";

  var Group =  {
    "resource": RESOURCE,
    "create": function(params) {
      OpenNebulaAction.create(params, RESOURCE);
    },
    "del": function(params) {
      OpenNebulaAction.del(params, RESOURCE);
    },
    "list": function(params) {
      OpenNebulaAction.list(params, RESOURCE, null, function(response) {
        var list = OpenNebulaHelper.pool(RESOURCE, response);

        QuotaDefaults.setDefaultGroupQuotas(
          QuotaDefaults.default_quotas(response.GROUP_POOL.DEFAULT_GROUP_QUOTAS)
        );

        // Inject the VM quota. This info is returned separately in the
        // pool info call, but the elementArray expects it inside the GROUP,
        // as it is returned by the individual info call
        var quotas_hash = OpenNebulaHelper.pool_hash_processing(
                                            'GROUP_POOL', 'QUOTAS', response);

        $.each(list,function(){
          var q = quotas_hash[this[RESOURCE].ID];

          if (q != undefined) {
              this[RESOURCE].VM_QUOTA = q.QUOTAS.VM_QUOTA;
          }
        });

        return list;
      });
    },
    "update": function(params) {
      var action_obj = {"template_raw" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
    },
    "set_quota" : function(params) {
      var action_obj = {quotas :  params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "set_quota", action_obj);
    },
    "show" : function(params) {
      var callback = params.success;

      // Before calling the true callback, we update the default quotas
      // included in the .show response
      params.success = function(request, response) {
        QuotaDefaults.setDefaultGroupQuotas(
          QuotaDefaults.default_quotas(response.GROUP.DEFAULT_GROUP_QUOTAS)
        );

        return callback ? callback(request, response) : null;
      };

      OpenNebulaAction.show(params, RESOURCE);
    },
    "accounting" : function(params) {
      OpenNebulaAction.monitor(params, RESOURCE, false);
    },
    "add_admin" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "add_admin", action_obj);
    },
    "del_admin" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "del_admin", action_obj);
    },
    "getName": function(id){
      return OpenNebulaAction.getName(id, RESOURCE);
    }
  };

  return Group;
});
define('opennebula/host',['require','./action','utils/locale','./error','./helper'],function(require) {
  var OpenNebulaAction = require('./action');
  var Locale = require('utils/locale');
  var OpenNebulaError = require('./error');
  var OpenNebulaHelper = require('./helper');


  var pcisCache;
  var pcisWaiting = false;
  var pcisCallbacks = [];

  var CACHE_EXPIRE = 300000; //ms

  var _clearCache = function() {
    pcisCache = null;
    //console.log("Host.pciDevices. Cache cleaned");
  };

  var RESOURCE = "HOST";

  var STATES_STR = [
    Locale.tr("INIT"),
    Locale.tr("MONITORING_MONITORED"),
    Locale.tr("MONITORED"),
    Locale.tr("ERROR"),
    Locale.tr("DISABLED"),
    Locale.tr("MONITORING_ERROR"),
    Locale.tr("MONITORING_INIT"),
    Locale.tr("MONITORING_DISABLED")
  ];

  var SIMPLE_STATES_STR = [
    Locale.tr("INIT"),
    Locale.tr("UPDATE"),
    Locale.tr("ON"),
    Locale.tr("ERROR"),
    Locale.tr("OFF"),
    Locale.tr("RETRY"),
    Locale.tr("INIT"),
    Locale.tr("OFF")
  ];

  var STATES = {
    INIT                 : 0,
    MONITORING_MONITORED : 1,
    MONITORED            : 2,
    ERROR                : 3,
    DISABLED             : 4,
    MONITORING_ERROR     : 5,
    MONITORING_INIT      : 6,
    MONITORING_DISABLED  : 7
  };

  var Host = {
    "resource": RESOURCE,
    "stateStr": function(stateId) {
      return STATES_STR[stateId];
    },
    "simpleStateStr": function(stateId) {
      return SIMPLE_STATES_STR[stateId];
    },
    "STATES": STATES,
    "create": function(params) {
      OpenNebulaAction.create(params, RESOURCE);
      _clearCache();
    },
    "del": function(params) {
      OpenNebulaAction.del(params, RESOURCE);
      _clearCache();
    },
    "list": function(params) {
      OpenNebulaAction.list(params, RESOURCE);
    },
    "list_in_zone" : function(params) {
      OpenNebulaAction.list_in_zone(params, RESOURCE);
    },
    "show": function(params) {
      OpenNebulaAction.show(params, RESOURCE);
    },
    "update": function(params) {
      var action_obj = {"template_raw" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
      _clearCache();
    },
    "enable": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "enable");
      _clearCache();
    },
    "disable": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "disable");
      _clearCache();
    },
    "monitor" : function(params) {
      OpenNebulaAction.monitor(params, RESOURCE, false);
    },
    "pool_monitor" : function(params) {
      OpenNebulaAction.monitor(params, RESOURCE, true);
    },
    "rename" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "rename", action_obj);
    },
    "getName": function(id){
      return OpenNebulaAction.getName(id, RESOURCE);
    },
    "pciDevices": function(params){
      var callback = params.success;
      var callbackError = params.error;
      var request = OpenNebulaHelper.request(RESOURCE, "infrastructure");

      if (pcisCache &&
          pcisCache["timestamp"] + CACHE_EXPIRE > new Date().getTime()) {

        //console.log("Host.pciDevices. Cache used");

        return callback ?
            callback(request, pcisCache["data"]) : null;
      }

      pcisCallbacks.push({
        success : callback,
        error : callbackError
      });

      //console.log("Host.pciDevices. Callback queued");

      if (pcisWaiting) {
        return;
      }

      pcisWaiting = true;

      //console.log("Host.pciDevices. NO cache, calling ajax");

      $.ajax({
        url: "infrastructure",
        type: "GET",
        dataType: "json",
        success: function(response) {
          var pcis = response.pci_devices;

          if (pcis == undefined){
            pcis = [];
          }

          if (!$.isArray(pcis)){ // If only 1 convert to array
            pcis = [pcis];
          }

          pcisCache = {
            timestamp   : new Date().getTime(),
            data        : pcis
          };

          pcisWaiting = false;

          for (var i = 0; i < pcisCallbacks.length; i++) {
            var callback = pcisCallbacks[i].success;

            if (callback) {
              //console.log("Host.pciDevices. Callback called");
              callback(request, pcis);
            }
          }

          pcisCallbacks = [];

          return;
        },
        error: function(response) {
          pcisWaiting = false;

          for (var i = 0; i < pcisCallbacks.length; i++) {
            var callback = pcisCallbacks[i].error;

            if (callback) {
              //console.log("Host.pciDevices. ERROR Callback called");
              callback(request, OpenNebulaError(response));
            }
          }

          pcisCallbacks = [];

          return;
        }
      });
    }
  }

  return Host;
})
;
define('opennebula/image',['require','./action','utils/locale'],function(require) {
  var OpenNebulaAction = require('./action');
  var Locale = require('utils/locale');

  var RESOURCE = "IMAGE";
  var STATES_STR = [
    Locale.tr("INIT"),
    Locale.tr("READY"),
    Locale.tr("USED"),
    Locale.tr("DISABLED"),
    Locale.tr("LOCKED"),
    Locale.tr("ERROR"),
    Locale.tr("CLONE"),
    Locale.tr("DELETE"),
    Locale.tr("USED_PERS")
  ];

  var TYPES_STR = [
    Locale.tr("OS"),
    Locale.tr("CDROM"),
    Locale.tr("DATABLOCK"),
    Locale.tr("KERNEL"),
    Locale.tr("RAMDISK"),
    Locale.tr("CONTEXT")
  ];

  var STATES = {
    INIT      : 0,
    READY     : 1,
    USED      : 2,
    DISABLED  : 3,
    LOCKED    : 4,
    ERROR     : 5,
    CLONE     : 6,
    DELETE    : 7,
    USED_PERS : 8
  };

  var TYPES = {
    OS        : 0,
    CDROM     : 1,
    DATABLOCK : 2,
    KERNEL    : 3,
    RAMDISK   : 4,
    CONTEXT   : 5
  };

  var Image = {
    "resource": RESOURCE,
    "stateStr": function(stateId) {
      return STATES_STR[stateId];
    },
    "STATES": STATES,
    "typeStr": function(typeId) {
      return TYPES_STR[typeId];
    },
    "TYPES": TYPES,
    "create": function(params) {
      OpenNebulaAction.create(params, RESOURCE);
    },
    "del": function(params) {
      OpenNebulaAction.del(params, RESOURCE);
    },
    "list": function(params) {
      OpenNebulaAction.list(params, RESOURCE);
    },
    "show": function(params) {
      OpenNebulaAction.show(params, RESOURCE);
    },
    "chown" : function(params) {
      OpenNebulaAction.chown(params, RESOURCE);
    },
    "chgrp" : function(params) {
      OpenNebulaAction.chgrp(params, RESOURCE);
    },
    "chmod" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "chmod", action_obj);
    },
    "update": function(params) {
      var action_obj = {"template_raw" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
    },
    "enable": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "enable");
    },
    "disable": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "disable");
    },
    "persistent": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "persistent");
    },
    "nonpersistent": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "nonpersistent");
    },
    "chtype": function(params) {
      var action_obj = {"type" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "chtype", action_obj);
    },
    "clone" : function(params) {
      var action_obj = params.data.extra_param ? params.data.extra_param : {};
      OpenNebulaAction.simple_action(params, RESOURCE, "clone", action_obj);
    },
    "rename" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "rename", action_obj);
    },
    "snapshot_flatten": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "snapshot_flatten", action_obj);
    },
    "snapshot_revert": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "snapshot_revert", action_obj);
    },
    "snapshot_delete": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "snapshot_delete", action_obj);
    },
    "getName": function(id){
      return OpenNebulaAction.getName(id, RESOURCE);
    }
  }

  return Image;
})
;
define('opennebula/marketplace',['require','./action','./error','./helper','utils/notifier','utils/locale'],function(require) {
  var OpenNebulaAction = require('./action');
  var OpenNebulaError = require('./error');
  var OpenNebulaHelper = require('./helper');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');

  var RESOURCE = "MARKETPLACE";

  var Marketplace = {
    "resource": RESOURCE,
    "show" : function(params) {
      params.error = function() {
        return Notifier.notifyError(Locale.tr("Cannot connect to OpenNebula Marketplace"));
      };
      OpenNebulaAction.show(params, RESOURCE);
    },
    "list" : function(params) {
      //Custom list request function, since the contents do not come
      //in the same format as the rest of opennebula resources.
      var callback = params.success;
      var callback_error = params.error;
      var timeout = params.timeout || false;
      var request = OpenNebulaHelper.request('MARKETPLACE', 'list');

      $.ajax({
        url: 'marketplace',
        type: 'GET',
        data: {timeout: timeout},
        dataType: "json",
        success: function(response) {
          $(".marketplace_error_message").hide();
          return callback ? callback(request, response) : null;
        },
        error: function(res) {
          $(".marketplace_error_message").show();
          return callback_error ? callback_error(request, OpenNebulaError(res)) : null;
        }
      });
    }
  };

  return Marketplace;
});
define('opennebula/network',['require','./action'],function(require) {
  var OpenNebulaAction = require('./action');

  var RESOURCE = "VNET";

  var Network =  {
    "resource": RESOURCE,
    "create": function(params) {
      OpenNebulaAction.create(params, RESOURCE);
    },
    "del": function(params) {
      OpenNebulaAction.del(params, RESOURCE);
    },
    "list": function(params) {
      OpenNebulaAction.list(params, RESOURCE);
    },
    "list_in_zone" : function(params) {
      OpenNebulaAction.list_in_zone(params, RESOURCE);
    },
    "show": function(params) {
      OpenNebulaAction.show(params, RESOURCE);
    },
    "chown" : function(params) {
      OpenNebulaAction.chown(params, RESOURCE);
    },
    "chgrp" : function(params) {
      OpenNebulaAction.chgrp(params, RESOURCE);
    },
    "chmod" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "chmod", action_obj);
    },
    "publish": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "publish");
    },
    "unpublish": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "unpublish");
    },
    "hold" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "hold", action_obj);
    },
    "release" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "release", action_obj);
    },
    "add_ar" : function(params) {
      var action_obj = {"ar_template" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "add_ar", action_obj);
    },
    "rm_ar" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "rm_ar", action_obj);
    },
    "update_ar": function(params) {
      var action_obj = {"ar_template" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update_ar", action_obj);
    },
    "reserve": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "reserve", action_obj);
    },
    "update": function(params) {
      var action_obj = {"template_raw" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
    },
    "rename" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "rename", action_obj);
    },
    "getName": function(id){
      return OpenNebulaAction.getName(id, RESOURCE);
    }
  }

  return Network;
})
;
define('tabs/oneflow-services-tab/tabId',['require'],function(require){
  return 'oneflow-services';
});
define('tabs/oneflow-services-tab/panels/roles/panelId',['require'],function(require){
  return 'service_roles_tab';
});
define('opennebula/role',['require','./action','./error','./helper','utils/locale','tabs/oneflow-services-tab/tabId','tabs/oneflow-services-tab/panels/roles/panelId'],function(require) {
  var OpenNebulaAction = require('./action');
  var OpenNebulaError = require('./error');
  var OpenNebulaHelper = require('./helper');
  var Locale = require('utils/locale');

  var RESOURCE = "DOCUMENT";
  var PATH = 'service';

  var TAB_ID = require('tabs/oneflow-services-tab/tabId');
  var PANEL_ID = require('tabs/oneflow-services-tab/panels/roles/panelId');

  function generate_batch_action_params() {
    var context = $('#'+TAB_ID+' #'+PANEL_ID);

    var action_obj = {
        "period" : $("#batch_action_period", context).val(),
        "number" : $("#batch_action_number", context).val()};

    return action_obj;
  }

  var Role = {
    "resource": RESOURCE,
    "state" : function(state_int) {
      state_int = state_int ? state_int : 0;
      var state = [
          Locale.tr("PENDING"),
          Locale.tr("DEPLOYING"),
          Locale.tr("RUNNING"),
          Locale.tr("UNDEPLOYING"),
          Locale.tr("WARNING"),
          Locale.tr("DONE"),
          Locale.tr("FAILED_UNDEPLOYING"),
          Locale.tr("FAILED_DEPLOYING"),
          Locale.tr("SCALING"),
          Locale.tr("FAILED_SCALING"),
          Locale.tr("COOLDOWN")
      ][state_int]
      return state ? state : state_int;
    },
    "hold" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "hold",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "release" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "release",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "suspend" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "suspend",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "resume" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "resume",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "stop" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "stop",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "boot" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "boot",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "delete_recreate" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "delete-recreate",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "reboot" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "reboot",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "reboot_hard" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "reboot-hard",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "poweroff" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "poweroff",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "poweroff_hard" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "poweroff-hard",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "undeploy" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "undeploy",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "undeploy_hard" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "undeploy-hard",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "snapshot_create" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "snapshot-create",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "shutdown" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "shutdown",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "shutdown_hard" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "shutdown-hard",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "del" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "delete",
                                      generate_batch_action_params(),
                                      PATH);
    },
    "recover" : function(params) {
      OpenNebulaAction.simple_action(params,
                                      RESOURCE,
                                      "recover",
                                      null,
                                      PATH);
    },
    "update" : function(params) {
      var request = OpenNebulaHelper.request(RESOURCE, "update", params.data.id);

      $.ajax({
        url: PATH + "/" + params.data.id,
        type: "PUT",
        dataType: "json",
        contentType: "application/json; charset=utf-8",
        data: JSON.stringify(params.data.extra_param),
        success: function(response) {
          return params.success ? params.success(request, response) : null;
        },
        error: function(response) {
          return params.error ? params.error(request, OpenNebulaError(response)) : null;
        }
      });
    }
  }

  return Role;
})
;
define('opennebula/securitygroup',['require','./action'],function(require) {
  var OpenNebulaAction = require('./action');

  var RESOURCE = "SECURITY_GROUP";

  var SecurityGroup = {
    "resource" : "SECURITY_GROUP",

    "create" : function(params) {
      OpenNebulaAction.create(params, RESOURCE);
    },
    "del" : function(params) {
      OpenNebulaAction.del(params, RESOURCE);
    },
    "list" : function(params) {
      OpenNebulaAction.list(params, RESOURCE);
    },
    "show" : function(params) {
      OpenNebulaAction.show(params, RESOURCE);
    },
    "chown" : function(params) {
      OpenNebulaAction.chown(params, RESOURCE);
    },
    "chgrp" : function(params) {
      OpenNebulaAction.chgrp(params, RESOURCE);
    },
    "chmod" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "chmod", action_obj);
    },
    "update": function(params) {
      var action_obj = {"template_raw" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
    },
    "clone" : function(params) {
      var name = params.data.extra_param ? params.data.extra_param : "";
      var action_obj = {"name" : name};
      OpenNebulaAction.simple_action(params, RESOURCE, "clone", action_obj);
    },
    "rename" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "rename", action_obj);
    },
    "getName": function(id){
      return OpenNebulaAction.getName(id, RESOURCE);
    }
  }

  return SecurityGroup;
})
;
define('opennebula/service',['require','./action','utils/locale'],function(require) {
  var OpenNebulaAction = require('./action');
  var Locale = require('utils/locale');

  var RESOURCE = "DOCUMENT";
  var PATH = 'service';
  var CACHE_NAME = 'SERVICE'

  var STATES_STR = [
    Locale.tr("PENDING"),
    Locale.tr("DEPLOYING"),
    Locale.tr("RUNNING"),
    Locale.tr("UNDEPLOYING"),
    Locale.tr("WARNING"),
    Locale.tr("DONE"),
    Locale.tr("FAILED_UNDEPLOYING"),
    Locale.tr("FAILED_DEPLOYING"),
    Locale.tr("SCALING"),
    Locale.tr("FAILED_SCALING"),
    Locale.tr("COOLDOWN")
  ];

  var STATES = {
    PENDING             : 0,
    DEPLOYING           : 1,
    RUNNING             : 2,
    UNDEPLOYING         : 3,
    WARNING             : 4,
    DONE                : 5,
    FAILED_UNDEPLOYING  : 6,
    FAILED_DEPLOYING    : 7,
    SCALING             : 8,
    FAILED_SCALING      : 9,
    COOLDOWN            : 10
  };

  var Service = {
    "resource": RESOURCE,
    "del": function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.del(params, RESOURCE, PATH);
    },
    "list" : function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.list(params, RESOURCE, PATH)
    },
    "show" : function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.show(params, RESOURCE, false, PATH)
    },
    "chown" : function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.chown(params, RESOURCE, PATH);
    },
    "chgrp" : function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.chgrp(params, RESOURCE, PATH);
    },
    "chmod" : function(params) {
      params.cache_name = CACHE_NAME;
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "chmod", action_obj, PATH);
    },
    "shutdown" : function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.simple_action(params, RESOURCE, "shutdown", null, PATH);
    },
    "recover" : function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.simple_action(params, RESOURCE, "recover", null, PATH);
    },
    "stateStr" : function(stateId) {
      return STATES_STR[stateId];
    },
    "STATES": STATES,
    "getName": function(id){
      return OpenNebulaAction.getName(id, CACHE_NAME);
    }
  }

  return Service;
})

;
define('opennebula/servicetemplate',['require','./action'],function(require) {
  var OpenNebulaAction = require('./action');

  var RESOURCE = "DOCUMENT";
  var PATH = 'service_template';
  var CACHE_NAME = "SERVICE_TEMPLATE";

  var ServiceTemplate = {
    "resource": RESOURCE,
    "create": function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.create(params, RESOURCE, PATH);
    },

    "instantiate": function(params) {
      params.cache_name = CACHE_NAME;
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "instantiate", action_obj, PATH);
    },
    "update": function(params) {
      params.cache_name = CACHE_NAME;
      var action_obj = {"template_json" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj, PATH);
    },
    "del": function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.del(params, RESOURCE, PATH);
    },
    "list" : function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.list(params, RESOURCE, PATH)
    },
    "show" : function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.show(params, RESOURCE, false, PATH)
    },
    "chown" : function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.chown(params, RESOURCE, PATH);
    },
    "chgrp" : function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.chgrp(params, RESOURCE, PATH);
    },
    "chmod" : function(params) {
      params.cache_name = CACHE_NAME;
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "chmod", action_obj, PATH);
    },
    "getName": function(id){
      return OpenNebulaAction.getName(id, CACHE_NAME);
    }
  }

  return ServiceTemplate;
})
;
define('opennebula/support',['require','./action'],function(require) {
  var OpenNebulaAction = require('./action');

  var RESOURCE = "REQUEST";
  var PATH = 'support/request';
  var CACHE_NAME = "REQUEST";

  var Support = {
    "resource": RESOURCE,
    "create": function(params) {
      OpenNebulaAction.create(params, RESOURCE, PATH);
    },
    "update": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "update", params.data.extra_param, PATH);
    },
    "list" : function(params) {
      params.cache_name = CACHE_NAME;
      OpenNebulaAction.clear_cache(params.cache_name);
      OpenNebulaAction.list(params, RESOURCE, PATH)
    },
    "show" : function(params) {
      OpenNebulaAction.show(params, RESOURCE, false, PATH)
    }
  }

  return Support;
})
;
define('opennebula/template',['require','./action'],function(require) {
  var OpenNebulaAction = require('./action');

  var RESOURCE = "VMTEMPLATE";

  var Template = {
    "resource": RESOURCE,
    "create" : function(params) {
      OpenNebulaAction.create(params, RESOURCE);
    },
    "del" : function(params) {
      OpenNebulaAction.del(params, RESOURCE);
    },
    "delete_from_provision": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "delete_from_provision");
    },
    "list" : function(params) {
      OpenNebulaAction.list(params, RESOURCE);
    },
    "show" : function(params) {
      OpenNebulaAction.show(params, RESOURCE);
    },
    "chown" : function(params) {
      OpenNebulaAction.chown(params, RESOURCE);
    },
    "chgrp" : function(params) {
      OpenNebulaAction.chgrp(params, RESOURCE);
    },
    "chmod" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "chmod", action_obj);
    },
    "chmod_from_provision": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "chmod_from_provision", action_obj);
    },
    "update" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
    },
    "publish" : function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "publish");
    },
    "unpublish" : function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "unpublish");
    },
    "instantiate" : function(params) {
      var action_obj = params.data.extra_param ? params.data.extra_param : {};
      OpenNebulaAction.simple_action(params, RESOURCE, "instantiate", action_obj);
    },
    "clone" : function(params) {
      var name = params.data.extra_param ? params.data.extra_param : "";
      var action_obj = {"name" : name};
      OpenNebulaAction.simple_action(params, RESOURCE, "clone", action_obj);
    },
    "rename" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "rename", action_obj);
    },
    "getName": function(id){
      return OpenNebulaAction.getName(id, RESOURCE);
    }
  }

  return Template;
})
;
define('opennebula/user',['require','utils/quotas/quota-defaults','./action','./helper','./error'],function(require) {

  var QuotaDefaults = require('utils/quotas/quota-defaults');

  var OpenNebulaAction = require('./action'),
      OpenNebulaHelper = require('./helper'),
      OpenNebulaError  = require('./error');

  var RESOURCE = "USER";

  var User = {
    "resource": RESOURCE,
    "create": function(params) {
      OpenNebulaAction.create(params, RESOURCE);
    },
    "del": function(params) {
      OpenNebulaAction.del(params, RESOURCE);
    },
    "list": function(params) {
      OpenNebulaAction.list(params, RESOURCE, null, function(response) {
        var list = OpenNebulaHelper.pool(RESOURCE, response);

        QuotaDefaults.setDefaultUserQuotas(
          QuotaDefaults.default_quotas(response.USER_POOL.DEFAULT_USER_QUOTAS)
        );

        // Inject the VM user quota. This info is returned separately in the
        // pool info call, but the userElementArray expects it inside the USER,
        // as it is returned by the individual info call
        var quotas_hash = OpenNebulaHelper.pool_hash_processing(
                                            'USER_POOL', 'QUOTAS', response);

        $.each(list,function(){
          var q = quotas_hash[this[RESOURCE].ID];

          if (q != undefined) {
              this[RESOURCE].VM_QUOTA = q.QUOTAS.VM_QUOTA;
          }
        });

        return list;
      });
    },
    "show" : function(params) {
      var callback = params.success;

      // Before calling the true callback, we update the default quotas
      // included in the .show response
      params.success = function(request, response) {
        QuotaDefaults.setDefaultUserQuotas(
          QuotaDefaults.default_quotas(response.USER.DEFAULT_USER_QUOTAS)
        );

        return callback ? callback(request, response) : null;
      };

      OpenNebulaAction.show(params, RESOURCE);
    },
    "passwd": function(params) {
      var action_obj = {"password": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "passwd", action_obj);
    },
    "chgrp" : function(params) {
      var action_obj = {"group_id": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "chgrp", action_obj);
    },
    "chauth" : function(params) {
      var action_obj = {"auth_driver" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "chauth", action_obj);
    },
    "update": function(params) {
      var action_obj = {"template_raw" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
    },
    "append": function(params) {
      var action_obj = {"template_raw" : params.data.extra_param, append : true};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
    },
    "accounting" : function(params) {
      OpenNebulaAction.monitor(params, RESOURCE, false);
    },
    "set_quota" : function(params) {
      var action_obj = {quotas :  params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "set_quota", action_obj);
    },
    "addgroup" : function(params) {
      var action_obj = {"group_id": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "addgroup", action_obj);
    },
    "delgroup" : function(params) {
      var action_obj = {"group_id": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "delgroup", action_obj);
    },
    "getName": function(id){
      return OpenNebulaAction.getName(id, RESOURCE);
    }
  };

  return User;
});
define('opennebula/vdc',['require','./action'],function(require) {
  var OpenNebulaAction = require('./action');

  var RESOURCE = "VDC";

  var Vdc = {
    "resource": RESOURCE,
    "create" : function(params) {
      OpenNebulaAction.create(params, RESOURCE);
    },
    "del" : function(params) {
      OpenNebulaAction.del(params, RESOURCE);
    },
    "list" : function(params) {
      OpenNebulaAction.list(params, RESOURCE);
    },
    "show" : function(params) {
      OpenNebulaAction.show(params, RESOURCE);
    },
    "update" : function(params) {
      var action_obj = {"template_raw" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
    },
    "rename" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "rename", action_obj);
    },

    "add_group" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "add_group", action_obj);
    },
    "del_group" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "del_group", action_obj);
    },

    "add_cluster" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "add_cluster", action_obj);
    },
    "del_cluster" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "del_cluster", action_obj);
    },

    "add_host" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "add_host", action_obj);
    },
    "del_host" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "del_host", action_obj);
    },

    "add_datastore" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "add_datastore", action_obj);
    },
    "del_datastore" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "del_datastore", action_obj);
    },

    "add_vnet" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "add_vnet", action_obj);
    },
    "del_vnet" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "del_vnet", action_obj);
    },
    "getName": function(id){
      return OpenNebulaAction.getName(id, RESOURCE);
    }
  }

  return Vdc;
})
;
define('opennebula/vm',['require','./action','./helper','./error','utils/locale'],function(require) {
  var OpenNebulaAction = require('./action'),
      OpenNebulaHelper = require('./helper'),
      OpenNebulaError  = require('./error');
      Locale = require('utils/locale');

  var RESOURCE = "VM";

  var STATES_STR = [
    "INIT",
    "PENDING",
    "HOLD",
    "ACTIVE",
    "STOPPED",
    "SUSPENDED",
    "DONE",
    "FAILED",
    "POWEROFF",
    "UNDEPLOYED"
  ];

  var STATES = {
    INIT       : 0,
    PENDING    : 1,
    HOLD       : 2,
    ACTIVE     : 3,
    STOPPED    : 4,
    SUSPENDED  : 5,
    DONE       : 6,
    //FAILED   : 7,
    POWEROFF   : 8,
    UNDEPLOYED : 9
  };

  var LCM_STATES_STR = [
    "LCM_INIT",
    "PROLOG",
    "BOOT",
    "RUNNING",
    "MIGRATE",
    "SAVE_STOP",
    "SAVE_SUSPEND",
    "SAVE_MIGRATE",
    "PROLOG_MIGRATE",
    "PROLOG_RESUME",
    "EPILOG_STOP",
    "EPILOG",
    "SHUTDOWN",
    "CANCEL",
    "FAILURE",
    "CLEANUP_RESUBMIT",
    "UNKNOWN",
    "HOTPLUG",
    "SHUTDOWN_POWEROFF",
    "BOOT_UNKNOWN",
    "BOOT_POWEROFF",
    "BOOT_SUSPENDED",
    "BOOT_STOPPED",
    "CLEANUP_DELETE",
    "HOTPLUG_SNAPSHOT",
    "HOTPLUG_NIC",
    "HOTPLUG_SAVEAS",
    "HOTPLUG_SAVEAS_POWEROFF",
    "HOTPLUG_SAVEAS_SUSPENDED",
    "SHUTDOWN_UNDEPLOY",
    "EPILOG_UNDEPLOY",
    "PROLOG_UNDEPLOY",
    "BOOT_UNDEPLOY",
    "HOTPLUG_PROLOG_POWEROFF",
    "HOTPLUG_EPILOG_POWEROFF",
    "BOOT_MIGRATE",
    "BOOT_FAILURE",
    "BOOT_MIGRATE_FAILURE",
    "PROLOG_MIGRATE_FAILURE",
    "PROLOG_FAILURE",
    "EPILOG_FAILURE",
    "EPILOG_STOP_FAILURE",
    "EPILOG_UNDEPLOY_FAILURE",
    "PROLOG_MIGRATE_POWEROFF",
    "PROLOG_MIGRATE_POWEROFF_FAILURE",
    "PROLOG_MIGRATE_SUSPEND",
    "PROLOG_MIGRATE_SUSPEND_FAILURE",
    "BOOT_UNDEPLOY_FAILURE",
    "BOOT_STOPPED_FAILURE",
    "PROLOG_RESUME_FAILURE",
    "PROLOG_UNDEPLOY_FAILURE",
    "DISK_SNAPSHOT_POWEROFF",
    "DISK_SNAPSHOT_REVERT_POWEROFF",
    "DISK_SNAPSHOT_DELETE_POWEROFF",
    "DISK_SNAPSHOT_SUSPENDED",
    "DISK_SNAPSHOT_REVERT_SUSPENDED",
    "DISK_SNAPSHOT_DELETE_SUSPENDED",
    "DISK_SNAPSHOT",
    "DISK_SNAPSHOT_REVERT",
    "DISK_SNAPSHOT_DELETE",
  ];

  var LCM_STATES = {
    LCM_INIT                        : 0,
    PROLOG                          : 1,
    BOOT                            : 2,
    RUNNING                         : 3,
    MIGRATE                         : 4,
    SAVE_STOP                       : 5,
    SAVE_SUSPEND                    : 6,
    SAVE_MIGRATE                    : 7,
    PROLOG_MIGRATE                  : 8,
    PROLOG_RESUME                   : 9,
    EPILOG_STOP                     : 10,
    EPILOG                          : 11,
    SHUTDOWN                        : 12,
    //CANCEL                        : 13,
    //FAILURE                       : 14,
    CLEANUP_RESUBMIT                : 15,
    UNKNOWN                         : 16,
    HOTPLUG                         : 17,
    SHUTDOWN_POWEROFF               : 18,
    BOOT_UNKNOWN                    : 19,
    BOOT_POWEROFF                   : 20,
    BOOT_SUSPENDED                  : 21,
    BOOT_STOPPED                    : 22,
    CLEANUP_DELETE                  : 23,
    HOTPLUG_SNAPSHOT                : 24,
    HOTPLUG_NIC                     : 25,
    HOTPLUG_SAVEAS                  : 26,
    HOTPLUG_SAVEAS_POWEROFF         : 27,
    HOTPLUG_SAVEAS_SUSPENDED        : 28,
    SHUTDOWN_UNDEPLOY               : 29,
    EPILOG_UNDEPLOY                 : 30,
    PROLOG_UNDEPLOY                 : 31,
    BOOT_UNDEPLOY                   : 32,
    HOTPLUG_PROLOG_POWEROFF         : 33,
    HOTPLUG_EPILOG_POWEROFF         : 34,
    BOOT_MIGRATE                    : 35,
    BOOT_FAILURE                    : 36,
    BOOT_MIGRATE_FAILURE            : 37,
    PROLOG_MIGRATE_FAILURE          : 38,
    PROLOG_FAILURE                  : 39,
    EPILOG_FAILURE                  : 40,
    EPILOG_STOP_FAILURE             : 41,
    EPILOG_UNDEPLOY_FAILURE         : 42,
    PROLOG_MIGRATE_POWEROFF         : 43,
    PROLOG_MIGRATE_POWEROFF_FAILURE : 44,
    PROLOG_MIGRATE_SUSPEND          : 45,
    PROLOG_MIGRATE_SUSPEND_FAILURE  : 46,
    BOOT_UNDEPLOY_FAILURE           : 47,
    BOOT_STOPPED_FAILURE            : 48,
    PROLOG_RESUME_FAILURE           : 49,
    PROLOG_UNDEPLOY_FAILURE         : 50,
    DISK_SNAPSHOT_POWEROFF          : 51,
    DISK_SNAPSHOT_REVERT_POWEROFF   : 52,
    DISK_SNAPSHOT_DELETE_POWEROFF   : 53,
    DISK_SNAPSHOT_SUSPENDED         : 54,
    DISK_SNAPSHOT_REVERT_SUSPENDED  : 55,
    DISK_SNAPSHOT_DELETE_SUSPENDED  : 56,
    DISK_SNAPSHOT                   : 57,
    DISK_SNAPSHOT_REVERT            : 58,
    DISK_SNAPSHOT_DELETE            : 59
  };

  var SHORT_LCM_STATES_STR = [
    Locale.tr("LCM_INIT"),  // LCM_INIT
    Locale.tr("PROLOG"),    // PROLOG
    Locale.tr("BOOT"),      // BOOT
    Locale.tr("RUNNING"),   // RUNNING
    Locale.tr("MIGRATE"),   // MIGRATE
    Locale.tr("SAVE"),      // SAVE_STOP
    Locale.tr("SAVE"),      // SAVE_SUSPEND
    Locale.tr("SAVE"),      // SAVE_MIGRATE
    Locale.tr("MIGRATE"),   // PROLOG_MIGRATE
    Locale.tr("PROLOG"),    // PROLOG_RESUME
    Locale.tr("EPILOG"),    // EPILOG_STOP
    Locale.tr("EPILOG"),    // EPILOG
    Locale.tr("SHUTDOWN"),  // SHUTDOWN
    Locale.tr("SHUTDOWN"),  // CANCEL
    Locale.tr("FAILURE"),   // FAILURE
    Locale.tr("CLEANUP"),   // CLEANUP_RESUBMIT
    Locale.tr("UNKNOWN"),   // UNKNOWN
    Locale.tr("HOTPLUG"),   // HOTPLUG
    Locale.tr("SHUTDOWN"),  // SHUTDOWN_POWEROFF
    Locale.tr("BOOT"),      // BOOT_UNKNOWN
    Locale.tr("BOOT"),      // BOOT_POWEROFF
    Locale.tr("BOOT"),      // BOOT_SUSPENDED
    Locale.tr("BOOT"),      // BOOT_STOPPED
    Locale.tr("CLEANUP"),   // CLEANUP_DELETE
    Locale.tr("SNAPSHOT"),  // HOTPLUG_SNAPSHOT
    Locale.tr("HOTPLUG"),   // HOTPLUG_NIC
    Locale.tr("HOTPLUG"),   // HOTPLUG_SAVEAS
    Locale.tr("HOTPLUG"),   // HOTPLUG_SAVEAS_POWEROFF
    Locale.tr("HOTPLUG"),   // HOTPLUG_SAVEAS_SUSPENDED
    Locale.tr("SHUTDOWN"),  // SHUTDOWN_UNDEPLOY
    Locale.tr("EPILOG"),    // EPILOG_UNDEPLOY
    Locale.tr("PROLOG"),    // PROLOG_UNDEPLOY
    Locale.tr("BOOT"),      // BOOT_UNDEPLOY
    Locale.tr("HOTPLUG"),   // HOTPLUG_PROLOG_POWEROFF
    Locale.tr("HOTPLUG"),   // HOTPLUG_EPILOG_POWEROFF
    Locale.tr("BOOT"),      // BOOT_MIGRATE
    Locale.tr("FAILURE"),   // BOOT_FAILURE
    Locale.tr("FAILURE"),   // BOOT_MIGRATE_FAILURE
    Locale.tr("FAILURE"),   // PROLOG_MIGRATE_FAILURE
    Locale.tr("FAILURE"),   // PROLOG_FAILURE
    Locale.tr("FAILURE"),   // EPILOG_FAILURE
    Locale.tr("FAILURE"),   // EPILOG_STOP_FAILURE
    Locale.tr("FAILURE"),   // EPILOG_UNDEPLOY_FAILURE
    Locale.tr("MIGRATE"),   // PROLOG_MIGRATE_POWEROFF
    Locale.tr("FAILURE"),   // PROLOG_MIGRATE_POWEROFF_FAILURE
    Locale.tr("MIGRATE"),   // PROLOG_MIGRATE_SUSPEND
    Locale.tr("FAILURE"),   // PROLOG_MIGRATE_SUSPEND_FAILURE
    Locale.tr("FAILURE"),   // BOOT_UNDEPLOY_FAILURE
    Locale.tr("FAILURE"),   // BOOT_STOPPED_FAILURE
    Locale.tr("FAILURE"),   // PROLOG_RESUME_FAILURE
    Locale.tr("FAILURE"),   // PROLOG_UNDEPLOY_FAILURE
    Locale.tr("SNAPSHOT"),  // DISK_SNAPSHOT_POWEROFF
    Locale.tr("SNAPSHOT"),  // DISK_SNAPSHOT_REVERT_POWEROFF
    Locale.tr("SNAPSHOT"),  // DISK_SNAPSHOT_DELETE_POWEROFF
    Locale.tr("SNAPSHOT"),  // DISK_SNAPSHOT_SUSPENDED
    Locale.tr("SNAPSHOT"),  // DISK_SNAPSHOT_REVERT_SUSPENDED
    Locale.tr("SNAPSHOT"),  // DISK_SNAPSHOT_DELETE_SUSPENDED
    Locale.tr("SNAPSHOT"),  // DISK_SNAPSHOT
    Locale.tr("SNAPSHOT"),  // DISK_SNAPSHOT_REVERT
    Locale.tr("SNAPSHOT"),  // DISK_SNAPSHOT_DELETE
  ];

  var VNC_STATES = [
    LCM_STATES.RUNNING,
    LCM_STATES.MIGRATE,
    LCM_STATES.SHUTDOWN,
    LCM_STATES.CANCEL,
    LCM_STATES.UNKNOWN,
    LCM_STATES.HOTPLUG,
    LCM_STATES.SHUTDOWN_POWEROFF,
    LCM_STATES.HOTPLUG_SNAPSHOT,
    LCM_STATES.HOTPLUG_NIC,
    LCM_STATES.HOTPLUG_SAVEAS,
    LCM_STATES.HOTPLUG_SAVEAS_POWEROFF,
    LCM_STATES.HOTPLUG_SAVEAS_SUSPENDED,
    LCM_STATES.SHUTDOWN_UNDEPLOY
  ];

  var EXTERNAL_IP_ATTRS = [
    'GUEST_IP',
    'AWS_IP_ADDRESS',
    'AZ_IPADDRESS',
    'SL_PRIMARYIPADDRESS'
  ];

  var EXTERNAL_NETWORK_ATTRIBUTES = [
    'GUEST_IP',
    'AWS_IP_ADDRESS',
    'AWS_DNS_NAME',
    'AWS_PRIVATE_IP_ADDRESS',
    'AWS_PRIVATE_DNS_NAME',
    'AWS_SECURITY_GROUPS',
    'AZ_IPADDRESS',
    'SL_PRIMARYIPADDRESS'
  ];

  var MIGRATE_REASON = [
    "NONE",
    "ERROR",
    "USER"
  ];

  var MIGRATE_ACTION_STR = [
    "none",
    "migrate",
    "live-migrate",
    "shutdown",
    "shutdown-hard",
    "undeploy",
    "undeploy-hard",
    "hold",
    "release",
    "stop",
    "suspend",
    "resume",
    "boot",
    "delete",
    "delete-recreate",
    "reboot",
    "reboot-hard",
    "resched",
    "unresched",
    "poweroff",
    "poweroff-hard",
    "disk-attach",
    "disk-detach",
    "nic-attach",
    "nic-detach",
    "snap-create",
    "snap-delete"
  ];

  var VM = {
    "resource": RESOURCE,
    "create": function(params) {
      OpenNebulaAction.create(params, RESOURCE);
    },
    "del": function(params) {
      OpenNebulaAction.del(params, RESOURCE);
    },
    "list": function(params) {
      OpenNebulaAction.list(params, RESOURCE);
    },
    "show": function(params) {
      OpenNebulaAction.show(params, RESOURCE);
    },
    "chown" : function(params) {
      OpenNebulaAction.chown(params, RESOURCE);
    },
    "chgrp" : function(params) {
      OpenNebulaAction.chgrp(params, RESOURCE);
    },
    "chmod" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "chmod", action_obj);
    },
    "shutdown": function(params) {
      var action_obj = {"hard": false};
      OpenNebulaAction.simple_action(params, RESOURCE, "shutdown", action_obj);
    },
    "shutdown_hard" : function(params) {
      var action_obj = {"hard": true};
      OpenNebulaAction.simple_action(params, RESOURCE, "shutdown", action_obj);
    },
    "hold": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "hold");
    },
    "release": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "release");
    },
    "stop": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "stop");
    },
    "suspend": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "suspend");
    },
    "save_as_template": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "save_as_template");
    },
    "resume": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "resume");
    },
    "resubmit": function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "resubmit");
    },
    "poweroff" : function(params) {
      var action_obj = {"hard": false};
      OpenNebulaAction.simple_action(params, RESOURCE, "poweroff", action_obj);
    },
    "poweroff_hard" : function(params) {
      var action_obj = {"hard": true};
      OpenNebulaAction.simple_action(params, RESOURCE, "poweroff", action_obj);
    },
    "undeploy" : function(params) {
      var action_obj = {"hard": false};
      OpenNebulaAction.simple_action(params, RESOURCE, "undeploy", action_obj);
    },
    "undeploy_hard" : function(params) {
      var action_obj = {"hard": true};
      OpenNebulaAction.simple_action(params, RESOURCE, "undeploy", action_obj);
    },
    "reboot" : function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "reboot");
    },
    "reset" : function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "reset");
    },

    "log": function(params) {
      OpenNebulaAction.show(params, RESOURCE, "log");
    },
    "deploy": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "deploy", action_obj);
    },
    "livemigrate": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "livemigrate", action_obj);
    },
    "migrate": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "migrate", action_obj);
    },
    "disk_saveas": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "disk_saveas", action_obj);
    },
    "snapshot_create": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "snapshot_create", action_obj);
    },
    "snapshot_revert": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "snapshot_revert", action_obj);
    },
    "snapshot_delete": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "snapshot_delete", action_obj);
    },
    "disk_snapshot_create": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "disk_snapshot_create", action_obj);
    },
    "disk_snapshot_revert": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "disk_snapshot_revert", action_obj);
    },
    "disk_snapshot_delete": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "disk_snapshot_delete", action_obj);
    },
    "vnc" : function(params, startstop) {
      var callback = params.success;
      var callback_error = params.error;
      var id = params.data.id;
      var resource = RESOURCE;

      var method = startstop;
      var action = OpenNebulaHelper.action(method);
      var request = OpenNebulaHelper.request(resource, method, id);
      $.ajax({
        url: "vm/" + id + "/startvnc",
        type: "POST",
        dataType: "json",
        success: function(response) {
          return callback ? callback(request, response) : null;
        },
        error: function(response) {
          return callback_error ?
              callback_error(request, OpenNebulaError(response)) : null;
        }
      });
    },
    "update": function(params) {
      var action_obj = {"template_raw" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
    },
    "monitor" : function(params) {
      OpenNebulaAction.monitor(params, RESOURCE, false);
    },
    "pool_monitor" : function(params) {
      OpenNebulaAction.monitor(params, RESOURCE, true);
    },
    "resize" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "resize", action_obj);
    },
    "attachdisk" : function(params) {
      var action_obj = {"disk_template": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "attachdisk", action_obj);
    },
    "detachdisk" : function(params) {
      var action_obj = {"disk_id": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "detachdisk", action_obj);
    },
    "attachnic" : function(params) {
      var action_obj = {"nic_template": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "attachnic", action_obj);
    },
    "detachnic" : function(params) {
      var action_obj = {"nic_id": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "detachnic", action_obj);
    },
    "rename" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "rename", action_obj);
    },
    "resched" : function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "resched");
    },
    "unresched" : function(params) {
      OpenNebulaAction.simple_action(params, RESOURCE, "unresched");
    },
    "recover" : function(params) {
      var action_obj = {"result": params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "recover", action_obj);
    },
    "accounting": function(params) {
      OpenNebulaAction.accounting(params, RESOURCE);
    },
    "showback": function(params) {
      OpenNebulaAction.showback(params, RESOURCE);
    },
    "save_as_template": function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "save_as_template", action_obj);
    },
    "stateStr": function(stateId) {
      return STATES_STR[stateId];
    },
    "STATES": STATES,
    "lcmStateStr": function(stateId) {
      return LCM_STATES_STR[stateId];
    },
    "LCM_STATES": LCM_STATES,
    "shortLcmStateStr": function(stateId) {
      return SHORT_LCM_STATES_STR[stateId];
    },
    "isFailureState": function(lcmStateId) {
      switch(parseInt(lcmStateId)){
        case LCM_STATES.BOOT_FAILURE:
        case LCM_STATES.BOOT_MIGRATE_FAILURE:
        case LCM_STATES.PROLOG_MIGRATE_FAILURE:
        case LCM_STATES.PROLOG_FAILURE:
        case LCM_STATES.EPILOG_FAILURE:
        case LCM_STATES.EPILOG_STOP_FAILURE:
        case LCM_STATES.EPILOG_UNDEPLOY_FAILURE:
        case LCM_STATES.PROLOG_MIGRATE_POWEROFF_FAILURE:
        case LCM_STATES.PROLOG_MIGRATE_SUSPEND_FAILURE:
        case LCM_STATES.BOOT_UNDEPLOY_FAILURE:
        case LCM_STATES.BOOT_STOPPED_FAILURE:
        case LCM_STATES.PROLOG_RESUME_FAILURE:
        case LCM_STATES.PROLOG_UNDEPLOY_FAILURE:
          return true;

        default:
          return false;
      }
    },
    "hostnameStr": function(element) {
      var state = element.STATE;
      var hostname = "--";
      if (state == STATES.ACTIVE || state == STATES.SUSPENDED || state == STATES.POWEROFF) {
        var history = retrieveLastHistoryRecord(element)
        if (history) {
          hostname = history.HOSTNAME;
        };
      };

      return hostname;
    },
    "migrateActionStr": function(stateId) {
      return MIGRATE_ACTION_STR[stateId];
    },
    "migrateReasonStr": function(stateId) {
      return MIGRATE_REASON[stateId];
    },
    "ipsStr": ipsStr,
    "retrieveExternalIPs": retrieveExternalIPs,
    "retrieveExternalNetworkAttrs": retrieveExternalNetworkAttrs,
    "isNICGraphsSupported": isNICGraphsSupported,
    "isNICAttachSupported": isNICAttachSupported,
    "isVNCSupported": isVNCSupported,
    "isSPICESupported": isSPICESupported,
    "getName": function(id){
      return OpenNebulaAction.getName(id, RESOURCE);
    }
  }

  function retrieveLastHistoryRecord(element) {
    if (element.HISTORY_RECORDS && element.HISTORY_RECORDS.HISTORY) {
      var history = element.HISTORY_RECORDS.HISTORY;
      if (history.constructor == Array) {
        return history[history.length - 1];
      } else {
        return history;
      };
    } else {
      return null;
    }
  }

  // Return true if the VM has a hybrid section
  function isNICGraphsSupported(element) {
    var history = retrieveLastHistoryRecord(element)
    if (history) {
      return $.inArray(history.VMMMAD, ['vcenter', 'ec2', 'az', 'sl']) == -1;
    } else {
      return false;
    }
  }

  function isNICAttachSupported(element) {
    var history = retrieveLastHistoryRecord(element)
    if (history) {
      return $.inArray(history.VMMMAD, ['ec2', 'az', 'sl']) == -1;
    } else {
      return false;
    }
  }

  function retrieveExternalIPs(element) {
    var monitoring = element.MONITORING;
    var ips = {};
    var externalIP;

    $.each(EXTERNAL_IP_ATTRS, function(index, IPAttr) {
      externalIP = monitoring[IPAttr];
      if (externalIP) {
        ips[IPAttr] = externalIP;
      }
    });

    return ips;
  }

  function retrieveExternalNetworkAttrs(element) {
    var ips = {};
    var externalAttr;

    var monitoring = element.MONITORING;
    if (monitoring) {
      $.each(EXTERNAL_NETWORK_ATTRIBUTES, function(index, attr) {
        externalAttr = monitoring[attr];
        if (externalAttr) {
          ips[attr] = externalAttr;
        }
      });
    }

    return ips;
  }

  // Return the IP or several IPs of a VM
  function ipsStr(element, divider) {
    var divider = divider || "<br>"
    var nic = element.TEMPLATE.NIC;
    var ips = [];

    var monitoring = element.MONITORING;
    if (monitoring) {
      var externalIP;
      $.each(EXTERNAL_IP_ATTRS, function(index, IPAttr) {
        externalIP = monitoring[IPAttr];
        if (externalIP && ($.inArray(externalIP, ips) == -1)) {
          ips.push(externalIP);
        }
      })
    }

    if(ips.length==0)
    {
      if (nic != undefined) {
        if (!$.isArray(nic)) {
          nic = [nic];
        }

        $.each(nic, function(index, value) {
          if (value.IP) {
            ips.push(value.IP);
          }

          if (value.IP6_GLOBAL) {
            ips.push(value.IP6_GLOBAL);
          }

          if (value.IP6_ULA) {
            ips.push(value.IP6_ULA);
          }
        });
      }
    }

    if (ips.length > 0) {
      return ips.join(divider);
    } else {
      return '--';
    }
  };

  // returns true if the vnc button should be enabled
  function isVNCSupported(element) {
    var graphics = element.TEMPLATE.GRAPHICS;
    var state = parseInt(element.LCM_STATE);

    return (graphics &&
        graphics.TYPE &&
        graphics.TYPE.toLowerCase() == "vnc"  &&
        $.inArray(state, VNC_STATES) != -1);
  }

  function isSPICESupported(element) {
    var graphics = element.TEMPLATE.GRAPHICS;
    var state = parseInt(element.LCM_STATE);

    return (graphics &&
        graphics.TYPE &&
        graphics.TYPE.toLowerCase() == "spice" &&
        $.inArray(state, VNC_STATES) != -1);
  }

  return VM;
})
;
define('opennebula/zone',['require','./action'],function(require) {
  var OpenNebulaAction = require('./action');

  var RESOURCE = "ZONE";

  var Zone = {
    "resource": RESOURCE,
    "create" : function(params) {
      OpenNebulaAction.create(params, RESOURCE);
    },
    "del" : function(params) {
      OpenNebulaAction.del(params, RESOURCE);
    },
    "list" : function(params) {
      OpenNebulaAction.list(params, RESOURCE);
    },
    "show" : function(params) {
      OpenNebulaAction.show(params, RESOURCE);
    },
    "update" : function(params) {
      var action_obj = {"template_raw" : params.data.extra_param};
      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
    },
    "rename" : function(params) {
      var action_obj = params.data.extra_param;
      OpenNebulaAction.simple_action(params, RESOURCE, "rename", action_obj);
    },
    "getName": function(id){
      return OpenNebulaAction.getName(id, RESOURCE);
    }
  };

  return Zone;
});
define('opennebula',['require','jquery','./opennebula/helper','./opennebula/action','./opennebula/auth','./opennebula/error','./opennebula/acl','./opennebula/cluster','./opennebula/datastore','./opennebula/group','./opennebula/host','./opennebula/image','./opennebula/marketplace','./opennebula/network','./opennebula/role','./opennebula/securitygroup','./opennebula/service','./opennebula/servicetemplate','./opennebula/support','./opennebula/template','./opennebula/user','./opennebula/vdc','./opennebula/vm','./opennebula/zone'],function(require) {
  require('jquery');

  var Helper = require('./opennebula/helper'),
      Action = require('./opennebula/action'),
      Auth   = require('./opennebula/auth'),
      Error  = require('./opennebula/error'),

      Acl             = require('./opennebula/acl'),
      Cluster         = require('./opennebula/cluster'),
      Datastore       = require('./opennebula/datastore'),
      Group           = require('./opennebula/group'),
      Host            = require('./opennebula/host'),
      Image           = require('./opennebula/image'),
      Marketplace     = require('./opennebula/marketplace'),
      Network         = require('./opennebula/network'),
      Role            = require('./opennebula/role'),
      securitygroup   = require('./opennebula/securitygroup'),
      Service         = require('./opennebula/service'),
      ServiceTemplate = require('./opennebula/servicetemplate'),
      Support         = require('./opennebula/support'),
      Template        = require('./opennebula/template'),
      User            = require('./opennebula/user'),
      Vdc             = require('./opennebula/vdc'),
      Vm              = require('./opennebula/vm'),
      Zone            = require('./opennebula/zone')

  if (typeof(csrftoken) != "undefined") {
    $.ajaxPrefilter(function(options, originalOptions, jqXHR) {
      var params = originalOptions.data;

      if (typeof(params) == "string") {
        params = JSON.parse(params);
        params["csrftoken"] = csrftoken;
        options.data = JSON.stringify(params);
      } else {
        params = params || {};
        params["csrftoken"] = csrftoken;
        options.data = $.param(params);
      }
    });
  }

  $.ajaxSetup({
    converters: {
      "text json": function(textValue) {
        return jQuery.parseJSON(jQuery('<div/>').text(textValue).html());
      }
    }
  });

  var OpenNebula = {
    'Helper': Helper,
    'Action': Action,
    'Auth': Auth,
    'Error': Error,
    'Acl': Acl,
    'Cluster': Cluster,
    'Datastore': Datastore,
    'Group': Group,
    'Host': Host,
    'Image': Image,
    'File': Image,
    'Marketplace': Marketplace,
    'Network': Network,
    'Role': Role,
    'SecurityGroup': securitygroup,
    'Service': Service,
    'ServiceTemplate': ServiceTemplate,
    'Support': Support,
    'Template': Template,
    'User': User,
    'Vdc': Vdc,
    'VM': Vm,
    'Zone': Zone
  }

  return OpenNebula;
});

;(function ($, window, document, undefined) {
  'use strict';

  Foundation.libs.accordion = {
    name : 'accordion',

    version : '5.5.2',

    settings : {
      content_class : 'content',
      active_class : 'active',
      multi_expand : false,
      toggleable : true,
      callback : function () {}
    },

    init : function (scope, method, options) {
      this.bindings(method, options);
    },

    events : function (instance) {
      var self = this;
      var S = this.S;
      self.create(this.S(instance));

      S(this.scope)
      .off('.fndtn.accordion')
      .on('click.fndtn.accordion', '[' + this.attr_name() + '] > dd > a, [' + this.attr_name() + '] > li > a', function (e) {
        var accordion = S(this).closest('[' + self.attr_name() + ']'),
            groupSelector = self.attr_name() + '=' + accordion.attr(self.attr_name()),
            settings = accordion.data(self.attr_name(true) + '-init') || self.settings,
            target = S('#' + this.href.split('#')[1]),
            aunts = $('> dd, > li', accordion),
            siblings = aunts.children('.' + settings.content_class),
            active_content = siblings.filter('.' + settings.active_class);

        e.preventDefault();

        if (accordion.attr(self.attr_name())) {
          siblings = siblings.add('[' + groupSelector + '] dd > ' + '.' + settings.content_class + ', [' + groupSelector + '] li > ' + '.' + settings.content_class);
          aunts = aunts.add('[' + groupSelector + '] dd, [' + groupSelector + '] li');
        }

        if (settings.toggleable && target.is(active_content)) {
          target.parent('dd, li').toggleClass(settings.active_class, false);
          target.toggleClass(settings.active_class, false);
          S(this).attr('aria-expanded', function(i, attr){
              return attr === 'true' ? 'false' : 'true';
          });
          settings.callback(target);
          target.triggerHandler('toggled', [accordion]);
          accordion.triggerHandler('toggled', [target]);
          return;
        }

        if (!settings.multi_expand) {
          siblings.removeClass(settings.active_class);
          aunts.removeClass(settings.active_class);
          aunts.children('a').attr('aria-expanded','false');
        }

        target.addClass(settings.active_class).parent().addClass(settings.active_class);
        settings.callback(target);
        target.triggerHandler('toggled', [accordion]);
        accordion.triggerHandler('toggled', [target]);
        S(this).attr('aria-expanded','true');
      });
    },

    create: function($instance) {
      var self = this,
          accordion = $instance,
          aunts = $('> .accordion-navigation', accordion),
          settings = accordion.data(self.attr_name(true) + '-init') || self.settings;

      aunts.children('a').attr('aria-expanded','false');
      aunts.has('.' + settings.content_class + '.' + settings.active_class).children('a').attr('aria-expanded','true');

      if (settings.multi_expand) {
        $instance.attr('aria-multiselectable','true');
      }
    },

    off : function () {},

    reflow : function () {}
  };
}(jQuery, window, window.document));

define("foundation.accordion", ["foundation.core"], function(){});

/*! DataTables 1.10.7
 * ©2008-2014 SpryMedia Ltd - datatables.net/license
 */

/**
 * @summary     DataTables
 * @description Paginate, search and order HTML tables
 * @version     1.10.7
 * @file        jquery.dataTables.js
 * @author      SpryMedia Ltd (www.sprymedia.co.uk)
 * @contact     www.sprymedia.co.uk/contact
 * @copyright   Copyright 2008-2014 SpryMedia Ltd.
 *
 * This source file is free software, available under the following license:
 *   MIT license - http://datatables.net/license
 *
 * This source file is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
 *
 * For details please refer to: http://www.datatables.net
 */

/*jslint evil: true, undef: true, browser: true */
/*globals $,require,jQuery,define,_selector_run,_selector_opts,_selector_first,_selector_row_indexes,_ext,_Api,_api_register,_api_registerPlural,_re_new_lines,_re_html,_re_formatted_numeric,_re_escape_regex,_empty,_intVal,_numToDecimal,_isNumber,_isHtml,_htmlNumeric,_pluck,_pluck_order,_range,_stripHtml,_unique,_fnBuildAjax,_fnAjaxUpdate,_fnAjaxParameters,_fnAjaxUpdateDraw,_fnAjaxDataSrc,_fnAddColumn,_fnColumnOptions,_fnAdjustColumnSizing,_fnVisibleToColumnIndex,_fnColumnIndexToVisible,_fnVisbleColumns,_fnGetColumns,_fnColumnTypes,_fnApplyColumnDefs,_fnHungarianMap,_fnCamelToHungarian,_fnLanguageCompat,_fnBrowserDetect,_fnAddData,_fnAddTr,_fnNodeToDataIndex,_fnNodeToColumnIndex,_fnGetCellData,_fnSetCellData,_fnSplitObjNotation,_fnGetObjectDataFn,_fnSetObjectDataFn,_fnGetDataMaster,_fnClearTable,_fnDeleteIndex,_fnInvalidate,_fnGetRowElements,_fnCreateTr,_fnBuildHead,_fnDrawHead,_fnDraw,_fnReDraw,_fnAddOptionsHtml,_fnDetectHeader,_fnGetUniqueThs,_fnFeatureHtmlFilter,_fnFilterComplete,_fnFilterCustom,_fnFilterColumn,_fnFilter,_fnFilterCreateSearch,_fnEscapeRegex,_fnFilterData,_fnFeatureHtmlInfo,_fnUpdateInfo,_fnInfoMacros,_fnInitialise,_fnInitComplete,_fnLengthChange,_fnFeatureHtmlLength,_fnFeatureHtmlPaginate,_fnPageChange,_fnFeatureHtmlProcessing,_fnProcessingDisplay,_fnFeatureHtmlTable,_fnScrollDraw,_fnApplyToChildren,_fnCalculateColumnWidths,_fnThrottle,_fnConvertToWidth,_fnScrollingWidthAdjust,_fnGetWidestNode,_fnGetMaxLenString,_fnStringToCss,_fnScrollBarWidth,_fnSortFlatten,_fnSort,_fnSortAria,_fnSortListener,_fnSortAttachListener,_fnSortingClasses,_fnSortData,_fnSaveState,_fnLoadState,_fnSettingsFromNode,_fnLog,_fnMap,_fnBindAction,_fnCallbackReg,_fnCallbackFire,_fnLengthOverflow,_fnRenderer,_fnDataSource,_fnRowAttributes*/

(/** @lends <global> */function( window, document, undefined ) {

(function( factory ) {
	"use strict";

	if ( typeof define === 'function' && define.amd ) {
		// Define as an AMD module if possible
		define( 'datatables', ['jquery'], factory );
	}
    else if ( typeof exports === 'object' ) {
        // Node/CommonJS
        module.exports = factory( require( 'jquery' ) );
    }
	else if ( jQuery && !jQuery.fn.dataTable ) {
		// Define using browser globals otherwise
		// Prevent multiple instantiations if the script is loaded twice
		factory( jQuery );
	}
}
(/** @lends <global> */function( $ ) {
	"use strict";

	/**
	 * DataTables is a plug-in for the jQuery Javascript library. It is a highly
	 * flexible tool, based upon the foundations of progressive enhancement,
	 * which will add advanced interaction controls to any HTML table. For a
	 * full list of features please refer to
	 * [DataTables.net](href="http://datatables.net).
	 *
	 * Note that the `DataTable` object is not a global variable but is aliased
	 * to `jQuery.fn.DataTable` and `jQuery.fn.dataTable` through which it may
	 * be  accessed.
	 *
	 *  @class
	 *  @param {object} [init={}] Configuration object for DataTables. Options
	 *    are defined by {@link DataTable.defaults}
	 *  @requires jQuery 1.7+
	 *
	 *  @example
	 *    // Basic initialisation
	 *    $(document).ready( function {
	 *      $('#example').dataTable();
	 *    } );
	 *
	 *  @example
	 *    // Initialisation with configuration options - in this case, disable
	 *    // pagination and sorting.
	 *    $(document).ready( function {
	 *      $('#example').dataTable( {
	 *        "paginate": false,
	 *        "sort": false
	 *      } );
	 *    } );
	 */
	var DataTable;

	
	/*
	 * It is useful to have variables which are scoped locally so only the
	 * DataTables functions can access them and they don't leak into global space.
	 * At the same time these functions are often useful over multiple files in the
	 * core and API, so we list, or at least document, all variables which are used
	 * by DataTables as private variables here. This also ensures that there is no
	 * clashing of variable names and that they can easily referenced for reuse.
	 */
	
	
	// Defined else where
	//  _selector_run
	//  _selector_opts
	//  _selector_first
	//  _selector_row_indexes
	
	var _ext; // DataTable.ext
	var _Api; // DataTable.Api
	var _api_register; // DataTable.Api.register
	var _api_registerPlural; // DataTable.Api.registerPlural
	
	var _re_dic = {};
	var _re_new_lines = /[\r\n]/g;
	var _re_html = /<.*?>/g;
	var _re_date_start = /^[\w\+\-]/;
	var _re_date_end = /[\w\+\-]$/;
	
	// Escape regular expression special characters
	var _re_escape_regex = new RegExp( '(\\' + [ '/', '.', '*', '+', '?', '|', '(', ')', '[', ']', '{', '}', '\\', '$', '^', '-' ].join('|\\') + ')', 'g' );
	
	// http://en.wikipedia.org/wiki/Foreign_exchange_market
	// - \u20BD - Russian ruble.
	// - \u20a9 - South Korean Won
	// - \u20BA - Turkish Lira
	// - \u20B9 - Indian Rupee
	// - R - Brazil (R$) and South Africa
	// - fr - Swiss Franc
	// - kr - Swedish krona, Norwegian krone and Danish krone
	// - \u2009 is thin space and \u202F is narrow no-break space, both used in many
	//   standards as thousands separators.
	var _re_formatted_numeric = /[',$£€¥%\u2009\u202F\u20BD\u20a9\u20BArfk]/gi;
	
	
	var _empty = function ( d ) {
		return !d || d === true || d === '-' ? true : false;
	};
	
	
	var _intVal = function ( s ) {
		var integer = parseInt( s, 10 );
		return !isNaN(integer) && isFinite(s) ? integer : null;
	};
	
	// Convert from a formatted number with characters other than `.` as the
	// decimal place, to a Javascript number
	var _numToDecimal = function ( num, decimalPoint ) {
		// Cache created regular expressions for speed as this function is called often
		if ( ! _re_dic[ decimalPoint ] ) {
			_re_dic[ decimalPoint ] = new RegExp( _fnEscapeRegex( decimalPoint ), 'g' );
		}
		return typeof num === 'string' && decimalPoint !== '.' ?
			num.replace( /\./g, '' ).replace( _re_dic[ decimalPoint ], '.' ) :
			num;
	};
	
	
	var _isNumber = function ( d, decimalPoint, formatted ) {
		var strType = typeof d === 'string';
	
		// If empty return immediately so there must be a number if it is a
		// formatted string (this stops the string "k", or "kr", etc being detected
		// as a formatted number for currency
		if ( _empty( d ) ) {
			return true;
		}
	
		if ( decimalPoint && strType ) {
			d = _numToDecimal( d, decimalPoint );
		}
	
		if ( formatted && strType ) {
			d = d.replace( _re_formatted_numeric, '' );
		}
	
		return !isNaN( parseFloat(d) ) && isFinite( d );
	};
	
	
	// A string without HTML in it can be considered to be HTML still
	var _isHtml = function ( d ) {
		return _empty( d ) || typeof d === 'string';
	};
	
	
	var _htmlNumeric = function ( d, decimalPoint, formatted ) {
		if ( _empty( d ) ) {
			return true;
		}
	
		var html = _isHtml( d );
		return ! html ?
			null :
			_isNumber( _stripHtml( d ), decimalPoint, formatted ) ?
				true :
				null;
	};
	
	
	var _pluck = function ( a, prop, prop2 ) {
		var out = [];
		var i=0, ien=a.length;
	
		// Could have the test in the loop for slightly smaller code, but speed
		// is essential here
		if ( prop2 !== undefined ) {
			for ( ; i<ien ; i++ ) {
				if ( a[i] && a[i][ prop ] ) {
					out.push( a[i][ prop ][ prop2 ] );
				}
			}
		}
		else {
			for ( ; i<ien ; i++ ) {
				if ( a[i] ) {
					out.push( a[i][ prop ] );
				}
			}
		}
	
		return out;
	};
	
	
	// Basically the same as _pluck, but rather than looping over `a` we use `order`
	// as the indexes to pick from `a`
	var _pluck_order = function ( a, order, prop, prop2 )
	{
		var out = [];
		var i=0, ien=order.length;
	
		// Could have the test in the loop for slightly smaller code, but speed
		// is essential here
		if ( prop2 !== undefined ) {
			for ( ; i<ien ; i++ ) {
				if ( a[ order[i] ][ prop ] ) {
					out.push( a[ order[i] ][ prop ][ prop2 ] );
				}
			}
		}
		else {
			for ( ; i<ien ; i++ ) {
				out.push( a[ order[i] ][ prop ] );
			}
		}
	
		return out;
	};
	
	
	var _range = function ( len, start )
	{
		var out = [];
		var end;
	
		if ( start === undefined ) {
			start = 0;
			end = len;
		}
		else {
			end = start;
			start = len;
		}
	
		for ( var i=start ; i<end ; i++ ) {
			out.push( i );
		}
	
		return out;
	};
	
	
	var _removeEmpty = function ( a )
	{
		var out = [];
	
		for ( var i=0, ien=a.length ; i<ien ; i++ ) {
			if ( a[i] ) { // careful - will remove all falsy values!
				out.push( a[i] );
			}
		}
	
		return out;
	};
	
	
	var _stripHtml = function ( d ) {
		return d.replace( _re_html, '' );
	};
	
	
	/**
	 * Find the unique elements in a source array.
	 *
	 * @param  {array} src Source array
	 * @return {array} Array of unique items
	 * @ignore
	 */
	var _unique = function ( src )
	{
		// A faster unique method is to use object keys to identify used values,
		// but this doesn't work with arrays or objects, which we must also
		// consider. See jsperf.com/compare-array-unique-versions/4 for more
		// information.
		var
			out = [],
			val,
			i, ien=src.length,
			j, k=0;
	
		again: for ( i=0 ; i<ien ; i++ ) {
			val = src[i];
	
			for ( j=0 ; j<k ; j++ ) {
				if ( out[j] === val ) {
					continue again;
				}
			}
	
			out.push( val );
			k++;
		}
	
		return out;
	};
	
	
	
	/**
	 * Create a mapping object that allows camel case parameters to be looked up
	 * for their Hungarian counterparts. The mapping is stored in a private
	 * parameter called `_hungarianMap` which can be accessed on the source object.
	 *  @param {object} o
	 *  @memberof DataTable#oApi
	 */
	function _fnHungarianMap ( o )
	{
		var
			hungarian = 'a aa ai ao as b fn i m o s ',
			match,
			newKey,
			map = {};
	
		$.each( o, function (key, val) {
			match = key.match(/^([^A-Z]+?)([A-Z])/);
	
			if ( match && hungarian.indexOf(match[1]+' ') !== -1 )
			{
				newKey = key.replace( match[0], match[2].toLowerCase() );
				map[ newKey ] = key;
	
				if ( match[1] === 'o' )
				{
					_fnHungarianMap( o[key] );
				}
			}
		} );
	
		o._hungarianMap = map;
	}
	
	
	/**
	 * Convert from camel case parameters to Hungarian, based on a Hungarian map
	 * created by _fnHungarianMap.
	 *  @param {object} src The model object which holds all parameters that can be
	 *    mapped.
	 *  @param {object} user The object to convert from camel case to Hungarian.
	 *  @param {boolean} force When set to `true`, properties which already have a
	 *    Hungarian value in the `user` object will be overwritten. Otherwise they
	 *    won't be.
	 *  @memberof DataTable#oApi
	 */
	function _fnCamelToHungarian ( src, user, force )
	{
		if ( ! src._hungarianMap ) {
			_fnHungarianMap( src );
		}
	
		var hungarianKey;
	
		$.each( user, function (key, val) {
			hungarianKey = src._hungarianMap[ key ];
	
			if ( hungarianKey !== undefined && (force || user[hungarianKey] === undefined) )
			{
				// For objects, we need to buzz down into the object to copy parameters
				if ( hungarianKey.charAt(0) === 'o' )
				{
					// Copy the camelCase options over to the hungarian
					if ( ! user[ hungarianKey ] ) {
						user[ hungarianKey ] = {};
					}
					$.extend( true, user[hungarianKey], user[key] );
	
					_fnCamelToHungarian( src[hungarianKey], user[hungarianKey], force );
				}
				else {
					user[hungarianKey] = user[ key ];
				}
			}
		} );
	}
	
	
	/**
	 * Language compatibility - when certain options are given, and others aren't, we
	 * need to duplicate the values over, in order to provide backwards compatibility
	 * with older language files.
	 *  @param {object} oSettings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnLanguageCompat( lang )
	{
		var defaults = DataTable.defaults.oLanguage;
		var zeroRecords = lang.sZeroRecords;
	
		/* Backwards compatibility - if there is no sEmptyTable given, then use the same as
		 * sZeroRecords - assuming that is given.
		 */
		if ( ! lang.sEmptyTable && zeroRecords &&
			defaults.sEmptyTable === "No data available in table" )
		{
			_fnMap( lang, lang, 'sZeroRecords', 'sEmptyTable' );
		}
	
		/* Likewise with loading records */
		if ( ! lang.sLoadingRecords && zeroRecords &&
			defaults.sLoadingRecords === "Loading..." )
		{
			_fnMap( lang, lang, 'sZeroRecords', 'sLoadingRecords' );
		}
	
		// Old parameter name of the thousands separator mapped onto the new
		if ( lang.sInfoThousands ) {
			lang.sThousands = lang.sInfoThousands;
		}
	
		var decimal = lang.sDecimal;
		if ( decimal ) {
			_addNumericSort( decimal );
		}
	}
	
	
	/**
	 * Map one parameter onto another
	 *  @param {object} o Object to map
	 *  @param {*} knew The new parameter name
	 *  @param {*} old The old parameter name
	 */
	var _fnCompatMap = function ( o, knew, old ) {
		if ( o[ knew ] !== undefined ) {
			o[ old ] = o[ knew ];
		}
	};
	
	
	/**
	 * Provide backwards compatibility for the main DT options. Note that the new
	 * options are mapped onto the old parameters, so this is an external interface
	 * change only.
	 *  @param {object} init Object to map
	 */
	function _fnCompatOpts ( init )
	{
		_fnCompatMap( init, 'ordering',      'bSort' );
		_fnCompatMap( init, 'orderMulti',    'bSortMulti' );
		_fnCompatMap( init, 'orderClasses',  'bSortClasses' );
		_fnCompatMap( init, 'orderCellsTop', 'bSortCellsTop' );
		_fnCompatMap( init, 'order',         'aaSorting' );
		_fnCompatMap( init, 'orderFixed',    'aaSortingFixed' );
		_fnCompatMap( init, 'paging',        'bPaginate' );
		_fnCompatMap( init, 'pagingType',    'sPaginationType' );
		_fnCompatMap( init, 'pageLength',    'iDisplayLength' );
		_fnCompatMap( init, 'searching',     'bFilter' );
	
		// Column search objects are in an array, so it needs to be converted
		// element by element
		var searchCols = init.aoSearchCols;
	
		if ( searchCols ) {
			for ( var i=0, ien=searchCols.length ; i<ien ; i++ ) {
				if ( searchCols[i] ) {
					_fnCamelToHungarian( DataTable.models.oSearch, searchCols[i] );
				}
			}
		}
	}
	
	
	/**
	 * Provide backwards compatibility for column options. Note that the new options
	 * are mapped onto the old parameters, so this is an external interface change
	 * only.
	 *  @param {object} init Object to map
	 */
	function _fnCompatCols ( init )
	{
		_fnCompatMap( init, 'orderable',     'bSortable' );
		_fnCompatMap( init, 'orderData',     'aDataSort' );
		_fnCompatMap( init, 'orderSequence', 'asSorting' );
		_fnCompatMap( init, 'orderDataType', 'sortDataType' );
	
		// orderData can be given as an integer
		var dataSort = init.aDataSort;
		if ( dataSort && ! $.isArray( dataSort ) ) {
			init.aDataSort = [ dataSort ];
		}
	}
	
	
	/**
	 * Browser feature detection for capabilities, quirks
	 *  @param {object} settings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnBrowserDetect( settings )
	{
		var browser = settings.oBrowser;
	
		// Scrolling feature / quirks detection
		var n = $('<div/>')
			.css( {
				position: 'absolute',
				top: 0,
				left: 0,
				height: 1,
				width: 1,
				overflow: 'hidden'
			} )
			.append(
				$('<div/>')
					.css( {
						position: 'absolute',
						top: 1,
						left: 1,
						width: 100,
						overflow: 'scroll'
					} )
					.append(
						$('<div class="test"/>')
							.css( {
								width: '100%',
								height: 10
							} )
					)
			)
			.appendTo( 'body' );
	
		var test = n.find('.test');
	
		// IE6/7 will oversize a width 100% element inside a scrolling element, to
		// include the width of the scrollbar, while other browsers ensure the inner
		// element is contained without forcing scrolling
		browser.bScrollOversize = test[0].offsetWidth === 100;
	
		// In rtl text layout, some browsers (most, but not all) will place the
		// scrollbar on the left, rather than the right.
		browser.bScrollbarLeft = Math.round( test.offset().left ) !== 1;
	
		n.remove();
	}
	
	
	/**
	 * Array.prototype reduce[Right] method, used for browsers which don't support
	 * JS 1.6. Done this way to reduce code size, since we iterate either way
	 *  @param {object} settings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnReduce ( that, fn, init, start, end, inc )
	{
		var
			i = start,
			value,
			isSet = false;
	
		if ( init !== undefined ) {
			value = init;
			isSet = true;
		}
	
		while ( i !== end ) {
			if ( ! that.hasOwnProperty(i) ) {
				continue;
			}
	
			value = isSet ?
				fn( value, that[i], i, that ) :
				that[i];
	
			isSet = true;
			i += inc;
		}
	
		return value;
	}
	
	/**
	 * Add a column to the list used for the table with default values
	 *  @param {object} oSettings dataTables settings object
	 *  @param {node} nTh The th element for this column
	 *  @memberof DataTable#oApi
	 */
	function _fnAddColumn( oSettings, nTh )
	{
		// Add column to aoColumns array
		var oDefaults = DataTable.defaults.column;
		var iCol = oSettings.aoColumns.length;
		var oCol = $.extend( {}, DataTable.models.oColumn, oDefaults, {
			"nTh": nTh ? nTh : document.createElement('th'),
			"sTitle":    oDefaults.sTitle    ? oDefaults.sTitle    : nTh ? nTh.innerHTML : '',
			"aDataSort": oDefaults.aDataSort ? oDefaults.aDataSort : [iCol],
			"mData": oDefaults.mData ? oDefaults.mData : iCol,
			idx: iCol
		} );
		oSettings.aoColumns.push( oCol );
	
		// Add search object for column specific search. Note that the `searchCols[ iCol ]`
		// passed into extend can be undefined. This allows the user to give a default
		// with only some of the parameters defined, and also not give a default
		var searchCols = oSettings.aoPreSearchCols;
		searchCols[ iCol ] = $.extend( {}, DataTable.models.oSearch, searchCols[ iCol ] );
	
		// Use the default column options function to initialise classes etc
		_fnColumnOptions( oSettings, iCol, $(nTh).data() );
	}
	
	
	/**
	 * Apply options for a column
	 *  @param {object} oSettings dataTables settings object
	 *  @param {int} iCol column index to consider
	 *  @param {object} oOptions object with sType, bVisible and bSearchable etc
	 *  @memberof DataTable#oApi
	 */
	function _fnColumnOptions( oSettings, iCol, oOptions )
	{
		var oCol = oSettings.aoColumns[ iCol ];
		var oClasses = oSettings.oClasses;
		var th = $(oCol.nTh);
	
		// Try to get width information from the DOM. We can't get it from CSS
		// as we'd need to parse the CSS stylesheet. `width` option can override
		if ( ! oCol.sWidthOrig ) {
			// Width attribute
			oCol.sWidthOrig = th.attr('width') || null;
	
			// Style attribute
			var t = (th.attr('style') || '').match(/width:\s*(\d+[pxem%]+)/);
			if ( t ) {
				oCol.sWidthOrig = t[1];
			}
		}
	
		/* User specified column options */
		if ( oOptions !== undefined && oOptions !== null )
		{
			// Backwards compatibility
			_fnCompatCols( oOptions );
	
			// Map camel case parameters to their Hungarian counterparts
			_fnCamelToHungarian( DataTable.defaults.column, oOptions );
	
			/* Backwards compatibility for mDataProp */
			if ( oOptions.mDataProp !== undefined && !oOptions.mData )
			{
				oOptions.mData = oOptions.mDataProp;
			}
	
			if ( oOptions.sType )
			{
				oCol._sManualType = oOptions.sType;
			}
	
			// `class` is a reserved word in Javascript, so we need to provide
			// the ability to use a valid name for the camel case input
			if ( oOptions.className && ! oOptions.sClass )
			{
				oOptions.sClass = oOptions.className;
			}
	
			$.extend( oCol, oOptions );
			_fnMap( oCol, oOptions, "sWidth", "sWidthOrig" );
	
			/* iDataSort to be applied (backwards compatibility), but aDataSort will take
			 * priority if defined
			 */
			if ( oOptions.iDataSort !== undefined )
			{
				oCol.aDataSort = [ oOptions.iDataSort ];
			}
			_fnMap( oCol, oOptions, "aDataSort" );
		}
	
		/* Cache the data get and set functions for speed */
		var mDataSrc = oCol.mData;
		var mData = _fnGetObjectDataFn( mDataSrc );
		var mRender = oCol.mRender ? _fnGetObjectDataFn( oCol.mRender ) : null;
	
		var attrTest = function( src ) {
			return typeof src === 'string' && src.indexOf('@') !== -1;
		};
		oCol._bAttrSrc = $.isPlainObject( mDataSrc ) && (
			attrTest(mDataSrc.sort) || attrTest(mDataSrc.type) || attrTest(mDataSrc.filter)
		);
	
		oCol.fnGetData = function (rowData, type, meta) {
			var innerData = mData( rowData, type, undefined, meta );
	
			return mRender && type ?
				mRender( innerData, type, rowData, meta ) :
				innerData;
		};
		oCol.fnSetData = function ( rowData, val, meta ) {
			return _fnSetObjectDataFn( mDataSrc )( rowData, val, meta );
		};
	
		// Indicate if DataTables should read DOM data as an object or array
		// Used in _fnGetRowElements
		if ( typeof mDataSrc !== 'number' ) {
			oSettings._rowReadObject = true;
		}
	
		/* Feature sorting overrides column specific when off */
		if ( !oSettings.oFeatures.bSort )
		{
			oCol.bSortable = false;
			th.addClass( oClasses.sSortableNone ); // Have to add class here as order event isn't called
		}
	
		/* Check that the class assignment is correct for sorting */
		var bAsc = $.inArray('asc', oCol.asSorting) !== -1;
		var bDesc = $.inArray('desc', oCol.asSorting) !== -1;
		if ( !oCol.bSortable || (!bAsc && !bDesc) )
		{
			oCol.sSortingClass = oClasses.sSortableNone;
			oCol.sSortingClassJUI = "";
		}
		else if ( bAsc && !bDesc )
		{
			oCol.sSortingClass = oClasses.sSortableAsc;
			oCol.sSortingClassJUI = oClasses.sSortJUIAscAllowed;
		}
		else if ( !bAsc && bDesc )
		{
			oCol.sSortingClass = oClasses.sSortableDesc;
			oCol.sSortingClassJUI = oClasses.sSortJUIDescAllowed;
		}
		else
		{
			oCol.sSortingClass = oClasses.sSortable;
			oCol.sSortingClassJUI = oClasses.sSortJUI;
		}
	}
	
	
	/**
	 * Adjust the table column widths for new data. Note: you would probably want to
	 * do a redraw after calling this function!
	 *  @param {object} settings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnAdjustColumnSizing ( settings )
	{
		/* Not interested in doing column width calculation if auto-width is disabled */
		if ( settings.oFeatures.bAutoWidth !== false )
		{
			var columns = settings.aoColumns;
	
			_fnCalculateColumnWidths( settings );
			for ( var i=0 , iLen=columns.length ; i<iLen ; i++ )
			{
				columns[i].nTh.style.width = columns[i].sWidth;
			}
		}
	
		var scroll = settings.oScroll;
		if ( scroll.sY !== '' || scroll.sX !== '')
		{
			_fnScrollDraw( settings );
		}
	
		_fnCallbackFire( settings, null, 'column-sizing', [settings] );
	}
	
	
	/**
	 * Covert the index of a visible column to the index in the data array (take account
	 * of hidden columns)
	 *  @param {object} oSettings dataTables settings object
	 *  @param {int} iMatch Visible column index to lookup
	 *  @returns {int} i the data index
	 *  @memberof DataTable#oApi
	 */
	function _fnVisibleToColumnIndex( oSettings, iMatch )
	{
		var aiVis = _fnGetColumns( oSettings, 'bVisible' );
	
		return typeof aiVis[iMatch] === 'number' ?
			aiVis[iMatch] :
			null;
	}
	
	
	/**
	 * Covert the index of an index in the data array and convert it to the visible
	 *   column index (take account of hidden columns)
	 *  @param {int} iMatch Column index to lookup
	 *  @param {object} oSettings dataTables settings object
	 *  @returns {int} i the data index
	 *  @memberof DataTable#oApi
	 */
	function _fnColumnIndexToVisible( oSettings, iMatch )
	{
		var aiVis = _fnGetColumns( oSettings, 'bVisible' );
		var iPos = $.inArray( iMatch, aiVis );
	
		return iPos !== -1 ? iPos : null;
	}
	
	
	/**
	 * Get the number of visible columns
	 *  @param {object} oSettings dataTables settings object
	 *  @returns {int} i the number of visible columns
	 *  @memberof DataTable#oApi
	 */
	function _fnVisbleColumns( oSettings )
	{
		return _fnGetColumns( oSettings, 'bVisible' ).length;
	}
	
	
	/**
	 * Get an array of column indexes that match a given property
	 *  @param {object} oSettings dataTables settings object
	 *  @param {string} sParam Parameter in aoColumns to look for - typically
	 *    bVisible or bSearchable
	 *  @returns {array} Array of indexes with matched properties
	 *  @memberof DataTable#oApi
	 */
	function _fnGetColumns( oSettings, sParam )
	{
		var a = [];
	
		$.map( oSettings.aoColumns, function(val, i) {
			if ( val[sParam] ) {
				a.push( i );
			}
		} );
	
		return a;
	}
	
	
	/**
	 * Calculate the 'type' of a column
	 *  @param {object} settings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnColumnTypes ( settings )
	{
		var columns = settings.aoColumns;
		var data = settings.aoData;
		var types = DataTable.ext.type.detect;
		var i, ien, j, jen, k, ken;
		var col, cell, detectedType, cache;
	
		// For each column, spin over the 
		for ( i=0, ien=columns.length ; i<ien ; i++ ) {
			col = columns[i];
			cache = [];
	
			if ( ! col.sType && col._sManualType ) {
				col.sType = col._sManualType;
			}
			else if ( ! col.sType ) {
				for ( j=0, jen=types.length ; j<jen ; j++ ) {
					for ( k=0, ken=data.length ; k<ken ; k++ ) {
						// Use a cache array so we only need to get the type data
						// from the formatter once (when using multiple detectors)
						if ( cache[k] === undefined ) {
							cache[k] = _fnGetCellData( settings, k, i, 'type' );
						}
	
						detectedType = types[j]( cache[k], settings );
	
						// If null, then this type can't apply to this column, so
						// rather than testing all cells, break out. There is an
						// exception for the last type which is `html`. We need to
						// scan all rows since it is possible to mix string and HTML
						// types
						if ( ! detectedType && j !== types.length-1 ) {
							break;
						}
	
						// Only a single match is needed for html type since it is
						// bottom of the pile and very similar to string
						if ( detectedType === 'html' ) {
							break;
						}
					}
	
					// Type is valid for all data points in the column - use this
					// type
					if ( detectedType ) {
						col.sType = detectedType;
						break;
					}
				}
	
				// Fall back - if no type was detected, always use string
				if ( ! col.sType ) {
					col.sType = 'string';
				}
			}
		}
	}
	
	
	/**
	 * Take the column definitions and static columns arrays and calculate how
	 * they relate to column indexes. The callback function will then apply the
	 * definition found for a column to a suitable configuration object.
	 *  @param {object} oSettings dataTables settings object
	 *  @param {array} aoColDefs The aoColumnDefs array that is to be applied
	 *  @param {array} aoCols The aoColumns array that defines columns individually
	 *  @param {function} fn Callback function - takes two parameters, the calculated
	 *    column index and the definition for that column.
	 *  @memberof DataTable#oApi
	 */
	function _fnApplyColumnDefs( oSettings, aoColDefs, aoCols, fn )
	{
		var i, iLen, j, jLen, k, kLen, def;
		var columns = oSettings.aoColumns;
	
		// Column definitions with aTargets
		if ( aoColDefs )
		{
			/* Loop over the definitions array - loop in reverse so first instance has priority */
			for ( i=aoColDefs.length-1 ; i>=0 ; i-- )
			{
				def = aoColDefs[i];
	
				/* Each definition can target multiple columns, as it is an array */
				var aTargets = def.targets !== undefined ?
					def.targets :
					def.aTargets;
	
				if ( ! $.isArray( aTargets ) )
				{
					aTargets = [ aTargets ];
				}
	
				for ( j=0, jLen=aTargets.length ; j<jLen ; j++ )
				{
					if ( typeof aTargets[j] === 'number' && aTargets[j] >= 0 )
					{
						/* Add columns that we don't yet know about */
						while( columns.length <= aTargets[j] )
						{
							_fnAddColumn( oSettings );
						}
	
						/* Integer, basic index */
						fn( aTargets[j], def );
					}
					else if ( typeof aTargets[j] === 'number' && aTargets[j] < 0 )
					{
						/* Negative integer, right to left column counting */
						fn( columns.length+aTargets[j], def );
					}
					else if ( typeof aTargets[j] === 'string' )
					{
						/* Class name matching on TH element */
						for ( k=0, kLen=columns.length ; k<kLen ; k++ )
						{
							if ( aTargets[j] == "_all" ||
							     $(columns[k].nTh).hasClass( aTargets[j] ) )
							{
								fn( k, def );
							}
						}
					}
				}
			}
		}
	
		// Statically defined columns array
		if ( aoCols )
		{
			for ( i=0, iLen=aoCols.length ; i<iLen ; i++ )
			{
				fn( i, aoCols[i] );
			}
		}
	}
	
	/**
	 * Add a data array to the table, creating DOM node etc. This is the parallel to
	 * _fnGatherData, but for adding rows from a Javascript source, rather than a
	 * DOM source.
	 *  @param {object} oSettings dataTables settings object
	 *  @param {array} aData data array to be added
	 *  @param {node} [nTr] TR element to add to the table - optional. If not given,
	 *    DataTables will create a row automatically
	 *  @param {array} [anTds] Array of TD|TH elements for the row - must be given
	 *    if nTr is.
	 *  @returns {int} >=0 if successful (index of new aoData entry), -1 if failed
	 *  @memberof DataTable#oApi
	 */
	function _fnAddData ( oSettings, aDataIn, nTr, anTds )
	{
		/* Create the object for storing information about this new row */
		var iRow = oSettings.aoData.length;
		var oData = $.extend( true, {}, DataTable.models.oRow, {
			src: nTr ? 'dom' : 'data'
		} );
	
		oData._aData = aDataIn;
		oSettings.aoData.push( oData );
	
		/* Create the cells */
		var nTd, sThisType;
		var columns = oSettings.aoColumns;
		for ( var i=0, iLen=columns.length ; i<iLen ; i++ )
		{
			// When working with a row, the data source object must be populated. In
			// all other cases, the data source object is already populated, so we
			// don't overwrite it, which might break bindings etc
			if ( nTr ) {
				_fnSetCellData( oSettings, iRow, i, _fnGetCellData( oSettings, iRow, i ) );
			}
			columns[i].sType = null;
		}
	
		/* Add to the display array */
		oSettings.aiDisplayMaster.push( iRow );
	
		/* Create the DOM information, or register it if already present */
		if ( nTr || ! oSettings.oFeatures.bDeferRender )
		{
			_fnCreateTr( oSettings, iRow, nTr, anTds );
		}
	
		return iRow;
	}
	
	
	/**
	 * Add one or more TR elements to the table. Generally we'd expect to
	 * use this for reading data from a DOM sourced table, but it could be
	 * used for an TR element. Note that if a TR is given, it is used (i.e.
	 * it is not cloned).
	 *  @param {object} settings dataTables settings object
	 *  @param {array|node|jQuery} trs The TR element(s) to add to the table
	 *  @returns {array} Array of indexes for the added rows
	 *  @memberof DataTable#oApi
	 */
	function _fnAddTr( settings, trs )
	{
		var row;
	
		// Allow an individual node to be passed in
		if ( ! (trs instanceof $) ) {
			trs = $(trs);
		}
	
		return trs.map( function (i, el) {
			row = _fnGetRowElements( settings, el );
			return _fnAddData( settings, row.data, el, row.cells );
		} );
	}
	
	
	/**
	 * Take a TR element and convert it to an index in aoData
	 *  @param {object} oSettings dataTables settings object
	 *  @param {node} n the TR element to find
	 *  @returns {int} index if the node is found, null if not
	 *  @memberof DataTable#oApi
	 */
	function _fnNodeToDataIndex( oSettings, n )
	{
		return (n._DT_RowIndex!==undefined) ? n._DT_RowIndex : null;
	}
	
	
	/**
	 * Take a TD element and convert it into a column data index (not the visible index)
	 *  @param {object} oSettings dataTables settings object
	 *  @param {int} iRow The row number the TD/TH can be found in
	 *  @param {node} n The TD/TH element to find
	 *  @returns {int} index if the node is found, -1 if not
	 *  @memberof DataTable#oApi
	 */
	function _fnNodeToColumnIndex( oSettings, iRow, n )
	{
		return $.inArray( n, oSettings.aoData[ iRow ].anCells );
	}
	
	
	/**
	 * Get the data for a given cell from the internal cache, taking into account data mapping
	 *  @param {object} settings dataTables settings object
	 *  @param {int} rowIdx aoData row id
	 *  @param {int} colIdx Column index
	 *  @param {string} type data get type ('display', 'type' 'filter' 'sort')
	 *  @returns {*} Cell data
	 *  @memberof DataTable#oApi
	 */
	function _fnGetCellData( settings, rowIdx, colIdx, type )
	{
		var draw           = settings.iDraw;
		var col            = settings.aoColumns[colIdx];
		var rowData        = settings.aoData[rowIdx]._aData;
		var defaultContent = col.sDefaultContent;
		var cellData       = col.fnGetData( rowData, type, {
			settings: settings,
			row:      rowIdx,
			col:      colIdx
		} );
	
		if ( cellData === undefined ) {
			if ( settings.iDrawError != draw && defaultContent === null ) {
				_fnLog( settings, 0, "Requested unknown parameter "+
					(typeof col.mData=='function' ? '{function}' : "'"+col.mData+"'")+
					" for row "+rowIdx, 4 );
				settings.iDrawError = draw;
			}
			return defaultContent;
		}
	
		/* When the data source is null, we can use default column data */
		if ( (cellData === rowData || cellData === null) && defaultContent !== null ) {
			cellData = defaultContent;
		}
		else if ( typeof cellData === 'function' ) {
			// If the data source is a function, then we run it and use the return,
			// executing in the scope of the data object (for instances)
			return cellData.call( rowData );
		}
	
		if ( cellData === null && type == 'display' ) {
			return '';
		}
		return cellData;
	}
	
	
	/**
	 * Set the value for a specific cell, into the internal data cache
	 *  @param {object} settings dataTables settings object
	 *  @param {int} rowIdx aoData row id
	 *  @param {int} colIdx Column index
	 *  @param {*} val Value to set
	 *  @memberof DataTable#oApi
	 */
	function _fnSetCellData( settings, rowIdx, colIdx, val )
	{
		var col     = settings.aoColumns[colIdx];
		var rowData = settings.aoData[rowIdx]._aData;
	
		col.fnSetData( rowData, val, {
			settings: settings,
			row:      rowIdx,
			col:      colIdx
		}  );
	}
	
	
	// Private variable that is used to match action syntax in the data property object
	var __reArray = /\[.*?\]$/;
	var __reFn = /\(\)$/;
	
	/**
	 * Split string on periods, taking into account escaped periods
	 * @param  {string} str String to split
	 * @return {array} Split string
	 */
	function _fnSplitObjNotation( str )
	{
		return $.map( str.match(/(\\.|[^\.])+/g), function ( s ) {
			return s.replace(/\\./g, '.');
		} );
	}
	
	
	/**
	 * Return a function that can be used to get data from a source object, taking
	 * into account the ability to use nested objects as a source
	 *  @param {string|int|function} mSource The data source for the object
	 *  @returns {function} Data get function
	 *  @memberof DataTable#oApi
	 */
	function _fnGetObjectDataFn( mSource )
	{
		if ( $.isPlainObject( mSource ) )
		{
			/* Build an object of get functions, and wrap them in a single call */
			var o = {};
			$.each( mSource, function (key, val) {
				if ( val ) {
					o[key] = _fnGetObjectDataFn( val );
				}
			} );
	
			return function (data, type, row, meta) {
				var t = o[type] || o._;
				return t !== undefined ?
					t(data, type, row, meta) :
					data;
			};
		}
		else if ( mSource === null )
		{
			/* Give an empty string for rendering / sorting etc */
			return function (data) { // type, row and meta also passed, but not used
				return data;
			};
		}
		else if ( typeof mSource === 'function' )
		{
			return function (data, type, row, meta) {
				return mSource( data, type, row, meta );
			};
		}
		else if ( typeof mSource === 'string' && (mSource.indexOf('.') !== -1 ||
			      mSource.indexOf('[') !== -1 || mSource.indexOf('(') !== -1) )
		{
			/* If there is a . in the source string then the data source is in a
			 * nested object so we loop over the data for each level to get the next
			 * level down. On each loop we test for undefined, and if found immediately
			 * return. This allows entire objects to be missing and sDefaultContent to
			 * be used if defined, rather than throwing an error
			 */
			var fetchData = function (data, type, src) {
				var arrayNotation, funcNotation, out, innerSrc;
	
				if ( src !== "" )
				{
					var a = _fnSplitObjNotation( src );
	
					for ( var i=0, iLen=a.length ; i<iLen ; i++ )
					{
						// Check if we are dealing with special notation
						arrayNotation = a[i].match(__reArray);
						funcNotation = a[i].match(__reFn);
	
						if ( arrayNotation )
						{
							// Array notation
							a[i] = a[i].replace(__reArray, '');
	
							// Condition allows simply [] to be passed in
							if ( a[i] !== "" ) {
								data = data[ a[i] ];
							}
							out = [];
	
							// Get the remainder of the nested object to get
							a.splice( 0, i+1 );
							innerSrc = a.join('.');
	
							// Traverse each entry in the array getting the properties requested
							for ( var j=0, jLen=data.length ; j<jLen ; j++ ) {
								out.push( fetchData( data[j], type, innerSrc ) );
							}
	
							// If a string is given in between the array notation indicators, that
							// is used to join the strings together, otherwise an array is returned
							var join = arrayNotation[0].substring(1, arrayNotation[0].length-1);
							data = (join==="") ? out : out.join(join);
	
							// The inner call to fetchData has already traversed through the remainder
							// of the source requested, so we exit from the loop
							break;
						}
						else if ( funcNotation )
						{
							// Function call
							a[i] = a[i].replace(__reFn, '');
							data = data[ a[i] ]();
							continue;
						}
	
						if ( data === null || data[ a[i] ] === undefined )
						{
							return undefined;
						}
						data = data[ a[i] ];
					}
				}
	
				return data;
			};
	
			return function (data, type) { // row and meta also passed, but not used
				return fetchData( data, type, mSource );
			};
		}
		else
		{
			/* Array or flat object mapping */
			return function (data, type) { // row and meta also passed, but not used
				return data[mSource];
			};
		}
	}
	
	
	/**
	 * Return a function that can be used to set data from a source object, taking
	 * into account the ability to use nested objects as a source
	 *  @param {string|int|function} mSource The data source for the object
	 *  @returns {function} Data set function
	 *  @memberof DataTable#oApi
	 */
	function _fnSetObjectDataFn( mSource )
	{
		if ( $.isPlainObject( mSource ) )
		{
			/* Unlike get, only the underscore (global) option is used for for
			 * setting data since we don't know the type here. This is why an object
			 * option is not documented for `mData` (which is read/write), but it is
			 * for `mRender` which is read only.
			 */
			return _fnSetObjectDataFn( mSource._ );
		}
		else if ( mSource === null )
		{
			/* Nothing to do when the data source is null */
			return function () {};
		}
		else if ( typeof mSource === 'function' )
		{
			return function (data, val, meta) {
				mSource( data, 'set', val, meta );
			};
		}
		else if ( typeof mSource === 'string' && (mSource.indexOf('.') !== -1 ||
			      mSource.indexOf('[') !== -1 || mSource.indexOf('(') !== -1) )
		{
			/* Like the get, we need to get data from a nested object */
			var setData = function (data, val, src) {
				var a = _fnSplitObjNotation( src ), b;
				var aLast = a[a.length-1];
				var arrayNotation, funcNotation, o, innerSrc;
	
				for ( var i=0, iLen=a.length-1 ; i<iLen ; i++ )
				{
					// Check if we are dealing with an array notation request
					arrayNotation = a[i].match(__reArray);
					funcNotation = a[i].match(__reFn);
	
					if ( arrayNotation )
					{
						a[i] = a[i].replace(__reArray, '');
						data[ a[i] ] = [];
	
						// Get the remainder of the nested object to set so we can recurse
						b = a.slice();
						b.splice( 0, i+1 );
						innerSrc = b.join('.');
	
						// Traverse each entry in the array setting the properties requested
						for ( var j=0, jLen=val.length ; j<jLen ; j++ )
						{
							o = {};
							setData( o, val[j], innerSrc );
							data[ a[i] ].push( o );
						}
	
						// The inner call to setData has already traversed through the remainder
						// of the source and has set the data, thus we can exit here
						return;
					}
					else if ( funcNotation )
					{
						// Function call
						a[i] = a[i].replace(__reFn, '');
						data = data[ a[i] ]( val );
					}
	
					// If the nested object doesn't currently exist - since we are
					// trying to set the value - create it
					if ( data[ a[i] ] === null || data[ a[i] ] === undefined )
					{
						data[ a[i] ] = {};
					}
					data = data[ a[i] ];
				}
	
				// Last item in the input - i.e, the actual set
				if ( aLast.match(__reFn ) )
				{
					// Function call
					data = data[ aLast.replace(__reFn, '') ]( val );
				}
				else
				{
					// If array notation is used, we just want to strip it and use the property name
					// and assign the value. If it isn't used, then we get the result we want anyway
					data[ aLast.replace(__reArray, '') ] = val;
				}
			};
	
			return function (data, val) { // meta is also passed in, but not used
				return setData( data, val, mSource );
			};
		}
		else
		{
			/* Array or flat object mapping */
			return function (data, val) { // meta is also passed in, but not used
				data[mSource] = val;
			};
		}
	}
	
	
	/**
	 * Return an array with the full table data
	 *  @param {object} oSettings dataTables settings object
	 *  @returns array {array} aData Master data array
	 *  @memberof DataTable#oApi
	 */
	function _fnGetDataMaster ( settings )
	{
		return _pluck( settings.aoData, '_aData' );
	}
	
	
	/**
	 * Nuke the table
	 *  @param {object} oSettings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnClearTable( settings )
	{
		settings.aoData.length = 0;
		settings.aiDisplayMaster.length = 0;
		settings.aiDisplay.length = 0;
	}
	
	
	 /**
	 * Take an array of integers (index array) and remove a target integer (value - not
	 * the key!)
	 *  @param {array} a Index array to target
	 *  @param {int} iTarget value to find
	 *  @memberof DataTable#oApi
	 */
	function _fnDeleteIndex( a, iTarget, splice )
	{
		var iTargetIndex = -1;
	
		for ( var i=0, iLen=a.length ; i<iLen ; i++ )
		{
			if ( a[i] == iTarget )
			{
				iTargetIndex = i;
			}
			else if ( a[i] > iTarget )
			{
				a[i]--;
			}
		}
	
		if ( iTargetIndex != -1 && splice === undefined )
		{
			a.splice( iTargetIndex, 1 );
		}
	}
	
	
	/**
	 * Mark cached data as invalid such that a re-read of the data will occur when
	 * the cached data is next requested. Also update from the data source object.
	 *
	 * @param {object} settings DataTables settings object
	 * @param {int}    rowIdx   Row index to invalidate
	 * @param {string} [src]    Source to invalidate from: undefined, 'auto', 'dom'
	 *     or 'data'
	 * @param {int}    [colIdx] Column index to invalidate. If undefined the whole
	 *     row will be invalidated
	 * @memberof DataTable#oApi
	 *
	 * @todo For the modularisation of v1.11 this will need to become a callback, so
	 *   the sort and filter methods can subscribe to it. That will required
	 *   initialisation options for sorting, which is why it is not already baked in
	 */
	function _fnInvalidate( settings, rowIdx, src, colIdx )
	{
		var row = settings.aoData[ rowIdx ];
		var i, ien;
		var cellWrite = function ( cell, col ) {
			// This is very frustrating, but in IE if you just write directly
			// to innerHTML, and elements that are overwritten are GC'ed,
			// even if there is a reference to them elsewhere
			while ( cell.childNodes.length ) {
				cell.removeChild( cell.firstChild );
			}
	
			cell.innerHTML = _fnGetCellData( settings, rowIdx, col, 'display' );
		};
	
		// Are we reading last data from DOM or the data object?
		if ( src === 'dom' || ((! src || src === 'auto') && row.src === 'dom') ) {
			// Read the data from the DOM
			row._aData = _fnGetRowElements(
					settings, row, colIdx, colIdx === undefined ? undefined : row._aData
				)
				.data;
		}
		else {
			// Reading from data object, update the DOM
			var cells = row.anCells;
	
			if ( cells ) {
				if ( colIdx !== undefined ) {
					cellWrite( cells[colIdx], colIdx );
				}
				else {
					for ( i=0, ien=cells.length ; i<ien ; i++ ) {
						cellWrite( cells[i], i );
					}
				}
			}
		}
	
		// For both row and cell invalidation, the cached data for sorting and
		// filtering is nulled out
		row._aSortData = null;
		row._aFilterData = null;
	
		// Invalidate the type for a specific column (if given) or all columns since
		// the data might have changed
		var cols = settings.aoColumns;
		if ( colIdx !== undefined ) {
			cols[ colIdx ].sType = null;
		}
		else {
			for ( i=0, ien=cols.length ; i<ien ; i++ ) {
				cols[i].sType = null;
			}
	
			// Update DataTables special `DT_*` attributes for the row
			_fnRowAttributes( row );
		}
	}
	
	
	/**
	 * Build a data source object from an HTML row, reading the contents of the
	 * cells that are in the row.
	 *
	 * @param {object} settings DataTables settings object
	 * @param {node|object} TR element from which to read data or existing row
	 *   object from which to re-read the data from the cells
	 * @param {int} [colIdx] Optional column index
	 * @param {array|object} [d] Data source object. If `colIdx` is given then this
	 *   parameter should also be given and will be used to write the data into.
	 *   Only the column in question will be written
	 * @returns {object} Object with two parameters: `data` the data read, in
	 *   document order, and `cells` and array of nodes (they can be useful to the
	 *   caller, so rather than needing a second traversal to get them, just return
	 *   them from here).
	 * @memberof DataTable#oApi
	 */
	function _fnGetRowElements( settings, row, colIdx, d )
	{
		var
			tds = [],
			td = row.firstChild,
			name, col, o, i=0, contents,
			columns = settings.aoColumns,
			objectRead = settings._rowReadObject;
	
		// Allow the data object to be passed in, or construct
		d = d || objectRead ? {} : [];
	
		var attr = function ( str, td  ) {
			if ( typeof str === 'string' ) {
				var idx = str.indexOf('@');
	
				if ( idx !== -1 ) {
					var attr = str.substring( idx+1 );
					var setter = _fnSetObjectDataFn( str );
					setter( d, td.getAttribute( attr ) );
				}
			}
		};
	
		// Read data from a cell and store into the data object
		var cellProcess = function ( cell ) {
			if ( colIdx === undefined || colIdx === i ) {
				col = columns[i];
				contents = $.trim(cell.innerHTML);
	
				if ( col && col._bAttrSrc ) {
					var setter = _fnSetObjectDataFn( col.mData._ );
					setter( d, contents );
	
					attr( col.mData.sort, cell );
					attr( col.mData.type, cell );
					attr( col.mData.filter, cell );
				}
				else {
					// Depending on the `data` option for the columns the data can
					// be read to either an object or an array.
					if ( objectRead ) {
						if ( ! col._setter ) {
							// Cache the setter function
							col._setter = _fnSetObjectDataFn( col.mData );
						}
						col._setter( d, contents );
					}
					else {
						d[i] = contents;
					}
				}
			}
	
			i++;
		};
	
		if ( td ) {
			// `tr` element was passed in
			while ( td ) {
				name = td.nodeName.toUpperCase();
	
				if ( name == "TD" || name == "TH" ) {
					cellProcess( td );
					tds.push( td );
				}
	
				td = td.nextSibling;
			}
		}
		else {
			// Existing row object passed in
			tds = row.anCells;
			
			for ( var j=0, jen=tds.length ; j<jen ; j++ ) {
				cellProcess( tds[j] );
			}
		}
	
		return {
			data: d,
			cells: tds
		};
	}
	/**
	 * Create a new TR element (and it's TD children) for a row
	 *  @param {object} oSettings dataTables settings object
	 *  @param {int} iRow Row to consider
	 *  @param {node} [nTrIn] TR element to add to the table - optional. If not given,
	 *    DataTables will create a row automatically
	 *  @param {array} [anTds] Array of TD|TH elements for the row - must be given
	 *    if nTr is.
	 *  @memberof DataTable#oApi
	 */
	function _fnCreateTr ( oSettings, iRow, nTrIn, anTds )
	{
		var
			row = oSettings.aoData[iRow],
			rowData = row._aData,
			cells = [],
			nTr, nTd, oCol,
			i, iLen;
	
		if ( row.nTr === null )
		{
			nTr = nTrIn || document.createElement('tr');
	
			row.nTr = nTr;
			row.anCells = cells;
	
			/* Use a private property on the node to allow reserve mapping from the node
			 * to the aoData array for fast look up
			 */
			nTr._DT_RowIndex = iRow;
	
			/* Special parameters can be given by the data source to be used on the row */
			_fnRowAttributes( row );
	
			/* Process each column */
			for ( i=0, iLen=oSettings.aoColumns.length ; i<iLen ; i++ )
			{
				oCol = oSettings.aoColumns[i];
	
				nTd = nTrIn ? anTds[i] : document.createElement( oCol.sCellType );
				cells.push( nTd );
	
				// Need to create the HTML if new, or if a rendering function is defined
				if ( !nTrIn || oCol.mRender || oCol.mData !== i )
				{
					nTd.innerHTML = _fnGetCellData( oSettings, iRow, i, 'display' );
				}
	
				/* Add user defined class */
				if ( oCol.sClass )
				{
					nTd.className += ' '+oCol.sClass;
				}
	
				// Visibility - add or remove as required
				if ( oCol.bVisible && ! nTrIn )
				{
					nTr.appendChild( nTd );
				}
				else if ( ! oCol.bVisible && nTrIn )
				{
					nTd.parentNode.removeChild( nTd );
				}
	
				if ( oCol.fnCreatedCell )
				{
					oCol.fnCreatedCell.call( oSettings.oInstance,
						nTd, _fnGetCellData( oSettings, iRow, i ), rowData, iRow, i
					);
				}
			}
	
			_fnCallbackFire( oSettings, 'aoRowCreatedCallback', null, [nTr, rowData, iRow] );
		}
	
		// Remove once webkit bug 131819 and Chromium bug 365619 have been resolved
		// and deployed
		row.nTr.setAttribute( 'role', 'row' );
	}
	
	
	/**
	 * Add attributes to a row based on the special `DT_*` parameters in a data
	 * source object.
	 *  @param {object} DataTables row object for the row to be modified
	 *  @memberof DataTable#oApi
	 */
	function _fnRowAttributes( row )
	{
		var tr = row.nTr;
		var data = row._aData;
	
		if ( tr ) {
			if ( data.DT_RowId ) {
				tr.id = data.DT_RowId;
			}
	
			if ( data.DT_RowClass ) {
				// Remove any classes added by DT_RowClass before
				var a = data.DT_RowClass.split(' ');
				row.__rowc = row.__rowc ?
					_unique( row.__rowc.concat( a ) ) :
					a;
	
				$(tr)
					.removeClass( row.__rowc.join(' ') )
					.addClass( data.DT_RowClass );
			}
	
			if ( data.DT_RowAttr ) {
				$(tr).attr( data.DT_RowAttr );
			}
	
			if ( data.DT_RowData ) {
				$(tr).data( data.DT_RowData );
			}
		}
	}
	
	
	/**
	 * Create the HTML header for the table
	 *  @param {object} oSettings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnBuildHead( oSettings )
	{
		var i, ien, cell, row, column;
		var thead = oSettings.nTHead;
		var tfoot = oSettings.nTFoot;
		var createHeader = $('th, td', thead).length === 0;
		var classes = oSettings.oClasses;
		var columns = oSettings.aoColumns;
	
		if ( createHeader ) {
			row = $('<tr/>').appendTo( thead );
		}
	
		for ( i=0, ien=columns.length ; i<ien ; i++ ) {
			column = columns[i];
			cell = $( column.nTh ).addClass( column.sClass );
	
			if ( createHeader ) {
				cell.appendTo( row );
			}
	
			// 1.11 move into sorting
			if ( oSettings.oFeatures.bSort ) {
				cell.addClass( column.sSortingClass );
	
				if ( column.bSortable !== false ) {
					cell
						.attr( 'tabindex', oSettings.iTabIndex )
						.attr( 'aria-controls', oSettings.sTableId );
	
					_fnSortAttachListener( oSettings, column.nTh, i );
				}
			}
	
			if ( column.sTitle != cell.html() ) {
				cell.html( column.sTitle );
			}
	
			_fnRenderer( oSettings, 'header' )(
				oSettings, cell, column, classes
			);
		}
	
		if ( createHeader ) {
			_fnDetectHeader( oSettings.aoHeader, thead );
		}
		
		/* ARIA role for the rows */
	 	$(thead).find('>tr').attr('role', 'row');
	
		/* Deal with the footer - add classes if required */
		$(thead).find('>tr>th, >tr>td').addClass( classes.sHeaderTH );
		$(tfoot).find('>tr>th, >tr>td').addClass( classes.sFooterTH );
	
		// Cache the footer cells. Note that we only take the cells from the first
		// row in the footer. If there is more than one row the user wants to
		// interact with, they need to use the table().foot() method. Note also this
		// allows cells to be used for multiple columns using colspan
		if ( tfoot !== null ) {
			var cells = oSettings.aoFooter[0];
	
			for ( i=0, ien=cells.length ; i<ien ; i++ ) {
				column = columns[i];
				column.nTf = cells[i].cell;
	
				if ( column.sClass ) {
					$(column.nTf).addClass( column.sClass );
				}
			}
		}
	}
	
	
	/**
	 * Draw the header (or footer) element based on the column visibility states. The
	 * methodology here is to use the layout array from _fnDetectHeader, modified for
	 * the instantaneous column visibility, to construct the new layout. The grid is
	 * traversed over cell at a time in a rows x columns grid fashion, although each
	 * cell insert can cover multiple elements in the grid - which is tracks using the
	 * aApplied array. Cell inserts in the grid will only occur where there isn't
	 * already a cell in that position.
	 *  @param {object} oSettings dataTables settings object
	 *  @param array {objects} aoSource Layout array from _fnDetectHeader
	 *  @param {boolean} [bIncludeHidden=false] If true then include the hidden columns in the calc,
	 *  @memberof DataTable#oApi
	 */
	function _fnDrawHead( oSettings, aoSource, bIncludeHidden )
	{
		var i, iLen, j, jLen, k, kLen, n, nLocalTr;
		var aoLocal = [];
		var aApplied = [];
		var iColumns = oSettings.aoColumns.length;
		var iRowspan, iColspan;
	
		if ( ! aoSource )
		{
			return;
		}
	
		if (  bIncludeHidden === undefined )
		{
			bIncludeHidden = false;
		}
	
		/* Make a copy of the master layout array, but without the visible columns in it */
		for ( i=0, iLen=aoSource.length ; i<iLen ; i++ )
		{
			aoLocal[i] = aoSource[i].slice();
			aoLocal[i].nTr = aoSource[i].nTr;
	
			/* Remove any columns which are currently hidden */
			for ( j=iColumns-1 ; j>=0 ; j-- )
			{
				if ( !oSettings.aoColumns[j].bVisible && !bIncludeHidden )
				{
					aoLocal[i].splice( j, 1 );
				}
			}
	
			/* Prep the applied array - it needs an element for each row */
			aApplied.push( [] );
		}
	
		for ( i=0, iLen=aoLocal.length ; i<iLen ; i++ )
		{
			nLocalTr = aoLocal[i].nTr;
	
			/* All cells are going to be replaced, so empty out the row */
			if ( nLocalTr )
			{
				while( (n = nLocalTr.firstChild) )
				{
					nLocalTr.removeChild( n );
				}
			}
	
			for ( j=0, jLen=aoLocal[i].length ; j<jLen ; j++ )
			{
				iRowspan = 1;
				iColspan = 1;
	
				/* Check to see if there is already a cell (row/colspan) covering our target
				 * insert point. If there is, then there is nothing to do.
				 */
				if ( aApplied[i][j] === undefined )
				{
					nLocalTr.appendChild( aoLocal[i][j].cell );
					aApplied[i][j] = 1;
	
					/* Expand the cell to cover as many rows as needed */
					while ( aoLocal[i+iRowspan] !== undefined &&
					        aoLocal[i][j].cell == aoLocal[i+iRowspan][j].cell )
					{
						aApplied[i+iRowspan][j] = 1;
						iRowspan++;
					}
	
					/* Expand the cell to cover as many columns as needed */
					while ( aoLocal[i][j+iColspan] !== undefined &&
					        aoLocal[i][j].cell == aoLocal[i][j+iColspan].cell )
					{
						/* Must update the applied array over the rows for the columns */
						for ( k=0 ; k<iRowspan ; k++ )
						{
							aApplied[i+k][j+iColspan] = 1;
						}
						iColspan++;
					}
	
					/* Do the actual expansion in the DOM */
					$(aoLocal[i][j].cell)
						.attr('rowspan', iRowspan)
						.attr('colspan', iColspan);
				}
			}
		}
	}
	
	
	/**
	 * Insert the required TR nodes into the table for display
	 *  @param {object} oSettings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnDraw( oSettings )
	{
		/* Provide a pre-callback function which can be used to cancel the draw is false is returned */
		var aPreDraw = _fnCallbackFire( oSettings, 'aoPreDrawCallback', 'preDraw', [oSettings] );
		if ( $.inArray( false, aPreDraw ) !== -1 )
		{
			_fnProcessingDisplay( oSettings, false );
			return;
		}
	
		var i, iLen, n;
		var anRows = [];
		var iRowCount = 0;
		var asStripeClasses = oSettings.asStripeClasses;
		var iStripes = asStripeClasses.length;
		var iOpenRows = oSettings.aoOpenRows.length;
		var oLang = oSettings.oLanguage;
		var iInitDisplayStart = oSettings.iInitDisplayStart;
		var bServerSide = _fnDataSource( oSettings ) == 'ssp';
		var aiDisplay = oSettings.aiDisplay;
	
		oSettings.bDrawing = true;
	
		/* Check and see if we have an initial draw position from state saving */
		if ( iInitDisplayStart !== undefined && iInitDisplayStart !== -1 )
		{
			oSettings._iDisplayStart = bServerSide ?
				iInitDisplayStart :
				iInitDisplayStart >= oSettings.fnRecordsDisplay() ?
					0 :
					iInitDisplayStart;
	
			oSettings.iInitDisplayStart = -1;
		}
	
		var iDisplayStart = oSettings._iDisplayStart;
		var iDisplayEnd = oSettings.fnDisplayEnd();
	
		/* Server-side processing draw intercept */
		if ( oSettings.bDeferLoading )
		{
			oSettings.bDeferLoading = false;
			oSettings.iDraw++;
			_fnProcessingDisplay( oSettings, false );
		}
		else if ( !bServerSide )
		{
			oSettings.iDraw++;
		}
		else if ( !oSettings.bDestroying && !_fnAjaxUpdate( oSettings ) )
		{
			return;
		}
	
		if ( aiDisplay.length !== 0 )
		{
			var iStart = bServerSide ? 0 : iDisplayStart;
			var iEnd = bServerSide ? oSettings.aoData.length : iDisplayEnd;
	
			for ( var j=iStart ; j<iEnd ; j++ )
			{
				var iDataIndex = aiDisplay[j];
				var aoData = oSettings.aoData[ iDataIndex ];
				if ( aoData.nTr === null )
				{
					_fnCreateTr( oSettings, iDataIndex );
				}
	
				var nRow = aoData.nTr;
	
				/* Remove the old striping classes and then add the new one */
				if ( iStripes !== 0 )
				{
					var sStripe = asStripeClasses[ iRowCount % iStripes ];
					if ( aoData._sRowStripe != sStripe )
					{
						$(nRow).removeClass( aoData._sRowStripe ).addClass( sStripe );
						aoData._sRowStripe = sStripe;
					}
				}
	
				// Row callback functions - might want to manipulate the row
				// iRowCount and j are not currently documented. Are they at all
				// useful?
				_fnCallbackFire( oSettings, 'aoRowCallback', null,
					[nRow, aoData._aData, iRowCount, j] );
	
				anRows.push( nRow );
				iRowCount++;
			}
		}
		else
		{
			/* Table is empty - create a row with an empty message in it */
			var sZero = oLang.sZeroRecords;
			if ( oSettings.iDraw == 1 &&  _fnDataSource( oSettings ) == 'ajax' )
			{
				sZero = oLang.sLoadingRecords;
			}
			else if ( oLang.sEmptyTable && oSettings.fnRecordsTotal() === 0 )
			{
				sZero = oLang.sEmptyTable;
			}
	
			anRows[ 0 ] = $( '<tr/>', { 'class': iStripes ? asStripeClasses[0] : '' } )
				.append( $('<td />', {
					'valign':  'top',
					'colSpan': _fnVisbleColumns( oSettings ),
					'class':   oSettings.oClasses.sRowEmpty
				} ).html( sZero ) )[0];
		}
	
		/* Header and footer callbacks */
		_fnCallbackFire( oSettings, 'aoHeaderCallback', 'header', [ $(oSettings.nTHead).children('tr')[0],
			_fnGetDataMaster( oSettings ), iDisplayStart, iDisplayEnd, aiDisplay ] );
	
		_fnCallbackFire( oSettings, 'aoFooterCallback', 'footer', [ $(oSettings.nTFoot).children('tr')[0],
			_fnGetDataMaster( oSettings ), iDisplayStart, iDisplayEnd, aiDisplay ] );
	
		var body = $(oSettings.nTBody);
	
		body.children().detach();
		body.append( $(anRows) );
	
		/* Call all required callback functions for the end of a draw */
		_fnCallbackFire( oSettings, 'aoDrawCallback', 'draw', [oSettings] );
	
		/* Draw is complete, sorting and filtering must be as well */
		oSettings.bSorted = false;
		oSettings.bFiltered = false;
		oSettings.bDrawing = false;
	}
	
	
	/**
	 * Redraw the table - taking account of the various features which are enabled
	 *  @param {object} oSettings dataTables settings object
	 *  @param {boolean} [holdPosition] Keep the current paging position. By default
	 *    the paging is reset to the first page
	 *  @memberof DataTable#oApi
	 */
	function _fnReDraw( settings, holdPosition )
	{
		var
			features = settings.oFeatures,
			sort     = features.bSort,
			filter   = features.bFilter;
	
		if ( sort ) {
			_fnSort( settings );
		}
	
		if ( filter ) {
			_fnFilterComplete( settings, settings.oPreviousSearch );
		}
		else {
			// No filtering, so we want to just use the display master
			settings.aiDisplay = settings.aiDisplayMaster.slice();
		}
	
		if ( holdPosition !== true ) {
			settings._iDisplayStart = 0;
		}
	
		// Let any modules know about the draw hold position state (used by
		// scrolling internally)
		settings._drawHold = holdPosition;
	
		_fnDraw( settings );
	
		settings._drawHold = false;
	}
	
	
	/**
	 * Add the options to the page HTML for the table
	 *  @param {object} oSettings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnAddOptionsHtml ( oSettings )
	{
		var classes = oSettings.oClasses;
		var table = $(oSettings.nTable);
		var holding = $('<div/>').insertBefore( table ); // Holding element for speed
		var features = oSettings.oFeatures;
	
		// All DataTables are wrapped in a div
		var insert = $('<div/>', {
			id:      oSettings.sTableId+'_wrapper',
			'class': classes.sWrapper + (oSettings.nTFoot ? '' : ' '+classes.sNoFooter)
		} );
	
		oSettings.nHolding = holding[0];
		oSettings.nTableWrapper = insert[0];
		oSettings.nTableReinsertBefore = oSettings.nTable.nextSibling;
	
		/* Loop over the user set positioning and place the elements as needed */
		var aDom = oSettings.sDom.split('');
		var featureNode, cOption, nNewNode, cNext, sAttr, j;
		for ( var i=0 ; i<aDom.length ; i++ )
		{
			featureNode = null;
			cOption = aDom[i];
	
			if ( cOption == '<' )
			{
				/* New container div */
				nNewNode = $('<div/>')[0];
	
				/* Check to see if we should append an id and/or a class name to the container */
				cNext = aDom[i+1];
				if ( cNext == "'" || cNext == '"' )
				{
					sAttr = "";
					j = 2;
					while ( aDom[i+j] != cNext )
					{
						sAttr += aDom[i+j];
						j++;
					}
	
					/* Replace jQuery UI constants @todo depreciated */
					if ( sAttr == "H" )
					{
						sAttr = classes.sJUIHeader;
					}
					else if ( sAttr == "F" )
					{
						sAttr = classes.sJUIFooter;
					}
	
					/* The attribute can be in the format of "#id.class", "#id" or "class" This logic
					 * breaks the string into parts and applies them as needed
					 */
					if ( sAttr.indexOf('.') != -1 )
					{
						var aSplit = sAttr.split('.');
						nNewNode.id = aSplit[0].substr(1, aSplit[0].length-1);
						nNewNode.className = aSplit[1];
					}
					else if ( sAttr.charAt(0) == "#" )
					{
						nNewNode.id = sAttr.substr(1, sAttr.length-1);
					}
					else
					{
						nNewNode.className = sAttr;
					}
	
					i += j; /* Move along the position array */
				}
	
				insert.append( nNewNode );
				insert = $(nNewNode);
			}
			else if ( cOption == '>' )
			{
				/* End container div */
				insert = insert.parent();
			}
			// @todo Move options into their own plugins?
			else if ( cOption == 'l' && features.bPaginate && features.bLengthChange )
			{
				/* Length */
				featureNode = _fnFeatureHtmlLength( oSettings );
			}
			else if ( cOption == 'f' && features.bFilter )
			{
				/* Filter */
				featureNode = _fnFeatureHtmlFilter( oSettings );
			}
			else if ( cOption == 'r' && features.bProcessing )
			{
				/* pRocessing */
				featureNode = _fnFeatureHtmlProcessing( oSettings );
			}
			else if ( cOption == 't' )
			{
				/* Table */
				featureNode = _fnFeatureHtmlTable( oSettings );
			}
			else if ( cOption ==  'i' && features.bInfo )
			{
				/* Info */
				featureNode = _fnFeatureHtmlInfo( oSettings );
			}
			else if ( cOption == 'p' && features.bPaginate )
			{
				/* Pagination */
				featureNode = _fnFeatureHtmlPaginate( oSettings );
			}
			else if ( DataTable.ext.feature.length !== 0 )
			{
				/* Plug-in features */
				var aoFeatures = DataTable.ext.feature;
				for ( var k=0, kLen=aoFeatures.length ; k<kLen ; k++ )
				{
					if ( cOption == aoFeatures[k].cFeature )
					{
						featureNode = aoFeatures[k].fnInit( oSettings );
						break;
					}
				}
			}
	
			/* Add to the 2D features array */
			if ( featureNode )
			{
				var aanFeatures = oSettings.aanFeatures;
	
				if ( ! aanFeatures[cOption] )
				{
					aanFeatures[cOption] = [];
				}
	
				aanFeatures[cOption].push( featureNode );
				insert.append( featureNode );
			}
		}
	
		/* Built our DOM structure - replace the holding div with what we want */
		holding.replaceWith( insert );
	}
	
	
	/**
	 * Use the DOM source to create up an array of header cells. The idea here is to
	 * create a layout grid (array) of rows x columns, which contains a reference
	 * to the cell that that point in the grid (regardless of col/rowspan), such that
	 * any column / row could be removed and the new grid constructed
	 *  @param array {object} aLayout Array to store the calculated layout in
	 *  @param {node} nThead The header/footer element for the table
	 *  @memberof DataTable#oApi
	 */
	function _fnDetectHeader ( aLayout, nThead )
	{
		var nTrs = $(nThead).children('tr');
		var nTr, nCell;
		var i, k, l, iLen, jLen, iColShifted, iColumn, iColspan, iRowspan;
		var bUnique;
		var fnShiftCol = function ( a, i, j ) {
			var k = a[i];
	                while ( k[j] ) {
				j++;
			}
			return j;
		};
	
		aLayout.splice( 0, aLayout.length );
	
		/* We know how many rows there are in the layout - so prep it */
		for ( i=0, iLen=nTrs.length ; i<iLen ; i++ )
		{
			aLayout.push( [] );
		}
	
		/* Calculate a layout array */
		for ( i=0, iLen=nTrs.length ; i<iLen ; i++ )
		{
			nTr = nTrs[i];
			iColumn = 0;
	
			/* For every cell in the row... */
			nCell = nTr.firstChild;
			while ( nCell ) {
				if ( nCell.nodeName.toUpperCase() == "TD" ||
				     nCell.nodeName.toUpperCase() == "TH" )
				{
					/* Get the col and rowspan attributes from the DOM and sanitise them */
					iColspan = nCell.getAttribute('colspan') * 1;
					iRowspan = nCell.getAttribute('rowspan') * 1;
					iColspan = (!iColspan || iColspan===0 || iColspan===1) ? 1 : iColspan;
					iRowspan = (!iRowspan || iRowspan===0 || iRowspan===1) ? 1 : iRowspan;
	
					/* There might be colspan cells already in this row, so shift our target
					 * accordingly
					 */
					iColShifted = fnShiftCol( aLayout, i, iColumn );
	
					/* Cache calculation for unique columns */
					bUnique = iColspan === 1 ? true : false;
	
					/* If there is col / rowspan, copy the information into the layout grid */
					for ( l=0 ; l<iColspan ; l++ )
					{
						for ( k=0 ; k<iRowspan ; k++ )
						{
							aLayout[i+k][iColShifted+l] = {
								"cell": nCell,
								"unique": bUnique
							};
							aLayout[i+k].nTr = nTr;
						}
					}
				}
				nCell = nCell.nextSibling;
			}
		}
	}
	
	
	/**
	 * Get an array of unique th elements, one for each column
	 *  @param {object} oSettings dataTables settings object
	 *  @param {node} nHeader automatically detect the layout from this node - optional
	 *  @param {array} aLayout thead/tfoot layout from _fnDetectHeader - optional
	 *  @returns array {node} aReturn list of unique th's
	 *  @memberof DataTable#oApi
	 */
	function _fnGetUniqueThs ( oSettings, nHeader, aLayout )
	{
		var aReturn = [];
		if ( !aLayout )
		{
			aLayout = oSettings.aoHeader;
			if ( nHeader )
			{
				aLayout = [];
				_fnDetectHeader( aLayout, nHeader );
			}
		}
	
		for ( var i=0, iLen=aLayout.length ; i<iLen ; i++ )
		{
			for ( var j=0, jLen=aLayout[i].length ; j<jLen ; j++ )
			{
				if ( aLayout[i][j].unique &&
					 (!aReturn[j] || !oSettings.bSortCellsTop) )
				{
					aReturn[j] = aLayout[i][j].cell;
				}
			}
		}
	
		return aReturn;
	}
	
	/**
	 * Create an Ajax call based on the table's settings, taking into account that
	 * parameters can have multiple forms, and backwards compatibility.
	 *
	 * @param {object} oSettings dataTables settings object
	 * @param {array} data Data to send to the server, required by
	 *     DataTables - may be augmented by developer callbacks
	 * @param {function} fn Callback function to run when data is obtained
	 */
	function _fnBuildAjax( oSettings, data, fn )
	{
		// Compatibility with 1.9-, allow fnServerData and event to manipulate
		_fnCallbackFire( oSettings, 'aoServerParams', 'serverParams', [data] );
	
		// Convert to object based for 1.10+ if using the old array scheme which can
		// come from server-side processing or serverParams
		if ( data && $.isArray(data) ) {
			var tmp = {};
			var rbracket = /(.*?)\[\]$/;
	
			$.each( data, function (key, val) {
				var match = val.name.match(rbracket);
	
				if ( match ) {
					// Support for arrays
					var name = match[0];
	
					if ( ! tmp[ name ] ) {
						tmp[ name ] = [];
					}
					tmp[ name ].push( val.value );
				}
				else {
					tmp[val.name] = val.value;
				}
			} );
			data = tmp;
		}
	
		var ajaxData;
		var ajax = oSettings.ajax;
		var instance = oSettings.oInstance;
		var callback = function ( json ) {
			_fnCallbackFire( oSettings, null, 'xhr', [oSettings, json, oSettings.jqXHR] );
			fn( json );
		};
	
		if ( $.isPlainObject( ajax ) && ajax.data )
		{
			ajaxData = ajax.data;
	
			var newData = $.isFunction( ajaxData ) ?
				ajaxData( data, oSettings ) :  // fn can manipulate data or return
				ajaxData;                      // an object object or array to merge
	
			// If the function returned something, use that alone
			data = $.isFunction( ajaxData ) && newData ?
				newData :
				$.extend( true, data, newData );
	
			// Remove the data property as we've resolved it already and don't want
			// jQuery to do it again (it is restored at the end of the function)
			delete ajax.data;
		}
	
		var baseAjax = {
			"data": data,
			"success": function (json) {
				var error = json.error || json.sError;
				if ( error ) {
					_fnLog( oSettings, 0, error );
				}
	
				oSettings.json = json;
				callback( json );
			},
			"dataType": "json",
			"cache": false,
			"type": oSettings.sServerMethod,
			"error": function (xhr, error, thrown) {
				var ret = _fnCallbackFire( oSettings, null, 'xhr', [oSettings, null, oSettings.jqXHR] );
	
				if ( $.inArray( true, ret ) === -1 ) {
					if ( error == "parsererror" ) {
						_fnLog( oSettings, 0, 'Invalid JSON response', 1 );
					}
					else if ( xhr.readyState === 4 ) {
						_fnLog( oSettings, 0, 'Ajax error', 7 );
					}
				}
	
				_fnProcessingDisplay( oSettings, false );
			}
		};
	
		// Store the data submitted for the API
		oSettings.oAjaxData = data;
	
		// Allow plug-ins and external processes to modify the data
		_fnCallbackFire( oSettings, null, 'preXhr', [oSettings, data] );
	
		if ( oSettings.fnServerData )
		{
			// DataTables 1.9- compatibility
			oSettings.fnServerData.call( instance,
				oSettings.sAjaxSource,
				$.map( data, function (val, key) { // Need to convert back to 1.9 trad format
					return { name: key, value: val };
				} ),
				callback,
				oSettings
			);
		}
		else if ( oSettings.sAjaxSource || typeof ajax === 'string' )
		{
			// DataTables 1.9- compatibility
			oSettings.jqXHR = $.ajax( $.extend( baseAjax, {
				url: ajax || oSettings.sAjaxSource
			} ) );
		}
		else if ( $.isFunction( ajax ) )
		{
			// Is a function - let the caller define what needs to be done
			oSettings.jqXHR = ajax.call( instance, data, callback, oSettings );
		}
		else
		{
			// Object to extend the base settings
			oSettings.jqXHR = $.ajax( $.extend( baseAjax, ajax ) );
	
			// Restore for next time around
			ajax.data = ajaxData;
		}
	}
	
	
	/**
	 * Update the table using an Ajax call
	 *  @param {object} settings dataTables settings object
	 *  @returns {boolean} Block the table drawing or not
	 *  @memberof DataTable#oApi
	 */
	function _fnAjaxUpdate( settings )
	{
		if ( settings.bAjaxDataGet ) {
			settings.iDraw++;
			_fnProcessingDisplay( settings, true );
	
			_fnBuildAjax(
				settings,
				_fnAjaxParameters( settings ),
				function(json) {
					_fnAjaxUpdateDraw( settings, json );
				}
			);
	
			return false;
		}
		return true;
	}
	
	
	/**
	 * Build up the parameters in an object needed for a server-side processing
	 * request. Note that this is basically done twice, is different ways - a modern
	 * method which is used by default in DataTables 1.10 which uses objects and
	 * arrays, or the 1.9- method with is name / value pairs. 1.9 method is used if
	 * the sAjaxSource option is used in the initialisation, or the legacyAjax
	 * option is set.
	 *  @param {object} oSettings dataTables settings object
	 *  @returns {bool} block the table drawing or not
	 *  @memberof DataTable#oApi
	 */
	function _fnAjaxParameters( settings )
	{
		var
			columns = settings.aoColumns,
			columnCount = columns.length,
			features = settings.oFeatures,
			preSearch = settings.oPreviousSearch,
			preColSearch = settings.aoPreSearchCols,
			i, data = [], dataProp, column, columnSearch,
			sort = _fnSortFlatten( settings ),
			displayStart = settings._iDisplayStart,
			displayLength = features.bPaginate !== false ?
				settings._iDisplayLength :
				-1;
	
		var param = function ( name, value ) {
			data.push( { 'name': name, 'value': value } );
		};
	
		// DataTables 1.9- compatible method
		param( 'sEcho',          settings.iDraw );
		param( 'iColumns',       columnCount );
		param( 'sColumns',       _pluck( columns, 'sName' ).join(',') );
		param( 'iDisplayStart',  displayStart );
		param( 'iDisplayLength', displayLength );
	
		// DataTables 1.10+ method
		var d = {
			draw:    settings.iDraw,
			columns: [],
			order:   [],
			start:   displayStart,
			length:  displayLength,
			search:  {
				value: preSearch.sSearch,
				regex: preSearch.bRegex
			}
		};
	
		for ( i=0 ; i<columnCount ; i++ ) {
			column = columns[i];
			columnSearch = preColSearch[i];
			dataProp = typeof column.mData=="function" ? 'function' : column.mData ;
	
			d.columns.push( {
				data:       dataProp,
				name:       column.sName,
				searchable: column.bSearchable,
				orderable:  column.bSortable,
				search:     {
					value: columnSearch.sSearch,
					regex: columnSearch.bRegex
				}
			} );
	
			param( "mDataProp_"+i, dataProp );
	
			if ( features.bFilter ) {
				param( 'sSearch_'+i,     columnSearch.sSearch );
				param( 'bRegex_'+i,      columnSearch.bRegex );
				param( 'bSearchable_'+i, column.bSearchable );
			}
	
			if ( features.bSort ) {
				param( 'bSortable_'+i, column.bSortable );
			}
		}
	
		if ( features.bFilter ) {
			param( 'sSearch', preSearch.sSearch );
			param( 'bRegex', preSearch.bRegex );
		}
	
		if ( features.bSort ) {
			$.each( sort, function ( i, val ) {
				d.order.push( { column: val.col, dir: val.dir } );
	
				param( 'iSortCol_'+i, val.col );
				param( 'sSortDir_'+i, val.dir );
			} );
	
			param( 'iSortingCols', sort.length );
		}
	
		// If the legacy.ajax parameter is null, then we automatically decide which
		// form to use, based on sAjaxSource
		var legacy = DataTable.ext.legacy.ajax;
		if ( legacy === null ) {
			return settings.sAjaxSource ? data : d;
		}
	
		// Otherwise, if legacy has been specified then we use that to decide on the
		// form
		return legacy ? data : d;
	}
	
	
	/**
	 * Data the data from the server (nuking the old) and redraw the table
	 *  @param {object} oSettings dataTables settings object
	 *  @param {object} json json data return from the server.
	 *  @param {string} json.sEcho Tracking flag for DataTables to match requests
	 *  @param {int} json.iTotalRecords Number of records in the data set, not accounting for filtering
	 *  @param {int} json.iTotalDisplayRecords Number of records in the data set, accounting for filtering
	 *  @param {array} json.aaData The data to display on this page
	 *  @param {string} [json.sColumns] Column ordering (sName, comma separated)
	 *  @memberof DataTable#oApi
	 */
	function _fnAjaxUpdateDraw ( settings, json )
	{
		// v1.10 uses camelCase variables, while 1.9 uses Hungarian notation.
		// Support both
		var compat = function ( old, modern ) {
			return json[old] !== undefined ? json[old] : json[modern];
		};
	
		var data = _fnAjaxDataSrc( settings, json );
		var draw            = compat( 'sEcho',                'draw' );
		var recordsTotal    = compat( 'iTotalRecords',        'recordsTotal' );
		var recordsFiltered = compat( 'iTotalDisplayRecords', 'recordsFiltered' );
	
		if ( draw ) {
			// Protect against out of sequence returns
			if ( draw*1 < settings.iDraw ) {
				return;
			}
			settings.iDraw = draw * 1;
		}
	
		_fnClearTable( settings );
		settings._iRecordsTotal   = parseInt(recordsTotal, 10);
		settings._iRecordsDisplay = parseInt(recordsFiltered, 10);
	
		for ( var i=0, ien=data.length ; i<ien ; i++ ) {
			_fnAddData( settings, data[i] );
		}
		settings.aiDisplay = settings.aiDisplayMaster.slice();
	
		settings.bAjaxDataGet = false;
		_fnDraw( settings );
	
		if ( ! settings._bInitComplete ) {
			_fnInitComplete( settings, json );
		}
	
		settings.bAjaxDataGet = true;
		_fnProcessingDisplay( settings, false );
	}
	
	
	/**
	 * Get the data from the JSON data source to use for drawing a table. Using
	 * `_fnGetObjectDataFn` allows the data to be sourced from a property of the
	 * source object, or from a processing function.
	 *  @param {object} oSettings dataTables settings object
	 *  @param  {object} json Data source object / array from the server
	 *  @return {array} Array of data to use
	 */
	function _fnAjaxDataSrc ( oSettings, json )
	{
		var dataSrc = $.isPlainObject( oSettings.ajax ) && oSettings.ajax.dataSrc !== undefined ?
			oSettings.ajax.dataSrc :
			oSettings.sAjaxDataProp; // Compatibility with 1.9-.
	
		// Compatibility with 1.9-. In order to read from aaData, check if the
		// default has been changed, if not, check for aaData
		if ( dataSrc === 'data' ) {
			return json.aaData || json[dataSrc];
		}
	
		return dataSrc !== "" ?
			_fnGetObjectDataFn( dataSrc )( json ) :
			json;
	}
	
	/**
	 * Generate the node required for filtering text
	 *  @returns {node} Filter control element
	 *  @param {object} oSettings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnFeatureHtmlFilter ( settings )
	{
		var classes = settings.oClasses;
		var tableId = settings.sTableId;
		var language = settings.oLanguage;
		var previousSearch = settings.oPreviousSearch;
		var features = settings.aanFeatures;
		var input = '<input type="search" class="'+classes.sFilterInput+'"/>';
	
		var str = language.sSearch;
		str = str.match(/_INPUT_/) ?
			str.replace('_INPUT_', input) :
			str+input;
	
		var filter = $('<div/>', {
				'id': ! features.f ? tableId+'_filter' : null,
				'class': classes.sFilter
			} )
			.append( $('<label/>' ).append( str ) );
	
		var searchFn = function() {
			/* Update all other filter input elements for the new display */
			var n = features.f;
			var val = !this.value ? "" : this.value; // mental IE8 fix :-(
	
			/* Now do the filter */
			if ( val != previousSearch.sSearch ) {
				_fnFilterComplete( settings, {
					"sSearch": val,
					"bRegex": previousSearch.bRegex,
					"bSmart": previousSearch.bSmart ,
					"bCaseInsensitive": previousSearch.bCaseInsensitive
				} );
	
				// Need to redraw, without resorting
				settings._iDisplayStart = 0;
				_fnDraw( settings );
			}
		};
	
		var searchDelay = settings.searchDelay !== null ?
			settings.searchDelay :
			_fnDataSource( settings ) === 'ssp' ?
				400 :
				0;
	
		var jqFilter = $('input', filter)
			.val( previousSearch.sSearch )
			.attr( 'placeholder', language.sSearchPlaceholder )
			.bind(
				'keyup.DT search.DT input.DT paste.DT cut.DT',
				searchDelay ?
					_fnThrottle( searchFn, searchDelay ) :
					searchFn
			)
			.bind( 'keypress.DT', function(e) {
				/* Prevent form submission */
				if ( e.keyCode == 13 ) {
					return false;
				}
			} )
			.attr('aria-controls', tableId);
	
		// Update the input elements whenever the table is filtered
		$(settings.nTable).on( 'search.dt.DT', function ( ev, s ) {
			if ( settings === s ) {
				// IE9 throws an 'unknown error' if document.activeElement is used
				// inside an iframe or frame...
				try {
					if ( jqFilter[0] !== document.activeElement ) {
						jqFilter.val( previousSearch.sSearch );
					}
				}
				catch ( e ) {}
			}
		} );
	
		return filter[0];
	}
	
	
	/**
	 * Filter the table using both the global filter and column based filtering
	 *  @param {object} oSettings dataTables settings object
	 *  @param {object} oSearch search information
	 *  @param {int} [iForce] force a research of the master array (1) or not (undefined or 0)
	 *  @memberof DataTable#oApi
	 */
	function _fnFilterComplete ( oSettings, oInput, iForce )
	{
		var oPrevSearch = oSettings.oPreviousSearch;
		var aoPrevSearch = oSettings.aoPreSearchCols;
		var fnSaveFilter = function ( oFilter ) {
			/* Save the filtering values */
			oPrevSearch.sSearch = oFilter.sSearch;
			oPrevSearch.bRegex = oFilter.bRegex;
			oPrevSearch.bSmart = oFilter.bSmart;
			oPrevSearch.bCaseInsensitive = oFilter.bCaseInsensitive;
		};
		var fnRegex = function ( o ) {
			// Backwards compatibility with the bEscapeRegex option
			return o.bEscapeRegex !== undefined ? !o.bEscapeRegex : o.bRegex;
		};
	
		// Resolve any column types that are unknown due to addition or invalidation
		// @todo As per sort - can this be moved into an event handler?
		_fnColumnTypes( oSettings );
	
		/* In server-side processing all filtering is done by the server, so no point hanging around here */
		if ( _fnDataSource( oSettings ) != 'ssp' )
		{
			/* Global filter */
			_fnFilter( oSettings, oInput.sSearch, iForce, fnRegex(oInput), oInput.bSmart, oInput.bCaseInsensitive );
			fnSaveFilter( oInput );
	
			/* Now do the individual column filter */
			for ( var i=0 ; i<aoPrevSearch.length ; i++ )
			{
				_fnFilterColumn( oSettings, aoPrevSearch[i].sSearch, i, fnRegex(aoPrevSearch[i]),
					aoPrevSearch[i].bSmart, aoPrevSearch[i].bCaseInsensitive );
			}
	
			/* Custom filtering */
			_fnFilterCustom( oSettings );
		}
		else
		{
			fnSaveFilter( oInput );
		}
	
		/* Tell the draw function we have been filtering */
		oSettings.bFiltered = true;
		_fnCallbackFire( oSettings, null, 'search', [oSettings] );
	}
	
	
	/**
	 * Apply custom filtering functions
	 *  @param {object} oSettings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnFilterCustom( settings )
	{
		var filters = DataTable.ext.search;
		var displayRows = settings.aiDisplay;
		var row, rowIdx;
	
		for ( var i=0, ien=filters.length ; i<ien ; i++ ) {
			var rows = [];
	
			// Loop over each row and see if it should be included
			for ( var j=0, jen=displayRows.length ; j<jen ; j++ ) {
				rowIdx = displayRows[ j ];
				row = settings.aoData[ rowIdx ];
	
				if ( filters[i]( settings, row._aFilterData, rowIdx, row._aData, j ) ) {
					rows.push( rowIdx );
				}
			}
	
			// So the array reference doesn't break set the results into the
			// existing array
			displayRows.length = 0;
			displayRows.push.apply( displayRows, rows );
		}
	}
	
	
	/**
	 * Filter the table on a per-column basis
	 *  @param {object} oSettings dataTables settings object
	 *  @param {string} sInput string to filter on
	 *  @param {int} iColumn column to filter
	 *  @param {bool} bRegex treat search string as a regular expression or not
	 *  @param {bool} bSmart use smart filtering or not
	 *  @param {bool} bCaseInsensitive Do case insenstive matching or not
	 *  @memberof DataTable#oApi
	 */
	function _fnFilterColumn ( settings, searchStr, colIdx, regex, smart, caseInsensitive )
	{
		if ( searchStr === '' ) {
			return;
		}
	
		var data;
		var display = settings.aiDisplay;
		var rpSearch = _fnFilterCreateSearch( searchStr, regex, smart, caseInsensitive );
	
		for ( var i=display.length-1 ; i>=0 ; i-- ) {
			data = settings.aoData[ display[i] ]._aFilterData[ colIdx ];
	
			if ( ! rpSearch.test( data ) ) {
				display.splice( i, 1 );
			}
		}
	}
	
	
	/**
	 * Filter the data table based on user input and draw the table
	 *  @param {object} settings dataTables settings object
	 *  @param {string} input string to filter on
	 *  @param {int} force optional - force a research of the master array (1) or not (undefined or 0)
	 *  @param {bool} regex treat as a regular expression or not
	 *  @param {bool} smart perform smart filtering or not
	 *  @param {bool} caseInsensitive Do case insenstive matching or not
	 *  @memberof DataTable#oApi
	 */
	function _fnFilter( settings, input, force, regex, smart, caseInsensitive )
	{
		var rpSearch = _fnFilterCreateSearch( input, regex, smart, caseInsensitive );
		var prevSearch = settings.oPreviousSearch.sSearch;
		var displayMaster = settings.aiDisplayMaster;
		var display, invalidated, i;
	
		// Need to take account of custom filtering functions - always filter
		if ( DataTable.ext.search.length !== 0 ) {
			force = true;
		}
	
		// Check if any of the rows were invalidated
		invalidated = _fnFilterData( settings );
	
		// If the input is blank - we just want the full data set
		if ( input.length <= 0 ) {
			settings.aiDisplay = displayMaster.slice();
		}
		else {
			// New search - start from the master array
			if ( invalidated ||
				 force ||
				 prevSearch.length > input.length ||
				 input.indexOf(prevSearch) !== 0 ||
				 settings.bSorted // On resort, the display master needs to be
				                  // re-filtered since indexes will have changed
			) {
				settings.aiDisplay = displayMaster.slice();
			}
	
			// Search the display array
			display = settings.aiDisplay;
	
			for ( i=display.length-1 ; i>=0 ; i-- ) {
				if ( ! rpSearch.test( settings.aoData[ display[i] ]._sFilterRow ) ) {
					display.splice( i, 1 );
				}
			}
		}
	}
	
	
	/**
	 * Build a regular expression object suitable for searching a table
	 *  @param {string} sSearch string to search for
	 *  @param {bool} bRegex treat as a regular expression or not
	 *  @param {bool} bSmart perform smart filtering or not
	 *  @param {bool} bCaseInsensitive Do case insensitive matching or not
	 *  @returns {RegExp} constructed object
	 *  @memberof DataTable#oApi
	 */
	function _fnFilterCreateSearch( search, regex, smart, caseInsensitive )
	{
		search = regex ?
			search :
			_fnEscapeRegex( search );
		
		if ( smart ) {
			/* For smart filtering we want to allow the search to work regardless of
			 * word order. We also want double quoted text to be preserved, so word
			 * order is important - a la google. So this is what we want to
			 * generate:
			 * 
			 * ^(?=.*?\bone\b)(?=.*?\btwo three\b)(?=.*?\bfour\b).*$
			 */
			var a = $.map( search.match( /"[^"]+"|[^ ]+/g ) || [''], function ( word ) {
				if ( word.charAt(0) === '"' ) {
					var m = word.match( /^"(.*)"$/ );
					word = m ? m[1] : word;
				}
	
				return word.replace('"', '');
			} );
	
			search = '^(?=.*?'+a.join( ')(?=.*?' )+').*$';
		}
	
		return new RegExp( search, caseInsensitive ? 'i' : '' );
	}
	
	
	/**
	 * Escape a string such that it can be used in a regular expression
	 *  @param {string} sVal string to escape
	 *  @returns {string} escaped string
	 *  @memberof DataTable#oApi
	 */
	function _fnEscapeRegex ( sVal )
	{
		return sVal.replace( _re_escape_regex, '\\$1' );
	}
	
	
	
	var __filter_div = $('<div>')[0];
	var __filter_div_textContent = __filter_div.textContent !== undefined;
	
	// Update the filtering data for each row if needed (by invalidation or first run)
	function _fnFilterData ( settings )
	{
		var columns = settings.aoColumns;
		var column;
		var i, j, ien, jen, filterData, cellData, row;
		var fomatters = DataTable.ext.type.search;
		var wasInvalidated = false;
	
		for ( i=0, ien=settings.aoData.length ; i<ien ; i++ ) {
			row = settings.aoData[i];
	
			if ( ! row._aFilterData ) {
				filterData = [];
	
				for ( j=0, jen=columns.length ; j<jen ; j++ ) {
					column = columns[j];
	
					if ( column.bSearchable ) {
						cellData = _fnGetCellData( settings, i, j, 'filter' );
	
						if ( fomatters[ column.sType ] ) {
							cellData = fomatters[ column.sType ]( cellData );
						}
	
						// Search in DataTables 1.10 is string based. In 1.11 this
						// should be altered to also allow strict type checking.
						if ( cellData === null ) {
							cellData = '';
						}
	
						if ( typeof cellData !== 'string' && cellData.toString ) {
							cellData = cellData.toString();
						}
					}
					else {
						cellData = '';
					}
	
					// If it looks like there is an HTML entity in the string,
					// attempt to decode it so sorting works as expected. Note that
					// we could use a single line of jQuery to do this, but the DOM
					// method used here is much faster http://jsperf.com/html-decode
					if ( cellData.indexOf && cellData.indexOf('&') !== -1 ) {
						__filter_div.innerHTML = cellData;
						cellData = __filter_div_textContent ?
							__filter_div.textContent :
							__filter_div.innerText;
					}
	
					if ( cellData.replace ) {
						cellData = cellData.replace(/[\r\n]/g, '');
					}
	
					filterData.push( cellData );
				}
	
				row._aFilterData = filterData;
				row._sFilterRow = filterData.join('  ');
				wasInvalidated = true;
			}
		}
	
		return wasInvalidated;
	}
	
	
	/**
	 * Convert from the internal Hungarian notation to camelCase for external
	 * interaction
	 *  @param {object} obj Object to convert
	 *  @returns {object} Inverted object
	 *  @memberof DataTable#oApi
	 */
	function _fnSearchToCamel ( obj )
	{
		return {
			search:          obj.sSearch,
			smart:           obj.bSmart,
			regex:           obj.bRegex,
			caseInsensitive: obj.bCaseInsensitive
		};
	}
	
	
	
	/**
	 * Convert from camelCase notation to the internal Hungarian. We could use the
	 * Hungarian convert function here, but this is cleaner
	 *  @param {object} obj Object to convert
	 *  @returns {object} Inverted object
	 *  @memberof DataTable#oApi
	 */
	function _fnSearchToHung ( obj )
	{
		return {
			sSearch:          obj.search,
			bSmart:           obj.smart,
			bRegex:           obj.regex,
			bCaseInsensitive: obj.caseInsensitive
		};
	}
	
	/**
	 * Generate the node required for the info display
	 *  @param {object} oSettings dataTables settings object
	 *  @returns {node} Information element
	 *  @memberof DataTable#oApi
	 */
	function _fnFeatureHtmlInfo ( settings )
	{
		var
			tid = settings.sTableId,
			nodes = settings.aanFeatures.i,
			n = $('<div/>', {
				'class': settings.oClasses.sInfo,
				'id': ! nodes ? tid+'_info' : null
			} );
	
		if ( ! nodes ) {
			// Update display on each draw
			settings.aoDrawCallback.push( {
				"fn": _fnUpdateInfo,
				"sName": "information"
			} );
	
			n
				.attr( 'role', 'status' )
				.attr( 'aria-live', 'polite' );
	
			// Table is described by our info div
			$(settings.nTable).attr( 'aria-describedby', tid+'_info' );
		}
	
		return n[0];
	}
	
	
	/**
	 * Update the information elements in the display
	 *  @param {object} settings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnUpdateInfo ( settings )
	{
		/* Show information about the table */
		var nodes = settings.aanFeatures.i;
		if ( nodes.length === 0 ) {
			return;
		}
	
		var
			lang  = settings.oLanguage,
			start = settings._iDisplayStart+1,
			end   = settings.fnDisplayEnd(),
			max   = settings.fnRecordsTotal(),
			total = settings.fnRecordsDisplay(),
			out   = total ?
				lang.sInfo :
				lang.sInfoEmpty;
	
		if ( total !== max ) {
			/* Record set after filtering */
			out += ' ' + lang.sInfoFiltered;
		}
	
		// Convert the macros
		out += lang.sInfoPostFix;
		out = _fnInfoMacros( settings, out );
	
		var callback = lang.fnInfoCallback;
		if ( callback !== null ) {
			out = callback.call( settings.oInstance,
				settings, start, end, max, total, out
			);
		}
	
		$(nodes).html( out );
	}
	
	
	function _fnInfoMacros ( settings, str )
	{
		// When infinite scrolling, we are always starting at 1. _iDisplayStart is used only
		// internally
		var
			formatter  = settings.fnFormatNumber,
			start      = settings._iDisplayStart+1,
			len        = settings._iDisplayLength,
			vis        = settings.fnRecordsDisplay(),
			all        = len === -1;
	
		return str.
			replace(/_START_/g, formatter.call( settings, start ) ).
			replace(/_END_/g,   formatter.call( settings, settings.fnDisplayEnd() ) ).
			replace(/_MAX_/g,   formatter.call( settings, settings.fnRecordsTotal() ) ).
			replace(/_TOTAL_/g, formatter.call( settings, vis ) ).
			replace(/_PAGE_/g,  formatter.call( settings, all ? 1 : Math.ceil( start / len ) ) ).
			replace(/_PAGES_/g, formatter.call( settings, all ? 1 : Math.ceil( vis / len ) ) );
	}
	
	
	
	/**
	 * Draw the table for the first time, adding all required features
	 *  @param {object} settings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnInitialise ( settings )
	{
		var i, iLen, iAjaxStart=settings.iInitDisplayStart;
		var columns = settings.aoColumns, column;
		var features = settings.oFeatures;
	
		/* Ensure that the table data is fully initialised */
		if ( ! settings.bInitialised ) {
			setTimeout( function(){ _fnInitialise( settings ); }, 200 );
			return;
		}
	
		/* Show the display HTML options */
		_fnAddOptionsHtml( settings );
	
		/* Build and draw the header / footer for the table */
		_fnBuildHead( settings );
		_fnDrawHead( settings, settings.aoHeader );
		_fnDrawHead( settings, settings.aoFooter );
	
		/* Okay to show that something is going on now */
		_fnProcessingDisplay( settings, true );
	
		/* Calculate sizes for columns */
		if ( features.bAutoWidth ) {
			_fnCalculateColumnWidths( settings );
		}
	
		for ( i=0, iLen=columns.length ; i<iLen ; i++ ) {
			column = columns[i];
	
			if ( column.sWidth ) {
				column.nTh.style.width = _fnStringToCss( column.sWidth );
			}
		}
	
		// If there is default sorting required - let's do it. The sort function
		// will do the drawing for us. Otherwise we draw the table regardless of the
		// Ajax source - this allows the table to look initialised for Ajax sourcing
		// data (show 'loading' message possibly)
		_fnReDraw( settings );
	
		// Server-side processing init complete is done by _fnAjaxUpdateDraw
		var dataSrc = _fnDataSource( settings );
		if ( dataSrc != 'ssp' ) {
			// if there is an ajax source load the data
			if ( dataSrc == 'ajax' ) {
				_fnBuildAjax( settings, [], function(json) {
					var aData = _fnAjaxDataSrc( settings, json );
	
					// Got the data - add it to the table
					for ( i=0 ; i<aData.length ; i++ ) {
						_fnAddData( settings, aData[i] );
					}
	
					// Reset the init display for cookie saving. We've already done
					// a filter, and therefore cleared it before. So we need to make
					// it appear 'fresh'
					settings.iInitDisplayStart = iAjaxStart;
	
					_fnReDraw( settings );
	
					_fnProcessingDisplay( settings, false );
					_fnInitComplete( settings, json );
				}, settings );
			}
			else {
				_fnProcessingDisplay( settings, false );
				_fnInitComplete( settings );
			}
		}
	}
	
	
	/**
	 * Draw the table for the first time, adding all required features
	 *  @param {object} oSettings dataTables settings object
	 *  @param {object} [json] JSON from the server that completed the table, if using Ajax source
	 *    with client-side processing (optional)
	 *  @memberof DataTable#oApi
	 */
	function _fnInitComplete ( settings, json )
	{
		settings._bInitComplete = true;
	
		// On an Ajax load we now have data and therefore want to apply the column
		// sizing
		if ( json ) {
			_fnAdjustColumnSizing( settings );
		}
	
		_fnCallbackFire( settings, 'aoInitComplete', 'init', [settings, json] );
	}
	
	
	function _fnLengthChange ( settings, val )
	{
		var len = parseInt( val, 10 );
		settings._iDisplayLength = len;
	
		_fnLengthOverflow( settings );
	
		// Fire length change event
		_fnCallbackFire( settings, null, 'length', [settings, len] );
	}
	
	
	/**
	 * Generate the node required for user display length changing
	 *  @param {object} settings dataTables settings object
	 *  @returns {node} Display length feature node
	 *  @memberof DataTable#oApi
	 */
	function _fnFeatureHtmlLength ( settings )
	{
		var
			classes  = settings.oClasses,
			tableId  = settings.sTableId,
			menu     = settings.aLengthMenu,
			d2       = $.isArray( menu[0] ),
			lengths  = d2 ? menu[0] : menu,
			language = d2 ? menu[1] : menu;
	
		var select = $('<select/>', {
			'name':          tableId+'_length',
			'aria-controls': tableId,
			'class':         classes.sLengthSelect
		} );
	
		for ( var i=0, ien=lengths.length ; i<ien ; i++ ) {
			select[0][ i ] = new Option( language[i], lengths[i] );
		}
	
		var div = $('<div><label/></div>').addClass( classes.sLength );
		if ( ! settings.aanFeatures.l ) {
			div[0].id = tableId+'_length';
		}
	
		div.children().append(
			settings.oLanguage.sLengthMenu.replace( '_MENU_', select[0].outerHTML )
		);
	
		// Can't use `select` variable as user might provide their own and the
		// reference is broken by the use of outerHTML
		$('select', div)
			.val( settings._iDisplayLength )
			.bind( 'change.DT', function(e) {
				_fnLengthChange( settings, $(this).val() );
				_fnDraw( settings );
			} );
	
		// Update node value whenever anything changes the table's length
		$(settings.nTable).bind( 'length.dt.DT', function (e, s, len) {
			if ( settings === s ) {
				$('select', div).val( len );
			}
		} );
	
		return div[0];
	}
	
	
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * Note that most of the paging logic is done in
	 * DataTable.ext.pager
	 */
	
	/**
	 * Generate the node required for default pagination
	 *  @param {object} oSettings dataTables settings object
	 *  @returns {node} Pagination feature node
	 *  @memberof DataTable#oApi
	 */
	function _fnFeatureHtmlPaginate ( settings )
	{
		var
			type   = settings.sPaginationType,
			plugin = DataTable.ext.pager[ type ],
			modern = typeof plugin === 'function',
			redraw = function( settings ) {
				_fnDraw( settings );
			},
			node = $('<div/>').addClass( settings.oClasses.sPaging + type )[0],
			features = settings.aanFeatures;
	
		if ( ! modern ) {
			plugin.fnInit( settings, node, redraw );
		}
	
		/* Add a draw callback for the pagination on first instance, to update the paging display */
		if ( ! features.p )
		{
			node.id = settings.sTableId+'_paginate';
	
			settings.aoDrawCallback.push( {
				"fn": function( settings ) {
					if ( modern ) {
						var
							start      = settings._iDisplayStart,
							len        = settings._iDisplayLength,
							visRecords = settings.fnRecordsDisplay(),
							all        = len === -1,
							page = all ? 0 : Math.ceil( start / len ),
							pages = all ? 1 : Math.ceil( visRecords / len ),
							buttons = plugin(page, pages),
							i, ien;
	
						for ( i=0, ien=features.p.length ; i<ien ; i++ ) {
							_fnRenderer( settings, 'pageButton' )(
								settings, features.p[i], i, buttons, page, pages
							);
						}
					}
					else {
						plugin.fnUpdate( settings, redraw );
					}
				},
				"sName": "pagination"
			} );
		}
	
		return node;
	}
	
	
	/**
	 * Alter the display settings to change the page
	 *  @param {object} settings DataTables settings object
	 *  @param {string|int} action Paging action to take: "first", "previous",
	 *    "next" or "last" or page number to jump to (integer)
	 *  @param [bool] redraw Automatically draw the update or not
	 *  @returns {bool} true page has changed, false - no change
	 *  @memberof DataTable#oApi
	 */
	function _fnPageChange ( settings, action, redraw )
	{
		var
			start     = settings._iDisplayStart,
			len       = settings._iDisplayLength,
			records   = settings.fnRecordsDisplay();
	
		if ( records === 0 || len === -1 )
		{
			start = 0;
		}
		else if ( typeof action === "number" )
		{
			start = action * len;
	
			if ( start > records )
			{
				start = 0;
			}
		}
		else if ( action == "first" )
		{
			start = 0;
		}
		else if ( action == "previous" )
		{
			start = len >= 0 ?
				start - len :
				0;
	
			if ( start < 0 )
			{
			  start = 0;
			}
		}
		else if ( action == "next" )
		{
			if ( start + len < records )
			{
				start += len;
			}
		}
		else if ( action == "last" )
		{
			start = Math.floor( (records-1) / len) * len;
		}
		else
		{
			_fnLog( settings, 0, "Unknown paging action: "+action, 5 );
		}
	
		var changed = settings._iDisplayStart !== start;
		settings._iDisplayStart = start;
	
		if ( changed ) {
			_fnCallbackFire( settings, null, 'page', [settings] );
	
			if ( redraw ) {
				_fnDraw( settings );
			}
		}
	
		return changed;
	}
	
	
	
	/**
	 * Generate the node required for the processing node
	 *  @param {object} settings dataTables settings object
	 *  @returns {node} Processing element
	 *  @memberof DataTable#oApi
	 */
	function _fnFeatureHtmlProcessing ( settings )
	{
		return $('<div/>', {
				'id': ! settings.aanFeatures.r ? settings.sTableId+'_processing' : null,
				'class': settings.oClasses.sProcessing
			} )
			.html( settings.oLanguage.sProcessing )
			.insertBefore( settings.nTable )[0];
	}
	
	
	/**
	 * Display or hide the processing indicator
	 *  @param {object} settings dataTables settings object
	 *  @param {bool} show Show the processing indicator (true) or not (false)
	 *  @memberof DataTable#oApi
	 */
	function _fnProcessingDisplay ( settings, show )
	{
		if ( settings.oFeatures.bProcessing ) {
			$(settings.aanFeatures.r).css( 'display', show ? 'block' : 'none' );
		}
	
		_fnCallbackFire( settings, null, 'processing', [settings, show] );
	}
	
	/**
	 * Add any control elements for the table - specifically scrolling
	 *  @param {object} settings dataTables settings object
	 *  @returns {node} Node to add to the DOM
	 *  @memberof DataTable#oApi
	 */
	function _fnFeatureHtmlTable ( settings )
	{
		var table = $(settings.nTable);
	
		// Add the ARIA grid role to the table
		table.attr( 'role', 'grid' );
	
		// Scrolling from here on in
		var scroll = settings.oScroll;
	
		if ( scroll.sX === '' && scroll.sY === '' ) {
			return settings.nTable;
		}
	
		var scrollX = scroll.sX;
		var scrollY = scroll.sY;
		var classes = settings.oClasses;
		var caption = table.children('caption');
		var captionSide = caption.length ? caption[0]._captionSide : null;
		var headerClone = $( table[0].cloneNode(false) );
		var footerClone = $( table[0].cloneNode(false) );
		var footer = table.children('tfoot');
		var _div = '<div/>';
		var size = function ( s ) {
			return !s ? null : _fnStringToCss( s );
		};
	
		// This is fairly messy, but with x scrolling enabled, if the table has a
		// width attribute, regardless of any width applied using the column width
		// options, the browser will shrink or grow the table as needed to fit into
		// that 100%. That would make the width options useless. So we remove it.
		// This is okay, under the assumption that width:100% is applied to the
		// table in CSS (it is in the default stylesheet) which will set the table
		// width as appropriate (the attribute and css behave differently...)
		if ( scroll.sX && table.attr('width') === '100%' ) {
			table.removeAttr('width');
		}
	
		if ( ! footer.length ) {
			footer = null;
		}
	
		/*
		 * The HTML structure that we want to generate in this function is:
		 *  div - scroller
		 *    div - scroll head
		 *      div - scroll head inner
		 *        table - scroll head table
		 *          thead - thead
		 *    div - scroll body
		 *      table - table (master table)
		 *        thead - thead clone for sizing
		 *        tbody - tbody
		 *    div - scroll foot
		 *      div - scroll foot inner
		 *        table - scroll foot table
		 *          tfoot - tfoot
		 */
		var scroller = $( _div, { 'class': classes.sScrollWrapper } )
			.append(
				$(_div, { 'class': classes.sScrollHead } )
					.css( {
						overflow: 'hidden',
						position: 'relative',
						border: 0,
						width: scrollX ? size(scrollX) : '100%'
					} )
					.append(
						$(_div, { 'class': classes.sScrollHeadInner } )
							.css( {
								'box-sizing': 'content-box',
								width: scroll.sXInner || '100%'
							} )
							.append(
								headerClone
									.removeAttr('id')
									.css( 'margin-left', 0 )
									.append( captionSide === 'top' ? caption : null )
									.append(
										table.children('thead')
									)
							)
					)
			)
			.append(
				$(_div, { 'class': classes.sScrollBody } )
					.css( {
						overflow: 'auto',
						height: size( scrollY ),
						width: size( scrollX )
					} )
					.append( table )
			);
	
		if ( footer ) {
			scroller.append(
				$(_div, { 'class': classes.sScrollFoot } )
					.css( {
						overflow: 'hidden',
						border: 0,
						width: scrollX ? size(scrollX) : '100%'
					} )
					.append(
						$(_div, { 'class': classes.sScrollFootInner } )
							.append(
								footerClone
									.removeAttr('id')
									.css( 'margin-left', 0 )
									.append( captionSide === 'bottom' ? caption : null )
									.append(
										table.children('tfoot')
									)
							)
					)
			);
		}
	
		var children = scroller.children();
		var scrollHead = children[0];
		var scrollBody = children[1];
		var scrollFoot = footer ? children[2] : null;
	
		// When the body is scrolled, then we also want to scroll the headers
		if ( scrollX ) {
			$(scrollBody).on( 'scroll.DT', function (e) {
				var scrollLeft = this.scrollLeft;
	
				scrollHead.scrollLeft = scrollLeft;
	
				if ( footer ) {
					scrollFoot.scrollLeft = scrollLeft;
				}
			} );
		}
	
		settings.nScrollHead = scrollHead;
		settings.nScrollBody = scrollBody;
		settings.nScrollFoot = scrollFoot;
	
		// On redraw - align columns
		settings.aoDrawCallback.push( {
			"fn": _fnScrollDraw,
			"sName": "scrolling"
		} );
	
		return scroller[0];
	}
	
	
	
	/**
	 * Update the header, footer and body tables for resizing - i.e. column
	 * alignment.
	 *
	 * Welcome to the most horrible function DataTables. The process that this
	 * function follows is basically:
	 *   1. Re-create the table inside the scrolling div
	 *   2. Take live measurements from the DOM
	 *   3. Apply the measurements to align the columns
	 *   4. Clean up
	 *
	 *  @param {object} settings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnScrollDraw ( settings )
	{
		// Given that this is such a monster function, a lot of variables are use
		// to try and keep the minimised size as small as possible
		var
			scroll         = settings.oScroll,
			scrollX        = scroll.sX,
			scrollXInner   = scroll.sXInner,
			scrollY        = scroll.sY,
			barWidth       = scroll.iBarWidth,
			divHeader      = $(settings.nScrollHead),
			divHeaderStyle = divHeader[0].style,
			divHeaderInner = divHeader.children('div'),
			divHeaderInnerStyle = divHeaderInner[0].style,
			divHeaderTable = divHeaderInner.children('table'),
			divBodyEl      = settings.nScrollBody,
			divBody        = $(divBodyEl),
			divBodyStyle   = divBodyEl.style,
			divFooter      = $(settings.nScrollFoot),
			divFooterInner = divFooter.children('div'),
			divFooterTable = divFooterInner.children('table'),
			header         = $(settings.nTHead),
			table          = $(settings.nTable),
			tableEl        = table[0],
			tableStyle     = tableEl.style,
			footer         = settings.nTFoot ? $(settings.nTFoot) : null,
			browser        = settings.oBrowser,
			ie67           = browser.bScrollOversize,
			headerTrgEls, footerTrgEls,
			headerSrcEls, footerSrcEls,
			headerCopy, footerCopy,
			headerWidths=[], footerWidths=[],
			headerContent=[],
			idx, correction, sanityWidth,
			zeroOut = function(nSizer) {
				var style = nSizer.style;
				style.paddingTop = "0";
				style.paddingBottom = "0";
				style.borderTopWidth = "0";
				style.borderBottomWidth = "0";
				style.height = 0;
			};
	
		/*
		 * 1. Re-create the table inside the scrolling div
		 */
	
		// Remove the old minimised thead and tfoot elements in the inner table
		table.children('thead, tfoot').remove();
	
		// Clone the current header and footer elements and then place it into the inner table
		headerCopy = header.clone().prependTo( table );
		headerTrgEls = header.find('tr'); // original header is in its own table
		headerSrcEls = headerCopy.find('tr');
		headerCopy.find('th, td').removeAttr('tabindex');
	
		if ( footer ) {
			footerCopy = footer.clone().prependTo( table );
			footerTrgEls = footer.find('tr'); // the original tfoot is in its own table and must be sized
			footerSrcEls = footerCopy.find('tr');
		}
	
	
		/*
		 * 2. Take live measurements from the DOM - do not alter the DOM itself!
		 */
	
		// Remove old sizing and apply the calculated column widths
		// Get the unique column headers in the newly created (cloned) header. We want to apply the
		// calculated sizes to this header
		if ( ! scrollX )
		{
			divBodyStyle.width = '100%';
			divHeader[0].style.width = '100%';
		}
	
		$.each( _fnGetUniqueThs( settings, headerCopy ), function ( i, el ) {
			idx = _fnVisibleToColumnIndex( settings, i );
			el.style.width = settings.aoColumns[idx].sWidth;
		} );
	
		if ( footer ) {
			_fnApplyToChildren( function(n) {
				n.style.width = "";
			}, footerSrcEls );
		}
	
		// If scroll collapse is enabled, when we put the headers back into the body for sizing, we
		// will end up forcing the scrollbar to appear, making our measurements wrong for when we
		// then hide it (end of this function), so add the header height to the body scroller.
		if ( scroll.bCollapse && scrollY !== "" ) {
			divBodyStyle.height = (divBody[0].offsetHeight + header[0].offsetHeight)+"px";
		}
	
		// Size the table as a whole
		sanityWidth = table.outerWidth();
		if ( scrollX === "" ) {
			// No x scrolling
			tableStyle.width = "100%";
	
			// IE7 will make the width of the table when 100% include the scrollbar
			// - which is shouldn't. When there is a scrollbar we need to take this
			// into account.
			if ( ie67 && (table.find('tbody').height() > divBodyEl.offsetHeight ||
				divBody.css('overflow-y') == "scroll")
			) {
				tableStyle.width = _fnStringToCss( table.outerWidth() - barWidth);
			}
		}
		else
		{
			// x scrolling
			if ( scrollXInner !== "" ) {
				// x scroll inner has been given - use it
				tableStyle.width = _fnStringToCss(scrollXInner);
			}
			else if ( sanityWidth == divBody.width() && divBody.height() < table.height() ) {
				// There is y-scrolling - try to take account of the y scroll bar
				tableStyle.width = _fnStringToCss( sanityWidth-barWidth );
				if ( table.outerWidth() > sanityWidth-barWidth ) {
					// Not possible to take account of it
					tableStyle.width = _fnStringToCss( sanityWidth );
				}
			}
			else {
				// When all else fails
				tableStyle.width = _fnStringToCss( sanityWidth );
			}
		}
	
		// Recalculate the sanity width - now that we've applied the required width,
		// before it was a temporary variable. This is required because the column
		// width calculation is done before this table DOM is created.
		sanityWidth = table.outerWidth();
	
		// Hidden header should have zero height, so remove padding and borders. Then
		// set the width based on the real headers
	
		// Apply all styles in one pass
		_fnApplyToChildren( zeroOut, headerSrcEls );
	
		// Read all widths in next pass
		_fnApplyToChildren( function(nSizer) {
			headerContent.push( nSizer.innerHTML );
			headerWidths.push( _fnStringToCss( $(nSizer).css('width') ) );
		}, headerSrcEls );
	
		// Apply all widths in final pass
		_fnApplyToChildren( function(nToSize, i) {
			nToSize.style.width = headerWidths[i];
		}, headerTrgEls );
	
		$(headerSrcEls).height(0);
	
		/* Same again with the footer if we have one */
		if ( footer )
		{
			_fnApplyToChildren( zeroOut, footerSrcEls );
	
			_fnApplyToChildren( function(nSizer) {
				footerWidths.push( _fnStringToCss( $(nSizer).css('width') ) );
			}, footerSrcEls );
	
			_fnApplyToChildren( function(nToSize, i) {
				nToSize.style.width = footerWidths[i];
			}, footerTrgEls );
	
			$(footerSrcEls).height(0);
		}
	
	
		/*
		 * 3. Apply the measurements
		 */
	
		// "Hide" the header and footer that we used for the sizing. We need to keep
		// the content of the cell so that the width applied to the header and body
		// both match, but we want to hide it completely. We want to also fix their
		// width to what they currently are
		_fnApplyToChildren( function(nSizer, i) {
			nSizer.innerHTML = '<div class="dataTables_sizing" style="height:0;overflow:hidden;">'+headerContent[i]+'</div>';
			nSizer.style.width = headerWidths[i];
		}, headerSrcEls );
	
		if ( footer )
		{
			_fnApplyToChildren( function(nSizer, i) {
				nSizer.innerHTML = "";
				nSizer.style.width = footerWidths[i];
			}, footerSrcEls );
		}
	
		// Sanity check that the table is of a sensible width. If not then we are going to get
		// misalignment - try to prevent this by not allowing the table to shrink below its min width
		if ( table.outerWidth() < sanityWidth )
		{
			// The min width depends upon if we have a vertical scrollbar visible or not */
			correction = ((divBodyEl.scrollHeight > divBodyEl.offsetHeight ||
				divBody.css('overflow-y') == "scroll")) ?
					sanityWidth+barWidth :
					sanityWidth;
	
			// IE6/7 are a law unto themselves...
			if ( ie67 && (divBodyEl.scrollHeight >
				divBodyEl.offsetHeight || divBody.css('overflow-y') == "scroll")
			) {
				tableStyle.width = _fnStringToCss( correction-barWidth );
			}
	
			// And give the user a warning that we've stopped the table getting too small
			if ( scrollX === "" || scrollXInner !== "" ) {
				_fnLog( settings, 1, 'Possible column misalignment', 6 );
			}
		}
		else
		{
			correction = '100%';
		}
	
		// Apply to the container elements
		divBodyStyle.width = _fnStringToCss( correction );
		divHeaderStyle.width = _fnStringToCss( correction );
	
		if ( footer ) {
			settings.nScrollFoot.style.width = _fnStringToCss( correction );
		}
	
	
		/*
		 * 4. Clean up
		 */
		if ( ! scrollY ) {
			/* IE7< puts a vertical scrollbar in place (when it shouldn't be) due to subtracting
			 * the scrollbar height from the visible display, rather than adding it on. We need to
			 * set the height in order to sort this. Don't want to do it in any other browsers.
			 */
			if ( ie67 ) {
				divBodyStyle.height = _fnStringToCss( tableEl.offsetHeight+barWidth );
			}
		}
	
		if ( scrollY && scroll.bCollapse ) {
			divBodyStyle.height = _fnStringToCss( scrollY );
	
			var iExtra = (scrollX && tableEl.offsetWidth > divBodyEl.offsetWidth) ?
				barWidth :
				0;
	
			if ( tableEl.offsetHeight < divBodyEl.offsetHeight ) {
				divBodyStyle.height = _fnStringToCss( tableEl.offsetHeight+iExtra );
			}
		}
	
		/* Finally set the width's of the header and footer tables */
		var iOuterWidth = table.outerWidth();
		divHeaderTable[0].style.width = _fnStringToCss( iOuterWidth );
		divHeaderInnerStyle.width = _fnStringToCss( iOuterWidth );
	
		// Figure out if there are scrollbar present - if so then we need a the header and footer to
		// provide a bit more space to allow "overflow" scrolling (i.e. past the scrollbar)
		var bScrolling = table.height() > divBodyEl.clientHeight || divBody.css('overflow-y') == "scroll";
		var padding = 'padding' + (browser.bScrollbarLeft ? 'Left' : 'Right' );
		divHeaderInnerStyle[ padding ] = bScrolling ? barWidth+"px" : "0px";
	
		if ( footer ) {
			divFooterTable[0].style.width = _fnStringToCss( iOuterWidth );
			divFooterInner[0].style.width = _fnStringToCss( iOuterWidth );
			divFooterInner[0].style[padding] = bScrolling ? barWidth+"px" : "0px";
		}
	
		/* Adjust the position of the header in case we loose the y-scrollbar */
		divBody.scroll();
	
		// If sorting or filtering has occurred, jump the scrolling back to the top
		// only if we aren't holding the position
		if ( (settings.bSorted || settings.bFiltered) && ! settings._drawHold ) {
			divBodyEl.scrollTop = 0;
		}
	}
	
	
	
	/**
	 * Apply a given function to the display child nodes of an element array (typically
	 * TD children of TR rows
	 *  @param {function} fn Method to apply to the objects
	 *  @param array {nodes} an1 List of elements to look through for display children
	 *  @param array {nodes} an2 Another list (identical structure to the first) - optional
	 *  @memberof DataTable#oApi
	 */
	function _fnApplyToChildren( fn, an1, an2 )
	{
		var index=0, i=0, iLen=an1.length;
		var nNode1, nNode2;
	
		while ( i < iLen ) {
			nNode1 = an1[i].firstChild;
			nNode2 = an2 ? an2[i].firstChild : null;
	
			while ( nNode1 ) {
				if ( nNode1.nodeType === 1 ) {
					if ( an2 ) {
						fn( nNode1, nNode2, index );
					}
					else {
						fn( nNode1, index );
					}
	
					index++;
				}
	
				nNode1 = nNode1.nextSibling;
				nNode2 = an2 ? nNode2.nextSibling : null;
			}
	
			i++;
		}
	}
	
	
	
	var __re_html_remove = /<.*?>/g;
	
	
	/**
	 * Calculate the width of columns for the table
	 *  @param {object} oSettings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnCalculateColumnWidths ( oSettings )
	{
		var
			table = oSettings.nTable,
			columns = oSettings.aoColumns,
			scroll = oSettings.oScroll,
			scrollY = scroll.sY,
			scrollX = scroll.sX,
			scrollXInner = scroll.sXInner,
			columnCount = columns.length,
			visibleColumns = _fnGetColumns( oSettings, 'bVisible' ),
			headerCells = $('th', oSettings.nTHead),
			tableWidthAttr = table.getAttribute('width'), // from DOM element
			tableContainer = table.parentNode,
			userInputs = false,
			i, column, columnIdx, width, outerWidth;
	
		var styleWidth = table.style.width;
		if ( styleWidth && styleWidth.indexOf('%') !== -1 ) {
			tableWidthAttr = styleWidth;
		}
	
		/* Convert any user input sizes into pixel sizes */
		for ( i=0 ; i<visibleColumns.length ; i++ ) {
			column = columns[ visibleColumns[i] ];
	
			if ( column.sWidth !== null ) {
				column.sWidth = _fnConvertToWidth( column.sWidthOrig, tableContainer );
	
				userInputs = true;
			}
		}
	
		/* If the number of columns in the DOM equals the number that we have to
		 * process in DataTables, then we can use the offsets that are created by
		 * the web- browser. No custom sizes can be set in order for this to happen,
		 * nor scrolling used
		 */
		if ( ! userInputs && ! scrollX && ! scrollY &&
		    columnCount == _fnVisbleColumns( oSettings ) &&
			columnCount == headerCells.length
		) {
			for ( i=0 ; i<columnCount ; i++ ) {
				columns[i].sWidth = _fnStringToCss( headerCells.eq(i).width() );
			}
		}
		else
		{
			// Otherwise construct a single row, worst case, table with the widest
			// node in the data, assign any user defined widths, then insert it into
			// the DOM and allow the browser to do all the hard work of calculating
			// table widths
			var tmpTable = $(table).clone() // don't use cloneNode - IE8 will remove events on the main table
				.css( 'visibility', 'hidden' )
				.removeAttr( 'id' );
	
			// Clean up the table body
			tmpTable.find('tbody tr').remove();
			var tr = $('<tr/>').appendTo( tmpTable.find('tbody') );
	
			// Remove any assigned widths from the footer (from scrolling)
			tmpTable.find('tfoot th, tfoot td').css('width', '');
	
			// Apply custom sizing to the cloned header
			headerCells = _fnGetUniqueThs( oSettings, tmpTable.find('thead')[0] );
	
			for ( i=0 ; i<visibleColumns.length ; i++ ) {
				column = columns[ visibleColumns[i] ];
	
				headerCells[i].style.width = column.sWidthOrig !== null && column.sWidthOrig !== '' ?
					_fnStringToCss( column.sWidthOrig ) :
					'';
			}
	
			// Find the widest cell for each column and put it into the table
			if ( oSettings.aoData.length ) {
				for ( i=0 ; i<visibleColumns.length ; i++ ) {
					columnIdx = visibleColumns[i];
					column = columns[ columnIdx ];
	
					$( _fnGetWidestNode( oSettings, columnIdx ) )
						.clone( false )
						.append( column.sContentPadding )
						.appendTo( tr );
				}
			}
	
			// Table has been built, attach to the document so we can work with it
			tmpTable.appendTo( tableContainer );
	
			// When scrolling (X or Y) we want to set the width of the table as 
			// appropriate. However, when not scrolling leave the table width as it
			// is. This results in slightly different, but I think correct behaviour
			if ( scrollX && scrollXInner ) {
				tmpTable.width( scrollXInner );
			}
			else if ( scrollX ) {
				tmpTable.css( 'width', 'auto' );
	
				if ( tmpTable.width() < tableContainer.offsetWidth ) {
					tmpTable.width( tableContainer.offsetWidth );
				}
			}
			else if ( scrollY ) {
				tmpTable.width( tableContainer.offsetWidth );
			}
			else if ( tableWidthAttr ) {
				tmpTable.width( tableWidthAttr );
			}
	
			// Take into account the y scrollbar
			_fnScrollingWidthAdjust( oSettings, tmpTable[0] );
	
			// Browsers need a bit of a hand when a width is assigned to any columns
			// when x-scrolling as they tend to collapse the table to the min-width,
			// even if we sent the column widths. So we need to keep track of what
			// the table width should be by summing the user given values, and the
			// automatic values
			if ( scrollX )
			{
				var total = 0;
	
				for ( i=0 ; i<visibleColumns.length ; i++ ) {
					column = columns[ visibleColumns[i] ];
					outerWidth = $(headerCells[i]).outerWidth();
	
					total += column.sWidthOrig === null ?
						outerWidth :
						parseInt( column.sWidth, 10 ) + outerWidth - $(headerCells[i]).width();
				}
	
				tmpTable.width( _fnStringToCss( total ) );
				table.style.width = _fnStringToCss( total );
			}
	
			// Get the width of each column in the constructed table
			for ( i=0 ; i<visibleColumns.length ; i++ ) {
				column = columns[ visibleColumns[i] ];
				width = $(headerCells[i]).width();
	
				if ( width ) {
					column.sWidth = _fnStringToCss( width );
				}
			}
	
			table.style.width = _fnStringToCss( tmpTable.css('width') );
	
			// Finished with the table - ditch it
			tmpTable.remove();
		}
	
		// If there is a width attr, we want to attach an event listener which
		// allows the table sizing to automatically adjust when the window is
		// resized. Use the width attr rather than CSS, since we can't know if the
		// CSS is a relative value or absolute - DOM read is always px.
		if ( tableWidthAttr ) {
			table.style.width = _fnStringToCss( tableWidthAttr );
		}
	
		if ( (tableWidthAttr || scrollX) && ! oSettings._reszEvt ) {
			var bindResize = function () {
				$(window).bind('resize.DT-'+oSettings.sInstance, _fnThrottle( function () {
					_fnAdjustColumnSizing( oSettings );
				} ) );
			};
	
			// IE6/7 will crash if we bind a resize event handler on page load.
			// To be removed in 1.11 which drops IE6/7 support
			if ( oSettings.oBrowser.bScrollOversize ) {
				setTimeout( bindResize, 1000 );
			}
			else {
				bindResize();
			}
	
			oSettings._reszEvt = true;
		}
	}
	
	
	/**
	 * Throttle the calls to a function. Arguments and context are maintained for
	 * the throttled function
	 *  @param {function} fn Function to be called
	 *  @param {int} [freq=200] call frequency in mS
	 *  @returns {function} wrapped function
	 *  @memberof DataTable#oApi
	 */
	function _fnThrottle( fn, freq ) {
		var
			frequency = freq !== undefined ? freq : 200,
			last,
			timer;
	
		return function () {
			var
				that = this,
				now  = +new Date(),
				args = arguments;
	
			if ( last && now < last + frequency ) {
				clearTimeout( timer );
	
				timer = setTimeout( function () {
					last = undefined;
					fn.apply( that, args );
				}, frequency );
			}
			else {
				last = now;
				fn.apply( that, args );
			}
		};
	}
	
	
	/**
	 * Convert a CSS unit width to pixels (e.g. 2em)
	 *  @param {string} width width to be converted
	 *  @param {node} parent parent to get the with for (required for relative widths) - optional
	 *  @returns {int} width in pixels
	 *  @memberof DataTable#oApi
	 */
	function _fnConvertToWidth ( width, parent )
	{
		if ( ! width ) {
			return 0;
		}
	
		var n = $('<div/>')
			.css( 'width', _fnStringToCss( width ) )
			.appendTo( parent || document.body );
	
		var val = n[0].offsetWidth;
		n.remove();
	
		return val;
	}
	
	
	/**
	 * Adjust a table's width to take account of vertical scroll bar
	 *  @param {object} oSettings dataTables settings object
	 *  @param {node} n table node
	 *  @memberof DataTable#oApi
	 */
	
	function _fnScrollingWidthAdjust ( settings, n )
	{
		var scroll = settings.oScroll;
	
		if ( scroll.sX || scroll.sY ) {
			// When y-scrolling only, we want to remove the width of the scroll bar
			// so the table + scroll bar will fit into the area available, otherwise
			// we fix the table at its current size with no adjustment
			var correction = ! scroll.sX ? scroll.iBarWidth : 0;
			n.style.width = _fnStringToCss( $(n).outerWidth() - correction );
		}
	}
	
	
	/**
	 * Get the widest node
	 *  @param {object} settings dataTables settings object
	 *  @param {int} colIdx column of interest
	 *  @returns {node} widest table node
	 *  @memberof DataTable#oApi
	 */
	function _fnGetWidestNode( settings, colIdx )
	{
		var idx = _fnGetMaxLenString( settings, colIdx );
		if ( idx < 0 ) {
			return null;
		}
	
		var data = settings.aoData[ idx ];
		return ! data.nTr ? // Might not have been created when deferred rendering
			$('<td/>').html( _fnGetCellData( settings, idx, colIdx, 'display' ) )[0] :
			data.anCells[ colIdx ];
	}
	
	
	/**
	 * Get the maximum strlen for each data column
	 *  @param {object} settings dataTables settings object
	 *  @param {int} colIdx column of interest
	 *  @returns {string} max string length for each column
	 *  @memberof DataTable#oApi
	 */
	function _fnGetMaxLenString( settings, colIdx )
	{
		var s, max=-1, maxIdx = -1;
	
		for ( var i=0, ien=settings.aoData.length ; i<ien ; i++ ) {
			s = _fnGetCellData( settings, i, colIdx, 'display' )+'';
			s = s.replace( __re_html_remove, '' );
	
			if ( s.length > max ) {
				max = s.length;
				maxIdx = i;
			}
		}
	
		return maxIdx;
	}
	
	
	/**
	 * Append a CSS unit (only if required) to a string
	 *  @param {string} value to css-ify
	 *  @returns {string} value with css unit
	 *  @memberof DataTable#oApi
	 */
	function _fnStringToCss( s )
	{
		if ( s === null ) {
			return '0px';
		}
	
		if ( typeof s == 'number' ) {
			return s < 0 ?
				'0px' :
				s+'px';
		}
	
		// Check it has a unit character already
		return s.match(/\d$/) ?
			s+'px' :
			s;
	}
	
	
	/**
	 * Get the width of a scroll bar in this browser being used
	 *  @returns {int} width in pixels
	 *  @memberof DataTable#oApi
	 */
	function _fnScrollBarWidth ()
	{
		// On first run a static variable is set, since this is only needed once.
		// Subsequent runs will just use the previously calculated value
		var width = DataTable.__scrollbarWidth;
	
		if ( width === undefined ) {
			var sizer = $('<p/>').css( {
					position: 'absolute',
					top: 0,
					left: 0,
					width: '100%',
					height: 150,
					padding: 0,
					overflow: 'scroll',
					visibility: 'hidden'
				} )
				.appendTo('body');
	
			width = sizer[0].offsetWidth - sizer[0].clientWidth;
			DataTable.__scrollbarWidth = width;
	
			sizer.remove();
		}
	
		return width;
	}
	
	
	
	function _fnSortFlatten ( settings )
	{
		var
			i, iLen, k, kLen,
			aSort = [],
			aiOrig = [],
			aoColumns = settings.aoColumns,
			aDataSort, iCol, sType, srcCol,
			fixed = settings.aaSortingFixed,
			fixedObj = $.isPlainObject( fixed ),
			nestedSort = [],
			add = function ( a ) {
				if ( a.length && ! $.isArray( a[0] ) ) {
					// 1D array
					nestedSort.push( a );
				}
				else {
					// 2D array
					nestedSort.push.apply( nestedSort, a );
				}
			};
	
		// Build the sort array, with pre-fix and post-fix options if they have been
		// specified
		if ( $.isArray( fixed ) ) {
			add( fixed );
		}
	
		if ( fixedObj && fixed.pre ) {
			add( fixed.pre );
		}
	
		add( settings.aaSorting );
	
		if (fixedObj && fixed.post ) {
			add( fixed.post );
		}
	
		for ( i=0 ; i<nestedSort.length ; i++ )
		{
			srcCol = nestedSort[i][0];
			aDataSort = aoColumns[ srcCol ].aDataSort;
	
			for ( k=0, kLen=aDataSort.length ; k<kLen ; k++ )
			{
				iCol = aDataSort[k];
				sType = aoColumns[ iCol ].sType || 'string';
	
				if ( nestedSort[i]._idx === undefined ) {
					nestedSort[i]._idx = $.inArray( nestedSort[i][1], aoColumns[iCol].asSorting );
				}
	
				aSort.push( {
					src:       srcCol,
					col:       iCol,
					dir:       nestedSort[i][1],
					index:     nestedSort[i]._idx,
					type:      sType,
					formatter: DataTable.ext.type.order[ sType+"-pre" ]
				} );
			}
		}
	
		return aSort;
	}
	
	/**
	 * Change the order of the table
	 *  @param {object} oSettings dataTables settings object
	 *  @memberof DataTable#oApi
	 *  @todo This really needs split up!
	 */
	function _fnSort ( oSettings )
	{
		var
			i, ien, iLen, j, jLen, k, kLen,
			sDataType, nTh,
			aiOrig = [],
			oExtSort = DataTable.ext.type.order,
			aoData = oSettings.aoData,
			aoColumns = oSettings.aoColumns,
			aDataSort, data, iCol, sType, oSort,
			formatters = 0,
			sortCol,
			displayMaster = oSettings.aiDisplayMaster,
			aSort;
	
		// Resolve any column types that are unknown due to addition or invalidation
		// @todo Can this be moved into a 'data-ready' handler which is called when
		//   data is going to be used in the table?
		_fnColumnTypes( oSettings );
	
		aSort = _fnSortFlatten( oSettings );
	
		for ( i=0, ien=aSort.length ; i<ien ; i++ ) {
			sortCol = aSort[i];
	
			// Track if we can use the fast sort algorithm
			if ( sortCol.formatter ) {
				formatters++;
			}
	
			// Load the data needed for the sort, for each cell
			_fnSortData( oSettings, sortCol.col );
		}
	
		/* No sorting required if server-side or no sorting array */
		if ( _fnDataSource( oSettings ) != 'ssp' && aSort.length !== 0 )
		{
			// Create a value - key array of the current row positions such that we can use their
			// current position during the sort, if values match, in order to perform stable sorting
			for ( i=0, iLen=displayMaster.length ; i<iLen ; i++ ) {
				aiOrig[ displayMaster[i] ] = i;
			}
	
			/* Do the sort - here we want multi-column sorting based on a given data source (column)
			 * and sorting function (from oSort) in a certain direction. It's reasonably complex to
			 * follow on it's own, but this is what we want (example two column sorting):
			 *  fnLocalSorting = function(a,b){
			 *    var iTest;
			 *    iTest = oSort['string-asc']('data11', 'data12');
			 *      if (iTest !== 0)
			 *        return iTest;
			 *    iTest = oSort['numeric-desc']('data21', 'data22');
			 *    if (iTest !== 0)
			 *      return iTest;
			 *    return oSort['numeric-asc']( aiOrig[a], aiOrig[b] );
			 *  }
			 * Basically we have a test for each sorting column, if the data in that column is equal,
			 * test the next column. If all columns match, then we use a numeric sort on the row
			 * positions in the original data array to provide a stable sort.
			 *
			 * Note - I know it seems excessive to have two sorting methods, but the first is around
			 * 15% faster, so the second is only maintained for backwards compatibility with sorting
			 * methods which do not have a pre-sort formatting function.
			 */
			if ( formatters === aSort.length ) {
				// All sort types have formatting functions
				displayMaster.sort( function ( a, b ) {
					var
						x, y, k, test, sort,
						len=aSort.length,
						dataA = aoData[a]._aSortData,
						dataB = aoData[b]._aSortData;
	
					for ( k=0 ; k<len ; k++ ) {
						sort = aSort[k];
	
						x = dataA[ sort.col ];
						y = dataB[ sort.col ];
	
						test = x<y ? -1 : x>y ? 1 : 0;
						if ( test !== 0 ) {
							return sort.dir === 'asc' ? test : -test;
						}
					}
	
					x = aiOrig[a];
					y = aiOrig[b];
					return x<y ? -1 : x>y ? 1 : 0;
				} );
			}
			else {
				// Depreciated - remove in 1.11 (providing a plug-in option)
				// Not all sort types have formatting methods, so we have to call their sorting
				// methods.
				displayMaster.sort( function ( a, b ) {
					var
						x, y, k, l, test, sort, fn,
						len=aSort.length,
						dataA = aoData[a]._aSortData,
						dataB = aoData[b]._aSortData;
	
					for ( k=0 ; k<len ; k++ ) {
						sort = aSort[k];
	
						x = dataA[ sort.col ];
						y = dataB[ sort.col ];
	
						fn = oExtSort[ sort.type+"-"+sort.dir ] || oExtSort[ "string-"+sort.dir ];
						test = fn( x, y );
						if ( test !== 0 ) {
							return test;
						}
					}
	
					x = aiOrig[a];
					y = aiOrig[b];
					return x<y ? -1 : x>y ? 1 : 0;
				} );
			}
		}
	
		/* Tell the draw function that we have sorted the data */
		oSettings.bSorted = true;
	}
	
	
	function _fnSortAria ( settings )
	{
		var label;
		var nextSort;
		var columns = settings.aoColumns;
		var aSort = _fnSortFlatten( settings );
		var oAria = settings.oLanguage.oAria;
	
		// ARIA attributes - need to loop all columns, to update all (removing old
		// attributes as needed)
		for ( var i=0, iLen=columns.length ; i<iLen ; i++ )
		{
			var col = columns[i];
			var asSorting = col.asSorting;
			var sTitle = col.sTitle.replace( /<.*?>/g, "" );
			var th = col.nTh;
	
			// IE7 is throwing an error when setting these properties with jQuery's
			// attr() and removeAttr() methods...
			th.removeAttribute('aria-sort');
	
			/* In ARIA only the first sorting column can be marked as sorting - no multi-sort option */
			if ( col.bSortable ) {
				if ( aSort.length > 0 && aSort[0].col == i ) {
					th.setAttribute('aria-sort', aSort[0].dir=="asc" ? "ascending" : "descending" );
					nextSort = asSorting[ aSort[0].index+1 ] || asSorting[0];
				}
				else {
					nextSort = asSorting[0];
				}
	
				label = sTitle + ( nextSort === "asc" ?
					oAria.sSortAscending :
					oAria.sSortDescending
				);
			}
			else {
				label = sTitle;
			}
	
			th.setAttribute('aria-label', label);
		}
	}
	
	
	/**
	 * Function to run on user sort request
	 *  @param {object} settings dataTables settings object
	 *  @param {node} attachTo node to attach the handler to
	 *  @param {int} colIdx column sorting index
	 *  @param {boolean} [append=false] Append the requested sort to the existing
	 *    sort if true (i.e. multi-column sort)
	 *  @param {function} [callback] callback function
	 *  @memberof DataTable#oApi
	 */
	function _fnSortListener ( settings, colIdx, append, callback )
	{
		var col = settings.aoColumns[ colIdx ];
		var sorting = settings.aaSorting;
		var asSorting = col.asSorting;
		var nextSortIdx;
		var next = function ( a, overflow ) {
			var idx = a._idx;
			if ( idx === undefined ) {
				idx = $.inArray( a[1], asSorting );
			}
	
			return idx+1 < asSorting.length ?
				idx+1 :
				overflow ?
					null :
					0;
		};
	
		// Convert to 2D array if needed
		if ( typeof sorting[0] === 'number' ) {
			sorting = settings.aaSorting = [ sorting ];
		}
	
		// If appending the sort then we are multi-column sorting
		if ( append && settings.oFeatures.bSortMulti ) {
			// Are we already doing some kind of sort on this column?
			var sortIdx = $.inArray( colIdx, _pluck(sorting, '0') );
	
			if ( sortIdx !== -1 ) {
				// Yes, modify the sort
				nextSortIdx = next( sorting[sortIdx], true );
	
				if ( nextSortIdx === null && sorting.length === 1 ) {
					nextSortIdx = 0; // can't remove sorting completely
				}
	
				if ( nextSortIdx === null ) {
					sorting.splice( sortIdx, 1 );
				}
				else {
					sorting[sortIdx][1] = asSorting[ nextSortIdx ];
					sorting[sortIdx]._idx = nextSortIdx;
				}
			}
			else {
				// No sort on this column yet
				sorting.push( [ colIdx, asSorting[0], 0 ] );
				sorting[sorting.length-1]._idx = 0;
			}
		}
		else if ( sorting.length && sorting[0][0] == colIdx ) {
			// Single column - already sorting on this column, modify the sort
			nextSortIdx = next( sorting[0] );
	
			sorting.length = 1;
			sorting[0][1] = asSorting[ nextSortIdx ];
			sorting[0]._idx = nextSortIdx;
		}
		else {
			// Single column - sort only on this column
			sorting.length = 0;
			sorting.push( [ colIdx, asSorting[0] ] );
			sorting[0]._idx = 0;
		}
	
		// Run the sort by calling a full redraw
		_fnReDraw( settings );
	
		// callback used for async user interaction
		if ( typeof callback == 'function' ) {
			callback( settings );
		}
	}
	
	
	/**
	 * Attach a sort handler (click) to a node
	 *  @param {object} settings dataTables settings object
	 *  @param {node} attachTo node to attach the handler to
	 *  @param {int} colIdx column sorting index
	 *  @param {function} [callback] callback function
	 *  @memberof DataTable#oApi
	 */
	function _fnSortAttachListener ( settings, attachTo, colIdx, callback )
	{
		var col = settings.aoColumns[ colIdx ];
	
		_fnBindAction( attachTo, {}, function (e) {
			/* If the column is not sortable - don't to anything */
			if ( col.bSortable === false ) {
				return;
			}
	
			// If processing is enabled use a timeout to allow the processing
			// display to be shown - otherwise to it synchronously
			if ( settings.oFeatures.bProcessing ) {
				_fnProcessingDisplay( settings, true );
	
				setTimeout( function() {
					_fnSortListener( settings, colIdx, e.shiftKey, callback );
	
					// In server-side processing, the draw callback will remove the
					// processing display
					if ( _fnDataSource( settings ) !== 'ssp' ) {
						_fnProcessingDisplay( settings, false );
					}
				}, 0 );
			}
			else {
				_fnSortListener( settings, colIdx, e.shiftKey, callback );
			}
		} );
	}
	
	
	/**
	 * Set the sorting classes on table's body, Note: it is safe to call this function
	 * when bSort and bSortClasses are false
	 *  @param {object} oSettings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnSortingClasses( settings )
	{
		var oldSort = settings.aLastSort;
		var sortClass = settings.oClasses.sSortColumn;
		var sort = _fnSortFlatten( settings );
		var features = settings.oFeatures;
		var i, ien, colIdx;
	
		if ( features.bSort && features.bSortClasses ) {
			// Remove old sorting classes
			for ( i=0, ien=oldSort.length ; i<ien ; i++ ) {
				colIdx = oldSort[i].src;
	
				// Remove column sorting
				$( _pluck( settings.aoData, 'anCells', colIdx ) )
					.removeClass( sortClass + (i<2 ? i+1 : 3) );
			}
	
			// Add new column sorting
			for ( i=0, ien=sort.length ; i<ien ; i++ ) {
				colIdx = sort[i].src;
	
				$( _pluck( settings.aoData, 'anCells', colIdx ) )
					.addClass( sortClass + (i<2 ? i+1 : 3) );
			}
		}
	
		settings.aLastSort = sort;
	}
	
	
	// Get the data to sort a column, be it from cache, fresh (populating the
	// cache), or from a sort formatter
	function _fnSortData( settings, idx )
	{
		// Custom sorting function - provided by the sort data type
		var column = settings.aoColumns[ idx ];
		var customSort = DataTable.ext.order[ column.sSortDataType ];
		var customData;
	
		if ( customSort ) {
			customData = customSort.call( settings.oInstance, settings, idx,
				_fnColumnIndexToVisible( settings, idx )
			);
		}
	
		// Use / populate cache
		var row, cellData;
		var formatter = DataTable.ext.type.order[ column.sType+"-pre" ];
	
		for ( var i=0, ien=settings.aoData.length ; i<ien ; i++ ) {
			row = settings.aoData[i];
	
			if ( ! row._aSortData ) {
				row._aSortData = [];
			}
	
			if ( ! row._aSortData[idx] || customSort ) {
				cellData = customSort ?
					customData[i] : // If there was a custom sort function, use data from there
					_fnGetCellData( settings, i, idx, 'sort' );
	
				row._aSortData[ idx ] = formatter ?
					formatter( cellData ) :
					cellData;
			}
		}
	}
	
	
	
	/**
	 * Save the state of a table
	 *  @param {object} oSettings dataTables settings object
	 *  @memberof DataTable#oApi
	 */
	function _fnSaveState ( settings )
	{
		if ( !settings.oFeatures.bStateSave || settings.bDestroying )
		{
			return;
		}
	
		/* Store the interesting variables */
		var state = {
			time:    +new Date(),
			start:   settings._iDisplayStart,
			length:  settings._iDisplayLength,
			order:   $.extend( true, [], settings.aaSorting ),
			search:  _fnSearchToCamel( settings.oPreviousSearch ),
			columns: $.map( settings.aoColumns, function ( col, i ) {
				return {
					visible: col.bVisible,
					search: _fnSearchToCamel( settings.aoPreSearchCols[i] )
				};
			} )
		};
	
		_fnCallbackFire( settings, "aoStateSaveParams", 'stateSaveParams', [settings, state] );
	
		settings.oSavedState = state;
		settings.fnStateSaveCallback.call( settings.oInstance, settings, state );
	}
	
	
	/**
	 * Attempt to load a saved table state
	 *  @param {object} oSettings dataTables settings object
	 *  @param {object} oInit DataTables init object so we can override settings
	 *  @memberof DataTable#oApi
	 */
	function _fnLoadState ( settings, oInit )
	{
		var i, ien;
		var columns = settings.aoColumns;
	
		if ( ! settings.oFeatures.bStateSave ) {
			return;
		}
	
		var state = settings.fnStateLoadCallback.call( settings.oInstance, settings );
		if ( ! state || ! state.time ) {
			return;
		}
	
		/* Allow custom and plug-in manipulation functions to alter the saved data set and
		 * cancelling of loading by returning false
		 */
		var abStateLoad = _fnCallbackFire( settings, 'aoStateLoadParams', 'stateLoadParams', [settings, state] );
		if ( $.inArray( false, abStateLoad ) !== -1 ) {
			return;
		}
	
		/* Reject old data */
		var duration = settings.iStateDuration;
		if ( duration > 0 && state.time < +new Date() - (duration*1000) ) {
			return;
		}
	
		// Number of columns have changed - all bets are off, no restore of settings
		if ( columns.length !== state.columns.length ) {
			return;
		}
	
		// Store the saved state so it might be accessed at any time
		settings.oLoadedState = $.extend( true, {}, state );
	
		// Restore key features - todo - for 1.11 this needs to be done by
		// subscribed events
		if ( state.start !== undefined ) {
			settings._iDisplayStart    = state.start;
			settings.iInitDisplayStart = state.start;
		}
		if ( state.length !== undefined ) {
			settings._iDisplayLength   = state.length;
		}
	
		// Order
		if ( state.order !== undefined ) {
			settings.aaSorting = [];
			$.each( state.order, function ( i, col ) {
				settings.aaSorting.push( col[0] >= columns.length ?
					[ 0, col[1] ] :
					col
				);
			} );
		}
	
		// Search
		if ( state.search !== undefined ) {
			$.extend( settings.oPreviousSearch, _fnSearchToHung( state.search ) );
		}
	
		// Columns
		for ( i=0, ien=state.columns.length ; i<ien ; i++ ) {
			var col = state.columns[i];
	
			// Visibility
			if ( col.visible !== undefined ) {
				columns[i].bVisible = col.visible;
			}
	
			// Search
			if ( col.search !== undefined ) {
				$.extend( settings.aoPreSearchCols[i], _fnSearchToHung( col.search ) );
			}
		}
	
		_fnCallbackFire( settings, 'aoStateLoaded', 'stateLoaded', [settings, state] );
	}
	
	
	/**
	 * Return the settings object for a particular table
	 *  @param {node} table table we are using as a dataTable
	 *  @returns {object} Settings object - or null if not found
	 *  @memberof DataTable#oApi
	 */
	function _fnSettingsFromNode ( table )
	{
		var settings = DataTable.settings;
		var idx = $.inArray( table, _pluck( settings, 'nTable' ) );
	
		return idx !== -1 ?
			settings[ idx ] :
			null;
	}
	
	
	/**
	 * Log an error message
	 *  @param {object} settings dataTables settings object
	 *  @param {int} level log error messages, or display them to the user
	 *  @param {string} msg error message
	 *  @param {int} tn Technical note id to get more information about the error.
	 *  @memberof DataTable#oApi
	 */
	function _fnLog( settings, level, msg, tn )
	{
		msg = 'DataTables warning: '+
			(settings!==null ? 'table id='+settings.sTableId+' - ' : '')+msg;
	
		if ( tn ) {
			msg += '. For more information about this error, please see '+
			'http://datatables.net/tn/'+tn;
		}
	
		if ( ! level  ) {
			// Backwards compatibility pre 1.10
			var ext = DataTable.ext;
			var type = ext.sErrMode || ext.errMode;
	
			_fnCallbackFire( settings, null, 'error', [ settings, tn, msg ] );
	
			if ( type == 'alert' ) {
				alert( msg );
			}
			else if ( type == 'throw' ) {
				throw new Error(msg);
			}
			else if ( typeof type == 'function' ) {
				type( settings, tn, msg );
			}
		}
		else if ( window.console && console.log ) {
			console.log( msg );
		}
	}
	
	
	/**
	 * See if a property is defined on one object, if so assign it to the other object
	 *  @param {object} ret target object
	 *  @param {object} src source object
	 *  @param {string} name property
	 *  @param {string} [mappedName] name to map too - optional, name used if not given
	 *  @memberof DataTable#oApi
	 */
	function _fnMap( ret, src, name, mappedName )
	{
		if ( $.isArray( name ) ) {
			$.each( name, function (i, val) {
				if ( $.isArray( val ) ) {
					_fnMap( ret, src, val[0], val[1] );
				}
				else {
					_fnMap( ret, src, val );
				}
			} );
	
			return;
		}
	
		if ( mappedName === undefined ) {
			mappedName = name;
		}
	
		if ( src[name] !== undefined ) {
			ret[mappedName] = src[name];
		}
	}
	
	
	/**
	 * Extend objects - very similar to jQuery.extend, but deep copy objects, and
	 * shallow copy arrays. The reason we need to do this, is that we don't want to
	 * deep copy array init values (such as aaSorting) since the dev wouldn't be
	 * able to override them, but we do want to deep copy arrays.
	 *  @param {object} out Object to extend
	 *  @param {object} extender Object from which the properties will be applied to
	 *      out
	 *  @param {boolean} breakRefs If true, then arrays will be sliced to take an
	 *      independent copy with the exception of the `data` or `aaData` parameters
	 *      if they are present. This is so you can pass in a collection to
	 *      DataTables and have that used as your data source without breaking the
	 *      references
	 *  @returns {object} out Reference, just for convenience - out === the return.
	 *  @memberof DataTable#oApi
	 *  @todo This doesn't take account of arrays inside the deep copied objects.
	 */
	function _fnExtend( out, extender, breakRefs )
	{
		var val;
	
		for ( var prop in extender ) {
			if ( extender.hasOwnProperty(prop) ) {
				val = extender[prop];
	
				if ( $.isPlainObject( val ) ) {
					if ( ! $.isPlainObject( out[prop] ) ) {
						out[prop] = {};
					}
					$.extend( true, out[prop], val );
				}
				else if ( breakRefs && prop !== 'data' && prop !== 'aaData' && $.isArray(val) ) {
					out[prop] = val.slice();
				}
				else {
					out[prop] = val;
				}
			}
		}
	
		return out;
	}
	
	
	/**
	 * Bind an event handers to allow a click or return key to activate the callback.
	 * This is good for accessibility since a return on the keyboard will have the
	 * same effect as a click, if the element has focus.
	 *  @param {element} n Element to bind the action to
	 *  @param {object} oData Data object to pass to the triggered function
	 *  @param {function} fn Callback function for when the event is triggered
	 *  @memberof DataTable#oApi
	 */
	function _fnBindAction( n, oData, fn )
	{
		$(n)
			.bind( 'click.DT', oData, function (e) {
					n.blur(); // Remove focus outline for mouse users
					fn(e);
				} )
			.bind( 'keypress.DT', oData, function (e){
					if ( e.which === 13 ) {
						e.preventDefault();
						fn(e);
					}
				} )
			.bind( 'selectstart.DT', function () {
					/* Take the brutal approach to cancelling text selection */
					return false;
				} );
	}
	
	
	/**
	 * Register a callback function. Easily allows a callback function to be added to
	 * an array store of callback functions that can then all be called together.
	 *  @param {object} oSettings dataTables settings object
	 *  @param {string} sStore Name of the array storage for the callbacks in oSettings
	 *  @param {function} fn Function to be called back
	 *  @param {string} sName Identifying name for the callback (i.e. a label)
	 *  @memberof DataTable#oApi
	 */
	function _fnCallbackReg( oSettings, sStore, fn, sName )
	{
		if ( fn )
		{
			oSettings[sStore].push( {
				"fn": fn,
				"sName": sName
			} );
		}
	}
	
	
	/**
	 * Fire callback functions and trigger events. Note that the loop over the
	 * callback array store is done backwards! Further note that you do not want to
	 * fire off triggers in time sensitive applications (for example cell creation)
	 * as its slow.
	 *  @param {object} settings dataTables settings object
	 *  @param {string} callbackArr Name of the array storage for the callbacks in
	 *      oSettings
	 *  @param {string} eventName Name of the jQuery custom event to trigger. If
	 *      null no trigger is fired
	 *  @param {array} args Array of arguments to pass to the callback function /
	 *      trigger
	 *  @memberof DataTable#oApi
	 */
	function _fnCallbackFire( settings, callbackArr, eventName, args )
	{
		var ret = [];
	
		if ( callbackArr ) {
			ret = $.map( settings[callbackArr].slice().reverse(), function (val, i) {
				return val.fn.apply( settings.oInstance, args );
			} );
		}
	
		if ( eventName !== null ) {
			var e = $.Event( eventName+'.dt' );
	
			$(settings.nTable).trigger( e, args );
	
			ret.push( e.result );
		}
	
		return ret;
	}
	
	
	function _fnLengthOverflow ( settings )
	{
		var
			start = settings._iDisplayStart,
			end = settings.fnDisplayEnd(),
			len = settings._iDisplayLength;
	
		/* If we have space to show extra rows (backing up from the end point - then do so */
		if ( start >= end )
		{
			start = end - len;
		}
	
		// Keep the start record on the current page
		start -= (start % len);
	
		if ( len === -1 || start < 0 )
		{
			start = 0;
		}
	
		settings._iDisplayStart = start;
	}
	
	
	function _fnRenderer( settings, type )
	{
		var renderer = settings.renderer;
		var host = DataTable.ext.renderer[type];
	
		if ( $.isPlainObject( renderer ) && renderer[type] ) {
			// Specific renderer for this type. If available use it, otherwise use
			// the default.
			return host[renderer[type]] || host._;
		}
		else if ( typeof renderer === 'string' ) {
			// Common renderer - if there is one available for this type use it,
			// otherwise use the default
			return host[renderer] || host._;
		}
	
		// Use the default
		return host._;
	}
	
	
	/**
	 * Detect the data source being used for the table. Used to simplify the code
	 * a little (ajax) and to make it compress a little smaller.
	 *
	 *  @param {object} settings dataTables settings object
	 *  @returns {string} Data source
	 *  @memberof DataTable#oApi
	 */
	function _fnDataSource ( settings )
	{
		if ( settings.oFeatures.bServerSide ) {
			return 'ssp';
		}
		else if ( settings.ajax || settings.sAjaxSource ) {
			return 'ajax';
		}
		return 'dom';
	}
	

	DataTable = function( options )
	{
		/**
		 * Perform a jQuery selector action on the table's TR elements (from the tbody) and
		 * return the resulting jQuery object.
		 *  @param {string|node|jQuery} sSelector jQuery selector or node collection to act on
		 *  @param {object} [oOpts] Optional parameters for modifying the rows to be included
		 *  @param {string} [oOpts.filter=none] Select TR elements that meet the current filter
		 *    criterion ("applied") or all TR elements (i.e. no filter).
		 *  @param {string} [oOpts.order=current] Order of the TR elements in the processed array.
		 *    Can be either 'current', whereby the current sorting of the table is used, or
		 *    'original' whereby the original order the data was read into the table is used.
		 *  @param {string} [oOpts.page=all] Limit the selection to the currently displayed page
		 *    ("current") or not ("all"). If 'current' is given, then order is assumed to be
		 *    'current' and filter is 'applied', regardless of what they might be given as.
		 *  @returns {object} jQuery object, filtered by the given selector.
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *
		 *      // Highlight every second row
		 *      oTable.$('tr:odd').css('backgroundColor', 'blue');
		 *    } );
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *
		 *      // Filter to rows with 'Webkit' in them, add a background colour and then
		 *      // remove the filter, thus highlighting the 'Webkit' rows only.
		 *      oTable.fnFilter('Webkit');
		 *      oTable.$('tr', {"search": "applied"}).css('backgroundColor', 'blue');
		 *      oTable.fnFilter('');
		 *    } );
		 */
		this.$ = function ( sSelector, oOpts )
		{
			return this.api(true).$( sSelector, oOpts );
		};
		
		
		/**
		 * Almost identical to $ in operation, but in this case returns the data for the matched
		 * rows - as such, the jQuery selector used should match TR row nodes or TD/TH cell nodes
		 * rather than any descendants, so the data can be obtained for the row/cell. If matching
		 * rows are found, the data returned is the original data array/object that was used to
		 * create the row (or a generated array if from a DOM source).
		 *
		 * This method is often useful in-combination with $ where both functions are given the
		 * same parameters and the array indexes will match identically.
		 *  @param {string|node|jQuery} sSelector jQuery selector or node collection to act on
		 *  @param {object} [oOpts] Optional parameters for modifying the rows to be included
		 *  @param {string} [oOpts.filter=none] Select elements that meet the current filter
		 *    criterion ("applied") or all elements (i.e. no filter).
		 *  @param {string} [oOpts.order=current] Order of the data in the processed array.
		 *    Can be either 'current', whereby the current sorting of the table is used, or
		 *    'original' whereby the original order the data was read into the table is used.
		 *  @param {string} [oOpts.page=all] Limit the selection to the currently displayed page
		 *    ("current") or not ("all"). If 'current' is given, then order is assumed to be
		 *    'current' and filter is 'applied', regardless of what they might be given as.
		 *  @returns {array} Data for the matched elements. If any elements, as a result of the
		 *    selector, were not TR, TD or TH elements in the DataTable, they will have a null
		 *    entry in the array.
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *
		 *      // Get the data from the first row in the table
		 *      var data = oTable._('tr:first');
		 *
		 *      // Do something useful with the data
		 *      alert( "First cell is: "+data[0] );
		 *    } );
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *
		 *      // Filter to 'Webkit' and get all data for
		 *      oTable.fnFilter('Webkit');
		 *      var data = oTable._('tr', {"search": "applied"});
		 *
		 *      // Do something with the data
		 *      alert( data.length+" rows matched the search" );
		 *    } );
		 */
		this._ = function ( sSelector, oOpts )
		{
			return this.api(true).rows( sSelector, oOpts ).data();
		};
		
		
		/**
		 * Create a DataTables Api instance, with the currently selected tables for
		 * the Api's context.
		 * @param {boolean} [traditional=false] Set the API instance's context to be
		 *   only the table referred to by the `DataTable.ext.iApiIndex` option, as was
		 *   used in the API presented by DataTables 1.9- (i.e. the traditional mode),
		 *   or if all tables captured in the jQuery object should be used.
		 * @return {DataTables.Api}
		 */
		this.api = function ( traditional )
		{
			return traditional ?
				new _Api(
					_fnSettingsFromNode( this[ _ext.iApiIndex ] )
				) :
				new _Api( this );
		};
		
		
		/**
		 * Add a single new row or multiple rows of data to the table. Please note
		 * that this is suitable for client-side processing only - if you are using
		 * server-side processing (i.e. "bServerSide": true), then to add data, you
		 * must add it to the data source, i.e. the server-side, through an Ajax call.
		 *  @param {array|object} data The data to be added to the table. This can be:
		 *    <ul>
		 *      <li>1D array of data - add a single row with the data provided</li>
		 *      <li>2D array of arrays - add multiple rows in a single call</li>
		 *      <li>object - data object when using <i>mData</i></li>
		 *      <li>array of objects - multiple data objects when using <i>mData</i></li>
		 *    </ul>
		 *  @param {bool} [redraw=true] redraw the table or not
		 *  @returns {array} An array of integers, representing the list of indexes in
		 *    <i>aoData</i> ({@link DataTable.models.oSettings}) that have been added to
		 *    the table.
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    // Global var for counter
		 *    var giCount = 2;
		 *
		 *    $(document).ready(function() {
		 *      $('#example').dataTable();
		 *    } );
		 *
		 *    function fnClickAddRow() {
		 *      $('#example').dataTable().fnAddData( [
		 *        giCount+".1",
		 *        giCount+".2",
		 *        giCount+".3",
		 *        giCount+".4" ]
		 *      );
		 *
		 *      giCount++;
		 *    }
		 */
		this.fnAddData = function( data, redraw )
		{
			var api = this.api( true );
		
			/* Check if we want to add multiple rows or not */
			var rows = $.isArray(data) && ( $.isArray(data[0]) || $.isPlainObject(data[0]) ) ?
				api.rows.add( data ) :
				api.row.add( data );
		
			if ( redraw === undefined || redraw ) {
				api.draw();
			}
		
			return rows.flatten().toArray();
		};
		
		
		/**
		 * This function will make DataTables recalculate the column sizes, based on the data
		 * contained in the table and the sizes applied to the columns (in the DOM, CSS or
		 * through the sWidth parameter). This can be useful when the width of the table's
		 * parent element changes (for example a window resize).
		 *  @param {boolean} [bRedraw=true] Redraw the table or not, you will typically want to
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable( {
		 *        "sScrollY": "200px",
		 *        "bPaginate": false
		 *      } );
		 *
		 *      $(window).bind('resize', function () {
		 *        oTable.fnAdjustColumnSizing();
		 *      } );
		 *    } );
		 */
		this.fnAdjustColumnSizing = function ( bRedraw )
		{
			var api = this.api( true ).columns.adjust();
			var settings = api.settings()[0];
			var scroll = settings.oScroll;
		
			if ( bRedraw === undefined || bRedraw ) {
				api.draw( false );
			}
			else if ( scroll.sX !== "" || scroll.sY !== "" ) {
				/* If not redrawing, but scrolling, we want to apply the new column sizes anyway */
				_fnScrollDraw( settings );
			}
		};
		
		
		/**
		 * Quickly and simply clear a table
		 *  @param {bool} [bRedraw=true] redraw the table or not
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *
		 *      // Immediately 'nuke' the current rows (perhaps waiting for an Ajax callback...)
		 *      oTable.fnClearTable();
		 *    } );
		 */
		this.fnClearTable = function( bRedraw )
		{
			var api = this.api( true ).clear();
		
			if ( bRedraw === undefined || bRedraw ) {
				api.draw();
			}
		};
		
		
		/**
		 * The exact opposite of 'opening' a row, this function will close any rows which
		 * are currently 'open'.
		 *  @param {node} nTr the table row to 'close'
		 *  @returns {int} 0 on success, or 1 if failed (can't find the row)
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable;
		 *
		 *      // 'open' an information row when a row is clicked on
		 *      $('#example tbody tr').click( function () {
		 *        if ( oTable.fnIsOpen(this) ) {
		 *          oTable.fnClose( this );
		 *        } else {
		 *          oTable.fnOpen( this, "Temporary row opened", "info_row" );
		 *        }
		 *      } );
		 *
		 *      oTable = $('#example').dataTable();
		 *    } );
		 */
		this.fnClose = function( nTr )
		{
			this.api( true ).row( nTr ).child.hide();
		};
		
		
		/**
		 * Remove a row for the table
		 *  @param {mixed} target The index of the row from aoData to be deleted, or
		 *    the TR element you want to delete
		 *  @param {function|null} [callBack] Callback function
		 *  @param {bool} [redraw=true] Redraw the table or not
		 *  @returns {array} The row that was deleted
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *
		 *      // Immediately remove the first row
		 *      oTable.fnDeleteRow( 0 );
		 *    } );
		 */
		this.fnDeleteRow = function( target, callback, redraw )
		{
			var api = this.api( true );
			var rows = api.rows( target );
			var settings = rows.settings()[0];
			var data = settings.aoData[ rows[0][0] ];
		
			rows.remove();
		
			if ( callback ) {
				callback.call( this, settings, data );
			}
		
			if ( redraw === undefined || redraw ) {
				api.draw();
			}
		
			return data;
		};
		
		
		/**
		 * Restore the table to it's original state in the DOM by removing all of DataTables
		 * enhancements, alterations to the DOM structure of the table and event listeners.
		 *  @param {boolean} [remove=false] Completely remove the table from the DOM
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      // This example is fairly pointless in reality, but shows how fnDestroy can be used
		 *      var oTable = $('#example').dataTable();
		 *      oTable.fnDestroy();
		 *    } );
		 */
		this.fnDestroy = function ( remove )
		{
			this.api( true ).destroy( remove );
		};
		
		
		/**
		 * Redraw the table
		 *  @param {bool} [complete=true] Re-filter and resort (if enabled) the table before the draw.
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *
		 *      // Re-draw the table - you wouldn't want to do it here, but it's an example :-)
		 *      oTable.fnDraw();
		 *    } );
		 */
		this.fnDraw = function( complete )
		{
			// Note that this isn't an exact match to the old call to _fnDraw - it takes
			// into account the new data, but can hold position.
			this.api( true ).draw( complete );
		};
		
		
		/**
		 * Filter the input based on data
		 *  @param {string} sInput String to filter the table on
		 *  @param {int|null} [iColumn] Column to limit filtering to
		 *  @param {bool} [bRegex=false] Treat as regular expression or not
		 *  @param {bool} [bSmart=true] Perform smart filtering or not
		 *  @param {bool} [bShowGlobal=true] Show the input global filter in it's input box(es)
		 *  @param {bool} [bCaseInsensitive=true] Do case-insensitive matching (true) or not (false)
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *
		 *      // Sometime later - filter...
		 *      oTable.fnFilter( 'test string' );
		 *    } );
		 */
		this.fnFilter = function( sInput, iColumn, bRegex, bSmart, bShowGlobal, bCaseInsensitive )
		{
			var api = this.api( true );
		
			if ( iColumn === null || iColumn === undefined ) {
				api.search( sInput, bRegex, bSmart, bCaseInsensitive );
			}
			else {
				api.column( iColumn ).search( sInput, bRegex, bSmart, bCaseInsensitive );
			}
		
			api.draw();
		};
		
		
		/**
		 * Get the data for the whole table, an individual row or an individual cell based on the
		 * provided parameters.
		 *  @param {int|node} [src] A TR row node, TD/TH cell node or an integer. If given as
		 *    a TR node then the data source for the whole row will be returned. If given as a
		 *    TD/TH cell node then iCol will be automatically calculated and the data for the
		 *    cell returned. If given as an integer, then this is treated as the aoData internal
		 *    data index for the row (see fnGetPosition) and the data for that row used.
		 *  @param {int} [col] Optional column index that you want the data of.
		 *  @returns {array|object|string} If mRow is undefined, then the data for all rows is
		 *    returned. If mRow is defined, just data for that row, and is iCol is
		 *    defined, only data for the designated cell is returned.
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    // Row data
		 *    $(document).ready(function() {
		 *      oTable = $('#example').dataTable();
		 *
		 *      oTable.$('tr').click( function () {
		 *        var data = oTable.fnGetData( this );
		 *        // ... do something with the array / object of data for the row
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Individual cell data
		 *    $(document).ready(function() {
		 *      oTable = $('#example').dataTable();
		 *
		 *      oTable.$('td').click( function () {
		 *        var sData = oTable.fnGetData( this );
		 *        alert( 'The cell clicked on had the value of '+sData );
		 *      } );
		 *    } );
		 */
		this.fnGetData = function( src, col )
		{
			var api = this.api( true );
		
			if ( src !== undefined ) {
				var type = src.nodeName ? src.nodeName.toLowerCase() : '';
		
				return col !== undefined || type == 'td' || type == 'th' ?
					api.cell( src, col ).data() :
					api.row( src ).data() || null;
			}
		
			return api.data().toArray();
		};
		
		
		/**
		 * Get an array of the TR nodes that are used in the table's body. Note that you will
		 * typically want to use the '$' API method in preference to this as it is more
		 * flexible.
		 *  @param {int} [iRow] Optional row index for the TR element you want
		 *  @returns {array|node} If iRow is undefined, returns an array of all TR elements
		 *    in the table's body, or iRow is defined, just the TR element requested.
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *
		 *      // Get the nodes from the table
		 *      var nNodes = oTable.fnGetNodes( );
		 *    } );
		 */
		this.fnGetNodes = function( iRow )
		{
			var api = this.api( true );
		
			return iRow !== undefined ?
				api.row( iRow ).node() :
				api.rows().nodes().flatten().toArray();
		};
		
		
		/**
		 * Get the array indexes of a particular cell from it's DOM element
		 * and column index including hidden columns
		 *  @param {node} node this can either be a TR, TD or TH in the table's body
		 *  @returns {int} If nNode is given as a TR, then a single index is returned, or
		 *    if given as a cell, an array of [row index, column index (visible),
		 *    column index (all)] is given.
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      $('#example tbody td').click( function () {
		 *        // Get the position of the current data from the node
		 *        var aPos = oTable.fnGetPosition( this );
		 *
		 *        // Get the data array for this row
		 *        var aData = oTable.fnGetData( aPos[0] );
		 *
		 *        // Update the data array and return the value
		 *        aData[ aPos[1] ] = 'clicked';
		 *        this.innerHTML = 'clicked';
		 *      } );
		 *
		 *      // Init DataTables
		 *      oTable = $('#example').dataTable();
		 *    } );
		 */
		this.fnGetPosition = function( node )
		{
			var api = this.api( true );
			var nodeName = node.nodeName.toUpperCase();
		
			if ( nodeName == 'TR' ) {
				return api.row( node ).index();
			}
			else if ( nodeName == 'TD' || nodeName == 'TH' ) {
				var cell = api.cell( node ).index();
		
				return [
					cell.row,
					cell.columnVisible,
					cell.column
				];
			}
			return null;
		};
		
		
		/**
		 * Check to see if a row is 'open' or not.
		 *  @param {node} nTr the table row to check
		 *  @returns {boolean} true if the row is currently open, false otherwise
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable;
		 *
		 *      // 'open' an information row when a row is clicked on
		 *      $('#example tbody tr').click( function () {
		 *        if ( oTable.fnIsOpen(this) ) {
		 *          oTable.fnClose( this );
		 *        } else {
		 *          oTable.fnOpen( this, "Temporary row opened", "info_row" );
		 *        }
		 *      } );
		 *
		 *      oTable = $('#example').dataTable();
		 *    } );
		 */
		this.fnIsOpen = function( nTr )
		{
			return this.api( true ).row( nTr ).child.isShown();
		};
		
		
		/**
		 * This function will place a new row directly after a row which is currently
		 * on display on the page, with the HTML contents that is passed into the
		 * function. This can be used, for example, to ask for confirmation that a
		 * particular record should be deleted.
		 *  @param {node} nTr The table row to 'open'
		 *  @param {string|node|jQuery} mHtml The HTML to put into the row
		 *  @param {string} sClass Class to give the new TD cell
		 *  @returns {node} The row opened. Note that if the table row passed in as the
		 *    first parameter, is not found in the table, this method will silently
		 *    return.
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable;
		 *
		 *      // 'open' an information row when a row is clicked on
		 *      $('#example tbody tr').click( function () {
		 *        if ( oTable.fnIsOpen(this) ) {
		 *          oTable.fnClose( this );
		 *        } else {
		 *          oTable.fnOpen( this, "Temporary row opened", "info_row" );
		 *        }
		 *      } );
		 *
		 *      oTable = $('#example').dataTable();
		 *    } );
		 */
		this.fnOpen = function( nTr, mHtml, sClass )
		{
			return this.api( true )
				.row( nTr )
				.child( mHtml, sClass )
				.show()
				.child()[0];
		};
		
		
		/**
		 * Change the pagination - provides the internal logic for pagination in a simple API
		 * function. With this function you can have a DataTables table go to the next,
		 * previous, first or last pages.
		 *  @param {string|int} mAction Paging action to take: "first", "previous", "next" or "last"
		 *    or page number to jump to (integer), note that page 0 is the first page.
		 *  @param {bool} [bRedraw=true] Redraw the table or not
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *      oTable.fnPageChange( 'next' );
		 *    } );
		 */
		this.fnPageChange = function ( mAction, bRedraw )
		{
			var api = this.api( true ).page( mAction );
		
			if ( bRedraw === undefined || bRedraw ) {
				api.draw(false);
			}
		};
		
		
		/**
		 * Show a particular column
		 *  @param {int} iCol The column whose display should be changed
		 *  @param {bool} bShow Show (true) or hide (false) the column
		 *  @param {bool} [bRedraw=true] Redraw the table or not
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *
		 *      // Hide the second column after initialisation
		 *      oTable.fnSetColumnVis( 1, false );
		 *    } );
		 */
		this.fnSetColumnVis = function ( iCol, bShow, bRedraw )
		{
			var api = this.api( true ).column( iCol ).visible( bShow );
		
			if ( bRedraw === undefined || bRedraw ) {
				api.columns.adjust().draw();
			}
		};
		
		
		/**
		 * Get the settings for a particular table for external manipulation
		 *  @returns {object} DataTables settings object. See
		 *    {@link DataTable.models.oSettings}
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *      var oSettings = oTable.fnSettings();
		 *
		 *      // Show an example parameter from the settings
		 *      alert( oSettings._iDisplayStart );
		 *    } );
		 */
		this.fnSettings = function()
		{
			return _fnSettingsFromNode( this[_ext.iApiIndex] );
		};
		
		
		/**
		 * Sort the table by a particular column
		 *  @param {int} iCol the data index to sort on. Note that this will not match the
		 *    'display index' if you have hidden data entries
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *
		 *      // Sort immediately with columns 0 and 1
		 *      oTable.fnSort( [ [0,'asc'], [1,'asc'] ] );
		 *    } );
		 */
		this.fnSort = function( aaSort )
		{
			this.api( true ).order( aaSort ).draw();
		};
		
		
		/**
		 * Attach a sort listener to an element for a given column
		 *  @param {node} nNode the element to attach the sort listener to
		 *  @param {int} iColumn the column that a click on this node will sort on
		 *  @param {function} [fnCallback] callback function when sort is run
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *
		 *      // Sort on column 1, when 'sorter' is clicked on
		 *      oTable.fnSortListener( document.getElementById('sorter'), 1 );
		 *    } );
		 */
		this.fnSortListener = function( nNode, iColumn, fnCallback )
		{
			this.api( true ).order.listener( nNode, iColumn, fnCallback );
		};
		
		
		/**
		 * Update a table cell or row - this method will accept either a single value to
		 * update the cell with, an array of values with one element for each column or
		 * an object in the same format as the original data source. The function is
		 * self-referencing in order to make the multi column updates easier.
		 *  @param {object|array|string} mData Data to update the cell/row with
		 *  @param {node|int} mRow TR element you want to update or the aoData index
		 *  @param {int} [iColumn] The column to update, give as null or undefined to
		 *    update a whole row.
		 *  @param {bool} [bRedraw=true] Redraw the table or not
		 *  @param {bool} [bAction=true] Perform pre-draw actions or not
		 *  @returns {int} 0 on success, 1 on error
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *      oTable.fnUpdate( 'Example update', 0, 0 ); // Single cell
		 *      oTable.fnUpdate( ['a', 'b', 'c', 'd', 'e'], $('tbody tr')[0] ); // Row
		 *    } );
		 */
		this.fnUpdate = function( mData, mRow, iColumn, bRedraw, bAction )
		{
			var api = this.api( true );
		
			if ( iColumn === undefined || iColumn === null ) {
				api.row( mRow ).data( mData );
			}
			else {
				api.cell( mRow, iColumn ).data( mData );
			}
		
			if ( bAction === undefined || bAction ) {
				api.columns.adjust();
			}
		
			if ( bRedraw === undefined || bRedraw ) {
				api.draw();
			}
			return 0;
		};
		
		
		/**
		 * Provide a common method for plug-ins to check the version of DataTables being used, in order
		 * to ensure compatibility.
		 *  @param {string} sVersion Version string to check for, in the format "X.Y.Z". Note that the
		 *    formats "X" and "X.Y" are also acceptable.
		 *  @returns {boolean} true if this version of DataTables is greater or equal to the required
		 *    version, or false if this version of DataTales is not suitable
		 *  @method
		 *  @dtopt API
		 *  @deprecated Since v1.10
		 *
		 *  @example
		 *    $(document).ready(function() {
		 *      var oTable = $('#example').dataTable();
		 *      alert( oTable.fnVersionCheck( '1.9.0' ) );
		 *    } );
		 */
		this.fnVersionCheck = _ext.fnVersionCheck;
		

		var _that = this;
		var emptyInit = options === undefined;
		var len = this.length;

		if ( emptyInit ) {
			options = {};
		}

		this.oApi = this.internal = _ext.internal;

		// Extend with old style plug-in API methods
		for ( var fn in DataTable.ext.internal ) {
			if ( fn ) {
				this[fn] = _fnExternApiFunc(fn);
			}
		}

		this.each(function() {
			// For each initialisation we want to give it a clean initialisation
			// object that can be bashed around
			var o = {};
			var oInit = len > 1 ? // optimisation for single table case
				_fnExtend( o, options, true ) :
				options;

			/*global oInit,_that,emptyInit*/
			var i=0, iLen, j, jLen, k, kLen;
			var sId = this.getAttribute( 'id' );
			var bInitHandedOff = false;
			var defaults = DataTable.defaults;
			var $this = $(this);
			
			
			/* Sanity check */
			if ( this.nodeName.toLowerCase() != 'table' )
			{
				_fnLog( null, 0, 'Non-table node initialisation ('+this.nodeName+')', 2 );
				return;
			}
			
			/* Backwards compatibility for the defaults */
			_fnCompatOpts( defaults );
			_fnCompatCols( defaults.column );
			
			/* Convert the camel-case defaults to Hungarian */
			_fnCamelToHungarian( defaults, defaults, true );
			_fnCamelToHungarian( defaults.column, defaults.column, true );
			
			/* Setting up the initialisation object */
			_fnCamelToHungarian( defaults, $.extend( oInit, $this.data() ) );
			
			
			
			/* Check to see if we are re-initialising a table */
			var allSettings = DataTable.settings;
			for ( i=0, iLen=allSettings.length ; i<iLen ; i++ )
			{
				var s = allSettings[i];
			
				/* Base check on table node */
				if ( s.nTable == this || s.nTHead.parentNode == this || (s.nTFoot && s.nTFoot.parentNode == this) )
				{
					var bRetrieve = oInit.bRetrieve !== undefined ? oInit.bRetrieve : defaults.bRetrieve;
					var bDestroy = oInit.bDestroy !== undefined ? oInit.bDestroy : defaults.bDestroy;
			
					if ( emptyInit || bRetrieve )
					{
						return s.oInstance;
					}
					else if ( bDestroy )
					{
						s.oInstance.fnDestroy();
						break;
					}
					else
					{
						_fnLog( s, 0, 'Cannot reinitialise DataTable', 3 );
						return;
					}
				}
			
				/* If the element we are initialising has the same ID as a table which was previously
				 * initialised, but the table nodes don't match (from before) then we destroy the old
				 * instance by simply deleting it. This is under the assumption that the table has been
				 * destroyed by other methods. Anyone using non-id selectors will need to do this manually
				 */
				if ( s.sTableId == this.id )
				{
					allSettings.splice( i, 1 );
					break;
				}
			}
			
			/* Ensure the table has an ID - required for accessibility */
			if ( sId === null || sId === "" )
			{
				sId = "DataTables_Table_"+(DataTable.ext._unique++);
				this.id = sId;
			}
			
			/* Create the settings object for this table and set some of the default parameters */
			var oSettings = $.extend( true, {}, DataTable.models.oSettings, {
				"sDestroyWidth": $this[0].style.width,
				"sInstance":     sId,
				"sTableId":      sId
			} );
			oSettings.nTable = this;
			oSettings.oApi   = _that.internal;
			oSettings.oInit  = oInit;
			
			allSettings.push( oSettings );
			
			// Need to add the instance after the instance after the settings object has been added
			// to the settings array, so we can self reference the table instance if more than one
			oSettings.oInstance = (_that.length===1) ? _that : $this.dataTable();
			
			// Backwards compatibility, before we apply all the defaults
			_fnCompatOpts( oInit );
			
			if ( oInit.oLanguage )
			{
				_fnLanguageCompat( oInit.oLanguage );
			}
			
			// If the length menu is given, but the init display length is not, use the length menu
			if ( oInit.aLengthMenu && ! oInit.iDisplayLength )
			{
				oInit.iDisplayLength = $.isArray( oInit.aLengthMenu[0] ) ?
					oInit.aLengthMenu[0][0] : oInit.aLengthMenu[0];
			}
			
			// Apply the defaults and init options to make a single init object will all
			// options defined from defaults and instance options.
			oInit = _fnExtend( $.extend( true, {}, defaults ), oInit );
			
			
			// Map the initialisation options onto the settings object
			_fnMap( oSettings.oFeatures, oInit, [
				"bPaginate",
				"bLengthChange",
				"bFilter",
				"bSort",
				"bSortMulti",
				"bInfo",
				"bProcessing",
				"bAutoWidth",
				"bSortClasses",
				"bServerSide",
				"bDeferRender"
			] );
			_fnMap( oSettings, oInit, [
				"asStripeClasses",
				"ajax",
				"fnServerData",
				"fnFormatNumber",
				"sServerMethod",
				"aaSorting",
				"aaSortingFixed",
				"aLengthMenu",
				"sPaginationType",
				"sAjaxSource",
				"sAjaxDataProp",
				"iStateDuration",
				"sDom",
				"bSortCellsTop",
				"iTabIndex",
				"fnStateLoadCallback",
				"fnStateSaveCallback",
				"renderer",
				"searchDelay",
				[ "iCookieDuration", "iStateDuration" ], // backwards compat
				[ "oSearch", "oPreviousSearch" ],
				[ "aoSearchCols", "aoPreSearchCols" ],
				[ "iDisplayLength", "_iDisplayLength" ],
				[ "bJQueryUI", "bJUI" ]
			] );
			_fnMap( oSettings.oScroll, oInit, [
				[ "sScrollX", "sX" ],
				[ "sScrollXInner", "sXInner" ],
				[ "sScrollY", "sY" ],
				[ "bScrollCollapse", "bCollapse" ]
			] );
			_fnMap( oSettings.oLanguage, oInit, "fnInfoCallback" );
			
			/* Callback functions which are array driven */
			_fnCallbackReg( oSettings, 'aoDrawCallback',       oInit.fnDrawCallback,      'user' );
			_fnCallbackReg( oSettings, 'aoServerParams',       oInit.fnServerParams,      'user' );
			_fnCallbackReg( oSettings, 'aoStateSaveParams',    oInit.fnStateSaveParams,   'user' );
			_fnCallbackReg( oSettings, 'aoStateLoadParams',    oInit.fnStateLoadParams,   'user' );
			_fnCallbackReg( oSettings, 'aoStateLoaded',        oInit.fnStateLoaded,       'user' );
			_fnCallbackReg( oSettings, 'aoRowCallback',        oInit.fnRowCallback,       'user' );
			_fnCallbackReg( oSettings, 'aoRowCreatedCallback', oInit.fnCreatedRow,        'user' );
			_fnCallbackReg( oSettings, 'aoHeaderCallback',     oInit.fnHeaderCallback,    'user' );
			_fnCallbackReg( oSettings, 'aoFooterCallback',     oInit.fnFooterCallback,    'user' );
			_fnCallbackReg( oSettings, 'aoInitComplete',       oInit.fnInitComplete,      'user' );
			_fnCallbackReg( oSettings, 'aoPreDrawCallback',    oInit.fnPreDrawCallback,   'user' );
			
			var oClasses = oSettings.oClasses;
			
			// @todo Remove in 1.11
			if ( oInit.bJQueryUI )
			{
				/* Use the JUI classes object for display. You could clone the oStdClasses object if
				 * you want to have multiple tables with multiple independent classes
				 */
				$.extend( oClasses, DataTable.ext.oJUIClasses, oInit.oClasses );
			
				if ( oInit.sDom === defaults.sDom && defaults.sDom === "lfrtip" )
				{
					/* Set the DOM to use a layout suitable for jQuery UI's theming */
					oSettings.sDom = '<"H"lfr>t<"F"ip>';
				}
			
				if ( ! oSettings.renderer ) {
					oSettings.renderer = 'jqueryui';
				}
				else if ( $.isPlainObject( oSettings.renderer ) && ! oSettings.renderer.header ) {
					oSettings.renderer.header = 'jqueryui';
				}
			}
			else
			{
				$.extend( oClasses, DataTable.ext.classes, oInit.oClasses );
			}
			$this.addClass( oClasses.sTable );
			
			/* Calculate the scroll bar width and cache it for use later on */
			if ( oSettings.oScroll.sX !== "" || oSettings.oScroll.sY !== "" )
			{
				oSettings.oScroll.iBarWidth = _fnScrollBarWidth();
			}
			if ( oSettings.oScroll.sX === true ) { // Easy initialisation of x-scrolling
				oSettings.oScroll.sX = '100%';
			}
			
			if ( oSettings.iInitDisplayStart === undefined )
			{
				/* Display start point, taking into account the save saving */
				oSettings.iInitDisplayStart = oInit.iDisplayStart;
				oSettings._iDisplayStart = oInit.iDisplayStart;
			}
			
			if ( oInit.iDeferLoading !== null )
			{
				oSettings.bDeferLoading = true;
				var tmp = $.isArray( oInit.iDeferLoading );
				oSettings._iRecordsDisplay = tmp ? oInit.iDeferLoading[0] : oInit.iDeferLoading;
				oSettings._iRecordsTotal = tmp ? oInit.iDeferLoading[1] : oInit.iDeferLoading;
			}
			
			/* Language definitions */
			var oLanguage = oSettings.oLanguage;
			$.extend( true, oLanguage, oInit.oLanguage );
			
			if ( oLanguage.sUrl !== "" )
			{
				/* Get the language definitions from a file - because this Ajax call makes the language
				 * get async to the remainder of this function we use bInitHandedOff to indicate that
				 * _fnInitialise will be fired by the returned Ajax handler, rather than the constructor
				 */
				$.ajax( {
					dataType: 'json',
					url: oLanguage.sUrl,
					success: function ( json ) {
						_fnLanguageCompat( json );
						_fnCamelToHungarian( defaults.oLanguage, json );
						$.extend( true, oLanguage, json );
						_fnInitialise( oSettings );
					},
					error: function () {
						// Error occurred loading language file, continue on as best we can
						_fnInitialise( oSettings );
					}
				} );
				bInitHandedOff = true;
			}
			
			/*
			 * Stripes
			 */
			if ( oInit.asStripeClasses === null )
			{
				oSettings.asStripeClasses =[
					oClasses.sStripeOdd,
					oClasses.sStripeEven
				];
			}
			
			/* Remove row stripe classes if they are already on the table row */
			var stripeClasses = oSettings.asStripeClasses;
			var rowOne = $this.children('tbody').find('tr').eq(0);
			if ( $.inArray( true, $.map( stripeClasses, function(el, i) {
				return rowOne.hasClass(el);
			} ) ) !== -1 ) {
				$('tbody tr', this).removeClass( stripeClasses.join(' ') );
				oSettings.asDestroyStripes = stripeClasses.slice();
			}
			
			/*
			 * Columns
			 * See if we should load columns automatically or use defined ones
			 */
			var anThs = [];
			var aoColumnsInit;
			var nThead = this.getElementsByTagName('thead');
			if ( nThead.length !== 0 )
			{
				_fnDetectHeader( oSettings.aoHeader, nThead[0] );
				anThs = _fnGetUniqueThs( oSettings );
			}
			
			/* If not given a column array, generate one with nulls */
			if ( oInit.aoColumns === null )
			{
				aoColumnsInit = [];
				for ( i=0, iLen=anThs.length ; i<iLen ; i++ )
				{
					aoColumnsInit.push( null );
				}
			}
			else
			{
				aoColumnsInit = oInit.aoColumns;
			}
			
			/* Add the columns */
			for ( i=0, iLen=aoColumnsInit.length ; i<iLen ; i++ )
			{
				_fnAddColumn( oSettings, anThs ? anThs[i] : null );
			}
			
			/* Apply the column definitions */
			_fnApplyColumnDefs( oSettings, oInit.aoColumnDefs, aoColumnsInit, function (iCol, oDef) {
				_fnColumnOptions( oSettings, iCol, oDef );
			} );
			
			/* HTML5 attribute detection - build an mData object automatically if the
			 * attributes are found
			 */
			if ( rowOne.length ) {
				var a = function ( cell, name ) {
					return cell.getAttribute( 'data-'+name ) !== null ? name : null;
				};
			
				$.each( _fnGetRowElements( oSettings, rowOne[0] ).cells, function (i, cell) {
					var col = oSettings.aoColumns[i];
			
					if ( col.mData === i ) {
						var sort = a( cell, 'sort' ) || a( cell, 'order' );
						var filter = a( cell, 'filter' ) || a( cell, 'search' );
			
						if ( sort !== null || filter !== null ) {
							col.mData = {
								_:      i+'.display',
								sort:   sort !== null   ? i+'.@data-'+sort   : undefined,
								type:   sort !== null   ? i+'.@data-'+sort   : undefined,
								filter: filter !== null ? i+'.@data-'+filter : undefined
							};
			
							_fnColumnOptions( oSettings, i );
						}
					}
				} );
			}
			
			var features = oSettings.oFeatures;
			
			/* Must be done after everything which can be overridden by the state saving! */
			if ( oInit.bStateSave )
			{
				features.bStateSave = true;
				_fnLoadState( oSettings, oInit );
				_fnCallbackReg( oSettings, 'aoDrawCallback', _fnSaveState, 'state_save' );
			}
			
			
			/*
			 * Sorting
			 * @todo For modularisation (1.11) this needs to do into a sort start up handler
			 */
			
			// If aaSorting is not defined, then we use the first indicator in asSorting
			// in case that has been altered, so the default sort reflects that option
			if ( oInit.aaSorting === undefined )
			{
				var sorting = oSettings.aaSorting;
				for ( i=0, iLen=sorting.length ; i<iLen ; i++ )
				{
					sorting[i][1] = oSettings.aoColumns[ i ].asSorting[0];
				}
			}
			
			/* Do a first pass on the sorting classes (allows any size changes to be taken into
			 * account, and also will apply sorting disabled classes if disabled
			 */
			_fnSortingClasses( oSettings );
			
			if ( features.bSort )
			{
				_fnCallbackReg( oSettings, 'aoDrawCallback', function () {
					if ( oSettings.bSorted ) {
						var aSort = _fnSortFlatten( oSettings );
						var sortedColumns = {};
			
						$.each( aSort, function (i, val) {
							sortedColumns[ val.src ] = val.dir;
						} );
			
						_fnCallbackFire( oSettings, null, 'order', [oSettings, aSort, sortedColumns] );
						_fnSortAria( oSettings );
					}
				} );
			}
			
			_fnCallbackReg( oSettings, 'aoDrawCallback', function () {
				if ( oSettings.bSorted || _fnDataSource( oSettings ) === 'ssp' || features.bDeferRender ) {
					_fnSortingClasses( oSettings );
				}
			}, 'sc' );
			
			
			/*
			 * Final init
			 * Cache the header, body and footer as required, creating them if needed
			 */
			
			/* Browser support detection */
			_fnBrowserDetect( oSettings );
			
			// Work around for Webkit bug 83867 - store the caption-side before removing from doc
			var captions = $this.children('caption').each( function () {
				this._captionSide = $this.css('caption-side');
			} );
			
			var thead = $this.children('thead');
			if ( thead.length === 0 )
			{
				thead = $('<thead/>').appendTo(this);
			}
			oSettings.nTHead = thead[0];
			
			var tbody = $this.children('tbody');
			if ( tbody.length === 0 )
			{
				tbody = $('<tbody/>').appendTo(this);
			}
			oSettings.nTBody = tbody[0];
			
			var tfoot = $this.children('tfoot');
			if ( tfoot.length === 0 && captions.length > 0 && (oSettings.oScroll.sX !== "" || oSettings.oScroll.sY !== "") )
			{
				// If we are a scrolling table, and no footer has been given, then we need to create
				// a tfoot element for the caption element to be appended to
				tfoot = $('<tfoot/>').appendTo(this);
			}
			
			if ( tfoot.length === 0 || tfoot.children().length === 0 ) {
				$this.addClass( oClasses.sNoFooter );
			}
			else if ( tfoot.length > 0 ) {
				oSettings.nTFoot = tfoot[0];
				_fnDetectHeader( oSettings.aoFooter, oSettings.nTFoot );
			}
			
			/* Check if there is data passing into the constructor */
			if ( oInit.aaData )
			{
				for ( i=0 ; i<oInit.aaData.length ; i++ )
				{
					_fnAddData( oSettings, oInit.aaData[ i ] );
				}
			}
			else if ( oSettings.bDeferLoading || _fnDataSource( oSettings ) == 'dom' )
			{
				/* Grab the data from the page - only do this when deferred loading or no Ajax
				 * source since there is no point in reading the DOM data if we are then going
				 * to replace it with Ajax data
				 */
				_fnAddTr( oSettings, $(oSettings.nTBody).children('tr') );
			}
			
			/* Copy the data index array */
			oSettings.aiDisplay = oSettings.aiDisplayMaster.slice();
			
			/* Initialisation complete - table can be drawn */
			oSettings.bInitialised = true;
			
			/* Check if we need to initialise the table (it might not have been handed off to the
			 * language processor)
			 */
			if ( bInitHandedOff === false )
			{
				_fnInitialise( oSettings );
			}
		} );
		_that = null;
		return this;
	};

	
	
	/**
	 * Computed structure of the DataTables API, defined by the options passed to
	 * `DataTable.Api.register()` when building the API.
	 *
	 * The structure is built in order to speed creation and extension of the Api
	 * objects since the extensions are effectively pre-parsed.
	 *
	 * The array is an array of objects with the following structure, where this
	 * base array represents the Api prototype base:
	 *
	 *     [
	 *       {
	 *         name:      'data'                -- string   - Property name
	 *         val:       function () {},       -- function - Api method (or undefined if just an object
	 *         methodExt: [ ... ],              -- array    - Array of Api object definitions to extend the method result
	 *         propExt:   [ ... ]               -- array    - Array of Api object definitions to extend the property
	 *       },
	 *       {
	 *         name:     'row'
	 *         val:       {},
	 *         methodExt: [ ... ],
	 *         propExt:   [
	 *           {
	 *             name:      'data'
	 *             val:       function () {},
	 *             methodExt: [ ... ],
	 *             propExt:   [ ... ]
	 *           },
	 *           ...
	 *         ]
	 *       }
	 *     ]
	 *
	 * @type {Array}
	 * @ignore
	 */
	var __apiStruct = [];
	
	
	/**
	 * `Array.prototype` reference.
	 *
	 * @type object
	 * @ignore
	 */
	var __arrayProto = Array.prototype;
	
	
	/**
	 * Abstraction for `context` parameter of the `Api` constructor to allow it to
	 * take several different forms for ease of use.
	 *
	 * Each of the input parameter types will be converted to a DataTables settings
	 * object where possible.
	 *
	 * @param  {string|node|jQuery|object} mixed DataTable identifier. Can be one
	 *   of:
	 *
	 *   * `string` - jQuery selector. Any DataTables' matching the given selector
	 *     with be found and used.
	 *   * `node` - `TABLE` node which has already been formed into a DataTable.
	 *   * `jQuery` - A jQuery object of `TABLE` nodes.
	 *   * `object` - DataTables settings object
	 *   * `DataTables.Api` - API instance
	 * @return {array|null} Matching DataTables settings objects. `null` or
	 *   `undefined` is returned if no matching DataTable is found.
	 * @ignore
	 */
	var _toSettings = function ( mixed )
	{
		var idx, jq;
		var settings = DataTable.settings;
		var tables = $.map( settings, function (el, i) {
			return el.nTable;
		} );
	
		if ( ! mixed ) {
			return [];
		}
		else if ( mixed.nTable && mixed.oApi ) {
			// DataTables settings object
			return [ mixed ];
		}
		else if ( mixed.nodeName && mixed.nodeName.toLowerCase() === 'table' ) {
			// Table node
			idx = $.inArray( mixed, tables );
			return idx !== -1 ? [ settings[idx] ] : null;
		}
		else if ( mixed && typeof mixed.settings === 'function' ) {
			return mixed.settings().toArray();
		}
		else if ( typeof mixed === 'string' ) {
			// jQuery selector
			jq = $(mixed);
		}
		else if ( mixed instanceof $ ) {
			// jQuery object (also DataTables instance)
			jq = mixed;
		}
	
		if ( jq ) {
			return jq.map( function(i) {
				idx = $.inArray( this, tables );
				return idx !== -1 ? settings[idx] : null;
			} ).toArray();
		}
	};
	
	
	/**
	 * DataTables API class - used to control and interface with  one or more
	 * DataTables enhanced tables.
	 *
	 * The API class is heavily based on jQuery, presenting a chainable interface
	 * that you can use to interact with tables. Each instance of the API class has
	 * a "context" - i.e. the tables that it will operate on. This could be a single
	 * table, all tables on a page or a sub-set thereof.
	 *
	 * Additionally the API is designed to allow you to easily work with the data in
	 * the tables, retrieving and manipulating it as required. This is done by
	 * presenting the API class as an array like interface. The contents of the
	 * array depend upon the actions requested by each method (for example
	 * `rows().nodes()` will return an array of nodes, while `rows().data()` will
	 * return an array of objects or arrays depending upon your table's
	 * configuration). The API object has a number of array like methods (`push`,
	 * `pop`, `reverse` etc) as well as additional helper methods (`each`, `pluck`,
	 * `unique` etc) to assist your working with the data held in a table.
	 *
	 * Most methods (those which return an Api instance) are chainable, which means
	 * the return from a method call also has all of the methods available that the
	 * top level object had. For example, these two calls are equivalent:
	 *
	 *     // Not chained
	 *     api.row.add( {...} );
	 *     api.draw();
	 *
	 *     // Chained
	 *     api.row.add( {...} ).draw();
	 *
	 * @class DataTable.Api
	 * @param {array|object|string|jQuery} context DataTable identifier. This is
	 *   used to define which DataTables enhanced tables this API will operate on.
	 *   Can be one of:
	 *
	 *   * `string` - jQuery selector. Any DataTables' matching the given selector
	 *     with be found and used.
	 *   * `node` - `TABLE` node which has already been formed into a DataTable.
	 *   * `jQuery` - A jQuery object of `TABLE` nodes.
	 *   * `object` - DataTables settings object
	 * @param {array} [data] Data to initialise the Api instance with.
	 *
	 * @example
	 *   // Direct initialisation during DataTables construction
	 *   var api = $('#example').DataTable();
	 *
	 * @example
	 *   // Initialisation using a DataTables jQuery object
	 *   var api = $('#example').dataTable().api();
	 *
	 * @example
	 *   // Initialisation as a constructor
	 *   var api = new $.fn.DataTable.Api( 'table.dataTable' );
	 */
	_Api = function ( context, data )
	{
		if ( ! (this instanceof _Api) ) {
			return new _Api( context, data );
		}
	
		var settings = [];
		var ctxSettings = function ( o ) {
			var a = _toSettings( o );
			if ( a ) {
				settings.push.apply( settings, a );
			}
		};
	
		if ( $.isArray( context ) ) {
			for ( var i=0, ien=context.length ; i<ien ; i++ ) {
				ctxSettings( context[i] );
			}
		}
		else {
			ctxSettings( context );
		}
	
		// Remove duplicates
		this.context = _unique( settings );
	
		// Initial data
		if ( data ) {
			this.push.apply( this, data.toArray ? data.toArray() : data );
		}
	
		// selector
		this.selector = {
			rows: null,
			cols: null,
			opts: null
		};
	
		_Api.extend( this, this, __apiStruct );
	};
	
	DataTable.Api = _Api;
	
	_Api.prototype = /** @lends DataTables.Api */{
		any: function ()
		{
			return this.flatten().length !== 0;
		},
	
	
		concat:  __arrayProto.concat,
	
	
		context: [], // array of table settings objects
	
	
		each: function ( fn )
		{
			for ( var i=0, ien=this.length ; i<ien; i++ ) {
				fn.call( this, this[i], i, this );
			}
	
			return this;
		},
	
	
		eq: function ( idx )
		{
			var ctx = this.context;
	
			return ctx.length > idx ?
				new _Api( ctx[idx], this[idx] ) :
				null;
		},
	
	
		filter: function ( fn )
		{
			var a = [];
	
			if ( __arrayProto.filter ) {
				a = __arrayProto.filter.call( this, fn, this );
			}
			else {
				// Compatibility for browsers without EMCA-252-5 (JS 1.6)
				for ( var i=0, ien=this.length ; i<ien ; i++ ) {
					if ( fn.call( this, this[i], i, this ) ) {
						a.push( this[i] );
					}
				}
			}
	
			return new _Api( this.context, a );
		},
	
	
		flatten: function ()
		{
			var a = [];
			return new _Api( this.context, a.concat.apply( a, this.toArray() ) );
		},
	
	
		join:    __arrayProto.join,
	
	
		indexOf: __arrayProto.indexOf || function (obj, start)
		{
			for ( var i=(start || 0), ien=this.length ; i<ien ; i++ ) {
				if ( this[i] === obj ) {
					return i;
				}
			}
			return -1;
		},
	
		iterator: function ( flatten, type, fn, alwaysNew ) {
			var
				a = [], ret,
				i, ien, j, jen,
				context = this.context,
				rows, items, item,
				selector = this.selector;
	
			// Argument shifting
			if ( typeof flatten === 'string' ) {
				alwaysNew = fn;
				fn = type;
				type = flatten;
				flatten = false;
			}
	
			for ( i=0, ien=context.length ; i<ien ; i++ ) {
				var apiInst = new _Api( context[i] );
	
				if ( type === 'table' ) {
					ret = fn.call( apiInst, context[i], i );
	
					if ( ret !== undefined ) {
						a.push( ret );
					}
				}
				else if ( type === 'columns' || type === 'rows' ) {
					// this has same length as context - one entry for each table
					ret = fn.call( apiInst, context[i], this[i], i );
	
					if ( ret !== undefined ) {
						a.push( ret );
					}
				}
				else if ( type === 'column' || type === 'column-rows' || type === 'row' || type === 'cell' ) {
					// columns and rows share the same structure.
					// 'this' is an array of column indexes for each context
					items = this[i];
	
					if ( type === 'column-rows' ) {
						rows = _selector_row_indexes( context[i], selector.opts );
					}
	
					for ( j=0, jen=items.length ; j<jen ; j++ ) {
						item = items[j];
	
						if ( type === 'cell' ) {
							ret = fn.call( apiInst, context[i], item.row, item.column, i, j );
						}
						else {
							ret = fn.call( apiInst, context[i], item, i, j, rows );
						}
	
						if ( ret !== undefined ) {
							a.push( ret );
						}
					}
				}
			}
	
			if ( a.length || alwaysNew ) {
				var api = new _Api( context, flatten ? a.concat.apply( [], a ) : a );
				var apiSelector = api.selector;
				apiSelector.rows = selector.rows;
				apiSelector.cols = selector.cols;
				apiSelector.opts = selector.opts;
				return api;
			}
			return this;
		},
	
	
		lastIndexOf: __arrayProto.lastIndexOf || function (obj, start)
		{
			// Bit cheeky...
			return this.indexOf.apply( this.toArray.reverse(), arguments );
		},
	
	
		length:  0,
	
	
		map: function ( fn )
		{
			var a = [];
	
			if ( __arrayProto.map ) {
				a = __arrayProto.map.call( this, fn, this );
			}
			else {
				// Compatibility for browsers without EMCA-252-5 (JS 1.6)
				for ( var i=0, ien=this.length ; i<ien ; i++ ) {
					a.push( fn.call( this, this[i], i ) );
				}
			}
	
			return new _Api( this.context, a );
		},
	
	
		pluck: function ( prop )
		{
			return this.map( function ( el ) {
				return el[ prop ];
			} );
		},
	
		pop:     __arrayProto.pop,
	
	
		push:    __arrayProto.push,
	
	
		// Does not return an API instance
		reduce: __arrayProto.reduce || function ( fn, init )
		{
			return _fnReduce( this, fn, init, 0, this.length, 1 );
		},
	
	
		reduceRight: __arrayProto.reduceRight || function ( fn, init )
		{
			return _fnReduce( this, fn, init, this.length-1, -1, -1 );
		},
	
	
		reverse: __arrayProto.reverse,
	
	
		// Object with rows, columns and opts
		selector: null,
	
	
		shift:   __arrayProto.shift,
	
	
		sort:    __arrayProto.sort, // ? name - order?
	
	
		splice:  __arrayProto.splice,
	
	
		toArray: function ()
		{
			return __arrayProto.slice.call( this );
		},
	
	
		to$: function ()
		{
			return $( this );
		},
	
	
		toJQuery: function ()
		{
			return $( this );
		},
	
	
		unique: function ()
		{
			return new _Api( this.context, _unique(this) );
		},
	
	
		unshift: __arrayProto.unshift
	};
	
	
	_Api.extend = function ( scope, obj, ext )
	{
		// Only extend API instances and static properties of the API
		if ( ! ext.length || ! obj || ( ! (obj instanceof _Api) && ! obj.__dt_wrapper ) ) {
			return;
		}
	
		var
			i, ien,
			j, jen,
			struct, inner,
			methodScoping = function ( scope, fn, struc ) {
				return function () {
					var ret = fn.apply( scope, arguments );
	
					// Method extension
					_Api.extend( ret, ret, struc.methodExt );
					return ret;
				};
			};
	
		for ( i=0, ien=ext.length ; i<ien ; i++ ) {
			struct = ext[i];
	
			// Value
			obj[ struct.name ] = typeof struct.val === 'function' ?
				methodScoping( scope, struct.val, struct ) :
				$.isPlainObject( struct.val ) ?
					{} :
					struct.val;
	
			obj[ struct.name ].__dt_wrapper = true;
	
			// Property extension
			_Api.extend( scope, obj[ struct.name ], struct.propExt );
		}
	};
	
	
	// @todo - Is there need for an augment function?
	// _Api.augment = function ( inst, name )
	// {
	// 	// Find src object in the structure from the name
	// 	var parts = name.split('.');
	
	// 	_Api.extend( inst, obj );
	// };
	
	
	//     [
	//       {
	//         name:      'data'                -- string   - Property name
	//         val:       function () {},       -- function - Api method (or undefined if just an object
	//         methodExt: [ ... ],              -- array    - Array of Api object definitions to extend the method result
	//         propExt:   [ ... ]               -- array    - Array of Api object definitions to extend the property
	//       },
	//       {
	//         name:     'row'
	//         val:       {},
	//         methodExt: [ ... ],
	//         propExt:   [
	//           {
	//             name:      'data'
	//             val:       function () {},
	//             methodExt: [ ... ],
	//             propExt:   [ ... ]
	//           },
	//           ...
	//         ]
	//       }
	//     ]
	
	_Api.register = _api_register = function ( name, val )
	{
		if ( $.isArray( name ) ) {
			for ( var j=0, jen=name.length ; j<jen ; j++ ) {
				_Api.register( name[j], val );
			}
			return;
		}
	
		var
			i, ien,
			heir = name.split('.'),
			struct = __apiStruct,
			key, method;
	
		var find = function ( src, name ) {
			for ( var i=0, ien=src.length ; i<ien ; i++ ) {
				if ( src[i].name === name ) {
					return src[i];
				}
			}
			return null;
		};
	
		for ( i=0, ien=heir.length ; i<ien ; i++ ) {
			method = heir[i].indexOf('()') !== -1;
			key = method ?
				heir[i].replace('()', '') :
				heir[i];
	
			var src = find( struct, key );
			if ( ! src ) {
				src = {
					name:      key,
					val:       {},
					methodExt: [],
					propExt:   []
				};
				struct.push( src );
			}
	
			if ( i === ien-1 ) {
				src.val = val;
			}
			else {
				struct = method ?
					src.methodExt :
					src.propExt;
			}
		}
	};
	
	
	_Api.registerPlural = _api_registerPlural = function ( pluralName, singularName, val ) {
		_Api.register( pluralName, val );
	
		_Api.register( singularName, function () {
			var ret = val.apply( this, arguments );
	
			if ( ret === this ) {
				// Returned item is the API instance that was passed in, return it
				return this;
			}
			else if ( ret instanceof _Api ) {
				// New API instance returned, want the value from the first item
				// in the returned array for the singular result.
				return ret.length ?
					$.isArray( ret[0] ) ?
						new _Api( ret.context, ret[0] ) : // Array results are 'enhanced'
						ret[0] :
					undefined;
			}
	
			// Non-API return - just fire it back
			return ret;
		} );
	};
	
	
	/**
	 * Selector for HTML tables. Apply the given selector to the give array of
	 * DataTables settings objects.
	 *
	 * @param {string|integer} [selector] jQuery selector string or integer
	 * @param  {array} Array of DataTables settings objects to be filtered
	 * @return {array}
	 * @ignore
	 */
	var __table_selector = function ( selector, a )
	{
		// Integer is used to pick out a table by index
		if ( typeof selector === 'number' ) {
			return [ a[ selector ] ];
		}
	
		// Perform a jQuery selector on the table nodes
		var nodes = $.map( a, function (el, i) {
			return el.nTable;
		} );
	
		return $(nodes)
			.filter( selector )
			.map( function (i) {
				// Need to translate back from the table node to the settings
				var idx = $.inArray( this, nodes );
				return a[ idx ];
			} )
			.toArray();
	};
	
	
	
	/**
	 * Context selector for the API's context (i.e. the tables the API instance
	 * refers to.
	 *
	 * @name    DataTable.Api#tables
	 * @param {string|integer} [selector] Selector to pick which tables the iterator
	 *   should operate on. If not given, all tables in the current context are
	 *   used. This can be given as a jQuery selector (for example `':gt(0)'`) to
	 *   select multiple tables or as an integer to select a single table.
	 * @returns {DataTable.Api} Returns a new API instance if a selector is given.
	 */
	_api_register( 'tables()', function ( selector ) {
		// A new instance is created if there was a selector specified
		return selector ?
			new _Api( __table_selector( selector, this.context ) ) :
			this;
	} );
	
	
	_api_register( 'table()', function ( selector ) {
		var tables = this.tables( selector );
		var ctx = tables.context;
	
		// Truncate to the first matched table
		return ctx.length ?
			new _Api( ctx[0] ) :
			tables;
	} );
	
	
	_api_registerPlural( 'tables().nodes()', 'table().node()' , function () {
		return this.iterator( 'table', function ( ctx ) {
			return ctx.nTable;
		}, 1 );
	} );
	
	
	_api_registerPlural( 'tables().body()', 'table().body()' , function () {
		return this.iterator( 'table', function ( ctx ) {
			return ctx.nTBody;
		}, 1 );
	} );
	
	
	_api_registerPlural( 'tables().header()', 'table().header()' , function () {
		return this.iterator( 'table', function ( ctx ) {
			return ctx.nTHead;
		}, 1 );
	} );
	
	
	_api_registerPlural( 'tables().footer()', 'table().footer()' , function () {
		return this.iterator( 'table', function ( ctx ) {
			return ctx.nTFoot;
		}, 1 );
	} );
	
	
	_api_registerPlural( 'tables().containers()', 'table().container()' , function () {
		return this.iterator( 'table', function ( ctx ) {
			return ctx.nTableWrapper;
		}, 1 );
	} );
	
	
	
	/**
	 * Redraw the tables in the current context.
	 *
	 * @param {boolean} [reset=true] Reset (default) or hold the current paging
	 *   position. A full re-sort and re-filter is performed when this method is
	 *   called, which is why the pagination reset is the default action.
	 * @returns {DataTables.Api} this
	 */
	_api_register( 'draw()', function ( resetPaging ) {
		return this.iterator( 'table', function ( settings ) {
			_fnReDraw( settings, resetPaging===false );
		} );
	} );
	
	
	
	/**
	 * Get the current page index.
	 *
	 * @return {integer} Current page index (zero based)
	 *//**
	 * Set the current page.
	 *
	 * Note that if you attempt to show a page which does not exist, DataTables will
	 * not throw an error, but rather reset the paging.
	 *
	 * @param {integer|string} action The paging action to take. This can be one of:
	 *  * `integer` - The page index to jump to
	 *  * `string` - An action to take:
	 *    * `first` - Jump to first page.
	 *    * `next` - Jump to the next page
	 *    * `previous` - Jump to previous page
	 *    * `last` - Jump to the last page.
	 * @returns {DataTables.Api} this
	 */
	_api_register( 'page()', function ( action ) {
		if ( action === undefined ) {
			return this.page.info().page; // not an expensive call
		}
	
		// else, have an action to take on all tables
		return this.iterator( 'table', function ( settings ) {
			_fnPageChange( settings, action );
		} );
	} );
	
	
	/**
	 * Paging information for the first table in the current context.
	 *
	 * If you require paging information for another table, use the `table()` method
	 * with a suitable selector.
	 *
	 * @return {object} Object with the following properties set:
	 *  * `page` - Current page index (zero based - i.e. the first page is `0`)
	 *  * `pages` - Total number of pages
	 *  * `start` - Display index for the first record shown on the current page
	 *  * `end` - Display index for the last record shown on the current page
	 *  * `length` - Display length (number of records). Note that generally `start
	 *    + length = end`, but this is not always true, for example if there are
	 *    only 2 records to show on the final page, with a length of 10.
	 *  * `recordsTotal` - Full data set length
	 *  * `recordsDisplay` - Data set length once the current filtering criterion
	 *    are applied.
	 */
	_api_register( 'page.info()', function ( action ) {
		if ( this.context.length === 0 ) {
			return undefined;
		}
	
		var
			settings   = this.context[0],
			start      = settings._iDisplayStart,
			len        = settings._iDisplayLength,
			visRecords = settings.fnRecordsDisplay(),
			all        = len === -1;
	
		return {
			"page":           all ? 0 : Math.floor( start / len ),
			"pages":          all ? 1 : Math.ceil( visRecords / len ),
			"start":          start,
			"end":            settings.fnDisplayEnd(),
			"length":         len,
			"recordsTotal":   settings.fnRecordsTotal(),
			"recordsDisplay": visRecords
		};
	} );
	
	
	/**
	 * Get the current page length.
	 *
	 * @return {integer} Current page length. Note `-1` indicates that all records
	 *   are to be shown.
	 *//**
	 * Set the current page length.
	 *
	 * @param {integer} Page length to set. Use `-1` to show all records.
	 * @returns {DataTables.Api} this
	 */
	_api_register( 'page.len()', function ( len ) {
		// Note that we can't call this function 'length()' because `length`
		// is a Javascript property of functions which defines how many arguments
		// the function expects.
		if ( len === undefined ) {
			return this.context.length !== 0 ?
				this.context[0]._iDisplayLength :
				undefined;
		}
	
		// else, set the page length
		return this.iterator( 'table', function ( settings ) {
			_fnLengthChange( settings, len );
		} );
	} );
	
	
	
	var __reload = function ( settings, holdPosition, callback ) {
		// Use the draw event to trigger a callback
		if ( callback ) {
			var api = new _Api( settings );
	
			api.one( 'draw', function () {
				callback( api.ajax.json() );
			} );
		}
	
		if ( _fnDataSource( settings ) == 'ssp' ) {
			_fnReDraw( settings, holdPosition );
		}
		else {
			// Trigger xhr
			_fnProcessingDisplay( settings, true );
	
			_fnBuildAjax( settings, [], function( json ) {
				_fnClearTable( settings );
	
				var data = _fnAjaxDataSrc( settings, json );
				for ( var i=0, ien=data.length ; i<ien ; i++ ) {
					_fnAddData( settings, data[i] );
				}
	
				_fnReDraw( settings, holdPosition );
				_fnProcessingDisplay( settings, false );
			} );
		}
	};
	
	
	/**
	 * Get the JSON response from the last Ajax request that DataTables made to the
	 * server. Note that this returns the JSON from the first table in the current
	 * context.
	 *
	 * @return {object} JSON received from the server.
	 */
	_api_register( 'ajax.json()', function () {
		var ctx = this.context;
	
		if ( ctx.length > 0 ) {
			return ctx[0].json;
		}
	
		// else return undefined;
	} );
	
	
	/**
	 * Get the data submitted in the last Ajax request
	 */
	_api_register( 'ajax.params()', function () {
		var ctx = this.context;
	
		if ( ctx.length > 0 ) {
			return ctx[0].oAjaxData;
		}
	
		// else return undefined;
	} );
	
	
	/**
	 * Reload tables from the Ajax data source. Note that this function will
	 * automatically re-draw the table when the remote data has been loaded.
	 *
	 * @param {boolean} [reset=true] Reset (default) or hold the current paging
	 *   position. A full re-sort and re-filter is performed when this method is
	 *   called, which is why the pagination reset is the default action.
	 * @returns {DataTables.Api} this
	 */
	_api_register( 'ajax.reload()', function ( callback, resetPaging ) {
		return this.iterator( 'table', function (settings) {
			__reload( settings, resetPaging===false, callback );
		} );
	} );
	
	
	/**
	 * Get the current Ajax URL. Note that this returns the URL from the first
	 * table in the current context.
	 *
	 * @return {string} Current Ajax source URL
	 *//**
	 * Set the Ajax URL. Note that this will set the URL for all tables in the
	 * current context.
	 *
	 * @param {string} url URL to set.
	 * @returns {DataTables.Api} this
	 */
	_api_register( 'ajax.url()', function ( url ) {
		var ctx = this.context;
	
		if ( url === undefined ) {
			// get
			if ( ctx.length === 0 ) {
				return undefined;
			}
			ctx = ctx[0];
	
			return ctx.ajax ?
				$.isPlainObject( ctx.ajax ) ?
					ctx.ajax.url :
					ctx.ajax :
				ctx.sAjaxSource;
		}
	
		// set
		return this.iterator( 'table', function ( settings ) {
			if ( $.isPlainObject( settings.ajax ) ) {
				settings.ajax.url = url;
			}
			else {
				settings.ajax = url;
			}
			// No need to consider sAjaxSource here since DataTables gives priority
			// to `ajax` over `sAjaxSource`. So setting `ajax` here, renders any
			// value of `sAjaxSource` redundant.
		} );
	} );
	
	
	/**
	 * Load data from the newly set Ajax URL. Note that this method is only
	 * available when `ajax.url()` is used to set a URL. Additionally, this method
	 * has the same effect as calling `ajax.reload()` but is provided for
	 * convenience when setting a new URL. Like `ajax.reload()` it will
	 * automatically redraw the table once the remote data has been loaded.
	 *
	 * @returns {DataTables.Api} this
	 */
	_api_register( 'ajax.url().load()', function ( callback, resetPaging ) {
		// Same as a reload, but makes sense to present it for easy access after a
		// url change
		return this.iterator( 'table', function ( ctx ) {
			__reload( ctx, resetPaging===false, callback );
		} );
	} );
	
	
	
	
	var _selector_run = function ( type, selector, selectFn, settings, opts )
	{
		var
			out = [], res,
			a, i, ien, j, jen,
			selectorType = typeof selector;
	
		// Can't just check for isArray here, as an API or jQuery instance might be
		// given with their array like look
		if ( ! selector || selectorType === 'string' || selectorType === 'function' || selector.length === undefined ) {
			selector = [ selector ];
		}
	
		for ( i=0, ien=selector.length ; i<ien ; i++ ) {
			a = selector[i] && selector[i].split ?
				selector[i].split(',') :
				[ selector[i] ];
	
			for ( j=0, jen=a.length ; j<jen ; j++ ) {
				res = selectFn( typeof a[j] === 'string' ? $.trim(a[j]) : a[j] );
	
				if ( res && res.length ) {
					out.push.apply( out, res );
				}
			}
		}
	
		// selector extensions
		var ext = _ext.selector[ type ];
		if ( ext.length ) {
			for ( i=0, ien=ext.length ; i<ien ; i++ ) {
				out = ext[i]( settings, opts, out );
			}
		}
	
		return out;
	};
	
	
	var _selector_opts = function ( opts )
	{
		if ( ! opts ) {
			opts = {};
		}
	
		// Backwards compatibility for 1.9- which used the terminology filter rather
		// than search
		if ( opts.filter && opts.search === undefined ) {
			opts.search = opts.filter;
		}
	
		return $.extend( {
			search: 'none',
			order: 'current',
			page: 'all'
		}, opts );
	};
	
	
	var _selector_first = function ( inst )
	{
		// Reduce the API instance to the first item found
		for ( var i=0, ien=inst.length ; i<ien ; i++ ) {
			if ( inst[i].length > 0 ) {
				// Assign the first element to the first item in the instance
				// and truncate the instance and context
				inst[0] = inst[i];
				inst[0].length = 1;
				inst.length = 1;
				inst.context = [ inst.context[i] ];
	
				return inst;
			}
		}
	
		// Not found - return an empty instance
		inst.length = 0;
		return inst;
	};
	
	
	var _selector_row_indexes = function ( settings, opts )
	{
		var
			i, ien, tmp, a=[],
			displayFiltered = settings.aiDisplay,
			displayMaster = settings.aiDisplayMaster;
	
		var
			search = opts.search,  // none, applied, removed
			order  = opts.order,   // applied, current, index (original - compatibility with 1.9)
			page   = opts.page;    // all, current
	
		if ( _fnDataSource( settings ) == 'ssp' ) {
			// In server-side processing mode, most options are irrelevant since
			// rows not shown don't exist and the index order is the applied order
			// Removed is a special case - for consistency just return an empty
			// array
			return search === 'removed' ?
				[] :
				_range( 0, displayMaster.length );
		}
		else if ( page == 'current' ) {
			// Current page implies that order=current and fitler=applied, since it is
			// fairly senseless otherwise, regardless of what order and search actually
			// are
			for ( i=settings._iDisplayStart, ien=settings.fnDisplayEnd() ; i<ien ; i++ ) {
				a.push( displayFiltered[i] );
			}
		}
		else if ( order == 'current' || order == 'applied' ) {
			a = search == 'none' ?
				displayMaster.slice() :                      // no search
				search == 'applied' ?
					displayFiltered.slice() :                // applied search
					$.map( displayMaster, function (el, i) { // removed search
						return $.inArray( el, displayFiltered ) === -1 ? el : null;
					} );
		}
		else if ( order == 'index' || order == 'original' ) {
			for ( i=0, ien=settings.aoData.length ; i<ien ; i++ ) {
				if ( search == 'none' ) {
					a.push( i );
				}
				else { // applied | removed
					tmp = $.inArray( i, displayFiltered );
	
					if ((tmp === -1 && search == 'removed') ||
						(tmp >= 0   && search == 'applied') )
					{
						a.push( i );
					}
				}
			}
		}
	
		return a;
	};
	
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * Rows
	 *
	 * {}          - no selector - use all available rows
	 * {integer}   - row aoData index
	 * {node}      - TR node
	 * {string}    - jQuery selector to apply to the TR elements
	 * {array}     - jQuery array of nodes, or simply an array of TR nodes
	 *
	 */
	
	
	var __row_selector = function ( settings, selector, opts )
	{
		var run = function ( sel ) {
			var selInt = _intVal( sel );
			var i, ien;
	
			// Short cut - selector is a number and no options provided (default is
			// all records, so no need to check if the index is in there, since it
			// must be - dev error if the index doesn't exist).
			if ( selInt !== null && ! opts ) {
				return [ selInt ];
			}
	
			var rows = _selector_row_indexes( settings, opts );
	
			if ( selInt !== null && $.inArray( selInt, rows ) !== -1 ) {
				// Selector - integer
				return [ selInt ];
			}
			else if ( ! sel ) {
				// Selector - none
				return rows;
			}
	
			// Selector - function
			if ( typeof sel === 'function' ) {
				return $.map( rows, function (idx) {
					var row = settings.aoData[ idx ];
					return sel( idx, row._aData, row.nTr ) ? idx : null;
				} );
			}
	
			// Get nodes in the order from the `rows` array with null values removed
			var nodes = _removeEmpty(
				_pluck_order( settings.aoData, rows, 'nTr' )
			);
	
			// Selector - node
			if ( sel.nodeName ) {
				if ( $.inArray( sel, nodes ) !== -1 ) {
					return [ sel._DT_RowIndex ]; // sel is a TR node that is in the table
					                             // and DataTables adds a prop for fast lookup
				}
			}
	
			// Selector - jQuery selector string, array of nodes or jQuery object/
			// As jQuery's .filter() allows jQuery objects to be passed in filter,
			// it also allows arrays, so this will cope with all three options
			return $(nodes)
				.filter( sel )
				.map( function () {
					return this._DT_RowIndex;
				} )
				.toArray();
		};
	
		return _selector_run( 'row', selector, run, settings, opts );
	};
	
	
	_api_register( 'rows()', function ( selector, opts ) {
		// argument shifting
		if ( selector === undefined ) {
			selector = '';
		}
		else if ( $.isPlainObject( selector ) ) {
			opts = selector;
			selector = '';
		}
	
		opts = _selector_opts( opts );
	
		var inst = this.iterator( 'table', function ( settings ) {
			return __row_selector( settings, selector, opts );
		}, 1 );
	
		// Want argument shifting here and in __row_selector?
		inst.selector.rows = selector;
		inst.selector.opts = opts;
	
		return inst;
	} );
	
	_api_register( 'rows().nodes()', function () {
		return this.iterator( 'row', function ( settings, row ) {
			return settings.aoData[ row ].nTr || undefined;
		}, 1 );
	} );
	
	_api_register( 'rows().data()', function () {
		return this.iterator( true, 'rows', function ( settings, rows ) {
			return _pluck_order( settings.aoData, rows, '_aData' );
		}, 1 );
	} );
	
	_api_registerPlural( 'rows().cache()', 'row().cache()', function ( type ) {
		return this.iterator( 'row', function ( settings, row ) {
			var r = settings.aoData[ row ];
			return type === 'search' ? r._aFilterData : r._aSortData;
		}, 1 );
	} );
	
	_api_registerPlural( 'rows().invalidate()', 'row().invalidate()', function ( src ) {
		return this.iterator( 'row', function ( settings, row ) {
			_fnInvalidate( settings, row, src );
		} );
	} );
	
	_api_registerPlural( 'rows().indexes()', 'row().index()', function () {
		return this.iterator( 'row', function ( settings, row ) {
			return row;
		}, 1 );
	} );
	
	_api_registerPlural( 'rows().remove()', 'row().remove()', function () {
		var that = this;
	
		return this.iterator( 'row', function ( settings, row, thatIdx ) {
			var data = settings.aoData;
	
			data.splice( row, 1 );
	
			// Update the _DT_RowIndex parameter on all rows in the table
			for ( var i=0, ien=data.length ; i<ien ; i++ ) {
				if ( data[i].nTr !== null ) {
					data[i].nTr._DT_RowIndex = i;
				}
			}
	
			// Remove the target row from the search array
			var displayIndex = $.inArray( row, settings.aiDisplay );
	
			// Delete from the display arrays
			_fnDeleteIndex( settings.aiDisplayMaster, row );
			_fnDeleteIndex( settings.aiDisplay, row );
			_fnDeleteIndex( that[ thatIdx ], row, false ); // maintain local indexes
	
			// Check for an 'overflow' they case for displaying the table
			_fnLengthOverflow( settings );
		} );
	} );
	
	
	_api_register( 'rows.add()', function ( rows ) {
		var newRows = this.iterator( 'table', function ( settings ) {
				var row, i, ien;
				var out = [];
	
				for ( i=0, ien=rows.length ; i<ien ; i++ ) {
					row = rows[i];
	
					if ( row.nodeName && row.nodeName.toUpperCase() === 'TR' ) {
						out.push( _fnAddTr( settings, row )[0] );
					}
					else {
						out.push( _fnAddData( settings, row ) );
					}
				}
	
				return out;
			}, 1 );
	
		// Return an Api.rows() extended instance, so rows().nodes() etc can be used
		var modRows = this.rows( -1 );
		modRows.pop();
		modRows.push.apply( modRows, newRows.toArray() );
	
		return modRows;
	} );
	
	
	
	
	
	/**
	 *
	 */
	_api_register( 'row()', function ( selector, opts ) {
		return _selector_first( this.rows( selector, opts ) );
	} );
	
	
	_api_register( 'row().data()', function ( data ) {
		var ctx = this.context;
	
		if ( data === undefined ) {
			// Get
			return ctx.length && this.length ?
				ctx[0].aoData[ this[0] ]._aData :
				undefined;
		}
	
		// Set
		ctx[0].aoData[ this[0] ]._aData = data;
	
		// Automatically invalidate
		_fnInvalidate( ctx[0], this[0], 'data' );
	
		return this;
	} );
	
	
	_api_register( 'row().node()', function () {
		var ctx = this.context;
	
		return ctx.length && this.length ?
			ctx[0].aoData[ this[0] ].nTr || null :
			null;
	} );
	
	
	_api_register( 'row.add()', function ( row ) {
		// Allow a jQuery object to be passed in - only a single row is added from
		// it though - the first element in the set
		if ( row instanceof $ && row.length ) {
			row = row[0];
		}
	
		var rows = this.iterator( 'table', function ( settings ) {
			if ( row.nodeName && row.nodeName.toUpperCase() === 'TR' ) {
				return _fnAddTr( settings, row )[0];
			}
			return _fnAddData( settings, row );
		} );
	
		// Return an Api.rows() extended instance, with the newly added row selected
		return this.row( rows[0] );
	} );
	
	
	
	var __details_add = function ( ctx, row, data, klass )
	{
		// Convert to array of TR elements
		var rows = [];
		var addRow = function ( r, k ) {
			// Recursion to allow for arrays of jQuery objects
			if ( $.isArray( r ) || r instanceof $ ) {
				for ( var i=0, ien=r.length ; i<ien ; i++ ) {
					addRow( r[i], k );
				}
				return;
			}
	
			// If we get a TR element, then just add it directly - up to the dev
			// to add the correct number of columns etc
			if ( r.nodeName && r.nodeName.toLowerCase() === 'tr' ) {
				rows.push( r );
			}
			else {
				// Otherwise create a row with a wrapper
				var created = $('<tr><td/></tr>').addClass( k );
				$('td', created)
					.addClass( k )
					.html( r )
					[0].colSpan = _fnVisbleColumns( ctx );
	
				rows.push( created[0] );
			}
		};
	
		addRow( data, klass );
	
		if ( row._details ) {
			row._details.remove();
		}
	
		row._details = $(rows);
	
		// If the children were already shown, that state should be retained
		if ( row._detailsShow ) {
			row._details.insertAfter( row.nTr );
		}
	};
	
	
	var __details_remove = function ( api, idx )
	{
		var ctx = api.context;
	
		if ( ctx.length ) {
			var row = ctx[0].aoData[ idx !== undefined ? idx : api[0] ];
	
			if ( row._details ) {
				row._details.remove();
	
				row._detailsShow = undefined;
				row._details = undefined;
			}
		}
	};
	
	
	var __details_display = function ( api, show ) {
		var ctx = api.context;
	
		if ( ctx.length && api.length ) {
			var row = ctx[0].aoData[ api[0] ];
	
			if ( row._details ) {
				row._detailsShow = show;
	
				if ( show ) {
					row._details.insertAfter( row.nTr );
				}
				else {
					row._details.detach();
				}
	
				__details_events( ctx[0] );
			}
		}
	};
	
	
	var __details_events = function ( settings )
	{
		var api = new _Api( settings );
		var namespace = '.dt.DT_details';
		var drawEvent = 'draw'+namespace;
		var colvisEvent = 'column-visibility'+namespace;
		var destroyEvent = 'destroy'+namespace;
		var data = settings.aoData;
	
		api.off( drawEvent +' '+ colvisEvent +' '+ destroyEvent );
	
		if ( _pluck( data, '_details' ).length > 0 ) {
			// On each draw, insert the required elements into the document
			api.on( drawEvent, function ( e, ctx ) {
				if ( settings !== ctx ) {
					return;
				}
	
				api.rows( {page:'current'} ).eq(0).each( function (idx) {
					// Internal data grab
					var row = data[ idx ];
	
					if ( row._detailsShow ) {
						row._details.insertAfter( row.nTr );
					}
				} );
			} );
	
			// Column visibility change - update the colspan
			api.on( colvisEvent, function ( e, ctx, idx, vis ) {
				if ( settings !== ctx ) {
					return;
				}
	
				// Update the colspan for the details rows (note, only if it already has
				// a colspan)
				var row, visible = _fnVisbleColumns( ctx );
	
				for ( var i=0, ien=data.length ; i<ien ; i++ ) {
					row = data[i];
	
					if ( row._details ) {
						row._details.children('td[colspan]').attr('colspan', visible );
					}
				}
			} );
	
			// Table destroyed - nuke any child rows
			api.on( destroyEvent, function ( e, ctx ) {
				if ( settings !== ctx ) {
					return;
				}
	
				for ( var i=0, ien=data.length ; i<ien ; i++ ) {
					if ( data[i]._details ) {
						__details_remove( api, i );
					}
				}
			} );
		}
	};
	
	// Strings for the method names to help minification
	var _emp = '';
	var _child_obj = _emp+'row().child';
	var _child_mth = _child_obj+'()';
	
	// data can be:
	//  tr
	//  string
	//  jQuery or array of any of the above
	_api_register( _child_mth, function ( data, klass ) {
		var ctx = this.context;
	
		if ( data === undefined ) {
			// get
			return ctx.length && this.length ?
				ctx[0].aoData[ this[0] ]._details :
				undefined;
		}
		else if ( data === true ) {
			// show
			this.child.show();
		}
		else if ( data === false ) {
			// remove
			__details_remove( this );
		}
		else if ( ctx.length && this.length ) {
			// set
			__details_add( ctx[0], ctx[0].aoData[ this[0] ], data, klass );
		}
	
		return this;
	} );
	
	
	_api_register( [
		_child_obj+'.show()',
		_child_mth+'.show()' // only when `child()` was called with parameters (without
	], function ( show ) {   // it returns an object and this method is not executed)
		__details_display( this, true );
		return this;
	} );
	
	
	_api_register( [
		_child_obj+'.hide()',
		_child_mth+'.hide()' // only when `child()` was called with parameters (without
	], function () {         // it returns an object and this method is not executed)
		__details_display( this, false );
		return this;
	} );
	
	
	_api_register( [
		_child_obj+'.remove()',
		_child_mth+'.remove()' // only when `child()` was called with parameters (without
	], function () {           // it returns an object and this method is not executed)
		__details_remove( this );
		return this;
	} );
	
	
	_api_register( _child_obj+'.isShown()', function () {
		var ctx = this.context;
	
		if ( ctx.length && this.length ) {
			// _detailsShown as false or undefined will fall through to return false
			return ctx[0].aoData[ this[0] ]._detailsShow || false;
		}
		return false;
	} );
	
	
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * Columns
	 *
	 * {integer}           - column index (>=0 count from left, <0 count from right)
	 * "{integer}:visIdx"  - visible column index (i.e. translate to column index)  (>=0 count from left, <0 count from right)
	 * "{integer}:visible" - alias for {integer}:visIdx  (>=0 count from left, <0 count from right)
	 * "{string}:name"     - column name
	 * "{string}"          - jQuery selector on column header nodes
	 *
	 */
	
	// can be an array of these items, comma separated list, or an array of comma
	// separated lists
	
	var __re_column_selector = /^(.+):(name|visIdx|visible)$/;
	
	
	// r1 and r2 are redundant - but it means that the parameters match for the
	// iterator callback in columns().data()
	var __columnData = function ( settings, column, r1, r2, rows ) {
		var a = [];
		for ( var row=0, ien=rows.length ; row<ien ; row++ ) {
			a.push( _fnGetCellData( settings, rows[row], column ) );
		}
		return a;
	};
	
	
	var __column_selector = function ( settings, selector, opts )
	{
		var
			columns = settings.aoColumns,
			names = _pluck( columns, 'sName' ),
			nodes = _pluck( columns, 'nTh' );
	
		var run = function ( s ) {
			var selInt = _intVal( s );
	
			// Selector - all
			if ( s === '' ) {
				return _range( columns.length );
			}
			
			// Selector - index
			if ( selInt !== null ) {
				return [ selInt >= 0 ?
					selInt : // Count from left
					columns.length + selInt // Count from right (+ because its a negative value)
				];
			}
			
			// Selector = function
			if ( typeof s === 'function' ) {
				var rows = _selector_row_indexes( settings, opts );
	
				return $.map( columns, function (col, idx) {
					return s(
							idx,
							__columnData( settings, idx, 0, 0, rows ),
							nodes[ idx ]
						) ? idx : null;
				} );
			}
	
			// jQuery or string selector
			var match = typeof s === 'string' ?
				s.match( __re_column_selector ) :
				'';
	
			if ( match ) {
				switch( match[2] ) {
					case 'visIdx':
					case 'visible':
						var idx = parseInt( match[1], 10 );
						// Visible index given, convert to column index
						if ( idx < 0 ) {
							// Counting from the right
							var visColumns = $.map( columns, function (col,i) {
								return col.bVisible ? i : null;
							} );
							return [ visColumns[ visColumns.length + idx ] ];
						}
						// Counting from the left
						return [ _fnVisibleToColumnIndex( settings, idx ) ];
	
					case 'name':
						// match by name. `names` is column index complete and in order
						return $.map( names, function (name, i) {
							return name === match[1] ? i : null;
						} );
				}
			}
			else {
				// jQuery selector on the TH elements for the columns
				return $( nodes )
					.filter( s )
					.map( function () {
						return $.inArray( this, nodes ); // `nodes` is column index complete and in order
					} )
					.toArray();
			}
		};
	
		return _selector_run( 'column', selector, run, settings, opts );
	};
	
	
	var __setColumnVis = function ( settings, column, vis, recalc ) {
		var
			cols = settings.aoColumns,
			col  = cols[ column ],
			data = settings.aoData,
			row, cells, i, ien, tr;
	
		// Get
		if ( vis === undefined ) {
			return col.bVisible;
		}
	
		// Set
		// No change
		if ( col.bVisible === vis ) {
			return;
		}
	
		if ( vis ) {
			// Insert column
			// Need to decide if we should use appendChild or insertBefore
			var insertBefore = $.inArray( true, _pluck(cols, 'bVisible'), column+1 );
	
			for ( i=0, ien=data.length ; i<ien ; i++ ) {
				tr = data[i].nTr;
				cells = data[i].anCells;
	
				if ( tr ) {
					// insertBefore can act like appendChild if 2nd arg is null
					tr.insertBefore( cells[ column ], cells[ insertBefore ] || null );
				}
			}
		}
		else {
			// Remove column
			$( _pluck( settings.aoData, 'anCells', column ) ).detach();
		}
	
		// Common actions
		col.bVisible = vis;
		_fnDrawHead( settings, settings.aoHeader );
		_fnDrawHead( settings, settings.aoFooter );
	
		if ( recalc === undefined || recalc ) {
			// Automatically adjust column sizing
			_fnAdjustColumnSizing( settings );
	
			// Realign columns for scrolling
			if ( settings.oScroll.sX || settings.oScroll.sY ) {
				_fnScrollDraw( settings );
			}
		}
	
		_fnCallbackFire( settings, null, 'column-visibility', [settings, column, vis] );
	
		_fnSaveState( settings );
	};
	
	
	_api_register( 'columns()', function ( selector, opts ) {
		// argument shifting
		if ( selector === undefined ) {
			selector = '';
		}
		else if ( $.isPlainObject( selector ) ) {
			opts = selector;
			selector = '';
		}
	
		opts = _selector_opts( opts );
	
		var inst = this.iterator( 'table', function ( settings ) {
			return __column_selector( settings, selector, opts );
		}, 1 );
	
		// Want argument shifting here and in _row_selector?
		inst.selector.cols = selector;
		inst.selector.opts = opts;
	
		return inst;
	} );
	
	_api_registerPlural( 'columns().header()', 'column().header()', function ( selector, opts ) {
		return this.iterator( 'column', function ( settings, column ) {
			return settings.aoColumns[column].nTh;
		}, 1 );
	} );
	
	_api_registerPlural( 'columns().footer()', 'column().footer()', function ( selector, opts ) {
		return this.iterator( 'column', function ( settings, column ) {
			return settings.aoColumns[column].nTf;
		}, 1 );
	} );
	
	_api_registerPlural( 'columns().data()', 'column().data()', function () {
		return this.iterator( 'column-rows', __columnData, 1 );
	} );
	
	_api_registerPlural( 'columns().dataSrc()', 'column().dataSrc()', function () {
		return this.iterator( 'column', function ( settings, column ) {
			return settings.aoColumns[column].mData;
		}, 1 );
	} );
	
	_api_registerPlural( 'columns().cache()', 'column().cache()', function ( type ) {
		return this.iterator( 'column-rows', function ( settings, column, i, j, rows ) {
			return _pluck_order( settings.aoData, rows,
				type === 'search' ? '_aFilterData' : '_aSortData', column
			);
		}, 1 );
	} );
	
	_api_registerPlural( 'columns().nodes()', 'column().nodes()', function () {
		return this.iterator( 'column-rows', function ( settings, column, i, j, rows ) {
			return _pluck_order( settings.aoData, rows, 'anCells', column ) ;
		}, 1 );
	} );
	
	_api_registerPlural( 'columns().visible()', 'column().visible()', function ( vis, calc ) {
		return this.iterator( 'column', function ( settings, column ) {
			if ( vis === undefined ) {
				return settings.aoColumns[ column ].bVisible;
			} // else
			__setColumnVis( settings, column, vis, calc );
		} );
	} );
	
	_api_registerPlural( 'columns().indexes()', 'column().index()', function ( type ) {
		return this.iterator( 'column', function ( settings, column ) {
			return type === 'visible' ?
				_fnColumnIndexToVisible( settings, column ) :
				column;
		}, 1 );
	} );
	
	_api_register( 'columns.adjust()', function () {
		return this.iterator( 'table', function ( settings ) {
			_fnAdjustColumnSizing( settings );
		}, 1 );
	} );
	
	_api_register( 'column.index()', function ( type, idx ) {
		if ( this.context.length !== 0 ) {
			var ctx = this.context[0];
	
			if ( type === 'fromVisible' || type === 'toData' ) {
				return _fnVisibleToColumnIndex( ctx, idx );
			}
			else if ( type === 'fromData' || type === 'toVisible' ) {
				return _fnColumnIndexToVisible( ctx, idx );
			}
		}
	} );
	
	_api_register( 'column()', function ( selector, opts ) {
		return _selector_first( this.columns( selector, opts ) );
	} );
	
	
	
	
	var __cell_selector = function ( settings, selector, opts )
	{
		var data = settings.aoData;
		var rows = _selector_row_indexes( settings, opts );
		var cells = _removeEmpty( _pluck_order( data, rows, 'anCells' ) );
		var allCells = $( [].concat.apply([], cells) );
		var row;
		var columns = settings.aoColumns.length;
		var a, i, ien, j, o, host;
	
		var run = function ( s ) {
			var fnSelector = typeof s === 'function';
	
			if ( s === null || s === undefined || fnSelector ) {
				// All cells and function selectors
				a = [];
	
				for ( i=0, ien=rows.length ; i<ien ; i++ ) {
					row = rows[i];
	
					for ( j=0 ; j<columns ; j++ ) {
						o = {
							row: row,
							column: j
						};
	
						if ( fnSelector ) {
							// Selector - function
							host = settings.aoData[ row ];
	
							if ( s( o, _fnGetCellData(settings, row, j), host.anCells ? host.anCells[j] : null ) ) {
								a.push( o );
							}
						}
						else {
							// Selector - all
							a.push( o );
						}
					}
				}
	
				return a;
			}
			
			// Selector - index
			if ( $.isPlainObject( s ) ) {
				return [s];
			}
	
			// Selector - jQuery filtered cells
			return allCells
				.filter( s )
				.map( function (i, el) {
					row = el.parentNode._DT_RowIndex;
	
					return {
						row: row,
						column: $.inArray( el, data[ row ].anCells )
					};
				} )
				.toArray();
		};
	
		return _selector_run( 'cell', selector, run, settings, opts );
	};
	
	
	
	
	_api_register( 'cells()', function ( rowSelector, columnSelector, opts ) {
		// Argument shifting
		if ( $.isPlainObject( rowSelector ) ) {
			// Indexes
			if ( rowSelector.row === undefined ) {
				// Selector options in first parameter
				opts = rowSelector;
				rowSelector = null;
			}
			else {
				// Cell index objects in first parameter
				opts = columnSelector;
				columnSelector = null;
			}
		}
		if ( $.isPlainObject( columnSelector ) ) {
			opts = columnSelector;
			columnSelector = null;
		}
	
		// Cell selector
		if ( columnSelector === null || columnSelector === undefined ) {
			return this.iterator( 'table', function ( settings ) {
				return __cell_selector( settings, rowSelector, _selector_opts( opts ) );
			} );
		}
	
		// Row + column selector
		var columns = this.columns( columnSelector, opts );
		var rows = this.rows( rowSelector, opts );
		var a, i, ien, j, jen;
	
		var cells = this.iterator( 'table', function ( settings, idx ) {
			a = [];
	
			for ( i=0, ien=rows[idx].length ; i<ien ; i++ ) {
				for ( j=0, jen=columns[idx].length ; j<jen ; j++ ) {
					a.push( {
						row:    rows[idx][i],
						column: columns[idx][j]
					} );
				}
			}
	
			return a;
		}, 1 );
	
		$.extend( cells.selector, {
			cols: columnSelector,
			rows: rowSelector,
			opts: opts
		} );
	
		return cells;
	} );
	
	
	_api_registerPlural( 'cells().nodes()', 'cell().node()', function () {
		return this.iterator( 'cell', function ( settings, row, column ) {
			var cells = settings.aoData[ row ].anCells;
			return cells ?
				cells[ column ] :
				undefined;
		}, 1 );
	} );
	
	
	_api_register( 'cells().data()', function () {
		return this.iterator( 'cell', function ( settings, row, column ) {
			return _fnGetCellData( settings, row, column );
		}, 1 );
	} );
	
	
	_api_registerPlural( 'cells().cache()', 'cell().cache()', function ( type ) {
		type = type === 'search' ? '_aFilterData' : '_aSortData';
	
		return this.iterator( 'cell', function ( settings, row, column ) {
			return settings.aoData[ row ][ type ][ column ];
		}, 1 );
	} );
	
	
	_api_registerPlural( 'cells().render()', 'cell().render()', function ( type ) {
		return this.iterator( 'cell', function ( settings, row, column ) {
			return _fnGetCellData( settings, row, column, type );
		}, 1 );
	} );
	
	
	_api_registerPlural( 'cells().indexes()', 'cell().index()', function () {
		return this.iterator( 'cell', function ( settings, row, column ) {
			return {
				row: row,
				column: column,
				columnVisible: _fnColumnIndexToVisible( settings, column )
			};
		}, 1 );
	} );
	
	
	_api_registerPlural( 'cells().invalidate()', 'cell().invalidate()', function ( src ) {
		return this.iterator( 'cell', function ( settings, row, column ) {
			_fnInvalidate( settings, row, src, column );
		} );
	} );
	
	
	
	_api_register( 'cell()', function ( rowSelector, columnSelector, opts ) {
		return _selector_first( this.cells( rowSelector, columnSelector, opts ) );
	} );
	
	
	_api_register( 'cell().data()', function ( data ) {
		var ctx = this.context;
		var cell = this[0];
	
		if ( data === undefined ) {
			// Get
			return ctx.length && cell.length ?
				_fnGetCellData( ctx[0], cell[0].row, cell[0].column ) :
				undefined;
		}
	
		// Set
		_fnSetCellData( ctx[0], cell[0].row, cell[0].column, data );
		_fnInvalidate( ctx[0], cell[0].row, 'data', cell[0].column );
	
		return this;
	} );
	
	
	
	/**
	 * Get current ordering (sorting) that has been applied to the table.
	 *
	 * @returns {array} 2D array containing the sorting information for the first
	 *   table in the current context. Each element in the parent array represents
	 *   a column being sorted upon (i.e. multi-sorting with two columns would have
	 *   2 inner arrays). The inner arrays may have 2 or 3 elements. The first is
	 *   the column index that the sorting condition applies to, the second is the
	 *   direction of the sort (`desc` or `asc`) and, optionally, the third is the
	 *   index of the sorting order from the `column.sorting` initialisation array.
	 *//**
	 * Set the ordering for the table.
	 *
	 * @param {integer} order Column index to sort upon.
	 * @param {string} direction Direction of the sort to be applied (`asc` or `desc`)
	 * @returns {DataTables.Api} this
	 *//**
	 * Set the ordering for the table.
	 *
	 * @param {array} order 1D array of sorting information to be applied.
	 * @param {array} [...] Optional additional sorting conditions
	 * @returns {DataTables.Api} this
	 *//**
	 * Set the ordering for the table.
	 *
	 * @param {array} order 2D array of sorting information to be applied.
	 * @returns {DataTables.Api} this
	 */
	_api_register( 'order()', function ( order, dir ) {
		var ctx = this.context;
	
		if ( order === undefined ) {
			// get
			return ctx.length !== 0 ?
				ctx[0].aaSorting :
				undefined;
		}
	
		// set
		if ( typeof order === 'number' ) {
			// Simple column / direction passed in
			order = [ [ order, dir ] ];
		}
		else if ( ! $.isArray( order[0] ) ) {
			// Arguments passed in (list of 1D arrays)
			order = Array.prototype.slice.call( arguments );
		}
		// otherwise a 2D array was passed in
	
		return this.iterator( 'table', function ( settings ) {
			settings.aaSorting = order.slice();
		} );
	} );
	
	
	/**
	 * Attach a sort listener to an element for a given column
	 *
	 * @param {node|jQuery|string} node Identifier for the element(s) to attach the
	 *   listener to. This can take the form of a single DOM node, a jQuery
	 *   collection of nodes or a jQuery selector which will identify the node(s).
	 * @param {integer} column the column that a click on this node will sort on
	 * @param {function} [callback] callback function when sort is run
	 * @returns {DataTables.Api} this
	 */
	_api_register( 'order.listener()', function ( node, column, callback ) {
		return this.iterator( 'table', function ( settings ) {
			_fnSortAttachListener( settings, node, column, callback );
		} );
	} );
	
	
	// Order by the selected column(s)
	_api_register( [
		'columns().order()',
		'column().order()'
	], function ( dir ) {
		var that = this;
	
		return this.iterator( 'table', function ( settings, i ) {
			var sort = [];
	
			$.each( that[i], function (j, col) {
				sort.push( [ col, dir ] );
			} );
	
			settings.aaSorting = sort;
		} );
	} );
	
	
	
	_api_register( 'search()', function ( input, regex, smart, caseInsen ) {
		var ctx = this.context;
	
		if ( input === undefined ) {
			// get
			return ctx.length !== 0 ?
				ctx[0].oPreviousSearch.sSearch :
				undefined;
		}
	
		// set
		return this.iterator( 'table', function ( settings ) {
			if ( ! settings.oFeatures.bFilter ) {
				return;
			}
	
			_fnFilterComplete( settings, $.extend( {}, settings.oPreviousSearch, {
				"sSearch": input+"",
				"bRegex":  regex === null ? false : regex,
				"bSmart":  smart === null ? true  : smart,
				"bCaseInsensitive": caseInsen === null ? true : caseInsen
			} ), 1 );
		} );
	} );
	
	
	_api_registerPlural(
		'columns().search()',
		'column().search()',
		function ( input, regex, smart, caseInsen ) {
			return this.iterator( 'column', function ( settings, column ) {
				var preSearch = settings.aoPreSearchCols;
	
				if ( input === undefined ) {
					// get
					return preSearch[ column ].sSearch;
				}
	
				// set
				if ( ! settings.oFeatures.bFilter ) {
					return;
				}
	
				$.extend( preSearch[ column ], {
					"sSearch": input+"",
					"bRegex":  regex === null ? false : regex,
					"bSmart":  smart === null ? true  : smart,
					"bCaseInsensitive": caseInsen === null ? true : caseInsen
				} );
	
				_fnFilterComplete( settings, settings.oPreviousSearch, 1 );
			} );
		}
	);
	
	/*
	 * State API methods
	 */
	
	_api_register( 'state()', function () {
		return this.context.length ?
			this.context[0].oSavedState :
			null;
	} );
	
	
	_api_register( 'state.clear()', function () {
		return this.iterator( 'table', function ( settings ) {
			// Save an empty object
			settings.fnStateSaveCallback.call( settings.oInstance, settings, {} );
		} );
	} );
	
	
	_api_register( 'state.loaded()', function () {
		return this.context.length ?
			this.context[0].oLoadedState :
			null;
	} );
	
	
	_api_register( 'state.save()', function () {
		return this.iterator( 'table', function ( settings ) {
			_fnSaveState( settings );
		} );
	} );
	
	
	
	/**
	 * Provide a common method for plug-ins to check the version of DataTables being
	 * used, in order to ensure compatibility.
	 *
	 *  @param {string} version Version string to check for, in the format "X.Y.Z".
	 *    Note that the formats "X" and "X.Y" are also acceptable.
	 *  @returns {boolean} true if this version of DataTables is greater or equal to
	 *    the required version, or false if this version of DataTales is not
	 *    suitable
	 *  @static
	 *  @dtopt API-Static
	 *
	 *  @example
	 *    alert( $.fn.dataTable.versionCheck( '1.9.0' ) );
	 */
	DataTable.versionCheck = DataTable.fnVersionCheck = function( version )
	{
		var aThis = DataTable.version.split('.');
		var aThat = version.split('.');
		var iThis, iThat;
	
		for ( var i=0, iLen=aThat.length ; i<iLen ; i++ ) {
			iThis = parseInt( aThis[i], 10 ) || 0;
			iThat = parseInt( aThat[i], 10 ) || 0;
	
			// Parts are the same, keep comparing
			if (iThis === iThat) {
				continue;
			}
	
			// Parts are different, return immediately
			return iThis > iThat;
		}
	
		return true;
	};
	
	
	/**
	 * Check if a `<table>` node is a DataTable table already or not.
	 *
	 *  @param {node|jquery|string} table Table node, jQuery object or jQuery
	 *      selector for the table to test. Note that if more than more than one
	 *      table is passed on, only the first will be checked
	 *  @returns {boolean} true the table given is a DataTable, or false otherwise
	 *  @static
	 *  @dtopt API-Static
	 *
	 *  @example
	 *    if ( ! $.fn.DataTable.isDataTable( '#example' ) ) {
	 *      $('#example').dataTable();
	 *    }
	 */
	DataTable.isDataTable = DataTable.fnIsDataTable = function ( table )
	{
		var t = $(table).get(0);
		var is = false;
	
		$.each( DataTable.settings, function (i, o) {
			var head = o.nScrollHead ? $('table', o.nScrollHead)[0] : null;
			var foot = o.nScrollFoot ? $('table', o.nScrollFoot)[0] : null;
	
			if ( o.nTable === t || head === t || foot === t ) {
				is = true;
			}
		} );
	
		return is;
	};
	
	
	/**
	 * Get all DataTable tables that have been initialised - optionally you can
	 * select to get only currently visible tables.
	 *
	 *  @param {boolean} [visible=false] Flag to indicate if you want all (default)
	 *    or visible tables only.
	 *  @returns {array} Array of `table` nodes (not DataTable instances) which are
	 *    DataTables
	 *  @static
	 *  @dtopt API-Static
	 *
	 *  @example
	 *    $.each( $.fn.dataTable.tables(true), function () {
	 *      $(table).DataTable().columns.adjust();
	 *    } );
	 */
	DataTable.tables = DataTable.fnTables = function ( visible )
	{
		return $.map( DataTable.settings, function (o) {
			if ( !visible || (visible && $(o.nTable).is(':visible')) ) {
				return o.nTable;
			}
		} );
	};
	
	
	/**
	 * DataTables utility methods
	 * 
	 * This namespace provides helper methods that DataTables uses internally to
	 * create a DataTable, but which are not exclusively used only for DataTables.
	 * These methods can be used by extension authors to save the duplication of
	 * code.
	 *
	 *  @namespace
	 */
	DataTable.util = {
		/**
		 * Throttle the calls to a function. Arguments and context are maintained
		 * for the throttled function.
		 *
		 * @param {function} fn Function to be called
		 * @param {integer} freq Call frequency in mS
		 * @return {function} Wrapped function
		 */
		throttle: _fnThrottle,
	
	
		/**
		 * Escape a string such that it can be used in a regular expression
		 *
		 *  @param {string} sVal string to escape
		 *  @returns {string} escaped string
		 */
		escapeRegex: _fnEscapeRegex
	};
	
	
	/**
	 * Convert from camel case parameters to Hungarian notation. This is made public
	 * for the extensions to provide the same ability as DataTables core to accept
	 * either the 1.9 style Hungarian notation, or the 1.10+ style camelCase
	 * parameters.
	 *
	 *  @param {object} src The model object which holds all parameters that can be
	 *    mapped.
	 *  @param {object} user The object to convert from camel case to Hungarian.
	 *  @param {boolean} force When set to `true`, properties which already have a
	 *    Hungarian value in the `user` object will be overwritten. Otherwise they
	 *    won't be.
	 */
	DataTable.camelToHungarian = _fnCamelToHungarian;
	
	
	
	/**
	 *
	 */
	_api_register( '$()', function ( selector, opts ) {
		var
			rows   = this.rows( opts ).nodes(), // Get all rows
			jqRows = $(rows);
	
		return $( [].concat(
			jqRows.filter( selector ).toArray(),
			jqRows.find( selector ).toArray()
		) );
	} );
	
	
	// jQuery functions to operate on the tables
	$.each( [ 'on', 'one', 'off' ], function (i, key) {
		_api_register( key+'()', function ( /* event, handler */ ) {
			var args = Array.prototype.slice.call(arguments);
	
			// Add the `dt` namespace automatically if it isn't already present
			if ( ! args[0].match(/\.dt\b/) ) {
				args[0] += '.dt';
			}
	
			var inst = $( this.tables().nodes() );
			inst[key].apply( inst, args );
			return this;
		} );
	} );
	
	
	_api_register( 'clear()', function () {
		return this.iterator( 'table', function ( settings ) {
			_fnClearTable( settings );
		} );
	} );
	
	
	_api_register( 'settings()', function () {
		return new _Api( this.context, this.context );
	} );
	
	
	_api_register( 'init()', function () {
		var ctx = this.context;
		return ctx.length ? ctx[0].oInit : null;
	} );
	
	
	_api_register( 'data()', function () {
		return this.iterator( 'table', function ( settings ) {
			return _pluck( settings.aoData, '_aData' );
		} ).flatten();
	} );
	
	
	_api_register( 'destroy()', function ( remove ) {
		remove = remove || false;
	
		return this.iterator( 'table', function ( settings ) {
			var orig      = settings.nTableWrapper.parentNode;
			var classes   = settings.oClasses;
			var table     = settings.nTable;
			var tbody     = settings.nTBody;
			var thead     = settings.nTHead;
			var tfoot     = settings.nTFoot;
			var jqTable   = $(table);
			var jqTbody   = $(tbody);
			var jqWrapper = $(settings.nTableWrapper);
			var rows      = $.map( settings.aoData, function (r) { return r.nTr; } );
			var i, ien;
	
			// Flag to note that the table is currently being destroyed - no action
			// should be taken
			settings.bDestroying = true;
	
			// Fire off the destroy callbacks for plug-ins etc
			_fnCallbackFire( settings, "aoDestroyCallback", "destroy", [settings] );
	
			// If not being removed from the document, make all columns visible
			if ( ! remove ) {
				new _Api( settings ).columns().visible( true );
			}
	
			// Blitz all `DT` namespaced events (these are internal events, the
			// lowercase, `dt` events are user subscribed and they are responsible
			// for removing them
			jqWrapper.unbind('.DT').find(':not(tbody *)').unbind('.DT');
			$(window).unbind('.DT-'+settings.sInstance);
	
			// When scrolling we had to break the table up - restore it
			if ( table != thead.parentNode ) {
				jqTable.children('thead').detach();
				jqTable.append( thead );
			}
	
			if ( tfoot && table != tfoot.parentNode ) {
				jqTable.children('tfoot').detach();
				jqTable.append( tfoot );
			}
	
			// Remove the DataTables generated nodes, events and classes
			jqTable.detach();
			jqWrapper.detach();
	
			settings.aaSorting = [];
			settings.aaSortingFixed = [];
			_fnSortingClasses( settings );
	
			$( rows ).removeClass( settings.asStripeClasses.join(' ') );
	
			$('th, td', thead).removeClass( classes.sSortable+' '+
				classes.sSortableAsc+' '+classes.sSortableDesc+' '+classes.sSortableNone
			);
	
			if ( settings.bJUI ) {
				$('th span.'+classes.sSortIcon+ ', td span.'+classes.sSortIcon, thead).detach();
				$('th, td', thead).each( function () {
					var wrapper = $('div.'+classes.sSortJUIWrapper, this);
					$(this).append( wrapper.contents() );
					wrapper.detach();
				} );
			}
	
			if ( ! remove && orig ) {
				// insertBefore acts like appendChild if !arg[1]
				orig.insertBefore( table, settings.nTableReinsertBefore );
			}
	
			// Add the TR elements back into the table in their original order
			jqTbody.children().detach();
			jqTbody.append( rows );
	
			// Restore the width of the original table - was read from the style property,
			// so we can restore directly to that
			jqTable
				.css( 'width', settings.sDestroyWidth )
				.removeClass( classes.sTable );
	
			// If the were originally stripe classes - then we add them back here.
			// Note this is not fool proof (for example if not all rows had stripe
			// classes - but it's a good effort without getting carried away
			ien = settings.asDestroyStripes.length;
	
			if ( ien ) {
				jqTbody.children().each( function (i) {
					$(this).addClass( settings.asDestroyStripes[i % ien] );
				} );
			}
	
			/* Remove the settings object from the settings array */
			var idx = $.inArray( settings, DataTable.settings );
			if ( idx !== -1 ) {
				DataTable.settings.splice( idx, 1 );
			}
		} );
	} );
	
	
	// Add the `every()` method for rows, columns and cells in a compact form
	$.each( [ 'column', 'row', 'cell' ], function ( i, type ) {
		_api_register( type+'s().every()', function ( fn ) {
			return this.iterator( type, function ( settings, idx, idx2 ) {
				// idx2 is undefined for rows and columns.
				fn.call( new _Api( settings )[ type ]( idx, idx2 ) );
			} );
		} );
	} );
	
	
	// i18n method for extensions to be able to use the language object from the
	// DataTable
	_api_register( 'i18n()', function ( token, def, plural ) {
		var ctx = this.context[0];
		var resolved = _fnGetObjectDataFn( token )( ctx.oLanguage );
	
		if ( resolved === undefined ) {
			resolved = def;
		}
	
		if ( plural !== undefined && $.isPlainObject( resolved ) ) {
			resolved = resolved[ plural ] !== undefined ?
				resolved[ plural ] :
				resolved._;
		}
	
		return resolved.replace( '%d', plural ); // nb: plural might be undefined,
	} );

	/**
	 * Version string for plug-ins to check compatibility. Allowed format is
	 * `a.b.c-d` where: a:int, b:int, c:int, d:string(dev|beta|alpha). `d` is used
	 * only for non-release builds. See http://semver.org/ for more information.
	 *  @member
	 *  @type string
	 *  @default Version number
	 */
	DataTable.version = "1.10.7";

	/**
	 * Private data store, containing all of the settings objects that are
	 * created for the tables on a given page.
	 *
	 * Note that the `DataTable.settings` object is aliased to
	 * `jQuery.fn.dataTableExt` through which it may be accessed and
	 * manipulated, or `jQuery.fn.dataTable.settings`.
	 *  @member
	 *  @type array
	 *  @default []
	 *  @private
	 */
	DataTable.settings = [];

	/**
	 * Object models container, for the various models that DataTables has
	 * available to it. These models define the objects that are used to hold
	 * the active state and configuration of the table.
	 *  @namespace
	 */
	DataTable.models = {};
	
	
	
	/**
	 * Template object for the way in which DataTables holds information about
	 * search information for the global filter and individual column filters.
	 *  @namespace
	 */
	DataTable.models.oSearch = {
		/**
		 * Flag to indicate if the filtering should be case insensitive or not
		 *  @type boolean
		 *  @default true
		 */
		"bCaseInsensitive": true,
	
		/**
		 * Applied search term
		 *  @type string
		 *  @default <i>Empty string</i>
		 */
		"sSearch": "",
	
		/**
		 * Flag to indicate if the search term should be interpreted as a
		 * regular expression (true) or not (false) and therefore and special
		 * regex characters escaped.
		 *  @type boolean
		 *  @default false
		 */
		"bRegex": false,
	
		/**
		 * Flag to indicate if DataTables is to use its smart filtering or not.
		 *  @type boolean
		 *  @default true
		 */
		"bSmart": true
	};
	
	
	
	
	/**
	 * Template object for the way in which DataTables holds information about
	 * each individual row. This is the object format used for the settings
	 * aoData array.
	 *  @namespace
	 */
	DataTable.models.oRow = {
		/**
		 * TR element for the row
		 *  @type node
		 *  @default null
		 */
		"nTr": null,
	
		/**
		 * Array of TD elements for each row. This is null until the row has been
		 * created.
		 *  @type array nodes
		 *  @default []
		 */
		"anCells": null,
	
		/**
		 * Data object from the original data source for the row. This is either
		 * an array if using the traditional form of DataTables, or an object if
		 * using mData options. The exact type will depend on the passed in
		 * data from the data source, or will be an array if using DOM a data
		 * source.
		 *  @type array|object
		 *  @default []
		 */
		"_aData": [],
	
		/**
		 * Sorting data cache - this array is ostensibly the same length as the
		 * number of columns (although each index is generated only as it is
		 * needed), and holds the data that is used for sorting each column in the
		 * row. We do this cache generation at the start of the sort in order that
		 * the formatting of the sort data need be done only once for each cell
		 * per sort. This array should not be read from or written to by anything
		 * other than the master sorting methods.
		 *  @type array
		 *  @default null
		 *  @private
		 */
		"_aSortData": null,
	
		/**
		 * Per cell filtering data cache. As per the sort data cache, used to
		 * increase the performance of the filtering in DataTables
		 *  @type array
		 *  @default null
		 *  @private
		 */
		"_aFilterData": null,
	
		/**
		 * Filtering data cache. This is the same as the cell filtering cache, but
		 * in this case a string rather than an array. This is easily computed with
		 * a join on `_aFilterData`, but is provided as a cache so the join isn't
		 * needed on every search (memory traded for performance)
		 *  @type array
		 *  @default null
		 *  @private
		 */
		"_sFilterRow": null,
	
		/**
		 * Cache of the class name that DataTables has applied to the row, so we
		 * can quickly look at this variable rather than needing to do a DOM check
		 * on className for the nTr property.
		 *  @type string
		 *  @default <i>Empty string</i>
		 *  @private
		 */
		"_sRowStripe": "",
	
		/**
		 * Denote if the original data source was from the DOM, or the data source
		 * object. This is used for invalidating data, so DataTables can
		 * automatically read data from the original source, unless uninstructed
		 * otherwise.
		 *  @type string
		 *  @default null
		 *  @private
		 */
		"src": null
	};
	
	
	/**
	 * Template object for the column information object in DataTables. This object
	 * is held in the settings aoColumns array and contains all the information that
	 * DataTables needs about each individual column.
	 *
	 * Note that this object is related to {@link DataTable.defaults.column}
	 * but this one is the internal data store for DataTables's cache of columns.
	 * It should NOT be manipulated outside of DataTables. Any configuration should
	 * be done through the initialisation options.
	 *  @namespace
	 */
	DataTable.models.oColumn = {
		/**
		 * Column index. This could be worked out on-the-fly with $.inArray, but it
		 * is faster to just hold it as a variable
		 *  @type integer
		 *  @default null
		 */
		"idx": null,
	
		/**
		 * A list of the columns that sorting should occur on when this column
		 * is sorted. That this property is an array allows multi-column sorting
		 * to be defined for a column (for example first name / last name columns
		 * would benefit from this). The values are integers pointing to the
		 * columns to be sorted on (typically it will be a single integer pointing
		 * at itself, but that doesn't need to be the case).
		 *  @type array
		 */
		"aDataSort": null,
	
		/**
		 * Define the sorting directions that are applied to the column, in sequence
		 * as the column is repeatedly sorted upon - i.e. the first value is used
		 * as the sorting direction when the column if first sorted (clicked on).
		 * Sort it again (click again) and it will move on to the next index.
		 * Repeat until loop.
		 *  @type array
		 */
		"asSorting": null,
	
		/**
		 * Flag to indicate if the column is searchable, and thus should be included
		 * in the filtering or not.
		 *  @type boolean
		 */
		"bSearchable": null,
	
		/**
		 * Flag to indicate if the column is sortable or not.
		 *  @type boolean
		 */
		"bSortable": null,
	
		/**
		 * Flag to indicate if the column is currently visible in the table or not
		 *  @type boolean
		 */
		"bVisible": null,
	
		/**
		 * Store for manual type assignment using the `column.type` option. This
		 * is held in store so we can manipulate the column's `sType` property.
		 *  @type string
		 *  @default null
		 *  @private
		 */
		"_sManualType": null,
	
		/**
		 * Flag to indicate if HTML5 data attributes should be used as the data
		 * source for filtering or sorting. True is either are.
		 *  @type boolean
		 *  @default false
		 *  @private
		 */
		"_bAttrSrc": false,
	
		/**
		 * Developer definable function that is called whenever a cell is created (Ajax source,
		 * etc) or processed for input (DOM source). This can be used as a compliment to mRender
		 * allowing you to modify the DOM element (add background colour for example) when the
		 * element is available.
		 *  @type function
		 *  @param {element} nTd The TD node that has been created
		 *  @param {*} sData The Data for the cell
		 *  @param {array|object} oData The data for the whole row
		 *  @param {int} iRow The row index for the aoData data store
		 *  @default null
		 */
		"fnCreatedCell": null,
	
		/**
		 * Function to get data from a cell in a column. You should <b>never</b>
		 * access data directly through _aData internally in DataTables - always use
		 * the method attached to this property. It allows mData to function as
		 * required. This function is automatically assigned by the column
		 * initialisation method
		 *  @type function
		 *  @param {array|object} oData The data array/object for the array
		 *    (i.e. aoData[]._aData)
		 *  @param {string} sSpecific The specific data type you want to get -
		 *    'display', 'type' 'filter' 'sort'
		 *  @returns {*} The data for the cell from the given row's data
		 *  @default null
		 */
		"fnGetData": null,
	
		/**
		 * Function to set data for a cell in the column. You should <b>never</b>
		 * set the data directly to _aData internally in DataTables - always use
		 * this method. It allows mData to function as required. This function
		 * is automatically assigned by the column initialisation method
		 *  @type function
		 *  @param {array|object} oData The data array/object for the array
		 *    (i.e. aoData[]._aData)
		 *  @param {*} sValue Value to set
		 *  @default null
		 */
		"fnSetData": null,
	
		/**
		 * Property to read the value for the cells in the column from the data
		 * source array / object. If null, then the default content is used, if a
		 * function is given then the return from the function is used.
		 *  @type function|int|string|null
		 *  @default null
		 */
		"mData": null,
	
		/**
		 * Partner property to mData which is used (only when defined) to get
		 * the data - i.e. it is basically the same as mData, but without the
		 * 'set' option, and also the data fed to it is the result from mData.
		 * This is the rendering method to match the data method of mData.
		 *  @type function|int|string|null
		 *  @default null
		 */
		"mRender": null,
	
		/**
		 * Unique header TH/TD element for this column - this is what the sorting
		 * listener is attached to (if sorting is enabled.)
		 *  @type node
		 *  @default null
		 */
		"nTh": null,
	
		/**
		 * Unique footer TH/TD element for this column (if there is one). Not used
		 * in DataTables as such, but can be used for plug-ins to reference the
		 * footer for each column.
		 *  @type node
		 *  @default null
		 */
		"nTf": null,
	
		/**
		 * The class to apply to all TD elements in the table's TBODY for the column
		 *  @type string
		 *  @default null
		 */
		"sClass": null,
	
		/**
		 * When DataTables calculates the column widths to assign to each column,
		 * it finds the longest string in each column and then constructs a
		 * temporary table and reads the widths from that. The problem with this
		 * is that "mmm" is much wider then "iiii", but the latter is a longer
		 * string - thus the calculation can go wrong (doing it properly and putting
		 * it into an DOM object and measuring that is horribly(!) slow). Thus as
		 * a "work around" we provide this option. It will append its value to the
		 * text that is found to be the longest string for the column - i.e. padding.
		 *  @type string
		 */
		"sContentPadding": null,
	
		/**
		 * Allows a default value to be given for a column's data, and will be used
		 * whenever a null data source is encountered (this can be because mData
		 * is set to null, or because the data source itself is null).
		 *  @type string
		 *  @default null
		 */
		"sDefaultContent": null,
	
		/**
		 * Name for the column, allowing reference to the column by name as well as
		 * by index (needs a lookup to work by name).
		 *  @type string
		 */
		"sName": null,
	
		/**
		 * Custom sorting data type - defines which of the available plug-ins in
		 * afnSortData the custom sorting will use - if any is defined.
		 *  @type string
		 *  @default std
		 */
		"sSortDataType": 'std',
	
		/**
		 * Class to be applied to the header element when sorting on this column
		 *  @type string
		 *  @default null
		 */
		"sSortingClass": null,
	
		/**
		 * Class to be applied to the header element when sorting on this column -
		 * when jQuery UI theming is used.
		 *  @type string
		 *  @default null
		 */
		"sSortingClassJUI": null,
	
		/**
		 * Title of the column - what is seen in the TH element (nTh).
		 *  @type string
		 */
		"sTitle": null,
	
		/**
		 * Column sorting and filtering type
		 *  @type string
		 *  @default null
		 */
		"sType": null,
	
		/**
		 * Width of the column
		 *  @type string
		 *  @default null
		 */
		"sWidth": null,
	
		/**
		 * Width of the column when it was first "encountered"
		 *  @type string
		 *  @default null
		 */
		"sWidthOrig": null
	};
	
	
	/*
	 * Developer note: The properties of the object below are given in Hungarian
	 * notation, that was used as the interface for DataTables prior to v1.10, however
	 * from v1.10 onwards the primary interface is camel case. In order to avoid
	 * breaking backwards compatibility utterly with this change, the Hungarian
	 * version is still, internally the primary interface, but is is not documented
	 * - hence the @name tags in each doc comment. This allows a Javascript function
	 * to create a map from Hungarian notation to camel case (going the other direction
	 * would require each property to be listed, which would at around 3K to the size
	 * of DataTables, while this method is about a 0.5K hit.
	 *
	 * Ultimately this does pave the way for Hungarian notation to be dropped
	 * completely, but that is a massive amount of work and will break current
	 * installs (therefore is on-hold until v2).
	 */
	
	/**
	 * Initialisation options that can be given to DataTables at initialisation
	 * time.
	 *  @namespace
	 */
	DataTable.defaults = {
		/**
		 * An array of data to use for the table, passed in at initialisation which
		 * will be used in preference to any data which is already in the DOM. This is
		 * particularly useful for constructing tables purely in Javascript, for
		 * example with a custom Ajax call.
		 *  @type array
		 *  @default null
		 *
		 *  @dtopt Option
		 *  @name DataTable.defaults.data
		 *
		 *  @example
		 *    // Using a 2D array data source
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "data": [
		 *          ['Trident', 'Internet Explorer 4.0', 'Win 95+', 4, 'X'],
		 *          ['Trident', 'Internet Explorer 5.0', 'Win 95+', 5, 'C'],
		 *        ],
		 *        "columns": [
		 *          { "title": "Engine" },
		 *          { "title": "Browser" },
		 *          { "title": "Platform" },
		 *          { "title": "Version" },
		 *          { "title": "Grade" }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using an array of objects as a data source (`data`)
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "data": [
		 *          {
		 *            "engine":   "Trident",
		 *            "browser":  "Internet Explorer 4.0",
		 *            "platform": "Win 95+",
		 *            "version":  4,
		 *            "grade":    "X"
		 *          },
		 *          {
		 *            "engine":   "Trident",
		 *            "browser":  "Internet Explorer 5.0",
		 *            "platform": "Win 95+",
		 *            "version":  5,
		 *            "grade":    "C"
		 *          }
		 *        ],
		 *        "columns": [
		 *          { "title": "Engine",   "data": "engine" },
		 *          { "title": "Browser",  "data": "browser" },
		 *          { "title": "Platform", "data": "platform" },
		 *          { "title": "Version",  "data": "version" },
		 *          { "title": "Grade",    "data": "grade" }
		 *        ]
		 *      } );
		 *    } );
		 */
		"aaData": null,
	
	
		/**
		 * If ordering is enabled, then DataTables will perform a first pass sort on
		 * initialisation. You can define which column(s) the sort is performed
		 * upon, and the sorting direction, with this variable. The `sorting` array
		 * should contain an array for each column to be sorted initially containing
		 * the column's index and a direction string ('asc' or 'desc').
		 *  @type array
		 *  @default [[0,'asc']]
		 *
		 *  @dtopt Option
		 *  @name DataTable.defaults.order
		 *
		 *  @example
		 *    // Sort by 3rd column first, and then 4th column
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "order": [[2,'asc'], [3,'desc']]
		 *      } );
		 *    } );
		 *
		 *    // No initial sorting
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "order": []
		 *      } );
		 *    } );
		 */
		"aaSorting": [[0,'asc']],
	
	
		/**
		 * This parameter is basically identical to the `sorting` parameter, but
		 * cannot be overridden by user interaction with the table. What this means
		 * is that you could have a column (visible or hidden) which the sorting
		 * will always be forced on first - any sorting after that (from the user)
		 * will then be performed as required. This can be useful for grouping rows
		 * together.
		 *  @type array
		 *  @default null
		 *
		 *  @dtopt Option
		 *  @name DataTable.defaults.orderFixed
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "orderFixed": [[0,'asc']]
		 *      } );
		 *    } )
		 */
		"aaSortingFixed": [],
	
	
		/**
		 * DataTables can be instructed to load data to display in the table from a
		 * Ajax source. This option defines how that Ajax call is made and where to.
		 *
		 * The `ajax` property has three different modes of operation, depending on
		 * how it is defined. These are:
		 *
		 * * `string` - Set the URL from where the data should be loaded from.
		 * * `object` - Define properties for `jQuery.ajax`.
		 * * `function` - Custom data get function
		 *
		 * `string`
		 * --------
		 *
		 * As a string, the `ajax` property simply defines the URL from which
		 * DataTables will load data.
		 *
		 * `object`
		 * --------
		 *
		 * As an object, the parameters in the object are passed to
		 * [jQuery.ajax](http://api.jquery.com/jQuery.ajax/) allowing fine control
		 * of the Ajax request. DataTables has a number of default parameters which
		 * you can override using this option. Please refer to the jQuery
		 * documentation for a full description of the options available, although
		 * the following parameters provide additional options in DataTables or
		 * require special consideration:
		 *
		 * * `data` - As with jQuery, `data` can be provided as an object, but it
		 *   can also be used as a function to manipulate the data DataTables sends
		 *   to the server. The function takes a single parameter, an object of
		 *   parameters with the values that DataTables has readied for sending. An
		 *   object may be returned which will be merged into the DataTables
		 *   defaults, or you can add the items to the object that was passed in and
		 *   not return anything from the function. This supersedes `fnServerParams`
		 *   from DataTables 1.9-.
		 *
		 * * `dataSrc` - By default DataTables will look for the property `data` (or
		 *   `aaData` for compatibility with DataTables 1.9-) when obtaining data
		 *   from an Ajax source or for server-side processing - this parameter
		 *   allows that property to be changed. You can use Javascript dotted
		 *   object notation to get a data source for multiple levels of nesting, or
		 *   it my be used as a function. As a function it takes a single parameter,
		 *   the JSON returned from the server, which can be manipulated as
		 *   required, with the returned value being that used by DataTables as the
		 *   data source for the table. This supersedes `sAjaxDataProp` from
		 *   DataTables 1.9-.
		 *
		 * * `success` - Should not be overridden it is used internally in
		 *   DataTables. To manipulate / transform the data returned by the server
		 *   use `ajax.dataSrc`, or use `ajax` as a function (see below).
		 *
		 * `function`
		 * ----------
		 *
		 * As a function, making the Ajax call is left up to yourself allowing
		 * complete control of the Ajax request. Indeed, if desired, a method other
		 * than Ajax could be used to obtain the required data, such as Web storage
		 * or an AIR database.
		 *
		 * The function is given four parameters and no return is required. The
		 * parameters are:
		 *
		 * 1. _object_ - Data to send to the server
		 * 2. _function_ - Callback function that must be executed when the required
		 *    data has been obtained. That data should be passed into the callback
		 *    as the only parameter
		 * 3. _object_ - DataTables settings object for the table
		 *
		 * Note that this supersedes `fnServerData` from DataTables 1.9-.
		 *
		 *  @type string|object|function
		 *  @default null
		 *
		 *  @dtopt Option
		 *  @name DataTable.defaults.ajax
		 *  @since 1.10.0
		 *
		 * @example
		 *   // Get JSON data from a file via Ajax.
		 *   // Note DataTables expects data in the form `{ data: [ ...data... ] }` by default).
		 *   $('#example').dataTable( {
		 *     "ajax": "data.json"
		 *   } );
		 *
		 * @example
		 *   // Get JSON data from a file via Ajax, using `dataSrc` to change
		 *   // `data` to `tableData` (i.e. `{ tableData: [ ...data... ] }`)
		 *   $('#example').dataTable( {
		 *     "ajax": {
		 *       "url": "data.json",
		 *       "dataSrc": "tableData"
		 *     }
		 *   } );
		 *
		 * @example
		 *   // Get JSON data from a file via Ajax, using `dataSrc` to read data
		 *   // from a plain array rather than an array in an object
		 *   $('#example').dataTable( {
		 *     "ajax": {
		 *       "url": "data.json",
		 *       "dataSrc": ""
		 *     }
		 *   } );
		 *
		 * @example
		 *   // Manipulate the data returned from the server - add a link to data
		 *   // (note this can, should, be done using `render` for the column - this
		 *   // is just a simple example of how the data can be manipulated).
		 *   $('#example').dataTable( {
		 *     "ajax": {
		 *       "url": "data.json",
		 *       "dataSrc": function ( json ) {
		 *         for ( var i=0, ien=json.length ; i<ien ; i++ ) {
		 *           json[i][0] = '<a href="/message/'+json[i][0]+'>View message</a>';
		 *         }
		 *         return json;
		 *       }
		 *     }
		 *   } );
		 *
		 * @example
		 *   // Add data to the request
		 *   $('#example').dataTable( {
		 *     "ajax": {
		 *       "url": "data.json",
		 *       "data": function ( d ) {
		 *         return {
		 *           "extra_search": $('#extra').val()
		 *         };
		 *       }
		 *     }
		 *   } );
		 *
		 * @example
		 *   // Send request as POST
		 *   $('#example').dataTable( {
		 *     "ajax": {
		 *       "url": "data.json",
		 *       "type": "POST"
		 *     }
		 *   } );
		 *
		 * @example
		 *   // Get the data from localStorage (could interface with a form for
		 *   // adding, editing and removing rows).
		 *   $('#example').dataTable( {
		 *     "ajax": function (data, callback, settings) {
		 *       callback(
		 *         JSON.parse( localStorage.getItem('dataTablesData') )
		 *       );
		 *     }
		 *   } );
		 */
		"ajax": null,
	
	
		/**
		 * This parameter allows you to readily specify the entries in the length drop
		 * down menu that DataTables shows when pagination is enabled. It can be
		 * either a 1D array of options which will be used for both the displayed
		 * option and the value, or a 2D array which will use the array in the first
		 * position as the value, and the array in the second position as the
		 * displayed options (useful for language strings such as 'All').
		 *
		 * Note that the `pageLength` property will be automatically set to the
		 * first value given in this array, unless `pageLength` is also provided.
		 *  @type array
		 *  @default [ 10, 25, 50, 100 ]
		 *
		 *  @dtopt Option
		 *  @name DataTable.defaults.lengthMenu
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "lengthMenu": [[10, 25, 50, -1], [10, 25, 50, "All"]]
		 *      } );
		 *    } );
		 */
		"aLengthMenu": [ 10, 25, 50, 100 ],
	
	
		/**
		 * The `columns` option in the initialisation parameter allows you to define
		 * details about the way individual columns behave. For a full list of
		 * column options that can be set, please see
		 * {@link DataTable.defaults.column}. Note that if you use `columns` to
		 * define your columns, you must have an entry in the array for every single
		 * column that you have in your table (these can be null if you don't which
		 * to specify any options).
		 *  @member
		 *
		 *  @name DataTable.defaults.column
		 */
		"aoColumns": null,
	
		/**
		 * Very similar to `columns`, `columnDefs` allows you to target a specific
		 * column, multiple columns, or all columns, using the `targets` property of
		 * each object in the array. This allows great flexibility when creating
		 * tables, as the `columnDefs` arrays can be of any length, targeting the
		 * columns you specifically want. `columnDefs` may use any of the column
		 * options available: {@link DataTable.defaults.column}, but it _must_
		 * have `targets` defined in each object in the array. Values in the `targets`
		 * array may be:
		 *   <ul>
		 *     <li>a string - class name will be matched on the TH for the column</li>
		 *     <li>0 or a positive integer - column index counting from the left</li>
		 *     <li>a negative integer - column index counting from the right</li>
		 *     <li>the string "_all" - all columns (i.e. assign a default)</li>
		 *   </ul>
		 *  @member
		 *
		 *  @name DataTable.defaults.columnDefs
		 */
		"aoColumnDefs": null,
	
	
		/**
		 * Basically the same as `search`, this parameter defines the individual column
		 * filtering state at initialisation time. The array must be of the same size
		 * as the number of columns, and each element be an object with the parameters
		 * `search` and `escapeRegex` (the latter is optional). 'null' is also
		 * accepted and the default will be used.
		 *  @type array
		 *  @default []
		 *
		 *  @dtopt Option
		 *  @name DataTable.defaults.searchCols
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "searchCols": [
		 *          null,
		 *          { "search": "My filter" },
		 *          null,
		 *          { "search": "^[0-9]", "escapeRegex": false }
		 *        ]
		 *      } );
		 *    } )
		 */
		"aoSearchCols": [],
	
	
		/**
		 * An array of CSS classes that should be applied to displayed rows. This
		 * array may be of any length, and DataTables will apply each class
		 * sequentially, looping when required.
		 *  @type array
		 *  @default null <i>Will take the values determined by the `oClasses.stripe*`
		 *    options</i>
		 *
		 *  @dtopt Option
		 *  @name DataTable.defaults.stripeClasses
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "stripeClasses": [ 'strip1', 'strip2', 'strip3' ]
		 *      } );
		 *    } )
		 */
		"asStripeClasses": null,
	
	
		/**
		 * Enable or disable automatic column width calculation. This can be disabled
		 * as an optimisation (it takes some time to calculate the widths) if the
		 * tables widths are passed in using `columns`.
		 *  @type boolean
		 *  @default true
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.autoWidth
		 *
		 *  @example
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "autoWidth": false
		 *      } );
		 *    } );
		 */
		"bAutoWidth": true,
	
	
		/**
		 * Deferred rendering can provide DataTables with a huge speed boost when you
		 * are using an Ajax or JS data source for the table. This option, when set to
		 * true, will cause DataTables to defer the creation of the table elements for
		 * each row until they are needed for a draw - saving a significant amount of
		 * time.
		 *  @type boolean
		 *  @default false
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.deferRender
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "ajax": "sources/arrays.txt",
		 *        "deferRender": true
		 *      } );
		 *    } );
		 */
		"bDeferRender": false,
	
	
		/**
		 * Replace a DataTable which matches the given selector and replace it with
		 * one which has the properties of the new initialisation object passed. If no
		 * table matches the selector, then the new DataTable will be constructed as
		 * per normal.
		 *  @type boolean
		 *  @default false
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.destroy
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "srollY": "200px",
		 *        "paginate": false
		 *      } );
		 *
		 *      // Some time later....
		 *      $('#example').dataTable( {
		 *        "filter": false,
		 *        "destroy": true
		 *      } );
		 *    } );
		 */
		"bDestroy": false,
	
	
		/**
		 * Enable or disable filtering of data. Filtering in DataTables is "smart" in
		 * that it allows the end user to input multiple words (space separated) and
		 * will match a row containing those words, even if not in the order that was
		 * specified (this allow matching across multiple columns). Note that if you
		 * wish to use filtering in DataTables this must remain 'true' - to remove the
		 * default filtering input box and retain filtering abilities, please use
		 * {@link DataTable.defaults.dom}.
		 *  @type boolean
		 *  @default true
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.searching
		 *
		 *  @example
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "searching": false
		 *      } );
		 *    } );
		 */
		"bFilter": true,
	
	
		/**
		 * Enable or disable the table information display. This shows information
		 * about the data that is currently visible on the page, including information
		 * about filtered data if that action is being performed.
		 *  @type boolean
		 *  @default true
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.info
		 *
		 *  @example
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "info": false
		 *      } );
		 *    } );
		 */
		"bInfo": true,
	
	
		/**
		 * Enable jQuery UI ThemeRoller support (required as ThemeRoller requires some
		 * slightly different and additional mark-up from what DataTables has
		 * traditionally used).
		 *  @type boolean
		 *  @default false
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.jQueryUI
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "jQueryUI": true
		 *      } );
		 *    } );
		 */
		"bJQueryUI": false,
	
	
		/**
		 * Allows the end user to select the size of a formatted page from a select
		 * menu (sizes are 10, 25, 50 and 100). Requires pagination (`paginate`).
		 *  @type boolean
		 *  @default true
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.lengthChange
		 *
		 *  @example
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "lengthChange": false
		 *      } );
		 *    } );
		 */
		"bLengthChange": true,
	
	
		/**
		 * Enable or disable pagination.
		 *  @type boolean
		 *  @default true
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.paging
		 *
		 *  @example
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "paging": false
		 *      } );
		 *    } );
		 */
		"bPaginate": true,
	
	
		/**
		 * Enable or disable the display of a 'processing' indicator when the table is
		 * being processed (e.g. a sort). This is particularly useful for tables with
		 * large amounts of data where it can take a noticeable amount of time to sort
		 * the entries.
		 *  @type boolean
		 *  @default false
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.processing
		 *
		 *  @example
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "processing": true
		 *      } );
		 *    } );
		 */
		"bProcessing": false,
	
	
		/**
		 * Retrieve the DataTables object for the given selector. Note that if the
		 * table has already been initialised, this parameter will cause DataTables
		 * to simply return the object that has already been set up - it will not take
		 * account of any changes you might have made to the initialisation object
		 * passed to DataTables (setting this parameter to true is an acknowledgement
		 * that you understand this). `destroy` can be used to reinitialise a table if
		 * you need.
		 *  @type boolean
		 *  @default false
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.retrieve
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      initTable();
		 *      tableActions();
		 *    } );
		 *
		 *    function initTable ()
		 *    {
		 *      return $('#example').dataTable( {
		 *        "scrollY": "200px",
		 *        "paginate": false,
		 *        "retrieve": true
		 *      } );
		 *    }
		 *
		 *    function tableActions ()
		 *    {
		 *      var table = initTable();
		 *      // perform API operations with oTable
		 *    }
		 */
		"bRetrieve": false,
	
	
		/**
		 * When vertical (y) scrolling is enabled, DataTables will force the height of
		 * the table's viewport to the given height at all times (useful for layout).
		 * However, this can look odd when filtering data down to a small data set,
		 * and the footer is left "floating" further down. This parameter (when
		 * enabled) will cause DataTables to collapse the table's viewport down when
		 * the result set will fit within the given Y height.
		 *  @type boolean
		 *  @default false
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.scrollCollapse
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "scrollY": "200",
		 *        "scrollCollapse": true
		 *      } );
		 *    } );
		 */
		"bScrollCollapse": false,
	
	
		/**
		 * Configure DataTables to use server-side processing. Note that the
		 * `ajax` parameter must also be given in order to give DataTables a
		 * source to obtain the required data for each draw.
		 *  @type boolean
		 *  @default false
		 *
		 *  @dtopt Features
		 *  @dtopt Server-side
		 *  @name DataTable.defaults.serverSide
		 *
		 *  @example
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "serverSide": true,
		 *        "ajax": "xhr.php"
		 *      } );
		 *    } );
		 */
		"bServerSide": false,
	
	
		/**
		 * Enable or disable sorting of columns. Sorting of individual columns can be
		 * disabled by the `sortable` option for each column.
		 *  @type boolean
		 *  @default true
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.ordering
		 *
		 *  @example
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "ordering": false
		 *      } );
		 *    } );
		 */
		"bSort": true,
	
	
		/**
		 * Enable or display DataTables' ability to sort multiple columns at the
		 * same time (activated by shift-click by the user).
		 *  @type boolean
		 *  @default true
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.orderMulti
		 *
		 *  @example
		 *    // Disable multiple column sorting ability
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "orderMulti": false
		 *      } );
		 *    } );
		 */
		"bSortMulti": true,
	
	
		/**
		 * Allows control over whether DataTables should use the top (true) unique
		 * cell that is found for a single column, or the bottom (false - default).
		 * This is useful when using complex headers.
		 *  @type boolean
		 *  @default false
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.orderCellsTop
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "orderCellsTop": true
		 *      } );
		 *    } );
		 */
		"bSortCellsTop": false,
	
	
		/**
		 * Enable or disable the addition of the classes `sorting\_1`, `sorting\_2` and
		 * `sorting\_3` to the columns which are currently being sorted on. This is
		 * presented as a feature switch as it can increase processing time (while
		 * classes are removed and added) so for large data sets you might want to
		 * turn this off.
		 *  @type boolean
		 *  @default true
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.orderClasses
		 *
		 *  @example
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "orderClasses": false
		 *      } );
		 *    } );
		 */
		"bSortClasses": true,
	
	
		/**
		 * Enable or disable state saving. When enabled HTML5 `localStorage` will be
		 * used to save table display information such as pagination information,
		 * display length, filtering and sorting. As such when the end user reloads
		 * the page the display display will match what thy had previously set up.
		 *
		 * Due to the use of `localStorage` the default state saving is not supported
		 * in IE6 or 7. If state saving is required in those browsers, use
		 * `stateSaveCallback` to provide a storage solution such as cookies.
		 *  @type boolean
		 *  @default false
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.stateSave
		 *
		 *  @example
		 *    $(document).ready( function () {
		 *      $('#example').dataTable( {
		 *        "stateSave": true
		 *      } );
		 *    } );
		 */
		"bStateSave": false,
	
	
		/**
		 * This function is called when a TR element is created (and all TD child
		 * elements have been inserted), or registered if using a DOM source, allowing
		 * manipulation of the TR element (adding classes etc).
		 *  @type function
		 *  @param {node} row "TR" element for the current row
		 *  @param {array} data Raw data array for this row
		 *  @param {int} dataIndex The index of this row in the internal aoData array
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.createdRow
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "createdRow": function( row, data, dataIndex ) {
		 *          // Bold the grade for all 'A' grade browsers
		 *          if ( data[4] == "A" )
		 *          {
		 *            $('td:eq(4)', row).html( '<b>A</b>' );
		 *          }
		 *        }
		 *      } );
		 *    } );
		 */
		"fnCreatedRow": null,
	
	
		/**
		 * This function is called on every 'draw' event, and allows you to
		 * dynamically modify any aspect you want about the created DOM.
		 *  @type function
		 *  @param {object} settings DataTables settings object
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.drawCallback
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "drawCallback": function( settings ) {
		 *          alert( 'DataTables has redrawn the table' );
		 *        }
		 *      } );
		 *    } );
		 */
		"fnDrawCallback": null,
	
	
		/**
		 * Identical to fnHeaderCallback() but for the table footer this function
		 * allows you to modify the table footer on every 'draw' event.
		 *  @type function
		 *  @param {node} foot "TR" element for the footer
		 *  @param {array} data Full table data (as derived from the original HTML)
		 *  @param {int} start Index for the current display starting point in the
		 *    display array
		 *  @param {int} end Index for the current display ending point in the
		 *    display array
		 *  @param {array int} display Index array to translate the visual position
		 *    to the full data array
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.footerCallback
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "footerCallback": function( tfoot, data, start, end, display ) {
		 *          tfoot.getElementsByTagName('th')[0].innerHTML = "Starting index is "+start;
		 *        }
		 *      } );
		 *    } )
		 */
		"fnFooterCallback": null,
	
	
		/**
		 * When rendering large numbers in the information element for the table
		 * (i.e. "Showing 1 to 10 of 57 entries") DataTables will render large numbers
		 * to have a comma separator for the 'thousands' units (e.g. 1 million is
		 * rendered as "1,000,000") to help readability for the end user. This
		 * function will override the default method DataTables uses.
		 *  @type function
		 *  @member
		 *  @param {int} toFormat number to be formatted
		 *  @returns {string} formatted string for DataTables to show the number
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.formatNumber
		 *
		 *  @example
		 *    // Format a number using a single quote for the separator (note that
		 *    // this can also be done with the language.thousands option)
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "formatNumber": function ( toFormat ) {
		 *          return toFormat.toString().replace(
		 *            /\B(?=(\d{3})+(?!\d))/g, "'"
		 *          );
		 *        };
		 *      } );
		 *    } );
		 */
		"fnFormatNumber": function ( toFormat ) {
			return toFormat.toString().replace(
				/\B(?=(\d{3})+(?!\d))/g,
				this.oLanguage.sThousands
			);
		},
	
	
		/**
		 * This function is called on every 'draw' event, and allows you to
		 * dynamically modify the header row. This can be used to calculate and
		 * display useful information about the table.
		 *  @type function
		 *  @param {node} head "TR" element for the header
		 *  @param {array} data Full table data (as derived from the original HTML)
		 *  @param {int} start Index for the current display starting point in the
		 *    display array
		 *  @param {int} end Index for the current display ending point in the
		 *    display array
		 *  @param {array int} display Index array to translate the visual position
		 *    to the full data array
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.headerCallback
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "fheaderCallback": function( head, data, start, end, display ) {
		 *          head.getElementsByTagName('th')[0].innerHTML = "Displaying "+(end-start)+" records";
		 *        }
		 *      } );
		 *    } )
		 */
		"fnHeaderCallback": null,
	
	
		/**
		 * The information element can be used to convey information about the current
		 * state of the table. Although the internationalisation options presented by
		 * DataTables are quite capable of dealing with most customisations, there may
		 * be times where you wish to customise the string further. This callback
		 * allows you to do exactly that.
		 *  @type function
		 *  @param {object} oSettings DataTables settings object
		 *  @param {int} start Starting position in data for the draw
		 *  @param {int} end End position in data for the draw
		 *  @param {int} max Total number of rows in the table (regardless of
		 *    filtering)
		 *  @param {int} total Total number of rows in the data set, after filtering
		 *  @param {string} pre The string that DataTables has formatted using it's
		 *    own rules
		 *  @returns {string} The string to be displayed in the information element.
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.infoCallback
		 *
		 *  @example
		 *    $('#example').dataTable( {
		 *      "infoCallback": function( settings, start, end, max, total, pre ) {
		 *        return start +" to "+ end;
		 *      }
		 *    } );
		 */
		"fnInfoCallback": null,
	
	
		/**
		 * Called when the table has been initialised. Normally DataTables will
		 * initialise sequentially and there will be no need for this function,
		 * however, this does not hold true when using external language information
		 * since that is obtained using an async XHR call.
		 *  @type function
		 *  @param {object} settings DataTables settings object
		 *  @param {object} json The JSON object request from the server - only
		 *    present if client-side Ajax sourced data is used
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.initComplete
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "initComplete": function(settings, json) {
		 *          alert( 'DataTables has finished its initialisation.' );
		 *        }
		 *      } );
		 *    } )
		 */
		"fnInitComplete": null,
	
	
		/**
		 * Called at the very start of each table draw and can be used to cancel the
		 * draw by returning false, any other return (including undefined) results in
		 * the full draw occurring).
		 *  @type function
		 *  @param {object} settings DataTables settings object
		 *  @returns {boolean} False will cancel the draw, anything else (including no
		 *    return) will allow it to complete.
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.preDrawCallback
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "preDrawCallback": function( settings ) {
		 *          if ( $('#test').val() == 1 ) {
		 *            return false;
		 *          }
		 *        }
		 *      } );
		 *    } );
		 */
		"fnPreDrawCallback": null,
	
	
		/**
		 * This function allows you to 'post process' each row after it have been
		 * generated for each table draw, but before it is rendered on screen. This
		 * function might be used for setting the row class name etc.
		 *  @type function
		 *  @param {node} row "TR" element for the current row
		 *  @param {array} data Raw data array for this row
		 *  @param {int} displayIndex The display index for the current table draw
		 *  @param {int} displayIndexFull The index of the data in the full list of
		 *    rows (after filtering)
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.rowCallback
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "rowCallback": function( row, data, displayIndex, displayIndexFull ) {
		 *          // Bold the grade for all 'A' grade browsers
		 *          if ( data[4] == "A" ) {
		 *            $('td:eq(4)', row).html( '<b>A</b>' );
		 *          }
		 *        }
		 *      } );
		 *    } );
		 */
		"fnRowCallback": null,
	
	
		/**
		 * __Deprecated__ The functionality provided by this parameter has now been
		 * superseded by that provided through `ajax`, which should be used instead.
		 *
		 * This parameter allows you to override the default function which obtains
		 * the data from the server so something more suitable for your application.
		 * For example you could use POST data, or pull information from a Gears or
		 * AIR database.
		 *  @type function
		 *  @member
		 *  @param {string} source HTTP source to obtain the data from (`ajax`)
		 *  @param {array} data A key/value pair object containing the data to send
		 *    to the server
		 *  @param {function} callback to be called on completion of the data get
		 *    process that will draw the data on the page.
		 *  @param {object} settings DataTables settings object
		 *
		 *  @dtopt Callbacks
		 *  @dtopt Server-side
		 *  @name DataTable.defaults.serverData
		 *
		 *  @deprecated 1.10. Please use `ajax` for this functionality now.
		 */
		"fnServerData": null,
	
	
		/**
		 * __Deprecated__ The functionality provided by this parameter has now been
		 * superseded by that provided through `ajax`, which should be used instead.
		 *
		 *  It is often useful to send extra data to the server when making an Ajax
		 * request - for example custom filtering information, and this callback
		 * function makes it trivial to send extra information to the server. The
		 * passed in parameter is the data set that has been constructed by
		 * DataTables, and you can add to this or modify it as you require.
		 *  @type function
		 *  @param {array} data Data array (array of objects which are name/value
		 *    pairs) that has been constructed by DataTables and will be sent to the
		 *    server. In the case of Ajax sourced data with server-side processing
		 *    this will be an empty array, for server-side processing there will be a
		 *    significant number of parameters!
		 *  @returns {undefined} Ensure that you modify the data array passed in,
		 *    as this is passed by reference.
		 *
		 *  @dtopt Callbacks
		 *  @dtopt Server-side
		 *  @name DataTable.defaults.serverParams
		 *
		 *  @deprecated 1.10. Please use `ajax` for this functionality now.
		 */
		"fnServerParams": null,
	
	
		/**
		 * Load the table state. With this function you can define from where, and how, the
		 * state of a table is loaded. By default DataTables will load from `localStorage`
		 * but you might wish to use a server-side database or cookies.
		 *  @type function
		 *  @member
		 *  @param {object} settings DataTables settings object
		 *  @return {object} The DataTables state object to be loaded
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.stateLoadCallback
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "stateSave": true,
		 *        "stateLoadCallback": function (settings) {
		 *          var o;
		 *
		 *          // Send an Ajax request to the server to get the data. Note that
		 *          // this is a synchronous request.
		 *          $.ajax( {
		 *            "url": "/state_load",
		 *            "async": false,
		 *            "dataType": "json",
		 *            "success": function (json) {
		 *              o = json;
		 *            }
		 *          } );
		 *
		 *          return o;
		 *        }
		 *      } );
		 *    } );
		 */
		"fnStateLoadCallback": function ( settings ) {
			try {
				return JSON.parse(
					(settings.iStateDuration === -1 ? sessionStorage : localStorage).getItem(
						'DataTables_'+settings.sInstance+'_'+location.pathname
					)
				);
			} catch (e) {}
		},
	
	
		/**
		 * Callback which allows modification of the saved state prior to loading that state.
		 * This callback is called when the table is loading state from the stored data, but
		 * prior to the settings object being modified by the saved state. Note that for
		 * plug-in authors, you should use the `stateLoadParams` event to load parameters for
		 * a plug-in.
		 *  @type function
		 *  @param {object} settings DataTables settings object
		 *  @param {object} data The state object that is to be loaded
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.stateLoadParams
		 *
		 *  @example
		 *    // Remove a saved filter, so filtering is never loaded
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "stateSave": true,
		 *        "stateLoadParams": function (settings, data) {
		 *          data.oSearch.sSearch = "";
		 *        }
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Disallow state loading by returning false
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "stateSave": true,
		 *        "stateLoadParams": function (settings, data) {
		 *          return false;
		 *        }
		 *      } );
		 *    } );
		 */
		"fnStateLoadParams": null,
	
	
		/**
		 * Callback that is called when the state has been loaded from the state saving method
		 * and the DataTables settings object has been modified as a result of the loaded state.
		 *  @type function
		 *  @param {object} settings DataTables settings object
		 *  @param {object} data The state object that was loaded
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.stateLoaded
		 *
		 *  @example
		 *    // Show an alert with the filtering value that was saved
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "stateSave": true,
		 *        "stateLoaded": function (settings, data) {
		 *          alert( 'Saved filter was: '+data.oSearch.sSearch );
		 *        }
		 *      } );
		 *    } );
		 */
		"fnStateLoaded": null,
	
	
		/**
		 * Save the table state. This function allows you to define where and how the state
		 * information for the table is stored By default DataTables will use `localStorage`
		 * but you might wish to use a server-side database or cookies.
		 *  @type function
		 *  @member
		 *  @param {object} settings DataTables settings object
		 *  @param {object} data The state object to be saved
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.stateSaveCallback
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "stateSave": true,
		 *        "stateSaveCallback": function (settings, data) {
		 *          // Send an Ajax request to the server with the state object
		 *          $.ajax( {
		 *            "url": "/state_save",
		 *            "data": data,
		 *            "dataType": "json",
		 *            "method": "POST"
		 *            "success": function () {}
		 *          } );
		 *        }
		 *      } );
		 *    } );
		 */
		"fnStateSaveCallback": function ( settings, data ) {
			try {
				(settings.iStateDuration === -1 ? sessionStorage : localStorage).setItem(
					'DataTables_'+settings.sInstance+'_'+location.pathname,
					JSON.stringify( data )
				);
			} catch (e) {}
		},
	
	
		/**
		 * Callback which allows modification of the state to be saved. Called when the table
		 * has changed state a new state save is required. This method allows modification of
		 * the state saving object prior to actually doing the save, including addition or
		 * other state properties or modification. Note that for plug-in authors, you should
		 * use the `stateSaveParams` event to save parameters for a plug-in.
		 *  @type function
		 *  @param {object} settings DataTables settings object
		 *  @param {object} data The state object to be saved
		 *
		 *  @dtopt Callbacks
		 *  @name DataTable.defaults.stateSaveParams
		 *
		 *  @example
		 *    // Remove a saved filter, so filtering is never saved
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "stateSave": true,
		 *        "stateSaveParams": function (settings, data) {
		 *          data.oSearch.sSearch = "";
		 *        }
		 *      } );
		 *    } );
		 */
		"fnStateSaveParams": null,
	
	
		/**
		 * Duration for which the saved state information is considered valid. After this period
		 * has elapsed the state will be returned to the default.
		 * Value is given in seconds.
		 *  @type int
		 *  @default 7200 <i>(2 hours)</i>
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.stateDuration
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "stateDuration": 60*60*24; // 1 day
		 *      } );
		 *    } )
		 */
		"iStateDuration": 7200,
	
	
		/**
		 * When enabled DataTables will not make a request to the server for the first
		 * page draw - rather it will use the data already on the page (no sorting etc
		 * will be applied to it), thus saving on an XHR at load time. `deferLoading`
		 * is used to indicate that deferred loading is required, but it is also used
		 * to tell DataTables how many records there are in the full table (allowing
		 * the information element and pagination to be displayed correctly). In the case
		 * where a filtering is applied to the table on initial load, this can be
		 * indicated by giving the parameter as an array, where the first element is
		 * the number of records available after filtering and the second element is the
		 * number of records without filtering (allowing the table information element
		 * to be shown correctly).
		 *  @type int | array
		 *  @default null
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.deferLoading
		 *
		 *  @example
		 *    // 57 records available in the table, no filtering applied
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "serverSide": true,
		 *        "ajax": "scripts/server_processing.php",
		 *        "deferLoading": 57
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // 57 records after filtering, 100 without filtering (an initial filter applied)
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "serverSide": true,
		 *        "ajax": "scripts/server_processing.php",
		 *        "deferLoading": [ 57, 100 ],
		 *        "search": {
		 *          "search": "my_filter"
		 *        }
		 *      } );
		 *    } );
		 */
		"iDeferLoading": null,
	
	
		/**
		 * Number of rows to display on a single page when using pagination. If
		 * feature enabled (`lengthChange`) then the end user will be able to override
		 * this to a custom setting using a pop-up menu.
		 *  @type int
		 *  @default 10
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.pageLength
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "pageLength": 50
		 *      } );
		 *    } )
		 */
		"iDisplayLength": 10,
	
	
		/**
		 * Define the starting point for data display when using DataTables with
		 * pagination. Note that this parameter is the number of records, rather than
		 * the page number, so if you have 10 records per page and want to start on
		 * the third page, it should be "20".
		 *  @type int
		 *  @default 0
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.displayStart
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "displayStart": 20
		 *      } );
		 *    } )
		 */
		"iDisplayStart": 0,
	
	
		/**
		 * By default DataTables allows keyboard navigation of the table (sorting, paging,
		 * and filtering) by adding a `tabindex` attribute to the required elements. This
		 * allows you to tab through the controls and press the enter key to activate them.
		 * The tabindex is default 0, meaning that the tab follows the flow of the document.
		 * You can overrule this using this parameter if you wish. Use a value of -1 to
		 * disable built-in keyboard navigation.
		 *  @type int
		 *  @default 0
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.tabIndex
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "tabIndex": 1
		 *      } );
		 *    } );
		 */
		"iTabIndex": 0,
	
	
		/**
		 * Classes that DataTables assigns to the various components and features
		 * that it adds to the HTML table. This allows classes to be configured
		 * during initialisation in addition to through the static
		 * {@link DataTable.ext.oStdClasses} object).
		 *  @namespace
		 *  @name DataTable.defaults.classes
		 */
		"oClasses": {},
	
	
		/**
		 * All strings that DataTables uses in the user interface that it creates
		 * are defined in this object, allowing you to modified them individually or
		 * completely replace them all as required.
		 *  @namespace
		 *  @name DataTable.defaults.language
		 */
		"oLanguage": {
			/**
			 * Strings that are used for WAI-ARIA labels and controls only (these are not
			 * actually visible on the page, but will be read by screenreaders, and thus
			 * must be internationalised as well).
			 *  @namespace
			 *  @name DataTable.defaults.language.aria
			 */
			"oAria": {
				/**
				 * ARIA label that is added to the table headers when the column may be
				 * sorted ascending by activing the column (click or return when focused).
				 * Note that the column header is prefixed to this string.
				 *  @type string
				 *  @default : activate to sort column ascending
				 *
				 *  @dtopt Language
				 *  @name DataTable.defaults.language.aria.sortAscending
				 *
				 *  @example
				 *    $(document).ready( function() {
				 *      $('#example').dataTable( {
				 *        "language": {
				 *          "aria": {
				 *            "sortAscending": " - click/return to sort ascending"
				 *          }
				 *        }
				 *      } );
				 *    } );
				 */
				"sSortAscending": ": activate to sort column ascending",
	
				/**
				 * ARIA label that is added to the table headers when the column may be
				 * sorted descending by activing the column (click or return when focused).
				 * Note that the column header is prefixed to this string.
				 *  @type string
				 *  @default : activate to sort column ascending
				 *
				 *  @dtopt Language
				 *  @name DataTable.defaults.language.aria.sortDescending
				 *
				 *  @example
				 *    $(document).ready( function() {
				 *      $('#example').dataTable( {
				 *        "language": {
				 *          "aria": {
				 *            "sortDescending": " - click/return to sort descending"
				 *          }
				 *        }
				 *      } );
				 *    } );
				 */
				"sSortDescending": ": activate to sort column descending"
			},
	
			/**
			 * Pagination string used by DataTables for the built-in pagination
			 * control types.
			 *  @namespace
			 *  @name DataTable.defaults.language.paginate
			 */
			"oPaginate": {
				/**
				 * Text to use when using the 'full_numbers' type of pagination for the
				 * button to take the user to the first page.
				 *  @type string
				 *  @default First
				 *
				 *  @dtopt Language
				 *  @name DataTable.defaults.language.paginate.first
				 *
				 *  @example
				 *    $(document).ready( function() {
				 *      $('#example').dataTable( {
				 *        "language": {
				 *          "paginate": {
				 *            "first": "First page"
				 *          }
				 *        }
				 *      } );
				 *    } );
				 */
				"sFirst": "First",
	
	
				/**
				 * Text to use when using the 'full_numbers' type of pagination for the
				 * button to take the user to the last page.
				 *  @type string
				 *  @default Last
				 *
				 *  @dtopt Language
				 *  @name DataTable.defaults.language.paginate.last
				 *
				 *  @example
				 *    $(document).ready( function() {
				 *      $('#example').dataTable( {
				 *        "language": {
				 *          "paginate": {
				 *            "last": "Last page"
				 *          }
				 *        }
				 *      } );
				 *    } );
				 */
				"sLast": "Last",
	
	
				/**
				 * Text to use for the 'next' pagination button (to take the user to the
				 * next page).
				 *  @type string
				 *  @default Next
				 *
				 *  @dtopt Language
				 *  @name DataTable.defaults.language.paginate.next
				 *
				 *  @example
				 *    $(document).ready( function() {
				 *      $('#example').dataTable( {
				 *        "language": {
				 *          "paginate": {
				 *            "next": "Next page"
				 *          }
				 *        }
				 *      } );
				 *    } );
				 */
				"sNext": "Next",
	
	
				/**
				 * Text to use for the 'previous' pagination button (to take the user to
				 * the previous page).
				 *  @type string
				 *  @default Previous
				 *
				 *  @dtopt Language
				 *  @name DataTable.defaults.language.paginate.previous
				 *
				 *  @example
				 *    $(document).ready( function() {
				 *      $('#example').dataTable( {
				 *        "language": {
				 *          "paginate": {
				 *            "previous": "Previous page"
				 *          }
				 *        }
				 *      } );
				 *    } );
				 */
				"sPrevious": "Previous"
			},
	
			/**
			 * This string is shown in preference to `zeroRecords` when the table is
			 * empty of data (regardless of filtering). Note that this is an optional
			 * parameter - if it is not given, the value of `zeroRecords` will be used
			 * instead (either the default or given value).
			 *  @type string
			 *  @default No data available in table
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.emptyTable
			 *
			 *  @example
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "emptyTable": "No data available in table"
			 *        }
			 *      } );
			 *    } );
			 */
			"sEmptyTable": "No data available in table",
	
	
			/**
			 * This string gives information to the end user about the information
			 * that is current on display on the page. The following tokens can be
			 * used in the string and will be dynamically replaced as the table
			 * display updates. This tokens can be placed anywhere in the string, or
			 * removed as needed by the language requires:
			 *
			 * * `\_START\_` - Display index of the first record on the current page
			 * * `\_END\_` - Display index of the last record on the current page
			 * * `\_TOTAL\_` - Number of records in the table after filtering
			 * * `\_MAX\_` - Number of records in the table without filtering
			 * * `\_PAGE\_` - Current page number
			 * * `\_PAGES\_` - Total number of pages of data in the table
			 *
			 *  @type string
			 *  @default Showing _START_ to _END_ of _TOTAL_ entries
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.info
			 *
			 *  @example
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "info": "Showing page _PAGE_ of _PAGES_"
			 *        }
			 *      } );
			 *    } );
			 */
			"sInfo": "Showing _START_ to _END_ of _TOTAL_ entries",
	
	
			/**
			 * Display information string for when the table is empty. Typically the
			 * format of this string should match `info`.
			 *  @type string
			 *  @default Showing 0 to 0 of 0 entries
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.infoEmpty
			 *
			 *  @example
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "infoEmpty": "No entries to show"
			 *        }
			 *      } );
			 *    } );
			 */
			"sInfoEmpty": "Showing 0 to 0 of 0 entries",
	
	
			/**
			 * When a user filters the information in a table, this string is appended
			 * to the information (`info`) to give an idea of how strong the filtering
			 * is. The variable _MAX_ is dynamically updated.
			 *  @type string
			 *  @default (filtered from _MAX_ total entries)
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.infoFiltered
			 *
			 *  @example
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "infoFiltered": " - filtering from _MAX_ records"
			 *        }
			 *      } );
			 *    } );
			 */
			"sInfoFiltered": "(filtered from _MAX_ total entries)",
	
	
			/**
			 * If can be useful to append extra information to the info string at times,
			 * and this variable does exactly that. This information will be appended to
			 * the `info` (`infoEmpty` and `infoFiltered` in whatever combination they are
			 * being used) at all times.
			 *  @type string
			 *  @default <i>Empty string</i>
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.infoPostFix
			 *
			 *  @example
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "infoPostFix": "All records shown are derived from real information."
			 *        }
			 *      } );
			 *    } );
			 */
			"sInfoPostFix": "",
	
	
			/**
			 * This decimal place operator is a little different from the other
			 * language options since DataTables doesn't output floating point
			 * numbers, so it won't ever use this for display of a number. Rather,
			 * what this parameter does is modify the sort methods of the table so
			 * that numbers which are in a format which has a character other than
			 * a period (`.`) as a decimal place will be sorted numerically.
			 *
			 * Note that numbers with different decimal places cannot be shown in
			 * the same table and still be sortable, the table must be consistent.
			 * However, multiple different tables on the page can use different
			 * decimal place characters.
			 *  @type string
			 *  @default 
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.decimal
			 *
			 *  @example
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "decimal": ","
			 *          "thousands": "."
			 *        }
			 *      } );
			 *    } );
			 */
			"sDecimal": "",
	
	
			/**
			 * DataTables has a build in number formatter (`formatNumber`) which is
			 * used to format large numbers that are used in the table information.
			 * By default a comma is used, but this can be trivially changed to any
			 * character you wish with this parameter.
			 *  @type string
			 *  @default ,
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.thousands
			 *
			 *  @example
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "thousands": "'"
			 *        }
			 *      } );
			 *    } );
			 */
			"sThousands": ",",
	
	
			/**
			 * Detail the action that will be taken when the drop down menu for the
			 * pagination length option is changed. The '_MENU_' variable is replaced
			 * with a default select list of 10, 25, 50 and 100, and can be replaced
			 * with a custom select box if required.
			 *  @type string
			 *  @default Show _MENU_ entries
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.lengthMenu
			 *
			 *  @example
			 *    // Language change only
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "lengthMenu": "Display _MENU_ records"
			 *        }
			 *      } );
			 *    } );
			 *
			 *  @example
			 *    // Language and options change
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "lengthMenu": 'Display <select>'+
			 *            '<option value="10">10</option>'+
			 *            '<option value="20">20</option>'+
			 *            '<option value="30">30</option>'+
			 *            '<option value="40">40</option>'+
			 *            '<option value="50">50</option>'+
			 *            '<option value="-1">All</option>'+
			 *            '</select> records'
			 *        }
			 *      } );
			 *    } );
			 */
			"sLengthMenu": "Show _MENU_ entries",
	
	
			/**
			 * When using Ajax sourced data and during the first draw when DataTables is
			 * gathering the data, this message is shown in an empty row in the table to
			 * indicate to the end user the the data is being loaded. Note that this
			 * parameter is not used when loading data by server-side processing, just
			 * Ajax sourced data with client-side processing.
			 *  @type string
			 *  @default Loading...
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.loadingRecords
			 *
			 *  @example
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "loadingRecords": "Please wait - loading..."
			 *        }
			 *      } );
			 *    } );
			 */
			"sLoadingRecords": "Loading...",
	
	
			/**
			 * Text which is displayed when the table is processing a user action
			 * (usually a sort command or similar).
			 *  @type string
			 *  @default Processing...
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.processing
			 *
			 *  @example
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "processing": "DataTables is currently busy"
			 *        }
			 *      } );
			 *    } );
			 */
			"sProcessing": "Processing...",
	
	
			/**
			 * Details the actions that will be taken when the user types into the
			 * filtering input text box. The variable "_INPUT_", if used in the string,
			 * is replaced with the HTML text box for the filtering input allowing
			 * control over where it appears in the string. If "_INPUT_" is not given
			 * then the input box is appended to the string automatically.
			 *  @type string
			 *  @default Search:
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.search
			 *
			 *  @example
			 *    // Input text box will be appended at the end automatically
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "search": "Filter records:"
			 *        }
			 *      } );
			 *    } );
			 *
			 *  @example
			 *    // Specify where the filter should appear
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "search": "Apply filter _INPUT_ to table"
			 *        }
			 *      } );
			 *    } );
			 */
			"sSearch": "Search:",
	
	
			/**
			 * Assign a `placeholder` attribute to the search `input` element
			 *  @type string
			 *  @default 
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.searchPlaceholder
			 */
			"sSearchPlaceholder": "",
	
	
			/**
			 * All of the language information can be stored in a file on the
			 * server-side, which DataTables will look up if this parameter is passed.
			 * It must store the URL of the language file, which is in a JSON format,
			 * and the object has the same properties as the oLanguage object in the
			 * initialiser object (i.e. the above parameters). Please refer to one of
			 * the example language files to see how this works in action.
			 *  @type string
			 *  @default <i>Empty string - i.e. disabled</i>
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.url
			 *
			 *  @example
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "url": "http://www.sprymedia.co.uk/dataTables/lang.txt"
			 *        }
			 *      } );
			 *    } );
			 */
			"sUrl": "",
	
	
			/**
			 * Text shown inside the table records when the is no information to be
			 * displayed after filtering. `emptyTable` is shown when there is simply no
			 * information in the table at all (regardless of filtering).
			 *  @type string
			 *  @default No matching records found
			 *
			 *  @dtopt Language
			 *  @name DataTable.defaults.language.zeroRecords
			 *
			 *  @example
			 *    $(document).ready( function() {
			 *      $('#example').dataTable( {
			 *        "language": {
			 *          "zeroRecords": "No records to display"
			 *        }
			 *      } );
			 *    } );
			 */
			"sZeroRecords": "No matching records found"
		},
	
	
		/**
		 * This parameter allows you to have define the global filtering state at
		 * initialisation time. As an object the `search` parameter must be
		 * defined, but all other parameters are optional. When `regex` is true,
		 * the search string will be treated as a regular expression, when false
		 * (default) it will be treated as a straight string. When `smart`
		 * DataTables will use it's smart filtering methods (to word match at
		 * any point in the data), when false this will not be done.
		 *  @namespace
		 *  @extends DataTable.models.oSearch
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.search
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "search": {"search": "Initial search"}
		 *      } );
		 *    } )
		 */
		"oSearch": $.extend( {}, DataTable.models.oSearch ),
	
	
		/**
		 * __Deprecated__ The functionality provided by this parameter has now been
		 * superseded by that provided through `ajax`, which should be used instead.
		 *
		 * By default DataTables will look for the property `data` (or `aaData` for
		 * compatibility with DataTables 1.9-) when obtaining data from an Ajax
		 * source or for server-side processing - this parameter allows that
		 * property to be changed. You can use Javascript dotted object notation to
		 * get a data source for multiple levels of nesting.
		 *  @type string
		 *  @default data
		 *
		 *  @dtopt Options
		 *  @dtopt Server-side
		 *  @name DataTable.defaults.ajaxDataProp
		 *
		 *  @deprecated 1.10. Please use `ajax` for this functionality now.
		 */
		"sAjaxDataProp": "data",
	
	
		/**
		 * __Deprecated__ The functionality provided by this parameter has now been
		 * superseded by that provided through `ajax`, which should be used instead.
		 *
		 * You can instruct DataTables to load data from an external
		 * source using this parameter (use aData if you want to pass data in you
		 * already have). Simply provide a url a JSON object can be obtained from.
		 *  @type string
		 *  @default null
		 *
		 *  @dtopt Options
		 *  @dtopt Server-side
		 *  @name DataTable.defaults.ajaxSource
		 *
		 *  @deprecated 1.10. Please use `ajax` for this functionality now.
		 */
		"sAjaxSource": null,
	
	
		/**
		 * This initialisation variable allows you to specify exactly where in the
		 * DOM you want DataTables to inject the various controls it adds to the page
		 * (for example you might want the pagination controls at the top of the
		 * table). DIV elements (with or without a custom class) can also be added to
		 * aid styling. The follow syntax is used:
		 *   <ul>
		 *     <li>The following options are allowed:
		 *       <ul>
		 *         <li>'l' - Length changing</li>
		 *         <li>'f' - Filtering input</li>
		 *         <li>'t' - The table!</li>
		 *         <li>'i' - Information</li>
		 *         <li>'p' - Pagination</li>
		 *         <li>'r' - pRocessing</li>
		 *       </ul>
		 *     </li>
		 *     <li>The following constants are allowed:
		 *       <ul>
		 *         <li>'H' - jQueryUI theme "header" classes ('fg-toolbar ui-widget-header ui-corner-tl ui-corner-tr ui-helper-clearfix')</li>
		 *         <li>'F' - jQueryUI theme "footer" classes ('fg-toolbar ui-widget-header ui-corner-bl ui-corner-br ui-helper-clearfix')</li>
		 *       </ul>
		 *     </li>
		 *     <li>The following syntax is expected:
		 *       <ul>
		 *         <li>'&lt;' and '&gt;' - div elements</li>
		 *         <li>'&lt;"class" and '&gt;' - div with a class</li>
		 *         <li>'&lt;"#id" and '&gt;' - div with an ID</li>
		 *       </ul>
		 *     </li>
		 *     <li>Examples:
		 *       <ul>
		 *         <li>'&lt;"wrapper"flipt&gt;'</li>
		 *         <li>'&lt;lf&lt;t&gt;ip&gt;'</li>
		 *       </ul>
		 *     </li>
		 *   </ul>
		 *  @type string
		 *  @default lfrtip <i>(when `jQueryUI` is false)</i> <b>or</b>
		 *    <"H"lfr>t<"F"ip> <i>(when `jQueryUI` is true)</i>
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.dom
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "dom": '&lt;"top"i&gt;rt&lt;"bottom"flp&gt;&lt;"clear"&gt;'
		 *      } );
		 *    } );
		 */
		"sDom": "lfrtip",
	
	
		/**
		 * Search delay option. This will throttle full table searches that use the
		 * DataTables provided search input element (it does not effect calls to
		 * `dt-api search()`, providing a delay before the search is made.
		 *  @type integer
		 *  @default 0
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.searchDelay
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "searchDelay": 200
		 *      } );
		 *    } )
		 */
		"searchDelay": null,
	
	
		/**
		 * DataTables features four different built-in options for the buttons to
		 * display for pagination control:
		 *
		 * * `simple` - 'Previous' and 'Next' buttons only
		 * * 'simple_numbers` - 'Previous' and 'Next' buttons, plus page numbers
		 * * `full` - 'First', 'Previous', 'Next' and 'Last' buttons
		 * * `full_numbers` - 'First', 'Previous', 'Next' and 'Last' buttons, plus
		 *   page numbers
		 *  
		 * Further methods can be added using {@link DataTable.ext.oPagination}.
		 *  @type string
		 *  @default simple_numbers
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.pagingType
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "pagingType": "full_numbers"
		 *      } );
		 *    } )
		 */
		"sPaginationType": "simple_numbers",
	
	
		/**
		 * Enable horizontal scrolling. When a table is too wide to fit into a
		 * certain layout, or you have a large number of columns in the table, you
		 * can enable x-scrolling to show the table in a viewport, which can be
		 * scrolled. This property can be `true` which will allow the table to
		 * scroll horizontally when needed, or any CSS unit, or a number (in which
		 * case it will be treated as a pixel measurement). Setting as simply `true`
		 * is recommended.
		 *  @type boolean|string
		 *  @default <i>blank string - i.e. disabled</i>
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.scrollX
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "scrollX": true,
		 *        "scrollCollapse": true
		 *      } );
		 *    } );
		 */
		"sScrollX": "",
	
	
		/**
		 * This property can be used to force a DataTable to use more width than it
		 * might otherwise do when x-scrolling is enabled. For example if you have a
		 * table which requires to be well spaced, this parameter is useful for
		 * "over-sizing" the table, and thus forcing scrolling. This property can by
		 * any CSS unit, or a number (in which case it will be treated as a pixel
		 * measurement).
		 *  @type string
		 *  @default <i>blank string - i.e. disabled</i>
		 *
		 *  @dtopt Options
		 *  @name DataTable.defaults.scrollXInner
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "scrollX": "100%",
		 *        "scrollXInner": "110%"
		 *      } );
		 *    } );
		 */
		"sScrollXInner": "",
	
	
		/**
		 * Enable vertical scrolling. Vertical scrolling will constrain the DataTable
		 * to the given height, and enable scrolling for any data which overflows the
		 * current viewport. This can be used as an alternative to paging to display
		 * a lot of data in a small area (although paging and scrolling can both be
		 * enabled at the same time). This property can be any CSS unit, or a number
		 * (in which case it will be treated as a pixel measurement).
		 *  @type string
		 *  @default <i>blank string - i.e. disabled</i>
		 *
		 *  @dtopt Features
		 *  @name DataTable.defaults.scrollY
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "scrollY": "200px",
		 *        "paginate": false
		 *      } );
		 *    } );
		 */
		"sScrollY": "",
	
	
		/**
		 * __Deprecated__ The functionality provided by this parameter has now been
		 * superseded by that provided through `ajax`, which should be used instead.
		 *
		 * Set the HTTP method that is used to make the Ajax call for server-side
		 * processing or Ajax sourced data.
		 *  @type string
		 *  @default GET
		 *
		 *  @dtopt Options
		 *  @dtopt Server-side
		 *  @name DataTable.defaults.serverMethod
		 *
		 *  @deprecated 1.10. Please use `ajax` for this functionality now.
		 */
		"sServerMethod": "GET",
	
	
		/**
		 * DataTables makes use of renderers when displaying HTML elements for
		 * a table. These renderers can be added or modified by plug-ins to
		 * generate suitable mark-up for a site. For example the Bootstrap
		 * integration plug-in for DataTables uses a paging button renderer to
		 * display pagination buttons in the mark-up required by Bootstrap.
		 *
		 * For further information about the renderers available see
		 * DataTable.ext.renderer
		 *  @type string|object
		 *  @default null
		 *
		 *  @name DataTable.defaults.renderer
		 *
		 */
		"renderer": null
	};
	
	_fnHungarianMap( DataTable.defaults );
	
	
	
	/*
	 * Developer note - See note in model.defaults.js about the use of Hungarian
	 * notation and camel case.
	 */
	
	/**
	 * Column options that can be given to DataTables at initialisation time.
	 *  @namespace
	 */
	DataTable.defaults.column = {
		/**
		 * Define which column(s) an order will occur on for this column. This
		 * allows a column's ordering to take multiple columns into account when
		 * doing a sort or use the data from a different column. For example first
		 * name / last name columns make sense to do a multi-column sort over the
		 * two columns.
		 *  @type array|int
		 *  @default null <i>Takes the value of the column index automatically</i>
		 *
		 *  @name DataTable.defaults.column.orderData
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columnDefs`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [
		 *          { "orderData": [ 0, 1 ], "targets": [ 0 ] },
		 *          { "orderData": [ 1, 0 ], "targets": [ 1 ] },
		 *          { "orderData": 2, "targets": [ 2 ] }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          { "orderData": [ 0, 1 ] },
		 *          { "orderData": [ 1, 0 ] },
		 *          { "orderData": 2 },
		 *          null,
		 *          null
		 *        ]
		 *      } );
		 *    } );
		 */
		"aDataSort": null,
		"iDataSort": -1,
	
	
		/**
		 * You can control the default ordering direction, and even alter the
		 * behaviour of the sort handler (i.e. only allow ascending ordering etc)
		 * using this parameter.
		 *  @type array
		 *  @default [ 'asc', 'desc' ]
		 *
		 *  @name DataTable.defaults.column.orderSequence
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columnDefs`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [
		 *          { "orderSequence": [ "asc" ], "targets": [ 1 ] },
		 *          { "orderSequence": [ "desc", "asc", "asc" ], "targets": [ 2 ] },
		 *          { "orderSequence": [ "desc" ], "targets": [ 3 ] }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          null,
		 *          { "orderSequence": [ "asc" ] },
		 *          { "orderSequence": [ "desc", "asc", "asc" ] },
		 *          { "orderSequence": [ "desc" ] },
		 *          null
		 *        ]
		 *      } );
		 *    } );
		 */
		"asSorting": [ 'asc', 'desc' ],
	
	
		/**
		 * Enable or disable filtering on the data in this column.
		 *  @type boolean
		 *  @default true
		 *
		 *  @name DataTable.defaults.column.searchable
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columnDefs`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [
		 *          { "searchable": false, "targets": [ 0 ] }
		 *        ] } );
		 *    } );
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          { "searchable": false },
		 *          null,
		 *          null,
		 *          null,
		 *          null
		 *        ] } );
		 *    } );
		 */
		"bSearchable": true,
	
	
		/**
		 * Enable or disable ordering on this column.
		 *  @type boolean
		 *  @default true
		 *
		 *  @name DataTable.defaults.column.orderable
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columnDefs`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [
		 *          { "orderable": false, "targets": [ 0 ] }
		 *        ] } );
		 *    } );
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          { "orderable": false },
		 *          null,
		 *          null,
		 *          null,
		 *          null
		 *        ] } );
		 *    } );
		 */
		"bSortable": true,
	
	
		/**
		 * Enable or disable the display of this column.
		 *  @type boolean
		 *  @default true
		 *
		 *  @name DataTable.defaults.column.visible
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columnDefs`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [
		 *          { "visible": false, "targets": [ 0 ] }
		 *        ] } );
		 *    } );
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          { "visible": false },
		 *          null,
		 *          null,
		 *          null,
		 *          null
		 *        ] } );
		 *    } );
		 */
		"bVisible": true,
	
	
		/**
		 * Developer definable function that is called whenever a cell is created (Ajax source,
		 * etc) or processed for input (DOM source). This can be used as a compliment to mRender
		 * allowing you to modify the DOM element (add background colour for example) when the
		 * element is available.
		 *  @type function
		 *  @param {element} td The TD node that has been created
		 *  @param {*} cellData The Data for the cell
		 *  @param {array|object} rowData The data for the whole row
		 *  @param {int} row The row index for the aoData data store
		 *  @param {int} col The column index for aoColumns
		 *
		 *  @name DataTable.defaults.column.createdCell
		 *  @dtopt Columns
		 *
		 *  @example
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [ {
		 *          "targets": [3],
		 *          "createdCell": function (td, cellData, rowData, row, col) {
		 *            if ( cellData == "1.7" ) {
		 *              $(td).css('color', 'blue')
		 *            }
		 *          }
		 *        } ]
		 *      });
		 *    } );
		 */
		"fnCreatedCell": null,
	
	
		/**
		 * This parameter has been replaced by `data` in DataTables to ensure naming
		 * consistency. `dataProp` can still be used, as there is backwards
		 * compatibility in DataTables for this option, but it is strongly
		 * recommended that you use `data` in preference to `dataProp`.
		 *  @name DataTable.defaults.column.dataProp
		 */
	
	
		/**
		 * This property can be used to read data from any data source property,
		 * including deeply nested objects / properties. `data` can be given in a
		 * number of different ways which effect its behaviour:
		 *
		 * * `integer` - treated as an array index for the data source. This is the
		 *   default that DataTables uses (incrementally increased for each column).
		 * * `string` - read an object property from the data source. There are
		 *   three 'special' options that can be used in the string to alter how
		 *   DataTables reads the data from the source object:
		 *    * `.` - Dotted Javascript notation. Just as you use a `.` in
		 *      Javascript to read from nested objects, so to can the options
		 *      specified in `data`. For example: `browser.version` or
		 *      `browser.name`. If your object parameter name contains a period, use
		 *      `\\` to escape it - i.e. `first\\.name`.
		 *    * `[]` - Array notation. DataTables can automatically combine data
		 *      from and array source, joining the data with the characters provided
		 *      between the two brackets. For example: `name[, ]` would provide a
		 *      comma-space separated list from the source array. If no characters
		 *      are provided between the brackets, the original array source is
		 *      returned.
		 *    * `()` - Function notation. Adding `()` to the end of a parameter will
		 *      execute a function of the name given. For example: `browser()` for a
		 *      simple function on the data source, `browser.version()` for a
		 *      function in a nested property or even `browser().version` to get an
		 *      object property if the function called returns an object. Note that
		 *      function notation is recommended for use in `render` rather than
		 *      `data` as it is much simpler to use as a renderer.
		 * * `null` - use the original data source for the row rather than plucking
		 *   data directly from it. This action has effects on two other
		 *   initialisation options:
		 *    * `defaultContent` - When null is given as the `data` option and
		 *      `defaultContent` is specified for the column, the value defined by
		 *      `defaultContent` will be used for the cell.
		 *    * `render` - When null is used for the `data` option and the `render`
		 *      option is specified for the column, the whole data source for the
		 *      row is used for the renderer.
		 * * `function` - the function given will be executed whenever DataTables
		 *   needs to set or get the data for a cell in the column. The function
		 *   takes three parameters:
		 *    * Parameters:
		 *      * `{array|object}` The data source for the row
		 *      * `{string}` The type call data requested - this will be 'set' when
		 *        setting data or 'filter', 'display', 'type', 'sort' or undefined
		 *        when gathering data. Note that when `undefined` is given for the
		 *        type DataTables expects to get the raw data for the object back<
		 *      * `{*}` Data to set when the second parameter is 'set'.
		 *    * Return:
		 *      * The return value from the function is not required when 'set' is
		 *        the type of call, but otherwise the return is what will be used
		 *        for the data requested.
		 *
		 * Note that `data` is a getter and setter option. If you just require
		 * formatting of data for output, you will likely want to use `render` which
		 * is simply a getter and thus simpler to use.
		 *
		 * Note that prior to DataTables 1.9.2 `data` was called `mDataProp`. The
		 * name change reflects the flexibility of this property and is consistent
		 * with the naming of mRender. If 'mDataProp' is given, then it will still
		 * be used by DataTables, as it automatically maps the old name to the new
		 * if required.
		 *
		 *  @type string|int|function|null
		 *  @default null <i>Use automatically calculated column index</i>
		 *
		 *  @name DataTable.defaults.column.data
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Read table data from objects
		 *    // JSON structure for each row:
		 *    //   {
		 *    //      "engine": {value},
		 *    //      "browser": {value},
		 *    //      "platform": {value},
		 *    //      "version": {value},
		 *    //      "grade": {value}
		 *    //   }
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "ajaxSource": "sources/objects.txt",
		 *        "columns": [
		 *          { "data": "engine" },
		 *          { "data": "browser" },
		 *          { "data": "platform" },
		 *          { "data": "version" },
		 *          { "data": "grade" }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Read information from deeply nested objects
		 *    // JSON structure for each row:
		 *    //   {
		 *    //      "engine": {value},
		 *    //      "browser": {value},
		 *    //      "platform": {
		 *    //         "inner": {value}
		 *    //      },
		 *    //      "details": [
		 *    //         {value}, {value}
		 *    //      ]
		 *    //   }
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "ajaxSource": "sources/deep.txt",
		 *        "columns": [
		 *          { "data": "engine" },
		 *          { "data": "browser" },
		 *          { "data": "platform.inner" },
		 *          { "data": "platform.details.0" },
		 *          { "data": "platform.details.1" }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using `data` as a function to provide different information for
		 *    // sorting, filtering and display. In this case, currency (price)
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [ {
		 *          "targets": [ 0 ],
		 *          "data": function ( source, type, val ) {
		 *            if (type === 'set') {
		 *              source.price = val;
		 *              // Store the computed dislay and filter values for efficiency
		 *              source.price_display = val=="" ? "" : "$"+numberFormat(val);
		 *              source.price_filter  = val=="" ? "" : "$"+numberFormat(val)+" "+val;
		 *              return;
		 *            }
		 *            else if (type === 'display') {
		 *              return source.price_display;
		 *            }
		 *            else if (type === 'filter') {
		 *              return source.price_filter;
		 *            }
		 *            // 'sort', 'type' and undefined all just use the integer
		 *            return source.price;
		 *          }
		 *        } ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using default content
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [ {
		 *          "targets": [ 0 ],
		 *          "data": null,
		 *          "defaultContent": "Click to edit"
		 *        } ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using array notation - outputting a list from an array
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [ {
		 *          "targets": [ 0 ],
		 *          "data": "name[, ]"
		 *        } ]
		 *      } );
		 *    } );
		 *
		 */
		"mData": null,
	
	
		/**
		 * This property is the rendering partner to `data` and it is suggested that
		 * when you want to manipulate data for display (including filtering,
		 * sorting etc) without altering the underlying data for the table, use this
		 * property. `render` can be considered to be the the read only companion to
		 * `data` which is read / write (then as such more complex). Like `data`
		 * this option can be given in a number of different ways to effect its
		 * behaviour:
		 *
		 * * `integer` - treated as an array index for the data source. This is the
		 *   default that DataTables uses (incrementally increased for each column).
		 * * `string` - read an object property from the data source. There are
		 *   three 'special' options that can be used in the string to alter how
		 *   DataTables reads the data from the source object:
		 *    * `.` - Dotted Javascript notation. Just as you use a `.` in
		 *      Javascript to read from nested objects, so to can the options
		 *      specified in `data`. For example: `browser.version` or
		 *      `browser.name`. If your object parameter name contains a period, use
		 *      `\\` to escape it - i.e. `first\\.name`.
		 *    * `[]` - Array notation. DataTables can automatically combine data
		 *      from and array source, joining the data with the characters provided
		 *      between the two brackets. For example: `name[, ]` would provide a
		 *      comma-space separated list from the source array. If no characters
		 *      are provided between the brackets, the original array source is
		 *      returned.
		 *    * `()` - Function notation. Adding `()` to the end of a parameter will
		 *      execute a function of the name given. For example: `browser()` for a
		 *      simple function on the data source, `browser.version()` for a
		 *      function in a nested property or even `browser().version` to get an
		 *      object property if the function called returns an object.
		 * * `object` - use different data for the different data types requested by
		 *   DataTables ('filter', 'display', 'type' or 'sort'). The property names
		 *   of the object is the data type the property refers to and the value can
		 *   defined using an integer, string or function using the same rules as
		 *   `render` normally does. Note that an `_` option _must_ be specified.
		 *   This is the default value to use if you haven't specified a value for
		 *   the data type requested by DataTables.
		 * * `function` - the function given will be executed whenever DataTables
		 *   needs to set or get the data for a cell in the column. The function
		 *   takes three parameters:
		 *    * Parameters:
		 *      * {array|object} The data source for the row (based on `data`)
		 *      * {string} The type call data requested - this will be 'filter',
		 *        'display', 'type' or 'sort'.
		 *      * {array|object} The full data source for the row (not based on
		 *        `data`)
		 *    * Return:
		 *      * The return value from the function is what will be used for the
		 *        data requested.
		 *
		 *  @type string|int|function|object|null
		 *  @default null Use the data source value.
		 *
		 *  @name DataTable.defaults.column.render
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Create a comma separated list from an array of objects
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "ajaxSource": "sources/deep.txt",
		 *        "columns": [
		 *          { "data": "engine" },
		 *          { "data": "browser" },
		 *          {
		 *            "data": "platform",
		 *            "render": "[, ].name"
		 *          }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Execute a function to obtain data
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [ {
		 *          "targets": [ 0 ],
		 *          "data": null, // Use the full data source object for the renderer's source
		 *          "render": "browserName()"
		 *        } ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // As an object, extracting different data for the different types
		 *    // This would be used with a data source such as:
		 *    //   { "phone": 5552368, "phone_filter": "5552368 555-2368", "phone_display": "555-2368" }
		 *    // Here the `phone` integer is used for sorting and type detection, while `phone_filter`
		 *    // (which has both forms) is used for filtering for if a user inputs either format, while
		 *    // the formatted phone number is the one that is shown in the table.
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [ {
		 *          "targets": [ 0 ],
		 *          "data": null, // Use the full data source object for the renderer's source
		 *          "render": {
		 *            "_": "phone",
		 *            "filter": "phone_filter",
		 *            "display": "phone_display"
		 *          }
		 *        } ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Use as a function to create a link from the data source
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [ {
		 *          "targets": [ 0 ],
		 *          "data": "download_link",
		 *          "render": function ( data, type, full ) {
		 *            return '<a href="'+data+'">Download</a>';
		 *          }
		 *        } ]
		 *      } );
		 *    } );
		 */
		"mRender": null,
	
	
		/**
		 * Change the cell type created for the column - either TD cells or TH cells. This
		 * can be useful as TH cells have semantic meaning in the table body, allowing them
		 * to act as a header for a row (you may wish to add scope='row' to the TH elements).
		 *  @type string
		 *  @default td
		 *
		 *  @name DataTable.defaults.column.cellType
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Make the first column use TH cells
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [ {
		 *          "targets": [ 0 ],
		 *          "cellType": "th"
		 *        } ]
		 *      } );
		 *    } );
		 */
		"sCellType": "td",
	
	
		/**
		 * Class to give to each cell in this column.
		 *  @type string
		 *  @default <i>Empty string</i>
		 *
		 *  @name DataTable.defaults.column.class
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columnDefs`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [
		 *          { "class": "my_class", "targets": [ 0 ] }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          { "class": "my_class" },
		 *          null,
		 *          null,
		 *          null,
		 *          null
		 *        ]
		 *      } );
		 *    } );
		 */
		"sClass": "",
	
		/**
		 * When DataTables calculates the column widths to assign to each column,
		 * it finds the longest string in each column and then constructs a
		 * temporary table and reads the widths from that. The problem with this
		 * is that "mmm" is much wider then "iiii", but the latter is a longer
		 * string - thus the calculation can go wrong (doing it properly and putting
		 * it into an DOM object and measuring that is horribly(!) slow). Thus as
		 * a "work around" we provide this option. It will append its value to the
		 * text that is found to be the longest string for the column - i.e. padding.
		 * Generally you shouldn't need this!
		 *  @type string
		 *  @default <i>Empty string<i>
		 *
		 *  @name DataTable.defaults.column.contentPadding
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          null,
		 *          null,
		 *          null,
		 *          {
		 *            "contentPadding": "mmm"
		 *          }
		 *        ]
		 *      } );
		 *    } );
		 */
		"sContentPadding": "",
	
	
		/**
		 * Allows a default value to be given for a column's data, and will be used
		 * whenever a null data source is encountered (this can be because `data`
		 * is set to null, or because the data source itself is null).
		 *  @type string
		 *  @default null
		 *
		 *  @name DataTable.defaults.column.defaultContent
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columnDefs`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [
		 *          {
		 *            "data": null,
		 *            "defaultContent": "Edit",
		 *            "targets": [ -1 ]
		 *          }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          null,
		 *          null,
		 *          null,
		 *          {
		 *            "data": null,
		 *            "defaultContent": "Edit"
		 *          }
		 *        ]
		 *      } );
		 *    } );
		 */
		"sDefaultContent": null,
	
	
		/**
		 * This parameter is only used in DataTables' server-side processing. It can
		 * be exceptionally useful to know what columns are being displayed on the
		 * client side, and to map these to database fields. When defined, the names
		 * also allow DataTables to reorder information from the server if it comes
		 * back in an unexpected order (i.e. if you switch your columns around on the
		 * client-side, your server-side code does not also need updating).
		 *  @type string
		 *  @default <i>Empty string</i>
		 *
		 *  @name DataTable.defaults.column.name
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columnDefs`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [
		 *          { "name": "engine", "targets": [ 0 ] },
		 *          { "name": "browser", "targets": [ 1 ] },
		 *          { "name": "platform", "targets": [ 2 ] },
		 *          { "name": "version", "targets": [ 3 ] },
		 *          { "name": "grade", "targets": [ 4 ] }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          { "name": "engine" },
		 *          { "name": "browser" },
		 *          { "name": "platform" },
		 *          { "name": "version" },
		 *          { "name": "grade" }
		 *        ]
		 *      } );
		 *    } );
		 */
		"sName": "",
	
	
		/**
		 * Defines a data source type for the ordering which can be used to read
		 * real-time information from the table (updating the internally cached
		 * version) prior to ordering. This allows ordering to occur on user
		 * editable elements such as form inputs.
		 *  @type string
		 *  @default std
		 *
		 *  @name DataTable.defaults.column.orderDataType
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columnDefs`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [
		 *          { "orderDataType": "dom-text", "targets": [ 2, 3 ] },
		 *          { "type": "numeric", "targets": [ 3 ] },
		 *          { "orderDataType": "dom-select", "targets": [ 4 ] },
		 *          { "orderDataType": "dom-checkbox", "targets": [ 5 ] }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          null,
		 *          null,
		 *          { "orderDataType": "dom-text" },
		 *          { "orderDataType": "dom-text", "type": "numeric" },
		 *          { "orderDataType": "dom-select" },
		 *          { "orderDataType": "dom-checkbox" }
		 *        ]
		 *      } );
		 *    } );
		 */
		"sSortDataType": "std",
	
	
		/**
		 * The title of this column.
		 *  @type string
		 *  @default null <i>Derived from the 'TH' value for this column in the
		 *    original HTML table.</i>
		 *
		 *  @name DataTable.defaults.column.title
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columnDefs`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [
		 *          { "title": "My column title", "targets": [ 0 ] }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          { "title": "My column title" },
		 *          null,
		 *          null,
		 *          null,
		 *          null
		 *        ]
		 *      } );
		 *    } );
		 */
		"sTitle": null,
	
	
		/**
		 * The type allows you to specify how the data for this column will be
		 * ordered. Four types (string, numeric, date and html (which will strip
		 * HTML tags before ordering)) are currently available. Note that only date
		 * formats understood by Javascript's Date() object will be accepted as type
		 * date. For example: "Mar 26, 2008 5:03 PM". May take the values: 'string',
		 * 'numeric', 'date' or 'html' (by default). Further types can be adding
		 * through plug-ins.
		 *  @type string
		 *  @default null <i>Auto-detected from raw data</i>
		 *
		 *  @name DataTable.defaults.column.type
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columnDefs`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [
		 *          { "type": "html", "targets": [ 0 ] }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          { "type": "html" },
		 *          null,
		 *          null,
		 *          null,
		 *          null
		 *        ]
		 *      } );
		 *    } );
		 */
		"sType": null,
	
	
		/**
		 * Defining the width of the column, this parameter may take any CSS value
		 * (3em, 20px etc). DataTables applies 'smart' widths to columns which have not
		 * been given a specific width through this interface ensuring that the table
		 * remains readable.
		 *  @type string
		 *  @default null <i>Automatic</i>
		 *
		 *  @name DataTable.defaults.column.width
		 *  @dtopt Columns
		 *
		 *  @example
		 *    // Using `columnDefs`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columnDefs": [
		 *          { "width": "20%", "targets": [ 0 ] }
		 *        ]
		 *      } );
		 *    } );
		 *
		 *  @example
		 *    // Using `columns`
		 *    $(document).ready( function() {
		 *      $('#example').dataTable( {
		 *        "columns": [
		 *          { "width": "20%" },
		 *          null,
		 *          null,
		 *          null,
		 *          null
		 *        ]
		 *      } );
		 *    } );
		 */
		"sWidth": null
	};
	
	_fnHungarianMap( DataTable.defaults.column );
	
	
	
	/**
	 * DataTables settings object - this holds all the information needed for a
	 * given table, including configuration, data and current application of the
	 * table options. DataTables does not have a single instance for each DataTable
	 * with the settings attached to that instance, but rather instances of the
	 * DataTable "class" are created on-the-fly as needed (typically by a
	 * $().dataTable() call) and the settings object is then applied to that
	 * instance.
	 *
	 * Note that this object is related to {@link DataTable.defaults} but this
	 * one is the internal data store for DataTables's cache of columns. It should
	 * NOT be manipulated outside of DataTables. Any configuration should be done
	 * through the initialisation options.
	 *  @namespace
	 *  @todo Really should attach the settings object to individual instances so we
	 *    don't need to create new instances on each $().dataTable() call (if the
	 *    table already exists). It would also save passing oSettings around and
	 *    into every single function. However, this is a very significant
	 *    architecture change for DataTables and will almost certainly break
	 *    backwards compatibility with older installations. This is something that
	 *    will be done in 2.0.
	 */
	DataTable.models.oSettings = {
		/**
		 * Primary features of DataTables and their enablement state.
		 *  @namespace
		 */
		"oFeatures": {
	
			/**
			 * Flag to say if DataTables should automatically try to calculate the
			 * optimum table and columns widths (true) or not (false).
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bAutoWidth": null,
	
			/**
			 * Delay the creation of TR and TD elements until they are actually
			 * needed by a driven page draw. This can give a significant speed
			 * increase for Ajax source and Javascript source data, but makes no
			 * difference at all fro DOM and server-side processing tables.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bDeferRender": null,
	
			/**
			 * Enable filtering on the table or not. Note that if this is disabled
			 * then there is no filtering at all on the table, including fnFilter.
			 * To just remove the filtering input use sDom and remove the 'f' option.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bFilter": null,
	
			/**
			 * Table information element (the 'Showing x of y records' div) enable
			 * flag.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bInfo": null,
	
			/**
			 * Present a user control allowing the end user to change the page size
			 * when pagination is enabled.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bLengthChange": null,
	
			/**
			 * Pagination enabled or not. Note that if this is disabled then length
			 * changing must also be disabled.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bPaginate": null,
	
			/**
			 * Processing indicator enable flag whenever DataTables is enacting a
			 * user request - typically an Ajax request for server-side processing.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bProcessing": null,
	
			/**
			 * Server-side processing enabled flag - when enabled DataTables will
			 * get all data from the server for every draw - there is no filtering,
			 * sorting or paging done on the client-side.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bServerSide": null,
	
			/**
			 * Sorting enablement flag.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bSort": null,
	
			/**
			 * Multi-column sorting
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bSortMulti": null,
	
			/**
			 * Apply a class to the columns which are being sorted to provide a
			 * visual highlight or not. This can slow things down when enabled since
			 * there is a lot of DOM interaction.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bSortClasses": null,
	
			/**
			 * State saving enablement flag.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bStateSave": null
		},
	
	
		/**
		 * Scrolling settings for a table.
		 *  @namespace
		 */
		"oScroll": {
			/**
			 * When the table is shorter in height than sScrollY, collapse the
			 * table container down to the height of the table (when true).
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type boolean
			 */
			"bCollapse": null,
	
			/**
			 * Width of the scrollbar for the web-browser's platform. Calculated
			 * during table initialisation.
			 *  @type int
			 *  @default 0
			 */
			"iBarWidth": 0,
	
			/**
			 * Viewport width for horizontal scrolling. Horizontal scrolling is
			 * disabled if an empty string.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type string
			 */
			"sX": null,
	
			/**
			 * Width to expand the table to when using x-scrolling. Typically you
			 * should not need to use this.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type string
			 *  @deprecated
			 */
			"sXInner": null,
	
			/**
			 * Viewport height for vertical scrolling. Vertical scrolling is disabled
			 * if an empty string.
			 * Note that this parameter will be set by the initialisation routine. To
			 * set a default use {@link DataTable.defaults}.
			 *  @type string
			 */
			"sY": null
		},
	
		/**
		 * Language information for the table.
		 *  @namespace
		 *  @extends DataTable.defaults.oLanguage
		 */
		"oLanguage": {
			/**
			 * Information callback function. See
			 * {@link DataTable.defaults.fnInfoCallback}
			 *  @type function
			 *  @default null
			 */
			"fnInfoCallback": null
		},
	
		/**
		 * Browser support parameters
		 *  @namespace
		 */
		"oBrowser": {
			/**
			 * Indicate if the browser incorrectly calculates width:100% inside a
			 * scrolling element (IE6/7)
			 *  @type boolean
			 *  @default false
			 */
			"bScrollOversize": false,
	
			/**
			 * Determine if the vertical scrollbar is on the right or left of the
			 * scrolling container - needed for rtl language layout, although not
			 * all browsers move the scrollbar (Safari).
			 *  @type boolean
			 *  @default false
			 */
			"bScrollbarLeft": false
		},
	
	
		"ajax": null,
	
	
		/**
		 * Array referencing the nodes which are used for the features. The
		 * parameters of this object match what is allowed by sDom - i.e.
		 *   <ul>
		 *     <li>'l' - Length changing</li>
		 *     <li>'f' - Filtering input</li>
		 *     <li>'t' - The table!</li>
		 *     <li>'i' - Information</li>
		 *     <li>'p' - Pagination</li>
		 *     <li>'r' - pRocessing</li>
		 *   </ul>
		 *  @type array
		 *  @default []
		 */
		"aanFeatures": [],
	
		/**
		 * Store data information - see {@link DataTable.models.oRow} for detailed
		 * information.
		 *  @type array
		 *  @default []
		 */
		"aoData": [],
	
		/**
		 * Array of indexes which are in the current display (after filtering etc)
		 *  @type array
		 *  @default []
		 */
		"aiDisplay": [],
	
		/**
		 * Array of indexes for display - no filtering
		 *  @type array
		 *  @default []
		 */
		"aiDisplayMaster": [],
	
		/**
		 * Store information about each column that is in use
		 *  @type array
		 *  @default []
		 */
		"aoColumns": [],
	
		/**
		 * Store information about the table's header
		 *  @type array
		 *  @default []
		 */
		"aoHeader": [],
	
		/**
		 * Store information about the table's footer
		 *  @type array
		 *  @default []
		 */
		"aoFooter": [],
	
		/**
		 * Store the applied global search information in case we want to force a
		 * research or compare the old search to a new one.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @namespace
		 *  @extends DataTable.models.oSearch
		 */
		"oPreviousSearch": {},
	
		/**
		 * Store the applied search for each column - see
		 * {@link DataTable.models.oSearch} for the format that is used for the
		 * filtering information for each column.
		 *  @type array
		 *  @default []
		 */
		"aoPreSearchCols": [],
	
		/**
		 * Sorting that is applied to the table. Note that the inner arrays are
		 * used in the following manner:
		 * <ul>
		 *   <li>Index 0 - column number</li>
		 *   <li>Index 1 - current sorting direction</li>
		 * </ul>
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type array
		 *  @todo These inner arrays should really be objects
		 */
		"aaSorting": null,
	
		/**
		 * Sorting that is always applied to the table (i.e. prefixed in front of
		 * aaSorting).
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type array
		 *  @default []
		 */
		"aaSortingFixed": [],
	
		/**
		 * Classes to use for the striping of a table.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type array
		 *  @default []
		 */
		"asStripeClasses": null,
	
		/**
		 * If restoring a table - we should restore its striping classes as well
		 *  @type array
		 *  @default []
		 */
		"asDestroyStripes": [],
	
		/**
		 * If restoring a table - we should restore its width
		 *  @type int
		 *  @default 0
		 */
		"sDestroyWidth": 0,
	
		/**
		 * Callback functions array for every time a row is inserted (i.e. on a draw).
		 *  @type array
		 *  @default []
		 */
		"aoRowCallback": [],
	
		/**
		 * Callback functions for the header on each draw.
		 *  @type array
		 *  @default []
		 */
		"aoHeaderCallback": [],
	
		/**
		 * Callback function for the footer on each draw.
		 *  @type array
		 *  @default []
		 */
		"aoFooterCallback": [],
	
		/**
		 * Array of callback functions for draw callback functions
		 *  @type array
		 *  @default []
		 */
		"aoDrawCallback": [],
	
		/**
		 * Array of callback functions for row created function
		 *  @type array
		 *  @default []
		 */
		"aoRowCreatedCallback": [],
	
		/**
		 * Callback functions for just before the table is redrawn. A return of
		 * false will be used to cancel the draw.
		 *  @type array
		 *  @default []
		 */
		"aoPreDrawCallback": [],
	
		/**
		 * Callback functions for when the table has been initialised.
		 *  @type array
		 *  @default []
		 */
		"aoInitComplete": [],
	
	
		/**
		 * Callbacks for modifying the settings to be stored for state saving, prior to
		 * saving state.
		 *  @type array
		 *  @default []
		 */
		"aoStateSaveParams": [],
	
		/**
		 * Callbacks for modifying the settings that have been stored for state saving
		 * prior to using the stored values to restore the state.
		 *  @type array
		 *  @default []
		 */
		"aoStateLoadParams": [],
	
		/**
		 * Callbacks for operating on the settings object once the saved state has been
		 * loaded
		 *  @type array
		 *  @default []
		 */
		"aoStateLoaded": [],
	
		/**
		 * Cache the table ID for quick access
		 *  @type string
		 *  @default <i>Empty string</i>
		 */
		"sTableId": "",
	
		/**
		 * The TABLE node for the main table
		 *  @type node
		 *  @default null
		 */
		"nTable": null,
	
		/**
		 * Permanent ref to the thead element
		 *  @type node
		 *  @default null
		 */
		"nTHead": null,
	
		/**
		 * Permanent ref to the tfoot element - if it exists
		 *  @type node
		 *  @default null
		 */
		"nTFoot": null,
	
		/**
		 * Permanent ref to the tbody element
		 *  @type node
		 *  @default null
		 */
		"nTBody": null,
	
		/**
		 * Cache the wrapper node (contains all DataTables controlled elements)
		 *  @type node
		 *  @default null
		 */
		"nTableWrapper": null,
	
		/**
		 * Indicate if when using server-side processing the loading of data
		 * should be deferred until the second draw.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type boolean
		 *  @default false
		 */
		"bDeferLoading": false,
	
		/**
		 * Indicate if all required information has been read in
		 *  @type boolean
		 *  @default false
		 */
		"bInitialised": false,
	
		/**
		 * Information about open rows. Each object in the array has the parameters
		 * 'nTr' and 'nParent'
		 *  @type array
		 *  @default []
		 */
		"aoOpenRows": [],
	
		/**
		 * Dictate the positioning of DataTables' control elements - see
		 * {@link DataTable.model.oInit.sDom}.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type string
		 *  @default null
		 */
		"sDom": null,
	
		/**
		 * Search delay (in mS)
		 *  @type integer
		 *  @default null
		 */
		"searchDelay": null,
	
		/**
		 * Which type of pagination should be used.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type string
		 *  @default two_button
		 */
		"sPaginationType": "two_button",
	
		/**
		 * The state duration (for `stateSave`) in seconds.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type int
		 *  @default 0
		 */
		"iStateDuration": 0,
	
		/**
		 * Array of callback functions for state saving. Each array element is an
		 * object with the following parameters:
		 *   <ul>
		 *     <li>function:fn - function to call. Takes two parameters, oSettings
		 *       and the JSON string to save that has been thus far created. Returns
		 *       a JSON string to be inserted into a json object
		 *       (i.e. '"param": [ 0, 1, 2]')</li>
		 *     <li>string:sName - name of callback</li>
		 *   </ul>
		 *  @type array
		 *  @default []
		 */
		"aoStateSave": [],
	
		/**
		 * Array of callback functions for state loading. Each array element is an
		 * object with the following parameters:
		 *   <ul>
		 *     <li>function:fn - function to call. Takes two parameters, oSettings
		 *       and the object stored. May return false to cancel state loading</li>
		 *     <li>string:sName - name of callback</li>
		 *   </ul>
		 *  @type array
		 *  @default []
		 */
		"aoStateLoad": [],
	
		/**
		 * State that was saved. Useful for back reference
		 *  @type object
		 *  @default null
		 */
		"oSavedState": null,
	
		/**
		 * State that was loaded. Useful for back reference
		 *  @type object
		 *  @default null
		 */
		"oLoadedState": null,
	
		/**
		 * Source url for AJAX data for the table.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type string
		 *  @default null
		 */
		"sAjaxSource": null,
	
		/**
		 * Property from a given object from which to read the table data from. This
		 * can be an empty string (when not server-side processing), in which case
		 * it is  assumed an an array is given directly.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type string
		 */
		"sAjaxDataProp": null,
	
		/**
		 * Note if draw should be blocked while getting data
		 *  @type boolean
		 *  @default true
		 */
		"bAjaxDataGet": true,
	
		/**
		 * The last jQuery XHR object that was used for server-side data gathering.
		 * This can be used for working with the XHR information in one of the
		 * callbacks
		 *  @type object
		 *  @default null
		 */
		"jqXHR": null,
	
		/**
		 * JSON returned from the server in the last Ajax request
		 *  @type object
		 *  @default undefined
		 */
		"json": undefined,
	
		/**
		 * Data submitted as part of the last Ajax request
		 *  @type object
		 *  @default undefined
		 */
		"oAjaxData": undefined,
	
		/**
		 * Function to get the server-side data.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type function
		 */
		"fnServerData": null,
	
		/**
		 * Functions which are called prior to sending an Ajax request so extra
		 * parameters can easily be sent to the server
		 *  @type array
		 *  @default []
		 */
		"aoServerParams": [],
	
		/**
		 * Send the XHR HTTP method - GET or POST (could be PUT or DELETE if
		 * required).
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type string
		 */
		"sServerMethod": null,
	
		/**
		 * Format numbers for display.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type function
		 */
		"fnFormatNumber": null,
	
		/**
		 * List of options that can be used for the user selectable length menu.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type array
		 *  @default []
		 */
		"aLengthMenu": null,
	
		/**
		 * Counter for the draws that the table does. Also used as a tracker for
		 * server-side processing
		 *  @type int
		 *  @default 0
		 */
		"iDraw": 0,
	
		/**
		 * Indicate if a redraw is being done - useful for Ajax
		 *  @type boolean
		 *  @default false
		 */
		"bDrawing": false,
	
		/**
		 * Draw index (iDraw) of the last error when parsing the returned data
		 *  @type int
		 *  @default -1
		 */
		"iDrawError": -1,
	
		/**
		 * Paging display length
		 *  @type int
		 *  @default 10
		 */
		"_iDisplayLength": 10,
	
		/**
		 * Paging start point - aiDisplay index
		 *  @type int
		 *  @default 0
		 */
		"_iDisplayStart": 0,
	
		/**
		 * Server-side processing - number of records in the result set
		 * (i.e. before filtering), Use fnRecordsTotal rather than
		 * this property to get the value of the number of records, regardless of
		 * the server-side processing setting.
		 *  @type int
		 *  @default 0
		 *  @private
		 */
		"_iRecordsTotal": 0,
	
		/**
		 * Server-side processing - number of records in the current display set
		 * (i.e. after filtering). Use fnRecordsDisplay rather than
		 * this property to get the value of the number of records, regardless of
		 * the server-side processing setting.
		 *  @type boolean
		 *  @default 0
		 *  @private
		 */
		"_iRecordsDisplay": 0,
	
		/**
		 * Flag to indicate if jQuery UI marking and classes should be used.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type boolean
		 */
		"bJUI": null,
	
		/**
		 * The classes to use for the table
		 *  @type object
		 *  @default {}
		 */
		"oClasses": {},
	
		/**
		 * Flag attached to the settings object so you can check in the draw
		 * callback if filtering has been done in the draw. Deprecated in favour of
		 * events.
		 *  @type boolean
		 *  @default false
		 *  @deprecated
		 */
		"bFiltered": false,
	
		/**
		 * Flag attached to the settings object so you can check in the draw
		 * callback if sorting has been done in the draw. Deprecated in favour of
		 * events.
		 *  @type boolean
		 *  @default false
		 *  @deprecated
		 */
		"bSorted": false,
	
		/**
		 * Indicate that if multiple rows are in the header and there is more than
		 * one unique cell per column, if the top one (true) or bottom one (false)
		 * should be used for sorting / title by DataTables.
		 * Note that this parameter will be set by the initialisation routine. To
		 * set a default use {@link DataTable.defaults}.
		 *  @type boolean
		 */
		"bSortCellsTop": null,
	
		/**
		 * Initialisation object that is used for the table
		 *  @type object
		 *  @default null
		 */
		"oInit": null,
	
		/**
		 * Destroy callback functions - for plug-ins to attach themselves to the
		 * destroy so they can clean up markup and events.
		 *  @type array
		 *  @default []
		 */
		"aoDestroyCallback": [],
	
	
		/**
		 * Get the number of records in the current record set, before filtering
		 *  @type function
		 */
		"fnRecordsTotal": function ()
		{
			return _fnDataSource( this ) == 'ssp' ?
				this._iRecordsTotal * 1 :
				this.aiDisplayMaster.length;
		},
	
		/**
		 * Get the number of records in the current record set, after filtering
		 *  @type function
		 */
		"fnRecordsDisplay": function ()
		{
			return _fnDataSource( this ) == 'ssp' ?
				this._iRecordsDisplay * 1 :
				this.aiDisplay.length;
		},
	
		/**
		 * Get the display end point - aiDisplay index
		 *  @type function
		 */
		"fnDisplayEnd": function ()
		{
			var
				len      = this._iDisplayLength,
				start    = this._iDisplayStart,
				calc     = start + len,
				records  = this.aiDisplay.length,
				features = this.oFeatures,
				paginate = features.bPaginate;
	
			if ( features.bServerSide ) {
				return paginate === false || len === -1 ?
					start + records :
					Math.min( start+len, this._iRecordsDisplay );
			}
			else {
				return ! paginate || calc>records || len===-1 ?
					records :
					calc;
			}
		},
	
		/**
		 * The DataTables object for this table
		 *  @type object
		 *  @default null
		 */
		"oInstance": null,
	
		/**
		 * Unique identifier for each instance of the DataTables object. If there
		 * is an ID on the table node, then it takes that value, otherwise an
		 * incrementing internal counter is used.
		 *  @type string
		 *  @default null
		 */
		"sInstance": null,
	
		/**
		 * tabindex attribute value that is added to DataTables control elements, allowing
		 * keyboard navigation of the table and its controls.
		 */
		"iTabIndex": 0,
	
		/**
		 * DIV container for the footer scrolling table if scrolling
		 */
		"nScrollHead": null,
	
		/**
		 * DIV container for the footer scrolling table if scrolling
		 */
		"nScrollFoot": null,
	
		/**
		 * Last applied sort
		 *  @type array
		 *  @default []
		 */
		"aLastSort": [],
	
		/**
		 * Stored plug-in instances
		 *  @type object
		 *  @default {}
		 */
		"oPlugins": {}
	};

	/**
	 * Extension object for DataTables that is used to provide all extension
	 * options.
	 *
	 * Note that the `DataTable.ext` object is available through
	 * `jQuery.fn.dataTable.ext` where it may be accessed and manipulated. It is
	 * also aliased to `jQuery.fn.dataTableExt` for historic reasons.
	 *  @namespace
	 *  @extends DataTable.models.ext
	 */
	
	
	/**
	 * DataTables extensions
	 * 
	 * This namespace acts as a collection area for plug-ins that can be used to
	 * extend DataTables capabilities. Indeed many of the build in methods
	 * use this method to provide their own capabilities (sorting methods for
	 * example).
	 *
	 * Note that this namespace is aliased to `jQuery.fn.dataTableExt` for legacy
	 * reasons
	 *
	 *  @namespace
	 */
	DataTable.ext = _ext = {
		/**
		 * Buttons. For use with the Buttons extension for DataTables. This is
		 * defined here so other extensions can define buttons regardless of load
		 * order. It is _not_ used by DataTables core.
		 *
		 *  @type object
		 *  @default {}
		 */
		buttons: {},
	
	
		/**
		 * Element class names
		 *
		 *  @type object
		 *  @default {}
		 */
		classes: {},
	
	
		/**
		 * Error reporting.
		 * 
		 * How should DataTables report an error. Can take the value 'alert',
		 * 'throw', 'none' or a function.
		 *
		 *  @type string|function
		 *  @default alert
		 */
		errMode: "alert",
	
	
		/**
		 * Feature plug-ins.
		 * 
		 * This is an array of objects which describe the feature plug-ins that are
		 * available to DataTables. These feature plug-ins are then available for
		 * use through the `dom` initialisation option.
		 * 
		 * Each feature plug-in is described by an object which must have the
		 * following properties:
		 * 
		 * * `fnInit` - function that is used to initialise the plug-in,
		 * * `cFeature` - a character so the feature can be enabled by the `dom`
		 *   instillation option. This is case sensitive.
		 *
		 * The `fnInit` function has the following input parameters:
		 *
		 * 1. `{object}` DataTables settings object: see
		 *    {@link DataTable.models.oSettings}
		 *
		 * And the following return is expected:
		 * 
		 * * {node|null} The element which contains your feature. Note that the
		 *   return may also be void if your plug-in does not require to inject any
		 *   DOM elements into DataTables control (`dom`) - for example this might
		 *   be useful when developing a plug-in which allows table control via
		 *   keyboard entry
		 *
		 *  @type array
		 *
		 *  @example
		 *    $.fn.dataTable.ext.features.push( {
		 *      "fnInit": function( oSettings ) {
		 *        return new TableTools( { "oDTSettings": oSettings } );
		 *      },
		 *      "cFeature": "T"
		 *    } );
		 */
		feature: [],
	
	
		/**
		 * Row searching.
		 * 
		 * This method of searching is complimentary to the default type based
		 * searching, and a lot more comprehensive as it allows you complete control
		 * over the searching logic. Each element in this array is a function
		 * (parameters described below) that is called for every row in the table,
		 * and your logic decides if it should be included in the searching data set
		 * or not.
		 *
		 * Searching functions have the following input parameters:
		 *
		 * 1. `{object}` DataTables settings object: see
		 *    {@link DataTable.models.oSettings}
		 * 2. `{array|object}` Data for the row to be processed (same as the
		 *    original format that was passed in as the data source, or an array
		 *    from a DOM data source
		 * 3. `{int}` Row index ({@link DataTable.models.oSettings.aoData}), which
		 *    can be useful to retrieve the `TR` element if you need DOM interaction.
		 *
		 * And the following return is expected:
		 *
		 * * {boolean} Include the row in the searched result set (true) or not
		 *   (false)
		 *
		 * Note that as with the main search ability in DataTables, technically this
		 * is "filtering", since it is subtractive. However, for consistency in
		 * naming we call it searching here.
		 *
		 *  @type array
		 *  @default []
		 *
		 *  @example
		 *    // The following example shows custom search being applied to the
		 *    // fourth column (i.e. the data[3] index) based on two input values
		 *    // from the end-user, matching the data in a certain range.
		 *    $.fn.dataTable.ext.search.push(
		 *      function( settings, data, dataIndex ) {
		 *        var min = document.getElementById('min').value * 1;
		 *        var max = document.getElementById('max').value * 1;
		 *        var version = data[3] == "-" ? 0 : data[3]*1;
		 *
		 *        if ( min == "" && max == "" ) {
		 *          return true;
		 *        }
		 *        else if ( min == "" && version < max ) {
		 *          return true;
		 *        }
		 *        else if ( min < version && "" == max ) {
		 *          return true;
		 *        }
		 *        else if ( min < version && version < max ) {
		 *          return true;
		 *        }
		 *        return false;
		 *      }
		 *    );
		 */
		search: [],
	
	
		/**
		 * Selector extensions
		 *
		 * The `selector` option can be used to extend the options available for the
		 * selector modifier options (`selector-modifier` object data type) that
		 * each of the three built in selector types offer (row, column and cell +
		 * their plural counterparts). For example the Select extension uses this
		 * mechanism to provide an option to select only rows, columns and cells
		 * that have been marked as selected by the end user (`{selected: true}`),
		 * which can be used in conjunction with the existing built in selector
		 * options.
		 *
		 * Each property is an array to which functions can be pushed. The functions
		 * take three attributes:
		 *
		 * * Settings object for the host table
		 * * Options object (`selector-modifier` object type)
		 * * Array of selected item indexes
		 *
		 * The return is an array of the resulting item indexes after the custom
		 * selector has been applied.
		 *
		 *  @type object
		 */
		selector: {
			cell: [],
			column: [],
			row: []
		},
	
	
		/**
		 * Internal functions, exposed for used in plug-ins.
		 * 
		 * Please note that you should not need to use the internal methods for
		 * anything other than a plug-in (and even then, try to avoid if possible).
		 * The internal function may change between releases.
		 *
		 *  @type object
		 *  @default {}
		 */
		internal: {},
	
	
		/**
		 * Legacy configuration options. Enable and disable legacy options that
		 * are available in DataTables.
		 *
		 *  @type object
		 */
		legacy: {
			/**
			 * Enable / disable DataTables 1.9 compatible server-side processing
			 * requests
			 *
			 *  @type boolean
			 *  @default null
			 */
			ajax: null
		},
	
	
		/**
		 * Pagination plug-in methods.
		 * 
		 * Each entry in this object is a function and defines which buttons should
		 * be shown by the pagination rendering method that is used for the table:
		 * {@link DataTable.ext.renderer.pageButton}. The renderer addresses how the
		 * buttons are displayed in the document, while the functions here tell it
		 * what buttons to display. This is done by returning an array of button
		 * descriptions (what each button will do).
		 *
		 * Pagination types (the four built in options and any additional plug-in
		 * options defined here) can be used through the `paginationType`
		 * initialisation parameter.
		 *
		 * The functions defined take two parameters:
		 *
		 * 1. `{int} page` The current page index
		 * 2. `{int} pages` The number of pages in the table
		 *
		 * Each function is expected to return an array where each element of the
		 * array can be one of:
		 *
		 * * `first` - Jump to first page when activated
		 * * `last` - Jump to last page when activated
		 * * `previous` - Show previous page when activated
		 * * `next` - Show next page when activated
		 * * `{int}` - Show page of the index given
		 * * `{array}` - A nested array containing the above elements to add a
		 *   containing 'DIV' element (might be useful for styling).
		 *
		 * Note that DataTables v1.9- used this object slightly differently whereby
		 * an object with two functions would be defined for each plug-in. That
		 * ability is still supported by DataTables 1.10+ to provide backwards
		 * compatibility, but this option of use is now decremented and no longer
		 * documented in DataTables 1.10+.
		 *
		 *  @type object
		 *  @default {}
		 *
		 *  @example
		 *    // Show previous, next and current page buttons only
		 *    $.fn.dataTableExt.oPagination.current = function ( page, pages ) {
		 *      return [ 'previous', page, 'next' ];
		 *    };
		 */
		pager: {},
	
	
		renderer: {
			pageButton: {},
			header: {}
		},
	
	
		/**
		 * Ordering plug-ins - custom data source
		 * 
		 * The extension options for ordering of data available here is complimentary
		 * to the default type based ordering that DataTables typically uses. It
		 * allows much greater control over the the data that is being used to
		 * order a column, but is necessarily therefore more complex.
		 * 
		 * This type of ordering is useful if you want to do ordering based on data
		 * live from the DOM (for example the contents of an 'input' element) rather
		 * than just the static string that DataTables knows of.
		 * 
		 * The way these plug-ins work is that you create an array of the values you
		 * wish to be ordering for the column in question and then return that
		 * array. The data in the array much be in the index order of the rows in
		 * the table (not the currently ordering order!). Which order data gathering
		 * function is run here depends on the `dt-init columns.orderDataType`
		 * parameter that is used for the column (if any).
		 *
		 * The functions defined take two parameters:
		 *
		 * 1. `{object}` DataTables settings object: see
		 *    {@link DataTable.models.oSettings}
		 * 2. `{int}` Target column index
		 *
		 * Each function is expected to return an array:
		 *
		 * * `{array}` Data for the column to be ordering upon
		 *
		 *  @type array
		 *
		 *  @example
		 *    // Ordering using `input` node values
		 *    $.fn.dataTable.ext.order['dom-text'] = function  ( settings, col )
		 *    {
		 *      return this.api().column( col, {order:'index'} ).nodes().map( function ( td, i ) {
		 *        return $('input', td).val();
		 *      } );
		 *    }
		 */
		order: {},
	
	
		/**
		 * Type based plug-ins.
		 *
		 * Each column in DataTables has a type assigned to it, either by automatic
		 * detection or by direct assignment using the `type` option for the column.
		 * The type of a column will effect how it is ordering and search (plug-ins
		 * can also make use of the column type if required).
		 *
		 * @namespace
		 */
		type: {
			/**
			 * Type detection functions.
			 *
			 * The functions defined in this object are used to automatically detect
			 * a column's type, making initialisation of DataTables super easy, even
			 * when complex data is in the table.
			 *
			 * The functions defined take two parameters:
			 *
		     *  1. `{*}` Data from the column cell to be analysed
		     *  2. `{settings}` DataTables settings object. This can be used to
		     *     perform context specific type detection - for example detection
		     *     based on language settings such as using a comma for a decimal
		     *     place. Generally speaking the options from the settings will not
		     *     be required
			 *
			 * Each function is expected to return:
			 *
			 * * `{string|null}` Data type detected, or null if unknown (and thus
			 *   pass it on to the other type detection functions.
			 *
			 *  @type array
			 *
			 *  @example
			 *    // Currency type detection plug-in:
			 *    $.fn.dataTable.ext.type.detect.push(
			 *      function ( data, settings ) {
			 *        // Check the numeric part
			 *        if ( ! $.isNumeric( data.substring(1) ) ) {
			 *          return null;
			 *        }
			 *
			 *        // Check prefixed by currency
			 *        if ( data.charAt(0) == '$' || data.charAt(0) == '&pound;' ) {
			 *          return 'currency';
			 *        }
			 *        return null;
			 *      }
			 *    );
			 */
			detect: [],
	
	
			/**
			 * Type based search formatting.
			 *
			 * The type based searching functions can be used to pre-format the
			 * data to be search on. For example, it can be used to strip HTML
			 * tags or to de-format telephone numbers for numeric only searching.
			 *
			 * Note that is a search is not defined for a column of a given type,
			 * no search formatting will be performed.
			 * 
			 * Pre-processing of searching data plug-ins - When you assign the sType
			 * for a column (or have it automatically detected for you by DataTables
			 * or a type detection plug-in), you will typically be using this for
			 * custom sorting, but it can also be used to provide custom searching
			 * by allowing you to pre-processing the data and returning the data in
			 * the format that should be searched upon. This is done by adding
			 * functions this object with a parameter name which matches the sType
			 * for that target column. This is the corollary of <i>afnSortData</i>
			 * for searching data.
			 *
			 * The functions defined take a single parameter:
			 *
		     *  1. `{*}` Data from the column cell to be prepared for searching
			 *
			 * Each function is expected to return:
			 *
			 * * `{string|null}` Formatted string that will be used for the searching.
			 *
			 *  @type object
			 *  @default {}
			 *
			 *  @example
			 *    $.fn.dataTable.ext.type.search['title-numeric'] = function ( d ) {
			 *      return d.replace(/\n/g," ").replace( /<.*?>/g, "" );
			 *    }
			 */
			search: {},
	
	
			/**
			 * Type based ordering.
			 *
			 * The column type tells DataTables what ordering to apply to the table
			 * when a column is sorted upon. The order for each type that is defined,
			 * is defined by the functions available in this object.
			 *
			 * Each ordering option can be described by three properties added to
			 * this object:
			 *
			 * * `{type}-pre` - Pre-formatting function
			 * * `{type}-asc` - Ascending order function
			 * * `{type}-desc` - Descending order function
			 *
			 * All three can be used together, only `{type}-pre` or only
			 * `{type}-asc` and `{type}-desc` together. It is generally recommended
			 * that only `{type}-pre` is used, as this provides the optimal
			 * implementation in terms of speed, although the others are provided
			 * for compatibility with existing Javascript sort functions.
			 *
			 * `{type}-pre`: Functions defined take a single parameter:
			 *
		     *  1. `{*}` Data from the column cell to be prepared for ordering
			 *
			 * And return:
			 *
			 * * `{*}` Data to be sorted upon
			 *
			 * `{type}-asc` and `{type}-desc`: Functions are typical Javascript sort
			 * functions, taking two parameters:
			 *
		     *  1. `{*}` Data to compare to the second parameter
		     *  2. `{*}` Data to compare to the first parameter
			 *
			 * And returning:
			 *
			 * * `{*}` Ordering match: <0 if first parameter should be sorted lower
			 *   than the second parameter, ===0 if the two parameters are equal and
			 *   >0 if the first parameter should be sorted height than the second
			 *   parameter.
			 * 
			 *  @type object
			 *  @default {}
			 *
			 *  @example
			 *    // Numeric ordering of formatted numbers with a pre-formatter
			 *    $.extend( $.fn.dataTable.ext.type.order, {
			 *      "string-pre": function(x) {
			 *        a = (a === "-" || a === "") ? 0 : a.replace( /[^\d\-\.]/g, "" );
			 *        return parseFloat( a );
			 *      }
			 *    } );
			 *
			 *  @example
			 *    // Case-sensitive string ordering, with no pre-formatting method
			 *    $.extend( $.fn.dataTable.ext.order, {
			 *      "string-case-asc": function(x,y) {
			 *        return ((x < y) ? -1 : ((x > y) ? 1 : 0));
			 *      },
			 *      "string-case-desc": function(x,y) {
			 *        return ((x < y) ? 1 : ((x > y) ? -1 : 0));
			 *      }
			 *    } );
			 */
			order: {}
		},
	
		/**
		 * Unique DataTables instance counter
		 *
		 * @type int
		 * @private
		 */
		_unique: 0,
	
	
		//
		// Depreciated
		// The following properties are retained for backwards compatiblity only.
		// The should not be used in new projects and will be removed in a future
		// version
		//
	
		/**
		 * Version check function.
		 *  @type function
		 *  @depreciated Since 1.10
		 */
		fnVersionCheck: DataTable.fnVersionCheck,
	
	
		/**
		 * Index for what 'this' index API functions should use
		 *  @type int
		 *  @deprecated Since v1.10
		 */
		iApiIndex: 0,
	
	
		/**
		 * jQuery UI class container
		 *  @type object
		 *  @deprecated Since v1.10
		 */
		oJUIClasses: {},
	
	
		/**
		 * Software version
		 *  @type string
		 *  @deprecated Since v1.10
		 */
		sVersion: DataTable.version
	};
	
	
	//
	// Backwards compatibility. Alias to pre 1.10 Hungarian notation counter parts
	//
	$.extend( _ext, {
		afnFiltering: _ext.search,
		aTypes:       _ext.type.detect,
		ofnSearch:    _ext.type.search,
		oSort:        _ext.type.order,
		afnSortData:  _ext.order,
		aoFeatures:   _ext.feature,
		oApi:         _ext.internal,
		oStdClasses:  _ext.classes,
		oPagination:  _ext.pager
	} );
	
	
	$.extend( DataTable.ext.classes, {
		"sTable": "dataTable",
		"sNoFooter": "no-footer",
	
		/* Paging buttons */
		"sPageButton": "paginate_button",
		"sPageButtonActive": "current",
		"sPageButtonDisabled": "disabled",
	
		/* Striping classes */
		"sStripeOdd": "odd",
		"sStripeEven": "even",
	
		/* Empty row */
		"sRowEmpty": "dataTables_empty",
	
		/* Features */
		"sWrapper": "dataTables_wrapper",
		"sFilter": "dataTables_filter",
		"sInfo": "dataTables_info",
		"sPaging": "dataTables_paginate paging_", /* Note that the type is postfixed */
		"sLength": "dataTables_length",
		"sProcessing": "dataTables_processing",
	
		/* Sorting */
		"sSortAsc": "sorting_asc",
		"sSortDesc": "sorting_desc",
		"sSortable": "sorting", /* Sortable in both directions */
		"sSortableAsc": "sorting_asc_disabled",
		"sSortableDesc": "sorting_desc_disabled",
		"sSortableNone": "sorting_disabled",
		"sSortColumn": "sorting_", /* Note that an int is postfixed for the sorting order */
	
		/* Filtering */
		"sFilterInput": "",
	
		/* Page length */
		"sLengthSelect": "",
	
		/* Scrolling */
		"sScrollWrapper": "dataTables_scroll",
		"sScrollHead": "dataTables_scrollHead",
		"sScrollHeadInner": "dataTables_scrollHeadInner",
		"sScrollBody": "dataTables_scrollBody",
		"sScrollFoot": "dataTables_scrollFoot",
		"sScrollFootInner": "dataTables_scrollFootInner",
	
		/* Misc */
		"sHeaderTH": "",
		"sFooterTH": "",
	
		// Deprecated
		"sSortJUIAsc": "",
		"sSortJUIDesc": "",
		"sSortJUI": "",
		"sSortJUIAscAllowed": "",
		"sSortJUIDescAllowed": "",
		"sSortJUIWrapper": "",
		"sSortIcon": "",
		"sJUIHeader": "",
		"sJUIFooter": ""
	} );
	
	
	(function() {
	
	// Reused strings for better compression. Closure compiler appears to have a
	// weird edge case where it is trying to expand strings rather than use the
	// variable version. This results in about 200 bytes being added, for very
	// little preference benefit since it this run on script load only.
	var _empty = '';
	_empty = '';
	
	var _stateDefault = _empty + 'ui-state-default';
	var _sortIcon     = _empty + 'css_right ui-icon ui-icon-';
	var _headerFooter = _empty + 'fg-toolbar ui-toolbar ui-widget-header ui-helper-clearfix';
	
	$.extend( DataTable.ext.oJUIClasses, DataTable.ext.classes, {
		/* Full numbers paging buttons */
		"sPageButton":         "fg-button ui-button "+_stateDefault,
		"sPageButtonActive":   "ui-state-disabled",
		"sPageButtonDisabled": "ui-state-disabled",
	
		/* Features */
		"sPaging": "dataTables_paginate fg-buttonset ui-buttonset fg-buttonset-multi "+
			"ui-buttonset-multi paging_", /* Note that the type is postfixed */
	
		/* Sorting */
		"sSortAsc":            _stateDefault+" sorting_asc",
		"sSortDesc":           _stateDefault+" sorting_desc",
		"sSortable":           _stateDefault+" sorting",
		"sSortableAsc":        _stateDefault+" sorting_asc_disabled",
		"sSortableDesc":       _stateDefault+" sorting_desc_disabled",
		"sSortableNone":       _stateDefault+" sorting_disabled",
		"sSortJUIAsc":         _sortIcon+"triangle-1-n",
		"sSortJUIDesc":        _sortIcon+"triangle-1-s",
		"sSortJUI":            _sortIcon+"carat-2-n-s",
		"sSortJUIAscAllowed":  _sortIcon+"carat-1-n",
		"sSortJUIDescAllowed": _sortIcon+"carat-1-s",
		"sSortJUIWrapper":     "DataTables_sort_wrapper",
		"sSortIcon":           "DataTables_sort_icon",
	
		/* Scrolling */
		"sScrollHead": "dataTables_scrollHead "+_stateDefault,
		"sScrollFoot": "dataTables_scrollFoot "+_stateDefault,
	
		/* Misc */
		"sHeaderTH":  _stateDefault,
		"sFooterTH":  _stateDefault,
		"sJUIHeader": _headerFooter+" ui-corner-tl ui-corner-tr",
		"sJUIFooter": _headerFooter+" ui-corner-bl ui-corner-br"
	} );
	
	}());
	
	
	
	var extPagination = DataTable.ext.pager;
	
	function _numbers ( page, pages ) {
		var
			numbers = [],
			buttons = extPagination.numbers_length,
			half = Math.floor( buttons / 2 ),
			i = 1;
	
		if ( pages <= buttons ) {
			numbers = _range( 0, pages );
		}
		else if ( page <= half ) {
			numbers = _range( 0, buttons-2 );
			numbers.push( 'ellipsis' );
			numbers.push( pages-1 );
		}
		else if ( page >= pages - 1 - half ) {
			numbers = _range( pages-(buttons-2), pages );
			numbers.splice( 0, 0, 'ellipsis' ); // no unshift in ie6
			numbers.splice( 0, 0, 0 );
		}
		else {
			numbers = _range( page-half+2, page+half-1 );
			numbers.push( 'ellipsis' );
			numbers.push( pages-1 );
			numbers.splice( 0, 0, 'ellipsis' );
			numbers.splice( 0, 0, 0 );
		}
	
		numbers.DT_el = 'span';
		return numbers;
	}
	
	
	$.extend( extPagination, {
		simple: function ( page, pages ) {
			return [ 'previous', 'next' ];
		},
	
		full: function ( page, pages ) {
			return [  'first', 'previous', 'next', 'last' ];
		},
	
		simple_numbers: function ( page, pages ) {
			return [ 'previous', _numbers(page, pages), 'next' ];
		},
	
		full_numbers: function ( page, pages ) {
			return [ 'first', 'previous', _numbers(page, pages), 'next', 'last' ];
		},
	
		// For testing and plug-ins to use
		_numbers: _numbers,
	
		// Number of number buttons (including ellipsis) to show. _Must be odd!_
		numbers_length: 7
	} );
	
	
	$.extend( true, DataTable.ext.renderer, {
		pageButton: {
			_: function ( settings, host, idx, buttons, page, pages ) {
				var classes = settings.oClasses;
				var lang = settings.oLanguage.oPaginate;
				var btnDisplay, btnClass, counter=0;
	
				var attach = function( container, buttons ) {
					var i, ien, node, button;
					var clickHandler = function ( e ) {
						_fnPageChange( settings, e.data.action, true );
					};
	
					for ( i=0, ien=buttons.length ; i<ien ; i++ ) {
						button = buttons[i];
	
						if ( $.isArray( button ) ) {
							var inner = $( '<'+(button.DT_el || 'div')+'/>' )
								.appendTo( container );
							attach( inner, button );
						}
						else {
							btnDisplay = '';
							btnClass = '';
	
							switch ( button ) {
								case 'ellipsis':
									container.append('<span class="ellipsis">&#x2026;</span>');
									break;
	
								case 'first':
									btnDisplay = lang.sFirst;
									btnClass = button + (page > 0 ?
										'' : ' '+classes.sPageButtonDisabled);
									break;
	
								case 'previous':
									btnDisplay = lang.sPrevious;
									btnClass = button + (page > 0 ?
										'' : ' '+classes.sPageButtonDisabled);
									break;
	
								case 'next':
									btnDisplay = lang.sNext;
									btnClass = button + (page < pages-1 ?
										'' : ' '+classes.sPageButtonDisabled);
									break;
	
								case 'last':
									btnDisplay = lang.sLast;
									btnClass = button + (page < pages-1 ?
										'' : ' '+classes.sPageButtonDisabled);
									break;
	
								default:
									btnDisplay = button + 1;
									btnClass = page === button ?
										classes.sPageButtonActive : '';
									break;
							}
	
							if ( btnDisplay ) {
								node = $('<a>', {
										'class': classes.sPageButton+' '+btnClass,
										'aria-controls': settings.sTableId,
										'data-dt-idx': counter,
										'tabindex': settings.iTabIndex,
										'id': idx === 0 && typeof button === 'string' ?
											settings.sTableId +'_'+ button :
											null
									} )
									.html( btnDisplay )
									.appendTo( container );
	
								_fnBindAction(
									node, {action: button}, clickHandler
								);
	
								counter++;
							}
						}
					}
				};
	
				// IE9 throws an 'unknown error' if document.activeElement is used
				// inside an iframe or frame. Try / catch the error. Not good for
				// accessibility, but neither are frames.
				var activeEl;
	
				try {
					// Because this approach is destroying and recreating the paging
					// elements, focus is lost on the select button which is bad for
					// accessibility. So we want to restore focus once the draw has
					// completed
					activeEl = $(document.activeElement).data('dt-idx');
				}
				catch (e) {}
	
				attach( $(host).empty(), buttons );
	
				if ( activeEl ) {
					$(host).find( '[data-dt-idx='+activeEl+']' ).focus();
				}
			}
		}
	} );
	
	
	
	// Built in type detection. See model.ext.aTypes for information about
	// what is required from this methods.
	$.extend( DataTable.ext.type.detect, [
		// Plain numbers - first since V8 detects some plain numbers as dates
		// e.g. Date.parse('55') (but not all, e.g. Date.parse('22')...).
		function ( d, settings )
		{
			var decimal = settings.oLanguage.sDecimal;
			return _isNumber( d, decimal ) ? 'num'+decimal : null;
		},
	
		// Dates (only those recognised by the browser's Date.parse)
		function ( d, settings )
		{
			// V8 will remove any unknown characters at the start and end of the
			// expression, leading to false matches such as `$245.12` or `10%` being
			// a valid date. See forum thread 18941 for detail.
			if ( d && !(d instanceof Date) && ( ! _re_date_start.test(d) || ! _re_date_end.test(d) ) ) {
				return null;
			}
			var parsed = Date.parse(d);
			return (parsed !== null && !isNaN(parsed)) || _empty(d) ? 'date' : null;
		},
	
		// Formatted numbers
		function ( d, settings )
		{
			var decimal = settings.oLanguage.sDecimal;
			return _isNumber( d, decimal, true ) ? 'num-fmt'+decimal : null;
		},
	
		// HTML numeric
		function ( d, settings )
		{
			var decimal = settings.oLanguage.sDecimal;
			return _htmlNumeric( d, decimal ) ? 'html-num'+decimal : null;
		},
	
		// HTML numeric, formatted
		function ( d, settings )
		{
			var decimal = settings.oLanguage.sDecimal;
			return _htmlNumeric( d, decimal, true ) ? 'html-num-fmt'+decimal : null;
		},
	
		// HTML (this is strict checking - there must be html)
		function ( d, settings )
		{
			return _empty( d ) || (typeof d === 'string' && d.indexOf('<') !== -1) ?
				'html' : null;
		}
	] );
	
	
	
	// Filter formatting functions. See model.ext.ofnSearch for information about
	// what is required from these methods.
	// 
	// Note that additional search methods are added for the html numbers and
	// html formatted numbers by `_addNumericSort()` when we know what the decimal
	// place is
	
	
	$.extend( DataTable.ext.type.search, {
		html: function ( data ) {
			return _empty(data) ?
				data :
				typeof data === 'string' ?
					data
						.replace( _re_new_lines, " " )
						.replace( _re_html, "" ) :
					'';
		},
	
		string: function ( data ) {
			return _empty(data) ?
				data :
				typeof data === 'string' ?
					data.replace( _re_new_lines, " " ) :
					data;
		}
	} );
	
	
	
	var __numericReplace = function ( d, decimalPlace, re1, re2 ) {
		if ( d !== 0 && (!d || d === '-') ) {
			return -Infinity;
		}
	
		// If a decimal place other than `.` is used, it needs to be given to the
		// function so we can detect it and replace with a `.` which is the only
		// decimal place Javascript recognises - it is not locale aware.
		if ( decimalPlace ) {
			d = _numToDecimal( d, decimalPlace );
		}
	
		if ( d.replace ) {
			if ( re1 ) {
				d = d.replace( re1, '' );
			}
	
			if ( re2 ) {
				d = d.replace( re2, '' );
			}
		}
	
		return d * 1;
	};
	
	
	// Add the numeric 'deformatting' functions for sorting and search. This is done
	// in a function to provide an easy ability for the language options to add
	// additional methods if a non-period decimal place is used.
	function _addNumericSort ( decimalPlace ) {
		$.each(
			{
				// Plain numbers
				"num": function ( d ) {
					return __numericReplace( d, decimalPlace );
				},
	
				// Formatted numbers
				"num-fmt": function ( d ) {
					return __numericReplace( d, decimalPlace, _re_formatted_numeric );
				},
	
				// HTML numeric
				"html-num": function ( d ) {
					return __numericReplace( d, decimalPlace, _re_html );
				},
	
				// HTML numeric, formatted
				"html-num-fmt": function ( d ) {
					return __numericReplace( d, decimalPlace, _re_html, _re_formatted_numeric );
				}
			},
			function ( key, fn ) {
				// Add the ordering method
				_ext.type.order[ key+decimalPlace+'-pre' ] = fn;
	
				// For HTML types add a search formatter that will strip the HTML
				if ( key.match(/^html\-/) ) {
					_ext.type.search[ key+decimalPlace ] = _ext.type.search.html;
				}
			}
		);
	}
	
	
	// Default sort methods
	$.extend( _ext.type.order, {
		// Dates
		"date-pre": function ( d ) {
			return Date.parse( d ) || 0;
		},
	
		// html
		"html-pre": function ( a ) {
			return _empty(a) ?
				'' :
				a.replace ?
					a.replace( /<.*?>/g, "" ).toLowerCase() :
					a+'';
		},
	
		// string
		"string-pre": function ( a ) {
			// This is a little complex, but faster than always calling toString,
			// http://jsperf.com/tostring-v-check
			return _empty(a) ?
				'' :
				typeof a === 'string' ?
					a.toLowerCase() :
					! a.toString ?
						'' :
						a.toString();
		},
	
		// string-asc and -desc are retained only for compatibility with the old
		// sort methods
		"string-asc": function ( x, y ) {
			return ((x < y) ? -1 : ((x > y) ? 1 : 0));
		},
	
		"string-desc": function ( x, y ) {
			return ((x < y) ? 1 : ((x > y) ? -1 : 0));
		}
	} );
	
	
	// Numeric sorting types - order doesn't matter here
	_addNumericSort( '' );
	
	
	$.extend( true, DataTable.ext.renderer, {
		header: {
			_: function ( settings, cell, column, classes ) {
				// No additional mark-up required
				// Attach a sort listener to update on sort - note that using the
				// `DT` namespace will allow the event to be removed automatically
				// on destroy, while the `dt` namespaced event is the one we are
				// listening for
				$(settings.nTable).on( 'order.dt.DT', function ( e, ctx, sorting, columns ) {
					if ( settings !== ctx ) { // need to check this this is the host
						return;               // table, not a nested one
					}
	
					var colIdx = column.idx;
	
					cell
						.removeClass(
							column.sSortingClass +' '+
							classes.sSortAsc +' '+
							classes.sSortDesc
						)
						.addClass( columns[ colIdx ] == 'asc' ?
							classes.sSortAsc : columns[ colIdx ] == 'desc' ?
								classes.sSortDesc :
								column.sSortingClass
						);
				} );
			},
	
			jqueryui: function ( settings, cell, column, classes ) {
				$('<div/>')
					.addClass( classes.sSortJUIWrapper )
					.append( cell.contents() )
					.append( $('<span/>')
						.addClass( classes.sSortIcon+' '+column.sSortingClassJUI )
					)
					.appendTo( cell );
	
				// Attach a sort listener to update on sort
				$(settings.nTable).on( 'order.dt.DT', function ( e, ctx, sorting, columns ) {
					if ( settings !== ctx ) {
						return;
					}
	
					var colIdx = column.idx;
	
					cell
						.removeClass( classes.sSortAsc +" "+classes.sSortDesc )
						.addClass( columns[ colIdx ] == 'asc' ?
							classes.sSortAsc : columns[ colIdx ] == 'desc' ?
								classes.sSortDesc :
								column.sSortingClass
						);
	
					cell
						.find( 'span.'+classes.sSortIcon )
						.removeClass(
							classes.sSortJUIAsc +" "+
							classes.sSortJUIDesc +" "+
							classes.sSortJUI +" "+
							classes.sSortJUIAscAllowed +" "+
							classes.sSortJUIDescAllowed
						)
						.addClass( columns[ colIdx ] == 'asc' ?
							classes.sSortJUIAsc : columns[ colIdx ] == 'desc' ?
								classes.sSortJUIDesc :
								column.sSortingClassJUI
						);
				} );
			}
		}
	} );
	
	/*
	 * Public helper functions. These aren't used internally by DataTables, or
	 * called by any of the options passed into DataTables, but they can be used
	 * externally by developers working with DataTables. They are helper functions
	 * to make working with DataTables a little bit easier.
	 */
	
	/**
	 * Helpers for `columns.render`.
	 *
	 * The options defined here can be used with the `columns.render` initialisation
	 * option to provide a display renderer. The following functions are defined:
	 *
	 * * `number` - Will format numeric data (defined by `columns.data`) for
	 *   display, retaining the original unformatted data for sorting and filtering.
	 *   It takes 4 parameters:
	 *   * `string` - Thousands grouping separator
	 *   * `string` - Decimal point indicator
	 *   * `integer` - Number of decimal points to show
	 *   * `string` (optional) - Prefix.
	 *
	 * @example
	 *   // Column definition using the number renderer
	 *   {
	 *     data: "salary",
	 *     render: $.fn.dataTable.render.number( '\'', '.', 0, '$' )
	 *   }
	 *
	 * @namespace
	 */
	DataTable.render = {
		number: function ( thousands, decimal, precision, prefix ) {
			return {
				display: function ( d ) {
					if ( typeof d !== 'number' && typeof d !== 'string' ) {
						return d;
					}
	
					var negative = d < 0 ? '-' : '';
					d = Math.abs( parseFloat( d ) );
	
					var intPart = parseInt( d, 10 );
					var floatPart = precision ?
						decimal+(d - intPart).toFixed( precision ).substring( 2 ):
						'';
	
					return negative + (prefix||'') +
						intPart.toString().replace(
							/\B(?=(\d{3})+(?!\d))/g, thousands
						) +
						floatPart;
				}
			};
		}
	};
	
	
	/*
	 * This is really a good bit rubbish this method of exposing the internal methods
	 * publicly... - To be fixed in 2.0 using methods on the prototype
	 */
	
	
	/**
	 * Create a wrapper function for exporting an internal functions to an external API.
	 *  @param {string} fn API function name
	 *  @returns {function} wrapped function
	 *  @memberof DataTable#internal
	 */
	function _fnExternApiFunc (fn)
	{
		return function() {
			var args = [_fnSettingsFromNode( this[DataTable.ext.iApiIndex] )].concat(
				Array.prototype.slice.call(arguments)
			);
			return DataTable.ext.internal[fn].apply( this, args );
		};
	}
	
	
	/**
	 * Reference to internal functions for use by plug-in developers. Note that
	 * these methods are references to internal functions and are considered to be
	 * private. If you use these methods, be aware that they are liable to change
	 * between versions.
	 *  @namespace
	 */
	$.extend( DataTable.ext.internal, {
		_fnExternApiFunc: _fnExternApiFunc,
		_fnBuildAjax: _fnBuildAjax,
		_fnAjaxUpdate: _fnAjaxUpdate,
		_fnAjaxParameters: _fnAjaxParameters,
		_fnAjaxUpdateDraw: _fnAjaxUpdateDraw,
		_fnAjaxDataSrc: _fnAjaxDataSrc,
		_fnAddColumn: _fnAddColumn,
		_fnColumnOptions: _fnColumnOptions,
		_fnAdjustColumnSizing: _fnAdjustColumnSizing,
		_fnVisibleToColumnIndex: _fnVisibleToColumnIndex,
		_fnColumnIndexToVisible: _fnColumnIndexToVisible,
		_fnVisbleColumns: _fnVisbleColumns,
		_fnGetColumns: _fnGetColumns,
		_fnColumnTypes: _fnColumnTypes,
		_fnApplyColumnDefs: _fnApplyColumnDefs,
		_fnHungarianMap: _fnHungarianMap,
		_fnCamelToHungarian: _fnCamelToHungarian,
		_fnLanguageCompat: _fnLanguageCompat,
		_fnBrowserDetect: _fnBrowserDetect,
		_fnAddData: _fnAddData,
		_fnAddTr: _fnAddTr,
		_fnNodeToDataIndex: _fnNodeToDataIndex,
		_fnNodeToColumnIndex: _fnNodeToColumnIndex,
		_fnGetCellData: _fnGetCellData,
		_fnSetCellData: _fnSetCellData,
		_fnSplitObjNotation: _fnSplitObjNotation,
		_fnGetObjectDataFn: _fnGetObjectDataFn,
		_fnSetObjectDataFn: _fnSetObjectDataFn,
		_fnGetDataMaster: _fnGetDataMaster,
		_fnClearTable: _fnClearTable,
		_fnDeleteIndex: _fnDeleteIndex,
		_fnInvalidate: _fnInvalidate,
		_fnGetRowElements: _fnGetRowElements,
		_fnCreateTr: _fnCreateTr,
		_fnBuildHead: _fnBuildHead,
		_fnDrawHead: _fnDrawHead,
		_fnDraw: _fnDraw,
		_fnReDraw: _fnReDraw,
		_fnAddOptionsHtml: _fnAddOptionsHtml,
		_fnDetectHeader: _fnDetectHeader,
		_fnGetUniqueThs: _fnGetUniqueThs,
		_fnFeatureHtmlFilter: _fnFeatureHtmlFilter,
		_fnFilterComplete: _fnFilterComplete,
		_fnFilterCustom: _fnFilterCustom,
		_fnFilterColumn: _fnFilterColumn,
		_fnFilter: _fnFilter,
		_fnFilterCreateSearch: _fnFilterCreateSearch,
		_fnEscapeRegex: _fnEscapeRegex,
		_fnFilterData: _fnFilterData,
		_fnFeatureHtmlInfo: _fnFeatureHtmlInfo,
		_fnUpdateInfo: _fnUpdateInfo,
		_fnInfoMacros: _fnInfoMacros,
		_fnInitialise: _fnInitialise,
		_fnInitComplete: _fnInitComplete,
		_fnLengthChange: _fnLengthChange,
		_fnFeatureHtmlLength: _fnFeatureHtmlLength,
		_fnFeatureHtmlPaginate: _fnFeatureHtmlPaginate,
		_fnPageChange: _fnPageChange,
		_fnFeatureHtmlProcessing: _fnFeatureHtmlProcessing,
		_fnProcessingDisplay: _fnProcessingDisplay,
		_fnFeatureHtmlTable: _fnFeatureHtmlTable,
		_fnScrollDraw: _fnScrollDraw,
		_fnApplyToChildren: _fnApplyToChildren,
		_fnCalculateColumnWidths: _fnCalculateColumnWidths,
		_fnThrottle: _fnThrottle,
		_fnConvertToWidth: _fnConvertToWidth,
		_fnScrollingWidthAdjust: _fnScrollingWidthAdjust,
		_fnGetWidestNode: _fnGetWidestNode,
		_fnGetMaxLenString: _fnGetMaxLenString,
		_fnStringToCss: _fnStringToCss,
		_fnScrollBarWidth: _fnScrollBarWidth,
		_fnSortFlatten: _fnSortFlatten,
		_fnSort: _fnSort,
		_fnSortAria: _fnSortAria,
		_fnSortListener: _fnSortListener,
		_fnSortAttachListener: _fnSortAttachListener,
		_fnSortingClasses: _fnSortingClasses,
		_fnSortData: _fnSortData,
		_fnSaveState: _fnSaveState,
		_fnLoadState: _fnLoadState,
		_fnSettingsFromNode: _fnSettingsFromNode,
		_fnLog: _fnLog,
		_fnMap: _fnMap,
		_fnBindAction: _fnBindAction,
		_fnCallbackReg: _fnCallbackReg,
		_fnCallbackFire: _fnCallbackFire,
		_fnLengthOverflow: _fnLengthOverflow,
		_fnRenderer: _fnRenderer,
		_fnDataSource: _fnDataSource,
		_fnRowAttributes: _fnRowAttributes,
		_fnCalculateEnd: function () {} // Used by a lot of plug-ins, but redundant
		                                // in 1.10, so this dead-end function is
		                                // added to prevent errors
	} );
	

	// jQuery access
	$.fn.dataTable = DataTable;

	// Legacy aliases
	$.fn.dataTableSettings = DataTable.settings;
	$.fn.dataTableExt = DataTable.ext;

	// With a capital `D` we return a DataTables API instance rather than a
	// jQuery object
	$.fn.DataTable = function ( opts ) {
		return $(this).dataTable( opts ).api();
	};

	// All properties that are available to $.fn.dataTable should also be
	// available on $.fn.DataTable
	$.each( DataTable, function ( prop, val ) {
		$.fn.DataTable[ prop ] = val;
	} );


	// Information about events fired by DataTables - for documentation.
	/**
	 * Draw event, fired whenever the table is redrawn on the page, at the same
	 * point as fnDrawCallback. This may be useful for binding events or
	 * performing calculations when the table is altered at all.
	 *  @name DataTable#draw.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} o DataTables settings object {@link DataTable.models.oSettings}
	 */

	/**
	 * Search event, fired when the searching applied to the table (using the
	 * built-in global search, or column filters) is altered.
	 *  @name DataTable#search.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} o DataTables settings object {@link DataTable.models.oSettings}
	 */

	/**
	 * Page change event, fired when the paging of the table is altered.
	 *  @name DataTable#page.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} o DataTables settings object {@link DataTable.models.oSettings}
	 */

	/**
	 * Order event, fired when the ordering applied to the table is altered.
	 *  @name DataTable#order.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} o DataTables settings object {@link DataTable.models.oSettings}
	 */

	/**
	 * DataTables initialisation complete event, fired when the table is fully
	 * drawn, including Ajax data loaded, if Ajax data is required.
	 *  @name DataTable#init.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} oSettings DataTables settings object
	 *  @param {object} json The JSON object request from the server - only
	 *    present if client-side Ajax sourced data is used</li></ol>
	 */

	/**
	 * State save event, fired when the table has changed state a new state save
	 * is required. This event allows modification of the state saving object
	 * prior to actually doing the save, including addition or other state
	 * properties (for plug-ins) or modification of a DataTables core property.
	 *  @name DataTable#stateSaveParams.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} oSettings DataTables settings object
	 *  @param {object} json The state information to be saved
	 */

	/**
	 * State load event, fired when the table is loading state from the stored
	 * data, but prior to the settings object being modified by the saved state
	 * - allowing modification of the saved state is required or loading of
	 * state for a plug-in.
	 *  @name DataTable#stateLoadParams.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} oSettings DataTables settings object
	 *  @param {object} json The saved state information
	 */

	/**
	 * State loaded event, fired when state has been loaded from stored data and
	 * the settings object has been modified by the loaded data.
	 *  @name DataTable#stateLoaded.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} oSettings DataTables settings object
	 *  @param {object} json The saved state information
	 */

	/**
	 * Processing event, fired when DataTables is doing some kind of processing
	 * (be it, order, searcg or anything else). It can be used to indicate to
	 * the end user that there is something happening, or that something has
	 * finished.
	 *  @name DataTable#processing.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} oSettings DataTables settings object
	 *  @param {boolean} bShow Flag for if DataTables is doing processing or not
	 */

	/**
	 * Ajax (XHR) event, fired whenever an Ajax request is completed from a
	 * request to made to the server for new data. This event is called before
	 * DataTables processed the returned data, so it can also be used to pre-
	 * process the data returned from the server, if needed.
	 *
	 * Note that this trigger is called in `fnServerData`, if you override
	 * `fnServerData` and which to use this event, you need to trigger it in you
	 * success function.
	 *  @name DataTable#xhr.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} o DataTables settings object {@link DataTable.models.oSettings}
	 *  @param {object} json JSON returned from the server
	 *
	 *  @example
	 *     // Use a custom property returned from the server in another DOM element
	 *     $('#table').dataTable().on('xhr.dt', function (e, settings, json) {
	 *       $('#status').html( json.status );
	 *     } );
	 *
	 *  @example
	 *     // Pre-process the data returned from the server
	 *     $('#table').dataTable().on('xhr.dt', function (e, settings, json) {
	 *       for ( var i=0, ien=json.aaData.length ; i<ien ; i++ ) {
	 *         json.aaData[i].sum = json.aaData[i].one + json.aaData[i].two;
	 *       }
	 *       // Note no return - manipulate the data directly in the JSON object.
	 *     } );
	 */

	/**
	 * Destroy event, fired when the DataTable is destroyed by calling fnDestroy
	 * or passing the bDestroy:true parameter in the initialisation object. This
	 * can be used to remove bound events, added DOM nodes, etc.
	 *  @name DataTable#destroy.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} o DataTables settings object {@link DataTable.models.oSettings}
	 */

	/**
	 * Page length change event, fired when number of records to show on each
	 * page (the length) is changed.
	 *  @name DataTable#length.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} o DataTables settings object {@link DataTable.models.oSettings}
	 *  @param {integer} len New length
	 */

	/**
	 * Column sizing has changed.
	 *  @name DataTable#column-sizing.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} o DataTables settings object {@link DataTable.models.oSettings}
	 */

	/**
	 * Column visibility has changed.
	 *  @name DataTable#column-visibility.dt
	 *  @event
	 *  @param {event} e jQuery event object
	 *  @param {object} o DataTables settings object {@link DataTable.models.oSettings}
	 *  @param {int} column Column index
	 *  @param {bool} vis `false` if column now hidden, or `true` if visible
	 */

	return $.fn.dataTable;
}));

}(window, document));


/*! DataTables Foundation integration
 * ©2011-2014 SpryMedia Ltd - datatables.net/license
 */

/**
 * DataTables integration for Foundation. This requires Foundation 5 and
 * DataTables 1.10 or newer.
 *
 * This file sets the defaults and adds options to DataTables to style its
 * controls using Foundation. See http://datatables.net/manual/styling/foundation
 * for further information.
 */
(function(window, document, undefined){

var factory = function( $, DataTable ) {
"use strict";


$.extend( DataTable.ext.classes, {
	sWrapper: "dataTables_wrapper dt-foundation"
} );


/* Set the defaults for DataTables initialisation */
$.extend( true, DataTable.defaults, {
	dom:
		"<'row'<'small-6 columns'l><'small-6 columns'f>r>"+
		"t"+
		"<'row'<'small-6 columns'i><'small-6 columns'p>>",
	renderer: 'foundation'
} );


/* Page button renderer */
DataTable.ext.renderer.pageButton.foundation = function ( settings, host, idx, buttons, page, pages ) {
	var api = new DataTable.Api( settings );
	var classes = settings.oClasses;
	var lang = settings.oLanguage.oPaginate;
	var btnDisplay, btnClass;

	var attach = function( container, buttons ) {
		var i, ien, node, button;
		var clickHandler = function ( e ) {
			e.preventDefault();
			if ( e.data.action !== 'ellipsis' ) {
				api.page( e.data.action ).draw( false );
			}
		};

		for ( i=0, ien=buttons.length ; i<ien ; i++ ) {
			button = buttons[i];

			if ( $.isArray( button ) ) {
				attach( container, button );
			}
			else {
				btnDisplay = '';
				btnClass = '';

				switch ( button ) {
					case 'ellipsis':
						btnDisplay = '&hellip;';
						btnClass = 'unavailable';
						break;

					case 'first':
						btnDisplay = lang.sFirst;
						btnClass = button + (page > 0 ?
							'' : ' unavailable');
						break;

					case 'previous':
						btnDisplay = lang.sPrevious;
						btnClass = button + (page > 0 ?
							'' : ' unavailable');
						break;

					case 'next':
						btnDisplay = lang.sNext;
						btnClass = button + (page < pages-1 ?
							'' : ' unavailable');
						break;

					case 'last':
						btnDisplay = lang.sLast;
						btnClass = button + (page < pages-1 ?
							'' : ' unavailable');
						break;

					default:
						btnDisplay = button + 1;
						btnClass = page === button ?
							'current' : '';
						break;
				}

				if ( btnDisplay ) {
					node = $('<li>', {
							'class': classes.sPageButton+' '+btnClass,
							'aria-controls': settings.sTableId,
							'tabindex': settings.iTabIndex,
							'id': idx === 0 && typeof button === 'string' ?
								settings.sTableId +'_'+ button :
								null
						} )
						.append( $('<a>', {
								'href': '#'
							} )
							.html( btnDisplay )
						)
						.appendTo( container );

					settings.oApi._fnBindAction(
						node, {action: button}, clickHandler
					);
				}
			}
		}
	};

	attach(
		$(host).empty().html('<ul class="pagination"/>').children('ul'),
		buttons
	);
};


/*
 * TableTools Foundation compatibility
 * Required TableTools 2.1+
 */
if ( DataTable.TableTools ) {
	// Set the classes that TableTools uses to something suitable for Foundation
	$.extend( true, DataTable.TableTools.classes, {
		"container": "DTTT button-group",
		"buttons": {
			"normal": "button",
			"disabled": "disabled"
		},
		"collection": {
			"container": "DTTT_dropdown dropdown-menu",
			"buttons": {
				"normal": "",
				"disabled": "disabled"
			}
		},
		"select": {
			"row": "active"
		}
	} );

	// Have the collection use a bootstrap compatible dropdown
	$.extend( true, DataTable.TableTools.DEFAULTS.oTags, {
		"collection": {
			"container": "ul",
			"button": "li",
			"liner": "a"
		}
	} );
}

}; // /factory


// Define as an AMD module if possible
if ( typeof define === 'function' && define.amd ) {
	define( 'foundation-datatables',['jquery', 'datatables'], factory );
}
else if ( typeof exports === 'object' ) {
    // Node/CommonJS
    factory( require('jquery'), require('datatables') );
}
else if ( jQuery ) {
	// Otherwise simply initialise as normal, stopping multiple evaluation
	factory( jQuery, jQuery.fn.dataTable );
}


})(window, document);


define('utils/resource-select',['require','utils/notifier','utils/locale','opennebula/cluster','opennebula/user','opennebula/group'],function(require) {
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  require('opennebula/cluster');
  require('opennebula/user');
  require('opennebula/group');

  var _insert = function(id, context, resource, init_val, empty_value,
      extra_options, filter_att, filter_val, trigger_change_init_val, only_name) {

    var Resource = require('opennebula/' + resource.toLowerCase());
    $(id, context).html('<i class="fa fa-spinner fa-spin"></i>');

    Resource.list({
      timeout: true,
      success: function (request, obj_list) {
        var select_str = '<select class="resource_list_select">';

        if (empty_value) {
          select_str += '<option class="empty_value" value="">' +
                          Locale.tr("Please select") + '</option>';
        }

        if (resource == "Cluster") {
          if (!extra_options) {
            extra_options = "";
          }

          extra_options += '<option value="-1">Default (none)</option>';
        }

        if (extra_options) {
          select_str += extra_options;
        }

        if (!filter_att) {
          filter_att = [];
        }

        var res_name = Resource.resource;
        $.each(obj_list, function() {
          var id = this[res_name].ID;
          var name = this[res_name].NAME;
          var add = true;

          for (var i = 0; i < filter_att.length; i++) {
            if (this[res_name][filter_att[i]] == filter_val[i]) {
              add = false;
              break;
            }
          }

          if (add) {
            select_str += '<option elem_id="' + id + '" value="' + id + '">'
            if (!only_name) {
              select_str += id + ': '
            }
            select_str += name + '</option>';
          }
        });

        select_str += "</select>";

        $(id, context).html(select_str);

        if (init_val) {
          $(id + " .resource_list_select", context).val(init_val);
          if (trigger_change_init_val) {
            $(id + " .resource_list_select", context).change();
          }
        }
      },
      error: Notifier.onError
    });
  }

  return {
    'insert': _insert
  }
})
;
define('utils/progress-bar',['require'],function(require) {
  var _quotaBarHtml = function(usage, limit, info_str, not_html) {
    percentage = 0;

    if (limit > 0) {
      percentage = Math.floor((usage / limit) * 100);

      if (percentage > 100) {
        percentage = 100;
      }
    } else if (limit == 0 && usage > 0) {
      percentage = 100;
    }

    info_str = info_str || (usage + ' / ' + ((limit >= 0) ? limit : '-'));

    if (not_html) {
      return {
        "percentage": percentage,
        "str": info_str
      }
    } else {
      html = '<span class="progress-text right" style="font-size: 12px">' + info_str + '</span><br><div class="progress radius" style="height: 10px; margin-bottom:0px"><span class="meter" style="width: '          + percentage + '%"></div>';

      return html;
    }
  }

  return {
    'html': _quotaBarHtml
  }
})
;
define('utils/humanize',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  /*
    CONSTRUCTOR
   */

  return {
    'size': _size,
    'sizeFromB': _sizeFromB,
    'sizeFromKB': _sizeFromKB,
    'sizeFromMB': _sizeFromMB,
    'prettyDuration': _prettyDuration,
    'prettyTime': _prettyTime,
    'prettyTimeAxis': _prettyTimeAxis,
    'prettyPrintJSON': _prettyPrintJSON,
    'prettyTimeAgo': _format_date
  }

  /*
    FUNCTION DEFINITIONS
   */

  /*
    Returns a human readable size in Kilo, Mega, Giga or Tera bytes
    if no from_bytes, assumes value comes in Ks
  */
  function _size(value, from_bytes, sufix) {
    if (typeof(value) === "undefined") {
      value = 0;
    }
    var binarySufix = ["", "K", "M", "G", "T"];

    var i = from_bytes ? 0 : 1;
    while (value >= 1024 && i < 4) {
      value = value / 1024;
      i++;
    }
    value = Math.round(value * 10) / 10;

    if (value - Math.round(value) == 0) {
      value = Math.round(value);
    }

    if (sufix == undefined) {
      sufix = "B";
    }

    var st = value + binarySufix[i] + sufix;
    return st;
  }

  function _sizeFromB(value) {
    return _size(value, true);
  }

  function _sizeFromKB(value) {
    return _size(value);
  }

  function _sizeFromMB(value) {
    if (typeof(value) === "undefined") {
      value = 0;
    }
    var binarySufix =  ["MB", "GB", "TB"];
    var i = 0;
    while (value >= 1024 && i < 2) {
      value = value / 1024;
      i++;
    }
    value = Math.round(value * 10) / 10;

    if (value - Math.round(value) == 0) {
      value = Math.round(value);
    }

    var st = value + binarySufix[i];
    return st;
  }

  function _prettyDuration(duration) {
    var days = Math.floor(duration / 86400);
    duration -= days * 86400;

    var hours = Math.floor(duration / 3600) % 24;
    duration -= hours * 3600;

    var minutes = Math.floor(duration / 60) % 60;
    duration -= minutes * 60;

    var seconds = duration % 60;

    var str = "";
    if (days > 0) { str += days + 'd '};
    if (hours > 0) { str += hours + 'h '};
    str += minutes + 'm ';
    return str;
  }

  //introduces 0s before a number until in reaches 'length'.
  function _pad(number, length) {
    var str = '' + number;
    while (str.length < length)
        str = '0' + str;
    return str;
  }

  //turns a Unix-formatted time into a human readable string
  function _prettyTime(seconds) {
    var d = new Date();
    d.setTime(seconds * 1000);

    var secs = _pad(d.getSeconds(), 2);
    var hour = _pad(d.getHours(), 2);
    var mins = _pad(d.getMinutes(), 2);
    var day = _pad(d.getDate(), 2);
    var month = _pad(d.getMonth() + 1, 2); //getMonths returns 0-11
    var year = d.getFullYear();

    return hour + ":" + mins + ":" + secs + " " + day + "/" + month + "/" + year;
  }

  // Format time for plot axis
  // If show date, only date information is shown
  function _prettyTimeAxis(time, show_date) {
    var d = new Date();
    d.setTime(time * 1000);

    var secs = _pad(d.getSeconds(), 2);
    var hour = _pad(d.getHours(), 2);
    var mins = _pad(d.getMinutes(), 2);
    var day = _pad(d.getDate(), 2);
    var month = _pad(d.getMonth() + 1, 2); //getMonths returns 0-11
    var year = d.getFullYear();

    if (show_date)
        return day + "/" + month;
    else
        return hour + ":" + mins;
  }

  // Returns an HTML string with the json keys and values
  // Attempts to css format output, giving different values to
  // margins etc. according to depth level etc.
  // See example of use in plugins.
  function _prettyPrintJSON(template_json, padding, weight, border_bottom, padding_top_bottom) {
    var str = ""
    if (!template_json) { return "Not defined";}
    if (!padding) {padding = 10};
    if (!weight) {weight = "none";}
    if (!border_bottom) {border_bottom = "0px solid #efefef";}
    if (!padding_top_bottom) {padding_top_bottom = 5;}
    var field = null;

    if (template_json.constructor == Array) {
      for (field = 0; field < template_json.length; ++field) {
        str += _prettyPrintRowJSON(field, template_json[field], padding, weight, border_bottom, padding_top_bottom);
      }
    } else {
      for (field in template_json) {
        str += _prettyPrintRowJSON(field, template_json[field], padding, weight, border_bottom, padding_top_bottom);
      }
    }
    return str;
  }

  function _prettyPrintRowJSON(field, value, padding, weight, border_bottom, padding_top_bottom) {
    var str = "";

    if (typeof value == 'object') {
      //name of field row
      str += '<tr>\
        <td class="key_td" style=\
            "padding-left:' + padding + 'px;\
             font-weight:' + weight + ';\
             border-bottom:' + border_bottom + ';\
             padding-top:' + padding_top_bottom + 'px;\
             padding-bottom:' + padding_top_bottom + 'px;">' +
          field +
        '</td>\
        <td class="value_td" style=\
            "border-bottom:' + border_bottom + ';\
             padding-top:' + padding_top_bottom + 'px;\
             padding-bottom:' + padding_top_bottom + 'px">\
        </td>\
      </tr>';
      //attributes rows
      //empty row - prettyprint - empty row
      str += _prettyPrintJSON(value, padding + 25, "normal", "0", 1);
    } else {
      str += '<tr>\
        <td class="key_td" style="\
            padding-left:' + padding + 'px;\
            font-weight:' + weight + ';\
            border-bottom:' + border_bottom + ';\
            padding-top:' + padding_top_bottom + 'px;\
            padding-bottom:' + padding_top_bottom + 'px">' +
          field +
        '</td>\
        <td class="value_td" style="\
            border-bottom:' + border_bottom + ';\
            padding-top:' + padding_top_bottom + 'px;\
            padding-bottom:' + padding_top_bottom + 'px">' +
          value +
        '</td>\
      </tr>';
    };

    return str;
  }

  function _format_date(unix_timestamp) {
    var difference_in_seconds = (Math.round((new Date()).getTime() / 1000)) - unix_timestamp,
        current_date = new Date(unix_timestamp * 1000), minutes, hours;

    if(difference_in_seconds < 60) {
      return difference_in_seconds + "s" + " ago";
    } else if (difference_in_seconds < 60*60) {
      minutes = Math.floor(difference_in_seconds/60);
      return minutes + "m" + " ago";
    } else if (difference_in_seconds < 60*60*24) {
      hours = Math.floor(difference_in_seconds/60/60);
      return hours + "h" + " ago";
    } else if (difference_in_seconds > 60*60*24){
      if(current_date.getYear() !== new Date().getYear())
        return current_date.getDay() + " " + Locale.months[current_date.getMonth()].substr(0,3) + " " + _fourdigits(current_date.getYear());
      else {
          return current_date.getDay() + " " + Locale.months[current_date.getMonth()].substr(0,3);
      }
    }

    return difference_in_seconds;

    function _fourdigits(number)  {
          return (number < 1000) ? number + 1900 : number;}

    //function _plural(number) {
    //  if(parseInt(number) === 1) {
    //    return "";
    //  }
    //  return "s";
    //}
  }
})
;
define('utils/quotas/quota-widgets',['require','utils/progress-bar','utils/humanize','utils/locale','sunstone','utils/resource-select','./quota-limits','utils/quotas/quota-defaults'],function(require) {
  // Dependencies
  var ProgressBar = require('utils/progress-bar');
  var Humanize = require('utils/humanize');
  var Locale = require('utils/locale');
  var Sunstone = require('sunstone');
  var ResourceSelect = require('utils/resource-select');
  var QuotaLimits = require('./quota-limits');
  var QuotaDefaults = require('utils/quotas/quota-defaults');


  // Constants
  var QUOTA_LIMIT_DEFAULT   = QuotaLimits.QUOTA_LIMIT_DEFAULT;
  var QUOTA_LIMIT_UNLIMITED = QuotaLimits.QUOTA_LIMIT_UNLIMITED;


  /**
   * Returns true if the quotas are empty
   * @param  {Object} resource_info User/Group object
   * @return {boolean} true if the quotas are empty
   */
  function _emptyQuotas(resource_info){
    return ($.isEmptyObject(resource_info.VM_QUOTA) &&
            $.isEmptyObject(resource_info.DATASTORE_QUOTA) &&
            $.isEmptyObject(resource_info.IMAGE_QUOTA) &&
            $.isEmptyObject(resource_info.NETWORK_QUOTA) );
  }

  // If the VM quotas are empty, inits the VM counters to 0, and sets the limit
  // to 'default'. It is not applied to oneadmin user/group
  function _initEmptyQuotas(resource){
    if ($.isEmptyObject(resource.VM_QUOTA) && resource.ID != 0){
      resource.VM_QUOTA = {
        VM: {
          VMS         : QUOTA_LIMIT_DEFAULT,
          VMS_USED    : 0,
          CPU         : QUOTA_LIMIT_DEFAULT,
          CPU_USED    : 0,
          MEMORY      : QUOTA_LIMIT_DEFAULT,
          MEMORY_USED : 0,
          SYSTEM_DISK_SIZE      : QUOTA_LIMIT_DEFAULT,
          SYSTEM_DISK_SIZE_USED : 0
        }
      };
    }
  }

  /**
   * Returns a widget with the VM quotas
   * @param  {Object} info User/Group object
   * @param  {Object} default_quotas default quotas for Users/Groups
   * @return {string} html string
   */
  function _vmsWidget(info, default_quotas){
    var empty_quotas = $.isEmptyObject(info.VM_QUOTA);

    var quotas_tab_html = "";

    if (empty_quotas){
      quotas_tab_html +=
        '<fieldset style="display: none" class="editable_quota">';
    } else {
      quotas_tab_html +=
        '<fieldset>';
    }

    var vms_bar;

    if (!empty_quotas){
      vms_bar = _editableQuotaBar(
        info.VM_QUOTA.VM.VMS_USED,
        info.VM_QUOTA.VM.VMS,
        default_quotas.VM_QUOTA.VM.VMS,
        { quota_name: "VM_VMS"});
    } else {
      vms_bar = _editableQuotaBar(
        0,
        QUOTA_LIMIT_DEFAULT,
        default_quotas.VM_QUOTA.VM.VMS,
        { quota_name: "VM_VMS"});
    }

    quotas_tab_html +=
        '<legend>' + Locale.tr("VMs") + '</legend>\
        <div>'+vms_bar+'</div>\
        <br>\
      </fieldset>'

    return quotas_tab_html;
  }

  /**
   * Returns a widget with the CPU quotas
   * @param  {Object} info User/Group object
   * @param  {Object} default_quotas default quotas for Users/Groups
   * @return {string} html string
   */
  function _cpuWidget(info, default_quotas){
    var empty_quotas = $.isEmptyObject(info.VM_QUOTA);

    var quotas_tab_html = "";

    if (empty_quotas){
      quotas_tab_html +=
        '<fieldset style="display: none" class="editable_quota">';
    } else {
      quotas_tab_html +=
        '<fieldset>';
    }

    var cpu_bar;

    if (!empty_quotas){
      cpu_bar = _editableQuotaBar(
            info.VM_QUOTA.VM.CPU_USED,
            info.VM_QUOTA.VM.CPU,
            default_quotas.VM_QUOTA.VM.CPU,
            {   is_float: true,
                quota_name: "VM_CPU"
            });
    } else {
      cpu_bar = _editableQuotaBar(
            0,
            QUOTA_LIMIT_DEFAULT,
            default_quotas.VM_QUOTA.VM.CPU,
            {   is_float: true,
                quota_name: "VM_CPU"
            });
    }

    quotas_tab_html +=
        '<legend>' + Locale.tr("CPU") + '</legend>\
        <div>'+cpu_bar+'</div>\
        <br>\
        </fieldset>'

    return quotas_tab_html;
  }

  /**
   * Returns a widget with the memory quotas
   * @param  {Object} info User/Group object
   * @param  {Object} default_quotas default quotas for Users/Groups
   * @return {string} html string
   */
  function _memoryWidget(info, default_quotas){
    var empty_quotas = $.isEmptyObject(info.VM_QUOTA);

    var quotas_tab_html = "";

    if (empty_quotas){
      quotas_tab_html +=
        '<fieldset style="display: none" class="editable_quota">';
    } else {
      quotas_tab_html +=
        '<fieldset>';
    }

    var memory_bar;

    if (!empty_quotas){
      memory_bar = _editableQuotaBar(
            info.VM_QUOTA.VM.MEMORY_USED,
            info.VM_QUOTA.VM.MEMORY,
            default_quotas.VM_QUOTA.VM.MEMORY,
            {   mb: true,
                quota_name: "VM_MEMORY"
            });
    } else {
      memory_bar = _editableQuotaBar(
            0,
            QUOTA_LIMIT_DEFAULT,
            default_quotas.VM_QUOTA.VM.MEMORY,
            {   mb: true,
                quota_name: "VM_MEMORY"
            });
    }

    quotas_tab_html +=
        '<legend>' + Locale.tr("Memory") + '</legend>\
        <div>'+memory_bar+'</div>\
        <br>\
        </fieldset>'

    return quotas_tab_html;
  }

  /**
   * Returns a widget with the system disk quotas
   * @param  {Object} info User/Group object
   * @param  {Object} default_quotas default quotas for Users/Groups
   * @return {string} html string
   */
  function _systemDiskWidget(info, default_quotas){
    var empty_quotas = $.isEmptyObject(info.VM_QUOTA);

    var quotas_tab_html = "";

    if (empty_quotas){
      quotas_tab_html +=
        '<fieldset style="display: none" class="editable_quota">';
    } else {
      quotas_tab_html +=
        '<fieldset>';
    }

    var system_bar;

    if (!empty_quotas){
      system_bar = _editableQuotaBar(
            info.VM_QUOTA.VM.SYSTEM_DISK_SIZE_USED,
            info.VM_QUOTA.VM.SYSTEM_DISK_SIZE,
            default_quotas.VM_QUOTA.VM.SYSTEM_DISK_SIZE,
            {   mb: true,
                quota_name: "VM_SYSTEM_DISK_SIZE"
            });
    } else {
      system_bar = _editableQuotaBar(
            0,
            QUOTA_LIMIT_DEFAULT,
            default_quotas.VM_QUOTA.VM.SYSTEM_DISK_SIZE,
            {   mb: true,
                quota_name: "VM_SYSTEM_DISK_SIZE"
            });
    }

    quotas_tab_html +=
        '<legend>' + Locale.tr("System disks") + '</legend>\
        <div>'+system_bar+'</div>\
        <br>\
        </fieldset>'

    return quotas_tab_html;
  }

  /**
   * Returns a widget with the image quotas
   * @param  {Object} info User/Group object
   * @param  {Object} default_quotas default quotas for Users/Groups
   * @return {string} html string
   */
  function _imageWidget(info, default_quotas) {
    var empty_quotas = $.isEmptyObject(info.IMAGE_QUOTA);

    var quotas_tab_html = "";

    if (empty_quotas){
      quotas_tab_html +=
        '<fieldset style="padding: 5px 15px; display: none" class="editable_quota">';
    } else {
      quotas_tab_html +=
        '<fieldset style="padding: 5px 15px">';
    }

    quotas_tab_html +=
          '<legend>'+Locale.tr("Image")+'</legend>\
          <table class="quota_table extended_table image_quota_table">\
          <thead>\
              <tr>\
                  <th style="width:16%">'+Locale.tr("ID")+'</th>\
                  <th style="width:84%">'+Locale.tr("Running VMs")+'</th>\
              </tr>\
          </thead>\
          <tbody>';

    var img_quotas = [];

    if (!empty_quotas){
      if ($.isArray(info.IMAGE_QUOTA.IMAGE))
        img_quotas = info.IMAGE_QUOTA.IMAGE;
      else if (info.IMAGE_QUOTA.IMAGE.ID)
        img_quotas = [info.IMAGE_QUOTA.IMAGE];
    }

    for (var i=0; i < img_quotas.length; i++){

      var default_img_quotas = default_quotas.IMAGE_QUOTA[img_quotas[i].ID]

      if (default_img_quotas == undefined){
        default_img_quotas = {
          "RVMS"  : QUOTA_LIMIT_UNLIMITED
        }
      }

      var rvms_bar = _editableQuotaBar(
        img_quotas[i].RVMS_USED,
        img_quotas[i].RVMS,
        default_img_quotas.RVMS,
        { quota_name: "IMAGE_RVMS"});

      quotas_tab_html +=
        '<tr class="image_quota_tr" quota_id="'+img_quotas[i].ID+'">\
          <td>'+img_quotas[i].ID+'</td>\
          <td>'+rvms_bar+'</td>\
        </tr>';
    }

    quotas_tab_html +=
              '</tbody>\
              <tfoot>\
                  <tr class="editable_quota" style="display: none">\
                      <td colspan="2">\
                          <a type="button" \
                            class="button small radius small-12" \
                            id="image_add_quota_btn"><i class="fa fa-plus"></i>\
                            '+Locale.tr("Add a new quota")+'\
                          </a>\
                      </td>\
                  </tr>\
              </tfoot>\
          </table>\
      </fieldset>';

    return quotas_tab_html;
  }

  /**
   * @param  {Object} context jquery object, e.g. $("panel-id")
   * @param  {Object} default_quotas default quotas for Users/Groups
   */
  function _setupImageWidget(context, default_quotas){
    context.off("click", "#image_add_quota_btn");
    context.on("click", "#image_add_quota_btn", function(){

      $(".image_quota_table tbody", context).append(
        '<tr class="image_quota_tr" quota_id="-1">\
          <td class="image_select" colspan="2"></td>\
          <td class="rvms_bar"></td>\
        </tr>');

      ResourceSelect.insert(
        'td.image_select',
        $(".image_quota_table tbody tr", context).last(),
        "Image",
        null, true);

      $(".image_quota_table tbody tr", context).last().off(
                                            "change", ".resource_list_select");

      $(".image_quota_table tbody tr", context).last().on(
                                  "change", ".resource_list_select", function(){

          $(this).parents("td").attr("colspan", "1");

          var image_id = $(this).val();
          var tr = $(this).parents("tr");
          tr.attr("quota_id", image_id);

          var default_img_quotas = default_quotas.IMAGE_QUOTA[image_id];

          if (default_img_quotas == undefined){
            default_img_quotas = {
              "RVMS"  : QUOTA_LIMIT_UNLIMITED
            }
          }

          var rvms_bar = _editableQuotaBar(
            0,
            QUOTA_LIMIT_DEFAULT,
            default_img_quotas.RVMS,
            { quota_name: "IMAGE_RVMS"});

          $("td.rvms_bar", tr).html(rvms_bar);

          $(".editable_quota", tr).show();
          $(".non_editable_quota", tr).hide();

          $.each($("input", tr), function(){
            initQuotaInputValue(this);
          });
        });

      return false;
    });
  }

  /**
   * Returns a widget with the datastore quotas
   * @param  {Object} info User/Group object
   * @param  {Object} default_quotas default quotas for Users/Groups
   * @return {string} html string
   */
  function _datastoreWidget(info, default_quotas) {
    var empty_quotas = $.isEmptyObject(info.DATASTORE_QUOTA);

    var quotas_tab_html = "";

    if (empty_quotas){
      quotas_tab_html +=
        '<fieldset style="padding: 5px 15px; display: none" class="editable_quota">';
    } else {
      quotas_tab_html +=
        '<fieldset style="padding: 5px 15px">';
    }

    quotas_tab_html +=
        '<legend>'+Locale.tr("Datastore")+'</legend>\
        <table class="quota_table extended_table ds_quota_table">\
          <thead>\
              <tr>\
                <th style="width:16%">'+Locale.tr("ID")+'</th>\
                <th style="width:42%">'+Locale.tr("Images")+'</th>\
                <th style="width:42%">'+Locale.tr("Size")+'</th>\
              </tr>\
          </thead>\
          <tbody>';

    var ds_quotas = [];

    if (!empty_quotas){
      if ($.isArray(info.DATASTORE_QUOTA.DATASTORE))
        ds_quotas = info.DATASTORE_QUOTA.DATASTORE;
      else if (info.DATASTORE_QUOTA.DATASTORE.ID)
        ds_quotas = [info.DATASTORE_QUOTA.DATASTORE];
    }

    for (var i=0; i < ds_quotas.length; i++){

      var default_ds_quotas = default_quotas.DATASTORE_QUOTA[ds_quotas[i].ID]

      if (default_ds_quotas == undefined){
        default_ds_quotas = {
          "IMAGES"    : QUOTA_LIMIT_UNLIMITED,
          "SIZE"      : QUOTA_LIMIT_UNLIMITED
        }
      }

      var img_bar = _editableQuotaBar(
          ds_quotas[i].IMAGES_USED,
          ds_quotas[i].IMAGES,
          default_ds_quotas.IMAGES,
          { quota_name: "DS_IMAGES" });

      var size_bar = _editableQuotaBar(
          ds_quotas[i].SIZE_USED,
          ds_quotas[i].SIZE,
          default_ds_quotas.SIZE,
          {   mb: true,
              quota_name: "DS_SIZE"
          });

      quotas_tab_html +=
      '<tr class="ds_quota_tr" quota_id="'+ds_quotas[i].ID+'">\
        <td>'+ds_quotas[i].ID+'</td>\
        <td>'+img_bar+'</td>\
        <td>'+size_bar+'</td>\
      </tr>';
    }

    quotas_tab_html +=
            '</tbody>\
            <tfoot>\
                <tr class="editable_quota" style="display: none">\
                    <td colspan="3">\
                        <a type="button" \
                          class="button small radius small-12" \
                          id="ds_add_quota_btn"><i class="fa fa-plus"></i>\
                          '+Locale.tr("Add a new quota")+'\
                        </a>\
                    </td>\
                </tr>\
            </tfoot>\
        </table>\
        <div class="">\
        </div>\
    </fieldset>';

    return quotas_tab_html;
  }

  /**
   * @param  {Object} context jquery object, e.g. $("panel-id")
   * @param  {Object} default_quotas default quotas for Users/Groups
   */
  function _setupDatastoreWidget(context, default_quotas){
    context.off("click", "#ds_add_quota_btn");
    context.on("click", "#ds_add_quota_btn", function(){

      $(".ds_quota_table tbody", context).append(
        '<tr class="ds_quota_tr" quota_id="-1">\
          <td class="ds_select" colspan="3"></td>\
          <td class="img_bar"></td>\
          <td class="size_bar"></td>\
        </tr>');

      ResourceSelect.insert(
        'td.ds_select',
        $(".ds_quota_table tbody tr", context).last(),
        "Datastore",
        null, true);

      $(".ds_quota_table tbody tr", context).last().off(
                                            "change", ".resource_list_select");

      $(".ds_quota_table tbody tr", context).last().on(
                                "change", ".resource_list_select", function(){

        $(this).parents("td").attr("colspan", "1");

        var ds_id = $(this).val();
        var tr = $(this).parents("tr");
        tr.attr("quota_id", ds_id);

        var default_ds_quotas = default_quotas.DATASTORE_QUOTA[ds_id];

        if (default_ds_quotas == undefined){
            default_ds_quotas = {
                "IMAGES"    : QUOTA_LIMIT_UNLIMITED,
                "SIZE"      : QUOTA_LIMIT_UNLIMITED
            }
        }

        var img_bar = _editableQuotaBar(
            0,
            QUOTA_LIMIT_DEFAULT,
            default_ds_quotas.IMAGES,
            { quota_name: "DS_IMAGES" });

        var size_bar = _editableQuotaBar(
            0,
            QUOTA_LIMIT_DEFAULT,
            default_ds_quotas.SIZE,
            {   mb: true,
                quota_name: "DS_SIZE"
            });

        $("td.img_bar", tr).html(img_bar);
        $("td.size_bar", tr).html(size_bar);

        $(".editable_quota", tr).show();
        $(".non_editable_quota", tr).hide();

        $.each($("input", tr), function(){
          initQuotaInputValue(this);
        });
      });

      return false;
    });

    return false;
  }

  /**
   * Returns a widget with the network quotas
   * @param  {Object} info User/Group object
   * @param  {Object} default_quotas default quotas for Users/Groups
   * @return {string} html string
   */
  function _networkWidget(info, default_quotas) {
    var empty_quotas = $.isEmptyObject(info.NETWORK_QUOTA);

    var quotas_tab_html = "";

    if (empty_quotas){
      quotas_tab_html +=
        '<fieldset style="padding: 5px 15px; display: none" class="editable_quota">';
    } else {
      quotas_tab_html +=
        '<fieldset style="padding: 5px 15px">';
    }

    quotas_tab_html +=
        '<legend>'+Locale.tr("Network")+'</legend>\
        <table class="quota_table extended_table network_quota_table">\
            <thead>\
                <tr>\
                    <th style="width:16%">'+Locale.tr("ID")+'</th>\
                    <th style="width:84%">'+Locale.tr("Leases")+'</th>\
                </tr>\
            </thead>\
            <tbody>';

    var net_quotas = [];

    if (!empty_quotas){
      if ($.isArray(info.NETWORK_QUOTA.NETWORK))
        net_quotas = info.NETWORK_QUOTA.NETWORK;
      else if (info.NETWORK_QUOTA.NETWORK.ID)
        net_quotas = [info.NETWORK_QUOTA.NETWORK];
    }

    for (var i=0; i < net_quotas.length; i++){
      var default_net_quotas = default_quotas.NETWORK_QUOTA[net_quotas[i].ID]

      if (default_net_quotas == undefined){
        default_net_quotas = {
          "LEASES" : QUOTA_LIMIT_UNLIMITED
        }
      }

      var leases_bar = _editableQuotaBar(
          net_quotas[i].LEASES_USED,
          net_quotas[i].LEASES,
          default_net_quotas.LEASES,
          { quota_name: "NETWORK_LEASES" });

      quotas_tab_html +=
        '<tr class="network_quota_tr" quota_id="'+net_quotas[i].ID+'">\
          <td>'+net_quotas[i].ID+'</td>\
          <td>'+leases_bar+'</td>\
        </tr>';
    }

    quotas_tab_html +=
            '</tbody>\
            <tfoot>\
              <tr class="editable_quota" style="display: none">\
                <td colspan="2">\
                  <a type="button" \
                    class="button small radius small-12" \
                    id="network_add_quota_btn"><i class="fa fa-plus"></i>\
                    '+Locale.tr("Add a new quota")+'\
                  </a>\
                </td>\
              </tr>\
            </tfoot>\
          </table>\
        </fieldset>';

    return quotas_tab_html;
  }

  /**
   * @param  {Object} context jquery object, e.g. $("panel-id")
   * @param  {Object} default_quotas default quotas for Users/Groups
   */
  function _setupNetworkWidget(context, default_quotas){
    context.off("click", "#network_add_quota_btn");
    context.on("click", "#network_add_quota_btn", function(){

      $(".network_quota_table tbody", context).append(
          '<tr class="network_quota_tr" quota_id="-1">\
              <td class="network_select" colspan="2"></td>\
              <td class="leases_bar"></td>\
          </tr>');

      ResourceSelect.insert(
          'td.network_select',
          $(".network_quota_table tbody tr", context).last(),
          "Network",
          null, true);

      $(".network_quota_table tbody tr", context).last().off(
                                            "change", ".resource_list_select");

      $(".network_quota_table tbody tr", context).last().on(
                                  "change", ".resource_list_select", function(){

        $(this).parents("td").attr("colspan", "1");

        var network_id = $(this).val();
        var tr = $(this).parents("tr");
        tr.attr("quota_id", network_id);

        var default_net_quotas = default_quotas.NETWORK_QUOTA[network_id];

        if (default_net_quotas == undefined){
          default_net_quotas = {
            "LEASES" : QUOTA_LIMIT_UNLIMITED
          }
        }

        var leases_bar = _editableQuotaBar(
            0,
            QUOTA_LIMIT_DEFAULT,
            default_net_quotas.LEASES,
            { quota_name: "NETWORK_LEASES" });

        $("td.leases_bar", tr).html(leases_bar);

        $(".editable_quota", tr).show();
        $(".non_editable_quota", tr).hide();

        $.each($("input", tr), function(){
          initQuotaInputValue(this);
        });
      });

      return false;
    });

    return false;
  }

  /**
   * Creates a quotas panel
   * @param  {Object} resourece_info User/Group object
   * @param  {Object} default_quotas default quotas for Users/Groups
   * @param  {boolean} edit_enabled true to show the edit button and functionality
   * @return {string} html string with all the quota widgets
   */
  function _initQuotasPanel(resource_info, default_quotas, edit_enabled){
    _initEmptyQuotas(resource_info);

    var vms_quota = _vmsWidget(resource_info, default_quotas);
    var cpu_quota = _cpuWidget(resource_info, default_quotas);
    var memory_quota = _memoryWidget(resource_info, default_quotas);
    var system_disk_size_quota = _systemDiskWidget(resource_info, default_quotas);

    var image_quota = _imageWidget(resource_info, default_quotas);
    var network_quota = _networkWidget(resource_info, default_quotas);
    var datastore_quota = _datastoreWidget(resource_info, default_quotas);

    var quotas_html;

    quotas_html = '<div class="quotas">';

    if (edit_enabled) {
        quotas_html +=
        '<div class="row">\
          <div class="large-12 columns">\
            <span class="right">\
              <button class="button secondary small radius" id="edit_quotas_button">\
                <span class="fa fa-pencil-square-o"></span> '+Locale.tr("Edit")+'\
              </button>\
              <button class="button alert small radius" id="cancel_quotas_button" style="display: none">\
                '+Locale.tr("Cancel")+'\
              </button>\
              <button class="button success small radius" id="submit_quotas_button" style="display: none">\
                '+Locale.tr("Apply")+'\
              </button>\
            </span>\
          </div>\
        </div>';
    }

    if (_emptyQuotas(resource_info)) {
        quotas_html +=
        '<div class="row non_editable_quota">\
          <div class="large-8 large-centered columns">\
            <div class="text-center">\
              <span class="fa-stack fa-5x" style="color: #dfdfdf">\
                <i class="fa fa-cloud fa-stack-2x"></i>\
                <i class="fa fa-align-left fa-stack-1x fa-inverse"></i>\
              </span>\
              <br>\
              <p style="font-size: 18px; color: #999">\
                '+Locale.tr("There are no quotas defined")+'\
              </p>\
            </div>\
          </div>\
        </div>';
    }

    quotas_html +=
        '<div class="row">\
          <div class="large-6 columns">' + vms_quota + '</div>\
          <div class="large-6 columns">' + cpu_quota + '</div>\
        </div>\
        <div class="row">\
          <div class="large-6 columns">' + memory_quota + '</div>\
          <div class="large-6 columns">' + system_disk_size_quota+ '</div>\
        </div>\
        <br><br>\
        <div class="row">\
          <div class="large-6 columns">' + image_quota + '</div>\
          <div class="large-6 columns right">' + network_quota + '</div>\
        </div>\
        <br><br>\
        <div class="row">\
          <div class="large-12 columns">' + datastore_quota + '</div>\
        </div>\
      </div>';

    return quotas_html;
  }

  function input_val(input){
    switch(input.attr("quota_mode")) {
      case "edit":
        return input.val();
      case "default":
        return QUOTA_LIMIT_DEFAULT;
      case "unlimited":
        return QUOTA_LIMIT_UNLIMITED;
    }
  }

  function initQuotaInputValue(input){
    switch($(input).val()) {
      case QUOTA_LIMIT_DEFAULT:
        $(input).parents(".quotabar_container").find(".quotabar_default_btn").click();
        break;
      case QUOTA_LIMIT_UNLIMITED:
        $(input).parents(".quotabar_container").find(".quotabar_unlimited_btn").click();
        break;
      default:
        break;
    }
  }

  function quotasPanelEditAction(parent_container){
    $("#edit_quotas_button", parent_container).hide();
    $("#cancel_quotas_button", parent_container).show();
    $("#submit_quotas_button", parent_container).show();

    $.each($("div.quotabar_container input", parent_container), function(){
      initQuotaInputValue(this);
    });

    $(".editable_quota", parent_container).show();
    $(".non_editable_quota", parent_container).hide();

    return false;
  }

  function setupQuotasBarButtons(resource_info, parent_container){
    parent_container.off("click", ".quotabar_edit_btn");
    parent_container.on("click",  ".quotabar_edit_btn", function() {
      var input = $(this).parents(".quotabar_container").find("input");

      if(input.attr("quota_mode") != "edit"){
        input.attr("quota_mode", "edit");
        input.attr("disabled", false);
        input.val( input.attr("quota_limit") >= 0 ? input.attr("quota_limit") : "0" );
      }

      return false;
    });

    parent_container.off("click", ".quotabar_default_btn");
    parent_container.on("click",  ".quotabar_default_btn", function() {
      var input = $(this).parents(".quotabar_container").find("input");

      var default_value = input.attr("quota_default");

      if (default_value == QUOTA_LIMIT_UNLIMITED) {
        default_value = "∞";
      }

      input.val( Locale.tr("Default") + " (" + default_value + ")" );
      input.attr("quota_mode", "default");
      input.attr("disabled", "disabled");

      return false;
    });

    parent_container.off("click", ".quotabar_unlimited_btn");
    parent_container.on("click",  ".quotabar_unlimited_btn", function() {
      var input = $(this).parents(".quotabar_container").find("input");

      input.val(Locale.tr("Unlimited"));
      input.attr("quota_mode", "unlimited");
      input.attr("disabled", "disabled");

      return false;
    });
  }

  function retrieveQuotasValues(parent_container){
    var obj = {};

    obj["VM"] = {
      "CPU"           : input_val( $("div[quota_name=VM_CPU] input", parent_container) ),
      "MEMORY"        : input_val( $("div[quota_name=VM_MEMORY] input", parent_container) ),
      "VMS"           : input_val( $("div[quota_name=VM_VMS] input", parent_container) ),
      "SYSTEM_DISK_SIZE" : input_val( $("div[quota_name=VM_SYSTEM_DISK_SIZE] input", parent_container) )
    };

    $.each($("tr.image_quota_tr", parent_container), function(){
      if($(this).attr("quota_id") != "-1"){
        if (obj["IMAGE"] == undefined) {
          obj["IMAGE"] = [];
        }

        obj["IMAGE"].push({
          "ID"    : $(this).attr("quota_id"),
          "RVMS"  : input_val( $("div[quota_name=IMAGE_RVMS] input", this) )
        });
      }
    });

    $.each($("tr.network_quota_tr", parent_container), function(){
      if($(this).attr("quota_id") != "-1"){
        if (obj["NETWORK"] == undefined) {
          obj["NETWORK"] = [];
        }

        obj["NETWORK"].push({
          "ID"    : $(this).attr("quota_id"),
          "LEASES": input_val( $("div[quota_name=NETWORK_LEASES] input", this) )
        });
      }
    });

    $.each($("tr.ds_quota_tr", parent_container), function(){
      if($(this).attr("quota_id") != "-1"){
        if (obj["DATASTORE"] == undefined) {
          obj["DATASTORE"] = [];
        }

        obj["DATASTORE"].push({
          "ID"    : $(this).attr("quota_id"),
          "IMAGES": input_val( $("div[quota_name=DS_IMAGES] input", this) ),
          "SIZE"  : input_val( $("div[quota_name=DS_SIZE] input", this) )
        });
      }
    });

    return obj;
  }

  /**
   * Setups the html returned by initQuotasPanel, adding listeners for the edit
   * buttons
   * @param  {Object} resourece_info User/Group object
   * @param  {Object} parent_container jquery object, e.g. $("panel-id")
   * @param  {boolean} edit_enabled true to show the edit button and functionality
   * @param  {string} resource_name User or Group
   */
  function _setupQuotasPanel(resource_info, parent_container, edit_enabled, resource_name){
    if (edit_enabled) {
      parent_container.off("click", "#edit_quotas_button");
      parent_container.on("click",  "#edit_quotas_button", function() {
        return quotasPanelEditAction(parent_container);
      });

      parent_container.off("click", "#cancel_quotas_button");
      parent_container.on("click",  "#cancel_quotas_button", function() {
        Sunstone.runAction(resource_name+".show", resource_info.ID);
        return false;
      });

      parent_container.off("click", "#submit_quotas_button");
      parent_container.on("click",  "#submit_quotas_button", function() {
        var obj = retrieveQuotasValues(parent_container);
        Sunstone.runAction(resource_name+".set_quota", [resource_info.ID], obj);

        return false;
      });

      setupQuotasBarButtons(resource_info, parent_container);

      var default_quotas = QuotaDefaults.getDefaultQuotas(resource_name);

      _setupImageWidget(parent_container, default_quotas);
      _setupNetworkWidget(parent_container, default_quotas);
      _setupDatastoreWidget(parent_container, default_quotas);
    }
  }

  /*
   * opts.is_float : true to parse quota_limit and default_limit as floats instead of int
   * opts.mb : true if the quota is in MB
   * opts.quota_name : string to identify the quota widget
   */
  function _editableQuotaBar(usage, quota_limit, default_limit, opts){

      if (!opts) opts = {};
      if (!opts.quota_name) opts.quota_name = "";

      var limit;

      if (opts.is_float){
          usage = parseFloat(usage, 10);
          limit = _quotaFloatLimit(quota_limit, default_limit);
      } else {
          usage = parseInt(usage, 10);
          limit = _quotaIntLimit(quota_limit, default_limit);
      }

      percentage = 0;

      if (limit > 0){
          percentage = Math.floor((usage / limit) * 100);

          if (percentage > 100){
              percentage = 100;
          }
      } else if (limit == 0 && usage > 0){
          percentage = 100;
      }

      var info_str;

      if (opts.mb){
          info_str = Humanize.size(usage * 1024)+' / '
              +((limit >= 0) ? Humanize.size(limit * 1024) : '-')
      } else {
          info_str = usage+' / '+((limit >= 0) ? limit : '-');
      }

      html =
      '<div class="quotabar_container" quota_name="'+opts.quota_name+'">\
        <div class="row collapse editable_quota" style="font-size: 12px; display: none">\
          <div class="small-2 columns">\
            <label style="font-size: 12px; margin: 0px" class="inline right">'+ usage + ' /&nbsp;</label>\
          </div>';


      if (opts.mb){
          html +=
          '<div class="small-4 columns">';
      }else{
          html +=
          '<div class="small-5 columns">';
      }

      html +=
            '<input type="text" style="font-size: 12px; margin: 0px" quota_mode="edit" quota_limit="'+quota_limit+'" quota_default="'+default_limit+'" value="'+quota_limit+'"/>\
          </div>';

      if (opts.mb){
          html +=
          '<div class="small-1 columns">\
            <span style="font-size: 12px; height: 2.0625rem !important; line-height: 2.0625rem !important;" class="postfix">MB</span>\
          </div>';
      }

      html +=
          '<div class="small-5 columns">\
            <ul class="button-group">\
              <li><a style="font-size: 1em; margin: 0px" class="button tiny secondary quotabar_edit_btn"><span class="fa fa-pencil"></span></a></li>\
              <li><a style="font-size: 1em; margin: 0px" class="button tiny secondary quotabar_default_btn"><span class="fa fa-file-o"></span></a></li>\
              <li><a style="font-size: 1em; margin: 0px" class="button tiny secondary quotabar_unlimited_btn"><strong>&infin;</strong></a></li>\
            </ul>\
          </div>\
        </div>\
        <div class="row collapse non_editable_quota">\
          <div class="large-12 columns">\
            <div class="progress-text right" style="font-size: 12px">\
              '+info_str+'\
            </div>\
            <br>\
            <div class="progress radius" style="height: 10px; margin-bottom:0px">\
              <span class="meter" style="width: '
                +percentage+'%" />\
            </div>\
          </div>\
        </div>\
      </div>';
      return html;
  }

  function _quotaInfo(usage, limit, default_limit, not_html){
      var int_usage = parseInt(usage, 10);
      var int_limit = _quotaIntLimit(limit, default_limit);
      return _quotaBaseInfo(int_usage, int_limit, null, not_html);
  }

  function _quotaMBInfo(usage, limit, default_limit, not_html){
      var int_usage = parseInt(usage, 10);
      var int_limit = _quotaIntLimit(limit, default_limit);

      info_str = Humanize.size(int_usage * 1024)+' / '
              +((int_limit >= 0) ? Humanize.size(int_limit * 1024) : '-')

      return _quotaBaseInfo(int_usage, int_limit, info_str, not_html);
  }

  function _quotaFloatInfo(usage, limit, default_limit, not_html){
      var float_usage = parseFloat(usage, 10);
      var float_limit = _quotaFloatLimit(limit, default_limit);
      return _quotaBaseInfo(float_usage, float_limit, null, not_html);
  }

  function _quotaBaseInfo(usage, limit, info_str, not_html){
      percentage = 0;

      if (limit > 0){
          percentage = Math.floor((usage / limit) * 100);

          if (percentage > 100){
              percentage = 100;
          }
      } else if (limit == 0 && usage > 0){
          percentage = 100;
      }

      info_str = info_str || ( usage+' / '+((limit >= 0) ? limit : '-') );

      if (not_html) {
          return {
              "percentage": percentage,
              "str": info_str
          }
      } else {
          html = '<span class="progress-text right" style="font-size: 12px">'+info_str+'</span><br><div class="progress radius" style="height: 10px; margin-bottom:0px"><span class="meter" style="width: '
              +percentage+'%"></div>';

          return html;
      }
  }

  function _quotaBar(usage, limit, default_limit){
    var int_usage = parseInt(usage, 10);
    var int_limit = _quotaIntLimit(limit, default_limit);
    return ProgressBar.html(int_usage, int_limit, null);
  }

  function _quotaBarMB(usage, limit, default_limit){
    var int_usage = parseInt(usage, 10);
    var int_limit = _quotaIntLimit(limit, default_limit);

    info_str = Humanize.size(int_usage * 1024)+' / ' +
                  ((int_limit >= 0) ? Humanize.size(int_limit * 1024) : '-');

    return ProgressBar.html(int_usage, int_limit, info_str);
  }

  function _quotaBarFloat(usage, limit, default_limit){
    var float_usage = parseFloat(usage, 10);
    var float_limit = _quotaFloatLimit(limit, default_limit);
    return ProgressBar.html(float_usage, float_limit, null);
  }

  function _quotaIntLimit(limit, default_limit){
    i_limit = parseInt(limit, 10);
    i_default_limit = parseInt(default_limit, 10);

    if (limit == QUOTA_LIMIT_DEFAULT){
      i_limit = i_default_limit;
    }

    if (isNaN(i_limit))
    {
      i_limit = 0;
    }

    return i_limit
  }

  function _quotaFloatLimit(limit, default_limit){
    f_limit = parseFloat(limit, 10);
    f_default_limit = parseFloat(default_limit, 10);

    if (f_limit == parseFloat(QUOTA_LIMIT_DEFAULT, 10)){
      f_limit = f_default_limit;
    }

    if (isNaN(f_limit))
    {
      f_limit = 0;
    }

    return f_limit
  }

  //============================================================================
  // Quotas dialog
  //============================================================================

  function _quotas_tmpl(){
    return '<div class="row">\
        <div class="large-12 columns">\
          <dl class="tabs right-info-tabs text-center" data-tab>\
               <dd class="active"><a href="#vm_quota"><i class="fa fa-cloud"></i><br>'+Locale.tr("VM")+'</a></dd>\
               <dd><a href="#datastore_quota"><i class="fa fa-folder-open"></i><br>'+Locale.tr("Datastore")+'</a></dd>\
               <dd><a href="#image_quota"><i class="fa fa-upload"></i><br>'+Locale.tr("Image")+'</a></dd>\
               <dd><a href="#network_quota"><i class="fa fa-globe"></i><br>'+Locale.tr("VNet")+'</a></dd>\
          </dl>\
        </div>\
      </div>\
      <div class="row">\
        <div class="tabs-content">\
          <div id="vm_quota" class="content active">\
          </div>\
          <div id="datastore_quota" class="content">\
          </div>\
          <div id="image_quota" class="content">\
          </div>\
          <div id="network_quota" class="content">\
          </div>\
        </div>\
      </div>';
  }

  /**
   * Sets up a dialog to edit and update user and group quotas
   */
  function _setupQuotasDialog(context){
    $('form', context).submit(function(){
      var obj = retrieveQuotasValues(context);
      var action = $('div.form_buttons button',this).val();
      var sel_elems = Sunstone.getAction(action).elements();
      Sunstone.runAction(action,sel_elems,obj);

      return false;
    });
  }

  function _populateQuotasDialog(resource_info, default_quotas, context){
    var vms_quota = _vmsWidget(resource_info, default_quotas);
    var cpu_quota = _cpuWidget(resource_info, default_quotas);
    var memory_quota = _memoryWidget(resource_info, default_quotas);
    var system_disk_size_quota = _systemDiskWidget(resource_info, default_quotas);

    var image_quota = _imageWidget(resource_info, default_quotas);
    var network_quota = _networkWidget(resource_info, default_quotas);
    var datastore_quota = _datastoreWidget(resource_info, default_quotas);

    $("#vm_quota", context).html(
        '<div class="large-6 columns">' + vms_quota + '</div>\
        <div class="large-6 columns">' + cpu_quota + '</div>\
        <div class="large-6 columns">' + memory_quota + '</div>\
        <div class="large-6 columns">' + system_disk_size_quota+ '</div>');

    $("#datastore_quota", context).html(
        '<div class="large-12 columns">' + datastore_quota + '</div>');

    $("#image_quota", context).html(
        '<div class="large-12 columns">' + image_quota + '</div>');

    $("#network_quota", context).html(
        '<div class="large-12 columns">' + network_quota + '</div>');

    setupQuotasBarButtons(resource_info, context);

    _setupImageWidget(context, default_quotas);
    _setupNetworkWidget(context, default_quotas);
    _setupDatastoreWidget(context, default_quotas);

    quotasPanelEditAction(context);

    context.foundation();
  }

  return {
    'initEmptyQuotas': _initEmptyQuotas,
    'quotaBar': _quotaBar,
    'quotaBarMB': _quotaBarMB,
    'quotaBarFloat': _quotaBarFloat,
    'initQuotasPanel': _initQuotasPanel,
    'setupQuotasPanel': _setupQuotasPanel,
    'dialogHTML': _quotas_tmpl,
    'setupQuotasDialog': _setupQuotasDialog,
    'populateQuotasDialog': _populateQuotasDialog,
    'quotaFloatInfo': _quotaFloatInfo,
    'quotaMBInfo': _quotaMBInfo,
    'quotaInfo': _quotaInfo
  };
});

define('templates/helpers/advancedSection',['require','hbs/handlebars'],function(require) {
  var Handlebars = require('hbs/handlebars');

  var id = 0;

  var advancedSection = function(title, options) {
    id += 1;

    var html_id = "advanced_section_" + id;

    return new Handlebars.SafeString(
      '<div class="accordion_advanced">'+
        '<a href="#'+html_id+'" class="accordion_advanced_toggle">'+
            '<i class="fa fa-fw fa-chevron-down"/>'+
            '<i class="fa fa-fw fa-chevron-up"/>'+
            title+
        '</a>'+
        '<div id="'+html_id+'" class="content hidden">'+
          options.fn(this) +
        '</div>'+
      '</div>'
    );
  };

  Handlebars.registerHelper('advancedSection', advancedSection);

  return advancedSection;
});

/* START_TEMPLATE */
define('hbs!utils/accounting/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/advancedSection'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "    <div class=\"row acct_table\">\n      <div class=\"large-12 columns graph_legend\">\n        <h3 class=\"subheader\"><small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU hours", {"name":"tr","hash":{},"data":data})))
    + "</small></h3>\n      </div>\n      <div class=\"large-12 columns\" style=\"overflow:auto\">\n        <table id=\"acct_cpu_datatable\" class=\"datatable twelve\">\n          <thead>\n            <tr>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Date", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            </tr>\n          </thead>\n          <tbody id=\"tbody_acct_cpu_datatable\">\n          </tbody>\n        </table>\n      </div>\n    </div>\n    <div class=\"row acct_table\">\n      <div class=\"large-12 columns graph_legend\">\n        <h3 class=\"subheader\"><small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Memory GB hours", {"name":"tr","hash":{},"data":data})))
    + "</small></h3>\n      </div>\n      <div class=\"large-12 columns\" style=\"overflow:auto\">\n        <table id=\"acct_mem_datatable\" class=\"datatable twelve\">\n          <thead>\n            <tr>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Date", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            </tr>\n          </thead>\n          <tbody id=\"tbody_acct_mem_datatable\">\n          </tbody>\n        </table>\n      </div>\n    </div>\n    <div class=\"row acct_table\">\n      <div class=\"large-12 columns graph_legend\">\n        <h3 class=\"subheader\"><small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk MB hours", {"name":"tr","hash":{},"data":data})))
    + "</small></h3>\n      </div>\n      <div class=\"large-12 columns\" style=\"overflow:auto\">\n        <table id=\"acct_disk_datatable\" class=\"datatable twelve\">\n          <thead>\n            <tr>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Date", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            </tr>\n          </thead>\n          <tbody id=\"tbody_acct_disk_datatable\">\n          </tbody>\n        </table>\n      </div>\n    </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row\">\n  <div id=\"acct_start_time_container\" class=\"left columns\">\n    <label for=\"acct_start_time\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Start time", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input id=\"acct_start_time\" type=\"date\" placeholder=\"2013-06-30\"/>\n  </div>\n  <div id=\"acct_end_time_container\" class=\"left columns\">\n    <label for=\"acct_end_time\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "End time", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input id=\"acct_end_time\" type=\"date\" placeholder=\"2013-12-30\"/>\n  </div>\n  <div id=\"acct_group_by_container\" class=\"left columns\">\n    <label for=\"acct_group_by\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group by", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <select id=\"acct_group_by\" name=\"acct_group_by\">\n      <option value=\"user\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"group\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"vm\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n    </select>\n  </div>\n  <div id=\"acct_owner_container\" class=\"left columns\">\n    <label for=\"acct_owner\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filter", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <div class=\"row\">\n      <div class=\"large-5 columns\">\n        <select id=\"acct_owner\" name=\"acct_owner\">\n          <option value=\"acct_owner_all\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "All", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          <option value=\"acct_owner_group\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          <option value=\"acct_owner_user\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        </select>\n      </div>\n      <div class=\"large-7 columns\">\n        <div id=\"acct_owner_select\"/>\n      </div>\n    </div>\n  </div>\n  <div id=\"acct_button_container\" class=\"left columns\" style=\"margin-top: 15px\">\n    <button class=\"button radius success large-12\" id=\"acct_submit\" type=\"button\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Get Accounting", {"name":"tr","hash":{},"data":data})))
    + "\n    </button>\n  </div>\n</div>\n<br>\n<div id=\"acct_placeholder\">\n  <div class=\"row\">\n    <div class=\"large-8 large-centered columns\">\n      <div class=\"text-center\">\n        <span class=\"fa-stack fa-5x\" style=\"color: #dfdfdf\">\n          <i class=\"fa fa-cloud fa-stack-2x\"></i>\n          <i class=\"fa fa-bar-chart-o fa-stack-1x fa-inverse\"></i>\n        </span>\n        <div id=\"acct_no_data\" class=\"hidden\">\n          <br>\n          <p style=\"font-size: 18px; color: #999\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "There are no accounting records", {"name":"tr","hash":{},"data":data})))
    + "\n          </p>\n        </div>\n      </div>\n    </div>\n  </div>\n</div>\n<div id=\"acct_content\" class=\"hidden\">\n  <div class=\"row\">\n    <div class=\"large-12 columns graph_legend\">\n      <h3 class=\"subheader\"><small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU hours", {"name":"tr","hash":{},"data":data})))
    + "</small></h3>\n    </div>\n    <div class=\"large-12 columns\">\n      <div class=\"large-12 columns centered graph\" id=\"acct_cpu_graph\" style=\"height: 200px;\">\n      </div>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns graph_legend\">\n      <h3 class=\"subheader\"><small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Memory GB hours", {"name":"tr","hash":{},"data":data})))
    + "</small></h3>\n    </div>\n    <div class=\"large-12 columns\">\n      <div class=\"large-12 columns centered graph\" id=\"acct_mem_graph\" style=\"height: 200px;\">\n      </div>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns graph_legend\">\n      <h3 class=\"subheader\"><small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk MB hours", {"name":"tr","hash":{},"data":data})))
    + "</small></h3>\n    </div>\n    <div class=\"large-12 columns\">\n      <div class=\"large-12 columns centered graph\" id=\"acct_disk_graph\" style=\"height: 200px;\">\n      </div>\n    </div>\n  </div>\n  <br>\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Accounting Tables", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n";
},"useData":true});
Handlebars.registerPartial('utils/accounting/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!utils/accounting/empty-graph',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<span class=\"fa-stack fa-2x\" style=\"color: #dfdfdf\"> <i class=\"fa fa-cloud fa-stack-2x\"></i>\n</span>\n<br>\n<span style=\"color: #cfcfcf\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "There is no information available", {"name":"tr","hash":{},"data":data})))
    + "</span>";
},"useData":true});
Handlebars.registerPartial('utils/accounting/empty-graph', t);
return t;
});
/* END_TEMPLATE */
;
/* Javascript plotting library for jQuery, version 0.8.3.

Copyright (c) 2007-2014 IOLA and Ole Laursen.
Licensed under the MIT license.

*/

// first an inline dependency, jquery.colorhelpers.js, we inline it here
// for convenience

/* Plugin for jQuery for working with colors.
 *
 * Version 1.1.
 *
 * Inspiration from jQuery color animation plugin by John Resig.
 *
 * Released under the MIT license by Ole Laursen, October 2009.
 *
 * Examples:
 *
 *   $.color.parse("#fff").scale('rgb', 0.25).add('a', -0.5).toString()
 *   var c = $.color.extract($("#mydiv"), 'background-color');
 *   console.log(c.r, c.g, c.b, c.a);
 *   $.color.make(100, 50, 25, 0.4).toString() // returns "rgba(100,50,25,0.4)"
 *
 * Note that .scale() and .add() return the same modified object
 * instead of making a new one.
 *
 * V. 1.1: Fix error handling so e.g. parsing an empty string does
 * produce a color rather than just crashing.
 */
(function($){$.color={};$.color.make=function(r,g,b,a){var o={};o.r=r||0;o.g=g||0;o.b=b||0;o.a=a!=null?a:1;o.add=function(c,d){for(var i=0;i<c.length;++i)o[c.charAt(i)]+=d;return o.normalize()};o.scale=function(c,f){for(var i=0;i<c.length;++i)o[c.charAt(i)]*=f;return o.normalize()};o.toString=function(){if(o.a>=1){return"rgb("+[o.r,o.g,o.b].join(",")+")"}else{return"rgba("+[o.r,o.g,o.b,o.a].join(",")+")"}};o.normalize=function(){function clamp(min,value,max){return value<min?min:value>max?max:value}o.r=clamp(0,parseInt(o.r),255);o.g=clamp(0,parseInt(o.g),255);o.b=clamp(0,parseInt(o.b),255);o.a=clamp(0,o.a,1);return o};o.clone=function(){return $.color.make(o.r,o.b,o.g,o.a)};return o.normalize()};$.color.extract=function(elem,css){var c;do{c=elem.css(css).toLowerCase();if(c!=""&&c!="transparent")break;elem=elem.parent()}while(elem.length&&!$.nodeName(elem.get(0),"body"));if(c=="rgba(0, 0, 0, 0)")c="transparent";return $.color.parse(c)};$.color.parse=function(str){var res,m=$.color.make;if(res=/rgb\(\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*\)/.exec(str))return m(parseInt(res[1],10),parseInt(res[2],10),parseInt(res[3],10));if(res=/rgba\(\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*,\s*([0-9]+(?:\.[0-9]+)?)\s*\)/.exec(str))return m(parseInt(res[1],10),parseInt(res[2],10),parseInt(res[3],10),parseFloat(res[4]));if(res=/rgb\(\s*([0-9]+(?:\.[0-9]+)?)\%\s*,\s*([0-9]+(?:\.[0-9]+)?)\%\s*,\s*([0-9]+(?:\.[0-9]+)?)\%\s*\)/.exec(str))return m(parseFloat(res[1])*2.55,parseFloat(res[2])*2.55,parseFloat(res[3])*2.55);if(res=/rgba\(\s*([0-9]+(?:\.[0-9]+)?)\%\s*,\s*([0-9]+(?:\.[0-9]+)?)\%\s*,\s*([0-9]+(?:\.[0-9]+)?)\%\s*,\s*([0-9]+(?:\.[0-9]+)?)\s*\)/.exec(str))return m(parseFloat(res[1])*2.55,parseFloat(res[2])*2.55,parseFloat(res[3])*2.55,parseFloat(res[4]));if(res=/#([a-fA-F0-9]{2})([a-fA-F0-9]{2})([a-fA-F0-9]{2})/.exec(str))return m(parseInt(res[1],16),parseInt(res[2],16),parseInt(res[3],16));if(res=/#([a-fA-F0-9])([a-fA-F0-9])([a-fA-F0-9])/.exec(str))return m(parseInt(res[1]+res[1],16),parseInt(res[2]+res[2],16),parseInt(res[3]+res[3],16));var name=$.trim(str).toLowerCase();if(name=="transparent")return m(255,255,255,0);else{res=lookupColors[name]||[0,0,0];return m(res[0],res[1],res[2])}};var lookupColors={aqua:[0,255,255],azure:[240,255,255],beige:[245,245,220],black:[0,0,0],blue:[0,0,255],brown:[165,42,42],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgrey:[169,169,169],darkgreen:[0,100,0],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkviolet:[148,0,211],fuchsia:[255,0,255],gold:[255,215,0],green:[0,128,0],indigo:[75,0,130],khaki:[240,230,140],lightblue:[173,216,230],lightcyan:[224,255,255],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightyellow:[255,255,224],lime:[0,255,0],magenta:[255,0,255],maroon:[128,0,0],navy:[0,0,128],olive:[128,128,0],orange:[255,165,0],pink:[255,192,203],purple:[128,0,128],violet:[128,0,128],red:[255,0,0],silver:[192,192,192],white:[255,255,255],yellow:[255,255,0]}})(jQuery);

// the actual Flot code
(function($) {

	// Cache the prototype hasOwnProperty for faster access

	var hasOwnProperty = Object.prototype.hasOwnProperty;

    // A shim to provide 'detach' to jQuery versions prior to 1.4.  Using a DOM
    // operation produces the same effect as detach, i.e. removing the element
    // without touching its jQuery data.

    // Do not merge this into Flot 0.9, since it requires jQuery 1.4.4+.

    if (!$.fn.detach) {
        $.fn.detach = function() {
            return this.each(function() {
                if (this.parentNode) {
                    this.parentNode.removeChild( this );
                }
            });
        };
    }

	///////////////////////////////////////////////////////////////////////////
	// The Canvas object is a wrapper around an HTML5 <canvas> tag.
	//
	// @constructor
	// @param {string} cls List of classes to apply to the canvas.
	// @param {element} container Element onto which to append the canvas.
	//
	// Requiring a container is a little iffy, but unfortunately canvas
	// operations don't work unless the canvas is attached to the DOM.

	function Canvas(cls, container) {

		var element = container.children("." + cls)[0];

		if (element == null) {

			element = document.createElement("canvas");
			element.className = cls;

			$(element).css({ direction: "ltr", position: "absolute", left: 0, top: 0 })
				.appendTo(container);

			// If HTML5 Canvas isn't available, fall back to [Ex|Flash]canvas

			if (!element.getContext) {
				if (window.G_vmlCanvasManager) {
					element = window.G_vmlCanvasManager.initElement(element);
				} else {
					throw new Error("Canvas is not available. If you're using IE with a fall-back such as Excanvas, then there's either a mistake in your conditional include, or the page has no DOCTYPE and is rendering in Quirks Mode.");
				}
			}
		}

		this.element = element;

		var context = this.context = element.getContext("2d");

		// Determine the screen's ratio of physical to device-independent
		// pixels.  This is the ratio between the canvas width that the browser
		// advertises and the number of pixels actually present in that space.

		// The iPhone 4, for example, has a device-independent width of 320px,
		// but its screen is actually 640px wide.  It therefore has a pixel
		// ratio of 2, while most normal devices have a ratio of 1.

		var devicePixelRatio = window.devicePixelRatio || 1,
			backingStoreRatio =
				context.webkitBackingStorePixelRatio ||
				context.mozBackingStorePixelRatio ||
				context.msBackingStorePixelRatio ||
				context.oBackingStorePixelRatio ||
				context.backingStorePixelRatio || 1;

		this.pixelRatio = devicePixelRatio / backingStoreRatio;

		// Size the canvas to match the internal dimensions of its container

		this.resize(container.width(), container.height());

		// Collection of HTML div layers for text overlaid onto the canvas

		this.textContainer = null;
		this.text = {};

		// Cache of text fragments and metrics, so we can avoid expensively
		// re-calculating them when the plot is re-rendered in a loop.

		this._textCache = {};
	}

	// Resizes the canvas to the given dimensions.
	//
	// @param {number} width New width of the canvas, in pixels.
	// @param {number} width New height of the canvas, in pixels.

	Canvas.prototype.resize = function(width, height) {

		if (width <= 0 || height <= 0) {
			throw new Error("Invalid dimensions for plot, width = " + width + ", height = " + height);
		}

		var element = this.element,
			context = this.context,
			pixelRatio = this.pixelRatio;

		// Resize the canvas, increasing its density based on the display's
		// pixel ratio; basically giving it more pixels without increasing the
		// size of its element, to take advantage of the fact that retina
		// displays have that many more pixels in the same advertised space.

		// Resizing should reset the state (excanvas seems to be buggy though)

		if (this.width != width) {
			element.width = width * pixelRatio;
			element.style.width = width + "px";
			this.width = width;
		}

		if (this.height != height) {
			element.height = height * pixelRatio;
			element.style.height = height + "px";
			this.height = height;
		}

		// Save the context, so we can reset in case we get replotted.  The
		// restore ensure that we're really back at the initial state, and
		// should be safe even if we haven't saved the initial state yet.

		context.restore();
		context.save();

		// Scale the coordinate space to match the display density; so even though we
		// may have twice as many pixels, we still want lines and other drawing to
		// appear at the same size; the extra pixels will just make them crisper.

		context.scale(pixelRatio, pixelRatio);
	};

	// Clears the entire canvas area, not including any overlaid HTML text

	Canvas.prototype.clear = function() {
		this.context.clearRect(0, 0, this.width, this.height);
	};

	// Finishes rendering the canvas, including managing the text overlay.

	Canvas.prototype.render = function() {

		var cache = this._textCache;

		// For each text layer, add elements marked as active that haven't
		// already been rendered, and remove those that are no longer active.

		for (var layerKey in cache) {
			if (hasOwnProperty.call(cache, layerKey)) {

				var layer = this.getTextLayer(layerKey),
					layerCache = cache[layerKey];

				layer.hide();

				for (var styleKey in layerCache) {
					if (hasOwnProperty.call(layerCache, styleKey)) {
						var styleCache = layerCache[styleKey];
						for (var key in styleCache) {
							if (hasOwnProperty.call(styleCache, key)) {

								var positions = styleCache[key].positions;

								for (var i = 0, position; position = positions[i]; i++) {
									if (position.active) {
										if (!position.rendered) {
											layer.append(position.element);
											position.rendered = true;
										}
									} else {
										positions.splice(i--, 1);
										if (position.rendered) {
											position.element.detach();
										}
									}
								}

								if (positions.length == 0) {
									delete styleCache[key];
								}
							}
						}
					}
				}

				layer.show();
			}
		}
	};

	// Creates (if necessary) and returns the text overlay container.
	//
	// @param {string} classes String of space-separated CSS classes used to
	//     uniquely identify the text layer.
	// @return {object} The jQuery-wrapped text-layer div.

	Canvas.prototype.getTextLayer = function(classes) {

		var layer = this.text[classes];

		// Create the text layer if it doesn't exist

		if (layer == null) {

			// Create the text layer container, if it doesn't exist

			if (this.textContainer == null) {
				this.textContainer = $("<div class='flot-text'></div>")
					.css({
						position: "absolute",
						top: 0,
						left: 0,
						bottom: 0,
						right: 0,
						'font-size': "smaller",
						color: "#545454"
					})
					.insertAfter(this.element);
			}

			layer = this.text[classes] = $("<div></div>")
				.addClass(classes)
				.css({
					position: "absolute",
					top: 0,
					left: 0,
					bottom: 0,
					right: 0
				})
				.appendTo(this.textContainer);
		}

		return layer;
	};

	// Creates (if necessary) and returns a text info object.
	//
	// The object looks like this:
	//
	// {
	//     width: Width of the text's wrapper div.
	//     height: Height of the text's wrapper div.
	//     element: The jQuery-wrapped HTML div containing the text.
	//     positions: Array of positions at which this text is drawn.
	// }
	//
	// The positions array contains objects that look like this:
	//
	// {
	//     active: Flag indicating whether the text should be visible.
	//     rendered: Flag indicating whether the text is currently visible.
	//     element: The jQuery-wrapped HTML div containing the text.
	//     x: X coordinate at which to draw the text.
	//     y: Y coordinate at which to draw the text.
	// }
	//
	// Each position after the first receives a clone of the original element.
	//
	// The idea is that that the width, height, and general 'identity' of the
	// text is constant no matter where it is placed; the placements are a
	// secondary property.
	//
	// Canvas maintains a cache of recently-used text info objects; getTextInfo
	// either returns the cached element or creates a new entry.
	//
	// @param {string} layer A string of space-separated CSS classes uniquely
	//     identifying the layer containing this text.
	// @param {string} text Text string to retrieve info for.
	// @param {(string|object)=} font Either a string of space-separated CSS
	//     classes or a font-spec object, defining the text's font and style.
	// @param {number=} angle Angle at which to rotate the text, in degrees.
	//     Angle is currently unused, it will be implemented in the future.
	// @param {number=} width Maximum width of the text before it wraps.
	// @return {object} a text info object.

	Canvas.prototype.getTextInfo = function(layer, text, font, angle, width) {

		var textStyle, layerCache, styleCache, info;

		// Cast the value to a string, in case we were given a number or such

		text = "" + text;

		// If the font is a font-spec object, generate a CSS font definition

		if (typeof font === "object") {
			textStyle = font.style + " " + font.variant + " " + font.weight + " " + font.size + "px/" + font.lineHeight + "px " + font.family;
		} else {
			textStyle = font;
		}

		// Retrieve (or create) the cache for the text's layer and styles

		layerCache = this._textCache[layer];

		if (layerCache == null) {
			layerCache = this._textCache[layer] = {};
		}

		styleCache = layerCache[textStyle];

		if (styleCache == null) {
			styleCache = layerCache[textStyle] = {};
		}

		info = styleCache[text];

		// If we can't find a matching element in our cache, create a new one

		if (info == null) {

			var element = $("<div></div>").html(text)
				.css({
					position: "absolute",
					'max-width': width,
					top: -9999
				})
				.appendTo(this.getTextLayer(layer));

			if (typeof font === "object") {
				element.css({
					font: textStyle,
					color: font.color
				});
			} else if (typeof font === "string") {
				element.addClass(font);
			}

			info = styleCache[text] = {
				width: element.outerWidth(true),
				height: element.outerHeight(true),
				element: element,
				positions: []
			};

			element.detach();
		}

		return info;
	};

	// Adds a text string to the canvas text overlay.
	//
	// The text isn't drawn immediately; it is marked as rendering, which will
	// result in its addition to the canvas on the next render pass.
	//
	// @param {string} layer A string of space-separated CSS classes uniquely
	//     identifying the layer containing this text.
	// @param {number} x X coordinate at which to draw the text.
	// @param {number} y Y coordinate at which to draw the text.
	// @param {string} text Text string to draw.
	// @param {(string|object)=} font Either a string of space-separated CSS
	//     classes or a font-spec object, defining the text's font and style.
	// @param {number=} angle Angle at which to rotate the text, in degrees.
	//     Angle is currently unused, it will be implemented in the future.
	// @param {number=} width Maximum width of the text before it wraps.
	// @param {string=} halign Horizontal alignment of the text; either "left",
	//     "center" or "right".
	// @param {string=} valign Vertical alignment of the text; either "top",
	//     "middle" or "bottom".

	Canvas.prototype.addText = function(layer, x, y, text, font, angle, width, halign, valign) {

		var info = this.getTextInfo(layer, text, font, angle, width),
			positions = info.positions;

		// Tweak the div's position to match the text's alignment

		if (halign == "center") {
			x -= info.width / 2;
		} else if (halign == "right") {
			x -= info.width;
		}

		if (valign == "middle") {
			y -= info.height / 2;
		} else if (valign == "bottom") {
			y -= info.height;
		}

		// Determine whether this text already exists at this position.
		// If so, mark it for inclusion in the next render pass.

		for (var i = 0, position; position = positions[i]; i++) {
			if (position.x == x && position.y == y) {
				position.active = true;
				return;
			}
		}

		// If the text doesn't exist at this position, create a new entry

		// For the very first position we'll re-use the original element,
		// while for subsequent ones we'll clone it.

		position = {
			active: true,
			rendered: false,
			element: positions.length ? info.element.clone() : info.element,
			x: x,
			y: y
		};

		positions.push(position);

		// Move the element to its final position within the container

		position.element.css({
			top: Math.round(y),
			left: Math.round(x),
			'text-align': halign	// In case the text wraps
		});
	};

	// Removes one or more text strings from the canvas text overlay.
	//
	// If no parameters are given, all text within the layer is removed.
	//
	// Note that the text is not immediately removed; it is simply marked as
	// inactive, which will result in its removal on the next render pass.
	// This avoids the performance penalty for 'clear and redraw' behavior,
	// where we potentially get rid of all text on a layer, but will likely
	// add back most or all of it later, as when redrawing axes, for example.
	//
	// @param {string} layer A string of space-separated CSS classes uniquely
	//     identifying the layer containing this text.
	// @param {number=} x X coordinate of the text.
	// @param {number=} y Y coordinate of the text.
	// @param {string=} text Text string to remove.
	// @param {(string|object)=} font Either a string of space-separated CSS
	//     classes or a font-spec object, defining the text's font and style.
	// @param {number=} angle Angle at which the text is rotated, in degrees.
	//     Angle is currently unused, it will be implemented in the future.

	Canvas.prototype.removeText = function(layer, x, y, text, font, angle) {
		if (text == null) {
			var layerCache = this._textCache[layer];
			if (layerCache != null) {
				for (var styleKey in layerCache) {
					if (hasOwnProperty.call(layerCache, styleKey)) {
						var styleCache = layerCache[styleKey];
						for (var key in styleCache) {
							if (hasOwnProperty.call(styleCache, key)) {
								var positions = styleCache[key].positions;
								for (var i = 0, position; position = positions[i]; i++) {
									position.active = false;
								}
							}
						}
					}
				}
			}
		} else {
			var positions = this.getTextInfo(layer, text, font, angle).positions;
			for (var i = 0, position; position = positions[i]; i++) {
				if (position.x == x && position.y == y) {
					position.active = false;
				}
			}
		}
	};

	///////////////////////////////////////////////////////////////////////////
	// The top-level container for the entire plot.

    function Plot(placeholder, data_, options_, plugins) {
        // data is on the form:
        //   [ series1, series2 ... ]
        // where series is either just the data as [ [x1, y1], [x2, y2], ... ]
        // or { data: [ [x1, y1], [x2, y2], ... ], label: "some label", ... }

        var series = [],
            options = {
                // the color theme used for graphs
                colors: ["#edc240", "#afd8f8", "#cb4b4b", "#4da74d", "#9440ed"],
                legend: {
                    show: true,
                    noColumns: 1, // number of colums in legend table
                    labelFormatter: null, // fn: string -> string
                    labelBoxBorderColor: "#ccc", // border color for the little label boxes
                    container: null, // container (as jQuery object) to put legend in, null means default on top of graph
                    position: "ne", // position of default legend container within plot
                    margin: 5, // distance from grid edge to default legend container within plot
                    backgroundColor: null, // null means auto-detect
                    backgroundOpacity: 0.85, // set to 0 to avoid background
                    sorted: null    // default to no legend sorting
                },
                xaxis: {
                    show: null, // null = auto-detect, true = always, false = never
                    position: "bottom", // or "top"
                    mode: null, // null or "time"
                    font: null, // null (derived from CSS in placeholder) or object like { size: 11, lineHeight: 13, style: "italic", weight: "bold", family: "sans-serif", variant: "small-caps" }
                    color: null, // base color, labels, ticks
                    tickColor: null, // possibly different color of ticks, e.g. "rgba(0,0,0,0.15)"
                    transform: null, // null or f: number -> number to transform axis
                    inverseTransform: null, // if transform is set, this should be the inverse function
                    min: null, // min. value to show, null means set automatically
                    max: null, // max. value to show, null means set automatically
                    autoscaleMargin: null, // margin in % to add if auto-setting min/max
                    ticks: null, // either [1, 3] or [[1, "a"], 3] or (fn: axis info -> ticks) or app. number of ticks for auto-ticks
                    tickFormatter: null, // fn: number -> string
                    labelWidth: null, // size of tick labels in pixels
                    labelHeight: null,
                    reserveSpace: null, // whether to reserve space even if axis isn't shown
                    tickLength: null, // size in pixels of ticks, or "full" for whole line
                    alignTicksWithAxis: null, // axis number or null for no sync
                    tickDecimals: null, // no. of decimals, null means auto
                    tickSize: null, // number or [number, "unit"]
                    minTickSize: null // number or [number, "unit"]
                },
                yaxis: {
                    autoscaleMargin: 0.02,
                    position: "left" // or "right"
                },
                xaxes: [],
                yaxes: [],
                series: {
                    points: {
                        show: false,
                        radius: 3,
                        lineWidth: 2, // in pixels
                        fill: true,
                        fillColor: "#ffffff",
                        symbol: "circle" // or callback
                    },
                    lines: {
                        // we don't put in show: false so we can see
                        // whether lines were actively disabled
                        lineWidth: 2, // in pixels
                        fill: false,
                        fillColor: null,
                        steps: false
                        // Omit 'zero', so we can later default its value to
                        // match that of the 'fill' option.
                    },
                    bars: {
                        show: false,
                        lineWidth: 2, // in pixels
                        barWidth: 1, // in units of the x axis
                        fill: true,
                        fillColor: null,
                        align: "left", // "left", "right", or "center"
                        horizontal: false,
                        zero: true
                    },
                    shadowSize: 3,
                    highlightColor: null
                },
                grid: {
                    show: true,
                    aboveData: false,
                    color: "#545454", // primary color used for outline and labels
                    backgroundColor: null, // null for transparent, else color
                    borderColor: null, // set if different from the grid color
                    tickColor: null, // color for the ticks, e.g. "rgba(0,0,0,0.15)"
                    margin: 0, // distance from the canvas edge to the grid
                    labelMargin: 5, // in pixels
                    axisMargin: 8, // in pixels
                    borderWidth: 2, // in pixels
                    minBorderMargin: null, // in pixels, null means taken from points radius
                    markings: null, // array of ranges or fn: axes -> array of ranges
                    markingsColor: "#f4f4f4",
                    markingsLineWidth: 2,
                    // interactive stuff
                    clickable: false,
                    hoverable: false,
                    autoHighlight: true, // highlight in case mouse is near
                    mouseActiveRadius: 10 // how far the mouse can be away to activate an item
                },
                interaction: {
                    redrawOverlayInterval: 1000/60 // time between updates, -1 means in same flow
                },
                hooks: {}
            },
        surface = null,     // the canvas for the plot itself
        overlay = null,     // canvas for interactive stuff on top of plot
        eventHolder = null, // jQuery object that events should be bound to
        ctx = null, octx = null,
        xaxes = [], yaxes = [],
        plotOffset = { left: 0, right: 0, top: 0, bottom: 0},
        plotWidth = 0, plotHeight = 0,
        hooks = {
            processOptions: [],
            processRawData: [],
            processDatapoints: [],
            processOffset: [],
            drawBackground: [],
            drawSeries: [],
            draw: [],
            bindEvents: [],
            drawOverlay: [],
            shutdown: []
        },
        plot = this;

        // public functions
        plot.setData = setData;
        plot.setupGrid = setupGrid;
        plot.draw = draw;
        plot.getPlaceholder = function() { return placeholder; };
        plot.getCanvas = function() { return surface.element; };
        plot.getPlotOffset = function() { return plotOffset; };
        plot.width = function () { return plotWidth; };
        plot.height = function () { return plotHeight; };
        plot.offset = function () {
            var o = eventHolder.offset();
            o.left += plotOffset.left;
            o.top += plotOffset.top;
            return o;
        };
        plot.getData = function () { return series; };
        plot.getAxes = function () {
            var res = {}, i;
            $.each(xaxes.concat(yaxes), function (_, axis) {
                if (axis)
                    res[axis.direction + (axis.n != 1 ? axis.n : "") + "axis"] = axis;
            });
            return res;
        };
        plot.getXAxes = function () { return xaxes; };
        plot.getYAxes = function () { return yaxes; };
        plot.c2p = canvasToAxisCoords;
        plot.p2c = axisToCanvasCoords;
        plot.getOptions = function () { return options; };
        plot.highlight = highlight;
        plot.unhighlight = unhighlight;
        plot.triggerRedrawOverlay = triggerRedrawOverlay;
        plot.pointOffset = function(point) {
            return {
                left: parseInt(xaxes[axisNumber(point, "x") - 1].p2c(+point.x) + plotOffset.left, 10),
                top: parseInt(yaxes[axisNumber(point, "y") - 1].p2c(+point.y) + plotOffset.top, 10)
            };
        };
        plot.shutdown = shutdown;
        plot.destroy = function () {
            shutdown();
            placeholder.removeData("plot").empty();

            series = [];
            options = null;
            surface = null;
            overlay = null;
            eventHolder = null;
            ctx = null;
            octx = null;
            xaxes = [];
            yaxes = [];
            hooks = null;
            highlights = [];
            plot = null;
        };
        plot.resize = function () {
        	var width = placeholder.width(),
        		height = placeholder.height();
            surface.resize(width, height);
            overlay.resize(width, height);
        };

        // public attributes
        plot.hooks = hooks;

        // initialize
        initPlugins(plot);
        parseOptions(options_);
        setupCanvases();
        setData(data_);
        setupGrid();
        draw();
        bindEvents();


        function executeHooks(hook, args) {
            args = [plot].concat(args);
            for (var i = 0; i < hook.length; ++i)
                hook[i].apply(this, args);
        }

        function initPlugins() {

            // References to key classes, allowing plugins to modify them

            var classes = {
                Canvas: Canvas
            };

            for (var i = 0; i < plugins.length; ++i) {
                var p = plugins[i];
                p.init(plot, classes);
                if (p.options)
                    $.extend(true, options, p.options);
            }
        }

        function parseOptions(opts) {

            $.extend(true, options, opts);

            // $.extend merges arrays, rather than replacing them.  When less
            // colors are provided than the size of the default palette, we
            // end up with those colors plus the remaining defaults, which is
            // not expected behavior; avoid it by replacing them here.

            if (opts && opts.colors) {
            	options.colors = opts.colors;
            }

            if (options.xaxis.color == null)
                options.xaxis.color = $.color.parse(options.grid.color).scale('a', 0.22).toString();
            if (options.yaxis.color == null)
                options.yaxis.color = $.color.parse(options.grid.color).scale('a', 0.22).toString();

            if (options.xaxis.tickColor == null) // grid.tickColor for back-compatibility
                options.xaxis.tickColor = options.grid.tickColor || options.xaxis.color;
            if (options.yaxis.tickColor == null) // grid.tickColor for back-compatibility
                options.yaxis.tickColor = options.grid.tickColor || options.yaxis.color;

            if (options.grid.borderColor == null)
                options.grid.borderColor = options.grid.color;
            if (options.grid.tickColor == null)
                options.grid.tickColor = $.color.parse(options.grid.color).scale('a', 0.22).toString();

            // Fill in defaults for axis options, including any unspecified
            // font-spec fields, if a font-spec was provided.

            // If no x/y axis options were provided, create one of each anyway,
            // since the rest of the code assumes that they exist.

            var i, axisOptions, axisCount,
                fontSize = placeholder.css("font-size"),
                fontSizeDefault = fontSize ? +fontSize.replace("px", "") : 13,
                fontDefaults = {
                    style: placeholder.css("font-style"),
                    size: Math.round(0.8 * fontSizeDefault),
                    variant: placeholder.css("font-variant"),
                    weight: placeholder.css("font-weight"),
                    family: placeholder.css("font-family")
                };

            axisCount = options.xaxes.length || 1;
            for (i = 0; i < axisCount; ++i) {

                axisOptions = options.xaxes[i];
                if (axisOptions && !axisOptions.tickColor) {
                    axisOptions.tickColor = axisOptions.color;
                }

                axisOptions = $.extend(true, {}, options.xaxis, axisOptions);
                options.xaxes[i] = axisOptions;

                if (axisOptions.font) {
                    axisOptions.font = $.extend({}, fontDefaults, axisOptions.font);
                    if (!axisOptions.font.color) {
                        axisOptions.font.color = axisOptions.color;
                    }
                    if (!axisOptions.font.lineHeight) {
                        axisOptions.font.lineHeight = Math.round(axisOptions.font.size * 1.15);
                    }
                }
            }

            axisCount = options.yaxes.length || 1;
            for (i = 0; i < axisCount; ++i) {

                axisOptions = options.yaxes[i];
                if (axisOptions && !axisOptions.tickColor) {
                    axisOptions.tickColor = axisOptions.color;
                }

                axisOptions = $.extend(true, {}, options.yaxis, axisOptions);
                options.yaxes[i] = axisOptions;

                if (axisOptions.font) {
                    axisOptions.font = $.extend({}, fontDefaults, axisOptions.font);
                    if (!axisOptions.font.color) {
                        axisOptions.font.color = axisOptions.color;
                    }
                    if (!axisOptions.font.lineHeight) {
                        axisOptions.font.lineHeight = Math.round(axisOptions.font.size * 1.15);
                    }
                }
            }

            // backwards compatibility, to be removed in future
            if (options.xaxis.noTicks && options.xaxis.ticks == null)
                options.xaxis.ticks = options.xaxis.noTicks;
            if (options.yaxis.noTicks && options.yaxis.ticks == null)
                options.yaxis.ticks = options.yaxis.noTicks;
            if (options.x2axis) {
                options.xaxes[1] = $.extend(true, {}, options.xaxis, options.x2axis);
                options.xaxes[1].position = "top";
                // Override the inherit to allow the axis to auto-scale
                if (options.x2axis.min == null) {
                    options.xaxes[1].min = null;
                }
                if (options.x2axis.max == null) {
                    options.xaxes[1].max = null;
                }
            }
            if (options.y2axis) {
                options.yaxes[1] = $.extend(true, {}, options.yaxis, options.y2axis);
                options.yaxes[1].position = "right";
                // Override the inherit to allow the axis to auto-scale
                if (options.y2axis.min == null) {
                    options.yaxes[1].min = null;
                }
                if (options.y2axis.max == null) {
                    options.yaxes[1].max = null;
                }
            }
            if (options.grid.coloredAreas)
                options.grid.markings = options.grid.coloredAreas;
            if (options.grid.coloredAreasColor)
                options.grid.markingsColor = options.grid.coloredAreasColor;
            if (options.lines)
                $.extend(true, options.series.lines, options.lines);
            if (options.points)
                $.extend(true, options.series.points, options.points);
            if (options.bars)
                $.extend(true, options.series.bars, options.bars);
            if (options.shadowSize != null)
                options.series.shadowSize = options.shadowSize;
            if (options.highlightColor != null)
                options.series.highlightColor = options.highlightColor;

            // save options on axes for future reference
            for (i = 0; i < options.xaxes.length; ++i)
                getOrCreateAxis(xaxes, i + 1).options = options.xaxes[i];
            for (i = 0; i < options.yaxes.length; ++i)
                getOrCreateAxis(yaxes, i + 1).options = options.yaxes[i];

            // add hooks from options
            for (var n in hooks)
                if (options.hooks[n] && options.hooks[n].length)
                    hooks[n] = hooks[n].concat(options.hooks[n]);

            executeHooks(hooks.processOptions, [options]);
        }

        function setData(d) {
            series = parseData(d);
            fillInSeriesOptions();
            processData();
        }

        function parseData(d) {
            var res = [];
            for (var i = 0; i < d.length; ++i) {
                var s = $.extend(true, {}, options.series);

                if (d[i].data != null) {
                    s.data = d[i].data; // move the data instead of deep-copy
                    delete d[i].data;

                    $.extend(true, s, d[i]);

                    d[i].data = s.data;
                }
                else
                    s.data = d[i];
                res.push(s);
            }

            return res;
        }

        function axisNumber(obj, coord) {
            var a = obj[coord + "axis"];
            if (typeof a == "object") // if we got a real axis, extract number
                a = a.n;
            if (typeof a != "number")
                a = 1; // default to first axis
            return a;
        }

        function allAxes() {
            // return flat array without annoying null entries
            return $.grep(xaxes.concat(yaxes), function (a) { return a; });
        }

        function canvasToAxisCoords(pos) {
            // return an object with x/y corresponding to all used axes
            var res = {}, i, axis;
            for (i = 0; i < xaxes.length; ++i) {
                axis = xaxes[i];
                if (axis && axis.used)
                    res["x" + axis.n] = axis.c2p(pos.left);
            }

            for (i = 0; i < yaxes.length; ++i) {
                axis = yaxes[i];
                if (axis && axis.used)
                    res["y" + axis.n] = axis.c2p(pos.top);
            }

            if (res.x1 !== undefined)
                res.x = res.x1;
            if (res.y1 !== undefined)
                res.y = res.y1;

            return res;
        }

        function axisToCanvasCoords(pos) {
            // get canvas coords from the first pair of x/y found in pos
            var res = {}, i, axis, key;

            for (i = 0; i < xaxes.length; ++i) {
                axis = xaxes[i];
                if (axis && axis.used) {
                    key = "x" + axis.n;
                    if (pos[key] == null && axis.n == 1)
                        key = "x";

                    if (pos[key] != null) {
                        res.left = axis.p2c(pos[key]);
                        break;
                    }
                }
            }

            for (i = 0; i < yaxes.length; ++i) {
                axis = yaxes[i];
                if (axis && axis.used) {
                    key = "y" + axis.n;
                    if (pos[key] == null && axis.n == 1)
                        key = "y";

                    if (pos[key] != null) {
                        res.top = axis.p2c(pos[key]);
                        break;
                    }
                }
            }

            return res;
        }

        function getOrCreateAxis(axes, number) {
            if (!axes[number - 1])
                axes[number - 1] = {
                    n: number, // save the number for future reference
                    direction: axes == xaxes ? "x" : "y",
                    options: $.extend(true, {}, axes == xaxes ? options.xaxis : options.yaxis)
                };

            return axes[number - 1];
        }

        function fillInSeriesOptions() {

            var neededColors = series.length, maxIndex = -1, i;

            // Subtract the number of series that already have fixed colors or
            // color indexes from the number that we still need to generate.

            for (i = 0; i < series.length; ++i) {
                var sc = series[i].color;
                if (sc != null) {
                    neededColors--;
                    if (typeof sc == "number" && sc > maxIndex) {
                        maxIndex = sc;
                    }
                }
            }

            // If any of the series have fixed color indexes, then we need to
            // generate at least as many colors as the highest index.

            if (neededColors <= maxIndex) {
                neededColors = maxIndex + 1;
            }

            // Generate all the colors, using first the option colors and then
            // variations on those colors once they're exhausted.

            var c, colors = [], colorPool = options.colors,
                colorPoolSize = colorPool.length, variation = 0;

            for (i = 0; i < neededColors; i++) {

                c = $.color.parse(colorPool[i % colorPoolSize] || "#666");

                // Each time we exhaust the colors in the pool we adjust
                // a scaling factor used to produce more variations on
                // those colors. The factor alternates negative/positive
                // to produce lighter/darker colors.

                // Reset the variation after every few cycles, or else
                // it will end up producing only white or black colors.

                if (i % colorPoolSize == 0 && i) {
                    if (variation >= 0) {
                        if (variation < 0.5) {
                            variation = -variation - 0.2;
                        } else variation = 0;
                    } else variation = -variation;
                }

                colors[i] = c.scale('rgb', 1 + variation);
            }

            // Finalize the series options, filling in their colors

            var colori = 0, s;
            for (i = 0; i < series.length; ++i) {
                s = series[i];

                // assign colors
                if (s.color == null) {
                    s.color = colors[colori].toString();
                    ++colori;
                }
                else if (typeof s.color == "number")
                    s.color = colors[s.color].toString();

                // turn on lines automatically in case nothing is set
                if (s.lines.show == null) {
                    var v, show = true;
                    for (v in s)
                        if (s[v] && s[v].show) {
                            show = false;
                            break;
                        }
                    if (show)
                        s.lines.show = true;
                }

                // If nothing was provided for lines.zero, default it to match
                // lines.fill, since areas by default should extend to zero.

                if (s.lines.zero == null) {
                    s.lines.zero = !!s.lines.fill;
                }

                // setup axes
                s.xaxis = getOrCreateAxis(xaxes, axisNumber(s, "x"));
                s.yaxis = getOrCreateAxis(yaxes, axisNumber(s, "y"));
            }
        }

        function processData() {
            var topSentry = Number.POSITIVE_INFINITY,
                bottomSentry = Number.NEGATIVE_INFINITY,
                fakeInfinity = Number.MAX_VALUE,
                i, j, k, m, length,
                s, points, ps, x, y, axis, val, f, p,
                data, format;

            function updateAxis(axis, min, max) {
                if (min < axis.datamin && min != -fakeInfinity)
                    axis.datamin = min;
                if (max > axis.datamax && max != fakeInfinity)
                    axis.datamax = max;
            }

            $.each(allAxes(), function (_, axis) {
                // init axis
                axis.datamin = topSentry;
                axis.datamax = bottomSentry;
                axis.used = false;
            });

            for (i = 0; i < series.length; ++i) {
                s = series[i];
                s.datapoints = { points: [] };

                executeHooks(hooks.processRawData, [ s, s.data, s.datapoints ]);
            }

            // first pass: clean and copy data
            for (i = 0; i < series.length; ++i) {
                s = series[i];

                data = s.data;
                format = s.datapoints.format;

                if (!format) {
                    format = [];
                    // find out how to copy
                    format.push({ x: true, number: true, required: true });
                    format.push({ y: true, number: true, required: true });

                    if (s.bars.show || (s.lines.show && s.lines.fill)) {
                        var autoscale = !!((s.bars.show && s.bars.zero) || (s.lines.show && s.lines.zero));
                        format.push({ y: true, number: true, required: false, defaultValue: 0, autoscale: autoscale });
                        if (s.bars.horizontal) {
                            delete format[format.length - 1].y;
                            format[format.length - 1].x = true;
                        }
                    }

                    s.datapoints.format = format;
                }

                if (s.datapoints.pointsize != null)
                    continue; // already filled in

                s.datapoints.pointsize = format.length;

                ps = s.datapoints.pointsize;
                points = s.datapoints.points;

                var insertSteps = s.lines.show && s.lines.steps;
                s.xaxis.used = s.yaxis.used = true;

                for (j = k = 0; j < data.length; ++j, k += ps) {
                    p = data[j];

                    var nullify = p == null;
                    if (!nullify) {
                        for (m = 0; m < ps; ++m) {
                            val = p[m];
                            f = format[m];

                            if (f) {
                                if (f.number && val != null) {
                                    val = +val; // convert to number
                                    if (isNaN(val))
                                        val = null;
                                    else if (val == Infinity)
                                        val = fakeInfinity;
                                    else if (val == -Infinity)
                                        val = -fakeInfinity;
                                }

                                if (val == null) {
                                    if (f.required)
                                        nullify = true;

                                    if (f.defaultValue != null)
                                        val = f.defaultValue;
                                }
                            }

                            points[k + m] = val;
                        }
                    }

                    if (nullify) {
                        for (m = 0; m < ps; ++m) {
                            val = points[k + m];
                            if (val != null) {
                                f = format[m];
                                // extract min/max info
                                if (f.autoscale !== false) {
                                    if (f.x) {
                                        updateAxis(s.xaxis, val, val);
                                    }
                                    if (f.y) {
                                        updateAxis(s.yaxis, val, val);
                                    }
                                }
                            }
                            points[k + m] = null;
                        }
                    }
                    else {
                        // a little bit of line specific stuff that
                        // perhaps shouldn't be here, but lacking
                        // better means...
                        if (insertSteps && k > 0
                            && points[k - ps] != null
                            && points[k - ps] != points[k]
                            && points[k - ps + 1] != points[k + 1]) {
                            // copy the point to make room for a middle point
                            for (m = 0; m < ps; ++m)
                                points[k + ps + m] = points[k + m];

                            // middle point has same y
                            points[k + 1] = points[k - ps + 1];

                            // we've added a point, better reflect that
                            k += ps;
                        }
                    }
                }
            }

            // give the hooks a chance to run
            for (i = 0; i < series.length; ++i) {
                s = series[i];

                executeHooks(hooks.processDatapoints, [ s, s.datapoints]);
            }

            // second pass: find datamax/datamin for auto-scaling
            for (i = 0; i < series.length; ++i) {
                s = series[i];
                points = s.datapoints.points;
                ps = s.datapoints.pointsize;
                format = s.datapoints.format;

                var xmin = topSentry, ymin = topSentry,
                    xmax = bottomSentry, ymax = bottomSentry;

                for (j = 0; j < points.length; j += ps) {
                    if (points[j] == null)
                        continue;

                    for (m = 0; m < ps; ++m) {
                        val = points[j + m];
                        f = format[m];
                        if (!f || f.autoscale === false || val == fakeInfinity || val == -fakeInfinity)
                            continue;

                        if (f.x) {
                            if (val < xmin)
                                xmin = val;
                            if (val > xmax)
                                xmax = val;
                        }
                        if (f.y) {
                            if (val < ymin)
                                ymin = val;
                            if (val > ymax)
                                ymax = val;
                        }
                    }
                }

                if (s.bars.show) {
                    // make sure we got room for the bar on the dancing floor
                    var delta;

                    switch (s.bars.align) {
                        case "left":
                            delta = 0;
                            break;
                        case "right":
                            delta = -s.bars.barWidth;
                            break;
                        default:
                            delta = -s.bars.barWidth / 2;
                    }

                    if (s.bars.horizontal) {
                        ymin += delta;
                        ymax += delta + s.bars.barWidth;
                    }
                    else {
                        xmin += delta;
                        xmax += delta + s.bars.barWidth;
                    }
                }

                updateAxis(s.xaxis, xmin, xmax);
                updateAxis(s.yaxis, ymin, ymax);
            }

            $.each(allAxes(), function (_, axis) {
                if (axis.datamin == topSentry)
                    axis.datamin = null;
                if (axis.datamax == bottomSentry)
                    axis.datamax = null;
            });
        }

        function setupCanvases() {

            // Make sure the placeholder is clear of everything except canvases
            // from a previous plot in this container that we'll try to re-use.

            placeholder.css("padding", 0) // padding messes up the positioning
                .children().filter(function(){
                    return !$(this).hasClass("flot-overlay") && !$(this).hasClass('flot-base');
                }).remove();

            if (placeholder.css("position") == 'static')
                placeholder.css("position", "relative"); // for positioning labels and overlay

            surface = new Canvas("flot-base", placeholder);
            overlay = new Canvas("flot-overlay", placeholder); // overlay canvas for interactive features

            ctx = surface.context;
            octx = overlay.context;

            // define which element we're listening for events on
            eventHolder = $(overlay.element).unbind();

            // If we're re-using a plot object, shut down the old one

            var existing = placeholder.data("plot");

            if (existing) {
                existing.shutdown();
                overlay.clear();
            }

            // save in case we get replotted
            placeholder.data("plot", plot);
        }

        function bindEvents() {
            // bind events
            if (options.grid.hoverable) {
                eventHolder.mousemove(onMouseMove);

                // Use bind, rather than .mouseleave, because we officially
                // still support jQuery 1.2.6, which doesn't define a shortcut
                // for mouseenter or mouseleave.  This was a bug/oversight that
                // was fixed somewhere around 1.3.x.  We can return to using
                // .mouseleave when we drop support for 1.2.6.

                eventHolder.bind("mouseleave", onMouseLeave);
            }

            if (options.grid.clickable)
                eventHolder.click(onClick);

            executeHooks(hooks.bindEvents, [eventHolder]);
        }

        function shutdown() {
            if (redrawTimeout)
                clearTimeout(redrawTimeout);

            eventHolder.unbind("mousemove", onMouseMove);
            eventHolder.unbind("mouseleave", onMouseLeave);
            eventHolder.unbind("click", onClick);

            executeHooks(hooks.shutdown, [eventHolder]);
        }

        function setTransformationHelpers(axis) {
            // set helper functions on the axis, assumes plot area
            // has been computed already

            function identity(x) { return x; }

            var s, m, t = axis.options.transform || identity,
                it = axis.options.inverseTransform;

            // precompute how much the axis is scaling a point
            // in canvas space
            if (axis.direction == "x") {
                s = axis.scale = plotWidth / Math.abs(t(axis.max) - t(axis.min));
                m = Math.min(t(axis.max), t(axis.min));
            }
            else {
                s = axis.scale = plotHeight / Math.abs(t(axis.max) - t(axis.min));
                s = -s;
                m = Math.max(t(axis.max), t(axis.min));
            }

            // data point to canvas coordinate
            if (t == identity) // slight optimization
                axis.p2c = function (p) { return (p - m) * s; };
            else
                axis.p2c = function (p) { return (t(p) - m) * s; };
            // canvas coordinate to data point
            if (!it)
                axis.c2p = function (c) { return m + c / s; };
            else
                axis.c2p = function (c) { return it(m + c / s); };
        }

        function measureTickLabels(axis) {

            var opts = axis.options,
                ticks = axis.ticks || [],
                labelWidth = opts.labelWidth || 0,
                labelHeight = opts.labelHeight || 0,
                maxWidth = labelWidth || (axis.direction == "x" ? Math.floor(surface.width / (ticks.length || 1)) : null),
                legacyStyles = axis.direction + "Axis " + axis.direction + axis.n + "Axis",
                layer = "flot-" + axis.direction + "-axis flot-" + axis.direction + axis.n + "-axis " + legacyStyles,
                font = opts.font || "flot-tick-label tickLabel";

            for (var i = 0; i < ticks.length; ++i) {

                var t = ticks[i];

                if (!t.label)
                    continue;

                var info = surface.getTextInfo(layer, t.label, font, null, maxWidth);

                labelWidth = Math.max(labelWidth, info.width);
                labelHeight = Math.max(labelHeight, info.height);
            }

            axis.labelWidth = opts.labelWidth || labelWidth;
            axis.labelHeight = opts.labelHeight || labelHeight;
        }

        function allocateAxisBoxFirstPhase(axis) {
            // find the bounding box of the axis by looking at label
            // widths/heights and ticks, make room by diminishing the
            // plotOffset; this first phase only looks at one
            // dimension per axis, the other dimension depends on the
            // other axes so will have to wait

            var lw = axis.labelWidth,
                lh = axis.labelHeight,
                pos = axis.options.position,
                isXAxis = axis.direction === "x",
                tickLength = axis.options.tickLength,
                axisMargin = options.grid.axisMargin,
                padding = options.grid.labelMargin,
                innermost = true,
                outermost = true,
                first = true,
                found = false;

            // Determine the axis's position in its direction and on its side

            $.each(isXAxis ? xaxes : yaxes, function(i, a) {
                if (a && (a.show || a.reserveSpace)) {
                    if (a === axis) {
                        found = true;
                    } else if (a.options.position === pos) {
                        if (found) {
                            outermost = false;
                        } else {
                            innermost = false;
                        }
                    }
                    if (!found) {
                        first = false;
                    }
                }
            });

            // The outermost axis on each side has no margin

            if (outermost) {
                axisMargin = 0;
            }

            // The ticks for the first axis in each direction stretch across

            if (tickLength == null) {
                tickLength = first ? "full" : 5;
            }

            if (!isNaN(+tickLength))
                padding += +tickLength;

            if (isXAxis) {
                lh += padding;

                if (pos == "bottom") {
                    plotOffset.bottom += lh + axisMargin;
                    axis.box = { top: surface.height - plotOffset.bottom, height: lh };
                }
                else {
                    axis.box = { top: plotOffset.top + axisMargin, height: lh };
                    plotOffset.top += lh + axisMargin;
                }
            }
            else {
                lw += padding;

                if (pos == "left") {
                    axis.box = { left: plotOffset.left + axisMargin, width: lw };
                    plotOffset.left += lw + axisMargin;
                }
                else {
                    plotOffset.right += lw + axisMargin;
                    axis.box = { left: surface.width - plotOffset.right, width: lw };
                }
            }

             // save for future reference
            axis.position = pos;
            axis.tickLength = tickLength;
            axis.box.padding = padding;
            axis.innermost = innermost;
        }

        function allocateAxisBoxSecondPhase(axis) {
            // now that all axis boxes have been placed in one
            // dimension, we can set the remaining dimension coordinates
            if (axis.direction == "x") {
                axis.box.left = plotOffset.left - axis.labelWidth / 2;
                axis.box.width = surface.width - plotOffset.left - plotOffset.right + axis.labelWidth;
            }
            else {
                axis.box.top = plotOffset.top - axis.labelHeight / 2;
                axis.box.height = surface.height - plotOffset.bottom - plotOffset.top + axis.labelHeight;
            }
        }

        function adjustLayoutForThingsStickingOut() {
            // possibly adjust plot offset to ensure everything stays
            // inside the canvas and isn't clipped off

            var minMargin = options.grid.minBorderMargin,
                axis, i;

            // check stuff from the plot (FIXME: this should just read
            // a value from the series, otherwise it's impossible to
            // customize)
            if (minMargin == null) {
                minMargin = 0;
                for (i = 0; i < series.length; ++i)
                    minMargin = Math.max(minMargin, 2 * (series[i].points.radius + series[i].points.lineWidth/2));
            }

            var margins = {
                left: minMargin,
                right: minMargin,
                top: minMargin,
                bottom: minMargin
            };

            // check axis labels, note we don't check the actual
            // labels but instead use the overall width/height to not
            // jump as much around with replots
            $.each(allAxes(), function (_, axis) {
                if (axis.reserveSpace && axis.ticks && axis.ticks.length) {
                    if (axis.direction === "x") {
                        margins.left = Math.max(margins.left, axis.labelWidth / 2);
                        margins.right = Math.max(margins.right, axis.labelWidth / 2);
                    } else {
                        margins.bottom = Math.max(margins.bottom, axis.labelHeight / 2);
                        margins.top = Math.max(margins.top, axis.labelHeight / 2);
                    }
                }
            });

            plotOffset.left = Math.ceil(Math.max(margins.left, plotOffset.left));
            plotOffset.right = Math.ceil(Math.max(margins.right, plotOffset.right));
            plotOffset.top = Math.ceil(Math.max(margins.top, plotOffset.top));
            plotOffset.bottom = Math.ceil(Math.max(margins.bottom, plotOffset.bottom));
        }

        function setupGrid() {
            var i, axes = allAxes(), showGrid = options.grid.show;

            // Initialize the plot's offset from the edge of the canvas

            for (var a in plotOffset) {
                var margin = options.grid.margin || 0;
                plotOffset[a] = typeof margin == "number" ? margin : margin[a] || 0;
            }

            executeHooks(hooks.processOffset, [plotOffset]);

            // If the grid is visible, add its border width to the offset

            for (var a in plotOffset) {
                if(typeof(options.grid.borderWidth) == "object") {
                    plotOffset[a] += showGrid ? options.grid.borderWidth[a] : 0;
                }
                else {
                    plotOffset[a] += showGrid ? options.grid.borderWidth : 0;
                }
            }

            $.each(axes, function (_, axis) {
                var axisOpts = axis.options;
                axis.show = axisOpts.show == null ? axis.used : axisOpts.show;
                axis.reserveSpace = axisOpts.reserveSpace == null ? axis.show : axisOpts.reserveSpace;
                setRange(axis);
            });

            if (showGrid) {

                var allocatedAxes = $.grep(axes, function (axis) {
                    return axis.show || axis.reserveSpace;
                });

                $.each(allocatedAxes, function (_, axis) {
                    // make the ticks
                    setupTickGeneration(axis);
                    setTicks(axis);
                    snapRangeToTicks(axis, axis.ticks);
                    // find labelWidth/Height for axis
                    measureTickLabels(axis);
                });

                // with all dimensions calculated, we can compute the
                // axis bounding boxes, start from the outside
                // (reverse order)
                for (i = allocatedAxes.length - 1; i >= 0; --i)
                    allocateAxisBoxFirstPhase(allocatedAxes[i]);

                // make sure we've got enough space for things that
                // might stick out
                adjustLayoutForThingsStickingOut();

                $.each(allocatedAxes, function (_, axis) {
                    allocateAxisBoxSecondPhase(axis);
                });
            }

            plotWidth = surface.width - plotOffset.left - plotOffset.right;
            plotHeight = surface.height - plotOffset.bottom - plotOffset.top;

            // now we got the proper plot dimensions, we can compute the scaling
            $.each(axes, function (_, axis) {
                setTransformationHelpers(axis);
            });

            if (showGrid) {
                drawAxisLabels();
            }

            insertLegend();
        }

        function setRange(axis) {
            var opts = axis.options,
                min = +(opts.min != null ? opts.min : axis.datamin),
                max = +(opts.max != null ? opts.max : axis.datamax),
                delta = max - min;

            if (delta == 0.0) {
                // degenerate case
                var widen = max == 0 ? 1 : 0.01;

                if (opts.min == null)
                    min -= widen;
                // always widen max if we couldn't widen min to ensure we
                // don't fall into min == max which doesn't work
                if (opts.max == null || opts.min != null)
                    max += widen;
            }
            else {
                // consider autoscaling
                var margin = opts.autoscaleMargin;
                if (margin != null) {
                    if (opts.min == null) {
                        min -= delta * margin;
                        // make sure we don't go below zero if all values
                        // are positive
                        if (min < 0 && axis.datamin != null && axis.datamin >= 0)
                            min = 0;
                    }
                    if (opts.max == null) {
                        max += delta * margin;
                        if (max > 0 && axis.datamax != null && axis.datamax <= 0)
                            max = 0;
                    }
                }
            }
            axis.min = min;
            axis.max = max;
        }

        function setupTickGeneration(axis) {
            var opts = axis.options;

            // estimate number of ticks
            var noTicks;
            if (typeof opts.ticks == "number" && opts.ticks > 0)
                noTicks = opts.ticks;
            else
                // heuristic based on the model a*sqrt(x) fitted to
                // some data points that seemed reasonable
                noTicks = 0.3 * Math.sqrt(axis.direction == "x" ? surface.width : surface.height);

            var delta = (axis.max - axis.min) / noTicks,
                dec = -Math.floor(Math.log(delta) / Math.LN10),
                maxDec = opts.tickDecimals;

            if (maxDec != null && dec > maxDec) {
                dec = maxDec;
            }

            var magn = Math.pow(10, -dec),
                norm = delta / magn, // norm is between 1.0 and 10.0
                size;

            if (norm < 1.5) {
                size = 1;
            } else if (norm < 3) {
                size = 2;
                // special case for 2.5, requires an extra decimal
                if (norm > 2.25 && (maxDec == null || dec + 1 <= maxDec)) {
                    size = 2.5;
                    ++dec;
                }
            } else if (norm < 7.5) {
                size = 5;
            } else {
                size = 10;
            }

            size *= magn;

            if (opts.minTickSize != null && size < opts.minTickSize) {
                size = opts.minTickSize;
            }

            axis.delta = delta;
            axis.tickDecimals = Math.max(0, maxDec != null ? maxDec : dec);
            axis.tickSize = opts.tickSize || size;

            // Time mode was moved to a plug-in in 0.8, and since so many people use it
            // we'll add an especially friendly reminder to make sure they included it.

            if (opts.mode == "time" && !axis.tickGenerator) {
                throw new Error("Time mode requires the flot.time plugin.");
            }

            // Flot supports base-10 axes; any other mode else is handled by a plug-in,
            // like flot.time.js.

            if (!axis.tickGenerator) {

                axis.tickGenerator = function (axis) {

                    var ticks = [],
                        start = floorInBase(axis.min, axis.tickSize),
                        i = 0,
                        v = Number.NaN,
                        prev;

                    do {
                        prev = v;
                        v = start + i * axis.tickSize;
                        ticks.push(v);
                        ++i;
                    } while (v < axis.max && v != prev);
                    return ticks;
                };

				axis.tickFormatter = function (value, axis) {

					var factor = axis.tickDecimals ? Math.pow(10, axis.tickDecimals) : 1;
					var formatted = "" + Math.round(value * factor) / factor;

					// If tickDecimals was specified, ensure that we have exactly that
					// much precision; otherwise default to the value's own precision.

					if (axis.tickDecimals != null) {
						var decimal = formatted.indexOf(".");
						var precision = decimal == -1 ? 0 : formatted.length - decimal - 1;
						if (precision < axis.tickDecimals) {
							return (precision ? formatted : formatted + ".") + ("" + factor).substr(1, axis.tickDecimals - precision);
						}
					}

                    return formatted;
                };
            }

            if ($.isFunction(opts.tickFormatter))
                axis.tickFormatter = function (v, axis) { return "" + opts.tickFormatter(v, axis); };

            if (opts.alignTicksWithAxis != null) {
                var otherAxis = (axis.direction == "x" ? xaxes : yaxes)[opts.alignTicksWithAxis - 1];
                if (otherAxis && otherAxis.used && otherAxis != axis) {
                    // consider snapping min/max to outermost nice ticks
                    var niceTicks = axis.tickGenerator(axis);
                    if (niceTicks.length > 0) {
                        if (opts.min == null)
                            axis.min = Math.min(axis.min, niceTicks[0]);
                        if (opts.max == null && niceTicks.length > 1)
                            axis.max = Math.max(axis.max, niceTicks[niceTicks.length - 1]);
                    }

                    axis.tickGenerator = function (axis) {
                        // copy ticks, scaled to this axis
                        var ticks = [], v, i;
                        for (i = 0; i < otherAxis.ticks.length; ++i) {
                            v = (otherAxis.ticks[i].v - otherAxis.min) / (otherAxis.max - otherAxis.min);
                            v = axis.min + v * (axis.max - axis.min);
                            ticks.push(v);
                        }
                        return ticks;
                    };

                    // we might need an extra decimal since forced
                    // ticks don't necessarily fit naturally
                    if (!axis.mode && opts.tickDecimals == null) {
                        var extraDec = Math.max(0, -Math.floor(Math.log(axis.delta) / Math.LN10) + 1),
                            ts = axis.tickGenerator(axis);

                        // only proceed if the tick interval rounded
                        // with an extra decimal doesn't give us a
                        // zero at end
                        if (!(ts.length > 1 && /\..*0$/.test((ts[1] - ts[0]).toFixed(extraDec))))
                            axis.tickDecimals = extraDec;
                    }
                }
            }
        }

        function setTicks(axis) {
            var oticks = axis.options.ticks, ticks = [];
            if (oticks == null || (typeof oticks == "number" && oticks > 0))
                ticks = axis.tickGenerator(axis);
            else if (oticks) {
                if ($.isFunction(oticks))
                    // generate the ticks
                    ticks = oticks(axis);
                else
                    ticks = oticks;
            }

            // clean up/labelify the supplied ticks, copy them over
            var i, v;
            axis.ticks = [];
            for (i = 0; i < ticks.length; ++i) {
                var label = null;
                var t = ticks[i];
                if (typeof t == "object") {
                    v = +t[0];
                    if (t.length > 1)
                        label = t[1];
                }
                else
                    v = +t;
                if (label == null)
                    label = axis.tickFormatter(v, axis);
                if (!isNaN(v))
                    axis.ticks.push({ v: v, label: label });
            }
        }

        function snapRangeToTicks(axis, ticks) {
            if (axis.options.autoscaleMargin && ticks.length > 0) {
                // snap to ticks
                if (axis.options.min == null)
                    axis.min = Math.min(axis.min, ticks[0].v);
                if (axis.options.max == null && ticks.length > 1)
                    axis.max = Math.max(axis.max, ticks[ticks.length - 1].v);
            }
        }

        function draw() {

            surface.clear();

            executeHooks(hooks.drawBackground, [ctx]);

            var grid = options.grid;

            // draw background, if any
            if (grid.show && grid.backgroundColor)
                drawBackground();

            if (grid.show && !grid.aboveData) {
                drawGrid();
            }

            for (var i = 0; i < series.length; ++i) {
                executeHooks(hooks.drawSeries, [ctx, series[i]]);
                drawSeries(series[i]);
            }

            executeHooks(hooks.draw, [ctx]);

            if (grid.show && grid.aboveData) {
                drawGrid();
            }

            surface.render();

            // A draw implies that either the axes or data have changed, so we
            // should probably update the overlay highlights as well.

            triggerRedrawOverlay();
        }

        function extractRange(ranges, coord) {
            var axis, from, to, key, axes = allAxes();

            for (var i = 0; i < axes.length; ++i) {
                axis = axes[i];
                if (axis.direction == coord) {
                    key = coord + axis.n + "axis";
                    if (!ranges[key] && axis.n == 1)
                        key = coord + "axis"; // support x1axis as xaxis
                    if (ranges[key]) {
                        from = ranges[key].from;
                        to = ranges[key].to;
                        break;
                    }
                }
            }

            // backwards-compat stuff - to be removed in future
            if (!ranges[key]) {
                axis = coord == "x" ? xaxes[0] : yaxes[0];
                from = ranges[coord + "1"];
                to = ranges[coord + "2"];
            }

            // auto-reverse as an added bonus
            if (from != null && to != null && from > to) {
                var tmp = from;
                from = to;
                to = tmp;
            }

            return { from: from, to: to, axis: axis };
        }

        function drawBackground() {
            ctx.save();
            ctx.translate(plotOffset.left, plotOffset.top);

            ctx.fillStyle = getColorOrGradient(options.grid.backgroundColor, plotHeight, 0, "rgba(255, 255, 255, 0)");
            ctx.fillRect(0, 0, plotWidth, plotHeight);
            ctx.restore();
        }

        function drawGrid() {
            var i, axes, bw, bc;

            ctx.save();
            ctx.translate(plotOffset.left, plotOffset.top);

            // draw markings
            var markings = options.grid.markings;
            if (markings) {
                if ($.isFunction(markings)) {
                    axes = plot.getAxes();
                    // xmin etc. is backwards compatibility, to be
                    // removed in the future
                    axes.xmin = axes.xaxis.min;
                    axes.xmax = axes.xaxis.max;
                    axes.ymin = axes.yaxis.min;
                    axes.ymax = axes.yaxis.max;

                    markings = markings(axes);
                }

                for (i = 0; i < markings.length; ++i) {
                    var m = markings[i],
                        xrange = extractRange(m, "x"),
                        yrange = extractRange(m, "y");

                    // fill in missing
                    if (xrange.from == null)
                        xrange.from = xrange.axis.min;
                    if (xrange.to == null)
                        xrange.to = xrange.axis.max;
                    if (yrange.from == null)
                        yrange.from = yrange.axis.min;
                    if (yrange.to == null)
                        yrange.to = yrange.axis.max;

                    // clip
                    if (xrange.to < xrange.axis.min || xrange.from > xrange.axis.max ||
                        yrange.to < yrange.axis.min || yrange.from > yrange.axis.max)
                        continue;

                    xrange.from = Math.max(xrange.from, xrange.axis.min);
                    xrange.to = Math.min(xrange.to, xrange.axis.max);
                    yrange.from = Math.max(yrange.from, yrange.axis.min);
                    yrange.to = Math.min(yrange.to, yrange.axis.max);

                    var xequal = xrange.from === xrange.to,
                        yequal = yrange.from === yrange.to;

                    if (xequal && yequal) {
                        continue;
                    }

                    // then draw
                    xrange.from = Math.floor(xrange.axis.p2c(xrange.from));
                    xrange.to = Math.floor(xrange.axis.p2c(xrange.to));
                    yrange.from = Math.floor(yrange.axis.p2c(yrange.from));
                    yrange.to = Math.floor(yrange.axis.p2c(yrange.to));

                    if (xequal || yequal) {
                        var lineWidth = m.lineWidth || options.grid.markingsLineWidth,
                            subPixel = lineWidth % 2 ? 0.5 : 0;
                        ctx.beginPath();
                        ctx.strokeStyle = m.color || options.grid.markingsColor;
                        ctx.lineWidth = lineWidth;
                        if (xequal) {
                            ctx.moveTo(xrange.to + subPixel, yrange.from);
                            ctx.lineTo(xrange.to + subPixel, yrange.to);
                        } else {
                            ctx.moveTo(xrange.from, yrange.to + subPixel);
                            ctx.lineTo(xrange.to, yrange.to + subPixel);                            
                        }
                        ctx.stroke();
                    } else {
                        ctx.fillStyle = m.color || options.grid.markingsColor;
                        ctx.fillRect(xrange.from, yrange.to,
                                     xrange.to - xrange.from,
                                     yrange.from - yrange.to);
                    }
                }
            }

            // draw the ticks
            axes = allAxes();
            bw = options.grid.borderWidth;

            for (var j = 0; j < axes.length; ++j) {
                var axis = axes[j], box = axis.box,
                    t = axis.tickLength, x, y, xoff, yoff;
                if (!axis.show || axis.ticks.length == 0)
                    continue;

                ctx.lineWidth = 1;

                // find the edges
                if (axis.direction == "x") {
                    x = 0;
                    if (t == "full")
                        y = (axis.position == "top" ? 0 : plotHeight);
                    else
                        y = box.top - plotOffset.top + (axis.position == "top" ? box.height : 0);
                }
                else {
                    y = 0;
                    if (t == "full")
                        x = (axis.position == "left" ? 0 : plotWidth);
                    else
                        x = box.left - plotOffset.left + (axis.position == "left" ? box.width : 0);
                }

                // draw tick bar
                if (!axis.innermost) {
                    ctx.strokeStyle = axis.options.color;
                    ctx.beginPath();
                    xoff = yoff = 0;
                    if (axis.direction == "x")
                        xoff = plotWidth + 1;
                    else
                        yoff = plotHeight + 1;

                    if (ctx.lineWidth == 1) {
                        if (axis.direction == "x") {
                            y = Math.floor(y) + 0.5;
                        } else {
                            x = Math.floor(x) + 0.5;
                        }
                    }

                    ctx.moveTo(x, y);
                    ctx.lineTo(x + xoff, y + yoff);
                    ctx.stroke();
                }

                // draw ticks

                ctx.strokeStyle = axis.options.tickColor;

                ctx.beginPath();
                for (i = 0; i < axis.ticks.length; ++i) {
                    var v = axis.ticks[i].v;

                    xoff = yoff = 0;

                    if (isNaN(v) || v < axis.min || v > axis.max
                        // skip those lying on the axes if we got a border
                        || (t == "full"
                            && ((typeof bw == "object" && bw[axis.position] > 0) || bw > 0)
                            && (v == axis.min || v == axis.max)))
                        continue;

                    if (axis.direction == "x") {
                        x = axis.p2c(v);
                        yoff = t == "full" ? -plotHeight : t;

                        if (axis.position == "top")
                            yoff = -yoff;
                    }
                    else {
                        y = axis.p2c(v);
                        xoff = t == "full" ? -plotWidth : t;

                        if (axis.position == "left")
                            xoff = -xoff;
                    }

                    if (ctx.lineWidth == 1) {
                        if (axis.direction == "x")
                            x = Math.floor(x) + 0.5;
                        else
                            y = Math.floor(y) + 0.5;
                    }

                    ctx.moveTo(x, y);
                    ctx.lineTo(x + xoff, y + yoff);
                }

                ctx.stroke();
            }


            // draw border
            if (bw) {
                // If either borderWidth or borderColor is an object, then draw the border
                // line by line instead of as one rectangle
                bc = options.grid.borderColor;
                if(typeof bw == "object" || typeof bc == "object") {
                    if (typeof bw !== "object") {
                        bw = {top: bw, right: bw, bottom: bw, left: bw};
                    }
                    if (typeof bc !== "object") {
                        bc = {top: bc, right: bc, bottom: bc, left: bc};
                    }

                    if (bw.top > 0) {
                        ctx.strokeStyle = bc.top;
                        ctx.lineWidth = bw.top;
                        ctx.beginPath();
                        ctx.moveTo(0 - bw.left, 0 - bw.top/2);
                        ctx.lineTo(plotWidth, 0 - bw.top/2);
                        ctx.stroke();
                    }

                    if (bw.right > 0) {
                        ctx.strokeStyle = bc.right;
                        ctx.lineWidth = bw.right;
                        ctx.beginPath();
                        ctx.moveTo(plotWidth + bw.right / 2, 0 - bw.top);
                        ctx.lineTo(plotWidth + bw.right / 2, plotHeight);
                        ctx.stroke();
                    }

                    if (bw.bottom > 0) {
                        ctx.strokeStyle = bc.bottom;
                        ctx.lineWidth = bw.bottom;
                        ctx.beginPath();
                        ctx.moveTo(plotWidth + bw.right, plotHeight + bw.bottom / 2);
                        ctx.lineTo(0, plotHeight + bw.bottom / 2);
                        ctx.stroke();
                    }

                    if (bw.left > 0) {
                        ctx.strokeStyle = bc.left;
                        ctx.lineWidth = bw.left;
                        ctx.beginPath();
                        ctx.moveTo(0 - bw.left/2, plotHeight + bw.bottom);
                        ctx.lineTo(0- bw.left/2, 0);
                        ctx.stroke();
                    }
                }
                else {
                    ctx.lineWidth = bw;
                    ctx.strokeStyle = options.grid.borderColor;
                    ctx.strokeRect(-bw/2, -bw/2, plotWidth + bw, plotHeight + bw);
                }
            }

            ctx.restore();
        }

        function drawAxisLabels() {

            $.each(allAxes(), function (_, axis) {
                var box = axis.box,
                    legacyStyles = axis.direction + "Axis " + axis.direction + axis.n + "Axis",
                    layer = "flot-" + axis.direction + "-axis flot-" + axis.direction + axis.n + "-axis " + legacyStyles,
                    font = axis.options.font || "flot-tick-label tickLabel",
                    tick, x, y, halign, valign;

                // Remove text before checking for axis.show and ticks.length;
                // otherwise plugins, like flot-tickrotor, that draw their own
                // tick labels will end up with both theirs and the defaults.

                surface.removeText(layer);

                if (!axis.show || axis.ticks.length == 0)
                    return;

                for (var i = 0; i < axis.ticks.length; ++i) {

                    tick = axis.ticks[i];
                    if (!tick.label || tick.v < axis.min || tick.v > axis.max)
                        continue;

                    if (axis.direction == "x") {
                        halign = "center";
                        x = plotOffset.left + axis.p2c(tick.v);
                        if (axis.position == "bottom") {
                            y = box.top + box.padding;
                        } else {
                            y = box.top + box.height - box.padding;
                            valign = "bottom";
                        }
                    } else {
                        valign = "middle";
                        y = plotOffset.top + axis.p2c(tick.v);
                        if (axis.position == "left") {
                            x = box.left + box.width - box.padding;
                            halign = "right";
                        } else {
                            x = box.left + box.padding;
                        }
                    }

                    surface.addText(layer, x, y, tick.label, font, null, null, halign, valign);
                }
            });
        }

        function drawSeries(series) {
            if (series.lines.show)
                drawSeriesLines(series);
            if (series.bars.show)
                drawSeriesBars(series);
            if (series.points.show)
                drawSeriesPoints(series);
        }

        function drawSeriesLines(series) {
            function plotLine(datapoints, xoffset, yoffset, axisx, axisy) {
                var points = datapoints.points,
                    ps = datapoints.pointsize,
                    prevx = null, prevy = null;

                ctx.beginPath();
                for (var i = ps; i < points.length; i += ps) {
                    var x1 = points[i - ps], y1 = points[i - ps + 1],
                        x2 = points[i], y2 = points[i + 1];

                    if (x1 == null || x2 == null)
                        continue;

                    // clip with ymin
                    if (y1 <= y2 && y1 < axisy.min) {
                        if (y2 < axisy.min)
                            continue;   // line segment is outside
                        // compute new intersection point
                        x1 = (axisy.min - y1) / (y2 - y1) * (x2 - x1) + x1;
                        y1 = axisy.min;
                    }
                    else if (y2 <= y1 && y2 < axisy.min) {
                        if (y1 < axisy.min)
                            continue;
                        x2 = (axisy.min - y1) / (y2 - y1) * (x2 - x1) + x1;
                        y2 = axisy.min;
                    }

                    // clip with ymax
                    if (y1 >= y2 && y1 > axisy.max) {
                        if (y2 > axisy.max)
                            continue;
                        x1 = (axisy.max - y1) / (y2 - y1) * (x2 - x1) + x1;
                        y1 = axisy.max;
                    }
                    else if (y2 >= y1 && y2 > axisy.max) {
                        if (y1 > axisy.max)
                            continue;
                        x2 = (axisy.max - y1) / (y2 - y1) * (x2 - x1) + x1;
                        y2 = axisy.max;
                    }

                    // clip with xmin
                    if (x1 <= x2 && x1 < axisx.min) {
                        if (x2 < axisx.min)
                            continue;
                        y1 = (axisx.min - x1) / (x2 - x1) * (y2 - y1) + y1;
                        x1 = axisx.min;
                    }
                    else if (x2 <= x1 && x2 < axisx.min) {
                        if (x1 < axisx.min)
                            continue;
                        y2 = (axisx.min - x1) / (x2 - x1) * (y2 - y1) + y1;
                        x2 = axisx.min;
                    }

                    // clip with xmax
                    if (x1 >= x2 && x1 > axisx.max) {
                        if (x2 > axisx.max)
                            continue;
                        y1 = (axisx.max - x1) / (x2 - x1) * (y2 - y1) + y1;
                        x1 = axisx.max;
                    }
                    else if (x2 >= x1 && x2 > axisx.max) {
                        if (x1 > axisx.max)
                            continue;
                        y2 = (axisx.max - x1) / (x2 - x1) * (y2 - y1) + y1;
                        x2 = axisx.max;
                    }

                    if (x1 != prevx || y1 != prevy)
                        ctx.moveTo(axisx.p2c(x1) + xoffset, axisy.p2c(y1) + yoffset);

                    prevx = x2;
                    prevy = y2;
                    ctx.lineTo(axisx.p2c(x2) + xoffset, axisy.p2c(y2) + yoffset);
                }
                ctx.stroke();
            }

            function plotLineArea(datapoints, axisx, axisy) {
                var points = datapoints.points,
                    ps = datapoints.pointsize,
                    bottom = Math.min(Math.max(0, axisy.min), axisy.max),
                    i = 0, top, areaOpen = false,
                    ypos = 1, segmentStart = 0, segmentEnd = 0;

                // we process each segment in two turns, first forward
                // direction to sketch out top, then once we hit the
                // end we go backwards to sketch the bottom
                while (true) {
                    if (ps > 0 && i > points.length + ps)
                        break;

                    i += ps; // ps is negative if going backwards

                    var x1 = points[i - ps],
                        y1 = points[i - ps + ypos],
                        x2 = points[i], y2 = points[i + ypos];

                    if (areaOpen) {
                        if (ps > 0 && x1 != null && x2 == null) {
                            // at turning point
                            segmentEnd = i;
                            ps = -ps;
                            ypos = 2;
                            continue;
                        }

                        if (ps < 0 && i == segmentStart + ps) {
                            // done with the reverse sweep
                            ctx.fill();
                            areaOpen = false;
                            ps = -ps;
                            ypos = 1;
                            i = segmentStart = segmentEnd + ps;
                            continue;
                        }
                    }

                    if (x1 == null || x2 == null)
                        continue;

                    // clip x values

                    // clip with xmin
                    if (x1 <= x2 && x1 < axisx.min) {
                        if (x2 < axisx.min)
                            continue;
                        y1 = (axisx.min - x1) / (x2 - x1) * (y2 - y1) + y1;
                        x1 = axisx.min;
                    }
                    else if (x2 <= x1 && x2 < axisx.min) {
                        if (x1 < axisx.min)
                            continue;
                        y2 = (axisx.min - x1) / (x2 - x1) * (y2 - y1) + y1;
                        x2 = axisx.min;
                    }

                    // clip with xmax
                    if (x1 >= x2 && x1 > axisx.max) {
                        if (x2 > axisx.max)
                            continue;
                        y1 = (axisx.max - x1) / (x2 - x1) * (y2 - y1) + y1;
                        x1 = axisx.max;
                    }
                    else if (x2 >= x1 && x2 > axisx.max) {
                        if (x1 > axisx.max)
                            continue;
                        y2 = (axisx.max - x1) / (x2 - x1) * (y2 - y1) + y1;
                        x2 = axisx.max;
                    }

                    if (!areaOpen) {
                        // open area
                        ctx.beginPath();
                        ctx.moveTo(axisx.p2c(x1), axisy.p2c(bottom));
                        areaOpen = true;
                    }

                    // now first check the case where both is outside
                    if (y1 >= axisy.max && y2 >= axisy.max) {
                        ctx.lineTo(axisx.p2c(x1), axisy.p2c(axisy.max));
                        ctx.lineTo(axisx.p2c(x2), axisy.p2c(axisy.max));
                        continue;
                    }
                    else if (y1 <= axisy.min && y2 <= axisy.min) {
                        ctx.lineTo(axisx.p2c(x1), axisy.p2c(axisy.min));
                        ctx.lineTo(axisx.p2c(x2), axisy.p2c(axisy.min));
                        continue;
                    }

                    // else it's a bit more complicated, there might
                    // be a flat maxed out rectangle first, then a
                    // triangular cutout or reverse; to find these
                    // keep track of the current x values
                    var x1old = x1, x2old = x2;

                    // clip the y values, without shortcutting, we
                    // go through all cases in turn

                    // clip with ymin
                    if (y1 <= y2 && y1 < axisy.min && y2 >= axisy.min) {
                        x1 = (axisy.min - y1) / (y2 - y1) * (x2 - x1) + x1;
                        y1 = axisy.min;
                    }
                    else if (y2 <= y1 && y2 < axisy.min && y1 >= axisy.min) {
                        x2 = (axisy.min - y1) / (y2 - y1) * (x2 - x1) + x1;
                        y2 = axisy.min;
                    }

                    // clip with ymax
                    if (y1 >= y2 && y1 > axisy.max && y2 <= axisy.max) {
                        x1 = (axisy.max - y1) / (y2 - y1) * (x2 - x1) + x1;
                        y1 = axisy.max;
                    }
                    else if (y2 >= y1 && y2 > axisy.max && y1 <= axisy.max) {
                        x2 = (axisy.max - y1) / (y2 - y1) * (x2 - x1) + x1;
                        y2 = axisy.max;
                    }

                    // if the x value was changed we got a rectangle
                    // to fill
                    if (x1 != x1old) {
                        ctx.lineTo(axisx.p2c(x1old), axisy.p2c(y1));
                        // it goes to (x1, y1), but we fill that below
                    }

                    // fill triangular section, this sometimes result
                    // in redundant points if (x1, y1) hasn't changed
                    // from previous line to, but we just ignore that
                    ctx.lineTo(axisx.p2c(x1), axisy.p2c(y1));
                    ctx.lineTo(axisx.p2c(x2), axisy.p2c(y2));

                    // fill the other rectangle if it's there
                    if (x2 != x2old) {
                        ctx.lineTo(axisx.p2c(x2), axisy.p2c(y2));
                        ctx.lineTo(axisx.p2c(x2old), axisy.p2c(y2));
                    }
                }
            }

            ctx.save();
            ctx.translate(plotOffset.left, plotOffset.top);
            ctx.lineJoin = "round";

            var lw = series.lines.lineWidth,
                sw = series.shadowSize;
            // FIXME: consider another form of shadow when filling is turned on
            if (lw > 0 && sw > 0) {
                // draw shadow as a thick and thin line with transparency
                ctx.lineWidth = sw;
                ctx.strokeStyle = "rgba(0,0,0,0.1)";
                // position shadow at angle from the mid of line
                var angle = Math.PI/18;
                plotLine(series.datapoints, Math.sin(angle) * (lw/2 + sw/2), Math.cos(angle) * (lw/2 + sw/2), series.xaxis, series.yaxis);
                ctx.lineWidth = sw/2;
                plotLine(series.datapoints, Math.sin(angle) * (lw/2 + sw/4), Math.cos(angle) * (lw/2 + sw/4), series.xaxis, series.yaxis);
            }

            ctx.lineWidth = lw;
            ctx.strokeStyle = series.color;
            var fillStyle = getFillStyle(series.lines, series.color, 0, plotHeight);
            if (fillStyle) {
                ctx.fillStyle = fillStyle;
                plotLineArea(series.datapoints, series.xaxis, series.yaxis);
            }

            if (lw > 0)
                plotLine(series.datapoints, 0, 0, series.xaxis, series.yaxis);
            ctx.restore();
        }

        function drawSeriesPoints(series) {
            function plotPoints(datapoints, radius, fillStyle, offset, shadow, axisx, axisy, symbol) {
                var points = datapoints.points, ps = datapoints.pointsize;

                for (var i = 0; i < points.length; i += ps) {
                    var x = points[i], y = points[i + 1];
                    if (x == null || x < axisx.min || x > axisx.max || y < axisy.min || y > axisy.max)
                        continue;

                    ctx.beginPath();
                    x = axisx.p2c(x);
                    y = axisy.p2c(y) + offset;
                    if (symbol == "circle")
                        ctx.arc(x, y, radius, 0, shadow ? Math.PI : Math.PI * 2, false);
                    else
                        symbol(ctx, x, y, radius, shadow);
                    ctx.closePath();

                    if (fillStyle) {
                        ctx.fillStyle = fillStyle;
                        ctx.fill();
                    }
                    ctx.stroke();
                }
            }

            ctx.save();
            ctx.translate(plotOffset.left, plotOffset.top);

            var lw = series.points.lineWidth,
                sw = series.shadowSize,
                radius = series.points.radius,
                symbol = series.points.symbol;

            // If the user sets the line width to 0, we change it to a very 
            // small value. A line width of 0 seems to force the default of 1.
            // Doing the conditional here allows the shadow setting to still be 
            // optional even with a lineWidth of 0.

            if( lw == 0 )
                lw = 0.0001;

            if (lw > 0 && sw > 0) {
                // draw shadow in two steps
                var w = sw / 2;
                ctx.lineWidth = w;
                ctx.strokeStyle = "rgba(0,0,0,0.1)";
                plotPoints(series.datapoints, radius, null, w + w/2, true,
                           series.xaxis, series.yaxis, symbol);

                ctx.strokeStyle = "rgba(0,0,0,0.2)";
                plotPoints(series.datapoints, radius, null, w/2, true,
                           series.xaxis, series.yaxis, symbol);
            }

            ctx.lineWidth = lw;
            ctx.strokeStyle = series.color;
            plotPoints(series.datapoints, radius,
                       getFillStyle(series.points, series.color), 0, false,
                       series.xaxis, series.yaxis, symbol);
            ctx.restore();
        }

        function drawBar(x, y, b, barLeft, barRight, fillStyleCallback, axisx, axisy, c, horizontal, lineWidth) {
            var left, right, bottom, top,
                drawLeft, drawRight, drawTop, drawBottom,
                tmp;

            // in horizontal mode, we start the bar from the left
            // instead of from the bottom so it appears to be
            // horizontal rather than vertical
            if (horizontal) {
                drawBottom = drawRight = drawTop = true;
                drawLeft = false;
                left = b;
                right = x;
                top = y + barLeft;
                bottom = y + barRight;

                // account for negative bars
                if (right < left) {
                    tmp = right;
                    right = left;
                    left = tmp;
                    drawLeft = true;
                    drawRight = false;
                }
            }
            else {
                drawLeft = drawRight = drawTop = true;
                drawBottom = false;
                left = x + barLeft;
                right = x + barRight;
                bottom = b;
                top = y;

                // account for negative bars
                if (top < bottom) {
                    tmp = top;
                    top = bottom;
                    bottom = tmp;
                    drawBottom = true;
                    drawTop = false;
                }
            }

            // clip
            if (right < axisx.min || left > axisx.max ||
                top < axisy.min || bottom > axisy.max)
                return;

            if (left < axisx.min) {
                left = axisx.min;
                drawLeft = false;
            }

            if (right > axisx.max) {
                right = axisx.max;
                drawRight = false;
            }

            if (bottom < axisy.min) {
                bottom = axisy.min;
                drawBottom = false;
            }

            if (top > axisy.max) {
                top = axisy.max;
                drawTop = false;
            }

            left = axisx.p2c(left);
            bottom = axisy.p2c(bottom);
            right = axisx.p2c(right);
            top = axisy.p2c(top);

            // fill the bar
            if (fillStyleCallback) {
                c.fillStyle = fillStyleCallback(bottom, top);
                c.fillRect(left, top, right - left, bottom - top)
            }

            // draw outline
            if (lineWidth > 0 && (drawLeft || drawRight || drawTop || drawBottom)) {
                c.beginPath();

                // FIXME: inline moveTo is buggy with excanvas
                c.moveTo(left, bottom);
                if (drawLeft)
                    c.lineTo(left, top);
                else
                    c.moveTo(left, top);
                if (drawTop)
                    c.lineTo(right, top);
                else
                    c.moveTo(right, top);
                if (drawRight)
                    c.lineTo(right, bottom);
                else
                    c.moveTo(right, bottom);
                if (drawBottom)
                    c.lineTo(left, bottom);
                else
                    c.moveTo(left, bottom);
                c.stroke();
            }
        }

        function drawSeriesBars(series) {
            function plotBars(datapoints, barLeft, barRight, fillStyleCallback, axisx, axisy) {
                var points = datapoints.points, ps = datapoints.pointsize;

                for (var i = 0; i < points.length; i += ps) {
                    if (points[i] == null)
                        continue;
                    drawBar(points[i], points[i + 1], points[i + 2], barLeft, barRight, fillStyleCallback, axisx, axisy, ctx, series.bars.horizontal, series.bars.lineWidth);
                }
            }

            ctx.save();
            ctx.translate(plotOffset.left, plotOffset.top);

            // FIXME: figure out a way to add shadows (for instance along the right edge)
            ctx.lineWidth = series.bars.lineWidth;
            ctx.strokeStyle = series.color;

            var barLeft;

            switch (series.bars.align) {
                case "left":
                    barLeft = 0;
                    break;
                case "right":
                    barLeft = -series.bars.barWidth;
                    break;
                default:
                    barLeft = -series.bars.barWidth / 2;
            }

            var fillStyleCallback = series.bars.fill ? function (bottom, top) { return getFillStyle(series.bars, series.color, bottom, top); } : null;
            plotBars(series.datapoints, barLeft, barLeft + series.bars.barWidth, fillStyleCallback, series.xaxis, series.yaxis);
            ctx.restore();
        }

        function getFillStyle(filloptions, seriesColor, bottom, top) {
            var fill = filloptions.fill;
            if (!fill)
                return null;

            if (filloptions.fillColor)
                return getColorOrGradient(filloptions.fillColor, bottom, top, seriesColor);

            var c = $.color.parse(seriesColor);
            c.a = typeof fill == "number" ? fill : 0.4;
            c.normalize();
            return c.toString();
        }

        function insertLegend() {

            if (options.legend.container != null) {
                $(options.legend.container).html("");
            } else {
                placeholder.find(".legend").remove();
            }

            if (!options.legend.show) {
                return;
            }

            var fragments = [], entries = [], rowStarted = false,
                lf = options.legend.labelFormatter, s, label;

            // Build a list of legend entries, with each having a label and a color

            for (var i = 0; i < series.length; ++i) {
                s = series[i];
                if (s.label) {
                    label = lf ? lf(s.label, s) : s.label;
                    if (label) {
                        entries.push({
                            label: label,
                            color: s.color
                        });
                    }
                }
            }

            // Sort the legend using either the default or a custom comparator

            if (options.legend.sorted) {
                if ($.isFunction(options.legend.sorted)) {
                    entries.sort(options.legend.sorted);
                } else if (options.legend.sorted == "reverse") {
                	entries.reverse();
                } else {
                    var ascending = options.legend.sorted != "descending";
                    entries.sort(function(a, b) {
                        return a.label == b.label ? 0 : (
                            (a.label < b.label) != ascending ? 1 : -1   // Logical XOR
                        );
                    });
                }
            }

            // Generate markup for the list of entries, in their final order

            for (var i = 0; i < entries.length; ++i) {

                var entry = entries[i];

                if (i % options.legend.noColumns == 0) {
                    if (rowStarted)
                        fragments.push('</tr>');
                    fragments.push('<tr>');
                    rowStarted = true;
                }

                fragments.push(
                    '<td class="legendColorBox"><div style="border:1px solid ' + options.legend.labelBoxBorderColor + ';padding:1px"><div style="width:4px;height:0;border:5px solid ' + entry.color + ';overflow:hidden"></div></div></td>' +
                    '<td class="legendLabel">' + entry.label + '</td>'
                );
            }

            if (rowStarted)
                fragments.push('</tr>');

            if (fragments.length == 0)
                return;

            var table = '<table style="font-size:smaller;color:' + options.grid.color + '">' + fragments.join("") + '</table>';
            if (options.legend.container != null)
                $(options.legend.container).html(table);
            else {
                var pos = "",
                    p = options.legend.position,
                    m = options.legend.margin;
                if (m[0] == null)
                    m = [m, m];
                if (p.charAt(0) == "n")
                    pos += 'top:' + (m[1] + plotOffset.top) + 'px;';
                else if (p.charAt(0) == "s")
                    pos += 'bottom:' + (m[1] + plotOffset.bottom) + 'px;';
                if (p.charAt(1) == "e")
                    pos += 'right:' + (m[0] + plotOffset.right) + 'px;';
                else if (p.charAt(1) == "w")
                    pos += 'left:' + (m[0] + plotOffset.left) + 'px;';
                var legend = $('<div class="legend">' + table.replace('style="', 'style="position:absolute;' + pos +';') + '</div>').appendTo(placeholder);
                if (options.legend.backgroundOpacity != 0.0) {
                    // put in the transparent background
                    // separately to avoid blended labels and
                    // label boxes
                    var c = options.legend.backgroundColor;
                    if (c == null) {
                        c = options.grid.backgroundColor;
                        if (c && typeof c == "string")
                            c = $.color.parse(c);
                        else
                            c = $.color.extract(legend, 'background-color');
                        c.a = 1;
                        c = c.toString();
                    }
                    var div = legend.children();
                    $('<div style="position:absolute;width:' + div.width() + 'px;height:' + div.height() + 'px;' + pos +'background-color:' + c + ';"> </div>').prependTo(legend).css('opacity', options.legend.backgroundOpacity);
                }
            }
        }


        // interactive features

        var highlights = [],
            redrawTimeout = null;

        // returns the data item the mouse is over, or null if none is found
        function findNearbyItem(mouseX, mouseY, seriesFilter) {
            var maxDistance = options.grid.mouseActiveRadius,
                smallestDistance = maxDistance * maxDistance + 1,
                item = null, foundPoint = false, i, j, ps;

            for (i = series.length - 1; i >= 0; --i) {
                if (!seriesFilter(series[i]))
                    continue;

                var s = series[i],
                    axisx = s.xaxis,
                    axisy = s.yaxis,
                    points = s.datapoints.points,
                    mx = axisx.c2p(mouseX), // precompute some stuff to make the loop faster
                    my = axisy.c2p(mouseY),
                    maxx = maxDistance / axisx.scale,
                    maxy = maxDistance / axisy.scale;

                ps = s.datapoints.pointsize;
                // with inverse transforms, we can't use the maxx/maxy
                // optimization, sadly
                if (axisx.options.inverseTransform)
                    maxx = Number.MAX_VALUE;
                if (axisy.options.inverseTransform)
                    maxy = Number.MAX_VALUE;

                if (s.lines.show || s.points.show) {
                    for (j = 0; j < points.length; j += ps) {
                        var x = points[j], y = points[j + 1];
                        if (x == null)
                            continue;

                        // For points and lines, the cursor must be within a
                        // certain distance to the data point
                        if (x - mx > maxx || x - mx < -maxx ||
                            y - my > maxy || y - my < -maxy)
                            continue;

                        // We have to calculate distances in pixels, not in
                        // data units, because the scales of the axes may be different
                        var dx = Math.abs(axisx.p2c(x) - mouseX),
                            dy = Math.abs(axisy.p2c(y) - mouseY),
                            dist = dx * dx + dy * dy; // we save the sqrt

                        // use <= to ensure last point takes precedence
                        // (last generally means on top of)
                        if (dist < smallestDistance) {
                            smallestDistance = dist;
                            item = [i, j / ps];
                        }
                    }
                }

                if (s.bars.show && !item) { // no other point can be nearby

                    var barLeft, barRight;

                    switch (s.bars.align) {
                        case "left":
                            barLeft = 0;
                            break;
                        case "right":
                            barLeft = -s.bars.barWidth;
                            break;
                        default:
                            barLeft = -s.bars.barWidth / 2;
                    }

                    barRight = barLeft + s.bars.barWidth;

                    for (j = 0; j < points.length; j += ps) {
                        var x = points[j], y = points[j + 1], b = points[j + 2];
                        if (x == null)
                            continue;

                        // for a bar graph, the cursor must be inside the bar
                        if (series[i].bars.horizontal ?
                            (mx <= Math.max(b, x) && mx >= Math.min(b, x) &&
                             my >= y + barLeft && my <= y + barRight) :
                            (mx >= x + barLeft && mx <= x + barRight &&
                             my >= Math.min(b, y) && my <= Math.max(b, y)))
                                item = [i, j / ps];
                    }
                }
            }

            if (item) {
                i = item[0];
                j = item[1];
                ps = series[i].datapoints.pointsize;

                return { datapoint: series[i].datapoints.points.slice(j * ps, (j + 1) * ps),
                         dataIndex: j,
                         series: series[i],
                         seriesIndex: i };
            }

            return null;
        }

        function onMouseMove(e) {
            if (options.grid.hoverable)
                triggerClickHoverEvent("plothover", e,
                                       function (s) { return s["hoverable"] != false; });
        }

        function onMouseLeave(e) {
            if (options.grid.hoverable)
                triggerClickHoverEvent("plothover", e,
                                       function (s) { return false; });
        }

        function onClick(e) {
            triggerClickHoverEvent("plotclick", e,
                                   function (s) { return s["clickable"] != false; });
        }

        // trigger click or hover event (they send the same parameters
        // so we share their code)
        function triggerClickHoverEvent(eventname, event, seriesFilter) {
            var offset = eventHolder.offset(),
                canvasX = event.pageX - offset.left - plotOffset.left,
                canvasY = event.pageY - offset.top - plotOffset.top,
            pos = canvasToAxisCoords({ left: canvasX, top: canvasY });

            pos.pageX = event.pageX;
            pos.pageY = event.pageY;

            var item = findNearbyItem(canvasX, canvasY, seriesFilter);

            if (item) {
                // fill in mouse pos for any listeners out there
                item.pageX = parseInt(item.series.xaxis.p2c(item.datapoint[0]) + offset.left + plotOffset.left, 10);
                item.pageY = parseInt(item.series.yaxis.p2c(item.datapoint[1]) + offset.top + plotOffset.top, 10);
            }

            if (options.grid.autoHighlight) {
                // clear auto-highlights
                for (var i = 0; i < highlights.length; ++i) {
                    var h = highlights[i];
                    if (h.auto == eventname &&
                        !(item && h.series == item.series &&
                          h.point[0] == item.datapoint[0] &&
                          h.point[1] == item.datapoint[1]))
                        unhighlight(h.series, h.point);
                }

                if (item)
                    highlight(item.series, item.datapoint, eventname);
            }

            placeholder.trigger(eventname, [ pos, item ]);
        }

        function triggerRedrawOverlay() {
            var t = options.interaction.redrawOverlayInterval;
            if (t == -1) {      // skip event queue
                drawOverlay();
                return;
            }

            if (!redrawTimeout)
                redrawTimeout = setTimeout(drawOverlay, t);
        }

        function drawOverlay() {
            redrawTimeout = null;

            // draw highlights
            octx.save();
            overlay.clear();
            octx.translate(plotOffset.left, plotOffset.top);

            var i, hi;
            for (i = 0; i < highlights.length; ++i) {
                hi = highlights[i];

                if (hi.series.bars.show)
                    drawBarHighlight(hi.series, hi.point);
                else
                    drawPointHighlight(hi.series, hi.point);
            }
            octx.restore();

            executeHooks(hooks.drawOverlay, [octx]);
        }

        function highlight(s, point, auto) {
            if (typeof s == "number")
                s = series[s];

            if (typeof point == "number") {
                var ps = s.datapoints.pointsize;
                point = s.datapoints.points.slice(ps * point, ps * (point + 1));
            }

            var i = indexOfHighlight(s, point);
            if (i == -1) {
                highlights.push({ series: s, point: point, auto: auto });

                triggerRedrawOverlay();
            }
            else if (!auto)
                highlights[i].auto = false;
        }

        function unhighlight(s, point) {
            if (s == null && point == null) {
                highlights = [];
                triggerRedrawOverlay();
                return;
            }

            if (typeof s == "number")
                s = series[s];

            if (typeof point == "number") {
                var ps = s.datapoints.pointsize;
                point = s.datapoints.points.slice(ps * point, ps * (point + 1));
            }

            var i = indexOfHighlight(s, point);
            if (i != -1) {
                highlights.splice(i, 1);

                triggerRedrawOverlay();
            }
        }

        function indexOfHighlight(s, p) {
            for (var i = 0; i < highlights.length; ++i) {
                var h = highlights[i];
                if (h.series == s && h.point[0] == p[0]
                    && h.point[1] == p[1])
                    return i;
            }
            return -1;
        }

        function drawPointHighlight(series, point) {
            var x = point[0], y = point[1],
                axisx = series.xaxis, axisy = series.yaxis,
                highlightColor = (typeof series.highlightColor === "string") ? series.highlightColor : $.color.parse(series.color).scale('a', 0.5).toString();

            if (x < axisx.min || x > axisx.max || y < axisy.min || y > axisy.max)
                return;

            var pointRadius = series.points.radius + series.points.lineWidth / 2;
            octx.lineWidth = pointRadius;
            octx.strokeStyle = highlightColor;
            var radius = 1.5 * pointRadius;
            x = axisx.p2c(x);
            y = axisy.p2c(y);

            octx.beginPath();
            if (series.points.symbol == "circle")
                octx.arc(x, y, radius, 0, 2 * Math.PI, false);
            else
                series.points.symbol(octx, x, y, radius, false);
            octx.closePath();
            octx.stroke();
        }

        function drawBarHighlight(series, point) {
            var highlightColor = (typeof series.highlightColor === "string") ? series.highlightColor : $.color.parse(series.color).scale('a', 0.5).toString(),
                fillStyle = highlightColor,
                barLeft;

            switch (series.bars.align) {
                case "left":
                    barLeft = 0;
                    break;
                case "right":
                    barLeft = -series.bars.barWidth;
                    break;
                default:
                    barLeft = -series.bars.barWidth / 2;
            }

            octx.lineWidth = series.bars.lineWidth;
            octx.strokeStyle = highlightColor;

            drawBar(point[0], point[1], point[2] || 0, barLeft, barLeft + series.bars.barWidth,
                    function () { return fillStyle; }, series.xaxis, series.yaxis, octx, series.bars.horizontal, series.bars.lineWidth);
        }

        function getColorOrGradient(spec, bottom, top, defaultColor) {
            if (typeof spec == "string")
                return spec;
            else {
                // assume this is a gradient spec; IE currently only
                // supports a simple vertical gradient properly, so that's
                // what we support too
                var gradient = ctx.createLinearGradient(0, top, 0, bottom);

                for (var i = 0, l = spec.colors.length; i < l; ++i) {
                    var c = spec.colors[i];
                    if (typeof c != "string") {
                        var co = $.color.parse(defaultColor);
                        if (c.brightness != null)
                            co = co.scale('rgb', c.brightness);
                        if (c.opacity != null)
                            co.a *= c.opacity;
                        c = co.toString();
                    }
                    gradient.addColorStop(i / (l - 1), c);
                }

                return gradient;
            }
        }
    }

    // Add the plot function to the top level of the jQuery object

    $.plot = function(placeholder, data, options) {
        //var t0 = new Date();
        var plot = new Plot($(placeholder), data, options, $.plot.plugins);
        //(window.console ? console.log : alert)("time used (msecs): " + ((new Date()).getTime() - t0.getTime()));
        return plot;
    };

    $.plot.version = "0.8.3";

    $.plot.plugins = [];

    // Also add the plot function as a chainable property

    $.fn.plot = function(data, options) {
        return this.each(function() {
            $.plot(this, data, options);
        });
    };

    // round to nearby lower multiple of base
    function floorInBase(n, base) {
        return base * Math.floor(n / base);
    }

})(jQuery);

define("flot", ["jquery"], function(){});

/* Flot plugin for stacking data sets rather than overlyaing them.

Copyright (c) 2007-2014 IOLA and Ole Laursen.
Licensed under the MIT license.

The plugin assumes the data is sorted on x (or y if stacking horizontally).
For line charts, it is assumed that if a line has an undefined gap (from a
null point), then the line above it should have the same gap - insert zeros
instead of "null" if you want another behaviour. This also holds for the start
and end of the chart. Note that stacking a mix of positive and negative values
in most instances doesn't make sense (so it looks weird).

Two or more series are stacked when their "stack" attribute is set to the same
key (which can be any number or string or just "true"). To specify the default
stack, you can set the stack option like this:

	series: {
		stack: null/false, true, or a key (number/string)
	}

You can also specify it for a single series, like this:

	$.plot( $("#placeholder"), [{
		data: [ ... ],
		stack: true
	}])

The stacking order is determined by the order of the data series in the array
(later series end up on top of the previous).

Internally, the plugin modifies the datapoints in each series, adding an
offset to the y value. For line series, extra data points are inserted through
interpolation. If there's a second y value, it's also adjusted (e.g for bar
charts or filled areas).

*/

(function ($) {
    var options = {
        series: { stack: null } // or number/string
    };
    
    function init(plot) {
        function findMatchingSeries(s, allseries) {
            var res = null;
            for (var i = 0; i < allseries.length; ++i) {
                if (s == allseries[i])
                    break;
                
                if (allseries[i].stack == s.stack)
                    res = allseries[i];
            }
            
            return res;
        }
        
        function stackData(plot, s, datapoints) {
            if (s.stack == null || s.stack === false)
                return;

            var other = findMatchingSeries(s, plot.getData());
            if (!other)
                return;

            var ps = datapoints.pointsize,
                points = datapoints.points,
                otherps = other.datapoints.pointsize,
                otherpoints = other.datapoints.points,
                newpoints = [],
                px, py, intery, qx, qy, bottom,
                withlines = s.lines.show,
                horizontal = s.bars.horizontal,
                withbottom = ps > 2 && (horizontal ? datapoints.format[2].x : datapoints.format[2].y),
                withsteps = withlines && s.lines.steps,
                fromgap = true,
                keyOffset = horizontal ? 1 : 0,
                accumulateOffset = horizontal ? 0 : 1,
                i = 0, j = 0, l, m;

            while (true) {
                if (i >= points.length)
                    break;

                l = newpoints.length;

                if (points[i] == null) {
                    // copy gaps
                    for (m = 0; m < ps; ++m)
                        newpoints.push(points[i + m]);
                    i += ps;
                }
                else if (j >= otherpoints.length) {
                    // for lines, we can't use the rest of the points
                    if (!withlines) {
                        for (m = 0; m < ps; ++m)
                            newpoints.push(points[i + m]);
                    }
                    i += ps;
                }
                else if (otherpoints[j] == null) {
                    // oops, got a gap
                    for (m = 0; m < ps; ++m)
                        newpoints.push(null);
                    fromgap = true;
                    j += otherps;
                }
                else {
                    // cases where we actually got two points
                    px = points[i + keyOffset];
                    py = points[i + accumulateOffset];
                    qx = otherpoints[j + keyOffset];
                    qy = otherpoints[j + accumulateOffset];
                    bottom = 0;

                    if (px == qx) {
                        for (m = 0; m < ps; ++m)
                            newpoints.push(points[i + m]);

                        newpoints[l + accumulateOffset] += qy;
                        bottom = qy;
                        
                        i += ps;
                        j += otherps;
                    }
                    else if (px > qx) {
                        // we got past point below, might need to
                        // insert interpolated extra point
                        if (withlines && i > 0 && points[i - ps] != null) {
                            intery = py + (points[i - ps + accumulateOffset] - py) * (qx - px) / (points[i - ps + keyOffset] - px);
                            newpoints.push(qx);
                            newpoints.push(intery + qy);
                            for (m = 2; m < ps; ++m)
                                newpoints.push(points[i + m]);
                            bottom = qy; 
                        }

                        j += otherps;
                    }
                    else { // px < qx
                        if (fromgap && withlines) {
                            // if we come from a gap, we just skip this point
                            i += ps;
                            continue;
                        }
                            
                        for (m = 0; m < ps; ++m)
                            newpoints.push(points[i + m]);
                        
                        // we might be able to interpolate a point below,
                        // this can give us a better y
                        if (withlines && j > 0 && otherpoints[j - otherps] != null)
                            bottom = qy + (otherpoints[j - otherps + accumulateOffset] - qy) * (px - qx) / (otherpoints[j - otherps + keyOffset] - qx);

                        newpoints[l + accumulateOffset] += bottom;
                        
                        i += ps;
                    }

                    fromgap = false;
                    
                    if (l != newpoints.length && withbottom)
                        newpoints[l + 2] += bottom;
                }

                // maintain the line steps invariant
                if (withsteps && l != newpoints.length && l > 0
                    && newpoints[l] != null
                    && newpoints[l] != newpoints[l - ps]
                    && newpoints[l + 1] != newpoints[l - ps + 1]) {
                    for (m = 0; m < ps; ++m)
                        newpoints[l + ps + m] = newpoints[l + m];
                    newpoints[l + 1] = newpoints[l - ps + 1];
                }
            }

            datapoints.points = newpoints;
        }
        
        plot.hooks.processDatapoints.push(stackData);
    }
    
    $.plot.plugins.push({
        init: init,
        options: options,
        name: 'stack',
        version: '1.2'
    });
})(jQuery);

define("flot.stack", ["flot"], function(){});

/* Flot plugin for automatically redrawing plots as the placeholder resizes.

Copyright (c) 2007-2014 IOLA and Ole Laursen.
Licensed under the MIT license.

It works by listening for changes on the placeholder div (through the jQuery
resize event plugin) - if the size changes, it will redraw the plot.

There are no options. If you need to disable the plugin for some plots, you
can just fix the size of their placeholders.

*/

/* Inline dependency:
 * jQuery resize event - v1.1 - 3/14/2010
 * http://benalman.com/projects/jquery-resize-plugin/
 *
 * Copyright (c) 2010 "Cowboy" Ben Alman
 * Dual licensed under the MIT and GPL licenses.
 * http://benalman.com/about/license/
 */
(function($,e,t){"$:nomunge";var i=[],n=$.resize=$.extend($.resize,{}),a,r=false,s="setTimeout",u="resize",m=u+"-special-event",o="pendingDelay",l="activeDelay",f="throttleWindow";n[o]=200;n[l]=20;n[f]=true;$.event.special[u]={setup:function(){if(!n[f]&&this[s]){return false}var e=$(this);i.push(this);e.data(m,{w:e.width(),h:e.height()});if(i.length===1){a=t;h()}},teardown:function(){if(!n[f]&&this[s]){return false}var e=$(this);for(var t=i.length-1;t>=0;t--){if(i[t]==this){i.splice(t,1);break}}e.removeData(m);if(!i.length){if(r){cancelAnimationFrame(a)}else{clearTimeout(a)}a=null}},add:function(e){if(!n[f]&&this[s]){return false}var i;function a(e,n,a){var r=$(this),s=r.data(m)||{};s.w=n!==t?n:r.width();s.h=a!==t?a:r.height();i.apply(this,arguments)}if($.isFunction(e)){i=e;return a}else{i=e.handler;e.handler=a}}};function h(t){if(r===true){r=t||1}for(var s=i.length-1;s>=0;s--){var l=$(i[s]);if(l[0]==e||l.is(":visible")){var f=l.width(),c=l.height(),d=l.data(m);if(d&&(f!==d.w||c!==d.h)){l.trigger(u,[d.w=f,d.h=c]);r=t||true}}else{d=l.data(m);d.w=0;d.h=0}}if(a!==null){if(r&&(t==null||t-r<1e3)){a=e.requestAnimationFrame(h)}else{a=setTimeout(h,n[o]);r=false}}}if(!e.requestAnimationFrame){e.requestAnimationFrame=function(){return e.webkitRequestAnimationFrame||e.mozRequestAnimationFrame||e.oRequestAnimationFrame||e.msRequestAnimationFrame||function(t,i){return e.setTimeout(function(){t((new Date).getTime())},n[l])}}()}if(!e.cancelAnimationFrame){e.cancelAnimationFrame=function(){return e.webkitCancelRequestAnimationFrame||e.mozCancelRequestAnimationFrame||e.oCancelRequestAnimationFrame||e.msCancelRequestAnimationFrame||clearTimeout}()}})(jQuery,this);

(function ($) {
    var options = { }; // no options

    function init(plot) {
        function onResize() {
            var placeholder = plot.getPlaceholder();

            // somebody might have hidden us and we can't plot
            // when we don't have the dimensions
            if (placeholder.width() == 0 || placeholder.height() == 0)
                return;

            plot.resize();
            plot.setupGrid();
            plot.draw();
        }
        
        function bindEvents(plot, eventHolder) {
            plot.getPlaceholder().resize(onResize);
        }

        function shutdown(plot, eventHolder) {
            plot.getPlaceholder().unbind("resize", onResize);
        }
        
        plot.hooks.bindEvents.push(bindEvents);
        plot.hooks.shutdown.push(shutdown);
    }
    
    $.plot.plugins.push({
        init: init,
        options: options,
        name: 'resize',
        version: '1.0'
    });
})(jQuery);

define("flot.resize", ["flot"], function(){});

/*
 * jquery.flot.tooltip
 * 
 * description: easy-to-use tooltips for Flot charts
 * version: 0.8.4
 * authors: Krzysztof Urbas @krzysu [myviews.pl],Evan Steinkerchner @Roundaround
 * website: https://github.com/krzysu/flot.tooltip
 * 
 * build on 2014-08-04
 * released under MIT License, 2012
*/ 
(function ($) {
    // plugin options, default values
    var defaultOptions = {
        tooltip: false,
        tooltipOpts: {
            id: "flotTip",
            content: "%s | X: %x | Y: %y",
            // allowed templates are:
            // %s -> series label,
            // %lx -> x axis label (requires flot-axislabels plugin https://github.com/markrcote/flot-axislabels),
            // %ly -> y axis label (requires flot-axislabels plugin https://github.com/markrcote/flot-axislabels),
            // %x -> X value,
            // %y -> Y value,
            // %x.2 -> precision of X value,
            // %p -> percent
            xDateFormat: null,
            yDateFormat: null,
            monthNames: null,
            dayNames: null,
            shifts: {
                x: 10,
                y: 20
            },
            defaultTheme: true,
            lines: {
                track: false,
                threshold: 0.05
            },

            // callbacks
            onHover: function (flotItem, $tooltipEl) {},

            $compat: false
        }
    };

    // object
    var FlotTooltip = function (plot) {
        // variables
        this.tipPosition = {x: 0, y: 0};

        this.init(plot);
    };

    // main plugin function
    FlotTooltip.prototype.init = function (plot) {
        var that = this;

        // detect other flot plugins
        var plotPluginsLength = $.plot.plugins.length;
        this.plotPlugins = [];

        if (plotPluginsLength) {
            for (var p = 0; p < plotPluginsLength; p++) {
                this.plotPlugins.push($.plot.plugins[p].name);
            }
        }

        plot.hooks.bindEvents.push(function (plot, eventHolder) {

            // get plot options
            that.plotOptions = plot.getOptions();

            // if not enabled return
            if (that.plotOptions.tooltip === false || typeof that.plotOptions.tooltip === 'undefined') return;

            // shortcut to access tooltip options
            that.tooltipOptions = that.plotOptions.tooltipOpts;

            if (that.tooltipOptions.$compat) {
                that.wfunc = 'width';
                that.hfunc = 'height';
            } else {
                that.wfunc = 'innerWidth';
                that.hfunc = 'innerHeight';
            }

            // create tooltip DOM element
            var $tip = that.getDomElement();

            // bind event
            $( plot.getPlaceholder() ).bind("plothover", plothover);

            $(eventHolder).bind('mousemove', mouseMove);
        });

        plot.hooks.shutdown.push(function (plot, eventHolder){
            $(plot.getPlaceholder()).unbind("plothover", plothover);
            $(eventHolder).unbind("mousemove", mouseMove);
        });

        function mouseMove(e){
            var pos = {};
            pos.x = e.pageX;
            pos.y = e.pageY;
            plot.setTooltipPosition(pos);
        }

        function plothover(event, pos, item) {
            // Simple distance formula.
            var lineDistance = function (p1x, p1y, p2x, p2y) {
                return Math.sqrt((p2x - p1x) * (p2x - p1x) + (p2y - p1y) * (p2y - p1y));
            };

            // Here is some voodoo magic for determining the distance to a line form a given point {x, y}.
            var dotLineLength = function (x, y, x0, y0, x1, y1, o) {
                if (o && !(o =
                    function (x, y, x0, y0, x1, y1) {
                        if (typeof x0 !== 'undefined') return { x: x0, y: y };
                        else if (typeof y0 !== 'undefined') return { x: x, y: y0 };

                        var left,
                            tg = -1 / ((y1 - y0) / (x1 - x0));

                        return {
                            x: left = (x1 * (x * tg - y + y0) + x0 * (x * -tg + y - y1)) / (tg * (x1 - x0) + y0 - y1),
                            y: tg * left - tg * x + y
                        };
                    } (x, y, x0, y0, x1, y1),
                    o.x >= Math.min(x0, x1) && o.x <= Math.max(x0, x1) && o.y >= Math.min(y0, y1) && o.y <= Math.max(y0, y1))
                ) {
                    var l1 = lineDistance(x, y, x0, y0), l2 = lineDistance(x, y, x1, y1);
                    return l1 > l2 ? l2 : l1;
                } else {
                    var a = y0 - y1, b = x1 - x0, c = x0 * y1 - y0 * x1;
                    return Math.abs(a * x + b * y + c) / Math.sqrt(a * a + b * b);
                }
            };

            if (item) {
                plot.showTooltip(item, pos);
            } else if (that.plotOptions.series.lines.show && that.tooltipOptions.lines.track === true) {
                var closestTrace = {
                    distance: -1
                };

                $.each(plot.getData(), function (i, series) {
                    var xBeforeIndex = 0,
                        xAfterIndex = -1;

                    // Our search here assumes our data is sorted via the x-axis.
                    // TODO: Improve efficiency somehow - search smaller sets of data.
                    for (var j = 1; j < series.data.length; j++) {
                        if (series.data[j - 1][0] <= pos.x && series.data[j][0] >= pos.x) {
                            xBeforeIndex = j - 1;
                            xAfterIndex = j;
                        }
                    }

                    if (xAfterIndex === -1) {
                        plot.hideTooltip();
                        return;
                    }

                    var pointPrev = { x: series.data[xBeforeIndex][0], y: series.data[xBeforeIndex][1] },
                        pointNext = { x: series.data[xAfterIndex][0], y: series.data[xAfterIndex][1] };

                    var distToLine = dotLineLength(pos.x, pos.y, pointPrev.x, pointPrev.y, pointNext.x, pointNext.y, false);

                    if (distToLine < that.tooltipOptions.lines.threshold) {

                        var closestIndex = lineDistance(pointPrev.x, pointPrev.y, pos.x, pos.y) <
                            lineDistance(pos.x, pos.y, pointNext.x, pointNext.y) ? xBeforeIndex : xAfterIndex;

                        var pointSize = series.datapoints.pointsize;

                        // Calculate the point on the line vertically closest to our cursor.
                        var pointOnLine = [
                            pos.x,
                            pointPrev.y + ((pointNext.y - pointPrev.y) * ((pos.x - pointPrev.x) / (pointNext.x - pointPrev.x)))
                        ];

                        var item = {
                            datapoint: pointOnLine,
                            dataIndex: closestIndex,
                            series: series,
                            seriesIndex: i
                        };

                        if (closestTrace.distance === -1 || distToLine < closestTrace.distance) {
                            closestTrace = {
                                distance: distToLine,
                                item: item
                            };
                        }
                    }
                });

                if (closestTrace.distance !== -1)
                    plot.showTooltip(closestTrace.item, pos);
                else
                    plot.hideTooltip();
            } else {
                plot.hideTooltip();
            }
        }

	    // Quick little function for setting the tooltip position.
	    plot.setTooltipPosition = function (pos) {
	        var $tip = that.getDomElement();

	        var totalTipWidth = $tip.outerWidth() + that.tooltipOptions.shifts.x;
	        var totalTipHeight = $tip.outerHeight() + that.tooltipOptions.shifts.y;
	        if ((pos.x - $(window).scrollLeft()) > ($(window)[that.wfunc]() - totalTipWidth)) {
	            pos.x -= totalTipWidth;
	        }
	        if ((pos.y - $(window).scrollTop()) > ($(window)[that.hfunc]() - totalTipHeight)) {
	            pos.y -= totalTipHeight;
	        }
	        that.tipPosition.x = pos.x;
	        that.tipPosition.y = pos.y;
	    };

	    // Quick little function for showing the tooltip.
	    plot.showTooltip = function (target, position) {
	        var $tip = that.getDomElement();

	        // convert tooltip content template to real tipText
	        var tipText = that.stringFormat(that.tooltipOptions.content, target);

	        $tip.html(tipText);
	        plot.setTooltipPosition({ x: position.pageX, y: position.pageY });
	        $tip.css({
	            left: that.tipPosition.x + that.tooltipOptions.shifts.x,
	            top: that.tipPosition.y + that.tooltipOptions.shifts.y
	        }).show();

	        // run callback
	        if (typeof that.tooltipOptions.onHover === 'function') {
	            that.tooltipOptions.onHover(target, $tip);
	        }
	    };

	    // Quick little function for hiding the tooltip.
	    plot.hideTooltip = function () {
	        that.getDomElement().hide().html('');
	    };
    };

    /**
     * get or create tooltip DOM element
     * @return jQuery object
     */
    FlotTooltip.prototype.getDomElement = function () {
        var $tip = $('#' + this.tooltipOptions.id);

        if( $tip.length === 0 ){
            $tip = $('<div />').attr('id', this.tooltipOptions.id);
            $tip.appendTo('body').hide().css({position: 'absolute'});

            if(this.tooltipOptions.defaultTheme) {
                $tip.css({
                    'background': '#fff',
                    'z-index': '1040',
                    'padding': '0.4em 0.6em',
                    'border-radius': '0.5em',
                    'font-size': '0.8em',
                    'border': '1px solid #111',
                    'display': 'none',
                    'white-space': 'nowrap'
                });
            }
        }

        return $tip;
    };

    /**
     * core function, create tooltip content
     * @param  {string} content - template with tooltip content
     * @param  {object} item - Flot item
     * @return {string} real tooltip content for current item
     */
    FlotTooltip.prototype.stringFormat = function (content, item) {

        var percentPattern = /%p\.{0,1}(\d{0,})/;
        var seriesPattern = /%s/;
        var xLabelPattern = /%lx/; // requires flot-axislabels plugin https://github.com/markrcote/flot-axislabels, will be ignored if plugin isn't loaded
        var yLabelPattern = /%ly/; // requires flot-axislabels plugin https://github.com/markrcote/flot-axislabels, will be ignored if plugin isn't loaded
        var xPattern = /%x\.{0,1}(\d{0,})/;
        var yPattern = /%y\.{0,1}(\d{0,})/;
        var xPatternWithoutPrecision = "%x";
        var yPatternWithoutPrecision = "%y";
        var customTextPattern = "%ct";

        var x, y, customText, p;

        // for threshold plugin we need to read data from different place
        if (typeof item.series.threshold !== "undefined") {
            x = item.datapoint[0];
            y = item.datapoint[1];
            customText = item.datapoint[2];
        } else if (typeof item.series.lines !== "undefined" && item.series.lines.steps) {
            x = item.series.datapoints.points[item.dataIndex * 2];
            y = item.series.datapoints.points[item.dataIndex * 2 + 1];
            // TODO: where to find custom text in this variant?
            customText = "";
        } else {
            x = item.series.data[item.dataIndex][0];
            y = item.series.data[item.dataIndex][1];
            customText = item.series.data[item.dataIndex][2];
        }

        // I think this is only in case of threshold plugin
        if (item.series.label === null && item.series.originSeries) {
            item.series.label = item.series.originSeries.label;
        }

        // if it is a function callback get the content string
        if (typeof(content) === 'function') {
            content = content(item.series.label, x, y, item);
        }

        // percent match for pie charts and stacked percent
        if (typeof (item.series.percent) !== 'undefined') {
            p = item.series.percent;
        } else if (typeof (item.series.percents) !== 'undefined') {
            p = item.series.percents[item.dataIndex];
        }        
        if (typeof p === 'number') {
            content = this.adjustValPrecision(percentPattern, content, p);
        }

        // series match
        if (typeof(item.series.label) !== 'undefined') {
            content = content.replace(seriesPattern, item.series.label);
        } else {
            //remove %s if label is undefined
            content = content.replace(seriesPattern, "");
        }

        // x axis label match
        if (this.hasAxisLabel('xaxis', item)) {
            content = content.replace(xLabelPattern, item.series.xaxis.options.axisLabel);
        } else {
            //remove %lx if axis label is undefined or axislabels plugin not present
            content = content.replace(xLabelPattern, "");
        }

        // y axis label match
        if (this.hasAxisLabel('yaxis', item)) {
            content = content.replace(yLabelPattern, item.series.yaxis.options.axisLabel);
        } else {
            //remove %ly if axis label is undefined or axislabels plugin not present
            content = content.replace(yLabelPattern, "");
        }

        // time mode axes with custom dateFormat
        if (this.isTimeMode('xaxis', item) && this.isXDateFormat(item)) {
            content = content.replace(xPattern, this.timestampToDate(x, this.tooltipOptions.xDateFormat, item.series.xaxis.options));
        }
		if (this.isTimeMode('yaxis', item) && this.isYDateFormat(item)) {
            content = content.replace(yPattern, this.timestampToDate(y, this.tooltipOptions.yDateFormat, item.series.yaxis.options));
        }

        // set precision if defined
        if (typeof x === 'number') {
            content = this.adjustValPrecision(xPattern, content, x);
        }
        if (typeof y === 'number') {
            content = this.adjustValPrecision(yPattern, content, y);
        }

        // change x from number to given label, if given
        if (typeof item.series.xaxis.ticks !== 'undefined') {

            var ticks;
            if (this.hasRotatedXAxisTicks(item)) {
                // xaxis.ticks will be an empty array if tickRotor is being used, but the values are available in rotatedTicks
                ticks = 'rotatedTicks';
            } else {
                ticks = 'ticks';
            }

            // see https://github.com/krzysu/flot.tooltip/issues/65
            var tickIndex = item.dataIndex + item.seriesIndex;

            if (item.series.xaxis[ticks].length > tickIndex && !this.isTimeMode('xaxis', item)) {
                var valueX = (this.isCategoriesMode('xaxis', item)) ? item.series.xaxis[ticks][tickIndex].label : item.series.xaxis[ticks][tickIndex].v;
                if (valueX === x) {
                    content = content.replace(xPattern, item.series.xaxis[ticks][tickIndex].label);
                }
            }
        }

        // change y from number to given label, if given
        if (typeof item.series.yaxis.ticks !== 'undefined') {
            for (var index in item.series.yaxis.ticks) {
                if (item.series.yaxis.ticks.hasOwnProperty(index)) {
                    var valueY = (this.isCategoriesMode('yaxis', item)) ? item.series.yaxis.ticks[index].label : item.series.yaxis.ticks[index].v;
                    if (valueY === y) {
                        content = content.replace(yPattern, item.series.yaxis.ticks[index].label);
                    }
                }
            }
        }

        // if no value customization, use tickFormatter by default
        if (typeof item.series.xaxis.tickFormatter !== 'undefined') {
            //escape dollar
            content = content.replace(xPatternWithoutPrecision, item.series.xaxis.tickFormatter(x, item.series.xaxis).replace(/\$/g, '$$'));
        }
        if (typeof item.series.yaxis.tickFormatter !== 'undefined') {
            //escape dollar
            content = content.replace(yPatternWithoutPrecision, item.series.yaxis.tickFormatter(y, item.series.yaxis).replace(/\$/g, '$$'));
        }

        if (customText)
            content = content.replace(customTextPattern, customText);

        return content;
    };

    // helpers just for readability
    FlotTooltip.prototype.isTimeMode = function (axisName, item) {
        return (typeof item.series[axisName].options.mode !== 'undefined' && item.series[axisName].options.mode === 'time');
    };

    FlotTooltip.prototype.isXDateFormat = function (item) {
        return (typeof this.tooltipOptions.xDateFormat !== 'undefined' && this.tooltipOptions.xDateFormat !== null);
    };

    FlotTooltip.prototype.isYDateFormat = function (item) {
        return (typeof this.tooltipOptions.yDateFormat !== 'undefined' && this.tooltipOptions.yDateFormat !== null);
    };

    FlotTooltip.prototype.isCategoriesMode = function (axisName, item) {
        return (typeof item.series[axisName].options.mode !== 'undefined' && item.series[axisName].options.mode === 'categories');
    };

    //
    FlotTooltip.prototype.timestampToDate = function (tmst, dateFormat, options) {
        var theDate = $.plot.dateGenerator(tmst, options);
        return $.plot.formatDate(theDate, dateFormat, this.tooltipOptions.monthNames, this.tooltipOptions.dayNames);
    };

    //
    FlotTooltip.prototype.adjustValPrecision = function (pattern, content, value) {

        var precision;
        var matchResult = content.match(pattern);
        if( matchResult !== null ) {
            if(RegExp.$1 !== '') {
                precision = RegExp.$1;
                value = value.toFixed(precision);

                // only replace content if precision exists, in other case use thickformater
                content = content.replace(pattern, value);
            }
        }
        return content;
    };

    // other plugins detection below

    // check if flot-axislabels plugin (https://github.com/markrcote/flot-axislabels) is used and that an axis label is given
    FlotTooltip.prototype.hasAxisLabel = function (axisName, item) {
        return ($.inArray(this.plotPlugins, 'axisLabels') !== -1 && typeof item.series[axisName].options.axisLabel !== 'undefined' && item.series[axisName].options.axisLabel.length > 0);
    };

    // check whether flot-tickRotor, a plugin which allows rotation of X-axis ticks, is being used
    FlotTooltip.prototype.hasRotatedXAxisTicks = function (item) {
        return ($.inArray(this.plotPlugins, 'tickRotor') !== -1 && typeof item.series.xaxis.rotatedTicks !== 'undefined');
    };

    //
    var init = function (plot) {
      new FlotTooltip(plot);
    };

    // define Flot plugin
    $.plot.plugins.push({
        init: init,
        options: defaultOptions,
        name: 'tooltip',
        version: '0.8.4'
    });

})(jQuery);

define("flot.tooltip", ["flot"], function(){});

/* Pretty handling of time axes.

Copyright (c) 2007-2014 IOLA and Ole Laursen.
Licensed under the MIT license.

Set axis.mode to "time" to enable. See the section "Time series data" in
API.txt for details.

*/

(function($) {

	var options = {
		xaxis: {
			timezone: null,		// "browser" for local to the client or timezone for timezone-js
			timeformat: null,	// format string to use
			twelveHourClock: false,	// 12 or 24 time in time mode
			monthNames: null	// list of names of months
		}
	};

	// round to nearby lower multiple of base

	function floorInBase(n, base) {
		return base * Math.floor(n / base);
	}

	// Returns a string with the date d formatted according to fmt.
	// A subset of the Open Group's strftime format is supported.

	function formatDate(d, fmt, monthNames, dayNames) {

		if (typeof d.strftime == "function") {
			return d.strftime(fmt);
		}

		var leftPad = function(n, pad) {
			n = "" + n;
			pad = "" + (pad == null ? "0" : pad);
			return n.length == 1 ? pad + n : n;
		};

		var r = [];
		var escape = false;
		var hours = d.getHours();
		var isAM = hours < 12;

		if (monthNames == null) {
			monthNames = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];
		}

		if (dayNames == null) {
			dayNames = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];
		}

		var hours12;

		if (hours > 12) {
			hours12 = hours - 12;
		} else if (hours == 0) {
			hours12 = 12;
		} else {
			hours12 = hours;
		}

		for (var i = 0; i < fmt.length; ++i) {

			var c = fmt.charAt(i);

			if (escape) {
				switch (c) {
					case 'a': c = "" + dayNames[d.getDay()]; break;
					case 'b': c = "" + monthNames[d.getMonth()]; break;
					case 'd': c = leftPad(d.getDate()); break;
					case 'e': c = leftPad(d.getDate(), " "); break;
					case 'h':	// For back-compat with 0.7; remove in 1.0
					case 'H': c = leftPad(hours); break;
					case 'I': c = leftPad(hours12); break;
					case 'l': c = leftPad(hours12, " "); break;
					case 'm': c = leftPad(d.getMonth() + 1); break;
					case 'M': c = leftPad(d.getMinutes()); break;
					// quarters not in Open Group's strftime specification
					case 'q':
						c = "" + (Math.floor(d.getMonth() / 3) + 1); break;
					case 'S': c = leftPad(d.getSeconds()); break;
					case 'y': c = leftPad(d.getFullYear() % 100); break;
					case 'Y': c = "" + d.getFullYear(); break;
					case 'p': c = (isAM) ? ("" + "am") : ("" + "pm"); break;
					case 'P': c = (isAM) ? ("" + "AM") : ("" + "PM"); break;
					case 'w': c = "" + d.getDay(); break;
				}
				r.push(c);
				escape = false;
			} else {
				if (c == "%") {
					escape = true;
				} else {
					r.push(c);
				}
			}
		}

		return r.join("");
	}

	// To have a consistent view of time-based data independent of which time
	// zone the client happens to be in we need a date-like object independent
	// of time zones.  This is done through a wrapper that only calls the UTC
	// versions of the accessor methods.

	function makeUtcWrapper(d) {

		function addProxyMethod(sourceObj, sourceMethod, targetObj, targetMethod) {
			sourceObj[sourceMethod] = function() {
				return targetObj[targetMethod].apply(targetObj, arguments);
			};
		};

		var utc = {
			date: d
		};

		// support strftime, if found

		if (d.strftime != undefined) {
			addProxyMethod(utc, "strftime", d, "strftime");
		}

		addProxyMethod(utc, "getTime", d, "getTime");
		addProxyMethod(utc, "setTime", d, "setTime");

		var props = ["Date", "Day", "FullYear", "Hours", "Milliseconds", "Minutes", "Month", "Seconds"];

		for (var p = 0; p < props.length; p++) {
			addProxyMethod(utc, "get" + props[p], d, "getUTC" + props[p]);
			addProxyMethod(utc, "set" + props[p], d, "setUTC" + props[p]);
		}

		return utc;
	};

	// select time zone strategy.  This returns a date-like object tied to the
	// desired timezone

	function dateGenerator(ts, opts) {
		if (opts.timezone == "browser") {
			return new Date(ts);
		} else if (!opts.timezone || opts.timezone == "utc") {
			return makeUtcWrapper(new Date(ts));
		} else if (typeof timezoneJS != "undefined" && typeof timezoneJS.Date != "undefined") {
			var d = new timezoneJS.Date();
			// timezone-js is fickle, so be sure to set the time zone before
			// setting the time.
			d.setTimezone(opts.timezone);
			d.setTime(ts);
			return d;
		} else {
			return makeUtcWrapper(new Date(ts));
		}
	}
	
	// map of app. size of time units in milliseconds

	var timeUnitSize = {
		"second": 1000,
		"minute": 60 * 1000,
		"hour": 60 * 60 * 1000,
		"day": 24 * 60 * 60 * 1000,
		"month": 30 * 24 * 60 * 60 * 1000,
		"quarter": 3 * 30 * 24 * 60 * 60 * 1000,
		"year": 365.2425 * 24 * 60 * 60 * 1000
	};

	// the allowed tick sizes, after 1 year we use
	// an integer algorithm

	var baseSpec = [
		[1, "second"], [2, "second"], [5, "second"], [10, "second"],
		[30, "second"], 
		[1, "minute"], [2, "minute"], [5, "minute"], [10, "minute"],
		[30, "minute"], 
		[1, "hour"], [2, "hour"], [4, "hour"],
		[8, "hour"], [12, "hour"],
		[1, "day"], [2, "day"], [3, "day"],
		[0.25, "month"], [0.5, "month"], [1, "month"],
		[2, "month"]
	];

	// we don't know which variant(s) we'll need yet, but generating both is
	// cheap

	var specMonths = baseSpec.concat([[3, "month"], [6, "month"],
		[1, "year"]]);
	var specQuarters = baseSpec.concat([[1, "quarter"], [2, "quarter"],
		[1, "year"]]);

	function init(plot) {
		plot.hooks.processOptions.push(function (plot, options) {
			$.each(plot.getAxes(), function(axisName, axis) {

				var opts = axis.options;

				if (opts.mode == "time") {
					axis.tickGenerator = function(axis) {

						var ticks = [];
						var d = dateGenerator(axis.min, opts);
						var minSize = 0;

						// make quarter use a possibility if quarters are
						// mentioned in either of these options

						var spec = (opts.tickSize && opts.tickSize[1] ===
							"quarter") ||
							(opts.minTickSize && opts.minTickSize[1] ===
							"quarter") ? specQuarters : specMonths;

						if (opts.minTickSize != null) {
							if (typeof opts.tickSize == "number") {
								minSize = opts.tickSize;
							} else {
								minSize = opts.minTickSize[0] * timeUnitSize[opts.minTickSize[1]];
							}
						}

						for (var i = 0; i < spec.length - 1; ++i) {
							if (axis.delta < (spec[i][0] * timeUnitSize[spec[i][1]]
											  + spec[i + 1][0] * timeUnitSize[spec[i + 1][1]]) / 2
								&& spec[i][0] * timeUnitSize[spec[i][1]] >= minSize) {
								break;
							}
						}

						var size = spec[i][0];
						var unit = spec[i][1];

						// special-case the possibility of several years

						if (unit == "year") {

							// if given a minTickSize in years, just use it,
							// ensuring that it's an integer

							if (opts.minTickSize != null && opts.minTickSize[1] == "year") {
								size = Math.floor(opts.minTickSize[0]);
							} else {

								var magn = Math.pow(10, Math.floor(Math.log(axis.delta / timeUnitSize.year) / Math.LN10));
								var norm = (axis.delta / timeUnitSize.year) / magn;

								if (norm < 1.5) {
									size = 1;
								} else if (norm < 3) {
									size = 2;
								} else if (norm < 7.5) {
									size = 5;
								} else {
									size = 10;
								}

								size *= magn;
							}

							// minimum size for years is 1

							if (size < 1) {
								size = 1;
							}
						}

						axis.tickSize = opts.tickSize || [size, unit];
						var tickSize = axis.tickSize[0];
						unit = axis.tickSize[1];

						var step = tickSize * timeUnitSize[unit];

						if (unit == "second") {
							d.setSeconds(floorInBase(d.getSeconds(), tickSize));
						} else if (unit == "minute") {
							d.setMinutes(floorInBase(d.getMinutes(), tickSize));
						} else if (unit == "hour") {
							d.setHours(floorInBase(d.getHours(), tickSize));
						} else if (unit == "month") {
							d.setMonth(floorInBase(d.getMonth(), tickSize));
						} else if (unit == "quarter") {
							d.setMonth(3 * floorInBase(d.getMonth() / 3,
								tickSize));
						} else if (unit == "year") {
							d.setFullYear(floorInBase(d.getFullYear(), tickSize));
						}

						// reset smaller components

						d.setMilliseconds(0);

						if (step >= timeUnitSize.minute) {
							d.setSeconds(0);
						}
						if (step >= timeUnitSize.hour) {
							d.setMinutes(0);
						}
						if (step >= timeUnitSize.day) {
							d.setHours(0);
						}
						if (step >= timeUnitSize.day * 4) {
							d.setDate(1);
						}
						if (step >= timeUnitSize.month * 2) {
							d.setMonth(floorInBase(d.getMonth(), 3));
						}
						if (step >= timeUnitSize.quarter * 2) {
							d.setMonth(floorInBase(d.getMonth(), 6));
						}
						if (step >= timeUnitSize.year) {
							d.setMonth(0);
						}

						var carry = 0;
						var v = Number.NaN;
						var prev;

						do {

							prev = v;
							v = d.getTime();
							ticks.push(v);

							if (unit == "month" || unit == "quarter") {
								if (tickSize < 1) {

									// a bit complicated - we'll divide the
									// month/quarter up but we need to take
									// care of fractions so we don't end up in
									// the middle of a day

									d.setDate(1);
									var start = d.getTime();
									d.setMonth(d.getMonth() +
										(unit == "quarter" ? 3 : 1));
									var end = d.getTime();
									d.setTime(v + carry * timeUnitSize.hour + (end - start) * tickSize);
									carry = d.getHours();
									d.setHours(0);
								} else {
									d.setMonth(d.getMonth() +
										tickSize * (unit == "quarter" ? 3 : 1));
								}
							} else if (unit == "year") {
								d.setFullYear(d.getFullYear() + tickSize);
							} else {
								d.setTime(v + step);
							}
						} while (v < axis.max && v != prev);

						return ticks;
					};

					axis.tickFormatter = function (v, axis) {

						var d = dateGenerator(v, axis.options);

						// first check global format

						if (opts.timeformat != null) {
							return formatDate(d, opts.timeformat, opts.monthNames, opts.dayNames);
						}

						// possibly use quarters if quarters are mentioned in
						// any of these places

						var useQuarters = (axis.options.tickSize &&
								axis.options.tickSize[1] == "quarter") ||
							(axis.options.minTickSize &&
								axis.options.minTickSize[1] == "quarter");

						var t = axis.tickSize[0] * timeUnitSize[axis.tickSize[1]];
						var span = axis.max - axis.min;
						var suffix = (opts.twelveHourClock) ? " %p" : "";
						var hourCode = (opts.twelveHourClock) ? "%I" : "%H";
						var fmt;

						if (t < timeUnitSize.minute) {
							fmt = hourCode + ":%M:%S" + suffix;
						} else if (t < timeUnitSize.day) {
							if (span < 2 * timeUnitSize.day) {
								fmt = hourCode + ":%M" + suffix;
							} else {
								fmt = "%b %d " + hourCode + ":%M" + suffix;
							}
						} else if (t < timeUnitSize.month) {
							fmt = "%b %d";
						} else if ((useQuarters && t < timeUnitSize.quarter) ||
							(!useQuarters && t < timeUnitSize.year)) {
							if (span < timeUnitSize.year) {
								fmt = "%b";
							} else {
								fmt = "%b %Y";
							}
						} else if (useQuarters && t < timeUnitSize.year) {
							if (span < timeUnitSize.year) {
								fmt = "Q%q";
							} else {
								fmt = "Q%q %Y";
							}
						} else {
							fmt = "%Y";
						}

						var rt = formatDate(d, fmt, opts.monthNames, opts.dayNames);

						return rt;
					};
				}
			});
		});
	}

	$.plot.plugins.push({
		init: init,
		options: options,
		name: 'time',
		version: '1.0'
	});

	// Time-axis support used to be in Flot core, which exposed the
	// formatDate function on the plot object.  Various plugins depend
	// on the function, so we need to re-expose it here.

	$.plot.formatDate = formatDate;
	$.plot.dateGenerator = dateGenerator;

})(jQuery);

define("flot.time", ["flot"], function(){});

define('utils/accounting',['require','hbs!./accounting/html','hbs!./accounting/empty-graph','utils/locale','opennebula/vm','utils/notifier','utils/resource-select','flot','flot.stack','flot.resize','flot.tooltip','flot.time'],function(require) {

  var TemplateHTML = require('hbs!./accounting/html');
  var TemplateEmptyGraph = require('hbs!./accounting/empty-graph');
  var Locale = require('utils/locale');
  var OpenNebulaVM = require('opennebula/vm');
  var Notifier = require('utils/notifier');
  var ResourceSelect = require('utils/resource-select');

  require('flot');
  require('flot.stack');
  require('flot.resize');
  require('flot.tooltip');
  require('flot.time');

  function _html(){
    var html = TemplateHTML({});

    return html;
  };

  // context is a jQuery selector
  // The following options can be set:
  //   fixed_user     fix an owner user ID
  //   fixed_group    fix an owner group ID
  //   init_group_by  "user", "group", "vm". init the group-by selector
  //   fixed_group_by "user", "group", "vm". set a fixed group-by selector
  function _setup(context, opt) {
    if (opt == undefined){
      opt = {};
    }

    //--------------------------------------------------------------------------
    // Set column width
    //--------------------------------------------------------------------------

    var n_columns = 3; // start, end time, button

    if (opt.fixed_user == undefined && opt.fixed_group == undefined){
      n_columns += 1;     //acct_owner_container
    }

    if(opt.fixed_group_by == undefined){
      n_columns += 1;     //acct_group_by_container
    }

    if (n_columns > 4){
      // In this case the first row will have 4 inputs, and the
      // get accounting button will overflow to the second row
      n_columns = 4;
    }

    var width = parseInt(12 / n_columns);

    $("#acct_start_time_container", context).addClass("medium-"+width*2).addClass("large-"+width);
    $("#acct_end_time_container",   context).addClass("medium-"+width*2).addClass("large-"+width);
    $("#acct_group_by_container",   context).addClass("medium-"+width*2).addClass("large-"+width);
    $("#acct_owner_container",      context).addClass("medium-"+width*2).addClass("large-"+width);
    $("#acct_button_container",     context).addClass("medium-"+width*2).addClass("large-"+width);

    //--------------------------------------------------------------------------
    // Init start time to 1st of last month
    //--------------------------------------------------------------------------
    var d = new Date();

    d.setDate(1);
    d.setMonth(d.getMonth() - 1);

    $("#acct_start_time", context).val(
      d.getFullYear() + '-' + ('0'+(d.getMonth()+1)).slice(-2) + '-' + ('0'+d.getDate()).slice(-2));

    //--------------------------------------------------------------------------
    // Init end time to today
    //--------------------------------------------------------------------------

    d = new Date();

    $("#acct_end_time", context).val(
      d.getFullYear() + '-' + ('0'+(d.getMonth()+1)).slice(-2) + '-' + ('0'+d.getDate()).slice(-2));

    //--------------------------------------------------------------------------
    // VM owner: all, group, user
    //--------------------------------------------------------------------------

    if (opt.fixed_user != undefined || opt.fixed_group != undefined){
      $("#acct_owner_container", context).hide();
    } else {
      $("select#acct_owner", context).change(function(){
        var value = $(this).val();

        switch (value){
        case "acct_owner_all":
          $("#acct_owner_select", context).hide();
          break;

        case "acct_owner_group":
          $("#acct_owner_select", context).show();
          ResourceSelect.insert("#acct_owner_select", context, "Group");
          break;

        case "acct_owner_user":
          $("#acct_owner_select", context).show();
          ResourceSelect.insert("#acct_owner_select", context, "User", -1, false,
              '<option value="-1">'+Locale.tr("<< me >>")+'</option>');
          break;
        }
      });
    }

    //--------------------------------------------------------------------------
    // Init group by select
    //--------------------------------------------------------------------------

    if(opt.init_group_by != undefined){
      $("#acct_group_by", context).val(opt.init_group_by);
    }else if(opt.fixed_group_by != undefined){
      $("#acct_group_by", context).val(opt.fixed_group_by);
      $("#acct_group_by_container", context).hide();
    }

    //--------------------------------------------------------------------------
    // Submit request
    //--------------------------------------------------------------------------
    function dateFromString(str) {
      var a = $.map(str.split(/[^0-9]/), function(s) { return parseInt(s, 10) });
      return Date.UTC(a[0], a[1]-1 || 0, a[2] || 1, a[3] || 0, a[4] || 0, a[5] || 0, a[6] || 0);
    }

    $("#acct_submit", context).on("click", function(){
      var start_time = -1;
      var end_time = -1;

      var v = $("#acct_start_time", context).val();
      if (v == ""){
        Notifier.notifyError(Locale.tr("Time range start is mandatory"));
        return false;
      }else{
        start_time = dateFromString(v)
        //start_time = Date.parse(v+' UTC');

        if (isNaN(start_time)){
          Notifier.notifyError(Locale.tr("Time range start is not a valid date. It must be YYYY/MM/DD"));
          return false;
        }

        // ms to s
        start_time = start_time / 1000;
      }

      var v = $("#acct_end_time", context).val();
      if (v != ""){
        end_time = new Date(dateFromString(v));

        if (isNaN(end_time)){
          Notifier.notifyError(Locale.tr("Time range end is not a valid date. It must be YYYY/MM/DD"));
          return false;
        }

        // Add 1 to end_date, because the date is initialized at 00:00.
        // The difference for this range  01/01, 31/01  is:
        //   without adjustment: [01, 31)
        //             adjusted: [01, 31]
        end_time.setDate(end_time.getDate() + 1);

        // ms to s
        end_time = end_time.getTime() / 1000;
      }

      var options = {
        "start_time": start_time,
        "end_time": end_time
      };

      if (opt.fixed_user != undefined){
        options.userfilter = opt.fixed_user;
      } else if (opt.fixed_group != undefined){
        options.group = opt.fixed_group;
      } else {
        var select_val = $("#acct_owner_select .resource_list_select", context).val();

        switch ($("select#acct_owner", context).val()){
        case "acct_owner_all":
          break;

        case "acct_owner_group":
          if(select_val != ""){
            options.group = select_val;
          }
          break;

        case "acct_owner_user":
          if(select_val != ""){
            options.userfilter = select_val;
          }
          break;
        }
      }

      OpenNebulaVM.accounting({
        //timeout: true,
        success: function(req, response){
          _fillAccounting(context, req, response, false);
        },
        error: Notifier.onError,
        data: options
      });

      return false;
    });
  };

  function _fillAccounting(context, req, response, no_table) {
    $("#acct_cpu_graph", context).html(TemplateEmptyGraph());
    $("#acct_mem_graph", context).html(TemplateEmptyGraph());
    $("#acct_disk_graph", context).html(TemplateEmptyGraph());

    var options = req.request.data[0];

    //--------------------------------------------------------------------------
    // Time slots
    //--------------------------------------------------------------------------

    // start_time is mandatory
    var start = new Date(options.start_time * 1000);
    start.setUTCHours(0,0,0,0);

    var end;
    var now = new Date();

    if (options.end_time != undefined && options.end_time != -1) {
      end = new Date(options.end_time * 1000)
      if (end > now) {
        end = now;
      }
    } else {
      end = now;
    }

    // granularity of 1 day
    var times = [];

    var tmp_time = start;

    while (tmp_time < end) {
      times.push(tmp_time.getTime());

      // day += 1
      tmp_time.setUTCDate( tmp_time.getUTCDate() + 1 );
    }

    // End time is the start of the last time slot. For the last slot,
    // we don't add one day if the date is the current day, we add "up to now".
    if (tmp_time > now) {
      tmp_time = now;
    }

    times.push(tmp_time.getTime());

    //--------------------------------------------------------------------------
    // Flot options
    //--------------------------------------------------------------------------

    var options = {
      colors: ["#0098C3","#0A00C2","#AB00C2","#C20037","#C26B00","#78C200","#00C22A","#00B8C2"],

      xaxis : {
        mode: "time",
        timeformat: "%y/%m/%d",
        color: "#efefef",
        font: {
          color: "#999",
          size: 10
        },
        ticks: 4,
        minTickSize: [1, "day"]
      },
      yaxis : { min: 0,
        color: "#efefef",
        font: {
          color: "#999",
          size: 10
        }
      },
      series: {
        bars: {
          show: true,
          lineWidth: 0,
          fill: true,
          barWidth: 24*60*60*1000 * 0.8,
          align: "center"
        },
        stack: true
      },
      legend : {
        show : false
      },
      grid: {
        borderWidth: 1,
        borderColor: "#efefef",
        hoverable: true
      },
      tooltip: true,
      tooltipOpts: {
        content: "%x | %s | %y"
      }
    };

    //--------------------------------------------------------------------------
    // Group by
    //--------------------------------------------------------------------------

    // TODO: Allow to change group by dynamically, instead of calling oned again
    var group_by_fn;
    var group_by_name;
    var group_by_prefix;
    switch ($("#acct_group_by", context).val()){
    case "user":
      group_by_fn = function(history){
        return history.VM.UID;
      }

      group_by_name = function(history){
        return history.VM.UNAME;
      }

      group_by_prefix = Locale.tr("User");

      break;

    case "group":
      group_by_fn = function(history){
        return history.VM.GID;
      }

      group_by_name = function(history){
        return history.VM.GNAME;
      }

      group_by_prefix = Locale.tr("Group");

      break;

    case "vm":
      group_by_fn = function(history){
        return history.OID;
      }

      group_by_name = function(history){
        return history.VM.NAME;
      }

      group_by_prefix = Locale.tr("VM");

      break;
    }

    //--------------------------------------------------------------------------
    // Filter history entries
    //--------------------------------------------------------------------------

    // TODO filter
    // True to proccess, false to discard
    var filter_by_fn = function(history){
      // return history.OID == 3605 || history.OID == 2673;
      return true;
    }

    //--------------------------------------------------------------------------
    // Process data series for flot
    //--------------------------------------------------------------------------

    var series = {};

    $("#acct_no_data", context).hide();

    if(response.HISTORY_RECORDS == undefined){
      $("#acct_placeholder", context).show();
      $("#acct_content", context).hide();

      $("#acct_no_data", context).show();
      return false;
    }

    $.each(response.HISTORY_RECORDS.HISTORY, function(index, history){

      /*
      if(!filter_by_fn(history)){
        return true; //continue
      }
      */
      var group_by = group_by_fn(history);

      if (series[group_by] == undefined){
        series[group_by] = {};
        series[group_by].data_points = {};

        series[group_by].data_points[times[0]] = {};
        series[group_by].data_points[times[times.length-2]] = {};

        series[group_by].data_points[times[0]].CPU_HOURS = 0;
        series[group_by].data_points[times[times.length-2]].CPU_HOURS = 0;

        series[group_by].data_points[times[0]].MEM_HOURS = 0;
        series[group_by].data_points[times[times.length-2]].MEM_HOURS = 0;

        series[group_by].data_points[times[0]].DISK_HOURS = 0;
        series[group_by].data_points[times[times.length-2]].DISK_HOURS = 0;

        var name = group_by_name(history);
        series[group_by].name = name;
        series[group_by].label = group_by_prefix+" "+group_by+" "+name;
      }

      var serie = series[group_by].data_points;

      for (var i = 0; i<times.length-1; i++){

        var t = times[i];
        var t_next = times[i+1];

        // To stack values properly, flot needs an entry for all
        // the time slots
        if(serie[t] == undefined){
          serie[t] = {};
          serie[t].CPU_HOURS = 0;
          serie[t].MEM_HOURS = 0;
          serie[t].DISK_HOURS = 0;
        }

        if( (history.ETIME*1000 > t || history.ETIME == 0) &&
            (history.STIME != 0 && history.STIME*1000 <= t_next) ) {

          var stime = t;
          if(history.STIME != 0){
            stime = Math.max(t, history.STIME*1000);
          }

          var etime = t_next;
          if(history.ETIME != 0){
            etime = Math.min(t_next, history.ETIME*1000);
          }

          var n_hours = (etime - stime) / 1000 / 60 / 60;

          var template = history.VM.TEMPLATE;

          // --- cpu ---

          var val = parseFloat(template.CPU) * n_hours;

          if (!isNaN(val)){
            serie[t].CPU_HOURS += val;
          }

          // --- mem ---

          var val = parseInt(template.MEMORY)/1024 * n_hours;

          if (!isNaN(val)){
            serie[t].MEM_HOURS += val;
          }

          // --- DISK ---

          var disks = [];
          if ($.isArray(template.DISK))
            disks = template.DISK;
          else if (!$.isEmptyObject(template.DISK))
            disks = [template.DISK];

          var snapshots = [];
          if ($.isArray(template.SNAPSHOTS)){
            snapshots = template.SNAPSHOTS;
          } else if (!$.isEmptyObject(template.SNAPSHOTS)) {
            snapshots = [template.SNAPSHOTS];
          }

          var val = 0;
          $.each(disks, function(index, disk){
            val += parseInt(disk.SIZE) * n_hours;
          })

          // snapshots is an array that contains the snapshots for each DISK
          $.each(snapshots, function(index, snapshotForDisk){
            var diskSnapshots = []
            if (!$.isArray(snapshotForDisk.SNAPSHOT)){
              diskSnapshots = [snapshotForDisk.SNAPSHOT];
            }

            // diskSnapshots contains the snapshots for a specific DISK
            $.each(diskSnapshots, function(index, snapshot){
              val += parseInt(snapshot.SIZE) * n_hours;
            })
          })

          if (!isNaN(val)){
            serie[t].DISK_HOURS += val;
          }
        }
      }
    });

    //--------------------------------------------------------------------------
    // Create series, draw plots
    //--------------------------------------------------------------------------

    var cpu_plot_series = [];
    var mem_plot_series = [];
    var disk_plot_series = [];

    $.each(series, function(key, val){
      var cpu_data = [];
      var mem_data = [];
      var disk_data = [];

      $.each(val.data_points, function(time,num){
        cpu_data.push([parseInt(time),num.CPU_HOURS]);
        mem_data.push([parseInt(time),num.MEM_HOURS]);
        disk_data.push([parseInt(time),num.DISK_HOURS]);
      });

      cpu_plot_series.push(
      {
        label: val.label,
        name: val.name,
        id: key,
        data: cpu_data
      });

      mem_plot_series.push(
      {
        label: val.label,
        name: val.name,
        id: key,
        data: mem_data
      });

      disk_plot_series.push(
      {
        label: val.label,
        name: val.name,
        id: key,
        data: disk_data
      });
    });

    var cpu_plot = $.plot($("#acct_cpu_graph", context), cpu_plot_series, options);
    var mem_plot = $.plot($("#acct_mem_graph", context), mem_plot_series, options);
    var disk_plot = $.plot($("#acct_disk_graph", context), disk_plot_series, options);

    //--------------------------------------------------------------------------
    // Init dataTables
    //--------------------------------------------------------------------------

    if (no_table) {
      $(".acct_table",context).hide();
    } else {
      $("#acct_cpu_datatable",context).dataTable().fnClearTable();
      $("#acct_cpu_datatable",context).dataTable().fnDestroy();

      $("#acct_cpu_datatable thead",context).remove();
      $("#acct_cpu_datatable",context).width("100%");


      $("#acct_mem_datatable",context).dataTable().fnClearTable();
      $("#acct_mem_datatable",context).dataTable().fnDestroy();

      $("#acct_mem_datatable thead",context).remove();
      $("#acct_mem_datatable",context).width("100%");


      $("#acct_disk_datatable",context).dataTable().fnClearTable();
      $("#acct_disk_datatable",context).dataTable().fnDestroy();

      $("#acct_disk_datatable thead",context).remove();
      $("#acct_disk_datatable",context).width("100%");


      cpu_plot_data = cpu_plot.getData();
      mem_plot_data = mem_plot.getData();
      disk_plot_data = disk_plot.getData();

      var thead =
        '<thead>\
          <tr>\
            <th>'+Locale.tr("Date UTC")+'</th>\
            <th>'+Locale.tr("Total")+'</th>';

      $.each(cpu_plot_data, function(i, serie){
        thead += '<th style="border-bottom: '+serie.color+' 4px solid !important;'+
              ' border-left: 10px solid white; border-right: 5px solid white;'+
              ' white-space: nowrap">'+
              group_by_prefix+' '+serie.id+'<br/>'+serie.name+'</th>';
      });

      thead += '</tr></thead>';

      $("#acct_cpu_datatable",context).append(thead);

      thead =
        '<thead>\
          <tr>\
            <th>'+Locale.tr("Date UTC")+'</th>\
            <th>'+Locale.tr("Total")+'</th>';

      $.each(mem_plot_data, function(i, serie){
        thead += '<th style="border-bottom: '+serie.color+' 4px solid !important;'+
              ' border-left: 10px solid white; border-right: 5px solid white;'+
              ' white-space: nowrap">'+
              group_by_prefix+' '+serie.id+'<br/>'+serie.name+'</th>';
      });

      thead += '</tr></thead>';

      $("#acct_mem_datatable",context).append(thead);

      thead =
        '<thead>\
          <tr>\
            <th>'+Locale.tr("Date UTC")+'</th>\
            <th>'+Locale.tr("Total")+'</th>';

      $.each(disk_plot_data, function(i, serie){
        thead += '<th style="border-bottom: '+serie.color+' 4px solid !important;'+
              ' border-left: 10px solid white; border-right: 5px solid white;'+
              ' white-space: nowrap">'+
              group_by_prefix+' '+serie.id+'<br/>'+serie.name+'</th>';
      });

      thead += '</tr></thead>';

      $("#acct_disk_datatable",context).append(thead);


      var cpu_dataTable_data = [];
      var mem_dataTable_data = [];
      var disk_dataTable_data = [];

      for (var i = 0; i<times.length-1; i++){
        var t = times[i];

        var cpu_row = [];
        var mem_row = [];
        var disk_row = [];

        var time_st = time_UTC(t);

        cpu_row.push(time_st);
        mem_row.push(time_st);
        disk_row.push(time_st);

        cpu_row.push(0);
        mem_row.push(0);
        disk_row.push(0);

        var cpu_total = 0;
        var mem_total = 0;
        var disk_total = 0;

        $.each(series, function(key, val){
          var v = val.data_points[t];

          if(v != undefined){
            var cpu_v = (v.CPU_HOURS * 100).toFixed() / 100;
            var mem_v = (v.MEM_HOURS * 100).toFixed() / 100;
            var disk_v = (v.DISK_HOURS * 100).toFixed() / 100;

            cpu_total += cpu_v;
            mem_total += mem_v;
            disk_total += disk_v;

            cpu_row.push(cpu_v);
            mem_row.push(mem_v);
            disk_row.push(disk_v);
          } else {
            cpu_row.push(0);
            mem_row.push(0);
            disk_row.push(0);
          }
        });

        cpu_row[1] = (cpu_total * 100).toFixed() / 100;
        mem_row[1] = (mem_total * 100).toFixed() / 100;
        disk_row[1] = (disk_total * 100).toFixed() / 100;

        cpu_dataTable_data.push(cpu_row);
        mem_dataTable_data.push(mem_row);
        disk_dataTable_data.push(disk_row);
      }

      var acct_cpu_dataTable = $("#acct_cpu_datatable",context).dataTable({
        "bSortClasses" : false,
        "bDeferRender": true,
        "aoColumnDefs": [
        { "bSortable": false, "aTargets": ['_all'] },
        ]
      });

      var acct_mem_dataTable = $("#acct_mem_datatable",context).dataTable({
        "bSortClasses" : false,
        "bDeferRender": true,
        "aoColumnDefs": [
        { "bSortable": false, "aTargets": ['_all'] },
        ]
      });

      var acct_disk_dataTable = $("#acct_disk_datatable",context).dataTable({
        "bSortClasses" : false,
        "bDeferRender": true,
        "aoColumnDefs": [
        { "bSortable": false, "aTargets": ['_all'] },
        ]
      });

      if (cpu_dataTable_data.length > 0) {
        acct_cpu_dataTable.fnAddData(cpu_dataTable_data);
      }

      if (mem_dataTable_data.length > 0) {
        acct_mem_dataTable.fnAddData(mem_dataTable_data);
      }

      if (disk_dataTable_data.length > 0) {
        acct_disk_dataTable.fnAddData(disk_dataTable_data);
      }
    }

    $("#acct_placeholder", context).hide();
    $("#acct_content", context).show();
  }

  /**
   * Format time in UTC, YYYY/MM/DD
   * time is in ms
   */
  function time_UTC(time){
    var d = new Date(time);

    return d.getUTCFullYear() + '/' + (d.getUTCMonth()+1) + '/' + d.getUTCDate();
  }

  return {
    'html': _html,
    'setup': _setup,
    'fillAccounting': _fillAccounting
  };
});


/* START_TEMPLATE */
define('hbs!utils/showback/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"row\">\n  <div id=\"showback_user_container\" class=\"left medium-4 columns\">\n    <label for=\"showback_user_select\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filter by user", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <div id=\"showback_user_select\"/>\n  </div>\n  <div id=\"showback_group_container\" class=\"left medium-4 columns\">\n    <label for=\"showback_group_select\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filter by group", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <div id=\"showback_group_select\"/>\n  </div>\n  <div id=\"showback_button_container\" class=\"right medium-4 columns\">\n    <button class=\"button radius success right large-12\" id=\"showback_submit\" type=\"button\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Get Showback", {"name":"tr","hash":{},"data":data})))
    + "\n    </button>\n  </div>\n</div>\n<div id=\"showback_placeholder\">\n  <div class=\"row\">\n    <div class=\"large-8 large-centered columns\">\n      <div class=\"text-center\">\n        <span class=\"fa-stack fa-5x\" style=\"color: #dfdfdf\">\n          <i class=\"fa fa-cloud fa-stack-2x\"></i>\n          <i class=\"fa fa-money fa-stack-1x fa-inverse\"></i>\n        </span>\n        <div id=\"showback_no_data\" class=\"hidden\">\n          <br>\n          <p style=\"font-size: 18px; color: #999\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "There are no showback records", {"name":"tr","hash":{},"data":data})))
    + "\n          </p>\n          </div>\n        </div>\n      </div>\n    </div>\n  </div>\n  <div id=\"showback_content\" class=\"hidden\">\n    <div class=\"row showback_table\">\n      <div class=\"large-12 columns graph_legend\">\n        <h3 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Showback", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n      </div>\n      <div class=\"medium-6 columns\" style=\"overflow:auto\">\n        <table id=\"showback_datatable\" class=\"datatable twelve\">\n          <thead>\n            <tr>\n              <th>dateint</th>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Year", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Month", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Date", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cost", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            </tr>\n          </thead>\n          <tbody id=\"tbody_showback_datatable\">\n          </tbody>\n        </table>\n        <span class=\"label secondary radius showback_select_a_row\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select a row to get detailed information of the month", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </div>\n      <div class=\"medium-6 columns\">\n        <div class=\"large-12 columns centered graph\" id=\"showback_graph\" style=\"height: 200px;\">\n        </div>\n      </div>\n    </div>\n    <div class=\"row showback_vms_table hidden\">\n      <div class=\"large-12 columns graph_legend\">\n        <h3 class=\"subheader\" id=\"showback_vms_title\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMs", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n      </div>\n      <div class=\"large-12 columns\" style=\"overflow:auto\">\n        <table id=\"showback_vms_datatable\" class=\"datatable twelve\">\n          <thead>\n            <tr>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Owner", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hours", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cost", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            </tr>\n          </thead>\n          <tbody id=\"tbody_showback_datatable\">\n          </tbody>\n        </table>\n      </div>\n    </div>\n  </div>";
},"useData":true});
Handlebars.registerPartial('utils/showback/html', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/showback',['require','hbs!./showback/html','utils/locale','opennebula/vm','utils/notifier','utils/resource-select','flot','flot.stack','flot.resize','flot.tooltip','flot.time'],function(require) {

  var TemplateHTML = require('hbs!./showback/html');
  var Locale = require('utils/locale');
  var OpenNebulaVM = require('opennebula/vm');
  var Notifier = require('utils/notifier');
  var ResourceSelect = require('utils/resource-select');

  require('flot');
  require('flot.stack');
  require('flot.resize');
  require('flot.tooltip');
  require('flot.time');

  function _html(){
    var html = TemplateHTML({});

    return html;
  }

  // context is a jQuery selector
  // The following options can be set:
  //   fixed_user     fix an owner user ID. Use "" to fix to "any user"
  //   fixed_group    fix an owner group ID. Use "" to fix to "any group"
  function _setup(context, opt) {
    if (opt == undefined){
      opt = {};
    }

    //--------------------------------------------------------------------------
    // VM owner: all, group, user
    //--------------------------------------------------------------------------

    if (opt.fixed_user != undefined){
      $("#showback_user_container", context).hide();
    } else {
      ResourceSelect.insert("#showback_user_select", context, "User", -1, false,
        '<option value="-1">'+Locale.tr("<< me >>")+'</option>');
    }

    if (opt.fixed_group != undefined){
      $("#showback_group_container", context).hide();
    } else {
      ResourceSelect.insert("#showback_group_select", context, "Group", "", true);
    }

    showback_dataTable = $("#showback_datatable",context).dataTable({
      "bSortClasses" : false,
      "bDeferRender": true,
      "iDisplayLength": 6,
      "sDom": "t<'row collapse'<'small-12 columns'p>>",
      "aoColumnDefs": [
          { "bVisible": false, "aTargets": [0,1,2]}
        ]
    });

    showback_dataTable.fnSort( [ [0, "desc"] ] );

    showback_vms_dataTable = $("#showback_vms_datatable",context).dataTable({
      "bSortClasses" : false,
      "bDeferRender": true
    });

    showback_dataTable.on("click", "tbody tr", function(){
      var cells = showback_dataTable.fnGetData(this);
      var year = cells[1];
      var month = cells[2];

      showback_vms_dataTable.fnClearTable();
      showback_vms_dataTable.fnAddData(
                    showback_dataTable.data("vms_per_date")[year][month].VMS);

      $("#showback_vms_title", context).text(
                  Locale.months[month-1] + " " + year + " " + Locale.tr("VMs"));
      $(".showback_vms_table", context).show();
      $(".showback_select_a_row", context).hide();
    });

    //--------------------------------------------------------------------------
    // Submit request
    //--------------------------------------------------------------------------

    $("#showback_submit", context).on("click", function(){
      var options = {};

      var userfilter;
      var group;

      if (opt.fixed_user != undefined){
        userfilter = opt.fixed_user;
      } else {
        userfilter = $("#showback_user_select .resource_list_select", context).val();
      }

      if (opt.fixed_group != undefined){
        group = opt.fixed_group;
      } else {
        group = $("#showback_group_select .resource_list_select", context).val();
      }

      if(userfilter != ""){
        options.userfilter = userfilter;
      }

      if(group != ""){
        options.group = group;
      }

      OpenNebulaVM.showback({
        // timeout: true,
        success: function(req, response){
          _fillShowback(context, req, response);
        },
        error: Notifier.onError,
        data: options
      });

      return false;
    });
  }

  function _fillShowback(context, req, response) {
    $("#showback_no_data", context).hide();

    if(response.SHOWBACK_RECORDS == undefined){
      $("#showback_placeholder", context).show();
      $("#showback_content", context).hide();

      $("#showback_no_data", context).show();
      return false;
    }

    var vms_per_date = {};
    $.each(response.SHOWBACK_RECORDS.SHOWBACK, function(index, showback){
      if (vms_per_date[showback.YEAR] == undefined) {
        vms_per_date[showback.YEAR] = {};
      }

      if (vms_per_date[showback.YEAR][showback.MONTH] == undefined) {
        vms_per_date[showback.YEAR][showback.MONTH] = {
          "VMS": [],
          "TOTAL": 0
        };
      }

      vms_per_date[showback.YEAR][showback.MONTH].VMS.push(
        [ showback.VMID,
          showback.VMNAME,
          showback.UNAME,
          showback.HOURS,
          showback.TOTAL_COST
        ]);

      vms_per_date[showback.YEAR][showback.MONTH].TOTAL += parseFloat(showback.TOTAL_COST);
    });

    var series = []
    var showback_data = [];
    $.each(vms_per_date, function(year, months){
      $.each(months, function(month, value){
        series.push(
          [ (new Date(year, month-1)).getTime(),
            year,
            month,
            Locale.months[month-1] + " " + year, value.TOTAL.toFixed(2)
          ]);

        showback_data.push([(new Date(year, month-1)), value.TOTAL.toFixed(2) ]);
      });
    });

    showback_dataTable.fnClearTable();
    if (series.length > 0) {
      showback_dataTable.data("vms_per_date", vms_per_date);
      showback_dataTable.fnAddData(series);
    }

    var showback_plot_series = [];
    showback_plot_series.push(
    {
      label: Locale.tr("Showback"),
      data: showback_data
    });

    var options = {
      // colors: [ "#cdebf5", "#2ba6cb", "#6f6f6f" ]
      colors: [ "#2ba6cb", "#707D85", "#AC5A62" ],
      legend: {
        show: false
      },
      xaxis : {
        mode: "time",
        color: "#efefef",
        size: 8,
        minTickSize: [1, "month"]
      },
      yaxis : {
        show: false
      },
      series: {
        bars: {
          show: true,
          lineWidth: 0,
          barWidth: 24 * 60 * 60 * 1000 * 20,
          fill: true,
          align: "left"
        }
      },
      grid: {
        borderWidth: 1,
        borderColor: "#efefef",
        hoverable: true
      }
      //tooltip: true,
      //tooltipOpts: {
      //    content: "%x"
      //}
    };

    var showback_plot = $.plot(
        $("#showback_graph", context), showback_plot_series, options);

    $("#showback_placeholder", context).hide();
    $("#showback_content", context).show();
  }

  return {
    'html': _html,
    'setup': _setup
  };
});

define('utils/graphs',['require','flot','flot.stack','flot.resize','flot.time','utils/humanize'],function(require) {
  /*
    DEPENDENCIES
   */

  require('flot');
  require('flot.stack');
  require('flot.resize');
  // TODO Check if necessary require('flot.tooltip');
  require('flot.time');
  var Humanize = require('utils/humanize');

  /*
    CONSTRUCTOR
   */

  return {
    'plot': _plotGraph
  }

  /*
    FUNCTION DEFINITIONS
   */

  function _plotGraph(response, info) {
    series = [];

    var attributes = info.monitor_resources.split(',');

    if (info.labels) {
      labels = info.labels.split(',')
    }

    for (var i = 0; i < attributes.length; i++) {
      var attribute = attributes[i];

      var data = response.monitoring[attribute];

      if (data) {
        if (info.derivative == true) {
          derivative(data);
        }

        series.push({
          stack: attribute,
          // Turns label TEMPLATE/BLABLA into BLABLA
          label: labels ? labels[i] : attribute[i].split('/').pop(),
          data: data
        });
      }
    }

    var humanize = info.humanize_figures ?
        Humanize.size : function(val) { return (val * 100).toFixed() / 100 };

    var options = {
      //        colors: [ "#cdebf5", "#2ba6cb", "#6f6f6f" ]
      colors: ["#2ba6cb", "#707D85", "#AC5A62"],
      legend : {show : (info.div_legend != undefined),
                 noColumns: attributes.length,
                 container: info.div_legend
               },
      xaxis : {
        tickFormatter: function(val, axis) {
          return Humanize.prettyTimeAxis(val, info.show_date);
        },
        color: "#efefef",
        font: {
          color: "#999",
          size: 10
        }
      },
      yaxis : {
        tickFormatter: function(val, axis) {
          return humanize(val, info.convert_from_bytes, info.y_sufix);
        },
        min: 0,
        color: "#efefef",
        font: {
          color: "#999",
          size: 10
        }
      },
      series: {
        lines: {
          lineWidth: 1
        }
      },
      grid: {
        borderWidth: 1,
        borderColor: "#efefef"
      }
    };

    if (series.length > 0) {
      $.plot(info.div_graph, series, options);
    };
  }

  function derivative(data) {
    for (var i = 0; i < data.length - 1; i++) {
      // Each elem is [timestamp, cumulative value]
      var first = data[i];
      var second = data[i + 1];

      // value now - value before / seconds
      var speed = (second[1] - first[1]) / (second[0] - first[0]);

      // The first element is replaced with the second one
      data[i] = [first[0], speed];
    }

    // The last elem must be removed
    data.pop();
  }
});

;(function ($, window, document, undefined) {
  'use strict';

  Foundation.libs.tooltip = {
    name : 'tooltip',

    version : '5.5.2',

    settings : {
      additional_inheritable_classes : [],
      tooltip_class : '.tooltip',
      append_to : 'body',
      touch_close_text : 'Tap To Close',
      disable_for_touch : false,
      hover_delay : 200,
      show_on : 'all',
      tip_template : function (selector, content) {
        return '<span data-selector="' + selector + '" id="' + selector + '" class="'
          + Foundation.libs.tooltip.settings.tooltip_class.substring(1)
          + '" role="tooltip">' + content + '<span class="nub"></span></span>';
      }
    },

    cache : {},

    init : function (scope, method, options) {
      Foundation.inherit(this, 'random_str');
      this.bindings(method, options);
    },

    should_show : function (target, tip) {
      var settings = $.extend({}, this.settings, this.data_options(target));

      if (settings.show_on === 'all') {
        return true;
      } else if (this.small() && settings.show_on === 'small') {
        return true;
      } else if (this.medium() && settings.show_on === 'medium') {
        return true;
      } else if (this.large() && settings.show_on === 'large') {
        return true;
      }
      return false;
    },

    medium : function () {
      return matchMedia(Foundation.media_queries['medium']).matches;
    },

    large : function () {
      return matchMedia(Foundation.media_queries['large']).matches;
    },

    events : function (instance) {
      var self = this,
          S = self.S;

      self.create(this.S(instance));

      function _startShow(elt, $this, immediate) {
        if (elt.timer) {
          return;
        }

        if (immediate) {
          elt.timer = null;
          self.showTip($this);
        } else {
          elt.timer = setTimeout(function () {
            elt.timer = null;
            self.showTip($this);
          }.bind(elt), self.settings.hover_delay);
        }
      }

      function _startHide(elt, $this) {
        if (elt.timer) {
          clearTimeout(elt.timer);
          elt.timer = null;
        }

        self.hide($this);
      }

      $(this.scope)
        .off('.tooltip')
        .on('mouseenter.fndtn.tooltip mouseleave.fndtn.tooltip touchstart.fndtn.tooltip MSPointerDown.fndtn.tooltip',
          '[' + this.attr_name() + ']', function (e) {
          var $this = S(this),
              settings = $.extend({}, self.settings, self.data_options($this)),
              is_touch = false;

          if (Modernizr.touch && /touchstart|MSPointerDown/i.test(e.type) && S(e.target).is('a')) {
            return false;
          }

          if (/mouse/i.test(e.type) && self.ie_touch(e)) {
            return false;
          }
          
          if ($this.hasClass('open')) {
            if (Modernizr.touch && /touchstart|MSPointerDown/i.test(e.type)) {
              e.preventDefault();
            }
            self.hide($this);
          } else {
            if (settings.disable_for_touch && Modernizr.touch && /touchstart|MSPointerDown/i.test(e.type)) {
              return;
            } else if (!settings.disable_for_touch && Modernizr.touch && /touchstart|MSPointerDown/i.test(e.type)) {
              e.preventDefault();
              S(settings.tooltip_class + '.open').hide();
              is_touch = true;
              // close other open tooltips on touch
              if ($('.open[' + self.attr_name() + ']').length > 0) {
               var prevOpen = S($('.open[' + self.attr_name() + ']')[0]);
               self.hide(prevOpen);
              }
            }

            if (/enter|over/i.test(e.type)) {
              _startShow(this, $this);

            } else if (e.type === 'mouseout' || e.type === 'mouseleave') {
              _startHide(this, $this);
            } else {
              _startShow(this, $this, true);
            }
          }
        })
        .on('mouseleave.fndtn.tooltip touchstart.fndtn.tooltip MSPointerDown.fndtn.tooltip', '[' + this.attr_name() + '].open', function (e) {
          if (/mouse/i.test(e.type) && self.ie_touch(e)) {
            return false;
          }

          if ($(this).data('tooltip-open-event-type') == 'touch' && e.type == 'mouseleave') {
            return;
          } else if ($(this).data('tooltip-open-event-type') == 'mouse' && /MSPointerDown|touchstart/i.test(e.type)) {
            self.convert_to_touch($(this));
          } else {
            _startHide(this, $(this));
          }
        })
        .on('DOMNodeRemoved DOMAttrModified', '[' + this.attr_name() + ']:not(a)', function (e) {
          _startHide(this, S(this));
        });
    },

    ie_touch : function (e) {
      // How do I distinguish between IE11 and Windows Phone 8?????
      return false;
    },

    showTip : function ($target) {
      var $tip = this.getTip($target);
      if (this.should_show($target, $tip)) {
        return this.show($target);
      }
      return;
    },

    getTip : function ($target) {
      var selector = this.selector($target),
          settings = $.extend({}, this.settings, this.data_options($target)),
          tip = null;

      if (selector) {
        tip = this.S('span[data-selector="' + selector + '"]' + settings.tooltip_class);
      }

      return (typeof tip === 'object') ? tip : false;
    },

    selector : function ($target) {
      var dataSelector = $target.attr(this.attr_name()) || $target.attr('data-selector');

      if (typeof dataSelector != 'string') {
        dataSelector = this.random_str(6);
        $target
          .attr('data-selector', dataSelector)
          .attr('aria-describedby', dataSelector);
      }

      return dataSelector;
    },

    create : function ($target) {
      var self = this,
          settings = $.extend({}, this.settings, this.data_options($target)),
          tip_template = this.settings.tip_template;

      if (typeof settings.tip_template === 'string' && window.hasOwnProperty(settings.tip_template)) {
        tip_template = window[settings.tip_template];
      }

      var $tip = $(tip_template(this.selector($target), $('<div></div>').html($target.attr('title')).html())),
          classes = this.inheritable_classes($target);

      $tip.addClass(classes).appendTo(settings.append_to);

      if (Modernizr.touch) {
        $tip.append('<span class="tap-to-close">' + settings.touch_close_text + '</span>');
        $tip.on('touchstart.fndtn.tooltip MSPointerDown.fndtn.tooltip', function (e) {
          self.hide($target);
        });
      }

      $target.removeAttr('title').attr('title', '');
    },

    reposition : function (target, tip, classes) {
      var width, nub, nubHeight, nubWidth, column, objPos;

      tip.css('visibility', 'hidden').show();

      width = target.data('width');
      nub = tip.children('.nub');
      nubHeight = nub.outerHeight();
      nubWidth = nub.outerHeight();

      if (this.small()) {
        tip.css({'width' : '100%'});
      } else {
        tip.css({'width' : (width) ? width : 'auto'});
      }

      objPos = function (obj, top, right, bottom, left, width) {
        return obj.css({
          'top' : (top) ? top : 'auto',
          'bottom' : (bottom) ? bottom : 'auto',
          'left' : (left) ? left : 'auto',
          'right' : (right) ? right : 'auto'
        }).end();
      };

      objPos(tip, (target.offset().top + target.outerHeight() + 10), 'auto', 'auto', target.offset().left);

      if (this.small()) {
        objPos(tip, (target.offset().top + target.outerHeight() + 10), 'auto', 'auto', 12.5, $(this.scope).width());
        tip.addClass('tip-override');
        objPos(nub, -nubHeight, 'auto', 'auto', target.offset().left);
      } else {
        var left = target.offset().left;
        if (Foundation.rtl) {
          nub.addClass('rtl');
          left = target.offset().left + target.outerWidth() - tip.outerWidth();
        }

        objPos(tip, (target.offset().top + target.outerHeight() + 10), 'auto', 'auto', left);
        // reset nub from small styles, if they've been applied
        if (nub.attr('style')) {
          nub.removeAttr('style');
        }
        
        tip.removeClass('tip-override');
        if (classes && classes.indexOf('tip-top') > -1) {
          if (Foundation.rtl) {
            nub.addClass('rtl');
          }
          objPos(tip, (target.offset().top - tip.outerHeight()), 'auto', 'auto', left)
            .removeClass('tip-override');
        } else if (classes && classes.indexOf('tip-left') > -1) {
          objPos(tip, (target.offset().top + (target.outerHeight() / 2) - (tip.outerHeight() / 2)), 'auto', 'auto', (target.offset().left - tip.outerWidth() - nubHeight))
            .removeClass('tip-override');
          nub.removeClass('rtl');
        } else if (classes && classes.indexOf('tip-right') > -1) {
          objPos(tip, (target.offset().top + (target.outerHeight() / 2) - (tip.outerHeight() / 2)), 'auto', 'auto', (target.offset().left + target.outerWidth() + nubHeight))
            .removeClass('tip-override');
          nub.removeClass('rtl');
        }
      }

      tip.css('visibility', 'visible').hide();
    },

    small : function () {
      return matchMedia(Foundation.media_queries.small).matches &&
        !matchMedia(Foundation.media_queries.medium).matches;
    },

    inheritable_classes : function ($target) {
      var settings = $.extend({}, this.settings, this.data_options($target)),
          inheritables = ['tip-top', 'tip-left', 'tip-bottom', 'tip-right', 'radius', 'round'].concat(settings.additional_inheritable_classes),
          classes = $target.attr('class'),
          filtered = classes ? $.map(classes.split(' '), function (el, i) {
            if ($.inArray(el, inheritables) !== -1) {
              return el;
            }
          }).join(' ') : '';

      return $.trim(filtered);
    },

    convert_to_touch : function ($target) {
      var self = this,
          $tip = self.getTip($target),
          settings = $.extend({}, self.settings, self.data_options($target));

      if ($tip.find('.tap-to-close').length === 0) {
        $tip.append('<span class="tap-to-close">' + settings.touch_close_text + '</span>');
        $tip.on('click.fndtn.tooltip.tapclose touchstart.fndtn.tooltip.tapclose MSPointerDown.fndtn.tooltip.tapclose', function (e) {
          self.hide($target);
        });
      }

      $target.data('tooltip-open-event-type', 'touch');
    },

    show : function ($target) {
      var $tip = this.getTip($target);

      if ($target.data('tooltip-open-event-type') == 'touch') {
        this.convert_to_touch($target);
      }

      this.reposition($target, $tip, $target.attr('class'));
      $target.addClass('open');
      $tip.fadeIn(150);
    },

    hide : function ($target) {
      var $tip = this.getTip($target);
      $tip.fadeOut(150, function () {
        $tip.find('.tap-to-close').remove();
        $tip.off('click.fndtn.tooltip.tapclose MSPointerDown.fndtn.tapclose');
        $target.removeClass('open');
      });
    },

    off : function () {
      var self = this;
      this.S(this.scope).off('.fndtn.tooltip');
      this.S(this.settings.tooltip_class).each(function (i) {
        $('[' + self.attr_name() + ']').eq(i).attr('title', $(this).text());
      }).remove();
    },

    reflow : function () {}
  };
}(jQuery, window, window.document));

define("foundation.tooltip", ["foundation.core"], function(){});

define('utils/tips',['require','foundation.tooltip'],function(require) {
  require('foundation.tooltip');
  //Replaces all class"tip" divs with an information icon that
  //displays the tip information on mouseover.
  var _setup = function(context, position) {
    if (!context) {
      context = $(document);
    }

    //For each tip in this context
    $('.tip', context).each(function() {
      var obj = $(this);
      obj.removeClass('tip');
      var tip = obj.html();

      var tip_classes = ['has-tip']
      if (position) {
        tip_classes.push(position)
      }
      //replace the text with an icon and spans
      //obj.html('<span data-tooltip class="' + tip_classes.join(' ') + '" data-width="210" title="' + tip + '"><i class="fa fa-question-circle"></i></span>');
      obj.html('<span title="' + tip + '"><i class="fa fa-question-circle"></i></span>');
    });
  }

  var _html = function(str) {
    //return '<span data-tooltip class="" data-width="210" title="' + str + '"><i class="fa fa-question-circle"></i></span>'
    return '<span title="' + str + '"><i class="fa fa-question-circle"></i></span>';
  }

  return {
    'setup': _setup,
    'html': _html
  }
})
;
define('templates/helpers/tip',['require','hbs/handlebars','utils/locale','utils/tips'],function(require) {
  var Handlebars = require('hbs/handlebars');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');

  var tip = function(context, options) {
    return Tips.html(context);
  };

  Handlebars.registerHelper('tip', tip);
  return tip;
})
;

/* START_TEMPLATE */
define('hbs!utils/range-slider/html',['hbs','hbs/handlebars','templates/helpers/tip'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, buffer = "    ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, (depth0 != null ? depth0.tip : depth0), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  \n";
},"3":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "    <div class=\"small-9 columns\">\n      <input type=\"text\" id=\""
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "\" name=\""
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "\"/>\n    </div>\n    <div class=\"small-3 columns\">\n      <span style=\"font-size: 18px; color: #999;\">"
    + escapeExpression(((helper = (helper = helpers.unitLabel || (depth0 != null ? depth0.unitLabel : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unitLabel","hash":{},"data":data}) : helper)))
    + "</span>\n    </div>\n";
},"5":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "    <input type=\"text\" wizard_field=\""
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "\" id=\""
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "\" name=\""
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "\"/>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<label class=\"\" for=\""
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "\">\n  "
    + escapeExpression(((helper = (helper = helpers.label || (depth0 != null ? depth0.label : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"label","hash":{},"data":data}) : helper)))
    + "\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.tip : depth0), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "</label>\n<div class=\"medium-10 columns\">\n  <div id=\""
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "Slider\" class=\"range-slider radius\" data-slider data-options=\"start: "
    + escapeExpression(((helper = (helper = helpers.start || (depth0 != null ? depth0.start : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"start","hash":{},"data":data}) : helper)))
    + "; end: "
    + escapeExpression(((helper = (helper = helpers.end || (depth0 != null ? depth0.end : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"end","hash":{},"data":data}) : helper)))
    + "; step: "
    + escapeExpression(((helper = (helper = helpers.step || (depth0 != null ? depth0.step : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"step","hash":{},"data":data}) : helper)))
    + ";\">\n    <span class=\"range-slider-handle\"></span>\n    <span class=\"range-slider-active-segment\"></span>\n    <input type=\"hidden\"></div>\n</div>\n<div class=\"medium-2 columns\">\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.unitLabel : depth0), {"name":"if","hash":{},"fn":this.program(3, data),"inverse":this.program(5, data),"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>";
},"useData":true});
Handlebars.registerPartial('utils/range-slider/html', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/range-slider',['require','hbs!./range-slider/html'],function(require) {
  var TemplateHTML = require('hbs!./range-slider/html');

  return {
    'insert': _insert
  }

  /*
    Insert a range slider
    @param {Object} opts Options for the slider
      opts.label Label for the input
      opts.tip String with the despription
      opts.name Name of the input, it will be used for the wizard_field value
        and the ID of the input
      opts.start Start value of the slider
      opts.end End value of the slider
      opts.step Step value of the slider
      opts.startValue Initialize the slider with this value
    @param {Object} context div to insert the range slider
    @returns {String} HTML row
   */
  function _insert(opts, context) {
    context.html(TemplateHTML(opts));
    context.foundation('slider', 'reflow');
    
    // Define the cpu slider
    var input = $("#" + opts.name, context);
    var slider = $("#" + opts.name + "Slider", context)

    slider.on('change.fndtn.slider', function(){
      if ($(this).attr('data-slider') >= 0) {
        input.val($(this).attr('data-slider'));
      }
    });

    input.on('change', function() {
      if (this.value && this.value >= 0) {
        slider.foundation('slider', 'set_value', this.value);
      } else {
        slider.foundation('slider', 'set_value', -1);
      }
    });

    if (opts.startValue) {
      slider.foundation('slider', 'set_value', opts.startValue);
      input.val(opts.startValue);
    }
  }
});

define('utils/disks-resize',['require','utils/locale','sunstone-config','opennebula','utils/range-slider'],function(require){
  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var OpenNebula = require('opennebula');
  var RangeSlider = require('utils/range-slider');

  return {
    'insert': _insert,
    'retrieve': _retrieve
  };

  function _insert(template_json, disksContext) {
    var template_disk = template_json.VMTEMPLATE.TEMPLATE.DISK
    var disks = []
    if ($.isArray(template_disk)) {
      disks = template_disk
    } else if (!$.isEmptyObject(template_disk)) {
      disks = [template_disk]
    }

    if (disks.length > 0) {
      disksContext.html(
        '<div class="row">'+
          '<div class="large-12 columns">'+
            '<h3 class="subheader text-right">'+
              '<span class="left">'+
                '<i class="fa fa-tasks fa-lg"></i>&emsp;'+
                Locale.tr("Disks")+
              '</span>'+
              '<span class="provision_create_template_disk_cost_div hidden">' +
                '<span class="cost_value">0.00</span> '+
                '<small style="color: #999;">'+Locale.tr("COST")+' / ' + Locale.tr("HOUR") + '</small>'+
              '</span>'+
              '<br>'+
            '</h3>'+
          '</div>'+
        '</div>'+
        '<div class="row">'+
          '<div class="large-12 large-centered columns disksContainer">'+
            '<span class="text-center" style="font-size:80px">'+
              '<i class="fa fa-spinner fa-spin"></i>'+
            '</span>'+
          '</div>'+
        '</div>')

      OpenNebula.Template.show({
        data : {
            id: template_json.VMTEMPLATE.ID,
            extended: true
        },
        success: function(request, extendedTemplateJSON) {
          var extendedTemplateDisk = extendedTemplateJSON.VMTEMPLATE.TEMPLATE.DISK;
          var extendedDisks = []
          if ($.isArray(extendedTemplateDisk)) {
            extendedDisks = extendedTemplateDisk
          } else if (!$.isEmptyObject(extendedTemplateDisk)) {
            extendedDisks = [extendedTemplateDisk]
          }


          var disk_cost = template_json.VMTEMPLATE.TEMPLATE.DISK_COST;
          if (disk_cost && Config.isFeatureEnabled("showback")) {
            $(".provision_create_template_disk_cost_div", disksContext).show();

            disksContext.on("change.fndtn.slider", '.range-slider', function(){
              /*if ($(this).attr('data-slider') <= 0) {
                var diskContainer = $(this).parent('.diskContainer');
                $("#SIZE", diskContainer).val(diskContainer.data('original_size'));
              }*/

              var cost = 0;
              $('.range-slider', disksContext).each(function(){
                if ($(this).attr('data-slider') > 0) {
                  cost += $(this).attr('data-slider') * disk_cost
                }
              })
              $(".cost_value", disksContext).html(cost.toFixed(2));
            });
          } else {
            $(".provision_create_template_disk_cost_div", disksContext).hide();
          }

          var diskContext;
          $(".disksContainer", disksContext).html("");
          $.each(extendedDisks, function(disk_id, disk) {
            diskContext = $('<div class="row diskContainer">'+
                '<div class="large-12 columns diskSlider">' +
                '</div>' +
              '</div>').appendTo($(".disksContainer", disksContext));

            diskContext.data('template_disk', disks[disk_id]);

            var sizeGB = disk.SIZE / 1024;
            diskContext.data('original_size', sizeGB);
            RangeSlider.insert({
              'label': disk.IMAGE,
              'unitLabel': 'GB',
              'name': 'SIZE',
              'start': sizeGB,
              'end': sizeGB + 500,
              'step': 10,
              'startValue': sizeGB
            }, $(".diskSlider", diskContext))
          })
        }
      })
    } else {
      disksContext.html("");
    }
  }

  function _retrieve(context) {
    var disks = [];
    var disk, size;
    $(".diskContainer", context).each(function(){
      if ($(this).data("template_disk")) {
        disk = $(this).data("template_disk");
        original_size = $(this).data("original_size");
        size = $("#SIZE", this).val();
        if (size && size > original_size && size > 0) {
          disk['SIZE'] = Math.ceil(size * 1024);
        }
      }

      if (disk) {
        disks.push(disk);
      }
    });

    return disks;
  }
});
define('utils/nics-section',['require','utils/locale','utils/notifier','opennebula'],function(require) {
  var Locale = require('utils/locale');
  var Notifier = require('utils/notifier');
  var OpenNebula = require('opennebula');

  var provision_nic_accordion_id = 0;
  var provision_nic_accordion_dd_id = 0;

  return {
    'insert': _insert,
    'retrieve': _retrieve
  }

  function _insert(template_json, context) {
    if (template_json.VMTEMPLATE.TEMPLATE.SUNSTONE_NETWORK_SELECT != "NO") {
      var template_nic = template_json.VMTEMPLATE.TEMPLATE.NIC
      var nics = []
      if ($.isArray(template_nic))
          nics = template_nic
      else if (!$.isEmptyObject(template_nic))
          nics = [template_nic]
        
      generate_provision_network_accordion(
        $(".provision_network_selector", context));

      $.each(nics, function(index, nic) {
        generate_provision_network_table(
          $(".provision_nic_accordion", context),
          nic);
      })
    }
  }

  function _retrieve(context) {
    var nics = [];
    var nic;
    $(".selected_network", context).each(function() {
      if ($(this).attr("template_nic")) {
        nic = JSON.parse($(this).attr("template_nic"))
      } else if ($(this).attr("opennebula_id")) {
        nic = {
          'network_id': $(this).attr("opennebula_id")
        }
      } else {
        nic = undefined;
      }

      if (nic) {
        nics.push(nic);
      }
    });

    return nics
  }

  function generate_provision_network_table(context, nic, vnet_attr) {
    context.off();
    var nic_span;

    if (nic) {
      nic_span = '<span class="selected_network" template_nic=\'' + JSON.stringify(nic) + '\'>' +
          '<span style="color: #999; font-size: 14px">' + Locale.tr("INTERFACE") + "</span>&emsp;&emsp;" +
          '<span style="color: #777;">' + (nic.NETWORK || nic.NETWORK_ID) + "</span>" +
        '</span>' +
        '<span class="has-tip right provision_remove_nic" style="cursor: pointer;">' +
          '<i class="fa fa-times"/>' +
        '</span>' +
        '<span class="has-tip right" style="cursor: pointer; margin-right:10px">' +
          '<i class="fa fa-pencil"/>' +
        '</span>';
    } else if (vnet_attr) {
      nic_span = '<span style="color: #777; font-size: 16px">' + vnet_attr.description + "</span><br>" +
        '<span class="selected_network only-not-active" attr_name=\'' + vnet_attr.name + '\' style="color: #777;">' +
          '<span style="color: #999; font-size: 14px">' + Locale.tr("INTERFACE") + "</span>&emsp;&emsp;" +
          '<span class="button radius small">' + Locale.tr("Select a Network") + "</span>" +
        '</span>' +
        '<span class="only-active" style="color:#555">' +
          Locale.tr("Select a Network for this interface") +
        '</span>' +
        '<span class="has-tip right only-not-active" style="cursor: pointer; margin-right:10px">' +
          '<i class="fa fa-pencil"/>' +
        '</span>';
    } else {
      nic_span =
        '<span class="selected_network only-not-active" style="color: #777;">' +
          '<span style="color: #999; font-size: 14px">' + Locale.tr("INTERFACE") + "</span>&emsp;&emsp;" +
          '<span class="button radius small">' + Locale.tr("Select a Network") + "</span>" +
        '</span>' +
        '<span class="only-active" style="color:#555">' +
          Locale.tr("Select a Network for this interface") +
        '</span>' +
        '<span class="has-tip right provision_remove_nic" style="cursor: pointer;">' +
          '<i class="fa fa-times"/>' +
        '</span>' +
        '<span class="has-tip right only-not-active" style="cursor: pointer; margin-right:10px">' +
          '<i class="fa fa-pencil"/>' +
        '</span>';
    }

    var dd_context = $('<dd style="border-bottom: 1px solid #efefef;" class="accordion-navigation">' +
      '<a href="#provision_accordion_dd_' + provision_nic_accordion_dd_id + '" style="background: #fff; font-size: 24px">' +
        nic_span +
      '</a>' +
      '<div id="provision_accordion_dd_' + provision_nic_accordion_dd_id + '" class="content">' +
        '<div class="row">' +
          '<div class="large-12 large-centered columns">' +
            '<h3 class="subheader text-right">' +
              '<input type="search" class="provision-search-input right" placeholder="Search"/>' +
            '</h3>' +
            '<br>' +
          '</div>' +
        '</div>' +
        '<div class="row">' +
          '<div class="large-12 large-centered columns">' +
            '<table class="provision_networks_table">' +
              '<thead class="hidden">' +
                '<tr>' +
                  '<th>' + Locale.tr("ID") + '</th>' +
                  '<th>' + Locale.tr("Name") + '</th>' +
                '</tr>' +
              '</thead>' +
              '<tbody class="hidden">' +
              '</tbody>' +
            '</table>' +
            '<br>' +
          '</div>' +
        '</div>' +
        '</div>' +
      '</dd>').appendTo(context);

    provision_nic_accordion_dd_id += 1;

    var provision_networks_datatable = $('.provision_networks_table', dd_context).dataTable({
      "iDisplayLength": 6,
      "sDom" : '<"H">t<"F"lp>',
      "aLengthMenu": [[6, 12, 36, 72], [6, 12, 36, 72]],
      "aoColumnDefs": [
          {"bVisible": false, "aTargets": ["all"]}
      ],
      "aoColumns": [
          {"mDataProp": "VNET.ID"},
          {"mDataProp": "VNET.NAME"}
      ],
      "fnPreDrawCallback": function (oSettings) {
        // create a thumbs container if it doesn't exist. put it in the dataTables_scrollbody div
        if (this.$('tr', {"filter": "applied"}).length == 0) {
          this.html('<div class="text-center">' +
            '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
              '<i class="fa fa-cloud fa-stack-2x"></i>' +
              '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>' +
            '</span>' +
            '<br>' +
            '<br>' +
            '<span style="font-size: 18px; color: #999">' +
              Locale.tr("There are no networks available. Please contact your cloud administrator") +
            '</span>' +
            '</div>');
        } else {
          $(".provision_networks_table", dd_context).html(
            '<ul class="provision_networks_ul large-block-grid-3 medium-block-grid-3 small-block-grid-1 text-center">' +
            '</ul>');
        }

        return true;
      },
      "fnRowCallback": function(nRow, aData, iDisplayIndex, iDisplayIndexFull) {
        var data = aData.VNET;
        $(".provision_networks_ul", dd_context).append(
          '<li>' +
            '<ul class="provision-pricing-table hoverable more-than-one" opennebula_id="' + data.ID + '" opennebula_name="' + data.NAME + '">' +
              '<li class="provision-title" title="' + data.NAME + '">' +
                data.NAME +
              '</li>' +
              '<li class="provision-bullet-item">' +
                '<i class="fa fa-fw fa-globe" style="font-size:40px;"/>' +
              '</li>' +
              '<li class="provision-description">' +
                (data.TEMPLATE.DESCRIPTION || '...') +
              '</li>' +
            '</ul>' +
          '</li>');

        return nRow;
      }
    });

    $('.provision-search-input', dd_context).on('input', function() {
      provision_networks_datatable.fnFilter($(this).val());
    })

    dd_context.on("click", ".provision-pricing-table.more-than-one" , function() {
      $(".selected_network", dd_context).html(
          '<span style="color: #999; font-size: 14px">' + Locale.tr("INTERFACE") + "</span>&emsp;&emsp;" +
          '<span style="color: #777;">' + $(this).attr("opennebula_name") + "</span>");

      $(".selected_network", dd_context).attr("opennebula_id", $(this).attr("opennebula_id"))
      $(".selected_network", dd_context).removeAttr("template_nic")

      $('a', dd_context).first().trigger("click");
    })

    dd_context.on("click", ".provision_remove_nic" , function() {
      dd_context.remove();
      return false;
    });

    if (!nic && !vnet_attr) {
      $('a', dd_context).trigger("click");
    }

    update_provision_networks_datatable(provision_networks_datatable);
  }

  function generate_provision_network_accordion(context, hide_add_button) {
    context.off();
    context.html(
      '<br>' +
      '<div class="row">' +
        '<div class="large-12 columns">' +
          '<h3 class="subheader text-right">' +
            '<span class="left">' +
              '<i class="fa fa-globe fa-lg"></i>&emsp;' +
              Locale.tr("Network") +
            '</span>' +
          '</h3>' +
          '<br>' +
        '</div>' +
      '</div>' +
      '<div class="row">' +
        '<div class="large-12 large-centered columns">' +
          '<dl class="accordion provision_nic_accordion" data-accordion="provision_accordion_' + provision_nic_accordion_id + '">' +
          '</dl>' +
          '<br>' +
          '<a class="button radius secondary provision_add_network_interface" style="width:inherit; padding: 1rem; color: #555; ' + (hide_add_button ? 'display:none;' : '') + '">' +
            Locale.tr("Add another Network Interface") +
          '</a>' +
        '</div>' +
      '</div>' +
      '<br>')

    provision_nic_accordion_id += 1;

    $(".provision_add_network_interface", context).on("click", function() {
      generate_provision_network_table($(".accordion", context));
    })

    $(document).foundation();
  }

  function update_provision_networks_datatable(datatable) {
    datatable.html('<div class="text-center">' +
      '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
        '<i class="fa fa-cloud fa-stack-2x"></i>' +
        '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>' +
      '</span>' +
      '<br>' +
      '<br>' +
      '<span style="font-size: 18px; color: #999">' +
      '</span>' +
      '</div>');

    OpenNebula.Network.list({
      timeout: true,
      success: function (request, item_list) {
        datatable.fnClearTable(true);
        if (item_list.length == 0) {
          datatable.html('<div class="text-center">' +
            '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
              '<i class="fa fa-cloud fa-stack-2x"></i>' +
              '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>' +
            '</span>' +
            '<br>' +
            '<br>' +
            '<span style="font-size: 18px; color: #999">' +
              Locale.tr("There are no networks available.") +
            '</span>' +
            '</div>');
        } else {
          datatable.fnAddData(item_list);
        }
      },
      error: Notifier.onError
    });
  }
})
;
;(function ($, window, document, undefined) {
  'use strict';

  Foundation.libs.slider = {
    name : 'slider',

    version : '5.5.2',

    settings : {
      start : 0,
      end : 100,
      step : 1,
      precision : null,
      initial : null,
      display_selector : '',
      vertical : false,
      trigger_input_change : false,
      on_change : function () {}
    },

    cache : {},

    init : function (scope, method, options) {
      Foundation.inherit(this, 'throttle');
      this.bindings(method, options);
      this.reflow();
    },

    events : function () {
      var self = this;

      $(this.scope)
        .off('.slider')
        .on('mousedown.fndtn.slider touchstart.fndtn.slider pointerdown.fndtn.slider',
        '[' + self.attr_name() + ']:not(.disabled, [disabled]) .range-slider-handle', function (e) {
          if (!self.cache.active) {
            e.preventDefault();
            self.set_active_slider($(e.target));
          }
        })
        .on('mousemove.fndtn.slider touchmove.fndtn.slider pointermove.fndtn.slider', function (e) {
          if (!!self.cache.active) {
            e.preventDefault();
            if ($.data(self.cache.active[0], 'settings').vertical) {
              var scroll_offset = 0;
              if (!e.pageY) {
                scroll_offset = window.scrollY;
              }
              self.calculate_position(self.cache.active, self.get_cursor_position(e, 'y') + scroll_offset);
            } else {
              self.calculate_position(self.cache.active, self.get_cursor_position(e, 'x'));
            }
          }
        })
        .on('mouseup.fndtn.slider touchend.fndtn.slider pointerup.fndtn.slider', function (e) {
          self.remove_active_slider();
        })
        .on('change.fndtn.slider', function (e) {
          self.settings.on_change();
        });

      self.S(window)
        .on('resize.fndtn.slider', self.throttle(function (e) {
          self.reflow();
        }, 300));

      // update slider value as users change input value
      this.S('[' + this.attr_name() + ']').each(function () {
        var slider = $(this),
            handle = slider.children('.range-slider-handle')[0],
            settings = self.initialize_settings(handle);

        if (settings.display_selector != '') {
          $(settings.display_selector).each(function(){
            if (this.hasOwnProperty('value')) {
              $(this).change(function(){
                // is there a better way to do this?
                slider.foundation("slider", "set_value", $(this).val());
              });
            }
          });
        }
      });
    },

    get_cursor_position : function (e, xy) {
      var pageXY = 'page' + xy.toUpperCase(),
          clientXY = 'client' + xy.toUpperCase(),
          position;

      if (typeof e[pageXY] !== 'undefined') {
        position = e[pageXY];
      } else if (typeof e.originalEvent[clientXY] !== 'undefined') {
        position = e.originalEvent[clientXY];
      } else if (e.originalEvent.touches && e.originalEvent.touches[0] && typeof e.originalEvent.touches[0][clientXY] !== 'undefined') {
        position = e.originalEvent.touches[0][clientXY];
      } else if (e.currentPoint && typeof e.currentPoint[xy] !== 'undefined') {
        position = e.currentPoint[xy];
      }

      return position;
    },

    set_active_slider : function ($handle) {
      this.cache.active = $handle;
    },

    remove_active_slider : function () {
      this.cache.active = null;
    },

    calculate_position : function ($handle, cursor_x) {
      var self = this,
          settings = $.data($handle[0], 'settings'),
          handle_l = $.data($handle[0], 'handle_l'),
          handle_o = $.data($handle[0], 'handle_o'),
          bar_l = $.data($handle[0], 'bar_l'),
          bar_o = $.data($handle[0], 'bar_o');

      requestAnimationFrame(function () {
        var pct;

        if (Foundation.rtl && !settings.vertical) {
          pct = self.limit_to(((bar_o + bar_l - cursor_x) / bar_l), 0, 1);
        } else {
          pct = self.limit_to(((cursor_x - bar_o) / bar_l), 0, 1);
        }

        pct = settings.vertical ? 1 - pct : pct;

        var norm = self.normalized_value(pct, settings.start, settings.end, settings.step, settings.precision);

        self.set_ui($handle, norm);
      });
    },

    set_ui : function ($handle, value) {
      var settings = $.data($handle[0], 'settings'),
          handle_l = $.data($handle[0], 'handle_l'),
          bar_l = $.data($handle[0], 'bar_l'),
          norm_pct = this.normalized_percentage(value, settings.start, settings.end),
          handle_offset = norm_pct * (bar_l - handle_l) - 1,
          progress_bar_length = norm_pct * 100,
          $handle_parent = $handle.parent(),
          $hidden_inputs = $handle.parent().children('input[type=hidden]');

      if (Foundation.rtl && !settings.vertical) {
        handle_offset = -handle_offset;
      }

      handle_offset = settings.vertical ? -handle_offset + bar_l - handle_l + 1 : handle_offset;
      this.set_translate($handle, handle_offset, settings.vertical);

      if (settings.vertical) {
        $handle.siblings('.range-slider-active-segment').css('height', progress_bar_length + '%');
      } else {
        $handle.siblings('.range-slider-active-segment').css('width', progress_bar_length + '%');
      }

      $handle_parent.attr(this.attr_name(), value).trigger('change.fndtn.slider');

      $hidden_inputs.val(value);
      if (settings.trigger_input_change) {
          $hidden_inputs.trigger('change.fndtn.slider');
      }

      if (!$handle[0].hasAttribute('aria-valuemin')) {
        $handle.attr({
          'aria-valuemin' : settings.start,
          'aria-valuemax' : settings.end
        });
      }
      $handle.attr('aria-valuenow', value);

      if (settings.display_selector != '') {
        $(settings.display_selector).each(function () {
          if (this.hasAttribute('value')) {
            $(this).val(value);
          } else {
            $(this).text(value);
          }
        });
      }

    },

    normalized_percentage : function (val, start, end) {
      return Math.min(1, (val - start) / (end - start));
    },

    normalized_value : function (val, start, end, step, precision) {
      var range = end - start,
          point = val * range,
          mod = (point - (point % step)) / step,
          rem = point % step,
          round = ( rem >= step * 0.5 ? step : 0);
      return ((mod * step + round) + start).toFixed(precision);
    },

    set_translate : function (ele, offset, vertical) {
      if (vertical) {
        $(ele)
          .css('-webkit-transform', 'translateY(' + offset + 'px)')
          .css('-moz-transform', 'translateY(' + offset + 'px)')
          .css('-ms-transform', 'translateY(' + offset + 'px)')
          .css('-o-transform', 'translateY(' + offset + 'px)')
          .css('transform', 'translateY(' + offset + 'px)');
      } else {
        $(ele)
          .css('-webkit-transform', 'translateX(' + offset + 'px)')
          .css('-moz-transform', 'translateX(' + offset + 'px)')
          .css('-ms-transform', 'translateX(' + offset + 'px)')
          .css('-o-transform', 'translateX(' + offset + 'px)')
          .css('transform', 'translateX(' + offset + 'px)');
      }
    },

    limit_to : function (val, min, max) {
      return Math.min(Math.max(val, min), max);
    },

    initialize_settings : function (handle) {
      var settings = $.extend({}, this.settings, this.data_options($(handle).parent())),
          decimal_places_match_result;

      if (settings.precision === null) {
        decimal_places_match_result = ('' + settings.step).match(/\.([\d]*)/);
        settings.precision = decimal_places_match_result && decimal_places_match_result[1] ? decimal_places_match_result[1].length : 0;
      }

      if (settings.vertical) {
        $.data(handle, 'bar_o', $(handle).parent().offset().top);
        $.data(handle, 'bar_l', $(handle).parent().outerHeight());
        $.data(handle, 'handle_o', $(handle).offset().top);
        $.data(handle, 'handle_l', $(handle).outerHeight());
      } else {
        $.data(handle, 'bar_o', $(handle).parent().offset().left);
        $.data(handle, 'bar_l', $(handle).parent().outerWidth());
        $.data(handle, 'handle_o', $(handle).offset().left);
        $.data(handle, 'handle_l', $(handle).outerWidth());
      }

      $.data(handle, 'bar', $(handle).parent());
      return $.data(handle, 'settings', settings);
    },

    set_initial_position : function ($ele) {
      var settings = $.data($ele.children('.range-slider-handle')[0], 'settings'),
          initial = ((typeof settings.initial == 'number' && !isNaN(settings.initial)) ? settings.initial : Math.floor((settings.end - settings.start) * 0.5 / settings.step) * settings.step + settings.start),
          $handle = $ele.children('.range-slider-handle');
      this.set_ui($handle, initial);
    },

    set_value : function (value) {
      var self = this;
      $('[' + self.attr_name() + ']', this.scope).each(function () {
        $(this).attr(self.attr_name(), value);
      });
      if (!!$(this.scope).attr(self.attr_name())) {
        $(this.scope).attr(self.attr_name(), value);
      }
      self.reflow();
    },

    reflow : function () {
      var self = this;
      self.S('[' + this.attr_name() + ']').each(function () {
        var handle = $(this).children('.range-slider-handle')[0],
            val = $(this).attr(self.attr_name());
        self.initialize_settings(handle);

        if (val) {
          self.set_ui($(handle), parseFloat(val));
        } else {
          self.set_initial_position($(this));
        }
      });
    }
  };

}(jQuery, window, window.document));

define("foundation.slider", ["foundation.core"], function(){});

define('tabs/provision-tab/users/quota-widget',['require','foundation.slider','utils/quotas/quota-limits'],function(require) {
  require('foundation.slider');
  var QuotaLimits = require('utils/quotas/quota-limits');

  return {
    'setup': setup_provision_quota_widget,
    'reset': reset_provision_quota_widget,
    'retrieve': retrieve_provision_quota_widget
  }

  function setup_provision_quota_widget(context) {
    // Mode selector, for the 3 sliders
    $("select.provision_quota_select", context).on('change', function() {
        var row = $(this).closest(".row");

        switch ($(this).val()) {
          case "edit":
            $("div.provision_quota_edit", row).show();
            $("div.provision_quota_default", row).hide();
            $("div.provision_quota_unlimited", row).hide();

            $("input", row).change();

            break;

          case "default":
            $("div.provision_quota_edit", row).hide();
            $("div.provision_quota_default", row).show();
            $("div.provision_quota_unlimited", row).hide();

            break;

          case "unlimited":
            $("div.provision_quota_edit", row).hide();
            $("div.provision_quota_default", row).hide();
            $("div.provision_quota_unlimited", row).show();

            break;
        }

        return false;
      });

    var provision_rvms_quota_input = $(".provision_rvms_quota_input", context);

    $(".provision_rvms_quota_slider", context).on('change.fndtn.slider', function() {
        provision_rvms_quota_input.val($(this).attr('data-slider'))
      });

    provision_rvms_quota_input.change(function() {
      $(".provision_rvms_quota_slider", context).foundation(
                                          'slider', 'set_value', this.value);
    });

    var provision_cpu_quota_input = $(".provision_cpu_quota_input", context);

    $(".provision_cpu_quota_slider", context).on('change.fndtn.slider', function() {
        provision_cpu_quota_input.val($(this).attr('data-slider'))
      });

    provision_cpu_quota_input.change(function() {
      $(".provision_cpu_quota_slider", context).foundation(
                                          'slider', 'set_value', this.value);
    });

    var provision_memory_quota_input = $(".provision_memory_quota_input", context);
    var provision_memory_quota_tmp_input = $(".provision_memory_quota_tmp_input", context);

    var update_final_memory_input = function() {
        var value = provision_memory_quota_tmp_input.val();
        if (value > 0) {
          provision_memory_quota_input.val(Math.floor(value * 1024));
        } else {
          provision_memory_quota_input.val(value);
        }
      }

    $(".provision_memory_quota_slider", context).on('change.fndtn.slider', function() {
        provision_memory_quota_tmp_input.val($(this).attr('data-slider'));
        update_final_memory_input();
      });

    provision_memory_quota_tmp_input.change(function() {
      update_final_memory_input();
      $(".provision_memory_quota_slider", context).foundation(
                                          'slider', 'set_value', this.value);
    });

    $(".provision_rvms_quota_input", context).val('').change();
    $(".provision_memory_quota_input", context).val('').change();
    $(".provision_memory_quota_tmp_input", context).val('').change();
    $(".provision_cpu_quota_input", context).val('').change();
  }

  function reset_provision_quota_widget(context) {
    $("select.provision_quota_select", context).val('edit').change();

    $(".provision_rvms_quota_input", context).val('').change();
    $(".provision_memory_quota_input", context).val('').change();
    $(".provision_memory_quota_tmp_input", context).val('').change();
    $(".provision_cpu_quota_input", context).val('').change();
  }

  function retrieve_provision_quota_widget(context) {
    var retrieve_quota = function(select, input) {
      switch (select.val()) {
        case "edit":
          return input.val();
        case "default":
          return QuotaLimits.QUOTA_LIMIT_DEFAULT;
        case "unlimited":
          return QuotaLimits.QUOTA_LIMIT_UNLIMITED;
      }
    }

    var vms_limit = retrieve_quota(
          $(".provision_rvms_quota select.provision_quota_select", context),
          $(".provision_rvms_quota_input", context));

    var cpu_limit = retrieve_quota(
          $(".provision_cpu_quota select.provision_quota_select", context),
          $(".provision_cpu_quota_input", context));

    var mem_limit = retrieve_quota(
          $(".provision_memory_quota select.provision_quota_select", context),
          $(".provision_memory_quota_input", context));

    return {
      "VM" : {
        "SYSTEM_DISK_SIZE": QuotaLimits.QUOTA_LIMIT_DEFAULT,
        "VMS":    vms_limit,
        "MEMORY": mem_limit,
        "CPU":    cpu_limit
      }
    };
  }
});

;(function ($, window, document, undefined) {
  'use strict';

  Foundation.libs.alert = {
    name : 'alert',

    version : '5.5.2',

    settings : {
      callback : function () {}
    },

    init : function (scope, method, options) {
      this.bindings(method, options);
    },

    events : function () {
      var self = this,
          S = this.S;

      $(this.scope).off('.alert').on('click.fndtn.alert', '[' + this.attr_name() + '] .close', function (e) {
        var alertBox = S(this).closest('[' + self.attr_name() + ']'),
            settings = alertBox.data(self.attr_name(true) + '-init') || self.settings;

        e.preventDefault();
        if (Modernizr.csstransitions) {
          alertBox.addClass('alert-close');
          alertBox.on('transitionend webkitTransitionEnd oTransitionEnd', function (e) {
            S(this).trigger('close.fndtn.alert').remove();
            settings.callback();
          });
        } else {
          alertBox.fadeOut(300, function () {
            S(this).trigger('close.fndtn.alert').remove();
            settings.callback();
          });
        }
      });
    },

    reflow : function () {}
  };
}(jQuery, window, window.document));

define("foundation.alert", ["foundation.core"], function(){});

define('templates/helpers/isTabPanelEnabled',['require','hbs/handlebars','sunstone-config'],function(require) {
  var Handlebars = require('hbs/handlebars');
  var Config = require('sunstone-config');

  var isTabPanelEnabled = function(tabName, panel, options) {
    if (Config.isTabPanelEnabled(tabName, panel)) {
      return options.fn(this);
    } else {
      return options.inverse(this);
    }
  };

  Handlebars.registerHelper('isTabPanelEnabled', isTabPanelEnabled);

  return isTabPanelEnabled;
});

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/vms/info',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/isTabPanelEnabled'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "The main disk of the Virtual Machine will be saved in a new Image", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"left button small radius success provision_snapshot_button tip-top\">\n            <i class=\"fa fa-fw fa-lg fa-save\"/>\n          </a>\n          <a title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "You have to power-off the virtual machine first", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"left button small radius white provision_snapshot_button_disabled tip-top\" style=\"color: #999\">\n            <i class=\"fa fa-fw fa-lg fa-save\"/>\n          </a>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"text-center provision_info_vm_loading\">\n  <span class=\"fa-stack fa-5x\" style=\"color: #dfdfdf\"> <i class=\"fa fa-cloud fa-stack-2x\"></i> <i class=\"fa  fa-spinner fa-spin fa-stack-1x fa-inverse\"></i>\n  </span>\n  <br>\n  <br>\n  <span style=\"font-size: 18px; color: #999\"></span>\n</div>\n<div class=\"provision_info_vm\">\n  <div class=\"row\">\n    <div class=\"large-12 large-centered columns\">\n      <ul class=\"inline-list provision_action_icons\">\n        <li>\n          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Open a remote console in a new window", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"left button small radius provision_vnc_button tip-top\">\n            <i class=\"fa fa-fw fa-lg fa-desktop\"/></a>\n          <a title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "You have to boot the Virtual Machine first", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"left button small radius white provision_vnc_button_disabled tip-top\" style=\"color: #999\">\n            <i class=\"fa fa-fw fa-lg fa-desktop\"/></a>\n";
  stack1 = ((helpers.isTabPanelEnabled || (depth0 && depth0.isTabPanelEnabled) || helperMissing).call(depth0, "provision-tab", "templates", {"name":"isTabPanelEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "        </li>\n        <li class=\"right\">\n          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Delete", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"button small radius alert provision_delete_confirm_button tip-top right\">\n            <i class=\"fa fa-fw fa-lg fa-trash-o\"/>\n          </a>\n          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Delete", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"button small radius alert provision_shutdownhard_confirm_button tip-top right\">\n            <i class=\"fa fa-fw fa-lg fa-trash-o\"/>\n          </a>\n          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Power off", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"button small radius secondary provision_poweroff_confirm_button tip-top right\">\n            <i class=\"fa fa-fw fa-lg fa-power-off\"/>\n          </a>\n          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Power on", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"button small radius secondary provision_poweron_button tip-top right\">\n            <i class=\"fa fa-fw fa-lg fa-play\"/>\n          </a>\n          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Reboot", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"button small radius secondary provision_reboot_confirm_button tip-top right\">\n            <i class=\"fa fa-fw fa-lg fa-repeat\"/>\n          </a>\n        </li>\n      </ul>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"provision_confirm_action large-12 large-centered columns\"></div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-5 columns\">\n      <ul class=\"provision-pricing-table_vm_info provision-pricing-table\" style=\"border: 0px !important; margin-top: 20px;\"></ul>\n    </div>\n    <div class=\"large-6 columns right\">\n      <div class=\"row\">\n        <div class=\"large-6 medium-6  columns\">\n          <div class=\"row text-center\">\n            <div class=\"large-12 columns\">\n              <h4 class=\"subheader\" style=\"margin: 0px\">\n                <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU", {"name":"tr","hash":{},"data":data})))
    + "</small>\n              </h4>\n            </div>\n          </div>\n          <div class=\"row\">\n            <div class=\"large-12 columns\">\n              <div class=\"large-12 columns centered graph vm_cpu_graph\" style=\"height: 100px;\"></div>\n            </div>\n          </div>\n          <div class=\"row graph_legend\">\n            <div class=\"large-12 columns centered vm_cpu_legend\"></div>\n          </div>\n        </div>\n        <div class=\"large-6 medium-6 columns\">\n          <div class=\"row text-center\">\n            <div class=\"large-12 columns\">\n              <h4 class=\"subheader\" style=\"margin: 0px\">\n                <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Memory", {"name":"tr","hash":{},"data":data})))
    + "</small>\n              </h4>\n            </div>\n          </div>\n          <div class=\"row\">\n            <div class=\"large-12 columns\">\n              <div class=\"large-12 columns centered graph vm_memory_graph\" style=\"height: 100px;\"></div>\n            </div>\n          </div>\n          <div class=\"row graph_legend\">\n            <div class=\"large-12 columns centered vm_memory_legend\"></div>\n          </div>\n        </div>\n      </div>\n    </div>\n    <div class=\"large-12 columns\">\n      <br>\n      <div class=\"row\">\n      <div class=\"large-3 medium-6 columns\">\n        <div class=\"row text-center\">\n          <div class=\"large-12 columns\">\n            <h4 class=\"subheader\" style=\"margin: 0px\">\n              <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Net RX", {"name":"tr","hash":{},"data":data})))
    + "</small>\n            </h4>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <div class=\"large-12 columns centered graph vm_net_rx_graph\" style=\"height: 100px;\"></div>\n          </div>\n        </div>\n        <div class=\"row graph_legend\">\n          <div class=\"large-12 columns centered vm_net_rx_legend\"></div>\n        </div>\n      </div>\n      <div class=\"large-3 medium-6  columns\">\n        <div class=\"row text-center\">\n          <div class=\"large-12 columns\">\n            <h4 class=\"subheader\" style=\"margin: 0px\">\n              <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Net TX", {"name":"tr","hash":{},"data":data})))
    + "</small>\n            </h4>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <div class=\"large-12 columns centered graph vm_net_tx_graph\" style=\"height: 100px;\"></div>\n          </div>\n        </div>\n        <div class=\"row graph_legend\">\n          <div class=\"large-12 columns centered vm_net_tx_legend\"></div>\n        </div>\n      </div>\n      <div class=\"large-3 medium-6  columns\">\n        <div class=\"row text-center\">\n          <div class=\"large-12 columns\">\n            <h4 class=\"subheader\" style=\"margin: 0px\">\n              <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Net Download Speed", {"name":"tr","hash":{},"data":data})))
    + "</small>\n            </h4>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <div class=\"large-12 columns centered graph vm_net_rx_speed_graph\" style=\"height: 100px;\"></div>\n          </div>\n        </div>\n        <div class=\"row graph_legend\">\n          <div class=\"large-12 columns centered vm_net_rx_speed_legend\"></div>\n        </div>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <div class=\"row text-center\">\n          <div class=\"large-12 columns\">\n            <h4 class=\"subheader\" style=\"margin: 0px\">\n              <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Net Upload Speed", {"name":"tr","hash":{},"data":data})))
    + "</small>\n            </h4>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <div class=\"large-12 columns centered graph vm_net_tx_speed_graph\" style=\"height: 100px;\"></div>\n          </div>\n        </div>\n        <div class=\"row graph_legend\">\n          <div class=\"large-12 columns centered vm_net_tx_speed_legend\"></div>\n        </div>\n      </div>\n    </div>\n  </div>\n  <br>\n  <br></div>";
},"useData":true});
Handlebars.registerPartial('tabs/provision-tab/vms/info', t);
Handlebars.registerPartial('tabs/provision-tab/vms/info', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/vms/list',['hbs','hbs/handlebars', 'hbs!tabs/provision-tab/vms/info','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  return "active";
  },"3":function(depth0,helpers,partials,data) {
  return "style=\"display:none\"";
  },"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<dl class=\"accordion accordion_list provision_list_vms\" data-accordion>\n  <dd class=\"";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.active : stack1), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += " accordion-navigation\">\n    <a href=\"#provision_list_vm_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\" class=\"provision_back right only-not-active\">\n      <span class=\"button small radius\"> <i class=\"fa fa-fw fa-lg fa-th\"/><i class=\"fa fa-fw fa-lg fa-chevron-left\"/>\n      </span>\n    </a>\n    <h3 class=\"subheader\">\n      <span style=\"color: #777\">\n      </span>\n      <span class=\"provision_info_vm_name only-not-active\"></span>\n      <span href\"#\" class=\"right only-active button small radius secondary provision_vms_list_refresh_button\" \n                ";
  stack1 = helpers.unless.call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.refresh : stack1), {"name":"unless","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += " title=\" "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Refresh", {"name":"tr","hash":{},"data":data})))
    + "\">\n        <i class=\"fa fa-fw fa-lg fa-refresh\"/></span>\n      <span href\"#\" class=\"right only-not-active button small radius secondary provision_refresh_info\" \n                ";
  stack1 = helpers.unless.call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.refresh : stack1), {"name":"unless","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += " title=\" "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Refresh", {"name":"tr","hash":{},"data":data})))
    + "\">\n        <i class=\"fa fa-fw fa-lg fa-refresh\"/>\n      </span>\n      <span href\"#\" class=\"right only-active button small radius secondary provision_vms_list_filter_button\" \n                ";
  stack1 = helpers.unless.call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.filter : stack1), {"name":"unless","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += " title=\" "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filter by User", {"name":"tr","hash":{},"data":data})))
    + "\">\n        <i class=\"fa fa-fw fa-lg fa-filter\"/>\n      </span>\n      <span class=\"right only-active provision_list_vms_filter\" style=\"display: none\"></span>\n      <span>\n        <span href\"#\" class=\"right  only-active button small radius success provision_create_vm_button\" \n                ";
  stack1 = helpers.unless.call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.create : stack1), {"name":"unless","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += ">\n          <i class=\"fa fa-fw fa-lg fa-plus-square\"/>\n        </span>\n        <input type=\"search\" class=\"only-active provision_list_vms_search provision-search-input \" placeholder=\"Search VMs\"/>\n      </h3>\n      <div id=\"provision_list_vm_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\" class=\"content ";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.active : stack1), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "\">\n        <div class=\"row\">\n          <div class=\"large-12 large-centered columns\">\n            <table class=\"provision_vms_table\">\n              <thead class=\"hidden\">\n                <tr>\n                  <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                  <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                  <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                </tr>\n              </thead>\n              <tbody class=\"hidden\"></tbody>\n            </table>\n            <br></div>\n        </div>\n      </div>\n    </dd>\n    <dd class=\"accordion-navigation\">\n      <a class=\"provision_show_vm_accordion\" href=\"#provision_show_vm_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\"></a>\n      <div id=\"provision_show_vm_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\" class=\"content\">";
  stack1 = this.invokePartial(partials['tabs/provision-tab/vms/info'], '', 'tabs/provision-tab/vms/info', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n    </dd>\n  </dl>";
},"usePartial":true,"useData":true});
Handlebars.registerPartial('tabs/provision-tab/vms/list', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/provision-tab/vms/list',['require','foundation.alert','opennebula','opennebula/vm','utils/locale','sunstone-config','utils/notifier','utils/humanize','utils/resource-select','utils/graphs','hbs!./list'],function(require) {
  require('foundation.alert');
  var OpenNebula = require('opennebula');
  var OpenNebulaVM = require('opennebula/vm');
  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var Notifier = require('utils/notifier');
  var Humanize = require('utils/humanize');
  var ResourceSelect = require('utils/resource-select');
  var Graphs = require('utils/graphs');

  var TemplateVmsList = require('hbs!./list');

  var _accordionId = 0;

  return {
    'generate': generate_provision_vms_list,
    'show': show_provision_vm_list,
    'state': get_provision_vm_state
  };

  function show_provision_vm_list(timeout, context) {
    $(".section_content").hide();
    $(".provision_vms_list_section").fadeIn();

    $("dd:not(.active) .provision_back", $(".provision_vms_list_section")).trigger("click");
    $(".provision_vms_list_refresh_button", $(".provision_vms_list_section")).trigger("click");
  }

  function generate_provision_vms_list(context, opts) {
    context.off();
    context.html(html(opts));

    if (opts.data) {
      $(".provision_vms_table", context).data("opennebula", opts.data)
    }

    setup_provision_vms_list(context, opts);
    setup_info_vm(context);
  }

  function html(opts_arg) {
    opts = $.extend({
        title: Locale.tr("Virtual Machines"),
        refresh: true,
        create: true,
        filter: true
      }, opts_arg)

    _accordionId += 1;
    return TemplateVmsList({'accordionId': _accordionId, 'opts': opts});
  }

  function fill_provision_vms_datatable(datatable, item_list) {
    datatable.fnClearTable(true);
    if (item_list.length == 0) {
      datatable.html('<div class="text-center">' +
        '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
          '<i class="fa fa-cloud fa-stack-2x"></i>' +
          '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>' +
        '</span>' +
        '<br>' +
        '<br>' +
        '<span style="font-size: 18px; color: #999">' +
          Locale.tr("There are no Virtual Machines") +
        '</span>' +
        '<br>' +
        '<br>' +
        '</div>');
    } else {
      datatable.fnAddData(item_list);
    }
  }

  function update_provision_vms_datatable(datatable, timeout) {
    datatable.html('<div class="text-center">' +
      '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
        '<i class="fa fa-cloud fa-stack-2x"></i>' +
        '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>' +
      '</span>' +
      '<br>' +
      '<br>' +
      '<span style="font-size: 18px; color: #999">' +
      '</span>' +
      '</div>');

    var data = datatable.data('opennebula');
    if (data) {
      fill_provision_vms_datatable(datatable, data)
    } else {
      setTimeout(function() {
          OpenNebula.VM.list({
            timeout: true,
            success: function (request, item_list) {
              fill_provision_vms_datatable(datatable, item_list)
            },
            error: Notifier.onError
          })
        }, timeout);
    }
  }

  function setup_provision_vms_list(context, opts) {
    var provision_vms_datatable = $('.provision_vms_table', context).dataTable({
      "iDisplayLength": 6,
      "sDom" : '<"H">t<"F"lp>',
      "aLengthMenu": [[6, 12, 36, 72], [6, 12, 36, 72]],
      "aaSorting"  : [[0, "desc"]],
      "aoColumnDefs": [
          { "bVisible": false, "aTargets": ["all"]}
      ],
      "aoColumns": [
          { "mDataProp": "VM.ID" },
          { "mDataProp": "VM.NAME" },
          { "mDataProp": "VM.UID" }
      ],
      "fnPreDrawCallback": function (oSettings) {
        // create a thumbs container if it doesn't exist. put it in the dataTables_scrollbody div
        if (this.$('tr', {"filter": "applied"} ).length == 0) {
          this.html('<div class="text-center">'+
            '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
              '<i class="fa fa-cloud fa-stack-2x"></i>'+
              '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
            '</span>'+
            '<br>'+
            '<br>'+
            '<span style="font-size: 18px; color: #999">'+
              Locale.tr("There are no Virtual Machines")+
            '</span>'+
            '</div>');
        } else {
          $(".provision_vms_table", context).html('<ul class="provision_vms_ul large-block-grid-3 medium-block-grid-3 small-block-grid-1 text-center"></ul>');
        }

        return true;
      },
      "fnRowCallback": function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
        var data = aData.VM;
        var state = get_provision_vm_state(data);

        $(".provision_vms_ul", context).append('<li>'+
            '<ul class="provision-pricing-table" opennebula_id="'+data.ID+'" datatable_index="'+iDisplayIndexFull+'">'+
              '<li class="provision-title text-left">'+
                '<a class="provision_info_vm_button" style="color:#555" href="#">'+
                '<span class="'+ state.color +'-color" title="'+state.str+'">'+
                  '<i class="fa fa-fw fa-square"/> '+
                '</span>'+
                data.NAME + '</a>'+
              '</li>'+
              '<li class="provision-bullet-item text-left" >'+
                '<i class="fa fa-fw fa-lg fa-laptop"/> '+
                'x'+data.TEMPLATE.CPU+' - '+
                ((data.TEMPLATE.MEMORY > 1000) ?
                  (Math.floor(data.TEMPLATE.MEMORY/1024)+'GB') :
                  (data.TEMPLATE.MEMORY+'MB'))+
                ' - '+
                get_provision_disk_image(data) +
              '</li>'+
              '<li class="provision-bullet-item text-left" >'+
                '<span class="">'+
                  get_provision_ips(data) +
                '</span>'+
              '</li>'+
              '<li class="provision-bullet-item-last text-left" >'+
                '<span class="">'+
                  '<i class="fa fa-fw fa-lg fa-user"/> '+
                  data.UNAME+
                '</span>'+
                '<span class="right">'+
                  Humanize.prettyTimeAgo(data.STIME)+
                '</span>'+
              '</li>'+
            '</ul>'+
          '</li>');

        return nRow;
      }
    });

    $('.provision_list_vms_search', context).on('input',function(){
      provision_vms_datatable.fnFilter( $(this).val() );
    })

    context.on("click", ".provision_vms_list_refresh_button", function(){
      OpenNebula.Action.clear_cache("VM");
      update_provision_vms_datatable(provision_vms_datatable, 0);
      return false;
    });

    $(".provision_list_vms_filter", context).on("change", ".resource_list_select", function(){
      if ($(this).val() != "-2"){
        provision_vms_datatable.fnFilter("^" + $(this).val() + "$", 2, true, false);
      } else {
        provision_vms_datatable.fnFilter("", 2);
      }
    })

    ResourceSelect.insert(
      ".provision_list_vms_filter",
      context,
      "User",
      (opts.filter_expression ? opts.filter_expression : "-2"),
      false,
      '<option value="-2">'+Locale.tr("ALL")+'</option>',
      null,
      null,
      true,
      true);

    context.on("click", ".provision_vms_list_filter_button", function(){
      $(".provision_list_vms_filter", context).fadeIn();
      return false;
    });

    OpenNebula.Action.clear_cache("VM");
    update_provision_vms_datatable(provision_vms_datatable, 0);

    $(document).foundation();
  }

  function setup_info_vm(context) {
    function update_provision_vm_info(vm_id, context) {
      //var tempScrollTop = $(window).scrollTop();
      $(".provision_info_vm_name", context).text("");
      $(".provision_info_vm_loading", context).show();
      $(".provision_info_vm", context).css('visibility', 'hidden');

      OpenNebula.VM.show({
        data : {
          id: vm_id
        },
        error: Notifier.onError,
        success: function(request, response){
          var data = response.VM
          var state = get_provision_vm_state(data);

          switch (state.color) {
            case "deploying":
              $(".provision_reboot_confirm_button", context).hide();
              $(".provision_poweroff_confirm_button", context).hide();
              $(".provision_poweron_button", context).hide();
              $(".provision_delete_confirm_button", context).show();
              $(".provision_shutdownhard_confirm_button", context).hide();
              $(".provision_snapshot_button", context).hide();
              $(".provision_vnc_button", context).hide();
              $(".provision_snapshot_button_disabled", context).hide();
              $(".provision_vnc_button_disabled", context).hide();
              break;
            case "running":
              $(".provision_reboot_confirm_button", context).show();
              $(".provision_poweroff_confirm_button", context).show();
              $(".provision_poweron_button", context).hide();
              $(".provision_delete_confirm_button", context).hide();
              $(".provision_shutdownhard_confirm_button", context).show();
              $(".provision_snapshot_button", context).hide();
              $(".provision_vnc_button", context).show();
              $(".provision_snapshot_button_disabled", context).show();
              $(".provision_vnc_button_disabled", context).hide();
              break;
            case "off":
              $(".provision_reboot_confirm_button", context).hide();
              $(".provision_poweroff_confirm_button", context).hide();
              $(".provision_poweron_button", context).show();
              $(".provision_delete_confirm_button", context).show();
              $(".provision_shutdownhard_confirm_button", context).hide();
              $(".provision_snapshot_button", context).show();
              $(".provision_vnc_button", context).hide();
              $(".provision_snapshot_button_disabled", context).hide();
              $(".provision_vnc_button_disabled", context).show();
              break;
            case "powering_off":
            case "error":
              $(".provision_reboot_confirm_button", context).hide();
              $(".provision_poweroff_confirm_button", context).hide();
              $(".provision_poweron_button", context).hide();
              $(".provision_delete_confirm_button", context).show();
              $(".provision_shutdownhard_confirm_button", context).hide();
              $(".provision_snapshot_button", context).hide();
              $(".provision_vnc_button", context).hide();
              $(".provision_snapshot_button_disabled", context).hide();
              $(".provision_vnc_button_disabled", context).hide();
              break;
            default:
              color = 'secondary';
              $(".provision_reboot_confirm_button", context).hide();
              $(".provision_poweroff_confirm_button", context).hide();
              $(".provision_poweron_button", context).hide();
              $(".provision_delete_confirm_button", context).show();
              $(".provision_shutdownhard_confirm_button", context).hide();
              $(".provision_snapshot_button", context).hide();
              $(".provision_vnc_button", context).hide();
              $(".provision_snapshot_button_disabled", context).hide();
              $(".provision_vnc_button_disabled", context).hide();
              break;
          }

          if (!OpenNebula.VM.isVNCSupported(data) && !OpenNebula.VM.isSPICESupported(data)) {
              $(".provision_vnc_button", context).hide();
              $(".provision_vnc_button_disabled", context).hide();
          }

          $(".provision_info_vm", context).attr("vm_id", data.ID);
          $(".provision_info_vm", context).data("vm", data);

          $(".provision_info_vm_name", context).text(data.NAME);

          $(".provision-pricing-table_vm_info", context).html(
              '<li class="text-left provision-bullet-item">'+
                '<span class="'+ state.color +'-color">'+
                  '<i class="fa fa-fw fa-lg fa-square"/>&emsp;'+
                  state.str+
                '</span>'+
              '</li>'+
              '<li class="text-left provision-bullet-item">'+
                '<hr style="margin: 0px">'+
              '</li>'+
              '<li class="text-left provision-bullet-item" >'+
                '<span>'+
                  '<i class="fa fa-fw fa-lg fa-laptop"/>&emsp;'+
                  'x'+data.TEMPLATE.CPU+' - '+
                  ((data.TEMPLATE.MEMORY > 1000) ?
                    (Math.floor(data.TEMPLATE.MEMORY/1024)+'GB') :
                    (data.TEMPLATE.MEMORY+'MB'))+
                '</span>'+
                ' - '+
                '<span>'+
                  get_provision_disk_image(data) +
                '</span>'+
              '</li>'+
              '<li class="text-left provision-bullet-item" >'+
                '<span>'+
                  get_provision_ips(data) +
                '</span>'+
              '</li>'+
              //'<li  class="text-left provision-bullet-item" >'+
              //  '<span style="color: #afafaf;" style="font-size: 16px">'+
              //    "ID: " +
              //    data.ID+
              //  '</span>' +
              //'</li>'+
              '<li class="text-left provision-bullet-item">'+
                '<hr style="margin: 0px">'+
              '</li>'+
              '<li class="text-right provision-bullet-item">'+
                '<span class="left" style="color: #999;">'+
                  '<i class="fa fa-fw fa-lg fa-user"/>&emsp;'+
                  data.UNAME+
                '</span>'+
                '<span class="right" style="color: #999;">'+
                  '<i class="fa fa-fw fa-lg fa-clock-o"/>&emsp;'+
                  Humanize.prettyTimeAgo(data.STIME)+
                  ' - '+
                  'ID: '+
                  data.ID+
                '</span>'+
              '</li>');

          $(".provision_confirm_action:first", context).html("");

          $(".provision_info_vm", context).css('visibility', 'visible');
          $(".provision_info_vm_loading", context).hide();

          //$(window).scrollTop(tempScrollTop);

          OpenNebula.VM.monitor({
            data : {
              timeout: true,
              id: data.ID,
              monitor: {
                monitor_resources : "MONITORING/CPU,MONITORING/MEMORY,MONITORING/NETTX,MONITORING/NETRX"
              }
            },
            success: function(request, response){
              var vm_graphs = [
                  {
                      monitor_resources : "MONITORING/CPU",
                      labels : "Real CPU",
                      humanize_figures : false,
                      div_graph : $(".vm_cpu_graph", context)
                  },
                  {
                      monitor_resources : "MONITORING/MEMORY",
                      labels : "Real MEM",
                      humanize_figures : true,
                      div_graph : $(".vm_memory_graph", context)
                  },
                  {
                      labels : "Network reception",
                      monitor_resources : "MONITORING/NETRX",
                      humanize_figures : true,
                      convert_from_bytes : true,
                      div_graph : $(".vm_net_rx_graph", context)
                  },
                  {
                      labels : "Network transmission",
                      monitor_resources : "MONITORING/NETTX",
                      humanize_figures : true,
                      convert_from_bytes : true,
                      div_graph : $(".vm_net_tx_graph", context)
                  },
                  {
                      labels : "Network reception speed",
                      monitor_resources : "MONITORING/NETRX",
                      humanize_figures : true,
                      convert_from_bytes : true,
                      y_sufix : "B/s",
                      derivative : true,
                      div_graph : $(".vm_net_rx_speed_graph", context)
                  },
                  {
                      labels : "Network transmission speed",
                      monitor_resources : "MONITORING/NETTX",
                      humanize_figures : true,
                      convert_from_bytes : true,
                      y_sufix : "B/s",
                      derivative : true,
                      div_graph : $(".vm_net_tx_speed_graph", context)
                  }
              ];

              for(var i=0; i<vm_graphs.length; i++) {
                  Graphs.plot(
                      response,
                      vm_graphs[i]
                  );
              }
            }
          })
        }
      })
    }

    if (Config.isTabPanelEnabled("provision-tab", "templates")) {
      context.on("click", ".provision_snapshot_button", function(){
        $(".provision_confirm_action:first", context).html(
          '<div data-alert class="alert-box secondary radius">'+
            '<div class="row">'+
              '<div class="large-12 columns">'+
                '<span style="font-size: 14px; line-height: 20px">'+
                  Locale.tr("This Virtual Machine will be saved in a new Template. Only the main disk will be preserved!")+
                '<br>'+
                  Locale.tr("You can then create a new Virtual Machine using this Template.")+
                '</span>'+
              '</div>'+
            '</div>'+
            '<br>'+
            '<div class="row">'+
              '<div class="large-11 large-centered columns">'+
                '<input type="text" class="provision_snapshot_name" placeholder="'+Locale.tr("Template Name")+'" style="height: 40px !important; font-size: 16px; padding: 0.5rem  !important; margin: 0px"/>'+
              '</div>'+
            '</div>'+
            '<br>'+
            '<div class="row">'+
              '<div class="large-11 large-centered columns">'+
                '<a href"#" class="provision_snapshot_create_button success button large-12 radius right">'+Locale.tr("Save Virtual Machine to Template")+'</a>'+
              '</div>'+
            '</div>'+
            '<a href="#" class="close" style="top: 20px">&times;</a>'+
          '</div>');
      });

      context.on("click", ".provision_snapshot_create_button", function(){
        var button = $(this);
        button.attr("disabled", "disabled");
        var context = $(".provision_info_vm[vm_id]");

        var vm_id = context.attr("vm_id");
        var template_name = $('.provision_snapshot_name', context).val();

        OpenNebula.VM.save_as_template({
          data : {
            id: vm_id,
            extra_param: {
              name : template_name
            }
          },
          timeout: false,
          success: function(request, response){
            OpenNebula.Action.clear_cache("VMTEMPLATE");
            Notifier.notifyMessage(Locale.tr("VM Template") + ' ' + request.request.data[0][1].name + ' ' + Locale.tr("saved successfully"))
            update_provision_vm_info(vm_id, context);
            button.removeAttr("disabled");
          },
          error: function(request, response){
            if(response.error.http_status == 0){   // Failed due to cloning template taking too long
                OpenNebula.Action.clear_cache("VMTEMPLATE");
                update_provision_vm_info(vm_id, context);
                Notifier.notifyMessage(Locale.tr("VM cloning in the background. The Template will appear as soon as it is ready, and the VM unlocked."));
            } else {
                Notifier.onError(request, response);
            }
            button.removeAttr("disabled");
          }
        })

        return false;
      });
    }

    context.on("click", ".provision_delete_confirm_button", function(){
      $(".provision_confirm_action:first", context).html(
        '<div data-alert class="alert-box secondary radius">'+
          '<div class="row">'+
          '<div class="large-9 columns">'+
            '<span style="font-size: 14px; line-height: 20px">'+
              Locale.tr("Be careful, this action will inmediately destroy your Virtual Machine")+
              '<br>'+
              Locale.tr("All the information will be lost!")+
            '</span>'+
          '</div>'+
          '<div class="large-3 columns">'+
            '<a href"#" class="provision_delete_button alert button large-12 radius right" style="margin-right: 15px">'+Locale.tr("Delete")+'</a>'+
          '</div>'+
          '</div>'+
          '<a href="#" class="close">&times;</a>'+
        '</div>');
    });

    context.on("click", ".provision_shutdownhard_confirm_button", function(){
      $(".provision_confirm_action:first", context).html(
        '<div data-alert class="alert-box secondary radius">'+
          '<div class="row">'+
          '<div class="large-9 columns">'+
            '<span style="font-size: 14px; line-height: 20px">'+
              Locale.tr("Be careful, this action will inmediately destroy your Virtual Machine")+
              '<br>'+
              Locale.tr("All the information will be lost!")+
            '</span>'+
          '</div>'+
          '<div class="large-3 columns">'+
            '<a href"#" class="provision_shutdownhard_button alert button large-12 radius right" style="margin-right: 15px">'+Locale.tr("Delete")+'</a>'+
          '</div>'+
          '</div>'+
          '<a href="#" class="close" style="top: 20px">&times;</a>'+
        '</div>');
    });

    context.on("click", ".provision_poweroff_confirm_button", function(){
      $(".provision_confirm_action:first", context).html(
        '<div data-alert class="alert-box secondary radius">'+
          '<div class="row">'+
          '<div class="large-11 columns">'+
            '<span style="font-size: 14px; line-height: 20px">'+
              Locale.tr("This action will power off this Virtual Machine. The Virtual Machine will remain in the poweroff state, and can be powered on later")+
              '<br>'+
              '<br>'+
              Locale.tr("You can send the power off signal to the Virtual Machine (this is equivalent to execute the command from the console). If that doesn't effect your Virtual Machine, try to Power off the machine (this is equivalent to pressing the power off button in a physical computer).")+
            '</span>'+
          '</div>'+
          '</div>'+
          '<br>'+
          '<div class="row">'+
          '<div class="large-12 columns">'+
            '<a href"#" class="provision_poweroff_button button radius right" style="margin-right: 15px">'+Locale.tr("Power off")+'</a>'+
            '<label class="left" style="margin-left: 25px">'+
              '<input type="radio" name="provision_poweroff_radio" value="poweroff_hard" class="provision_poweroff_hard_radio">'+
              ' <i class="fa fa-fw fa-bolt"/> '+Locale.tr("Power off the machine")+
            '</label>'+
            '<label class="left" style="margin-left: 25px">'+
              '<input type="radio" name="provision_poweroff_radio" value="poweroff" class="provision_poweroff_radio" checked>'+
              ' <i class="fa fa-fw fa-power-off"/> '+Locale.tr("Send the power off signal")+
            '</label>'+
          '</div>'+
          '</div>'+
          '<a href="#" class="close" style="top: 20px">&times;</a>'+
        '</div>');
    });

    context.on("click", ".provision_reboot_confirm_button", function(){
      $(".provision_confirm_action:first", context).html(
        '<div data-alert class="alert-box secondary radius">'+
          '<div class="row">'+
          '<div class="large-11 columns">'+
            '<span style="font-size: 14px; line-height: 20px">'+
              Locale.tr("This action will reboot this Virtual Machine.")+
              '<br>'+
              '<br>'+
              Locale.tr("You can send the reboot signal to the Virtual Machine (this is equivalent to execute the reboot command form the console). If that doesn't effect your Virtual Machine, try to Reboot the machine (this is equivalent to pressing the reset button a physical computer).")+
            '</span>'+
          '</div>'+
          '</div>'+
          '<br>'+
          '<div class="row">'+
          '<div class="large-12 columns">'+
            '<a href"#" class="provision_reboot_button button radius right" style="margin-right: 15px">'+Locale.tr("Reboot")+'</a>'+
            '<label class="left" style="margin-left: 25px">'+
              '<input type="radio" name="provision_reboot_radio" value="reset" class="provision_reboot_hard_radio">'+
              ' <i class="fa fa-fw fa-bolt"/> '+Locale.tr("Reboot the machine")+
            '</label>'+
            '<label class="left" style="margin-left: 25px">'+
              '<input type="radio" name="provision_reboot_radio" value="reboot" class="provision_reboot_radio" checked>'+
              ' <i class="fa fa-fw fa-power-off"/> '+Locale.tr("Send the reboot signal")+
            '</label>'+
          '</div>'+
          '</div>'+
          '<a href="#" class="close" style="top: 20px">&times;</a>'+
        '</div>');
    });

    context.on("click", ".provision_delete_button", function(){
      var button = $(this);
      button.attr("disabled", "disabled");
      var vm_id = $(".provision_info_vm", context).attr("vm_id");

      OpenNebula.VM.del({
        data : {
          id: vm_id
        },
        success: function(request, response){
          $(".provision_back", context).click();
          $(".provision_vms_list_refresh_button", context).click();
          button.removeAttr("disabled");
        },
        error: function(request, response){
          Notifier.onError(request, response);
          button.removeAttr("disabled");
        }
      })

      return false;
    });

    context.on("click", ".provision_shutdownhard_button", function(){
      var button = $(this);
      button.attr("disabled", "disabled");
      var vm_id = $(".provision_info_vm", context).attr("vm_id");

      OpenNebula.VM.shutdown_hard({
        data : {
          id: vm_id
        },
        success: function(request, response){
          $(".provision_back", context).click();
          $(".provision_vms_list_refresh_button", context).click();
          button.removeAttr("disabled");
        },
        error: function(request, response){
          Notifier.onError(request, response);
          button.removeAttr("disabled");
        }
      })

      return false;
    });

    context.on("click", ".provision_poweroff_button", function(){
      var button = $(this);
      button.attr("disabled", "disabled");
      var vm_id = $(".provision_info_vm", context).attr("vm_id");
      var poweroff_action = $('input[name=provision_poweroff_radio]:checked').val()

      OpenNebula.VM[poweroff_action]({
        data : {
          id: vm_id
        },
        success: function(request, response){
          update_provision_vm_info(vm_id, context);
          button.removeAttr("disabled");
        },
        error: function(request, response){
          Notifier.onError(request, response);
          button.removeAttr("disabled");
        }
      })

      return false;
    });

    context.on("click", ".provision_reboot_button", function(){
      var button = $(this);
      button.attr("disabled", "disabled");

      var vm_id = $(".provision_info_vm", context).attr("vm_id");
      var reboot_action = $('input[name=provision_reboot_radio]:checked').val()

      OpenNebula.VM[reboot_action]({
        data : {
          id: vm_id
        },
        success: function(request, response){
          update_provision_vm_info(vm_id, context);
          button.removeAttr("disabled");
        },
        error: function(request, response){
          Notifier.onError(request, response);
          button.removeAttr("disabled");
        }
      })

      return false;
    });

    context.on("click", ".provision_poweron_button", function(){
      var button = $(this);
      button.attr("disabled", "disabled");
      var vm_id = $(".provision_info_vm", context).attr("vm_id");

      OpenNebula.VM.resume({
        data : {
          id: vm_id
        },
        success: function(request, response){
          update_provision_vm_info(vm_id, context);
          button.removeAttr("disabled");
        },
        error: function(request, response){
          Notifier.onError(request, response);
          button.removeAttr("disabled");
        }
      })

      return false;
    });

    context.on("click", ".provision_vnc_button", function(){
      var button = $(this);
      button.attr("disabled", "disabled");
      var vm_id = $(".provision_info_vm", context).attr("vm_id");
      var vm_data = $(".provision_info_vm", context).data("vm");

      OpenNebula.VM.vnc({
        data : {
          id: vm_id
        },
        success: function(request, response){
          if (OpenNebula.VM.isVNCSupported(vm_data)) {
            var proxy_host = window.location.hostname;
            var proxy_port = config['system_config']['vnc_proxy_port'];
            var pw = response["password"];
            var token = response["token"];
            var vm_name = response["vm_name"];
            var path = '?token='+token;

            var url = "vnc?";
            url += "host=" + proxy_host;
            url += "&port=" + proxy_port;
            url += "&token=" + token;
            url += "&password=" + pw;
            url += "&encrypt=" + config['user_config']['vnc_wss'];
            url += "&title=" + vm_name;

            window.open(url, '', '_blank');
            button.removeAttr("disabled");
          } else if (OpenNebula.VM.isSPICESupported(vm_data)) {
            var host, port, password, scheme = "ws://", uri, token, vm_name;

            if (config['user_config']['vnc_wss'] == "yes") {
                scheme = "wss://";
            }

            host = window.location.hostname;
            port = config['system_config']['vnc_proxy_port'];
            password = response["password"];
            token = response["token"];
            vm_name = response["vm_name"];

            uri = scheme + host + ":" + port + "?token=" + token;

            var url = "spice?";
            url += "host=" + host;
            url += "&port=" + port;
            url += "&token=" + token;
            url += "&password=" + password;
            url += "&encrypt=" + config['user_config']['vnc_wss'];
            url += "&title=" + vm_name;

            window.open(url, '', '_blank');
            button.removeAttr("disabled");
          } else {
            Notifier.notifyError("The remote console is not enabled for this VM")
          }
        },
        error: function(request, response){
          Notifier.onError(request, response);
          button.removeAttr("disabled");
        }
      })

      return false;
    });

    context.on("click", ".provision_refresh_info", function(){
      var vm_id = $(".provision_info_vm", context).attr("vm_id");
      update_provision_vm_info(vm_id, context);
      return false;
    });

    //
    // Info VM
    //

    $(".provision_list_vms", context).on("click", ".provision_info_vm_button", function(){
      $("a.provision_show_vm_accordion", context).trigger("click");
      // TODO loading

      var vm_id = $(this).parents(".provision-pricing-table").attr("opennebula_id")
      update_provision_vm_info(vm_id, context);
      return false;
    })
  }


  // @params
  //    data: and VM object
  //      Example: data.ID
  // @returns and object containing the following properties
  //    color: css class for this state.
  //      color + '-color' font color class
  //      color + '-bg' background class
  //    str: user friendly state string
  function get_provision_vm_state(data) {
    var state = parseInt(data.STATE);
    var state_color;
    var state_str;

    switch (state) {
      case OpenNebulaVM.STATES.INIT:
      case OpenNebulaVM.STATES.PENDING:
      case OpenNebulaVM.STATES.HOLD:
        state_color = 'deploying';
        state_str = Locale.tr("DEPLOYING") + " (1/3)";
        break;
      case OpenNebulaVM.STATES.ACTIVE:
        var lcm_state = parseInt(data.LCM_STATE);

        switch (lcm_state) {
          case OpenNebulaVM.LCM_STATES.LCM_INIT:
            state_color = 'deploying';
            state_str = Locale.tr("DEPLOYING") + " (1/3)";
            break;
          case OpenNebulaVM.LCM_STATES.PROLOG:
          case OpenNebulaVM.LCM_STATES.PROLOG_RESUME:
          case OpenNebulaVM.LCM_STATES.PROLOG_UNDEPLOY:
            state_color = 'deploying';
            state_str = Locale.tr("DEPLOYING") + " (2/3)";
            break;
          case OpenNebulaVM.LCM_STATES.BOOT:
          case OpenNebulaVM.LCM_STATES.BOOT_UNKNOWN:
          case OpenNebulaVM.LCM_STATES.BOOT_POWEROFF:
          case OpenNebulaVM.LCM_STATES.BOOT_SUSPENDED:
          case OpenNebulaVM.LCM_STATES.BOOT_STOPPED:
          case OpenNebulaVM.LCM_STATES.BOOT_UNDEPLOY:
            state_color = 'deploying';
            state_str = Locale.tr("DEPLOYING") + " (3/3)";
            break;
          case OpenNebulaVM.LCM_STATES.RUNNING:
          case OpenNebulaVM.LCM_STATES.HOTPLUG_SNAPSHOT:
          case OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_POWEROFF:
          case OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_REVERT_POWEROFF:
          case OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_DELETE_POWEROFF:
          case OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_SUSPENDED:
          case OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_REVERT_SUSPENDED:
          case OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_DELETE_SUSPENDED:
          case OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT:
          case OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_REVERT:
          case OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_DELETE:
          case OpenNebulaVM.LCM_STATES.MIGRATE:
          case OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE:
          case OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_POWEROFF:
          case OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_SUSPEND:
            state_color = 'running';
            state_str = Locale.tr("RUNNING");
            break;
          case OpenNebulaVM.LCM_STATES.HOTPLUG:
          case OpenNebulaVM.LCM_STATES.HOTPLUG_NIC:
          case OpenNebulaVM.LCM_STATES.HOTPLUG_SAVEAS:
          case OpenNebulaVM.LCM_STATES.HOTPLUG_SAVEAS_POWEROFF:
          case OpenNebulaVM.LCM_STATES.HOTPLUG_SAVEAS_SUSPENDED:
          case OpenNebulaVM.LCM_STATES.HOTPLUG_PROLOG_POWEROFF:
          case OpenNebulaVM.LCM_STATES.HOTPLUG_EPILOG_POWEROFF:
            state_color = 'deploying';
            state_str = Locale.tr("SAVING IMAGE");
            break;
          case OpenNebulaVM.LCM_STATES.FAILURE:
          case OpenNebulaVM.LCM_STATES.BOOT_FAILURE:
          case OpenNebulaVM.LCM_STATES.BOOT_MIGRATE_FAILURE:
          case OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_FAILURE:
          case OpenNebulaVM.LCM_STATES.PROLOG_FAILURE:
          case OpenNebulaVM.LCM_STATES.EPILOG_FAILURE:
          case OpenNebulaVM.LCM_STATES.EPILOG_STOP_FAILURE:
          case OpenNebulaVM.LCM_STATES.EPILOG_UNDEPLOY_FAILURE:
          case OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_POWEROFF_FAILURE:
          case OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_SUSPEND_FAILURE:
          case OpenNebulaVM.LCM_STATES.BOOT_UNDEPLOY_FAILURE:
          case OpenNebulaVM.LCM_STATES.BOOT_STOPPED_FAILURE:
          case OpenNebulaVM.LCM_STATES.PROLOG_RESUME_FAILURE:
          case OpenNebulaVM.LCM_STATES.PROLOG_UNDEPLOY_FAILURE:
            state_color = 'error';
            state_str = Locale.tr("ERROR");
            break;
          case OpenNebulaVM.LCM_STATES.SAVE_STOP:
          case OpenNebulaVM.LCM_STATES.SAVE_SUSPEND:
          case OpenNebulaVM.LCM_STATES.SAVE_MIGRATE:
          case OpenNebulaVM.LCM_STATES.EPILOG_STOP:
          case OpenNebulaVM.LCM_STATES.EPILOG:
          case OpenNebulaVM.LCM_STATES.EPILOG_UNDEPLOY:
          case OpenNebulaVM.LCM_STATES.SHUTDOWN:
          case OpenNebulaVM.LCM_STATES.CANCEL:
          case OpenNebulaVM.LCM_STATES.SHUTDOWN_POWEROFF:
          case OpenNebulaVM.LCM_STATES.SHUTDOWN_UNDEPLOY:
          case OpenNebulaVM.LCM_STATES.CLEANUP_RESUBMIT:
          case OpenNebulaVM.LCM_STATES.CLEANUP_DELETE:
            state_color = 'powering_off';
            state_str = Locale.tr("POWERING OFF");
            break;
          case OpenNebulaVM.LCM_STATES.UNKNOWN:
            state_color = 'powering_off';
            state_str = Locale.tr("UNKNOWN");
            break;
          default:
            state_color = 'powering_off';
            state_str = Locale.tr("UNKNOWN");
            break;
        }

        break;
      case OpenNebulaVM.STATES.STOPPED:
      case OpenNebulaVM.STATES.SUSPENDED:
      case OpenNebulaVM.STATES.POWEROFF:
      case OpenNebulaVM.STATES.UNDEPLOYED:
      case OpenNebulaVM.STATES.DONE:
        state_color = 'off';
        state_str = Locale.tr("OFF");

        break;
      default:
        state_color = 'powering_off';
        state_str = Locale.tr("UNKNOWN");
        break;
    }

    return {
      color: state_color,
      str: state_str
    }
  }

  function get_provision_disk_image(data) {
    var disks = []
    if ($.isArray(data.TEMPLATE.DISK))
        disks = data.TEMPLATE.DISK
    else if (!$.isEmptyObject(data.TEMPLATE.DISK))
        disks = [data.TEMPLATE.DISK]

    if (disks.length > 0) {
      return disks[0].IMAGE;
    } else {
      return '';
    }
  }

  function get_provision_ips(data) {
    return '<i class="fa fa-fw fa-lg fa-globe"></i> ' + OpenNebula.VM.ipsStr(data, ', ');
  }

  // @params
  //    data: and IMAGE object
  //      Example: data.ID
  // @returns and object containing the following properties
  //    color: css class for this state.
  //      color + '-color' font color class
  //      color + '-bg' background class
  //    str: user friendly state string
  function get_provision_image_state(data) {
    var state = OpenNebula.Image.stateStr(data.STATE);
    var state_color;
    var state_str;

    switch (state) {
      case "READY":
      case "USED":
        state_color = 'running';
        state_str = Locale.tr("READY");
        break;
      case "DISABLED":
      case "USED_PERS":
        state_color = 'off';
        state_str = Locale.tr("OFF");
        break;
      case "LOCKED":
      case "CLONE":
      case "INIT":
        state_color = 'deploying';
        state_str = Locale.tr("DEPLOYING") + " (1/3)";
        break;
      case "ERROR":
        state_color = 'error';
        state_str = Locale.tr("ERROR");
        break;
      case "DELETE":
        state_color = 'error';
        state_str = Locale.tr("DELETING");
        break;
      default:
        state_color = 'powering_off';
        state_str = Locale.tr("UNKNOWN");
        break;
    }

    return {
      color: state_color,
      str: state_str
    }
  }
});


/* START_TEMPLATE */
define('hbs!tabs/provision-tab/templates/list',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  return "active";
  },"3":function(depth0,helpers,partials,data) {
  return "style=\"display:none\"";
  },"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<dl class=\"accordion accordion_list provision_list_templates\" data-accordion=\"dfsaf\">\n  <dd class=\"";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.active : stack1), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += " accordion-navigation\">\n    <a class=\"right only-not-active\" href=\"#provision_list_template_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\">\n      <span class=\"button small radius\"> <i class=\"fa fa-fw fa-lg fa-th\"/><i class=\"fa fa-fw fa-lg fa-chevron-left\"/>\n      </span>\n    </a>\n    <h3 class=\"subheader\">\n      <span href\"#\" class=\"right only-active button small radius secondary provision_templates_list_refresh_button\" \n          ";
  stack1 = helpers.unless.call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.refresh : stack1), {"name":"unless","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += " title=\" "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Refresh", {"name":"tr","hash":{},"data":data})))
    + "\">\n        <i class=\"fa fa-fw fa-lg fa-refresh\"/></span>\n      <span href\"#\" class=\"right only-active button small radius secondary provision_templates_list_filter_button\" \n          ";
  stack1 = helpers.unless.call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.filter : stack1), {"name":"unless","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += " title=\" "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filter by User", {"name":"tr","hash":{},"data":data})))
    + "\">\n        <i class=\"fa fa-fw fa-lg fa-filter\"/>\n      </span>\n      <span class=\"right only-active provision_list_templates_filter\" style=\"display: none\"></span>\n      <input type=\"search\" class=\"provision_list_templates_search only-active provision-search-input\" placeholder=\"Search Templates\"/>\n    </h3>\n    <div id=\"provision_list_template_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\" class=\"content ";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.active : stack1), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\">\n      <div class=\"row\">\n        <div class=\"provision_confirm_delete_template_div large-12 large-centered columns\"></div>\n      </div>\n      <div class=\"row\">\n        <div class=\"large-12 large-centered columns\">\n          <table class=\"provision_templates_table\">\n            <thead class=\"hidden\">\n              <tr>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Saved", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              </tr>\n            </thead>\n            <tbody class=\"hidden\"></tbody>\n          </table>\n          <br></div>\n      </div>\n    </div>\n  </dd>\n  <dd class=\"accordion-navigation\">\n    <a class=\"provision_show_template_accordion\" href=\"#provision_show_template_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\"></a>\n    <div id=\"provision_show_template_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\" class=\"content\"></div>\n  </dd>\n</dl>";
},"useData":true});
Handlebars.registerPartial('tabs/provision-tab/templates/list', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/provision-tab/templates/list',['require','foundation.alert','opennebula','utils/locale','sunstone-config','utils/notifier','utils/humanize','utils/resource-select','hbs!./list'],function(require) {
  require('foundation.alert');
  var OpenNebula = require('opennebula');
  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var Notifier = require('utils/notifier');
  var Humanize = require('utils/humanize');
  var ResourceSelect = require('utils/resource-select');

  var TemplateTemplatesList = require('hbs!./list');

  var _accordionId = 0;

  return {
    'generate': generate_provision_templates_list,
    'show': show_provision_template_list,
    'updateDatatable': update_provision_templates_datatable,
  };

  function show_provision_template_list(timeout) {
    $(".section_content").hide();
    $(".provision_templates_list_section").fadeIn();

    $(".provision_templates_list_refresh_button", $(".provision_templates_list_section")).trigger("click");
  }

  function generate_provision_templates_list(context, opts) {
    context.off();
    context.html(html(opts));
    setup_provision_templates_list(context, opts);
  }

  function html(opts_arg){
    opts = $.extend({
        title: Locale.tr("Saved Templates"),
        refresh: true,
        create: true,
        active: true,
        filter: true
      },opts_arg)

    _accordionId += 1;
    return TemplateTemplatesList({'accordionId': _accordionId, 'opts': opts});
  }

  function update_provision_templates_datatable(datatable, timeout) {
    datatable.html('<div class="text-center">'+
      '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
        '<i class="fa fa-cloud fa-stack-2x"></i>'+
        '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>'+
      '</span>'+
      '<br>'+
      '<br>'+
      '<span style="font-size: 18px; color: #999">'+
      '</span>'+
      '</div>');

    setTimeout( function(){
      OpenNebula.Template.list({
        timeout: true,
        success: function (request, item_list){
          datatable.fnClearTable(true);
          if (item_list.length == 0) {
            datatable.html('<div class="text-center">'+
              '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
                '<i class="fa fa-cloud fa-stack-2x"></i>'+
                '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
              '</span>'+
              '<br>'+
              '<br>'+
              '<span style="font-size: 18px; color: #999">'+
                Locale.tr("There are no templates available")+
              '</span>'+
              '</div>');
          } else {
            datatable.fnAddData(item_list);
          }
        },
        error: Notifier.onError
      });
    }, timeout);
  }

  function setup_provision_templates_list(context, opts) {
    var provision_templates_datatable = $('.provision_templates_table', context).dataTable({
      "iDisplayLength": 8,
      "sDom" : '<"H">t<"F"lp>',
      "aLengthMenu": [[6, 12, 36, 72], [6, 12, 36, 72]],
      "aaSorting"  : [[0, "desc"]],
      "aoColumnDefs": [
          { "bVisible": false, "aTargets": ["all"]}
      ],
      "aoColumns": [
          { "mDataProp": "VMTEMPLATE.ID" },
          { "mDataProp": "VMTEMPLATE.NAME" },
          { "mDataProp": "VMTEMPLATE.TEMPLATE.SAVED_TEMPLATE_ID", "sDefaultContent" : "-"  },
          { "mDataProp": "VMTEMPLATE.UID" }
      ],
      "fnPreDrawCallback": function (oSettings) {
        // create a thumbs container if it doesn't exist. put it in the dataTables_scrollbody div
        if (this.$('tr', {"filter": "applied"} ).length == 0) {
          this.html('<div class="text-center">'+
            '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
              '<i class="fa fa-cloud fa-stack-2x"></i>'+
              '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
            '</span>'+
            '<br>'+
            '<br>'+
            '<span style="font-size: 18px; color: #999">'+
              Locale.tr("There are no saved templates available")+
              '<br>'+
              Locale.tr("Create a template by saving a running Virtual Machine")+
            '</span>'+
            '</div>');
        } else {
          $(".provision_templates_table", context).html('<ul class="provision_templates_ul large-block-grid-3 medium-block-grid-3 small-block-grid-1 text-center"></ul>');
        }
        return true;
      },
      "fnRowCallback": function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
        var data = aData.VMTEMPLATE;
        var actions_html = "";
        if (Config.isTabActionEnabled("provision-tab", "Template.chmod")) {
          if (data.UID == config['user_id']) {

            if (data.PERMISSIONS.GROUP_U == "1") {
              actions_html += '<a class="provision_confirm_unshare_template_button left" title="'+ Locale.tr("Unshare")+'" style="color:#555" href="#"><i class="fa fa-fw fa-lg fa-ban only-on-hover"/></a>';
              actions_html += '<span style="font-size:12px; color: #777">' + Locale.tr("SHARED") + '</span>';
            } else {
              actions_html += '<a class="provision_confirm_chmod_template_button left" title="'+ Locale.tr("Share")+'" style="color:#555" href="#"><i class="fa fa-fw fa-lg fa-share-alt only-on-hover"/></a>';
            }
          }
        }

        if (Config.isTabActionEnabled("provision-tab", "Template.delete")) {
          actions_html += '<a class="provision_confirm_delete_template_button" title="'+ Locale.tr("Delete")+'"  style="color:#555" href="#"><i class="fa fa-fw fa-lg fa-trash-o right only-on-hover"/></a>';
        }

        $(".provision_templates_ul", context).append('<li>'+
            '<ul class="provision-pricing-table" opennebula_id="'+data.ID+'" saved_to_image_id="'+data.TEMPLATE.SAVED_TO_IMAGE_ID+'" datatable_index="'+iDisplayIndexFull+'">'+
              '<li class="provision-title text-left" title="'+data.NAME+'">'+
                data.NAME +
              '</li>'+
              '<li class="provision-bullet-item text-left" >'+
                '<i class="fa fa-fw fa-lg fa-laptop"/> '+
                'x'+data.TEMPLATE.CPU+' - '+
                ((data.TEMPLATE.MEMORY > 1000) ?
                  (Math.floor(data.TEMPLATE.MEMORY/1024)+'GB') :
                  (data.TEMPLATE.MEMORY+'MB'))+
                ' - '+
                get_provision_disk_image(data) +
              '</li>'+
              '<li class="provision-description text-left" style="padding-top:0px; padding-bottom: 5px">'+
                (data.TEMPLATE.DESCRIPTION || '')+
              '</li>'+
              '<li class="provision-bullet-item" style="padding-top:10px">'+
                actions_html+
              '</li>'+
              '<li class="provision-bullet-item-last text-left" >'+
                '<span class="">'+
                  '<i class="fa fa-fw fa-lg fa-user"/> '+
                  data.UNAME+
                '</span>'+
                '<span class="right">'+
                  Humanize.prettyTimeAgo(data.REGTIME)+
                '</span>'+
              '</li>'+
            '</ul>'+
          '</li>');

        return nRow;
      }
    });

    provision_templates_datatable.fnFilter("^(?!\-$)", 2, true, false);

    $('.provision_list_templates_search', context).on('input',function(){
      provision_templates_datatable.fnFilter( $(this).val() );
    })

    context.on("click", ".provision_templates_list_refresh_button", function(){
      OpenNebula.Action.clear_cache("VMTEMPLATE");
      $(".provision_confirm_delete_template_div", context).html("");
      update_provision_templates_datatable(provision_templates_datatable, 0);
      return false;
    });

    context.on("click", ".provision_templates_list_search_button", function(){
      $(".provision_list_templates_search", context).fadeIn();
    });

    $(".provision_list_templates_filter", context).on("change", ".resource_list_select", function(){
      if ($(this).val() != "-2"){
        provision_templates_datatable.fnFilter("^" + $(this).val() + "$", 3, true, false);
      } else {
        provision_templates_datatable.fnFilter("", 3);
      }
    })

    ResourceSelect.insert(
      ".provision_list_templates_filter",
      context,
      "User",
      (opts.filter_expression ? opts.filter_expression : "-2"),
      false,
      '<option value="-2">'+Locale.tr("ALL")+'</option>',
      null,
      null,
      true,
      true);

    context.on("click", ".provision_templates_list_filter_button", function(){
      $(".provision_list_templates_filter", context).fadeIn();
      return false;
    });

    if (Config.isTabActionEnabled("provision-tab", "Template.delete")) {
      context.on("click", ".provision_confirm_delete_template_button", function(){
        var ul_context = $(this).parents(".provision-pricing-table");
        var template_id = ul_context.attr("opennebula_id");
        var template_name = $(".provision-title", ul_context).text();

        $(".provision_confirm_delete_template_div", context).html(
          '<div data-alert class="alert-box secondary radius">'+
            '<div class="row">'+
            '<div class="large-9 columns">'+
              '<span style="font-size: 14px; line-height: 20px">'+
                Locale.tr("Handle with care! This action will inmediately destroy the template")+
                ' "' + template_name + '" ' +
                Locale.tr("and the image associated.") +
              '</span>'+
            '</div>'+
            '<div class="large-3 columns">'+
              '<a href"#" class="provision_delete_template_button alert button large-12 radius right" style="margin-right: 15px" template_id="'+template_id+'">'+Locale.tr("Delete")+'</a>'+
            '</div>'+
            '</div>'+
            '<a href="#" class="close">&times;</a>'+
          '</div>');
      });

      context.on("click", ".provision_delete_template_button", function(){

        var button = $(this);
        button.attr("disabled", "disabled");

        var template_id = $(this).attr("template_id");

        OpenNebula.Template.delete_from_provision({
          timeout: true,
          data : {
            id : template_id
          },
          success: function (){
            $(".provision_templates_list_refresh_button", context).trigger("click");
          },
          error: function (request,error_json, container) {
            Notifier.onError(request, error_json, container);
            $(".provision_templates_list_refresh_button", context).trigger("click");
          }
        })
      });
    }


    if (Config.isTabActionEnabled("provision-tab", "Template.chmod")) {
      context.on("click", ".provision_confirm_chmod_template_button", function(){
        var ul_context = $(this).parents(".provision-pricing-table");
        var template_id = ul_context.attr("opennebula_id");
        var template_name = $(".provision-title", ul_context).text();

        $(".provision_confirm_delete_template_div", context).html(
          '<div data-alert class="alert-box secondary radius">'+
            '<div class="row">'+
            '<div class="large-8 columns">'+
              '<span style="font-size: 14px; line-height: 20px">'+
                Locale.tr("The template")+
                ' "' + template_name + '" ' +
                Locale.tr("and the image associated will be shared and all the users will be able to instantiate new VMs using this template.") +
              '</span>'+
            '</div>'+
            '<div class="large-4 columns">'+
              '<a href"#" class="provision_chmod_template_button success button large-12 radius right" style="margin-right: 15px" template_id="'+template_id+'">'+Locale.tr("Share template")+'</a>'+
            '</div>'+
            '</div>'+
            '<a href="#" class="close">&times;</a>'+
          '</div>');
      });

      context.on("click", ".provision_chmod_template_button", function(){

        var button = $(this);
        button.attr("disabled", "disabled");

        var template_id = $(this).attr("template_id");

        OpenNebula.Template.chmod_from_provision({
          timeout: true,
          data : {
            id : template_id,
            extra_param: {'group_u': 1}
          },
          success: function (){
            $(".provision_templates_list_refresh_button", context).trigger("click");
          },
          error: Notifier.onError
        })
      });

      context.on("click", ".provision_confirm_unshare_template_button", function(){
        var ul_context = $(this).parents(".provision-pricing-table");
        var template_id = ul_context.attr("opennebula_id");
        var template_name = $(".provision-title", ul_context).first().text();

        $(".provision_confirm_delete_template_div", context).html(
          '<div data-alert class="alert-box secondary radius">'+
            '<div class="row">'+
            '<div class="large-8 columns">'+
              '<span style="font-size: 14px; line-height: 20px">'+
                Locale.tr("The template")+
                ' "' + template_name + '" ' +
                Locale.tr("and the image associated will be unshared and the users will not be able to instantiate new VMs using this template.") +
              '</span>'+
            '</div>'+
            '<div class="large-4 columns">'+
              '<a href"#" class="provision_unshare_template_button success button large-12 radius right" style="margin-right: 15px" template_id="'+template_id+'">'+Locale.tr("Unshare template")+'</a>'+
            '</div>'+
            '</div>'+
            '<a href="#" class="close">&times;</a>'+
          '</div>');
      });

      context.on("click", ".provision_unshare_template_button", function(){
        var button = $(this);
        button.attr("disabled", "disabled");

        var template_id = $(this).attr("template_id");

        OpenNebula.Template.chmod_from_provision({
          timeout: true,
          data : {
            id : template_id,
            extra_param: {'group_u': 0}
          },
          success: function (){
            $(".provision_templates_list_refresh_button", context).trigger("click");
          },
          error: Notifier.onError
        })
      });
    }

    OpenNebula.Action.clear_cache("VMTEMPLATE");
    update_provision_templates_datatable(provision_templates_datatable, 0);
    context.foundation();
  }

  function get_provision_disk_image(data) {
    var disks = []
    if ($.isArray(data.TEMPLATE.DISK))
        disks = data.TEMPLATE.DISK
    else if (!$.isEmptyObject(data.TEMPLATE.DISK))
        disks = [data.TEMPLATE.DISK]

    if (disks.length > 0 && disks[0].IMAGE) {
      return disks[0].IMAGE;
    } else {
      return '';
    }
  }
});


/* START_TEMPLATE */
define('hbs!tabs/provision-tab/flows/info',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"text-center provision_info_flow_loading\">\n  <span class=\"fa-stack fa-5x\" style=\"color: #dfdfdf\"> <i class=\"fa fa-cloud fa-stack-2x\"></i> <i class=\"fa  fa-spinner fa-spin fa-stack-1x fa-inverse\"></i>\n  </span>\n  <br>\n  <br>\n  <span style=\"font-size: 18px; color: #999\"></span>\n</div>\n<div class=\"provision_info_flow\">\n  <div class=\"row\">\n    <div class=\"large-12 large-centered columns\">\n      <ul class=\"inline-list provision_action_icons\">\n        <li>\n          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Recover a failed service, cleaning the failed VMs", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"left button small radius success provision_recover_button\">\n            <i class=\"fa fa-fw fa-lg fa-wrench\"/></a>\n        </li>\n        <li class=\"right\">\n          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Delete", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"button small radius alert provision_delete_confirm_button tip-top right\">\n            <i class=\"fa fa-fw fa-lg fa-trash-o\"/></a>\n          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shutdown", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"button small radius secondary provision_shutdown_confirm_button tip-top right\">\n            <i class=\"fa fa-fw fa-lg fa-power-off\"/>\n          </a>\n        </li>\n      </ul>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"provision_confirm_action large-10 large-centered columns\"></div>\n  </div>\n  <br>\n  <div class=\"row\">\n    <div class=\"large-4 columns\">\n      <ul class=\"provision-pricing-table_flow_info provision-pricing-table\" style=\"border: 0px !important; background: #fff;\"></ul>\n    </div>\n    <div class=\"large-8 columns\">\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <ul class=\"provision_roles_ul large-block-grid-2 medium-block-grid-2 small-block-grid-1 text-center\"></ul>\n        </div>\n      </div>\n    </div>\n  </div>\n  <br>\n  <br>\n  <div class=\"row\">\n    <div class=\"provision_role_vms_container large-12 large-centered columns\"></div>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/provision-tab/flows/info', t);
Handlebars.registerPartial('tabs/provision-tab/flows/info', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/flows/list',['hbs','hbs/handlebars', 'hbs!tabs/provision-tab/flows/info','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  return "active";
  },"3":function(depth0,helpers,partials,data) {
  return "style=\"display:none\"";
  },"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<dl class=\"accordion accordion_list provision_list_flows\" data-accordion>\n  <dd class=\"";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.active : stack1), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += " accordion-navigation\">\n    <a class=\"provision_list_flow_accordion right only-not-active provision_back\" href=\"#provision_list_flow_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\">\n      <span class=\"button small radius\"> <i class=\"fa fa-fw fa-lg fa-th\"/><i class=\"fa fa-fw fa-lg fa-chevron-left\"/>\n      </span>\n    </a>\n    <h3 class=\"subheader\">\n      <span class=\"provision_info_flow_name only-not-active\"></span>\n      <span href\"#\" class=\"only-active right button radius small secondary provision_flows_list_refresh_button\" \n              ";
  stack1 = helpers.unless.call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.refresh : stack1), {"name":"unless","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += " title=\" "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Refresh", {"name":"tr","hash":{},"data":data})))
    + "\">\n        <i class=\"fa fa-fw fa-lg fa-refresh\"/></span>\n      <span href\"#\" class=\"only-not-active right button small radius secondary provision_refresh_info\" \n              ";
  stack1 = helpers.unless.call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.refresh : stack1), {"name":"unless","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += " title=\" "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Refresh", {"name":"tr","hash":{},"data":data})))
    + "\">\n        <i class=\"fa fa-fw fa-lg fa-refresh\"/>\n      </span>\n      <span href\"#\" class=\"only-active right button radius small secondary provision_flows_list_filter_button\" \n              ";
  stack1 = helpers.unless.call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.filter : stack1), {"name":"unless","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += " title=\" "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filter by User", {"name":"tr","hash":{},"data":data})))
    + "\">\n        <i class=\"fa fa-fw fa-lg fa-filter\"/>\n      </span>\n      <span class=\"only-active right provision_list_flows_filter\" style=\"display: none\"></span>\n      <span>\n        <input type=\"search\" class=\"only-active provision_list_flows_search provision-search-input\" placeholder=\"Search Services\"/>\n        <span href\"#\" class=\"only-active right button radius small success provision_create_flow_button\" \n              ";
  stack1 = helpers.unless.call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.create : stack1), {"name":"unless","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += ">\n          <i class=\"fa fa-fw fa-lg fa-plus-square\"/>\n        </span>\n      </h3>\n      <div id=\"provision_list_flow_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\" class=\"content ";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.opts : depth0)) != null ? stack1.active : stack1), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "\">\n        <div class=\"\">\n          <div class=\"row\">\n            <div class=\"large-12 large-centered columns\">\n              <table class=\"provision_flows_table\">\n                <thead class=\"hidden\">\n                  <tr>\n                    <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                    <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                    <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                  </tr>\n                </thead>\n                <tbody class=\"hidden\"></tbody>\n              </table>\n              <br></div>\n          </div>\n        </div>\n      </div>\n    </dd>\n    <dd class=\"accordion-navigation\">\n      <a class=\"provision_show_flow_accordion\" href=\"#provision_show_flow_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\"></a>\n      <div id=\"provision_show_flow_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\" class=\"content\">";
  stack1 = this.invokePartial(partials['tabs/provision-tab/flows/info'], '', 'tabs/provision-tab/flows/info', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n    </dd>\n  </dl>";
},"usePartial":true,"useData":true});
Handlebars.registerPartial('tabs/provision-tab/flows/list', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/provision-tab/flows/list',['require','foundation.alert','opennebula','utils/locale','utils/notifier','utils/humanize','utils/resource-select','tabs/provision-tab/vms/list','hbs!./list'],function(require) {
  require('foundation.alert');
  var OpenNebula = require('opennebula');
  var Locale = require('utils/locale');
  var Notifier = require('utils/notifier');
  var Humanize = require('utils/humanize');
  var ResourceSelect = require('utils/resource-select');

  var ProvisionVmsList = require('tabs/provision-tab/vms/list');

  var TemplateFlowsList = require('hbs!./list');

  var _accordionId = 0;

  return {
    'generate': generate_provision_flows_list,
    'show': show_provision_flow_list
  };


  function show_provision_flow_list(timeout) {
    $(".section_content").hide();
    $(".provision_flows_list_section").fadeIn();

    $("dd:not(.active) .provision_back", $(".provision_flows_list_section")).trigger("click");
    $(".provision_flows_list_refresh_button", $(".provision_flows_list_section")).trigger("click");
  }

  function generate_provision_flows_list(context, opts) {
    context.off();
    context.html(html(opts));
    setup_provision_flows_list(context, opts);
    setup_info_flow(context);
  }

  function html(opts_arg){
    opts = $.extend({
        title: Locale.tr("Services"),
        active: true,
        refresh: true,
        create: true,
        filter: true
      },opts_arg)

    _accordionId += 1;
    return TemplateFlowsList({'accordionId': _accordionId, 'opts': opts});
  }

  function update_provision_flows_datatable(datatable, timeout) {
    datatable.html('<div class="text-center">'+
      '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
        '<i class="fa fa-cloud fa-stack-2x"></i>'+
        '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>'+
      '</span>'+
      '<br>'+
      '<br>'+
      '<span style="font-size: 18px; color: #999">'+
      '</span>'+
      '</div>');

    setTimeout( function(){
      OpenNebula.Service.list({
        timeout: true,
        success: function (request, item_list){
          $(".flow_error_message").hide();
          datatable.fnClearTable(true);
          if (item_list.length == 0) {
            datatable.html('<div class="text-center">'+
              '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
                '<i class="fa fa-cloud fa-stack-2x"></i>'+
                '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
              '</span>'+
              '<br>'+
              '<br>'+
              '<span style="font-size: 18px; color: #999">'+
                Locale.tr("There are no Services")+
              '</span>'+
              '<br>'+
              '<br>'+
              '</div>');
          } else {
            datatable.fnAddData(item_list);
          }
        },
        error: function(request, error_json) {
          datatable.html('<div class="text-center">'+
            '<br>'+
            '<br>'+
            '<div class="row flow_error_message" id="" hidden>'+
              '<div class="small-6 columns small-centered text-center">'+
                  '<div class="alert-box alert radius">'+Locale.tr("Cannot connect to OneFlow server")+'</div>'+
              '</div>'+
            '</div>'+
            '<br>'+
            '<br>'+
            '<span style="font-size: 18px; color: #999">'+
            '</span>'+
            '</div>');

            Notifier.onError(request, error_json, $(".flow_error_message"));
        }
      })
    }, timeout );
  }

  function setup_provision_flows_list(context, opts){
    //
    // List Flows
    //

    provision_flows_datatable = $('.provision_flows_table', context).dataTable({
      "iDisplayLength": 6,
      "sDom" : '<"H">t<"F"lp>',
      "aLengthMenu": [[6, 12, 36, 72], [6, 12, 36, 72]],
      "aaSorting"  : [[0, "desc"]],
      "aoColumnDefs": [
          { "bVisible": false, "aTargets": ["all"]}
      ],
      "aoColumns": [
          { "mDataProp": "DOCUMENT.ID" },
          { "mDataProp": "DOCUMENT.NAME" },
          { "mDataProp": "DOCUMENT.UID" }
      ],
      "fnPreDrawCallback": function (oSettings) {
        // create a thumbs container if it doesn't exist. put it in the dataTables_scrollbody div
        if (this.$('tr', {"filter": "applied"} ).length == 0) {
          this.html('<div class="text-center">'+
            '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
              '<i class="fa fa-cloud fa-stack-2x"></i>'+
              '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
            '</span>'+
            '<br>'+
            '<br>'+
            '<span style="font-size: 18px; color: #999">'+
              Locale.tr("Looks like you don't have any Service. Click the button below to get started")+
            '</span>'+
            '<br>'+
            '<br>'+
            '<div class="row">'+
              '<div class="large-6 large-centered columns">'+
                '<a href"#" class="medium large-12 button radius provision_create_flow_button"">'+Locale.tr("Create a new Service")+'</a>'+
              '</div>'+
            '</div>'+
            '<br>'+
            '<br>'+
            '</div>');
        } else {
          $(".provision_flows_table", context).html('<ul class="provision_flows_ul large-block-grid-3 medium-block-grid-3 small-block-grid-1 text-center"></ul>');
        }

        return true;
      },
      "fnRowCallback": function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
        var data = aData.DOCUMENT;
        var body = data.TEMPLATE.BODY;
        var state = get_provision_flow_state(body);
        var start_time = get_provision_flow_start_time(body);

        var roles_li = "";
        if (body.roles) {
          $.each(body.roles, function(index, role) {
            var role_state = get_provision_flow_state(role);
            var rvms = {
              str : (role.nodes ? role.nodes.length : 0) + " / " + role.cardinality ,
              percentage : Math.floor((role.nodes ? role.nodes.length : 0) / role.cardinality)*100
            }

            roles_li +=
              '<li class="provision-bullet-item text-left"">'+
                '<i class="fa fa-fw fa-lg fa-cube"/>&emsp;'+
                role.name+
                '<span class="right" style="font-size: 12px">'+rvms.str+" VMs</span>"+
              '</li>';
          });
        }

        $(".provision_flows_ul", context).append('<li>'+
            '<ul class="provision-pricing-table" opennebula_id="'+data.ID+'" datatable_index="'+iDisplayIndexFull+'">'+
              '<li class="provision-title text-left">'+
                '<a class="provision_info_flow_button" style="color:#555" href="#">'+
                  '<span class="'+ state.color +'-color" title="'+ state.str +'">'+
                    '<i class="fa fa-fw fa-lg fa-square"/>&emsp;'+
                  '</span>'+
                  data.NAME +
                '</a>'+
              '</li>'+
              roles_li +
              '<li class="provision-bullet-item-last text-right">'+
                '<span class="left">'+
                  '<i class="fa fa-fw fa-lg fa-user"/>&emsp;'+
                  data.UNAME+
                '</span>'+
                '<span>'+
                  (start_time ? Humanize.prettyTimeAgo(start_time) : '-') +
                '</span>'+
              '</li>'+
            '</ul>'+
          '</li>');

        return nRow;
      }
    });

    $('.provision_list_flows_search', context).on('input',function(){
      provision_flows_datatable.fnFilter( $(this).val() );
    })

    context.on("click", ".provision_flows_list_refresh_button", function(){
      OpenNebula.Action.clear_cache("SERVICE");
      update_provision_flows_datatable(provision_flows_datatable, 0);
      return false;
    });

    context.on("click", ".provision_flows_list_search_button", function(){
      $(".provision_list_flows_search", context).fadeIn();
    });

    $(".provision_list_flows_filter", context).on("change", ".resource_list_select", function(){
      if ($(this).val() != "-2"){
        provision_flows_datatable.fnFilter("^" + $(this).val() + "$", 2, true, false);
      } else {
        provision_flows_datatable.fnFilter("", 2);
      }
    })

    ResourceSelect.insert(
      ".provision_list_flows_filter",
      context,
      "User",
      (opts.filter_expression ? opts.filter_expression : "-2"),
      false,
      '<option value="-2">'+Locale.tr("ALL")+'</option>',
      null,
      null,
      true,
      true);

    context.on("click", ".provision_flows_list_filter_button", function(){
      $(".provision_list_flows_filter", context).fadeIn();
      return false;
    });

    OpenNebula.Action.clear_cache("SERVICE");
    update_provision_flows_datatable(provision_flows_datatable, 0);

    $(document).foundation();
  }

  function setup_info_flow(context) {
    function update_provision_flow_info(flow_id, context, role_id) {
      $(".provision_info_flow_name", context).text("");
      $(".provision_info_flow", context).css('visibility', 'hidden');
      $(".provision_info_flow_loading", context).fadeIn();
      $(".provision_role_vms_container").html("");

      OpenNebula.Service.show({
        data : {
          id: flow_id
        },
        error: Notifier.onError,
        success: function(request, response){
          var data = response.DOCUMENT
          var body = data.TEMPLATE.BODY;
          var state = get_provision_flow_state(body);
          var start_time = get_provision_flow_start_time(body);

          switch (state.color) {
            case "deploying":
              $(".provision_recover_button", context).hide();
              $(".provision_delete_confirm_button", context).show();
              $(".provision_shutdown_confirm_button", context).show();
              break;
            case "running":
              $(".provision_recover_button", context).hide();
              $(".provision_delete_confirm_button", context).show();
              $(".provision_shutdown_confirm_button", context).show();
              break;
            case "off":
              $(".provision_recover_button", context).hide();
              $(".provision_delete_confirm_button", context).show();
              $(".provision_shutdown_confirm_button", context).hide();
              break;
            case "powering_off":
            case "error":
              $(".provision_recover_button", context).show();
              $(".provision_delete_confirm_button", context).show();
              $(".provision_shutdown_confirm_button", context).show();
              break;
            default:
              $(".provision_recover_button", context).show();
              $(".provision_delete_confirm_button", context).show();
              $(".provision_shutdown_confirm_button", context).show();
              break;
          }

          $(".provision_info_flow", context).attr("flow_id", data.ID);
          $(".provision_info_flow_name", context).text(data.NAME);

          $(".provision-pricing-table_flow_info", context).html(
              '<li class="text-left provision-bullet-item">'+
                '<span class="'+ state.color +'-color">'+
                  '<i class="fa fa-fw fa-lg fa-square"/>&emsp;'+
                  state.str+
                '</span>'+
              '</li>'+
              '<li class="text-left provision-bullet-item">'+
                '<hr style="margin: 0px">'+
              '</li>'+
              '<li class="text-left provision-bullet-item">'+
                '<span style="color: #999;">'+
                  '<i class="fa fa-fw fa-lg fa-user"/>&emsp;'+
                  data.UNAME+
                '</span>'+
                '<span class="right" style="color: #999;">'+
                  '<i class="fa fa-fw fa-lg fa-clock-o"/>&emsp;'+
                  (start_time ? Humanize.prettyTimeAgo(start_time) : "...") +
                  ' - '+
                  'ID: '+
                  data.ID+
                '</span>'+
              '</li>'+
            '</ul>');

          $(".provision_roles_ul", context).html("");
          if (body.roles) {
            $.each(body.roles, function(index, role) {
              var role_state = get_provision_flow_state(role);
              var rvms = {
                str : (role.nodes ? role.nodes.length : 0) + " / " + role.cardinality ,
                percentage : Math.floor((role.nodes ? role.nodes.length : 0) / role.cardinality)*100
              }

              var li = $(
                '<li>'+
                  '<ul class="provision_role_ul provision-pricing-table">'+
                    '<li class="provision-title text-left">'+
                      '<i class="fa fa-fw fa-cube"/>&emsp;'+
                      role.name+
                    '</li>'+
                    '<li class="provision-bullet-item text-left" style="padding-top: 5px; margin-left: 10px; margin-right: 10px">'+
                      '<div class="progress small radius" style="margin-bottom:0px">'+
                      '  <span class="meter" style="width: '+rvms.percentage+'%;"></span>'+
                      '</div>'+
                    '</li>'+
                    '<li class="provision-bullet-item text-left" style="padding-top: 0px; margin-left: 10px; margin-right: 10px; font-size: 14px">'+
                      '<span class="'+ role_state.color +'-color">'+
                        role_state.str+
                      '</span>'+
                      '<span class="right">'+rvms.str+" VMs</span>"+
                    '</li>'+
                    '<li class="text-left provision-bullet-item">'+
                      '<br>'+
                    '</li>'+
                    '<li class="provision-bullet-item text-left" style="padding-top: 5px; margin-left: 10px; margin-right: 10px">'+
                      '<a class="provision_role_vms_button button medium radius">'+
                        '<i class="fa fa-th fa-lg"></i>'+
                      '</a>'+
                      '<a class="provision_role_cardinality_button button medium success right radius">'+
                        '<i class="fa fa-arrows-h fa-lg"></i>'+
                      '</a>'+
                    '</li>'+
                  '</ul>'+
                '</li>').appendTo($(".provision_roles_ul", context));

                $(".provision_role_ul", li).data("role", role);
                if (role_id && role_id == role.name) {
                  $(".provision_role_vms_button", li).trigger("click");
                }
            });
          }

          $(".provision_info_flow_state_hr", context).html('<div style="height:1px; margin-top:5px; margin-bottom: 5px; background: #cfcfcf"></div>');

          $(".provision_confirm_action:first", context).html("");

          $(".provision_info_flow_loading", context).hide();
          $(".provision_info_flow", context).css('visibility', 'visible');
        }
      })
    }

    context.on("click", ".provision_role_vms_button", function(){
      $(".provision_role_vms_container", context).html('<div class="text-center">'+
        '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
          '<i class="fa fa-cloud fa-stack-2x"></i>'+
          '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>'+
        '</span>'+
        '<br>'+
        '<br>'+
        '<span style="font-size: 18px; color: #999">'+
        '</span>'+
        '</div>');

      var role = $(this).closest(".provision_role_ul").data('role');
      $(".provision_info_flow", context).data("role_id", role.name);
      var vms = []

      if (role.nodes && role.nodes.length > 0) {
        $.each(role.nodes, function(index, node){
          vms.push(node.vm_info);
        })
      }

      ProvisionVmsList.generate(
        $(".provision_role_vms_container", context),
        {
          title: role.name + ' ' + Locale.tr("VMs"),
          active: true,
          refresh: false,
          create: false,
          filter: false,
          data: vms
        });
    })

    context.on("click", ".provision_role_cardinality_button", function(){
      var role = $(this).closest(".provision_role_ul").data('role');
      var min_vms = (role.min_vms||1);
      var max_vms = (role.max_vms||100);

      $(".provision_confirm_action:first", context).html(
        '<div data-alert class="alert-box secondary radius">'+
          '<div class="row">'+
            '<div class="large-12 large-centered columns">'+
              '<div class="row">'+
                '<div class="large-4 text-center columns">'+
                  '<span class="cardinality_value" style="color: #777; font-size:60px">'+role.cardinality+'</span>'+
                  '<br>'+
                  '<span style="color: #999; font-size:20px">'+role.name + ' ' + Locale.tr("VMs")+'</span>'+
                '</div>'+
                '<div class="large-8 columns">'+
                '<div class="cardinality_slider_div">'+
                  '<br>'+
                  '<span class="left" style="color: #999;">'+min_vms+'</span>'+
                  '<span class="right" style="color: #999;">'+max_vms+'</span>'+
                  '<br>'+
                  '<div class="range-slider cardinality_slider" data-slider>'+
                    '<span class="range-slider-handle" role="slider" tabindex="0"></span>'+
                    '<span class="range-slider-active-segment"></span>'+
                    '<input type="hidden">'+
                  '</div>'+
                  '<br>'+
                  '<a href"#" class="provision_change_cardinality_button success button radius large-12" role_id="'+role.name+'">'+Locale.tr("Change Cardinality")+'</a>'+
                '</div>'+
                '<div class="cardinality_no_slider_div">'+
                  '<br>'+
                  '<br>'+
                  '<span class="" style="color: #999;">'+Locale.tr("The cardinality for this role cannot be changed")+'</span>'+
                '</div>'+
              '</div>'+
            '</div>'+
          '</div>'+
          '<a href="#" class="close" style="top: 20px">&times;</a>'+
        '</div>');

      context.foundation('slider', 'reflow');
      if (max_vms > min_vms) {
        $( ".cardinality_slider_div", context).show();
        $( ".cardinality_no_slider_div", context).hide();

        $( ".cardinality_slider", context).foundation('slider', 'set_value', role.cardinality);
        $( ".cardinality_slider", context).on('change.fndtn.slider', function(){
          $(".cardinality_value",context).html($(this).attr('data-slider'))
        });
      } else {
        $( ".cardinality_slider_div", context).hide();
        $( ".cardinality_no_slider_div", context).show();
      }

      return false;
    });

    context.on("click", ".provision_change_cardinality_button", function(){
      var flow_id = $(".provision_info_flow", context).attr("flow_id");
      var cardinality = $(".cardinality_slider", context).attr('data-slider');

      OpenNebula.Role.update({
        data : {
          id: flow_id + '/role/' + $(this).attr("role_id"),
          extra_param: {
            cardinality: cardinality
          }
        },
        success: function(request, response){
          OpenNebula.Action.clear_cache("SERVICE");
          $(".provision_refresh_info", context).trigger("click");
        },
        error: Notifier.onError
      })
    });

    context.on("click", ".provision_delete_confirm_button", function(){
      $(".provision_confirm_action:first", context).html(
        '<div data-alert class="alert-box secondary radius">'+
          '<div class="row">'+
          '<div class="large-9 columns">'+
            '<span style="font-size: 14px; line-height: 20px">'+
              Locale.tr("Be careful, this action will inmediately destroy your Service")+
              '<br>'+
              Locale.tr("All the information will be lost!")+
            '</span>'+
          '</div>'+
          '<div class="large-3 columns">'+
            '<a href"#" class="provision_delete_button alert button large-12 radius right" style="margin-right: 15px">'+Locale.tr("Delete")+'</a>'+
          '</div>'+
          '</div>'+
          '<a href="#" class="close">&times;</a>'+
        '</div>');
    });

    context.on("click", ".provision_shutdown_confirm_button", function(){
      $(".provision_confirm_action:first", context).html(
        '<div data-alert class="alert-box secondary radius">'+
          '<div class="row">'+
          '<div class="large-9 columns">'+
            '<span style="font-size: 14px; line-height: 20px">'+
              Locale.tr("Be careful, this action will inmediately shutdown your Service")+
              '<br>'+
              Locale.tr("All the information will be lost!")+
            '</span>'+
          '</div>'+
          '<div class="large-3 columns">'+
            '<a href"#" class="provision_shutdown_button alert button large-12 radius right" style="margin-right: 15px">'+Locale.tr("Shutdown")+'</a>'+
          '</div>'+
          '</div>'+
          '<a href="#" class="close">&times;</a>'+
        '</div>');
    });

    context.on("click", ".provision_recover_button", function(){
      var flow_id = $(".provision_info_flow", context).attr("flow_id");

      OpenNebula.Service.recover({
        data : {
          id: flow_id
        },
        success: function(request, response){
          update_provision_flow_info(flow_id, context);
        },
        error: Notifier.onError
      })
    });

    context.on("click", ".provision_shutdown_button", function(){
      var flow_id = $(".provision_info_flow", context).attr("flow_id");

      OpenNebula.Service.shutdown({
        data : {
          id: flow_id
        },
        success: function(request, response){
          update_provision_flow_info(flow_id, context);
        },
        error: Notifier.onError
      })
    });

    context.on("click", ".provision_delete_button", function(){
      var button = $(this);
      button.attr("disabled", "disabled");
      var flow_id = $(".provision_info_flow", context).attr("flow_id");

      OpenNebula.Service.del({
        data : {
          id: flow_id
        },
        success: function(request, response){
          $(".provision_back", context).click();
          $(".provision_flows_list_refresh_button", context).click();
          button.removeAttr("disabled");
        },
        error: function(request, response){
          Notifier.onError(request, response);
          button.removeAttr("disabled");
        }
      })
    });

    context.on("click", ".provision_refresh_info", function(){
      var flow_id = $(".provision_info_flow", context).attr("flow_id");
      var role_id = $(".provision_info_flow", context).data("role_id");
      update_provision_flow_info(flow_id, context, role_id);
      //$(".provision_flows_list_refresh_button", $(".provision_flows_list_section")).trigger("click");
      return false;
    });

    //
    // Info Flow
    //

    $(".provision_list_flows", context).on("click", ".provision_info_flow_button", function(){
      $("a.provision_show_flow_accordion", context).trigger("click");

      var flow_id = $(this).parents(".provision-pricing-table").attr("opennebula_id")
      update_provision_flow_info(flow_id, context);
      return false;
    })
  }

  function get_provision_flow_start_time(data) {
    if (data.log) {
      return data.log[0].timestamp
    } else {
      return null;
    }
  }

  // @params
  //    data: and BODY object of the Document representing the Service
  //      Example: data.ID
  // @returns and object containing the following properties
  //    color: css class for this state.
  //      color + '-color' font color class
  //      color + '-bg' background class
  //    str: user friendly state string
  function get_provision_flow_state(data) {
    var state_color;
    var state_str;

    switch (parseInt(data.state)) {
      case OpenNebula.Service.STATES.PENDING:
        state_color = 'deploying';
        state_str = Locale.tr("PENDING");
        break;
      case OpenNebula.Service.STATES.DEPLOYING:
        state_color = 'deploying';
        state_str = Locale.tr("DEPLOYING");
        break;
      case OpenNebula.Service.STATES.UNDEPLOYING:
        state_color = 'powering_off';
        state_str = Locale.tr("UNDEPLOYING");
        break;
      case OpenNebula.Service.STATES.FAILED_UNDEPLOYING:
        state_color = 'error';
        state_str = Locale.tr("FAILED UNDEPLOYING");
        break;
      case OpenNebula.Service.STATES.FAILED_DEPLOYING:
        state_color = 'error';
        state_str = Locale.tr("FAILED DEPLOYING");
        break;
      case OpenNebula.Service.STATES.FAILED_SCALING:
        state_color = 'error';
        state_str = Locale.tr("FAILED SCALING");
        break;
      case OpenNebula.Service.STATES.WARNING:
        state_color = 'error';
        state_str = Locale.tr("WARNING");
        break;
      case OpenNebula.Service.STATES.RUNNING:
        state_color = 'running';
        state_str = Locale.tr("RUNNING");
        break;
      case OpenNebula.Service.STATES.SCALING:
        state_color = 'deploying';
        state_str = Locale.tr("SCALING");
        break;
      case OpenNebula.Service.STATES.COOLDOWN:
        state_color = 'error';
        state_str = Locale.tr("COOLDOWN");
        break;
      case OpenNebula.Service.STATES.DONE:
        state_color = 'off';
        state_str = Locale.tr("DONE");
        break;
      default:
        state_color = 'powering_off';
        state_str = Locale.tr("UNKNOWN");
        break;
    }

    return {
      color: state_color,
      str: state_str
    }
  }
});


/* START_TEMPLATE */
define('hbs!tabs/provision-tab/users/quota-widget/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"row\">\n  <div class=\"large-12 large-centered columns\">\n    <h5 class=\"subheader text-right\">\n      <span class=\"left\" style=\"margin-bottom: 0.5em\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Running VMs", {"name":"tr","hash":{},"data":data})))
    + "</span>\n    </h5>\n  </div>\n</div>\n<div class=\"row provision_rvms_quota\">\n  <div class=\"medium-3 small-12 columns\">\n    <select class=\"provision_quota_select\">\n      <option value=\"edit\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Manual", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"unlimited\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Unlimited", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"default\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Default", {"name":"tr","hash":{},"data":data})))
    + "</option>\n    </select>\n  </div>\n  <div class=\"medium-9 small-12 columns provision_quota_edit\">\n    <div class=\"row collapse\">\n      <div class=\"small-9 columns\">\n        <div class=\"range-slider radius provision_rvms_quota_slider\" data-slider data-options=\"start: 0; end: 50;\">\n          <span class=\"range-slider-handle\"></span>\n          <span class=\"range-slider-active-segment\"></span>\n          <input type=\"hidden\"></div>\n      </div>\n      <div class=\"large-2 small-2 columns\">\n        <input type=\"text\"  class=\"provision-input provision_rvms_quota_input\" style=\"margin-top: -7px; height: 40px !important; font-size: 16px; padding: 0.5rem  !important;\"/>\n      </div>\n    </div>\n  </div>\n  <div class=\"medium-9 small-12 columns provision_quota_unlimited\" style=\"display:none\">\n    <span style=\"font-size: 18px; color: #999\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Unlimited. Group quotas will still apply", {"name":"tr","hash":{},"data":data})))
    + "</span>\n  </div>\n  <div class=\"medium-9 small-12 columns provision_quota_default\" style=\"display:none\">\n    <span style=\"font-size: 18px; color: #999\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Use the default system quotas set by the cloud adminstrator", {"name":"tr","hash":{},"data":data})))
    + "\n    </span>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-12 large-centered columns\">\n    <h5 class=\"subheader text-right\">\n      <span class=\"left\" style=\"margin-bottom: 0.5em\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU", {"name":"tr","hash":{},"data":data})))
    + "</span>\n    </h5>\n  </div>\n</div>\n<div class=\"row provision_cpu_quota\">\n  <div class=\"medium-3 small-12 columns\">\n    <select class=\"provision_quota_select\">\n      <option value=\"edit\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Manual", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"unlimited\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Unlimited", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"default\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Default", {"name":"tr","hash":{},"data":data})))
    + "</option>\n    </select>\n  </div>\n  <div class=\"medium-9 small-12 columns provision_quota_edit\">\n    <div class=\"row collapse\">\n      <div class=\"small-9 columns\">\n        <div class=\"range-slider radius provision_cpu_quota_slider\" data-slider data-options=\"start: 0; end: 50;\">\n          <span class=\"range-slider-handle\"></span>\n          <span class=\"range-slider-active-segment\"></span>\n          <input type=\"hidden\"></div>\n      </div>\n      <div class=\"large-2 small-2 columns\">\n        <input type=\"text\"  class=\"provision-input provision_cpu_quota_input\" style=\"margin-top: -7px; height: 40px !important; font-size: 16px; padding: 0.5rem  !important;\"/>\n      </div>\n    </div>\n  </div>\n  <div class=\"medium-9 small-12 columns provision_quota_unlimited\" style=\"display:none\">\n    <span style=\"font-size: 18px; color: #999\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Unlimited. Group quotas will still apply", {"name":"tr","hash":{},"data":data})))
    + "</span>\n  </div>\n  <div class=\"medium-9 small-12 columns provision_quota_default\" style=\"display:none\">\n    <span style=\"font-size: 18px; color: #999\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Use the default system quotas set by the cloud adminstrator", {"name":"tr","hash":{},"data":data})))
    + "\n    </span>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-12 large-centered columns\">\n    <h5 class=\"subheader text-right\">\n      <span class=\"left\" style=\"margin-bottom: 0.5em\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Memory (GB)", {"name":"tr","hash":{},"data":data})))
    + "</span>\n    </h5>\n  </div>\n</div>\n<div class=\"vm_param\">\n  <input type=\"hidden\" class=\"provision_memory_quota_input\"/>\n</div>\n<div class=\"row provision_memory_quota\">\n  <div class=\"medium-3 small-12 columns\">\n    <select class=\"provision_quota_select\">\n      <option value=\"edit\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Manual", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"unlimited\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Unlimited", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"default\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Default", {"name":"tr","hash":{},"data":data})))
    + "</option>\n    </select>\n  </div>\n  <div class=\"medium-9 small-12 columns provision_quota_edit\">\n    <div class=\"row collapse\">\n      <div class=\"small-9 columns\">\n        <div class=\"range-slider radius provision_memory_quota_slider\" data-slider data-options=\"start: 0; end: 50;\">\n          <span class=\"range-slider-handle\"></span>\n          <span class=\"range-slider-active-segment\"></span>\n          <input type=\"hidden\"></div>\n      </div>\n      <div class=\"large-2 small-2 columns\">\n        <input type=\"text\"  class=\"provision-input provision_memory_quota_tmp_input\" style=\"margin-top: -7px; height: 40px !important; font-size: 16px; padding: 0.5rem  !important;\"/>\n      </div>\n    </div>\n  </div>\n  <div class=\"medium-9 small-12 columns provision_quota_unlimited\" style=\"display:none\">\n    <span style=\"font-size: 18px; color: #999\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Unlimited. Group quotas will still apply", {"name":"tr","hash":{},"data":data})))
    + "</span>\n  </div>\n  <div class=\"medium-9 small-12 columns provision_quota_default\" style=\"display:none\">\n    <span style=\"font-size: 18px; color: #999\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Use the default system quotas set by the cloud adminstrator", {"name":"tr","hash":{},"data":data})))
    + "\n    </span>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/provision-tab/users/quota-widget/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/users/empty-graphs',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<span class=\"fa-stack fa-2x\" style=\"color: #dfdfdf\"> <i class=\"fa fa-cloud fa-stack-2x\"></i>\n</span>\n<br>\n<span style=\"color: #cfcfcf\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "There is no information available", {"name":"tr","hash":{},"data":data})))
    + "</span>";
},"useData":true});
Handlebars.registerPartial('tabs/provision-tab/users/empty-graphs', t);
Handlebars.registerPartial('./empty-graphs', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/users/info',['hbs','hbs/handlebars', 'hbs!tabs/provision-tab/users/empty-graphs','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"text-center provision_info_vdc_user_loading\">\n  <span class=\"fa-stack fa-5x\" style=\"color: #dfdfdf\"> <i class=\"fa fa-cloud fa-stack-2x\"></i> <i class=\"fa  fa-spinner fa-spin fa-stack-1x fa-inverse\"></i>\n  </span>\n  <br>\n  <br>\n  <span style=\"font-size: 18px; color: #999\"></span>\n</div>\n<div class=\"provision_info_vdc_user\">\n  <div class=\"row\">\n    <div class=\"large-12 large-centered columns\">\n      <ul class=\"inline-list provision_action_icons\">\n        <li class=\"\">\n          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update the User Quotas", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"left button small secondary radius provision_vdc_user_quota_confirm_button tip-top\">\n            <i class=\"fa fa-fw fa-lg fa-align-left\"/></a>\n          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Change the password of the User", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"left button small secondary radius provision_vdc_user_password_confirm_button tip-top\">\n            <i class=\"fa fa-fw fa-lg fa-lock\"/></a>\n        </li>\n        <li class=\"right\">\n          <a href\"#\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Delete the User", {"name":"tr","hash":{},"data":data})))
    + "\" class=\"right button small radius alert provision_vdc_user_delete_confirm_button tip-top\">\n            <i class=\"fa fa-fw fa-lg fa-trash-o\"/>\n          </a>\n        </li>\n      </ul>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"provision_vdc_user_confirm_action large-12 columns\"></div>\n  </div>\n  <br>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <ul class=\"provision-pricing-table_user_info provision-pricing-table\" style=\"border: 0px !important; background: #fff;\"></ul>\n    </div>\n    <div class=\"large-12 columns\">\n      <div class=\"row dashboard_vm_accounting\">\n        <div id=\"acct_content\">\n          <div class=\"large-4 columns\">\n            <div class=\"row\">\n              <div class=\"large-12 columns graph_legend text-center\">\n                <span style=\"color: #777; font-size: 14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk MB hours", {"name":"tr","hash":{},"data":data})))
    + "</span>\n              </div>\n              <div class=\"large-12 columns\">\n                <div class=\"large-12 columns centered graph\" id=\"acct_disk_graph\" style=\"height: 100px;\"></div>\n              </div>\n            </div>\n          </div>\n          <div class=\"large-4 columns\">\n            <input style=\"display:none;\" value=\"vm\" id=\"acct_group_by\"/>\n            <div class=\"row\">\n              <div class=\"large-12 columns graph_legend text-center\">\n                <span style=\"color: #777; font-size: 14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU hours", {"name":"tr","hash":{},"data":data})))
    + "</span>\n              </div>\n              <br>\n              <div class=\"large-12 columns\">\n                <div class=\"large-12 columns centered graph\" id=\"acct_cpu_graph\" style=\"height: 100px;\"></div>\n              </div>\n            </div>\n          </div>\n          <div class=\"large-4 columns\">\n            <div class=\"row\">\n              <div class=\"large-12 columns graph_legend text-center\">\n                <span style=\"color: #777; font-size: 14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Memory GB hours", {"name":"tr","hash":{},"data":data})))
    + "</span>\n              </div>\n              <div class=\"large-12 columns\">\n                <div class=\"large-12 columns centered graph\" id=\"acct_mem_graph\" style=\"height: 100px;\"></div>\n              </div>\n            </div>\n          </div>\n        </div>\n        <div id=\"acct_no_data\">\n          <div class=\"row\">\n            <div class=\"large-12 columns text-center\">";
  stack1 = this.invokePartial(partials['./empty-graphs'], '', './empty-graphs', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n          </div>\n        </div>\n      </div>\n    </div>\n  </div>\n  <br>\n  <div class=\"row\">\n    <div class=\"provision-user-resource-header large-12 columns text-center\"></div>\n  </div>\n  <br>\n  <div class=\"row\">\n    <div class=\"provision_vdc_info_container large-12 columns\"></div>\n  </div>\n  <br></div>\n";
},"usePartial":true,"useData":true});
Handlebars.registerPartial('tabs/provision-tab/users/info', t);
Handlebars.registerPartial('tabs/provision-tab/users/info', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/users/list',['hbs','hbs/handlebars', 'hbs!tabs/provision-tab/users/info','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<dl class=\"accordion accordion_list provision_list_users\" data-accordion>\n  <dd class=\"active accordion-navigation\">\n    <a href=\"#provision_list_user_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\" class=\"provision_back right only-not-active\">\n      <span class=\"button small radius\"> <i class=\"fa fa-fw fa-lg fa-th\"/><i class=\"fa fa-fw fa-lg fa-chevron-left\"/>\n      </span>\n    </a>\n    <h3 class=\"subheader\">\n      <span class=\"provision_info_vdc_user_name only-not-active\"></span>\n      <span href\"#\" class=\"right only-active provision_users_list_refresh_button button small radius secondary\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Refresh", {"name":"tr","hash":{},"data":data})))
    + "\">\n        <i class=\"fa fa-fw fa-lg fa-refresh\"/></span>\n      <span href\"#\" class=\"right only-not-active provision_refresh_info button small radius secondary\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Refresh", {"name":"tr","hash":{},"data":data})))
    + "\">\n        <i class=\"fa fa-fw fa-lg fa-refresh\"/>\n      </span>\n      <input type=\"search\" class=\"provision_list_users_search provision-search-input only-active\" placeholder=\"Search Users\"/>\n      <span href\"#\" class=\"right only-active provision_create_user_button button small radius success\">\n        <i class=\"fa fa-fw fa-lg fa-plus-square\"/>\n      </span>\n    </h3>\n    <div id=\"provision_list_user_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\" class=\"content active\">\n      <div class=\"row\">\n        <div class=\"large-12 large-centered columns\">\n          <table class=\"provision_users_table\">\n            <thead class=\"hidden\">\n              <tr>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              </tr>\n            </thead>\n            <tbody class=\"hidden\"></tbody>\n          </table>\n          <br></div>\n      </div>\n    </div>\n  </dd>\n  <dd class=\"accordion-navigation\">\n    <a class=\"provision_show_user_accordion\" href=\"#provision_show_user_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\"></a>\n    <div id=\"provision_show_user_accordion"
    + escapeExpression(((helper = (helper = helpers.accordionId || (depth0 != null ? depth0.accordionId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"accordionId","hash":{},"data":data}) : helper)))
    + "\" class=\"content\">";
  stack1 = this.invokePartial(partials['tabs/provision-tab/users/info'], '', 'tabs/provision-tab/users/info', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n  </dd>\n</dl>";
},"usePartial":true,"useData":true});
Handlebars.registerPartial('tabs/provision-tab/users/list', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/provision-tab/users/list',['require','foundation.alert','opennebula','utils/locale','sunstone-config','utils/notifier','utils/quotas/quota-widgets','utils/quotas/quota-limits','utils/quotas/quota-defaults','utils/accounting','utils/showback','./quota-widget','tabs/provision-tab/vms/list','tabs/provision-tab/templates/list','tabs/provision-tab/flows/list','hbs!./quota-widget/html','hbs!./list'],function(require) {
  require('foundation.alert');
  var OpenNebula = require('opennebula');
  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var Notifier = require('utils/notifier');
  var QuotaWidgets = require('utils/quotas/quota-widgets');
  var QuotaLimits = require('utils/quotas/quota-limits');
  var QuotaDefaults = require('utils/quotas/quota-defaults');
  var Accounting = require('utils/accounting');
  var Showback = require('utils/showback');

  var ProvisionQuotaWidget = require('./quota-widget');
  var ProvisionVmsList = require('tabs/provision-tab/vms/list');
  var ProvisionTemplatesList = require('tabs/provision-tab/templates/list');
  var ProvisionFlowsList = require('tabs/provision-tab/flows/list');

  var TemplateProvisionQuotaWidget = require('hbs!./quota-widget/html');
  var TemplateUsersList = require('hbs!./list');

  var _accordionId = 0;

  return {
    'generate': generate_provision_users_list,
    'show': show_provision_user_list
  };

  function show_provision_user_list(timeout) {
    $(".section_content").hide();
    $(".provision_users_list_section").fadeIn();

    $("dd:not(.active) .provision_back", $(".provision_users_list_section")).trigger("click");
    $(".provision_users_list_refresh_button", $(".provision_users_list_section")).trigger("click");
  }

  function generate_provision_users_list(context, opts) {
    context.off();
    context.html(html(opts));
    setup_provision_users_list(context);
    setup_provision_user_info(context);
  }

  function html(opts_arg) {
    _accordionId += 1;
    return TemplateUsersList({'accordionId': _accordionId});
  }

  function update_provision_users_datatable(datatable, timeout) {
    datatable.html('<div class="text-center">' +
      '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
        '<i class="fa fa-cloud fa-stack-2x"></i>' +
        '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>' +
      '</span>' +
      '<br>' +
      '<br>' +
      '<span style="font-size: 18px; color: #999">' +
      '</span>' +
      '</div>');

    setTimeout(function() {
      OpenNebula.User.list({
        timeout: true,
        success: function (request, item_list, quotas_list) {
          datatable.fnClearTable(true);
          if (item_list.length == 0) {
            datatable.html('<div class="text-center">' +
              '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
                '<i class="fa fa-cloud fa-stack-2x"></i>' +
                '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>' +
              '</span>' +
              '<br>' +
              '<br>' +
              '<span style="font-size: 18px; color: #999">' +
                Locale.tr("The list of users is empty") +
              '</span>' +
              '</div>');
          } else {
            provision_quotas_list = quotas_list;
            datatable.fnAddData(item_list);
          }
        },
        error: Notifier.onError
      })
    }, timeout);
  }

  function setup_provision_users_list(context) {
    var provision_users_datatable = $('.provision_users_table', context).dataTable({
      "iDisplayLength": 6,
      "sDom" : '<"H">t<"F"lp>',
      "aLengthMenu": [[6, 12, 36, 72], [6, 12, 36, 72]],
      "aaSorting"  : [[0, "desc"]],
      "aoColumnDefs": [
          {"bVisible": false, "aTargets": ["all"]}
      ],
      "aoColumns": [
          {"mDataProp": "USER.ID"},
          {"mDataProp": "USER.NAME"}
      ],
      "fnPreDrawCallback": function (oSettings) {
        // create a thumbs container if it doesn't exist. put it in the dataTables_scrollbody div
        if (this.$('tr', {"filter": "applied"}).length == 0) {
          this.html('<div class="text-center">' +
            '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
              '<i class="fa fa-cloud fa-stack-2x"></i>' +
              '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>' +
            '</span>' +
            '<br>' +
            '<br>' +
            '<span style="font-size: 18px; color: #999">' +
              Locale.tr("The list of users is empty") +
            '</span>' +
            '</div>');
        } else {
          $(".provision_users_table", context).html('<ul class="provision_users_ul large-block-grid-3 medium-block-grid-3 small-block-grid-1 text-center"></ul>');
        }

        return true;
      },
      "fnRowCallback": function(nRow, aData, iDisplayIndex, iDisplayIndexFull) {
        var data = aData.USER;
        //var state = get_provision_vm_state(data);
        var vms = "";
        var memory = "";
        var cpu = "";

        var quotas_html;
        QuotaWidgets.initEmptyQuotas(data);

        if (!$.isEmptyObject(data.VM_QUOTA)) {
          var default_user_quotas = QuotaDefaults.getDefaultUserQuotas();

          quotas = QuotaWidgets.quotaFloatInfo(
              data.VM_QUOTA.VM.VMS_USED,
              data.VM_QUOTA.VM.VMS,
              default_user_quotas.VM_QUOTA.VM.VMS,
              true);

          quotas_html = "";
          quotas_html += '<li class="provision-bullet-item text-left">' +
            '<i class="fa fa-fw fa-th"></i> '+ Locale.tr("VMs") +
            '<span class="right">' + quotas.str + "</span>" +
          '</li>';

          quotas = QuotaWidgets.quotaFloatInfo(
              data.VM_QUOTA.VM.CPU_USED,
              data.VM_QUOTA.VM.CPU,
              default_user_quotas.VM_QUOTA.VM.CPU,
              true);

          quotas_html += '<li class="provision-bullet-item text-left">' +
            '<i class="fa fa-fw fa-tachometer"></i> '+ Locale.tr("CPU") +
            '<span class="right">' + quotas.str + "</span>" +
          '</li>';

          quotas = QuotaWidgets.quotaMBInfo(
              data.VM_QUOTA.VM.MEMORY_USED,
              data.VM_QUOTA.VM.MEMORY,
              default_user_quotas.VM_QUOTA.VM.MEMORY,
              true);

          quotas_html += '<li class="provision-bullet-item text-left">' +
            '<i class="fa fa-fw fa-align-left"></i> '+ Locale.tr("Memory") +
            '<span class="right">' + quotas.str + "</span>" +
          '</li>';
        } else {
          quotas = QuotaWidgets.quotaFloatInfo(0, 0, null, true);

          quotas_html = "";
          quotas_html += '<li class="provision-bullet-item text-left">' +
            '<i class="fa fa-fw fa-th"></i> '+ Locale.tr("VMs") +
            '<span class="right">' + quotas.str + "</span>" +
          '</li>';

          quotas = QuotaWidgets.quotaFloatInfo(0, 0, null, true);

          quotas_html += '<li class="provision-bullet-item text-left">' +
            '<i class="fa fa-fw fa-tachometer"></i> '+ Locale.tr("CPU") +
            '<span class="right">' + quotas.str + "</span>" +
          '</li>';

          quotas = QuotaWidgets.quotaMBInfo(0, 0, null, true);

          quotas_html += '<li class="provision-bullet-item text-left">' +
            '<i class="fa fa-fw fa-align-left"></i> '+ Locale.tr("Memory") +
            '<span class="right">' + quotas.str + "</span>" +
          '</li>';
        }

        $(".provision_users_ul", context).append('<li>' +
            '<ul class="provision-pricing-table" opennebula_id="' + data.ID + '" datatable_index="' + iDisplayIndexFull + '">' +
              '<li class="provision-title text-left" style="padding-bottom: 10px">' +
                '<a class="provision_info_user_button" style="color:#555" href="#"><i class="fa fa-fw fa-lg fa-sign-in right only-on-hover"/>' + data.NAME + '</a>' +
              '</li>' +
                quotas_html +
            '</ul>' +
          '</li>');

        return nRow;
      }
    });

    $('.provision_list_users_search', context).on('input',function(){
      provision_users_datatable.fnFilter($(this).val());
    })

    context.on("click", ".provision_users_list_refresh_button", function() {
      OpenNebula.Action.clear_cache("USER");
      update_provision_users_datatable(provision_users_datatable, 0);
      return false;
    });

    $(document).foundation();
  }

  function setup_provision_user_info(context) {
    function update_provision_vdc_user_info(user_id, context) {
      $(".provision_info_vdc_user_name", context).text("");
      $(".provision_vdc_info_container", context).html("");
      $(".provision_info_vdc_user", context).hide();
      $(".provision_info_vdc_user_loading", context).fadeIn();

      OpenNebula.User.show({
        data : {
          id: user_id
        },
        error: Notifier.onError,
        success: function(request, response) {
          var data = response.USER

          $(".provision_vdc_user_confirm_action", context).html("");
          $(".provision_info_vdc_user_acct", context).html("");

          $(".provision_info_vdc_user", context).attr("opennebula_id", data.ID);
          $(".provision_info_vdc_user", context).attr("uname", data.NAME);
          $(".provision_info_vdc_user", context).attr("quotas", JSON.stringify(data.VM_QUOTA));
          $(".provision_info_vdc_user_name", context).text(data.NAME);

          $(".provision-pricing-table_user_info", context).html("");

          QuotaWidgets.initEmptyQuotas(data);

          if (!$.isEmptyObject(data.VM_QUOTA)) {
            var default_user_quotas = QuotaDefaults.default_quotas(data.DEFAULT_USER_QUOTAS);
            vms_quotas = QuotaWidgets.quotaFloatInfo(
                data.VM_QUOTA.VM.VMS_USED,
                data.VM_QUOTA.VM.VMS,
                default_user_quotas.VM_QUOTA.VM.VMS,
                true);

            cpu_quotas = QuotaWidgets.quotaFloatInfo(
                data.VM_QUOTA.VM.CPU_USED,
                data.VM_QUOTA.VM.CPU,
                default_user_quotas.VM_QUOTA.VM.CPU,
                true);

            mem_quotas = QuotaWidgets.quotaMBInfo(
                data.VM_QUOTA.VM.MEMORY_USED,
                data.VM_QUOTA.VM.MEMORY,
                default_user_quotas.VM_QUOTA.VM.MEMORY,
                true);

            $(".provision-pricing-table_user_info", context).append('<li class="provision-bullet-item text-left">' +
              '<div class="large-4 columns">'+
                '<i class="fa fa-fw fa-th"></i> '+ Locale.tr("VMs") +
                '<span class="right">' + vms_quotas.str + "</span>" +
              '</div>'+
              '<div class="large-4 columns">'+
                '<i class="fa fa-fw fa-tachometer"></i> '+ Locale.tr("CPU") +
                '<span class="right">' + cpu_quotas.str + "</span>" +
              '</div>'+
              '<div class="large-4 columns">'+
                '<i class="fa fa-fw fa-align-left"></i> '+ Locale.tr("Memory") +
                '<span class="right">' + mem_quotas.str + "</span>" +
              '</div>'+
            '</li>' +
            '<li class="provision-bullet-item text-left">' +
              '<div class="large-4 columns">'+
                '<div class="progress small radius" style="background: #f7f7f7">' +
                  '  <span class="meter" style="width: ' + vms_quotas.percentage + '%;"></span>' +
                '</div>' +
              '</div>'+
              '<div class="large-4 columns">'+
                '<div class="progress small radius" style="background: #f7f7f7">' +
                  '  <span class="meter" style="width: ' + cpu_quotas.percentage + '%;"></span>' +
                '</div>' +
              '</div>'+
              '<div class="large-4 columns">'+
                '<div class="progress small radius" style="background: #f7f7f7">' +
                  '  <span class="meter" style="width: ' + mem_quotas.percentage + '%;"></span>' +
                '</div>' +
              '</div>'+
            '</li>');
          } else {
            vms_quotas = QuotaWidgets.quotaFloatInfo(0, 0, null, true);
            cpu_quotas = QuotaWidgets.quotaFloatInfo(0, 0, null, true);
            mem_quotas = QuotaWidgets.quotaMBInfo(0, 0, null, true);

            $(".provision-pricing-table_user_info", context).append('<li class="provision-bullet-item text-left">' +
              '<div class="large-4 columns">'+
                '<i class="fa fa-fw fa-th"></i> '+ Locale.tr("VMs") +
                '<span class="right">' + vms_quotas.str + "</span>" +
              '</div>'+
              '<div class="large-4 columns">'+
                '<i class="fa fa-fw fa-tachometer"></i> '+ Locale.tr("CPU") +
                '<span class="right">' + cpu_quotas.str + "</span>" +
              '</div>'+
              '<div class="large-4 columns">'+
                '<i class="fa fa-fw fa-align-left"></i> '+ Locale.tr("Memory") +
                '<span class="right">' + mem_quotas.str + "</span>" +
              '</div>'+
            '</li>' +
            '<li class="provision-bullet-item text-left">' +
              '<div class="large-4 columns">'+
                '<div class="progress small radius" style="background: #f7f7f7">' +
                  '  <span class="meter" style="width: ' + vms_quotas.percentage + '%;"></span>' +
                '</div>' +
              '</div>'+
              '<div class="large-4 columns">'+
                '<div class="progress small radius" style="background: #f7f7f7">' +
                  '  <span class="meter" style="width: ' + cpu_quotas.percentage + '%;"></span>' +
                '</div>' +
              '</div>'+
              '<div class="large-4 columns">'+
                '<div class="progress small radius" style="background: #f7f7f7">' +
                  '  <span class="meter" style="width: ' + mem_quotas.percentage + '%;"></span>' +
                '</div>' +
              '</div>'+
            '</li>');
          }

          $(".provision-user-resource-header", context).html(
                '<span class="provision_vdc_user_info_show_vms button medium radius" title="' + Locale.tr("User Virtual Machines") + '" style="margin-right: 10px">' +
                  '<i class="fa fa-th fa-lg"></i>' +
                '</span>' +
                '<span class="provision_vdc_user_info_show_templates button medium radius" title="' + Locale.tr("User Saved Templates") + '" style="margin-right: 10px">' +
                  '<i class="fa fa-save fa-lg"></i>' +
                '</span>' +
                '<span class="provision_vdc_user_info_show_flows button medium radius" title="' + Locale.tr("User Services") + '" style="margin-right: 10px">' +
                  '<i class="fa fa-cubes fa-lg"></i>' +
                '</span>' +
                '<span class="provision_vdc_user_info_show_acct button medium radius" title="' + Locale.tr("User Accounting") + '" style="margin-right: 10px">' +
                  '<i class="fa fa-bar-chart-o fa-lg"></i>' +
                '</span>' +
                (Config.isFeatureEnabled("showback") ? '<span class="provision_vdc_user_info_show_showback button medium radius" title="' + Locale.tr("User Showback") + '" style="margin-right: 10px">' +
                  '<i class="fa fa-money fa-lg"></i>' +
                '</span>' : ''))

          var start_time =  Math.floor(new Date().getTime() / 1000);
          // ms to s

          // 604800 = 7 days = 7*24*60*60
          start_time = start_time - 604800;

          // today
          var end_time = -1;

          var options = {
            "start_time": start_time,
            "end_time": end_time,
            "userfilter": user_id
          }

          var no_table = true;

          OpenNebula.VM.accounting({
            success: function(req, response) {
              Accounting.fillAccounting($(".dashboard_vm_accounting", context), req, response, no_table);
            },
            error: Notifier.onError,
            data: options
          });

          $(".provision_info_vdc_user", context).show();
          $(".provision_info_vdc_user_loading", context).hide();

          $(document).foundation();
          //$("#provision_info_vdc_quotas").html(quotas_html);
        }
      })
    }
    //
    // Info User
    //

    $(".provision_list_users", context).on("click", ".provision_info_user_button", function() {
      $("a.provision_show_user_accordion", context).trigger("click");
      // TODO loading

      var user_id = $(this).parents(".provision-pricing-table").attr("opennebula_id")
      update_provision_vdc_user_info(user_id, context);
    })

    context.on("click", ".provision_vdc_user_info_show_vms", function() {
      $(".provision_vdc_info_container", context).html('<div class="text-center">' +
        '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
          '<i class="fa fa-cloud fa-stack-2x"></i>' +
          '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>' +
        '</span>' +
        '<br>' +
        '<br>' +
        '<span style="font-size: 18px; color: #999">' +
        '</span>' +
        '</div>');

      ProvisionVmsList.generate(
        $(".provision_vdc_info_container", context),
        {
          title:  $(".provision_info_vdc_user", context).attr("uname") + ' ' + Locale.tr("VMs"),
          active: true,
          refresh: true,
          create: false,
          filter: false,
          filter_expression:  $(".provision_info_vdc_user", context).attr("opennebula_id")
        });
    })

    context.on("click", ".provision_vdc_user_info_show_templates", function() {
      $(".provision_vdc_info_container", context).html('<div class="text-center">' +
        '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
          '<i class="fa fa-cloud fa-stack-2x"></i>' +
          '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>' +
        '</span>' +
        '<br>' +
        '<br>' +
        '<span style="font-size: 18px; color: #999">' +
        '</span>' +
        '</div>');

      ProvisionTemplatesList.generate(
        $(".provision_vdc_info_container", context),
        {
          title:  $(".provision_info_vdc_user", context).attr("uname") + ' ' + Locale.tr("Templates"),
          active: true,
          refresh: true,
          create: false,
          filter: false,
          filter_expression:  $(".provision_info_vdc_user", context).attr("opennebula_id")
        });
    })

    context.on("click", ".provision_vdc_user_info_show_flows", function() {
      $(".provision_vdc_info_container", context).html('<div class="text-center">' +
        '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
          '<i class="fa fa-cloud fa-stack-2x"></i>' +
          '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>' +
        '</span>' +
        '<br>' +
        '<br>' +
        '<span style="font-size: 18px; color: #999">' +
        '</span>' +
        '</div>');

      ProvisionFlowsList.generate(
        $(".provision_vdc_info_container", context),
        {
          title:  $(".provision_info_vdc_user", context).attr("uname") + ' ' + Locale.tr("Services"),
          active: true,
          refresh: true,
          create: false,
          filter: false,
          filter_expression:  $(".provision_info_vdc_user", context).attr("opennebula_id")
        });
    })

    context.on("click", ".provision_vdc_user_info_show_acct", function() {
      $(".provision_vdc_info_container", context).html("");

      $(".provision_vdc_info_container", context).html(Accounting.html());
      Accounting.setup(
        $(".provision_vdc_info_container", context),
          {fixed_user: $(".provision_info_vdc_user", context).attr("opennebula_id"),
            init_group_by: "vm"});

      $(".provision_vdc_info_container", context).prepend(
        '<h2 class="subheader">' +
          $(".provision_info_vdc_user", context).attr("uname") + ' ' + Locale.tr("Accounting") +
        '</h2>')
    })

    if (Config.isFeatureEnabled("showback")) {
      context.on("click", ".provision_vdc_user_info_show_showback", function() {
        $(".provision_vdc_info_container", context).html("");

        $(".provision_vdc_info_container", context).html(Showback.html());
        Showback.setup(
          $(".provision_vdc_info_container", context),
            {fixed_user: $(".provision_info_vdc_user", context).attr("opennebula_id"),
              fixed_group: ""});

        $(".provision_vdc_info_container", context).prepend(
          '<h2 class="subheader">' +
            $(".provision_info_vdc_user", context).attr("uname") + ' ' + Locale.tr("Showback") +
          '</h2>')
      })
    };

    context.on("click", ".provision_vdc_user_delete_confirm_button", function() {
      $(".provision_vdc_user_confirm_action", context).html(
        '<div data-alert class="alert-box secondary radius">' +
          '<div class="row">' +
            '<div class="large-10 large-centered columns">' +
              '<span style="font-size: 14px; line-height: 20px">' +
                Locale.tr("Be careful, this action will inmediately remove the User from OpenNebula") +
              '</span>' +
            '</div>' +
          '</div>' +
          '<br>' +
          '<div class="row">' +
            '<div class="large-10 large-centered columns">' +
              '<a href"#" class="provision_delete_button alert button large-12 large radius">' + Locale.tr("Delete User") + '</a>' +
            '</div>' +
          '</div>' +
          '<a href="#" class="close" style="top: 20px">&times;</a>' +
        '</div>');
    });

    context.on("click", ".provision_vdc_user_password_confirm_button", function() {
      $(".provision_vdc_user_confirm_action", context).html(
        '<div data-alert class="alert-box secondary radius">' +
          '<div class="row">' +
            '<div class="large-10 large-centered columns">' +
              '<input type="password" class="provision_vdc_user_new_password provision-input" placeholder="' + Locale.tr("New Password") + '" style="height: 40px !important; font-size: 16px; padding: 0.5rem  !important;"/>' +
            '</div>' +
          '</div>' +
          '<div class="row">' +
            '<div class="large-10 large-centered columns">' +
              '<input type="password" class="provision_vdc_user_new_confirm_password provision-input" placeholder="' + Locale.tr("Confirm Password") + '" style="height: 40px !important; font-size: 16px; padding: 0.5rem  !important;"/>' +
              '<br>' +
            '</div>' +
          '</div>' +
          '<div class="row">' +
            '<div class="large-10 large-centered columns">' +
              '<button href"#" type="submit" class="provision_vdc_user_change_password_button button success large radius large-12 small-12">' + Locale.tr("Update Password") + '</button>' +
            '</div>' +
          '</div>' +
          '<a href="#" class="close" style="top: 20px">&times;</a>' +
        '</div>');

      context.on("click", ".provision_vdc_user_change_password_button", function() {
          var button = $(this);
          button.attr("disabled", "disabled");
          var user_id = $(".provision_info_vdc_user", context).attr("opennebula_id");
          var pw = $('.provision_vdc_user_new_password', context).val();
          var confirm_password = $('.provision_vdc_user_new_confirm_password', context).val();

          if (!pw.length) {
            Notifier.notifyError(Locale.tr("Fill in a new password"));
            return false;
          }

          if (pw !== confirm_password) {
            Notifier.notifyError(Locale.tr("Passwords do not match"));
            return false;
          }

          OpenNebula.User.passwd({
            data : {
              id: user_id,
              extra_param: pw
            },
            success: function(request, response) {
              update_provision_vdc_user_info(user_id, context);
              button.removeAttr("disabled");
            },
            error: function(request, response) {
              Notifier.onError(request, response);
              button.removeAttr("disabled");
            }
          })
          return false;
        });
    });

    context.on("click", ".provision_vdc_user_quota_confirm_button", function() {
      $(".provision_vdc_user_confirm_action", context).html(
        '<div data-alert class="alert-box secondary radius">' +
          TemplateProvisionQuotaWidget() +
          '<br>' +
          '<br>' +
          '<div class="row">' +
            '<div class="large-10 large-centered columns">' +
              '<a href"#" class="provision_update_quota_button success large button large-12 radius" style="margin-right: 15px">' + Locale.tr("Update User Quota") + '</a>' +
            '</div>' +
          '</div>' +
          '<a href="#" class="close" style="top: 20px">&times;</a>' +
        '</div>');

      ProvisionQuotaWidget.setup(context);

      $(document).foundation();

      var quotas_str = $(".provision_info_vdc_user", context).attr("quotas");
      if (quotas_str) {
        var quotas = JSON.parse(quotas_str);

        var vms_limit = QuotaLimits.QUOTA_LIMIT_DEFAULT;
        var cpu_limit = QuotaLimits.QUOTA_LIMIT_DEFAULT;
        var mem_limit = QuotaLimits.QUOTA_LIMIT_DEFAULT;

        if (quotas.VM != undefined) {
          vms_limit = quotas.VM.VMS;
          cpu_limit = quotas.VM.CPU;
          mem_limit = quotas.VM.MEMORY;

          if (mem_limit != QuotaLimits.QUOTA_LIMIT_UNLIMITED &&
             mem_limit != QuotaLimits.QUOTA_LIMIT_DEFAULT) {

            mem_limit = quotas.VM.MEMORY / 1024;
          }
        }

        var fill_limits = function(limit, select, input) {
            switch (limit){
              case QuotaLimits.QUOTA_LIMIT_DEFAULT:
                select.val('default').change();
                input.val('').change();
                break;

              case QuotaLimits.QUOTA_LIMIT_UNLIMITED:
                select.val('unlimited').change();
                input.val('').change();
                break;

              default:
                select.val('edit').change();
                input.val(limit).change();
            }
          }

        fill_limits(
          vms_limit,
          $("div.provision_rvms_quota select.provision_quota_select", context),
          $(".provision_rvms_quota_input", context));

        fill_limits(
          cpu_limit,
          $("div.provision_cpu_quota select.provision_quota_select", context),
          $(".provision_cpu_quota_input", context));

        fill_limits(
          mem_limit,
          $("div.provision_memory_quota select.provision_quota_select", context),
          $(".provision_memory_quota_tmp_input", context));
      }
    });

    context.on("click", ".provision_delete_button", function() {
      var button = $(this);
      button.attr("disabled", "disabled");
      var user_id = $(".provision_info_vdc_user", context).attr("opennebula_id");
      OpenNebula.User.del({
        data : {
          id: user_id
        },
        success: function(request, response) {
          $(".provision_back", context).click();
          $(".provision_users_list_refresh_button", context).click();
          button.removeAttr("disabled");
        },
        error: function(request, response) {
          Notifier.onError(request, response);
          button.removeAttr("disabled");
        }
      })
    });

    context.on("click", ".provision_update_quota_button", function() {
      var button = $(this);
      button.attr("disabled", "disabled");
      var user_id = $(".provision_info_vdc_user", context).attr("opennebula_id");

      quota_json = ProvisionQuotaWidget.retrieve(context);

      OpenNebula.User.set_quota({
        data : {
          id: user_id,
          extra_param: quota_json
        },
        success: function(request, response) {
          update_provision_vdc_user_info(user_id, context);
          button.removeAttr("disabled");
        },
        error: function(request, response) {
          Notifier.onError(request, response);
          button.removeAttr("disabled");
        }
      })
    });

    context.on("click", ".provision_refresh_info", function() {
      var user_id = $(".provision_info_vdc_user", context).attr("opennebula_id");
      update_provision_vdc_user_info(user_id, context);
      return false;
    });
  }
});


/* START_TEMPLATE */
define('hbs!tabs/provision-tab/vms/create',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form id=\"provision_create_vm\" class=\"hidden section_content\">\n  <div class=\"row\">\n    <div class=\"large-10 large-centered columns\">\n      <h2 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Create Virtual Machine", {"name":"tr","hash":{},"data":data})))
    + "</h2>\n      <br></div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-8 large-centered columns\">\n      <input type=\"text\" id=\"vm_name\"  class=\"provision-input\" placeholder=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Machine Name", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"height: 40px !important; font-size: 16px; padding: 0.5rem  !important;\"/>\n      <br></div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-10 large-centered columns\">\n      <dl class=\"accordion provision_accordion_template\" data-accordion=\"provision_accordion_template\">\n        <dd style=\"border-bottom: 1px solid #efefef;\" class=\"active accordion-navigation\">\n          <a href=\"#provision_dd_template\" style=\"background: #fff; font-size: 30px\">\n            <span class=\"select_template\" style=\"color:#555\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select a Template", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            <span class=\"selected_template\" style=\"display:none; color:#555\">\n              <span class=\"provision-logo selected_template_logo\" style=\"line-height: 80px\"></span>\n              <span class=\"selected_template_name\"></span>\n              <span class=\"has-tip right\" style=\"cursor: pointer; margin-right:10px; line-height: 80px\"> <i class=\"fa fa-pencil\"/></span>\n            </span>\n          </a>\n          <div id=\"provision_dd_template\" class=\"active content\">\n            <div class=\"row provision_select_template\">\n              <div class=\"large-12 large-centered columns\">\n                <dl class=\"tabs text-center\" data-tab style=\"width: 100%\">\n                  <dd class=\"active\" style=\"width: 33%;\">\n                    <a href=\"#provision_system_templates_selector\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "System", {"name":"tr","hash":{},"data":data})))
    + "</a>\n                  </dd>\n                  <dd style=\"width: 33%;\">\n                    <a href=\"#provision_vdc_templates_selector\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group", {"name":"tr","hash":{},"data":data})))
    + "</a>\n                  </dd>\n                  <dd style=\"width: 34%;\">\n                    <a href=\"#provision_saved_templates_selector\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Saved", {"name":"tr","hash":{},"data":data})))
    + "</a>\n                  </dd>\n                </dl>\n                <div class=\"row\">\n                  <div class=\"large-12 large-centered columns\">\n                    <h3 class=\"subheader text-right\">\n                      <input type=\"search\" class=\"provision-search-input right\" placeholder=\"Search\" id=\"provision_create_template_search\"/>\n                    </h3>\n                    <br></div>\n                </div>\n                <div class=\"tabs-content\">\n                  <div class=\"content active\" id=\"provision_system_templates_selector\">\n                    <table id=\"provision_system_templates_table\">\n                      <thead class=\"hidden\">\n                        <tr>\n                          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Saved", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                        </tr>\n                      </thead>\n                      <tbody class=\"hidden\"></tbody>\n                    </table>\n                  </div>\n                  <div class=\"content\" id=\"provision_vdc_templates_selector\">\n                    <table id=\"provision_vdc_templates_table\">\n                      <thead class=\"hidden\">\n                        <tr>\n                          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Saved", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shared", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                        </tr>\n                      </thead>\n                      <tbody class=\"hidden\"></tbody>\n                    </table>\n                  </div>\n                  <div class=\"content\" id=\"provision_saved_templates_selector\">\n                    <table id=\"provision_saved_templates_table\">\n                      <thead class=\"hidden\">\n                        <tr>\n                          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Saved", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shared", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                        </tr>\n                      </thead>\n                      <tbody class=\"hidden\"></tbody>\n                    </table>\n                  </div>\n                </div>\n              </div>\n            </div>\n          </div>\n        </dd>\n      </dl>\n    </div>\n  </div>\n  <br>\n  <div class=\"row\">\n    <div class=\"large-10 large-centered columns\">\n      <div class=\"provision_capacity_selector\"></div>\n      <br>\n      <div class=\"provision_disk_selector\"></div>\n      <br>\n      <div class=\"provision_network_selector\"></div>\n      <br>\n      <div class=\"provision_custom_attributes_selector\"></div>\n    </div>\n  </div>\n  <br>\n  <br>\n  <br>\n  <div class=\"row\">\n    <div class=\"large-7 columns large-centered\">\n      <div data-alert class=\"alert-box alert-box-error radius text-center hidden\"></div>\n      <button href=\"#\" class=\"button large success radius large-12 small-12\" type=\"submit\" style=\"height: 59px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Create", {"name":"tr","hash":{},"data":data})))
    + "</button>\n    </div>\n  </div>\n  <br>\n  <br></form>";
},"useData":true});
Handlebars.registerPartial('tabs/provision-tab/vms/create', t);
Handlebars.registerPartial('tabs/provision-tab/vms/create', t);
return t;
});
/* END_TEMPLATE */
;
define('templates/helpers/isFeatureEnabled',['require','hbs/handlebars','sunstone-config'],function(require) {
  var Handlebars = require('hbs/handlebars');
  var Config = require('sunstone-config');

  var isFeatureEnabled = function(feature, options) {
    if (Config.isFeatureEnabled(feature)) {
      return options.fn(this);
    } else {
      return options.inverse(this);
    }
  };

  Handlebars.registerHelper('isFeatureEnabled', isFeatureEnabled);

  return isFeatureEnabled;
})
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/group/info',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/isFeatureEnabled'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "  <div class=\"row\">\n    <div class=\"large-11 large-centered columns\">\n      <h3 class=\"subheader text-right\">\n        <span class=\"left\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group Showback", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </h3>\n    </div>\n  </div>\n  <br>\n  <div class=\"row\">\n    <div  id=\"provision_info_vdc_group_showback\" class=\"large-10 large-centered columns\"></div>\n  </div>\n  <br>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\"provision_manage_vdc\" class=\"hidden section_content\">\n  <div class=\"row\">\n    <div class=\"large-11 large-centered columns\">\n      <h3 class=\"subheader text-right\">\n        <span class=\"left\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group Accounting", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </h3>\n    </div>\n  </div>\n  <br>\n  <div class=\"row\">\n    <div  id=\"provision_info_vdc_group_acct\" class=\"large-10 large-centered columns\"></div>\n  </div>\n  <br>\n";
  stack1 = ((helpers.isFeatureEnabled || (depth0 && depth0.isFeatureEnabled) || helperMissing).call(depth0, "showback", {"name":"isFeatureEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  <div class=\"row\">\n    <div class=\"large-11 large-centered columns\">\n      <h3 class=\"subheader text-right\">\n        <span class=\"left\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group Quotas", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </h3>\n    </div>\n  </div>\n  <br>\n  <div class=\"row\">\n    <div class=\"large-11 columns large-centered quotas\" id=\"provision_vdc_quotas_div\"></div>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/provision-tab/group/info', t);
Handlebars.registerPartial('tabs/provision-tab/group/info', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/users/create',['hbs','hbs/handlebars', 'hbs!tabs/provision-tab/users/quota-widget/html','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form id=\"provision_create_user\" class=\"hidden section_content\">\n  <div class=\"row\">\n    <div class=\"large-10 large-centered columns\">\n      <h2 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Create User", {"name":"tr","hash":{},"data":data})))
    + "</h2>\n      <br></div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-8 large-centered columns\">\n      <input type=\"text\" id=\"username\"  class=\"provision-input\" placeholder=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Username", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"height: 40px !important; font-size: 16px; padding: 0.5rem  !important;\"/>\n      <br></div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-8 large-centered columns\">\n      <input type=\"password\" id=\"password\"  class=\"provision-input\" placeholder=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Password", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"height: 40px !important; font-size: 16px; padding: 0.5rem  !important;\"/>\n      <br></div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-8 large-centered columns\">\n      <input type=\"password\" id=\"repeat_password\"  class=\"provision-input\" placeholder=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Repeat Password", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"height: 40px !important; font-size: 16px; padding: 0.5rem  !important;\"/>\n      <br></div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-10 large-centered columns\">\n      <h3 class=\"subheader text-right\">\n        <span class=\"left\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Define Quotas", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </h3>\n      <br></div>\n  </div>\n  <br>\n  <div class=\"row\">\n    <div class=\"large-8 large-centered columns\">\n      <dl class=\"tabs text-center\" data-tab style=\"width: 100%\">\n        <dd class=\"active\" style=\"width: 50%;\">\n          <a href=\"#provision_create_user_default_quota\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Default", {"name":"tr","hash":{},"data":data})))
    + "</a>\n        </dd>\n        <dd style=\"width: 50%;\">\n          <a href=\"#provision_create_user_manual_quota\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Manual", {"name":"tr","hash":{},"data":data})))
    + "</a>\n        </dd>\n      </dl>\n      <div class=\"tabs-content\">\n        <div class=\"content active\" id=\"provision_create_user_default_quota\">\n          <div class=\"row\">\n            <div class=\"large-12 large-centered columns\">\n              <span style=\"font-size: 18px; color: #999\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Use the default system quotas set by the cloud adminstrator", {"name":"tr","hash":{},"data":data})))
    + "\n              </span>\n            </div>\n          </div>\n        </div>\n        <div class=\"content\" id=\"provision_create_user_manual_quota\">";
  stack1 = this.invokePartial(partials['tabs/provision-tab/users/quota-widget/html'], '', 'tabs/provision-tab/users/quota-widget/html', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n      </div>\n    </div>\n  </div>\n  <br>\n  <br>\n  <div class=\"row\">\n    <div class=\"large-5 large-centered columns\">\n      <hr>\n      <br></div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-7 columns large-centered\">\n      <div data-alert class=\"alert-box alert-box-error radius text-center hidden\"></div>\n      <button href=\"#\" class=\"button large radius large-12 small-12\" type=\"submit\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add User", {"name":"tr","hash":{},"data":data})))
    + "</button>\n    </div>\n  </div>\n  <br></form>";
},"usePartial":true,"useData":true});
Handlebars.registerPartial('tabs/provision-tab/users/create', t);
Handlebars.registerPartial('tabs/provision-tab/users/create', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/flows/create',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form id=\"provision_create_flow\" class=\"hidden section_content\">\n  <div class=\"row\">\n    <div class=\"large-10 large-centered columns\">\n      <h2 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Create Service", {"name":"tr","hash":{},"data":data})))
    + "</h2>\n      <br></div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-8 large-centered columns\">\n      <input type=\"text\" id=\"flow_name\"  class=\"provision-input\" placeholder=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Service Name", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"height: 40px !important; font-size: 16px; padding: 0.5rem  !important;\"/>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-10 large-centered columns\">\n      <dl class=\"accordion provision_accordion_flow_template\" data-accordion=\"provision_accordion_flow_template\">\n        <dd style=\"border-bottom: 1px solid #efefef;\" class=\"active accordion-navigation\">\n          <a href=\"#provision_dd_flow_template\" style=\"background: #fff; font-size: 30px\">\n            <span class=\"select_template\" style=\"color:#555\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select a Template", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            <span class=\"selected_template\" style=\"display:none; color:#555\">\n              <span class=\"provision-logo selected_template_logo\" style=\"color:#555\"></span>\n              <span class=\"selected_template_name\" style=\"color:#555\"></span>\n              <span class=\"has-tip right\" style=\"cursor: pointer; margin-right:10px;\"> <i class=\"fa fa-pencil\"/></span>\n            </span>\n          </a>\n          <div id=\"provision_dd_flow_template\" class=\"provision_select_flow_template active content\">\n            <div class=\"row\">\n              <div class=\"large-12 large-centered columns\">\n                <h3 class=\"subheader text-right\">\n                  <input type=\"search\" class=\"provision-search-input right\" placeholder=\"Search\" id=\"provision_create_flow_template_search\"/>\n                </h3>\n                <br></div>\n            </div>\n            <div class=\"row\">\n              <div class=\"large-12 large-centered columns\">\n                <table id=\"provision_flow_templates_table\">\n                  <thead class=\"hidden\">\n                    <tr>\n                      <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                      <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                    </tr>\n                  </thead>\n                  <tbody class=\"hidden\"></tbody>\n                </table>\n              </div>\n            </div>\n          </div>\n        </dd>\n      </dl>\n    </div>\n  </div>\n  <br>\n  <br>\n  <br>\n  <div class=\"row\">\n    <div class=\"provision_network_selector large-9 large-centered columns\"></div>\n  </div>\n  <br>\n  <div class=\"row\">\n    <div class=\"provision_custom_attributes_selector large-9 large-centered columns\"></div>\n  </div>\n  <div id=\"provision_customize_flow_template\" style=\"display: none\"></div>\n  <br>\n  <br>\n  <div class=\"row\">\n    <div class=\"large-7 columns large-centered\">\n      <div data-alert class=\"alert-box alert-box-error radius text-center hidden\"></div>\n      <button href=\"#\" class=\"button large success radius large-12 small-12\" type=\"submit\" style=\"height: 59px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Create", {"name":"tr","hash":{},"data":data})))
    + "</button>\n    </div>\n  </div>\n  <br>\n  <br></form>";
},"useData":true});
Handlebars.registerPartial('tabs/provision-tab/flows/create', t);
Handlebars.registerPartial('tabs/provision-tab/flows/create', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/content',['hbs','hbs/handlebars', 'hbs!tabs/provision-tab/vms/create', 'hbs!tabs/provision-tab/group/info', 'hbs!tabs/provision-tab/users/create', 'hbs!tabs/provision-tab/flows/create','templates/helpers/isTabPanelEnabled'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  return "<div class=\"provision_templates_list_section hidden section_content\"></div>\n";
  },"3":function(depth0,helpers,partials,data) {
  var stack1, buffer = "";
  stack1 = this.invokePartial(partials['tabs/provision-tab/group/info'], '', 'tabs/provision-tab/group/info', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  stack1 = this.invokePartial(partials['tabs/provision-tab/users/create'], '', 'tabs/provision-tab/users/create', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "<div class=\"provision_users_list_section hidden section_content\"></div>\n";
},"5":function(depth0,helpers,partials,data) {
  var stack1, buffer = "";
  stack1 = this.invokePartial(partials['tabs/provision-tab/flows/create'], '', 'tabs/provision-tab/flows/create', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "<div class=\"provision_flows_list_section hidden section_content\"></div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, buffer = "<div id=\"provision_dashboard\" class=\"section_content\"></div>\n\n";
  stack1 = this.invokePartial(partials['tabs/provision-tab/vms/create'], '', 'tabs/provision-tab/vms/create', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  buffer += "<div class=\"provision_vms_list_section hidden section_content\"></div>\n\n";
  stack1 = ((helpers.isTabPanelEnabled || (depth0 && depth0.isTabPanelEnabled) || helperMissing).call(depth0, "provision-tab", "templates", {"name":"isTabPanelEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n";
  stack1 = ((helpers.isTabPanelEnabled || (depth0 && depth0.isTabPanelEnabled) || helperMissing).call(depth0, "provision-tab", "users", {"name":"isTabPanelEnabled","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n";
  stack1 = ((helpers.isTabPanelEnabled || (depth0 && depth0.isTabPanelEnabled) || helperMissing).call(depth0, "provision-tab", "flows", {"name":"isTabPanelEnabled","hash":{},"fn":this.program(5, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"usePartial":true,"useData":true});
Handlebars.registerPartial('tabs/provision-tab/content', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/header',['hbs','hbs/handlebars','templates/helpers/isTabPanelEnabled','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "        <li>\n          <a href\"#\" class=\"medium off-color\" id=\"provision_vdc_info_button\">\n            <i class=\"fa fa-fw fa-lg fa-bar-chart-o\"/>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group Info", {"name":"tr","hash":{},"data":data})))
    + "\n          </a>\n        </li>\n        <li>\n          <a href\"#\" class=\"medium off-color provision_users_list_button\" id=\"\">\n          <i class=\"fa fa-fw fa-lg fa-users\"/>\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Users", {"name":"tr","hash":{},"data":data})))
    + "\n          </a>\n        </li>\n";
},"3":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "        <li>\n          <a href\"#\" class=\"medium off-color provision_templates_list_button\">\n            <i class=\"fa fa-fw fa-lg fa-save\"/>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Templates", {"name":"tr","hash":{},"data":data})))
    + "\n          </a>\n        </li>\n";
},"5":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "        <li>\n          <a href\"#\" class=\"medium off-color provision_flows_list_button\" id=\"\">\n            <i class=\"fa fa-lg fa-fw fa-cubes\"/>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Services", {"name":"tr","hash":{},"data":data})))
    + "\n          </a>\n        </li>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div style=\" padding: 15px 0px 10px 0px; margin-bottom: 10px\">\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <ul class=\"inline-list provision-header text-center\">\n";
  stack1 = ((helpers.isTabPanelEnabled || (depth0 && depth0.isTabPanelEnabled) || helperMissing).call(depth0, "provision-tab", "users", {"name":"isTabPanelEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "        <li>\n          <a href\"#\" class=\"medium off-color provision_vms_list_button\" id=\"\">\n            <i class=\"fa fa-fw fa-lg fa-th\"/>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMs", {"name":"tr","hash":{},"data":data})))
    + "\n          </a>\n        </li>\n";
  stack1 = ((helpers.isTabPanelEnabled || (depth0 && depth0.isTabPanelEnabled) || helperMissing).call(depth0, "provision-tab", "templates", {"name":"isTabPanelEnabled","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  stack1 = ((helpers.isTabPanelEnabled || (depth0 && depth0.isTabPanelEnabled) || helperMissing).call(depth0, "provision-tab", "flows", {"name":"isTabPanelEnabled","hash":{},"fn":this.program(5, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "      </ul>\n    </div>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/provision-tab/header', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/dashboard/quotas',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"row\">\n  <div class=\"large-11 large-centered columns\">\n    <h2 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Quotas", {"name":"tr","hash":{},"data":data})))
    + "</h2>\n    <br></div>\n</div>\n<div class=\"row\">\n  <div class=\"large-10 large-centered columns\">\n    <div class=\"row\">\n      <div class=\"large-4 columns text-center\">\n        <span id=\"provision_dashboard_rvms_percentage\"  style=\"font-size:50px\">0</span>\n        <span style=\"font-size:20px; color: #999\">%</span></div>\n        <div class=\"large-4 columns text-center\">\n          <span id=\"provision_dashboard_cpu_percentage\"  style=\"font-size:50px\">0</span>\n          <span style=\"font-size:20px; color: #999\">%</span></div>\n          <div class=\"large-4 columns text-center\">\n            <span id=\"provision_dashboard_memory_percentage\"  style=\"font-size:50px\">0</span>\n            <span style=\"font-size:20px; color: #999\">%</span></div>\n          </div>\n          <div class=\"row\">\n            <div class=\"large-4 columns text-center\">\n              <div class=\"progress large radius\">\n                <span id=\"provision_dashboard_rvms_meter\" class=\"meter\"></span>\n              </div>\n            </div>\n            <div class=\"large-4 columns text-center\">\n              <div class=\"progress large radius\">\n                <span id=\"provision_dashboard_cpu_meter\" class=\"meter\"></span>\n              </div>\n            </div>\n            <div class=\"large-4 columns text-center\">\n              <div class=\"progress large radius\">\n                <span id=\"provision_dashboard_memory_meter\" class=\"meter\"></span>\n              </div>\n            </div>\n          </div>\n          <div class=\"row\">\n            <div class=\"large-4 columns text-center\">\n              <span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "RUNNING VMS", {"name":"tr","hash":{},"data":data})))
    + "</span>\n              <br>\n              <span id=\"provision_dashboard_rvms_str\" style=\"color: #999; font-size: 14px;\"></span>\n            </div>\n            <div class=\"large-4 columns text-center\">\n              <span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU", {"name":"tr","hash":{},"data":data})))
    + "</span>\n              <br>\n              <span id=\"provision_dashboard_cpu_str\" style=\"color: #999; font-size: 14px;\"></span>\n            </div>\n            <div class=\"large-4 columns text-center\">\n              <span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "MEMORY", {"name":"tr","hash":{},"data":data})))
    + "</span>\n              <br>\n              <span id=\"provision_dashboard_memory_str\" style=\"color: #999; font-size: 14px;\"></span>\n            </div>\n          </div>\n        </div>\n      </div>\n      <br>\n      <br>";
},"useData":true});
Handlebars.registerPartial('tabs/provision-tab/dashboard/quotas', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/dashboard/vdc-quotas',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"row\">\n  <div class=\"large-12 large-centered columns\">\n    <h3 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group Quotas", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n    <br></div>\n</div>\n<div class=\"row\">\n  <div class=\"large-12 large-centered columns\">\n    <div class=\"row\">\n      <div class=\"large-4 columns text-center\">\n        <span id=\"provision_dashboard_vdc_rvms_percentage\" style=\"font-size:50px\">0</span>\n        <span style=\"font-size:20px; color: #999\">%</span>\n      </div>\n      <div class=\"large-4 columns text-center\">\n        <span id=\"provision_dashboard_vdc_cpu_percentage\" style=\"font-size:50px\">0</span>\n        <span style=\"font-size:20px; color: #999\">%</span>\n      </div>\n      <div class=\"large-4 columns text-center\">\n        <span id=\"provision_dashboard_vdc_memory_percentage\" style=\"font-size:50px\">0</span>\n        <span style=\"font-size:20px; color: #999\">%</span>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-4 columns text-center\">\n        <div class=\"progress large radius\">\n          <span id=\"provision_dashboard_vdc_rvms_meter\" class=\"meter\"></span>\n        </div>\n      </div>\n      <div class=\"large-4 columns text-center\">\n        <div class=\"progress large radius\">\n          <span id=\"provision_dashboard_vdc_cpu_meter\" class=\"meter\"></span>\n        </div>\n      </div>\n      <div class=\"large-4 columns text-center\">\n        <div class=\"progress large radius\">\n          <span id=\"provision_dashboard_vdc_memory_meter\" class=\"meter\"></span>\n        </div>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-4 columns text-center\">\n        <span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "RUNNING VMS", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        <br>\n        <span id=\"provision_dashboard_vdc_rvms_str\" style=\"color: #999; font-size: 14px;\"></span>\n      </div>\n      <div class=\"large-4 columns text-center\">\n        <span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        <br>\n        <span id=\"provision_dashboard_vdc_cpu_str\" style=\"color: #999; font-size: 14px;\"></span>\n      </div>\n      <div class=\"large-4 columns text-center\">\n        <span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "MEMORY", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        <br>\n        <span id=\"provision_dashboard_vdc_memory_str\" style=\"color: #999; font-size: 14px;\"></span>\n      </div>\n    </div>\n  </div>\n</div>\n<br>\n<br>";
},"useData":true});
Handlebars.registerPartial('tabs/provision-tab/dashboard/vdc-quotas', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/dashboard/empty-graphs',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<span class=\"fa-stack fa-2x\" style=\"color: #dfdfdf\"> <i class=\"fa fa-cloud fa-stack-2x\"></i>\n</span>\n<br>\n<span style=\"color: #cfcfcf\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "There is no information available", {"name":"tr","hash":{},"data":data})))
    + "</span>";
},"useData":true});
Handlebars.registerPartial('tabs/provision-tab/dashboard/empty-graphs', t);
Handlebars.registerPartial('./empty-graphs', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/dashboard/vms',['hbs','hbs/handlebars', 'hbs!tabs/provision-tab/dashboard/empty-graphs','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row\">\n  <div class=\"large-11 large-centered columns\">\n    <h2 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Machines", {"name":"tr","hash":{},"data":data})))
    + "</h2>\n    <br></div>\n</div>\n<div class=\"row\" id=\"provision_vms_dashboard\">\n  <div class=\"large-11 large-centered columns\">\n    <div class=\"row\">\n      <div class=\"large-4 columns text-center\">\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <span  id=\"provision_dashboard_total\" style=\"font-size:80px\"> <i class=\"fa fa-spinner fa-spin\"></i>\n            </span>\n            <br>\n            <span style=\"color: #999;\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "TOTAL", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </div>\n        </div>\n        <br>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns text-center\">\n            <a href\"#\" class=\"medium button success large radius provision_create_vm_button\" style=\"margin-left: 5px; margin-right: 5px\"> <i class=\"fa fa-lg fa-plus-square\"/></a>\n            <a href\"#\" class=\"medium button large radius provision_vms_list_button\" style=\"margin-left: 5px; margin-right: 5px\">\n              <i class=\"fa fa-lg fa-th\"/></a>\n          </div>\n        </div>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns\"></div>\n        </div>\n      </div>\n      <div class=\"large-2 columns\">\n        <div class=\"row\">\n          <div class=\"large-12 columns text-center\">\n            <span id=\"provision_dashboard_running\" style=\"font-size:40px\">\n              <i class=\"fa fa-spinner fa-spin\"></i>\n            </span>\n            <br>\n            <span  style=\"color: #999; font-size: 14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "RUNNING", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </div>\n        </div>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns text-center \">\n            <span id=\"provision_dashboard_deploying\"  style=\"font-size:40px\">\n              <i class=\"fa fa-spinner fa-spin\"></i>\n            </span>\n            <br>\n            <span  style=\"color: #999; font-size: 14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "DEPLOYING", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </div>\n        </div>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns text-center\">\n            <span  id=\"provision_dashboard_off\" style=\"font-size:40px\">\n              <i class=\"fa fa-spinner fa-spin\"></i>\n            </span>\n            <br>\n            <span  style=\"color: #999; font-size: 14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "OFF", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </div>\n        </div>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns text-center \">\n            <span  id=\"provision_dashboard_error\" style=\"font-size:40px\">\n              <i class=\"fa fa-spinner fa-spin\"></i>\n            </span>\n            <br>\n            <span  style=\"color: #999; font-size: 14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ERROR", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </div>\n        </div>\n      </div>\n      <div class=\"large-6 columns\" id=\"dashboard_vm_accounting\">\n        <input style=\"display:none;\" value=\"vm\" id=\"acct_group_by\"/>\n        <div class=\"row\">\n          <div class=\"large-12 columns graph_legend\">\n            <h3 class=\"subheader\">\n              <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU hours", {"name":"tr","hash":{},"data":data})))
    + "</small>\n            </h3>\n          </div>\n          <div class=\"large-12 columns\">\n            <div class=\"large-12 columns centered graph text-center\" id=\"acct_cpu_graph\" style=\"height: 100px;\">";
  stack1 = this.invokePartial(partials['./empty-graphs'], '', './empty-graphs', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-12 columns graph_legend\">\n            <h3 class=\"subheader\">\n              <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Memory GB hours", {"name":"tr","hash":{},"data":data})))
    + "</small>\n            </h3>\n          </div>\n          <div class=\"large-12 columns\">\n            <div class=\"large-12 columns centered graph text-center\" id=\"acct_mem_graph\" style=\"height: 100px;\">";
  stack1 = this.invokePartial(partials['./empty-graphs'], '', './empty-graphs', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-12 columns graph_legend\">\n            <h3 class=\"subheader\">\n              <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk MB hours", {"name":"tr","hash":{},"data":data})))
    + "</small>\n            </h3>\n          </div>\n          <div class=\"large-12 columns\">\n            <div class=\"large-12 columns centered graph text-center\" id=\"acct_disk_graph\" style=\"height: 100px;\">";
  stack1 = this.invokePartial(partials['./empty-graphs'], '', './empty-graphs', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n          </div>\n        </div>\n      </div>\n    </div>\n  </div>\n</div>\n<br>\n<br>\n";
},"usePartial":true,"useData":true});
Handlebars.registerPartial('tabs/provision-tab/dashboard/vms', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/dashboard/vdc-vms',['hbs','hbs/handlebars', 'hbs!tabs/provision-tab/dashboard/empty-graphs','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row\">\n  <div class=\"large-12 large-centered columns\">\n    <h3 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group Virtual Machines", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n    <br></div>\n</div>\n<div class=\"row\" id=\"provision_vdc_vms_dashboard\">\n  <div class=\"large-12 large-centered columns\">\n    <div class=\"row\">\n      <div class=\"large-4 columns text-center\">\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <span  id=\"provision_dashboard_vdc_total\" style=\"font-size:80px\"> <i class=\"fa fa-spinner fa-spin\"></i>\n            </span>\n            <br>\n            <span style=\"color: #999;\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "TOTAL", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </div>\n        </div>\n        <br>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns text-center\">\n            <a href\"#\" class=\"medium button success large radius provision_create_vm_button\" style=\"margin-left: 5px; margin-right: 5px\"> <i class=\"fa fa-lg fa-plus-square\"/></a>\n            <a href\"#\" class=\"medium button large radius provision_vms_list_button\" style=\"margin-left: 5px; margin-right: 5px\">\n              <i class=\"fa fa-lg fa-th\"/></a>\n          </div>\n        </div>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns\"></div>\n        </div>\n      </div>\n      <div class=\"large-2 columns\">\n        <div class=\"row\">\n          <div class=\"large-12 columns text-center\">\n            <span id=\"provision_dashboard_vdc_running\" style=\"font-size:40px\">\n              <i class=\"fa fa-spinner fa-spin\"></i>\n            </span>\n            <br>\n            <span  style=\"color: #999; font-size: 14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "RUNNING", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </div>\n        </div>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns text-center \">\n            <span id=\"provision_dashboard_vdc_deploying\"  style=\"font-size:40px\">\n              <i class=\"fa fa-spinner fa-spin\"></i>\n            </span>\n            <br>\n            <span  style=\"color: #999; font-size: 14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "DEPLOYING", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </div>\n        </div>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns text-center\">\n            <span  id=\"provision_dashboard_vdc_off\" style=\"font-size:40px\">\n              <i class=\"fa fa-spinner fa-spin\"></i>\n            </span>\n            <br>\n            <span  style=\"color: #999; font-size: 14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "OFF", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </div>\n        </div>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns text-center \">\n            <span  id=\"provision_dashboard_vdc_error\" style=\"font-size:40px\">\n              <i class=\"fa fa-spinner fa-spin\"></i>\n            </span>\n            <br>\n            <span  style=\"color: #999; font-size: 14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ERROR", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </div>\n        </div>\n      </div>\n      <div class=\"large-6 columns\" id=\"dashboard_vdc_vm_accounting\">\n        <input style=\"display:none;\" value=\"vm\" id=\"acct_group_by\"/>\n        <div class=\"row\">\n          <div class=\"large-12 columns graph_legend\">\n            <h3 class=\"subheader\">\n              <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU hours", {"name":"tr","hash":{},"data":data})))
    + "</small>\n            </h3>\n          </div>\n          <div class=\"large-12 columns\">\n            <div class=\"large-12 columns centered graph text-center\" id=\"acct_cpu_graph\" style=\"height: 100px;\">";
  stack1 = this.invokePartial(partials['./empty-graphs'], '', './empty-graphs', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-12 columns graph_legend\">\n            <h3 class=\"subheader\">\n              <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Memory GB hours", {"name":"tr","hash":{},"data":data})))
    + "</small>\n            </h3>\n          </div>\n          <div class=\"large-12 columns\">\n            <div class=\"large-12 columns centered graph text-center\" id=\"acct_mem_graph\" style=\"height: 100px;\">";
  stack1 = this.invokePartial(partials['./empty-graphs'], '', './empty-graphs', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-12 columns graph_legend\">\n            <h3 class=\"subheader\">\n              <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk MB hours", {"name":"tr","hash":{},"data":data})))
    + "</small>\n            </h3>\n          </div>\n          <div class=\"large-12 columns\">\n            <div class=\"large-12 columns centered graph text-center\" id=\"acct_disk_graph\" style=\"height: 100px;\">";
  stack1 = this.invokePartial(partials['./empty-graphs'], '', './empty-graphs', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n          </div>\n        </div>\n      </div>\n    </div>\n  </div>\n</div>\n<br>\n<br>\n";
},"usePartial":true,"useData":true});
Handlebars.registerPartial('tabs/provision-tab/dashboard/vdc-vms', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/provision-tab/dashboard/users',['hbs','hbs/handlebars', 'hbs!tabs/provision-tab/dashboard/empty-graphs','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "    <div class=\"row\">\n      <div class=\"large-11 large-centered columns\">\n        <h3 class=\"subheader\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Users", {"name":"tr","hash":{},"data":data})))
    + "\n        </h3>\n        <br>\n      </div>\n    </div>\n    <div class=\"row\" id=\"provision_users_dashboard\">\n      <div class=\"large-12 large-centered columns\">\n          <div class=\"row\">\n            <div class=\"large-4 columns text-center\">\n              <br>\n              <div class=\"row\">\n                <div class=\"large-12 columns\">\n                  <span  id=\"provision_dashboard_users_total\" style=\"font-size:80px\">\n                    <i class=\"fa fa-spinner fa-spin\"></i>\n                  </span>\n                  <br>\n                  <span style=\"color: #999;\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "TOTAL", {"name":"tr","hash":{},"data":data})))
    + "</span>\n                </div>\n              </div>\n              <br>\n              <br>\n              <div class=\"row\">\n                <div class=\"large-12 columns\">\n                  <a href\"#\" class=\"medium button large success radius provision_create_user_button\" style=\"margin-left: 5px; margin-right: 5px\"><i class=\"fa fa-lg fa-plus-square\"/></a>\n                  <a href\"#\" class=\"medium button large radius provision_users_list_button\" style=\"margin-left: 5px; margin-right: 5px\"><i class=\"fa fa-lg fa-users\"/></a>\n                </div>\n              </div>\n              <br>\n              <div class=\"row\">\n                <div class=\"large-12 columns\">\n                </div>\n              </div>\n            </div>\n            <div class=\"large-8 columns\" id=\"dashboard_vdc_user_accounting\">\n              <input style=\"display:none;\" value=\"user\" id=\"acct_group_by\"/>\n              <div class=\"row\">\n                <div class=\"large-12 columns graph_legend\">\n                  <h3 class=\"subheader\"><small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU hours", {"name":"tr","hash":{},"data":data})))
    + "</small></h3>\n                </div>\n                <div class=\"large-12 columns\">\n                  <div class=\"large-12 columns centered graph text-center\" id=\"acct_cpu_graph\" style=\"height: 100px;\">\n";
  stack1 = this.invokePartial(partials['./empty-graphs'], '                    ', './empty-graphs', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  buffer += "                  </div>\n                </div>\n              </div>\n              <div class=\"row\">\n                <div class=\"large-12 columns graph_legend\">\n                  <h3 class=\"subheader\"><small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Memory GB hours", {"name":"tr","hash":{},"data":data})))
    + "</small></h3>\n                </div>\n                <div class=\"large-12 columns\">\n                  <div class=\"large-12 columns centered graph text-center\" id=\"acct_mem_graph\" style=\"height: 100px;\">\n";
  stack1 = this.invokePartial(partials['./empty-graphs'], '                    ', './empty-graphs', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  buffer += "                  </div>\n                </div>\n              </div>\n              <div class=\"row\">\n                <div class=\"large-12 columns graph_legend\">\n                  <h3 class=\"subheader\"><small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk MB hours", {"name":"tr","hash":{},"data":data})))
    + "</small></h3>\n                </div>\n                <div class=\"large-12 columns\">\n                  <div class=\"large-12 columns centered graph text-center\" id=\"acct_disk_graph\" style=\"height: 100px;\">\n";
  stack1 = this.invokePartial(partials['./empty-graphs'], '                    ', './empty-graphs', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "                  </div>\n                </div>\n              </div>\n            </div>\n          </div>\n      </div>\n    </div>\n    <br>\n    <br>\n";
},"usePartial":true,"useData":true});
Handlebars.registerPartial('tabs/provision-tab/dashboard/users', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/provision-tab',['require','foundation.core','foundation.accordion','foundation-datatables','utils/locale','sunstone-config','opennebula','sunstone','utils/notifier','utils/resource-select','utils/quotas/quota-widgets','utils/quotas/quota-defaults','utils/accounting','utils/showback','utils/humanize','utils/quotas/quota-limits','utils/graphs','utils/range-slider','utils/disks-resize','utils/nics-section','./provision-tab/users/quota-widget','./provision-tab/vms/list','./provision-tab/templates/list','./provision-tab/users/list','./provision-tab/flows/list','hbs!./provision-tab/content','hbs!./provision-tab/header','hbs!./provision-tab/dashboard/quotas','hbs!./provision-tab/dashboard/vdc-quotas','hbs!./provision-tab/dashboard/vms','hbs!./provision-tab/dashboard/vdc-vms','hbs!./provision-tab/dashboard/users','hbs!./provision-tab/group/info','./provision-tab/tabId'],function(require) {
  require('foundation.core');
  require('foundation.accordion');
  require('foundation-datatables');
  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var OpenNebula = require('opennebula');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var ResourceSelect = require('utils/resource-select');
  var QuotaWidgets = require('utils/quotas/quota-widgets');
  var QuotaDefaults = require('utils/quotas/quota-defaults');
  var Accounting = require('utils/accounting');
  var Showback = require('utils/showback');
  var Humanize = require('utils/humanize');
  var QuotaLimits = require('utils/quotas/quota-limits');
  var Graphs = require('utils/graphs');
  var RangeSlider = require('utils/range-slider');
  var DisksResize = require('utils/disks-resize');
  var NicsSection = require('utils/nics-section');

  var ProvisionQuotaWidget = require('./provision-tab/users/quota-widget');

  var ProvisionVmsList = require('./provision-tab/vms/list');
  var ProvisionTemplatesList = require('./provision-tab/templates/list');
  var ProvisionUsersList = require('./provision-tab/users/list');
  var ProvisionFlowsList = require('./provision-tab/flows/list');

  // Templates
  var TemplateContent = require('hbs!./provision-tab/content');
  var TemplateHeader = require('hbs!./provision-tab/header');

  var TemplateDashboardQuotas = require('hbs!./provision-tab/dashboard/quotas');
  var TemplateDashboardVdcQuotas = require('hbs!./provision-tab/dashboard/vdc-quotas');
  var TemplateDashboardVms = require('hbs!./provision-tab/dashboard/vms');
  var TemplateDashboardVdcVms = require('hbs!./provision-tab/dashboard/vdc-vms');
  var TemplateDashboardUsers = require('hbs!./provision-tab/dashboard/users');

  var TemplateGroupInfo = require('hbs!./provision-tab/group/info');

  var TAB_ID = require('./provision-tab/tabId');

  var povision_actions = {
    "Provision.User.create" : {
        type: "create",
        call: OpenNebula.User.create,
        callback: function(request, response) {
          if ( $("div#provision_create_user_manual_quota",
               $("#provision_create_user")).hasClass("active") ){

            quota_json = ProvisionQuotaWidget.retrieve($("#provision_create_user"));

            Sunstone.runAction("Provision.User.set_quota",
                                [response.USER.ID], quota_json);
          } else {
            clear_provision_create_user();
          }
        },
        error: Notifier.onError
    },

    "Provision.User.set_quota" : {
        type: "multiple",
        call: OpenNebula.User.set_quota,
        callback: function(request) {
          clear_provision_create_user();
        },
        error: Notifier.onError
    },

    "Provision.Group.show" : {
        type: "single",
        call: OpenNebula.Group.show,
        callback: show_provision_group_info_callback,
        error: Notifier.onError
    },

    "Provision.Flow.instantiate" : {
      type: "single",
      call: OpenNebula.ServiceTemplate.instantiate,
      callback: function(){
        OpenNebula.Action.clear_cache("SERVICE");
        ProvisionFlowsList.show(0);
        var context = $("#provision_create_flow");
        $("#flow_name", context).val('');
        //$(".provision_selected_networks").html("");
        $(".provision-pricing-table", context).removeClass("selected");
        //$('a[href="#provision_system_templates_selector"]', context).click();
      },
      error: Notifier.onError
    },

    "Provision.instantiate" : {
      type: "single",
      call: OpenNebula.Template.instantiate,
      callback: function(){
        OpenNebula.Action.clear_cache("VM");
        ProvisionVmsList.show(0);
        var context = $("#provision_create_vm");
        $("#vm_name", context).val('');
        $(".provision_selected_networks").html("");
        $(".provision-pricing-table", context).removeClass("selected");
        $(".alert-box-error", context).hide();
        $('a[href="#provision_system_templates_selector"]', context).click();
      },
      error: Notifier.onError
    }
  }

  $(document).foundation();

  function generate_custom_attrs(context, custom_attrs) {
    context.off();
    var text_attrs = [];

    $.each(custom_attrs, function(key, value){
      var parts = value.split("|");
      // 0 mandatory; 1 type; 2 desc;
      var attrs = {
        "name": key,
        "mandatory": parts[0],
        "type": parts[1],
        "description": parts[2],
      }

      switch (parts[1]) {
        case "text":
          text_attrs.push(attrs)
          break;
        case "password":
          text_attrs.push(attrs)
          break;
      }
    })

    if (text_attrs.length > 0) {
      context.html(
        '<br>'+
        '<div class="row">'+
          '<div class="large-12 large-centered columns">'+
            '<h3 class="subheader text-right">'+
              '<span class="left">'+
                '<i class="fa fa-th fa-gears"></i>&emsp;'+
                Locale.tr("Custom Attributes")+
              '</span>'+
            '</h3>'+
            '<br>'+
          '</div>'+
        '</div>'+
        '<br>'+
        '<div class="provision_custom_attributes">'+
        '</div>'+
        '<br>'+
        '<br>'+
        '<br>');


      $.each(text_attrs, function(index, custom_attr){
        $(".provision_custom_attributes", context).append(
          '<br>'+
          '<div class="row">'+
            '<div class="large-10 large-centered columns">'+
              '<label style="font-size: 16px">' +
                '<i class="fa fa-asterisk" style="color:#0099c3"/> '+
                custom_attr.description +
                '<input type="'+custom_attr.type+'" attr_name="'+custom_attr.name+'" class="provision_custom_attribute provision-input" style="height: 40px !important; font-size: 16px; padding: 0.5rem  !important;"/>'+
              '</label>'+
            '</div>'+
          '</div>');
      })
    } else {
      context.html("");
    }
  }

  function generate_cardinality_selector(context, role_template, template_json) {
    context.off();
    var min_vms = (role_template.min_vms||1);
    var max_vms = (role_template.max_vms||20);

    context.html(
      '<br>'+
      '<div class="row">'+
        '<div class="large-12 large-centered columns">'+
          '<h3 class="subheader text-right">'+
            '<span class="left">'+
              '<i class="fa fa-th fa-lg"></i>&emsp;'+
              Locale.tr("Cardinality")+
            '</span>'+
          '</h3>'+
          '<br>'+
        '</div>'+
      '</div>'+
      '<br>'+
      '<br>'+
      '<div class="row">'+
        '<div class="large-12 columns">'+
          '<div class="row">'+
            '<div class="large-2 text-center columns">'+
              '<span class="cardinality_value" style="color: #777; font-size:40px">'+role_template.cardinality+'</span>'+
              '<br>'+
              '<span style="color: #999;">'+Locale.tr("VMs")+'</span>'+
            '</div>'+
            '<div class="large-6 columns">'+
              '<div class="cardinality_slider_div">'+
                '<span class="" style="color: #777;">'+Locale.tr("Change cardinality")+'</span>'+
                '<br>'+
                '<div class="range-slider radius cardinality_slider" data-slider data-options="start: 1; end: 50;">'+
                  '<span class="range-slider-handle"></span>'+
                  '<span class="range-slider-active-segment"></span>'+
                  '<input type="hidden">'+
                '</div>'+
                '<span class="left" style="color: #999;">'+min_vms+'</span>'+
                '<span class="right" style="color: #999;">'+max_vms+'</span>'+
              '</div>'+
              '<div class="cardinality_no_slider_div">'+
                '<br>'+
                '<br>'+
                '<span class="" style="color: #999;">'+Locale.tr("The cardinality for this role cannot be changed")+'</span>'+
              '</div>'+
            '</div>'+
            '<div class="large-4 columns text-center provision_create_service_cost_div hidden">'+
              '<span class="cost_value" style="color: #777; font-size:40px"></span>'+
              '<br>'+
              '<span style="color: #999;">'+Locale.tr("COST")+' / ' + Locale.tr("HOUR") + '</span>'+
            '</div>'+
          '</div>'+
        '</div>'+
      '</div>');

      var capacity = template_json.VMTEMPLATE.TEMPLATE;
      var cost = 0;
      if (capacity.CPU_COST || capacity.MEMORY_COST && Config.isFeatureEnabled("showback")) {
        $(".provision_create_service_cost_div").show();

        if (capacity.CPU && capacity.CPU_COST) {
          cost += capacity.CPU * capacity.CPU_COST
          $(".cost_value", context).data("CPU_COST", capacity.CPU_COST);
        }

        if (capacity.MEMORY && capacity.MEMORY_COST) {
          cost += capacity.MEMORY * capacity.MEMORY_COST
          $(".cost_value", context).data("MEMORY_COST", capacity.MEMORY_COST);
        }

        $(".provision_create_service_cost_div", context).data("cost", cost)
        var cost_value = cost*parseInt(role_template.cardinality);
        $(".cost_value", context).html(cost_value.toFixed(2));
      } else {
        $(".provision_create_service_cost_div").hide();
      }

      if (max_vms > min_vms) {
        $( ".cardinality_slider", context).attr('data-options', 'start: '+min_vms+'; end: '+max_vms+';')
        context.foundation();
        $( ".cardinality_slider_div", context).show();
        $( ".cardinality_no_slider_div", context).hide();

        $( ".cardinality_slider", context).foundation('slider', 'set_value', role_template.cardinality);

        $( ".cardinality_slider", context).on('change.fndtn.slider', function(){
          $(".cardinality_value",context).html($(this).attr('data-slider'))
          var cost_value = $(".provision_create_service_cost_div", context).data("cost")*$(this).attr('data-slider');
          $(".cost_value", context).html(cost_value.toFixed(2));
        });
      } else {
        $( ".cardinality_slider_div", context).hide();
        $( ".cardinality_no_slider_div", context).show();
      }
  }

  var provision_instance_type_accordion_id = 0;

  function generate_provision_instance_type_accordion(context, capacity) {
    context.off();
    var memory_value;
    var memory_unit;

    if (capacity.MEMORY > 1000){
      memory_value = Math.floor(capacity.MEMORY/1024);
      memory_unit = "GB";
    } else {
      memory_value = (capacity.MEMORY ? capacity.MEMORY : '-');
      memory_unit = "MB";
    }

    context.html(
      '<br>'+
      '<div class="row">'+
        '<div class="large-12 large-centered columns">'+
          '<h3 class="subheader text-right">'+
            '<span class="left">'+
              '<i class="fa fa-laptop fa-lg"></i>&emsp;'+
              Locale.tr("Capacity")+
            '</span>'+
            '<span>'+
              '<span class="cpu_value">'+(capacity.CPU ? capacity.CPU : '-')+'</span> '+
              '<small style="color: #999; margin-right: 10px">'+Locale.tr("CPU")+'</small>'+
              '<span class="memory_value">'+memory_value+'</span>'+
              ' '+
              '<span class="memory_unit">'+memory_unit+'</span> '+
              '<small style="color: #999; margin-right: 10px">'+Locale.tr("MEMORY")+'</small>'+
              '<span class="provision_create_template_cost_div hidden">' +
                '<span class="cost_value">0.00</span> '+
                '<small style="color: #999;">'+Locale.tr("COST")+' / ' + Locale.tr("HOUR") + '</small>'+
              '</span>'+
            '</span>'+
          '</h3>'+
          '<br>'+
        '</div>'+
      '</div>'+
      (Config.provision.create_vm.isEnabled("capacity_select") && (capacity.SUNSTONE_CAPACITY_SELECT != "NO") ?
      '<div class="row">'+
        '<div class="large-12 large-centered columns">'+
          '<dl class="accordion" data-accordion="provision_accordion_'+provision_instance_type_accordion_id+'">'+
            '<dd class="accordion-navigation">'+
              '<a href="#provision_instance_type_dd_'+provision_instance_type_accordion_id+'" class="button large-12 medium radius" style="color: #555;">'+
                Locale.tr("Change Capacity")+
              '</a>'+
              '<div id="provision_instance_type_dd_'+provision_instance_type_accordion_id+'" class="content">'+
                '<div class="row">'+
                  '<div class="large-12 large-centered columns">'+
                    '<h3 class="subheader text-right">'+
                      '<input type="search" class="provision-search-input right" placeholder="Search"/>'+
                    '</h3>'+
                    '<br>'+
                  '</div>'+
                '</div>'+
                '<div class="row">'+
                  '<div class="large-12 large-centered columns">'+
                    '<table class="provision_instance_types_table">'+
                      '<thead class="hidden">'+
                        '<tr>'+
                          '<th>'+Locale.tr("Name")+'</th>'+
                        '</tr>'+
                      '</thead>'+
                      '<tbody class="hidden">'+
                      '</tbody>'+
                    '</table>'+
                    '<br>'+
                  '</div>'+
                '</div>'+
              '</div>'+
            '</dd>'+
          '</dl>'+
        '</div>'+
      '</div>' : '' ) +
      '<br>');

    var cost = 0;
    if (capacity.CPU_COST || capacity.MEMORY_COST && Config.isFeatureEnabled("showback")) {
      $(".provision_create_template_cost_div").show();

      if (capacity.CPU && capacity.CPU_COST) {
        cost += capacity.CPU * capacity.CPU_COST
        $(".cost_value").data("CPU_COST", capacity.CPU_COST);
      }

      if (capacity.MEMORY && capacity.MEMORY_COST) {
        cost += capacity.MEMORY * capacity.MEMORY_COST
        $(".cost_value").data("MEMORY_COST", capacity.MEMORY_COST);
      }

      $(".cost_value").html(cost.toFixed(2));
    } else {
      $(".provision_create_template_cost_div").hide();
    }

    if (Config.provision.create_vm.isEnabled("capacity_select") && (capacity.SUNSTONE_CAPACITY_SELECT != "NO")) {
      provision_instance_type_accordion_id += 1;

      var provision_instance_types_datatable = $('.provision_instance_types_table', context).dataTable({
        "iDisplayLength": 6,
        "sDom" : '<"H">t<"F"lp>',
        "bSort" : false,
        "aLengthMenu": [[6, 12, 36, 72], [6, 12, 36, 72]],
        "aoColumnDefs": [
            { "bVisible": false, "aTargets": ["all"]}
        ],
        "aoColumns": [
            { "mDataProp": "name" }
        ],
        "fnPreDrawCallback": function (oSettings) {
          // create a thumbs container if it doesn't exist. put it in the dataTables_scrollbody div
          if (this.$('tr', {"filter": "applied"} ).length == 0) {
            this.html('<div class="text-center">'+
              '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
                '<i class="fa fa-cloud fa-stack-2x"></i>'+
                '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
              '</span>'+
              '<br>'+
              '<br>'+
              '<span style="font-size: 18px; color: #999">'+
                Locale.tr("There are no instance_types available. Please contact your cloud administrator")+
              '</span>'+
              '</div>');
          } else {
            $(".provision_instance_types_table", context).html(
              '<ul class="provision_instance_types_ul large-block-grid-3 medium-block-grid-3 small-block-grid-1 text-center">'+
              '</ul>');
          }

          return true;
        },
        "fnRowCallback": function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
          var data = aData;

          var li = $('<li>'+
              '<ul class="provision-pricing-table hoverable only-one" cpu="'+data.cpu+'" memory="'+data.memory+'">'+
                '<li class="provision-title" title="'+data.name+'">'+
                  data.name+
                '</li>'+
                '<li class="provision-bullet-item">'+
                  '<span style="font-size: 40px">'+
                  '<i class="fa fa-fw fa-laptop"/>&emsp;'+
                  '<span style="vertical-align: middle; font-size:14px">'+
                    'x'+data.cpu+' - '+
                    ((data.memory > 1000) ?
                      (Math.floor(data.memory/1024)+'GB') :
                      (data.memory+'MB'))+
                  '</span>'+
                  '</span>'+
                '</li>'+
                '<li class="provision-description">'+
                  (data.description || '')+
                '</li>'+
              '</ul>'+
            '</li>').appendTo($(".provision_instance_types_ul", context));

          $(".provision-pricing-table", li).data("opennebula", data)

          return nRow;
        }
      });


      $('.provision-search-input', context).on('input',function(){
        provision_instance_types_datatable.fnFilter( $(this).val() );
      })

      context.on("click", ".provision-pricing-table.only-one" , function(){
        $(".cpu_value", context).html($(this).attr("cpu"));

        var memory_value;
        var memory_unit;

        if ($(this).attr("memory") > 1000){
          memory_value = Math.floor($(this).attr("memory")/1024);
          memory_unit = "GB";
        } else {
          memory_value = $(this).attr("memory");
          memory_unit = "MB";
        }

        $(".memory_value", context).html(memory_value);
        $(".memory_unit", context).html(memory_unit);

        if (Config.isFeatureEnabled("showback")) {
          var cost = 0;

          if ($(".cost_value").data("CPU_COST")) {
            cost += $(this).attr("cpu") * $(".cost_value").data("CPU_COST")
          }

          if ($(".cost_value").data("MEMORY_COST")) {
            cost += $(this).attr("memory") * $(".cost_value").data("MEMORY_COST")
          }

          $(".cost_value").html(cost.toFixed(2));
        }

        $('.accordion a', context).first().trigger("click");
      })

      $(document).foundation();

      update_provision_instance_types_datatable(provision_instance_types_datatable);
    }
  }

  function show_provision_dashboard() {
    $(".section_content").hide();
    $("#provision_dashboard").fadeIn();

    $("#provision_dashboard").html("");

    if (Config.provision.dashboard.isEnabled("vms")) {
      $("#provision_dashboard").append(TemplateDashboardVms());

      var start_time =  Math.floor(new Date().getTime() / 1000);
      // ms to s

      // 604800 = 7 days = 7*24*60*60
      start_time = start_time - 604800;

      // today
      var end_time = -1;

      var options = {
        "start_time": start_time,
        "end_time": end_time,
        "userfilter": config["user_id"]
      }

      var no_table = true;

      OpenNebula.VM.accounting({
          success: function(req, response){
              Accounting.fillAccounting($("#dashboard_vm_accounting"), req, response, no_table);
          },
          error: Notifier.onError,
          data: options
      });

      OpenNebula.VM.list({
        timeout: true,
        success: function (request, item_list){
          var total = 0;
          var running = 0;
          var off = 0;
          var error = 0;
          var deploying = 0;

          $.each(item_list, function(index, vm){
            if (vm.VM.UID == config["user_id"]) {
              var state = ProvisionVmsList.state(vm.VM);

              total = total + 1;

              switch (state.color) {
                case "deploying":
                  deploying = deploying + 1;
                  break;
                case "error":
                  error = error + 1;
                  break;
                case "running":
                  running = running + 1;
                  break;
                case "powering_off":
                  off = off + 1;
                  break;
                case "off":
                  off = off + 1;
                  break;
              }
            }
          })

          var context = $("#provision_vms_dashboard");
          $("#provision_dashboard_total", context).html(total);
          $("#provision_dashboard_running", context).html(running);
          $("#provision_dashboard_off", context).html(off);
          $("#provision_dashboard_error", context).html(error);
          $("#provision_dashboard_deploying", context).html(deploying);
        },
        error: Notifier.onError
      });
    }

    if (Config.provision.dashboard.isEnabled("vdcvms")) {
      $("#provision_dashboard").append(TemplateDashboardVdcVms());

      var start_time =  Math.floor(new Date().getTime() / 1000);
      // ms to s

      // 604800 = 7 days = 7*24*60*60
      start_time = start_time - 604800;

      // today
      var end_time = -1;

      var options = {
        "start_time": start_time,
        "end_time": end_time
      }

      var no_table = true;

      OpenNebula.VM.accounting({
          success: function(req, response){
              Accounting.fillAccounting($("#dashboard_vdc_vm_accounting"), req, response, no_table);
          },
          error: Notifier.onError,
          data: options
      });


      OpenNebula.VM.list({
        timeout: true,
        success: function (request, item_list){
          var total = 0;
          var running = 0;
          var off = 0;
          var error = 0;
          var deploying = 0;

          $.each(item_list, function(index, vm){
              var state = ProvisionVmsList.state(vm.VM);

              total = total + 1;

              switch (state.color) {
                case "deploying":
                  deploying = deploying + 1;
                  break;
                case "error":
                  error = error + 1;
                  break;
                case "running":
                  running = running + 1;
                  break;
                case "powering_off":
                  off = off + 1;
                  break;
                case "off":
                  off = off + 1;
                  break;
                default:
                  break;
              }
          })

          var context = $("#provision_vdc_vms_dashboard");
          $("#provision_dashboard_vdc_total", context).html(total);
          $("#provision_dashboard_vdc_running", context).html(running);
          $("#provision_dashboard_vdc_off", context).html(off);
          $("#provision_dashboard_vdc_error", context).html(error);
          $("#provision_dashboard_vdc_deploying", context).html(deploying);
        },
        error: Notifier.onError
      });
    }

    if (Config.provision.dashboard.isEnabled("users")) {
      $("#provision_dashboard").append(TemplateDashboardUsers());

      var start_time =  Math.floor(new Date().getTime() / 1000);
      // ms to s

      // 604800 = 7 days = 7*24*60*60
      start_time = start_time - 604800;

      // today
      var end_time = -1;

      var options = {
        "start_time": start_time,
        "end_time": end_time,
        "group": config["user_gid"]
      }

      var no_table = true;

      OpenNebula.VM.accounting({
          success: function(req, response){
              Accounting.fillAccounting($("#dashboard_vdc_user_accounting"), req, response, no_table);
          },
          error: Notifier.onError,
          data: options
      });

      OpenNebula.User.list({
        timeout: true,
        success: function (request, item_list){
          var total = item_list.length || 0;

          var context = $("#provision_users_dashboard");
          $("#provision_dashboard_users_total", context).html(total);
        },
        error: Notifier.onError
      });
    }

    if (Config.provision.dashboard.isEnabled("quotas")) {
      $("#provision_dashboard").append(TemplateDashboardQuotas());


      OpenNebula.User.show({
        data : {
            id: "-1"
        },
        success: function(request,user_json){
          var user = user_json.USER;

          QuotaWidgets.initEmptyQuotas(user);

          if (!$.isEmptyObject(user.VM_QUOTA)){
              var default_user_quotas = QuotasDefault.default_quotas(user.DEFAULT_USER_QUOTAS);

              var vms = QuotaWidgets.quotaInfo(
                  user.VM_QUOTA.VM.VMS_USED,
                  user.VM_QUOTA.VM.VMS,
                  default_user_quotas.VM_QUOTA.VM.VMS,
                  true);

              $("#provision_dashboard_rvms_percentage").html(vms["percentage"]);
              $("#provision_dashboard_rvms_str").html(vms["str"]);
              $("#provision_dashboard_rvms_meter").css("width", vms["percentage"]+"%");

              var memory = QuotaWidgets.quotaMBInfo(
                  user.VM_QUOTA.VM.MEMORY_USED,
                  user.VM_QUOTA.VM.MEMORY,
                  default_user_quotas.VM_QUOTA.VM.MEMORY,
                  true);

              $("#provision_dashboard_memory_percentage").html(memory["percentage"]);
              $("#provision_dashboard_memory_str").html(memory["str"]);
              $("#provision_dashboard_memory_meter").css("width", memory["percentage"]+"%");

              var cpu = QuotaWidgets.quotaFloatInfo(
                  user.VM_QUOTA.VM.CPU_USED,
                  user.VM_QUOTA.VM.CPU,
                  default_user_quotas.VM_QUOTA.VM.CPU,
                  true);

              $("#provision_dashboard_cpu_percentage").html(cpu["percentage"]);
              $("#provision_dashboard_cpu_str").html(cpu["str"]);
              $("#provision_dashboard_cpu_meter").css("width", cpu["percentage"]+"%");
          }
        }
      })
    }

    if (Config.provision.dashboard.isEnabled("vdcquotas")) {
      $("#provision_dashboard").append(TemplateDashboardVdcQuotas());


      OpenNebula.Group.show({
        data : {
            id: "-1"
        },
        success: function(request,group_json){
          var group = group_json.GROUP;

          QuotaWidgets.initEmptyQuotas(group);

          if (!$.isEmptyObject(group.VM_QUOTA)){
              var default_group_quotas = QuotaDefaults.default_quotas(group.DEFAULT_GROUP_QUOTAS);

              var vms = QuotaWidgets.quotaInfo(
                  group.VM_QUOTA.VM.VMS_USED,
                  group.VM_QUOTA.VM.VMS,
                  default_group_quotas.VM_QUOTA.VM.VMS,
                  true);

              $("#provision_dashboard_vdc_rvms_percentage").html(vms["percentage"]);
              $("#provision_dashboard_vdc_rvms_str").html(vms["str"]);
              $("#provision_dashboard_vdc_rvms_meter").css("width", vms["percentage"]+"%");

              var memory = QuotaWidgets.quotaMBInfo(
                  group.VM_QUOTA.VM.MEMORY_USED,
                  group.VM_QUOTA.VM.MEMORY,
                  default_group_quotas.VM_QUOTA.VM.MEMORY,
                  true);

              $("#provision_dashboard_vdc_memory_percentage").html(memory["percentage"]);
              $("#provision_dashboard_vdc_memory_str").html(memory["str"]);
              $("#provision_dashboard_vdc_memory_meter").css("width", memory["percentage"]+"%");

              var cpu = QuotaWidgets.quotaFloatInfo(
                  group.VM_QUOTA.VM.CPU_USED,
                  group.VM_QUOTA.VM.CPU,
                  default_group_quotas.VM_QUOTA.VM.CPU,
                  true);

              $("#provision_dashboard_vdc_cpu_percentage").html(cpu["percentage"]);
              $("#provision_dashboard_vdc_cpu_str").html(cpu["str"]);
              $("#provision_dashboard_vdc_cpu_meter").css("width", cpu["percentage"]+"%");
          }
        }
      })
    }
  }


  function show_provision_user_info() {
    Sunstone.runAction("Provision.User.show", "-1");
    $(".section_content").hide();
    $("#provision_user_info").fadeIn();
    $("dd.active a", $("#provision_user_info")).trigger("click");
  }



  function show_provision_group_info_callback(request, response) {
    var info = response.GROUP;

    var context = $("#provision_manage_vdc");

    var default_group_quotas = QuotaDefaults.default_quotas(info.DEFAULT_GROUP_QUOTAS);

    var quotas_tab_html = QuotaWidgets.initQuotasPanel(info, default_group_quotas,
                                        "#provision_vdc_quotas_div", false);

    $("#provision_vdc_quotas_div").html(quotas_tab_html);

    QuotaWidgets.setupQuotasPanel(info,
        "#provision_vdc_quotas_div",
        false,
        "Group");

    $("#provision_info_vdc_group_acct", context).html(Accounting.html());
    Accounting.setup(
      $("#provision_info_vdc_group_acct", context),
      {   fixed_group: info.ID,
          init_group_by: "user" });

    if (Config.isFeatureEnabled("showback")) {
      $("#provision_info_vdc_group_showback", context).html(Showback.html());
      Showback.setup(
        $("#provision_info_vdc_group_showback", context),
        {   fixed_user: "", fixed_group: info.ID });
    }

    $("#acct_placeholder", context).hide();
  }

  function show_provision_create_vm() {
    OpenNebula.Action.clear_cache("VMTEMPLATE");
    ProvisionTemplatesList.updateDatatable(provision_system_templates_datatable);
    provision_system_templates_datatable.fnFilter("^-$", 2, true, false)

    ProvisionTemplatesList.updateDatatable(provision_vdc_templates_datatable);
    provision_vdc_templates_datatable.fnFilter("^(?!\-$)", 2, true, false);
    provision_vdc_templates_datatable.fnFilter("^1$", 3, true, false);

    if (Config.isTabPanelEnabled("provision-tab", "templates")) {
      ProvisionTemplatesList.updateDatatable(provision_saved_templates_datatable);
      provision_saved_templates_datatable.fnFilter("^(?!\-$)", 2, true, false);
      provision_saved_templates_datatable.fnFilter("^0$", 3, true, false);
    }

    $(".provision_accordion_template .selected_template").hide();
    $(".provision_accordion_template .select_template").show();

    $("#provision_create_vm .provision_capacity_selector").html("");
    $("#provision_create_vm .provision_disk_selector").html("");
    $("#provision_create_vm .provision_network_selector").html("");
    $("#provision_create_vm .provision_custom_attributes_selector").html("")

    $("#provision_create_vm dd:not(.active) a[href='#provision_dd_template']").trigger("click")

    $(".section_content").hide();
    $("#provision_create_vm").fadeIn();
  }

  function show_provision_create_flow() {
    update_provision_flow_templates_datatable(provision_flow_templates_datatable);

    var context = $("#provision_create_flow");

    $("#provision_customize_flow_template", context).hide();
    $("#provision_customize_flow_template", context).html("");

    $(".provision_network_selector", context).html("")
    $(".provision_custom_attributes_selector", context).html("")

    $(".provision_accordion_flow_template .selected_template", context).hide();
    $(".provision_accordion_flow_template .select_template", context).show();

    $("dd:not(.active) a[href='#provision_dd_flow_template']", context).trigger("click")

    $(".alert-box-error", context).hide();

    $(".section_content").hide();
    $("#provision_create_flow").fadeIn();
  }

  function show_provision_create_user() {
    $(".section_content").hide();
    $("#provision_create_user").fadeIn();
    $(document).foundation();
  }

  function show_provision_vdc_info() {
    $(".section_content").hide();
    $("#provision_manage_vdc").fadeIn();

    Sunstone.runAction('Provision.Group.show', "-1");
  }

  function update_provision_instance_types_datatable(datatable) {
      datatable.fnClearTable(true);
      if (!config['instance_types'] || config['instance_types'].length == 0) {
        datatable.html('<div class="text-center">'+
          '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
            '<i class="fa fa-cloud fa-stack-2x"></i>'+
            '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
          '</span>'+
          '<br>'+
          '<br>'+
          '<span style="font-size: 18px; color: #999">'+
            Locale.tr("There are no instance types available")+
          '</span>'+
          '</div>');
      } else {
        datatable.fnAddData(config['instance_types']);
      }
  }

  function update_provision_flow_templates_datatable(datatable, timeout) {
    datatable.html('<div class="text-center">'+
      '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
        '<i class="fa fa-cloud fa-stack-2x"></i>'+
        '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>'+
      '</span>'+
      '<br>'+
      '<br>'+
      '<span style="font-size: 18px; color: #999">'+
      '</span>'+
      '</div>');

    setTimeout( function(){
      OpenNebula.ServiceTemplate.list({
        timeout: true,
        success: function (request, item_list){
          datatable.fnClearTable(true);
          if (item_list.length == 0) {
            datatable.html('<div class="text-center">'+
              '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
                '<i class="fa fa-cloud fa-stack-2x"></i>'+
                '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
              '</span>'+
              '<br>'+
              '<br>'+
              '<span style="font-size: 18px; color: #999">'+
                Locale.tr("There are no templates available")+
              '</span>'+
              '</div>');
          } else {
            datatable.fnAddData(item_list);
          }
        },
        error: Notifier.onError
      });
    }, timeout);
  }

  // Closes and resets the create user wizard
  function clear_provision_create_user(){
    OpenNebula.Action.clear_cache("USER");
    ProvisionUsersList.show(0);

    var context = $("#provision_create_user");
    $("#username", context).val('');
    $("#password", context).val('');
    $("#repeat_password", context).val('');

    ProvisionQuotaWidget.reset(context);

    $(".alert-box-error", context).hide();
    $(".alert-box-error", context).html("");
  }

  var Tab = {
    tabId: TAB_ID,
    list_header: "",
    actions: povision_actions,
    content: TemplateContent(),
    setup: _setup
  };

  return Tab;

  function _setup() {
    $(document).ready(function(){
      var tab_name = 'provision-tab';
      var tab = $("#"+tab_name);

      if (Config.isTabEnabled(tab_name)) {
        $('.right-header').prepend(TemplateHeader());

        $(".right-content").addClass("large-centered small-centered");
        $("#footer").removeClass("right");
        $("#footer").addClass("large-centered small-centered");

        ProvisionVmsList.generate($(".provision_vms_list_section"), {active: true});

        if (Config.isTabPanelEnabled("provision-tab", "templates")) {
          ProvisionTemplatesList.generate($(".provision_templates_list_section"), {active: true});
        }

        // TODO check if active
        ProvisionFlowsList.generate($(".provision_flows_list_section"), {active: true});
        ProvisionUsersList.generate($(".provision_users_list_section"), {active: true});

        //
        // Dashboard
        //

        $(".provision_image_header").on("click", function(){
          Sunstone.showTab(TAB_ID);
          $('li', '.provision-header').removeClass("active");
          show_provision_dashboard();
        })

        $(".configuration").on("click", function(){
          $('li', '.provision-header').removeClass("active");
        })

        show_provision_dashboard();

        $('.provision-header').on('click', 'li', function(){
          Sunstone.showTab(TAB_ID);
          $('li', '.provision-header').removeClass("active");
          $(this).closest('li').addClass("active");
        })

        $(document).on("click", ".provision_vms_list_button", function(){
          OpenNebula.Action.clear_cache("VM");
          ProvisionVmsList.show(0);
        });

        $(document).on("click", ".provision_templates_list_button", function(){
          OpenNebula.Action.clear_cache("VMTEMPLATE");
          ProvisionTemplatesList.show(0);
        });

        $(document).on("click", ".provision_flows_list_button", function(){
          OpenNebula.Action.clear_cache("SERVICE");
          ProvisionFlowsList.show(0);
        });

        $(document).on("click", ".provision_users_list_button", function(){
          OpenNebula.Action.clear_cache("USER");
          ProvisionUsersList.show(0);
        });

        //
        // Create VM
        //

        function appendTemplateCard(aData, tableID) {
          var data = aData.VMTEMPLATE;
          var logo;

          if (data.TEMPLATE.LOGO) {
            logo = '<span class="provision-logo" href="#">'+
                '<img  src="'+data.TEMPLATE.LOGO+'">'+
              '</span>';
          } else {
            logo = '<span style="color: #bfbfbf; font-size: 60px;">'+
              '<i class="fa fa-fw fa-file-text-o"/>'+
            '</span>';
          }

          var li = $('<li>'+
              '<ul class="provision-pricing-table hoverable only-one" opennebula_id="'+data.ID+'">'+
                '<li class="provision-title" title="'+data.NAME+'">'+
                  data.NAME+
                '</li>'+
                '<li style="height: 85px" class="provision-bullet-item">'+
                  logo +
                '</li>'+
                '<li class="provision-description">'+
                  (data.TEMPLATE.DESCRIPTION || '...')+
                '</li>'+
              '</ul>'+
            '</li>').appendTo($("#"+tableID+'_ul'));

          $(".provision-pricing-table", li).data("opennebula", aData);
        }

        function initializeTemplateCards(context, tableID) {
          // create a thumbs container if it doesn't exist. put it in the dataTables_scrollbody div
          if (context.$('tr', {"filter": "applied"} ).length == 0) {
            context.html('<div class="text-center">'+
              '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
                '<i class="fa fa-cloud fa-stack-2x"></i>'+
                '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
              '</span>'+
              '<br>'+
              '<br>'+
              '<span style="font-size: 18px; color: #999">'+
                Locale.tr("There are no templates available")+
              '</span>'+
              '</div>');
          } else {
            $('#'+tableID+'_table').html(
              '<ul id="'+tableID+'_ul" class="large-block-grid-3 medium-block-grid-3 small-block-grid-1 text-center"></ul>');
          }

          return true;
        }

        provision_system_templates_datatable = $('#provision_system_templates_table').dataTable({
          "iDisplayLength": 6,
          "sDom" : '<"H">t<"F"lp>',
          "aLengthMenu": [[6, 12, 36, 72], [6, 12, 36, 72]],
          "aoColumnDefs": [
              { "bVisible": false, "aTargets": ["all"]}
          ],
          "aoColumns": [
              { "mDataProp": "VMTEMPLATE.ID" },
              { "mDataProp": "VMTEMPLATE.NAME" },
              { "mDataProp": "VMTEMPLATE.TEMPLATE.SAVED_TEMPLATE_ID", "sDefaultContent" : "-"  }
          ],
          "fnPreDrawCallback": function (oSettings) {
            initializeTemplateCards(this, "provision_system_templates")
          },
          "fnRowCallback": function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
            appendTemplateCard(aData, "provision_system_templates");
            return nRow;
          }
        });


        provision_vdc_templates_datatable = $('#provision_vdc_templates_table').dataTable({
          "iDisplayLength": 6,
          "sDom" : '<"H">t<"F"lp>',
          "aLengthMenu": [[6, 12, 36, 72], [6, 12, 36, 72]],
          "aoColumnDefs": [
              { "bVisible": false, "aTargets": ["all"]}
          ],
          "aoColumns": [
              { "mDataProp": "VMTEMPLATE.ID" },
              { "mDataProp": "VMTEMPLATE.NAME" },
              { "mDataProp": "VMTEMPLATE.TEMPLATE.SAVED_TEMPLATE_ID", "sDefaultContent" : "-"  },
              { "mDataProp": "VMTEMPLATE.PERMISSIONS.GROUP_U" }
          ],
          "fnPreDrawCallback": function (oSettings) {
            initializeTemplateCards(this, "provision_vdc_templates")
          },
          "fnRowCallback": function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
            appendTemplateCard(aData, "provision_vdc_templates");
            return nRow;
          }
        });


        provision_saved_templates_datatable = $('#provision_saved_templates_table').dataTable({
          "iDisplayLength": 6,
          "sDom" : '<"H">t<"F"lp>',
          "aLengthMenu": [[6, 12, 36, 72], [6, 12, 36, 72]],
          "aoColumnDefs": [
              { "bVisible": false, "aTargets": ["all"]}
          ],
          "aoColumns": [
              { "mDataProp": "VMTEMPLATE.ID" },
              { "mDataProp": "VMTEMPLATE.NAME" },
              { "mDataProp": "VMTEMPLATE.TEMPLATE.SAVED_TEMPLATE_ID", "sDefaultContent" : "-"  },
              { "mDataProp": "VMTEMPLATE.PERMISSIONS.GROUP_U" }
          ],
          "fnPreDrawCallback": function (oSettings) {
            initializeTemplateCards(this, "provision_saved_templates")
          },
          "fnRowCallback": function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
            appendTemplateCard(aData, "provision_saved_templates");
            return nRow;
          }
        });


        $('#provision_create_template_search').on('input',function(){
          provision_system_templates_datatable.fnFilter( $(this).val() );
          provision_saved_templates_datatable.fnFilter( $(this).val() );
          provision_vdc_templates_datatable.fnFilter( $(this).val() );
        })

        $("#provision_create_template_refresh_button").click(function(){
          OpenNebula.Action.clear_cache("VMTEMPLATE");
          ProvisionTemplatesList.updateDatatable(provision_system_templates_datatable);
          ProvisionTemplatesList.updateDatatable(provision_saved_templates_datatable);
          ProvisionTemplatesList.updateDatatable(provision_vdc_templates_datatable);

        });

        tab.on("click", "#provision_create_vm .provision_select_template .provision-pricing-table.only-one" , function(){
          var create_vm_context = $("#provision_create_vm");

          if ($(this).hasClass("selected")){
            $(".provision_disk_selector", create_vm_context).html("");
            $(".provision_network_selector", create_vm_context).html("");
            $(".provision_capacity_selector", create_vm_context).html("");

            $(".provision_accordion_template .selected_template").hide();
            $(".provision_accordion_template .select_template").show();
          } else {
            var template_id = $(this).attr("opennebula_id");
            var template_json = $(this).data("opennebula");

            $(".provision_accordion_template .selected_template").show();
            $(".provision_accordion_template .select_template").hide();
            $(".provision_accordion_template .selected_template_name").html(template_json.VMTEMPLATE.NAME)
            if (template_json.VMTEMPLATE.TEMPLATE.LOGO) {
              $(".provision_accordion_template .selected_template_logo").html('<img  src="'+template_json.VMTEMPLATE.TEMPLATE.LOGO+'">');
            } else {
              $(".provision_accordion_template .selected_template_logo").html('<i class="fa fa-file-text-o fa-lg"/>&emsp;');
            }

            $(".provision_accordion_template a").first().trigger("click");

            generate_provision_instance_type_accordion(
              $(".provision_capacity_selector", create_vm_context),
              template_json.VMTEMPLATE.TEMPLATE);

            var disksContext = $(".provision_disk_selector", create_vm_context);
            if (Config.provision.create_vm.isEnabled("disk_resize")) {
              DisksResize.insert(template_json, disksContext);
            } else {
              disksContext.html("");
            }

            if (Config.provision.create_vm.isEnabled("network_select")) {
              NicsSection.insert(template_json, create_vm_context);
            } else {
              $(".provision_network_selector", create_vm_context).html("");
            }

            if (template_json.VMTEMPLATE.TEMPLATE.USER_INPUTS) {
              generate_custom_attrs(
                $(".provision_custom_attributes_selector", create_vm_context),
                template_json.VMTEMPLATE.TEMPLATE.USER_INPUTS);
            } else {
              $(".provision_custom_attributes_selector", create_vm_context).html("");
            }
          }
        })

        tab.on("click", "#provision_create_vm .provision-pricing-table.only-one" , function(){
          if ($(this).hasClass("selected")){
            $(this).removeClass("selected");
          } else {
            $(".provision-pricing-table", $(this).parents(".large-block-grid-3,.large-block-grid-2")).removeClass("selected")
            $(this).addClass("selected");
          }
        })

        $("#provision_create_vm").submit(function(){
          var context = $(this);

          var template_id = $(".tabs-content .content.active .selected", context).attr("opennebula_id");
          if (!template_id) {
            $(".alert-box-error", context).fadeIn().html(Locale.tr("You must select at least a template configuration"));
            return false;
          }

          var vm_name = $("#vm_name", context).val();
          var nics = NicsSection.retrieve(context);
          var disks = DisksResize.retrieve($(".provision_disk_selector", context));
          var instance_type = $(".provision_instance_types_ul .selected", context);

          var extra_info = {
            'vm_name' : vm_name,
            'template': {
            }
          }

          if (nics.length > 0) {
            extra_info.template.nic = nics;
          }

          if (disks.length > 0) {
            extra_info.template.DISK = disks;
          }

          if (instance_type.length > 0) {
            var instance_typa_data = instance_type.data("opennebula");
            delete instance_typa_data.name;

            $.extend(extra_info.template, instance_typa_data)
          }

          var missing_attr = false;
          var user_inputs_values = {};
          if ($(".provision_custom_attributes", $(this))) {
            $(".provision_custom_attribute", $(".provision_custom_attributes", $(this))).each(function(){
              if (!$(this).val()) {
                $(this).parent("label").css("color", "red");
                missing_attr = true;
              } else {
                $(this).parent("label").css("color", "#777");
                user_inputs_values[$(this).attr("attr_name")] = $(this).val();
              }
            })
          }

          if (missing_attr) {
            $(".alert-box-error", $(this)).fadeIn().html(Locale.tr("You have not specified all the Custom Atrributes for this VM"));
            return false;
          }

          if (!$.isEmptyObject(user_inputs_values)) {
             $.extend(extra_info.template, user_inputs_values)
          }

          Sunstone.runAction("Provision.instantiate", template_id, extra_info);
          return false;
        })

        $(document).on("click", ".provision_create_vm_button", function(){
          show_provision_create_vm();
        });


        //
        // Create FLOW
        //

        provision_flow_templates_datatable = $('#provision_flow_templates_table').dataTable({
          "iDisplayLength": 6,
          "sDom" : '<"H">t<"F"lp>',
          "aLengthMenu": [[6, 12, 36, 72], [6, 12, 36, 72]],
          "aaSorting"  : [[1, "asc"]],
          "aoColumnDefs": [
              { "bVisible": false, "aTargets": ["all"]}
          ],
          "aoColumns": [
              { "mDataProp": "DOCUMENT.ID" },
              { "mDataProp": "DOCUMENT.NAME" }
          ],
          "fnPreDrawCallback": function (oSettings) {
            // create a thumbs container if it doesn't exist. put it in the dataTables_scrollbody div
            if (this.$('tr', {"filter": "applied"} ).length == 0) {
              this.html('<div class="text-center">'+
                '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
                  '<i class="fa fa-cloud fa-stack-2x"></i>'+
                  '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
                '</span>'+
                '<br>'+
                '<br>'+
                '<span style="font-size: 18px; color: #999">'+
                  Locale.tr("There are no templates available")+
                '</span>'+
                '</div>');
            } else {
              $("#provision_flow_templates_table").html('<ul id="provision_flow_templates_ul" class="large-block-grid-3 medium-block-grid-3 small-block-grid-1 text-center"></ul>');
            }

            return true;
          },
          "fnRowCallback": function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
            var data = aData.DOCUMENT;
            var body = data.TEMPLATE.BODY;
            var logo;

            var roles_li = "";
            if (body.roles) {
              $.each(body.roles, function(index, role) {
                roles_li +=
                  '<li class="provision-bullet-item text-left" style="margin-left: 10px;margin-right: 10px;">'+
                    '<i class="fa fa-fw fa-cube"/>&emsp;'+
                    role.name+
                    '<span class="right">'+role.cardinality+" VMs</span>"+
                  '</li>';
              });
            }

            if (body.LOGO) {
              logo = '<span class="provision-logo" href="#">'+
                  '<img  src="'+body.LOGO+'">'+
                '</span>';
            } else {
              logo = '<span style="color: #bfbfbf; font-size: 60px;">'+
                '<i class="fa fa-fw fa-cubes"/>'+
              '</span>';
            }

            var li = $('<li>'+
                '<ul class="provision-pricing-table hoverable only-one" opennebula_id="'+data.ID+'">'+
                  '<li class="provision-title" title="'+data.NAME+'">'+
                    data.NAME+
                  '</li>'+
                  '<li style="height: 85px" class="provision-bullet-item">'+
                    logo +
                  '</li>'+
                  roles_li +
                  '<li class="provision-description" style="padding-top:0px">'+
                    (data.TEMPLATE.DESCRIPTION || '')+
                  '</li>'+
                '</ul>'+
              '</li>').appendTo($("#provision_flow_templates_ul"));

            $(".provision-pricing-table", li).data("opennebula", aData);

            return nRow;
          }
        });

        $('#provision_create_flow_template_search').on('input',function(){
          provision_flow_templates_datatable.fnFilter( $(this).val() );
        })

        $("#provision_create_flow_template_refresh_button").click(function(){
          OpenNebula.Action.clear_cache("SERVICE_TEMPLATE");
          update_provision_flow_templates_datatable(provision_flow_templates_datatable);

        });

        tab.on("click", ".provision_select_flow_template .provision-pricing-table.only-one" , function(){
          var context = $("#provision_create_flow");

          if ($(this).hasClass("selected")){
            $("#provision_customize_flow_template").hide();
            $("#provision_customize_flow_template").html("");
            $(".provision_network_selector", context).html("")
            $(".provision_custom_attributes_selector", context).html("")

            $(".provision_accordion_flow_template .selected_template").hide();
            $(".provision_accordion_flow_template .select_template").show();
          } else {
            $("#provision_customize_flow_template").show();
            $("#provision_customize_flow_template").html("");

            var data = $(this).data("opennebula");
            var body = data.DOCUMENT.TEMPLATE.BODY;

            $(".provision_accordion_flow_template .selected_template").show();
            $(".provision_accordion_flow_template .select_template").hide();
            $(".provision_accordion_flow_template .selected_template_name").html(body.name)
            $(".provision_accordion_flow_template .selected_template_logo").html('<i class="fa fa-cubes fa-lg"/>&emsp;');
            $(".provision_accordion_flow_template a").first().trigger("click");

            var context = $("#provision_create_flow");

            if (body.custom_attrs) {
              var network_attrs = [];
              var text_attrs = [];

              $.each(body.custom_attrs, function(key, value){
                var parts = value.split("|");
                // 0 mandatory; 1 type; 2 desc;
                var attrs = {
                  "name": key,
                  "mandatory": parts[0],
                  "type": parts[1],
                  "description": parts[2],
                }

                switch (parts[1]) {
                  case "vnet_id":
                    network_attrs.push(attrs)
                    break;
                  case "text":
                    text_attrs.push(attrs)
                    break;
                  case "password":
                    text_attrs.push(attrs)
                    break;
                }
              })

              if (network_attrs.length > 0) {
                generate_provision_network_accordion(
                  $(".provision_network_selector", context), true);

                $.each(network_attrs, function(index, vnet_attr){
                  generate_provision_network_table(
                    $(".provision_nic_accordion", context),
                    null,
                    vnet_attr);
                });
              }

              //if (text_attrs.length > 0) {
              //  generate_custom_attrs(
              //    $(".provision_custom_attributes_selector", context),
              //    text_attrs);
              //}
            } else {
              $(".provision_network_selector", context).html("")
              $(".provision_custom_attributes_selector", context).html("")
            }

            $.each(body.roles, function(index, role){
              var context = $('<div id="provision_create_flow_role_'+index+'" class="provision_create_flow_role">'+
                '<div class="row">'+
                  '<div class="large-10 large-centered columns">'+
                    '<h2 class="subheader">'+
                      '<i class="fa fa-cube fa-lg"></i>&emsp;'+
                      role.name+
                    '</h2>'+
                    '<br>'+
                  '</div>'+
                '</div>'+
                '<div class="row">'+
                  '<div class="provision_cardinality_selector large-9 large-centered columns">'+
                  '</div>'+
                '</div>'+
                '<br>'+
                '<div class="row">'+
                  '<div class="provision_custom_attributes_selector large-9 large-centered columns">'+
                  '</div>'+
                '</div>'+
              '</div>'+
              '<br>'+
              '<br>').appendTo($("#provision_customize_flow_template"))

              context.data("opennebula", role);

              var template_id = role.vm_template;
              var role_html_id = "#provision_create_flow_role_"+index;

              OpenNebula.Template.show({
                data : {
                    id: template_id
                },
                success: function(request,template_json){
                  var role_context = $(role_html_id)

                  generate_cardinality_selector(
                    $(".provision_cardinality_selector", context),
                    role,
                    template_json);

                  if (template_json.VMTEMPLATE.TEMPLATE.USER_INPUTS) {
                    generate_custom_attrs(
                      $(".provision_custom_attributes_selector", role_context),
                      template_json.VMTEMPLATE.TEMPLATE.USER_INPUTS);
                  } else {
                    $(".provision_custom_attributes_selector", role_context).html("");
                  }
                }
              })


            })

            $(document).foundation();
          }
        })

        tab.on("click", "#provision_create_flow .provision-pricing-table.only-one" , function(){
          if ($(this).hasClass("selected")){
            $(this).removeClass("selected");
          } else {
            $(".provision-pricing-table", $(this).parents(".large-block-grid-3,.large-block-grid-2")).removeClass("selected")
            $(this).addClass("selected");
          }
        })

        $("#provision_create_flow").submit(function(){
          var context = $(this);

          var flow_name = $("#flow_name", context).val();
          var template_id = $(".provision_select_flow_template .selected", context).attr("opennebula_id");

          if (!template_id) {
            $(".alert-box-error", context).fadeIn().html(Locale.tr("You must select at least a template configuration"));
            return false;
          }

          var custom_attrs = {}
          var missing_network = false;
          if ($(".provision_nic_accordion", context)) {
            $(".selected_network", $(".provision_nic_accordion", context)).each(function(){
              if (!$(this).attr("opennebula_id")) {
                $(this).css("color", "red");
                missing_network = true;
              } else {
                $(this).css("color", "#777");
                custom_attrs[$(this).attr("attr_name")] = $(this).attr("opennebula_id");
              }
            })
          }

          if (missing_network) {
            $(".alert-box-error", context).fadeIn().html(Locale.tr("You have not specified all the Networks for this Service"));
            return false;
          }

          var roles = [];
          var missing_attr = false;

          $(".provision_create_flow_role", context).each(function(){
            var user_inputs_values = {};
            if ($(".provision_custom_attributes", $(this))) {
              $(".provision_custom_attribute", $(".provision_custom_attributes", $(this))).each(function(){
                if (!$(this).val()) {
                  $(this).parent("label").css("color", "red");
                  missing_attr = true;
                } else {
                  $(this).parent("label").css("color", "#777");
                  user_inputs_values[$(this).attr("attr_name")] = $(this).val();
                }
              })
            }

            var role_template = $(this).data("opennebula");

            $.each(role_template.elasticity_policies, function(i, pol){
                pol.expression = htmlDecode(pol.expression);
            });

            roles.push($.extend(role_template, {
              "cardinality": $(".cardinality_value", $(this)).text(),
              "user_inputs_values": user_inputs_values
            }));
          })

          var extra_info = {
            'merge_template': {
              "name" : flow_name,
              "roles" : roles,
              "custom_attrs_values": custom_attrs
            }
          }

          if (missing_attr) {
            $(".alert-box-error", $(this)).fadeIn().html(Locale.tr("You have not specified all the Custom Atrributes for this Service"));
            return false;
          }

          Sunstone.runAction("Provision.Flow.instantiate", template_id, extra_info);
          return false;
        })

        $(".provision_create_flow_button").on("click", function(){
          show_provision_create_flow();
        });

        //
        // Group Info
        //


        $("#provision_vdc_info_button").on("click", function(){
          OpenNebula.Action.clear_cache("GROUP");
          show_provision_vdc_info();
        });

        //
        // Create User
        //

        var context = $("#provision_create_user");

        ProvisionQuotaWidget.setup(context);

        // Workaround to fix sliders. Apparently the setup fails while they are hidden
        context.on('click', 'a[href="#provision_create_user_manual_quota"]', function(){
          $(".provision_rvms_quota_input", context).change();
          $(".provision_memory_quota_input", context).change();
          $(".provision_memory_quota_tmp_input", context).change();
          $(".provision_cpu_quota_input", context).change();

          // Workaround until hidden Foundation.slider can be initialized
          setInterval(function() {
            context.foundation('slider', 'reflow');
          }, 20);
        });

        $("#provision_create_user").submit(function(){
          var context = $(this);

          var username = $("#username", context).val();
          var password = $("#password", context).val();
          var repeat_password = $("#repeat_password", context).val();

          // TODO driver
          var driver = 'core';

          if (!username.length || !password.length){
            $(".alert-box-error", context).fadeOut();
            $(".alert-box-error", context).fadeIn().html(Locale.tr("You have to provide a username and password"));
            return false;
          }

          if (password !== repeat_password){
            $(".alert-box-error", context).fadeOut();
            $(".alert-box-error", context).fadeIn().html(Locale.tr("Passwords do not match"));
            return false;
          }

          var user_json = { "user" :
                            { "name" : username,
                              "password" : password,
                              "auth_driver" : driver
                            }
                          };

          Sunstone.runAction("Provision.User.create",user_json);
          $(".alert-box-error", context).html('<div class="text-center">'+
            '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
              '<i class="fa fa-cloud fa-stack-2x"></i>'+
              '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>'+
            '</span>'+
            '<br>'+
            '<br>'+
            '<span style="font-size: 18px; color: #999">'+
            '</span>'+
            '</div>');

          return false;
        });

        $(document).on("click", ".provision_create_user_button", function(){
          show_provision_create_user();
        });
      }
    });
  }

});

define('tabs/users-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "User.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "User.create_dialog" : {
      type: "create_dialog",
      layout: "create"
    },
    "User.update_password" : {
      type : "action",
      layout: "main_buttons",
      text : Locale.tr("Password")
    },
    "User.change_authentication" : {
      type : "action",
      layout: "main_buttons",
      text : Locale.tr("Auth")
    },
    "User.quotas_dialog" : {
      type : "action",
      layout: "main_buttons",
      text : Locale.tr("Quotas")
    },
    "User.chgrp" : {
      type: "confirm_with_select",
      text: Locale.tr("Change group"),
      layout: "user_select",
      select: "Group",
      tip: Locale.tr("This will change the main group of the selected users. Select the new group")+":"
    },
    "User.addgroup" : {
      type: "confirm_with_select",
      text: Locale.tr("Add to group"),
      layout: "user_select",
      select: "Group",
      tip: Locale.tr("This will add the user to a secondary group. Select the new group")+":"
    },
    "User.delgroup" : {
      type: "confirm_with_select",
      text: Locale.tr("Remove from group"),
      layout: "user_select",
      select: "Group",
      tip: Locale.tr("This will remove the user from a secondary group. Select the group")+":"
    },
    "User.delete" : {
      type: "confirm",
      text: Locale.tr("Delete"),
      layout: "del"
    }
  };

  return Buttons;
});


/* START_TEMPLATE */
define('hbs!utils/tab-datatable/empty-table',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"text-center\" style=\"font-size: 18px; color: #999\">\n  <br>\n  <span class=\"fa-stack fa-3x\" style=\"color: #dfdfdf\"> \n    <i class=\"fa fa-cloud fa-stack-2x\"></i> \n    <i class=\"fa fa-info-circle fa-stack-1x fa-inverse\"></i>\n  </span>\n  <br>\n  <span style=\" color: #999\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "There is no data available", {"name":"tr","hash":{},"data":data})))
    + "</span>\n</div>\n<br>";
},"useData":true});
Handlebars.registerPartial('utils/tab-datatable/empty-table', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!utils/tab-datatable/table',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "  <div class=\"row\">\n    <div class=\"large-8 small-2 columns\">\n      <a id=\"refresh_button_"
    + escapeExpression(((helper = (helper = helpers.dataTableId || (depth0 != null ? depth0.dataTableId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableId","hash":{},"data":data}) : helper)))
    + "\" href=\"#\" class=\"button small radius secondary\"> <i class=\"fa fa-refresh\" /></a>\n    </div>\n    <div class=\"large-4 small-10 columns\">\n      <input id=\""
    + escapeExpression(((helper = (helper = helpers.dataTableId || (depth0 != null ? depth0.dataTableId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableId","hash":{},"data":data}) : helper)))
    + "_search\" class=\"search\" type=\"search\" placeholder=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Search", {"name":"tr","hash":{},"data":data})))
    + "\"/>\n    </div>\n  </div>\n";
},"3":function(depth0,helpers,partials,data) {
  var lambda=this.lambda, escapeExpression=this.escapeExpression;
  return "        <th>"
    + escapeExpression(lambda(depth0, depth0))
    + "</th>\n";
},"5":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda;
  return "  <div class=\"row\">\n    <div class=\"large-12 columns\" id=\"selected_ids_row_"
    + escapeExpression(((helper = (helper = helpers.dataTableId || (depth0 != null ? depth0.dataTableId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableId","hash":{},"data":data}) : helper)))
    + "\">\n      <label for=\"selected_resource_id_"
    + escapeExpression(((helper = (helper = helpers.dataTableId || (depth0 != null ? depth0.dataTableId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableId","hash":{},"data":data}) : helper)))
    + "\" id=\"select_resource_"
    + escapeExpression(((helper = (helper = helpers.dataTableId || (depth0 != null ? depth0.dataTableId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableId","hash":{},"data":data}) : helper)))
    + "\" class=\"radius secondary label\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.selectOptions : depth0)) != null ? stack1.select_resource : stack1), depth0))
    + "</label>\n      <span id=\"selected_resource_"
    + escapeExpression(((helper = (helper = helpers.dataTableId || (depth0 != null ? depth0.dataTableId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableId","hash":{},"data":data}) : helper)))
    + "\" class=\"radius secondary label\" style=\"display: none;\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.selectOptions : depth0)) != null ? stack1.you_selected : stack1), depth0))
    + "</span>\n      <span id=\"select_resource_multiple_"
    + escapeExpression(((helper = (helper = helpers.dataTableId || (depth0 != null ? depth0.dataTableId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableId","hash":{},"data":data}) : helper)))
    + "\" class=\"radius secondary label\" style=\"display: none;\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.selectOptions : depth0)) != null ? stack1.select_resource_multiple : stack1), depth0))
    + "</span>\n      <span id=\"selected_resource_multiple_"
    + escapeExpression(((helper = (helper = helpers.dataTableId || (depth0 != null ? depth0.dataTableId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableId","hash":{},"data":data}) : helper)))
    + "\" class=\"radius secondary label\" style=\"display: none;\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.selectOptions : depth0)) != null ? stack1.you_selected_multiple : stack1), depth0))
    + "</span>\n      <span id=\"selected_resource_name_"
    + escapeExpression(((helper = (helper = helpers.dataTableId || (depth0 != null ? depth0.dataTableId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableId","hash":{},"data":data}) : helper)))
    + "\" class=\"radius label\" type=\"text\"></span>\n      <input id=\"selected_resource_id_"
    + escapeExpression(((helper = (helper = helpers.dataTableId || (depth0 != null ? depth0.dataTableId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableId","hash":{},"data":data}) : helper)))
    + "\" type=\"text\" style=\"visibility: hidden; width:0px !important; height:0px !important;padding: 0px !important;margin: 0px !important;display: inline;\"/>\n    </div>\n  </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dataTableId || (depth0 != null ? depth0.dataTableId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableId","hash":{},"data":data}) : helper)))
    + "Container\">\n";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.conf : depth0)) != null ? stack1.select : stack1), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "  <table id=\""
    + escapeExpression(((helper = (helper = helpers.dataTableId || (depth0 != null ? depth0.dataTableId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableId","hash":{},"data":data}) : helper)))
    + "\" class=\"datatable twelve\">\n    <thead>\n      <tr>\n        <th class=\"check\">\n          <input type=\"checkbox\" class=\"check_all\" value=\"\"/>\n        </th>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.columns : depth0), {"name":"each","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "      </tr>\n    </thead>\n    <tbody></tbody>\n  </table>\n";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.conf : depth0)) != null ? stack1.select : stack1), {"name":"if","hash":{},"fn":this.program(5, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>";
},"useData":true});
Handlebars.registerPartial('utils/tab-datatable/table', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!utils/tab-datatable/search-input',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<input id=\""
    + escapeExpression(((helper = (helper = helpers.dataTableSearchId || (depth0 != null ? depth0.dataTableSearchId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dataTableSearchId","hash":{},"data":data}) : helper)))
    + "\" type=\"search\" placeholder=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Search", {"name":"tr","hash":{},"data":data})))
    + "\" />";
},"useData":true});
Handlebars.registerPartial('utils/tab-datatable/search-input', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/tab-datatable',['require','foundation-datatables','hbs!./tab-datatable/empty-table','sunstone','sunstone-config','utils/locale','utils/tips','opennebula','utils/notifier','opennebula/user','hbs!./tab-datatable/table','hbs!./tab-datatable/search-input'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation-datatables');
  var TemplateEmptyTable = require('hbs!./tab-datatable/empty-table');
  var Sunstone = require('sunstone');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var OpenNebula = require('opennebula');
  var Notifier = require('utils/notifier');
  var OpenNebulaUser = require('opennebula/user');

  /*
    TEMPLATES
   */

  var TemplateDataTableHTML = require('hbs!./tab-datatable/table');
  var TemplateSearchInputHTML = require('hbs!./tab-datatable/search-input');

  /*
    CONSTANTS
   */

  var SPINNER = '<img src="images/ajax-loader.gif" alt="retrieving" class="loading_img"/>';

  /*
    GLOBAL INITIALIZATION
   */

  /* Set the defaults for DataTables initialisation */
  $.extend(true, $.fn.dataTable.defaults, {
    dom: "t<'row collapse'<'small-6 columns'i><'small-6 columns'lp>>",
    renderer: 'foundation',
    language: {
      "sLengthMenu": "_MENU_",
      "emptyTable": TemplateEmptyTable()
    }
  });

  /*
    CONSTRUCTOR
   */

  /* Child class must define:
    this.dataTableId
    this.resource
    this.dataTableOptions
    this.columns
    this.conf = {
      'info': true,     enable on click row will show the element
      'action': true,   enable actions on row elements
      'select': true,   enable selecting elements from the table
      'selectOptions': {
        'filter_fn': function(ds) { return ds.TYPE == 0; }
      }
      'customTabContext': jquery selector used when the datatable has associated
                          buttons. By default it will be the parent tab
      'customTrListener': function executed when a tr is clicked. Arguments
                          are (tableObj, tr)
    }

    1. The table HTML is returned calling the table.dataTableHTML attr
    2. The table must be initialized after including it in the DOM,
        using the table.initilize() method
    3. After that all the methods can be called on the table,
        depending on the functionalities enabled (info, action, select)
  */
  function TabDatatable() {
    var that = this;
    if (that.conf.select) {
      if (!that.selectOptions.select_resource) {
        that.selectOptions.select_resource = Locale.tr("Please select a resource from the list");
      }

      if (!that.selectOptions.you_selected) {
        that.selectOptions.you_selected = Locale.tr("You selected the following resource:");
      }

      if (that.selectOptions.id_index == undefined) {
        that.selectOptions.id_index = 0;
      }

      $.extend(that.selectOptions, that.conf.selectOptions);

      that.selectOptions.fixed_ids_map_orig = {};
      if (that.selectOptions.fixed_ids != undefined) {
        $.each(that.selectOptions.fixed_ids, function() {
          that.selectOptions.fixed_ids_map_orig[this] = true;
        });
      }

      that.selectOptions.starred_ids_map = {};
      if (that.selectOptions.starred_ids != undefined) {
        $.each(that.selectOptions.starred_ids, function() {
          that.selectOptions.starred_ids_map[this] = true;
        });
      }

      if (that.selectOptions.multiple_choice == undefined) {
        that.selectOptions.multiple_choice = false;
      }
    }

    that.dataTableHTML = TemplateDataTableHTML({
                          'dataTableId': this.dataTableId,
                          'columns': this.columns,
                          'conf': this.conf,
                          'selectOptions': this.selectOptions});

    that.searchInputHTML = TemplateSearchInputHTML({'dataTableSearchId': this.dataTableId + 'Search'});

    return that;
  }

  TabDatatable.prototype = {
    'initialize': _initialize,
    'initCheckAllBoxes': _initCheckAllBoxes,
    'tableCheckboxesListener': _tableCheckboxesListener,
    'onlyOneCheckboxListener': _onlyOneCheckboxListener,
    'infoListener': _infoListener,
    'addElement': _addElement,
    'deleteElement': _deleteElement,
    'updateElement': _updateElement,
    'elements': _elements,
    'updateView': _updateView,
    'getElementData': _getElementData,
    'waitingNodes': _waitingNodes,
    'recountCheckboxes': _recountCheckboxes,
    'filter': _filter,
    'resetResourceTableSelect': _resetResourceTableSelect,
    'refreshResourceTableSelect': _refreshResourceTableSelect,
    'selectResourceTableSelect': _selectResourceTableSelect,
    'retrieveResourceTableSelect': _retrieveResourceTableSelect,
    'idInput': _idInput,
    'initSelectResourceTableSelect': _initSelectResourceTableSelect,
    'updateFn': _updateFn,
    'list': _list
  }

  return TabDatatable;

  /*
    FUNCTION DEFINITIONS
   */

  function _initialize(opts) {
    if (this.conf.select) {
      if (opts && opts.selectOptions) {
        $.extend(this.selectOptions, opts.selectOptions);
      }

      this.initSelectResourceTableSelect();
    } else {
      this.dataTableOptions.pageLength = parseInt(config['page_length']);
    }

    this.dataTable = $('#' + this.dataTableId).dataTable(this.dataTableOptions);

    // Remember page length only for non selectable datatables
    if (!this.conf.select) {
      this.dataTable.on( 'length.dt', function ( e, settings, len ) {
        config['page_length'] = len;

        var template_str = 'TABLE_DEFAULT_PAGE_LENGTH = "'+len+'"';

        Sunstone.runAction("User.append_template", config['user_id'], template_str);
       });
    }

    var that = this;
    $('#' + this.dataTableId + 'Search').on('input', function() {
      that.dataTable.fnFilter($(this).val());
      return false;
    });

    this.dataTable.on('draw.dt', function() {
      that.recountCheckboxes();
    })

    if (this.selectOptions && this.selectOptions.id_index) {
      this.dataTable.fnSort([[this.selectOptions.id_index, config['user_config']['table_order']]]);
    } else {
      this.dataTable.fnSort([[1, SunstoneConfig.tableOrder]]);
    }

    if (this.conf.actions) {
      this.initCheckAllBoxes();
      this.tableCheckboxesListener();
    }

    if (this.conf.oneSelection == true) {
      this.onlyOneCheckboxListener();
      $(".check_all", that.dataTable).hide();
    }

    if (this.conf.info) {
      this.infoListener(_defaultTrListener);
    } else if (this.conf.customTrListener) {
      this.infoListener(this.conf.customTrListener);
    } else if (!this.conf.select){
      this.infoListener();
    }

    if (this.conf.select) {
      that.dataTable.fnSetColumnVis(0, false);
    }
  }

  function _defaultTrListener(tableObj, tr) {
    var aData = tableObj.dataTable.fnGetData(tr);
    if (!aData) return true;
    var id = $(aData[0]).val();
    if (!id) return true;

    Sunstone.showElement(tableObj.tabId, tableObj.resource + ".show", id);

    return false;
  }

  //Shows run a custom action when clicking on rows.
  function _infoListener(info_action) {
    var that = this;
    this.dataTable.on("click", 'tbody tr', function(e) {
      if ($(e.target).is('input') || $(e.target).is('select') || $(e.target).is('option')) {
        return true;
      }

      if (info_action) {
        //If ctrl is hold down, make check_box click
        if (e.ctrlKey || e.metaKey || $(e.target).is('input')) {
          $('.check_item', this).trigger('click');
        } else {
          info_action(that, this);
        }
      } else {
        $('.check_item', this).trigger('click');
      }

      return false;
    });
  }

  //call back for actions creating a zone element
  function _addElement(request, element_json) {
    var element = this.elementArray(element_json);
    this.dataTable.fnAddData(element);
  }

  //deletes an element with id 'tag' from a dataTable
  function _deleteElement(req) {
    var tag = '#' + this.resource.toLowerCase() + '_' + req.request.data;
    var tr = $(tag, this.dataTable).parents('tr')[0];
    this.dataTable.fnDeleteRow(tr);
    this.recountCheckboxes();

    var tab = this.dataTable.parents(".tab");
    if (Sunstone.rightInfoVisible(tab)) {
      $("a[href='back']", tab).click();
    }
  }

  //Add a listener to the check-all box of a datatable, enabling it to
  //check and uncheck all the checkboxes of its elements.
  function _initCheckAllBoxes() {
    var that = this;
    this.dataTable.on("change", '.check_all', function() {
      var table = $(this).closest('.dataTables_wrapper');
      if ($(this).is(":checked")) { //check all
        $('tbody input.check_item', table).prop('checked', true).change();
        $('td', table).addClass('markrowchecked');
      } else { //uncheck all
        $('tbody input.check_item', table).prop('checked', false).change();
        $('td', table).removeClass('markrowchecked');
      };

      that.recountCheckboxes();
    });
  }

  //Handle the activation of action buttons and the check_all box
  //when elements in a datatable are modified.
  function _recountCheckboxes() {
    var table = $('tbody', this.dataTable);

    var context;
    if (this.conf.customTabContext) {
      context = this.conf.customTabContext;
    } else {
      context = table.parents('.tab');
      if ($(".right-info", context).is(':visible')) {
        return;
      }
    }

    var nodes = $('tr', table); //visible nodes
    var total_length = nodes.length;
    var checked_length = $('input.check_item:checked', nodes).length;
    var last_action_b = $('.last_action_button', context);

    if (checked_length) { //at least 1 element checked
      //enable action buttons
      $('.top_button, .list_button', context).prop('disabled', false);

      //enable checkall box
      if (total_length == checked_length) {
        $('.check_all', this.dataTable).prop('checked', true);
      } else {
        $('.check_all', this.dataTable).prop('checked', false);
      };
    } else { //no elements cheked
      //disable action buttons, uncheck checkAll
      $('.check_all', this.dataTable).prop('checked', false);
      $('.top_button, .list_button', context).prop('disabled', true).attr('disabled', 'disabled');
    };

    //any case the create dialog buttons should always be enabled.
    $('.create_dialog_button', context).prop('disabled', false);
    $('.alwaysActive', context).prop('disabled', false);
  }

  //Init action buttons and checkboxes listeners
  function _tableCheckboxesListener() {
    //Initialization - disable all buttons
    var context = this.conf.customTabContext || this.dataTable.parents('.tab');

    $('.last_action_button', context).prop('disabled', true);
    $('.top_button, .list_button', context).prop('disabled', true);
    //These are always enabled
    $('.create_dialog_button', context).prop('disabled', false);
    $('.alwaysActive', context).prop('disabled', false);

    //listen to changes in the visible inputs
    var that = this;
    this.dataTable.on("change", 'tbody input.check_item', function() {
      var datatable = $(this).parents('table');

      if ($(this).is(":checked")) {
        $(this).parents('tr').children().addClass('markrowchecked');
      } else {
        $(this).parents('tr').children().removeClass('markrowchecked');
      }

      that.recountCheckboxes();
    });
  }

  /*
   * onlyOneCheckboxListener: Only one box can be checked
   */

  function _onlyOneCheckboxListener() {
    var that = this;
    this.dataTable.on("change", 'tbody input.check_item', function() {
      var checked = $(this).is(':checked');
      $('td', that.dataTable).removeClass('markrowchecked');
      $('input.check_item:checked', that.dataTable).prop('checked', false);
      $("td", $(this).closest('tr')).addClass('markrowchecked')
      $(this).prop('checked', checked);
    });
  }

  // Updates a data_table, with a 2D array containing the new values
  // Does a partial redraw, so the filter and pagination are kept
  // fromArray if true do not process the list since it is already an array of elements
  function _updateView(request, list, fromArray) {
    var selected_row_id = null;
    var checked_row_ids = new Array();
    var that = this;

    if (that.preUpdateView) {
      that.preUpdateView();
    }

    var row_id_index = this.dataTable.attr("row_id");

    if (row_id_index != undefined) {
      $.each($(this.dataTable.fnGetNodes()), function() {
        if ($('td.markrow', this).length != 0) {
          var aData = that.dataTable.fnGetData(this);

          selected_row_id = aData[row_id_index];

        }
      });
    }

    $.each($(this.dataTable.fnGetNodes()), function() {
      if ($('td.markrowchecked', this).length != 0) {
        if (!isNaN($($('td', $(this))[1]).html())) {
          checked_row_ids.push($($('td', $(this))[1]).html());
        } else {
          checked_row_ids.push($($('td', $(this))[0]).html());
        }
      }
    });

    // dataTable.fnSettings is undefined when the table has been detached from
    // the DOM

    if (this.dataTable && this.dataTable.fnSettings()) {
      var dTable_settings = this.dataTable.fnSettings();
      var prev_start = dTable_settings._iDisplayStart;

      this.dataTable.fnClearTable(false);

      var item_list;
      if (fromArray) {
        item_list = list;
      } else {
        item_list = [];
        $.each(list, function() {
          var item = that.elementArray(this);
          if (item){
            item_list.push(item);
          }
        });
      }

      var that = this;
      if (item_list.length > 0) {
        that.dataTable.fnAddData(item_list, false);
      }

      var new_start = prev_start;

      if (new_start > item_list.length - 1) {
        if (item_list.length > 0)
            new_start = item_list.length - 1;
        else
            new_start = 0;
      }

      dTable_settings.iInitDisplayStart = new_start;

      this.dataTable.fnDraw(true);
    };

    if (selected_row_id != undefined) {
      $.each($(this.dataTable.fnGetNodes()), function() {

        var aData = that.dataTable.fnGetData(this);

        if (aData[row_id_index] == selected_row_id) {
          $('td', this)[0].click();
        }
      });
    }

    if (checked_row_ids.length != 0) {
      $.each($(this.dataTable.fnGetNodes()), function() {
        var current_id = $($('td', this)[1]).html();

        if (isNaN(current_id)) {
          current_id = $($('td', this)[0]).html();
        }

        if (current_id) {
          if ($.inArray(current_id, checked_row_ids) != -1) {
            $('input.check_item:not(:checked)', this).first().click();
            $('td', this).addClass('markrowchecked');
          }
        }
      });
    }

    if (that.postUpdateView) {
      that.postUpdateView();
    }
  }

  //replaces an element with id 'tag' in a dataTable with a new one
  function _updateElement(request, element_json) {
    var id = element_json[this.xmlRoot].ID;
    var element = this.elementArray(element_json);
    var tag = '#' + this.resource.toLowerCase() + '_' + id;
    // fnGetData should be used instead, otherwise it depends on the visible columns
    var nodes = this.dataTable.fnGetNodes();
    var tr = $(tag, nodes).parents('tr')[0];
    if (tr) {
      var checked_val = $('input.check_item', tr).prop('checked');
      var position = this.dataTable.fnGetPosition(tr);
      this.dataTable.fnUpdate(element, position, undefined, false);
      $('input.check_item', tr).prop('checked', checked_val);
      this.recountCheckboxes();
    }
  }

  function _getElementData(id, resource_tag) {
    var nodes = this.dataTable.fnGetNodes();
    var tr = $(resource_tag + '_' + id, nodes).parents('tr')[0];
    return this.dataTable.fnGetData(tr);
  }

  function _waitingNodes() {
    $('tr input.check_item:visible', this.dataTable).replaceWith(SPINNER);
  }

  //returns an array of ids of selected elements in a dataTable
  function _elements(forceDataTable) {
    var selected_nodes = [];
    if (this.dataTable) {
      var tab = this.dataTable.parents(".tab")
      if (Sunstone.rightInfoVisible(tab) && !forceDataTable) {
        selected_nodes.push(Sunstone.rightInfoResourceId(tab));
      } else {
        //Which rows of the datatable are checked?
        var nodes = $('tbody input.check_item:checked', this.dataTable);
        $.each(nodes, function() {
          selected_nodes.push($(this).val());
        });
      }
    };
    return selected_nodes;
  }

  function _filter(value, columnId) {
    this.dataTable.fnFilter(value, columnId);
  }

  /*
    SELECT RESOURCE FUNCTION DEFINITIONS
   */

  function _initSelectResourceTableSelect() {
    var that = this;
    var section = $('#' + that.dataTableId + 'Container');

    if (that.selectOptions.id_index == undefined) {
      that.selectOptions.id_index = 0;
    }

    if (that.selectOptions.name_index == undefined) {
      that.selectOptions.name_index = 1;
    }

    if (that.selectOptions.dataTable_options == undefined) {
      that.selectOptions.dataTable_options = {};
    }

    if (that.selectOptions.select_callback == undefined) {
      that.selectOptions.select_callback = function() {};
    }

    if (that.selectOptions.unselect_callback == undefined) {
      that.selectOptions.unselect_callback = function() {};
    }

    if (that.selectOptions.multiple_choice) {
      that.dataTableOptions.fnRowCallback = function(nRow, aData, iDisplayIndex, iDisplayIndexFull) {
        var row_id = aData[that.selectOptions.id_index];

        var ids = $('#selected_ids_row_' + that.dataTableId, section).data("ids");
        if (ids[row_id]) {
          $("td", nRow).addClass('markrowchecked');
          $('input.check_item', nRow).prop('checked', true);
        } else {
          $("td", nRow).removeClass('markrowchecked');
          $('input.check_item', nRow).prop('checked', false);
        }
      };
    } else {
      that.dataTableOptions.fnRowCallback = function(nRow, aData, iDisplayIndex, iDisplayIndexFull) {
        var row_id = aData[that.selectOptions.id_index];

        var selected_id = $('#selected_resource_id_' + that.dataTableId, section).val();

        if (row_id == selected_id) {
          $("td", nRow).addClass('markrow');
          $('input.check_item', nRow).prop('checked', true);
        } else {
          $("td", nRow).removeClass('markrow');
          $('input.check_item', nRow).prop('checked', false);
        }
      };
    }

    $('#refresh_button_' + that.dataTableId, section).off("click");
    section.on('click', '#refresh_button_' + that.dataTableId, function() {
      that.updateFn();
      return false;
    });

    $('#' + that.dataTableId + '_search', section).on('input', function() {
      that.dataTable.fnFilter($(this).val());
      return false;
    })

    if (that.selectOptions.read_only) {
      $('#selected_ids_row_' + that.dataTableId, section).hide();
    } else if (that.selectOptions.multiple_choice) {
      $('#selected_resource_' + that.dataTableId, section).hide();
      $('#select_resource_' + that.dataTableId, section).hide();

      $('#selected_resource_multiple_' + that.dataTableId, section).hide();
      $('#select_resource_multiple_' + that.dataTableId, section).show();
    }

    $('#selected_resource_name_' + that.dataTableId, section).hide();

    $('#selected_ids_row_' + that.dataTableId, section).data("options", that.selectOptions);

    if (that.selectOptions.read_only) {

    } else if (that.selectOptions.multiple_choice) {
      $('#selected_ids_row_' + that.dataTableId, section).data("ids", {});

      function row_click(row, aData) {
        that.dataTable.unbind("draw");

        var row_id = aData[that.selectOptions.id_index];
        var row_name = aData[that.selectOptions.name_index];

        var ids = $('#selected_ids_row_' + that.dataTableId, section).data("ids");

        if (ids[row_id]) {
          delete ids[row_id];

          // Happens if row is not yet rendered (i.e. higher unvisited page)
          if (row != undefined) {
            $("td", row).removeClass('markrowchecked');
            $('input.check_item', row).prop('checked', false);
          }

          $('#selected_ids_row_' + that.dataTableId + ' span[row_id="' + row_id + '"]', section).remove();

          that.selectOptions.unselect_callback();
        } else {
          ids[row_id] = true;

          // Happens if row is not yet rendered (i.e. higher unvisited page)
          if (row != undefined) {
            $("td", row).addClass('markrowchecked');
            $('input.check_item', row).prop('checked', true);
          }

          $('#selected_ids_row_' + that.dataTableId, section).append('<span row_id="' + row_id + '" class="radius label">' + row_name + ' <span class="fa fa-times blue"></span></span> ');

          that.selectOptions.select_callback(aData, that.selectOptions);
        }

        if ($.isEmptyObject(ids)) {
          $('#selected_resource_multiple_' + that.dataTableId, section).hide();
          $('#select_resource_multiple_' + that.dataTableId, section).show();
        } else {
          $('#selected_resource_multiple_' + that.dataTableId, section).show();
          $('#select_resource_multiple_' + that.dataTableId, section).hide();
        }

        $('.alert-box', section).hide();

        return true;
      };

      $('#' + that.dataTableId + ' tbody', section).on("click", "tr", function(e) {
        var aData = that.dataTable.fnGetData(this);
        row_click(this, aData);
      });

      $(section).on("click", '#selected_ids_row_' + that.dataTableId + ' span.fa.fa-times', function() {
        var row_id = $(this).parent("span").attr('row_id');

        var found = false;

        var aData = that.dataTable.fnGetData();
        // TODO: improve preformance, linear search
        $.each(aData, function(index, row) {
          if (row[that.selectOptions.id_index] == row_id) {
            found = true;
            row_click(that.dataTable.fnGetNodes(index), row);
            return false;
          }
        });

        if (!found) {
          var ids = $('#selected_ids_row_' + that.dataTableId, section).data("ids");
          delete ids[row_id];
          $('#selected_ids_row_' + that.dataTableId + ' span[row_id="' + row_id + '"]', section).remove();

          if ($.isEmptyObject(ids)) {
            $('#selected_resource_multiple_' + that.dataTableId, section).hide();
            $('#select_resource_multiple_' + that.dataTableId, section).show();
          } else {
            $('#selected_resource_multiple_' + that.dataTableId, section).show();
            $('#select_resource_multiple_' + that.dataTableId, section).hide();
          }
        }

        that.selectOptions.unselect_callback(aData, that.selectOptions);
      });
    } else {
      $('#' + that.dataTableId + ' tbody', section).delegate("tr", "click", function(e) {
        that.dataTable.unbind("draw");
        var aData = that.dataTable.fnGetData(this);

        $("td.markrow", that.dataTable).removeClass('markrow');
        $('tbody input.check_item', that.dataTable).prop('checked', false);

        $('#selected_resource_' + that.dataTableId, section).show();
        $('#select_resource_' + that.dataTableId, section).hide();
        $('.alert-box', section).hide();

        $("td", this).addClass('markrow');
        $('input.check_item', this).prop('checked', true);

        $('#selected_resource_id_' + that.dataTableId, section).val(aData[that.selectOptions.id_index]).change();

        $('#selected_resource_name_' + that.dataTableId, section).text(aData[that.selectOptions.name_index]).change();
        $('#selected_resource_name_' + that.dataTableId, section).show();

        that.selectOptions.select_callback(aData, that.selectOptions);

        return true;
      });
    }

    Tips.setup(section);
  }

  function _resetResourceTableSelect() {
    var that = this;
    var section = $('#' + that.dataTableId + 'Container');

    // TODO: do for multiple_choice

    // TODO: works for more than one page?

    $("td.markrow", that.dataTable).removeClass('markrow');
    $('tbody input.check_item', that.dataTable).prop('checked', false);

    $('#' + that.dataTableId + '_search', section).val("").trigger("input");
    $('#refresh_button_' + that.dataTableId).click();

    $('#selected_resource_name_' + that.dataTableId, section).text("").hide();

    $('#selected_resource_' + that.dataTableId, section).hide();
    $('#select_resource_' + that.dataTableId, section).show();
  }

  // Returns an ID, or an array of IDs for that.selectOptions.multiple_choice
  function _retrieveResourceTableSelect() {
    var that = this;
    var section = $('#' + that.dataTableId + 'Container');

    if (that.selectOptions.multiple_choice) {
      var ids = $('#selected_ids_row_' + that.dataTableId, section).data("ids");

      var arr = [];

      $.each(ids, function(key, val) {
        arr.push(key);
      });

      return arr;
    } else {
      return $('#selected_resource_id_' + that.dataTableId, section).val();
    }
  }

  /**
   * Returns the jquery selector for the ID input. Can be used to add attributes
   * to it, such as 'wizard_field'
   * @return {Object} jquery selector for the ID input
   */
  function _idInput() {
    var that = this;
    var section = $('#' + that.dataTableId + 'Container');

    if (that.selectOptions.multiple_choice) {
      return $('#selected_ids_row_' + that.dataTableId, section);
    } else {
      return $('#selected_resource_id_' + that.dataTableId, section);
    }
  }

  // Clicks the refresh button
  function _refreshResourceTableSelect() {
    var that = this;
    var section = $('#' + that.dataTableId + 'Container');
    $('#refresh_button_' + that.dataTableId, section).click();
  }

  /**
   * Clears the current selection, and selects the given IDs
   * @param  {object} selectedResources Two alternatives, ids or names.
   *                - selectedResources.ids must be a single ID,
   *                            or an array of IDs for options.multiple_choice
   *                - selectedResources.names must be an array of {name, uname}
   */
  function _selectResourceTableSelect(selectedResources) {
    var that = this;
    var section = $('#' + that.dataTableId + 'Container');

    if (that.selectOptions.multiple_choice) {
      that.refreshResourceTableSelect(section, that.dataTableId);

      var data_ids = {};

      $('#selected_ids_row_' + that.dataTableId + ' span[row_id]', section).remove();

      if (selectedResources.ids == undefined) {
        selectedResources.ids = [];
      }

      // TODO: {name, uname} support for multiple_choice

      $.each(selectedResources.ids, function(index, row_id) {
        if (isNaN(row_id)) {
          return true;
        }

        data_ids[row_id] = true;

        var row_name = "" + row_id;

        // TODO: improve preformance, linear search. Needed to get the
        // name of the resource in the label. If function getName() was
        // indexed in the cache, it could be used here
        $.each(that.dataTable.fnGetData(), function(index, row) {
          if (row[that.selectOptions.id_index] == row_id) {
            row_name = row[that.selectOptions.name_index];
            return false;
          }
        });

        $('#selected_ids_row_' + that.dataTableId, section).append('<span row_id="' + row_id + '" class="radius label">' + row_name + ' <span class="fa fa-times blue"></span></span> ');
      });

      $('#selected_ids_row_' + that.dataTableId, section).data("ids", data_ids);

      if ($.isEmptyObject(data_ids)) {
        $('#selected_resource_multiple_' + that.dataTableId, section).hide();
        $('#select_resource_multiple_' + that.dataTableId, section).show();
      } else {
        $('#selected_resource_multiple_' + that.dataTableId, section).show();
        $('#select_resource_multiple_' + that.dataTableId, section).hide();
      }

      $('.alert-box', section).hide();

      that.dataTable.fnDraw();
    } else {
      $("td.markrow", that.dataTable).removeClass('markrow');
      $('tbody input.check_item', that.dataTable).prop('checked', false);

      $('#selected_resource_' + that.dataTableId, section).show();
      $('#select_resource_' + that.dataTableId, section).hide();
      $('.alert-box', section).hide();

      var row_id = undefined;
      var row_name = "";

      if (selectedResources.ids != undefined) {

        row_id = selectedResources.ids;

        row_name = "" + row_id;

        // TODO: improve preformance, linear search. Needed to get the
        // name of the resource in the label. If function getName() was
        // indexed in the cache, it could be used here
        $.each(that.dataTable.fnGetData(), function(index, row) {
          if (row[that.selectOptions.id_index] == row_id) {
            row_name = row[that.selectOptions.name_index];
            return false;
          }
        });
      } else if (selectedResources.names != undefined) {
        row_name = selectedResources.names.name;
        var row_uname = selectedResources.names.uname;

        $.each(that.dataTable.fnGetData(), function(index, row) {
          if (row[that.selectOptions.name_index] == row_name &&
             row[that.selectOptions.uname_index] == row_uname) {

            row_id = row[that.selectOptions.id_index];
            return false;
          }
        });
      }

      //        $("td", this).addClass('markrow');
      //        $('input.check_item', this).prop('checked', true);

      $('#selected_resource_id_' + that.dataTableId, section).val(row_id).change();

      $('#selected_resource_name_' + that.dataTableId, section).text(row_name).change();
      $('#selected_resource_name_' + that.dataTableId, section).show();

      that.refreshResourceTableSelect(section, that.dataTableId);
    }
  }

  function _updateFn() {
    var that = this;
    var success_func = function (request, resource_list) {
      var list_array = [];

      var fixed_ids_map = $.extend({}, that.selectOptions.fixed_ids_map_orig);

      $.each(resource_list, function() {
        var add = true;

        if (that.selectOptions.filter_fn) {
          add = that.selectOptions.filter_fn(this[that.xmlRoot]);
        }

        if (that.selectOptions.fixed_ids != undefined) {
          add = (add && fixed_ids_map[this[that.xmlRoot].ID]);
        }

        var elementArray;

        if (add) {
          elementArray = that.elementArray(this);
          add = (elementArray != false);
        }

        if (add) {
          if (that.selectOptions.starred_ids != undefined){
            if (that.selectOptions.starred_ids_map[this[that.xmlRoot].ID]){
              elementArray[that.selectOptions.name_index] =
                  ('<i class="fa fa-star fa-fw"></i> ' +
                    elementArray[that.selectOptions.name_index]);
            } else {
              elementArray[that.selectOptions.name_index] =
                  ('<i class="fa fa-fw"></i> ' +
                    elementArray[that.selectOptions.name_index]);
            }
          }

          list_array.push(elementArray);

          delete fixed_ids_map[this[that.xmlRoot].ID];
        }
      });

      var n_columns = that.columns.length + 1;

      $.each(fixed_ids_map, function(id, v) {
        var empty = [];

        for (var i = 0; i <= n_columns; i++) {
          empty.push("");
        }

        empty[that.selectOptions.id_index] = id;

        list_array.push(empty);
      });

      that.updateView(null, list_array, true);
    }

    var error_func = function(request, error_json, container) {
      success_func(request, []);
      Notifier.onError(request, error_json, container);
    }

    if (that.selectOptions.zone_id == undefined) {
      OpenNebula[that.resource].list({
        timeout: true,
        success: success_func,
        error: error_func
      });
    } else {
      OpenNebula[that.resource].list_in_zone({
        data: {zone_id: that.selectOptions.zone_id},
        timeout: true,
        success: success_func,
        error: error_func
      });
    }
  }
  // Used by panels that contain tables from other resources.
  // TODO: This is probably duplicated somewhere
  function _list() {
    var that = this;
    OpenNebula[that.resource].list({
      success: function(req, resp) {
        that.updateView(req, resp);
      },
      error: Notifier.onError
    });
  }
})
;
define('utils/template-utils',['require','utils/locale','sunstone'],function(require) {

  var Locale = require('utils/locale');
  var Sunstone = require('sunstone');

  //Escape doublequote in a string and return it
  function _escapeDoubleQuotes(string) {
    if (string != undefined) {
      return string.replace(/\\/g, '\\').replace(/"/g, '\\"');
    } else {
      return string;
    }
  }

  function _htmlDecode(value) {
    return $('<div/>').html(value).text();
  }

  // Convert from hash to string
  function _convert_template_to_string(template_json, unshown_values) {
    if (unshown_values)
      template_json = $.extend({}, template_json, unshown_values);

    var template_str = "\n";
    $.each(template_json, function(key, value) {
      // value can be an array
      if (!value) {
        template_str = template_str + key + "=\n";
      } else {
        if (value.constructor == Array) {
          var it = null;
          $.each(value, function(index, element) {
            if (!element) return true;
            // current value can be an object
            if (typeof element == 'object') {
              template_str += key + "=[";
              for (var current_key in element) {
                template_str += current_key + "=\"" + element[current_key].toString().replace(/"/g, "\\\"") + "\",";
              }
              template_str = template_str.substring(0, template_str.length - 1);
              template_str += "]\n";
            } else // or a string
              {
                template_str = template_str + key + "=\"" + element.toString().replace(/"/g, "\\\"") + "\"\n";
              }
          });
        } else // or a single value
        {
          // which in turn can be an object
          if (typeof value == 'object') {
            template_str += key + "=[";
            for (var current_key in value) {
              template_str += current_key + "=\"" + value[current_key].toString().replace(/"/g, "\\\"") + "\",";
            }
            template_str = template_str.substring(0, template_str.length - 1);
            template_str += "]\n";
          } else // or a string
          {
            template_str = template_str + key + "=\"" + value.toString().replace(/"/g, "\\\"") + "\"\n";
          }
        }
      }
    });

    return _htmlDecode(template_str);
  }

  return {
    'templateToString': _convert_template_to_string,
    'htmlDecode': _htmlDecode,
    'escapeDoubleQuotes': _escapeDoubleQuotes
  };
});


define('tabs/users-tab/tabId',['require'],function(require){
  return 'users-tab';
});
define('tabs/users-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','utils/quotas/quota-defaults','utils/quotas/quota-widgets','utils/template-utils','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');
  var QuotaDefaults = require('utils/quotas/quota-defaults');
  var QuotaWidgets = require('utils/quotas/quota-widgets');
  var TemplateUtils = require('utils/template-utils');

  /*
    CONSTANTS
   */

  var RESOURCE = "User";
  var XML_ROOT = "USER";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          { "bSortable": false, "aTargets": ["check",5,6,7] },
          {"sWidth": "35px", "aTargets": [0]},
          { "sWidth": "150px", "aTargets": [5,6,7] },
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    };

    this.columns = [
      Locale.tr("ID"),
      Locale.tr("Name"),
      Locale.tr("Group"),
      Locale.tr("Auth driver"),
      Locale.tr("VMs"),
      Locale.tr("Memory"),
      Locale.tr("CPU"),
      Locale.tr("Group ID"),
      Locale.tr("Hidden User Data")
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 2,
      "select_resource": Locale.tr("Please select a User from the list"),
      "you_selected": Locale.tr("You selected the following User:"),
      "select_resource_multiple": Locale.tr("Please select one or more users from the list"),
      "you_selected_multiple": Locale.tr("You selected the following users:")
    };

    this.totalUsers = 0;

    TabDataTable.call(this);
  }

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;
  Table.prototype.preUpdateView = _preUpdateView;
  Table.prototype.postUpdateView = _postUpdateView;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    this.totalUsers++;

    var element = element_json[XML_ROOT];

    var vms    = '<span class="progress-text right" style="font-size: 12px">-</span>';
    var memory = '<span class="progress-text right" style="font-size: 12px">-</span>';
    var cpu    = '<span class="progress-text right" style="font-size: 12px">-</span>';

    var default_user_quotas = QuotaDefaults.getDefaultUserQuotas();

    QuotaWidgets.initEmptyQuotas(element);

    if (!$.isEmptyObject(element.VM_QUOTA)){
      vms = QuotaWidgets.quotaBar(
        element.VM_QUOTA.VM.VMS_USED,
        element.VM_QUOTA.VM.VMS,
        default_user_quotas.VM_QUOTA.VM.VMS);

      memory = QuotaWidgets.quotaBarMB(
        element.VM_QUOTA.VM.MEMORY_USED,
        element.VM_QUOTA.VM.MEMORY,
        default_user_quotas.VM_QUOTA.VM.MEMORY);

      cpu = QuotaWidgets.quotaBarFloat(
        element.VM_QUOTA.VM.CPU_USED,
        element.VM_QUOTA.VM.CPU,
        default_user_quotas.VM_QUOTA.VM.CPU);
    }

    // Build hidden user template
    var hidden_template = TemplateUtils.templateToString(element);

    return [
      '<input class="check_item" type="checkbox" id="'+RESOURCE.toLowerCase()+'_' +
                           element.ID + '" name="selected_items" value="' +
                           element.ID + '"/>',
      element.ID,
      element.NAME,
      element.GNAME,
      element.AUTH_DRIVER,
      vms,
      memory,
      cpu,
      element.GID,
      hidden_template
    ];
  }

  function _preUpdateView() {
    this.totalUsers = 0;
  }

  function _postUpdateView() {
    $(".total_users").text(this.totalUsers);
  }
});

define('utils/common-actions',['require','sunstone','utils/notifier','utils/locale'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');

  /*
    CONSTRUCTOR
   */

  function CommonActions(openNebulaResource, resourceStr, tabId, xmlRoot) {
    this.openNebulaResource = openNebulaResource;
    this.tabId = tabId;
    this.resourceStr = resourceStr;
    this.xmlRoot = xmlRoot;
  }

  CommonActions.prototype.list = _list;
  CommonActions.prototype.show = _show;
  CommonActions.prototype.refresh = _refresh;
  CommonActions.prototype.del = _del;
  CommonActions.prototype.multipleAction = _multipleAction;
  CommonActions.prototype.singleAction = _singleAction;
  CommonActions.prototype.create = _create;
  CommonActions.prototype.showCreate = _showCreate;
  CommonActions.prototype.showUpdate = _showUpdate;
  CommonActions.prototype.checkAndShowUpdate = _checkAndShowUpdate;
  CommonActions.prototype.update = _update;
  CommonActions.prototype.updateTemplate = _updateTemplate;

  return CommonActions;

  function _list() {
    var that = this;
    return {
      type: "list",
      call: that.openNebulaResource.list,
      callback: function(request, response) {
        Sunstone.getDataTable(that.tabId).updateView(request, response);
      },
      error: Notifier.onError
    }
  }

  function _show() {
    var that = this;
    return {
      type: "single",
      call: that.openNebulaResource.show,
      callback: function(request, response) {
        Sunstone.getDataTable(that.tabId).updateElement(request, response);
        if (Sunstone.rightInfoVisible($('#' + that.tabId))) {
          Sunstone.insertPanels(that.tabId, response);
        }
      },
      error: Notifier.onError
    }
  }

  function _refresh() {
    var that = this;
    return {
      type: "custom",
      call: function() {
        var tab = $('#' + that.tabId);
        if (Sunstone.rightInfoVisible(tab)) {
          Sunstone.runAction(that.resourceStr + ".show", Sunstone.rightInfoResourceId(tab));
        } else {
          Sunstone.getDataTable(that.tabId).waitingNodes();
          Sunstone.runAction(that.resourceStr + ".list", {force: true});
        }
      },
      error: Notifier.onError
    }
  }

  function _del() {
    var that = this;
    return {
      type: "multiple",
      call : that.openNebulaResource.del,
      callback : function(request, response) {
        Sunstone.getDataTable(that.tabId).deleteElement(request, response);
      },
      elements: function() {
        return Sunstone.getDataTable(that.tabId).elements();
      },
      error: Notifier.onError,
      notify: true
    }
  }

  function _multipleAction(actionStr) {
    var that = this;
    return {
      type: "multiple",
      call: that.openNebulaResource[actionStr],
      callback: function (req) {
        Sunstone.runAction(that.resourceStr + ".show", req.request.data[0]);
      },
      elements: function() {
        return Sunstone.getDataTable(that.tabId).elements();
      },
      error: Notifier.onError,
      notify: true
    }
  }

  function _singleAction(actionStr) {
    var that = this;
    return {
      type: "single",
      call: that.openNebulaResource[actionStr],
      callback: function (req) {
        Sunstone.runAction(that.resourceStr + ".show", req.request.data[0][0]);
      },
      elements: function() {
        return Sunstone.getDataTable(that.tabId).elements();
      },
      error: Notifier.onError,
      notify: true
    }
  }

  function _create(formPanelId) {
    var that = this;
    return {
      type: "create",
      call: that.openNebulaResource.create,
      callback : function(request, response) {
        Sunstone.resetFormPanel(that.tabId, formPanelId);
        Sunstone.hideFormPanel(that.tabId);
        Sunstone.getDataTable(that.tabId).addElement(request, response);
      },
      error: function(request, response) {
        Sunstone.hideFormPanelLoading(that.tabId);
        Notifier.onError(request, response);
      },
      notify: true
    }
  }

  function _showCreate(formPanelId) {
    var that = this;
    return {
      type: "custom",
      call: function() {
        Sunstone.showFormPanel(that.tabId, formPanelId, "create");
      }
    }
  }

  function _showUpdate(formPanelId) {
    var that = this;
    return {
      type: "single",
      call: that.openNebulaResource.show,
      callback: function(request, response) {
        Sunstone.showFormPanel(that.tabId, formPanelId, "update",
          function(formPanelInstance, context) {
            formPanelInstance.fill(context, response[that.xmlRoot]);
          });
      },
      error: Notifier.onError
    }
  }

  function _checkAndShowUpdate() {
    var that = this;
    return {
      type: "single",
      call: function() {
        var selected_nodes = Sunstone.getDataTable(that.tabId).elements();
        if (selected_nodes.length != 1) {
          Notifier.notifyMessage("Please select one (and just one) resource to update.");
          return false;
        }

        var resource_id = "" + selected_nodes[0];
        Sunstone.runAction(that.resourceStr + ".show_to_update", resource_id);
      }
    }
  }

  function _update() {
    var that = this;
    return {
      type: "single",
      call: that.openNebulaResource.update,
      callback: function(request, response){
        Sunstone.hideFormPanel(that.tabId);
      },
      error: function(request, response){
        Sunstone.hideFormPanelLoading(that.tabId);
        Notifier.onError(request, response);
      }
    }
  }

  function _updateTemplate() {
    var that = this;
    return {
      type: "single",
      call: that.openNebulaResource.update,
      callback: function(request) {
        Sunstone.runAction(that.resourceStr + '.show', request.request.data[0][0]);
      },
      error: Notifier.onError
    }
  }
});

define('tabs/users-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createUserForm';
});
define('tabs/users-tab/dialogs/password/dialogId',['require'],function(require){
  return 'userPasswordDialog';
});
define('tabs/users-tab/dialogs/auth-driver/dialogId',['require'],function(require){
  return 'userAuthDriverDialog';
});
define('tabs/users-tab/dialogs/quotas/dialogId',['require'],function(require){
  return 'userQuotasDialog';
});
define('tabs/users-tab/actions',['require','sunstone','utils/notifier','utils/locale','./datatable','opennebula/user','utils/common-actions','./tabId','./form-panels/create/formPanelId','./dialogs/password/dialogId','./dialogs/auth-driver/dialogId','./dialogs/quotas/dialogId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var DataTable = require('./datatable');
  var OpenNebulaResource = require('opennebula/user');
  var CommonActions = require('utils/common-actions');

  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');
  var PASSWORD_DIALOG_ID = require('./dialogs/password/dialogId');
  var AUTH_DRIVER_DIALOG_ID = require('./dialogs/auth-driver/dialogId');
  var QUOTAS_DIALOG_ID = require('./dialogs/quotas/dialogId');

  var RESOURCE = "User";
  var XML_ROOT = "USER";

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "User.create" : _commonActions.create(CREATE_DIALOG_ID),
    "User.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID),
    "User.list" : _commonActions.list(),
    "User.show" : _commonActions.show(),
    "User.refresh" : _commonActions.refresh(),
    "User.delete" : _commonActions.del(),
    "User.chgrp": _commonActions.multipleAction('chgrp'),
    "User.addgroup": _commonActions.multipleAction('addgroup'),
    "User.delgroup": _commonActions.multipleAction('delgroup'),

    "User.update_password" : {
      type: "custom",
      call: function(){
        Sunstone.getDialog(PASSWORD_DIALOG_ID).setParams(
          {selectedElements: Sunstone.getDataTable(TAB_ID).elements()});
        Sunstone.getDialog(PASSWORD_DIALOG_ID).reset();
        Sunstone.getDialog(PASSWORD_DIALOG_ID).show();
      }
    },

    "User.passwd" : {
      type: "multiple",
      call: OpenNebulaResource.passwd,
      error: Notifier.onError
    },


    "User.change_authentication" : {
      type: "custom",
      call: function(){
        Sunstone.getDialog(AUTH_DRIVER_DIALOG_ID).show();
      }
    },

    "User.chauth" : {
      type: "multiple",
      call: OpenNebulaResource.chauth,
      error: Notifier.onError,
    },

    "User.update_template" : {
      type: "single",
      call: OpenNebulaResource.update,
      callback: function(request) {
        var reqId = request.request.data[0][0];

        Sunstone.runAction(RESOURCE+'.show',reqId);

        if (reqId == config['user_id'] || reqId == "-1") {
          Sunstone.runAction('Settings.refresh');

          $.ajax({
            url: 'config',
            type: "POST",
            dataType: "json",
            success: function() {
              return false;
            },
            error: function(response) {
            }
          });
        }
      },
      error: Notifier.onError
    },

    "User.append_template" : {
      type: "single",
      call: OpenNebulaResource.append,
      callback: function(request) {
        var reqId = request.request.data[0][0];

        Sunstone.runAction(RESOURCE+'.show',reqId);

        if (reqId == config['user_id'] || reqId == "-1") {
          Sunstone.runAction('Settings.refresh');

          $.ajax({
            url: 'config',
            type: "POST",
            dataType: "json",
            success: function() {
              return false;
            },
            error: function(response) {
            }
          });
        }
      },
      error: Notifier.onError
    },

    "User.append_template_refresh" : {
      type: "single",
      call: OpenNebulaResource.append,
      callback: function(request) {
        var reqId = request.request.data[0][0];

        if (reqId == config['user_id'] || reqId == "-1") {
          $.ajax({
            url: 'config',
            type: "POST",
            dataType: "json",
            success: function() {
              window.location.href = ".";
            },
            error: function(response) {
            }
          });
        } else {
          Sunstone.runAction(RESOURCE+'.show',reqId);
        }
      },
      error: Notifier.onError
    },

    "User.fetch_quotas" : {
      type: "single",
      call: OpenNebulaResource.show,
      callback: function (request,response) {
        Sunstone.getDialog(QUOTAS_DIALOG_ID).setParams({element: response[XML_ROOT]});
        Sunstone.getDialog(QUOTAS_DIALOG_ID).reset();
        Sunstone.getDialog(QUOTAS_DIALOG_ID).show();
      },
      error: Notifier.onError
    },

    "User.quotas_dialog" : {
      type: "custom",
      call: function() {
        var tab = $('#' + TAB_ID);
        if (Sunstone.rightInfoVisible(tab)) {
          $('a[href="#user_quotas_tab"]', tab).click();
          $('#edit_quotas_button', tab).click();
        } else {
          var sel_elems = Sunstone.getDataTable(TAB_ID).elements();
          //If only one user is selected we fecth the user's quotas
          if (sel_elems.length == 1){
            Sunstone.runAction(RESOURCE+'.fetch_quotas',sel_elems[0]);
          } else {
            // More than one, shows '0' usage
            Sunstone.getDialog(QUOTAS_DIALOG_ID).setParams({element: {}});
            Sunstone.getDialog(QUOTAS_DIALOG_ID).reset();
            Sunstone.getDialog(QUOTAS_DIALOG_ID).show();
          }
        }
      }
    },

    "User.set_quota" : {
      type: "multiple",
      call: OpenNebulaResource.set_quota,
      elements: function() {
        return Sunstone.getDataTable(TAB_ID).elements();
      },
      callback: function(request) {
        Sunstone.getDialog(QUOTAS_DIALOG_ID).hide();

        Sunstone.runAction(RESOURCE+'.show',request.request.data[0]);
      },
      error: Notifier.onError
    }
  };

  return _actions;
});

define('utils/dialogs/dialog',['require','foundation.reveal'],function(require) {
  require('foundation.reveal');

  function BaseDialog() {
    return this;
  }

  BaseDialog.prototype = {
    'insert': _insert,
    'show': _show,
    'hide': _hide,
    'reset': _reset,
  };

  return BaseDialog;

  function _insert(dialog) {
    var that = this;
    var dialogElement = $(that.html()).appendTo('div#dialogs');
    that.setup(dialogElement);
    dialogElement.foundation('reveal', 'reflow');

    dialogElement.on('opened.fndtn.reveal', function (e) {
      if (e.namespace !== 'fndtn.reveal') { return; }
      that.onShow(dialogElement);
    });

    dialogElement.on('close.fndtn.reveal', function (e) {
      if (e.namespace !== 'fndtn.reveal') { return; }
      if (that.onClose) {
        that.onClose(dialogElement);
      }
    });

    dialogElement.on('click', '.resetDialog', function() {
      that.reset();
      that.show();
    });

    that.dialogElement = dialogElement;

    return that.dialogElement;
  }

  function _show() {
    this.dialogElement.foundation('reveal', 'open');
    return false;
  }

  function _hide() {
    this.dialogElement.foundation('reveal', 'close');
  }

  function _reset() {
    this.dialogElement.remove();
    this.dialogElement = this.insert();
    return false;
  }
})
;

/* START_TEMPLATE */
define('hbs!tabs/users-tab/dialogs/password/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <h3 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update Password", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n  </div>\n  <form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\">\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        ";
  stack1 = ((helper = (helper = helpers.userCreationHTML || (depth0 != null ? depth0.userCreationHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"userCreationHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      </div>\n    </div>\n    <div class=\"form_buttons row\">\n      <button type=\"submit\" class=\"button radius right\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Change", {"name":"tr","hash":{},"data":data})))
    + "\n      </button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/users-tab/dialogs/password/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/users-tab/utils/user-creation/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function";
  return "<div class=\"row name_row\">\n  <div class=\"large-12 columns\">\n    <label for=\"username\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Username", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input required type=\"text\" name=\"username\" id=\""
    + escapeExpression(((helper = (helper = helpers.idPrefix || (depth0 != null ? depth0.idPrefix : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"idPrefix","hash":{},"data":data}) : helper)))
    + "_username\" />\n  </div>\n</div>\n<div class=\"row password_row\">\n  <div class=\"large-12 columns\">\n    <label for=\"pass\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Password", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input required type=\"password\" name=\"pass\" id=\""
    + escapeExpression(((helper = (helper = helpers.idPrefix || (depth0 != null ? depth0.idPrefix : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"idPrefix","hash":{},"data":data}) : helper)))
    + "_pass\" />\n  </div>\n  <div class=\"large-12 columns\">\n    <label for=\"confirm_password\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Confirm Password", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input data-equalto=\""
    + escapeExpression(((helper = (helper = helpers.idPrefix || (depth0 != null ? depth0.idPrefix : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"idPrefix","hash":{},"data":data}) : helper)))
    + "_pass\" type=\"password\" name=\"confirm_password\" id=\""
    + escapeExpression(((helper = (helper = helpers.idPrefix || (depth0 != null ? depth0.idPrefix : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"idPrefix","hash":{},"data":data}) : helper)))
    + "_confirm_password\" />\n    <small class=\"error\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Passwords do not match", {"name":"tr","hash":{},"data":data})))
    + "</small>\n  </div>\n</div>\n<div class=\"row auth_driver_row\">\n  <div class=\"large-12 columns\">\n    <label for=\"driver\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Authentication", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <select name=\"driver\" id=\""
    + escapeExpression(((helper = (helper = helpers.idPrefix || (depth0 != null ? depth0.idPrefix : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"idPrefix","hash":{},"data":data}) : helper)))
    + "_driver\">\n      <option value=\"core\" selected=\"selected\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Core", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"ssh\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "SSH", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"x509\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "x509", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"ldap\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "LDAP", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"public\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Public", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"custom\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom", {"name":"tr","hash":{},"data":data})))
    + "</option>\n    </select>\n    <div>\n      <input type=\"text\" id=\""
    + escapeExpression(((helper = (helper = helpers.idPrefix || (depth0 != null ? depth0.idPrefix : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"idPrefix","hash":{},"data":data}) : helper)))
    + "_custom_auth\" name=\"custom_auth\" />\n    </div>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/users-tab/utils/user-creation/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/users-tab/utils/user-creation',['require','hbs!./user-creation/html'],function(require) {

  var TemplateHTML = require('hbs!./user-creation/html');

  /**
   * @param {string} idPrefix
   * @param  {object} [options] Options to hide/show each field. Each field is
   *                            enabled by default.
   *                            - name: true, false
   *                            - password: true, false
   *                            - auth_driver: true, false
   */
  function UserCreation(idPrefix, options) {
    this.idPrefix = idPrefix;

    this.options = options;

    if (this.options == undefined){
      this.options = {};
    }

    if (this.options.name == undefined){
      this.options.name = true;
    }

    if (this.options.password == undefined){
      this.options.password = true;
    }

    if (this.options.auth_driver == undefined){
      this.options.auth_driver = true;
    }
  }

  UserCreation.prototype.constructor = UserCreation;
  UserCreation.prototype.html = _html;
  UserCreation.prototype.setup = _setup;
  UserCreation.prototype.retrieve = _retrieve;
  UserCreation.prototype.enable = _enable;
  UserCreation.prototype.disable = _disable;
  UserCreation.prototype.setName = _setName;

  return UserCreation;

  function _html(){
    return TemplateHTML({
      'idPrefix': this.idPrefix
    });
  }

  /**
   * Setups the html
   * @param  {object} context jquery selector
   */
  function _setup(context){
    var that = this;

    if (this.options.name == false){
      $('#'+that.idPrefix+'_username',context).removeAttr('required');
      $('.name_row', context).hide();
    }

    if (this.options.password == false){
      $('#'+that.idPrefix+'_pass',context).removeAttr('required');
      $('.password_row', context).hide();
    }

    if (this.options.auth_driver == false){
      $('.auth_driver_row', context).hide();
    }

    $('#'+that.idPrefix+'_driver', context).change(function(){
      if ($(this).val() == "ldap"){
        $('#'+that.idPrefix+'_pass',context).removeAttr('required');
        $('.password_row', context).hide();
      } else if (that.options.password) {
        $('#'+that.idPrefix+'_pass',context).attr('required', '');
        $('.password_row', context).show();
      }
    });

    $('input[name="custom_auth"]',context).parent().hide();
    $('select#'+that.idPrefix+'_driver',context).change(function(){
      if ($(this).val() == "custom"){
        $('input[name="custom_auth"]',context).parent().show();
        $('input[name="custom_auth"]',context).attr('required', '');
      } else {
        $('input[name="custom_auth"]',context).parent().hide();
        $('input[name="custom_auth"]',context).removeAttr('required');
      }
    });
  }

  /**
   * @param  {object} context jquery selector
   * @return {object}         Returns an object with the attributes:
   *                                  - name
   *                                  - password
   *                                  - auth_driver
   */
  function _retrieve(context){
    var that = this;

    var user_name = $('#'+that.idPrefix+'_username',context).val();
    var user_password = $('#'+that.idPrefix+'_pass',context).val();
    var driver = $('#'+that.idPrefix+'_driver', context).val();

    if (driver == 'custom'){
      driver = $('input[name="custom_auth"]', context).val();
    } else if (driver == "ldap") {
      user_password = "-";
    }

    return {
      "name" : user_name,
      "password" : user_password,
      "auth_driver" : driver
    };
  }

  /**
   * Disables all inputs, and removes the abide required tags
   * @param  {object} context jquery selector
   */
  function _disable(context){
    var that = this;

    $('#'+that.idPrefix+'_username',context).attr('disabled','disabled').removeAttr('required');
    $('#'+that.idPrefix+'_pass',context).attr('disabled','disabled').removeAttr('required');
    $('#'+that.idPrefix+'_confirm_password',context).attr('disabled','disabled').removeAttr('required');
    $('#'+that.idPrefix+'_driver',context).attr('disabled','disabled').removeAttr('required');
    $('#'+that.idPrefix+'_custom_auth',context).attr('disabled','disabled').removeAttr('required');
  }

  /**
   * Enables all inputs, and adds the abide required tags
   * @param  {object} context jquery selector
   */
  function _enable(context){
    var that = this;

    $('#'+that.idPrefix+'_username',context).removeAttr("disabled").attr('required', '');
    $('#'+that.idPrefix+'_pass',context).removeAttr("disabled").attr('required', '');
    $('#'+that.idPrefix+'_confirm_password',context).removeAttr("disabled").attr('required', '');
    $('#'+that.idPrefix+'_driver',context).removeAttr("disabled").attr('required', '');
    $('#'+that.idPrefix+'_custom_auth',context).removeAttr("disabled");

    $('select#'+that.idPrefix+'_driver',context).change();
  }

  function _setName(context, name){
    $('#'+this.idPrefix+'_username',context).val(name);
  }
});

define('tabs/users-tab/dialogs/password',['require','utils/dialogs/dialog','hbs!./password/html','sunstone','utils/notifier','utils/locale','tabs/users-tab/utils/user-creation','./password/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./password/html');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var UserCreation = require('tabs/users-tab/utils/user-creation');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./password/dialogId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    this.userCreation = new UserCreation(DIALOG_ID, {name: false, auth_driver: false});

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setParams = _setParams;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  /**
   * @param {object} params
   *        - params.selectedElements : Array of user ids
   */
  function _setParams(params) {
    this.selectedElements = params.selectedElements;
  }

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId,
      'userCreationHTML': this.userCreation.html()
    });
  }

  function _setup(context) {
    var that = this;

    this.userCreation.setup(context);

    context.foundation('abide', 'reflow');
    context.off('invalid.fndtn.abide', '#' + DIALOG_ID + 'Form');
    context.off('valid.fndtn.abide', '#' + DIALOG_ID + 'Form');

    context.on('invalid.fndtn.abide', '#' + DIALOG_ID + 'Form', function(e) {
      Notifier.notifyError(Locale.tr("One or more required fields are missing or malformed."));
    }).on('valid.fndtn.abide', '#' + DIALOG_ID + 'Form', function(e) {
      var inputs = that.userCreation.retrieve(context);

      Sunstone.runAction('User.passwd', that.selectedElements, inputs.password);

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      Sunstone.runAction('User.refresh');

      return false;
    });

    return false;
  }

  function _onShow(context) {

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/users-tab/dialogs/auth-driver/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <h3 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Change authentication", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n  </div>\n  <form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\">\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        ";
  stack1 = ((helper = (helper = helpers.userCreationHTML || (depth0 != null ? depth0.userCreationHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"userCreationHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      </div>\n    </div>\n    <div class=\"form_buttons row\">\n      <button type=\"submit\" class=\"button radius right\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Change", {"name":"tr","hash":{},"data":data})))
    + "\n      </button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/users-tab/dialogs/auth-driver/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/users-tab/dialogs/auth-driver',['require','utils/dialogs/dialog','hbs!./auth-driver/html','sunstone','utils/notifier','utils/locale','tabs/users-tab/utils/user-creation','./auth-driver/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./auth-driver/html');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var UserCreation = require('tabs/users-tab/utils/user-creation');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./auth-driver/dialogId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    this.userCreation = new UserCreation(DIALOG_ID, {name: false, password: false});

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId,
      'userCreationHTML': this.userCreation.html()
    });
  }

  function _setup(context) {
    var that = this;

    this.userCreation.setup(context);

    context.foundation('abide', 'reflow');
    context.off('invalid.fndtn.abide', '#' + DIALOG_ID + 'Form');
    context.off('valid.fndtn.abide', '#' + DIALOG_ID + 'Form');

    context.on('invalid.fndtn.abide', '#' + DIALOG_ID + 'Form', function(e) {
      Notifier.notifyError(Locale.tr("One or more required fields are missing or malformed."));
    }).on('valid.fndtn.abide', '#' + DIALOG_ID + 'Form', function(e) {
      var inputs = that.userCreation.retrieve(context);

      var selElems = Sunstone.getDataTable(TAB_ID).elements();

      Sunstone.runAction('User.chauth', selElems, inputs.auth_driver);

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      Sunstone.runAction('User.refresh');

      return false;
    });

    return false;
  }

  function _onShow(context) {

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/users-tab/dialogs/quotas/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large max-height\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <h3 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update Quota", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n  </div>\n  <form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\">\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        ";
  stack1 = ((helper = (helper = helpers.quotasHTML || (depth0 != null ? depth0.quotasHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"quotasHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      </div>\n    </div>\n    <div class=\"form_buttons row\">\n      <button type=\"submit\" class=\"button radius right\" value=\"User.set_quota\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Apply changes", {"name":"tr","hash":{},"data":data})))
    + "\n      </button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/users-tab/dialogs/quotas/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/users-tab/dialogs/quotas',['require','utils/dialogs/dialog','hbs!./quotas/html','utils/locale','utils/quotas/quota-defaults','utils/quotas/quota-widgets','./quotas/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./quotas/html');
  var Locale = require('utils/locale');
  var QuotaDefaults = require('utils/quotas/quota-defaults');
  var QuotaWidgets = require('utils/quotas/quota-widgets');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./quotas/dialogId');
  var TAB_ID = require('../tabId');
  var RESOURCE = "User";
  var XML_ROOT = "USER";

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    this.element = undefined;

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setParams = _setParams;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId,
      'quotasHTML': QuotaWidgets.dialogHTML()
    });
  }

  function _setup(context) {
    var that = this;

    QuotaWidgets.setupQuotasDialog(context);

    return false;
  }

  /**
   * @param {object} params
   *        - params.element : user object, or empty object {}
   */
  function _setParams(params) {
    this.element = params.element;
  }

  function _onShow(context) {
    QuotaWidgets.populateQuotasDialog(
      this.element,
      QuotaDefaults.getDefaultQuotas(RESOURCE),
      context);

    return false;
  }
});

define('templates/helpers/valOrDefault',['require','hbs/handlebars','utils/locale'],function(require) {
  var Handlebars = require('hbs/handlebars');
  var Locale = require('utils/locale');

  var valOrDefault = function(value, defaultValue, options) {
    var out = value || defaultValue;
    return new Handlebars.SafeString(out);
  };

  Handlebars.registerHelper('valOrDefault', valOrDefault);
  return valOrDefault;
})
;

/* START_TEMPLATE */
define('hbs!tabs/users-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/valOrDefault'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, lambda=this.lambda, escapeExpression=this.escapeExpression;
  return "        "
    + escapeExpression(lambda(((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.SSH_PUBLIC_KEY : stack1), depth0))
    + "\n";
},"3":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "You can provide a SSH Key for this User clicking on the edit button", {"name":"tr","hash":{},"data":data})))
    + "\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table id=\"info_user_table\" class=\"dataTable extended_table\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.NAME : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n        ";
  stack1 = ((helper = (helper = helpers.groupTrHTML || (depth0 != null ? depth0.groupTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"groupTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Secondary groups", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.secondaryGroups || (depth0 != null ? depth0.secondaryGroups : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"secondaryGroups","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Authentication driver", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.AUTH_DRIVER : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Password", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">\n            <button id=\"update_password\" type=\"button\" class=\"button tiny radius secondary\" >\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update password", {"name":"tr","hash":{},"data":data})))
    + " <a><i class=\"fa fa-pencil-square-o\"/></a>\n            </button>\n          </td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Table Order", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td_table_order\">"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.TABLE_ORDER : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td>\n            <div id=\"div_edit_table_order\">\n              <a id=\"div_edit_table_order_link\" class=\"edit_e\" href=\"#\"> <i class=\"fa fa-pencil-square-o right\"/></a>\n            </div>\n          </td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Language", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td_language\">"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.LANG : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td>\n            <div id=\"div_edit_language\">\n              <a id=\"div_edit_language_link\" class=\"edit_e\" href=\"#\"> <i class=\"fa fa-pencil-square-o right\"/></a>\n            </div>\n          </td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "View", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td_view\">"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.DEFAULT_VIEW : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td>\n            <div id=\"div_edit_view\">\n              <a id=\"div_edit_view_link\" class=\"edit_e\" href=\"#\"> <i class=\"fa fa-pencil-square-o right\"/></a>\n            </div>\n          </td>\n        </tr>\n      </tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">\n      <thead>\n        <tr>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Public SSH Key", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>\n            <a class=\"user_ssh_public_key_edit right\" href=\"#\"><i class=\"fa fa-pencil-square-o\"></i></a>\n          </th>\n        </tr>\n      </thead>\n    </table>\n    <textarea rows=\"6\" type=\"text\" id=\"user_ssh_public_key_textarea\" name=\"ssh_public_key\" class=\"hidden\">"
    + escapeExpression(lambda(((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.SSH_PUBLIC_KEY : stack1), depth0))
    + "</textarea>\n    <p id=\"user_ssh_public_key_text\" name=\"ssh_public_key\">\n";
  stack1 = helpers['if'].call(depth0, ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.SSH_PUBLIC_KEY : stack1), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.program(3, data),"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "    </p>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-9 columns\">\n    ";
  stack1 = ((helper = (helper = helpers.templateTableHTML || (depth0 != null ? depth0.templateTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templateTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/users-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/users-tab/panels/info/chgrp-tr',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda;
  return "<tr>\n  <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group", {"name":"tr","hash":{},"data":data})))
    + "</td>\n  <td id=\"value_td_group\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.GNAME : stack1), depth0))
    + "</td>\n  <td>\n    <div id=\"div_edit_chg_group\">\n      <a id=\"div_edit_chg_group_link\" class=\"edit_e\" href=\"#\"> <i class=\"fa fa-pencil-square-o right\"/></a>\n    </div>\n  </td>\n</tr>";
},"useData":true});
Handlebars.registerPartial('tabs/users-tab/panels/info/chgrp-tr', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/panel/template-table',['require','utils/locale','sunstone','utils/template-utils'],function(require) {
  /*
    This module insert a table with the template of the resource.
    New KEY=VALUE entries can be added and existing ones can be edited
   */

  var Locale = require('utils/locale');
  var Sunstone = require('sunstone');
  var TemplateUtils = require('utils/template-utils');

  /*
    Generate the table HTML with the template of the resource and an edit icon
    @param {Object} templateJSON Resource template (i.e: ZONE.TEMPLATE, IMAGE.TEMPLATE...)
    @param {String} resourceType Resource type (i.e: Zone, Host, Image...)
    @param {String} tableName Header of the table (i.e: Locale.tr("Attributes"))
    @returns {String} HTML table
   */
  function _html(templateJSON, resourceType, tableName) {
    var str = '<table id="' + resourceType.toLowerCase() + '_template_table" class="dataTable configuration_attrs"  cellpadding="0" cellspacing="0" border="0">\
                   <thead>\
                     <tr>\
                       <th colspan="3">'                     +
                      tableName +
                     '</th>\
                     </tr>\
                    </thead>' +
                  fromJSONtoHTMLTable(templateJSON, resourceType) +
                  '<tr>\
                      <td class="key_td"><input type="text" name="new_key" id="new_key" /></td>\
                      <td class="value_td"><textarea rows="1" type="text" name="new_value" id="new_value"></textarea></td>\
                      <td class="text-right"><button type="button" id="button_add_value" class="button small secondary">' + Locale.tr("Add") + '</button>\</td>\
                    </tr>'                  +
                 '</table>'

    return str;
  }

  /*
    Initialize the table, clicking the edit icon will add an input to edit the value
    @param {Object} templateJSON Resource template (i.e: ZONE.TEMPLATE, IMAGE.TEMPLATE...)
    @param {String} resourceType Resource type (i.e: Zone, Host, Image...)
    @param {String} resourceId ID of the resource
    @param {jQuery Object} context Selector including the tr
    @param {Object} unshownValues Values from the origianl resource template that
      have been deleted from the templateJSON param. Whithout this, a template
      update would permanently delete the missing values from OpenNebula
   */
  var _setup = function(templateJSON, resourceType, resourceId, context, unshownValues) {
    // Remove previous listeners
    context.off("keypress", "#new_key");
    context.off("keypress", "#new_value");
    context.off("keypress", "#new_value_vectorial");
    context.off("click", "#div_minus");
    context.off("click", "#div_edit");
    context.off("change", ".input_edit_value");
    context.off("click", "#div_edit_vectorial");
    context.off("change", ".input_edit_value_vectorial");
    context.off("click", "#div_minus_vectorial");
    context.off("click", "#button_add_value");
    context.off("click", "#button_add_value_vectorial");
    context.off("click", "#div_add_vectorial");

    // Add listener for add key and add value for Extended Template
    context.on("click", '#button_add_value', function() {
      new_value = $('#new_value', $(this).parent().parent()).val();
      new_key   = $('#new_key', $(this).parent().parent()).val();

      if (new_key != "") {
        var templateJSON_bk = $.extend({}, templateJSON);
        if (templateJSON[$.trim(new_key)] && (templateJSON[$.trim(new_key)] instanceof Array)) {
          templateJSON[$.trim(new_key)].push($.trim(new_value));
        } else {
          templateJSON[$.trim(new_key)] = $.trim(new_value);
        }
        template_str  = TemplateUtils.templateToString(templateJSON, unshownValues);

        Sunstone.runAction(resourceType + ".update_template", resourceId, template_str);
        templateJSON = templateJSON_bk;
      }
    });

    // Capture the enter key
    context.on("keypress", '#new_value', function(e) {
      var ev = e || window.event;
      var key = ev.keyCode;

      if (key == 13 && !ev.altKey) {
        //Get the button the user wants to have clicked
        $('#button_add_value', $(this).parent().parent()).click();
        ev.preventDefault();
      }
    })

    // Listener for single values

    // Listener for key,value pair remove action
    context.on("click", "#div_minus", function() {
      // Remove div_minus_ from the id
      field               = this.firstElementChild.id.substring(10, this.firstElementChild.id.length);
      var list_of_classes = this.firstElementChild.className.split(" ");
      var ocurrence = null;

      if (list_of_classes.length != 1) {
        $.each(list_of_classes, function(index, value) {
          if (value.match(/^ocurrence_/))
          ocurrence = value.substring(10, value.length);;
        });
      }

      // Erase the value from the template
      if (ocurrence != null)
          templateJSON[field].splice(ocurrence, 1);
      else
          delete templateJSON[field];

      template_str = TemplateUtils.templateToString(templateJSON, unshownValues);

      // Let OpenNebula know
      Sunstone.runAction(resourceType + ".update_template", resourceId, template_str);
    });

    // Listener for key,value pair edit action
    context.on("click", "#div_edit", function() {
      var key_str = this.firstElementChild.id.substring(9, this.firstElementChild.id.length);

      var value_str = $("#value_td_input_" + key_str).text();
      input = $("#value_td_input_" + key_str).html('<textarea class="input_edit_value" id="input_edit_' + key_str + '" type="text"></textarea>');
      $('#input_edit_' + key_str).val(value_str);

      // Capture the enter key
      context.off("keypress", '#input_edit_' + key_str);
      context.on("keypress", '#input_edit_' + key_str, function(e) {
        var ev = e || window.event;
        var key = ev.keyCode;

        if (key == 13 && !ev.altKey) {
          $('#input_edit_' + key_str).blur();
        }
      })

    });

    context.on("change", ".input_edit_value", function() {
      var key_str          = $.trim(this.id.substring(11, this.id.length));
      var value_str        = $.trim(this.value);
      var templateJSON_bk = $.extend({}, templateJSON);

      delete templateJSON[key_str];
      templateJSON[key_str] = value_str;

      template_str = TemplateUtils.templateToString(templateJSON, unshownValues);

      // Let OpenNebula know
      Sunstone.runAction(resourceType + ".update_template", resourceId, template_str);

      templateJSON = templateJSON_bk;
    });

    // Listeners for vectorial attributes
    // Listener for key,value pair edit action for subelement of vectorial key
    context.on("click", "#div_edit_vectorial", function() {
      var key_str         = $.trim(this.firstElementChild.id.substring(9, this.firstElementChild.id.length));
      var list_of_classes = this.firstElementChild.className.split(" ");
      var ocurrence       = " ";
      var vectorial_key   = null;

      if (list_of_classes.length != 1) {
        $.each(list_of_classes, function(index, value) {
          if (value.match(/^ocurrence_/))
              ocurrence += value + " ";
        });
      }

      if (list_of_classes.length != 1) {
        $.each(list_of_classes, function(index, value) {
          if (value.match(/^vectorial_key_/))
              vectorial_key = value;
        });
      }

      if (ocurrence != " ") {
        var value_str = $.trim($(".value_td_input_" + key_str + "." + ocurrence.substring(1, ocurrence.length - 1) + "." + vectorial_key).text());
        $(".value_td_input_" + key_str + "." + ocurrence.substring(1, ocurrence.length - 1) + "." + vectorial_key).html('<input class="input_edit_value_vectorial' + ocurrence + vectorial_key + '" id="input_edit_' + key_str + '" type="text" value="' + value_str + '"/>');

      } else {
        var value_str = $.trim($(".value_td_input_" + key_str + "." + vectorial_key).text());
        $(".value_td_input_" + key_str + "." + vectorial_key).html('<input class="input_edit_value_vectorial' + ocurrence + vectorial_key + '" id="input_edit_' + key_str + '" type="text" value="' + value_str + '"/>');
      }

    });

    context.on("change", ".input_edit_value_vectorial", function() {
      var key_str          = $.trim(this.id.substring(11, this.id.length));
      var value_str        = $.trim(this.value);
      var templateJSON_bk = $.extend({}, templateJSON);

      var list_of_classes  = this.className.split(" ");
      var ocurrence        = null;
      var vectorial_key    = null;

      if (list_of_classes.length != 1) {
        $.each(list_of_classes, function(index, value) {
          if (value.match(/^ocurrence_/))
              ocurrence = value.substring(10, value.length);
        });
      }

      if (list_of_classes.length != 1) {
        $.each(list_of_classes, function(index, value) {
          if (value.match(/^vectorial_key_/))
              vectorial_key = value.substring(14, value.length);
        });
      }

      if (ocurrence != null)
          templateJSON[vectorial_key][ocurrence][key_str] = value_str;
      else
          templateJSON[vectorial_key][key_str] = value_str;

      template_str = TemplateUtils.templateToString(templateJSON, unshownValues);

      // Let OpenNebula know
      Sunstone.runAction(resourceType + ".update_template", resourceId, template_str);

      templateJSON = templateJSON_bk;
    });

    // Listener for key,value pair remove action
    context.on("click", "#div_minus_vectorial", function() {
      // Remove div_minus_ from the id
      var field           = this.firstElementChild.id.substring(10, this.firstElementChild.id.length);
      var list_of_classes = this.firstElementChild.className.split(" ");
      var ocurrence       = null;
      var vectorial_key   = null;

      if (list_of_classes.length != 1) {
        $.each(list_of_classes, function(index, value) {
          if (value.match(/^ocurrence_/))
              ocurrence = value.substring(10, value.length);
        });
      }

      if (list_of_classes.length != 1) {
        $.each(list_of_classes, function(index, value) {
          if (value.match(/^vectorial_key_/))
              vectorial_key = value.substring(14, value.length);
        });
      }

      // Erase the value from the template
      if (ocurrence != null)
          delete templateJSON[vectorial_key][ocurrence][field];
      else
          delete templateJSON[vectorial_key][field];

      template_str = TemplateUtils.templateToString(templateJSON, unshownValues);

      // Let OpenNebula know
      Sunstone.runAction(resourceType + ".update_template", resourceId, template_str);
    });

    // Listener for vectorial key,value pair add action
    context.on("click", "#div_add_vectorial", function() {
      if (!$('#button_add_value_vectorial').html()) {
        var field           = this.firstElementChild.id.substring(18, this.firstElementChild.id.length);
        var list_of_classes = this.firstElementChild.className.split(" ");
        var ocurrence       = null;
        var vectorial_key   = null;

        if (list_of_classes.length != 1) {
          $.each(list_of_classes, function(index, value) {
            if (value.match(/^ocurrence_/))
                ocurrence = value;
          });
        }

        if (list_of_classes.length != 1) {
          $.each(list_of_classes, function(index, value) {
            if (value.match(/^vectorial_key_/))
                vectorial_key = value;
          });
        }

        $(this).parent().parent().after('<tr>\
                                                <td class="key_td"><input type="text" style="text-align:center" name="new_key_vectorial" id="new_key_vectorial" /></td>\
                                                <td class="value_td"><input type="text" name="new_value" id="new_value_vectorial" /></td>\
                                                <td class=""><button class="' + vectorial_key + " " + ocurrence + '" id="button_add_value_vectorial">' + Locale.tr("Add") + '</button>\</td>\
                                               </tr>');
      }
    });

    // Add listener for add key and add value for Extended Template
    context.on("click", '#button_add_value_vectorial', function() {
      if ($('#new_value_vectorial').val() != "" && $('#new_key_vectorial').val() != "") {
        var list_of_classes  = this.className.split(" ");
        var ocurrence        = null;
        var vectorial_key    = null;
        var templateJSON_bk = $.extend({}, templateJSON);

        if (list_of_classes.length != 1) {
          $.each(list_of_classes, function(index, value) {
            if (value.match(/^vectorial_key_/))
                vectorial_key = value;
          });
        }

        if (list_of_classes.length != 1) {
          $.each(list_of_classes, function(index, value) {
            if (value.match(/^ocurrence_/))
                ocurrence = value;
          });
        }

        vectorial_key = vectorial_key.substring(14, vectorial_key.length);

        if (ocurrence != null) {
          ocurrence = ocurrence.substring(10, ocurrence.length);
          templateJSON[vectorial_key][ocurrence][$('#new_key_vectorial').val()] = $.trim($('#new_value_vectorial').val());
        } else {
          templateJSON[vectorial_key][$('#new_key_vectorial').val()] = $.trim($('#new_value_vectorial').val());
        }

        template_str  = TemplateUtils.templateToString(templateJSON, unshownValues);

        Sunstone.runAction(resourceType + ".update_template", resourceId, template_str);
        // This avoids to get a messed template if the update fails
        templateJSON = templateJSON_bk;
      }
    });

    // Capture the enter key
    context.on("keypress", '#new_value_vectorial', function(e) {
      var ev = e || window.event;
      var key = ev.keyCode;

      if (key == 13) {
        //Get the button the user wants to have clicked
        $('#button_add_value_vectorial').click();
        ev.preventDefault();
      }
    })
  }

  // Returns an HTML string with the json keys and values
  function fromJSONtoHTMLTable(templateJSON, resourceType, vectorial, ocurrence) {
    var str = ""
    if (!templateJSON) { return "Not defined";}
    var field = null;

    // Iterate for each value in the JSON object
    for (field in templateJSON) {
      str += fromJSONtoHTMLRow(field,
                               templateJSON[field],
                               resourceType,
                               vectorial,
                               ocurrence);
    }

    return str;
  }

  // Helper for fromJSONtoHTMLTable function
  function fromJSONtoHTMLRow(field, value, resourceType, vectorial_key, ocurrence) {
    var str = "";

    // value can be an array
    if (value.constructor == Array) {
      var it = null;

      for (it = 0; it < value.length; ++it) {
        var current_value = value[it];

        // if value is object, we are dealing with a vectorial value
        if (typeof current_value == 'object') {
          str += '<tr id="' + resourceType.toLowerCase() + '_template_table_' + field + '">\
                             <td class="key_td key_vectorial_td">' + Locale.tr(field) + '</td>\
                             <td class="value_vectorial_td"></td>\
                             <td class="text-right">\
                               <span id="div_add_vectorial">\
                                 <a id="div_add_vectorial_' + field + '" class="add_vectorial_a ocurrence_' + it + ' vectorial_key_' + field + '" href="#"><i class="fa fa-plus-sign"/></a>\
                               </span>&emsp;\
                               <span id="div_minus">\
                                 <a id="div_minus_' + field + '" class="remove_vectorial_x ocurrence_' + it + '" href="#"><i class="fa fa-pencil-square-o"/><i class="fa fa-trash-o"/></a>\
                               </span>\
                             </td>'

          str += fromJSONtoHTMLTable(current_value,
                                     resourceType,
                                     field,
                                     it);
        } else {
          // if it is a single value, create the row for this occurence of the key
          str += fromJSONtoHTMLRow(field,
                                   current_value,
                                   resourceType,
                                   false,
                                   it);
        }
      }
    } else // or value can be a string
      {
        var ocurrence_str = "";
        if (ocurrence != null)
            ocurrence_str = " ocurrence_" + ocurrence;

        // If it comes from a vectorial daddy key, then reflect so in the html
        if (vectorial_key) {
          str += '<tr>\
                       <td class="key_td key_vectorial_td">&emsp;&emsp;' + Locale.tr(field) + '</td>\
                       <td class="value_td value_vectorial_td value_td_input_' + field + ocurrence_str + ' vectorial_key_' + vectorial_key + '" id="value_td_input_' + field + '">' + value + '</td>\
                       <td class="text-right">\
                         <span id="div_edit_vectorial">\
                           <a id="div_edit_' + field + '" class="edit_e' + ocurrence_str + ' vectorial_key_' + vectorial_key + '" href="#"><i class="fa fa-pencil-square-o"/></a>\
                         </span>&emsp;\
                         <span id="div_minus_vectorial">\
                           <a id="div_minus_' + field + '" class="remove_x' + ocurrence_str + ' vectorial_key_' + vectorial_key + '" href="#"><i class="fa fa-trash-o"/></a>\
                         </span>\
                       </td>\
                     </tr>'                 ;
        } else {
          // If it is not comming from a vectorial daddy key, it can still vectorial itself
          if (typeof value == 'object') {
            str += '<tr id="' + resourceType.toLowerCase() + '_template_table_' + field + '">\
                             <td class="key_td key_vectorial_td">'                           + Locale.tr(field) + '</td>\
                             <td class="value_vectorial_td"></td>\
                             <td class="text-right">\
                               <span id="div_add_vectorial">\
                                 <a id="div_add_vectorial_'                               + field + '" class="add_vectorial_a' + ocurrence_str + ' vectorial_key_' + field + '" href="#"><i class="fa fa-plus-sign"/></a>\
                               </span>&emsp;\
                               <span id="div_minus">\
                                 <a id="div_minus_'                               + field + '" class="remove_vectorial_x' + ocurrence_str + '" href="#"><i class="fa fa-trash-o"/></a>\
                               </span>\
                             </td>'
            str += fromJSONtoHTMLTable(value,
                       resourceType,
                       field,
                       ocurrence);
          } else // or, just a single value
             {
               str += '<tr>\
                           <td class="key_td">' + Locale.tr(field) + '</td>\
                           <td class="value_td" id="value_td_input_' + field + '">' + value + '</td>\
                           <td class="text-right">\
                             <span id="div_edit">\
                               <a id="div_edit_' + field + '" class="edit_e' + ocurrence_str + '" href="#"><i class="fa fa-pencil-square-o"/></a>\
                             </span>&emsp;\
                             <span id="div_minus">\
                               <a id="div_minus_' + field + '" class="remove_x' + ocurrence_str + '" href="#"><i class="fa fa-trash-o"/></a>\
                             </span>\
                           </td>\
                         </tr>'                      ;
             }
        }

      }

    return str;
  }

  return {
    'html': _html,
    'setup': _setup
  }
})
;
define('tabs/users-tab/panels/info/panelId',['require'],function(require){
  return 'user_info_tab';
});
define('tabs/users-tab/panels/info',['require','hbs!./info/html','hbs!./info/chgrp-tr','utils/resource-select','utils/template-utils','utils/locale','opennebula/user','sunstone','utils/panel/template-table','../tabId','./info/panelId','tabs/users-tab/dialogs/password/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TemplateInfo = require('hbs!./info/html');
  var TemplateChgrpTr = require('hbs!./info/chgrp-tr');
  var ResourceSelect = require('utils/resource-select');
  var TemplateUtils = require('utils/template-utils');
  var Locale = require('utils/locale');
  var OpenNebulaUser = require('opennebula/user');
  var Sunstone = require('sunstone');

  /*
    TEMPLATES
   */

  var TemplateTable = require('utils/panel/template-table');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "User";
  var XML_ROOT = "USER";
  var PASSWORD_DIALOG_ID = require('tabs/users-tab/dialogs/password/dialogId');

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var groupTrHTML = TemplateChgrpTr({'element': this.element});

    // TODO
    //$(".resource-info-header", $("#users-tab")).html(this.element.NAME);

    var secondaryGroups;

    if (typeof this.element.GROUPS.ID == "object") {
      secondaryGroups = this.element.GROUPS.ID.join(",");
    } else {
      secondaryGroups = "-";
    }

    // TODO: simplify interface?
    var strippedTemplate = $.extend({}, this.element.TEMPLATE);
    delete strippedTemplate["SSH_PUBLIC_KEY"];
    delete strippedTemplate["LANG"];
    delete strippedTemplate["TABLE_ORDER"];
    delete strippedTemplate["DEFAULT_VIEW"];
    delete strippedTemplate["TABLE_DEFAULT_PAGE_LENGTH"];

    var templateTableHTML = TemplateTable.html(strippedTemplate, RESOURCE,
                                              Locale.tr("Attributes"));
    //====

    return TemplateInfo({
      'element': this.element,
      'groupTrHTML': groupTrHTML,
      'secondaryGroups': secondaryGroups,
      'templateTableHTML': templateTableHTML
    });
  }

  function _setup(context) {
    var that = this;

    $('.resource-info-header', '#' + TAB_ID).text(that.element.NAME);

    // Template update
    // TODO: simplify interface?
    var strippedTemplate = $.extend({}, this.element.TEMPLATE);
    delete strippedTemplate["SSH_PUBLIC_KEY"];
    delete strippedTemplate["LANG"];
    delete strippedTemplate["TABLE_ORDER"];
    delete strippedTemplate["DEFAULT_VIEW"];
    delete strippedTemplate["TABLE_DEFAULT_PAGE_LENGTH"];

    var hiddenValues = {};

    if (this.element.TEMPLATE.SSH_PUBLIC_KEY != undefined) {
      hiddenValues.SSH_PUBLIC_KEY = this.element.TEMPLATE.SSH_PUBLIC_KEY;
    }
    if (this.element.TEMPLATE.LANG != undefined) {
      hiddenValues.LANG = this.element.TEMPLATE.LANG;
    }
    if (this.element.TEMPLATE.TABLE_ORDER != undefined) {
      hiddenValues.TABLE_ORDER = this.element.TEMPLATE.TABLE_ORDER;
    }
    if (this.element.TEMPLATE.DEFAULT_VIEW != undefined) {
      hiddenValues.DEFAULT_VIEW = this.element.TEMPLATE.DEFAULT_VIEW;
    }

    TemplateTable.setup(strippedTemplate, RESOURCE, this.element.ID, context, hiddenValues);
    //===

    // Chgrp
    context.off("click", "#div_edit_chg_group_link");
    context.on("click", "#div_edit_chg_group_link", function() {
      ResourceSelect.insert("#value_td_group", context, "Group", that.element.GID, false);
    });

    context.off("change", "#value_td_group .resource_list_select");
    context.on("change", "#value_td_group .resource_list_select", function() {
      var newGroupId = $(this).val();
      if (newGroupId != "") {
        Sunstone.runAction(RESOURCE + ".chgrp", [that.element.ID], newGroupId);
      }
    });

    // SSH input

    context.off("click", ".user_ssh_public_key_edit");
    context.on("click", ".user_ssh_public_key_edit", function() {
      $("#user_ssh_public_key_text", context).hide();
      $("#user_ssh_public_key_textarea", context).show().focus();
    });

    // Password button
    context.off("click", "#update_password");
    context.on("click", "#update_password", function(){
      Sunstone.getDialog(PASSWORD_DIALOG_ID).setParams(
        {selectedElements: [that.element.ID]});
      Sunstone.getDialog(PASSWORD_DIALOG_ID).reset();
      Sunstone.getDialog(PASSWORD_DIALOG_ID).show();
    });

    context.off("change", "#user_ssh_public_key_textarea");
    context.on("change", "#user_ssh_public_key_textarea", function() {
      var template_str = 'SSH_PUBLIC_KEY = "'+TemplateUtils.escapeDoubleQuotes($(this).val())+'"';

      Sunstone.runAction("User.append_template", that.element.ID, template_str);
    });

    context.off("focusout", "#user_ssh_public_key_textarea");
    context.on("focusout", "#user_ssh_public_key_textarea", function() {
      $("#user_ssh_public_key_text", context).show();
      $("#user_ssh_public_key_textarea", context).hide();
    });

    // Change table Order
    context.off("click", "#div_edit_table_order")
    context.on("click", "#div_edit_table_order", function() {
      $(".value_td_table_order", context).html('<select id="table_order_select">' +
         '<option value="asc">' + Locale.tr("ascending") + '</option>' +
         '<option value="desc">' + Locale.tr("descending") + '</option>' +
       '</select>');

      if (that.element.TEMPLATE.TABLE_ORDER) {
        $('#table_order_select', context).val(that.element.TEMPLATE.TABLE_ORDER);
      }
    });

    context.off("change", "#table_order_select")
    context.on("change", "#table_order_select", function() {
      var template_str = 'TABLE_ORDER = "'+$(this).val()+'"';

      Sunstone.runAction("User.append_template_refresh", that.element.ID, template_str);
    });

    // Change language
    context.off("click", "#div_edit_language")
    context.on("click", "#div_edit_language", function() {
      $(".value_td_language", context).html('<select id="language_select">' +
         Locale.language_options +
       '</select>');

      if (that.element.TEMPLATE.LANG) {
        $('#language_select', context).val(that.element.TEMPLATE.LANG);
      }
    });

    context.off("change", "#language_select")
    context.on("change", "#language_select", function() {
      var template_str = 'LANG = "'+$(this).val()+'"';

      Sunstone.runAction("User.append_template_refresh", that.element.ID, template_str);
    });

    // Change view
    context.off("click", "#div_edit_view")
    context.on("click", "#div_edit_view", function() {
      var options = '';
      $.each( config['available_views'], function(id, view) {
        options += '<option value="'+view+'">'+view+'</option>';
      });

      $(".value_td_view", context).html('<select id="view_select">' +
         options +
       '</select>');

      if (that.element.TEMPLATE.DEFAULT_VIEW) {
        $('#view_select', context).val(that.element.TEMPLATE.DEFAULT_VIEW);
      }
    });

    context.off("change", "#view_select")
    context.on("change", "#view_select", function() {
      var template_str = 'DEFAULT_VIEW = "'+$(this).val()+'"';

      Sunstone.runAction("User.append_template_refresh", that.element.ID, template_str);
    });

    return false;
  }
});

define('tabs/users-tab/panels/quotas/panelId',['require'],function(require){
  return 'user_quotas_tab';
});
define('tabs/users-tab/panels/quotas',['require','utils/locale','sunstone-config','utils/quotas/quota-defaults','utils/quotas/quota-widgets','../tabId','./quotas/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var QuotaDefaults = require('utils/quotas/quota-defaults');
  var QuotaWidgets = require('utils/quotas/quota-widgets');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./quotas/panelId');
  var RESOURCE = "User";
  var XML_ROOT = "USER";

  /*
    CONSTRUCTOR
   */

  function Panel(info, tabId) {
    this.tabId = tabId || TAB_ID;
    this.title = Locale.tr("Quotas");
    this.icon = "fa-align-left";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return QuotaWidgets.initQuotasPanel(
      this.element,
      QuotaDefaults.getDefaultQuotas(RESOURCE),
      Config.isTabActionEnabled(this.tabId, RESOURCE+".quotas_dialog"));
  }

  function _setup(context) {
    QuotaWidgets.setupQuotasPanel(
      this.element,
      context,
      Config.isTabActionEnabled(this.tabId, RESOURCE+".quotas_dialog"),
      RESOURCE);

    return false;
  }
});

define('tabs/users-tab/panels/accounting/panelId',['require'],function(require){
  return 'user_accounting_tab';
});
define('tabs/users-tab/panels/accounting',['require','utils/locale','utils/accounting','../tabId','./accounting/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Accounting = require('utils/accounting');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./accounting/panelId');
  var RESOURCE = "User";
  var XML_ROOT = "USER";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Accounting");
    this.icon = "fa-bar-chart-o";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return Accounting.html();
  }

  function _setup(context) {
    Accounting.setup(
      context,
      { fixed_user: this.element.ID,
        init_group_by: "vm"
      });
  }
});

define('tabs/users-tab/panels/showback/panelId',['require'],function(require){
  return 'user_showback_tab';
});
define('tabs/users-tab/panels/showback',['require','utils/locale','sunstone-config','utils/showback','../tabId','./showback/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var Showback = require('utils/showback');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./showback/panelId');
  var RESOURCE = "User";
  var XML_ROOT = "USER";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Showback");
    this.icon = "fa-money";

    this.element = info[XML_ROOT];

    if (!Config.isFeatureEnabled("showback")) {
      throw "Showback is disabled in the configuration";
    }

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return Showback.html();
  }

  function _setup(context) {
    Showback.setup(
      context,
      { fixed_user: this.element.ID,
        fixed_group: ""
      }
    );
  }
});

;(function ($, window, document, undefined) {
  'use strict';

  Foundation.libs.abide = {
    name : 'abide',

    version : '5.5.2',

    settings : {
      live_validate : true,
      validate_on_blur : true,
      // validate_on: 'tab', // tab (when user tabs between fields), change (input changes), manual (call custom events) 
      focus_on_invalid : true,
      error_labels : true, // labels with a for="inputId" will recieve an `error` class
      error_class : 'error',
      timeout : 1000,
      patterns : {
        alpha : /^[a-zA-Z]+$/,
        alpha_numeric : /^[a-zA-Z0-9]+$/,
        integer : /^[-+]?\d+$/,
        number : /^[-+]?\d*(?:[\.\,]\d+)?$/,

        // amex, visa, diners
        card : /^(?:4[0-9]{12}(?:[0-9]{3})?|5[1-5][0-9]{14}|6(?:011|5[0-9][0-9])[0-9]{12}|3[47][0-9]{13}|3(?:0[0-5]|[68][0-9])[0-9]{11}|(?:2131|1800|35\d{3})\d{11})$/,
        cvv : /^([0-9]){3,4}$/,

        // http://www.whatwg.org/specs/web-apps/current-work/multipage/states-of-the-type-attribute.html#valid-e-mail-address
        email : /^[a-zA-Z0-9.!#$%&'*+\/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)+$/,

        // http://blogs.lse.ac.uk/lti/2008/04/23/a-regular-expression-to-match-any-url/
        url: /^(https?|ftp|file|ssh):\/\/([-;:&=\+\$,\w]+@{1})?([-A-Za-z0-9\.]+)+:?(\d+)?((\/[-\+~%\/\.\w]+)?\??([-\+=&;%@\.\w]+)?#?([\w]+)?)?/,
        // abc.de
        domain : /^([a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?\.)+[a-zA-Z]{2,8}$/,

        datetime : /^([0-2][0-9]{3})\-([0-1][0-9])\-([0-3][0-9])T([0-5][0-9])\:([0-5][0-9])\:([0-5][0-9])(Z|([\-\+]([0-1][0-9])\:00))$/,
        // YYYY-MM-DD
        date : /(?:19|20)[0-9]{2}-(?:(?:0[1-9]|1[0-2])-(?:0[1-9]|1[0-9]|2[0-9])|(?:(?!02)(?:0[1-9]|1[0-2])-(?:30))|(?:(?:0[13578]|1[02])-31))$/,
        // HH:MM:SS
        time : /^(0[0-9]|1[0-9]|2[0-3])(:[0-5][0-9]){2}$/,
        dateISO : /^\d{4}[\/\-]\d{1,2}[\/\-]\d{1,2}$/,
        // MM/DD/YYYY
        month_day_year : /^(0[1-9]|1[012])[- \/.](0[1-9]|[12][0-9]|3[01])[- \/.]\d{4}$/,
        // DD/MM/YYYY
        day_month_year : /^(0[1-9]|[12][0-9]|3[01])[- \/.](0[1-9]|1[012])[- \/.]\d{4}$/,

        // #FFF or #FFFFFF
        color : /^#?([a-fA-F0-9]{6}|[a-fA-F0-9]{3})$/
      },
      validators : {
        equalTo : function (el, required, parent) {
          var from  = document.getElementById(el.getAttribute(this.add_namespace('data-equalto'))).value,
              to    = el.value,
              valid = (from === to);

          return valid;
        }
      }
    },

    timer : null,

    init : function (scope, method, options) {
      this.bindings(method, options);
    },

    events : function (scope) {
      var self = this,
          form = self.S(scope).attr('novalidate', 'novalidate'),
          settings = form.data(this.attr_name(true) + '-init') || {};

      this.invalid_attr = this.add_namespace('data-invalid');

      function validate(originalSelf, e) {
        clearTimeout(self.timer);
        self.timer = setTimeout(function () {
          self.validate([originalSelf], e);
        }.bind(originalSelf), settings.timeout);
      }


      form
        .off('.abide')
        .on('submit.fndtn.abide', function (e) {
          var is_ajax = /ajax/i.test(self.S(this).attr(self.attr_name()));
          return self.validate(self.S(this).find('input, textarea, select').not(":hidden, [data-abide-ignore]").get(), e, is_ajax);
        })
        .on('validate.fndtn.abide', function (e) {
          if (settings.validate_on === 'manual') {
            self.validate([e.target], e);
          }
        })
        .on('reset', function (e) {
          return self.reset($(this), e);          
        })
        .find('input, textarea, select').not(":hidden, [data-abide-ignore]")
          .off('.abide')
          .on('blur.fndtn.abide change.fndtn.abide', function (e) {
            // old settings fallback
            // will be deprecated with F6 release
            if (settings.validate_on_blur && settings.validate_on_blur === true) {
              validate(this, e);
            }
            // new settings combining validate options into one setting
            if (settings.validate_on === 'change') {
              validate(this, e);
            }
          })
          .on('keydown.fndtn.abide', function (e) {
            // old settings fallback
            // will be deprecated with F6 release
            if (settings.live_validate && settings.live_validate === true && e.which != 9) {
              validate(this, e);
            }
            // new settings combining validate options into one setting
            if (settings.validate_on === 'tab' && e.which === 9) {
              validate(this, e);
            }
            else if (settings.validate_on === 'change') {
              validate(this, e);
            }
          })
          .on('focus', function (e) {
            if (navigator.userAgent.match(/iPad|iPhone|Android|BlackBerry|Windows Phone|webOS/i)) {
              $('html, body').animate({
                  scrollTop: $(e.target).offset().top
              }, 100);
            } 
          });
    },

    reset : function (form, e) {
      var self = this;
      form.removeAttr(self.invalid_attr);

      $('[' + self.invalid_attr + ']', form).removeAttr(self.invalid_attr);
      $('.' + self.settings.error_class, form).not('small').removeClass(self.settings.error_class);
      $(':input', form).not(':button, :submit, :reset, :hidden, [data-abide-ignore]').val('').removeAttr(self.invalid_attr);
    },

    validate : function (els, e, is_ajax) {
      var validations = this.parse_patterns(els),
          validation_count = validations.length,
          form = this.S(els[0]).closest('form'),
          submit_event = /submit/.test(e.type);

      // Has to count up to make sure the focus gets applied to the top error
      for (var i = 0; i < validation_count; i++) {
        if (!validations[i] && (submit_event || is_ajax)) {
          if (this.settings.focus_on_invalid) {
            els[i].focus();
          }
          form.trigger('invalid.fndtn.abide');
          this.S(els[i]).closest('form').attr(this.invalid_attr, '');
          return false;
        }
      }

      if (submit_event || is_ajax) {
        form.trigger('valid.fndtn.abide');
      }

      form.removeAttr(this.invalid_attr);

      if (is_ajax) {
        return false;
      }

      return true;
    },

    parse_patterns : function (els) {
      var i = els.length,
          el_patterns = [];

      while (i--) {
        el_patterns.push(this.pattern(els[i]));
      }

      return this.check_validation_and_apply_styles(el_patterns);
    },

    pattern : function (el) {
      var type = el.getAttribute('type'),
          required = typeof el.getAttribute('required') === 'string';

      var pattern = el.getAttribute('pattern') || '';

      if (this.settings.patterns.hasOwnProperty(pattern) && pattern.length > 0) {
        return [el, this.settings.patterns[pattern], required];
      } else if (pattern.length > 0) {
        return [el, new RegExp(pattern), required];
      }

      if (this.settings.patterns.hasOwnProperty(type)) {
        return [el, this.settings.patterns[type], required];
      }

      pattern = /.*/;

      return [el, pattern, required];
    },

    // TODO: Break this up into smaller methods, getting hard to read.
    check_validation_and_apply_styles : function (el_patterns) {
      var i = el_patterns.length,
          validations = [],
          form = this.S(el_patterns[0][0]).closest('[data-' + this.attr_name(true) + ']'),
          settings = form.data(this.attr_name(true) + '-init') || {};
      while (i--) {
        var el = el_patterns[i][0],
            required = el_patterns[i][2],
            value = el.value.trim(),
            direct_parent = this.S(el).parent(),
            validator = el.getAttribute(this.add_namespace('data-abide-validator')),
            is_radio = el.type === 'radio',
            is_checkbox = el.type === 'checkbox',
            label = this.S('label[for="' + el.getAttribute('id') + '"]'),
            valid_length = (required) ? (el.value.length > 0) : true,
            el_validations = [];

        var parent, valid;

        // support old way to do equalTo validations
        if (el.getAttribute(this.add_namespace('data-equalto'))) { validator = 'equalTo' }

        if (!direct_parent.is('label')) {
          parent = direct_parent;
        } else {
          parent = direct_parent.parent();
        }

        if (is_radio && required) {
          el_validations.push(this.valid_radio(el, required));
        } else if (is_checkbox && required) {
          el_validations.push(this.valid_checkbox(el, required));

        } else if (validator) {
          // Validate using each of the specified (space-delimited) validators.
          var validators = validator.split(' ');
          var last_valid = true, all_valid = true;
          for (var iv = 0; iv < validators.length; iv++) {
              valid = this.settings.validators[validators[iv]].apply(this, [el, required, parent])
              el_validations.push(valid);
              all_valid = valid && last_valid;
              last_valid = valid;
          }
          if (all_valid) {
              this.S(el).removeAttr(this.invalid_attr);
              parent.removeClass('error');
              if (label.length > 0 && this.settings.error_labels) {
                label.removeClass(this.settings.error_class).removeAttr('role');
              }
              $(el).triggerHandler('valid');
          } else {
              this.S(el).attr(this.invalid_attr, '');
              parent.addClass('error');
              if (label.length > 0 && this.settings.error_labels) {
                label.addClass(this.settings.error_class).attr('role', 'alert');
              }
              $(el).triggerHandler('invalid');
          }
        } else {

          if (el_patterns[i][1].test(value) && valid_length ||
            !required && el.value.length < 1 || $(el).attr('disabled')) {
            el_validations.push(true);
          } else {
            el_validations.push(false);
          }

          el_validations = [el_validations.every(function (valid) {return valid;})];
          if (el_validations[0]) {
            this.S(el).removeAttr(this.invalid_attr);
            el.setAttribute('aria-invalid', 'false');
            el.removeAttribute('aria-describedby');
            parent.removeClass(this.settings.error_class);
            if (label.length > 0 && this.settings.error_labels) {
              label.removeClass(this.settings.error_class).removeAttr('role');
            }
            $(el).triggerHandler('valid');
          } else {
            this.S(el).attr(this.invalid_attr, '');
            el.setAttribute('aria-invalid', 'true');

            // Try to find the error associated with the input
            var errorElem = parent.find('small.' + this.settings.error_class, 'span.' + this.settings.error_class);
            var errorID = errorElem.length > 0 ? errorElem[0].id : '';
            if (errorID.length > 0) {
              el.setAttribute('aria-describedby', errorID);
            }

            // el.setAttribute('aria-describedby', $(el).find('.error')[0].id);
            parent.addClass(this.settings.error_class);
            if (label.length > 0 && this.settings.error_labels) {
              label.addClass(this.settings.error_class).attr('role', 'alert');
            }
            $(el).triggerHandler('invalid');
          }
        }
        validations = validations.concat(el_validations);
      }
      return validations;
    },

    valid_checkbox : function (el, required) {
      var el = this.S(el),
          valid = (el.is(':checked') || !required || el.get(0).getAttribute('disabled'));

      if (valid) {
        el.removeAttr(this.invalid_attr).parent().removeClass(this.settings.error_class);
        $(el).triggerHandler('valid');
      } else {
        el.attr(this.invalid_attr, '').parent().addClass(this.settings.error_class);
        $(el).triggerHandler('invalid');
      }

      return valid;
    },

    valid_radio : function (el, required) {
      var name = el.getAttribute('name'),
          group = this.S(el).closest('[data-' + this.attr_name(true) + ']').find("[name='" + name + "']"),
          count = group.length,
          valid = false,
          disabled = false;

      // Has to count up to make sure the focus gets applied to the top error
        for (var i=0; i < count; i++) {
            if( group[i].getAttribute('disabled') ){
                disabled=true;
                valid=true;
            } else {
                if (group[i].checked){
                    valid = true;
                } else {
                    if( disabled ){
                        valid = false;
                    }
                }
            }
        }

      // Has to count up to make sure the focus gets applied to the top error
      for (var i = 0; i < count; i++) {
        if (valid) {
          this.S(group[i]).removeAttr(this.invalid_attr).parent().removeClass(this.settings.error_class);
          $(group[i]).triggerHandler('valid');
        } else {
          this.S(group[i]).attr(this.invalid_attr, '').parent().addClass(this.settings.error_class);
          $(group[i]).triggerHandler('invalid');
        }
      }

      return valid;
    },

    valid_equal : function (el, required, parent) {
      var from  = document.getElementById(el.getAttribute(this.add_namespace('data-equalto'))).value,
          to    = el.value,
          valid = (from === to);

      if (valid) {
        this.S(el).removeAttr(this.invalid_attr);
        parent.removeClass(this.settings.error_class);
        if (label.length > 0 && settings.error_labels) {
          label.removeClass(this.settings.error_class);
        }
      } else {
        this.S(el).attr(this.invalid_attr, '');
        parent.addClass(this.settings.error_class);
        if (label.length > 0 && settings.error_labels) {
          label.addClass(this.settings.error_class);
        }
      }

      return valid;
    },

    valid_oneof : function (el, required, parent, doNotValidateOthers) {
      var el = this.S(el),
        others = this.S('[' + this.add_namespace('data-oneof') + ']'),
        valid = others.filter(':checked').length > 0;

      if (valid) {
        el.removeAttr(this.invalid_attr).parent().removeClass(this.settings.error_class);
      } else {
        el.attr(this.invalid_attr, '').parent().addClass(this.settings.error_class);
      }

      if (!doNotValidateOthers) {
        var _this = this;
        others.each(function () {
          _this.valid_oneof.call(_this, this, null, null, true);
        });
      }

      return valid;
    },

    reflow : function(scope, options) {
      var self = this,
          form = self.S('[' + this.attr_name() + ']').attr('novalidate', 'novalidate');
          self.S(form).each(function (idx, el) {
            self.events(el);
          });
    }
  };
}(jQuery, window, window.document));

define("foundation.abide", ["foundation.core"], function(){});

define('utils/form-panels/form-panel',['require','foundation.abide','utils/notifier','utils/locale','sunstone'],function(require) {
  // Dependencies
  require('foundation.abide');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var Sunstone = require('sunstone');

  function BaseFormPanel() {
    return this;
  }

  BaseFormPanel.prototype = {
    'insert': _insert,
    'reset': _reset,
    'setAction': _setAction,
    'title': _title,
    'buttonText': _buttonText,
    'resetButton': _resetButton,
    'actionOptions': _actionOptions
  }

  return BaseFormPanel;

  function _insert(context) {
    var that = this;
    this.wizardElement = $(that.htmlWizard()).appendTo( $("#wizardForms", context) );
    if (that.htmlAdvanced) {
      this.advancedElement = $(that.htmlAdvanced()).appendTo( $("#advancedForms", context) );
    }

    context.off('invalid.fndtn.abide', '#' + that.formPanelId + 'Wizard');
    context.off('valid.fndtn.abide', '#' + that.formPanelId + 'Wizard');
    context.on('invalid.fndtn.abide', '#' + that.formPanelId + 'Wizard', function(e) {
      Notifier.notifyError(Locale.tr("One or more required fields are missing or malformed."));
      Sunstone.hideFormPanelLoading(that.tabId);
    }).on('valid.fndtn.abide', '#' + that.formPanelId + 'Wizard', function(e) {
      that.submitWizard(this);
      return false;
    });

    context.off('invalid.fndtn.abide', '#' + that.formPanelId + 'Advanced');
    context.off('valid.fndtn.abide', '#' + that.formPanelId + 'Advanced');
    context.on('invalid.fndtn.abide', '#' + that.formPanelId + 'Advanced', function(e) {
      Notifier.notifyError(Locale.tr("One or more required fields are missing or malformed."));
      Sunstone.hideFormPanelLoading(that.tabId);
    }).on('valid.fndtn.abide', '#' + that.formPanelId + 'Advanced', function(e) {
      that.submitAdvanced(this);
      return false;
    });

    context.foundation('abide', 'reflow');

    that.setup(context);
  }

  function _reset(context) {
    this.wizardElement.remove();
    if (this.htmlAdvanced) {
      this.advancedElement.remove();
    }

    this.insert(context);
  }

  function _setAction(context, action) {
    var prevAction = this.action;

    this.action = action;

    if (prevAction != undefined && (prevAction != action || action == "update")) {
      this.reset(context);
    }
  }

  // @return [Object] actionOptions of the form based on the defined action
  function _actionOptions() {
    if (this.action) {
      var actionOptions = this.actions[this.action]
      if (actionOptions) {
        return actionOptions;
      } else {
        return null;
      }
    } else {
      return null;
    }
  }

  // @return [String] The title of the form based on the defined action
  function _title() {
    var actionOptions = this.actionOptions();
    if (actionOptions) {
      return actionOptions.title;
    } else {
      return "";
    }
  }

  // @return [String] The buttonText of the form based on the defined action
  function _buttonText() {
    var actionOptions = this.actionOptions();
    if (actionOptions) {
      return actionOptions.buttonText;
    } else {
      return "";
    }
  }

  // @return [Boolean] Is enabled the reset button based on the defined action
  function _resetButton() {
    var actionOptions = this.actionOptions();
    if (actionOptions) {
      return actionOptions.resetButton;
    } else {
      return false;
    }
  }
})
;

/* START_TEMPLATE */
define('hbs!tabs/users-tab/form-panels/create/wizard',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row\" >\n    <div class=\"medium-6 columns\">\n      ";
  stack1 = ((helper = (helper = helpers.userCreationHTML || (depth0 != null ? depth0.userCreationHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"userCreationHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/users-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/users-tab/form-panels/create',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','tabs/users-tab/utils/user-creation','hbs!./create/wizard','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var UserCreation = require('tabs/users-tab/utils/user-creation');

  /*
    TEMPLATES
   */

  var TemplateWizardHTML = require('hbs!./create/wizard');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create User"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      }
    };

    this.userCreation = new UserCreation(FORM_PANEL_ID);

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    return TemplateWizardHTML({
      'formPanelId': this.formPanelId,
      'userCreationHTML': this.userCreation.html()
    });
  }

  function _setup(context) {
    this.userCreation.setup(context);
  }

  function _submitWizard(context) {
    var user_json = {
      "user" : this.userCreation.retrieve(context)
    };

    Sunstone.runAction("User.create",user_json);
    return false;
  }

  function _onShow(context) {
  }
});

define('tabs/users-tab',['require','utils/locale','./users-tab/buttons','./users-tab/actions','./users-tab/datatable','./users-tab/tabId','./users-tab/dialogs/password','./users-tab/dialogs/auth-driver','./users-tab/dialogs/quotas','./users-tab/panels/info','./users-tab/panels/quotas','./users-tab/panels/accounting','./users-tab/panels/showback','./users-tab/form-panels/create'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./users-tab/buttons');
  var Actions = require('./users-tab/actions');
  var Table = require('./users-tab/datatable');

  var TAB_ID = require('./users-tab/tabId');
  var DATATABLE_ID = "dataTableUsers";

  var _dialogs = [
    require('./users-tab/dialogs/password'),
    require('./users-tab/dialogs/auth-driver'),
    require('./users-tab/dialogs/quotas')
  ];

  var _panels = [
    require('./users-tab/panels/info'),
    require('./users-tab/panels/quotas'),
    require('./users-tab/panels/accounting'),
    require('./users-tab/panels/showback')
  ];

  var _formPanels = [
    require('./users-tab/form-panels/create')
  ];

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("Users"),
    tabClass: "subTab",
    parentTab: "system-tab",
    listHeader: '<i class="fa fa-fw fa-user"></i>&emsp;'+Locale.tr("Users"),
    infoHeader: '<i class="fa fa-fw fa-user"></i>&emsp;'+Locale.tr("User"),
    subheader: '<span>\
        <span class="total_users"/> <small>'+Locale.tr("TOTAL")+'</small>\
      </span>',
    resource: 'User',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});


/* START_TEMPLATE */
define('hbs!tabs/settings-tab/panels/user-config/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <div class=\"row\">\n      <div class=\"large-12 large-centered columns\">\n        <dl class=\"accordion\" data-accordion>\n          <dd class=\"accordion-navigation\">\n            <a href=\"#provision_update_language_accordion\" class=\"text-center accordion-a\">\n              <div class=\"row only-not-active\">\n                <div class=\"large-12 large-centered columns\">\n                  <div class=\"text-center\">\n                    <span class=\"fa-stack fa-3x\" style=\"color: #777\"> <i class=\"fa fa-cloud fa-stack-2x\"></i> <i class=\"fa fa-comments fa-stack-1x fa-inverse\"></i>\n                    </span>\n                  </div>\n                </div>\n              </div>\n              <br class=\"only-not-active\">\n              <i class=\"fa fa-lg fa-comments only-active\"></i>\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Change Language", {"name":"tr","hash":{},"data":data})))
    + "\n            </a>\n            <div id=\"provision_update_language_accordion\" class=\"content\">\n              <br>\n              <form id=\"provision_change_language_form\">\n                <div class=\"row\">\n                  <div class=\"large-12 columns\">\n                    <select type=\"language\" id=\"provision_new_language\" class=\"provision-input\" style=\"height: 40px !important; font-size: 16px; padding: 0.5rem  !important;\">";
  stack1 = ((helper = (helper = helpers.languageOptions || (depth0 != null ? depth0.languageOptions : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"languageOptions","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</select>\n                  </div>\n                </div>\n                <div class=\"row\">\n                  <div class=\"large-12 columns\">\n                    <button href\"#\" type=\"submit\" class=\"button large radius large-12 small-12\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update Language", {"name":"tr","hash":{},"data":data})))
    + "</button>\n                  </div>\n                </div>\n              </form>\n            </div>\n          </dd>\n        </dl>\n      </div>\n    </div>\n  </div>\n  <div class=\"large-6 columns\">\n    <div class=\"row\">\n      <div class=\"large-12 large-centered columns\">\n        <dl class=\"accordion\" data-accordion>\n          <dd class=\"accordion-navigation\">\n            <a href=\"#provision_update_password_accordion\" class=\"text-center accordion-a\">\n              <div class=\"row only-not-active\">\n                <div class=\"large-12 large-centered columns\">\n                  <div class=\"text-center\">\n                    <span class=\"fa-stack fa-3x\" style=\"color: #777\">\n                      <i class=\"fa fa-cloud fa-stack-2x\"></i>\n                      <i class=\"fa fa-lock fa-stack-1x fa-inverse\"></i>\n                    </span>\n                  </div>\n                </div>\n              </div>\n              <br class=\"only-not-active\">\n              <i class=\"fa fa-lg fa-lock only-active\"></i>\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Change Password", {"name":"tr","hash":{},"data":data})))
    + "\n            </a>\n            <div id=\"provision_update_password_accordion\" class=\"content\">\n              <br>\n              <form id=\"provision_change_password_form\">\n                <div class=\"row\">\n                  <div class=\"large-12 columns\">\n                    <input type=\"password\" id=\"provision_new_password\" class=\"provision-input\" placeholder=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "New Password", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"height: 40px !important; font-size: 16px; padding: 0.5rem  !important;\"/>\n                  </div>\n                </div>\n                <div class=\"row\">\n                  <div class=\"large-12 columns\">\n                    <input type=\"password\" id=\"provision_new_confirm_password\" class=\"provision-input\" placeholder=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Confirm Password", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"height: 40px !important; font-size: 16px; padding: 0.5rem  !important;\"/>\n                  </div>\n                </div>\n                <div class=\"row\">\n                  <div class=\"large-12 columns\">\n                    <button href\"#\" type=\"submit\" class=\"button large radius large-12 small-12\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update Password", {"name":"tr","hash":{},"data":data})))
    + "</button>\n                  </div>\n                </div>\n              </form>\n            </div>\n          </dd>\n        </dl>\n      </div>\n    </div>\n  </div>\n</div>\n<br>\n<br>\n<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <div class=\"row\">\n      <div class=\"large-12 large-centered columns\">\n        <dl class=\"accordion\" data-accordion>\n          <dd class=\"accordion-navigation\">\n            <a href=\"#provision_update_view_accordion\" class=\"text-center accordion-a\">\n              <div class=\"row only-not-active\">\n                <div class=\"large-12 large-centered columns\">\n                  <div class=\"text-center\">\n                    <span class=\"fa-stack fa-3x\" style=\"color: #777\">\n                      <i class=\"fa fa-cloud fa-stack-2x\"></i>\n                      <i class=\"fa fa-picture-o fa-stack-1x fa-inverse\"></i>\n                    </span>\n                  </div>\n                </div>\n              </div>\n              <br class=\"only-not-active\">\n              <i class=\"fa fa-lg fa-picture-o only-active\"></i>\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Change view", {"name":"tr","hash":{},"data":data})))
    + "\n            </a>\n            <div id=\"provision_update_view_accordion\" class=\"content\">\n              <br>\n              <form id=\"provision_change_view_form\">\n                <div class=\"row\">\n                  <div class=\"large-12 columns\">\n                    <select id=\"provision_user_views_select\" class=\"provision-input\" style=\"height: 40px !important; font-size: 16px; padding: 0.5rem  !important;\"></select>\n                  </div>\n                </div>\n                <div class=\"row\">\n                  <div class=\"large-12 columns\">\n                    <button href\"#\" type=\"submit\" class=\"button large radius large-12 small-12\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update view", {"name":"tr","hash":{},"data":data})))
    + "</button>\n                  </div>\n                </div>\n              </form>\n            </div>\n          </dd>\n        </dl>\n      </div>\n    </div>\n  </div>\n  <div class=\"large-6 columns\">\n    <form id=\"provision_add_ssh_key_form\">\n      <div class=\"row\">\n        <div class=\"large-12 large-centered columns\">\n          <dl class=\"accordion\" data-accordion>\n            <dd class=\"accordion-navigation\">\n              <a href=\"#provision_add_ssh_key_accordion\" class=\"text-center accordion-a\">\n                <div class=\"row only-not-active\">\n                  <div class=\"large-12 large-centered columns\">\n                    <div class=\"text-center\">\n                      <span class=\"fa-stack fa-3x\" style=\"color: #777\">\n                        <i class=\"fa fa-cloud fa-stack-2x\"></i>\n                        <i class=\"fa fa-key fa-stack-1x fa-inverse\"></i>\n                      </span>\n                    </div>\n                  </div>\n                </div>\n                <br class=\"only-not-active\">\n                <i class=\"fa fa-key fa-lg only-active\"></i>\n                <span class=\"provision_add_ssh_key_button\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add SSH Key", {"name":"tr","hash":{},"data":data})))
    + "</span>\n                <span class=\"provision_update_ssh_key_button\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update SSH Key", {"name":"tr","hash":{},"data":data})))
    + "</span>\n              </a>\n              <div id=\"provision_add_ssh_key_accordion\" class=\"content\">\n                <br>\n                <p style=\"font-size: 16px; color: #999\">\n                  <span class=\"provision_add_ssh_key_button\">\n                    "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add a public SSH key to your account!", {"name":"tr","hash":{},"data":data})))
    + "\n                    <br>\n                    "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "You will be able to access your Virtual Machines without password", {"name":"tr","hash":{},"data":data})))
    + "\n                  </span>\n                  <span class=\"provision_update_ssh_key_button\">\n                    "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update your public SSH key!", {"name":"tr","hash":{},"data":data})))
    + "\n                    <br>\n                    "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "You will be able to access your Virtual Machines without password", {"name":"tr","hash":{},"data":data})))
    + "\n                  </span>\n                </p>\n                <div class=\"row\">\n                  <div class=\"large-12 large-centered columns\">\n                    <textarea id=\"provision_ssh_key\" style=\"height: 100px; font-size: 14px\" placeholder=\"SSH key\" class=\"provision-input\"></textarea>\n                  </div>\n                </div>\n                <div class=\"row\">\n                  <div class=\"large-12 large-centered columns\">\n                    <button href=\"#\" type=\"submit\" class=\"provision_add_ssh_key_button button large radius large-12 small-12\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add SSH Key", {"name":"tr","hash":{},"data":data})))
    + "</button>\n                    <button href=\"#\" type=\"submit\" class=\"provision_update_ssh_key_button button large radius large-12 small-12 hidden\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update SSH Key", {"name":"tr","hash":{},"data":data})))
    + "</button>\n                  </div>\n                </div>\n              </div>\n            </dd>\n          </dl>\n        </div>\n      </div>\n    </form>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/settings-tab/panels/user-config/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/settings-tab/panels/user-config/panelId',['require'],function(require) {
  return 'user_config_tab';
});

define('tabs/settings-tab/panels/user-config',['require','foundation.accordion','utils/locale','sunstone-config','opennebula','utils/template-utils','sunstone','utils/notifier','hbs!./user-config/html','../tabId','./user-config/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.accordion');
  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var OpenNebula = require('opennebula');
  var TemplateUtils = require('utils/template-utils');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');

  /*
    TEMPLATES
   */

  var TemplateEasyInfo = require('hbs!./user-config/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./user-config/panelId');
  var RESOURCE = "User";
  var XML_ROOT = "USER";

  /*
    CONSTRUCTOR
   */

  function Panel(info, tabId) {
    this.tabId = tabId || TAB_ID;
    this.title = Locale.tr("Config");
    this.icon = "fa-info";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateEasyInfo({'languageOptions': Locale.language_options});
  }

  function _setup(context) {
    $(document).foundation('accordion', 'reflow');
    var ssh_key = this.element.TEMPLATE.SSH_PUBLIC_KEY;
    if (ssh_key && ssh_key.length) {
      $("#provision_ssh_key", context).val(ssh_key);
      $(".provision_add_ssh_key_button", context).hide();
      $(".provision_update_ssh_key_button", context).show();
    } else {
      $(".provision_add_ssh_key_button", context).show();
      $(".provision_update_ssh_key_button", context).hide();
    }

    $('#provision_new_language option[value="' + config['user_config']["lang"] + '"]', context).attr('selected', 'selected');

    $.each(config['available_views'], function(id, view) {
      $('select#provision_user_views_select', context).append('<option value="' + view + '">' + view + '</option>')
    });

    $('#provision_user_views_select option[value="' + config['user_config']["default_view"] + '"]', context).attr('selected', 'selected');

    $("#provision_change_password_form").submit(function() {
      var pw = $('#provision_new_password', this).val();
      var confirm_password = $('#provision_new_confirm_password', this).val();

      if (!pw.length) {
        Notifier.notifyError(Locale.tr("Fill in a new password"));
        return false;
      }

      if (pw !== confirm_password) {
        Notifier.notifyError(Locale.tr("Passwords do not match"));
        return false;
      }

      Sunstone.runAction("User.passwd", ["-1"], pw);
      return false;
    });

    $("#provision_add_ssh_key_form").submit(function() {
      var keypair = $('#provision_ssh_key', this).val();

      if (!keypair.length) {
        Notifier.notifyError(Locale.tr("You have to provide an SSH key"));
        return false;
      }

      var template_str = 'SSH_PUBLIC_KEY = "'+TemplateUtils.escapeDoubleQuotes(keypair)+'"';

      Sunstone.runAction("User.append_template", "-1", template_str);

      return false;
    });

    $("#provision_change_view_form").submit(function() {
      var template_str = 'DEFAULT_VIEW = "'+$('#provision_user_views_select', this).val()+'"';

      Sunstone.runAction("User.append_template_refresh", "-1", template_str);

      return false;
    });

    $("#provision_change_language_form").submit(function() {
      var template_str = 'LANG = "'+$('#provision_new_language', this).val()+'"';

      Sunstone.runAction("User.append_template_refresh", "-1", template_str);

      return false;
    });

    return false;
  }
});

define('tabs/settings-tab/panels/group-quotas/panelId',['require'],function(require){
  return 'group_quotas_tab';
});
define('tabs/settings-tab/panels/group-quotas',['require','utils/locale','sunstone-config','utils/quotas/quota-defaults','utils/quotas/quota-widgets','opennebula/group','../tabId','./group-quotas/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var QuotaDefaults = require('utils/quotas/quota-defaults');
  var QuotaWidgets = require('utils/quotas/quota-widgets');
  var OpenNebulaGroup = require('opennebula/group');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./group-quotas/panelId');
  var RESOURCE = "User";
  var XML_ROOT = "USER";

  /*
    CONSTRUCTOR
   */

  function Panel(info, tabId) {
    this.tabId = tabId || TAB_ID;
    this.title = Locale.tr("Group Quotas");
    this.icon = "fa-align-left";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return '<div class="row">\
              <div class="large-6 columns">\
                <label>' + Locale.tr("Select group") + ':\
                  <select id="quota_group_sel">\
                  </select>\
                </label>\
              </div>\
            </div>\
            <div id="group_quotasTabBody" class="row">\
            </div>';
  }

  function _setup(context) {

    var groups = this.element.GROUPS.ID;

    if (!$.isArray(groups)){
      groups = [groups];
    }

    var groupDropdownOptions = "";
    $.each(groups, function(){
      groupDropdownOptions +=
      '<option elem_id="'+this+
      '" value="'+this+'">'+
      OpenNebulaGroup.getName(this)+' (id:'+this+')</option>';
    });

    $('select#quota_group_sel', context).html(groupDropdownOptions);

    $("#quota_group_sel", context).off("change");
    $("#quota_group_sel", context).on("change", function() {
        var value_str = $('select#quota_group_sel').val();
        if(value_str!="")
        {
          fillGroupQuotas(value_str, context);
        }
    });

    fillGroupQuotas('-1', context);

    return false;
  }

  function fillGroupQuotas(group_id, context){
    OpenNebulaGroup.show({
      data : {
        id: group_id
      },
      success: function(request,group_json){
        var info = group_json.GROUP;

        var quotas_tab_html = QuotaWidgets.initQuotasPanel(
          info,
          QuotaDefaults.getDefaultQuotas("Group"),
          false);

        $("#group_quotasTabBody", context).html(quotas_tab_html);

        QuotaWidgets.setupQuotasPanel(
          info,
          $("#group_quotasTabBody", context),
          false,
          "Group");

        $("select#quota_group_sel", context).val(info.ID);
      }
    });
  }
});


/* START_TEMPLATE */
define('hbs!tabs/acls-tab/form-panels/create/wizard',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <fieldset>\n        <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "This rule applies to", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n        <div class=\"row\">\n          <div class=\"large-4 columns\">\n            <input type=\"radio\" class=\"applies\" name=\"applies_select\" value=\"*\" id=\"applies_all\">\n            <label class=\"applies\" for=\"applies_all\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "All", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          </div>\n          <div class=\"large-4 columns\">\n            <input type=\"radio\" class=\"applies\" name=\"applies_select\" value=\"applies_to_user\" id=\"applies_id\">\n            <label class=\"applies\" for=\"applies_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          </div>\n          <div class=\"large-4 columns\">\n            <input type=\"radio\" class=\"applies\" name=\"applies_select\" value=\"applies_to_group\" id=\"applies_group\">\n            <label class=\"applies\" for=\"applies_group\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <div class=\"applies_to_user\">\n              <label for=\"applies_to_user\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n              <div name=\"applies_to_user\" id=\"applies_to_user\">\n              </div>\n            </div>\n            <div class=\"applies_to_group\">\n              <label for=\"applies_to_group\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n              <div name=\"applies_to_group\" id=\"applies_to_group\">\n              </div>\n            </div>\n          </div>\n        </div>\n      </fieldset>\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"zones_applies\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Zones where the rule applies", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      <div name=\"zones_applies\" id=\"zones_applies\">\n      </div>\n    </div>\n  </div>\n  <fieldset>\n    <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Affected resources", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n    <div class=\"row\">\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_host\" name=\"res_host\" class=\"resource_cb\" value=\"HOST\">\n        <label for=\"res_host\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hosts", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_cluster\" name=\"res_cluster\" class=\"resource_cb\" value=\"CLUSTER\">\n        <label for=\"res_cluster\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Clusters", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_datastore\" name=\"res_datastore\" class=\"resource_cb\" value=\"DATASTORE\">\n        <label for=\"res_datastore\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastores", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_vm\" name=\"res_vm\" class=\"resource_cb\" value=\"VM\">\n        <label for=\"res_vm\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Machines", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_net\" name=\"res_net\" class=\"resource_cb\" value=\"NET\">\n        <label for=\"res_net\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Networks", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_image\" name=\"res_image\" class=\"resource_cb\" value=\"IMAGE\">\n        <label for=\"res_image\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Images", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_template\" name=\"res_template\" class=\"resource_cb\" value=\"TEMPLATE\">\n        <label for=\"res_template\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Templates", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_user\" name=\"res_user\" class=\"resource_cb\" value=\"USER\">\n        <label for=\"res_user\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Users", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_group\" name=\"res_group\" class=\"resource_cb\" value=\"GROUP\">\n        <label for=\"res_group\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Groups", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_document\" name=\"res_document\" class=\"resource_cb\" value=\"DOCUMENT\">\n        <label for=\"res_document\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Documents", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_zone\" name=\"res_zone\" class=\"resource_cb\" value=\"ZONE\">\n        <label for=\"res_zone\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Zones", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_secgroup\" name=\"res_secgroup\" class=\"resource_cb\" value=\"SECGROUP\">\n        <label for=\"res_secgroup\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Security Groups", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"res_vdc\" name=\"res_vdc\" class=\"resource_cb\" value=\"VDC\">\n        <label for=\"res_vdc\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VDCs", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n    </div>\n  </fieldset>\n  <fieldset>\n    <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Resource subset", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n    <div class=\"row\">\n      <div class=\"large-3 columns\">\n        <input type=\"radio\" class=\"res_subgroup\" name=\"mode_select\" value=\"*\" id=\"res_subgroup_all\">\n        <label class=\"res_subgroup\" for=\"res_subgroup_all\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "All", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 columns\">\n        <input type=\"radio\" class=\"res_subgroup\" name=\"mode_select\" value=\"res_id\" id=\"res_subgroup_id\">\n        <label class=\"res_subgroup\" for=\"res_subgroup_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 columns\">\n        <input type=\"radio\" class=\"res_subgroup\" name=\"mode_select\" value=\"belonging_to\" id=\"res_subgroup_group\">\n        <label class=\"res_subgroup\" for=\"res_subgroup_group\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 columns\">\n        <input type=\"radio\" class=\"res_subgroup\" name=\"mode_select\" value=\"in_cluster\" id=\"res_subgroup_cluster\">\n        <label class=\"res_subgroup\" for=\"res_subgroup_cluster\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cluster", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-6 columns\">\n        <div class=\"res_id\">\n          <label for=\"res_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Resource ID", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input type=\"text\" name=\"res_id\" id=\"res_id\"></input>\n        </div>\n        <div class=\"belonging_to\">\n          <label for=\"belonging_to\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <div name=\"belonging_to\" id=\"belonging_to\">\n          </div>\n        </div>\n        <div class=\"in_cluster\">\n          <label for=\"in_cluster\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cluster", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <div name=\"in_cluster\" id=\"in_cluster\">\n          </div>\n        </div>\n      </div>\n    </div>\n  </fieldset>\n  <fieldset>\n    <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Allowed operations", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n    <div class=\"row\">\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"right_delete\" name=\"right_delete\" class=\"right_cb\" value=\"USE\">\n        <label for=\"right_delete\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Use", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"right_use\" name=\"right_use\" class=\"right_cb\" value=\"MANAGE\">\n        <label for=\"right_use\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Manage", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"right_manage\" name=\"right_manage\" class=\"right_cb\" value=\"ADMIN\">\n        <label for=\"right_manage\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Administrate", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n      <div class=\"large-3 medium-6 columns\">\n        <input type=\"checkbox\" id=\"right_create\" name=\"right_create\" class=\"right_cb\" value=\"CREATE\">\n        <label for=\"right_create\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Create", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n    </div>\n  </fieldset>\n  <br>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <label for=\"acl_preview\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ACL String preview", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n      <input type=\"text\" name=\"acl_preview\" id=\"acl_preview\"></input>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/acls-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/acls-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createAclForm';
});
define('tabs/acls-tab/tabId',['require'],function(require){
  return 'acls-tab';
});
define('tabs/acls-tab/form-panels/create',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','utils/notifier','utils/resource-select','hbs!./create/wizard','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var Notifier = require('utils/notifier');
  var ResourceSelect = require('utils/resource-select');

  /*
    TEMPLATES
   */

  var TemplateWizardHTML = require('hbs!./create/wizard');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create ACL"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      }
    };

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    return TemplateWizardHTML({
      'formPanelId': this.formPanelId
    });
  }

  function _setup(context) {
    //Default selected options
    $('#applies_all',context).attr('checked','checked');
    $('.applies_to_user',context).hide();
    $('.applies_to_group',context).hide();

    $('#res_subgroup_all',context).attr('checked','checked');
    $('.res_id',context).hide();
    $('.belonging_to',context).hide();
    $('.in_cluster',context).hide();

    //Applies to subset radio buttons
    $('.applies',context).click(function(){
      var value = $(this).val();
      switch (value) {
      case "*":
        $('.applies_to_user',context).hide();
        $('.applies_to_group',context).hide();
        break;
      case "applies_to_user":
        $('.applies_to_user',context).show();
        $('.applies_to_group',context).hide();
        break;
      case "applies_to_group":
        $('.applies_to_user',context).hide();
        $('.applies_to_group',context).show();
        break;
      }
    });

    //Resource subset radio buttons
    $('.res_subgroup',context).click(function(){
      var value = $(this).val();
      switch (value) {
      case "*":
        $('.res_id',context).hide();
        $('.belonging_to',context).hide();
        $('.in_cluster',context).hide();
        break;
      case "res_id":
        $('.res_id',context).show();
        $('.belonging_to').hide();
        $('.in_cluster',context).hide();
        break;
      case "belonging_to":
        $('.res_id',context).hide();
        $('.belonging_to',context).show();
        $('.in_cluster',context).hide();
        break;
      case "in_cluster":
        $('.res_id',context).hide();
        $('.belonging_to',context).hide();
        $('.in_cluster',context).show();
        break;
      }
    });

    //trigger ACL string preview on keyup
    $('input#res_id',context).keyup(function(){
      $(this).trigger("change");
    });

    //update the rule preview every time some field changes
    context.off('change', 'input,select');
    context.on('change', 'input,select', function(){
      var user="";
      var mode = $('.applies:checked',context).val();
      switch (mode) {
        case "*":
          user="*";
          break;
        case "applies_to_user":
          user="#"+$('div#applies_to_user .resource_list_select',context).val();
          break;
        case "applies_to_group":
          user="@"+$('div#applies_to_group .resource_list_select',context).val();
          break;
      }

      var resources = "";
      $('.resource_cb:checked',context).each(function(){
        resources+=$(this).val()+'+';
      });

      if (resources.length) {
        resources = resources.substring(0,resources.length-1);
      }

      var belonging="";
      var mode = $('.res_subgroup:checked',context).val();
      switch (mode) {
        case "*":
          belonging="*";
          break;
        case "res_id":
          belonging="#"+$('#res_id',context).val();
          break;
        case "belonging_to":
          belonging="@"+$('div#belonging_to .resource_list_select',context).val();
          break;
        case "in_cluster":
          belonging="%"+$('#in_cluster .resource_list_select',context).val();
          break;
      }

      var rights = "";
      $('.right_cb:checked',context).each(function(){
        rights+=$(this).val()+'+';
      });
      if (rights.length) {
        rights = rights.substring(0,rights.length-1);
      }

      var zone = $('#zones_applies .resource_list_select',context).val();

      if (zone != "*"){
        zone = '#'+zone;
      }

      var acl_string = user + ' ' + resources + '/' + belonging + ' ' +
                       rights + ' ' + zone;
      $('#acl_preview',context).val(acl_string);
    });
  }

  function _submitWizard(context) {
    var mode = $('.applies:checked',context).val();
    switch (mode) {
      case "applies_to_user":
        var l=$('#applies_to_user .resource_list_select',context).val().length;
        if (!l){
          Sunstone.hideFormPanelLoading(this.tabId);
          Notifier.notifyError(Locale.tr("Please select a user to whom the acl applies"));
          return false;
        }
        break;
      case "applies_to_group":
        var l=$('#applies_to_group .resource_list_select',context).val().length;
        if (!l){
          Sunstone.hideFormPanelLoading(this.tabId);
          Notifier.notifyError(Locale.tr("Please select a group to whom the acl applies"));
          return false;
        }
        break;
    }

    var resources = $('.resource_cb:checked',context).length;
    if (!resources) {
      Sunstone.hideFormPanelLoading(this.tabId);
      Notifier.notifyError(Locale.tr("Please select at least one resource"));
      return false;
    }

    var mode = $('.res_subgroup:checked',context).val();
    switch (mode) {
      case "res_id":
        var l=$('#res_id',context).val().length;
        if (!l){
          Sunstone.hideFormPanelLoading(this.tabId);
          Notifier.notifyError(Locale.tr("Please provide a resource ID for the resource(s) in this rule"));
          return false;
        }
        break;
      case "belonging_to":
        var l=$('#belonging_to .resource_list_select',context).val().length;
        if (!l){
          Sunstone.hideFormPanelLoading(this.tabId);
          Notifier.notifyError("Please select a group to which the selected resources belong to");
          return false;
        }
        break;
      case "in_cluster":
        var l=$('#in_cluster .resource_list_select',context).val().length;
        if (!l){
          Sunstone.hideFormPanelLoading(this.tabId);
          Notifier.notifyError("Please select a cluster to which the selected resources belong to");
          return false;
        }
        break;
    }

    var rights = $('.right_cb:checked',context).length;
    if (!rights) {
      Sunstone.hideFormPanelLoading(this.tabId);
      Notifier.notifyError("Please select at least one operation");
      return false;
    }

    var acl_string = $('#acl_preview',context).val();

    var acl_json = { "acl" : acl_string };
    Sunstone.runAction("Acl.create",acl_json);
    return false;
  }

  function _onShow(context) {
    ResourceSelect.insert('div#applies_to_user', context, "User", null, true);
    ResourceSelect.insert('div#applies_to_group', context, "Group", null, true);

    ResourceSelect.insert('div#belonging_to', context, "Group", null, true);
    ResourceSelect.insert('#in_cluster',context, "Cluster", null, true);

    // TODO BUG: doesn't work if the cluster.list callback is not finished yet
    // Delete cluster -1 option
    $('#in_cluster select option[value="-1"]',context).remove();

    ResourceSelect.insert('div#zones_applies', context, "Zone", "*", false,
                        '<option value="*">'+Locale.tr("All")+'</option>');

  }
});

define('tabs/settings-tab',['require','utils/locale','opennebula/user','sunstone','./users-tab/actions','./settings-tab/tabId','tabs/users-tab/tabId','tabs/users-tab/dialogs/password','tabs/users-tab/panels/info','tabs/settings-tab/panels/user-config','tabs/users-tab/panels/quotas','tabs/settings-tab/panels/group-quotas','tabs/users-tab/panels/accounting','tabs/users-tab/panels/showback','./acls-tab/form-panels/create'],function(require) {
  var Locale = require('utils/locale');
  var OpenNebulaUser = require('opennebula/user');
  var Sunstone = require('sunstone');
  var _actions = require('./users-tab/actions');

  var TAB_ID = require('./settings-tab/tabId');
  var USERS_TAB_ID = require('tabs/users-tab/tabId');

  _actions["Settings.refresh"] = {
    type: "custom",
    call: _onShow
  };

  var _dialogs = [
    require('tabs/users-tab/dialogs/password')
  ];

  var _panels = [
    require('tabs/users-tab/panels/info'),
    require('tabs/settings-tab/panels/user-config'),
    require('tabs/users-tab/panels/quotas'),
    require('tabs/settings-tab/panels/group-quotas'),
    require('tabs/users-tab/panels/accounting'),
    require('tabs/users-tab/panels/showback')
  ];

  var _formPanels = [
    require('./acls-tab/form-panels/create')
  ];

  var Tab = {
    tabId: TAB_ID,
    title: '<i class="fa fa-lg fa-fw fa-cog"></i>&emsp;' + Locale.tr("Settings"),
    listHeader: '<i class="fa fa-cog"></i>&emsp;' + Locale.tr("Settings"),
    resource: 'Settings',
    actions: _actions,
    content: '<span class="fa-stack fa-2x" style="color: #dfdfdf">' +
      '<i class="fa fa-cloud fa-stack-2x"></i>' +
      '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>' +
    '</span>',
    dialogs: _dialogs,
    panels: _panels,
  };

  return Tab;

  function _onShow() {
    OpenNebulaUser.show({
      data : {
        id: -1
      },
      success: function(request, user_json) {
        Sunstone.insertPanels(TAB_ID, user_json, TAB_ID, $(".right-list", $("#" + TAB_ID)))
      }
    });
  }
});


/* START_TEMPLATE */
define('hbs!tabs/dashboard-tab/html',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, lambda=this.lambda, buffer = "    <div class=\"small-12 large-12 columns\">\n      ";
  stack1 = lambda(depth0, depth0);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </div>\n";
},"3":function(depth0,helpers,partials,data) {
  var stack1, lambda=this.lambda, buffer = "    <div class=\"small-4 large-4 columns\">\n      ";
  stack1 = lambda(depth0, depth0);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </div>\n";
},"5":function(depth0,helpers,partials,data) {
  var stack1, lambda=this.lambda, buffer = "    <div class=\"small-6 large-6 columns\">\n      ";
  stack1 = lambda(depth0, depth0);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, buffer = "<div>\n  <div id=\"one_per_row\">\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.onePerRow : depth0), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "  </div>\n  <div id=\"three_per_row\" class=\"row\">\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.threePerRow : depth0), {"name":"each","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "  </div>\n  <div id=\"two_per_row\" class=\"row\">\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.twoPerRow : depth0), {"name":"each","hash":{},"fn":this.program(5, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "  </div>\n  <div id=\"one_footer\">\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.oneFooter : depth0), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/dashboard-tab/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vms-tab/tabId',['require'],function(require) {
  return "vms-tab";
})
;
define('tabs/hosts-tab/tabId',['require'],function(require){
  return 'hosts-tab';
});
define('tabs/images-tab/tabId',['require'],function(require){
  return 'images-tab';
});
define('tabs/vnets-tab/tabId',['require'],function(require){
  return 'vnets-tab';
});
define('tabs/vms-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createVMForm';
});
define('tabs/hosts-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createHostForm';
});
define('tabs/images-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createImageForm';
});
define('tabs/vnets-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createVNetForm';
});

/* START_TEMPLATE */
define('hbs!tabs/dashboard-tab/storage',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"row totals-info dashboard-widget-footer\">\n  <div class=\"small-12 large-12 columns text-center\">\n    <h2 class=\"subheader\">\n      <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IMAGES", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      <br>\n      <span class=\"total_images subheader\"></span>\n      <small><i class=\"fa fa-fw fa-upload\"></i> </small>\n    </h2>\n  </div>\n  <div class=\"small-12 large-12 columns text-center\">\n    <h5 class=\"subheader\">\n      <span class=\"size_images subheader\"></span>\n      <br>\n      <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "USED", {"name":"tr","hash":{},"data":data})))
    + "</small>\n    </h5>\n  </div>\n    <div class=\"small-12 large-12 columns text-center\">\n      <br>\n      <a class=\"button secondary radius tiny show_images_tab\"> <i class=\"fa fa-lg fa-list fa-fw\"></i>\n      </a>\n      <a class=\"button secondary  radius tiny show_create_image\">\n        <i class=\"fa fa-lg fa-plus fa-fw\"></i><i class=\"fa fa-fw fa-lg fa-upload\"></i>\n      </a>\n    </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/dashboard-tab/storage', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/dashboard-tab/empty-graph',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<span class=\"fa-stack fa-2x\" style=\"color: #dfdfdf\"> <i class=\"fa fa-cloud fa-stack-2x\"></i>\n</span>\n<br>\n<span style=\"color: #cfcfcf\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "There is no information available", {"name":"tr","hash":{},"data":data})))
    + "</span>";
},"useData":true});
Handlebars.registerPartial('tabs/dashboard-tab/empty-graph', t);
Handlebars.registerPartial('./empty-graph', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/dashboard-tab/users',['hbs','hbs/handlebars', 'hbs!tabs/dashboard-tab/empty-graph','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row totals-info\">\n  <div class=\"large-3 medium-6 small-12 columns\">\n    <br>\n    <div class=\"small-12 large-12 columns text-center\">\n      <h2 class=\"subheader\">\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "USERS", {"name":"tr","hash":{},"data":data})))
    + "</small>\n        <br>\n        <span class=\"subheader total_users\">-</span>\n        <small><i class=\"fa fa-fw fa-user\"></i> </small>\n      </h2>\n    </div>\n    <div class=\"small-12 large-12 columns text-center\" style=\"margin-top: 20px\">\n      <a class=\"button secondary radius tiny show_users_tab\"> <i class=\"fa fa-lg fa-list fa-fw\"></i></a>\n      <a class=\"button secondary radius tiny show_create_user\">\n        <i class=\"fa fa-lg fa-plus fa-fw\"></i><i class=\"fa fa-fw fa-lg fa-user\"></i></a>\n    </div>\n  </div>\n  <div class=\"large-9 medium-6 small-12 columns text-center\" id=\"dashboard_vdc_user_accounting\">\n    <input style=\"display:none;\" value=\"user\" id=\"acct_group_by\"/>\n    <div class=\"small-12 large-6 columns\">\n      <h4 class=\"subheader\" style=\"margin-bottom: 0px\">\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU hours", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h4>\n      <div class=\"large-12 columns centered graph text-center\" id=\"acct_cpu_graph\" style=\"height: 100px;\">";
  stack1 = this.invokePartial(partials['./empty-graph'], '', './empty-graph', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n    </div>\n    <div class=\"small-12 large-6 columns\">\n      <h4 class=\"subheader\" style=\"margin-bottom: 0px\">\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Memory GB hours", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h4>\n      <div class=\"large-12 columns centered graph text-center\" id=\"acct_mem_graph\" style=\"height: 100px;\">";
  stack1 = this.invokePartial(partials['./empty-graph'], '', './empty-graph', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n    </div>\n    <div class=\"small-12 large-6 left columns\">\n      <h4 class=\"subheader\" style=\"margin-bottom: 0px\">\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk MB hours", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h4>\n      <div class=\"large-12 columns centered graph text-center\" id=\"acct_disk_graph\" style=\"height: 100px;\">";
  stack1 = this.invokePartial(partials['./empty-graph'], '', './empty-graph', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n    </div>\n  </div>\n</div>\n<div class=\"row dashboard-widget-footer\">\n  <div class=\"small-3 large-3 columns text-center\"></div>\n</div>\n";
},"usePartial":true,"useData":true});
Handlebars.registerPartial('tabs/dashboard-tab/users', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/dashboard-tab/network',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"row totals-info dashboard-widget-footer\">\n  <div class=\"small-12 large-12 columns text-center\">\n    <h2 class=\"subheader\">\n      <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VNETs", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      <br>\n      <span class=\"total_vnets subheader\"></span>\n      <small><i class=\"fa fa-fw fa-sitemap\"></i> </small>\n    </h2>\n  </div>\n  <div class=\"small-12 large-12 columns text-center\">\n    <h5 class=\"subheader\">\n      <span class=\"addresses_vnets subheader\"></span>\n      <br>\n      <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "USED IPs", {"name":"tr","hash":{},"data":data})))
    + "</small>\n    </h5>\n  </div>\n    <div class=\"small-12 large-12 columns text-center\">\n      <br>\n      <a class=\"button secondary radius tiny show_vnets_tab\"> <i class=\"fa fa-lg fa-list fa-fw\"></i>\n      </a>\n      <a class=\"button secondary  radius tiny show_create_vnet\">\n        <i class=\"fa fa-lg fa-plus fa-fw\"></i><i class=\"fa fa-fw fa-lg fa-sitemap\"></i>\n      </a>\n    </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/dashboard-tab/network', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/dashboard-tab/hosts',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"row  totals-info\">\n  <div class=\"small-12 medium-6 large-3 columns centered\">\n    <div class=\"small-12 large-12 columns text-center\">\n      <h2 class=\"subheader\">\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "HOSTs", {"name":"tr","hash":{},"data":data})))
    + "</small>\n        <br>\n        <span class=\"total_hosts subheader\"/> \n        <small><i class=\"fa fa-fw fa-lg fa-hdd-o\"></i></small>\n      </h2>\n    </div>\n    <div class=\"small-4 large-4 columns text-center\">\n      <h5 class=\"subheader\">\n        <span class=\"on_hosts subheader success-color\"/>\n        <br>\n        <small class=\"success-color\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ON", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h5>\n    </div>\n    <div class=\"small-4 large-4 columns text-center\">\n      <h5 class=\"subheader\">\n        <span class=\"off_hosts subheader\"/>\n        <br>\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "OFF", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h5>\n    </div>\n    <div class=\"small-4 large-4 columns text-center\">\n      <h5 class=\"subheader\">\n        <span class=\"error_hosts subheader alert-color\"/>\n        <br>\n        <small class=\"alert-color\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ERROR", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h5>\n    </div>\n    <div class=\"small-12 large-12 columns text-center\">\n      <br>\n      <a class=\"button secondary radius tiny show_hosts_tab\"> <i class=\"fa fa-lg fa-list fa-fw\"></i>\n      </a>\n      <a class=\"button secondary  radius tiny show_create_host\">\n        <i class=\"fa fa-lg fa-plus fa-fw\"></i><i class=\"fa fa-fw fa-lg fa-hdd-o\"></i>\n      </a>\n    </div>\n  </div>\n  <div class=\"small-12 medium-6 large-9 columns\">\n    <div class=\"row\">\n      <div class=\"small-12 large-6 columns\">\n        <div id=\"dashboard_host_allocated_cpu\" style=\"padding: 0px 40px\"></div>\n        <br>\n        <div id=\"dashboard_host_real_cpu\" style=\"padding: 0px 40px\"></div>\n      </div>\n      <div class=\"small-12 large-6 columns\">\n        <div id=\"dashboard_host_allocated_mem\" style=\"padding: 0px 40px\"></div>\n        <br>\n        <div id=\"dashboard_host_real_mem\" style=\"padding: 0px 40px\"></div>\n      </div>\n    </div>\n  </div>\n</div>\n<div class=\"row dashboard-widget-footer\">\n  <div class=\"small-3 large-3 columns text-center\"></div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/dashboard-tab/hosts', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/dashboard-tab/vms',['hbs','hbs/handlebars', 'hbs!tabs/dashboard-tab/empty-graph','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row totals-info\">\n  <div class=\"small-12 medium-6 large-3 columns\">\n    <br>\n    <div class=\"small-12 large-12 columns text-center\">\n      <h2 class=\"subheader\">\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMs", {"name":"tr","hash":{},"data":data})))
    + "</small>\n        <br>\n        <span class=\"subheader total_vms\"></span>\n        <small><i class=\"fa fa-fw fa-th\"></i></small>\n      </h2>\n    </div>\n    <div class=\"small-4 large-4 columns text-center\">\n      <h5 class=\"subheader\">\n        <span class=\"subheader active_vms success-color\"></span>\n        <br>\n        <small class=\"success-color\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ACTIVE", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h5>\n    </div>\n    <div class=\"small-4 large-4 columns text-center\">\n      <h5 class=\"subheader\">\n        <span class=\"subheader pending_vms\"></span>\n        <br>\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PENDING", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h5>\n    </div>\n    <div class=\"small-4 large-4 columns text-center\">\n      <h5 class=\"subheader\">\n        <span class=\"subheader failed_vms alert-color\"></span>\n        <br>\n        <small class=\"alert-color\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "FAILED", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h5>\n    </div>\n    <div class=\"small-12 large-12 columns text-center\" style=\"margin-top: 20px\">\n      <a class=\"button secondary radius tiny show_vms_tab\"> <i class=\"fa fa-lg fa-list fa-fw\"></i>\n      </a>\n      <a class=\"button secondary radius tiny show_create_vm\">\n        <i class=\"fa fa-lg fa-plus fa-fw\"></i><i class=\"fa fa-fw fa-lg fa-th\"></i></a>\n      <br>\n    </div>\n  </div>\n  <div class=\"large-9 medium-6 small-12 columns text-center\" id=\"dashboard_vm_accounting\">\n    <input style=\"display:none;\" value=\"vm\" id=\"acct_group_by\"/>\n    <div class=\"small-12 large-6 columns\">\n      <h4 class=\"subheader\" style=\"margin-bottom: 0px\">\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU hours", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h4>\n      <div class=\"large-12 columns centered graph text-center\" id=\"acct_cpu_graph\" style=\"height: 100px;\">";
  stack1 = this.invokePartial(partials['./empty-graph'], '', './empty-graph', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n    </div>\n    <div class=\"small-12 large-6 columns\">\n      <h4 class=\"subheader\" style=\"margin-bottom: 0px\">\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Memory GB hours", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h4>\n      <div class=\"large-12 columns centered graph text-center\" id=\"acct_mem_graph\" style=\"height: 100px;\">";
  stack1 = this.invokePartial(partials['./empty-graph'], '', './empty-graph', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n    </div>\n    <div class=\"small-12 large-6 left columns\">\n      <h4 class=\"subheader\" style=\"margin-bottom: 0px\">\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk MB hours", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h4>\n      <div class=\"large-12 columns centered graph text-center\" id=\"acct_disk_graph\" style=\"height: 100px;\">";
  stack1 = this.invokePartial(partials['./empty-graph'], '', './empty-graph', depth0, undefined, helpers, partials, data);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n    </div>\n  </div>\n</div>\n<div class=\"row dashboard-widget-footer\">\n  <div class=\"small-3 large-3 columns text-center\"></div>\n</div>\n";
},"usePartial":true,"useData":true});
Handlebars.registerPartial('tabs/dashboard-tab/vms', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/dashboard-tab',['require','utils/locale','sunstone-config','sunstone','utils/notifier','opennebula/vm','utils/accounting','hbs!./dashboard-tab/html','tabs/vms-tab/tabId','tabs/hosts-tab/tabId','tabs/users-tab/tabId','tabs/images-tab/tabId','tabs/vnets-tab/tabId','tabs/vms-tab/form-panels/create/formPanelId','tabs/hosts-tab/form-panels/create/formPanelId','tabs/users-tab/form-panels/create/formPanelId','tabs/images-tab/form-panels/create/formPanelId','tabs/vnets-tab/form-panels/create/formPanelId','hbs!./dashboard-tab/storage','hbs!./dashboard-tab/users','hbs!./dashboard-tab/network','hbs!./dashboard-tab/hosts','hbs!./dashboard-tab/vms','./dashboard-tab/tabId'],function(require) {
  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var OpenNebulaVM = require('opennebula/vm');
  var Accounting = require('utils/accounting');

  var TemplateDashboard = require('hbs!./dashboard-tab/html');

  var VMS_TAB_ID = require('tabs/vms-tab/tabId');
  var HOSTS_TAB_ID = require('tabs/hosts-tab/tabId');
  var USERS_TAB_ID = require('tabs/users-tab/tabId');
  var IMAGES_TAB_ID = require('tabs/images-tab/tabId');
  var VNETS_TAB_ID = require('tabs/vnets-tab/tabId');

  var VMS_CREATE_FORM_PANEL_ID = require('tabs/vms-tab/form-panels/create/formPanelId');
  var HOSTS_CREATE_FORM_PANEL_ID = require('tabs/hosts-tab/form-panels/create/formPanelId');
  var USERS_CREATE_FORM_PANEL_ID = require('tabs/users-tab/form-panels/create/formPanelId');
  var IMAGES_CREATE_FORM_PANEL_ID = require('tabs/images-tab/form-panels/create/formPanelId');
  var VNETS_CREATE_FORM_PANEL_ID = require('tabs/vnets-tab/form-panels/create/formPanelId');

  var _initialized = false;
  var _activeWidgets = [];
  var _widgets = {
    'storage': {
      'html': require('hbs!./dashboard-tab/storage'),
      'onShow': function() {
        Sunstone.runAction("Image.list");
      }
    },
    'users': {
      'html': require('hbs!./dashboard-tab/users'),
      'onShow': function() {
        Sunstone.runAction("User.list");
        Sunstone.runAction("Group.list");

        var end_time = -1; // today
        var start_time =  Math.floor(new Date().getTime() / 1000);
        start_time = start_time - 604800; // 604800 = 7 days = 7*24*60*60

        var options = {
          "start_time": start_time,
          "end_time": end_time
        }

        var no_table = true;
        $("#acct_cpu_graph, #acct_mem_graph", "#dashboard_vdc_user_accounting").html('<span  id="provision_dashboard_total" style="font-size:80px">'+
          '<i class="fa fa-spinner fa-spin"></i>'+
        '</span>')
        OpenNebulaVM.accounting({
          success: function(req, response) {
            Accounting.fillAccounting($("#dashboard_vdc_user_accounting"), req, response, no_table);
          },
          error: Notifier.onError,
          data: options
        });
      }
    },
    'network': {
      'html': require('hbs!./dashboard-tab/network'),
      'onShow': function() {
        Sunstone.runAction("Network.list");
      }
    },
    'hosts': {
      'html': require('hbs!./dashboard-tab/hosts'),
      'onShow': function() {
        Sunstone.runAction("Host.list");
      }
    },
    'vms': {
      'html': require('hbs!./dashboard-tab/vms'),
      'onShow': function() {
        Sunstone.runAction("VM.list");

        var end_time = -1; // today
        var start_time =  Math.floor(new Date().getTime() / 1000);
        start_time = start_time - 604800; // 604800 = 7 days = 7*24*60*60

        var options = {
          "start_time": start_time,
          "end_time": end_time
        }

        var no_table = true;

        $("#acct_cpu_graph, #acct_mem_graph", "#dashboard_vm_accounting").html('<span  id="provision_dashboard_total" style="font-size:80px">'+
          '<i class="fa fa-spinner fa-spin"></i>'+
        '</span>')
        OpenNebulaVM.accounting({
          success: function(req, response) {
            Accounting.fillAccounting($("#dashboard_vm_accounting"), req, response, no_table);
          },
          error: Notifier.onError,
          data: options
        });
      }
    }
  }

  var _buttons = {
    "Dashboard.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    }
  };

  var _actions = {
    "Dashboard.refresh" : {
      type: "custom",
      call: _onShow
    },
  }

  var TAB_ID = require('./dashboard-tab/tabId');

  var Tab = {
    tabId: TAB_ID,
    resource: 'Dashboard',
    title: '<i class="fa fa-lg fa-fw fa-tachometer"></i>&emsp;' + Locale.tr("Dashboard"),
    listHeader: '<i class="fa fa-lg fa-fw fa-tachometer"></i>&emsp;' + Locale.tr("Dashboard"),
    buttons: _buttons,
    actions: _actions,
    content: _html()
  };

  return Tab;

  function _html() {
    var widgetsTemplates = {
      'threePerRow': [],
      'twoPerRow': [],
      'onePerRow': [],
      'oneFooter': []
    }

    $.each(Config.dashboardWidgets('widgets_three_per_row'), function(id, widget) {
      _activeWidgets.push(widget);
      widgetsTemplates['threePerRow'].push(_widgets[widget].html());
    })

    $.each(Config.dashboardWidgets('widgets_two_per_row'), function(id, widget) {
      _activeWidgets.push(widget);
      widgetsTemplates['twoPerRow'].push(_widgets[widget].html());
    })

    $.each(Config.dashboardWidgets('widgets_one_per_row'), function(id, widget) {
      _activeWidgets.push(widget);
      widgetsTemplates['onePerRow'].push(_widgets[widget].html());
    })

    $.each(Config.dashboardWidgets('widgets_one_footer'), function(id, widget) {
      _activeWidgets.push(widget);
      widgetsTemplates['oneFooter'].push(_widgets[widget].html());
    });

    return TemplateDashboard(widgetsTemplates);
  }

  function _onShow() {
    if (!_initialized) {
      _setup();
    }

    $.each(_activeWidgets, function(id, widgetId) {
      if (_widgets[widgetId].onShow) {
        _widgets[widgetId].onShow();
      }
    });
  }

  function _setup() {
    _initialized = true;


    $(document).on("click", ".show_vms_tab", function(){
      Sunstone.showTab(VMS_TAB_ID);
      return false;
    })

    $(document).on("click", ".show_hosts_tab", function(){
      Sunstone.showTab(HOSTS_TAB_ID);
      return false;
    })

    $(document).on("click", ".show_users_tab", function(){
      Sunstone.showTab(USERS_TAB_ID);
      return false;
    })

    $(document).on("click", ".show_images_tab", function(){
      Sunstone.showTab(IMAGES_TAB_ID);
      return false;
    })

    $(document).on("click", ".show_vnets_tab", function(){
      Sunstone.showTab(VNETS_TAB_ID);
      return false;
    })

    $(document).on("click", ".show_create_vm", function(){
      Sunstone.showTab(VMS_TAB_ID);
      Sunstone.showFormPanel(VMS_TAB_ID, VMS_CREATE_FORM_PANEL_ID, "create");
      return false;
    })

    $(document).on("click", ".show_create_host", function(){
      Sunstone.showTab(HOSTS_TAB_ID);
      Sunstone.showFormPanel(HOSTS_TAB_ID, HOSTS_CREATE_FORM_PANEL_ID, "create");
      return false;
    })

    $(document).on("click", ".show_create_user", function(){
      Sunstone.showTab(USERS_TAB_ID);
      Sunstone.showFormPanel(USERS_TAB_ID, USERS_CREATE_FORM_PANEL_ID, "create");
      return false;
    })

    $(document).on("click", ".show_create_image", function(){
      Sunstone.showTab(IMAGES_TAB_ID);
      Sunstone.showFormPanel(IMAGES_TAB_ID, IMAGES_CREATE_FORM_PANEL_ID, "create");
      return false;
    })

    $(document).on("click", ".show_create_vnet", function(){
      Sunstone.showTab(VNETS_TAB_ID);
      Sunstone.showFormPanel(VNETS_TAB_ID, VNETS_CREATE_FORM_PANEL_ID, "create");
      return false;
    })
  }
});

define('tabs/system-tab',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');
  var TAB_ID = 'system-tab';

  var Tab = {
    tabId: TAB_ID,
    title: '<i class="fa fa-lg fa-fw fa-cogs"></i>&emsp;'+Locale.tr("System"),
    no_content: true
  }

  return Tab;
});

define('tabs/groups-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "Group.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "Group.create_dialog" : {
      type: "create_dialog",
      layout: "create"
    },
    "Group.update_dialog" : {
      type : "action",
      layout: "main",
      text : Locale.tr("Update")
    },
    "Group.quotas_dialog" : {
      type : "action",
      text : Locale.tr("Quotas"),
      layout: "main"
    },
    "Group.delete" : {
      type: "confirm",
      text: Locale.tr("Delete"),
      layout: "del"
    }
  };

  return Buttons;
});

define('tabs/groups-tab/tabId',['require'],function(require){
  return 'groups-tab';
});
define('tabs/groups-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','utils/quotas/quota-defaults','utils/quotas/quota-widgets','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');
  var QuotaDefaults = require('utils/quotas/quota-defaults');
  var QuotaWidgets = require('utils/quotas/quota-widgets');

  /*
    CONSTANTS
   */

  var RESOURCE = "Group";
  var XML_ROOT = "GROUP";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check",4,5,6] },
          {"sWidth": "35px", "aTargets": [0]},
          {"sWidth": "150px", "aTargets": [4,5,6] },
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    };

    this.columns = [
      Locale.tr("ID"),
      Locale.tr("Name"),
      Locale.tr("Users"),
      Locale.tr("VMs"),
      Locale.tr("Memory"),
      Locale.tr("CPU")
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 2,
      "select_resource": Locale.tr("Please select a Group from the list"),
      "you_selected": Locale.tr("You selected the following Group:"),
      "select_resource_multiple": Locale.tr("Please select one or more groups from the list"),
      "you_selected_multiple": Locale.tr("You selected the following groups:")
    };

    this.totalGroups = 0;

    TabDataTable.call(this);
  }

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;
  Table.prototype.preUpdateView = _preUpdateView;
  Table.prototype.postUpdateView = _postUpdateView;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    this.totalGroups++;

    var element = element_json[XML_ROOT];

    var users_str = "0";

    if (element.USERS.ID){
      if ($.isArray(element.USERS.ID)){
        users_str = element.USERS.ID.length;
      } else {
        users_str = "1";
      }
    }

    var vms    = '<span class="progress-text right" style="font-size: 12px">-</span>';
    var memory = '<span class="progress-text right" style="font-size: 12px">-</span>';
    var cpu    = '<span class="progress-text right" style="font-size: 12px">-</span>';

    var default_group_quotas = QuotaDefaults.getDefaultGroupQuotas();

    QuotaWidgets.initEmptyQuotas(element);

    if (!$.isEmptyObject(element.VM_QUOTA)){
      vms = QuotaWidgets.quotaBar(
        element.VM_QUOTA.VM.VMS_USED,
        element.VM_QUOTA.VM.VMS,
        default_group_quotas.VM_QUOTA.VM.VMS);

      memory = QuotaWidgets.quotaBarMB(
        element.VM_QUOTA.VM.MEMORY_USED,
        element.VM_QUOTA.VM.MEMORY,
        default_group_quotas.VM_QUOTA.VM.MEMORY);

      cpu = QuotaWidgets.quotaBarFloat(
        element.VM_QUOTA.VM.CPU_USED,
        element.VM_QUOTA.VM.CPU,
        default_group_quotas.VM_QUOTA.VM.CPU);
    }

    return [
      '<input class="check_item" type="checkbox" id="'+RESOURCE.toLowerCase()+'_' +
                           element.ID + '" name="selected_items" value="' +
                           element.ID + '"/>',
      element.ID,
      element.NAME,
      users_str,
      vms,
      memory,
      cpu
    ];
  }

  function _preUpdateView() {
    this.totalGroups = 0;
  }

  function _postUpdateView() {
    $(".total_groups").text(this.totalGroups);
  }
});

define('tabs/groups-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createGroupForm';
});
define('tabs/groups-tab/dialogs/quotas/dialogId',['require'],function(require){
  return 'groupQuotasDialog';
});
define('tabs/groups-tab/actions',['require','sunstone','utils/notifier','utils/locale','./datatable','opennebula/group','utils/common-actions','./tabId','./form-panels/create/formPanelId','./dialogs/quotas/dialogId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var DataTable = require('./datatable');
  var OpenNebulaResource = require('opennebula/group');
  var CommonActions = require('utils/common-actions');

  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');
  var QUOTAS_DIALOG_ID = require('./dialogs/quotas/dialogId');

  var RESOURCE = "Group";
  var XML_ROOT = "GROUP";

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "Group.create" : _commonActions.create(CREATE_DIALOG_ID),
    "Group.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID),
    "Group.list" : _commonActions.list(),
    "Group.show" : _commonActions.show(),
    "Group.refresh" : _commonActions.refresh(),
    "Group.delete" : _commonActions.del(),
    "Group.update" : _commonActions.update(),
    "Group.update_template" : _commonActions.updateTemplate(),
    "Group.update_dialog" : _commonActions.checkAndShowUpdate(),
    "Group.show_to_update" : _commonActions.showUpdate(CREATE_DIALOG_ID),

    "Group.fetch_quotas" : {
      type: "single",
      call: OpenNebulaResource.show,
      callback: function (request,response) {
        Sunstone.getDialog(QUOTAS_DIALOG_ID).setParams({element: response[XML_ROOT]});
        Sunstone.getDialog(QUOTAS_DIALOG_ID).reset();
        Sunstone.getDialog(QUOTAS_DIALOG_ID).show();
      },
      error: Notifier.onError
    },

    "Group.quotas_dialog" : {
      type: "custom",
      call: function() {
        var tab = $('#' + TAB_ID);
        if (Sunstone.rightInfoVisible(tab)) {
          $('a[href="#group_quotas_tab"]', tab).click();
          $('#edit_quotas_button', tab).click();
        } else {
          var sel_elems = Sunstone.getDataTable(TAB_ID).elements();
          //If only one group is selected we fecth the group's quotas
          if (sel_elems.length == 1){
            Sunstone.runAction(RESOURCE+'.fetch_quotas',sel_elems[0]);
          } else {
            // More than one, shows '0' usage
            Sunstone.getDialog(QUOTAS_DIALOG_ID).setParams({element: {}});
            Sunstone.getDialog(QUOTAS_DIALOG_ID).reset();
            Sunstone.getDialog(QUOTAS_DIALOG_ID).show();
          }
        }
      }
    },

    "Group.set_quota" : {
      type: "multiple",
      call: OpenNebulaResource.set_quota,
      elements: function() {
        return Sunstone.getDataTable(TAB_ID).elements();
      },
      callback: function(request) {
        Sunstone.getDialog(QUOTAS_DIALOG_ID).hide();

        Sunstone.runAction(RESOURCE+'.show',request.request.data[0]);
      },
      error: Notifier.onError
    },

    "Group.add_admin" : {
      type: "single",
      call : OpenNebulaResource.add_admin,
      callback : function (req) {
        Sunstone.runAction(RESOURCE+'.show',req.request.data[0][0]);
      },
      error: Notifier.onError
    },

    "Group.del_admin" : {
      type: "single",
      call : OpenNebulaResource.del_admin,
      callback : function (req) {
        Sunstone.runAction(RESOURCE+'.show',req.request.data[0][0]);
      },
      error: Notifier.onError
    }
  };

  return _actions;
});


/* START_TEMPLATE */
define('hbs!tabs/groups-tab/dialogs/quotas/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large max-height\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <h3 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update Quota", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n  </div>\n  <form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\">\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        ";
  stack1 = ((helper = (helper = helpers.quotasHTML || (depth0 != null ? depth0.quotasHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"quotasHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      </div>\n    </div>\n    <div class=\"form_buttons row\">\n      <button type=\"submit\" class=\"button radius right\" value=\"Group.set_quota\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Apply changes", {"name":"tr","hash":{},"data":data})))
    + "\n      </button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/groups-tab/dialogs/quotas/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/groups-tab/dialogs/quotas',['require','utils/dialogs/dialog','hbs!./quotas/html','utils/locale','utils/quotas/quota-defaults','utils/quotas/quota-widgets','./quotas/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./quotas/html');
  var Locale = require('utils/locale');
  var QuotaDefaults = require('utils/quotas/quota-defaults');
  var QuotaWidgets = require('utils/quotas/quota-widgets');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./quotas/dialogId');
  var TAB_ID = require('../tabId');
  var RESOURCE = "Group";
  var XML_ROOT = "GROUP";

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    this.element = undefined;

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setParams = _setParams;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId,
      'quotasHTML': QuotaWidgets.dialogHTML()
    });
  }

  function _setup(context) {
    var that = this;

    QuotaWidgets.setupQuotasDialog(context);

    return false;
  }

  /**
   * @param {object} params
   *        - params.element : group object, or empty object {}
   */
  function _setParams(params) {
    this.element = params.element;
  }

  function _onShow(context) {
    QuotaWidgets.populateQuotasDialog(
      this.element,
      QuotaDefaults.getDefaultQuotas(RESOURCE),
      context);

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/groups-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "        <thead>\n          <tr>\n            <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group Admins Views", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          </tr>\n        </thead>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.adminViews : depth0), {"name":"each","hash":{},"fn":this.program(2, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"2":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "          <tr>\n            <td class=\"key_td\">\n              "
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.description : depth0), {"name":"if","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "            </td>\n          </tr>\n";
},"3":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "                <span class=\"tip\">"
    + escapeExpression(((helper = (helper = helpers.description || (depth0 != null ? depth0.description : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"description","hash":{},"data":data}) : helper)))
    + "</span>\n";
},"5":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "        <thead>\n          <tr>\n            <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group Users Views", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          </tr>\n        </thead>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.userViews : depth0), {"name":"each","hash":{},"fn":this.program(2, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.NAME : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n      </tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.adminViews : depth0), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.userViews : depth0), {"name":"if","hash":{},"fn":this.program(5, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "    </table>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-9 columns\">\n    ";
  stack1 = ((helper = (helper = helpers.templateTableHTML || (depth0 != null ? depth0.templateTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templateTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/groups-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/groups-tab/utils/views',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var _view_types = {
    advanced : {
      name: 'Advanced Layout',
      description : Locale.tr("This layout exposes a complete view of the cloud, allowing administrators and advanced users to have full control of any physical or virtual resource of the cloud."),
      preview: "advanced_layout.png"
    },
    cloud : {
      name: 'Cloud Layout',
      description : Locale.tr("This layout exposes a simplified version of the cloud where group administrators and cloud end-users will be able to manage any virtual resource of the cloud, without taking care of the physical resources management."),
      preview: "cloud_layout.png"
    },
    vcenter : {
      name: 'vCenter Layout',
      description : Locale.tr("Set of views to present the valid operation against a vCenter infrastructure"),
      preview: "vcenter_layout.png"
    },
    other : {
      name: 'Other Layouts',
      description : '',
      preview: null
    }
  };

  var _views_info = {
    admin : {
      id: 'admin',
      name: "Admin",
      description: Locale.tr("This view provides full control of the cloud"),
      type: "advanced"
    },
    user : {
      id: 'user',
      name: "User",
      description: Locale.tr("In this view users will not be able to manage nor retrieve the hosts and clusters of the cloud. They will be able to see Datastores and Virtual Networks in order to use them when creating a new Image or Virtual Machine, but they will not be able to create new ones."),
      type: "advanced"
    },
    groupadmin : {
      id: 'groupadmin',
      name: "Group Admin",
      description: Locale.tr("This view provides control of all the resources belonging to a group, but with no access to resources outside that group, that is, restricted to the physical and virtual resources of the group. This view features the ability to create new users within the group as well as set and keep track of user quotas."),
      type: "cloud"
    },
    cloud : {
      id: 'cloud',
      name: "Cloud",
      description: Locale.tr("This is a simplified view mainly intended for user that just require a portal where they can provision new virtual machines easily from pre-defined Templates."),
      type: "cloud"
    },
    admin_vcenter : {
      id: 'admin_vcenter',
      name: "Admin vCenter",
      description: Locale.tr("View designed to present the valid operations against a vCenter infrastructure to a cloud administrator"),
      type: "vcenter"
    },
    groupadmin_vcenter : {
      id: 'groupadmin_vcenter',
      name: "Group Admin vCenter",
      description: Locale.tr("View designed to present the valid operations agaist a vCenter infrastructure to a group administrator"),
      type: "vcenter"
    },
    cloud_vcenter : {
      id: 'cloud_vcenter',
      name: "Cloud vCenter",
      description: Locale.tr("View designed to present the valid operations against a vCenter infrastructure to a cloud consumer"),
      type: "vcenter"
    }
  };

  return {
    'info': _views_info,
    'types': _view_types
  };

});
define('tabs/groups-tab/panels/info/panelId',['require'],function(require){
  return 'group_info_tab';
});
define('tabs/groups-tab/panels/info',['require','hbs!./info/html','utils/locale','../utils/views','utils/tips','utils/panel/template-table','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TemplateInfo = require('hbs!./info/html');
  var Locale = require('utils/locale');
  var Views = require('../utils/views');
  var Tips = require('utils/tips');

  /*
    TEMPLATES
   */

  var TemplateTable = require('utils/panel/template-table');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "Group";
  var XML_ROOT = "GROUP";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var adminViews = _processViews(
      this.element.TEMPLATE.GROUP_ADMIN_VIEWS,
      this.element.TEMPLATE.GROUP_ADMIN_DEFAULT_VIEW);

    var userViews = _processViews(
      this.element.TEMPLATE.SUNSTONE_VIEWS,
      this.element.TEMPLATE.DEFAULT_VIEW);

    // TODO: simplify interface?
    var strippedTemplate = $.extend({}, this.element.TEMPLATE);
    delete strippedTemplate["GROUP_ADMIN_VIEWS"];
    delete strippedTemplate["SUNSTONE_VIEWS"];
    delete strippedTemplate["GROUP_ADMIN_DEFAULT_VIEW"];
    delete strippedTemplate["DEFAULT_VIEW"];
    delete strippedTemplate["TABLE_DEFAULT_PAGE_LENGTH"];

    var templateTableHTML = TemplateTable.html(strippedTemplate, RESOURCE,
                                              Locale.tr("Attributes"));
    //====

    return TemplateInfo({
      'element': this.element,
      'adminViews': adminViews,
      'userViews': userViews,
      'templateTableHTML': templateTableHTML
    });
  }

  function _setup(context) {
    var that = this;

    $('.resource-info-header', '#' + TAB_ID).text(that.element.NAME);

    Tips.setup(context);

    // Template update
    // TODO: simplify interface?
    var hiddenKeys = [
      "GROUP_ADMIN_VIEWS",
      "SUNSTONE_VIEWS",
      "GROUP_ADMIN_DEFAULT_VIEW",
      "DEFAULT_VIEW",
      "TABLE_DEFAULT_PAGE_LENGTH"];

    var strippedTemplate = $.extend({}, this.element.TEMPLATE);

    $.each(hiddenKeys, function(i, key){
      delete strippedTemplate[key];
    });

    var hiddenValues = {};

    $.each(hiddenKeys, function(i, key){
      if (that.element.TEMPLATE[key] != undefined){
          hiddenValues[key] = that.element.TEMPLATE[key];
      }
    });

    TemplateTable.setup(strippedTemplate, RESOURCE, this.element.ID, context, hiddenValues);
    //===

    return false;
  }

  function _processViews(viewsStr, defaultView){
    var viewsArray = [];

    if (viewsStr){
      $.each(viewsStr.split(','), function(index, view){
        var viewElem;

        var knownView = Views.info[view];
        if (knownView){
          viewElem = {
            'name': knownView.name,
            'description': knownView.description
          };
        } else {
          viewElem = {
            'name': view,
            'description': ''
          };
        }

        if (view == defaultView){
          viewElem.name += ' (' + Locale.tr("default") + ')';
        }

        viewsArray.push(viewElem);
      });
    }

    return viewsArray;
  }
});

define('templates/helpers/isTabActionEnabled',['require','hbs/handlebars','sunstone-config'],function(require) {
  var Handlebars = require('hbs/handlebars');
  var Config = require('sunstone-config');

  var isTabActionEnabled = function(tabName, action, options) {
    if (Config.isTabActionEnabled(tabName, action)) {
      return options.fn(this);
    } else {
      return options.inverse(this);
    }
  };

  Handlebars.registerHelper('isTabActionEnabled', isTabActionEnabled);

  return isTabActionEnabled;
})
;

/* START_TEMPLATE */
define('hbs!tabs/groups-tab/panels/users/html',['hbs','hbs/handlebars','templates/helpers/isTabActionEnabled','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "  <div class=\"row\">\n    <div class=\"large-4 small-10 columns right\">\n      <span>\n        <button class=\"button secondary small radius\" id=\"edit_admins_button\" style=\"width: 100%;\">\n          <span class=\"fa fa-pencil-square-o\"></span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Edit administrators", {"name":"tr","hash":{},"data":data})))
    + "\n        </button>\n        <button class=\"button alert small radius\" id=\"cancel_admins_button\" style=\"display: none\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cancel", {"name":"tr","hash":{},"data":data})))
    + "\n        </button>\n        <button class=\"button success small radius\" id=\"submit_admins_button\" style=\"display: none\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Apply", {"name":"tr","hash":{},"data":data})))
    + "\n        </button>\n      </span>\n    </div>\n  </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, functionType="function", buffer = "";
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "groups-tab", "Group.edit_admins", {"name":"isTabActionEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "<div class=\"group_users_info_table\">\n  ";
  stack1 = ((helper = (helper = helpers.usersTableHTML || (depth0 != null ? depth0.usersTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"usersTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n</div>\n\n\n";
},"useData":true});
Handlebars.registerPartial('tabs/groups-tab/panels/users/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/groups-tab/panels/users/panelId',['require'],function(require){
  return 'group_users_tab';
});
define('tabs/groups-tab/panels/users',['require','utils/locale','sunstone-config','hbs!./users/html','tabs/users-tab/datatable','sunstone','../tabId','./users/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var TemplateHTML = require('hbs!./users/html');
  var UsersTable = require('tabs/users-tab/datatable');
  var Sunstone = require('sunstone');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./users/panelId');
  var RESOURCE = "Group";
  var XML_ROOT = "GROUP";

  var USERS_TABLE_ID = PANEL_ID + "UsersTable";
  var USERS_EDIT_TABLE_ID = PANEL_ID + "UsersTableEdit";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Users");
    this.icon = "fa-users";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {

    this.users = [];

    if (this.element.USERS.ID != undefined){
      this.users = this.element.USERS.ID;

      if (!$.isArray(this.users)){
        this.users = [this.users];
      }
    }

    this.admins = [];

    if (this.element.ADMINS.ID != undefined){
      this.admins = this.element.ADMINS.ID;

      if (!$.isArray(this.admins)){
        this.admins = [this.admins];
      }
    }

    var opts = {
      info: true,
      select: true,
      selectOptions: {
        read_only: true,
        fixed_ids: this.users,
        starred_ids: this.admins
      }
    };

    this.usersTable = new UsersTable(USERS_TABLE_ID, opts);

    return TemplateHTML({
      'usersTableHTML': this.usersTable.dataTableHTML
    });
  }

  function _setup(context) {
    var that = this;

    this.usersTable.initialize();
    this.usersTable.refreshResourceTableSelect();

    if (Config.isTabActionEnabled("groups-tab", "Group.edit_admins")) {
      context.off("click", "#edit_admins_button");
      context.on("click",  "#edit_admins_button", function() {
        $("#edit_admins_button", context).hide();
        $("#cancel_admins_button", context).show();
        $("#submit_admins_button", context).show();

        var opts = {
          info: false,
          select: true,
          selectOptions: {
            multiple_choice: true,
            fixed_ids: that.users,
            starred_ids: that.admins
          }
        };

        that.usersTableEdit = new UsersTable(USERS_EDIT_TABLE_ID, opts);

        $("div.group_users_info_table", context).html(
          that.usersTableEdit.dataTableHTML );

        that.usersTableEdit.initialize();
        that.usersTableEdit.selectResourceTableSelect({ ids : that.admins });

        return false;
      });

      context.off("click", "#cancel_admins_button");
      context.on("click",  "#cancel_admins_button", function() {
        Sunstone.runAction("Group.show", that.element.ID);
        return false;
      });

      context.off("click", "#submit_admins_button");
      context.on("click",  "#submit_admins_button", function() {
        // Add/delete admins

        var selected_admins_list =
            that.usersTableEdit.retrieveResourceTableSelect(context,
                                                      "group_users_edit_list");

        $.each(selected_admins_list, function(i,admin_id){
          if (that.admins.indexOf(admin_id) == -1){
            Sunstone.runAction("Group.add_admin",
                  that.element.ID, {admin_id : admin_id});
          }
        });

        $.each(that.admins, function(i,admin_id){
          if (selected_admins_list.indexOf(admin_id) == -1){
            Sunstone.runAction("Group.del_admin",
                  that.element.ID, {admin_id : admin_id});
          }
        });

        return false;
      });
    }
  }
});

define('tabs/groups-tab/panels/quotas/panelId',['require'],function(require){
  return 'group_quotas_tab';
});
define('tabs/groups-tab/panels/quotas',['require','utils/locale','sunstone-config','utils/quotas/quota-defaults','utils/quotas/quota-widgets','../tabId','./quotas/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var QuotaDefaults = require('utils/quotas/quota-defaults');
  var QuotaWidgets = require('utils/quotas/quota-widgets');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./quotas/panelId');
  var RESOURCE = "Group";
  var XML_ROOT = "GROUP";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Quotas");
    this.icon = "fa-align-left";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return QuotaWidgets.initQuotasPanel(
      this.element,
      QuotaDefaults.getDefaultQuotas(RESOURCE),
      Config.isTabActionEnabled(TAB_ID, RESOURCE+".quotas_dialog"));
  }

  function _setup(context) {
    QuotaWidgets.setupQuotasPanel(
      this.element,
      context,
      Config.isTabActionEnabled(TAB_ID, RESOURCE+".quotas_dialog"),
      RESOURCE);

    return false;
  }
});

define('tabs/groups-tab/panels/accounting/panelId',['require'],function(require){
  return 'group_accounting_tab';
});
define('tabs/groups-tab/panels/accounting',['require','utils/locale','utils/accounting','../tabId','./accounting/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Accounting = require('utils/accounting');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./accounting/panelId');
  var RESOURCE = "Group";
  var XML_ROOT = "GROUP";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Accounting");
    this.icon = "fa-bar-chart-o";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return Accounting.html();
  }

  function _setup(context) {
    Accounting.setup(
      context,
      { fixed_group: this.element.ID,
        init_group_by: "user"
      });
  }
});

define('tabs/groups-tab/panels/showback/panelId',['require'],function(require){
  return 'group_showback_tab';
});
define('tabs/groups-tab/panels/showback',['require','utils/locale','sunstone-config','utils/showback','../tabId','./showback/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var Showback = require('utils/showback');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./showback/panelId');
  var RESOURCE = "Group";
  var XML_ROOT = "GROUP";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Showback");
    this.icon = "fa-money";

    this.element = info[XML_ROOT];

    if (!Config.isFeatureEnabled("showback")) {
      throw "Showback is disabled in the configuration";
    }

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return Showback.html();
  }

  function _setup(context) {
    Showback.setup(
      context,
      { fixed_user: "",
        fixed_group: this.element.ID
      }
    );
  }
});


/* START_TEMPLATE */
define('hbs!tabs/groups-tab/form-panels/create/wizard',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "        <div class=\"row\">\n          <div class=\"large-7 columns\">\n            <h4>"
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.description : depth0), {"name":"if","hash":{},"fn":this.program(2, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "            </h4>\n            <table class=\"dataTable extended_table\">\n              <tr>\n                <th style='width: 60%;'></th>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group Users", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group Admins", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              </tr>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.views : depth0), {"name":"each","hash":{},"fn":this.program(4, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "            </table>\n          </div>\n          <div class=\"large-5 columns\" style=\"text-align: center\">\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.preview : depth0), {"name":"if","hash":{},"fn":this.program(7, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "          </div>\n        </div>\n        <br>\n";
},"2":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "              <span class=\"tip\">"
    + escapeExpression(((helper = (helper = helpers.description || (depth0 != null ? depth0.description : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"description","hash":{},"data":data}) : helper)))
    + "</span>\n";
},"4":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "              <tr>\n                <td>\n                  "
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + " "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "View", {"name":"tr","hash":{},"data":data})))
    + "\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.description : depth0), {"name":"if","hash":{},"fn":this.program(5, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "                </td>\n                <td>\n                  <input class=\"user_view_input\" type=\"checkbox\"\n                  id=\"group_view_"
    + escapeExpression(((helper = (helper = helpers.id || (depth0 != null ? depth0.id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"id","hash":{},"data":data}) : helper)))
    + "\"\n                  value=\""
    + escapeExpression(((helper = (helper = helpers.id || (depth0 != null ? depth0.id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"id","hash":{},"data":data}) : helper)))
    + "\"/>\n                </td>\n                <td>\n                  <input class=\"admin_view_input\" type=\"checkbox\"\n                  id=\"group_admin_view_"
    + escapeExpression(((helper = (helper = helpers.id || (depth0 != null ? depth0.id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"id","hash":{},"data":data}) : helper)))
    + "\"\n                  value=\""
    + escapeExpression(((helper = (helper = helpers.id || (depth0 != null ? depth0.id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"id","hash":{},"data":data}) : helper)))
    + "\"/>\n                </td>\n              </tr>\n";
},"5":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "                  <span class=\"tip\">"
    + escapeExpression(((helper = (helper = helpers.description || (depth0 != null ? depth0.description : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"description","hash":{},"data":data}) : helper)))
    + "</span>\n";
},"7":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "            <img src=\"images/"
    + escapeExpression(((helper = (helper = helpers.preview || (depth0 != null ? depth0.preview : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"preview","hash":{},"data":data}) : helper)))
    + "\" style=\"height: 200px;\">\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"columns medium-5\">\n      <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + ":\n        <input required type=\"text\" name=\"name\" id=\"name\" />\n      </label>\n    </div>\n    <div class=\"columns medium-7\">\n      <dl class=\"tabs right-info-tabs text-center right\" data-tab>\n       <dd class=\"active\"><a href=\"#resource_views\"><i class=\"fa fa-eye\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Views", {"name":"tr","hash":{},"data":data})))
    + "</a></dd>\n       <dd><a href=\"#administrators\"><i class=\"fa fa-upload\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Admin", {"name":"tr","hash":{},"data":data})))
    + "</a></dd>\n       <dd><a href=\"#resource_creation\"><i class=\"fa fa-folder-open\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Permissions", {"name":"tr","hash":{},"data":data})))
    + "</a></dd>\n     </dl>\n   </div>\n </div>\n <div class=\"tabs-content\">\n  <div id=\"resource_views\" class=\"content active\">\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <p class=\"subheader\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Allow users in this group to use the following Sunstone views", {"name":"tr","hash":{},"data":data})))
    + "\n          &emsp;<span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Views available to the group users. If the default is unset, the one set in sunstone-views.yaml will be used", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </p>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <div class=\"row\">\n          <div class=\"medium-6 columns\">\n            <label for=\"user_view_default\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Default Users View", {"name":"tr","hash":{},"data":data})))
    + "\n            </label>\n            <select id=\"user_view_default\">\n            </select>\n          </div>\n          <div class=\"medium-6 columns\">\n            <label for=\"admin_view_default\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Default Admin View", {"name":"tr","hash":{},"data":data})))
    + "\n            </label>\n            <select id=\"admin_view_default\">\n            </select>\n          </div>\n        </div>\n        <br>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.viewTypes : depth0), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "      </div>\n    </div>\n  </div>\n  <div id=\"administrators\" class=\" content\">\n    <div class=\"row\">\n      <div class=\"medium-6 columns\" id=\"admin_user_wrapper\">\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <label>\n              <input type=\"checkbox\" id=\"admin_user\" name=\"admin_user\" value=\"YES\" />\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Create an administrator user", {"name":"tr","hash":{},"data":data})))
    + "\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "You can create now an administrator user. More administrators can be added later.", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n          </div>\n        </div>\n        ";
  stack1 = ((helper = (helper = helpers.userCreationHTML || (depth0 != null ? depth0.userCreationHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"userCreationHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      </div>\n    </div>\n  </div>\n  <div id=\"resource_creation\" class=\"content\">\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <label>\n          <input type=\"checkbox\" id=\"shared_resources\" name=\"shared_resources\" value=\"YES\" />\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Allow users to view the VMs and Services of other users in the same group", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "An ACL Rule will be created to give users in this group access to all the resources in the same group.", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </label>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <p class=\"subheader\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Allow users in this group to create the following resources", {"name":"tr","hash":{},"data":data})))
    + "\n          &emsp;<span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "This will create new ACL Rules to define which virtual resources this group's users will be able to create.", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </p>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <table class=\"dataTable\" style=\"table-layout:fixed\">\n          <thead><tr>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMs", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VNets", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Security Groups", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Images", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Templates", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Documents", {"name":"tr","hash":{},"data":data})))
    + "<span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Documents are a special tool used for general purposes, mainly by OneFlow. If you want to enable users of this group to use service composition via OneFlow, let it checked.", {"name":"tr","hash":{},"data":data})))
    + "</span></th>\n          </tr></thead>\n          <tbody>\n            <tr>\n              <td><input type=\"checkbox\" id=\"group_res_vm\" name=\"group_res_vm\" class=\"resource_cb\" value=\"VM\"></input></td>\n              <td><input type=\"checkbox\" id=\"group_res_net\" name=\"group_res_net\" class=\"resource_cb\" value=\"NET\"></input></td>\n              <td><input type=\"checkbox\" id=\"group_res_sg\" name=\"group_res_sg\" class=\"resource_cb\" value=\"SECGROUP\"></input></td>\n              <td><input type=\"checkbox\" id=\"group_res_image\" name=\"group_res_image\" class=\"resource_cb\" value=\"IMAGE\"></input></td>\n              <td><input type=\"checkbox\" id=\"group_res_template\" name=\"group_res_template\" class=\"resource_cb\" value=\"TEMPLATE\"></input></td>\n              <td><input type=\"checkbox\" id=\"group_res_document\" name=\"group_res_document\" class=\"resource_cb\" value=\"DOCUMENT\"></input></td>\n              <td/>\n            </tr>\n          </tbody>\n        </table>\n      </div>\n    </div>\n  </div>\n</div>\n<div class=\"reveal-footer\">\n  <div class=\"row collapse\" id=\"default_vdc_warning\">\n    <div class=\"large-12 columns text-right\">\n      <span class=\"radius secondary label\"><i class=\"fa fa-warning\"/> "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "New Groups are automatically added to the default VDC", {"name":"tr","hash":{},"data":data})))
    + "</span>\n    </div>\n  </div>\n</div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/groups-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/groups-tab/form-panels/create',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','tabs/users-tab/utils/user-creation','utils/tips','tabs/groups-tab/utils/views','utils/template-utils','hbs!./create/wizard','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var UserCreation = require('tabs/users-tab/utils/user-creation');
  var Tips = require('utils/tips');
  var Views = require('tabs/groups-tab/utils/views');
  var TemplateUtils = require('utils/template-utils');

  /*
    TEMPLATES
   */

  var TemplateWizardHTML = require('hbs!./create/wizard');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create Group"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      },
      'update': {
        'title': Locale.tr("Update Group"),
        'buttonText': Locale.tr("Update"),
        'resetButton': false
      }
    };

    this.userCreation = new UserCreation(FORM_PANEL_ID);

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.fill = _fill;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    var filtered_views = {
      cloud : [],
      advanced : [],
      vcenter : [],
      other : []
    };

    var view_info;
    $.each(config['all_views'], function(index, view_id) {
      view_info = Views.info[view_id];
      if (view_info) {
        switch (view_info.type) {
          case 'advanced':
            filtered_views.advanced.push(view_info);
            break;
          case 'cloud':
            filtered_views.cloud.push(view_info);
            break;
          case 'vcenter':
            filtered_views.vcenter.push(view_info);
            break;
          default:
            filtered_views.other.push({
              id: view_id,
              name: view_id,
              description: null,
              type: "other"
            });
            break;
        }
      } else {
        filtered_views.other.push({
          id: view_id,
          name: view_id,
          description: null,
          type: "other"
        });
      }
    });

    var viewTypes = [];

    $.each(filtered_views, function(view_type, views){
      if (views.length > 0) {
        viewTypes.push(
          {
            'name': Views.types[view_type].name,
            'description': Views.types[view_type].description,
            'preview': Views.types[view_type].preview,
            'views': views
          });
      }
    });

    return TemplateWizardHTML({
      'formPanelId': this.formPanelId,
      'userCreationHTML': this.userCreation.html(),
      'viewTypes': viewTypes
    });
  }

  function _setup(context) {
    var that = this;

    context.foundation('tab', 'reflow');
    this.userCreation.setup( $("#admin_user_wrapper",context) );
    Tips.setup(context);

    $('input#name', context).change(function(){
      var val = $(this).val();

      that.userCreation.setName(context, val + "-admin");
    });

    $('input#admin_user', context).change(function(){
      if ($(this).prop('checked')) {
        that.userCreation.enable( $("#admin_user_wrapper",context) );
      } else {
        that.userCreation.disable( $("#admin_user_wrapper",context) );
      }
    });

    this.userCreation.disable( $("#admin_user_wrapper",context) );

    $.each($('[id^="group_res"]', context), function(){
      $(this).prop("checked", true);
    });

    $("#group_res_net", context).prop("checked", false);

    $(context).off("change", ".admin_view_input");
    $(context).on("change", ".admin_view_input", function(){
      _generateViewsSelect(context, "admin");
    });

    $(context).off("change", ".user_view_input");
    $(context).on("change", ".user_view_input", function(){
      _generateViewsSelect(context, "user");
    });

    if (this.action == "create") {
      $("input#group_view_cloud").attr('checked','checked').change();
      $("input#group_admin_view_groupadmin").attr('checked','checked').change();

      _generateViewsSelect(context, "admin", "groupadmin");
      _generateViewsSelect(context, "user", "cloud");
    }
  }

  function _submitWizard(context) {
    var views = [];
    $.each($('[id^="group_view"]:checked', context), function(){
      views.push($(this).val());
    });

    var admin_views = [];
    $.each($('[id^="group_admin_view"]:checked', context), function(){
      admin_views.push($(this).val());
    });

    var default_view = $('#user_view_default', context).val();
    var default_admin_view = $('#admin_view_default', context).val();

    if (this.action == "create") {
      var name = $('#name',context).val();

      var user_json = null;

      if ( $('#admin_user', context).prop('checked') ){
        user_json = this.userCreation.retrieve($("#admin_user_wrapper",context));
      }

      var group_json = {
        "group" : {
          "name" : name
        }
      };

      if (user_json){
        group_json["group"]["group_admin"] = user_json;
      }

      var resources = "";
      var separator = "";

      $.each($('[id^="group_res"]:checked', context), function(){
        resources += (separator + $(this).val());
        separator = "+";
      });

      group_json['group']['resources'] = resources;

      if ( $('#shared_resources', context).prop('checked') ){
        group_json['group']['shared_resources'] = "VM+DOCUMENT";
      }

      group_json['group']['views'] = views;

      if (default_view != undefined){
        group_json['group']['default_view'] = default_view;
      }

      group_json['group']['admin_views'] = admin_views;

      if (default_admin_view != undefined){
        group_json['group']['default_admin_view'] = default_admin_view;
      }

      Sunstone.runAction("Group.create",group_json);
      return false;
    } else if (this.action == "update") {
      var template_json = this.element.TEMPLATE;

      delete template_json["SUNSTONE_VIEWS"];
      delete template_json["DEFAULT_VIEW"];
      delete template_json["GROUP_ADMIN_VIEWS"];
      delete template_json["GROUP_ADMIN_DEFAULT_VIEW"];

      if (views.length != 0){
        template_json["SUNSTONE_VIEWS"] = views.join(",");
      }

      if (default_view != undefined){
        template_json["DEFAULT_VIEW"] = default_view;
      }

      if (admin_views.length != 0){
        template_json["GROUP_ADMIN_VIEWS"] = admin_views.join(",");
      }

      if (default_admin_view != undefined){
        template_json["GROUP_ADMIN_DEFAULT_VIEW"] = default_admin_view;
      }

      var template_str = TemplateUtils.templateToString(template_json);

      Sunstone.runAction("Group.update",this.resourceId, template_str);
      return false;
    }
  }

  function _onShow(context) {
  }

  function _fill(context, element) {
    var that = this;

    if (this.action != "update") {return;}
    this.resourceId = element.ID;
    this.element = element;

    // Disable parts of the wizard
    $("input#name", context).attr("disabled", "disabled");

    $("a[href='#administrators']", context).parents("dd").hide();
    $("a[href='#resource_creation']", context).parents("dd").hide();

    $("input#name", context).val(element.NAME);

    var views_str = "";

    $('input[id^="group_view"]', context).removeAttr('checked');

    if (element.TEMPLATE.SUNSTONE_VIEWS){
      views_str = element.TEMPLATE.SUNSTONE_VIEWS;

      var views = views_str.split(",");
      $.each(views, function(){
        $('input[id^="group_view"][value="'+this.trim()+'"]',
          context).attr('checked','checked').change();
      });
    }

    $('input[id^="group_default_view"]', context).removeAttr('checked');

    if (element.TEMPLATE.DEFAULT_VIEW){
      $('#user_view_default', context).val(element.TEMPLATE.DEFAULT_VIEW.trim()).change();
    } else {
      $('#user_view_default', context).val("").change();
    }

    $('input[id^="group_admin_view"]', context).removeAttr('checked');

    if (element.TEMPLATE.GROUP_ADMIN_VIEWS){
      views_str = element.TEMPLATE.GROUP_ADMIN_VIEWS;

      var views = views_str.split(",");
      $.each(views, function(){
        $('input[id^="group_admin_view"][value="'+this.trim()+'"]',
          context).attr('checked','checked').change();
      });
    }

    $('input[id^="group_default_admin_view"]', context).removeAttr('checked');

    if (element.TEMPLATE.GROUP_ADMIN_DEFAULT_VIEW){
      $('#admin_view_default', context).val(element.TEMPLATE.GROUP_ADMIN_DEFAULT_VIEW.trim()).change();
    } else {
      $('#admin_view_default', context).val("").change();
    }
  }

  function _generateViewsSelect(context, idPrefix, value) {
    var views = [];
    var old_value = value || $("#"+idPrefix+"_view_default", context).val();

    var html = '<option id="" name="" value=""></option>';

    $("."+idPrefix+"_view_input:checked", context).each(function(){
      var name = (Views.info[this.value] ? Views.info[this.value].name : this.value);

      html += '<option value="'+this.value+'">'+name+'</option>';
    });

    $("select#"+idPrefix+"_view_default", context).html(html);

    if (old_value) {
      $("#"+idPrefix+"_view_default", context).val(old_value).change();
    }
  }

});

define('tabs/groups-tab',['require','utils/locale','./groups-tab/buttons','./groups-tab/actions','./groups-tab/datatable','./groups-tab/tabId','./groups-tab/dialogs/quotas','./groups-tab/panels/info','./groups-tab/panels/users','./groups-tab/panels/quotas','./groups-tab/panels/accounting','./groups-tab/panels/showback','./groups-tab/form-panels/create'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./groups-tab/buttons');
  var Actions = require('./groups-tab/actions');
  var Table = require('./groups-tab/datatable');

  var TAB_ID = require('./groups-tab/tabId');
  var DATATABLE_ID = "dataTableGroups";

  var _dialogs = [
    require('./groups-tab/dialogs/quotas')
  ];

  var _panels = [
    require('./groups-tab/panels/info'),
    require('./groups-tab/panels/users'),
    require('./groups-tab/panels/quotas'),
    require('./groups-tab/panels/accounting'),
    require('./groups-tab/panels/showback')
  ];

  var _formPanels = [
    require('./groups-tab/form-panels/create')
  ];

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("Groups"),
    tabClass: "subTab",
    parentTab: "system-tab",
    listHeader: '<i class="fa fa-fw fa-users"></i>&emsp;'+Locale.tr("Groups"),
    infoHeader: '<i class="fa fa-fw fa-users"></i>&emsp;'+Locale.tr("Group"),
    subheader: '<span>\
        <span class="total_groups"/> <small>'+Locale.tr("TOTAL")+'</small>\
      </span>',
    resource: 'Group',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});

define('tabs/vdcs-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "Vdc.refresh" : {
        type: "action",
        layout: "refresh",
        alwaysActive: true
    },
    "Vdc.create_dialog" : {
        type: "create_dialog",
        layout: "create"
    },
    "Vdc.update_dialog" : {
        type: "action",
        layout: "main",
        text: Locale.tr("Update")
    },
    "Vdc.delete" : {
        type: "confirm",
        layout: "del",
        text: Locale.tr("Delete")
    }
  };

  return Buttons;
});

define('tabs/vdcs-tab/tabId',['require'],function(require){
  return 'vdcs-tab';
});
define('tabs/vdcs-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var Config = require('sunstone-config');
  var Locale = require('utils/locale');

  /*
    CONSTANTS
   */

  var RESOURCE = "Vdc";
  var XML_ROOT = "VDC";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          { "bSortable": false, "aTargets": ["check"] },
          { "sWidth": "35px", "aTargets": [0] },
          { "bVisible": true, "aTargets": Config.tabTableColumns(TAB_NAME)},
          { "bVisible": false, "aTargets": ['_all']}
      ]
    };

    this.columns = [
      Locale.tr("ID"),
      Locale.tr("Name"),
      Locale.tr("Groups"),
      Locale.tr("Clusters"),
      Locale.tr("Hosts"),
      Locale.tr("VNets"),
      Locale.tr("Datastores")
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 2,
      "select_resource": Locale.tr("Please select a VDC from the list"),
      "you_selected": Locale.tr("You selected the following VDC:"),
      "select_resource_multiple": Locale.tr("Please select one or more VDCs from the list"),
      "you_selected_multiple": Locale.tr("You selected the following VDCs:")
    };

    this.totalElements = 0;

    TabDataTable.call(this);
  }

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;
  Table.prototype.preUpdateView = _preUpdateView;
  Table.prototype.postUpdateView = _postUpdateView;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    this.totalElements++;

    var element = element_json[XML_ROOT];

    return [
      '<input class="check_item" type="checkbox" id="'+RESOURCE.toLowerCase()+'_' +
                           element.ID + '" name="selected_items" value="' +
                           element.ID + '"/>',
      element.ID,
      element.NAME,
      _lengthOf(element.GROUPS.ID),
      _lengthOf(element.CLUSTERS.CLUSTER),
      _lengthOf(element.HOSTS.HOST),
      _lengthOf(element.VNETS.VNET),
      _lengthOf(element.DATASTORES.DATASTORE)
    ];
  }

  function _lengthOf(ids){
    var l = 0;
    if ($.isArray(ids))
      l = ids.length;
    else if (!$.isEmptyObject(ids))
      l = 1;

    return l;
  }

  function _preUpdateView() {
    this.totalElements = 0;
  }

  function _postUpdateView() {
    $(".total_vdcs").text(this.totalElements);
  }
});

define('tabs/vdcs-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createVDCForm';
});
define('tabs/vdcs-tab/actions',['require','sunstone','utils/notifier','utils/locale','./datatable','opennebula/vdc','utils/common-actions','./tabId','./form-panels/create/formPanelId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var DataTable = require('./datatable');
  var OpenNebulaResource = require('opennebula/vdc');
  var CommonActions = require('utils/common-actions');

  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');

  var RESOURCE = "Vdc";
  var XML_ROOT = "VDC";

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "Vdc.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID),
    "Vdc.list" : _commonActions.list(),
    "Vdc.show" : _commonActions.show(),
    "Vdc.refresh" : _commonActions.refresh(),
    "Vdc.delete" : _commonActions.del(),
    "Vdc.update" : _commonActions.update(),
    "Vdc.update_template" : _commonActions.updateTemplate(),
    "Vdc.update_dialog" : _commonActions.checkAndShowUpdate(),
    "Vdc.show_to_update" : _commonActions.showUpdate(CREATE_DIALOG_ID),
    "Vdc.rename": _commonActions.singleAction('rename'),

    "Vdc.create" : {
      type: "create",
      call: OpenNebulaResource.create,
      callback: function(request, response) {
        Sunstone.resetFormPanel(TAB_ID, CREATE_DIALOG_ID);
        Sunstone.hideFormPanel(TAB_ID);

        var group_ids = request.request.data[0].group_ids;
        if(group_ids !=undefined){
          $.each(group_ids,function(){
            Sunstone.runAction(RESOURCE+".add_group",
              response.VDC.ID,
              { group_id : parseInt(this)});
          });
        }

        var clusters = request.request.data[0].clusters;
        if(clusters !=undefined){
          $.each(clusters,function(){
            Sunstone.runAction(RESOURCE+".add_cluster",
              response.VDC.ID,
              this);
          });
        }

        var hosts = request.request.data[0].hosts;
        if(hosts !=undefined){
          $.each(hosts,function(){
            Sunstone.runAction(RESOURCE+".add_host",
              response.VDC.ID,
              this);
          });
        }

        var vnets = request.request.data[0].vnets;
        if(vnets !=undefined){
          $.each(vnets,function(){
            Sunstone.runAction(RESOURCE+".add_vnet",
              response.VDC.ID,
              this);
          });
        }

        var datastores = request.request.data[0].datastores;
        if(datastores !=undefined){
          $.each(datastores,function(){
            Sunstone.runAction(RESOURCE+".add_datastore",
              response.VDC.ID,
              this);
          });
        }

        // TODO: this vdc.show may get the information before the add/del
        // actions end, showing "outdated" information

        Sunstone.runAction(RESOURCE+'.show',request.request.data[0][0]);

        Sunstone.getDataTable(TAB_ID).addElement(request, response);
        Notifier.notifyCustom(Locale.tr("VDC created"), " ID: " + response.VDC.ID, false);
      },
      error: Notifier.onError
    },

    "Vdc.add_group" : {
      type: "single",
      call : OpenNebulaResource.add_group,
      error : Notifier.onError
    },

    "Vdc.del_group" : {
      type: "single",
      call : OpenNebulaResource.del_group,
      error : Notifier.onError
    },

    "Vdc.add_cluster" : {
      type: "single",
      call : OpenNebulaResource.add_cluster,
      error : Notifier.onError
    },

    "Vdc.del_cluster" : {
      type: "single",
      call : OpenNebulaResource.del_cluster,
      error : Notifier.onError
    },

    "Vdc.add_host" : {
      type: "single",
      call : OpenNebulaResource.add_host,
      error : Notifier.onError
    },

    "Vdc.del_host" : {
      type: "single",
      call : OpenNebulaResource.del_host,
      error : Notifier.onError
    },

    "Vdc.add_vnet" : {
      type: "single",
      call : OpenNebulaResource.add_vnet,
      error : Notifier.onError
    },

    "Vdc.del_vnet" : {
      type: "single",
      call : OpenNebulaResource.del_vnet,
      error : Notifier.onError
    },

    "Vdc.add_datastore" : {
      type: "single",
      call : OpenNebulaResource.add_datastore,
      error : Notifier.onError
    },

    "Vdc.del_datastore" : {
      type: "single",
      call : OpenNebulaResource.del_datastore,
      error : Notifier.onError
    }
  };

  return _actions;
})
;

/* START_TEMPLATE */
define('hbs!tabs/vdcs-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n       <tr>\n        <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n        <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n        <td></td>\n      </tr>\n      ";
  stack1 = ((helper = (helper = helpers.renameTrHTML || (depth0 != null ? depth0.renameTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"renameTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n    </tbody>\n  </table>\n</div>\n</div>\n<div class=\"row\">\n  <div class=\"large-9 columns\">\n    ";
  stack1 = ((helper = (helper = helpers.templateTableHTML || (depth0 != null ? depth0.templateTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templateTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vdcs-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!utils/panel/rename-tr/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<tr class=\""
    + escapeExpression(((helper = (helper = helpers.resourceType || (depth0 != null ? depth0.resourceType : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"resourceType","hash":{},"data":data}) : helper)))
    + "_rename\">\n  <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</td>\n  <td class=\"value_td_rename\">"
    + escapeExpression(((helper = (helper = helpers.resourceName || (depth0 != null ? depth0.resourceName : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"resourceName","hash":{},"data":data}) : helper)))
    + "</td>\n  <td>\n    <div id=\"div_edit_rename\">\n      <a id=\"div_edit_rename_link\" class=\"edit_e\" href=\"#\"> <i class=\"fa fa-pencil-square-o right\"/></a>\n    </div>\n  </td>\n</tr>";
},"useData":true});
Handlebars.registerPartial('utils/panel/rename-tr/html', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/panel/rename-tr',['require','hbs!./rename-tr/html','sunstone'],function(require) {
  /*
    This module insert a row with the name of the resource.
    The row can be edited and a rename action will be sent
   */

  var TemplateRenameTr = require('hbs!./rename-tr/html');
  var Sunstone = require('sunstone');

  /*
    Generate the tr HTML with the name of the resource and an edit icon
    @param {String} resourceType Resource type (i.e: Zone, Host, Image...)
    @param {String} resourceName Name of the resource
    @returns {String} HTML row
   */
  var _html = function(resourceType, resourceName) {
    var renameTrHTML = TemplateRenameTr({
      'resourceType': resourceType.toLowerCase(),
      'resourceName': resourceName
    })

    return renameTrHTML;
  };

  /*
    Initialize the row, clicking the edit icon will add an input to edit the name
    @param {String} resourceType Resource type (i.e: Zone, Host, Image...)
    @param {String} resourceId ID of the resource
    @param {jQuery Object} context Selector including the tr
   */
  var _setup = function(resourceType, resourceId, context) {
    context.off("click", "#div_edit_rename_link");
    context.on("click", "#div_edit_rename_link", function() {
      var valueStr = $(".value_td_rename", context).text();
      $(".value_td_rename", context).html('<input class="input_edit_value_rename" id="input_edit_rename" type="text" value="' + valueStr + '"/>');
    });

    context.off("change", ".input_edit_value_rename");
    context.on("change", ".input_edit_value_rename", function() {
      var valueStr = $(".input_edit_value_rename", context).val();
      if (valueStr != "") {
        var nameTemplate = {"name": valueStr};
        Sunstone.runAction(resourceType + ".rename", resourceId, nameTemplate);
      }
    });

    return false;
  }

  return {
    'html': _html,
    'setup': _setup
  }
});

define('tabs/vdcs-tab/panels/info/panelId',['require'],function(require){
  return 'vdc_info_tab';
});
define('tabs/vdcs-tab/panels/info',['require','hbs!./info/html','utils/locale','utils/panel/rename-tr','utils/panel/template-table','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TemplateHTML = require('hbs!./info/html');
  var Locale = require('utils/locale');
  var RenameTr = require('utils/panel/rename-tr');
  var TemplateTable = require('utils/panel/template-table');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "Vdc";
  var XML_ROOT = "VDC";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var renameTrHTML = RenameTr.html(RESOURCE, this.element.NAME);
    var templateTableHTML = TemplateTable.html(this.element.TEMPLATE, RESOURCE,
                                              Locale.tr("Attributes"));

    return TemplateHTML({
      'element': this.element,
      'renameTrHTML': renameTrHTML,
      'templateTableHTML': templateTableHTML
    });
  }

  function _setup(context) {
    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);

    RenameTr.setup(RESOURCE, this.element.ID, context);

    TemplateTable.setup(this.element.TEMPLATE, RESOURCE, this.element.ID, context);
  }
});

define('tabs/vdcs-tab/panels/groups/panelId',['require'],function(require){
  return 'vdc_groups_tab';
});
define('tabs/vdcs-tab/panels/groups',['require','utils/locale','tabs/groups-tab/datatable','./groups/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var GroupsTable = require('tabs/groups-tab/datatable');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./groups/panelId');
  var GROUPS_TABLE_ID = PANEL_ID + "GroupsTable";
  var RESOURCE = "Vdc";
  var XML_ROOT = "VDC";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Groups");
    this.icon = "fa-users";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var groups = [];

    if (this.element.GROUPS.ID != undefined){
      groups = this.element.GROUPS.ID;

      if (!$.isArray(groups)){
        groups = [groups];
      }
    }

    var opts = {
      info: true,
      select: true,
      selectOptions: {
        read_only: true,
        fixed_ids: groups
      }
    };

    this.groupsTable = new GroupsTable(GROUPS_TABLE_ID, opts);

    return this.groupsTable.dataTableHTML;
  }

  function _setup(context) {
    this.groupsTable.initialize();
    this.groupsTable.refreshResourceTableSelect();

    return false;
  }
});

define('tabs/vdcs-tab/utils/common',['require'],function(require) {
  /*
    Common functions for VDCs
   */

  var VDC_ALL_RESOURCES = "-10";

  /**
    Return an object with the VDC resources indexed by zone_id.

       {   zone_id :
           {   clusters   : [],
               hosts      : [],
               vnets      : [],
               datastores : []
           }
       }
  */
  function _indexedVdcResources(vdc){
    var resources = {};

    var clusters_array = [];
    var hosts_array = [];
    var vnets_array = [];
    var datastores_array = [];

    if (vdc.CLUSTERS.CLUSTER){
      clusters_array = vdc.CLUSTERS.CLUSTER;

      if (!$.isArray(clusters_array)){
        clusters_array = [clusters_array];
      }
    }

    if (vdc.HOSTS.HOST){
      hosts_array = vdc.HOSTS.HOST;

      if (!$.isArray(hosts_array)){
        hosts_array = [hosts_array];
      }
    }

    if (vdc.VNETS.VNET){
      vnets_array = vdc.VNETS.VNET;

      if (!$.isArray(vnets_array)){
        vnets_array = [vnets_array];
      }
    }

    if (vdc.DATASTORES.DATASTORE){
      datastores_array = vdc.DATASTORES.DATASTORE;

      if (!$.isArray(datastores_array)){
        datastores_array = [datastores_array];
      }
    }

    function init_resources_zone(zone_id){
      if (resources[zone_id] == undefined){
        resources[zone_id] = {
          clusters   : [],
          hosts      : [],
          vnets      : [],
          datastores : []
        }
      }
    }

    $.each(clusters_array, function(i,e){
      init_resources_zone(e.ZONE_ID);
      resources[e.ZONE_ID].clusters.push(e.CLUSTER_ID);
    });

    $.each(hosts_array, function(i,e){
      init_resources_zone(e.ZONE_ID);
      resources[e.ZONE_ID].hosts.push(e.HOST_ID);
    });

    $.each(vnets_array, function(i,e){
      init_resources_zone(e.ZONE_ID);
      resources[e.ZONE_ID].vnets.push(e.VNET_ID);
    });

    $.each(datastores_array, function(i,e){
      init_resources_zone(e.ZONE_ID);
      resources[e.ZONE_ID].datastores.push(e.DATASTORE_ID);
    });

    return resources;
  }

  return {
    'indexedVdcResources': _indexedVdcResources,
    'VDC_ALL_RESOURCES': VDC_ALL_RESOURCES
  };
});


/* START_TEMPLATE */
define('hbs!tabs/vdcs-tab/utils/resources-tab/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"row\">\n  <div class=\"large-8 medium-6 columns\">\n    <label class=\"inline right\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Zone", {"name":"tr","hash":{},"data":data})))
    + "</label>\n  </div>\n  <div class=\"large-4 medium-6 columns end\">\n    <select class=\"vdc_zones_select\">\n    </select>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <div class=\"tabs-content vdc_zones_tabs_content\"></div>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vdcs-tab/utils/resources-tab/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/vdcs-tab/utils/resources-zone/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <dl class=\"tabs right-info-tabs text-center\" data-tab>\n      <dd class=\"active\"><a href=\"#vdcClustersTab_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\"><i class=\"fa fa-th\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Clusters", {"name":"tr","hash":{},"data":data})))
    + "</a></dd>\n      <dd><a href=\"#vdcHostsTab_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\"><i class=\"fa fa-hdd-o\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hosts", {"name":"tr","hash":{},"data":data})))
    + "</a></dd>\n      <dd><a href=\"#vdcVnetsTab_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\"><i class=\"fa fa-globe\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VNets", {"name":"tr","hash":{},"data":data})))
    + "</a></dd>\n      <dd><a href=\"#vdcDatastoresTab_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\"><i class=\"fa fa-folder-open\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastores", {"name":"tr","hash":{},"data":data})))
    + "</a></dd>\n    </dl>\n    <div class=\"tabs-content\">\n      <div class=\"content active\" id=\"vdcClustersTab_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\" class=\"content\">\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <label for=\"all_clusters_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\">\n              <input type=\"checkbox\" name=\"all_clusters_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\" id=\"all_clusters_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\" />\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "All", {"name":"tr","hash":{},"data":data})))
    + "\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Selects all current and future clusters", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n          </div>\n        </div>\n        <div class=\"vdc_clusters_select\">\n          ";
  stack1 = ((helper = (helper = helpers.clustersTableHTML || (depth0 != null ? depth0.clustersTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"clustersTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </div>\n      </div>\n      <div id=\"vdcHostsTab_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\" class=\"content\">\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <label for=\"all_hosts_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\">\n              <input type=\"checkbox\" name=\"all_hosts_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\" id=\"all_hosts_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\" />\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "All", {"name":"tr","hash":{},"data":data})))
    + "\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Selects all current and future hosts", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n          </div>\n        </div>\n        <div class=\"vdc_hosts_select\">\n          ";
  stack1 = ((helper = (helper = helpers.hostsTableHTML || (depth0 != null ? depth0.hostsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"hostsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </div>\n      </div>\n      <div id=\"vdcVnetsTab_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\" class=\"content\">\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <label for=\"all_vnets_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\">\n              <input type=\"checkbox\" name=\"all_vnets_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\" id=\"all_vnets_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\" />\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "All", {"name":"tr","hash":{},"data":data})))
    + "\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Selects all current and future vnets", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n          </div>\n        </div>\n        <div class=\"vdc_vnets_select\">\n          ";
  stack1 = ((helper = (helper = helpers.vnetsTableHTML || (depth0 != null ? depth0.vnetsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"vnetsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </div>\n      </div>\n      <div id=\"vdcDatastoresTab_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\" class=\"content\">\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <label for=\"all_datastores_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\">\n              <input type=\"checkbox\" name=\"all_datastores_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\" id=\"all_datastores_"
    + escapeExpression(((helper = (helper = helpers.unique_id || (depth0 != null ? depth0.unique_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"unique_id","hash":{},"data":data}) : helper)))
    + "\" />\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "All", {"name":"tr","hash":{},"data":data})))
    + "\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Selects all current and future datastores", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n          </div>\n        </div>\n        <div class=\"vdc_datastores_select\">\n          ";
  stack1 = ((helper = (helper = helpers.datastoresTableHTML || (depth0 != null ? depth0.datastoresTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"datastoresTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n        </div>\n      </div>\n    </div>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vdcs-tab/utils/resources-zone/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/clusters-tab/tabId',['require'],function(require){
  return 'clusters-tab';
});
define('tabs/clusters-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');

  /*
    CONSTANTS
   */

  var RESOURCE = "Cluster";
  var XML_ROOT = "CLUSTER";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"] },
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    };

    this.columns = [
      Locale.tr("ID"),
      Locale.tr("Name"),
      Locale.tr("Hosts"),
      Locale.tr("VNets"),
      Locale.tr("Datastores")
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 2,
      "select_resource": Locale.tr("Please select a Cluster from the list"),
      "you_selected": Locale.tr("You selected the following Cluster:"),
      "select_resource_multiple": Locale.tr("Please select one or more clusters from the list"),
      "you_selected_multiple": Locale.tr("You selected the following clusters:")
    };

    TabDataTable.call(this);
  }

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json[XML_ROOT];

    return [
      '<input class="check_item" type="checkbox" id="'+RESOURCE.toLowerCase()+'_' +
                           element.ID + '" name="selected_items" value="' +
                           element.ID + '"/>',
      element.ID,
      element.NAME,
      _lengthOf(element.HOSTS.ID),
      _lengthOf(element.VNETS.ID),
      _lengthOf(element.DATASTORES.ID)
    ];
  }

  function _lengthOf(ids){
    var l = 0;
    if ($.isArray(ids))
      l = ids.length;
    else if (!$.isEmptyObject(ids))
      l = 1;

    return l;
  }
});

define('tabs/hosts-tab/utils/cpu-bars',['require','utils/progress-bar','utils/humanize'],function(require) {
  /*
    Generate an Object containing the html for the real and allocated CPU
   */

  var ProgressBar = require('utils/progress-bar');
  var Humanize = require('utils/humanize')

  /*
    @param {Object} info Object representing the Host as returned by OpenNebula
    @param {Boolean} hostShareFlag if true the info param is the HOST_SHARE element instead of HOSt
   */
  var _html = function(host, hostShareFlag) {
    var hostShare = hostShareFlag ? host : host.HOST_SHARE;
    var maxCPU = parseInt(hostShare.MAX_CPU);
    var infoStr;

    var allocatedCPUBar
    if (hostShare.CPU_USAGE) {
      var allocatedCPU = parseInt(hostShare.CPU_USAGE);

      if (maxCPU > 0) {
          var ratioAllocatedCPU = Math.round((allocatedCPU / maxCPU) * 100);
          infoStr = allocatedCPU + ' / ' + maxCPU + ' (' + ratioAllocatedCPU + '%)';
      } else {
          infoStr = "";
      }

      allocatedCPUBar = ProgressBar.html(allocatedCPU, maxCPU, infoStr);
    }

    var realCPUBar
    if (hostShare.USED_CPU) {
      var realCPU = parseInt(hostShare.USED_CPU);

      if (maxCPU > 0) {
          var ratioRealCPU = Math.round((realCPU / maxCPU) * 100);
          infoStr = realCPU + ' / ' + maxCPU + ' (' + ratioRealCPU + '%)';
      } else {
          infoStr = "";
      }

      realCPUBar = ProgressBar.html(realCPU, maxCPU, infoStr);
    }

    return {
      real: realCPUBar,
      allocated: allocatedCPUBar
    }
  }

  return {
    'html': _html
  }
});
define('tabs/hosts-tab/utils/memory-bars',['require','utils/progress-bar','utils/humanize'],function(require) {
  /*
    Generate an Object containing the html for the real and allocated MEMORY
   */

  var ProgressBar = require('utils/progress-bar');
  var Humanize = require('utils/humanize')

  /*
    @param {Object} info Object representing the Host as returned by OpenNebula
    @param {Boolean} hostShareFlag if true the info param is the HOST_SHARE element instead of HOSt
   */
  var _html = function(host, hostShareFlag) {
    var hostShare = hostShareFlag ? host : host.HOST_SHARE;
    var maxMem = parseInt(hostShare.MAX_MEM);
    var infoStr = "";

    var allocatedMemBar;
    if (hostShare.MEM_USAGE) {
      var allocatedMem = parseInt(hostShare.MEM_USAGE);

      if (maxMem > 0) {
          var ratioAllocatedMem = Math.round((allocatedMem / maxMem) * 100);
          infoStr = Humanize.size(allocatedMem) + ' / ' + Humanize.size(maxMem) + ' (' + ratioAllocatedMem + '%)';
      } else {
          infoStr = Humanize.size(allocatedMem) + ' / -';
      }

      allocatedMemBar = ProgressBar.html(allocatedMem, maxMem, infoStr);
    }

    var realMemBar;
    if (hostShare.USED_MEM) {
      var realMem = parseInt(hostShare.USED_MEM);

      if (maxMem > 0) {
          var ratioRealMem = Math.round((realMem / maxMem) * 100);
          infoStr = Humanize.size(realMem) + ' / ' + Humanize.size(maxMem) + ' (' + ratioRealMem + '%)';
      } else {
          infoStr = Humanize.size(realMem) + ' / -';
      }

      realMemBar = ProgressBar.html(realMem, maxMem, infoStr);
    }

    return {
      real: realMemBar,
      allocated: allocatedMemBar
    }
  }

  return {
    'html': _html
  }
});
define('tabs/hosts-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','utils/humanize','./utils/cpu-bars','./utils/memory-bars','opennebula/host','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var CPUBars = require('./utils/cpu-bars');
  var MemoryBars = require('./utils/memory-bars');
  var OpenNebulaHost = require('opennebula/host')

  /*
    CONSTANTS
   */

  var RESOURCE = "Host";
  var XML_ROOT = "HOST";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check", 5, 6, 7, 8]},
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    }

    this.columns = [
      Locale.tr("ID") ,
      Locale.tr("Name") ,
      Locale.tr("Cluster"),
      Locale.tr("RVMs"),
      Locale.tr("Real CPU"),
      Locale.tr("Allocated CPU"),
      Locale.tr("Real MEM"),
      Locale.tr("Allocated MEM"),
      Locale.tr("Status"),
      Locale.tr("IM MAD"),
      Locale.tr("VM MAD"),
      Locale.tr("Last monitored on")
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 2,
      "select_resource": Locale.tr("Please select a Host from the list"),
      "you_selected": Locale.tr("You selected the following Host:"),
      "select_resource_multiple": Locale.tr("Please select one or more hosts from the list"),
      "you_selected_multiple": Locale.tr("You selected the following hosts:")
    };

    this.totalHosts = 0;
    this.onHosts = 0;
    this.offHosts = 0;
    this.errorHosts = 0;
    this.maxCPU = 0;
    this.allocatedCPU = 0;
    this.realCPU = 0;
    this.maxMemory = 0;
    this.allocatedMemory = 0;
    this.realMemory = 0;

    TabDataTable.call(this);
  };

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;
  Table.prototype.preUpdateView = _preUpdateView;
  Table.prototype.postUpdateView = _postUpdateView;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json.HOST;

    var cpuBars = CPUBars.html(element);
    var memoryBars = MemoryBars.html(element);

    this.totalHosts++;

    switch (parseInt(element.STATE)) {
      case OpenNebulaHost.STATES.INIT:
      case OpenNebulaHost.STATES.MONITORING_INIT:
      case OpenNebulaHost.STATES.MONITORING_MONITORED:
      case OpenNebulaHost.STATES.MONITORED:
        this.onHosts++;
        break;
      case OpenNebulaHost.STATES.ERROR:
      case OpenNebulaHost.STATES.MONITORING_ERROR:
        this.errorHosts++;
        break;
      case OpenNebulaHost.STATES.DISABLED:
      case OpenNebulaHost.STATES.MONITORING_DISABLED:
        this.offHosts++;
        break;
      default:
        break;
    }

    this.maxCPU += parseInt(element.HOST_SHARE.MAX_CPU);
    this.allocatedCPU += parseInt(element.HOST_SHARE.CPU_USAGE);
    this.realCPU += parseInt(element.HOST_SHARE.USED_CPU);
    this.maxMemory += parseInt(element.HOST_SHARE.MAX_MEM);
    this.allocatedMemory += parseInt(element.HOST_SHARE.MEM_USAGE);
    this.realMemory += parseInt(element.HOST_SHARE.USED_MEM);

    return [
        '<input class="check_item" type="checkbox" id="' + RESOURCE.toLowerCase() + '_' +
                             element.ID + '" name="selected_items" value="' +
                             element.ID + '"/>',
        element.ID,
        element.NAME,
        element.CLUSTER.length ? element.CLUSTER : "-",
        element.HOST_SHARE.RUNNING_VMS, //rvm
        cpuBars.real,
        cpuBars.allocated,
        memoryBars.real,
        memoryBars.allocated,
        OpenNebulaHost.simpleStateStr(element.STATE),
        element.IM_MAD,
        element.VM_MAD,
        Humanize.prettyTime(element.LAST_MON_TIME)
    ];
  }

  function _preUpdateView() {
    this.totalHosts = 0;
    this.onHosts = 0;
    this.offHosts = 0;
    this.errorHosts = 0;
    this.maxCPU = 0;
    this.allocatedCPU = 0;
    this.realCPU = 0;
    this.maxMemory = 0;
    this.allocatedMemory = 0;
    this.realMemory = 0;
  }

  function _postUpdateView() {
    $(".total_hosts").text(this.totalHosts);
    $(".on_hosts").text(this.onHosts);
    $(".off_hosts").text(this.offHosts);
    $(".error_hosts").text(this.errorHosts);

    var ratio_allocated_cpu = 0;
    if (this.maxCPU > 0) {
      ratio_allocated_cpu = Math.round((this.allocatedCPU / this.maxCPU) * 100);
      info_str = this.allocatedCPU + ' / ' + this.maxCPU ;
    } else {
      info_str = "- / -";
    }

    //$("#dash_host_allocated_cpu").html(usageBarHtml(allocated_cpu, max_cpu, info_str, true));

    $("#dashboard_host_allocated_cpu").html(quotaDashboard(
      "dashboard_host_allocated_cpu",
      Locale.tr("Allocated CPU"),
      "30px",
      "14px",
      {"percentage": ratio_allocated_cpu, "str": info_str})
    );

    var ratio_real_cpu = 0;
    if (this.maxCPU > 0) {
      ratio_real_cpu = Math.round((this.realCPU / this.maxCPU) * 100);
      info_str = this.realCPU + ' / ' + this.maxCPU;
    } else {
      info_str = "- / -";
    }

    $("#dashboard_host_real_cpu").html(quotaDashboard(
      "dashboard_host_real_cpu",
      Locale.tr("Real CPU"),
      "30px",
      "14px",
      {"percentage": ratio_real_cpu, "str": info_str})
    );

    var ratio_allocated_mem = 0;
    if (this.maxMemory > 0) {
      ratio_allocated_mem = Math.round((this.allocatedMemory / this.maxMemory) * 100);
      info_str = Humanize.size(this.allocatedMemory) + ' / ' + Humanize.size(this.maxMemory);
    } else {
      info_str = Humanize.size(this.allocatedMemory) + ' / -';
    }

    $("#dashboard_host_allocated_mem").html(quotaDashboard(
      "dashboard_host_allocated_mem",
      Locale.tr("Allocated Memory"),
      "30px",
      "14px",
      {"percentage": ratio_allocated_mem, "str": info_str})
    );

    var ratio_real_mem = 0;
    if (this.maxMemory > 0) {
      ratio_real_mem = Math.round((this.realMemory / this.maxMemory) * 100);
      info_str = Humanize.size(this.realMemory) + ' / ' + Humanize.size(this.maxMemory);
    } else {
      info_str = Humanize.size(this.realMemory) + ' / -';
    }

    $("#dashboard_host_real_mem").html(quotaDashboard(
      "dashboard_host_real_mem",
      Locale.tr("Real Memory"),
      "30px",
      "14px",
      {"percentage": ratio_real_mem, "str": info_str})
    );

  }

  function quotaDashboard(html_tag, legend, font_large_size, font_small_size, quota) {
    var percentage = quota.percentage > 100 ? 100 : quota.percentage;

    return '<div class="row">' +
          '<div class="large-12 columns text-center" style="margin-bottom: 5px">' +
            '<h4 class="subheader">'+
              '<small>'+ legend +'</small>'+
            '</h4>'+
          '</div>' +
        '</div>' +
        '<div class="row">' +
          '<div class="large-12 columns text-center">' +
            '<div class="progress large radius">' +
            '  <span id="' + html_tag + '_meter" class="meter" style="width: ' + percentage + '%"></span>' +
            '</div>' +
          '</div>' +
        '</div>' +
        '<div class="row">' +
          '<div class="large-12 columns text-center">' +
            '<span id="' + html_tag + '_percentage" class="left" style="font-size:' + font_small_size + ';">' + quota.percentage + ' %</span>' +
            '<span id="' + html_tag + '_str" class="right" style="color: #999; font-size: ' + font_small_size + ';">' + quota.str + '</span>' +
          '</div>' +
        '</div>';
  }
});

define('tabs/vnets-tab/utils/common',['require'],function(require) {
  /*
    Common functions for VNets
   */


  /*
    @param {Object} info Object representing the VNet as returned by OpenNebula
   */
   function _getARList(info){
    var ar_list = info.AR_POOL.AR;

    if (!ar_list){ //empty
      ar_list = [];
    } else if (ar_list.constructor != Array) { //>1 lease
      ar_list = [ar_list];
    }

    return ar_list;
  }

  return {
    'getARList': _getARList
  }
})
;
define('tabs/vnets-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','utils/progress-bar','./utils/common','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');
  var ProgressBar = require('utils/progress-bar');
  var Utils = require('./utils/common');

  /*
    CONSTANTS
   */

  var RESOURCE = "Network";
  var XML_ROOT = "VNET";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"]},
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    }

    this.columns = [
      Locale.tr("ID"),
      Locale.tr("Owner"),
      Locale.tr("Group"),
      Locale.tr("Name"),
      Locale.tr("Reservation"),
      Locale.tr("Cluster"),
      Locale.tr("Bridge"),
      Locale.tr("Leases"),
      Locale.tr("VLAN ID")
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 4,
      "uname_index": 2,
      "select_resource": Locale.tr("Please select a network from the list"),
      "you_selected": Locale.tr("You selected the following network:"),
      "select_resource_multiple": Locale.tr("Please select one or more networks from the list"),
      "you_selected_multiple": Locale.tr("You selected the following networks:")
    };

    this.usedLeases = 0;
    this.totalVNets = 0;

    TabDataTable.call(this);
  };

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;
  Table.prototype.preUpdateView = _preUpdateView;
  Table.prototype.postUpdateView = _postUpdateView;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json[XML_ROOT];

    this.usedLeases = this.usedLeases + parseInt(element.USED_LEASES);
    this.totalVNets++;

    var total_size = 0;

    var arList = Utils.getARList(element);

    $.each(arList, function(){
      total_size += parseInt(this.SIZE);
    });

    return [
      '<input class="check_item" type="checkbox" id="' + RESOURCE.toLowerCase() + '_' +
                           element.ID + '" name="selected_items" value="' +
                           element.ID + '"/>',
      element.ID,
      element.UNAME,
      element.GNAME,
      element.NAME,
      element.PARENT_NETWORK_ID.length ? Locale.tr("Yes") : Locale.tr("No"),
      element.CLUSTER.length ? element.CLUSTER : "-",
      element.BRIDGE,
      ProgressBar.html(element.USED_LEASES, total_size),
      element.VLAN_ID.length ? element.VLAN_ID : "-"
    ];
  }

  function _preUpdateView() {
    this.totalVNets = 0;
    this.usedLeases = 0;
  }

  function _postUpdateView() {
    $(".total_vnets").text(this.totalVNets);
    $(".addresses_vnets").text(this.usedLeases);
  }
});

define('tabs/datastores-tab/utils/datastore-capacity-bar',['require','utils/progress-bar','utils/humanize'],function(require) {
  /*
    Generate the HTML representing the capacity of the Datastore
   */

  var ProgressBar = require('utils/progress-bar');
  var Humanize = require('utils/humanize')

  /*
    @param {Object} info Object representing the Datastore as returned by OpenNebula
   */
  var _html = function(info) {
    var total = parseInt(info.TOTAL_MB);
    var used = total - parseInt(info.FREE_MB);

    if (total > 0) {
      var ratio = Math.round((used / total) * 100);
      info_str = Humanize.sizeFromMB(used) + ' / ' + Humanize.sizeFromMB(total) + ' (' + ratio + '%)';
    } else {
      if (info.TYPE == 1) {
        info_str = '- / -';
      } else {
        info_str = Humanize.size(used) + ' / -';
      }
    }

    return ProgressBar.html(used, total, info_str);
  }

  return {
    'html': _html
  }
})
;
define('tabs/datastores-tab/tabId',['require'],function(require){
  return 'datastores-tab';
});
define('tabs/datastores-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','opennebula/datastore','./utils/datastore-capacity-bar','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');
  var OpenNebulaDatastore = require('opennebula/datastore');
  var DatastoreCapacityBar = require('./utils/datastore-capacity-bar');

  /*
    CONSTANTS
   */

  var RESOURCE = "Datastore";
  var XML_ROOT = "DATASTORE";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  /*
    @dataTableId
    @param {String} dataTableId unique identifier
    @param {Object} conf
      conf = {
        'info': true,     enable on click row will show the element
        'action': true,   enable actions on row elements
        'select': true,   enable selecting elements from the table
        'selectOptions': {
          'filter_fn': function(ds) { return ds.TYPE == 0; }
        }
      }
    @returns {Table} A new table object
   */
  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"]},
          {"sWidth": "35px", "aTargets": [0]},
          {"sWidth": "250px", "aTargets": [5]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    }

    this.columns = [
      Locale.tr("ID"),
      Locale.tr("Owner"),
      Locale.tr("Group"),
      Locale.tr("Name"),
      Locale.tr("Capacity"),
      Locale.tr("Cluster"),
      Locale.tr("Basepath"),
      Locale.tr("TM MAD"),
      Locale.tr("DS MAD"),
      Locale.tr("Type"),
      Locale.tr("Status"),
    ]

    this.selectOptions = {
      "id_index": 1,
      "name_index": 4,
      "uname_index": 2,
      "select_resource": Locale.tr("Please select a datastore from the list"),
      "you_selected": Locale.tr("You selected the following datastore:"),
      "select_resource_multiple": Locale.tr("Please select one or more datastores from the list"),
      "you_selected_multiple": Locale.tr("You selected the following datastores:")
    }

    TabDataTable.call(this);
  };

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json.DATASTORE;

    return [
        '<input class="check_item" type="checkbox" id="'+RESOURCE.toLowerCase()+'_' +
                             element.ID + '" name="selected_items" value="' +
                             element.ID + '"/>',
        element.ID,
        element.UNAME,
        element.GNAME,
        element.NAME,
        DatastoreCapacityBar.html(element),
        element.CLUSTER.length ? element.CLUSTER : "-",
        element.BASE_PATH,
        element.TM_MAD,
        element.DS_MAD,
        OpenNebulaDatastore.typeStr(element.TYPE),
        OpenNebulaDatastore.stateStr(element.STATE)
    ];
  }
});

define('tabs/vdcs-tab/utils/resources-zone',['require','hbs!./resources-zone/html','tabs/clusters-tab/datatable','tabs/hosts-tab/datatable','tabs/vnets-tab/datatable','tabs/datastores-tab/datatable','utils/tips','./common'],function(require) {

  var TemplateHTML = require('hbs!./resources-zone/html');

  var ClustersTable = require('tabs/clusters-tab/datatable');
  var HostsTable = require('tabs/hosts-tab/datatable');
  var VNetsTable = require('tabs/vnets-tab/datatable');
  var DatastoresTable = require('tabs/datastores-tab/datatable');
  var Tips = require('utils/tips');
  var Utils = require('./common');

  var VDC_ALL_RESOURCES = Utils.VDC_ALL_RESOURCES;

  function ResourcesZone(unique_id, zone_id, zone_name, context, indexed_resources) {
    this.unique_id = unique_id;
    this.zone_id = zone_id;

    this.resources = undefined;
    if(indexed_resources != undefined && indexed_resources[zone_id] != undefined){
      this.resources = indexed_resources[zone_id];
    }
  }

  ResourcesZone.prototype.constructor = ResourcesZone;
  ResourcesZone.prototype.html = _html;
  ResourcesZone.prototype.setup = _setup;
  ResourcesZone.prototype.onShow = _onShow;
  ResourcesZone.prototype.retrieve = _retrieve;
  ResourcesZone.prototype.fill = _fill;
  ResourcesZone.prototype.getZoneId = _getZoneId;

  return ResourcesZone;

  function _getZoneId() {
    return this.zone_id;
  }

  function _html() {
    var that = this;

    var opts = {};

    $.each(["clusters", "hosts", "vnets", "datastores"], function(i,res_name){
      if(that.resources != undefined){
        opts[res_name] = {
          info: true,
          select: true,
          selectOptions: {
            read_only: true,
            zone_id: that.zone_id
          }
        };

        if (!(that.resources[res_name].length == 1 &&
              that.resources[res_name][0] == VDC_ALL_RESOURCES) ){

          opts[res_name].selectOptions.fixed_ids = that.resources[res_name];
        }
      } else {
        opts[res_name] = {
          info: false,
          select: true,
          selectOptions: {
            multiple_choice: true,
            zone_id: that.zone_id
          }
        };
      }
    });

    that.clustersTable = new ClustersTable("vdc_clusters_"+that.unique_id, opts["clusters"]);
    that.hostsTable = new HostsTable("vdc_hosts_"+that.unique_id, opts["hosts"]);
    that.vnetsTable = new VNetsTable("vdc_vnets_"+that.unique_id, opts["vnets"]);
    that.datastoresTable = new DatastoresTable("vdc_datastores_"+that.unique_id, opts["datastores"]);

    return TemplateHTML({
      'unique_id': that.unique_id,
      'clustersTableHTML': that.clustersTable.dataTableHTML,
      'hostsTableHTML': that.hostsTable.dataTableHTML,
      'vnetsTableHTML': that.vnetsTable.dataTableHTML,
      'datastoresTableHTML': that.datastoresTable.dataTableHTML
    });
  }

  function _onShow(context) {
    var that = this;

    $.each([that.clustersTable, that.hostsTable,
            that.vnetsTable, that.datastoresTable], function(){

      this.refreshResourceTableSelect();
    });
  }

  function _setup(context){
    var that = this;

    context.foundation();

    $.each(["clusters", "hosts", "vnets", "datastores"], function(i,res_name){
      if(that.resources != undefined){
        if (that.resources[res_name].length == 1 &&
            that.resources[res_name][0] == VDC_ALL_RESOURCES){

          $("#all_"+res_name+"_"+that.unique_id, context).prop("checked", "checked");
        }

        $("#all_"+res_name+"_"+that.unique_id, context).prop("disabled", true);
      }

      $("input[name='all_"+res_name+"_"+that.unique_id+"']", context).change(function(){
        if ($(this).prop("checked")){
          $("div.vdc_"+res_name+"_select", context).hide();
        } else {
          $("div.vdc_"+res_name+"_select", context).show();
        }
      });
    });

    $.each([that.clustersTable, that.hostsTable,
            that.vnetsTable, that.datastoresTable], function(){

      this.initialize();
    });

    Tips.setup(context);
  }

  /*
  Return an object with the selected VDC resources in the zone tab
      {   clusters   : [],
          hosts      : [],
          vnets      : [],
          datastores : []
      }
  */
  function _retrieve(context) {
    var clusters;
    var hosts;
    var vnets;
    var datastores;

    if ( $("input[name='all_clusters_"+this.unique_id+"']", context).prop("checked") ){
      clusters = [VDC_ALL_RESOURCES];
    } else {
      clusters = this.clustersTable.retrieveResourceTableSelect();
    }

    if ( $("input[name='all_hosts_"+this.unique_id+"']", context).prop("checked") ){
      hosts = [VDC_ALL_RESOURCES];
    } else {
      hosts = this.hostsTable.retrieveResourceTableSelect();
    }

    if ( $("input[name='all_vnets_"+this.unique_id+"']", context).prop("checked") ){
      vnets = [VDC_ALL_RESOURCES];
    } else {
      vnets = this.vnetsTable.retrieveResourceTableSelect();
    }

    if ( $("input[name='all_datastores_"+this.unique_id+"']", context).prop("checked") ){
      datastores = [VDC_ALL_RESOURCES];
    } else {
      datastores = this.datastoresTable.retrieveResourceTableSelect();
    }

    var resources = {
      clusters   : clusters,
      hosts      : hosts,
      vnets      : vnets,
      datastores : datastores
    };

    return resources;
  }

  function _fill(context, selectedResources){

    if (selectedResources[this.zone_id] == undefined){
      selectedResources[this.zone_id] = {
        clusters   : [],
        hosts      : [],
        vnets      : [],
        datastores : []
      };
    }

    var resourcesZone = selectedResources[this.zone_id];

    if(resourcesZone.clusters.length == 1 &&
       resourcesZone.clusters[0] == VDC_ALL_RESOURCES){

      $("#all_clusters_"+this.unique_id, context).click();
    }else{
      this.clustersTable.selectResourceTableSelect(
            { ids : resourcesZone.clusters });
    }

    if(resourcesZone.hosts.length == 1 &&
       resourcesZone.hosts[0] == VDC_ALL_RESOURCES){

      $("#all_hosts_"+this.unique_id, context).click();
    }else{
      this.hostsTable.selectResourceTableSelect(
            { ids : resourcesZone.hosts });
    }

    if(resourcesZone.vnets.length == 1 &&
       resourcesZone.vnets[0] == VDC_ALL_RESOURCES){

      $("#all_vnets_"+this.unique_id, context).click();
    }else{
      this.vnetsTable.selectResourceTableSelect(
            { ids : resourcesZone.vnets });
    }

    if(resourcesZone.datastores.length == 1 &&
       resourcesZone.datastores[0] == VDC_ALL_RESOURCES){

      $("#all_datastores_"+this.unique_id, context).click();
    }else{
      this.datastoresTable.selectResourceTableSelect(
            { ids : resourcesZone.datastores });
    }
  }
});

define('tabs/vdcs-tab/utils/resources-tab',['require','hbs!./resources-tab/html','./resources-zone','./common'],function(require) {

  var TemplateHTML = require('hbs!./resources-tab/html');
  var ResourcesZone = require('./resources-zone');
  var Utils = require('./common');

  function ResourcesTab(unique_id_prefix) {
    this.unique_id_prefix = unique_id_prefix;
    this.zones = [];
  }

  ResourcesTab.prototype.constructor = ResourcesTab;
  ResourcesTab.prototype.html = _html;
  ResourcesTab.prototype.setup = _setup;
  ResourcesTab.prototype.onShow = _onShow;
  ResourcesTab.prototype.retrieve = _retrieve;
  ResourcesTab.prototype.retrieveIndexed = _retrieveIndexed;
  ResourcesTab.prototype.fill = _fill;
  ResourcesTab.prototype.addResourcesZone = _addResourcesZone;

  return ResourcesTab;

  function _html() {
    return TemplateHTML({});
  }

  function _onShow(context) {
    $.each(this.zones,function(i,resourcesZone){
      resourcesZone.onShow(context);
    });
  }

  function _setup(context) {
    var that = this;

    $("select.vdc_zones_select", context).change(function(){
      context.find(".vdc_zone_content").hide();
      $('div#'+that.unique_id_prefix+'_'+$(this).val()+'Tab', context).show();
    });

    $("select.vdc_zones_select", context)[0].selectedIndex = 0;
    $("select.vdc_zones_select", context).change();
  }

  /**
   * Returns the selected resources as needed by the Vdc.create call
   * @param  {objec} context jquery selector
   * @return {object}        Resources as:
   *                   {
   *                   "clusters" : {zone_id: zone_id, cluster_id: cluster_id},
   *                   "hosts" : {zone_id: zone_id, host_id: host_id}
   *                   "vnets" : {zone_id: zone_id, vnet_id: vnet_id}
   *                   "datastores" : {zone_id: zone_id, ds_id: ds_id}
   *                   }
   */
  function _retrieve(context) {
    var clusters    = [];
    var hosts       = [];
    var vnets       = [];
    var datastores  = [];

    $.each(this.zones,function(i,resourcesZone){
      var resources = resourcesZone.retrieve(context);
      var zone_id = resourcesZone.getZoneId();

      $.each(resources.clusters,function(j,cluster_id){
        clusters.push({zone_id: zone_id, cluster_id: cluster_id});
      });

      $.each(resources.hosts,function(j,host_id){
        hosts.push({zone_id: zone_id, host_id: host_id});
      });
      $.each(resources.vnets,function(j,vnet_id){
        vnets.push({zone_id: zone_id, vnet_id: vnet_id});
      });
      $.each(resources.datastores,function(j,ds_id){
        datastores.push({zone_id: zone_id, ds_id: ds_id});
      });
    });

    return {
      "clusters" : clusters,
      "hosts" : hosts,
      "vnets" : vnets,
      "datastores" : datastores
    };
  }

  function _retrieveIndexed(context) {
    var resources = {};

    $.each(this.zones,function(i,resourcesZone){
      resources[resourcesZone.getZoneId()] = resourcesZone.retrieve(context);
    });

    return resources;
  }

  function _fill(context, selectedResources){
    $.each(this.zones,function(i,resourcesZone){
      resourcesZone.fill(context, selectedResources);
    });
  }

  function _addResourcesZone(zone_id, zone_name, context, indexed_resources) {
    var unique_id = this.unique_id_prefix+'_'+zone_id;

    var resourcesZone = new ResourcesZone(unique_id, zone_id, zone_name, context, indexed_resources);

    // Append the new div containing the tab and add the tab to the list
    var html_tab_content =
      '<div id="'+unique_id+'Tab" class="vdc_zone_content">'+
        resourcesZone.html()+
      '</div>';

    $(html_tab_content).appendTo($(".vdc_zones_tabs_content", context));

    $("select.vdc_zones_select", context).append(
                        '<option value="'+zone_id+'">'+zone_name+'</option>');

    var zoneSection = $('#' +unique_id+'Tab', context);

    resourcesZone.setup(zoneSection);
    resourcesZone.onShow(zoneSection);

    this.zones.push(resourcesZone);
  }

});
define('tabs/vdcs-tab/panels/resources/panelId',['require'],function(require){
  return 'vdc_resources_tab';
});
define('tabs/vdcs-tab/panels/resources',['require','utils/locale','../utils/common','../utils/resources-tab','opennebula/zone','../tabId','./resources/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Utils = require('../utils/common');
  var ResourcesTab = require('../utils/resources-tab');
  var OpenNebulaZone = require('opennebula/zone');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./resources/panelId');
  var RESOURCE = "Vdc";
  var XML_ROOT = "VDC";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Resources");
    this.icon = "fa-th";

    this.element = info[XML_ROOT];

    this.resourcesTab = new ResourcesTab("vdc_info_panel");

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return this.resourcesTab.html();
  }

  function _setup(context) {
    var that = this;

    var indexed_resources = Utils.indexedVdcResources(this.element);

    $.each(indexed_resources, function(zone_id,objects){
      that.resourcesTab.addResourcesZone(
        zone_id,
        OpenNebulaZone.getName(zone_id),
        context,
        indexed_resources);
    });

    that.resourcesTab.setup(context);
    that.resourcesTab.onShow(context);
  }
});


/* START_TEMPLATE */
define('hbs!utils/custom-tags-table/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <table class=\"custom_tags dataTable policies_table\">\n      <thead>\n        <tr>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "KEY", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VALUE", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th></th>\n        </tr>\n      </thead>\n      <tbody>\n      </tbody>\n      <tfoot>\n        <tr>\n          <td colspan=\"3\">\n            <a type=\"button\" class=\"add_custom_tag button small small-12 secondary radius\">\n              <i class=\"fa fa-plus\"></i> "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add another attribute", {"name":"tr","hash":{},"data":data})))
    + "\n            </a>\n          </td>\n        </tr>\n      </tfoot>\n    </table>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('utils/custom-tags-table/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!utils/custom-tags-table/row',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<tr>\n  <td>\n    <input class=\"custom_tag_key\" type=\"text\" name=\"key\" value=\""
    + escapeExpression(((helper = (helper = helpers.key || (depth0 != null ? depth0.key : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"key","hash":{},"data":data}) : helper)))
    + "\">\n</td>\n<td>\n  <textarea class=\"custom_tag_value\" name=\"value\">"
    + escapeExpression(((helper = (helper = helpers.value || (depth0 != null ? depth0.value : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"value","hash":{},"data":data}) : helper)))
    + "</textarea>\n</td>\n<td>\n  <a href=\"#\"><i class=\"fa fa-times-circle remove-tab\"></i></a>\n</td>\n</tr>";
},"useData":true});
Handlebars.registerPartial('utils/custom-tags-table/row', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/custom-tags-table',['require','utils/locale','hbs!./custom-tags-table/html','hbs!./custom-tags-table/row','utils/template-utils'],function(require) {

  var Locale = require('utils/locale');
  var TemplateHTML = require('hbs!./custom-tags-table/html');
  var RowTemplateHTML = require('hbs!./custom-tags-table/row');
  var TemplateUtils = require('utils/template-utils');

  function _html(){
    return TemplateHTML();
  }

  function _setup(context){
    context.off("click", ".add_custom_tag");
    context.on("click", ".add_custom_tag", function(){
      $(".custom_tags tbody", context).append(RowTemplateHTML());
    });

    $(".add_custom_tag", context).trigger("click");

    context.on("click", ".custom_tags i.remove-tab", function(){
      var tr = $(this).closest('tr');
      tr.remove();
    });
  }

  // context is the container div of customTagsHtml()
  function _retrieveCustomTags(context){
    var template_json = {};

    $('.custom_tags tr', context).each(function(){
      if ($('.custom_tag_key', $(this)).val()) {
        template_json[$('.custom_tag_key', $(this)).val()] = $('.custom_tag_value', $(this)).val();
      }
    });

    return template_json;
  }

  // context is the container div of customTagsHtml()
  // template_json are the key:values that will be put into the table
  function _fillCustomTags(context, template_json){
    $(".custom_tags i.remove-tab", context).trigger("click");

    $.each(template_json, function(key, value){
      $(".custom_tags tbody", context).append(
                                    RowTemplateHTML({key: key, value: value}));
    });
  }

  return {
    'html': _html,
    'setup': _setup,
    'retrieve': _retrieveCustomTags,
    'fill': _fillCustomTags
  };
});
define('utils/wizard-fields',['require','utils/template-utils'],function(require) {
  var TemplateUtils = require('utils/template-utils');

  /*
    Inputs must define wizard_field="KEY"
    Inputs with the wizard_field attr will not be considered
   */

  return {
    'retrieve': _retrieveWizardFields,
    'fill': _fillWizardFields
  }

  // TODO: other types: radio, checkbox
  function _retrieveWizardFields(context) {
    var templateJSON = {};
    var fields = $('[wizard_field]', context);

    fields.each(function() {
      var field = $(this);

      if (field.prop('wizard_field_disabled') != true &&
            field.val() != null && field.val().length &&
            (field.attr("type") != "checkbox" || field.prop("checked")) &&
            (field.attr("type") != "radio" || field.prop("checked"))) {
        var field_name = field.attr('wizard_field');
        templateJSON[field_name] = field.val();
      }
    });

    return templateJSON;
  }

  function _fillWizardFields(context, templateJSON) {
    var fields = $('[wizard_field]', context);

    fields.each(function() {
      var field = $(this);
      var field_name = field.attr('wizard_field');
      if (templateJSON[field_name]) {
        switch (field.attr("type")){
        case "radio":
          var checked = (field.val() == templateJSON[field_name]);

          field.prop("checked", checked);

          if (checked) {
            field.change();
          }
          break;
        case "checkbox":
          var checked = (field.val().toUpperCase() ==
                          templateJSON[field_name].toUpperCase());

          field.prop("checked", checked);

          if (checked) {
            field.change();
          }
          break;
        default:
          field.val(
            TemplateUtils.escapeDoubleQuotes(
              TemplateUtils.htmlDecode(templateJSON[field_name])));
          field.change();
        }

        delete templateJSON[field_name];
      }
    });
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vdcs-tab/form-panels/create/wizard',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div>\n    <dl id=\"vdc_create_tabs\" class=\"tabs right-info-tabs text-center\" data-tab>\n      <dd class=\"active\"><a href=\"#vdcCreateGeneralTab\"><i class=\"fa fa-th\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "General", {"name":"tr","hash":{},"data":data})))
    + "</a></dd>\n      <dd><a href=\"#vdcCreateGroupsTab\"><i class=\"fa fa-users\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Groups", {"name":"tr","hash":{},"data":data})))
    + "</a></dd>\n      <dd><a href=\"#vdcCreateResourcesTab\"><i class=\"fa fa-hdd-o\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Resources", {"name":"tr","hash":{},"data":data})))
    + "</a></dd>\n    </dl>\n    <div id=\"vdc_create_tabs_content\" class=\"tabs-content\">\n      <div class=\"content active\" id=\"vdcCreateGeneralTab\">\n        <div class=\"row\">\n          <div class=\"large-6 medium-8 columns\">\n            <label for=\"name\" >"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name that the VDC will get for description purposes.", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n            <input type=\"text\" wizard_field=\"NAME\" required name=\"name\" id=\"name\"/>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-6 medium-8 columns\">\n            <label for=\"DESCRIPTION\" >"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description of the VDC", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n            <textarea type=\"text\" wizard_field=\"DESCRIPTION\" id=\"DESCRIPTION\" name=\"DESCRIPTION\"/>\n          </div>\n        </div>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom attributes", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            <br>\n            <br>\n          </div>\n        </div>\n        ";
  stack1 = ((helper = (helper = helpers.customTagsHTML || (depth0 != null ? depth0.customTagsHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customTagsHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </div>\n      <div id=\"vdcCreateGroupsTab\" class=\"content\">\n        ";
  stack1 = ((helper = (helper = helpers.groupsTableHTML || (depth0 != null ? depth0.groupsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"groupsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </div>\n      <div id=\"vdcCreateResourcesTab\" class=\"content\">\n        ";
  stack1 = ((helper = (helper = helpers.resourcesTabHTML || (depth0 != null ? depth0.resourcesTabHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"resourcesTabHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      </div>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/vdcs-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/vdcs-tab/form-panels/create/advanced',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Advanced\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <p>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Write the VDC template here", {"name":"tr","hash":{},"data":data})))
    + "</p>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <textarea id=\"template\" rows=\"15\" required></textarea>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/vdcs-tab/form-panels/create/advanced', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vdcs-tab/form-panels/create',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','utils/tips','utils/custom-tags-table','utils/wizard-fields','tabs/groups-tab/datatable','opennebula/zone','../utils/common','utils/notifier','../utils/resources-tab','utils/template-utils','hbs!./create/wizard','hbs!./create/advanced','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var CustomTagsTable = require('utils/custom-tags-table');
  var WizardFields = require('utils/wizard-fields');
  var GroupsTable = require('tabs/groups-tab/datatable');
  var OpenNebulaZone = require('opennebula/zone');
  var Utils = require('../utils/common');
  var Notifier = require('utils/notifier');
  var ResourcesTab = require('../utils/resources-tab');
  var TemplateUtils = require('utils/template-utils');

  /*
    TEMPLATES
   */

  var TemplateWizardHTML = require('hbs!./create/wizard');
  var TemplateAdvancedHTML = require('hbs!./create/advanced');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create Virtual Data Center"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      },
      'update': {
        'title': Locale.tr("Update Virtual Data Center"),
        'buttonText': Locale.tr("Update"),
        'resetButton': false
      }
    };

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.htmlAdvanced = _htmlAdvanced;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.submitAdvanced = _submitAdvanced;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.fill = _fill;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    var opts = {
      info: false,
      select: true,
      selectOptions: {"multiple_choice": true}
    };

    this.groupsTable = new GroupsTable("vdc_wizard_groups", opts);

    if (this.action == "create") {
      this.resourcesTab = new ResourcesTab("vdc_create_wizard");
    } else {
      this.resourcesTab = new ResourcesTab("vdc_update_wizard");
    }

    return TemplateWizardHTML({
      'formPanelId': this.formPanelId,
      'customTagsHTML': CustomTagsTable.html(),
      'groupsTableHTML': this.groupsTable.dataTableHTML,
      'resourcesTabHTML': this.resourcesTab.html()
    });
  }

  function _htmlAdvanced() {
    return TemplateAdvancedHTML({formPanelId: this.formPanelId});
  }

  function _setup(context) {
    var that = this;

    CustomTagsTable.setup($("#vdcCreateGeneralTab", context));
    this.groupsTable.initialize();

    // If this is an update, the _fill method may be called before the zone.list
    // finishes. And resourcesTab.fill only works if all the
    // resourcesTab.addResourcesZone have finished.
    // It's better to duplicate this code in _fill and make sure the
    // resourcesTab.fill is executed in the callback
    if (this.action != "update") {
      var zone_ids = [];

      OpenNebulaZone.list({
        timeout: true,
        success: function (request, obj_list){
          var zoneSection = $("#vdcCreateResourcesTab",context);

          $.each(obj_list,function(){
            zone_ids.push(this.ZONE.ID);

            that.resourcesTab.addResourcesZone(
              this.ZONE.ID,
              this.ZONE.NAME,
              zoneSection);
          });

          that.resourcesTab.setup(zoneSection);

          that.zone_ids = zone_ids;
        },
        error: Notifier.onError
      });
    }

    context.foundation();
    Tips.setup();
  }

  function _submitWizard(context) {
    var that = this;

    //Fetch values
    var vdc_json = {};

    $.extend(vdc_json, WizardFields.retrieve($("#vdcCreateGeneralTab", context)));

    $.extend(vdc_json, CustomTagsTable.retrieve($("#vdcCreateGeneralTab", context)));

    var group_ids = this.groupsTable.retrieveResourceTableSelect();

    if (this.action == "create") {
      var resources = this.resourcesTab.retrieve(context);

      vdc_json = {
        "vdc" : vdc_json,
        "group_ids" : group_ids,
        "clusters" : resources.clusters,
        "hosts" : resources.hosts,
        "vnets" : resources.vnets,
        "datastores" : resources.datastores
      };

      Sunstone.runAction("Vdc.create",vdc_json);
      return false;
    } else if (this.action == "update") {
      // Add/delete groups

      var selected_groups_list = this.groupsTable.retrieveResourceTableSelect();

      $.each(selected_groups_list, function(i,group_id){
        if (that.original_groups_list.indexOf(group_id) == -1){
          Sunstone.runAction("Vdc.add_group",
            that.resourceId, {group_id : group_id});
        }
      });

      $.each(that.original_groups_list, function(i,group_id){
        if (selected_groups_list.indexOf(group_id) == -1){
          Sunstone.runAction("Vdc.del_group",
            that.resourceId, {group_id : group_id});
        }
      });

      // Add/delete resources

      var selectedResources = that.resourcesTab.retrieveIndexed(context);

      for (var zoneId in selectedResources){
        var originalSelectedZone = that.originalSelectedResources[zoneId];

        $.each(selectedResources[zoneId].clusters, function(i,id){
          if (originalSelectedZone.clusters.indexOf(id) == -1){
            Sunstone.runAction(
              "Vdc.add_cluster",
              that.resourceId,
              {zone_id: zoneId, cluster_id: id});
          }
        });

        $.each(selectedResources[zoneId].hosts, function(i,id){
          if (originalSelectedZone.hosts.indexOf(id) == -1){
            Sunstone.runAction(
              "Vdc.add_host",
              that.resourceId,
              {zone_id: zoneId, host_id: id});
          }
        });

        $.each(selectedResources[zoneId].vnets, function(i,id){
          if (originalSelectedZone.vnets.indexOf(id) == -1){
            Sunstone.runAction(
              "Vdc.add_vnet",
              that.resourceId,
              {zone_id: zoneId, vnet_id: id});
          }
        });

        $.each(selectedResources[zoneId].datastores, function(i,id){
          if (originalSelectedZone.datastores.indexOf(id) == -1){
            Sunstone.runAction(
              "Vdc.add_datastore",
              that.resourceId,
              {zone_id: zoneId, ds_id: id});
          }
        });
      }

      for (var zoneId in that.originalSelectedResources){
        var selectedZone = selectedResources[zoneId];

        $.each(that.originalSelectedResources[zoneId].clusters, function(i,id){
          if (selectedZone.clusters.indexOf(id) == -1){
            Sunstone.runAction(
              "Vdc.del_cluster",
              that.resourceId,
              {zone_id: zoneId, cluster_id: id});
          }
        });

        $.each(that.originalSelectedResources[zoneId].hosts, function(i,id){
          if (selectedZone.hosts.indexOf(id) == -1){
            Sunstone.runAction(
              "Vdc.del_host",
              that.resourceId,
              {zone_id: zoneId, host_id: id});
          }
        });

        $.each(that.originalSelectedResources[zoneId].vnets, function(i,id){
          if (selectedZone.vnets.indexOf(id) == -1){
            Sunstone.runAction(
              "Vdc.del_vnet",
              that.resourceId,
              {zone_id: zoneId, vnet_id: id});
          }
        });

        $.each(that.originalSelectedResources[zoneId].datastores, function(i,id){
          if (selectedZone.datastores.indexOf(id) == -1){
            Sunstone.runAction(
              "Vdc.del_datastore",
              that.resourceId,
              {zone_id: zoneId, ds_id: id});
          }
        });
      }

      // TODO: this method ends now, but the add/del actions may still
      // be pending. A vdc.show now will get outdated information

      Sunstone.runAction("Vdc.update", that.resourceId, TemplateUtils.templateToString(vdc_json));
      return false;
    }

  }

  function _submitAdvanced(context) {
    if (this.action == "create") {
      var template = $('textarea#template',context).val();
      var vdc_json = {vdc: {vdc_raw: template}};
      Sunstone.runAction("Vdc.create",vdc_json);
      return false;
    } else if (this.action == "update") {
      var template_raw = $('textarea#template', context).val();
      Sunstone.runAction("Vdc.update", this.resourceId, template_raw);
      return false;
    }
  }

  function _onShow(context) {
    // TODO bug, does not work until the input is visible
    //$("input#name", context).focus();

    this.groupsTable.refreshResourceTableSelect();
    this.resourcesTab.onShow(context);
  }

  function _fill(context, element) {
    var that = this;

    this.resourceId = element.ID;

    // Populates the Avanced mode Tab
    $('#template', context).val(TemplateUtils.templateToString(element.TEMPLATE).replace(/^[\r\n]+$/g, ""));


    $('[wizard_field="NAME"]',context).val(
        TemplateUtils.escapeDoubleQuotes( TemplateUtils.htmlDecode(element.NAME) )).
        prop("disabled", true).
        prop('wizard_field_disabled', true);

    WizardFields.fill($("#vdcCreateGeneralTab", context), element.TEMPLATE);

    // Delete so these attributes don't end in the custom tags table also
    var fields = $('[wizard_field]', context);

    fields.each(function(){
      var field = $(this);
      var field_name = field.attr('wizard_field');

      delete element.TEMPLATE[field_name];
    });

    CustomTagsTable.fill($("#vdcCreateGeneralTab", context), element.TEMPLATE);

    // Fill groups table

    var group_ids = element.GROUPS.ID;

    if (typeof group_ids == 'string') {
      group_ids = [group_ids];
    }

    this.original_groups_list = [];

    if (group_ids) {
      this.original_groups_list = group_ids;
      this.groupsTable.selectResourceTableSelect({ ids : group_ids });
    }

    // Fill resource tables

    this.originalSelectedResources = Utils.indexedVdcResources(element);

    var zone_ids = [];

    OpenNebulaZone.list({
      timeout: true,
      success: function (request, obj_list){
        var zoneSection = $("#vdcCreateResourcesTab",context);

        $.each(obj_list,function(){
          zone_ids.push(this.ZONE.ID);

          that.resourcesTab.addResourcesZone(
            this.ZONE.ID,
            this.ZONE.NAME,
            zoneSection);
        });

        that.resourcesTab.setup(zoneSection);
        that.resourcesTab.fill(zoneSection, that.originalSelectedResources);

        that.zone_ids = zone_ids;
      },
      error: Notifier.onError
    });
  }
});

define('tabs/vdcs-tab',['require','utils/locale','./vdcs-tab/buttons','./vdcs-tab/actions','./vdcs-tab/datatable','./vdcs-tab/tabId','./vdcs-tab/panels/info','./vdcs-tab/panels/groups','./vdcs-tab/panels/resources','./vdcs-tab/form-panels/create'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./vdcs-tab/buttons');
  var Actions = require('./vdcs-tab/actions');
  var Table = require('./vdcs-tab/datatable');

  var TAB_ID = require('./vdcs-tab/tabId');
  var DATATABLE_ID = "dataTableVDCs";

  var _dialogs = [
  ];

  var _panels = [
    require('./vdcs-tab/panels/info'),
    require('./vdcs-tab/panels/groups'),
    require('./vdcs-tab/panels/resources')
  ];

  var _formPanels = [
    require('./vdcs-tab/form-panels/create')
  ];

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("VDCs"),
    tabClass: "subTab",
    parentTab: "system-tab",
    listHeader: '<i class="fa fa-fw fa-th"></i>&emsp;'+Locale.tr("Virtual Data Centers"),
    infoHeader: '<i class="fa fa-fw fa-th"></i>&emsp;'+Locale.tr("Virtual Data Center"),
    subheader: '<span>\
        <span class="total_vdcs"/> <small>'+Locale.tr("TOTAL")+'</small>\
      </span>',
    resource: 'Vdc',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});

define('tabs/acls-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "Acl.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "Acl.create_dialog" : {
      type: "create_dialog",
      layout: "create"
    },
    "Acl.delete" : {
      type: "confirm",
      text: Locale.tr("Delete"),
      layout: "del"
    },
  };

  return Buttons;
});

define('tabs/acls-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','opennebula/user','opennebula/group','opennebula/zone','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');

  var OpenNebulaUser = require('opennebula/user');
  var OpenNebulaGroup = require('opennebula/group');
  var OpenNebulaZone = require('opennebula/zone');

  /*
    CONSTANTS
   */

  var RESOURCE = "Acl";
  var XML_ROOT = "ACL";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          { "bSortable": false, "aTargets": ["check",2,3,4,5,6,7] },
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    };

    this.columns = [
      Locale.tr("ID"),
      Locale.tr("Applies to"),
      Locale.tr("Affected resources"),
      Locale.tr("Resource ID / Owned by"),
      Locale.tr("Allowed operations"),
      Locale.tr("Zone"),
      Locale.tr("ACL String")
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 1,
      "select_resource": Locale.tr("Please select an ACL rule from the list"),
      "you_selected": Locale.tr("You selected the following ACL rule:"),
      "select_resource_multiple": Locale.tr("Please select one or more ACL rules from the list"),
      "you_selected_multiple": Locale.tr("You selected the following ACL rules:")
    };

    TabDataTable.call(this);
  }

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json[XML_ROOT];

    var acl_string = element.STRING;

    var acl_array = _parseAclString(acl_string);

    return [
      '<input class="check_item" type="checkbox" id="'+RESOURCE.toLowerCase()+'_' +
                           element.ID + '" name="selected_items" value="' +
                           element.ID + '"/>',
      element.ID,
      acl_array[0],
      acl_array[1],
      acl_array[2],
      acl_array[3],
      Locale.tr(acl_array[4].charAt(0).toUpperCase()+acl_array[4].substring(1)), //capitalize 1st letter for translation
      element.STRING
    ];
  }


  //Parses a full ACL string, and translates it into
  //a legible array
  //to be put in the datatable fields.
  function _parseAclString(string) {
    var space_split = string.split(' ');
    var user = space_split[0];
    var resources = space_split[1];
    var rights = space_split[2];
    var zone = space_split[3];

    //User
    var user_str = _parseUserAcl(user);


    //Resources
    var resources_str="";
    var resources_array = resources.split('/');
    var belonging_to = _parseResourceAcl(resources_array[1]);
    resources_array = resources_array[0].split('+');
    for (var i=0; i<resources_array.length;i++){
      switch (resources_array[i]){
      case "HOST":
          resources_str+=Locale.tr("Hosts")+", ";
          break;
      case "VM":
          resources_str+=Locale.tr("Virtual Machines")+", ";
          break;
      case "NET":
          resources_str+=Locale.tr("Virtual Networks")+", ";
          break;
      case "IMAGE":
          resources_str+=(Locale.tr("Images")+", ");
          break;
      case "TEMPLATE":
          resources_str+=Locale.tr("VM Templates")+", ";
          break;
      case "USER":
          resources_str+=Locale.tr("Users")+", ";
          break;
      case "GROUP":
          resources_str+=Locale.tr("Groups")+", ";
          break;
      case "CLUSTER":
          resources_str+=Locale.tr("Clusters")+", ";
          break;
      case "DATASTORE":
          resources_str+=Locale.tr("Datastores")+", ";
          break;
      case "DOCUMENT":
          resources_str+=Locale.tr("Documents")+", ";
          break;
      case "ZONE":
          resources_str+=Locale.tr("Zones")+", ";
          break;
      case "SECGROUP":
          resources_str+=Locale.tr("Security Groups")+", ";
          break;
      case "VDC":
          resources_str+=Locale.tr("VDCs")+", ";
          break;
      }
    }
    //remove ", " from end
    resources_str = resources_str.substring(0,resources_str.length-2);

    //Ops
    var ops_str="";
    var ops_array = rights.split('+');
    for (var i=0; i<ops_array.length;i++){
        ops_str += ops_array[i].toLowerCase()+", ";
    }
    ops_str= ops_str.substring(0,ops_str.length-2);

    //Zone
    var zone_str = _parseZoneAcl(zone);

    return [user_str, resources_str, belonging_to, ops_str, zone_str];
  }

  //Receives a segment of an ACL and translates:
  // * -> All
  // @1 -> Group 1 (tries to translate "1" into group name)
  // #1 -> User 1 (tries to translate "1" into username)
  //Translation of usernames and groupnames depends on
  //group and user plugins tables.
  function _parseUserAcl(user){
    var user_str="";
    if (user[0] == '*'){
      user_str = Locale.tr("All");
    } else {
      if (user[0] == '#'){
        user_str=Locale.tr("User")+" ";
        user_str+= OpenNebulaUser.getName(user.substring(1));
      }
      else if (user[0] == '@'){
        user_str=Locale.tr("Group ");
        user_str+= OpenNebulaGroup.getName(user.substring(1));
      }
    }
    return user_str;
  }

  //Similar to above, but #1 means resource with "ID 1"
  function _parseResourceAcl(user){
    var user_str="";
    if (user[0] == '*'){
      user_str = Locale.tr("All");
    } else {
      if (user[0] == '#'){
        user_str=Locale.tr("ID")+" ";
        user_str+= user.substring(1);
      }
      else if (user[0] == '@'){
        user_str=Locale.tr("Group")+" ";
        user_str+= OpenNebulaGroup.getName(user.substring(1));
      }
      else if (user[0] == '%'){
        user_str=Locale.tr("Cluster ID")+" ";
        user_str+= user.substring(1);
      }
    }
    return user_str;
  }

  //Receives a segment of an ACL and translates:
  // * -> All
  // #1 -> Zone 1 (tries to translate "1" into zone name)
  //Translation of zone names depends on
  //zone plugins tables.
  function _parseZoneAcl(zone){
    var zone_str = "";

    if (zone[0] == '*'){
      zone_str = Locale.tr("All");
    } else if (zone[0] == '#'){
      zone_str = OpenNebulaZone.getName(zone.substring(1));
    }

    return zone_str;
  }

});

define('tabs/acls-tab/actions',['require','sunstone','utils/notifier','utils/locale','./datatable','opennebula/acl','utils/common-actions','./tabId','./form-panels/create/formPanelId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var DataTable = require('./datatable');
  var OpenNebulaResource = require('opennebula/acl');
  var CommonActions = require('utils/common-actions');

  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');

  var XML_ROOT = "ACL"
  var RESOURCE = "Acl"

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "Acl.list" : _commonActions.list(),
    "Acl.refresh" : _commonActions.refresh(),
    "Acl.delete" : _commonActions.del(),
    "Acl.create" : _commonActions.create(CREATE_DIALOG_ID),
    "Acl.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID)
  };

  return _actions;
});

define('tabs/acls-tab',['require','utils/locale','./acls-tab/buttons','./acls-tab/actions','./acls-tab/datatable','./acls-tab/tabId','./acls-tab/form-panels/create'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./acls-tab/buttons');
  var Actions = require('./acls-tab/actions');
  var Table = require('./acls-tab/datatable');

  var TAB_ID = require('./acls-tab/tabId');
  var DATATABLE_ID = "dataTableAcls";

  var _dialogs = [
  ];

  var _panels = [
  ];

  var _formPanels = [
    require('./acls-tab/form-panels/create')
  ];

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("ACLs"),
    tabClass: "subTab",
    parentTab: "system-tab",
    listHeader: '<i class="fa fa-key"></i>&emsp;'+Locale.tr("Access Control Lists"),
    subheader: '<span/><small></small>&emsp;',
    resource: 'Acl',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: false}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});

define('tabs/vresources-tab',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');
  var TAB_ID = 'vresources-tab';

  var VResourcesTab = {
    tabId: TAB_ID,
    title: '<i class="fa fa-lg fa-fw fa-cloud"></i>&emsp;' + Locale.tr("Virtual Resources"),
    no_content: true
  }

  return VResourcesTab;
});

define('tabs/vms-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "VM.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "VM.create_dialog" : {
      type: "action",
      layout: "create",
      alwaysActive: true
    },
    "VM.chown" : {
      type: "confirm_with_select",
      text: Locale.tr("Change owner"),
      select: "User",
      layout: "user_select",
      tip: Locale.tr("Select the new owner") + ":"
    },
    "VM.chgrp" : {
      type: "confirm_with_select",
      text: Locale.tr("Change group"),
      select: "Group",
      layout: "user_select",
      tip: Locale.tr("Select the new group") + ":"
    },
    "VM.deploy" : {
      type: "action",
      text: Locale.tr("Deploy"),
      tip: Locale.tr("This will deploy the selected VMs on the chosen host"),
      layout: "vmsplanification_buttons",
      custom_classes : "state-dependent"
    },
    "VM.migrate" : {
      type: "action",
      text: Locale.tr("Migrate"),
      tip: Locale.tr("This will migrate the selected VMs to the chosen host"),
      layout: "vmsplanification_buttons",
      custom_classes : "state-dependent"
    },
    "VM.migrate_live" : {
      type: "action",
      text: Locale.tr("Migrate") + ' <span class="label secondary radius">live</span>',
      tip: Locale.tr("This will live-migrate the selected VMs to the chosen host"),
      layout: "vmsplanification_buttons",
      custom_classes : "state-dependent"
    },
    "VM.hold" : {
      type: "action",
      text: Locale.tr("Hold"),
      tip: Locale.tr("This will hold selected pending VMs from being deployed"),
      layout: "vmsplanification_buttons",
      custom_classes : "state-dependent"
    },
    "VM.release" : {
      type: "action",
      text: Locale.tr("Release"),
      layout: "vmsplanification_buttons",
      tip: Locale.tr("This will release held machines"),
      custom_classes : "state-dependent"
    },
    "VM.suspend" : {
      type: "action",
      text: Locale.tr("Suspend"),
      layout: "vmspause_buttons",
      tip: Locale.tr("This will suspend selected machines"),
      custom_classes : "state-dependent"
    },
    "VM.resume" : {
      type: "action",
      text: '<i class="fa fa-play"/>',
      layout: "vmsplay_buttons",
      tip: Locale.tr("This will resume selected VMs"),
      custom_classes : "state-dependent"
    },
    "VM.stop" : {
      type: "action",
      text: Locale.tr("Stop"),
      layout: "vmsstop_buttons",
      tip: Locale.tr("This will stop selected VMs"),
      custom_classes : "state-dependent"
    },
    "VM.reboot" : {
      type: "action",
      text: Locale.tr("Reboot"),
      layout: "vmsrepeat_buttons",
      tip: Locale.tr("This will send a reboot action to running VMs"),
      custom_classes : "state-dependent"
    },
    "VM.reboot_hard" : {
      type: "action",
      text: Locale.tr("Reboot") + ' <span class="label secondary radius">hard</span>',
      layout: "vmsrepeat_buttons",
      tip: Locale.tr("This will perform a hard reboot on selected VMs"),
      custom_classes : "state-dependent"
    },
    "VM.poweroff" : {
      type: "action",
      text: Locale.tr("Power Off"),
      layout: "vmspause_buttons",
      tip: Locale.tr("This will send a power off signal to running VMs. They can be resumed later."),
      custom_classes : "state-dependent"
    },
    "VM.poweroff_hard" : {
      type: "action",
      text: Locale.tr("Power Off") + ' <span class="label secondary radius">hard</span>',
      layout: "vmspause_buttons",
      tip: Locale.tr("This will send a forced power off signal to running VMs. They can be resumed later."),
      custom_classes : "state-dependent"
    },
    "VM.undeploy" : {
      type: "action",
      text: Locale.tr("Undeploy"),
      layout: "vmsstop_buttons",
      tip: Locale.tr("Shuts down the given VM. The VM is saved in the system Datastore."),
      custom_classes : "state-dependent"
    },
    "VM.undeploy_hard" : {
      type: "action",
      text: Locale.tr("Undeploy") + ' <span class="label secondary radius">hard</span>',
      layout: "vmsstop_buttons",
      tip: Locale.tr("Shuts down the given VM. The VM is saved in the system Datastore."),
      custom_classes : "state-dependent"
    },
    "VM.shutdown" : {
      type: "confirm",
      text: Locale.tr("Shutdown"),
      layout: "vmsdelete_buttons",
      tip: Locale.tr("This will initiate the shutdown process in the selected VMs"),
      custom_classes : "state-dependent"
    },
    "VM.shutdown_hard" : {
      type: "confirm",
      text: Locale.tr("Shutdown") + ' <span class="label secondary radius">hard</span>',
      layout: "vmsdelete_buttons",
      tip: Locale.tr("This will initiate the shutdown-hard (forced) process in the selected VMs"),
      custom_classes : "state-dependent"
    },

    "VM.delete" : {
      type: "confirm",
      text: Locale.tr("Delete"),
      layout: "vmsdelete_buttons",
      tip: Locale.tr("This will delete the selected VMs from the database"),
      custom_classes : "state-dependent"
    },
    "VM.delete_recreate" : {
      type: "confirm",
      text: Locale.tr("Delete") + ' <span class="label secondary radius">recreate</span>',
      layout: "vmsrepeat_buttons",
      tip: Locale.tr("This will delete and recreate VMs to PENDING state"),
      custom_classes : "state-dependent"
    },
    "VM.resched" : {
      type: "action",
      text: Locale.tr("Reschedule"),
      layout: "vmsplanification_buttons",
      tip: Locale.tr("This will reschedule selected VMs"),
      custom_classes : "state-dependent"
    },
    "VM.unresched" : {
      type: "action",
      text: Locale.tr("Un-Reschedule"),
      layout: "vmsplanification_buttons",
      tip: Locale.tr("This will cancel the rescheduling for the selected VMs"),
      custom_classes : "state-dependent"
    },
    "VM.recover" : {
      type: "confirm_with_select",
      text: Locale.tr("Recover"),
      layout: "vmsplanification_buttons",
      custom_select: '<select class="resource_list_select">\
                    <option value="2">'              + Locale.tr("retry") + '</option>\
                    <option value="1">'              + Locale.tr("success") + '</option>\
                    <option value="0">'              + Locale.tr("failure") + '</option>\
                    </select>'              ,
      tip: Locale.tr("Recovers a stuck VM that is waiting for a driver operation. \
                    The recovery may be done by failing, succeeding or retrying the current operation. \
                    YOU NEED TO MANUALLY CHECK THE VM STATUS ON THE HOST, to decide if the operation \
                    was successful or not, or if it can be retried."),
      custom_classes : "state-dependent"
    },
    "VM.startvnc" : {
      type: "action",
      text: '<i class="fa fa-desktop" style="color: rgb(111, 111, 111)"/> ' + Locale.tr("VNC"),
      custom_classes: "only-right-info vnc-right-info",
      tip: Locale.tr("VNC")
    },
    "VM.startspice" : {
      type: "action",
      text: '<i class="fa fa-desktop" style="color: rgb(111, 111, 111)"/> ' + Locale.tr("SPICE"),
      custom_classes: "only-right-info spice-right-info",
      tip: Locale.tr("SPICE")
    },
    "VM.saveas_template" : {
      type: "action",
      text: '<i class="fa fa-save"/>',
      tip: Locale.tr("This Virtual Machine will be saved in a new Template. You can then create a new Virtual Machine using this Template"),
      custom_classes : "state-dependent"
    },
  }

  return Buttons;
})
;
/*
 * noVNC: HTML5 VNC client
 * Copyright (C) 2012 Joel Martin
 * Licensed under MPL 2.0 (see LICENSE.txt)
 *
 * See README.md for usage and integration instructions.
 */

/* jshint white: false, nonstandard: true */
/*global window, console, document, navigator, ActiveXObject, INCLUDE_URI */

// Globals defined here
var Util = {};


/*
 * Make arrays quack
 */

var addFunc = function (cl, name, func) {
    if (!cl.prototype[name]) {
        Object.defineProperty(cl.prototype, name, { enumerable: false, value: func });
    }
};

addFunc(Array, 'push8', function (num) {
    "use strict";
    this.push(num & 0xFF);
});

addFunc(Array, 'push16', function (num) {
    "use strict";
    this.push((num >> 8) & 0xFF,
              num & 0xFF);
});

addFunc(Array, 'push32', function (num) {
    "use strict";
    this.push((num >> 24) & 0xFF,
              (num >> 16) & 0xFF,
              (num >>  8) & 0xFF,
              num & 0xFF);
});

// IE does not support map (even in IE9)
//This prototype is provided by the Mozilla foundation and
//is distributed under the MIT license.
//http://www.ibiblio.org/pub/Linux/LICENSES/mit.license
addFunc(Array, 'map', function (fun /*, thisp*/) {
    "use strict";
    var len = this.length;
    if (typeof fun != "function") {
        throw new TypeError();
    }

    var res = new Array(len);
    var thisp = arguments[1];
    for (var i = 0; i < len; i++) {
        if (i in this) {
            res[i] = fun.call(thisp, this[i], i, this);
        }
    }

    return res;
});

// IE <9 does not support indexOf
//This prototype is provided by the Mozilla foundation and
//is distributed under the MIT license.
//http://www.ibiblio.org/pub/Linux/LICENSES/mit.license
addFunc(Array, 'indexOf', function (elt /*, from*/) {
    "use strict";
    var len = this.length >>> 0;

    var from = Number(arguments[1]) || 0;
    from = (from < 0) ? Math.ceil(from) : Math.floor(from);
    if (from < 0) {
        from += len;
    }

    for (; from < len; from++) {
        if (from in this &&
                this[from] === elt) {
            return from;
        }
    }
    return -1;
});

// From https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/keys
if (!Object.keys) {
    Object.keys = (function () {
        'use strict';
        var hasOwnProperty = Object.prototype.hasOwnProperty,
            hasDontEnumBug = !({toString: null}).propertyIsEnumerable('toString'),
            dontEnums = [
                'toString',
                'toLocaleString',
                'valueOf',
                'hasOwnProperty',
                'isPrototypeOf',
                'propertyIsEnumerable',
                'constructor'
            ],
            dontEnumsLength = dontEnums.length;

        return function (obj) {
            if (typeof obj !== 'object' && (typeof obj !== 'function' || obj === null)) {
                throw new TypeError('Object.keys called on non-object');
            }

            var result = [], prop, i;

            for (prop in obj) {
                if (hasOwnProperty.call(obj, prop)) {
                    result.push(prop);
                }
            }

            if (hasDontEnumBug) {
                for (i = 0; i < dontEnumsLength; i++) {
                    if (hasOwnProperty.call(obj, dontEnums[i])) {
                        result.push(dontEnums[i]);
                    }
                }
            }
            return result;
        };
    })();
}

// PhantomJS 1.x doesn't support bind,
// so leave this in until PhantomJS 2.0 is released
//This prototype is provided by the Mozilla foundation and
//is distributed under the MIT license.
//http://www.ibiblio.org/pub/Linux/LICENSES/mit.license
addFunc(Function, 'bind', function (oThis) {
    if (typeof this !== "function") {
        // closest thing possible to the ECMAScript 5
        // internal IsCallable function
        throw new TypeError("Function.prototype.bind - " +
                            "what is trying to be bound is not callable");
    }

    var aArgs = Array.prototype.slice.call(arguments, 1),
            fToBind = this,
            fNOP = function () {},
            fBound = function () {
                return fToBind.apply(this instanceof fNOP && oThis ? this
                                                                   : oThis,
                                     aArgs.concat(Array.prototype.slice.call(arguments)));
            };

    fNOP.prototype = this.prototype;
    fBound.prototype = new fNOP();

    return fBound;
});

//
// requestAnimationFrame shim with setTimeout fallback
//

window.requestAnimFrame = (function () {
    "use strict";
    return  window.requestAnimationFrame       ||
            window.webkitRequestAnimationFrame ||
            window.mozRequestAnimationFrame    ||
            window.oRequestAnimationFrame      ||
            window.msRequestAnimationFrame     ||
            function (callback) {
                window.setTimeout(callback, 1000 / 60);
            };
})();

/*
 * ------------------------------------------------------
 * Namespaced in Util
 * ------------------------------------------------------
 */

/*
 * Logging/debug routines
 */

Util._log_level = 'warn';
Util.init_logging = function (level) {
    "use strict";
    if (typeof level === 'undefined') {
        level = Util._log_level;
    } else {
        Util._log_level = level;
    }
    if (typeof window.console === "undefined") {
        if (typeof window.opera !== "undefined") {
            window.console = {
                'log'  : window.opera.postError,
                'warn' : window.opera.postError,
                'error': window.opera.postError
            };
        } else {
            window.console = {
                'log'  : function (m) {},
                'warn' : function (m) {},
                'error': function (m) {}
            };
        }
    }

    Util.Debug = Util.Info = Util.Warn = Util.Error = function (msg) {};
    /* jshint -W086 */
    switch (level) {
        case 'debug':
            Util.Debug = function (msg) { console.log(msg); };
        case 'info':
            Util.Info  = function (msg) { console.log(msg); };
        case 'warn':
            Util.Warn  = function (msg) { console.warn(msg); };
        case 'error':
            Util.Error = function (msg) { console.error(msg); };
        case 'none':
            break;
        default:
            throw new Error("invalid logging type '" + level + "'");
    }
    /* jshint +W086 */
};
Util.get_logging = function () {
    return Util._log_level;
};
// Initialize logging level
Util.init_logging();

Util.make_property = function (proto, name, mode, type) {
    "use strict";

    var getter;
    if (type === 'arr') {
        getter = function (idx) {
            if (typeof idx !== 'undefined') {
                return this['_' + name][idx];
            } else {
                return this['_' + name];
            }
        };
    } else {
        getter = function () {
            return this['_' + name];
        };
    }

    var make_setter = function (process_val) {
        if (process_val) {
            return function (val, idx) {
                if (typeof idx !== 'undefined') {
                    this['_' + name][idx] = process_val(val);
                } else {
                    this['_' + name] = process_val(val);
                }
            };
        } else {
            return function (val, idx) {
                if (typeof idx !== 'undefined') {
                    this['_' + name][idx] = val;
                } else {
                    this['_' + name] = val;
                }
            };
        }
    };

    var setter;
    if (type === 'bool') {
        setter = make_setter(function (val) {
            if (!val || (val in {'0': 1, 'no': 1, 'false': 1})) {
                return false;
            } else {
                return true;
            }
        });
    } else if (type === 'int') {
        setter = make_setter(function (val) { return parseInt(val, 10); });
    } else if (type === 'float') {
        setter = make_setter(parseFloat);
    } else if (type === 'str') {
        setter = make_setter(String);
    } else if (type === 'func') {
        setter = make_setter(function (val) {
            if (!val) {
                return function () {};
            } else {
                return val;
            }
        });
    } else if (type === 'arr' || type === 'dom' || type == 'raw') {
        setter = make_setter();
    } else {
        throw new Error('Unknown property type ' + type);  // some sanity checking
    }

    // set the getter
    if (typeof proto['get_' + name] === 'undefined') {
        proto['get_' + name] = getter;
    }

    // set the setter if needed
    if (typeof proto['set_' + name] === 'undefined') {
        if (mode === 'rw') {
            proto['set_' + name] = setter;
        } else if (mode === 'wo') {
            proto['set_' + name] = function (val, idx) {
                if (typeof this['_' + name] !== 'undefined') {
                    throw new Error(name + " can only be set once");
                }
                setter.call(this, val, idx);
            };
        }
    }

    // make a special setter that we can use in set defaults
    proto['_raw_set_' + name] = function (val, idx) {
        setter.call(this, val, idx);
        //delete this['_init_set_' + name];  // remove it after use
    };
};

Util.make_properties = function (constructor, arr) {
    "use strict";
    for (var i = 0; i < arr.length; i++) {
        Util.make_property(constructor.prototype, arr[i][0], arr[i][1], arr[i][2]);
    }
};

Util.set_defaults = function (obj, conf, defaults) {
    var defaults_keys = Object.keys(defaults);
    var conf_keys = Object.keys(conf);
    var keys_obj = {};
    var i;
    for (i = 0; i < defaults_keys.length; i++) { keys_obj[defaults_keys[i]] = 1; }
    for (i = 0; i < conf_keys.length; i++) { keys_obj[conf_keys[i]] = 1; }
    var keys = Object.keys(keys_obj);

    for (i = 0; i < keys.length; i++) {
        var setter = obj['_raw_set_' + keys[i]];
        if (!setter) {
          Util.Warn('Invalid property ' + keys[i]);
          continue;
        }

        if (keys[i] in conf) {
            setter.call(obj, conf[keys[i]]);
        } else {
            setter.call(obj, defaults[keys[i]]);
        }
    }
};

/*
 * Decode from UTF-8
 */
Util.decodeUTF8 = function (utf8string) {
    "use strict";
    return decodeURIComponent(escape(utf8string));
};



/*
 * Cross-browser routines
 */


// Dynamically load scripts without using document.write()
// Reference: http://unixpapa.com/js/dyna.html
//
// Handles the case where load_scripts is invoked from a script that
// itself is loaded via load_scripts. Once all scripts are loaded the
// window.onscriptsloaded handler is called (if set).
Util.get_include_uri = function () {
    return (typeof INCLUDE_URI !== "undefined") ? INCLUDE_URI : "include/";
};
Util._loading_scripts = [];
Util._pending_scripts = [];
Util.load_scripts = function (files) {
    "use strict";
    var head = document.getElementsByTagName('head')[0], script,
        ls = Util._loading_scripts, ps = Util._pending_scripts;

    var loadFunc = function (e) {
        while (ls.length > 0 && (ls[0].readyState === 'loaded' ||
                                 ls[0].readyState === 'complete')) {
            // For IE, append the script to trigger execution
            var s = ls.shift();
            //console.log("loaded script: " + s.src);
            head.appendChild(s);
        }
        if (!this.readyState ||
            (Util.Engine.presto && this.readyState === 'loaded') ||
            this.readyState === 'complete') {
            if (ps.indexOf(this) >= 0) {
                this.onload = this.onreadystatechange = null;
                //console.log("completed script: " + this.src);
                ps.splice(ps.indexOf(this), 1);

                // Call window.onscriptsload after last script loads
                if (ps.length === 0 && window.onscriptsload) {
                    window.onscriptsload();
                }
            }
        }
    };

    for (var f = 0; f < files.length; f++) {
        script = document.createElement('script');
        script.type = 'text/javascript';
        script.src = Util.get_include_uri() + files[f];
        //console.log("loading script: " + script.src);
        script.onload = script.onreadystatechange = loadFunc;
        // In-order script execution tricks
        if (Util.Engine.trident) {
            // For IE wait until readyState is 'loaded' before
            // appending it which will trigger execution
            // http://wiki.whatwg.org/wiki/Dynamic_Script_Execution_Order
            ls.push(script);
        } else {
            // For webkit and firefox set async=false and append now
            // https://developer.mozilla.org/en-US/docs/HTML/Element/script
            script.async = false;
            head.appendChild(script);
        }
        ps.push(script);
    }
};


// Get DOM element position on page
//  This solution is based based on http://www.greywyvern.com/?post=331
//  Thanks to Brian Huisman AKA GreyWyvern!
Util.getPosition = (function () {
    "use strict";
    function getStyle(obj, styleProp) {
        var y;
        if (obj.currentStyle) {
            y = obj.currentStyle[styleProp];
        } else if (window.getComputedStyle)
            y = window.getComputedStyle(obj, null)[styleProp];
        return y;
    }

    function scrollDist() {
        var myScrollTop = 0, myScrollLeft = 0;
        var html = document.getElementsByTagName('html')[0];

        // get the scrollTop part
        if (html.scrollTop && document.documentElement.scrollTop) {
            myScrollTop = html.scrollTop;
        } else if (html.scrollTop || document.documentElement.scrollTop) {
            myScrollTop = html.scrollTop + document.documentElement.scrollTop;
        } else if (document.body.scrollTop) {
            myScrollTop = document.body.scrollTop;
        } else {
            myScrollTop = 0;
        }

        // get the scrollLeft part
        if (html.scrollLeft && document.documentElement.scrollLeft) {
            myScrollLeft = html.scrollLeft;
        } else if (html.scrollLeft || document.documentElement.scrollLeft) {
            myScrollLeft = html.scrollLeft + document.documentElement.scrollLeft;
        } else if (document.body.scrollLeft) {
            myScrollLeft = document.body.scrollLeft;
        } else {
            myScrollLeft = 0;
        }

        return [myScrollLeft, myScrollTop];
    }

    return function (obj) {
        var curleft = 0, curtop = 0, scr = obj, fixed = false;
        while ((scr = scr.parentNode) && scr != document.body) {
            curleft -= scr.scrollLeft || 0;
            curtop -= scr.scrollTop || 0;
            if (getStyle(scr, "position") == "fixed") {
                fixed = true;
            }
        }
        if (fixed && !window.opera) {
            var scrDist = scrollDist();
            curleft += scrDist[0];
            curtop += scrDist[1];
        }

        do {
            curleft += obj.offsetLeft;
            curtop += obj.offsetTop;
        } while ((obj = obj.offsetParent));

        return {'x': curleft, 'y': curtop};
    };
})();


// Get mouse event position in DOM element
Util.getEventPosition = function (e, obj, scale) {
    "use strict";
    var evt, docX, docY, pos;
    //if (!e) evt = window.event;
    evt = (e ? e : window.event);
    evt = (evt.changedTouches ? evt.changedTouches[0] : evt.touches ? evt.touches[0] : evt);
    if (evt.pageX || evt.pageY) {
        docX = evt.pageX;
        docY = evt.pageY;
    } else if (evt.clientX || evt.clientY) {
        docX = evt.clientX + document.body.scrollLeft +
            document.documentElement.scrollLeft;
        docY = evt.clientY + document.body.scrollTop +
            document.documentElement.scrollTop;
    }
    pos = Util.getPosition(obj);
    if (typeof scale === "undefined") {
        scale = 1;
    }
    var realx = docX - pos.x;
    var realy = docY - pos.y;
    var x = Math.max(Math.min(realx, obj.width - 1), 0);
    var y = Math.max(Math.min(realy, obj.height - 1), 0);
    return {'x': x / scale, 'y': y / scale, 'realx': realx / scale, 'realy': realy / scale};
};


// Event registration. Based on: http://www.scottandrew.com/weblog/articles/cbs-events
Util.addEvent = function (obj, evType, fn) {
    "use strict";
    if (obj.attachEvent) {
        var r = obj.attachEvent("on" + evType, fn);
        return r;
    } else if (obj.addEventListener) {
        obj.addEventListener(evType, fn, false);
        return true;
    } else {
        throw new Error("Handler could not be attached");
    }
};

Util.removeEvent = function (obj, evType, fn) {
    "use strict";
    if (obj.detachEvent) {
        var r = obj.detachEvent("on" + evType, fn);
        return r;
    } else if (obj.removeEventListener) {
        obj.removeEventListener(evType, fn, false);
        return true;
    } else {
        throw new Error("Handler could not be removed");
    }
};

Util.stopEvent = function (e) {
    "use strict";
    if (e.stopPropagation) { e.stopPropagation(); }
    else                   { e.cancelBubble = true; }

    if (e.preventDefault)  { e.preventDefault(); }
    else                   { e.returnValue = false; }
};


// Set browser engine versions. Based on mootools.
Util.Features = {xpath: !!(document.evaluate), air: !!(window.runtime), query: !!(document.querySelector)};

(function () {
    "use strict";
    // 'presto': (function () { return (!window.opera) ? false : true; }()),
    var detectPresto = function () {
        return !!window.opera;
    };

    // 'trident': (function () { return (!window.ActiveXObject) ? false : ((window.XMLHttpRequest) ? ((document.querySelectorAll) ? 6 : 5) : 4);
    var detectTrident = function () {
        if (!window.ActiveXObject) {
            return false;
        } else {
            if (window.XMLHttpRequest) {
                return (document.querySelectorAll) ? 6 : 5;
            } else {
                return 4;
            }
        }
    };

    // 'webkit': (function () { try { return (navigator.taintEnabled) ? false : ((Util.Features.xpath) ? ((Util.Features.query) ? 525 : 420) : 419); } catch (e) { return false; } }()),
    var detectInitialWebkit = function () {
        try {
            if (navigator.taintEnabled) {
                return false;
            } else {
                if (Util.Features.xpath) {
                    return (Util.Features.query) ? 525 : 420;
                } else {
                    return 419;
                }
            }
        } catch (e) {
            return false;
        }
    };

    var detectActualWebkit = function (initial_ver) {
        var re = /WebKit\/([0-9\.]*) /;
        var str_ver = (navigator.userAgent.match(re) || ['', initial_ver])[1];
        return parseFloat(str_ver, 10);
    };

    // 'gecko': (function () { return (!document.getBoxObjectFor && window.mozInnerScreenX == null) ? false : ((document.getElementsByClassName) ? 19ssName) ? 19 : 18 : 18); }())
    var detectGecko = function () {
        /* jshint -W041 */
        if (!document.getBoxObjectFor && window.mozInnerScreenX == null) {
            return false;
        } else {
            return (document.getElementsByClassName) ? 19 : 18;
        }
        /* jshint +W041 */
    };

    Util.Engine = {
        // Version detection break in Opera 11.60 (errors on arguments.callee.caller reference)
        //'presto': (function() {
        //         return (!window.opera) ? false : ((arguments.callee.caller) ? 960 : ((document.getElementsByClassName) ? 950 : 925)); }()),
        'presto': detectPresto(),
        'trident': detectTrident(),
        'webkit': detectInitialWebkit(),
        'gecko': detectGecko(),
    };

    if (Util.Engine.webkit) {
        // Extract actual webkit version if available
        Util.Engine.webkit = detectActualWebkit(Util.Engine.webkit);
    }
})();

Util.Flash = (function () {
    "use strict";
    var v, version;
    try {
        v = navigator.plugins['Shockwave Flash'].description;
    } catch (err1) {
        try {
            v = new ActiveXObject('ShockwaveFlash.ShockwaveFlash').GetVariable('$version');
        } catch (err2) {
            v = '0 r0';
        }
    }
    version = v.match(/\d+/g);
    return {version: parseInt(version[0] || 0 + '.' + version[1], 10) || 0, build: parseInt(version[2], 10) || 0};
}());

define("vnc-util", (function (global) {
    return function () {
        var ret, fn;
        return ret || global.Util;
    };
}(this)));

/*
 * noVNC: HTML5 VNC client
 * Copyright (C) 2012 Joel Martin
 * Copyright (C) 2013 NTT corp.
 * Licensed under MPL 2.0 (see LICENSE.txt)
 *
 * See README.md for usage and integration instructions.
 */

/*jslint bitwise: false, white: false, browser: true, devel: true */
/*global Util, window, document */

// Globals defined here
var WebUtil = {}, $D;

/*
 * Simple DOM selector by ID
 */
if (!window.$D) {
    window.$D = function (id) {
        if (document.getElementById) {
            return document.getElementById(id);
        } else if (document.all) {
            return document.all[id];
        } else if (document.layers) {
            return document.layers[id];
        }
        return undefined;
    };
}


/*
 * ------------------------------------------------------
 * Namespaced in WebUtil
 * ------------------------------------------------------
 */

// init log level reading the logging HTTP param
WebUtil.init_logging = function (level) {
    "use strict";
    if (typeof level !== "undefined") {
        Util._log_level = level;
    } else {
        var param = document.location.href.match(/logging=([A-Za-z0-9\._\-]*)/);
        Util._log_level = (param || ['', Util._log_level])[1];
    }
    Util.init_logging();
};


WebUtil.dirObj = function (obj, depth, parent) {
    "use strict";
    if (! depth) { depth = 2; }
    if (! parent) { parent = ""; }

    // Print the properties of the passed-in object
    var msg = "";
    for (var i in obj) {
        if ((depth > 1) && (typeof obj[i] === "object")) {
            // Recurse attributes that are objects
            msg += WebUtil.dirObj(obj[i], depth - 1, parent + "." + i);
        } else {
            //val = new String(obj[i]).replace("\n", " ");
            var val = "";
            if (typeof(obj[i]) === "undefined") {
                val = "undefined";
            } else {
                val = obj[i].toString().replace("\n", " ");
            }
            if (val.length > 30) {
                val = val.substr(0, 30) + "...";
            }
            msg += parent + "." + i + ": " + val + "\n";
        }
    }
    return msg;
};

// Read a query string variable
WebUtil.getQueryVar = function (name, defVal) {
    "use strict";
    var re = new RegExp('.*[?&]' + name + '=([^&#]*)'),
        match = document.location.href.match(re);
    if (typeof defVal === 'undefined') { defVal = null; }
    if (match) {
        return decodeURIComponent(match[1]);
    } else {
        return defVal;
    }
};


/*
 * Cookie handling. Dervied from: http://www.quirksmode.org/js/cookies.html
 */

// No days means only for this browser session
WebUtil.createCookie = function (name, value, days) {
    "use strict";
    var date, expires;
    if (days) {
        date = new Date();
        date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
        expires = "; expires=" + date.toGMTString();
    } else {
        expires = "";
    }

    var secure;
    if (document.location.protocol === "https:") {
        secure = "; secure";
    } else {
        secure = "";
    }
    document.cookie = name + "=" + value + expires + "; path=/" + secure;
};

WebUtil.readCookie = function (name, defaultValue) {
    "use strict";
    var nameEQ = name + "=",
        ca = document.cookie.split(';');

    for (var i = 0; i < ca.length; i += 1) {
        var c = ca[i];
        while (c.charAt(0) === ' ') { c = c.substring(1, c.length); }
        if (c.indexOf(nameEQ) === 0) { return c.substring(nameEQ.length, c.length); }
    }
    return (typeof defaultValue !== 'undefined') ? defaultValue : null;
};

WebUtil.eraseCookie = function (name) {
    "use strict";
    WebUtil.createCookie(name, "", -1);
};

/*
 * Setting handling.
 */

WebUtil.initSettings = function (callback /*, ...callbackArgs */) {
    "use strict";
    var callbackArgs = Array.prototype.slice.call(arguments, 1);
    if (window.chrome && window.chrome.storage) {
        window.chrome.storage.sync.get(function (cfg) {
            WebUtil.settings = cfg;
            console.log(WebUtil.settings);
            if (callback) {
                callback.apply(this, callbackArgs);
            }
        });
    } else {
        // No-op
        if (callback) {
            callback.apply(this, callbackArgs);
        }
    }
};

// No days means only for this browser session
WebUtil.writeSetting = function (name, value) {
    "use strict";
    if (window.chrome && window.chrome.storage) {
        //console.log("writeSetting:", name, value);
        if (WebUtil.settings[name] !== value) {
            WebUtil.settings[name] = value;
            window.chrome.storage.sync.set(WebUtil.settings);
        }
    } else {
        localStorage.setItem(name, value);
    }
};

WebUtil.readSetting = function (name, defaultValue) {
    "use strict";
    var value;
    if (window.chrome && window.chrome.storage) {
        value = WebUtil.settings[name];
    } else {
        value = localStorage.getItem(name);
    }
    if (typeof value === "undefined") {
        value = null;
    }
    if (value === null && typeof defaultValue !== undefined) {
        return defaultValue;
    } else {
        return value;
    }
};

WebUtil.eraseSetting = function (name) {
    "use strict";
    if (window.chrome && window.chrome.storage) {
        window.chrome.storage.sync.remove(name);
        delete WebUtil.settings[name];
    } else {
        localStorage.removeItem(name);
    }
};

/*
 * Alternate stylesheet selection
 */
WebUtil.getStylesheets = function () {
    "use strict";
    var links = document.getElementsByTagName("link");
    var sheets = [];

    for (var i = 0; i < links.length; i += 1) {
        if (links[i].title &&
            links[i].rel.toUpperCase().indexOf("STYLESHEET") > -1) {
            sheets.push(links[i]);
        }
    }
    return sheets;
};

// No sheet means try and use value from cookie, null sheet used to
// clear all alternates.
WebUtil.selectStylesheet = function (sheet) {
    "use strict";
    if (typeof sheet === 'undefined') {
        sheet = 'default';
    }

    var sheets = WebUtil.getStylesheets();
    for (var i = 0; i < sheets.length; i += 1) {
        var link = sheets[i];
        if (link.title === sheet) {
            Util.Debug("Using stylesheet " + sheet);
            link.disabled = false;
        } else {
            //Util.Debug("Skipping stylesheet " + link.title);
            link.disabled = true;
        }
    }
    return sheet;
};

define("vnc-webutil", ["vnc-util"], function(){});

/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

// From: http://hg.mozilla.org/mozilla-central/raw-file/ec10630b1a54/js/src/devtools/jint/sunspider/string-base64.js

/*jslint white: false */
/*global console */

var Base64 = {
    /* Convert data (an array of integers) to a Base64 string. */
    toBase64Table : 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/='.split(''),
    base64Pad     : '=',

    encode: function (data) {
        "use strict";
        var result = '';
        var toBase64Table = Base64.toBase64Table;
        var length = data.length;
        var lengthpad = (length % 3);
        // Convert every three bytes to 4 ascii characters.

        for (var i = 0; i < (length - 2); i += 3) {
            result += toBase64Table[data[i] >> 2];
            result += toBase64Table[((data[i] & 0x03) << 4) + (data[i + 1] >> 4)];
            result += toBase64Table[((data[i + 1] & 0x0f) << 2) + (data[i + 2] >> 6)];
            result += toBase64Table[data[i + 2] & 0x3f];
        }

        // Convert the remaining 1 or 2 bytes, pad out to 4 characters.
        var j = 0;
        if (lengthpad === 2) {
            j = length - lengthpad;
            result += toBase64Table[data[j] >> 2];
            result += toBase64Table[((data[j] & 0x03) << 4) + (data[j + 1] >> 4)];
            result += toBase64Table[(data[j + 1] & 0x0f) << 2];
            result += toBase64Table[64];
        } else if (lengthpad === 1) {
            j = length - lengthpad;
            result += toBase64Table[data[j] >> 2];
            result += toBase64Table[(data[j] & 0x03) << 4];
            result += toBase64Table[64];
            result += toBase64Table[64];
        }

        return result;
    },

    /* Convert Base64 data to a string */
    /* jshint -W013 */
    toBinaryTable : [
        -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
        -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
        -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,62, -1,-1,-1,63,
        52,53,54,55, 56,57,58,59, 60,61,-1,-1, -1, 0,-1,-1,
        -1, 0, 1, 2,  3, 4, 5, 6,  7, 8, 9,10, 11,12,13,14,
        15,16,17,18, 19,20,21,22, 23,24,25,-1, -1,-1,-1,-1,
        -1,26,27,28, 29,30,31,32, 33,34,35,36, 37,38,39,40,
        41,42,43,44, 45,46,47,48, 49,50,51,-1, -1,-1,-1,-1
    ],
    /* jshint +W013 */

    decode: function (data, offset) {
        "use strict";
        offset = typeof(offset) !== 'undefined' ? offset : 0;
        var toBinaryTable = Base64.toBinaryTable;
        var base64Pad = Base64.base64Pad;
        var result, result_length;
        var leftbits = 0; // number of bits decoded, but yet to be appended
        var leftdata = 0; // bits decoded, but yet to be appended
        var data_length = data.indexOf('=') - offset;

        if (data_length < 0) { data_length = data.length - offset; }

        /* Every four characters is 3 resulting numbers */
        result_length = (data_length >> 2) * 3 + Math.floor((data_length % 4) / 1.5);
        result = new Array(result_length);

        // Convert one by one.
        for (var idx = 0, i = offset; i < data.length; i++) {
            var c = toBinaryTable[data.charCodeAt(i) & 0x7f];
            var padding = (data.charAt(i) === base64Pad);
            // Skip illegal characters and whitespace
            if (c === -1) {
                console.error("Illegal character code " + data.charCodeAt(i) + " at position " + i);
                continue;
            }
          
            // Collect data into leftdata, update bitcount
            leftdata = (leftdata << 6) | c;
            leftbits += 6;

            // If we have 8 or more bits, append 8 bits to the result
            if (leftbits >= 8) {
                leftbits -= 8;
                // Append if not padding.
                if (!padding) {
                    result[idx++] = (leftdata >> leftbits) & 0xff;
                }
                leftdata &= (1 << leftbits) - 1;
            }
        }

        // If there are any bits left, the base64 string was corrupted
        if (leftbits) {
            err = new Error('Corrupted base64 string');
            err.name = 'Base64-Error';
            throw err;
        }

        return result;
    }
}; /* End of Base64 namespace */
;
define("vnc-base64", ["vnc-util"], function(){});

/*
 * Websock: high-performance binary WebSockets
 * Copyright (C) 2012 Joel Martin
 * Licensed under MPL 2.0 (see LICENSE.txt)
 *
 * Websock is similar to the standard WebSocket object but Websock
 * enables communication with raw TCP sockets (i.e. the binary stream)
 * via websockify. This is accomplished by base64 encoding the data
 * stream between Websock and websockify.
 *
 * Websock has built-in receive queue buffering; the message event
 * does not contain actual data but is simply a notification that
 * there is new data available. Several rQ* methods are available to
 * read binary data off of the receive queue.
 */

/*jslint browser: true, bitwise: true */
/*global Util, Base64 */


// Load Flash WebSocket emulator if needed

// To force WebSocket emulator even when native WebSocket available
//window.WEB_SOCKET_FORCE_FLASH = true;
// To enable WebSocket emulator debug:
//window.WEB_SOCKET_DEBUG=1;

if (window.WebSocket && !window.WEB_SOCKET_FORCE_FLASH) {
    Websock_native = true;
} else if (window.MozWebSocket && !window.WEB_SOCKET_FORCE_FLASH) {
    Websock_native = true;
    window.WebSocket = window.MozWebSocket;
} else {
    /* no builtin WebSocket so load web_socket.js */

    Websock_native = false;
    (function () {
        window.WEB_SOCKET_SWF_LOCATION = Util.get_include_uri() +
                    "web-socket-js/WebSocketMain.swf";
        if (Util.Engine.trident) {
            Util.Debug("Forcing uncached load of WebSocketMain.swf");
            window.WEB_SOCKET_SWF_LOCATION += "?" + Math.random();
        }
        Util.load_scripts(["web-socket-js/swfobject.js",
                           "web-socket-js/web_socket.js"]);
    })();
}


function Websock() {
    "use strict";

    this._websocket = null;  // WebSocket object
    this._rQ = [];           // Receive queue
    this._rQi = 0;           // Receive queue index
    this._rQmax = 10000;     // Max receive queue size before compacting
    this._sQ = [];           // Send queue

    this._mode = 'base64';    // Current WebSocket mode: 'binary', 'base64'
    this.maxBufferedAmount = 200;

    this._eventHandlers = {
        'message': function () {},
        'open': function () {},
        'close': function () {},
        'error': function () {}
    };
}

(function () {
    "use strict";
    Websock.prototype = {
        // Getters and Setters
        get_sQ: function () {
            return this._sQ;
        },

        get_rQ: function () {
            return this._rQ;
        },

        get_rQi: function () {
            return this._rQi;
        },

        set_rQi: function (val) {
            this._rQi = val;
        },

        // Receive Queue
        rQlen: function () {
            return this._rQ.length - this._rQi;
        },

        rQpeek8: function () {
            return this._rQ[this._rQi];
        },

        rQshift8: function () {
            return this._rQ[this._rQi++];
        },

        rQskip8: function () {
            this._rQi++;
        },

        rQskipBytes: function (num) {
            this._rQi += num;
        },

        rQunshift8: function (num) {
            if (this._rQi === 0) {
                this._rQ.unshift(num);
            } else {
                this._rQi--;
                this._rQ[this._rQi] = num;
            }
        },

        rQshift16: function () {
            return (this._rQ[this._rQi++] << 8) +
                   this._rQ[this._rQi++];
        },

        rQshift32: function () {
            return (this._rQ[this._rQi++] << 24) +
                   (this._rQ[this._rQi++] << 16) +
                   (this._rQ[this._rQi++] << 8) +
                   this._rQ[this._rQi++];
        },

        rQshiftStr: function (len) {
            if (typeof(len) === 'undefined') { len = this.rQlen(); }
            var arr = this._rQ.slice(this._rQi, this._rQi + len);
            this._rQi += len;
            return String.fromCharCode.apply(null, arr);
        },

        rQshiftBytes: function (len) {
            if (typeof(len) === 'undefined') { len = this.rQlen(); }
            this._rQi += len;
            return this._rQ.slice(this._rQi - len, this._rQi);
        },

        rQslice: function (start, end) {
            if (end) {
                return this._rQ.slice(this._rQi + start, this._rQi + end);
            } else {
                return this._rQ.slice(this._rQi + start);
            }
        },

        // Check to see if we must wait for 'num' bytes (default to FBU.bytes)
        // to be available in the receive queue. Return true if we need to
        // wait (and possibly print a debug message), otherwise false.
        rQwait: function (msg, num, goback) {
            var rQlen = this._rQ.length - this._rQi; // Skip rQlen() function call
            if (rQlen < num) {
                if (goback) {
                    if (this._rQi < goback) {
                        throw new Error("rQwait cannot backup " + goback + " bytes");
                    }
                    this._rQi -= goback;
                }
                return true; // true means need more data
            }
            return false;
        },

        // Send Queue

        flush: function () {
            if (this._websocket.bufferedAmount !== 0) {
                Util.Debug("bufferedAmount: " + this._websocket.bufferedAmount);
            }

            if (this._websocket.bufferedAmount < this.maxBufferedAmount) {
                if (this._sQ.length > 0) {
                    this._websocket.send(this._encode_message());
                    this._sQ = [];
                }

                return true;
            } else {
                Util.Info("Delaying send, bufferedAmount: " +
                        this._websocket.bufferedAmount);
                return false;
            }
        },

        send: function (arr) {
           this._sQ = this._sQ.concat(arr);
           return this.flush();
        },

        send_string: function (str) {
            this.send(str.split('').map(function (chr) {
                return chr.charCodeAt(0);
            }));
        },

        // Event Handlers
        on: function (evt, handler) {
            this._eventHandlers[evt] = handler;
        },

        init: function (protocols, ws_schema) {
            this._rQ = [];
            this._rQi = 0;
            this._sQ = [];
            this._websocket = null;

            // Check for full typed array support
            var bt = false;
            if (('Uint8Array' in window) &&
                    ('set' in Uint8Array.prototype)) {
                bt = true;
            }

            // Check for full binary type support in WebSockets
            // Inspired by:
            // https://github.com/Modernizr/Modernizr/issues/370
            // https://github.com/Modernizr/Modernizr/blob/master/feature-detects/websockets/binary.js
            var wsbt = false;
            try {
                if (bt && ('binaryType' in WebSocket.prototype ||
                           !!(new WebSocket(ws_schema + '://.').binaryType))) {
                    Util.Info("Detected binaryType support in WebSockets");
                    wsbt = true;
                }
            } catch (exc) {
                // Just ignore failed test localhost connection
            }

            // Default protocols if not specified
            if (typeof(protocols) === "undefined") {
                if (wsbt) {
                    protocols = ['binary', 'base64'];
                } else {
                    protocols = 'base64';
                }
            }

            if (!wsbt) {
                if (protocols === 'binary') {
                    throw new Error('WebSocket binary sub-protocol requested but not supported');
                }

                if (typeof(protocols) === 'object') {
                    var new_protocols = [];

                    for (var i = 0; i < protocols.length; i++) {
                        if (protocols[i] === 'binary') {
                            Util.Error('Skipping unsupported WebSocket binary sub-protocol');
                        } else {
                            new_protocols.push(protocols[i]);
                        }
                    }

                    if (new_protocols.length > 0) {
                        protocols = new_protocols;
                    } else {
                        throw new Error("Only WebSocket binary sub-protocol was requested and is not supported.");
                    }
                }
            }

            return protocols;
        },

        open: function (uri, protocols) {
            var ws_schema = uri.match(/^([a-z]+):\/\//)[1];
            protocols = this.init(protocols, ws_schema);

            this._websocket = new WebSocket(uri, protocols);

            if (protocols.indexOf('binary') >= 0) {
                this._websocket.binaryType = 'arraybuffer';
            }

            this._websocket.onmessage = this._recv_message.bind(this);
            this._websocket.onopen = (function () {
                Util.Debug('>> WebSock.onopen');
                if (this._websocket.protocol) {
                    this._mode = this._websocket.protocol;
                    Util.Info("Server choose sub-protocol: " + this._websocket.protocol);
                } else {
                    this._mode = 'base64';
                    Util.Error('Server select no sub-protocol!: ' + this._websocket.protocol);
                }
                this._eventHandlers.open();
                Util.Debug("<< WebSock.onopen");
            }).bind(this);
            this._websocket.onclose = (function (e) {
                Util.Debug(">> WebSock.onclose");
                this._eventHandlers.close(e);
                Util.Debug("<< WebSock.onclose");
            }).bind(this);
            this._websocket.onerror = (function (e) {
                Util.Debug(">> WebSock.onerror: " + e);
                this._eventHandlers.error(e);
                Util.Debug("<< WebSock.onerror: " + e);
            }).bind(this);
        },

        close: function () {
            if (this._websocket) {
                if ((this._websocket.readyState === WebSocket.OPEN) ||
                        (this._websocket.readyState === WebSocket.CONNECTING)) {
                    Util.Info("Closing WebSocket connection");
                    this._websocket.close();
                }

                this._websocket.onmessage = function (e) { return; };
            }
        },

        // private methods
        _encode_message: function () {
            if (this._mode === 'binary') {
                // Put in a binary arraybuffer
                return (new Uint8Array(this._sQ)).buffer;
            } else {
                // base64 encode
                return Base64.encode(this._sQ);
            }
        },

        _decode_message: function (data) {
            if (this._mode === 'binary') {
                // push arraybuffer values onto the end
                var u8 = new Uint8Array(data);
                for (var i = 0; i < u8.length; i++) {
                    this._rQ.push(u8[i]);
                }
            } else {
                // base64 decode and concat to end
                this._rQ = this._rQ.concat(Base64.decode(data, 0));
            }
        },

        _recv_message: function (e) {
            try {
                this._decode_message(e.data);
                if (this.rQlen() > 0) {
                    this._eventHandlers.message();
                    // Compact the receive queue
                    if (this._rQ.length > this._rQmax) {
                        this._rQ = this._rQ.slice(this._rQi);
                        this._rQi = 0;
                    }
                } else {
                    Util.Debug("Ignoring empty message");
                }
            } catch (exc) {
                var exception_str = "";
                if (exc.name) {
                    exception_str += "\n    name: " + exc.name + "\n";
                    exception_str += "    message: " + exc.message + "\n";
                }

                if (typeof exc.description !== 'undefined') {
                    exception_str += "    description: " + exc.description + "\n";
                }

                if (typeof exc.stack !== 'undefined') {
                    exception_str += exc.stack;
                }

                if (exception_str.length > 0) {
                    Util.Error("recv_message, caught exception: " + exception_str);
                } else {
                    Util.Error("recv_message, caught exception: " + exc);
                }

                if (typeof exc.name !== 'undefined') {
                    this._eventHandlers.error(exc.name + ": " + exc.message);
                } else {
                    this._eventHandlers.error(exc);
                }
            }
        }
    };
})();

define("vnc-websock", ["vnc-util"], function(){});

/*
 * Ported from Flashlight VNC ActionScript implementation:
 *     http://www.wizhelp.com/flashlight-vnc/
 *
 * Full attribution follows:
 *
 * -------------------------------------------------------------------------
 *
 * This DES class has been extracted from package Acme.Crypto for use in VNC.
 * The unnecessary odd parity code has been removed.
 *
 * These changes are:
 *  Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *

 * DesCipher - the DES encryption method
 *
 * The meat of this code is by Dave Zimmerman <dzimm@widget.com>, and is:
 *
 * Copyright (c) 1996 Widget Workshop, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL or COMMERCIAL purposes and
 * without fee is hereby granted, provided that this copyright notice is kept 
 * intact. 
 * 
 * WIDGET WORKSHOP MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
 * OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. WIDGET WORKSHOP SHALL NOT BE LIABLE
 * FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  WIDGET WORKSHOP
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 *
 *
 * The rest is:
 *
 * Copyright (C) 1996 by Jef Poskanzer <jef@acme.com>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the ACME Labs Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.acme.com/java/
 */

/* jslint white: false */

function DES(passwd) {
    "use strict";

    // Tables, permutations, S-boxes, etc.
    // jshint -W013
    var PC2 = [13,16,10,23, 0, 4, 2,27,14, 5,20, 9,22,18,11, 3,
               25, 7,15, 6,26,19,12, 1,40,51,30,36,46,54,29,39,
               50,44,32,47,43,48,38,55,33,52,45,41,49,35,28,31 ],
        totrot = [ 1, 2, 4, 6, 8,10,12,14,15,17,19,21,23,25,27,28],
        z = 0x0, a,b,c,d,e,f, SP1,SP2,SP3,SP4,SP5,SP6,SP7,SP8,
        keys = [];

    // jshint -W015
    a=1<<16; b=1<<24; c=a|b; d=1<<2; e=1<<10; f=d|e;
    SP1 = [c|e,z|z,a|z,c|f,c|d,a|f,z|d,a|z,z|e,c|e,c|f,z|e,b|f,c|d,b|z,z|d,
           z|f,b|e,b|e,a|e,a|e,c|z,c|z,b|f,a|d,b|d,b|d,a|d,z|z,z|f,a|f,b|z,
           a|z,c|f,z|d,c|z,c|e,b|z,b|z,z|e,c|d,a|z,a|e,b|d,z|e,z|d,b|f,a|f,
           c|f,a|d,c|z,b|f,b|d,z|f,a|f,c|e,z|f,b|e,b|e,z|z,a|d,a|e,z|z,c|d];
    a=1<<20; b=1<<31; c=a|b; d=1<<5; e=1<<15; f=d|e;
    SP2 = [c|f,b|e,z|e,a|f,a|z,z|d,c|d,b|f,b|d,c|f,c|e,b|z,b|e,a|z,z|d,c|d,
           a|e,a|d,b|f,z|z,b|z,z|e,a|f,c|z,a|d,b|d,z|z,a|e,z|f,c|e,c|z,z|f,
           z|z,a|f,c|d,a|z,b|f,c|z,c|e,z|e,c|z,b|e,z|d,c|f,a|f,z|d,z|e,b|z,
           z|f,c|e,a|z,b|d,a|d,b|f,b|d,a|d,a|e,z|z,b|e,z|f,b|z,c|d,c|f,a|e];
    a=1<<17; b=1<<27; c=a|b; d=1<<3; e=1<<9; f=d|e;
    SP3 = [z|f,c|e,z|z,c|d,b|e,z|z,a|f,b|e,a|d,b|d,b|d,a|z,c|f,a|d,c|z,z|f,
           b|z,z|d,c|e,z|e,a|e,c|z,c|d,a|f,b|f,a|e,a|z,b|f,z|d,c|f,z|e,b|z,
           c|e,b|z,a|d,z|f,a|z,c|e,b|e,z|z,z|e,a|d,c|f,b|e,b|d,z|e,z|z,c|d,
           b|f,a|z,b|z,c|f,z|d,a|f,a|e,b|d,c|z,b|f,z|f,c|z,a|f,z|d,c|d,a|e];
    a=1<<13; b=1<<23; c=a|b; d=1<<0; e=1<<7; f=d|e;
    SP4 = [c|d,a|f,a|f,z|e,c|e,b|f,b|d,a|d,z|z,c|z,c|z,c|f,z|f,z|z,b|e,b|d,
           z|d,a|z,b|z,c|d,z|e,b|z,a|d,a|e,b|f,z|d,a|e,b|e,a|z,c|e,c|f,z|f,
           b|e,b|d,c|z,c|f,z|f,z|z,z|z,c|z,a|e,b|e,b|f,z|d,c|d,a|f,a|f,z|e,
           c|f,z|f,z|d,a|z,b|d,a|d,c|e,b|f,a|d,a|e,b|z,c|d,z|e,b|z,a|z,c|e];
    a=1<<25; b=1<<30; c=a|b; d=1<<8; e=1<<19; f=d|e;
    SP5 = [z|d,a|f,a|e,c|d,z|e,z|d,b|z,a|e,b|f,z|e,a|d,b|f,c|d,c|e,z|f,b|z,
           a|z,b|e,b|e,z|z,b|d,c|f,c|f,a|d,c|e,b|d,z|z,c|z,a|f,a|z,c|z,z|f,
           z|e,c|d,z|d,a|z,b|z,a|e,c|d,b|f,a|d,b|z,c|e,a|f,b|f,z|d,a|z,c|e,
           c|f,z|f,c|z,c|f,a|e,z|z,b|e,c|z,z|f,a|d,b|d,z|e,z|z,b|e,a|f,b|d];
    a=1<<22; b=1<<29; c=a|b; d=1<<4; e=1<<14; f=d|e;
    SP6 = [b|d,c|z,z|e,c|f,c|z,z|d,c|f,a|z,b|e,a|f,a|z,b|d,a|d,b|e,b|z,z|f,
           z|z,a|d,b|f,z|e,a|e,b|f,z|d,c|d,c|d,z|z,a|f,c|e,z|f,a|e,c|e,b|z,
           b|e,z|d,c|d,a|e,c|f,a|z,z|f,b|d,a|z,b|e,b|z,z|f,b|d,c|f,a|e,c|z,
           a|f,c|e,z|z,c|d,z|d,z|e,c|z,a|f,z|e,a|d,b|f,z|z,c|e,b|z,a|d,b|f];
    a=1<<21; b=1<<26; c=a|b; d=1<<1; e=1<<11; f=d|e;
    SP7 = [a|z,c|d,b|f,z|z,z|e,b|f,a|f,c|e,c|f,a|z,z|z,b|d,z|d,b|z,c|d,z|f,
           b|e,a|f,a|d,b|e,b|d,c|z,c|e,a|d,c|z,z|e,z|f,c|f,a|e,z|d,b|z,a|e,
           b|z,a|e,a|z,b|f,b|f,c|d,c|d,z|d,a|d,b|z,b|e,a|z,c|e,z|f,a|f,c|e,
           z|f,b|d,c|f,c|z,a|e,z|z,z|d,c|f,z|z,a|f,c|z,z|e,b|d,b|e,z|e,a|d];
    a=1<<18; b=1<<28; c=a|b; d=1<<6; e=1<<12; f=d|e;
    SP8 = [b|f,z|e,a|z,c|f,b|z,b|f,z|d,b|z,a|d,c|z,c|f,a|e,c|e,a|f,z|e,z|d,
           c|z,b|d,b|e,z|f,a|e,a|d,c|d,c|e,z|f,z|z,z|z,c|d,b|d,b|e,a|f,a|z,
           a|f,a|z,c|e,z|e,z|d,c|d,z|e,a|f,b|e,z|d,b|d,c|z,c|d,b|z,a|z,b|f,
           z|z,c|f,a|d,b|d,c|z,b|e,b|f,z|z,c|f,a|e,a|e,z|f,z|f,a|d,b|z,c|e];
    // jshint +W013,+W015

    // Set the key.
    function setKeys(keyBlock) {
        var i, j, l, m, n, o, pc1m = [], pcr = [], kn = [],
            raw0, raw1, rawi, KnLi;

        for (j = 0, l = 56; j < 56; ++j, l -= 8) {
            l += l < -5 ? 65 : l < -3 ? 31 : l < -1 ? 63 : l === 27 ? 35 : 0; // PC1
            m = l & 0x7;
            pc1m[j] = ((keyBlock[l >>> 3] & (1<<m)) !== 0) ? 1: 0;
        }

        for (i = 0; i < 16; ++i) {
            m = i << 1;
            n = m + 1;
            kn[m] = kn[n] = 0;
            for (o = 28; o < 59; o += 28) {
                for (j = o - 28; j < o; ++j) {
                    l = j + totrot[i];
                    if (l < o) {
                        pcr[j] = pc1m[l];
                    } else {
                        pcr[j] = pc1m[l - 28];
                    }
                }
            }
            for (j = 0; j < 24; ++j) {
                if (pcr[PC2[j]] !== 0) {
                    kn[m] |= 1 << (23 - j);
                }
                if (pcr[PC2[j + 24]] !== 0) {
                    kn[n] |= 1 << (23 - j);
                }
            }
        }

        // cookey
        for (i = 0, rawi = 0, KnLi = 0; i < 16; ++i) {
            raw0 = kn[rawi++];
            raw1 = kn[rawi++];
            keys[KnLi] = (raw0 & 0x00fc0000) << 6;
            keys[KnLi] |= (raw0 & 0x00000fc0) << 10;
            keys[KnLi] |= (raw1 & 0x00fc0000) >>> 10;
            keys[KnLi] |= (raw1 & 0x00000fc0) >>> 6;
            ++KnLi;
            keys[KnLi] = (raw0 & 0x0003f000) << 12;
            keys[KnLi] |= (raw0 & 0x0000003f) << 16;
            keys[KnLi] |= (raw1 & 0x0003f000) >>> 4;
            keys[KnLi] |= (raw1 & 0x0000003f);
            ++KnLi;
        }
    }

    // Encrypt 8 bytes of text
    function enc8(text) {
        var i = 0, b = text.slice(), fval, keysi = 0,
            l, r, x; // left, right, accumulator

        // Squash 8 bytes to 2 ints
        l = b[i++]<<24 | b[i++]<<16 | b[i++]<<8 | b[i++];
        r = b[i++]<<24 | b[i++]<<16 | b[i++]<<8 | b[i++];

        x = ((l >>> 4) ^ r) & 0x0f0f0f0f;
        r ^= x;
        l ^= (x << 4);
        x = ((l >>> 16) ^ r) & 0x0000ffff;
        r ^= x;
        l ^= (x << 16);
        x = ((r >>> 2) ^ l) & 0x33333333;
        l ^= x;
        r ^= (x << 2);
        x = ((r >>> 8) ^ l) & 0x00ff00ff;
        l ^= x;
        r ^= (x << 8);
        r = (r << 1) | ((r >>> 31) & 1);
        x = (l ^ r) & 0xaaaaaaaa;
        l ^= x;
        r ^= x;
        l = (l << 1) | ((l >>> 31) & 1);

        for (i = 0; i < 8; ++i) {
            x = (r << 28) | (r >>> 4);
            x ^= keys[keysi++];
            fval =  SP7[x & 0x3f];
            fval |= SP5[(x >>> 8) & 0x3f];
            fval |= SP3[(x >>> 16) & 0x3f];
            fval |= SP1[(x >>> 24) & 0x3f];
            x = r ^ keys[keysi++];
            fval |= SP8[x & 0x3f];
            fval |= SP6[(x >>> 8) & 0x3f];
            fval |= SP4[(x >>> 16) & 0x3f];
            fval |= SP2[(x >>> 24) & 0x3f];
            l ^= fval;
            x = (l << 28) | (l >>> 4);
            x ^= keys[keysi++];
            fval =  SP7[x & 0x3f];
            fval |= SP5[(x >>> 8) & 0x3f];
            fval |= SP3[(x >>> 16) & 0x3f];
            fval |= SP1[(x >>> 24) & 0x3f];
            x = l ^ keys[keysi++];
            fval |= SP8[x & 0x0000003f];
            fval |= SP6[(x >>> 8) & 0x3f];
            fval |= SP4[(x >>> 16) & 0x3f];
            fval |= SP2[(x >>> 24) & 0x3f];
            r ^= fval;
        }

        r = (r << 31) | (r >>> 1);
        x = (l ^ r) & 0xaaaaaaaa;
        l ^= x;
        r ^= x;
        l = (l << 31) | (l >>> 1);
        x = ((l >>> 8) ^ r) & 0x00ff00ff;
        r ^= x;
        l ^= (x << 8);
        x = ((l >>> 2) ^ r) & 0x33333333;
        r ^= x;
        l ^= (x << 2);
        x = ((r >>> 16) ^ l) & 0x0000ffff;
        l ^= x;
        r ^= (x << 16);
        x = ((r >>> 4) ^ l) & 0x0f0f0f0f;
        l ^= x;
        r ^= (x << 4);

        // Spread ints to bytes
        x = [r, l];
        for (i = 0; i < 8; i++) {
            b[i] = (x[i>>>2] >>> (8 * (3 - (i % 4)))) % 256;
            if (b[i] < 0) { b[i] += 256; } // unsigned
        }
        return b;
    }

    // Encrypt 16 bytes of text using passwd as key
    function encrypt(t) {
        return enc8(t.slice(0, 8)).concat(enc8(t.slice(8, 16)));
    }

    setKeys(passwd);             // Setup keys
    return {'encrypt': encrypt}; // Public interface

} // function DES
;
define("vnc-des", ["vnc-util"], function(){});

// This file describes mappings from Unicode codepoints to the keysym values
// (and optionally, key names) expected by the RFB protocol
// How this file was generated:
// node /Users/jalf/dev/mi/novnc/utils/parse.js /opt/X11/include/X11/keysymdef.h
var keysyms = (function(){
    "use strict";
    var keynames = null;
    var codepoints = {"32":32,"33":33,"34":34,"35":35,"36":36,"37":37,"38":38,"39":39,"40":40,"41":41,"42":42,"43":43,"44":44,"45":45,"46":46,"47":47,"48":48,"49":49,"50":50,"51":51,"52":52,"53":53,"54":54,"55":55,"56":56,"57":57,"58":58,"59":59,"60":60,"61":61,"62":62,"63":63,"64":64,"65":65,"66":66,"67":67,"68":68,"69":69,"70":70,"71":71,"72":72,"73":73,"74":74,"75":75,"76":76,"77":77,"78":78,"79":79,"80":80,"81":81,"82":82,"83":83,"84":84,"85":85,"86":86,"87":87,"88":88,"89":89,"90":90,"91":91,"92":92,"93":93,"94":94,"95":95,"96":96,"97":97,"98":98,"99":99,"100":100,"101":101,"102":102,"103":103,"104":104,"105":105,"106":106,"107":107,"108":108,"109":109,"110":110,"111":111,"112":112,"113":113,"114":114,"115":115,"116":116,"117":117,"118":118,"119":119,"120":120,"121":121,"122":122,"123":123,"124":124,"125":125,"126":126,"160":160,"161":161,"162":162,"163":163,"164":164,"165":165,"166":166,"167":167,"168":168,"169":169,"170":170,"171":171,"172":172,"173":173,"174":174,"175":175,"176":176,"177":177,"178":178,"179":179,"180":180,"181":181,"182":182,"183":183,"184":184,"185":185,"186":186,"187":187,"188":188,"189":189,"190":190,"191":191,"192":192,"193":193,"194":194,"195":195,"196":196,"197":197,"198":198,"199":199,"200":200,"201":201,"202":202,"203":203,"204":204,"205":205,"206":206,"207":207,"208":208,"209":209,"210":210,"211":211,"212":212,"213":213,"214":214,"215":215,"216":216,"217":217,"218":218,"219":219,"220":220,"221":221,"222":222,"223":223,"224":224,"225":225,"226":226,"227":227,"228":228,"229":229,"230":230,"231":231,"232":232,"233":233,"234":234,"235":235,"236":236,"237":237,"238":238,"239":239,"240":240,"241":241,"242":242,"243":243,"244":244,"245":245,"246":246,"247":247,"248":248,"249":249,"250":250,"251":251,"252":252,"253":253,"254":254,"255":255,"256":960,"257":992,"258":451,"259":483,"260":417,"261":433,"262":454,"263":486,"264":710,"265":742,"266":709,"267":741,"268":456,"269":488,"270":463,"271":495,"272":464,"273":496,"274":938,"275":954,"278":972,"279":1004,"280":458,"281":490,"282":460,"283":492,"284":728,"285":760,"286":683,"287":699,"288":725,"289":757,"290":939,"291":955,"292":678,"293":694,"294":673,"295":689,"296":933,"297":949,"298":975,"299":1007,"300":16777516,"301":16777517,"302":967,"303":999,"304":681,"305":697,"308":684,"309":700,"310":979,"311":1011,"312":930,"313":453,"314":485,"315":934,"316":950,"317":421,"318":437,"321":419,"322":435,"323":465,"324":497,"325":977,"326":1009,"327":466,"328":498,"330":957,"331":959,"332":978,"333":1010,"336":469,"337":501,"338":5052,"339":5053,"340":448,"341":480,"342":931,"343":947,"344":472,"345":504,"346":422,"347":438,"348":734,"349":766,"350":426,"351":442,"352":425,"353":441,"354":478,"355":510,"356":427,"357":443,"358":940,"359":956,"360":989,"361":1021,"362":990,"363":1022,"364":733,"365":765,"366":473,"367":505,"368":475,"369":507,"370":985,"371":1017,"372":16777588,"373":16777589,"374":16777590,"375":16777591,"376":5054,"377":428,"378":444,"379":431,"380":447,"381":430,"382":446,"399":16777615,"402":2294,"415":16777631,"416":16777632,"417":16777633,"431":16777647,"432":16777648,"437":16777653,"438":16777654,"439":16777655,"466":16777681,"486":16777702,"487":16777703,"601":16777817,"629":16777845,"658":16777874,"711":439,"728":418,"729":511,"731":434,"733":445,"901":1966,"902":1953,"904":1954,"905":1955,"906":1956,"908":1959,"910":1960,"911":1963,"912":1974,"913":1985,"914":1986,"915":1987,"916":1988,"917":1989,"918":1990,"919":1991,"920":1992,"921":1993,"922":1994,"923":1995,"924":1996,"925":1997,"926":1998,"927":1999,"928":2000,"929":2001,"931":2002,"932":2004,"933":2005,"934":2006,"935":2007,"936":2008,"937":2009,"938":1957,"939":1961,"940":1969,"941":1970,"942":1971,"943":1972,"944":1978,"945":2017,"946":2018,"947":2019,"948":2020,"949":2021,"950":2022,"951":2023,"952":2024,"953":2025,"954":2026,"955":2027,"956":2028,"957":2029,"958":2030,"959":2031,"960":2032,"961":2033,"962":2035,"963":2034,"964":2036,"965":2037,"966":2038,"967":2039,"968":2040,"969":2041,"970":1973,"971":1977,"972":1975,"973":1976,"974":1979,"1025":1715,"1026":1713,"1027":1714,"1028":1716,"1029":1717,"1030":1718,"1031":1719,"1032":1720,"1033":1721,"1034":1722,"1035":1723,"1036":1724,"1038":1726,"1039":1727,"1040":1761,"1041":1762,"1042":1783,"1043":1767,"1044":1764,"1045":1765,"1046":1782,"1047":1786,"1048":1769,"1049":1770,"1050":1771,"1051":1772,"1052":1773,"1053":1774,"1054":1775,"1055":1776,"1056":1778,"1057":1779,"1058":1780,"1059":1781,"1060":1766,"1061":1768,"1062":1763,"1063":1790,"1064":1787,"1065":1789,"1066":1791,"1067":1785,"1068":1784,"1069":1788,"1070":1760,"1071":1777,"1072":1729,"1073":1730,"1074":1751,"1075":1735,"1076":1732,"1077":1733,"1078":1750,"1079":1754,"1080":1737,"1081":1738,"1082":1739,"1083":1740,"1084":1741,"1085":1742,"1086":1743,"1087":1744,"1088":1746,"1089":1747,"1090":1748,"1091":1749,"1092":1734,"1093":1736,"1094":1731,"1095":1758,"1096":1755,"1097":1757,"1098":1759,"1099":1753,"1100":1752,"1101":1756,"1102":1728,"1103":1745,"1105":1699,"1106":1697,"1107":1698,"1108":1700,"1109":1701,"1110":1702,"1111":1703,"1112":1704,"1113":1705,"1114":1706,"1115":1707,"1116":1708,"1118":1710,"1119":1711,"1168":1725,"1169":1709,"1170":16778386,"1171":16778387,"1174":16778390,"1175":16778391,"1178":16778394,"1179":16778395,"1180":16778396,"1181":16778397,"1186":16778402,"1187":16778403,"1198":16778414,"1199":16778415,"1200":16778416,"1201":16778417,"1202":16778418,"1203":16778419,"1206":16778422,"1207":16778423,"1208":16778424,"1209":16778425,"1210":16778426,"1211":16778427,"1240":16778456,"1241":16778457,"1250":16778466,"1251":16778467,"1256":16778472,"1257":16778473,"1262":16778478,"1263":16778479,"1329":16778545,"1330":16778546,"1331":16778547,"1332":16778548,"1333":16778549,"1334":16778550,"1335":16778551,"1336":16778552,"1337":16778553,"1338":16778554,"1339":16778555,"1340":16778556,"1341":16778557,"1342":16778558,"1343":16778559,"1344":16778560,"1345":16778561,"1346":16778562,"1347":16778563,"1348":16778564,"1349":16778565,"1350":16778566,"1351":16778567,"1352":16778568,"1353":16778569,"1354":16778570,"1355":16778571,"1356":16778572,"1357":16778573,"1358":16778574,"1359":16778575,"1360":16778576,"1361":16778577,"1362":16778578,"1363":16778579,"1364":16778580,"1365":16778581,"1366":16778582,"1370":16778586,"1371":16778587,"1372":16778588,"1373":16778589,"1374":16778590,"1377":16778593,"1378":16778594,"1379":16778595,"1380":16778596,"1381":16778597,"1382":16778598,"1383":16778599,"1384":16778600,"1385":16778601,"1386":16778602,"1387":16778603,"1388":16778604,"1389":16778605,"1390":16778606,"1391":16778607,"1392":16778608,"1393":16778609,"1394":16778610,"1395":16778611,"1396":16778612,"1397":16778613,"1398":16778614,"1399":16778615,"1400":16778616,"1401":16778617,"1402":16778618,"1403":16778619,"1404":16778620,"1405":16778621,"1406":16778622,"1407":16778623,"1408":16778624,"1409":16778625,"1410":16778626,"1411":16778627,"1412":16778628,"1413":16778629,"1414":16778630,"1415":16778631,"1417":16778633,"1418":16778634,"1488":3296,"1489":3297,"1490":3298,"1491":3299,"1492":3300,"1493":3301,"1494":3302,"1495":3303,"1496":3304,"1497":3305,"1498":3306,"1499":3307,"1500":3308,"1501":3309,"1502":3310,"1503":3311,"1504":3312,"1505":3313,"1506":3314,"1507":3315,"1508":3316,"1509":3317,"1510":3318,"1511":3319,"1512":3320,"1513":3321,"1514":3322,"1548":1452,"1563":1467,"1567":1471,"1569":1473,"1570":1474,"1571":1475,"1572":1476,"1573":1477,"1574":1478,"1575":1479,"1576":1480,"1577":1481,"1578":1482,"1579":1483,"1580":1484,"1581":1485,"1582":1486,"1583":1487,"1584":1488,"1585":1489,"1586":1490,"1587":1491,"1588":1492,"1589":1493,"1590":1494,"1591":1495,"1592":1496,"1593":1497,"1594":1498,"1600":1504,"1601":1505,"1602":1506,"1603":1507,"1604":1508,"1605":1509,"1606":1510,"1607":1511,"1608":1512,"1609":1513,"1610":1514,"1611":1515,"1612":1516,"1613":1517,"1614":1518,"1615":1519,"1616":1520,"1617":1521,"1618":1522,"1619":16778835,"1620":16778836,"1621":16778837,"1632":16778848,"1633":16778849,"1634":16778850,"1635":16778851,"1636":16778852,"1637":16778853,"1638":16778854,"1639":16778855,"1640":16778856,"1641":16778857,"1642":16778858,"1648":16778864,"1657":16778873,"1662":16778878,"1670":16778886,"1672":16778888,"1681":16778897,"1688":16778904,"1700":16778916,"1705":16778921,"1711":16778927,"1722":16778938,"1726":16778942,"1729":16778945,"1740":16778956,"1746":16778962,"1748":16778964,"1776":16778992,"1777":16778993,"1778":16778994,"1779":16778995,"1780":16778996,"1781":16778997,"1782":16778998,"1783":16778999,"1784":16779000,"1785":16779001,"3458":16780674,"3459":16780675,"3461":16780677,"3462":16780678,"3463":16780679,"3464":16780680,"3465":16780681,"3466":16780682,"3467":16780683,"3468":16780684,"3469":16780685,"3470":16780686,"3471":16780687,"3472":16780688,"3473":16780689,"3474":16780690,"3475":16780691,"3476":16780692,"3477":16780693,"3478":16780694,"3482":16780698,"3483":16780699,"3484":16780700,"3485":16780701,"3486":16780702,"3487":16780703,"3488":16780704,"3489":16780705,"3490":16780706,"3491":16780707,"3492":16780708,"3493":16780709,"3494":16780710,"3495":16780711,"3496":16780712,"3497":16780713,"3498":16780714,"3499":16780715,"3500":16780716,"3501":16780717,"3502":16780718,"3503":16780719,"3504":16780720,"3505":16780721,"3507":16780723,"3508":16780724,"3509":16780725,"3510":16780726,"3511":16780727,"3512":16780728,"3513":16780729,"3514":16780730,"3515":16780731,"3517":16780733,"3520":16780736,"3521":16780737,"3522":16780738,"3523":16780739,"3524":16780740,"3525":16780741,"3526":16780742,"3530":16780746,"3535":16780751,"3536":16780752,"3537":16780753,"3538":16780754,"3539":16780755,"3540":16780756,"3542":16780758,"3544":16780760,"3545":16780761,"3546":16780762,"3547":16780763,"3548":16780764,"3549":16780765,"3550":16780766,"3551":16780767,"3570":16780786,"3571":16780787,"3572":16780788,"3585":3489,"3586":3490,"3587":3491,"3588":3492,"3589":3493,"3590":3494,"3591":3495,"3592":3496,"3593":3497,"3594":3498,"3595":3499,"3596":3500,"3597":3501,"3598":3502,"3599":3503,"3600":3504,"3601":3505,"3602":3506,"3603":3507,"3604":3508,"3605":3509,"3606":3510,"3607":3511,"3608":3512,"3609":3513,"3610":3514,"3611":3515,"3612":3516,"3613":3517,"3614":3518,"3615":3519,"3616":3520,"3617":3521,"3618":3522,"3619":3523,"3620":3524,"3621":3525,"3622":3526,"3623":3527,"3624":3528,"3625":3529,"3626":3530,"3627":3531,"3628":3532,"3629":3533,"3630":3534,"3631":3535,"3632":3536,"3633":3537,"3634":3538,"3635":3539,"3636":3540,"3637":3541,"3638":3542,"3639":3543,"3640":3544,"3641":3545,"3642":3546,"3647":3551,"3648":3552,"3649":3553,"3650":3554,"3651":3555,"3652":3556,"3653":3557,"3654":3558,"3655":3559,"3656":3560,"3657":3561,"3658":3562,"3659":3563,"3660":3564,"3661":3565,"3664":3568,"3665":3569,"3666":3570,"3667":3571,"3668":3572,"3669":3573,"3670":3574,"3671":3575,"3672":3576,"3673":3577,"4304":16781520,"4305":16781521,"4306":16781522,"4307":16781523,"4308":16781524,"4309":16781525,"4310":16781526,"4311":16781527,"4312":16781528,"4313":16781529,"4314":16781530,"4315":16781531,"4316":16781532,"4317":16781533,"4318":16781534,"4319":16781535,"4320":16781536,"4321":16781537,"4322":16781538,"4323":16781539,"4324":16781540,"4325":16781541,"4326":16781542,"4327":16781543,"4328":16781544,"4329":16781545,"4330":16781546,"4331":16781547,"4332":16781548,"4333":16781549,"4334":16781550,"4335":16781551,"4336":16781552,"4337":16781553,"4338":16781554,"4339":16781555,"4340":16781556,"4341":16781557,"4342":16781558,"7682":16784898,"7683":16784899,"7690":16784906,"7691":16784907,"7710":16784926,"7711":16784927,"7734":16784950,"7735":16784951,"7744":16784960,"7745":16784961,"7766":16784982,"7767":16784983,"7776":16784992,"7777":16784993,"7786":16785002,"7787":16785003,"7808":16785024,"7809":16785025,"7810":16785026,"7811":16785027,"7812":16785028,"7813":16785029,"7818":16785034,"7819":16785035,"7840":16785056,"7841":16785057,"7842":16785058,"7843":16785059,"7844":16785060,"7845":16785061,"7846":16785062,"7847":16785063,"7848":16785064,"7849":16785065,"7850":16785066,"7851":16785067,"7852":16785068,"7853":16785069,"7854":16785070,"7855":16785071,"7856":16785072,"7857":16785073,"7858":16785074,"7859":16785075,"7860":16785076,"7861":16785077,"7862":16785078,"7863":16785079,"7864":16785080,"7865":16785081,"7866":16785082,"7867":16785083,"7868":16785084,"7869":16785085,"7870":16785086,"7871":16785087,"7872":16785088,"7873":16785089,"7874":16785090,"7875":16785091,"7876":16785092,"7877":16785093,"7878":16785094,"7879":16785095,"7880":16785096,"7881":16785097,"7882":16785098,"7883":16785099,"7884":16785100,"7885":16785101,"7886":16785102,"7887":16785103,"7888":16785104,"7889":16785105,"7890":16785106,"7891":16785107,"7892":16785108,"7893":16785109,"7894":16785110,"7895":16785111,"7896":16785112,"7897":16785113,"7898":16785114,"7899":16785115,"7900":16785116,"7901":16785117,"7902":16785118,"7903":16785119,"7904":16785120,"7905":16785121,"7906":16785122,"7907":16785123,"7908":16785124,"7909":16785125,"7910":16785126,"7911":16785127,"7912":16785128,"7913":16785129,"7914":16785130,"7915":16785131,"7916":16785132,"7917":16785133,"7918":16785134,"7919":16785135,"7920":16785136,"7921":16785137,"7922":16785138,"7923":16785139,"7924":16785140,"7925":16785141,"7926":16785142,"7927":16785143,"7928":16785144,"7929":16785145,"8194":2722,"8195":2721,"8196":2723,"8197":2724,"8199":2725,"8200":2726,"8201":2727,"8202":2728,"8210":2747,"8211":2730,"8212":2729,"8213":1967,"8215":3295,"8216":2768,"8217":2769,"8218":2813,"8220":2770,"8221":2771,"8222":2814,"8224":2801,"8225":2802,"8226":2790,"8229":2735,"8230":2734,"8240":2773,"8242":2774,"8243":2775,"8248":2812,"8254":1150,"8304":16785520,"8308":16785524,"8309":16785525,"8310":16785526,"8311":16785527,"8312":16785528,"8313":16785529,"8320":16785536,"8321":16785537,"8322":16785538,"8323":16785539,"8324":16785540,"8325":16785541,"8326":16785542,"8327":16785543,"8328":16785544,"8329":16785545,"8352":16785568,"8353":16785569,"8354":16785570,"8355":16785571,"8356":16785572,"8357":16785573,"8358":16785574,"8359":16785575,"8360":16785576,"8361":3839,"8362":16785578,"8363":16785579,"8364":8364,"8453":2744,"8470":1712,"8471":2811,"8478":2772,"8482":2761,"8531":2736,"8532":2737,"8533":2738,"8534":2739,"8535":2740,"8536":2741,"8537":2742,"8538":2743,"8539":2755,"8540":2756,"8541":2757,"8542":2758,"8592":2299,"8593":2300,"8594":2301,"8595":2302,"8658":2254,"8660":2253,"8706":2287,"8709":16785925,"8711":2245,"8712":16785928,"8713":16785929,"8715":16785931,"8728":3018,"8730":2262,"8731":16785947,"8732":16785948,"8733":2241,"8734":2242,"8743":2270,"8744":2271,"8745":2268,"8746":2269,"8747":2239,"8748":16785964,"8749":16785965,"8756":2240,"8757":16785973,"8764":2248,"8771":2249,"8773":16785992,"8775":16785991,"8800":2237,"8801":2255,"8802":16786018,"8803":16786019,"8804":2236,"8805":2238,"8834":2266,"8835":2267,"8866":3068,"8867":3036,"8868":3010,"8869":3022,"8968":3027,"8970":3012,"8981":2810,"8992":2212,"8993":2213,"9109":3020,"9115":2219,"9117":2220,"9118":2221,"9120":2222,"9121":2215,"9123":2216,"9124":2217,"9126":2218,"9128":2223,"9132":2224,"9143":2209,"9146":2543,"9147":2544,"9148":2546,"9149":2547,"9225":2530,"9226":2533,"9227":2537,"9228":2531,"9229":2532,"9251":2732,"9252":2536,"9472":2211,"9474":2214,"9484":2210,"9488":2539,"9492":2541,"9496":2538,"9500":2548,"9508":2549,"9516":2551,"9524":2550,"9532":2542,"9618":2529,"9642":2791,"9643":2785,"9644":2779,"9645":2786,"9646":2783,"9647":2767,"9650":2792,"9651":2787,"9654":2781,"9655":2765,"9660":2793,"9661":2788,"9664":2780,"9665":2764,"9670":2528,"9675":2766,"9679":2782,"9702":2784,"9734":2789,"9742":2809,"9747":2762,"9756":2794,"9758":2795,"9792":2808,"9794":2807,"9827":2796,"9829":2798,"9830":2797,"9837":2806,"9839":2805,"10003":2803,"10007":2804,"10013":2777,"10016":2800,"10216":2748,"10217":2750,"10240":16787456,"10241":16787457,"10242":16787458,"10243":16787459,"10244":16787460,"10245":16787461,"10246":16787462,"10247":16787463,"10248":16787464,"10249":16787465,"10250":16787466,"10251":16787467,"10252":16787468,"10253":16787469,"10254":16787470,"10255":16787471,"10256":16787472,"10257":16787473,"10258":16787474,"10259":16787475,"10260":16787476,"10261":16787477,"10262":16787478,"10263":16787479,"10264":16787480,"10265":16787481,"10266":16787482,"10267":16787483,"10268":16787484,"10269":16787485,"10270":16787486,"10271":16787487,"10272":16787488,"10273":16787489,"10274":16787490,"10275":16787491,"10276":16787492,"10277":16787493,"10278":16787494,"10279":16787495,"10280":16787496,"10281":16787497,"10282":16787498,"10283":16787499,"10284":16787500,"10285":16787501,"10286":16787502,"10287":16787503,"10288":16787504,"10289":16787505,"10290":16787506,"10291":16787507,"10292":16787508,"10293":16787509,"10294":16787510,"10295":16787511,"10296":16787512,"10297":16787513,"10298":16787514,"10299":16787515,"10300":16787516,"10301":16787517,"10302":16787518,"10303":16787519,"10304":16787520,"10305":16787521,"10306":16787522,"10307":16787523,"10308":16787524,"10309":16787525,"10310":16787526,"10311":16787527,"10312":16787528,"10313":16787529,"10314":16787530,"10315":16787531,"10316":16787532,"10317":16787533,"10318":16787534,"10319":16787535,"10320":16787536,"10321":16787537,"10322":16787538,"10323":16787539,"10324":16787540,"10325":16787541,"10326":16787542,"10327":16787543,"10328":16787544,"10329":16787545,"10330":16787546,"10331":16787547,"10332":16787548,"10333":16787549,"10334":16787550,"10335":16787551,"10336":16787552,"10337":16787553,"10338":16787554,"10339":16787555,"10340":16787556,"10341":16787557,"10342":16787558,"10343":16787559,"10344":16787560,"10345":16787561,"10346":16787562,"10347":16787563,"10348":16787564,"10349":16787565,"10350":16787566,"10351":16787567,"10352":16787568,"10353":16787569,"10354":16787570,"10355":16787571,"10356":16787572,"10357":16787573,"10358":16787574,"10359":16787575,"10360":16787576,"10361":16787577,"10362":16787578,"10363":16787579,"10364":16787580,"10365":16787581,"10366":16787582,"10367":16787583,"10368":16787584,"10369":16787585,"10370":16787586,"10371":16787587,"10372":16787588,"10373":16787589,"10374":16787590,"10375":16787591,"10376":16787592,"10377":16787593,"10378":16787594,"10379":16787595,"10380":16787596,"10381":16787597,"10382":16787598,"10383":16787599,"10384":16787600,"10385":16787601,"10386":16787602,"10387":16787603,"10388":16787604,"10389":16787605,"10390":16787606,"10391":16787607,"10392":16787608,"10393":16787609,"10394":16787610,"10395":16787611,"10396":16787612,"10397":16787613,"10398":16787614,"10399":16787615,"10400":16787616,"10401":16787617,"10402":16787618,"10403":16787619,"10404":16787620,"10405":16787621,"10406":16787622,"10407":16787623,"10408":16787624,"10409":16787625,"10410":16787626,"10411":16787627,"10412":16787628,"10413":16787629,"10414":16787630,"10415":16787631,"10416":16787632,"10417":16787633,"10418":16787634,"10419":16787635,"10420":16787636,"10421":16787637,"10422":16787638,"10423":16787639,"10424":16787640,"10425":16787641,"10426":16787642,"10427":16787643,"10428":16787644,"10429":16787645,"10430":16787646,"10431":16787647,"10432":16787648,"10433":16787649,"10434":16787650,"10435":16787651,"10436":16787652,"10437":16787653,"10438":16787654,"10439":16787655,"10440":16787656,"10441":16787657,"10442":16787658,"10443":16787659,"10444":16787660,"10445":16787661,"10446":16787662,"10447":16787663,"10448":16787664,"10449":16787665,"10450":16787666,"10451":16787667,"10452":16787668,"10453":16787669,"10454":16787670,"10455":16787671,"10456":16787672,"10457":16787673,"10458":16787674,"10459":16787675,"10460":16787676,"10461":16787677,"10462":16787678,"10463":16787679,"10464":16787680,"10465":16787681,"10466":16787682,"10467":16787683,"10468":16787684,"10469":16787685,"10470":16787686,"10471":16787687,"10472":16787688,"10473":16787689,"10474":16787690,"10475":16787691,"10476":16787692,"10477":16787693,"10478":16787694,"10479":16787695,"10480":16787696,"10481":16787697,"10482":16787698,"10483":16787699,"10484":16787700,"10485":16787701,"10486":16787702,"10487":16787703,"10488":16787704,"10489":16787705,"10490":16787706,"10491":16787707,"10492":16787708,"10493":16787709,"10494":16787710,"10495":16787711,"12289":1188,"12290":1185,"12300":1186,"12301":1187,"12443":1246,"12444":1247,"12449":1191,"12450":1201,"12451":1192,"12452":1202,"12453":1193,"12454":1203,"12455":1194,"12456":1204,"12457":1195,"12458":1205,"12459":1206,"12461":1207,"12463":1208,"12465":1209,"12467":1210,"12469":1211,"12471":1212,"12473":1213,"12475":1214,"12477":1215,"12479":1216,"12481":1217,"12483":1199,"12484":1218,"12486":1219,"12488":1220,"12490":1221,"12491":1222,"12492":1223,"12493":1224,"12494":1225,"12495":1226,"12498":1227,"12501":1228,"12504":1229,"12507":1230,"12510":1231,"12511":1232,"12512":1233,"12513":1234,"12514":1235,"12515":1196,"12516":1236,"12517":1197,"12518":1237,"12519":1198,"12520":1238,"12521":1239,"12522":1240,"12523":1241,"12524":1242,"12525":1243,"12527":1244,"12530":1190,"12531":1245,"12539":1189,"12540":1200};

    function lookup(k) { return k ? {keysym: k, keyname: keynames ? keynames[k] : k} : undefined; }
    return {
        fromUnicode : function(u) { return lookup(codepoints[u]); },
        lookup : lookup
    };
})();

define("vnc-keysymdef", ["vnc-util"], function(){});

var kbdUtil = (function() {
    "use strict";

    function substituteCodepoint(cp) {
        // Any Unicode code points which do not have corresponding keysym entries
        // can be swapped out for another code point by adding them to this table
        var substitutions = {
            // {S,s} with comma below -> {S,s} with cedilla
            0x218 : 0x15e,
            0x219 : 0x15f,
            // {T,t} with comma below -> {T,t} with cedilla
            0x21a : 0x162,
            0x21b : 0x163
        };

        var sub = substitutions[cp];
        return sub ? sub : cp;
    }

    function isMac() {
        return navigator && !!(/mac/i).exec(navigator.platform);
    }
    function isWindows() {
        return navigator && !!(/win/i).exec(navigator.platform);
    }
    function isLinux() {
        return navigator && !!(/linux/i).exec(navigator.platform);
    }

    // Return true if a modifier which is not the specified char modifier (and is not shift) is down
    function hasShortcutModifier(charModifier, currentModifiers) {
        var mods = {};
        for (var key in currentModifiers) {
            if (parseInt(key) !== XK_Shift_L) {
                mods[key] = currentModifiers[key];
            }
        }

        var sum = 0;
        for (var k in currentModifiers) {
            if (mods[k]) {
                ++sum;
            }
        }
        if (hasCharModifier(charModifier, mods)) {
            return sum > charModifier.length;
        }
        else {
            return sum > 0;
        }
    }

    // Return true if the specified char modifier is currently down
    function hasCharModifier(charModifier, currentModifiers) {
        if (charModifier.length === 0) { return false; }

        for (var i = 0; i < charModifier.length; ++i) {
            if (!currentModifiers[charModifier[i]]) {
                return false;
            }
        }
        return true;
    }

    // Helper object tracking modifier key state
    // and generates fake key events to compensate if it gets out of sync
    function ModifierSync(charModifier) {
        if (!charModifier) {
            if (isMac()) {
                // on Mac, Option (AKA Alt) is used as a char modifier
                charModifier = [XK_Alt_L];
            }
            else if (isWindows()) {
                // on Windows, Ctrl+Alt is used as a char modifier
                charModifier = [XK_Alt_L, XK_Control_L];
            }
            else if (isLinux()) {
                // on Linux, ISO Level 3 Shift (AltGr) is used as a char modifier
                charModifier = [XK_ISO_Level3_Shift];
            }
            else {
                charModifier = [];
            }
        }

        var state = {};
        state[XK_Control_L] = false;
        state[XK_Alt_L] = false;
        state[XK_ISO_Level3_Shift] = false;
        state[XK_Shift_L] = false;
        state[XK_Meta_L] = false;

        function sync(evt, keysym) {
            var result = [];
            function syncKey(keysym) {
                return {keysym: keysyms.lookup(keysym), type: state[keysym] ? 'keydown' : 'keyup'};
            }

            if (evt.ctrlKey !== undefined &&
                evt.ctrlKey !== state[XK_Control_L] && keysym !== XK_Control_L) {
                state[XK_Control_L] = evt.ctrlKey;
                result.push(syncKey(XK_Control_L));
            }
            if (evt.altKey !== undefined &&
                evt.altKey !== state[XK_Alt_L] && keysym !== XK_Alt_L) {
                state[XK_Alt_L] = evt.altKey;
                result.push(syncKey(XK_Alt_L));
            }
            if (evt.altGraphKey !== undefined &&
                evt.altGraphKey !== state[XK_ISO_Level3_Shift] && keysym !== XK_ISO_Level3_Shift) {
                state[XK_ISO_Level3_Shift] = evt.altGraphKey;
                result.push(syncKey(XK_ISO_Level3_Shift));
            }
            if (evt.shiftKey !== undefined &&
                evt.shiftKey !== state[XK_Shift_L] && keysym !== XK_Shift_L) {
                state[XK_Shift_L] = evt.shiftKey;
                result.push(syncKey(XK_Shift_L));
            }
            if (evt.metaKey !== undefined &&
                evt.metaKey !== state[XK_Meta_L] && keysym !== XK_Meta_L) {
                state[XK_Meta_L] = evt.metaKey;
                result.push(syncKey(XK_Meta_L));
            }
            return result;
        }
        function syncKeyEvent(evt, down) {
            var obj = getKeysym(evt);
            var keysym = obj ? obj.keysym : null;

            // first, apply the event itself, if relevant
            if (keysym !== null && state[keysym] !== undefined) {
                state[keysym] = down;
            }
            return sync(evt, keysym);
        }

        return {
            // sync on the appropriate keyboard event
            keydown: function(evt) { return syncKeyEvent(evt, true);},
            keyup: function(evt) { return syncKeyEvent(evt, false);},
            // Call this with a non-keyboard event (such as mouse events) to use its modifier state to synchronize anyway
            syncAny: function(evt) { return sync(evt);},

            // is a shortcut modifier down?
            hasShortcutModifier: function() { return hasShortcutModifier(charModifier, state); },
            // if a char modifier is down, return the keys it consists of, otherwise return null
            activeCharModifier: function() { return hasCharModifier(charModifier, state) ? charModifier : null; }
        };
    }

    // Get a key ID from a keyboard event
    // May be a string or an integer depending on the available properties
    function getKey(evt){
        if ('keyCode' in evt && 'key' in evt) {
            return evt.key + ':' + evt.keyCode;
        }
        else if ('keyCode' in evt) {
            return evt.keyCode;
        }
        else {
            return evt.key;
        }
    }

    // Get the most reliable keysym value we can get from a key event
    // if char/charCode is available, prefer those, otherwise fall back to key/keyCode/which
    function getKeysym(evt){
        var codepoint;
        if (evt.char && evt.char.length === 1) {
            codepoint = evt.char.charCodeAt();
        }
        else if (evt.charCode) {
            codepoint = evt.charCode;
        }
        else if (evt.keyCode && evt.type === 'keypress') {
            // IE10 stores the char code as keyCode, and has no other useful properties
            codepoint = evt.keyCode;
        }
        if (codepoint) {
            var res = keysyms.fromUnicode(substituteCodepoint(codepoint));
            if (res) {
                return res;
            }
        }
        // we could check evt.key here.
        // Legal values are defined in http://www.w3.org/TR/DOM-Level-3-Events/#key-values-list,
        // so we "just" need to map them to keysym, but AFAIK this is only available in IE10, which also provides evt.key
        // so we don't *need* it yet
        if (evt.keyCode) {
            return keysyms.lookup(keysymFromKeyCode(evt.keyCode, evt.shiftKey));
        }
        if (evt.which) {
            return keysyms.lookup(keysymFromKeyCode(evt.which, evt.shiftKey));
        }
        return null;
    }

    // Given a keycode, try to predict which keysym it might be.
    // If the keycode is unknown, null is returned.
    function keysymFromKeyCode(keycode, shiftPressed) {
        if (typeof(keycode) !== 'number') {
            return null;
        }
        // won't be accurate for azerty
        if (keycode >= 0x30 && keycode <= 0x39) {
            return keycode; // digit
        }
        if (keycode >= 0x41 && keycode <= 0x5a) {
            // remap to lowercase unless shift is down
            return shiftPressed ? keycode : keycode + 32; // A-Z
        }
        if (keycode >= 0x60 && keycode <= 0x69) {
            return XK_KP_0 + (keycode - 0x60); // numpad 0-9
        }

        switch(keycode) {
            case 0x20: return XK_space;
            case 0x6a: return XK_KP_Multiply;
            case 0x6b: return XK_KP_Add;
            case 0x6c: return XK_KP_Separator;
            case 0x6d: return XK_KP_Subtract;
            case 0x6e: return XK_KP_Decimal;
            case 0x6f: return XK_KP_Divide;
            case 0xbb: return XK_plus;
            case 0xbc: return XK_comma;
            case 0xbd: return XK_minus;
            case 0xbe: return XK_period;
        }

        return nonCharacterKey({keyCode: keycode});
    }

    // if the key is a known non-character key (any key which doesn't generate character data)
    // return its keysym value. Otherwise return null
    function nonCharacterKey(evt) {
        // evt.key not implemented yet
        if (!evt.keyCode) { return null; }
        var keycode = evt.keyCode;

        if (keycode >= 0x70 && keycode <= 0x87) {
            return XK_F1 + keycode - 0x70; // F1-F24
        }
        switch (keycode) {

            case 8 : return XK_BackSpace;
            case 13 : return XK_Return;

            case 9 : return XK_Tab;

            case 27 : return XK_Escape;
            case 46 : return XK_Delete;

            case 36 : return XK_Home;
            case 35 : return XK_End;
            case 33 : return XK_Page_Up;
            case 34 : return XK_Page_Down;
            case 45 : return XK_Insert;

            case 37 : return XK_Left;
            case 38 : return XK_Up;
            case 39 : return XK_Right;
            case 40 : return XK_Down;

            case 16 : return XK_Shift_L;
            case 17 : return XK_Control_L;
            case 18 : return XK_Alt_L; // also: Option-key on Mac

            case 224 : return XK_Meta_L;
            case 225 : return XK_ISO_Level3_Shift; // AltGr
            case 91 : return XK_Super_L; // also: Windows-key
            case 92 : return XK_Super_R; // also: Windows-key
            case 93 : return XK_Menu; // also: Windows-Menu, Command on Mac
            default: return null;
        }
    }
    return {
        hasShortcutModifier : hasShortcutModifier,
        hasCharModifier : hasCharModifier,
        ModifierSync : ModifierSync,
        getKey : getKey,
        getKeysym : getKeysym,
        keysymFromKeyCode : keysymFromKeyCode,
        nonCharacterKey : nonCharacterKey,
        substituteCodepoint : substituteCodepoint
    };
})();

// Takes a DOM keyboard event and:
// - determines which keysym it represents
// - determines a keyId  identifying the key that was pressed (corresponding to the key/keyCode properties on the DOM event)
// - synthesizes events to synchronize modifier key state between which modifiers are actually down, and which we thought were down
// - marks each event with an 'escape' property if a modifier was down which should be "escaped"
// - generates a "stall" event in cases where it might be necessary to wait and see if a keypress event follows a keydown
// This information is collected into an object which is passed to the next() function. (one call per event)
function KeyEventDecoder(modifierState, next) {
    "use strict";
    function sendAll(evts) {
        for (var i = 0; i < evts.length; ++i) {
            next(evts[i]);
        }
    }
    function process(evt, type) {
        var result = {type: type};
        var keyId = kbdUtil.getKey(evt);
        if (keyId) {
            result.keyId = keyId;
        }

        var keysym = kbdUtil.getKeysym(evt);

        var hasModifier = modifierState.hasShortcutModifier() || !!modifierState.activeCharModifier();
        // Is this a case where we have to decide on the keysym right away, rather than waiting for the keypress?
        // "special" keys like enter, tab or backspace don't send keypress events,
        // and some browsers don't send keypresses at all if a modifier is down
        if (keysym && (type !== 'keydown' || kbdUtil.nonCharacterKey(evt) || hasModifier)) {
            result.keysym = keysym;
        }

        var isShift = evt.keyCode === 0x10 || evt.key === 'Shift';

        // Should we prevent the browser from handling the event?
        // Doing so on a keydown (in most browsers) prevents keypress from being generated
        // so only do that if we have to.
        var suppress = !isShift && (type !== 'keydown' || modifierState.hasShortcutModifier() || !!kbdUtil.nonCharacterKey(evt));

        // If a char modifier is down on a keydown, we need to insert a stall,
        // so VerifyCharModifier knows to wait and see if a keypress is comnig
        var stall = type === 'keydown' && modifierState.activeCharModifier() && !kbdUtil.nonCharacterKey(evt);

        // if a char modifier is pressed, get the keys it consists of (on Windows, AltGr is equivalent to Ctrl+Alt)
        var active = modifierState.activeCharModifier();

        // If we have a char modifier down, and we're able to determine a keysym reliably
        // then (a) we know to treat the modifier as a char modifier,
        // and (b) we'll have to "escape" the modifier to undo the modifier when sending the char.
        if (active && keysym) {
            var isCharModifier = false;
            for (var i  = 0; i < active.length; ++i) {
                if (active[i] === keysym.keysym) {
                    isCharModifier = true;
                }
            }
            if (type === 'keypress' && !isCharModifier) {
                result.escape = modifierState.activeCharModifier();
            }
        }

        if (stall) {
            // insert a fake "stall" event
            next({type: 'stall'});
        }
        next(result);

        return suppress;
    }

    return {
        keydown: function(evt) {
            sendAll(modifierState.keydown(evt));
            return process(evt, 'keydown');
        },
        keypress: function(evt) {
            return process(evt, 'keypress');
        },
        keyup: function(evt) {
            sendAll(modifierState.keyup(evt));
            return process(evt, 'keyup');
        },
        syncModifiers: function(evt) {
            sendAll(modifierState.syncAny(evt));
        },
        releaseAll: function() { next({type: 'releaseall'}); }
    };
}

// Combines keydown and keypress events where necessary to handle char modifiers.
// On some OS'es, a char modifier is sometimes used as a shortcut modifier.
// For example, on Windows, AltGr is synonymous with Ctrl-Alt. On a Danish keyboard layout, AltGr-2 yields a @, but Ctrl-Alt-D does nothing
// so when used with the '2' key, Ctrl-Alt counts as a char modifier (and should be escaped), but when used with 'D', it does not.
// The only way we can distinguish these cases is to wait and see if a keypress event arrives
// When we receive a "stall" event, wait a few ms before processing the next keydown. If a keypress has also arrived, merge the two
function VerifyCharModifier(next) {
    "use strict";
    var queue = [];
    var timer = null;
    function process() {
        if (timer) {
            return;
        }

        var delayProcess = function () {
            clearTimeout(timer);
            timer = null;
            process();
        };

        while (queue.length !== 0) {
            var cur = queue[0];
            queue = queue.splice(1);
            switch (cur.type) {
            case 'stall':
                // insert a delay before processing available events.
                /* jshint loopfunc: true */
                timer = setTimeout(delayProcess, 5);
                /* jshint loopfunc: false */
                return;
            case 'keydown':
                // is the next element a keypress? Then we should merge the two
                if (queue.length !== 0 && queue[0].type === 'keypress') {
                    // Firefox sends keypress even when no char is generated.
                    // so, if keypress keysym is the same as we'd have guessed from keydown,
                    // the modifier didn't have any effect, and should not be escaped
                    if (queue[0].escape && (!cur.keysym || cur.keysym.keysym !== queue[0].keysym.keysym)) {
                        cur.escape = queue[0].escape;
                    }
                    cur.keysym = queue[0].keysym;
                    queue = queue.splice(1);
                }
                break;
            }

            // swallow stall events, and pass all others to the next stage
            if (cur.type !== 'stall') {
                next(cur);
            }
        }
    }
    return function(evt) {
        queue.push(evt);
        process();
    };
}

// Keeps track of which keys we (and the server) believe are down
// When a keyup is received, match it against this list, to determine the corresponding keysym(s)
// in some cases, a single key may produce multiple keysyms, so the corresponding keyup event must release all of these chars
// key repeat events should be merged into a single entry.
// Because we can't always identify which entry a keydown or keyup event corresponds to, we sometimes have to guess
function TrackKeyState(next) {
    "use strict";
    var state = [];

    return function (evt) {
        var last = state.length !== 0 ? state[state.length-1] : null;

        switch (evt.type) {
        case 'keydown':
            // insert a new entry if last seen key was different.
            if (!last || !evt.keyId || last.keyId !== evt.keyId) {
                last = {keyId: evt.keyId, keysyms: {}};
                state.push(last);
            }
            if (evt.keysym) {
                // make sure last event contains this keysym (a single "logical" keyevent
                // can cause multiple key events to be sent to the VNC server)
                last.keysyms[evt.keysym.keysym] = evt.keysym;
                last.ignoreKeyPress = true;
                next(evt);
            }
            break;
        case 'keypress':
            if (!last) {
                last = {keyId: evt.keyId, keysyms: {}};
                state.push(last);
            }
            if (!evt.keysym) {
                console.log('keypress with no keysym:', evt);
            }

            // If we didn't expect a keypress, and already sent a keydown to the VNC server
            // based on the keydown, make sure to skip this event.
            if (evt.keysym && !last.ignoreKeyPress) {
                last.keysyms[evt.keysym.keysym] = evt.keysym;
                evt.type = 'keydown';
                next(evt);
            }
            break;
        case 'keyup':
            if (state.length === 0) {
                return;
            }
            var idx = null;
            // do we have a matching key tracked as being down?
            for (var i = 0; i !== state.length; ++i) {
                if (state[i].keyId === evt.keyId) {
                    idx = i;
                    break;
                }
            }
            // if we couldn't find a match (it happens), assume it was the last key pressed
            if (idx === null) {
                idx = state.length - 1;
            }

            var item = state.splice(idx, 1)[0];
            // for each keysym tracked by this key entry, clone the current event and override the keysym
            var clone = (function(){
                function Clone(){}
                return function (obj) { Clone.prototype=obj; return new Clone(); };
            }());
            for (var key in item.keysyms) {
                var out = clone(evt);
                out.keysym = item.keysyms[key];
                next(out);
            }
            break;
        case 'releaseall':
            /* jshint shadow: true */
            for (var i = 0; i < state.length; ++i) {
                for (var key in state[i].keysyms) {
                    var keysym = state[i].keysyms[key];
                    next({keyId: 0, keysym: keysym, type: 'keyup'});
                }
            }
            /* jshint shadow: false */
            state = [];
        }
    };
}

// Handles "escaping" of modifiers: if a char modifier is used to produce a keysym (such as AltGr-2 to generate an @),
// then the modifier must be "undone" before sending the @, and "redone" afterwards.
function EscapeModifiers(next) {
    "use strict";
    return function(evt) {
        if (evt.type !== 'keydown' || evt.escape === undefined) {
            next(evt);
            return;
        }
        // undo modifiers
        for (var i = 0; i < evt.escape.length; ++i) {
            next({type: 'keyup', keyId: 0, keysym: keysyms.lookup(evt.escape[i])});
        }
        // send the character event
        next(evt);
        // redo modifiers
        /* jshint shadow: true */
        for (var i = 0; i < evt.escape.length; ++i) {
            next({type: 'keydown', keyId: 0, keysym: keysyms.lookup(evt.escape[i])});
        }
        /* jshint shadow: false */
    };
}
;
define("vnc-keyboard", ["vnc-util"], function(){});

/*
 * noVNC: HTML5 VNC client
 * Copyright (C) 2012 Joel Martin
 * Copyright (C) 2013 Samuel Mannehed for Cendio AB
 * Licensed under MPL 2.0 or any later version (see LICENSE.txt)
 */

/*jslint browser: true, white: false */
/*global window, Util */

var Keyboard, Mouse;

(function () {
    "use strict";

    //
    // Keyboard event handler
    //

    Keyboard = function (defaults) {
        this._keyDownList = [];         // List of depressed keys
                                        // (even if they are happy)

        Util.set_defaults(this, defaults, {
            'target': document,
            'focused': true
        });

        // create the keyboard handler
        this._handler = new KeyEventDecoder(kbdUtil.ModifierSync(),
            VerifyCharModifier( /* jshint newcap: false */
                TrackKeyState(
                    EscapeModifiers(this._handleRfbEvent.bind(this))
                )
            )
        ); /* jshint newcap: true */

        // keep these here so we can refer to them later
        this._eventHandlers = {
            'keyup': this._handleKeyUp.bind(this),
            'keydown': this._handleKeyDown.bind(this),
            'keypress': this._handleKeyPress.bind(this),
            'blur': this._allKeysUp.bind(this)
        };
    };

    Keyboard.prototype = {
        // private methods

        _handleRfbEvent: function (e) {
            if (this._onKeyPress) {
                Util.Debug("onKeyPress " + (e.type == 'keydown' ? "down" : "up") +
                           ", keysym: " + e.keysym.keysym + "(" + e.keysym.keyname + ")");
                this._onKeyPress(e.keysym.keysym, e.type == 'keydown');
            }
        },

        _handleKeyDown: function (e) {
            if (!this._focused) { return true; }

            if (this._handler.keydown(e)) {
                // Suppress bubbling/default actions
                Util.stopEvent(e);
                return false;
            } else {
                // Allow the event to bubble and become a keyPress event which
                // will have the character code translated
                return true;
            }
        },

        _handleKeyPress: function (e) {
            if (!this._focused) { return true; }

            if (this._handler.keypress(e)) {
                // Suppress bubbling/default actions
                Util.stopEvent(e);
                return false;
            } else {
                // Allow the event to bubble and become a keyPress event which
                // will have the character code translated
                return true;
            }
        },

        _handleKeyUp: function (e) {
            if (!this._focused) { return true; }

            if (this._handler.keyup(e)) {
                // Suppress bubbling/default actions
                Util.stopEvent(e);
                return false;
            } else {
                // Allow the event to bubble and become a keyPress event which
                // will have the character code translated
                return true;
            }
        },

        _allKeysUp: function () {
            Util.Debug(">> Keyboard.allKeysUp");
            this._handler.releaseAll();
            Util.Debug("<< Keyboard.allKeysUp");
        },

        // Public methods

        grab: function () {
            //Util.Debug(">> Keyboard.grab");
            var c = this._target;

            Util.addEvent(c, 'keydown', this._eventHandlers.keydown);
            Util.addEvent(c, 'keyup', this._eventHandlers.keyup);
            Util.addEvent(c, 'keypress', this._eventHandlers.keypress);

            // Release (key up) if window loses focus
            Util.addEvent(window, 'blur', this._eventHandlers.blur);

            //Util.Debug("<< Keyboard.grab");
        },

        ungrab: function () {
            //Util.Debug(">> Keyboard.ungrab");
            var c = this._target;

            Util.removeEvent(c, 'keydown', this._eventHandlers.keydown);
            Util.removeEvent(c, 'keyup', this._eventHandlers.keyup);
            Util.removeEvent(c, 'keypress', this._eventHandlers.keypress);
            Util.removeEvent(window, 'blur', this._eventHandlers.blur);

            // Release (key up) all keys that are in a down state
            this._allKeysUp();

            //Util.Debug(">> Keyboard.ungrab");
        },

        sync: function (e) {
            this._handler.syncModifiers(e);
        }
    };

    Util.make_properties(Keyboard, [
        ['target',     'wo', 'dom'],  // DOM element that captures keyboard input
        ['focused',    'rw', 'bool'], // Capture and send key events

        ['onKeyPress', 'rw', 'func'] // Handler for key press/release
    ]);

    //
    // Mouse event handler
    //

    Mouse = function (defaults) {
        this._mouseCaptured  = false;

        this._doubleClickTimer = null;
        this._lastTouchPos = null;

        // Configuration attributes
        Util.set_defaults(this, defaults, {
            'target': document,
            'focused': true,
            'scale': 1.0,
            'touchButton': 1
        });

        this._eventHandlers = {
            'mousedown': this._handleMouseDown.bind(this),
            'mouseup': this._handleMouseUp.bind(this),
            'mousemove': this._handleMouseMove.bind(this),
            'mousewheel': this._handleMouseWheel.bind(this),
            'mousedisable': this._handleMouseDisable.bind(this)
        };
    };

    Mouse.prototype = {
        // private methods
        _captureMouse: function () {
            // capturing the mouse ensures we get the mouseup event
            if (this._target.setCapture) {
                this._target.setCapture();
            }

            // some browsers give us mouseup events regardless,
            // so if we never captured the mouse, we can disregard the event
            this._mouseCaptured = true;
        },

        _releaseMouse: function () {
            if (this._target.releaseCapture) {
                this._target.releaseCapture();
            }
            this._mouseCaptured = false;
        },

        _resetDoubleClickTimer: function () {
            this._doubleClickTimer = null;
        },

        _handleMouseButton: function (e, down) {
            if (!this._focused) { return true; }

            if (this._notify) {
                this._notify(e);
            }

            var evt = (e ? e : window.event);
            var pos = Util.getEventPosition(e, this._target, this._scale);

            var bmask;
            if (e.touches || e.changedTouches) {
                // Touch device

                // When two touches occur within 500 ms of each other and are
                // closer than 20 pixels together a double click is triggered.
                if (down == 1) {
                    if (this._doubleClickTimer === null) {
                        this._lastTouchPos = pos;
                    } else {
                        clearTimeout(this._doubleClickTimer);

                        // When the distance between the two touches is small enough
                        // force the position of the latter touch to the position of
                        // the first.

                        var xs = this._lastTouchPos.x - pos.x;
                        var ys = this._lastTouchPos.y - pos.y;
                        var d = Math.sqrt((xs * xs) + (ys * ys));

                        // The goal is to trigger on a certain physical width, the
                        // devicePixelRatio brings us a bit closer but is not optimal.
                        if (d < 20 * window.devicePixelRatio) {
                            pos = this._lastTouchPos;
                        }
                    }
                    this._doubleClickTimer = setTimeout(this._resetDoubleClickTimer.bind(this), 500);
                }
                bmask = this._touchButton;
                // If bmask is set
            } else if (evt.which) {
                /* everything except IE */
                bmask = 1 << evt.button;
            } else {
                /* IE including 9 */
                bmask = (evt.button & 0x1) +      // Left
                        (evt.button & 0x2) * 2 +  // Right
                        (evt.button & 0x4) / 2;   // Middle
            }

            if (this._onMouseButton) {
                Util.Debug("onMouseButton " + (down ? "down" : "up") +
                           ", x: " + pos.x + ", y: " + pos.y + ", bmask: " + bmask);
                this._onMouseButton(pos.x, pos.y, down, bmask);
            }
            Util.stopEvent(e);
            return false;
        },

        _handleMouseDown: function (e) {
            this._captureMouse();
            this._handleMouseButton(e, 1);
        },

        _handleMouseUp: function (e) {
            if (!this._mouseCaptured) { return; }

            this._handleMouseButton(e, 0);
            this._releaseMouse();
        },

        _handleMouseWheel: function (e) {
            if (!this._focused) { return true; }

            if (this._notify) {
                this._notify(e);
            }

            var evt = (e ? e : window.event);
            var pos = Util.getEventPosition(e, this._target, this._scale);
            var wheelData = evt.detail ? evt.detail * -1 : evt.wheelDelta / 40;
            var bmask;
            if (wheelData > 0) {
                bmask = 1 << 3;
            } else {
                bmask = 1 << 4;
            }

            if (this._onMouseButton) {
                this._onMouseButton(pos.x, pos.y, 1, bmask);
                this._onMouseButton(pos.x, pos.y, 0, bmask);
            }
            Util.stopEvent(e);
            return false;
        },

        _handleMouseMove: function (e) {
            if (! this._focused) { return true; }

            if (this._notify) {
                this._notify(e);
            }

            var evt = (e ? e : window.event);
            var pos = Util.getEventPosition(e, this._target, this._scale);
            if (this._onMouseMove) {
                this._onMouseMove(pos.x, pos.y);
            }
            Util.stopEvent(e);
            return false;
        },

        _handleMouseDisable: function (e) {
            if (!this._focused) { return true; }

            var evt = (e ? e : window.event);
            var pos = Util.getEventPosition(e, this._target, this._scale);

            /* Stop propagation if inside canvas area */
            if ((pos.realx >= 0) && (pos.realy >= 0) &&
                (pos.realx < this._target.offsetWidth) &&
                (pos.realy < this._target.offsetHeight)) {
                //Util.Debug("mouse event disabled");
                Util.stopEvent(e);
                return false;
            }

            return true;
        },


        // Public methods
        grab: function () {
            var c = this._target;

            if ('ontouchstart' in document.documentElement) {
                Util.addEvent(c, 'touchstart', this._eventHandlers.mousedown);
                Util.addEvent(window, 'touchend', this._eventHandlers.mouseup);
                Util.addEvent(c, 'touchend', this._eventHandlers.mouseup);
                Util.addEvent(c, 'touchmove', this._eventHandlers.mousemove);
            } else {
                Util.addEvent(c, 'mousedown', this._eventHandlers.mousedown);
                Util.addEvent(window, 'mouseup', this._eventHandlers.mouseup);
                Util.addEvent(c, 'mouseup', this._eventHandlers.mouseup);
                Util.addEvent(c, 'mousemove', this._eventHandlers.mousemove);
                Util.addEvent(c, (Util.Engine.gecko) ? 'DOMMouseScroll' : 'mousewheel',
                              this._eventHandlers.mousewheel);
            }

            /* Work around right and middle click browser behaviors */
            Util.addEvent(document, 'click', this._eventHandlers.mousedisable);
            Util.addEvent(document.body, 'contextmenu', this._eventHandlers.mousedisable);
        },

        ungrab: function () {
            var c = this._target;

            if ('ontouchstart' in document.documentElement) {
                Util.removeEvent(c, 'touchstart', this._eventHandlers.mousedown);
                Util.removeEvent(window, 'touchend', this._eventHandlers.mouseup);
                Util.removeEvent(c, 'touchend', this._eventHandlers.mouseup);
                Util.removeEvent(c, 'touchmove', this._eventHandlers.mousemove);
            } else {
                Util.removeEvent(c, 'mousedown', this._eventHandlers.mousedown);
                Util.removeEvent(window, 'mouseup', this._eventHandlers.mouseup);
                Util.removeEvent(c, 'mouseup', this._eventHandlers.mouseup);
                Util.removeEvent(c, 'mousemove', this._eventHandlers.mousemove);
                Util.removeEvent(c, (Util.Engine.gecko) ? 'DOMMouseScroll' : 'mousewheel',
                                 this._eventHandlers.mousewheel);
            }

            /* Work around right and middle click browser behaviors */
            Util.removeEvent(document, 'click', this._eventHandlers.mousedisable);
            Util.removeEvent(document.body, 'contextmenu', this._eventHandlers.mousedisable);

        }
    };

    Util.make_properties(Mouse, [
        ['target',         'ro', 'dom'],   // DOM element that captures mouse input
        ['notify',         'ro', 'func'],  // Function to call to notify whenever a mouse event is received
        ['focused',        'rw', 'bool'],  // Capture and send mouse clicks/movement
        ['scale',          'rw', 'float'], // Viewport scale factor 0.0 - 1.0

        ['onMouseButton',  'rw', 'func'],  // Handler for mouse button click/release
        ['onMouseMove',    'rw', 'func'],  // Handler for mouse movement
        ['touchButton',    'rw', 'int']    // Button mask (1, 2, 4) for touch devices (0 means ignore clicks)
    ]);
})();

define("vnc-input", ["vnc-util"], function(){});

/*
 * noVNC: HTML5 VNC client
 * Copyright (C) 2012 Joel Martin
 * Licensed under MPL 2.0 (see LICENSE.txt)
 *
 * See README.md for usage and integration instructions.
 */

/*jslint browser: true, white: false */
/*global Util, Base64, changeCursor */

var Display;

(function () {
    "use strict";

    Display = function (defaults) {
        this._drawCtx = null;
        this._c_forceCanvas = false;

        this._renderQ = [];  // queue drawing actions for in-oder rendering

        // the full frame buffer (logical canvas) size
        this._fb_width = 0;
        this._fb_height = 0;

        // the visible "physical canvas" viewport
        this._viewportLoc = { 'x': 0, 'y': 0, 'w': 0, 'h': 0 };
        this._cleanRect = { 'x1': 0, 'y1': 0, 'x2': -1, 'y2': -1 };

        this._prevDrawStyle = "";
        this._tile = null;
        this._tile16x16 = null;
        this._tile_x = 0;
        this._tile_y = 0;

        Util.set_defaults(this, defaults, {
            'true_color': true,
            'colourMap': [],
            'scale': 1.0,
            'viewport': false,
            'render_mode': ''
        });

        Util.Debug(">> Display.constructor");

        if (!this._target) {
            throw new Error("Target must be set");
        }

        if (typeof this._target === 'string') {
            throw new Error('target must be a DOM element');
        }

        if (!this._target.getContext) {
            throw new Error("no getContext method");
        }

        if (!this._drawCtx) {
            this._drawCtx = this._target.getContext('2d');
        }

        Util.Debug("User Agent: " + navigator.userAgent);
        if (Util.Engine.gecko) { Util.Debug("Browser: gecko " + Util.Engine.gecko); }
        if (Util.Engine.webkit) { Util.Debug("Browser: webkit " + Util.Engine.webkit); }
        if (Util.Engine.trident) { Util.Debug("Browser: trident " + Util.Engine.trident); }
        if (Util.Engine.presto) { Util.Debug("Browser: presto " + Util.Engine.presto); }

        this.clear();

        // Check canvas features
        if ('createImageData' in this._drawCtx) {
            this._render_mode = 'canvas rendering';
        } else {
            throw new Error("Canvas does not support createImageData");
        }

        if (this._prefer_js === null) {
            Util.Info("Prefering javascript operations");
            this._prefer_js = true;
        }

        // Determine browser support for setting the cursor via data URI scheme
        var curDat = [];
        for (var i = 0; i < 8 * 8 * 4; i++) {
            curDat.push(255);
        }
        try {
            var curSave = this._target.style.cursor;
            Display.changeCursor(this._target, curDat, curDat, 2, 2, 8, 8);
            if (this._target.style.cursor) {
                if (this._cursor_uri === null || this._cursor_uri === undefined) {
                    this._cursor_uri = true;
                }
                Util.Info("Data URI scheme cursor supported");
            } else {
                if (this._cursor_uri === null || this._cursor_uri === undefined) {
                    this._cursor_uri = false;
                }
                Util.Warn("Data URI scheme cursor not supported");
            }
            this._target.style.cursor = curSave;
        } catch (exc) {
            Util.Error("Data URI scheme cursor test exception: " + exc);
            this._cursor_uri = false;
        }

        Util.Debug("<< Display.constructor");
    };

    Display.prototype = {
        // Public methods
        viewportChange: function (deltaX, deltaY, width, height) {
            var vp = this._viewportLoc;
            var cr = this._cleanRect;
            var canvas = this._target;

            if (!this._viewport) {
                Util.Debug("Setting viewport to full display region");
                deltaX = -vp.w;  // clamped later of out of bounds
                deltaY = -vp.h;
                width = this._fb_width;
                height = this._fb_height;
            }

            if (typeof(deltaX) === "undefined") { deltaX = 0; }
            if (typeof(deltaY) === "undefined") { deltaY = 0; }
            if (typeof(width) === "undefined") { width = vp.w; }
            if (typeof(height) === "undefined") { height = vp.h; }

            // Size change
            if (width > this._fb_width) { width = this._fb_width; }
            if (height > this._fb_height) { height = this._fb_height; }

            if (vp.w !== width || vp.h !== height) {
                // Change width
                if (width < vp.w &&  cr.x2 > vp.x + width - 1) {
                    cr.x2 = vp.x + width - 1;
                }
                vp.w = width;

                // Change height
                if (height < vp.h &&  cr.y2 > vp.y + height - 1) {
                    cr.y2 = vp.y + height - 1;
                }
                vp.h = height;

                var saveImg = null;
                if (vp.w > 0 && vp.h > 0 && canvas.width > 0 && canvas.height > 0) {
                    var img_width = canvas.width < vp.w ? canvas.width : vp.w;
                    var img_height = canvas.height < vp.h ? canvas.height : vp.h;
                    saveImg = this._drawCtx.getImageData(0, 0, img_width, img_height);
                }

                canvas.width = vp.w;
                canvas.height = vp.h;

                if (saveImg) {
                    this._drawCtx.putImageData(saveImg, 0, 0);
                }
            }

            var vx2 = vp.x + vp.w - 1;
            var vy2 = vp.y + vp.h - 1;

            // Position change

            if (deltaX < 0 && vp.x + deltaX < 0) {
                deltaX = -vp.x;
            }
            if (vx2 + deltaX >= this._fb_width) {
                deltaX -= vx2 + deltaX - this._fb_width + 1;
            }

            if (vp.y + deltaY < 0) {
                deltaY = -vp.y;
            }
            if (vy2 + deltaY >= this._fb_height) {
                deltaY -= (vy2 + deltaY - this._fb_height + 1);
            }

            if (deltaX === 0 && deltaY === 0) {
                return;
            }
            Util.Debug("viewportChange deltaX: " + deltaX + ", deltaY: " + deltaY);

            vp.x += deltaX;
            vx2 += deltaX;
            vp.y += deltaY;
            vy2 += deltaY;

            // Update the clean rectangle
            if (vp.x > cr.x1) {
                cr.x1 = vp.x;
            }
            if (vx2 < cr.x2) {
                cr.x2 = vx2;
            }
            if (vp.y > cr.y1) {
                cr.y1 = vp.y;
            }
            if (vy2 < cr.y2) {
                cr.y2 = vy2;
            }

            var x1, w;
            if (deltaX < 0) {
                // Shift viewport left, redraw left section
                x1 = 0;
                w = -deltaX;
            } else {
                // Shift viewport right, redraw right section
                x1 = vp.w - deltaX;
                w = deltaX;
            }

            var y1, h;
            if (deltaY < 0) {
                // Shift viewport up, redraw top section
                y1 = 0;
                h = -deltaY;
            } else {
                // Shift viewport down, redraw bottom section
                y1 = vp.h - deltaY;
                h = deltaY;
            }

            // Copy the valid part of the viewport to the shifted location
            var saveStyle = this._drawCtx.fillStyle;
            this._drawCtx.fillStyle = "rgb(255,255,255)";
            if (deltaX !== 0) {
                this._drawCtx.drawImage(canvas, 0, 0, vp.w, vp.h, -deltaX, 0, vp.w, vp.h);
                this._drawCtx.fillRect(x1, 0, w, vp.h);
            }
            if (deltaY !== 0) {
                this._drawCtx.drawImage(canvas, 0, 0, vp.w, vp.h, 0, -deltaY, vp.w, vp.h);
                this._drawCtx.fillRect(0, y1, vp.w, h);
            }
            this._drawCtx.fillStyle = saveStyle;
        },

        // Return a map of clean and dirty areas of the viewport and reset the
        // tracking of clean and dirty areas
        //
        // Returns: { 'cleanBox': { 'x': x, 'y': y, 'w': w, 'h': h},
        //            'dirtyBoxes': [{ 'x': x, 'y': y, 'w': w, 'h': h }, ...] }
        getCleanDirtyReset: function () {
            var vp = this._viewportLoc;
            var cr = this._cleanRect;

            var cleanBox = { 'x': cr.x1, 'y': cr.y1,
                             'w': cr.x2 - cr.x1 + 1, 'h': cr.y2 - cr.y1 + 1 };

            var dirtyBoxes = [];
            if (cr.x1 >= cr.x2 || cr.y1 >= cr.y2) {
                // Whole viewport is dirty
                dirtyBoxes.push({ 'x': vp.x, 'y': vp.y, 'w': vp.w, 'h': vp.h });
            } else {
                // Redraw dirty regions
                var vx2 = vp.x + vp.w - 1;
                var vy2 = vp.y + vp.h - 1;

                if (vp.x < cr.x1) {
                    // left side dirty region
                    dirtyBoxes.push({'x': vp.x, 'y': vp.y,
                                     'w': cr.x1 - vp.x + 1, 'h': vp.h});
                }
                if (vx2 > cr.x2) {
                    // right side dirty region
                    dirtyBoxes.push({'x': cr.x2 + 1, 'y': vp.y,
                                     'w': vx2 - cr.x2, 'h': vp.h});
                }
                if(vp.y < cr.y1) {
                    // top/middle dirty region
                    dirtyBoxes.push({'x': cr.x1, 'y': vp.y,
                                     'w': cr.x2 - cr.x1 + 1, 'h': cr.y1 - vp.y});
                }
                if (vy2 > cr.y2) {
                    // bottom/middle dirty region
                    dirtyBoxes.push({'x': cr.x1, 'y': cr.y2 + 1,
                                     'w': cr.x2 - cr.x1 + 1, 'h': vy2 - cr.y2});
                }
            }

            this._cleanRect = {'x1': vp.x, 'y1': vp.y,
                               'x2': vp.x + vp.w - 1, 'y2': vp.y + vp.h - 1};

            return {'cleanBox': cleanBox, 'dirtyBoxes': dirtyBoxes};
        },

        absX: function (x) {
            return x + this._viewportLoc.x;
        },

        absY: function (y) {
            return y + this._viewportLoc.y;
        },

        resize: function (width, height) {
            this._prevDrawStyle = "";

            this._fb_width = width;
            this._fb_height = height;

            this._rescale(this._scale);

            this.viewportChange();
        },

        clear: function () {
            if (this._logo) {
                this.resize(this._logo.width, this._logo.height);
                this.blitStringImage(this._logo.data, 0, 0);
            } else {
                if (Util.Engine.trident === 6) {
                    // NB(directxman12): there's a bug in IE10 where we can fail to actually
                    //                   clear the canvas here because of the resize.
                    //                   Clearing the current viewport first fixes the issue
                    this._drawCtx.clearRect(0, 0, this._viewportLoc.w, this._viewportLoc.h);
                }
                this.resize(240, 20);
                this._drawCtx.clearRect(0, 0, this._viewportLoc.w, this._viewportLoc.h);
            }

            this._renderQ = [];
        },

        fillRect: function (x, y, width, height, color) {
            this._setFillColor(color);
            this._drawCtx.fillRect(x - this._viewportLoc.x, y - this._viewportLoc.y, width, height);
        },

        copyImage: function (old_x, old_y, new_x, new_y, w, h) {
            var x1 = old_x - this._viewportLoc.x;
            var y1 = old_y - this._viewportLoc.y;
            var x2 = new_x - this._viewportLoc.x;
            var y2 = new_y - this._viewportLoc.y;

            this._drawCtx.drawImage(this._target, x1, y1, w, h, x2, y2, w, h);
        },

        // start updating a tile
        startTile: function (x, y, width, height, color) {
            this._tile_x = x;
            this._tile_y = y;
            if (width === 16 && height === 16) {
                this._tile = this._tile16x16;
            } else {
                this._tile = this._drawCtx.createImageData(width, height);
            }

            if (this._prefer_js) {
                var bgr;
                if (this._true_color) {
                    bgr = color;
                } else {
                    bgr = this._colourMap[color[0]];
                }
                var red = bgr[2];
                var green = bgr[1];
                var blue = bgr[0];

                var data = this._tile.data;
                for (var i = 0; i < width * height * 4; i += 4) {
                    data[i] = red;
                    data[i + 1] = green;
                    data[i + 2] = blue;
                    data[i + 3] = 255;
                }
            } else {
                this.fillRect(x, y, width, height, color);
            }
        },

        // update sub-rectangle of the current tile
        subTile: function (x, y, w, h, color) {
            if (this._prefer_js) {
                var bgr;
                if (this._true_color) {
                    bgr = color;
                } else {
                    bgr = this._colourMap[color[0]];
                }
                var red = bgr[2];
                var green = bgr[1];
                var blue = bgr[0];
                var xend = x + w;
                var yend = y + h;

                var data = this._tile.data;
                var width = this._tile.width;
                for (var j = y; j < yend; j++) {
                    for (var i = x; i < xend; i++) {
                        var p = (i + (j * width)) * 4;
                        data[p] = red;
                        data[p + 1] = green;
                        data[p + 2] = blue;
                        data[p + 3] = 255;
                    }
                }
            } else {
                this.fillRect(this._tile_x + x, this._tile_y + y, w, h, color);
            }
        },

        // draw the current tile to the screen
        finishTile: function () {
            if (this._prefer_js) {
                this._drawCtx.putImageData(this._tile, this._tile_x - this._viewportLoc.x,
                                           this._tile_y - this._viewportLoc.y);
            }
            // else: No-op -- already done by setSubTile
        },

        blitImage: function (x, y, width, height, arr, offset) {
            if (this._true_color) {
                this._bgrxImageData(x, y, this._viewportLoc.x, this._viewportLoc.y, width, height, arr, offset);
            } else {
                this._cmapImageData(x, y, this._viewportLoc.x, this._viewportLoc.y, width, height, arr, offset);
            }
        },

        blitRgbImage: function (x, y , width, height, arr, offset) {
            if (this._true_color) {
                this._rgbImageData(x, y, this._viewportLoc.x, this._viewportLoc.y, width, height, arr, offset);
            } else {
                // probably wrong?
                this._cmapImageData(x, y, this._viewportLoc.x, this._viewportLoc.y, width, height, arr, offset);
            }
        },

        blitStringImage: function (str, x, y) {
            var img = new Image();
            img.onload = function () {
                this._drawCtx.drawImage(img, x - this._viewportLoc.x, y - this._viewportLoc.y);
            }.bind(this);
            img.src = str;
            return img; // for debugging purposes
        },

        // wrap ctx.drawImage but relative to viewport
        drawImage: function (img, x, y) {
            this._drawCtx.drawImage(img, x - this._viewportLoc.x, y - this._viewportLoc.y);
        },

        renderQ_push: function (action) {
            this._renderQ.push(action);
            if (this._renderQ.length === 1) {
                // If this can be rendered immediately it will be, otherwise
                // the scanner will start polling the queue (every
                // requestAnimationFrame interval)
                this._scan_renderQ();
            }
        },

        changeCursor: function (pixels, mask, hotx, hoty, w, h) {
            if (this._cursor_uri === false) {
                Util.Warn("changeCursor called but no cursor data URI support");
                return;
            }

            if (this._true_color) {
                Display.changeCursor(this._target, pixels, mask, hotx, hoty, w, h);
            } else {
                Display.changeCursor(this._target, pixels, mask, hotx, hoty, w, h, this._colourMap);
            }
        },

        defaultCursor: function () {
            this._target.style.cursor = "default";
        },

        // Overridden getters/setters
        get_context: function () {
            return this._drawCtx;
        },

        set_scale: function (scale) {
            this._rescale(scale);
        },

        set_width: function (w) {
            this.resize(w, this._fb_height);
        },
        get_width: function () {
            return this._fb_width;
        },

        set_height: function (h) {
            this.resize(this._fb_width, h);
        },
        get_height: function () {
            return this._fb_height;
        },

        // Private Methods
        _rescale: function (factor) {
            var canvas = this._target;
            var properties = ['transform', 'WebkitTransform', 'MozTransform'];
            var transform_prop;
            while ((transform_prop = properties.shift())) {
                if (typeof canvas.style[transform_prop] !== 'undefined') {
                    break;
                }
            }

            if (transform_prop === null) {
                Util.Debug("No scaling support");
                return;
            }

            if (typeof(factor) === "undefined") {
                factor = this._scale;
            } else if (factor > 1.0) {
                factor = 1.0;
            } else if (factor < 0.1) {
                factor = 0.1;
            }

            if (this._scale === factor) {
                return;
            }

            this._scale = factor;
            var x = canvas.width - (canvas.width * factor);
            var y = canvas.height - (canvas.height * factor);
            canvas.style[transform_prop] = 'scale(' + this._scale + ') translate(-' + x + 'px, -' + y + 'px)';
        },

        _setFillColor: function (color) {
            var bgr;
            if (this._true_color) {
                bgr = color;
            } else {
                bgr = this._colourMap[color[0]];
            }

            var newStyle = 'rgb(' + bgr[2] + ',' + bgr[1] + ',' + bgr[0] + ')';
            if (newStyle !== this._prevDrawStyle) {
                this._drawCtx.fillStyle = newStyle;
                this._prevDrawStyle = newStyle;
            }
        },

        _rgbImageData: function (x, y, vx, vy, width, height, arr, offset) {
            var img = this._drawCtx.createImageData(width, height);
            var data = img.data;
            for (var i = 0, j = offset; i < width * height * 4; i += 4, j += 3) {
                data[i]     = arr[j];
                data[i + 1] = arr[j + 1];
                data[i + 2] = arr[j + 2];
                data[i + 3] = 255;  // Alpha
            }
            this._drawCtx.putImageData(img, x - vx, y - vy);
        },

        _bgrxImageData: function (x, y, vx, vy, width, height, arr, offset) {
            var img = this._drawCtx.createImageData(width, height);
            var data = img.data;
            for (var i = 0, j = offset; i < width * height * 4; i += 4, j += 4) {
                data[i]     = arr[j + 2];
                data[i + 1] = arr[j + 1];
                data[i + 2] = arr[j];
                data[i + 3] = 255;  // Alpha
            }
            this._drawCtx.putImageData(img, x - vx, y - vy);
        },

        _cmapImageData: function (x, y, vx, vy, width, height, arr, offset) {
            var img = this._drawCtx.createImageData(width, height);
            var data = img.data;
            var cmap = this._colourMap;
            for (var i = 0, j = offset; i < width * height * 4; i += 4, j++) {
                var bgr = cmap[arr[j]];
                data[i]     = bgr[2];
                data[i + 1] = bgr[1];
                data[i + 2] = bgr[0];
                data[i + 3] = 255;  // Alpha
            }
            this._drawCtx.putImageData(img, x - vx, y - vy);
        },

        _scan_renderQ: function () {
            var ready = true;
            while (ready && this._renderQ.length > 0) {
                var a = this._renderQ[0];
                switch (a.type) {
                    case 'copy':
                        this.copyImage(a.old_x, a.old_y, a.x, a.y, a.width, a.height);
                        break;
                    case 'fill':
                        this.fillRect(a.x, a.y, a.width, a.height, a.color);
                        break;
                    case 'blit':
                        this.blitImage(a.x, a.y, a.width, a.height, a.data, 0);
                        break;
                    case 'blitRgb':
                        this.blitRgbImage(a.x, a.y, a.width, a.height, a.data, 0);
                        break;
                    case 'img':
                        if (a.img.complete) {
                            this.drawImage(a.img, a.x, a.y);
                        } else {
                            // We need to wait for this image to 'load'
                            // to keep things in-order
                            ready = false;
                        }
                        break;
                }

                if (ready) {
                    this._renderQ.shift();
                }
            }

            if (this._renderQ.length > 0) {
                requestAnimFrame(this._scan_renderQ.bind(this));
            }
        },
    };

    Util.make_properties(Display, [
        ['target', 'wo', 'dom'],       // Canvas element for rendering
        ['context', 'ro', 'raw'],      // Canvas 2D context for rendering (read-only)
        ['logo', 'rw', 'raw'],         // Logo to display when cleared: {"width": w, "height": h, "data": data}
        ['true_color', 'rw', 'bool'],  // Use true-color pixel data
        ['colourMap', 'rw', 'arr'],    // Colour map array (when not true-color)
        ['scale', 'rw', 'float'],      // Display area scale factor 0.0 - 1.0
        ['viewport', 'rw', 'bool'],    // Use a viewport set with viewportChange()
        ['width', 'rw', 'int'],        // Display area width
        ['height', 'rw', 'int'],       // Display area height

        ['render_mode', 'ro', 'str'],  // Canvas rendering mode (read-only)

        ['prefer_js', 'rw', 'str'],    // Prefer Javascript over canvas methods
        ['cursor_uri', 'rw', 'raw']    // Can we render cursor using data URI
    ]);

    // Class Methods
    Display.changeCursor = function (target, pixels, mask, hotx, hoty, w0, h0, cmap) {
        var w = w0;
        var h = h0;
        if (h < w) {
            h = w;  // increase h to make it square
        } else {
            w = h;  // increase w to make it square
        }

        var cur = [];

        // Push multi-byte little-endian values
        cur.push16le = function (num) {
            this.push(num & 0xFF, (num >> 8) & 0xFF);
        };
        cur.push32le = function (num) {
            this.push(num & 0xFF,
                      (num >> 8) & 0xFF,
                      (num >> 16) & 0xFF,
                      (num >> 24) & 0xFF);
        };

        var IHDRsz = 40;
        var RGBsz = w * h * 4;
        var XORsz = Math.ceil((w * h) / 8.0);
        var ANDsz = Math.ceil((w * h) / 8.0);

        cur.push16le(0);        // 0: Reserved
        cur.push16le(2);        // 2: .CUR type
        cur.push16le(1);        // 4: Number of images, 1 for non-animated ico

        // Cursor #1 header (ICONDIRENTRY)
        cur.push(w);            // 6: width
        cur.push(h);            // 7: height
        cur.push(0);            // 8: colors, 0 -> true-color
        cur.push(0);            // 9: reserved
        cur.push16le(hotx);     // 10: hotspot x coordinate
        cur.push16le(hoty);     // 12: hotspot y coordinate
        cur.push32le(IHDRsz + RGBsz + XORsz + ANDsz);
                                // 14: cursor data byte size
        cur.push32le(22);       // 18: offset of cursor data in the file

        // Cursor #1 InfoHeader (ICONIMAGE/BITMAPINFO)
        cur.push32le(IHDRsz);   // 22: InfoHeader size
        cur.push32le(w);        // 26: Cursor width
        cur.push32le(h * 2);    // 30: XOR+AND height
        cur.push16le(1);        // 34: number of planes
        cur.push16le(32);       // 36: bits per pixel
        cur.push32le(0);        // 38: Type of compression

        cur.push32le(XORsz + ANDsz);
                                // 42: Size of Image
        cur.push32le(0);        // 46: reserved
        cur.push32le(0);        // 50: reserved
        cur.push32le(0);        // 54: reserved
        cur.push32le(0);        // 58: reserved

        // 62: color data (RGBQUAD icColors[])
        var y, x;
        for (y = h - 1; y >= 0; y--) {
            for (x = 0; x < w; x++) {
                if (x >= w0 || y >= h0) {
                    cur.push(0);  // blue
                    cur.push(0);  // green
                    cur.push(0);  // red
                    cur.push(0);  // alpha
                } else {
                    var idx = y * Math.ceil(w0 / 8) + Math.floor(x / 8);
                    var alpha = (mask[idx] << (x % 8)) & 0x80 ? 255 : 0;
                    if (cmap) {
                        idx = (w0 * y) + x;
                        var rgb = cmap[pixels[idx]];
                        cur.push(rgb[2]);  // blue
                        cur.push(rgb[1]);  // green
                        cur.push(rgb[0]);  // red
                        cur.push(alpha);   // alpha
                    } else {
                        idx = ((w0 * y) + x) * 4;
                        cur.push(pixels[idx + 2]); // blue
                        cur.push(pixels[idx + 1]); // green
                        cur.push(pixels[idx]);     // red
                        cur.push(alpha);           // alpha
                    }
                }
            }
        }

        // XOR/bitmask data (BYTE icXOR[])
        // (ignored, just needs to be the right size)
        for (y = 0; y < h; y++) {
            for (x = 0; x < Math.ceil(w / 8); x++) {
                cur.push(0);
            }
        }

        // AND/bitmask data (BYTE icAND[])
        // (ignored, just needs to be the right size)
        for (y = 0; y < h; y++) {
            for (x = 0; x < Math.ceil(w / 8); x++) {
                cur.push(0);
            }
        }

        var url = 'data:image/x-icon;base64,' + Base64.encode(cur);
        target.style.cursor = 'url(' + url + ')' + hotx + ' ' + hoty + ', default';
    };
})();

define("vnc-display", ["vnc-util"], function(){});

/*
 * JSUnzip
 *
 * Copyright (c) 2011 by Erik Moller
 * All Rights Reserved
 *
 * This software is provided 'as-is', without any express
 * or implied warranty.  In no event will the authors be
 * held liable for any damages arising from the use of
 * this software.
 *
 * Permission is granted to anyone to use this software
 * for any purpose, including commercial applications,
 * and to alter it and redistribute it freely, subject to
 * the following restrictions:
 *
 * 1. The origin of this software must not be
 *    misrepresented; you must not claim that you
 *    wrote the original software. If you use this
 *    software in a product, an acknowledgment in
 *    the product documentation would be appreciated
 *    but is not required.
 *
 * 2. Altered source versions must be plainly marked
 *    as such, and must not be misrepresented as
 *    being the original software.
 *
 * 3. This notice may not be removed or altered from
 *    any source distribution.
 */
 
var tinf;

function JSUnzip() {

    this.getInt = function(offset, size) {
        switch (size) {
        case 4:
            return  (this.data.charCodeAt(offset + 3) & 0xff) << 24 | 
                    (this.data.charCodeAt(offset + 2) & 0xff) << 16 | 
                    (this.data.charCodeAt(offset + 1) & 0xff) << 8 | 
                    (this.data.charCodeAt(offset + 0) & 0xff);
            break;
        case 2:
            return  (this.data.charCodeAt(offset + 1) & 0xff) << 8 | 
                    (this.data.charCodeAt(offset + 0) & 0xff);
            break;
        default:
            return this.data.charCodeAt(offset) & 0xff;
            break;
        }
    };

    this.getDOSDate = function(dosdate, dostime) {
        var day = dosdate & 0x1f;
        var month = ((dosdate >> 5) & 0xf) - 1;
        var year = 1980 + ((dosdate >> 9) & 0x7f)
        var second = (dostime & 0x1f) * 2;
        var minute = (dostime >> 5) & 0x3f;
        hour = (dostime >> 11) & 0x1f;
        return new Date(year, month, day, hour, minute, second);
    }

    this.open = function(data) {
        this.data = data;
        this.files = [];

        if (this.data.length < 22)
            return { 'status' : false, 'error' : 'Invalid data' };
        var endOfCentralDirectory = this.data.length - 22;
        while (endOfCentralDirectory >= 0 && this.getInt(endOfCentralDirectory, 4) != 0x06054b50)
            --endOfCentralDirectory;
        if (endOfCentralDirectory < 0)
            return { 'status' : false, 'error' : 'Invalid data' };
        if (this.getInt(endOfCentralDirectory + 4, 2) != 0 || this.getInt(endOfCentralDirectory + 6, 2) != 0)
            return { 'status' : false, 'error' : 'No multidisk support' };

        var entriesInThisDisk = this.getInt(endOfCentralDirectory + 8, 2);
        var centralDirectoryOffset = this.getInt(endOfCentralDirectory + 16, 4);
        var globalCommentLength = this.getInt(endOfCentralDirectory + 20, 2);
        this.comment = this.data.slice(endOfCentralDirectory + 22, endOfCentralDirectory + 22 + globalCommentLength);

        var fileOffset = centralDirectoryOffset;

        for (var i = 0; i < entriesInThisDisk; ++i) {
            if (this.getInt(fileOffset + 0, 4) != 0x02014b50)
                return { 'status' : false, 'error' : 'Invalid data' };
            if (this.getInt(fileOffset + 6, 2) > 20)
                return { 'status' : false, 'error' : 'Unsupported version' };
            if (this.getInt(fileOffset + 8, 2) & 1)
                return { 'status' : false, 'error' : 'Encryption not implemented' };

            var compressionMethod = this.getInt(fileOffset + 10, 2);
            if (compressionMethod != 0 && compressionMethod != 8)
                return { 'status' : false, 'error' : 'Unsupported compression method' };

            var lastModFileTime = this.getInt(fileOffset + 12, 2);
            var lastModFileDate = this.getInt(fileOffset + 14, 2);
            var lastModifiedDate = this.getDOSDate(lastModFileDate, lastModFileTime);

            var crc = this.getInt(fileOffset + 16, 4);
            // TODO: crc

            var compressedSize = this.getInt(fileOffset + 20, 4);
            var uncompressedSize = this.getInt(fileOffset + 24, 4);

            var fileNameLength = this.getInt(fileOffset + 28, 2);
            var extraFieldLength = this.getInt(fileOffset + 30, 2);
            var fileCommentLength = this.getInt(fileOffset + 32, 2);

            var relativeOffsetOfLocalHeader = this.getInt(fileOffset + 42, 4);

            var fileName = this.data.slice(fileOffset + 46, fileOffset + 46 + fileNameLength);
            var fileComment = this.data.slice(fileOffset + 46 + fileNameLength + extraFieldLength, fileOffset + 46 + fileNameLength + extraFieldLength + fileCommentLength);

            if (this.getInt(relativeOffsetOfLocalHeader + 0, 4) != 0x04034b50)
                return { 'status' : false, 'error' : 'Invalid data' };
            var localFileNameLength = this.getInt(relativeOffsetOfLocalHeader + 26, 2);
            var localExtraFieldLength = this.getInt(relativeOffsetOfLocalHeader + 28, 2);
            var localFileContent = relativeOffsetOfLocalHeader + 30 + localFileNameLength + localExtraFieldLength;

            this.files[fileName] = 
            {
                'fileComment' : fileComment,
                'compressionMethod' : compressionMethod,
                'compressedSize' : compressedSize,
                'uncompressedSize' : uncompressedSize,
                'localFileContent' : localFileContent,
                'lastModifiedDate' : lastModifiedDate
            };

            fileOffset += 46 + fileNameLength + extraFieldLength + fileCommentLength;
        }
        return { 'status' : true }
    };     
    

    this.read = function(fileName) {
        var fileInfo = this.files[fileName];
        if (fileInfo) {
            if (fileInfo.compressionMethod == 8) {
                if (!tinf) {
                    tinf = new TINF();
                    tinf.init();
                }
                var result = tinf.uncompress(this.data, fileInfo.localFileContent);
                if (result.status == tinf.OK)
                    return { 'status' : true, 'data' : result.data };
                else
                    return { 'status' : false, 'error' : result.error };
            } else {
                return { 'status' : true, 'data' : this.data.slice(fileInfo.localFileContent, fileInfo.localFileContent + fileInfo.uncompressedSize) };
            }
        }
        return { 'status' : false, 'error' : "File '" + fileName + "' doesn't exist in zip" };
    };
    
};



/*
 * tinflate  -  tiny inflate
 *
 * Copyright (c) 2003 by Joergen Ibsen / Jibz
 * All Rights Reserved
 *
 * http://www.ibsensoftware.com/
 *
 * This software is provided 'as-is', without any express
 * or implied warranty.  In no event will the authors be
 * held liable for any damages arising from the use of
 * this software.
 *
 * Permission is granted to anyone to use this software
 * for any purpose, including commercial applications,
 * and to alter it and redistribute it freely, subject to
 * the following restrictions:
 *
 * 1. The origin of this software must not be
 *    misrepresented; you must not claim that you
 *    wrote the original software. If you use this
 *    software in a product, an acknowledgment in
 *    the product documentation would be appreciated
 *    but is not required.
 *
 * 2. Altered source versions must be plainly marked
 *    as such, and must not be misrepresented as
 *    being the original software.
 *
 * 3. This notice may not be removed or altered from
 *    any source distribution.
 */

/*
 * tinflate javascript port by Erik Moller in May 2011.
 * emoller@opera.com
 * 
 * read_bits() patched by mike@imidio.com to allow
 * reading more then 8 bits (needed in some zlib streams)
 */



function TINF() {
    
this.OK = 0;
this.DATA_ERROR = (-3);
this.WINDOW_SIZE = 32768;

/* ------------------------------ *
 * -- internal data structures -- *
 * ------------------------------ */

this.TREE = function() {
   this.table = new Array(16);  /* table of code length counts */
   this.trans = new Array(288); /* code -> symbol translation table */
};

this.DATA = function(that) {
   this.source = '';
   this.sourceIndex = 0;
   this.tag = 0;
   this.bitcount = 0;

   this.dest = [];
   
   this.history = [];

   this.ltree = new that.TREE(); /* dynamic length/symbol tree */
   this.dtree = new that.TREE(); /* dynamic distance tree */
};

/* --------------------------------------------------- *
 * -- uninitialized global data (static structures) -- *
 * --------------------------------------------------- */

this.sltree = new this.TREE(); /* fixed length/symbol tree */
this.sdtree = new this.TREE(); /* fixed distance tree */

/* extra bits and base tables for length codes */
this.length_bits = new Array(30);
this.length_base = new Array(30);

/* extra bits and base tables for distance codes */
this.dist_bits = new Array(30);
this.dist_base = new Array(30);

/* special ordering of code length codes */
this.clcidx = [
   16, 17, 18, 0, 8, 7, 9, 6,
   10, 5, 11, 4, 12, 3, 13, 2,
   14, 1, 15
];

/* ----------------------- *
 * -- utility functions -- *
 * ----------------------- */

/* build extra bits and base tables */
this.build_bits_base = function(bits, base, delta, first)
{
   var i, sum;

   /* build bits table */
   for (i = 0; i < delta; ++i) bits[i] = 0;
   for (i = 0; i < 30 - delta; ++i) bits[i + delta] = Math.floor(i / delta);

   /* build base table */
   for (sum = first, i = 0; i < 30; ++i)
   {
      base[i] = sum;
      sum += 1 << bits[i];
   }
}

/* build the fixed huffman trees */
this.build_fixed_trees = function(lt, dt)
{
   var i;

   /* build fixed length tree */
   for (i = 0; i < 7; ++i) lt.table[i] = 0;

   lt.table[7] = 24;
   lt.table[8] = 152;
   lt.table[9] = 112;

   for (i = 0; i < 24; ++i) lt.trans[i] = 256 + i;
   for (i = 0; i < 144; ++i) lt.trans[24 + i] = i;
   for (i = 0; i < 8; ++i) lt.trans[24 + 144 + i] = 280 + i;
   for (i = 0; i < 112; ++i) lt.trans[24 + 144 + 8 + i] = 144 + i;

   /* build fixed distance tree */
   for (i = 0; i < 5; ++i) dt.table[i] = 0;

   dt.table[5] = 32;

   for (i = 0; i < 32; ++i) dt.trans[i] = i;
}

/* given an array of code lengths, build a tree */
this.build_tree = function(t, lengths, loffset, num)
{
   var offs = new Array(16);
   var i, sum;

   /* clear code length count table */
   for (i = 0; i < 16; ++i) t.table[i] = 0;

   /* scan symbol lengths, and sum code length counts */
   for (i = 0; i < num; ++i) t.table[lengths[loffset + i]]++;

   t.table[0] = 0;

   /* compute offset table for distribution sort */
   for (sum = 0, i = 0; i < 16; ++i)
   {
      offs[i] = sum;
      sum += t.table[i];
   }

   /* create code->symbol translation table (symbols sorted by code) */
   for (i = 0; i < num; ++i)
   {
      if (lengths[loffset + i]) t.trans[offs[lengths[loffset + i]]++] = i;
   }
}

/* ---------------------- *
 * -- decode functions -- *
 * ---------------------- */

/* get one bit from source stream */
this.getbit = function(d)
{
   var bit;

   /* check if tag is empty */
   if (!d.bitcount--)
   {
      /* load next tag */
      d.tag = d.source[d.sourceIndex++] & 0xff;
      d.bitcount = 7;
   }

   /* shift bit out of tag */
   bit = d.tag & 0x01;
   d.tag >>= 1;

   return bit;
}

/* read a num bit value from a stream and add base */
function read_bits_direct(source, bitcount, tag, idx, num)
{
    var val = 0;
    while (bitcount < 24) {
        tag = tag | (source[idx++] & 0xff) << bitcount;
        bitcount += 8;
    }
    val = tag & (0xffff >> (16 - num));
    tag >>= num;
    bitcount -= num;
    return [bitcount, tag, idx, val];
}
this.read_bits = function(d, num, base)
{
    if (!num)
        return base;

    var ret = read_bits_direct(d.source, d.bitcount, d.tag, d.sourceIndex, num);
    d.bitcount = ret[0];
    d.tag = ret[1];
    d.sourceIndex = ret[2];
    return ret[3] + base;
}

/* given a data stream and a tree, decode a symbol */
this.decode_symbol = function(d, t)
{
    while (d.bitcount < 16) {
        d.tag = d.tag | (d.source[d.sourceIndex++] & 0xff) << d.bitcount;
        d.bitcount += 8;
    }
    
    var sum = 0, cur = 0, len = 0;
    do {
        cur = 2 * cur + ((d.tag & (1 << len)) >> len);

        ++len;

        sum += t.table[len];
        cur -= t.table[len];

    } while (cur >= 0);

    d.tag >>= len;
    d.bitcount -= len;

    return t.trans[sum + cur];
}

/* given a data stream, decode dynamic trees from it */
this.decode_trees = function(d, lt, dt)
{
   var code_tree = new this.TREE();
   var lengths = new Array(288+32);
   var hlit, hdist, hclen;
   var i, num, length;

   /* get 5 bits HLIT (257-286) */
   hlit = this.read_bits(d, 5, 257);

   /* get 5 bits HDIST (1-32) */
   hdist = this.read_bits(d, 5, 1);

   /* get 4 bits HCLEN (4-19) */
   hclen = this.read_bits(d, 4, 4);

   for (i = 0; i < 19; ++i) lengths[i] = 0;

   /* read code lengths for code length alphabet */
   for (i = 0; i < hclen; ++i)
   {
      /* get 3 bits code length (0-7) */
      var clen = this.read_bits(d, 3, 0);

      lengths[this.clcidx[i]] = clen;
   }

   /* build code length tree */
   this.build_tree(code_tree, lengths, 0, 19);

   /* decode code lengths for the dynamic trees */
   for (num = 0; num < hlit + hdist; )
   {
      var sym = this.decode_symbol(d, code_tree);

      switch (sym)
      {
      case 16:
         /* copy previous code length 3-6 times (read 2 bits) */
         {
            var prev = lengths[num - 1];
            for (length = this.read_bits(d, 2, 3); length; --length)
            {
               lengths[num++] = prev;
            }
         }
         break;
      case 17:
         /* repeat code length 0 for 3-10 times (read 3 bits) */
         for (length = this.read_bits(d, 3, 3); length; --length)
         {
            lengths[num++] = 0;
         }
         break;
      case 18:
         /* repeat code length 0 for 11-138 times (read 7 bits) */
         for (length = this.read_bits(d, 7, 11); length; --length)
         {
            lengths[num++] = 0;
         }
         break;
      default:
         /* values 0-15 represent the actual code lengths */
         lengths[num++] = sym;
         break;
      }
   }

   /* build dynamic trees */
   this.build_tree(lt, lengths, 0, hlit);
   this.build_tree(dt, lengths, hlit, hdist);
}

/* ----------------------------- *
 * -- block inflate functions -- *
 * ----------------------------- */

/* given a stream and two trees, inflate a block of data */
this.inflate_block_data = function(d, lt, dt)
{
   // js optimization.
   var ddest = d.dest;
   var ddestlength = ddest.length;

   while (1)
   {
      var sym = this.decode_symbol(d, lt);

      /* check for end of block */
      if (sym == 256)
      {
         return this.OK;
      }

      if (sym < 256)
      {
         ddest[ddestlength++] = sym; // ? String.fromCharCode(sym);
         d.history.push(sym);
      } else {

         var length, dist, offs;
         var i;

         sym -= 257;

         /* possibly get more bits from length code */
         length = this.read_bits(d, this.length_bits[sym], this.length_base[sym]);

         dist = this.decode_symbol(d, dt);

         /* possibly get more bits from distance code */
         offs = d.history.length - this.read_bits(d, this.dist_bits[dist], this.dist_base[dist]);

         if (offs < 0)
             throw ("Invalid zlib offset " + offs);
         
         /* copy match */
         for (i = offs; i < offs + length; ++i) {
            //ddest[ddestlength++] = ddest[i];
            ddest[ddestlength++] = d.history[i];
            d.history.push(d.history[i]);
         }
      }
   }
}

/* inflate an uncompressed block of data */
this.inflate_uncompressed_block = function(d)
{
   var length, invlength;
   var i;

   if (d.bitcount > 7) {
       var overflow = Math.floor(d.bitcount / 8);
       d.sourceIndex -= overflow;
       d.bitcount = 0;
       d.tag = 0;
   }
   
   /* get length */
   length = d.source[d.sourceIndex+1];
   length = 256*length + d.source[d.sourceIndex];

   /* get one's complement of length */
   invlength = d.source[d.sourceIndex+3];
   invlength = 256*invlength + d.source[d.sourceIndex+2];

   /* check length */
   if (length != (~invlength & 0x0000ffff)) return this.DATA_ERROR;

   d.sourceIndex += 4;

   /* copy block */
   for (i = length; i; --i) {
       d.history.push(d.source[d.sourceIndex]);
       d.dest[d.dest.length] = d.source[d.sourceIndex++];
   }

   /* make sure we start next block on a byte boundary */
   d.bitcount = 0;

   return this.OK;
}

/* inflate a block of data compressed with fixed huffman trees */
this.inflate_fixed_block = function(d)
{
   /* decode block using fixed trees */
   return this.inflate_block_data(d, this.sltree, this.sdtree);
}

/* inflate a block of data compressed with dynamic huffman trees */
this.inflate_dynamic_block = function(d)
{
   /* decode trees from stream */
   this.decode_trees(d, d.ltree, d.dtree);

   /* decode block using decoded trees */
   return this.inflate_block_data(d, d.ltree, d.dtree);
}

/* ---------------------- *
 * -- public functions -- *
 * ---------------------- */

/* initialize global (static) data */
this.init = function()
{
   /* build fixed huffman trees */
   this.build_fixed_trees(this.sltree, this.sdtree);

   /* build extra bits and base tables */
   this.build_bits_base(this.length_bits, this.length_base, 4, 3);
   this.build_bits_base(this.dist_bits, this.dist_base, 2, 1);

   /* fix a special case */
   this.length_bits[28] = 0;
   this.length_base[28] = 258;

   this.reset();   
}

this.reset = function()
{
   this.d = new this.DATA(this);
   delete this.header;
}

/* inflate stream from source to dest */
this.uncompress = function(source, offset)
{

   var d = this.d;
   var bfinal;

   /* initialise data */
   d.source = source;
   d.sourceIndex = offset;
   d.bitcount = 0;

   d.dest = [];

   // Skip zlib header at start of stream
   if (typeof this.header == 'undefined') {
       this.header = this.read_bits(d, 16, 0);
       /* byte 0: 0x78, 7 = 32k window size, 8 = deflate */
       /* byte 1: check bits for header and other flags */
   }

   var blocks = 0;
   
   do {

      var btype;
      var res;

      /* read final block flag */
      bfinal = this.getbit(d);

      /* read block type (2 bits) */
      btype = this.read_bits(d, 2, 0);

      /* decompress block */
      switch (btype)
      {
      case 0:
         /* decompress uncompressed block */
         res = this.inflate_uncompressed_block(d);
         break;
      case 1:
         /* decompress block with fixed huffman trees */
         res = this.inflate_fixed_block(d);
         break;
      case 2:
         /* decompress block with dynamic huffman trees */
         res = this.inflate_dynamic_block(d);
         break;
      default:
         return { 'status' : this.DATA_ERROR };
      }

      if (res != this.OK) return { 'status' : this.DATA_ERROR };
      blocks++;
      
   } while (!bfinal && d.sourceIndex < d.source.length);

   d.history = d.history.slice(-this.WINDOW_SIZE);
   
   return { 'status' : this.OK, 'data' : d.dest };
}

};

define("vnc-jsunzip", ["vnc-util"], function(){});

/*
 * noVNC: HTML5 VNC client
 * Copyright (C) 2012 Joel Martin
 * Copyright (C) 2013 Samuel Mannehed for Cendio AB
 * Licensed under MPL 2.0 (see LICENSE.txt)
 *
 * See README.md for usage and integration instructions.
 *
 * TIGHT decoder portion:
 * (c) 2012 Michael Tinglof, Joe Balaz, Les Piech (Mercuri.ca)
 */

/*jslint white: false, browser: true */
/*global window, Util, Display, Keyboard, Mouse, Websock, Websock_native, Base64, DES */

var RFB;

(function () {
    "use strict";
    RFB = function (defaults) {
        if (!defaults) {
            defaults = {};
        }

        this._rfb_host = '';
        this._rfb_port = 5900;
        this._rfb_password = '';
        this._rfb_path = '';

        this._rfb_state = 'disconnected';
        this._rfb_version = 0;
        this._rfb_max_version = 3.8;
        this._rfb_auth_scheme = '';

        this._rfb_tightvnc = false;
        this._rfb_xvp_ver = 0;

        // In preference order
        this._encodings = [
            ['COPYRECT',         0x01 ],
            ['TIGHT',            0x07 ],
            ['TIGHT_PNG',        -260 ],
            ['HEXTILE',          0x05 ],
            ['RRE',              0x02 ],
            ['RAW',              0x00 ],
            ['DesktopSize',      -223 ],
            ['Cursor',           -239 ],

            // Psuedo-encoding settings
            //['JPEG_quality_lo',   -32 ],
            ['JPEG_quality_med',    -26 ],
            //['JPEG_quality_hi',   -23 ],
            //['compress_lo',      -255 ],
            ['compress_hi',        -247 ],
            ['last_rect',          -224 ],
            ['xvp',                -309 ]
        ];

        this._encHandlers = {};
        this._encNames = {};
        this._encStats = {};

        this._sock = null;              // Websock object
        this._display = null;           // Display object
        this._keyboard = null;          // Keyboard input handler object
        this._mouse = null;             // Mouse input handler object
        this._sendTimer = null;         // Send Queue check timer
        this._disconnTimer = null;      // disconnection timer
        this._msgTimer = null;          // queued handle_msg timer

        // Frame buffer update state
        this._FBU = {
            rects: 0,
            subrects: 0,            // RRE
            lines: 0,               // RAW
            tiles: 0,               // HEXTILE
            bytes: 0,
            x: 0,
            y: 0,
            width: 0,
            height: 0,
            encoding: 0,
            subencoding: -1,
            background: null,
            zlib: []                // TIGHT zlib streams
        };

        this._fb_Bpp = 4;
        this._fb_depth = 3;
        this._fb_width = 0;
        this._fb_height = 0;
        this._fb_name = "";

        this._rre_chunk_sz = 100;

        this._timing = {
            last_fbu: 0,
            fbu_total: 0,
            fbu_total_cnt: 0,
            full_fbu_total: 0,
            full_fbu_cnt: 0,

            fbu_rt_start: 0,
            fbu_rt_total: 0,
            fbu_rt_cnt: 0,
            pixels: 0
        };

        // Mouse state
        this._mouse_buttonMask = 0;
        this._mouse_arr = [];
        this._viewportDragging = false;
        this._viewportDragPos = {};

        // set the default value on user-facing properties
        Util.set_defaults(this, defaults, {
            'target': 'null',                       // VNC display rendering Canvas object
            'focusContainer': document,             // DOM element that captures keyboard input
            'encrypt': false,                       // Use TLS/SSL/wss encryption
            'true_color': true,                     // Request true color pixel data
            'local_cursor': false,                  // Request locally rendered cursor
            'shared': true,                         // Request shared mode
            'view_only': false,                     // Disable client mouse/keyboard
            'xvp_password_sep': '@',                // Separator for XVP password fields
            'disconnectTimeout': 3,                 // Time (s) to wait for disconnection
            'wsProtocols': ['binary', 'base64'],    // Protocols to use in the WebSocket connection
            'repeaterID': '',                       // [UltraVNC] RepeaterID to connect to
            'viewportDrag': false,                  // Move the viewport on mouse drags

            // Callback functions
            'onUpdateState': function () { },       // onUpdateState(rfb, state, oldstate, statusMsg): state update/change
            'onPasswordRequired': function () { },  // onPasswordRequired(rfb): VNC password is required
            'onClipboard': function () { },         // onClipboard(rfb, text): RFB clipboard contents received
            'onBell': function () { },              // onBell(rfb): RFB Bell message received
            'onFBUReceive': function () { },        // onFBUReceive(rfb, fbu): RFB FBU received but not yet processed
            'onFBUComplete': function () { },       // onFBUComplete(rfb, fbu): RFB FBU received and processed
            'onFBResize': function () { },          // onFBResize(rfb, width, height): frame buffer resized
            'onDesktopName': function () { },       // onDesktopName(rfb, name): desktop name received
            'onXvpInit': function () { },           // onXvpInit(version): XVP extensions active for this connection
        });

        // main setup
        Util.Debug(">> RFB.constructor");

        // populate encHandlers with bound versions
        Object.keys(RFB.encodingHandlers).forEach(function (encName) {
            this._encHandlers[encName] = RFB.encodingHandlers[encName].bind(this);
        }.bind(this));

        // Create lookup tables based on encoding number
        for (var i = 0; i < this._encodings.length; i++) {
            this._encHandlers[this._encodings[i][1]] = this._encHandlers[this._encodings[i][0]];
            this._encNames[this._encodings[i][1]] = this._encodings[i][0];
            this._encStats[this._encodings[i][1]] = [0, 0];
        }

        try {
            this._display = new Display({target: this._target});
        } catch (exc) {
            Util.Error("Display exception: " + exc);
            this._updateState('fatal', "No working Display");
        }

        this._keyboard = new Keyboard({target: this._focusContainer,
                                       onKeyPress: this._handleKeyPress.bind(this)});

        this._mouse = new Mouse({target: this._target,
                                 onMouseButton: this._handleMouseButton.bind(this),
                                 onMouseMove: this._handleMouseMove.bind(this),
                                 notify: this._keyboard.sync.bind(this._keyboard)});

        this._sock = new Websock();
        this._sock.on('message', this._handle_message.bind(this));
        this._sock.on('open', function () {
            if (this._rfb_state === 'connect') {
                this._updateState('ProtocolVersion', "Starting VNC handshake");
            } else {
                this._fail("Got unexpected WebSocket connection");
            }
        }.bind(this));
        this._sock.on('close', function (e) {
            Util.Warn("WebSocket on-close event");
            var msg = "";
            if (e.code) {
                msg = " (code: " + e.code;
                if (e.reason) {
                    msg += ", reason: " + e.reason;
                }
                msg += ")";
            }
            if (this._rfb_state === 'disconnect') {
                this._updateState('disconnected', 'VNC disconnected' + msg);
            } else if (this._rfb_state === 'ProtocolVersion') {
                this._fail('Failed to connect to server' + msg);
            } else if (this._rfb_state in {'failed': 1, 'disconnected': 1}) {
                Util.Error("Received onclose while disconnected" + msg);
            } else {
                this._fail("Server disconnected" + msg);
            }
        }.bind(this));
        this._sock.on('error', function (e) {
            Util.Warn("WebSocket on-error event");
        });

        this._init_vars();

        var rmode = this._display.get_render_mode();
        if (Websock_native) {
            Util.Info("Using native WebSockets");
            this._updateState('loaded', 'noVNC ready: native WebSockets, ' + rmode);
        } else {
            Util.Warn("Using web-socket-js bridge.  Flash version: " + Util.Flash.version);
            if (!Util.Flash || Util.Flash.version < 9) {
                this._updateState('fatal', "WebSockets or <a href='http://get.adobe.com/flashplayer'>Adobe Flash</a> is required");
            } else if (document.location.href.substr(0, 7) === 'file://') {
                this._updateState('fatal', "'file://' URL is incompatible with Adobe Flash");
            } else {
                this._updateState('loaded', 'noVNC ready: WebSockets emulation, ' + rmode);
            }
        }

        Util.Debug("<< RFB.constructor");
    };

    RFB.prototype = {
        // Public methods
        connect: function (host, port, password, path) {
            this._rfb_host = host;
            this._rfb_port = port;
            this._rfb_password = (password !== undefined) ? password : "";
            this._rfb_path = (path !== undefined) ? path : "";

            if (!this._rfb_host || !this._rfb_port) {
                return this._fail("Must set host and port");
            }

            this._updateState('connect');
        },

        disconnect: function () {
            this._updateState('disconnect', 'Disconnecting');
        },

        sendPassword: function (passwd) {
            this._rfb_password = passwd;
            this._rfb_state = 'Authentication';
            setTimeout(this._init_msg.bind(this), 1);
        },

        sendCtrlAltDel: function () {
            if (this._rfb_state !== 'normal' || this._view_only) { return false; }
            Util.Info("Sending Ctrl-Alt-Del");

            var arr = [];
            arr = arr.concat(RFB.messages.keyEvent(XK_Control_L, 1));
            arr = arr.concat(RFB.messages.keyEvent(XK_Alt_L, 1));
            arr = arr.concat(RFB.messages.keyEvent(XK_Delete, 1));
            arr = arr.concat(RFB.messages.keyEvent(XK_Delete, 0));
            arr = arr.concat(RFB.messages.keyEvent(XK_Alt_L, 0));
            arr = arr.concat(RFB.messages.keyEvent(XK_Control_L, 0));
            this._sock.send(arr);
        },

        xvpOp: function (ver, op) {
            if (this._rfb_xvp_ver < ver) { return false; }
            Util.Info("Sending XVP operation " + op + " (version " + ver + ")");
            this._sock.send_string("\xFA\x00" + String.fromCharCode(ver) + String.fromCharCode(op));
            return true;
        },

        xvpShutdown: function () {
            return this.xvpOp(1, 2);
        },

        xvpReboot: function () {
            return this.xvpOp(1, 3);
        },

        xvpReset: function () {
            return this.xvpOp(1, 4);
        },

        // Send a key press. If 'down' is not specified then send a down key
        // followed by an up key.
        sendKey: function (code, down) {
            if (this._rfb_state !== "normal" || this._view_only) { return false; }
            var arr = [];
            if (typeof down !== 'undefined') {
                Util.Info("Sending key code (" + (down ? "down" : "up") + "): " + code);
                arr = arr.concat(RFB.messages.keyEvent(code, down ? 1 : 0));
            } else {
                Util.Info("Sending key code (down + up): " + code);
                arr = arr.concat(RFB.messages.keyEvent(code, 1));
                arr = arr.concat(RFB.messages.keyEvent(code, 0));
            }
            this._sock.send(arr);
        },

        clipboardPasteFrom: function (text) {
            if (this._rfb_state !== 'normal') { return; }
            this._sock.send(RFB.messages.clientCutText(text));
        },

        // Private methods

        _connect: function () {
            Util.Debug(">> RFB.connect");

            var uri;
            if (typeof UsingSocketIO !== 'undefined') {
                uri = 'http';
            } else {
                uri = this._encrypt ? 'wss' : 'ws';
            }

            uri += '://' + this._rfb_host + ':' + this._rfb_port + '/' + this._rfb_path;
            Util.Info("connecting to " + uri);

            this._sock.open(uri, this._wsProtocols);

            Util.Debug("<< RFB.connect");
        },

        _init_vars: function () {
            // reset state
            this._sock.init();

            this._FBU.rects        = 0;
            this._FBU.subrects     = 0;  // RRE and HEXTILE
            this._FBU.lines        = 0;  // RAW
            this._FBU.tiles        = 0;  // HEXTILE
            this._FBU.zlibs        = []; // TIGHT zlib encoders
            this._mouse_buttonMask = 0;
            this._mouse_arr        = [];
            this._rfb_tightvnc     = false;

            // Clear the per connection encoding stats
            var i;
            for (i = 0; i < this._encodings.length; i++) {
                this._encStats[this._encodings[i][1]][0] = 0;
            }

            for (i = 0; i < 4; i++) {
                this._FBU.zlibs[i] = new TINF();
                this._FBU.zlibs[i].init();
            }
        },

        _print_stats: function () {
            Util.Info("Encoding stats for this connection:");
            var i, s;
            for (i = 0; i < this._encodings.length; i++) {
                s = this._encStats[this._encodings[i][1]];
                if (s[0] + s[1] > 0) {
                    Util.Info("    " + this._encodings[i][0] + ": " + s[0] + " rects");
                }
            }

            Util.Info("Encoding stats since page load:");
            for (i = 0; i < this._encodings.length; i++) {
                s = this._encStats[this._encodings[i][1]];
                Util.Info("    " + this._encodings[i][0] + ": " + s[1] + " rects");
            }
        },


        /*
         * Page states:
         *   loaded       - page load, equivalent to disconnected
         *   disconnected - idle state
         *   connect      - starting to connect (to ProtocolVersion)
         *   normal       - connected
         *   disconnect   - starting to disconnect
         *   failed       - abnormal disconnect
         *   fatal        - failed to load page, or fatal error
         *
         * RFB protocol initialization states:
         *   ProtocolVersion
         *   Security
         *   Authentication
         *   password     - waiting for password, not part of RFB
         *   SecurityResult
         *   ClientInitialization - not triggered by server message
         *   ServerInitialization (to normal)
         */
        _updateState: function (state, statusMsg) {
            var oldstate = this._rfb_state;

            if (state === oldstate) {
                // Already here, ignore
                Util.Debug("Already in state '" + state + "', ignoring");
            }

            /*
             * These are disconnected states. A previous connect may
             * asynchronously cause a connection so make sure we are closed.
             */
            if (state in {'disconnected': 1, 'loaded': 1, 'connect': 1,
                          'disconnect': 1, 'failed': 1, 'fatal': 1}) {

                if (this._sendTimer) {
                    clearInterval(this._sendTimer);
                    this._sendTimer = null;
                }

                if (this._msgTimer) {
                    clearInterval(this._msgTimer);
                    this._msgTimer = null;
                }

                if (this._display && this._display.get_context()) {
                    this._keyboard.ungrab();
                    this._mouse.ungrab();
                    this._display.defaultCursor();
                    if (Util.get_logging() !== 'debug' || state === 'loaded') {
                        // Show noVNC logo on load and when disconnected, unless in
                        // debug mode
                        this._display.clear();
                    }
                }

                this._sock.close();
            }

            if (oldstate === 'fatal') {
                Util.Error('Fatal error, cannot continue');
            }

            var cmsg = typeof(statusMsg) !== 'undefined' ? (" Msg: " + statusMsg) : "";
            var fullmsg = "New state '" + state + "', was '" + oldstate + "'." + cmsg;
            if (state === 'failed' || state === 'fatal') {
                Util.Error(cmsg);
            } else {
                Util.Warn(cmsg);
            }

            if (oldstate === 'failed' && state === 'disconnected') {
                // do disconnect action, but stay in failed state
                this._rfb_state = 'failed';
            } else {
                this._rfb_state = state;
            }

            if (this._disconnTimer && this._rfb_state !== 'disconnect') {
                Util.Debug("Clearing disconnect timer");
                clearTimeout(this._disconnTimer);
                this._disconnTimer = null;
            }

            switch (state) {
                case 'normal':
                    if (oldstate === 'disconnected' || oldstate === 'failed') {
                        Util.Error("Invalid transition from 'disconnected' or 'failed' to 'normal'");
                    }
                    break;

                case 'connect':
                    this._init_vars();
                    this._connect();
                    // WebSocket.onopen transitions to 'ProtocolVersion'
                    break;

                case 'disconnect':
                    this._disconnTimer = setTimeout(function () {
                        this._fail("Disconnect timeout");
                    }.bind(this), this._disconnectTimeout * 1000);

                    this._print_stats();

                    // WebSocket.onclose transitions to 'disconnected'
                    break;

                case 'failed':
                    if (oldstate === 'disconnected') {
                        Util.Error("Invalid transition from 'disconnected' to 'failed'");
                    } else if (oldstate === 'normal') {
                        Util.Error("Error while connected.");
                    } else if (oldstate === 'init') {
                        Util.Error("Error while initializing.");
                    }

                    // Make sure we transition to disconnected
                    setTimeout(function () {
                        this._updateState('disconnected');
                    }.bind(this), 50);

                    break;

                default:
                    // No state change action to take
            }

            if (oldstate === 'failed' && state === 'disconnected') {
                this._onUpdateState(this, state, oldstate);
            } else {
                this._onUpdateState(this, state, oldstate, statusMsg);
            }
        },

        _fail: function (msg) {
            this._updateState('failed', msg);
            return false;
        },

        _handle_message: function () {
            if (this._sock.rQlen() === 0) {
                Util.Warn("handle_message called on an empty receive queue");
                return;
            }

            switch (this._rfb_state) {
                case 'disconnected':
                case 'failed':
                    Util.Error("Got data while disconnected");
                    break;
                case 'normal':
                    if (this._normal_msg() && this._sock.rQlen() > 0) {
                        // true means we can continue processing
                        // Give other events a chance to run
                        if (this._msgTimer === null) {
                            Util.Debug("More data to process, creating timer");
                            this._msgTimer = setTimeout(function () {
                                this._msgTimer = null;
                                this._handle_message();
                            }.bind(this), 10);
                        } else {
                            Util.Debug("More data to process, existing timer");
                        }
                    }
                    break;
                default:
                    this._init_msg();
                    break;
            }
        },

        _checkEvents: function () {
            if (this._rfb_state === 'normal' && !this._viewportDragging && this._mouse_arr.length > 0) {
                this._sock.send(this._mouse_arr);
                this._mouse_arr = [];
            }
        },

        _handleKeyPress: function (keysym, down) {
            if (this._view_only) { return; } // View only, skip keyboard, events
            this._sock.send(RFB.messages.keyEvent(keysym, down));
        },

        _handleMouseButton: function (x, y, down, bmask) {
            if (down) {
                this._mouse_buttonMask |= bmask;
            } else {
                this._mouse_buttonMask ^= bmask;
            }

            if (this._viewportDrag) {
                if (down && !this._viewportDragging) {
                    this._viewportDragging = true;
                    this._viewportDragPos = {'x': x, 'y': y};

                    // Skip sending mouse events
                    return;
                } else {
                    this._viewportDragging = false;
                }
            }

            if (this._view_only) { return; } // View only, skip mouse events

            this._mouse_arr = this._mouse_arr.concat(
                    RFB.messages.pointerEvent(this._display.absX(x), this._display.absY(y), this._mouse_buttonMask));
            this._sock.send(this._mouse_arr);
            this._mouse_arr = [];
        },

        _handleMouseMove: function (x, y) {
            if (this._viewportDragging) {
                var deltaX = this._viewportDragPos.x - x;
                var deltaY = this._viewportDragPos.y - y;
                this._viewportDragPos = {'x': x, 'y': y};

                this._display.viewportChange(deltaX, deltaY);

                // Skip sending mouse events
                return;
            }

            if (this._view_only) { return; } // View only, skip mouse events

            this._mouse_arr = this._mouse_arr.concat(
                    RFB.messages.pointerEvent(this._display.absX(x), this._display.absY(y), this._mouse_buttonMask));

            this._checkEvents();
        },

        // Message Handlers

        _negotiate_protocol_version: function () {
            if (this._sock.rQlen() < 12) {
                return this._fail("Incomplete protocol version");
            }

            var sversion = this._sock.rQshiftStr(12).substr(4, 7);
            Util.Info("Server ProtocolVersion: " + sversion);
            var is_repeater = 0;
            switch (sversion) {
                case "000.000":  // UltraVNC repeater
                    is_repeater = 1;
                    break;
                case "003.003":
                case "003.006":  // UltraVNC
                case "003.889":  // Apple Remote Desktop
                    this._rfb_version = 3.3;
                    break;
                case "003.007":
                    this._rfb_version = 3.7;
                    break;
                case "003.008":
                case "004.000":  // Intel AMT KVM
                case "004.001":  // RealVNC 4.6
                    this._rfb_version = 3.8;
                    break;
                default:
                    return this._fail("Invalid server version " + sversion);
            }

            if (is_repeater) {
                var repeaterID = this._repeaterID;
                while (repeaterID.length < 250) {
                    repeaterID += "\0";
                }
                this._sock.send_string(repeaterID);
                return true;
            }

            if (this._rfb_version > this._rfb_max_version) {
                this._rfb_version = this._rfb_max_version;
            }

            // Send updates either at a rate of 1 update per 50ms, or
            // whatever slower rate the network can handle
            this._sendTimer = setInterval(this._sock.flush.bind(this._sock), 50);

            var cversion = "00" + parseInt(this._rfb_version, 10) +
                           ".00" + ((this._rfb_version * 10) % 10);
            this._sock.send_string("RFB " + cversion + "\n");
            this._updateState('Security', 'Sent ProtocolVersion: ' + cversion);
        },

        _negotiate_security: function () {
            if (this._rfb_version >= 3.7) {
                // Server sends supported list, client decides
                var num_types = this._sock.rQshift8();
                if (this._sock.rQwait("security type", num_types, 1)) { return false; }

                if (num_types === 0) {
                    var strlen = this._sock.rQshift32();
                    var reason = this._sock.rQshiftStr(strlen);
                    return this._fail("Security failure: " + reason);
                }

                this._rfb_auth_scheme = 0;
                var types = this._sock.rQshiftBytes(num_types);
                Util.Debug("Server security types: " + types);
                for (var i = 0; i < types.length; i++) {
                    if (types[i] > this._rfb_auth_scheme && (types[i] <= 16 || types[i] == 22)) {
                        this._rfb_auth_scheme = types[i];
                    }
                }

                if (this._rfb_auth_scheme === 0) {
                    return this._fail("Unsupported security types: " + types);
                }

                this._sock.send([this._rfb_auth_scheme]);
            } else {
                // Server decides
                if (this._sock.rQwait("security scheme", 4)) { return false; }
                this._rfb_auth_scheme = this._sock.rQshift32();
            }

            this._updateState('Authentication', 'Authenticating using scheme: ' + this._rfb_auth_scheme);
            return this._init_msg(); // jump to authentication
        },

        // authentication
        _negotiate_xvp_auth: function () {
            var xvp_sep = this._xvp_password_sep;
            var xvp_auth = this._rfb_password.split(xvp_sep);
            if (xvp_auth.length < 3) {
                this._updateState('password', 'XVP credentials required (user' + xvp_sep +
                                  'target' + xvp_sep + 'password) -- got only ' + this._rfb_password);
                this._onPasswordRequired(this);
                return false;
            }

            var xvp_auth_str = String.fromCharCode(xvp_auth[0].length) +
                               String.fromCharCode(xvp_auth[1].length) +
                               xvp_auth[0] +
                               xvp_auth[1];
            this._sock.send_string(xvp_auth_str);
            this._rfb_password = xvp_auth.slice(2).join(xvp_sep);
            this._rfb_auth_scheme = 2;
            return this._negotiate_authentication();
        },

        _negotiate_std_vnc_auth: function () {
            if (this._rfb_password.length === 0) {
                // Notify via both callbacks since it's kind of
                // an RFB state change and a UI interface issue
                this._updateState('password', "Password Required");
                this._onPasswordRequired(this);
            }

            if (this._sock.rQwait("auth challenge", 16)) { return false; }

            var challenge = this._sock.rQshiftBytes(16);
            var response = RFB.genDES(this._rfb_password, challenge);
            this._sock.send(response);
            this._updateState("SecurityResult");
            return true;
        },

        _negotiate_tight_tunnels: function (numTunnels) {
            var clientSupportedTunnelTypes = {
                0: { vendor: 'TGHT', signature: 'NOTUNNEL' }
            };
            var serverSupportedTunnelTypes = {};
            // receive tunnel capabilities
            for (var i = 0; i < numTunnels; i++) {
                var cap_code = this._sock.rQshift32();
                var cap_vendor = this._sock.rQshiftStr(4);
                var cap_signature = this._sock.rQshiftStr(8);
                serverSupportedTunnelTypes[cap_code] = { vendor: cap_vendor, signature: cap_signature };
            }

            // choose the notunnel type
            if (serverSupportedTunnelTypes[0]) {
                if (serverSupportedTunnelTypes[0].vendor != clientSupportedTunnelTypes[0].vendor ||
                    serverSupportedTunnelTypes[0].signature != clientSupportedTunnelTypes[0].signature) {
                    return this._fail("Client's tunnel type had the incorrect vendor or signature");
                }
                this._sock.send([0, 0, 0, 0]);  // use NOTUNNEL
                return false; // wait until we receive the sub auth count to continue
            } else {
                return this._fail("Server wanted tunnels, but doesn't support the notunnel type");
            }
        },

        _negotiate_tight_auth: function () {
            if (!this._rfb_tightvnc) {  // first pass, do the tunnel negotiation
                if (this._sock.rQwait("num tunnels", 4)) { return false; }
                var numTunnels = this._sock.rQshift32();
                if (numTunnels > 0 && this._sock.rQwait("tunnel capabilities", 16 * numTunnels, 4)) { return false; }

                this._rfb_tightvnc = true;

                if (numTunnels > 0) {
                    this._negotiate_tight_tunnels(numTunnels);
                    return false;  // wait until we receive the sub auth to continue
                }
            }

            // second pass, do the sub-auth negotiation
            if (this._sock.rQwait("sub auth count", 4)) { return false; }
            var subAuthCount = this._sock.rQshift32();
            if (this._sock.rQwait("sub auth capabilities", 16 * subAuthCount, 4)) { return false; }

            var clientSupportedTypes = {
                'STDVNOAUTH__': 1,
                'STDVVNCAUTH_': 2
            };

            var serverSupportedTypes = [];

            for (var i = 0; i < subAuthCount; i++) {
                var capNum = this._sock.rQshift32();
                var capabilities = this._sock.rQshiftStr(12);
                serverSupportedTypes.push(capabilities);
            }

            for (var authType in clientSupportedTypes) {
                if (serverSupportedTypes.indexOf(authType) != -1) {
                    this._sock.send([0, 0, 0, clientSupportedTypes[authType]]);

                    switch (authType) {
                        case 'STDVNOAUTH__':  // no auth
                            this._updateState('SecurityResult');
                            return true;
                        case 'STDVVNCAUTH_': // VNC auth
                            this._rfb_auth_scheme = 2;
                            return this._init_msg();
                        default:
                            return this._fail("Unsupported tiny auth scheme: " + authType);
                    }
                }
            }

            this._fail("No supported sub-auth types!");
        },

        _negotiate_authentication: function () {
            switch (this._rfb_auth_scheme) {
                case 0:  // connection failed
                    if (this._sock.rQwait("auth reason", 4)) { return false; }
                    var strlen = this._sock.rQshift32();
                    var reason = this._sock.rQshiftStr(strlen);
                    return this._fail("Auth failure: " + reason);

                case 1:  // no auth
                    if (this._rfb_version >= 3.8) {
                        this._updateState('SecurityResult');
                        return true;
                    }
                    this._updateState('ClientInitialisation', "No auth required");
                    return this._init_msg();

                case 22:  // XVP auth
                    return this._negotiate_xvp_auth();

                case 2:  // VNC authentication
                    return this._negotiate_std_vnc_auth();

                case 16:  // TightVNC Security Type
                    return this._negotiate_tight_auth();

                default:
                    return this._fail("Unsupported auth scheme: " + this._rfb_auth_scheme);
            }
        },

        _handle_security_result: function () {
            if (this._sock.rQwait('VNC auth response ', 4)) { return false; }
            switch (this._sock.rQshift32()) {
                case 0:  // OK
                    this._updateState('ClientInitialisation', 'Authentication OK');
                    return this._init_msg();
                case 1:  // failed
                    if (this._rfb_version >= 3.8) {
                        var length = this._sock.rQshift32();
                        if (this._sock.rQwait("SecurityResult reason", length, 8)) { return false; }
                        var reason = this._sock.rQshiftStr(length);
                        return this._fail(reason);
                    } else {
                        return this._fail("Authentication failure");
                    }
                    return false;
                case 2:
                    return this._fail("Too many auth attempts");
            }
        },

        _negotiate_server_init: function () {
            if (this._sock.rQwait("server initialization", 24)) { return false; }

            /* Screen size */
            this._fb_width  = this._sock.rQshift16();
            this._fb_height = this._sock.rQshift16();

            /* PIXEL_FORMAT */
            var bpp         = this._sock.rQshift8();
            var depth       = this._sock.rQshift8();
            var big_endian  = this._sock.rQshift8();
            var true_color  = this._sock.rQshift8();

            var red_max     = this._sock.rQshift16();
            var green_max   = this._sock.rQshift16();
            var blue_max    = this._sock.rQshift16();
            var red_shift   = this._sock.rQshift8();
            var green_shift = this._sock.rQshift8();
            var blue_shift  = this._sock.rQshift8();
            this._sock.rQskipBytes(3);  // padding

            // NB(directxman12): we don't want to call any callbacks or print messages until
            //                   *after* we're past the point where we could backtrack

            /* Connection name/title */
            var name_length = this._sock.rQshift32();
            if (this._sock.rQwait('server init name', name_length, 24)) { return false; }
            this._fb_name = Util.decodeUTF8(this._sock.rQshiftStr(name_length));

            if (this._rfb_tightvnc) {
                if (this._sock.rQwait('TightVNC extended server init header', 8, 24 + name_length)) { return false; }
                // In TightVNC mode, ServerInit message is extended
                var numServerMessages = this._sock.rQshift16();
                var numClientMessages = this._sock.rQshift16();
                var numEncodings = this._sock.rQshift16();
                this._sock.rQskipBytes(2);  // padding

                var totalMessagesLength = (numServerMessages + numClientMessages + numEncodings) * 16;
                if (this._sock.rQwait('TightVNC extended server init header', totalMessagesLength, 32 + name_length)) { return false; }

                var i;
                for (i = 0; i < numServerMessages; i++) {
                    var srvMsg = this._sock.rQshiftStr(16);
                }

                for (i = 0; i < numClientMessages; i++) {
                    var clientMsg = this._sock.rQshiftStr(16);
                }

                for (i = 0; i < numEncodings; i++) {
                    var encoding = this._sock.rQshiftStr(16);
                }
            }

            // NB(directxman12): these are down here so that we don't run them multiple times
            //                   if we backtrack
            Util.Info("Screen: " + this._fb_width + "x" + this._fb_height +
                      ", bpp: " + bpp + ", depth: " + depth +
                      ", big_endian: " + big_endian +
                      ", true_color: " + true_color +
                      ", red_max: " + red_max +
                      ", green_max: " + green_max +
                      ", blue_max: " + blue_max +
                      ", red_shift: " + red_shift +
                      ", green_shift: " + green_shift +
                      ", blue_shift: " + blue_shift);

            if (big_endian !== 0) {
                Util.Warn("Server native endian is not little endian");
            }

            if (red_shift !== 16) {
                Util.Warn("Server native red-shift is not 16");
            }

            if (blue_shift !== 0) {
                Util.Warn("Server native blue-shift is not 0");
            }

            // we're past the point where we could backtrack, so it's safe to call this
            this._onDesktopName(this, this._fb_name);

            if (this._true_color && this._fb_name === "Intel(r) AMT KVM") {
                Util.Warn("Intel AMT KVM only supports 8/16 bit depths.  Disabling true color");
                this._true_color = false;
            }

            this._display.set_true_color(this._true_color);
            this._onFBResize(this, this._fb_width, this._fb_height);
            this._display.resize(this._fb_width, this._fb_height);
            this._keyboard.grab();
            this._mouse.grab();

            if (this._true_color) {
                this._fb_Bpp = 4;
                this._fb_depth = 3;
            } else {
                this._fb_Bpp = 1;
                this._fb_depth = 1;
            }

            var response = RFB.messages.pixelFormat(this._fb_Bpp, this._fb_depth, this._true_color);
            response = response.concat(
                            RFB.messages.clientEncodings(this._encodings, this._local_cursor, this._true_color));
            response = response.concat(
                            RFB.messages.fbUpdateRequests(this._display.getCleanDirtyReset(),
                                                          this._fb_width, this._fb_height));

            this._timing.fbu_rt_start = (new Date()).getTime();
            this._timing.pixels = 0;
            this._sock.send(response);

            this._checkEvents();

            if (this._encrypt) {
                this._updateState('normal', 'Connected (encrypted) to: ' + this._fb_name);
            } else {
                this._updateState('normal', 'Connected (unencrypted) to: ' + this._fb_name);
            }
        },

        _init_msg: function () {
            switch (this._rfb_state) {
                case 'ProtocolVersion':
                    return this._negotiate_protocol_version();

                case 'Security':
                    return this._negotiate_security();

                case 'Authentication':
                    return this._negotiate_authentication();

                case 'SecurityResult':
                    return this._handle_security_result();

                case 'ClientInitialisation':
                    this._sock.send([this._shared ? 1 : 0]); // ClientInitialisation
                    this._updateState('ServerInitialisation', "Authentication OK");
                    return true;

                case 'ServerInitialisation':
                    return this._negotiate_server_init();
            }
        },

        _handle_set_colour_map_msg: function () {
            Util.Debug("SetColorMapEntries");
            this._sock.rQskip8();  // Padding

            var first_colour = this._sock.rQshift16();
            var num_colours = this._sock.rQshift16();
            if (this._sock.rQwait('SetColorMapEntries', num_colours * 6, 6)) { return false; }

            for (var c = 0; c < num_colours; c++) {
                var red = parseInt(this._sock.rQshift16() / 256, 10);
                var green = parseInt(this._sock.rQshift16() / 256, 10);
                var blue = parseInt(this._sock.rQshift16() / 256, 10);
                this._display.set_colourMap([blue, green, red], first_colour + c);
            }
            Util.Debug("colourMap: " + this._display.get_colourMap());
            Util.Info("Registered " + num_colours + " colourMap entries");

            return true;
        },

        _handle_server_cut_text: function () {
            Util.Debug("ServerCutText");
            if (this._sock.rQwait("ServerCutText header", 7, 1)) { return false; }
            this._sock.rQskipBytes(3);  // Padding
            var length = this._sock.rQshift32();
            if (this._sock.rQwait("ServerCutText", length, 8)) { return false; }

            var text = this._sock.rQshiftStr(length);
            this._onClipboard(this, text);

            return true;
        },

        _handle_xvp_msg: function () {
            if (this._sock.rQwait("XVP version and message", 3, 1)) { return false; }
            this._sock.rQskip8();  // Padding
            var xvp_ver = this._sock.rQshift8();
            var xvp_msg = this._sock.rQshift8();

            switch (xvp_msg) {
                case 0:  // XVP_FAIL
                    this._updateState(this._rfb_state, "Operation Failed");
                    break;
                case 1:  // XVP_INIT
                    this._rfb_xvp_ver = xvp_ver;
                    Util.Info("XVP extensions enabled (version " + this._rfb_xvp_ver + ")");
                    this._onXvpInit(this._rfb_xvp_ver);
                    break;
                default:
                    this._fail("Disconnected: illegal server XVP message " + xvp_msg);
                    break;
            }

            return true;
        },

        _normal_msg: function () {
            var msg_type;

            if (this._FBU.rects > 0) {
                msg_type = 0;
            } else {
                msg_type = this._sock.rQshift8();
            }

            switch (msg_type) {
                case 0:  // FramebufferUpdate
                    var ret = this._framebufferUpdate();
                    if (ret) {
                        this._sock.send(RFB.messages.fbUpdateRequests(this._display.getCleanDirtyReset(),
                                                                      this._fb_width, this._fb_height));
                    }
                    return ret;

                case 1:  // SetColorMapEntries
                    return this._handle_set_colour_map_msg();

                case 2:  // Bell
                    Util.Debug("Bell");
                    this._onBell(this);
                    return true;

                case 3:  // ServerCutText
                    return this._handle_server_cut_text();

                case 250:  // XVP
                    return this._handle_xvp_msg();

                default:
                    this._fail("Disconnected: illegal server message type " + msg_type);
                    Util.Debug("sock.rQslice(0, 30): " + this._sock.rQslice(0, 30));
                    return true;
            }
        },

        _framebufferUpdate: function () {
            var ret = true;
            var now;

            if (this._FBU.rects === 0) {
                if (this._sock.rQwait("FBU header", 3, 1)) { return false; }
                this._sock.rQskip8();  // Padding
                this._FBU.rects = this._sock.rQshift16();
                this._FBU.bytes = 0;
                this._timing.cur_fbu = 0;
                if (this._timing.fbu_rt_start > 0) {
                    now = (new Date()).getTime();
                    Util.Info("First FBU latency: " + (now - this._timing.fbu_rt_start));
                }
            }

            while (this._FBU.rects > 0) {
                if (this._rfb_state !== "normal") { return false; }

                if (this._sock.rQwait("FBU", this._FBU.bytes)) { return false; }
                if (this._FBU.bytes === 0) {
                    if (this._sock.rQwait("rect header", 12)) { return false; }
                    /* New FramebufferUpdate */

                    var hdr = this._sock.rQshiftBytes(12);
                    this._FBU.x        = (hdr[0] << 8) + hdr[1];
                    this._FBU.y        = (hdr[2] << 8) + hdr[3];
                    this._FBU.width    = (hdr[4] << 8) + hdr[5];
                    this._FBU.height   = (hdr[6] << 8) + hdr[7];
                    this._FBU.encoding = parseInt((hdr[8] << 24) + (hdr[9] << 16) +
                                                  (hdr[10] << 8) + hdr[11], 10);

                    this._onFBUReceive(this,
                        {'x': this._FBU.x, 'y': this._FBU.y,
                         'width': this._FBU.width, 'height': this._FBU.height,
                         'encoding': this._FBU.encoding,
                         'encodingName': this._encNames[this._FBU.encoding]});

                    if (!this._encNames[this._FBU.encoding]) {
                        this._fail("Disconnected: unsupported encoding " +
                                   this._FBU.encoding);
                        return false;
                    }
                }

                this._timing.last_fbu = (new Date()).getTime();

                ret = this._encHandlers[this._FBU.encoding]();

                now = (new Date()).getTime();
                this._timing.cur_fbu += (now - this._timing.last_fbu);

                if (ret) {
                    this._encStats[this._FBU.encoding][0]++;
                    this._encStats[this._FBU.encoding][1]++;
                    this._timing.pixels += this._FBU.width * this._FBU.height;
                }

                if (this._timing.pixels >= (this._fb_width * this._fb_height)) {
                    if ((this._FBU.width === this._fb_width && this._FBU.height === this._fb_height) ||
                        this._timing.fbu_rt_start > 0) {
                        this._timing.full_fbu_total += this._timing.cur_fbu;
                        this._timing.full_fbu_cnt++;
                        Util.Info("Timing of full FBU, curr: " +
                                  this._timing.cur_fbu + ", total: " +
                                  this._timing.full_fbu_total + ", cnt: " +
                                  this._timing.full_fbu_cnt + ", avg: " +
                                  (this._timing.full_fbu_total / this._timing.full_fbu_cnt));
                    }

                    if (this._timing.fbu_rt_start > 0) {
                        var fbu_rt_diff = now - this._timing.fbu_rt_start;
                        this._timing.fbu_rt_total += fbu_rt_diff;
                        this._timing.fbu_rt_cnt++;
                        Util.Info("full FBU round-trip, cur: " +
                                  fbu_rt_diff + ", total: " +
                                  this._timing.fbu_rt_total + ", cnt: " +
                                  this._timing.fbu_rt_cnt + ", avg: " +
                                  (this._timing.fbu_rt_total / this._timing.fbu_rt_cnt));
                        this._timing.fbu_rt_start = 0;
                    }
                }

                if (!ret) { return ret; }  // need more data
            }

            this._onFBUComplete(this,
                    {'x': this._FBU.x, 'y': this._FBU.y,
                     'width': this._FBU.width, 'height': this._FBU.height,
                     'encoding': this._FBU.encoding,
                     'encodingName': this._encNames[this._FBU.encoding]});

            return true;  // We finished this FBU
        },
    };

    Util.make_properties(RFB, [
        ['target', 'wo', 'dom'],                // VNC display rendering Canvas object
        ['focusContainer', 'wo', 'dom'],        // DOM element that captures keyboard input
        ['encrypt', 'rw', 'bool'],              // Use TLS/SSL/wss encryption
        ['true_color', 'rw', 'bool'],           // Request true color pixel data
        ['local_cursor', 'rw', 'bool'],         // Request locally rendered cursor
        ['shared', 'rw', 'bool'],               // Request shared mode
        ['view_only', 'rw', 'bool'],            // Disable client mouse/keyboard
        ['xvp_password_sep', 'rw', 'str'],      // Separator for XVP password fields
        ['disconnectTimeout', 'rw', 'int'],     // Time (s) to wait for disconnection
        ['wsProtocols', 'rw', 'arr'],           // Protocols to use in the WebSocket connection
        ['repeaterID', 'rw', 'str'],            // [UltraVNC] RepeaterID to connect to
        ['viewportDrag', 'rw', 'bool'],         // Move the viewport on mouse drags

        // Callback functions
        ['onUpdateState', 'rw', 'func'],        // onUpdateState(rfb, state, oldstate, statusMsg): RFB state update/change
        ['onPasswordRequired', 'rw', 'func'],   // onPasswordRequired(rfb): VNC password is required
        ['onClipboard', 'rw', 'func'],          // onClipboard(rfb, text): RFB clipboard contents received
        ['onBell', 'rw', 'func'],               // onBell(rfb): RFB Bell message received
        ['onFBUReceive', 'rw', 'func'],         // onFBUReceive(rfb, fbu): RFB FBU received but not yet processed
        ['onFBUComplete', 'rw', 'func'],        // onFBUComplete(rfb, fbu): RFB FBU received and processed
        ['onFBResize', 'rw', 'func'],           // onFBResize(rfb, width, height): frame buffer resized
        ['onDesktopName', 'rw', 'func'],        // onDesktopName(rfb, name): desktop name received
        ['onXvpInit', 'rw', 'func'],            // onXvpInit(version): XVP extensions active for this connection
    ]);

    RFB.prototype.set_local_cursor = function (cursor) {
        if (!cursor || (cursor in {'0': 1, 'no': 1, 'false': 1})) {
            this._local_cursor = false;
        } else {
            if (this._display.get_cursor_uri()) {
                this._local_cursor = true;
            } else {
                Util.Warn("Browser does not support local cursor");
            }
        }
    };

    RFB.prototype.get_display = function () { return this._display; };
    RFB.prototype.get_keyboard = function () { return this._keyboard; };
    RFB.prototype.get_mouse = function () { return this._mouse; };

    // Class Methods
    RFB.messages = {
        keyEvent: function (keysym, down) {
            var arr = [4];
            arr.push8(down);
            arr.push16(0);
            arr.push32(keysym);
            return arr;
        },

        pointerEvent: function (x, y, mask) {
            var arr = [5];  // msg-type
            arr.push8(mask);
            arr.push16(x);
            arr.push16(y);
            return arr;
        },

        // TODO(directxman12): make this unicode compatible?
        clientCutText: function (text) {
            var arr = [6];  // msg-type
            arr.push8(0);   // padding
            arr.push8(0);   // padding
            arr.push8(0);   // padding
            arr.push32(text.length);
            var n = text.length;
            for (var i = 0; i < n; i++) {
                arr.push(text.charCodeAt(i));
            }

            return arr;
        },

        pixelFormat: function (bpp, depth, true_color) {
            var arr = [0]; // msg-type
            arr.push8(0);  // padding
            arr.push8(0);  // padding
            arr.push8(0);  // padding

            arr.push8(bpp * 8); // bits-per-pixel
            arr.push8(depth * 8); // depth
            arr.push8(0);  // little-endian
            arr.push8(true_color ? 1 : 0);  // true-color

            arr.push16(255);  // red-max
            arr.push16(255);  // green-max
            arr.push16(255);  // blue-max
            arr.push8(16);    // red-shift
            arr.push8(8);     // green-shift
            arr.push8(0);     // blue-shift

            arr.push8(0);     // padding
            arr.push8(0);     // padding
            arr.push8(0);     // padding
            return arr;
        },

        clientEncodings: function (encodings, local_cursor, true_color) {
            var i, encList = [];

            for (i = 0; i < encodings.length; i++) {
                if (encodings[i][0] === "Cursor" && !local_cursor) {
                    Util.Debug("Skipping Cursor pseudo-encoding");
                } else if (encodings[i][0] === "TIGHT" && !true_color) {
                    // TODO: remove this when we have tight+non-true-color
                    Util.Warn("Skipping tight as it is only supported with true color");
                } else {
                    encList.push(encodings[i][1]);
                }
            }

            var arr = [2];  // msg-type
            arr.push8(0);   // padding

            arr.push16(encList.length);  // encoding count
            for (i = 0; i < encList.length; i++) {
                arr.push32(encList[i]);
            }

            return arr;
        },

        fbUpdateRequests: function (cleanDirty, fb_width, fb_height) {
            var arr = [];

            var cb = cleanDirty.cleanBox;
            var w, h;
            if (cb.w > 0 && cb.h > 0) {
                w = typeof cb.w === "undefined" ? fb_width : cb.w;
                h = typeof cb.h === "undefined" ? fb_height : cb.h;
                // Request incremental for clean box
                arr = arr.concat(RFB.messages.fbUpdateRequest(1, cb.x, cb.y, w, h));
            }

            for (var i = 0; i < cleanDirty.dirtyBoxes.length; i++) {
                var db = cleanDirty.dirtyBoxes[i];
                // Force all (non-incremental) for dirty box
                w = typeof db.w === "undefined" ? fb_width : db.w;
                h = typeof db.h === "undefined" ? fb_height : db.h;
                arr = arr.concat(RFB.messages.fbUpdateRequest(0, db.x, db.y, w, h));
            }

            return arr;
        },

        fbUpdateRequest: function (incremental, x, y, w, h) {
            if (typeof(x) === "undefined") { x = 0; }
            if (typeof(y) === "undefined") { y = 0; }

            var arr = [3];  // msg-type
            arr.push8(incremental);
            arr.push16(x);
            arr.push16(y);
            arr.push16(w);
            arr.push16(h);

            return arr;
        }
    };

    RFB.genDES = function (password, challenge) {
        var passwd = [];
        for (var i = 0; i < password.length; i++) {
            passwd.push(password.charCodeAt(i));
        }
        return (new DES(passwd)).encrypt(challenge);
    };

    RFB.extract_data_uri = function (arr) {
        return ";base64," + Base64.encode(arr);
    };

    RFB.encodingHandlers = {
        RAW: function () {
            if (this._FBU.lines === 0) {
                this._FBU.lines = this._FBU.height;
            }

            this._FBU.bytes = this._FBU.width * this._fb_Bpp;  // at least a line
            if (this._sock.rQwait("RAW", this._FBU.bytes)) { return false; }
            var cur_y = this._FBU.y + (this._FBU.height - this._FBU.lines);
            var curr_height = Math.min(this._FBU.lines,
                                       Math.floor(this._sock.rQlen() / (this._FBU.width * this._fb_Bpp)));
            this._display.blitImage(this._FBU.x, cur_y, this._FBU.width,
                                    curr_height, this._sock.get_rQ(),
                                    this._sock.get_rQi());
            this._sock.rQskipBytes(this._FBU.width * curr_height * this._fb_Bpp);
            this._FBU.lines -= curr_height;

            if (this._FBU.lines > 0) {
                this._FBU.bytes = this._FBU.width * this._fb_Bpp;  // At least another line
            } else {
                this._FBU.rects--;
                this._FBU.bytes = 0;
            }

            return true;
        },

        COPYRECT: function () {
            this._FBU.bytes = 4;
            if (this._sock.rQwait("COPYRECT", 4)) { return false; }
            this._display.renderQ_push({
                'type': 'copy',
                'old_x': this._sock.rQshift16(),
                'old_y': this._sock.rQshift16(),
                'x': this._FBU.x,
                'y': this._FBU.y,
                'width': this._FBU.width,
                'height': this._FBU.height
            });
            this._FBU.rects--;
            this._FBU.bytes = 0;
            return true;
        },

        RRE: function () {
            var color;
            if (this._FBU.subrects === 0) {
                this._FBU.bytes = 4 + this._fb_Bpp;
                if (this._sock.rQwait("RRE", 4 + this._fb_Bpp)) { return false; }
                this._FBU.subrects = this._sock.rQshift32();
                color = this._sock.rQshiftBytes(this._fb_Bpp);  // Background
                this._display.fillRect(this._FBU.x, this._FBU.y, this._FBU.width, this._FBU.height, color);
            }

            while (this._FBU.subrects > 0 && this._sock.rQlen() >= (this._fb_Bpp + 8)) {
                color = this._sock.rQshiftBytes(this._fb_Bpp);
                var x = this._sock.rQshift16();
                var y = this._sock.rQshift16();
                var width = this._sock.rQshift16();
                var height = this._sock.rQshift16();
                this._display.fillRect(this._FBU.x + x, this._FBU.y + y, width, height, color);
                this._FBU.subrects--;
            }

            if (this._FBU.subrects > 0) {
                var chunk = Math.min(this._rre_chunk_sz, this._FBU.subrects);
                this._FBU.bytes = (this._fb_Bpp + 8) * chunk;
            } else {
                this._FBU.rects--;
                this._FBU.bytes = 0;
            }

            return true;
        },

        HEXTILE: function () {
            var rQ = this._sock.get_rQ();
            var rQi = this._sock.get_rQi();

            if (this._FBU.tiles === 0) {
                this._FBU.tiles_x = Math.ceil(this._FBU.width / 16);
                this._FBU.tiles_y = Math.ceil(this._FBU.height / 16);
                this._FBU.total_tiles = this._FBU.tiles_x * this._FBU.tiles_y;
                this._FBU.tiles = this._FBU.total_tiles;
            }

            while (this._FBU.tiles > 0) {
                this._FBU.bytes = 1;
                if (this._sock.rQwait("HEXTILE subencoding", this._FBU.bytes)) { return false; }
                var subencoding = rQ[rQi];  // Peek
                if (subencoding > 30) {  // Raw
                    this._fail("Disconnected: illegal hextile subencoding " + subencoding);
                    return false;
                }

                var subrects = 0;
                var curr_tile = this._FBU.total_tiles - this._FBU.tiles;
                var tile_x = curr_tile % this._FBU.tiles_x;
                var tile_y = Math.floor(curr_tile / this._FBU.tiles_x);
                var x = this._FBU.x + tile_x * 16;
                var y = this._FBU.y + tile_y * 16;
                var w = Math.min(16, (this._FBU.x + this._FBU.width) - x);
                var h = Math.min(16, (this._FBU.y + this._FBU.height) - y);

                // Figure out how much we are expecting
                if (subencoding & 0x01) {  // Raw
                    this._FBU.bytes += w * h * this._fb_Bpp;
                } else {
                    if (subencoding & 0x02) {  // Background
                        this._FBU.bytes += this._fb_Bpp;
                    }
                    if (subencoding & 0x04) {  // Foreground
                        this._FBU.bytes += this._fb_Bpp;
                    }
                    if (subencoding & 0x08) {  // AnySubrects
                        this._FBU.bytes++;  // Since we aren't shifting it off
                        if (this._sock.rQwait("hextile subrects header", this._FBU.bytes)) { return false; }
                        subrects = rQ[rQi + this._FBU.bytes - 1];  // Peek
                        if (subencoding & 0x10) {  // SubrectsColoured
                            this._FBU.bytes += subrects * (this._fb_Bpp + 2);
                        } else {
                            this._FBU.bytes += subrects * 2;
                        }
                    }
                }

                if (this._sock.rQwait("hextile", this._FBU.bytes)) { return false; }

                // We know the encoding and have a whole tile
                this._FBU.subencoding = rQ[rQi];
                rQi++;
                if (this._FBU.subencoding === 0) {
                    if (this._FBU.lastsubencoding & 0x01) {
                        // Weird: ignore blanks are RAW
                        Util.Debug("     Ignoring blank after RAW");
                    } else {
                        this._display.fillRect(x, y, w, h, this._FBU.background);
                    }
                } else if (this._FBU.subencoding & 0x01) {  // Raw
                    this._display.blitImage(x, y, w, h, rQ, rQi);
                    rQi += this._FBU.bytes - 1;
                } else {
                    if (this._FBU.subencoding & 0x02) {  // Background
                        this._FBU.background = rQ.slice(rQi, rQi + this._fb_Bpp);
                        rQi += this._fb_Bpp;
                    }
                    if (this._FBU.subencoding & 0x04) {  // Foreground
                        this._FBU.foreground = rQ.slice(rQi, rQi + this._fb_Bpp);
                        rQi += this._fb_Bpp;
                    }

                    this._display.startTile(x, y, w, h, this._FBU.background);
                    if (this._FBU.subencoding & 0x08) {  // AnySubrects
                        subrects = rQ[rQi];
                        rQi++;

                        for (var s = 0; s < subrects; s++) {
                            var color;
                            if (this._FBU.subencoding & 0x10) {  // SubrectsColoured
                                color = rQ.slice(rQi, rQi + this._fb_Bpp);
                                rQi += this._fb_Bpp;
                            } else {
                                color = this._FBU.foreground;
                            }
                            var xy = rQ[rQi];
                            rQi++;
                            var sx = (xy >> 4);
                            var sy = (xy & 0x0f);

                            var wh = rQ[rQi];
                            rQi++;
                            var sw = (wh >> 4) + 1;
                            var sh = (wh & 0x0f) + 1;

                            this._display.subTile(sx, sy, sw, sh, color);
                        }
                    }
                    this._display.finishTile();
                }
                this._sock.set_rQi(rQi);
                this._FBU.lastsubencoding = this._FBU.subencoding;
                this._FBU.bytes = 0;
                this._FBU.tiles--;
            }

            if (this._FBU.tiles === 0) {
                this._FBU.rects--;
            }

            return true;
        },

        getTightCLength: function (arr) {
            var header = 1, data = 0;
            data += arr[0] & 0x7f;
            if (arr[0] & 0x80) {
                header++;
                data += (arr[1] & 0x7f) << 7;
                if (arr[1] & 0x80) {
                    header++;
                    data += arr[2] << 14;
                }
            }
            return [header, data];
        },

        display_tight: function (isTightPNG) {
            if (this._fb_depth === 1) {
                this._fail("Tight protocol handler only implements true color mode");
            }

            this._FBU.bytes = 1;  // compression-control byte
            if (this._sock.rQwait("TIGHT compression-control", this._FBU.bytes)) { return false; }

            var checksum = function (data) {
                var sum = 0;
                for (var i = 0; i < data.length; i++) {
                    sum += data[i];
                    if (sum > 65536) sum -= 65536;
                }
                return sum;
            };

            var resetStreams = 0;
            var streamId = -1;
            var decompress = function (data) {
                for (var i = 0; i < 4; i++) {
                    if ((resetStreams >> i) & 1) {
                        this._FBU.zlibs[i].reset();
                        Util.Info("Reset zlib stream " + i);
                    }
                }

                var uncompressed = this._FBU.zlibs[streamId].uncompress(data, 0);
                if (uncompressed.status !== 0) {
                    Util.Error("Invalid data in zlib stream");
                }

                return uncompressed.data;
            }.bind(this);

            var indexedToRGB = function (data, numColors, palette, width, height) {
                // Convert indexed (palette based) image data to RGB
                // TODO: reduce number of calculations inside loop
                var dest = [];
                var x, y, dp, sp;
                if (numColors === 2) {
                    var w = Math.floor((width + 7) / 8);
                    var w1 = Math.floor(width / 8);

                    for (y = 0; y < height; y++) {
                        var b;
                        for (x = 0; x < w1; x++) {
                            for (b = 7; b >= 0; b--) {
                                dp = (y * width + x * 8 + 7 - b) * 3;
                                sp = (data[y * w + x] >> b & 1) * 3;
                                dest[dp] = palette[sp];
                                dest[dp + 1] = palette[sp + 1];
                                dest[dp + 2] = palette[sp + 2];
                            }
                        }

                        for (b = 7; b >= 8 - width % 8; b--) {
                            dp = (y * width + x * 8 + 7 - b) * 3;
                            sp = (data[y * w + x] >> b & 1) * 3;
                            dest[dp] = palette[sp];
                            dest[dp + 1] = palette[sp + 1];
                            dest[dp + 2] = palette[sp + 2];
                        }
                    }
                } else {
                    for (y = 0; y < height; y++) {
                        for (x = 0; x < width; x++) {
                            dp = (y * width + x) * 3;
                            sp = data[y * width + x] * 3;
                            dest[dp] = palette[sp];
                            dest[dp + 1] = palette[sp + 1];
                            dest[dp + 2] = palette[sp + 2];
                        }
                    }
                }

                return dest;
            }.bind(this);

            var rQ = this._sock.get_rQ();
            var rQi = this._sock.get_rQi();
            var cmode, clength, data;

            var handlePalette = function () {
                var numColors = rQ[rQi + 2] + 1;
                var paletteSize = numColors * this._fb_depth;
                this._FBU.bytes += paletteSize;
                if (this._sock.rQwait("TIGHT palette " + cmode, this._FBU.bytes)) { return false; }

                var bpp = (numColors <= 2) ? 1 : 8;
                var rowSize = Math.floor((this._FBU.width * bpp + 7) / 8);
                var raw = false;
                if (rowSize * this._FBU.height < 12) {
                    raw = true;
                    clength = [0, rowSize * this._FBU.height];
                } else {
                    clength = RFB.encodingHandlers.getTightCLength(this._sock.rQslice(3 + paletteSize,
                                                                                      3 + paletteSize + 3));
                }

                this._FBU.bytes += clength[0] + clength[1];
                if (this._sock.rQwait("TIGHT " + cmode, this._FBU.bytes)) { return false; }

                // Shift ctl, filter id, num colors, palette entries, and clength off
                this._sock.rQskipBytes(3);
                var palette = this._sock.rQshiftBytes(paletteSize);
                this._sock.rQskipBytes(clength[0]);

                if (raw) {
                    data = this._sock.rQshiftBytes(clength[1]);
                } else {
                    data = decompress(this._sock.rQshiftBytes(clength[1]));
                }

                // Convert indexed (palette based) image data to RGB
                var rgb = indexedToRGB(data, numColors, palette, this._FBU.width, this._FBU.height);

                this._display.renderQ_push({
                    'type': 'blitRgb',
                    'data': rgb,
                    'x': this._FBU.x,
                    'y': this._FBU.y,
                    'width': this._FBU.width,
                    'height': this._FBU.height
                });

                return true;
            }.bind(this);

            var handleCopy = function () {
                var raw = false;
                var uncompressedSize = this._FBU.width * this._FBU.height * this._fb_depth;
                if (uncompressedSize < 12) {
                    raw = true;
                    clength = [0, uncompressedSize];
                } else {
                    clength = RFB.encodingHandlers.getTightCLength(this._sock.rQslice(1, 4));
                }
                this._FBU.bytes = 1 + clength[0] + clength[1];
                if (this._sock.rQwait("TIGHT " + cmode, this._FBU.bytes)) { return false; }

                // Shift ctl, clength off
                this._sock.rQshiftBytes(1 + clength[0]);

                if (raw) {
                    data = this._sock.rQshiftBytes(clength[1]);
                } else {
                    data = decompress(this._sock.rQshiftBytes(clength[1]));
                }

                this._display.renderQ_push({
                    'type': 'blitRgb',
                    'data': data,
                    'x': this._FBU.x,
                    'y': this._FBU.y,
                    'width': this._FBU.width,
                    'height': this._FBU.height
                });

                return true;
            }.bind(this);

            var ctl = this._sock.rQpeek8();

            // Keep tight reset bits
            resetStreams = ctl & 0xF;

            // Figure out filter
            ctl = ctl >> 4;
            streamId = ctl & 0x3;

            if (ctl === 0x08)       cmode = "fill";
            else if (ctl === 0x09)  cmode = "jpeg";
            else if (ctl === 0x0A)  cmode = "png";
            else if (ctl & 0x04)    cmode = "filter";
            else if (ctl < 0x04)    cmode = "copy";
            else return this._fail("Illegal tight compression received, ctl: " + ctl);

            if (isTightPNG && (cmode === "filter" || cmode === "copy")) {
                return this._fail("filter/copy received in tightPNG mode");
            }

            switch (cmode) {
                // fill use fb_depth because TPIXELs drop the padding byte
                case "fill":  // TPIXEL
                    this._FBU.bytes += this._fb_depth;
                    break;
                case "jpeg":  // max clength
                    this._FBU.bytes += 3;
                    break;
                case "png":  // max clength
                    this._FBU.bytes += 3;
                    break;
                case "filter":  // filter id + num colors if palette
                    this._FBU.bytes += 2;
                    break;
                case "copy":
                    break;
            }

            if (this._sock.rQwait("TIGHT " + cmode, this._FBU.bytes)) { return false; }

            // Determine FBU.bytes
            switch (cmode) {
                case "fill":
                    this._sock.rQskip8();  // shift off ctl
                    var color = this._sock.rQshiftBytes(this._fb_depth);
                    this._display.renderQ_push({
                        'type': 'fill',
                        'x': this._FBU.x,
                        'y': this._FBU.y,
                        'width': this._FBU.width,
                        'height': this._FBU.height,
                        'color': [color[2], color[1], color[0]]
                    });
                    break;
                case "png":
                case "jpeg":
                    clength = RFB.encodingHandlers.getTightCLength(this._sock.rQslice(1, 4));
                    this._FBU.bytes = 1 + clength[0] + clength[1];  // ctl + clength size + jpeg-data
                    if (this._sock.rQwait("TIGHT " + cmode, this._FBU.bytes)) { return false; }

                    // We have everything, render it
                    this._sock.rQskipBytes(1 + clength[0]);  // shift off clt + compact length
                    var img = new Image();
                    img.src = "data: image/" + cmode +
                        RFB.extract_data_uri(this._sock.rQshiftBytes(clength[1]));
                    this._display.renderQ_push({
                        'type': 'img',
                        'img': img,
                        'x': this._FBU.x,
                        'y': this._FBU.y
                    });
                    img = null;
                    break;
                case "filter":
                    var filterId = rQ[rQi + 1];
                    if (filterId === 1) {
                        if (!handlePalette()) { return false; }
                    } else {
                        // Filter 0, Copy could be valid here, but servers don't send it as an explicit filter
                        // Filter 2, Gradient is valid but not use if jpeg is enabled
                        // TODO(directxman12): why aren't we just calling '_fail' here
                        throw new Error("Unsupported tight subencoding received, filter: " + filterId);
                    }
                    break;
                case "copy":
                    if (!handleCopy()) { return false; }
                    break;
            }


            this._FBU.bytes = 0;
            this._FBU.rects--;

            return true;
        },

        TIGHT: function () { return this._encHandlers.display_tight(false); },
        TIGHT_PNG: function () { return this._encHandlers.display_tight(true); },

        last_rect: function () {
            this._FBU.rects = 0;
            return true;
        },

        DesktopSize: function () {
            Util.Debug(">> set_desktopsize");
            this._fb_width = this._FBU.width;
            this._fb_height = this._FBU.height;
            this._onFBResize(this, this._fb_width, this._fb_height);
            this._display.resize(this._fb_width, this._fb_height);
            this._timing.fbu_rt_start = (new Date()).getTime();

            this._FBU.bytes = 0;
            this._FBU.rects--;

            Util.Debug("<< set_desktopsize");
            return true;
        },

        Cursor: function () {
            Util.Debug(">> set_cursor");
            var x = this._FBU.x;  // hotspot-x
            var y = this._FBU.y;  // hotspot-y
            var w = this._FBU.width;
            var h = this._FBU.height;

            var pixelslength = w * h * this._fb_Bpp;
            var masklength = Math.floor((w + 7) / 8) * h;

            this._FBU.bytes = pixelslength + masklength;
            if (this._sock.rQwait("cursor encoding", this._FBU.bytes)) { return false; }

            this._display.changeCursor(this._sock.rQshiftBytes(pixelslength),
                                       this._sock.rQshiftBytes(masklength),
                                       x, y, w, h);

            this._FBU.bytes = 0;
            this._FBU.rects--;

            Util.Debug("<< set_cursor");
            return true;
        },

        JPEG_quality_lo: function () {
            Util.Error("Server sent jpeg_quality pseudo-encoding");
        },

        compress_lo: function () {
            Util.Error("Server sent compress level pseudo-encoding");
        }
    };
})();

define("vnc-rfb", ["vnc-util"], function(){});

var XK_VoidSymbol =                0xffffff, /* Void symbol */

XK_BackSpace =                   0xff08, /* Back space, back char */
XK_Tab =                         0xff09,
XK_Linefeed =                    0xff0a, /* Linefeed, LF */
XK_Clear =                       0xff0b,
XK_Return =                      0xff0d, /* Return, enter */
XK_Pause =                       0xff13, /* Pause, hold */
XK_Scroll_Lock =                 0xff14,
XK_Sys_Req =                     0xff15,
XK_Escape =                      0xff1b,
XK_Delete =                      0xffff, /* Delete, rubout */

/* Cursor control & motion */

XK_Home =                        0xff50,
XK_Left =                        0xff51, /* Move left, left arrow */
XK_Up =                          0xff52, /* Move up, up arrow */
XK_Right =                       0xff53, /* Move right, right arrow */
XK_Down =                        0xff54, /* Move down, down arrow */
XK_Prior =                       0xff55, /* Prior, previous */
XK_Page_Up =                     0xff55,
XK_Next =                        0xff56, /* Next */
XK_Page_Down =                   0xff56,
XK_End =                         0xff57, /* EOL */
XK_Begin =                       0xff58, /* BOL */


/* Misc functions */

XK_Select =                      0xff60, /* Select, mark */
XK_Print =                       0xff61,
XK_Execute =                     0xff62, /* Execute, run, do */
XK_Insert =                      0xff63, /* Insert, insert here */
XK_Undo =                        0xff65,
XK_Redo =                        0xff66, /* Redo, again */
XK_Menu =                        0xff67,
XK_Find =                        0xff68, /* Find, search */
XK_Cancel =                      0xff69, /* Cancel, stop, abort, exit */
XK_Help =                        0xff6a, /* Help */
XK_Break =                       0xff6b,
XK_Mode_switch =                 0xff7e, /* Character set switch */
XK_script_switch =               0xff7e, /* Alias for mode_switch */
XK_Num_Lock =                    0xff7f,

/* Keypad functions, keypad numbers cleverly chosen to map to ASCII */

XK_KP_Space =                    0xff80, /* Space */
XK_KP_Tab =                      0xff89,
XK_KP_Enter =                    0xff8d, /* Enter */
XK_KP_F1 =                       0xff91, /* PF1, KP_A, ... */
XK_KP_F2 =                       0xff92,
XK_KP_F3 =                       0xff93,
XK_KP_F4 =                       0xff94,
XK_KP_Home =                     0xff95,
XK_KP_Left =                     0xff96,
XK_KP_Up =                       0xff97,
XK_KP_Right =                    0xff98,
XK_KP_Down =                     0xff99,
XK_KP_Prior =                    0xff9a,
XK_KP_Page_Up =                  0xff9a
XK_KP_Next =                     0xff9b,
XK_KP_Page_Down =                0xff9b,
XK_KP_End =                      0xff9c,
XK_KP_Begin =                    0xff9d,
XK_KP_Insert =                   0xff9e,
XK_KP_Delete =                   0xff9f,
XK_KP_Equal =                    0xffbd, /* Equals */
XK_KP_Multiply =                 0xffaa,
XK_KP_Add =                      0xffab,
XK_KP_Separator =                0xffac, /* Separator, often comma */
XK_KP_Subtract =                 0xffad,
XK_KP_Decimal =                  0xffae,
XK_KP_Divide =                   0xffaf,

XK_KP_0 =                        0xffb0,
XK_KP_1 =                        0xffb1,
XK_KP_2 =                        0xffb2,
XK_KP_3 =                        0xffb3,
XK_KP_4 =                        0xffb4,
XK_KP_5 =                        0xffb5,
XK_KP_6 =                        0xffb6,
XK_KP_7 =                        0xffb7,
XK_KP_8 =                        0xffb8,
XK_KP_9 =                        0xffb9,

/*
 * Auxiliary functions; note the duplicate definitions for left and right
 * function keys;  Sun keyboards and a few other manufacturers have such
 * function key groups on the left and/or right sides of the keyboard.
 * We've not found a keyboard with more than 35 function keys total.
 */

XK_F1 =                          0xffbe,
XK_F2 =                          0xffbf,
XK_F3 =                          0xffc0,
XK_F4 =                          0xffc1,
XK_F5 =                          0xffc2,
XK_F6 =                          0xffc3,
XK_F7 =                          0xffc4,
XK_F8 =                          0xffc5,
XK_F9 =                          0xffc6,
XK_F10 =                         0xffc7,
XK_F11 =                         0xffc8,
XK_L1 =                          0xffc8,
XK_F12 =                         0xffc9,
XK_L2 =                          0xffc9,
XK_F13 =                         0xffca,
XK_L3 =                          0xffca,
XK_F14 =                         0xffcb,
XK_L4 =                          0xffcb,
XK_F15 =                         0xffcc,
XK_L5 =                          0xffcc,
XK_F16 =                         0xffcd,
XK_L6 =                          0xffcd,
XK_F17 =                         0xffce,
XK_L7 =                          0xffce,
XK_F18 =                         0xffcf,
XK_L8 =                          0xffcf,
XK_F19 =                         0xffd0,
XK_L9 =                          0xffd0,
XK_F20 =                         0xffd1,
XK_L10 =                         0xffd1,
XK_F21 =                         0xffd2,
XK_R1 =                          0xffd2,
XK_F22 =                         0xffd3,
XK_R2 =                          0xffd3,
XK_F23 =                         0xffd4,
XK_R3 =                          0xffd4,
XK_F24 =                         0xffd5,
XK_R4 =                          0xffd5,
XK_F25 =                         0xffd6,
XK_R5 =                          0xffd6,
XK_F26 =                         0xffd7,
XK_R6 =                          0xffd7,
XK_F27 =                         0xffd8,
XK_R7 =                          0xffd8,
XK_F28 =                         0xffd9,
XK_R8 =                          0xffd9,
XK_F29 =                         0xffda,
XK_R9 =                          0xffda,
XK_F30 =                         0xffdb,
XK_R10 =                         0xffdb,
XK_F31 =                         0xffdc,
XK_R11 =                         0xffdc,
XK_F32 =                         0xffdd,
XK_R12 =                         0xffdd,
XK_F33 =                         0xffde,
XK_R13 =                         0xffde,
XK_F34 =                         0xffdf,
XK_R14 =                         0xffdf,
XK_F35 =                         0xffe0,
XK_R15 =                         0xffe0,

/* Modifiers */

XK_Shift_L =                     0xffe1, /* Left shift */
XK_Shift_R =                     0xffe2, /* Right shift */
XK_Control_L =                   0xffe3, /* Left control */
XK_Control_R =                   0xffe4, /* Right control */
XK_Caps_Lock =                   0xffe5, /* Caps lock */
XK_Shift_Lock =                  0xffe6, /* Shift lock */

XK_Meta_L =                      0xffe7, /* Left meta */
XK_Meta_R =                      0xffe8, /* Right meta */
XK_Alt_L =                       0xffe9, /* Left alt */
XK_Alt_R =                       0xffea, /* Right alt */
XK_Super_L =                     0xffeb, /* Left super */
XK_Super_R =                     0xffec, /* Right super */
XK_Hyper_L =                     0xffed, /* Left hyper */
XK_Hyper_R =                     0xffee, /* Right hyper */

XK_ISO_Level3_Shift =            0xfe03, /* AltGr */

/*
 * Latin 1
 * (ISO/IEC 8859-1 = Unicode U+0020..U+00FF)
 * Byte 3 = 0
 */

XK_space =                       0x0020, /* U+0020 SPACE */
XK_exclam =                      0x0021, /* U+0021 EXCLAMATION MARK */
XK_quotedbl =                    0x0022, /* U+0022 QUOTATION MARK */
XK_numbersign =                  0x0023, /* U+0023 NUMBER SIGN */
XK_dollar =                      0x0024, /* U+0024 DOLLAR SIGN */
XK_percent =                     0x0025, /* U+0025 PERCENT SIGN */
XK_ampersand =                   0x0026, /* U+0026 AMPERSAND */
XK_apostrophe =                  0x0027, /* U+0027 APOSTROPHE */
XK_quoteright =                  0x0027, /* deprecated */
XK_parenleft =                   0x0028, /* U+0028 LEFT PARENTHESIS */
XK_parenright =                  0x0029, /* U+0029 RIGHT PARENTHESIS */
XK_asterisk =                    0x002a, /* U+002A ASTERISK */
XK_plus =                        0x002b, /* U+002B PLUS SIGN */
XK_comma =                       0x002c, /* U+002C COMMA */
XK_minus =                       0x002d, /* U+002D HYPHEN-MINUS */
XK_period =                      0x002e, /* U+002E FULL STOP */
XK_slash =                       0x002f, /* U+002F SOLIDUS */
XK_0 =                           0x0030, /* U+0030 DIGIT ZERO */
XK_1 =                           0x0031, /* U+0031 DIGIT ONE */
XK_2 =                           0x0032, /* U+0032 DIGIT TWO */
XK_3 =                           0x0033, /* U+0033 DIGIT THREE */
XK_4 =                           0x0034, /* U+0034 DIGIT FOUR */
XK_5 =                           0x0035, /* U+0035 DIGIT FIVE */
XK_6 =                           0x0036, /* U+0036 DIGIT SIX */
XK_7 =                           0x0037, /* U+0037 DIGIT SEVEN */
XK_8 =                           0x0038, /* U+0038 DIGIT EIGHT */
XK_9 =                           0x0039, /* U+0039 DIGIT NINE */
XK_colon =                       0x003a, /* U+003A COLON */
XK_semicolon =                   0x003b, /* U+003B SEMICOLON */
XK_less =                        0x003c, /* U+003C LESS-THAN SIGN */
XK_equal =                       0x003d, /* U+003D EQUALS SIGN */
XK_greater =                     0x003e, /* U+003E GREATER-THAN SIGN */
XK_question =                    0x003f, /* U+003F QUESTION MARK */
XK_at =                          0x0040, /* U+0040 COMMERCIAL AT */
XK_A =                           0x0041, /* U+0041 LATIN CAPITAL LETTER A */
XK_B =                           0x0042, /* U+0042 LATIN CAPITAL LETTER B */
XK_C =                           0x0043, /* U+0043 LATIN CAPITAL LETTER C */
XK_D =                           0x0044, /* U+0044 LATIN CAPITAL LETTER D */
XK_E =                           0x0045, /* U+0045 LATIN CAPITAL LETTER E */
XK_F =                           0x0046, /* U+0046 LATIN CAPITAL LETTER F */
XK_G =                           0x0047, /* U+0047 LATIN CAPITAL LETTER G */
XK_H =                           0x0048, /* U+0048 LATIN CAPITAL LETTER H */
XK_I =                           0x0049, /* U+0049 LATIN CAPITAL LETTER I */
XK_J =                           0x004a, /* U+004A LATIN CAPITAL LETTER J */
XK_K =                           0x004b, /* U+004B LATIN CAPITAL LETTER K */
XK_L =                           0x004c, /* U+004C LATIN CAPITAL LETTER L */
XK_M =                           0x004d, /* U+004D LATIN CAPITAL LETTER M */
XK_N =                           0x004e, /* U+004E LATIN CAPITAL LETTER N */
XK_O =                           0x004f, /* U+004F LATIN CAPITAL LETTER O */
XK_P =                           0x0050, /* U+0050 LATIN CAPITAL LETTER P */
XK_Q =                           0x0051, /* U+0051 LATIN CAPITAL LETTER Q */
XK_R =                           0x0052, /* U+0052 LATIN CAPITAL LETTER R */
XK_S =                           0x0053, /* U+0053 LATIN CAPITAL LETTER S */
XK_T =                           0x0054, /* U+0054 LATIN CAPITAL LETTER T */
XK_U =                           0x0055, /* U+0055 LATIN CAPITAL LETTER U */
XK_V =                           0x0056, /* U+0056 LATIN CAPITAL LETTER V */
XK_W =                           0x0057, /* U+0057 LATIN CAPITAL LETTER W */
XK_X =                           0x0058, /* U+0058 LATIN CAPITAL LETTER X */
XK_Y =                           0x0059, /* U+0059 LATIN CAPITAL LETTER Y */
XK_Z =                           0x005a, /* U+005A LATIN CAPITAL LETTER Z */
XK_bracketleft =                 0x005b, /* U+005B LEFT SQUARE BRACKET */
XK_backslash =                   0x005c, /* U+005C REVERSE SOLIDUS */
XK_bracketright =                0x005d, /* U+005D RIGHT SQUARE BRACKET */
XK_asciicircum =                 0x005e, /* U+005E CIRCUMFLEX ACCENT */
XK_underscore =                  0x005f, /* U+005F LOW LINE */
XK_grave =                       0x0060, /* U+0060 GRAVE ACCENT */
XK_quoteleft =                   0x0060, /* deprecated */
XK_a =                           0x0061, /* U+0061 LATIN SMALL LETTER A */
XK_b =                           0x0062, /* U+0062 LATIN SMALL LETTER B */
XK_c =                           0x0063, /* U+0063 LATIN SMALL LETTER C */
XK_d =                           0x0064, /* U+0064 LATIN SMALL LETTER D */
XK_e =                           0x0065, /* U+0065 LATIN SMALL LETTER E */
XK_f =                           0x0066, /* U+0066 LATIN SMALL LETTER F */
XK_g =                           0x0067, /* U+0067 LATIN SMALL LETTER G */
XK_h =                           0x0068, /* U+0068 LATIN SMALL LETTER H */
XK_i =                           0x0069, /* U+0069 LATIN SMALL LETTER I */
XK_j =                           0x006a, /* U+006A LATIN SMALL LETTER J */
XK_k =                           0x006b, /* U+006B LATIN SMALL LETTER K */
XK_l =                           0x006c, /* U+006C LATIN SMALL LETTER L */
XK_m =                           0x006d, /* U+006D LATIN SMALL LETTER M */
XK_n =                           0x006e, /* U+006E LATIN SMALL LETTER N */
XK_o =                           0x006f, /* U+006F LATIN SMALL LETTER O */
XK_p =                           0x0070, /* U+0070 LATIN SMALL LETTER P */
XK_q =                           0x0071, /* U+0071 LATIN SMALL LETTER Q */
XK_r =                           0x0072, /* U+0072 LATIN SMALL LETTER R */
XK_s =                           0x0073, /* U+0073 LATIN SMALL LETTER S */
XK_t =                           0x0074, /* U+0074 LATIN SMALL LETTER T */
XK_u =                           0x0075, /* U+0075 LATIN SMALL LETTER U */
XK_v =                           0x0076, /* U+0076 LATIN SMALL LETTER V */
XK_w =                           0x0077, /* U+0077 LATIN SMALL LETTER W */
XK_x =                           0x0078, /* U+0078 LATIN SMALL LETTER X */
XK_y =                           0x0079, /* U+0079 LATIN SMALL LETTER Y */
XK_z =                           0x007a, /* U+007A LATIN SMALL LETTER Z */
XK_braceleft =                   0x007b, /* U+007B LEFT CURLY BRACKET */
XK_bar =                         0x007c, /* U+007C VERTICAL LINE */
XK_braceright =                  0x007d, /* U+007D RIGHT CURLY BRACKET */
XK_asciitilde =                  0x007e, /* U+007E TILDE */

XK_nobreakspace =                0x00a0, /* U+00A0 NO-BREAK SPACE */
XK_exclamdown =                  0x00a1, /* U+00A1 INVERTED EXCLAMATION MARK */
XK_cent =                        0x00a2, /* U+00A2 CENT SIGN */
XK_sterling =                    0x00a3, /* U+00A3 POUND SIGN */
XK_currency =                    0x00a4, /* U+00A4 CURRENCY SIGN */
XK_yen =                         0x00a5, /* U+00A5 YEN SIGN */
XK_brokenbar =                   0x00a6, /* U+00A6 BROKEN BAR */
XK_section =                     0x00a7, /* U+00A7 SECTION SIGN */
XK_diaeresis =                   0x00a8, /* U+00A8 DIAERESIS */
XK_copyright =                   0x00a9, /* U+00A9 COPYRIGHT SIGN */
XK_ordfeminine =                 0x00aa, /* U+00AA FEMININE ORDINAL INDICATOR */
XK_guillemotleft =               0x00ab, /* U+00AB LEFT-POINTING DOUBLE ANGLE QUOTATION MARK */
XK_notsign =                     0x00ac, /* U+00AC NOT SIGN */
XK_hyphen =                      0x00ad, /* U+00AD SOFT HYPHEN */
XK_registered =                  0x00ae, /* U+00AE REGISTERED SIGN */
XK_macron =                      0x00af, /* U+00AF MACRON */
XK_degree =                      0x00b0, /* U+00B0 DEGREE SIGN */
XK_plusminus =                   0x00b1, /* U+00B1 PLUS-MINUS SIGN */
XK_twosuperior =                 0x00b2, /* U+00B2 SUPERSCRIPT TWO */
XK_threesuperior =               0x00b3, /* U+00B3 SUPERSCRIPT THREE */
XK_acute =                       0x00b4, /* U+00B4 ACUTE ACCENT */
XK_mu =                          0x00b5, /* U+00B5 MICRO SIGN */
XK_paragraph =                   0x00b6, /* U+00B6 PILCROW SIGN */
XK_periodcentered =              0x00b7, /* U+00B7 MIDDLE DOT */
XK_cedilla =                     0x00b8, /* U+00B8 CEDILLA */
XK_onesuperior =                 0x00b9, /* U+00B9 SUPERSCRIPT ONE */
XK_masculine =                   0x00ba, /* U+00BA MASCULINE ORDINAL INDICATOR */
XK_guillemotright =              0x00bb, /* U+00BB RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK */
XK_onequarter =                  0x00bc, /* U+00BC VULGAR FRACTION ONE QUARTER */
XK_onehalf =                     0x00bd, /* U+00BD VULGAR FRACTION ONE HALF */
XK_threequarters =               0x00be, /* U+00BE VULGAR FRACTION THREE QUARTERS */
XK_questiondown =                0x00bf, /* U+00BF INVERTED QUESTION MARK */
XK_Agrave =                      0x00c0, /* U+00C0 LATIN CAPITAL LETTER A WITH GRAVE */
XK_Aacute =                      0x00c1, /* U+00C1 LATIN CAPITAL LETTER A WITH ACUTE */
XK_Acircumflex =                 0x00c2, /* U+00C2 LATIN CAPITAL LETTER A WITH CIRCUMFLEX */
XK_Atilde =                      0x00c3, /* U+00C3 LATIN CAPITAL LETTER A WITH TILDE */
XK_Adiaeresis =                  0x00c4, /* U+00C4 LATIN CAPITAL LETTER A WITH DIAERESIS */
XK_Aring =                       0x00c5, /* U+00C5 LATIN CAPITAL LETTER A WITH RING ABOVE */
XK_AE =                          0x00c6, /* U+00C6 LATIN CAPITAL LETTER AE */
XK_Ccedilla =                    0x00c7, /* U+00C7 LATIN CAPITAL LETTER C WITH CEDILLA */
XK_Egrave =                      0x00c8, /* U+00C8 LATIN CAPITAL LETTER E WITH GRAVE */
XK_Eacute =                      0x00c9, /* U+00C9 LATIN CAPITAL LETTER E WITH ACUTE */
XK_Ecircumflex =                 0x00ca, /* U+00CA LATIN CAPITAL LETTER E WITH CIRCUMFLEX */
XK_Ediaeresis =                  0x00cb, /* U+00CB LATIN CAPITAL LETTER E WITH DIAERESIS */
XK_Igrave =                      0x00cc, /* U+00CC LATIN CAPITAL LETTER I WITH GRAVE */
XK_Iacute =                      0x00cd, /* U+00CD LATIN CAPITAL LETTER I WITH ACUTE */
XK_Icircumflex =                 0x00ce, /* U+00CE LATIN CAPITAL LETTER I WITH CIRCUMFLEX */
XK_Idiaeresis =                  0x00cf, /* U+00CF LATIN CAPITAL LETTER I WITH DIAERESIS */
XK_ETH =                         0x00d0, /* U+00D0 LATIN CAPITAL LETTER ETH */
XK_Eth =                         0x00d0, /* deprecated */
XK_Ntilde =                      0x00d1, /* U+00D1 LATIN CAPITAL LETTER N WITH TILDE */
XK_Ograve =                      0x00d2, /* U+00D2 LATIN CAPITAL LETTER O WITH GRAVE */
XK_Oacute =                      0x00d3, /* U+00D3 LATIN CAPITAL LETTER O WITH ACUTE */
XK_Ocircumflex =                 0x00d4, /* U+00D4 LATIN CAPITAL LETTER O WITH CIRCUMFLEX */
XK_Otilde =                      0x00d5, /* U+00D5 LATIN CAPITAL LETTER O WITH TILDE */
XK_Odiaeresis =                  0x00d6, /* U+00D6 LATIN CAPITAL LETTER O WITH DIAERESIS */
XK_multiply =                    0x00d7, /* U+00D7 MULTIPLICATION SIGN */
XK_Oslash =                      0x00d8, /* U+00D8 LATIN CAPITAL LETTER O WITH STROKE */
XK_Ooblique =                    0x00d8, /* U+00D8 LATIN CAPITAL LETTER O WITH STROKE */
XK_Ugrave =                      0x00d9, /* U+00D9 LATIN CAPITAL LETTER U WITH GRAVE */
XK_Uacute =                      0x00da, /* U+00DA LATIN CAPITAL LETTER U WITH ACUTE */
XK_Ucircumflex =                 0x00db, /* U+00DB LATIN CAPITAL LETTER U WITH CIRCUMFLEX */
XK_Udiaeresis =                  0x00dc, /* U+00DC LATIN CAPITAL LETTER U WITH DIAERESIS */
XK_Yacute =                      0x00dd, /* U+00DD LATIN CAPITAL LETTER Y WITH ACUTE */
XK_THORN =                       0x00de, /* U+00DE LATIN CAPITAL LETTER THORN */
XK_Thorn =                       0x00de, /* deprecated */
XK_ssharp =                      0x00df, /* U+00DF LATIN SMALL LETTER SHARP S */
XK_agrave =                      0x00e0, /* U+00E0 LATIN SMALL LETTER A WITH GRAVE */
XK_aacute =                      0x00e1, /* U+00E1 LATIN SMALL LETTER A WITH ACUTE */
XK_acircumflex =                 0x00e2, /* U+00E2 LATIN SMALL LETTER A WITH CIRCUMFLEX */
XK_atilde =                      0x00e3, /* U+00E3 LATIN SMALL LETTER A WITH TILDE */
XK_adiaeresis =                  0x00e4, /* U+00E4 LATIN SMALL LETTER A WITH DIAERESIS */
XK_aring =                       0x00e5, /* U+00E5 LATIN SMALL LETTER A WITH RING ABOVE */
XK_ae =                          0x00e6, /* U+00E6 LATIN SMALL LETTER AE */
XK_ccedilla =                    0x00e7, /* U+00E7 LATIN SMALL LETTER C WITH CEDILLA */
XK_egrave =                      0x00e8, /* U+00E8 LATIN SMALL LETTER E WITH GRAVE */
XK_eacute =                      0x00e9, /* U+00E9 LATIN SMALL LETTER E WITH ACUTE */
XK_ecircumflex =                 0x00ea, /* U+00EA LATIN SMALL LETTER E WITH CIRCUMFLEX */
XK_ediaeresis =                  0x00eb, /* U+00EB LATIN SMALL LETTER E WITH DIAERESIS */
XK_igrave =                      0x00ec, /* U+00EC LATIN SMALL LETTER I WITH GRAVE */
XK_iacute =                      0x00ed, /* U+00ED LATIN SMALL LETTER I WITH ACUTE */
XK_icircumflex =                 0x00ee, /* U+00EE LATIN SMALL LETTER I WITH CIRCUMFLEX */
XK_idiaeresis =                  0x00ef, /* U+00EF LATIN SMALL LETTER I WITH DIAERESIS */
XK_eth =                         0x00f0, /* U+00F0 LATIN SMALL LETTER ETH */
XK_ntilde =                      0x00f1, /* U+00F1 LATIN SMALL LETTER N WITH TILDE */
XK_ograve =                      0x00f2, /* U+00F2 LATIN SMALL LETTER O WITH GRAVE */
XK_oacute =                      0x00f3, /* U+00F3 LATIN SMALL LETTER O WITH ACUTE */
XK_ocircumflex =                 0x00f4, /* U+00F4 LATIN SMALL LETTER O WITH CIRCUMFLEX */
XK_otilde =                      0x00f5, /* U+00F5 LATIN SMALL LETTER O WITH TILDE */
XK_odiaeresis =                  0x00f6, /* U+00F6 LATIN SMALL LETTER O WITH DIAERESIS */
XK_division =                    0x00f7, /* U+00F7 DIVISION SIGN */
XK_oslash =                      0x00f8, /* U+00F8 LATIN SMALL LETTER O WITH STROKE */
XK_ooblique =                    0x00f8, /* U+00F8 LATIN SMALL LETTER O WITH STROKE */
XK_ugrave =                      0x00f9, /* U+00F9 LATIN SMALL LETTER U WITH GRAVE */
XK_uacute =                      0x00fa, /* U+00FA LATIN SMALL LETTER U WITH ACUTE */
XK_ucircumflex =                 0x00fb, /* U+00FB LATIN SMALL LETTER U WITH CIRCUMFLEX */
XK_udiaeresis =                  0x00fc, /* U+00FC LATIN SMALL LETTER U WITH DIAERESIS */
XK_yacute =                      0x00fd, /* U+00FD LATIN SMALL LETTER Y WITH ACUTE */
XK_thorn =                       0x00fe, /* U+00FE LATIN SMALL LETTER THORN */
XK_ydiaeresis =                  0x00ff; /* U+00FF LATIN SMALL LETTER Y WITH DIAERESIS */
;
define("vnc-keysym", ["vnc-util"], function(){});

define('utils/vnc',['require','vnc-util','vnc-webutil','vnc-base64','vnc-websock','vnc-des','vnc-keysymdef','vnc-keyboard','vnc-input','vnc-display','vnc-jsunzip','vnc-rfb','vnc-keysym','sunstone-config'],function(require) {
  INCLUDE_URI = "bower_components/no-vnc/include/";
  require('vnc-util');
  require('vnc-webutil');
  require('vnc-base64');
  require('vnc-websock');
  require('vnc-des');
  require('vnc-keysymdef');
  require('vnc-keyboard');
  require('vnc-input');
  require('vnc-display');
  require('vnc-jsunzip');
  require('vnc-rfb');
  require('vnc-keysym');

  var Config = require('sunstone-config');

  var _lock = false;
  var _rfb;

  return {
    'lockStatus': lockStatus,
    'lock': lock,
    'unlock': unlock,
    'vncCallback': vncCallback,
    'disconnect': disconnect,
    'sendCtrlAltDel': sendCtrlAltDel
  }

  function lockStatus() {
    return _lock;
  }

  function lock() {
    _lock = true;
  }

  function unlock() {
    _lock = false;
  }

  function vncCallback(response) {
    _rfb = new RFB({'target':       $D('VNC_canvas'),
                   'encrypt':      Config.vncWSS == "yes",
                   'true_color':   true,
                   'local_cursor': true,
                   'shared':       true,
                   'onUpdateState':  updateVNCState});

    var proxy_host = window.location.hostname;
    var proxy_port = Config.vncProxyPort;
    var pw = response["password"];
    var token = response["token"];
    var vm_name = response["vm_name"];
    var path = '?token=' + token;

    var url = "vnc?";
    url += "host=" + proxy_host;
    url += "&port=" + proxy_port;
    url += "&token=" + token;
    url += "&password=" + pw;
    url += "&encrypt=" + Config.vncWSS;
    url += "&title=" + vm_name;

    $("#open_in_a_new_window").attr('href', url)
    _rfb.connect(proxy_host, proxy_port, pw, path);
  }

  function disconnect() {
    if (_rfb) { _rfb.disconnect(); }
  }

  function sendCtrlAltDel() {
    if (_rfb) { _rfb.sendCtrlAltDel(); }
  }

  //This is taken from noVNC examples
  function updateVNCState(rfb, state, oldstate, msg) {
    var s, sb, cad, klass;
    s = $D('VNC_status');
    sb = $D('VNC_status_bar');
    cad = $D('sendCtrlAltDelButton');
    switch (state) {
      case 'failed':       level = "error";  break;
      case 'fatal':        level = "error";  break;
      case 'normal':       level = "normal"; break;
      case 'disconnected': level = "normal"; break;
      case 'loaded':       level = "normal"; break;
      default:             level = "warn";   break;
    }

    if (state === "normal") {
      cad.disabled = false;
    } else {
      cad.disabled = true;
    }

    if (typeof(msg) !== 'undefined') {
      sb.setAttribute("class", "noVNC_status_" + level);
      s.innerHTML = msg;
    }
  }
});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*----------------------------------------------------------------------------
**  SpiceConn
**      This is the base Javascript class for establishing and
**  managing a connection to a Spice Server.
**  It is used to provide core functionality to the Spice main,
**  display, inputs, and cursor channels.  See main.js for 
**  usage.
**--------------------------------------------------------------------------*/
function SpiceConn(o)
{
    if (o === undefined || o.uri === undefined || ! o.uri)
        throw new Error("You must specify a uri");

    this.ws = new WebSocket(o.uri, 'binary');

    if (! this.ws.binaryType)
        throw new Error("WebSocket doesn't support binaryType.  Try a different browser.");

    this.connection_id = o.connection_id !== undefined ? o.connection_id : 0;
    this.type = o.type !== undefined ? o.type : SPICE_CHANNEL_MAIN;
    this.chan_id = o.chan_id !== undefined ? o.chan_id : 0;
    if (o.parent !== undefined)
    {
        this.parent = o.parent;
        this.message_id = o.parent.message_id;
        this.password = o.parent.password;
    }
    if (o.screen_id !== undefined)
        this.screen_id = o.screen_id;
    if (o.dump_id !== undefined)
        this.dump_id = o.dump_id;
    if (o.message_id !== undefined)
        this.message_id = o.message_id;
    if (o.password !== undefined)
        this.password = o.password;
    if (o.onerror !== undefined)
        this.onerror = o.onerror;
    if (o.onsuccess !== undefined)
        this.onsuccess = o.onsuccess;
    if (o.onagent !== undefined)
        this.onagent = o.onagent;

    this.state = "connecting";
    this.ws.parent = this;
    this.wire_reader = new SpiceWireReader(this, this.process_inbound);
    this.messages_sent = 0;
    this.warnings = [];

    this.ws.addEventListener('open', function(e) {
        DEBUG > 0 && console.log(">> WebSockets.onopen");
        DEBUG > 0 && console.log("id " + this.parent.connection_id +"; type " + this.parent.type);

        /***********************************************************************
        **          WHERE IT ALL REALLY BEGINS
        ***********************************************************************/
        this.parent.send_hdr();
        this.parent.wire_reader.request(SpiceLinkHeader.prototype.buffer_size());
        this.parent.state = "start";
    });
    this.ws.addEventListener('error', function(e) {
        if ('url' in e.target) {
            this.parent.log_err("WebSocket error: Can't connect to websocket on URL: " + e.target.url);
        }
        this.parent.report_error(e);
    });
    this.ws.addEventListener('close', function(e) {
        DEBUG > 0 && console.log(">> WebSockets.onclose");
        DEBUG > 0 && console.log("id " + this.parent.connection_id +"; type " + this.parent.type);
        DEBUG > 0 && console.log(e);
        if (this.parent.state != "closing" && this.parent.state != "error" && this.parent.onerror !== undefined)
        {
            var e;
            if (this.parent.state == "connecting")
                e = new Error("Connection refused.");
            else if (this.parent.state == "start" || this.parent.state == "link")
                e = new Error("Unexpected protocol mismatch.");
            else if (this.parent.state == "ticket")
                e = new Error("Bad password.");
            else
                e = new Error("Unexpected close while " + this.parent.state);

            this.parent.onerror(e);
            this.parent.log_err(e.toString());
        }
    });

    if (this.ws.readyState == 2 || this.ws.readyState == 3)
        throw new Error("Unable to connect to " + o.uri);

    this.timeout = window.setTimeout(spiceconn_timeout, SPICE_CONNECT_TIMEOUT, this);
}

SpiceConn.prototype =
{
    send_hdr : function ()
    {
        var hdr = new SpiceLinkHeader;
        var msg = new SpiceLinkMess;

        msg.connection_id = this.connection_id;
        msg.channel_type = this.type;
        // FIXME - we're not setting a channel_id...
        msg.common_caps.push(
            (1 << SPICE_COMMON_CAP_PROTOCOL_AUTH_SELECTION) |
            (1 << SPICE_COMMON_CAP_MINI_HEADER)
            );

        if (msg.channel_type == SPICE_CHANNEL_PLAYBACK)
            msg.channel_caps.push(
                (1 << SPICE_PLAYBACK_CAP_OPUS)
            );
        else if (msg.channel_type == SPICE_CHANNEL_MAIN)
            msg.channel_caps.push(
                (1 << SPICE_MAIN_CAP_AGENT_CONNECTED_TOKENS)
            );

        hdr.size = msg.buffer_size();

        var mb = new ArrayBuffer(hdr.buffer_size() + msg.buffer_size());
        hdr.to_buffer(mb);
        msg.to_buffer(mb, hdr.buffer_size());

        DEBUG > 1 && console.log("Sending header:");
        DEBUG > 2 && hexdump_buffer(mb);
        this.ws.send(mb);
    },

    send_ticket: function(ticket)
    {
        var hdr = new SpiceLinkAuthTicket();
        hdr.auth_mechanism = SPICE_COMMON_CAP_AUTH_SPICE;
        // FIXME - we need to implement RSA to make this work right
        hdr.encrypted_data = ticket;
        var mb = new ArrayBuffer(hdr.buffer_size());

        hdr.to_buffer(mb);
        DEBUG > 1 && console.log("Sending ticket:");
        DEBUG > 2 && hexdump_buffer(mb);
        this.ws.send(mb);
    },

    send_msg: function(msg)
    {
        var mb = new ArrayBuffer(msg.buffer_size());
        msg.to_buffer(mb);
        this.messages_sent++;
        DEBUG > 0 && console.log(">> hdr " + this.channel_type() + " type " + msg.type + " size " + mb.byteLength);
        DEBUG > 2 && hexdump_buffer(mb);
        this.ws.send(mb);
    },

    process_inbound: function(mb, saved_header)
    {
        DEBUG > 2 && console.log(this.type + ": processing message of size " + mb.byteLength + "; state is " + this.state);
        if (this.state == "ready")
        {
            if (saved_header == undefined)
            {
                var msg = new SpiceMiniData(mb);

                if (msg.type > 500)
                {
                    alert("Something has gone very wrong; we think we have message of type " + msg.type);
                    debugger;
                }

                if (msg.size == 0)
                {
                    this.process_message(msg);
                    this.wire_reader.request(SpiceMiniData.prototype.buffer_size());
                }
                else
                {
                    this.wire_reader.request(msg.size);
                    this.wire_reader.save_header(msg);
                }
            }
            else
            {
                saved_header.data = mb;
                this.process_message(saved_header);
                this.wire_reader.request(SpiceMiniData.prototype.buffer_size());
                this.wire_reader.save_header(undefined);
            }
        }

        else if (this.state == "start")
        {
            this.reply_hdr = new SpiceLinkHeader(mb);
            if (this.reply_hdr.magic != SPICE_MAGIC)
            {
                this.state = "error";
                var e = new Error('Error: magic mismatch: ' + this.reply_hdr.magic);
                this.report_error(e);
            }
            else
            {
                // FIXME - Determine major/minor version requirements
                this.wire_reader.request(this.reply_hdr.size);
                this.state = "link";
            }
        }

        else if (this.state == "link")
        {
            this.reply_link = new SpiceLinkReply(mb);
             // FIXME - Screen the caps - require minihdr at least, right?
            if (this.reply_link.error)
            {
                this.state = "error";
                var e = new Error('Error: reply link error ' + this.reply_link.error);
                this.report_error(e);
            }
            else
            {
                this.send_ticket(rsa_encrypt(this.reply_link.pub_key, this.password + String.fromCharCode(0)));
                this.state = "ticket";
                this.wire_reader.request(SpiceLinkAuthReply.prototype.buffer_size());
            }
        }

        else if (this.state == "ticket")
        {
            this.auth_reply = new SpiceLinkAuthReply(mb);
            if (this.auth_reply.auth_code == SPICE_LINK_ERR_OK)
            {
                DEBUG > 0 && console.log(this.type + ': Connected');

                if (this.type == SPICE_CHANNEL_DISPLAY)
                {
                    // FIXME - pixmap and glz dictionary config info?
                    var dinit = new SpiceMsgcDisplayInit();
                    var reply = new SpiceMiniData();
                    reply.build_msg(SPICE_MSGC_DISPLAY_INIT, dinit);
                    DEBUG > 0 && console.log("Request display init");
                    this.send_msg(reply);
                }
                this.state = "ready";
                this.wire_reader.request(SpiceMiniData.prototype.buffer_size());
                if (this.timeout)
                {
                    window.clearTimeout(this.timeout);
                    delete this.timeout;
                }
            }
            else
            {
                this.state = "error";
                if (this.auth_reply.auth_code == SPICE_LINK_ERR_PERMISSION_DENIED)
                {
                    var e = new Error("Permission denied.");
                }
                else
                {
                    var e = new Error("Unexpected link error " + this.auth_reply.auth_code);
                }
                this.report_error(e);
            }
        }
    },

    process_common_messages : function(msg)
    {
        if (msg.type == SPICE_MSG_SET_ACK)
        {
            var ack = new SpiceMsgSetAck(msg.data);
            // FIXME - what to do with generation?
            this.ack_window = ack.window;
            DEBUG > 1 && console.log(this.type + ": set ack to " + ack.window);
            this.msgs_until_ack = this.ack_window;
            var ackack = new SpiceMsgcAckSync(ack);
            var reply = new SpiceMiniData();
            reply.build_msg(SPICE_MSGC_ACK_SYNC, ackack);
            this.send_msg(reply);
            return true;
        }

        if (msg.type == SPICE_MSG_PING)
        {
            DEBUG > 1 && console.log("ping!");
            var pong = new SpiceMiniData;
            pong.type = SPICE_MSGC_PONG;
            if (msg.data)
            {
                pong.data = msg.data.slice(0, 12);
            }
            pong.size = pong.buffer_size();
            this.send_msg(pong);
            return true;
        }

        if (msg.type == SPICE_MSG_NOTIFY)
        {
            // FIXME - Visibility + what
            var notify = new SpiceMsgNotify(msg.data);
            if (notify.severity == SPICE_NOTIFY_SEVERITY_ERROR)
                this.log_err(notify.message);
            else if (notify.severity == SPICE_NOTIFY_SEVERITY_WARN )
                this.log_warn(notify.message);
            else
                this.log_info(notify.message);
            return true;
        }

        return false;

    },

    process_message: function(msg)
    {
        var rc;
        DEBUG > 0 && console.log("<< hdr " + this.channel_type() + " type " + msg.type + " size " + (msg.data && msg.data.byteLength));
        rc = this.process_common_messages(msg);
        if (! rc)
        {
            if (this.process_channel_message)
            {
                rc = this.process_channel_message(msg);
                if (! rc)
                    this.log_warn(this.type + ": Unknown message type " + msg.type + "!");
            }
            else
                this.log_err(this.type + ": No message handlers for this channel; message " + msg.type);
        }

        if (this.msgs_until_ack !== undefined && this.ack_window)
        {
            this.msgs_until_ack--;
            if (this.msgs_until_ack <= 0)
            {
                this.msgs_until_ack = this.ack_window;
                var ack = new SpiceMiniData();
                ack.type = SPICE_MSGC_ACK;
                this.send_msg(ack);
                DEBUG > 1 && console.log(this.type + ": sent ack");
            }
        }

        return rc;
    },

    channel_type: function()
    {
        if (this.type == SPICE_CHANNEL_MAIN)
            return "main";
        else if (this.type == SPICE_CHANNEL_DISPLAY)
            return "display";
        else if (this.type == SPICE_CHANNEL_INPUTS)
            return "inputs";
        else if (this.type == SPICE_CHANNEL_CURSOR)
            return "cursor";
        return "unknown-" + this.type;

    },

    log_info: function()
    {
        var msg = Array.prototype.join.call(arguments, " ");
        console.log(msg);
        if (this.message_id)
        {
            var p = document.createElement("p");
            p.appendChild(document.createTextNode(msg));
            p.className += "spice-message-info";
            document.getElementById(this.message_id).appendChild(p);
        }
    },

    log_warn: function()
    {
        var msg = Array.prototype.join.call(arguments, " ");
        console.log("WARNING: " + msg);
        if (this.message_id)
        {
            var p = document.createElement("p");
            p.appendChild(document.createTextNode(msg));
            p.className += "spice-message-warning";
            document.getElementById(this.message_id).appendChild(p);
        }
    },

    log_err: function()
    {
        var msg = Array.prototype.join.call(arguments, " ");
        console.log("ERROR: " + msg);
        if (this.message_id)
        {
            var p = document.createElement("p");
            p.appendChild(document.createTextNode(msg));
            p.className += "spice-message-error";
            document.getElementById(this.message_id).appendChild(p);
        }
    },

    known_unimplemented: function(type, msg)
    {
        if ( (!this.warnings[type]) || DEBUG > 1)
        {
            var str = "";
            if (DEBUG <= 1)
                str = " [ further notices suppressed ]";
            this.log_warn("Unimplemented function " + type + "(" + msg + ")" + str);
            this.warnings[type] = true;
        }
    },

    report_error: function(e)
    {
        this.log_err(e.toString());
        if (this.onerror != undefined)
            this.onerror(e);
        else
            throw(e);
    },

    report_success: function(m)
    {
        if (this.onsuccess != undefined)
            this.onsuccess(m);
    },

    cleanup: function()
    {
        if (this.timeout)
        {
            window.clearTimeout(this.timeout);
            delete this.timeout;
        }
        if (this.ws)
        {
            this.ws.close();
            this.ws = undefined;
        }
    },

    handle_timeout: function()
    {
        var e = new Error("Connection timed out.");
        this.report_error(e);
    },
}

function spiceconn_timeout(sc)
{
    SpiceConn.prototype.handle_timeout.call(sc);
}
;
define("spice-spiceconn", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*----------------------------------------------------------------------------
**  SpiceArrayBufferSlice
**    This function is a work around for IE 10, which has no slice()
**    method in it's subclass.
**--------------------------------------------------------------------------*/
function SpiceArrayBufferSlice(start, end)
{
    start = start || 0;
    end = end || this.byteLength;
    if (end < 0)
        end = this.byteLength + end;
    if (start < 0)
        start = this.byteLength + start;
    if (start < 0)
        start = 0;
    if (end < 0)
        end = 0;
    if (end > this.byteLength)
        end = this.byteLength;
    if (start > end)
        start = end;

    var ret = new ArrayBuffer(end - start);
    var in1 = new Uint8Array(this, start, end - start);
    var out = new Uint8Array(ret);
    var i;

    for (i = 0; i < end - start; i++)
        out[i] = in1[i];

    return ret;
}

if (! ArrayBuffer.prototype.slice)
{
    ArrayBuffer.prototype.slice = SpiceArrayBufferSlice;
    console.log("WARNING:  ArrayBuffer.slice() is missing; we are extending ArrayBuffer to compensate");
}
;
define("spice-spicearraybuffer", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/


/*----------------------------------------------------------------------------
**  enums.js
**      'constants' for Spice
**--------------------------------------------------------------------------*/
var SPICE_MAGIC         = "REDQ";
var SPICE_VERSION_MAJOR = 2;
var SPICE_VERSION_MINOR = 2;

var SPICE_CONNECT_TIMEOUT = (30 * 1000);

var SPICE_COMMON_CAP_PROTOCOL_AUTH_SELECTION = 0;
var SPICE_COMMON_CAP_AUTH_SPICE              = 1;
var SPICE_COMMON_CAP_AUTH_SASL               = 2;
var SPICE_COMMON_CAP_MINI_HEADER             = 3;

var SPICE_TICKET_KEY_PAIR_LENGTH             = 1024;
var SPICE_TICKET_PUBKEY_BYTES                = (SPICE_TICKET_KEY_PAIR_LENGTH / 8 + 34);

var SPICE_LINK_ERR_OK                        = 0,
    SPICE_LINK_ERR_ERROR                     = 1,
    SPICE_LINK_ERR_INVALID_MAGIC             = 2,
    SPICE_LINK_ERR_INVALID_DATA              = 3,
    SPICE_LINK_ERR_VERSION_MISMATCH          = 4,
    SPICE_LINK_ERR_NEED_SECURED              = 5,
    SPICE_LINK_ERR_NEED_UNSECURED            = 6,
    SPICE_LINK_ERR_PERMISSION_DENIED         = 7,
    SPICE_LINK_ERR_BAD_CONNECTION_ID         = 8,
    SPICE_LINK_ERR_CHANNEL_NOT_AVAILABLE     = 9;

var SPICE_MSG_MIGRATE                   = 1;
var SPICE_MSG_MIGRATE_DATA              = 2;
var SPICE_MSG_SET_ACK                   = 3;
var SPICE_MSG_PING                      = 4;
var SPICE_MSG_WAIT_FOR_CHANNELS         = 5;
var SPICE_MSG_DISCONNECTING             = 6;
var SPICE_MSG_NOTIFY                    = 7;
var SPICE_MSG_LIST                      = 8;

var SPICE_MSG_MAIN_MIGRATE_BEGIN        = 101;
var SPICE_MSG_MAIN_MIGRATE_CANCEL       = 102;
var SPICE_MSG_MAIN_INIT                 = 103;
var SPICE_MSG_MAIN_CHANNELS_LIST        = 104;
var SPICE_MSG_MAIN_MOUSE_MODE           = 105;
var SPICE_MSG_MAIN_MULTI_MEDIA_TIME     = 106;
var SPICE_MSG_MAIN_AGENT_CONNECTED      = 107;
var SPICE_MSG_MAIN_AGENT_DISCONNECTED   = 108;
var SPICE_MSG_MAIN_AGENT_DATA           = 109;
var SPICE_MSG_MAIN_AGENT_TOKEN          = 110;
var SPICE_MSG_MAIN_MIGRATE_SWITCH_HOST  = 111;
var SPICE_MSG_MAIN_MIGRATE_END          = 112;
var SPICE_MSG_MAIN_NAME                 = 113;
var SPICE_MSG_MAIN_UUID                 = 114;
var SPICE_MSG_MAIN_AGENT_CONNECTED_TOKENS = 115;
var SPICE_MSG_MAIN_MIGRATE_BEGIN_SEAMLESS = 116;
var SPICE_MSG_MAIN_MIGRATE_DST_SEAMLESS_ACK = 117;
var SPICE_MSG_MAIN_MIGRATE_DST_SEAMLESS_NACK = 118;
var SPICE_MSG_END_MAIN                  = 119;



var SPICE_MSGC_ACK_SYNC                 = 1;
var SPICE_MSGC_ACK                      = 2;
var SPICE_MSGC_PONG                     = 3;
var SPICE_MSGC_MIGRATE_FLUSH_MARK       = 4;
var SPICE_MSGC_MIGRATE_DATA             = 5;
var SPICE_MSGC_DISCONNECTING            = 6;


var SPICE_MSGC_MAIN_CLIENT_INFO         = 101;
var SPICE_MSGC_MAIN_MIGRATE_CONNECTED   = 102;
var SPICE_MSGC_MAIN_MIGRATE_CONNECT_ERROR = 103;
var SPICE_MSGC_MAIN_ATTACH_CHANNELS     = 104;
var SPICE_MSGC_MAIN_MOUSE_MODE_REQUEST  = 105;
var SPICE_MSGC_MAIN_AGENT_START         = 106;
var SPICE_MSGC_MAIN_AGENT_DATA          = 107;
var SPICE_MSGC_MAIN_AGENT_TOKEN         = 108;
var SPICE_MSGC_MAIN_MIGRATE_END         = 109;
var SPICE_MSGC_END_MAIN                 = 110;

var SPICE_MSG_DISPLAY_MODE              = 101;
var SPICE_MSG_DISPLAY_MARK              = 102;
var SPICE_MSG_DISPLAY_RESET             = 103;
var SPICE_MSG_DISPLAY_COPY_BITS         = 104;
var SPICE_MSG_DISPLAY_INVAL_LIST        = 105;
var SPICE_MSG_DISPLAY_INVAL_ALL_PIXMAPS = 106;
var SPICE_MSG_DISPLAY_INVAL_PALETTE     = 107;
var SPICE_MSG_DISPLAY_INVAL_ALL_PALETTES= 108;

var SPICE_MSG_DISPLAY_STREAM_CREATE     = 122;
var SPICE_MSG_DISPLAY_STREAM_DATA       = 123;
var SPICE_MSG_DISPLAY_STREAM_CLIP       = 124;
var SPICE_MSG_DISPLAY_STREAM_DESTROY    = 125;
var SPICE_MSG_DISPLAY_STREAM_DESTROY_ALL= 126;

var SPICE_MSG_DISPLAY_DRAW_FILL         = 302;
var SPICE_MSG_DISPLAY_DRAW_OPAQUE       = 303;
var SPICE_MSG_DISPLAY_DRAW_COPY         = 304;
var SPICE_MSG_DISPLAY_DRAW_BLEND        = 305;
var SPICE_MSG_DISPLAY_DRAW_BLACKNESS    = 306;
var SPICE_MSG_DISPLAY_DRAW_WHITENESS    = 307;
var SPICE_MSG_DISPLAY_DRAW_INVERS       = 308;
var SPICE_MSG_DISPLAY_DRAW_ROP3         = 309;
var SPICE_MSG_DISPLAY_DRAW_STROKE       = 310;
var SPICE_MSG_DISPLAY_DRAW_TEXT         = 311;
var SPICE_MSG_DISPLAY_DRAW_TRANSPARENT  = 312;
var SPICE_MSG_DISPLAY_DRAW_ALPHA_BLEND  = 313;
var SPICE_MSG_DISPLAY_SURFACE_CREATE    = 314;
var SPICE_MSG_DISPLAY_SURFACE_DESTROY   = 315;

var SPICE_MSGC_DISPLAY_INIT             = 101;

var SPICE_MSG_INPUTS_INIT               = 101;
var SPICE_MSG_INPUTS_KEY_MODIFIERS      = 102;

var SPICE_MSG_INPUTS_MOUSE_MOTION_ACK   = 111;

var SPICE_MSGC_INPUTS_KEY_DOWN          = 101;
var SPICE_MSGC_INPUTS_KEY_UP            = 102;
var SPICE_MSGC_INPUTS_KEY_MODIFIERS     = 103;

var SPICE_MSGC_INPUTS_MOUSE_MOTION      = 111;
var SPICE_MSGC_INPUTS_MOUSE_POSITION    = 112;
var SPICE_MSGC_INPUTS_MOUSE_PRESS       = 113;
var SPICE_MSGC_INPUTS_MOUSE_RELEASE     = 114;

var SPICE_MSG_CURSOR_INIT               = 101;
var SPICE_MSG_CURSOR_RESET              = 102;
var SPICE_MSG_CURSOR_SET                = 103;
var SPICE_MSG_CURSOR_MOVE               = 104;
var SPICE_MSG_CURSOR_HIDE               = 105;
var SPICE_MSG_CURSOR_TRAIL              = 106;
var SPICE_MSG_CURSOR_INVAL_ONE          = 107;
var SPICE_MSG_CURSOR_INVAL_ALL          = 108;

var SPICE_MSG_PLAYBACK_DATA             = 101;
var SPICE_MSG_PLAYBACK_MODE             = 102;
var SPICE_MSG_PLAYBACK_START            = 103;
var SPICE_MSG_PLAYBACK_STOP             = 104;
var SPICE_MSG_PLAYBACK_VOLUME           = 105;
var SPICE_MSG_PLAYBACK_MUTE             = 106;
var SPICE_MSG_PLAYBACK_LATENCY          = 107;

var SPICE_PLAYBACK_CAP_CELT_0_5_1       = 0;
var SPICE_PLAYBACK_CAP_VOLUME           = 1;
var SPICE_PLAYBACK_CAP_LATENCY          = 2;
var SPICE_PLAYBACK_CAP_OPUS             = 3;

var SPICE_MAIN_CAP_SEMI_SEAMLESS_MIGRATE  = 0;
var SPICE_MAIN_CAP_NAME_AND_UUID          = 1;
var SPICE_MAIN_CAP_AGENT_CONNECTED_TOKENS = 2;
var SPICE_MAIN_CAP_SEAMLESS_MIGRATE       = 3;

var SPICE_AUDIO_DATA_MODE_INVALID       = 0;
var SPICE_AUDIO_DATA_MODE_RAW           = 1;
var SPICE_AUDIO_DATA_MODE_CELT_0_5_1    = 2;
var SPICE_AUDIO_DATA_MODE_OPUS          = 3;

var SPICE_AUDIO_FMT_INVALID             = 0;
var SPICE_AUDIO_FMT_S16                 = 1;

var SPICE_CHANNEL_MAIN                  = 1;
var SPICE_CHANNEL_DISPLAY               = 2;
var SPICE_CHANNEL_INPUTS                = 3;
var SPICE_CHANNEL_CURSOR                = 4;
var SPICE_CHANNEL_PLAYBACK              = 5;
var SPICE_CHANNEL_RECORD                = 6;
var SPICE_CHANNEL_TUNNEL                = 7;
var SPICE_CHANNEL_SMARTCARD             = 8;
var SPICE_CHANNEL_USBREDIR              = 9;

var SPICE_SURFACE_FLAGS_PRIMARY = (1 << 0);

var SPICE_NOTIFY_SEVERITY_INFO  = 0;
var SPICE_NOTIFY_SEVERITY_WARN  = 1;
var SPICE_NOTIFY_SEVERITY_ERROR = 2;

var SPICE_MOUSE_MODE_SERVER = (1 << 0),
    SPICE_MOUSE_MODE_CLIENT = (1 << 1),
    SPICE_MOUSE_MODE_MASK = 0x3;

var SPICE_CLIP_TYPE_NONE            = 0;
var SPICE_CLIP_TYPE_RECTS           = 1;

var SPICE_IMAGE_TYPE_BITMAP         = 0;
var SPICE_IMAGE_TYPE_QUIC           = 1;
var SPICE_IMAGE_TYPE_RESERVED       = 2;
var SPICE_IMAGE_TYPE_LZ_PLT         = 100;
var SPICE_IMAGE_TYPE_LZ_RGB         = 101;
var SPICE_IMAGE_TYPE_GLZ_RGB        = 102;
var SPICE_IMAGE_TYPE_FROM_CACHE     = 103;
var SPICE_IMAGE_TYPE_SURFACE        = 104;
var SPICE_IMAGE_TYPE_JPEG           = 105;
var SPICE_IMAGE_TYPE_FROM_CACHE_LOSSLESS = 106;
var SPICE_IMAGE_TYPE_ZLIB_GLZ_RGB   = 107;
var SPICE_IMAGE_TYPE_JPEG_ALPHA     = 108;

var SPICE_IMAGE_FLAGS_CACHE_ME = (1 << 0),
    SPICE_IMAGE_FLAGS_HIGH_BITS_SET = (1 << 1),
    SPICE_IMAGE_FLAGS_CACHE_REPLACE_ME = (1 << 2);

var SPICE_BITMAP_FLAGS_PAL_CACHE_ME = (1 << 0),
    SPICE_BITMAP_FLAGS_PAL_FROM_CACHE = (1 << 1),
    SPICE_BITMAP_FLAGS_TOP_DOWN = (1 << 2),
    SPICE_BITMAP_FLAGS_MASK = 0x7;

var SPICE_BITMAP_FMT_INVALID        = 0,
    SPICE_BITMAP_FMT_1BIT_LE        = 1,
    SPICE_BITMAP_FMT_1BIT_BE        = 2,
    SPICE_BITMAP_FMT_4BIT_LE        = 3,
    SPICE_BITMAP_FMT_4BIT_BE        = 4,
    SPICE_BITMAP_FMT_8BIT           = 5,
    SPICE_BITMAP_FMT_16BIT          = 6,
    SPICE_BITMAP_FMT_24BIT          = 7,
    SPICE_BITMAP_FMT_32BIT          = 8,
    SPICE_BITMAP_FMT_RGBA           = 9;


var SPICE_CURSOR_FLAGS_NONE = (1 << 0),
    SPICE_CURSOR_FLAGS_CACHE_ME = (1 << 1),
    SPICE_CURSOR_FLAGS_FROM_CACHE = (1 << 2),
    SPICE_CURSOR_FLAGS_MASK = 0x7;

var SPICE_MOUSE_BUTTON_MASK_LEFT = (1 << 0),
    SPICE_MOUSE_BUTTON_MASK_MIDDLE = (1 << 1),
    SPICE_MOUSE_BUTTON_MASK_RIGHT = (1 << 2),
    SPICE_MOUSE_BUTTON_MASK_MASK = 0x7;
    
var SPICE_MOUSE_BUTTON_INVALID  = 0;
var SPICE_MOUSE_BUTTON_LEFT     = 1;
var SPICE_MOUSE_BUTTON_MIDDLE   = 2;
var SPICE_MOUSE_BUTTON_RIGHT    = 3;
var SPICE_MOUSE_BUTTON_UP       = 4;
var SPICE_MOUSE_BUTTON_DOWN     = 5;

var SPICE_BRUSH_TYPE_NONE = 0,
    SPICE_BRUSH_TYPE_SOLID = 1,
    SPICE_BRUSH_TYPE_PATTERN = 2;

var SPICE_SURFACE_FMT_INVALID = 0,
    SPICE_SURFACE_FMT_1_A = 1,
    SPICE_SURFACE_FMT_8_A = 8,
    SPICE_SURFACE_FMT_16_555 = 16,
    SPICE_SURFACE_FMT_32_xRGB = 32,
    SPICE_SURFACE_FMT_16_565 = 80,
    SPICE_SURFACE_FMT_32_ARGB = 96;

var SPICE_ROPD_INVERS_SRC = (1 << 0),
    SPICE_ROPD_INVERS_BRUSH = (1 << 1),
    SPICE_ROPD_INVERS_DEST = (1 << 2),
    SPICE_ROPD_OP_PUT = (1 << 3),
    SPICE_ROPD_OP_OR = (1 << 4),
    SPICE_ROPD_OP_AND = (1 << 5),
    SPICE_ROPD_OP_XOR = (1 << 6),
    SPICE_ROPD_OP_BLACKNESS = (1 << 7),
    SPICE_ROPD_OP_WHITENESS = (1 << 8),
    SPICE_ROPD_OP_INVERS = (1 << 9),
    SPICE_ROPD_INVERS_RES = (1 << 10),
    SPICE_ROPD_MASK = 0x7ff;

var LZ_IMAGE_TYPE_INVALID = 0,
    LZ_IMAGE_TYPE_PLT1_LE = 1,
    LZ_IMAGE_TYPE_PLT1_BE = 2,      // PLT stands for palette
    LZ_IMAGE_TYPE_PLT4_LE = 3,
    LZ_IMAGE_TYPE_PLT4_BE = 4,
    LZ_IMAGE_TYPE_PLT8    = 5,
    LZ_IMAGE_TYPE_RGB16   = 6,
    LZ_IMAGE_TYPE_RGB24   = 7,
    LZ_IMAGE_TYPE_RGB32   = 8,
    LZ_IMAGE_TYPE_RGBA    = 9,
    LZ_IMAGE_TYPE_XXXA    = 10;


var QUIC_IMAGE_TYPE_INVALID = 0,
    QUIC_IMAGE_TYPE_GRAY    = 1,
    QUIC_IMAGE_TYPE_RGB16   = 2,
    QUIC_IMAGE_TYPE_RGB24   = 3,
    QUIC_IMAGE_TYPE_RGB32   = 4,
    QUIC_IMAGE_TYPE_RGBA    = 5;

var SPICE_INPUT_MOTION_ACK_BUNCH = 4;


var SPICE_CURSOR_TYPE_ALPHA     = 0,
    SPICE_CURSOR_TYPE_MONO      = 1,
    SPICE_CURSOR_TYPE_COLOR4    = 2,
    SPICE_CURSOR_TYPE_COLOR8    = 3,
    SPICE_CURSOR_TYPE_COLOR16   = 4,
    SPICE_CURSOR_TYPE_COLOR24   = 5,
    SPICE_CURSOR_TYPE_COLOR32   = 6;

var SPICE_VIDEO_CODEC_TYPE_MJPEG = 1;

var VD_AGENT_PROTOCOL = 1;
var VD_AGENT_MAX_DATA_SIZE = 2048;

var VD_AGENT_MOUSE_STATE            = 1,
    VD_AGENT_MONITORS_CONFIG        = 2,
    VD_AGENT_REPLY                  = 3,
    VD_AGENT_CLIPBOARD              = 4,
    VD_AGENT_DISPLAY_CONFIG         = 5,
    VD_AGENT_ANNOUNCE_CAPABILITIES  = 6,
    VD_AGENT_CLIPBOARD_GRAB         = 7,
    VD_AGENT_CLIPBOARD_REQUEST      = 8,
    VD_AGENT_CLIPBOARD_RELEASE      = 9,
    VD_AGENT_FILE_XFER_START        =10,
    VD_AGENT_FILE_XFER_STATUS       =11,
    VD_AGENT_FILE_XFER_DATA         =12,
    VD_AGENT_CLIENT_DISCONNECTED    =13,
    VD_AGENT_MAX_CLIPBOARD          =14;

var VD_AGENT_CAP_MOUSE_STATE            = 0,
    VD_AGENT_CAP_MONITORS_CONFIG        = 1,
    VD_AGENT_CAP_REPLY                  = 2,
    VD_AGENT_CAP_CLIPBOARD              = 3,
    VD_AGENT_CAP_DISPLAY_CONFIG         = 4,
    VD_AGENT_CAP_CLIPBOARD_BY_DEMAND    = 5,
    VD_AGENT_CAP_CLIPBOARD_SELECTION    = 6,
    VD_AGENT_CAP_SPARSE_MONITORS_CONFIG = 7,
    VD_AGENT_CAP_GUEST_LINEEND_LF       = 8,
    VD_AGENT_CAP_GUEST_LINEEND_CRLF     = 9,
    VD_AGENT_CAP_MAX_CLIPBOARD          = 10,
    VD_AGENT_END_CAP                    = 11;

var VD_AGENT_FILE_XFER_STATUS_CAN_SEND_DATA = 0,
    VD_AGENT_FILE_XFER_STATUS_CANCELLED     = 1,
    VD_AGENT_FILE_XFER_STATUS_ERROR         = 2,
    VD_AGENT_FILE_XFER_STATUS_SUCCESS       = 3;

define("spice-enums", function(){});


/*
   Copyright (C) 2012 by Aric Stewart <aric@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Copyright 1990,91 by Thomas Roell, Dinkelscherben, Germany.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Thomas Roell not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Thomas Roell makes no representations
 * about the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * THOMAS ROELL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THOMAS ROELL BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */
/*
 * Copyright (c) 1994-2003 by The XFree86 Project, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER(S) OR AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of the copyright holder(s)
 * and author(s) shall not be used in advertising or otherwise to promote
 * the sale, use or other dealings in this Software without prior written
 * authorization from the copyright holder(s) and author(s).
 */

/*
 * NOTE: The AT/MF keyboards can generate (via the 8042) two (MF: three)
 *       sets of scancodes. Set3 can only be generated by a MF keyboard.
 *       Set2 sends a makecode for keypress, and the same code prefixed by a
 *       F0 for keyrelease. This is a little bit ugly to handle. Thus we use
 *       here for X386 the PC/XT compatible Set1. This set uses 8bit scancodes.
 *       Bit 7 ist set if the key is released. The code E0 switches to a
 *       different meaning to add the new MF cursorkeys, while not breaking old
 *       applications. E1 is another special prefix. Since I assume that there
 *       will be further versions of PC/XT scancode compatible keyboards, we
 *       may be in trouble one day.
 *
 * IDEA: 1) Use Set2 on AT84 keyboards and translate it to MF Set3.
 *       2) Use the keyboards native set and translate it to common keysyms.
 */

/*
 * definition of the AT84/MF101/MF102 Keyboard:
 * ============================================================
 *       Defined             Key Cap Glyphs       Pressed value
 *      Key Name            Main       Also       (hex)    (dec)
 *      ----------------   ---------- -------    ------    ------
 */

var KEY_Escape      =/* Escape                0x01  */    1
var KEY_1           =/* 1           !         0x02  */    2
var KEY_2           =/* 2           @         0x03  */    3
var KEY_3           =/* 3           #         0x04  */    4
var KEY_4           =/* 4           $         0x05  */    5
var KEY_5           =/* 5           %         0x06  */    6
var KEY_6           =/* 6           ^         0x07  */    7
var KEY_7           =/* 7           &         0x08  */    8
var KEY_8           =/* 8           *         0x09  */    9
var KEY_9           =/* 9           (         0x0a  */   10
var KEY_0           =/* 0           )         0x0b  */   11
var KEY_Minus       =/* - (Minus)   _ (Under) 0x0c  */   12
var KEY_Equal       =/* = (Equal)   +         0x0d  */   13
var KEY_BackSpace   =/* Back Space            0x0e  */   14
var KEY_Tab         =/* Tab                   0x0f  */   15
var KEY_Q           =/* Q                     0x10  */   16
var KEY_W           =/* W                     0x11  */   17
var KEY_E           =/* E                     0x12  */   18
var KEY_R           =/* R                     0x13  */   19
var KEY_T           =/* T                     0x14  */   20
var KEY_Y           =/* Y                     0x15  */   21
var KEY_U           =/* U                     0x16  */   22
var KEY_I           =/* I                     0x17  */   23
var KEY_O           =/* O                     0x18  */   24
var KEY_P           =/* P                     0x19  */   25
var KEY_LBrace      =/* [           {         0x1a  */   26
var KEY_RBrace      =/* ]           }         0x1b  */   27
var KEY_Enter       =/* Enter                 0x1c  */   28
var KEY_LCtrl       =/* Ctrl(left)            0x1d  */   29
var KEY_A           =/* A                     0x1e  */   30
var KEY_S           =/* S                     0x1f  */   31
var KEY_D           =/* D                     0x20  */   32
var KEY_F           =/* F                     0x21  */   33
var KEY_G           =/* G                     0x22  */   34
var KEY_H           =/* H                     0x23  */   35
var KEY_J           =/* J                     0x24  */   36
var KEY_K           =/* K                     0x25  */   37
var KEY_L           =/* L                     0x26  */   38
var KEY_SemiColon   =/* ;(SemiColon) :(Colon) 0x27  */   39
var KEY_Quote       =/* ' (Apostr)  " (Quote) 0x28  */   40
var KEY_Tilde       =/* ` (Accent)  ~ (Tilde) 0x29  */   41
var KEY_ShiftL      =/* Shift(left)           0x2a  */   42
var KEY_BSlash      =/* \(BckSlash) |(VertBar)0x2b  */   43
var KEY_Z           =/* Z                     0x2c  */   44
var KEY_X           =/* X                     0x2d  */   45
var KEY_C           =/* C                     0x2e  */   46
var KEY_V           =/* V                     0x2f  */   47
var KEY_B           =/* B                     0x30  */   48
var KEY_N           =/* N                     0x31  */   49
var KEY_M           =/* M                     0x32  */   50
var KEY_Comma       =/* , (Comma)   < (Less)  0x33  */   51
var KEY_Period      =/* . (Period)  >(Greater)0x34  */   52
var KEY_Slash       =/* / (Slash)   ?         0x35  */   53
var KEY_ShiftR      =/* Shift(right)          0x36  */   54
var KEY_KP_Multiply =/* *                     0x37  */   55
var KEY_Alt         =/* Alt(left)             0x38  */   56
var KEY_Space       =/*   (SpaceBar)          0x39  */   57
var KEY_CapsLock    =/* CapsLock              0x3a  */   58
var KEY_F1          =/* F1                    0x3b  */   59
var KEY_F2          =/* F2                    0x3c  */   60
var KEY_F3          =/* F3                    0x3d  */   61
var KEY_F4          =/* F4                    0x3e  */   62
var KEY_F5          =/* F5                    0x3f  */   63
var KEY_F6          =/* F6                    0x40  */   64
var KEY_F7          =/* F7                    0x41  */   65
var KEY_F8          =/* F8                    0x42  */   66
var KEY_F9          =/* F9                    0x43  */   67
var KEY_F10         =/* F10                   0x44  */   68
var KEY_NumLock     =/* NumLock               0x45  */   69
var KEY_ScrollLock  =/* ScrollLock            0x46  */   70
var KEY_KP_7        =/* 7           Home      0x47  */   71
var KEY_KP_8        =/* 8           Up        0x48  */   72
var KEY_KP_9        =/* 9           PgUp      0x49  */   73
var KEY_KP_Minus    =/* - (Minus)             0x4a  */   74
var KEY_KP_4        =/* 4           Left      0x4b  */   75
var KEY_KP_5        =/* 5                     0x4c  */   76
var KEY_KP_6        =/* 6           Right     0x4d  */   77
var KEY_KP_Plus     =/* + (Plus)              0x4e  */   78
var KEY_KP_1        =/* 1           End       0x4f  */   79
var KEY_KP_2        =/* 2           Down      0x50  */   80
var KEY_KP_3        =/* 3           PgDown    0x51  */   81
var KEY_KP_0        =/* 0           Insert    0x52  */   82
var KEY_KP_Decimal  =/* . (Decimal) Delete    0x53  */   83
var KEY_SysReqest   =/* SysReqest             0x54  */   84
                         /* NOTUSED               0x55  */
var KEY_Less        =/* < (Less)   >(Greater) 0x56  */   86
var KEY_F11         =/* F11                   0x57  */   87
var KEY_F12         =/* F12                   0x58  */   88

var KEY_Prefix0     =/* special               0x60  */   96
var KEY_Prefix1     =/* specail               0x61  */   97
;
define("spice-atKeynames", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*----------------------------------------------------------------------------
**  Utility settings and functions for Spice
**--------------------------------------------------------------------------*/
var DEBUG = 0;
var DUMP_DRAWS = false;
var DUMP_CANVASES = false;


/*----------------------------------------------------------------------------
**  combine_array_buffers
**      Combine two array buffers.
**      FIXME - this can't be optimal.  See wire.js about eliminating the need.
**--------------------------------------------------------------------------*/
function combine_array_buffers(a1, a2)
{
    var in1 = new Uint8Array(a1);
    var in2 = new Uint8Array(a2);
    var ret = new ArrayBuffer(a1.byteLength + a2.byteLength);
    var out = new Uint8Array(ret);
    var o = 0;
    var i;
    for (i = 0; i < in1.length; i++)
        out[o++] = in1[i];
    for (i = 0; i < in2.length; i++)
        out[o++] = in2[i];

    return ret;
}

/*----------------------------------------------------------------------------
**  hexdump_buffer
**--------------------------------------------------------------------------*/
function hexdump_buffer(a)
{
    var mg = new Uint8Array(a);
    var hex = "";
    var str = "";
    var last_zeros = 0;
    for (var i = 0; i < mg.length; i++)
    {
        var h = Number(mg[i]).toString(16);
        if (h.length == 1)
            hex += "0";
        hex += h + " ";

        if (mg[i] == 10 || mg[i] == 13 || mg[i] == 8)
            str += ".";
        else
            str += String.fromCharCode(mg[i]);

        if ((i % 16 == 15) || (i == (mg.length - 1)))
        {
            while (i % 16 != 15)
            {
                hex += "   ";
                i++;
            }

            if (last_zeros == 0)
                console.log(hex + " | " + str);

            if (hex == "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 ")
            {
                if (last_zeros == 1)
                {
                    console.log(".");
                    last_zeros++;
                }
                else if (last_zeros == 0)
                    last_zeros++;
            }
            else
                last_zeros = 0;

            hex = str = "";
        }
    }
}

/*----------------------------------------------------------------------------
** Converting keycodes to AT scancodes is very hard.
** luckly there are some resources on the web and in the Xorg driver that help
** us figure out what browser depenend keycodes match to what scancodes.
**
** This will most likely not work for non US keyboard and browsers other than
** modern Chrome and FireFox.
**--------------------------------------------------------------------------*/
var common_scanmap = [];
common_scanmap['Q'.charCodeAt(0)]  = KEY_Q;
common_scanmap['W'.charCodeAt(0)]  = KEY_W;
common_scanmap['E'.charCodeAt(0)]  = KEY_E;
common_scanmap['R'.charCodeAt(0)]  = KEY_R;
common_scanmap['T'.charCodeAt(0)]  = KEY_T;
common_scanmap['Y'.charCodeAt(0)]  = KEY_Y;
common_scanmap['U'.charCodeAt(0)]  = KEY_U;
common_scanmap['I'.charCodeAt(0)]  = KEY_I;
common_scanmap['O'.charCodeAt(0)]  = KEY_O;
common_scanmap['P'.charCodeAt(0)]  = KEY_P;
common_scanmap['A'.charCodeAt(0)]  = KEY_A;
common_scanmap['S'.charCodeAt(0)]  = KEY_S;
common_scanmap['D'.charCodeAt(0)]  = KEY_D;
common_scanmap['F'.charCodeAt(0)]  = KEY_F;
common_scanmap['G'.charCodeAt(0)]  = KEY_G;
common_scanmap['H'.charCodeAt(0)]  = KEY_H;
common_scanmap['J'.charCodeAt(0)]  = KEY_J;
common_scanmap['K'.charCodeAt(0)]  = KEY_K;
common_scanmap['L'.charCodeAt(0)]  = KEY_L;
common_scanmap['Z'.charCodeAt(0)]  = KEY_Z;
common_scanmap['X'.charCodeAt(0)]  = KEY_X;
common_scanmap['C'.charCodeAt(0)]  = KEY_C;
common_scanmap['V'.charCodeAt(0)]  = KEY_V;
common_scanmap['B'.charCodeAt(0)]  = KEY_B;
common_scanmap['N'.charCodeAt(0)]  = KEY_N;
common_scanmap['M'.charCodeAt(0)]  = KEY_M;
common_scanmap[' '.charCodeAt(0)]  = KEY_Space;
common_scanmap[13]                 = KEY_Enter;
common_scanmap[27]                 = KEY_Escape;
common_scanmap[8]                  = KEY_BackSpace;
common_scanmap[9]                  = KEY_Tab;
common_scanmap[16]                 = KEY_ShiftL;
common_scanmap[17]                 = KEY_LCtrl;
common_scanmap[18]                 = KEY_Alt;
common_scanmap[20]                 = KEY_CapsLock;
common_scanmap[144]                = KEY_NumLock;
common_scanmap[112]                = KEY_F1;
common_scanmap[113]                = KEY_F2;
common_scanmap[114]                = KEY_F3;
common_scanmap[115]                = KEY_F4;
common_scanmap[116]                = KEY_F5;
common_scanmap[117]                = KEY_F6;
common_scanmap[118]                = KEY_F7;
common_scanmap[119]                = KEY_F8;
common_scanmap[120]                = KEY_F9;
common_scanmap[121]                = KEY_F10;
common_scanmap[122]                = KEY_F11;
common_scanmap[123]                = KEY_F12;

/* These externded scancodes do not line up with values from atKeynames */
common_scanmap[42]                 = 99;
common_scanmap[19]                 = 101;    // Break
common_scanmap[111]                = 0xE035; // KP_Divide
common_scanmap[106]                = 0xE037; // KP_Multiply
common_scanmap[36]                 = 0xE047; // Home
common_scanmap[38]                 = 0xE048; // Up
common_scanmap[33]                 = 0xE049; // PgUp
common_scanmap[37]                 = 0xE04B; // Left
common_scanmap[39]                 = 0xE04D; // Right
common_scanmap[35]                 = 0xE04F; // End
common_scanmap[40]                 = 0xE050; // Down
common_scanmap[34]                 = 0xE051; // PgDown
common_scanmap[45]                 = 0xE052; // Insert
common_scanmap[46]                 = 0xE053; // Delete
common_scanmap[44]                 = 0x2A37; // Print

/* These are not common between ALL browsers but are between Firefox and DOM3 */
common_scanmap['1'.charCodeAt(0)]  = KEY_1;
common_scanmap['2'.charCodeAt(0)]  = KEY_2;
common_scanmap['3'.charCodeAt(0)]  = KEY_3;
common_scanmap['4'.charCodeAt(0)]  = KEY_4;
common_scanmap['5'.charCodeAt(0)]  = KEY_5;
common_scanmap['6'.charCodeAt(0)]  = KEY_6;
common_scanmap['7'.charCodeAt(0)]  = KEY_7;
common_scanmap['8'.charCodeAt(0)]  = KEY_8;
common_scanmap['9'.charCodeAt(0)]  = KEY_9;
common_scanmap['0'.charCodeAt(0)]  = KEY_0;
common_scanmap[145]                = KEY_ScrollLock;
common_scanmap[103]                = KEY_KP_7;
common_scanmap[104]                = KEY_KP_8;
common_scanmap[105]                = KEY_KP_9;
common_scanmap[100]                = KEY_KP_4;
common_scanmap[101]                = KEY_KP_5;
common_scanmap[102]                = KEY_KP_6;
common_scanmap[107]                = KEY_KP_Plus;
common_scanmap[97]                 = KEY_KP_1;
common_scanmap[98]                 = KEY_KP_2;
common_scanmap[99]                 = KEY_KP_3;
common_scanmap[96]                 = KEY_KP_0;
common_scanmap[110]                = KEY_KP_Decimal;
common_scanmap[191]                = KEY_Slash;
common_scanmap[190]                = KEY_Period;
common_scanmap[188]                = KEY_Comma;
common_scanmap[220]                = KEY_BSlash;
common_scanmap[192]                = KEY_Tilde;
common_scanmap[222]                = KEY_Quote;
common_scanmap[219]                = KEY_LBrace;
common_scanmap[221]                = KEY_RBrace;

common_scanmap[91]                 = 0xE05B; //KEY_LMeta
common_scanmap[92]                 = 0xE05C; //KEY_RMeta
common_scanmap[93]                 = 0xE05D; //KEY_Menu

/* Firefox/Mozilla codes */
var firefox_scanmap = [];
firefox_scanmap[173]                = KEY_Minus;
firefox_scanmap[109]                = KEY_Minus;
firefox_scanmap[61]                 = KEY_Equal;
firefox_scanmap[59]                 = KEY_SemiColon;

/* DOM3 codes */
var DOM_scanmap = [];
DOM_scanmap[189]                = KEY_Minus;
DOM_scanmap[187]                = KEY_Equal;
DOM_scanmap[186]                = KEY_SemiColon;

function get_scancode(code)
{
    if (common_scanmap[code] === undefined)
    {
        if (navigator.userAgent.indexOf("Firefox") != -1)
            return firefox_scanmap[code];
        else
            return DOM_scanmap[code];
    }
    else
        return common_scanmap[code];
}

function keycode_to_start_scan(code)
{
    var scancode = get_scancode(code);
    if (scancode === undefined)
    {
        alert('no map for ' + code);
        return 0;
    }

    if (scancode < 0x100) {
        return scancode;
    } else {
        return 0xe0 | ((scancode - 0x100) << 8);
    }
}

function keycode_to_end_scan(code)
{
    var scancode = get_scancode(code);
    if (scancode === undefined)
        return 0;

    if (scancode < 0x100) {
        return scancode | 0x80;
    } else {
        return 0x80e0 | ((scancode - 0x100) << 8);
    }
}
;
define("spice-utils", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*----------------------------------------------------------------------------
**  crc logic from rfc2083 ported to Javascript
**--------------------------------------------------------------------------*/

var rfc2083_crc_table = Array(256);
var rfc2083_crc_table_computed = 0;
/* Make the table for a fast CRC. */
function rfc2083_make_crc_table()
{
    var c;
    var n, k;
    for (n = 0; n < 256; n++)
    {
        c = n;
        for (k = 0; k < 8; k++)
        {
            if (c & 1)
                c = ((0xedb88320 ^ (c >>> 1)) >>> 0) & 0xffffffff;
            else
                c = c >>> 1;
        }
        rfc2083_crc_table[n] = c;
    }

    rfc2083_crc_table_computed = 1;
}

/* Update a running CRC with the bytes buf[0..len-1]--the CRC
     should be initialized to all 1's, and the transmitted value
     is the 1's complement of the final running CRC (see the
     crc() routine below)). */

function rfc2083_update_crc(crc, u8buf, at, len)
{
    var c = crc;
    var n;

    if (!rfc2083_crc_table_computed)
        rfc2083_make_crc_table();

    for (n = 0; n < len; n++)
    {
        c = rfc2083_crc_table[(c ^ u8buf[at + n]) & 0xff] ^ (c >>> 8);
    }

    return c;
}

function rfc2083_crc(u8buf, at, len)
{
    return rfc2083_update_crc(0xffffffff, u8buf, at, len) ^ 0xffffffff;
}

function crc32(mb, at, len)
{
    var u8 = new Uint8Array(mb);
    return rfc2083_crc(u8, at, len);
}

function PngIHDR(width, height)
{
    this.width = width;
    this.height = height;
    this.depth = 8;
    this.type = 6;
    this.compression = 0;
    this.filter = 0;
    this.interlace = 0;
}

PngIHDR.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var orig = at;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.buffer_size() - 12); at += 4;
        dv.setUint8(at, 'I'.charCodeAt(0)); at++;
        dv.setUint8(at, 'H'.charCodeAt(0)); at++;
        dv.setUint8(at, 'D'.charCodeAt(0)); at++;
        dv.setUint8(at, 'R'.charCodeAt(0)); at++;
        dv.setUint32(at, this.width); at += 4;
        dv.setUint32(at, this.height); at += 4;
        dv.setUint8(at, this.depth); at++;
        dv.setUint8(at, this.type); at++;
        dv.setUint8(at, this.compression); at++;
        dv.setUint8(at, this.filter); at++;
        dv.setUint8(at, this.interlace); at++;
        dv.setUint32(at, crc32(a, orig + 4, this.buffer_size() - 8)); at += 4;
        return at;
    },
    buffer_size: function()
    {
        return 12 + 13;
    }
}


function adler()
{
    this.s1 = 1;
    this.s2 = 0;
}

adler.prototype.update = function(b)
{
    this.s1 += b;
    this.s1 %= 65521;
    this.s2 += this.s1;
    this.s2 %= 65521;
}

function PngIDAT(width, height, bytes)
{
    if (bytes.byteLength > 65535)
    {
        throw new Error("Cannot handle more than 64K");
    }
    this.data = bytes;
    this.width = width;
    this.height = height;
}

PngIDAT.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var orig = at;
        var x, y, i, j;
        var dv = new SpiceDataView(a);
        var zsum = new adler();
        dv.setUint32(at, this.buffer_size() - 12); at += 4;
        dv.setUint8(at, 'I'.charCodeAt(0)); at++;
        dv.setUint8(at, 'D'.charCodeAt(0)); at++;
        dv.setUint8(at, 'A'.charCodeAt(0)); at++;
        dv.setUint8(at, 'T'.charCodeAt(0)); at++;

        /* zlib header.  */
        dv.setUint8(at, 0x78); at++;
        dv.setUint8(at, 0x01); at++;

        /* Deflate header.  Specifies uncompressed, final bit */
        dv.setUint8(at, 0x80); at++;
        dv.setUint16(at, this.data.byteLength + this.height); at += 2;
        dv.setUint16(at, ~(this.data.byteLength + this.height)); at += 2;
        var u8 = new Uint8Array(this.data);
        for (i = 0, y = 0; y < this.height; y++)
        {
            /* Filter type 0 - uncompressed */
            dv.setUint8(at, 0); at++;
            zsum.update(0);
            for (x = 0; x < this.width && i < this.data.byteLength; x++)
            {
                zsum.update(u8[i]);
                dv.setUint8(at, u8[i++]); at++;
                zsum.update(u8[i]);
                dv.setUint8(at, u8[i++]); at++;
                zsum.update(u8[i]);
                dv.setUint8(at, u8[i++]); at++;
                zsum.update(u8[i]);
                dv.setUint8(at, u8[i++]); at++;
            }
        }

        /* zlib checksum.   */
        dv.setUint16(at, zsum.s2); at+=2;
        dv.setUint16(at, zsum.s1); at+=2;

        /* FIXME - something is not quite right with the zlib code;
                   you get an error from libpng if you open the image in
                   gimp.  But it works, so it's good enough for now... */

        dv.setUint32(at, crc32(a, orig + 4, this.buffer_size() - 8)); at += 4;
        return at;
    },
    buffer_size: function()
    {
        return 12 + this.data.byteLength + this.height + 4 + 2 + 1 + 2 + 2;
    }
}


function PngIEND()
{
}

PngIEND.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var orig = at;
        var i;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.buffer_size() - 12); at += 4;
        dv.setUint8(at, 'I'.charCodeAt(0)); at++;
        dv.setUint8(at, 'E'.charCodeAt(0)); at++;
        dv.setUint8(at, 'N'.charCodeAt(0)); at++;
        dv.setUint8(at, 'D'.charCodeAt(0)); at++;
        dv.setUint32(at, crc32(a, orig + 4, this.buffer_size() - 8)); at += 4;
        return at;
    },
    buffer_size: function()
    {
        return 12;
    }
}


function create_rgba_png(width, height, bytes)
{
    var i;
    var ihdr = new PngIHDR(width, height);
    var idat = new PngIDAT(width, height, bytes);
    var iend = new PngIEND;

    var mb = new ArrayBuffer(ihdr.buffer_size() + idat.buffer_size() + iend.buffer_size());
    var at = ihdr.to_buffer(mb);
    at = idat.to_buffer(mb, at);
    at = iend.to_buffer(mb, at);

    var u8 = new Uint8Array(mb);
    var str = "";
    for (i = 0; i < at; i++)
    {
        str += "%";
        if (u8[i] < 16)
            str += "0";
        str += u8[i].toString(16);
    }


    return "%89PNG%0D%0A%1A%0A" + str;
}
;
define("spice-png", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/


/*----------------------------------------------------------------------------
**  lz.js
**      Functions for handling SPICE_IMAGE_TYPE_LZ_RGB
**  Adapted from lz.c .
**--------------------------------------------------------------------------*/
function lz_rgb32_decompress(in_buf, at, out_buf, type, default_alpha)
{
    var encoder = at;
    var op = 0;
    var ctrl;
    var ctr = 0;

    for (ctrl = in_buf[encoder++]; (op * 4) < out_buf.length; ctrl = in_buf[encoder++])
    {
        var ref = op;
        var len = ctrl >> 5;
        var ofs = (ctrl & 31) << 8;

//if (type == LZ_IMAGE_TYPE_RGBA)
//console.log(ctr++ + ": from " + (encoder + 28) + ", ctrl " + ctrl + ", len " + len + ", ofs " + ofs + ", op " + op);
        if (ctrl >= 32) {

            var code;
            len--;

            if (len == 7 - 1) {
                do {
                    code = in_buf[encoder++];
                    len += code;
                } while (code == 255);
            }
            code = in_buf[encoder++];
            ofs += code;


            if (code == 255) {
                if ((ofs - code) == (31 << 8)) {
                    ofs = in_buf[encoder++] << 8;
                    ofs += in_buf[encoder++];
                    ofs += 8191;
                }
            }
            len += 1;
            if (type == LZ_IMAGE_TYPE_RGBA)
                len += 2;

            ofs += 1;

            ref -= ofs;
            if (ref == (op - 1)) {
                var b = ref;
//if (type == LZ_IMAGE_TYPE_RGBA) console.log("alpha " + out_buf[(b*4)+3] + " dupped into pixel " + op + " through pixel " + (op + len));
                for (; len; --len) {
                    if (type == LZ_IMAGE_TYPE_RGBA)
                    {
                        out_buf[(op*4) + 3] = out_buf[(b*4)+3];
                    }
                    else
                    {
                        for (i = 0; i < 4; i++)
                            out_buf[(op*4) + i] = out_buf[(b*4)+i];
                    }
                    op++;
                }
            } else {
//if (type == LZ_IMAGE_TYPE_RGBA) console.log("alpha copied to pixel " + op + " through " + (op + len) + " from " + ref);
                for (; len; --len) {
                    if (type == LZ_IMAGE_TYPE_RGBA)
                    {
                        out_buf[(op*4) + 3] = out_buf[(ref*4)+3];
                    }
                    else
                    {
                        for (i = 0; i < 4; i++)
                            out_buf[(op*4) + i] = out_buf[(ref*4)+i];
                    }
                    op++; ref++;
                }
            }
        } else {
            ctrl++;

            if (type == LZ_IMAGE_TYPE_RGBA)
            {
//console.log("alpha " + in_buf[encoder] + " set into pixel " + op);
                out_buf[(op*4) + 3] = in_buf[encoder++];
            }
            else
            {
                out_buf[(op*4) + 0] = in_buf[encoder + 2];
                out_buf[(op*4) + 1] = in_buf[encoder + 1];
                out_buf[(op*4) + 2] = in_buf[encoder + 0];
                if (default_alpha)
                    out_buf[(op*4) + 3] = 255;
                encoder += 3;
            }
            op++;


            for (--ctrl; ctrl; ctrl--) {
                if (type == LZ_IMAGE_TYPE_RGBA)
                {
//console.log("alpha " + in_buf[encoder] + " set into pixel " + op);
                    out_buf[(op*4) + 3] = in_buf[encoder++];
                }
                else
                {
                    out_buf[(op*4) + 0] = in_buf[encoder + 2];
                    out_buf[(op*4) + 1] = in_buf[encoder + 1];
                    out_buf[(op*4) + 2] = in_buf[encoder + 0];
                    if (default_alpha)
                        out_buf[(op*4) + 3] = 255;
                    encoder += 3;
                }
                op++;
            }
        }

    }
    return encoder - 1;
}

function convert_spice_lz_to_web(context, lz_image)
{
    var at;
    if (lz_image.type === LZ_IMAGE_TYPE_RGB32 || lz_image.type === LZ_IMAGE_TYPE_RGBA)
    {
        var u8 = new Uint8Array(lz_image.data);
        var ret = context.createImageData(lz_image.width, lz_image.height);

        at = lz_rgb32_decompress(u8, 0, ret.data, LZ_IMAGE_TYPE_RGB32, lz_image.type != LZ_IMAGE_TYPE_RGBA);
        if (lz_image.type == LZ_IMAGE_TYPE_RGBA)
            lz_rgb32_decompress(u8, at, ret.data, LZ_IMAGE_TYPE_RGBA, false);
    }
    else if (lz_image.type === LZ_IMAGE_TYPE_XXXA)
    {
        var u8 = new Uint8Array(lz_image.data);
        var ret = context.createImageData(lz_image.width, lz_image.height);
        lz_rgb32_decompress(u8, 0, ret.data, LZ_IMAGE_TYPE_RGBA, false);
    }
    else
        return undefined;

    return ret;
}
;
define("spice-lz", function(){});

/*"use strict";*/
/* use strict is commented out because it results in a 5x slowdone in chrome */
/*
 *    Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>
 *    Copyright (C) 2012 by Aric Stewart <aric@codeweavers.com>
 *
 *    This file is part of spice-html5.
 *
 *    spice-html5 is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    spice-html5 is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
 */

var encoder;

var QUIC_IMAGE_TYPE_INVALID = 0;
var QUIC_IMAGE_TYPE_GRAY = 1;
var QUIC_IMAGE_TYPE_RGB16 = 2;
var QUIC_IMAGE_TYPE_RGB24 = 3;
var QUIC_IMAGE_TYPE_RGB32 = 4;
var QUIC_IMAGE_TYPE_RGBA = 5;
var DEFevol = 3;
var DEFwmimax = 6;
var DEFwminext = 2048;
var need_init = true;
var DEFmaxclen = 26;
var evol = DEFevol;
var wmimax = DEFwmimax;
var wminext = DEFwminext;
var family_5bpc = { nGRcodewords:[0,0,0,0,0,0,0,0],
                    notGRcwlen:[0,0,0,0,0,0,0,0],
                    notGRprefixmask:[0,0,0,0,0,0,0,0],
                    notGRsuffixlen:[0,0,0,0,0,0,0,0],
                    xlatU2L:[0,0,0,0,0,0,0,0],
                    xlatL2U:[0,0,0,0,0,0,0,0]
                  };
var family_8bpc = { nGRcodewords:[0,0,0,0,0,0,0,0],
                    notGRcwlen:[0,0,0,0,0,0,0,0],
                    notGRprefixmask:[0,0,0,0,0,0,0,0],
                    notGRsuffixlen:[0,0,0,0,0,0,0,0],
                    xlatU2L:[0,0,0,0,0,0,0,0],
                    xlatL2U:[0,0,0,0,0,0,0,0]
                  };
var bppmask = [ 0x00000000,
    0x00000001, 0x00000003, 0x00000007, 0x0000000f,
    0x0000001f, 0x0000003f, 0x0000007f, 0x000000ff,
    0x000001ff, 0x000003ff, 0x000007ff, 0x00000fff,
    0x00001fff, 0x00003fff, 0x00007fff, 0x0000ffff,
    0x0001ffff, 0x0003ffff, 0x0007ffff, 0x000fffff,
    0x001fffff, 0x003fffff, 0x007fffff, 0x00ffffff,
    0x01ffffff, 0x03ffffff, 0x07ffffff, 0x0fffffff,
    0x1fffffff, 0x3fffffff, 0x7fffffff, 0xffffffff];

var zeroLUT = [];

var besttrigtab = [
            [ 550, 900, 800, 700, 500, 350, 300, 200, 180, 180, 160],
            [ 110, 550, 900, 800, 550, 400, 350, 250, 140, 160, 140],
            [ 100, 120, 550, 900, 700, 500, 400, 300, 220, 250, 160]];

var J = [ 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 5, 5, 6, 6,
          7, 7, 8, 9, 10, 11, 12, 13, 14, 15];

var lzeroes = [
    8, 7, 6, 6, 5, 5, 5, 5, 4, 4, 4, 4, 4, 4, 4, 4, 3, 3, 3, 3, 3, 3, 3, 3, 3,
    3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0];

var tabrand_chaos = [
    0x02c57542, 0x35427717, 0x2f5a2153, 0x9244f155, 0x7bd26d07, 0x354c6052,
    0x57329b28, 0x2993868e, 0x6cd8808c, 0x147b46e0, 0x99db66af, 0xe32b4cac,
    0x1b671264, 0x9d433486, 0x62a4c192, 0x06089a4b, 0x9e3dce44, 0xdaabee13,
    0x222425ea, 0xa46f331d, 0xcd589250, 0x8bb81d7f, 0xc8b736b9, 0x35948d33,
    0xd7ac7fd0, 0x5fbe2803, 0x2cfbc105, 0x013dbc4e, 0x7a37820f, 0x39f88e9e,
    0xedd58794, 0xc5076689, 0xfcada5a4, 0x64c2f46d, 0xb3ba3243, 0x8974b4f9,
    0x5a05aebd, 0x20afcd00, 0x39e2b008, 0x88a18a45, 0x600bde29, 0xf3971ace,
    0xf37b0a6b, 0x7041495b, 0x70b707ab, 0x06beffbb, 0x4206051f, 0xe13c4ee3,
    0xc1a78327, 0x91aa067c, 0x8295f72a, 0x732917a6, 0x1d871b4d, 0x4048f136,
    0xf1840e7e, 0x6a6048c1, 0x696cb71a, 0x7ff501c3, 0x0fc6310b, 0x57e0f83d,
    0x8cc26e74, 0x11a525a2, 0x946934c7, 0x7cd888f0, 0x8f9d8604, 0x4f86e73b,
    0x04520316, 0xdeeea20c, 0xf1def496, 0x67687288, 0xf540c5b2, 0x22401484,
    0x3478658a, 0xc2385746, 0x01979c2c, 0x5dad73c8, 0x0321f58b, 0xf0fedbee,
    0x92826ddf, 0x284bec73, 0x5b1a1975, 0x03df1e11, 0x20963e01, 0xa17cf12b,
    0x740d776e, 0xa7a6bf3c, 0x01b5cce4, 0x1118aa76, 0xfc6fac0a, 0xce927e9b,
    0x00bf2567, 0x806f216c, 0xbca69056, 0x795bd3e9, 0xc9dc4557, 0x8929b6c2,
    0x789d52ec, 0x3f3fbf40, 0xb9197368, 0xa38c15b5, 0xc3b44fa8, 0xca8333b0,
    0xb7e8d590, 0xbe807feb, 0xbf5f8360, 0xd99e2f5c, 0x372928e1, 0x7c757c4c,
    0x0db5b154, 0xc01ede02, 0x1fc86e78, 0x1f3985be, 0xb4805c77, 0x00c880fa,
    0x974c1b12, 0x35ab0214, 0xb2dc840d, 0x5b00ae37, 0xd313b026, 0xb260969d,
    0x7f4c8879, 0x1734c4d3, 0x49068631, 0xb9f6a021, 0x6b863e6f, 0xcee5debf,
    0x29f8c9fb, 0x53dd6880, 0x72b61223, 0x1f67a9fd, 0x0a0f6993, 0x13e59119,
    0x11cca12e, 0xfe6b6766, 0x16b6effc, 0x97918fc4, 0xc2b8a563, 0x94f2f741,
    0x0bfa8c9a, 0xd1537ae8, 0xc1da349c, 0x873c60ca, 0x95005b85, 0x9b5c080e,
    0xbc8abbd9, 0xe1eab1d2, 0x6dac9070, 0x4ea9ebf1, 0xe0cf30d4, 0x1ef5bd7b,
    0xd161043e, 0x5d2fa2e2, 0xff5d3cae, 0x86ed9f87, 0x2aa1daa1, 0xbd731a34,
    0x9e8f4b22, 0xb1c2c67a, 0xc21758c9, 0xa182215d, 0xccb01948, 0x8d168df7,
    0x04238cfe, 0x368c3dbc, 0x0aeadca5, 0xbad21c24, 0x0a71fee5, 0x9fc5d872,
    0x54c152c6, 0xfc329483, 0x6783384a, 0xeddb3e1c, 0x65f90e30, 0x884ad098,
    0xce81675a, 0x4b372f7d, 0x68bf9a39, 0x43445f1e, 0x40f8d8cb, 0x90d5acb6,
    0x4cd07282, 0x349eeb06, 0x0c9d5332, 0x520b24ef, 0x80020447, 0x67976491,
    0x2f931ca3, 0xfe9b0535, 0xfcd30220, 0x61a9e6cc, 0xa487d8d7, 0x3f7c5dd1,
    0x7d0127c5, 0x48f51d15, 0x60dea871, 0xc9a91cb7, 0x58b53bb3, 0x9d5e0b2d,
    0x624a78b4, 0x30dbee1b, 0x9bdf22e7, 0x1df5c299, 0x2d5643a7, 0xf4dd35ff,
    0x03ca8fd6, 0x53b47ed8, 0x6f2c19aa, 0xfeb0c1f4, 0x49e54438, 0x2f2577e6,
    0xbf876969, 0x72440ea9, 0xfa0bafb8, 0x74f5b3a0, 0x7dd357cd, 0x89ce1358,
    0x6ef2cdda, 0x1e7767f3, 0xa6be9fdb, 0x4f5f88f8, 0xba994a3a, 0x08ca6b65,
    0xe0893818, 0x9e00a16a, 0xf42bfc8f, 0x9972eedc, 0x749c8b51, 0x32c05f5e,
    0xd706805f, 0x6bfbb7cf, 0xd9210a10, 0x31a1db97, 0x923a9559, 0x37a7a1f6,
    0x059f8861, 0xca493e62, 0x65157e81, 0x8f6467dd, 0xab85ff9f, 0x9331aff2,
    0x8616b9f5, 0xedbd5695, 0xee7e29b1, 0x313ac44f, 0xb903112f, 0x432ef649,
    0xdc0a36c0, 0x61cf2bba, 0x81474925, 0xa8b6c7ad, 0xee5931de, 0xb2f8158d,
    0x59fb7409, 0x2e3dfaed, 0x9af25a3f, 0xe1fed4d5 ];

var rgb32_pixel_pad = 3;
var rgb32_pixel_r = 2;
var rgb32_pixel_g = 1;
var rgb32_pixel_b = 0;
var rgb32_pixel_size = 4;

/* Helper Functions */

function ceil_log_2(val)
{
    if (val === 1)
        return 0;

    var result = 1;
    val -= 1;
    while (val = val >>> 1)
        result++;

    return result;
}

function family_init(family, bpc, limit)
{
    var l;
    for (l = 0; l < bpc; l++)
    {
        var altprefixlen, altcodewords;
        altprefixlen = limit - bpc;
        if (altprefixlen > bppmask[bpc - l])
            altprefixlen = bppmask[bpc - l];

        altcodewords = bppmask[bpc] + 1 - (altprefixlen << l);
        family.nGRcodewords[l] = (altprefixlen << l);
        family.notGRcwlen[l] = altprefixlen + ceil_log_2(altcodewords);
        family.notGRprefixmask[l] = bppmask[32 - altprefixlen]>>>0;
        family.notGRsuffixlen[l] = ceil_log_2(altcodewords);
    }

    /* decorelate_init */
    var pixelbitmask = bppmask[bpc];
    var pixelbitmaskshr = pixelbitmask >>> 1;
    var s;
    for (s = 0; s <= pixelbitmask; s++) {
        if (s <= pixelbitmaskshr) {
            family.xlatU2L[s] = s << 1;
        } else {
            family.xlatU2L[s] = ((pixelbitmask - s) << 1) + 1;
        }
    }

    /* corelate_init */
    for (s = 0; s <= pixelbitmask; s++) {
        if (s & 0x01) {
            family.xlatL2U[s] = pixelbitmask - (s >>> 1);
        } else {
            family.xlatL2U[s] = (s >>> 1);
        }
    }
}

function quic_image_bpc(type)
{
    switch (type) {
    case QUIC_IMAGE_TYPE_GRAY:
        return 8;
    case QUIC_IMAGE_TYPE_RGB16:
        return 5;
    case QUIC_IMAGE_TYPE_RGB24:
        return 8;
    case QUIC_IMAGE_TYPE_RGB32:
        return 8;
    case QUIC_IMAGE_TYPE_RGBA:
        return 8;
    case QUIC_IMAGE_TYPE_INVALID:
    default:
        console.log("quic: bad image type\n");
        return 0;
    }
}

function cnt_l_zeroes(bits)
{
    if (bits & 0xff800000) {
        return lzeroes[bits >>> 24];
    } else if (bits & 0xffff8000) {
        return 8 + lzeroes[(bits >>> 16) & 0x000000ff];
    } else if (bits & 0xffffff80) {
        return 16 + lzeroes[(bits >>> 8) & 0x000000ff];
    } else {
        return 24 + lzeroes[bits & 0x000000ff];
    }
}

function golomb_decoding_8bpc(l, bits)
{
    var rc;
    var cwlen;

    if (bits < 0 || bits > family_8bpc.notGRprefixmask[l])
    {
        var zeroprefix = cnt_l_zeroes(bits);
        cwlen = zeroprefix + 1 + l;
        rc = (zeroprefix << l) | (bits >> (32-cwlen)) & bppmask[l];
    }
    else
    {
        cwlen = family_8bpc.notGRcwlen[l];
        rc = family_8bpc.nGRcodewords[l] + ((bits >> (32-cwlen)) & bppmask[family_8bpc.notGRsuffixlen[l]]);
    }
    return {'codewordlen':cwlen, 'rc':rc};
}

function golomb_code_len_8bpc(n, l)
{
    if (n < family_8bpc.nGRcodewords[l]) {
        return (n >>> l) + 1 + l;
    } else {
        return family_8bpc.notGRcwlen[l];
    }
}

function QuicModel(bpc)
{
    var bstart;
    var bend = 0;

    this.levels = 0x1 << bpc;
    this.n_buckets_ptrs = 0;

    switch (evol) {
        case 1:
            this.repfirst = 3;
            this.firstsize = 1;
            this.repnext = 2;
            this.mulsize = 2;
            break;
        case 3:
            this.repfirst = 1;
            this.firstsize = 1;
            this.repnext = 1;
            this.mulsize = 2;
            break;
        case 5:
            this.repfirst = 1;
            this.firstsize = 1;
            this.repnext = 1;
            this.mulsize = 4;
            break;
        case 0:
        case 2:
        case 4:
            console.log("quic: findmodelparams(): evol value obsolete!!!\n");
        default:
            console.log("quic: findmodelparams(): evol out of range!!!\n");
    }

    this.n_buckets = 0;
    var repcntr = this.repfirst + 1;
    var bsize = this.firstsize;

    do {
        if (this.n_buckets) {
            bstart = bend + 1;
        } else {
            bstart = 0;
        }

        if (!--repcntr) {
            repcntr = this.repnext;
            bsize *= this.mulsize;
        }

        bend = bstart + bsize - 1;
        if (bend + bsize >= this.levels) {
            bend = this.levels - 1;
        }

        if (!this.n_buckets_ptrs) {
            this.n_buckets_ptrs = this.levels;
        }

        (this.n_buckets)++;
    } while (bend < this.levels - 1);
}

QuicModel.prototype = {
    n_buckets : 0,
    n_buckets_ptrs : 0,
    repfirst : 0,
    firstsize : 0,
    repnext : 0,
    mulsize : 0,
    levels :0
}

function QuicBucket()
{
    this.counters = [0,0,0,0,0,0,0,0];
}

QuicBucket.prototype = {
    bestcode: 0,

    reste : function (bpp)
    {
        this.bestcode = bpp;
        this.counters = [0,0,0,0,0,0,0,0];
    },

    update_model_8bpc : function (state, curval, bpp)
    {
        var i;

        var bestcode = bpp - 1;
        var bestcodelen = (this.counters[bestcode] += golomb_code_len_8bpc(curval, bestcode));

        for (i = bpp - 2; i >= 0; i--) {
            var ithcodelen = (this.counters[i] += golomb_code_len_8bpc(curval, i));

            if (ithcodelen < bestcodelen) {
                bestcode = i;
                bestcodelen = ithcodelen;
            }
        }

        this.bestcode = bestcode;

        if (bestcodelen > state.wm_trigger) {
            for (i = 0; i < bpp; i++) {
                this.counters[i] = this.counters[i] >>> 1;
            }
        }
    }
}

function QuicFamilyStat()
{
    this.buckets_ptrs = [];
    this.buckets_buf = [];
}

QuicFamilyStat.prototype = {

    fill_model_structures : function(model)
    {
        var bstart;
        var bend = 0;
        var bnumber = 0;

        var repcntr = model.repfirst + 1;
        var bsize = model.firstsize;

        do {
            if (bnumber) {
                bstart = bend + 1;
            } else {
                bstart = 0;
            }

            if (!--repcntr) {
                repcntr = model.repnext;
                bsize *= model.mulsize;
            }

            bend = bstart + bsize - 1;
            if (bend + bsize >= model.levels) {
                bend = model.levels - 1;
            }

            this.buckets_buf[bnumber] = new QuicBucket;

            var i;
            for (i = bstart; i <= bend; i++) {
                this.buckets_ptrs[i] = this.buckets_buf[bnumber];
            }

            bnumber++;
        } while (bend < model.levels - 1);
        return true;
    }
}

function QuicChannel(model_8bpc, model_5bpc)
{
    this.state = new CommonState;
    this.family_stat_8bpc = new QuicFamilyStat;
    this.family_stat_5bpc = new QuicFamilyStat;
    this.correlate_row = { zero: 0 , row:[] };
    this.model_8bpc = model_8bpc;
    this.model_5bpc = model_5bpc;
    this.buckets_ptrs = [];

    if (!this.family_stat_8bpc.fill_model_structures(this.model_8bpc))
        return undefined;

    if (!this.family_stat_5bpc.fill_model_structures(this.model_5bpc))
        return undefined;
}

QuicChannel.prototype = {

    reste : function (bpc)
    {
        var j;
        this.correlate_row = { zero: 0 , row: []};

        if (bpc == 8) {
            for (j = 0; j < this.model_8bpc.n_buckets; j++)
                this.family_stat_8bpc.buckets_buf[j].reste(7);
            this.buckets_ptrs = this.family_stat_8bpc.buckets_ptrs;
        } else if (bpc == 5) {
            for (j = 0; j < this.model_5bpc.n_buckets; j++)
                this.family_stat_8bpc.buckets_buf[j].reste(4);
            this.buckets_ptrs = this.family_stat_5bpc.buckets_ptrs;
        } else {
            console.log("quic: %s: bad bpc %d\n", __FUNCTION__, bpc);
            return false;
        }

        this.state.reste();
        return true;
    }
}

function CommonState()
{
}

CommonState.prototype = {
    waitcnt: 0,
    tabrand_seed: 0xff,
    wm_trigger: 0,
    wmidx: 0,
    wmileft: wminext,
    melcstate: 0,
    melclen: 0,
    melcorder: 0,

    set_wm_trigger : function()
    {
        var wm = this.wmidx;
        if (wm > 10) {
            wm = 10;
        }

        this.wm_trigger = besttrigtab[Math.floor(evol / 2)][wm];
    },

    reste : function()
    {
        this.waitcnt = 0;
        this.tabrand_seed = 0x0ff;
        this.wmidx = 0;
        this.wmileft = wminext;

        this.set_wm_trigger();

        this.melcstate = 0;
        this.melclen = J[0];
        this.melcorder = 1 << this.melclen;
    },

    tabrand : function()
    {
        this.tabrand_seed++;
        return tabrand_chaos[this.tabrand_seed & 0x0ff];
    }
}


function QuicEncoder()
{
    this.rgb_state = new CommonState;
    this.model_8bpc = new QuicModel(8);
    this.model_5bpc = new QuicModel(5);
    this.channels = [];

    var i;
    for (i = 0; i < 4; i++) {
        this.channels[i] = new QuicChannel(this.model_8bpc, this.model_5bpc);
        if (!this.channels[i])
        {
            console.log("quic: failed to create channel");
            return undefined;
        }
    }
}

QuicEncoder.prototype = {
                    type: 0,
                    width: 0,
                    height: 0,
                    io_idx: 0,
                    io_available_bits: 0,
                    io_word: 0,
                    io_next_word: 0,
                    io_now: 0,
                    io_end: 0,
                    rows_completed: 0,
              };

QuicEncoder.prototype.reste = function(io_ptr)
{
    this.rgb_state.reste();

    this.io_now = io_ptr;
    this.io_end = this.io_now.length;
    this.io_idx = 0;
    this.rows_completed = 0;
    return true;
}

QuicEncoder.prototype.read_io_word = function()
{
    if (this.io_idx >= this.io_end)
        throw("quic: out of data");
    this.io_next_word = this.io_now[this.io_idx++] | this.io_now[this.io_idx++]<<8 | this.io_now[this.io_idx++]<<16 | this.io_now[this.io_idx++]<<24;
}

QuicEncoder.prototype.decode_eatbits = function (len)
{
    this.io_word = this.io_word << len;

    var delta = (this.io_available_bits - len);
    if (delta >= 0)
    {
        this.io_available_bits = delta;
        this.io_word |= this.io_next_word >>> this.io_available_bits;
    }
    else
    {
        delta = -1 * delta;
        this.io_word |= this.io_next_word << delta;
        this.read_io_word();
        this.io_available_bits = 32 - delta;
        this.io_word |= this.io_next_word >>> this.io_available_bits;
    }
}

QuicEncoder.prototype.decode_eat32bits = function()
{
    this.decode_eatbits(16);
    this.decode_eatbits(16);
}

QuicEncoder.prototype.reste_channels = function(bpc)
{
    var i;

    for (i = 0; i < 4; i++)
        if (!this.channels[i].reste(bpc))
            return false;
    return true;
}

QuicEncoder.prototype.quic_decode_begin = function(io_ptr)
{
    if (!this.reste(io_ptr)) {
        return false;
    }

    this.io_idx = 0;
    this.io_next_word = this.io_now[this.io_idx++] | this.io_now[this.io_idx++]<<8 | this.io_now[this.io_idx++]<<16 | this.io_now[this.io_idx++]<<24;
    this.io_word = this.io_next_word;
    this.io_available_bits = 0;

    var magic = this.io_word;
    this.decode_eat32bits();
    if (magic != 0x43495551) /*QUIC*/ {
        console.log("quic: bad magic "+magic.toString(16));
        return false;
    }

    var version = this.io_word;
    this.decode_eat32bits();
    if (version != ((0 << 16) | (0 & 0xffff))) {
        console.log("quic: bad version "+version.toString(16));
        return false;
    }

    this.type = this.io_word;
    this.decode_eat32bits();

    this.width = this.io_word;
    this.decode_eat32bits();

    this.height = this.io_word;
    this.decode_eat32bits();

    var bpc = quic_image_bpc(this.type);

    if (!this.reste_channels(bpc))
        return false;

    return true;
}

QuicEncoder.prototype.quic_rgb32_uncompress_row0_seg = function (i, cur_row, end,
                                       waitmask, bpc, bpc_mask)
{
    var stopidx;
    var n_channels = 3;
    var c;
    var a;

    if (!i) {
        cur_row[rgb32_pixel_pad] = 0;
        c = 0;
        do
        {
            a = golomb_decoding_8bpc(this.channels[c].buckets_ptrs[this.channels[c].correlate_row.zero].bestcode, this.io_word);
            this.channels[c].correlate_row.row[0] = a.rc;
            cur_row[2-c] = (family_8bpc.xlatL2U[a.rc]&0xFF);
            this.decode_eatbits(a.codewordlen);
        } while (++c < n_channels);

        if (this.rgb_state.waitcnt) {
            --this.rgb_state.waitcnt;
        } else {
            this.rgb_state.waitcnt = (this.rgb_state.tabrand() & waitmask);
            c = 0;
            do
            {
                this.channels[c].buckets_ptrs[this.channels[c].correlate_row.zero].update_model_8bpc(this.rgb_state, this.channels[c].correlate_row.row[0], bpc);
            } while (++c < n_channels);
        }
        stopidx = ++i + this.rgb_state.waitcnt;
    } else {
        stopidx = i + this.rgb_state.waitcnt;
    }

    while (stopidx < end) {
        for (; i <= stopidx; i++) {
            cur_row[(i* rgb32_pixel_size)+rgb32_pixel_pad] = 0;
            c = 0;
            do
            {
                a = golomb_decoding_8bpc(this.channels[c].buckets_ptrs[this.channels[c].correlate_row.row[i - 1]].bestcode, this.io_word);
                this.channels[c].correlate_row.row[i] = a.rc;
                cur_row[(i* rgb32_pixel_size)+(2-c)] = (family_8bpc.xlatL2U[a.rc] + cur_row[((i-1) * rgb32_pixel_size) + (2-c)]) & bpc_mask;
                this.decode_eatbits(a.codewordlen);
            } while (++c < n_channels);
        }
        c = 0;
        do
        {
            this.channels[c].buckets_ptrs[this.channels[c].correlate_row.row[stopidx - 1]].update_model_8bpc(this.rgb_state, this.channels[c].correlate_row.row[stopidx], bpc);
        } while (++c < n_channels);
        stopidx = i + (this.rgb_state.tabrand() & waitmask);
    }

    for (; i < end; i++) {
        cur_row[(i* rgb32_pixel_size)+rgb32_pixel_pad] = 0;
        c = 0;
        do
        {
            a = golomb_decoding_8bpc(this.channels[c].buckets_ptrs[this.channels[c].correlate_row.row[i - 1]].bestcode, this.io_word);
            this.channels[c].correlate_row.row[i] = a.rc;
            cur_row[(i* rgb32_pixel_size)+(2-c)] = (family_8bpc.xlatL2U[a.rc] + cur_row[((i-1) * rgb32_pixel_size) + (2-c)]) & bpc_mask;
            this.decode_eatbits(a.codewordlen);
        } while (++c < n_channels);
    }
    this.rgb_state.waitcnt = stopidx - end;
}

QuicEncoder.prototype.quic_rgb32_uncompress_row0 = function (cur_row)
{
    var bpc = 8;
    var bpc_mask = 0xff;
    var pos = 0;
    var width = this.width;

    while ((wmimax > this.rgb_state.wmidx) && (this.rgb_state.wmileft <= width)) {
        if (this.rgb_state.wmileft) {
            this.quic_rgb32_uncompress_row0_seg(pos, cur_row,
                                       pos + this.rgb_state.wmileft,
                                       bppmask[this.rgb_state.wmidx],
                                       bpc, bpc_mask);
            pos += this.rgb_state.wmileft;
            width -= this.rgb_state.wmileft;
        }

        this.rgb_state.wmidx++;
        this.rgb_state.set_wm_trigger();
        this.rgb_state.wmileft = wminext;
    }

    if (width) {
        this.quic_rgb32_uncompress_row0_seg(pos, cur_row, pos + width,
                                   bppmask[this.rgb_state.wmidx], bpc, bpc_mask);
        if (wmimax > this.rgb_state.wmidx) {
            this.rgb_state.wmileft -= width;
        }
    }
}

QuicEncoder.prototype.quic_rgb32_uncompress_row_seg = function( prev_row, cur_row, i, end, bpc, bpc_mask)
{
    var n_channels = 3;
    var waitmask = bppmask[this.rgb_state.wmidx];

    var a;
    var run_index = 0;
    var stopidx = 0;
    var run_end = 0;
    var c;

    if (!i)
    {
        cur_row[rgb32_pixel_pad] = 0;

        c = 0;
        do {
            a = golomb_decoding_8bpc(this.channels[c].buckets_ptrs[this.channels[c].correlate_row.zero].bestcode, this.io_word);
            this.channels[c].correlate_row.row[0] = a.rc;
            cur_row[2-c] = (family_8bpc.xlatL2U[this.channels[c].correlate_row.row[0]] + prev_row[2-c]) & bpc_mask;
            this.decode_eatbits(a.codewordlen);
        } while (++c < n_channels);

        if (this.rgb_state.waitcnt) {
            --this.rgb_state.waitcnt;
        } else {
            this.rgb_state.waitcnt = (this.rgb_state.tabrand() & waitmask);
            c = 0;
            do {
                this.channels[c].buckets_ptrs[this.channels[c].correlate_row.zero].update_model_8bpc(this.rgb_state, this.channels[c].correlate_row.row[0], bpc);
            } while (++c < n_channels);
        }
        stopidx = ++i + this.rgb_state.waitcnt;
    } else {
        stopidx = i + this.rgb_state.waitcnt;
    }
    for (;;) {
        var rc = 0;
        while (stopidx < end && !rc) {
            for (; i <= stopidx && !rc; i++) {
                var pixel = i * rgb32_pixel_size;
                var pixelm1 = (i-1) * rgb32_pixel_size;
                var pixelm2 = (i-2) * rgb32_pixel_size;

                if ( prev_row[pixelm1+rgb32_pixel_r] == prev_row[pixel+rgb32_pixel_r] && prev_row[pixelm1+rgb32_pixel_g] == prev_row[pixel+rgb32_pixel_g] && prev_row[pixelm1 + rgb32_pixel_b] == prev_row[pixel+rgb32_pixel_b])
                {
                    if (run_index != i && i > 2 && (cur_row[pixelm1+rgb32_pixel_r] == cur_row[pixelm2+rgb32_pixel_r] && cur_row[pixelm1+rgb32_pixel_g] == cur_row[pixelm2+rgb32_pixel_g] && cur_row[pixelm1+rgb32_pixel_b] == cur_row[pixelm2+rgb32_pixel_b]))
                    {
                        /* do run */
                        this.rgb_state.waitcnt = stopidx - i;
                        run_index = i;
                        run_end = i + this.decode_run(this.rgb_state);

                        for (; i < run_end; i++) {
                            var pixel = i * rgb32_pixel_size;
                            var pixelm1 = (i-1) * rgb32_pixel_size;
                            cur_row[pixel+rgb32_pixel_pad] = 0;
                            cur_row[pixel+rgb32_pixel_r] = cur_row[pixelm1+rgb32_pixel_r];
                            cur_row[pixel+rgb32_pixel_g] = cur_row[pixelm1+rgb32_pixel_g];
                            cur_row[pixel+rgb32_pixel_b] = cur_row[pixelm1+rgb32_pixel_b];
                        }

                        if (i == end) {
                            return;
                        }
                        else
                        {
                            stopidx = i + this.rgb_state.waitcnt;
                            rc = 1;
                            break;
                        }
                    }
                }

                c = 0;
                cur_row[pixel+rgb32_pixel_pad] = 0;
                do {
                    var cc = this.channels[c];
                    var cr = cc.correlate_row;

                    a = golomb_decoding_8bpc(cc.buckets_ptrs[cr.row[i-1]].bestcode, this.io_word);
                    cr.row[i] = a.rc;
                cur_row[pixel+(2-c)] = (family_8bpc.xlatL2U[a.rc] + ((cur_row[pixelm1+(2-c)] + prev_row[pixel+(2-c)]) >> 1)) & bpc_mask;
                    this.decode_eatbits(a.codewordlen);
                } while (++c < n_channels);
            }
            if (rc)
                break;

            c = 0;
            do {
                this.channels[c].buckets_ptrs[this.channels[c].correlate_row.row[stopidx - 1]].update_model_8bpc(this.rgb_state, this.channels[c].correlate_row.row[stopidx], bpc);
            } while (++c < n_channels);

            stopidx = i + (this.rgb_state.tabrand() & waitmask);
        }

        for (; i < end && !rc; i++) {
            var pixel = i * rgb32_pixel_size;
            var pixelm1 = (i-1) * rgb32_pixel_size;
            var pixelm2 = (i-2) * rgb32_pixel_size;

            if (prev_row[pixelm1+rgb32_pixel_r] == prev_row[pixel+rgb32_pixel_r] && prev_row[pixelm1+rgb32_pixel_g] == prev_row[pixel+rgb32_pixel_g] && prev_row[pixelm1+rgb32_pixel_b] == prev_row[pixel+rgb32_pixel_b])
            {
                if (run_index != i && i > 2 && (cur_row[pixelm1+rgb32_pixel_r] == cur_row[pixelm2+rgb32_pixel_r] && cur_row[pixelm1+rgb32_pixel_g] == cur_row[pixelm2+rgb32_pixel_g] && cur_row[pixelm1+rgb32_pixel_b] == cur_row[pixelm2+rgb32_pixel_b]))
                {
                    /* do run */
                    this.rgb_state.waitcnt = stopidx - i;
                    run_index = i;
                    run_end = i + this.decode_run(this.rgb_state);

                    for (; i < run_end; i++) {
                        var pixel = i * rgb32_pixel_size;
                        var pixelm1 = (i-1) * rgb32_pixel_size;
                        cur_row[pixel+rgb32_pixel_pad] = 0;
                        cur_row[pixel+rgb32_pixel_r] = cur_row[pixelm1+rgb32_pixel_r];
                        cur_row[pixel+rgb32_pixel_g] = cur_row[pixelm1+rgb32_pixel_g];
                        cur_row[pixel+rgb32_pixel_b] = cur_row[pixelm1+rgb32_pixel_b];
                    }

                    if (i == end) {
                        return;
                    }
                    else
                    {
                        stopidx = i + this.rgb_state.waitcnt;
                        rc = 1;
                        break;
                    }
                }
            }

            cur_row[pixel+rgb32_pixel_pad] = 0;
            c = 0;
            do
            {
                a = golomb_decoding_8bpc(this.channels[c].buckets_ptrs[this.channels[c].correlate_row.row[i-1]].bestcode, this.io_word);
                this.channels[c].correlate_row.row[i] = a.rc;
                cur_row[pixel+(2-c)] = (family_8bpc.xlatL2U[a.rc] + ((cur_row[pixelm1+(2-c)] + prev_row[pixel+(2-c)]) >> 1)) & bpc_mask;
                this.decode_eatbits(a.codewordlen);
            } while (++c < n_channels);
        }

          if (!rc)
          {
            this.rgb_state.waitcnt = stopidx - end;
            return;
          }
        }
}

QuicEncoder.prototype.decode_run = function(state)
{
    var runlen = 0;

    do {
        var hits;
        var x = (~(this.io_word >>> 24)>>>0)&0xff;
        var temp = zeroLUT[x];

        for (hits = 1; hits <= temp; hits++) {
            runlen += state.melcorder;

            if (state.melcstate < 32) {
                state.melclen = J[++state.melcstate];
                state.melcorder = (1 << state.melclen);
            }
        }
        if (temp != 8) {
            this.decode_eatbits(temp + 1);

            break;
        }
        this.decode_eatbits(8);
    } while (true);

    if (state.melclen) {
        runlen += this.io_word >>> (32 - state.melclen);
        this.decode_eatbits(state.melclen);
    }

    if (state.melcstate) {
        state.melclen = J[--state.melcstate];
        state.melcorder = (1 << state.melclen);
    }

    return runlen;
}

QuicEncoder.prototype.quic_rgb32_uncompress_row = function (prev_row, cur_row)
{
    var bpc = 8;
    var bpc_mask = 0xff;
    var pos = 0;
    var width = this.width;

    while ((wmimax > this.rgb_state.wmidx) && (this.rgb_state.wmileft <= width)) {
        if (this.rgb_state.wmileft) {
            this.quic_rgb32_uncompress_row_seg(prev_row, cur_row, pos,
                                      pos + this.rgb_state.wmileft, bpc, bpc_mask);
            pos += this.rgb_state.wmileft;
            width -= this.rgb_state.wmileft;
        }

        this.rgb_state.wmidx++;
        this.rgb_state.set_wm_trigger();
        this.rgb_state.wmileft = wminext;
    }

    if (width) {
        this.quic_rgb32_uncompress_row_seg(prev_row, cur_row, pos,
                                  pos + width, bpc, bpc_mask);
        if (wmimax > this.rgb_state.wmidx) {
            this.rgb_state.wmileft -= width;
        }
    }
}

QuicEncoder.prototype.quic_four_uncompress_row0_seg = function (channel, i,
                                       correlate_row, cur_row, end, waitmask,
                                       bpc, bpc_mask)
{
    var stopidx;
    var a;

    if (i == 0) {
        a = golomb_decoding_8bpc(channel.buckets_ptrs[correlate_row.zero].bestcode, this.io_word);
        correlate_row.row[0] = a.rc;
        cur_row[rgb32_pixel_pad] = family_8bpc.xlatL2U[a.rc];
        this.decode_eatbits(a.codewordlen);

        if (channel.state.waitcnt) {
            --channel.state.waitcnt;
        } else {
            channel.state.waitcnt = (channel.state.tabrand() & waitmask);
            channel.buckets_ptrs[correlate_row.zero].update_model_8bpc(channel.state, correlate_row.row[0], bpc);
        }
        stopidx = ++i + channel.state.waitcnt;
    } else {
        stopidx = i + channel.state.waitcnt;
    }

    while (stopidx < end) {
        var pbucket;

        for (; i <= stopidx; i++) {
            pbucket = channel.buckets_ptrs[correlate_row.row[i - 1]];

            a = golomb_decoding_8bpc(pbucket.bestcode, this.io_word);
            correlate_row.row[i] = a.rc;
            cur_row[(i*rgb32_pixel_size)+rgb32_pixel_pad] = (family_8bpc.xlatL2U[a.rc] + cur_row[((i-1)*rgb32_pixel_size)+rgb32_pixel_pad]) & bpc_mask;
            this.decode_eatbits(a.codewordlen);
        }

        pbucket.update_model_8bpc(channel.state, correlate_row.row[stopidx], bpc);

        stopidx = i + (channel.state.tabrand() & waitmask);
    }

    for (; i < end; i++) {
        a = golomb_decoding_8bpc(channel.buckets_ptrs[correlate_row.row[i-1]].bestcode, this.io_word);

        correlate_row.row[i] = a.rc;
        cur_row[(i*rgb32_pixel_size)+rgb32_pixel_pad] = (family_8bpc.xlatL2U[a.rc] + cur_row[((i-1)*rgb32_pixel_size)+rgb32_pixel_pad]) & bpc_mask;
        this.decode_eatbits(a.codewordlen);
    }
    channel.state.waitcnt = stopidx - end;
}

QuicEncoder.prototype.quic_four_uncompress_row0 = function(channel, cur_row)
{
    var bpc = 8;
    var bpc_mask = 0xff;
    var correlate_row = channel.correlate_row;
    var pos = 0;
    var width = this.width;

    while ((wmimax > channel.state.wmidx) && (channel.state.wmileft <= width)) {
        if (channel.state.wmileft) {
            this.quic_four_uncompress_row0_seg(channel, pos, correlate_row, cur_row,
                                       pos + channel.state.wmileft, bppmask[channel.state.wmidx],
                                       bpc, bpc_mask);
            pos += channel.state.wmileft;
            width -= channel.state.wmileft;
        }

        channel.state.wmidx++;
        channel.state.set_wm_trigger();
        channel.state.wmileft = wminext;
    }

    if (width) {
        this.quic_four_uncompress_row0_seg(channel, pos, correlate_row, cur_row, pos + width,
                                   bppmask[channel.state.wmidx], bpc, bpc_mask);
        if (wmimax > channel.state.wmidx) {
            channel.state.wmileft -= width;
        }
    }
}

QuicEncoder.prototype.quic_four_uncompress_row_seg = function (channel,
                                      correlate_row, prev_row, cur_row, i,
                                      end, bpc, bpc_mask)
{
    var waitmask = bppmask[channel.state.wmidx];
    var stopidx;

    var run_index = 0;
    var run_end;

    var a;

    if (i == 0) {
        a = golomb_decoding_8bpc(channel.buckets_ptrs[correlate_row.zero].bestcode, this.io_word);

        correlate_row.row[0] = a.rc
        cur_row[rgb32_pixel_pad] = (family_8bpc.xlatL2U[a.rc] + prev_row[rgb32_pixel_pad]) & bpc_mask;
        this.decode_eatbits(a.codewordlen);

        if (channel.state.waitcnt) {
            --channel.state.waitcnt;
        } else {
            channel.state.waitcnt = (channel.state.tabrand() & waitmask);
            channel.buckets_ptrs[correlate_row.zero].update_model_8bpc(channel.state, correlate_row.row[0], bpc);
        }
        stopidx = ++i + channel.state.waitcnt;
    } else {
        stopidx = i + channel.state.waitcnt;
    }
    for (;;) {
        var rc = 0;
        while (stopidx < end && !rc) {
            var pbucket;
            for (; i <= stopidx && !rc; i++) {
                var pixel = i * rgb32_pixel_size;
                var pixelm1 = (i-1) * rgb32_pixel_size;
                var pixelm2 = (i-2) * rgb32_pixel_size;

                if (prev_row[pixelm1+rgb32_pixel_pad] == prev_row[pixel+rgb32_pixel_pad])
                {
                    if (run_index != i && i > 2 && cur_row[pixelm1+rgb32_pixel_pad] == cur_row[pixelm2+rgb32_pixel_pad])
                    {
                        /* do run */
                        channel.state.waitcnt = stopidx - i;
                        run_index = i;

                        run_end = i + this.decode_run(channel.state);

                        for (; i < run_end; i++) {
                            var pixel = i * rgb32_pixel_size;
                            var pixelm1 = (i-1) * rgb32_pixel_size;
                            cur_row[pixel+rgb32_pixel_pad] = cur_row[pixelm1+rgb32_pixel_pad];
                        }

                        if (i == end) {
                            return;
                        }
                        else
                        {
                            stopidx = i + channel.state.waitcnt;
                            rc = 1;
                            break;
                        }
                    }
                }

                pbucket = channel.buckets_ptrs[correlate_row.row[i - 1]];
                a = golomb_decoding_8bpc(pbucket.bestcode, this.io_word);
                correlate_row.row[i] = a.rc
                cur_row[pixel+rgb32_pixel_pad] = (family_8bpc.xlatL2U[a.rc] + ((cur_row[pixelm1+rgb32_pixel_pad] + prev_row[pixel+rgb32_pixel_pad]) >> 1)) & bpc_mask;
                this.decode_eatbits(a.codewordlen);
            }
            if (rc)
                break;

            pbucket.update_model_8bpc(channel.state, correlate_row.row[stopidx], bpc);

            stopidx = i + (channel.state.tabrand() & waitmask);
        }

        for (; i < end && !rc; i++) {
            var pixel = i * rgb32_pixel_size;
            var pixelm1 = (i-1) * rgb32_pixel_size;
            var pixelm2 = (i-2) * rgb32_pixel_size;
            if (prev_row[pixelm1+rgb32_pixel_pad] == prev_row[pixel+rgb32_pixel_pad])
            {
                if (run_index != i && i > 2 && cur_row[pixelm1+rgb32_pixel_pad] == cur_row[pixelm2+rgb32_pixel_pad])
                {
                    /* do run */
                    channel.state.waitcnt = stopidx - i;
                    run_index = i;

                    run_end = i + this.decode_run(channel.state);

                    for (; i < run_end; i++) {
                        var pixel = i * rgb32_pixel_size;
                        var pixelm1 = (i-1) * rgb32_pixel_size;
                        cur_row[pixel+rgb32_pixel_pad] = cur_row[pixelm1+rgb32_pixel_pad];
                    }

                    if (i == end) {
                        return;
                    }
                    else
                    {
                        stopidx = i + channel.state.waitcnt;
                        rc = 1;
                        break;
                    }
                }
            }

            a = golomb_decoding_8bpc(channel.buckets_ptrs[correlate_row.row[i-1]].bestcode, this.io_word);
            correlate_row.row[i] = a.rc;
            cur_row[pixel+rgb32_pixel_pad] = (family_8bpc.xlatL2U[a.rc] + ((cur_row[pixelm1+rgb32_pixel_pad] + prev_row[pixel+rgb32_pixel_pad]) >> 1)) & bpc_mask;
            this.decode_eatbits(a.codewordlen);
        }

        if (!rc)
        {
            channel.state.waitcnt = stopidx - end;
            return;
        }
    }
}

QuicEncoder.prototype.quic_four_uncompress_row = function(channel, prev_row,
                                                        cur_row)
{
    var bpc = 8;
    var bpc_mask = 0xff;
    var correlate_row = channel.correlate_row;
    var pos = 0;
    var width = this.width;

    while ((wmimax > channel.state.wmidx) && (channel.state.wmileft <= width)) {
        if (channel.state.wmileft) {
            this.quic_four_uncompress_row_seg(channel, correlate_row, prev_row, cur_row, pos,
                                      pos + channel.state.wmileft, bpc, bpc_mask);
            pos += channel.state.wmileft;
            width -= channel.state.wmileft;
        }

        channel.state.wmidx++;
        channel.state.set_wm_trigger();
        channel.state.wmileft = wminext;
    }

    if (width) {
        this.quic_four_uncompress_row_seg(channel, correlate_row, prev_row, cur_row, pos,
                                  pos + width, bpc, bpc_mask);
        if (wmimax > channel.state.wmidx) {
            channel.state.wmileft -= width;
        }
    }
}

/* We need to be generating rgb32 or rgba */
QuicEncoder.prototype.quic_decode = function(buf, stride)
{
    var row;

    switch (this.type)
    {
        case QUIC_IMAGE_TYPE_RGB32:
        case QUIC_IMAGE_TYPE_RGB24:
            this.channels[0].correlate_row.zero = 0;
            this.channels[1].correlate_row.zero = 0;
            this.channels[2].correlate_row.zero = 0;
            this.quic_rgb32_uncompress_row0(buf);

            this.rows_completed++;
            for (row = 1; row < this.height; row++)
            {
                var prev = buf;
                buf = prev.subarray(stride);
                this.channels[0].correlate_row.zero = this.channels[0].correlate_row.row[0];
                this.channels[1].correlate_row.zero = this.channels[1].correlate_row.row[0];
                this.channels[2].correlate_row.zero = this.channels[2].correlate_row.row[0];
                this.quic_rgb32_uncompress_row(prev, buf);
                this.rows_completed++;
            };
            break;
        case QUIC_IMAGE_TYPE_RGB16:
            console.log("quic: unsupported output format\n");
            return false;
            break;
        case QUIC_IMAGE_TYPE_RGBA:
            this.channels[0].correlate_row.zero = 0;
            this.channels[1].correlate_row.zero = 0;
            this.channels[2].correlate_row.zero = 0;
            this.quic_rgb32_uncompress_row0(buf);

            this.channels[3].correlate_row.zero = 0;
            this.quic_four_uncompress_row0(this.channels[3], buf);

            this.rows_completed++;
            for (row = 1; row < this.height; row++) {
                var prev = buf;
                buf = prev.subarray(stride);

                this.channels[0].correlate_row.zero = this.channels[0].correlate_row.row[0];
                this.channels[1].correlate_row.zero = this.channels[1].correlate_row.row[0];
                this.channels[2].correlate_row.zero = this.channels[2].correlate_row.row[0];
                this.quic_rgb32_uncompress_row(prev, buf);

                this.channels[3].correlate_row.zero = this.channels[3].correlate_row.row[0];
                this.quic_four_uncompress_row(encoder.channels[3], prev, buf);
                this.rows_completed++;
            }
            break;

        case QUIC_IMAGE_TYPE_GRAY:
            console.log("quic: unsupported output format\n");
            return false;
            break;

        case QUIC_IMAGE_TYPE_INVALID:
        default:
            console.log("quic: bad image type\n");
            return false;
    }
    return true;
}

QuicEncoder.prototype.simple_quic_decode = function(buf)
{
    var stride = 4; /* FIXME - proper stride calc please */
    if (!this.quic_decode_begin(buf))
        return undefined;
    if (this.type != QUIC_IMAGE_TYPE_RGB32 && this.type != QUIC_IMAGE_TYPE_RGB24
        && this.type != QUIC_IMAGE_TYPE_RGBA)
        return undefined;
    var out = new Uint8Array(this.width*this.height*4);
    out[0] = 69;
    if (this.quic_decode( out, (this.width * stride)))
        return out;
    return undefined;
}

function SpiceQuic()
{
}

SpiceQuic.prototype =
{
    from_dv: function(dv, at, mb)
    {
        if (!encoder)
            throw("quic: no quic encoder");
        this.data_size = dv.getUint32(at, true);
        at += 4;
        var buf = new Uint8Array(mb.slice(at));
        this.outptr = encoder.simple_quic_decode(buf);
        if (this.outptr)
        {
            this.type = encoder.type;
            this.width = encoder.width;
            this.height = encoder.height;
        }
        at += buf.length;
        return at;
    },
}

function convert_spice_quic_to_web(context, spice_quic)
{
    var ret = context.createImageData(spice_quic.width, spice_quic.height);
    var i;
    for (i = 0; i < (ret.width * ret.height * 4); i+=4)
    {
        ret.data[i + 0] = spice_quic.outptr[i + 2];
        ret.data[i + 1] = spice_quic.outptr[i + 1];
        ret.data[i + 2] = spice_quic.outptr[i + 0];
        if (spice_quic.type !== QUIC_IMAGE_TYPE_RGBA)
            ret.data[i + 3] = 255;
        else
            ret.data[i + 3] = 255 - spice_quic.outptr[i + 3];
    }
   return ret;
}

/* Module initialization */
if (need_init)
{
    need_init = false;

    family_init(family_8bpc, 8, DEFmaxclen);
    family_init(family_5bpc, 5, DEFmaxclen);
    /* init_zeroLUT */
    var i, j, k, l;

    j = k = 1;
    l = 8;
    for (i = 0; i < 256; ++i) {
        zeroLUT[i] = l;
        --k;
        if (k == 0) {
            k = j;
            --l;
            j *= 2;
        }
    }

    encoder = new QuicEncoder;

    if (!encoder)
        throw("quic: failed to create encoder");
}
;
define("spice-quic", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/


/*----------------------------------------------------------------------------
**  bitmap.js
**      Handle SPICE_IMAGE_TYPE_BITMAP
**--------------------------------------------------------------------------*/
function convert_spice_bitmap_to_web(context, spice_bitmap)
{
    var ret;
    var offset, x;
    var u8 = new Uint8Array(spice_bitmap.data);
    if (spice_bitmap.format != SPICE_BITMAP_FMT_32BIT &&
        spice_bitmap.format != SPICE_BITMAP_FMT_RGBA)
        return undefined;

    ret = context.createImageData(spice_bitmap.x, spice_bitmap.y);
    for (offset = 0; offset < (spice_bitmap.y * spice_bitmap.stride); )
        for (x = 0; x < spice_bitmap.x; x++, offset += 4)
        {
            ret.data[offset + 0 ] = u8[offset + 2];
            ret.data[offset + 1 ] = u8[offset + 1];
            ret.data[offset + 2 ] = u8[offset + 0];

            // FIXME - We effectively treat all images as having SPICE_IMAGE_FLAGS_HIGH_BITS_SET
            if (spice_bitmap.format == SPICE_BITMAP_FMT_32BIT)
                ret.data[offset + 3] = 255;
            else
                ret.data[offset + 3] = u8[offset];
        }

    return ret;
}
;
define("spice-bitmap", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*----------------------------------------------------------------------------
**  SpiceDataView
** FIXME FIXME 
**    This is used because Firefox does not have DataView yet.
**    We should use DataView if we have it, because it *has* to 
**    be faster than this code 
**--------------------------------------------------------------------------*/
function SpiceDataView(buffer, byteOffset, byteLength)
{
    if (byteOffset !== undefined)
    {
        if (byteLength !== undefined)
            this.u8 = new Uint8Array(buffer, byteOffset, byteLength);
        else
            this.u8 = new Uint8Array(buffer, byteOffset);
    }
    else
        this.u8 = new Uint8Array(buffer);
};

SpiceDataView.prototype = {
    getUint8:  function(byteOffset)
    {
        return this.u8[byteOffset];
    },
    getUint16:  function(byteOffset, littleEndian)
    {
        var low = 1, high = 0;
        if (littleEndian)
        {
            low = 0;
            high = 1;
        }

        return (this.u8[byteOffset + high] << 8) | this.u8[byteOffset + low];
    },
    getUint32:  function(byteOffset, littleEndian)
    {
        var low = 2, high = 0;
        if (littleEndian)
        {
            low = 0;
            high = 2;
        }

        return (this.getUint16(byteOffset + high, littleEndian) << 16) | 
                this.getUint16(byteOffset + low, littleEndian);
    },
    getUint64: function (byteOffset, littleEndian)
    {
        var low = 4, high = 0;
        if (littleEndian)
        {
            low = 0;
            high = 4;
        }

        return (this.getUint32(byteOffset + high, littleEndian) << 32) |
                this.getUint32(byteOffset + low, littleEndian);
    },
    setUint8:  function(byteOffset, b)
    {
        this.u8[byteOffset] = (b & 0xff);
    },
    setUint16:  function(byteOffset, i, littleEndian)
    {
        var low = 1, high = 0;
        if (littleEndian)
        {
            low = 0;
            high = 1;
        }
        this.u8[byteOffset + high] = (i & 0xffff) >> 8;
        this.u8[byteOffset + low]  = (i & 0x00ff);
    },
    setUint32:  function(byteOffset, w, littleEndian)
    {
        var low = 2, high = 0;
        if (littleEndian)
        {
            low = 0;
            high = 2;
        }

        this.setUint16(byteOffset + high, (w & 0xffffffff) >> 16, littleEndian);
        this.setUint16(byteOffset + low,  (w & 0x0000ffff), littleEndian);
    },
    setUint64:  function(byteOffset, w, littleEndian)
    {
        var low = 4, high = 0;
        if (littleEndian)
        {
            low = 0;
            high = 4;
        }

        this.setUint32(byteOffset + high, (w & 0xffffffffffffffff) >> 32, littleEndian);
        this.setUint32(byteOffset + low,  (w & 0x00000000ffffffff), littleEndian);
    },
}
;
define("spice-spicedataview", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*----------------------------------------------------------------------------
**  Spice types
**      This file contains classes for common spice types.
**  Generally, they are used as helpers in reading and writing messages
**  to and from the server.
**--------------------------------------------------------------------------*/

function SpiceChannelId()
{
}
SpiceChannelId.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.type = dv.getUint8(at, true); at ++;
        this.id = dv.getUint8(at, true); at ++;
        return at;
    },
}

function SpiceRect()
{
}

SpiceRect.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.top = dv.getUint32(at, true); at += 4;
        this.left = dv.getUint32(at, true); at += 4;
        this.bottom = dv.getUint32(at, true); at += 4;
        this.right = dv.getUint32(at, true); at += 4;
        return at;
    },
    is_same_size : function(r)
    {
        if ((this.bottom - this.top) == (r.bottom - r.top) &&
            (this.right - this.left) == (r.right - r.left) )
            return true;

        return false;
    },
}

function SpiceClipRects()
{
}

SpiceClipRects.prototype =
{
    from_dv: function(dv, at, mb)
    {
        var i;
        this.num_rects = dv.getUint32(at, true); at += 4;
        if (this.num_rects > 0)
            this.rects = [];
        for (i = 0; i < this.num_rects; i++)
        {
            this.rects[i] = new SpiceRect();
            at = this.rects[i].from_dv(dv, at, mb);
        }
        return at;
    },
}

function SpiceClip()
{
}

SpiceClip.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.type = dv.getUint8(at, true); at ++;
        if (this.type == SPICE_CLIP_TYPE_RECTS)
        {
            this.rects = new SpiceClipRects();
            at = this.rects.from_dv(dv, at, mb);
        }
        return at;
    },
}

function SpiceImageDescriptor()
{
}

SpiceImageDescriptor.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.id = dv.getUint64(at, true); at += 8;
        this.type  = dv.getUint8(at, true); at ++;
        this.flags = dv.getUint8(at, true); at ++;
        this.width = dv.getUint32(at, true); at += 4;
        this.height= dv.getUint32(at, true); at += 4;
        return at;
    },
}

function SpicePalette()
{
}

SpicePalette.prototype =
{
    from_dv: function(dv, at, mb)
    {
        var i;
        this.unique = dv.getUint64(at, true); at += 8;
        this.num_ents = dv.getUint16(at, true); at += 2;
        this.ents = [];
        for (i = 0; i < this.num_ents; i++)
        {
            this.ents[i] = dv.getUint32(at, true); at += 4;
        }
        return at;
    },
}

function SpiceBitmap()
{
}

SpiceBitmap.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.format = dv.getUint8(at, true); at++;
        this.flags  = dv.getUint8(at, true); at++;
        this.x = dv.getUint32(at, true); at += 4;
        this.y = dv.getUint32(at, true); at += 4;
        this.stride = dv.getUint32(at, true); at += 4;
        if (this.flags & SPICE_BITMAP_FLAGS_PAL_FROM_CACHE)
        {
            this.palette_id = dv.getUint64(at, true); at += 8;
        }
        else
        {
            var offset = dv.getUint32(at, true); at += 4;
            if (offset == 0)
                this.palette = null;
            else
            {
                this.palette = new SpicePalette;
                this.palette.from_dv(dv, offset, mb);
            }
        }
        // FIXME - should probably constrain this to the offset
        //          of palette, if non zero
        this.data   = mb.slice(at);
        at += this.data.byteLength;
        return at;
    },
}

function SpiceImage()
{
}

SpiceImage.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.descriptor = new SpiceImageDescriptor;
        at = this.descriptor.from_dv(dv, at, mb);

        if (this.descriptor.type == SPICE_IMAGE_TYPE_LZ_RGB)
        {
            this.lz_rgb = new Object();
            this.lz_rgb.length = dv.getUint32(at, true); at += 4;
            var initial_at = at;
            this.lz_rgb.magic = "";
            for (var i = 3; i >= 0; i--)
                this.lz_rgb.magic += String.fromCharCode(dv.getUint8(at + i));
            at += 4;

            // NOTE:  The endian change is *correct*
            this.lz_rgb.version = dv.getUint32(at); at += 4;
            this.lz_rgb.type = dv.getUint32(at); at += 4;
            this.lz_rgb.width = dv.getUint32(at); at += 4;
            this.lz_rgb.height = dv.getUint32(at); at += 4;
            this.lz_rgb.stride = dv.getUint32(at); at += 4;
            this.lz_rgb.top_down = dv.getUint32(at); at += 4;

            var header_size = at - initial_at;

            this.lz_rgb.data   = mb.slice(at, this.lz_rgb.length + at - header_size);
            at += this.lz_rgb.data.byteLength;

        }

        if (this.descriptor.type == SPICE_IMAGE_TYPE_BITMAP)
        {
            this.bitmap = new SpiceBitmap;
            at = this.bitmap.from_dv(dv, at, mb);
        }

        if (this.descriptor.type == SPICE_IMAGE_TYPE_SURFACE)
        {
            this.surface_id = dv.getUint32(at, true); at += 4;
        }

        if (this.descriptor.type == SPICE_IMAGE_TYPE_JPEG)
        {
            this.jpeg = new Object;
            this.jpeg.data_size = dv.getUint32(at, true); at += 4;
            this.jpeg.data = mb.slice(at);
            at += this.jpeg.data.byteLength;
        }

        if (this.descriptor.type == SPICE_IMAGE_TYPE_JPEG_ALPHA)
        {
            this.jpeg_alpha = new Object;
            this.jpeg_alpha.flags = dv.getUint8(at, true); at += 1;
            this.jpeg_alpha.jpeg_size = dv.getUint32(at, true); at += 4;
            this.jpeg_alpha.data_size = dv.getUint32(at, true); at += 4;
            this.jpeg_alpha.data = mb.slice(at, this.jpeg_alpha.jpeg_size + at);
            at += this.jpeg_alpha.data.byteLength;
            // Alpha channel is an LZ image
            this.jpeg_alpha.alpha = new Object();
            this.jpeg_alpha.alpha.length = this.jpeg_alpha.data_size - this.jpeg_alpha.jpeg_size;
            var initial_at = at;
            this.jpeg_alpha.alpha.magic = "";
            for (var i = 3; i >= 0; i--)
                this.jpeg_alpha.alpha.magic += String.fromCharCode(dv.getUint8(at + i));
            at += 4;

            // NOTE:  The endian change is *correct*
            this.jpeg_alpha.alpha.version = dv.getUint32(at); at += 4;
            this.jpeg_alpha.alpha.type = dv.getUint32(at); at += 4;
            this.jpeg_alpha.alpha.width = dv.getUint32(at); at += 4;
            this.jpeg_alpha.alpha.height = dv.getUint32(at); at += 4;
            this.jpeg_alpha.alpha.stride = dv.getUint32(at); at += 4;
            this.jpeg_alpha.alpha.top_down = dv.getUint32(at); at += 4;

            var header_size = at - initial_at;

            this.jpeg_alpha.alpha.data   = mb.slice(at, this.jpeg_alpha.alpha.length + at - header_size);
            at += this.jpeg_alpha.alpha.data.byteLength;
        }

        if (this.descriptor.type == SPICE_IMAGE_TYPE_QUIC)
        {
            this.quic = new SpiceQuic;
            at = this.quic.from_dv(dv, at, mb);
        }
        return at;
    },
}


function SpiceQMask()
{
}

SpiceQMask.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.flags  = dv.getUint8(at, true); at++;
        this.pos = new SpicePoint;
        at = this.pos.from_dv(dv, at, mb);
        var offset = dv.getUint32(at, true); at += 4;
        if (offset == 0)
        {
            this.bitmap = null;
            return at;
        }

        this.bitmap = new SpiceImage;
        return this.bitmap.from_dv(dv, offset, mb);
    },
}


function SpicePattern()
{
}

SpicePattern.prototype =
{
    from_dv: function(dv, at, mb)
    {
        var offset = dv.getUint32(at, true); at += 4;
        if (offset == 0)
        {
            this.pat = null;
        }
        else
        {
            this.pat = new SpiceImage;
            this.pat.from_dv(dv, offset, mb);
        }

        this.pos = new SpicePoint;
        return this.pos.from_dv(dv, at, mb);
    }
}

function SpiceBrush()
{
}

SpiceBrush.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.type = dv.getUint8(at, true); at ++;
        if (this.type == SPICE_BRUSH_TYPE_SOLID)
        {
            this.color = dv.getUint32(at, true); at += 4;
        }
        else if (this.type == SPICE_BRUSH_TYPE_PATTERN)
        {
            this.pattern = new SpicePattern;
            at = this.pattern.from_dv(dv, at, mb);
        }
        return at;
    },
}

function SpiceFill()
{
}

SpiceFill.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.brush = new SpiceBrush;
        at = this.brush.from_dv(dv, at, mb);
        this.rop_descriptor = dv.getUint16(at, true); at += 2;
        this.mask = new SpiceQMask;
        return this.mask.from_dv(dv, at, mb);
    },
}


function SpiceCopy()
{
}

SpiceCopy.prototype =
{
    from_dv: function(dv, at, mb)
    {
        var offset = dv.getUint32(at, true); at += 4;
        if (offset == 0)
        {
            this.src_bitmap = null;
        }
        else
        {
            this.src_bitmap = new SpiceImage;
            this.src_bitmap.from_dv(dv, offset, mb);
        }
        this.src_area = new SpiceRect;
        at = this.src_area.from_dv(dv, at, mb);
        this.rop_descriptor = dv.getUint16(at, true); at += 2;
        this.scale_mode = dv.getUint8(at, true); at ++;
        this.mask = new SpiceQMask;
        return this.mask.from_dv(dv, at, mb);
    },
}

function SpicePoint16()
{
}

SpicePoint16.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.x = dv.getUint16(at, true); at += 2;
        this.y = dv.getUint16(at, true); at += 2;
        return at;
    },
}

function SpicePoint()
{
}

SpicePoint.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.x = dv.getUint32(at, true); at += 4;
        this.y = dv.getUint32(at, true); at += 4;
        return at;
    },
}

function SpiceCursorHeader()
{
}

SpiceCursorHeader.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.unique = dv.getUint64(at, true); at += 8;
        this.type = dv.getUint8(at, true); at ++;
        this.width = dv.getUint16(at, true); at += 2;
        this.height = dv.getUint16(at, true); at += 2;
        this.hot_spot_x = dv.getUint16(at, true); at += 2;
        this.hot_spot_y = dv.getUint16(at, true); at += 2;
        return at;
    },
}

function SpiceCursor()
{
}

SpiceCursor.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.flags = dv.getUint16(at, true); at += 2;
        if (this.flags & SPICE_CURSOR_FLAGS_NONE)
            this.header = null;
        else
        {
            this.header = new SpiceCursorHeader;
            at = this.header.from_dv(dv, at, mb);
            this.data   = mb.slice(at);
            at += this.data.byteLength;
        }
        return at;
    },
}

function SpiceSurface()
{
}

SpiceSurface.prototype =
{
    from_dv: function(dv, at, mb)
    {
        this.surface_id = dv.getUint32(at, true); at += 4;
        this.width = dv.getUint32(at, true); at += 4;
        this.height = dv.getUint32(at, true); at += 4;
        this.format = dv.getUint32(at, true); at += 4;
        this.flags = dv.getUint32(at, true); at += 4;
        return at;
    },
}

/* FIXME - SpiceImage  types lz_plt, jpeg, zlib_glz, and jpeg_alpha are 
           completely unimplemented */
;
define("spice-spicetype", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*----------------------------------------------------------------------------
**  Spice messages
**      This file contains classes for passing messages to and from
**  a spice server.  This file should arguably be generated from 
**  spice.proto, but it was instead put together by hand.
**--------------------------------------------------------------------------*/
function SpiceLinkHeader(a, at)
{
    this.magic = SPICE_MAGIC;
    this.major_version = SPICE_VERSION_MAJOR;
    this.minor_version = SPICE_VERSION_MINOR;
    this.size = 0;
    if (a !== undefined)
        this.from_buffer(a, at);
}

SpiceLinkHeader.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.magic = "";
        for (var i = 0; i < 4; i++)
            this.magic += String.fromCharCode(dv.getUint8(at + i));
        at += 4;

        this.major_version = dv.getUint32(at, true); at += 4;
        this.minor_version = dv.getUint32(at, true); at += 4;
        this.size = dv.getUint32(at, true); at += 4;
    },

    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        for (var i = 0; i < 4; i++)
            dv.setUint8(at + i, this.magic.charCodeAt(i));
        at += 4;

        dv.setUint32(at, this.major_version, true); at += 4;
        dv.setUint32(at, this.minor_version, true); at += 4;
        dv.setUint32(at, this.size, true); at += 4;
    },
    buffer_size: function()
    { 
        return 16;
    },
}

function SpiceLinkMess(a, at)
{
    this.connection_id = 0;
    this.channel_type = 0;
    this.channel_id = 0;
    this.common_caps = [];
    this.channel_caps = [];

    if (a !== undefined)
        this.from_buffer(a, at);
}

SpiceLinkMess.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var i;
        var orig_at = at;
        var dv = new SpiceDataView(a);
        this.connection_id = dv.getUint32(at, true); at += 4;
        this.channel_type = dv.getUint8(at, true); at++;
        this.channel_id = dv.getUint8(at, true); at++;
        var num_common_caps = dv.getUint32(at, true); at += 4;
        var num_channel_caps  = dv.getUint32(at, true); at += 4;
        var caps_offset = dv.getUint32(at, true); at += 4;

        at = orig_at + caps_offset;
        this.common_caps = [];
        for (i = 0; i < num_common_caps; i++)
        {
            this.common_caps.unshift(dv.getUint32(at, true)); at += 4;
        }

        this.channel_caps = [];
        for (i = 0; i < num_channel_caps; i++)
        {
            this.channel_caps.unshift(dv.getUint32(at, true)); at += 4;
        }
    },

    to_buffer: function(a, at)
    {
        at = at || 0;
        var orig_at = at;
        var i;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.connection_id, true); at += 4;
        dv.setUint8(at, this.channel_type, true); at++;
        dv.setUint8(at, this.channel_id, true); at++;
        dv.setUint32(at, this.common_caps.length, true); at += 4;
        dv.setUint32(at, this.channel_caps.length, true); at += 4;
        dv.setUint32(at, (at - orig_at) + 4, true); at += 4;

        for (i = 0; i < this.common_caps.length; i++)
        {
            dv.setUint32(at, this.common_caps[i], true); at += 4;
        }

        for (i = 0; i < this.channel_caps.length; i++)
        {
            dv.setUint32(at, this.channel_caps[i], true); at += 4;
        }
    },
    buffer_size: function()
    {
        return 18 + (4 * this.common_caps.length) + (4 * this.channel_caps.length);
    }
}

function SpiceLinkReply(a, at)
{
    this.error = 0;
    this.pub_key = undefined;
    this.common_caps = [];
    this.channel_caps = [];

    if (a !== undefined)
        this.from_buffer(a, at);
}

SpiceLinkReply.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var i;
        var orig_at = at;
        var dv = new SpiceDataView(a);
        this.error = dv.getUint32(at, true); at += 4;

        this.pub_key = create_rsa_from_mb(a, at);
        at += SPICE_TICKET_PUBKEY_BYTES;

        var num_common_caps = dv.getUint32(at, true); at += 4;
        var num_channel_caps  = dv.getUint32(at, true); at += 4;
        var caps_offset = dv.getUint32(at, true); at += 4;

        at = orig_at + caps_offset;
        this.common_caps = [];
        for (i = 0; i < num_common_caps; i++)
        {
            this.common_caps.unshift(dv.getUint32(at, true)); at += 4;
        }

        this.channel_caps = [];
        for (i = 0; i < num_channel_caps; i++)
        {
            this.channel_caps.unshift(dv.getUint32(at, true)); at += 4;
        }
    },
}

function SpiceLinkAuthTicket(a, at)
{
    this.auth_mechanism = 0;
    this.encrypted_data = undefined;
}

SpiceLinkAuthTicket.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var i;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.auth_mechanism, true); at += 4;
        for (i = 0; i < SPICE_TICKET_KEY_PAIR_LENGTH / 8; i++)
        {
            if (this.encrypted_data && i < this.encrypted_data.length)
                dv.setUint8(at, this.encrypted_data[i], true);
            else
                dv.setUint8(at, 0, true);
            at++;
        }
    },
    buffer_size: function()
    {
        return 4 + (SPICE_TICKET_KEY_PAIR_LENGTH / 8);
    }
}

function SpiceLinkAuthReply(a, at)
{
    this.auth_code = 0;
    if (a !== undefined)
        this.from_buffer(a, at);
}

SpiceLinkAuthReply.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.auth_code = dv.getUint32(at, true); at += 4;
    },
    buffer_size: function()
    {
        return 4;
    }
}

function SpiceMiniData(a, at)
{
    this.type = 0;
    this.size = 0;
    this.data = undefined;
    if (a !== undefined)
        this.from_buffer(a, at);
}

SpiceMiniData.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var i;
        var dv = new SpiceDataView(a);
        this.type = dv.getUint16(at, true); at += 2;
        this.size = dv.getUint32(at, true); at += 4;
        if (a.byteLength > at)
        {
            this.data = a.slice(at);
            at += this.data.byteLength;
        }
    },
    to_buffer : function(a, at)
    {
        at = at || 0;
        var i;
        var dv = new SpiceDataView(a);
        dv.setUint16(at, this.type, true); at += 2;
        dv.setUint32(at, this.data ? this.data.byteLength : 0, true); at += 4;
        if (this.data && this.data.byteLength > 0)
        {
            var u8arr = new Uint8Array(this.data);
            for (i = 0; i < u8arr.length; i++, at++)
                dv.setUint8(at, u8arr[i], true);
        }
    },
    build_msg : function(in_type,  extra)
    {
        this.type = in_type;
        this.size = extra.buffer_size();
        this.data = new ArrayBuffer(this.size);
        extra.to_buffer(this.data);
    },
    buffer_size: function()
    {
        if (this.data)
            return 6 + this.data.byteLength;
        else
            return 6;
    },
}

function SpiceMsgChannels(a, at)
{
    this.num_of_channels = 0;
    this.channels = [];
    if (a !== undefined)
        this.from_buffer(a, at);
}

SpiceMsgChannels.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var i;
        var dv = new SpiceDataView(a);
        this.num_of_channels = dv.getUint32(at, true); at += 4;
        for (i = 0; i < this.num_of_channels; i++)
        {
            var chan = new SpiceChannelId();
            at = chan.from_dv(dv, at, a);
            this.channels.push(chan);
        }
    },
}

function SpiceMsgMainInit(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgMainInit.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.session_id = dv.getUint32(at, true); at += 4;
        this.display_channels_hint = dv.getUint32(at, true); at += 4;
        this.supported_mouse_modes = dv.getUint32(at, true); at += 4;
        this.current_mouse_mode = dv.getUint32(at, true); at += 4;
        this.agent_connected = dv.getUint32(at, true); at += 4;
        this.agent_tokens = dv.getUint32(at, true); at += 4;
        this.multi_media_time = dv.getUint32(at, true); at += 4;
        this.ram_hint = dv.getUint32(at, true); at += 4;
    },
}

function SpiceMsgMainMouseMode(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgMainMouseMode.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.supported_modes = dv.getUint16(at, true); at += 2;
        this.current_mode = dv.getUint16(at, true); at += 2;
    },
}

function SpiceMsgMainAgentData(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgMainAgentData.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.protocol = dv.getUint32(at, true); at += 4;
        this.type = dv.getUint32(at, true); at += 4;
        this.opaque = dv.getUint64(at, true); at += 8;
        this.size = dv.getUint32(at, true); at += 4;
        if (a.byteLength > at)
        {
            this.data = a.slice(at);
            at += this.data.byteLength;
        }
    }
}

function SpiceMsgMainAgentTokens(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgMainAgentTokens.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.num_tokens = dv.getUint32(at, true); at += 4;
    },
}

function SpiceMsgSetAck(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgSetAck.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.generation = dv.getUint32(at, true); at += 4;
        this.window = dv.getUint32(at, true); at += 4;
    },
}

function SpiceMsgcAckSync(ack)
{
    this.generation = ack.generation;
}

SpiceMsgcAckSync.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.generation, true); at += 4;
    },
    buffer_size: function()
    {
        return 4;
    }
}

function SpiceMsgcMainMouseModeRequest(mode)
{
    this.mode = mode;
}

SpiceMsgcMainMouseModeRequest.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint16(at, this.mode, true); at += 2;
    },
    buffer_size: function()
    {
        return 2;
    }
}

function SpiceMsgcMainAgentStart(num_tokens)
{
    this.num_tokens = num_tokens;
}

SpiceMsgcMainAgentStart.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.num_tokens, true); at += 4;
    },
    buffer_size: function()
    {
        return 4;
    }
}

function SpiceMsgcMainAgentData(type, data)
{
    this.protocol = VD_AGENT_PROTOCOL;
    this.type = type;
    this.opaque = 0;
    this.size = data.buffer_size();
    this.data = data;
}

SpiceMsgcMainAgentData.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.protocol, true); at += 4;
        dv.setUint32(at, this.type, true); at += 4;
        dv.setUint64(at, this.opaque, true); at += 8;
        dv.setUint32(at, this.size, true); at += 4;
        this.data.to_buffer(a, at);
    },
    buffer_size: function()
    {
        return 4 + 4 + 8 + 4 + this.data.buffer_size();
    }
}

function VDAgentAnnounceCapabilities(request, caps)
{
    if (caps)
    {
        this.request = request;
        this.caps = caps;
    }
    else
        this.from_buffer(request);
}

VDAgentAnnounceCapabilities.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.request, true); at += 4;
        dv.setUint32(at, this.caps, true); at += 4;
    },
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.request = dv.getUint32(at, true); at += 4;
        this.caps = dv.getUint32(at, true); at += 4;
        return at;
    },
    buffer_size: function()
    {
        return 8;
    }
}

function VDAgentMonitorsConfig(flags, width, height, depth, x, y)
{
    this.num_mon = 1;
    this.flags = flags;
    this.width = width;
    this.height = height;
    this.depth = depth;
    this.x = x;
    this.y = y;
}

VDAgentMonitorsConfig.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.num_mon, true); at += 4;
        dv.setUint32(at, this.flags, true); at += 4;
        dv.setUint32(at, this.height, true); at += 4;
        dv.setUint32(at, this.width, true); at += 4;
        dv.setUint32(at, this.depth, true); at += 4;
        dv.setUint32(at, this.x, true); at += 4;
        dv.setUint32(at, this.y, true); at += 4;
    },
    buffer_size: function()
    {
        return 28;
    }
}

function VDAgentFileXferStatusMessage(data, result)
{
    if (result)
    {
        this.id = data;
        this.result = result;
    }
    else
        this.from_buffer(data);
}

VDAgentFileXferStatusMessage.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.id, true); at += 4;
        dv.setUint32(at, this.result, true); at += 4;
    },
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.id = dv.getUint32(at, true); at += 4;
        this.result = dv.getUint32(at, true); at += 4;
        return at;
    },
    buffer_size: function()
    {
        return 8;
    }
}

function VDAgentFileXferStartMessage(id, name, size)
{
    this.id = id;
    this.string = "[vdagent-file-xfer]\n"+"name="+name+"\nsize="+size+"\n";
}

VDAgentFileXferStartMessage.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.id, true); at += 4;
        for (var i = 0; i < this.string.length; i++, at++)
            dv.setUint8(at, this.string.charCodeAt(i));
    },
    buffer_size: function()
    {
        return 4 + this.string.length + 1;
    }
}

function VDAgentFileXferDataMessage(id, size, data)
{
    this.id = id;
    this.size = size;
    this.data = data;
}

VDAgentFileXferDataMessage.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.id, true); at += 4;
        dv.setUint64(at, this.size, true); at += 8;
        if (this.data && this.data.byteLength > 0)
        {
            var u8arr = new Uint8Array(this.data);
            for (var i = 0; i < u8arr.length; i++, at++)
                dv.setUint8(at, u8arr[i]);
        }
    },
    buffer_size: function()
    {
        return 12 + this.size;
    }
}

function SpiceMsgNotify(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgNotify.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var i;
        var dv = new SpiceDataView(a);
        this.time_stamp = dv.getUint64(at, true); at += 8;
        this.severity = dv.getUint32(at, true); at += 4;
        this.visibility = dv.getUint32(at, true); at += 4;
        this.what = dv.getUint32(at, true); at += 4;
        this.message_len = dv.getUint32(at, true); at += 4;
        this.message = "";
        for (i = 0; i < this.message_len; i++)
        {
            var c = dv.getUint8(at, true); at++;
            this.message += String.fromCharCode(c);
        }
    },
}

function SpiceMsgcDisplayInit()
{
    this.pixmap_cache_id = 1;
    this.glz_dictionary_id = 0;
    this.pixmap_cache_size = 10 * 1024 * 1024;
    this.glz_dictionary_window_size = 0;
}

SpiceMsgcDisplayInit.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint8(at, this.pixmap_cache_id, true); at++;
        dv.setUint64(at, this.pixmap_cache_size, true); at += 8;
        dv.setUint8(at, this.glz_dictionary_id, true); at++;
        dv.setUint32(at, this.glz_dictionary_window_size, true); at += 4;
    },
    buffer_size: function()
    {
        return 14;
    }
}

function SpiceMsgDisplayBase()
{
}

SpiceMsgDisplayBase.prototype =
{
    from_dv : function(dv, at, mb)
    {
        this.surface_id = dv.getUint32(at, true); at += 4;
        this.box = new SpiceRect;
        at = this.box.from_dv(dv, at, mb);
        this.clip = new SpiceClip;
        return this.clip.from_dv(dv, at, mb);
    },
}

function SpiceMsgDisplayDrawCopy(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgDisplayDrawCopy.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.base = new SpiceMsgDisplayBase;
        at = this.base.from_dv(dv, at, a);
        this.data = new SpiceCopy;
        return this.data.from_dv(dv, at, a);
    },
}

function SpiceMsgDisplayDrawFill(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgDisplayDrawFill.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.base = new SpiceMsgDisplayBase;
        at = this.base.from_dv(dv, at, a);
        this.data = new SpiceFill;
        return this.data.from_dv(dv, at, a);
    },
}

function SpiceMsgDisplayCopyBits(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgDisplayCopyBits.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.base = new SpiceMsgDisplayBase;
        at = this.base.from_dv(dv, at, a);
        this.src_pos = new SpicePoint;
        return this.src_pos.from_dv(dv, at, a);
    },
}


function SpiceMsgSurfaceCreate(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgSurfaceCreate.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.surface = new SpiceSurface;
        return this.surface.from_dv(dv, at, a);
    },
}

function SpiceMsgSurfaceDestroy(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgSurfaceDestroy.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.surface_id = dv.getUint32(at, true); at += 4;
    },
}

function SpiceMsgInputsInit(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgInputsInit.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.keyboard_modifiers = dv.getUint16(at, true); at += 2;
        return at;
    },
}

function SpiceMsgInputsKeyModifiers(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgInputsKeyModifiers.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.keyboard_modifiers = dv.getUint16(at, true); at += 2;
        return at;
    },
}

function SpiceMsgCursorInit(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgCursorInit.prototype =
{
    from_buffer: function(a, at, mb)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.position = new SpicePoint16;
        at = this.position.from_dv(dv, at, mb);
        this.trail_length = dv.getUint16(at, true); at += 2;
        this.trail_frequency = dv.getUint16(at, true); at += 2;
        this.visible = dv.getUint8(at, true); at ++;
        this.cursor = new SpiceCursor;
        return this.cursor.from_dv(dv, at, a);
    },
}

function SpiceMsgPlaybackData(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgPlaybackData.prototype =
{
    from_buffer: function(a, at, mb)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.time = dv.getUint32(at, true); at += 4;
        if (a.byteLength > at)
        {
            this.data = a.slice(at);
            at += this.data.byteLength;
        }
        return at;
    },
}

function SpiceMsgPlaybackMode(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgPlaybackMode.prototype =
{
    from_buffer: function(a, at, mb)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.time = dv.getUint32(at, true); at += 4;
        this.mode = dv.getUint16(at, true); at += 2;
        if (a.byteLength > at)
        {
            this.data = a.slice(at);
            at += this.data.byteLength;
        }
        return at;
    },
}

function SpiceMsgPlaybackStart(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgPlaybackStart.prototype =
{
    from_buffer: function(a, at, mb)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.channels = dv.getUint32(at, true); at += 4;
        this.format = dv.getUint16(at, true); at += 2;
        this.frequency = dv.getUint32(at, true); at += 4;
        this.time = dv.getUint32(at, true); at += 4;
        return at;
    },
}



function SpiceMsgCursorSet(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgCursorSet.prototype =
{
    from_buffer: function(a, at, mb)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.position = new SpicePoint16;
        at = this.position.from_dv(dv, at, mb);
        this.visible = dv.getUint8(at, true); at ++;
        this.cursor = new SpiceCursor;
        return this.cursor.from_dv(dv, at, a);
    },
}


function SpiceMsgcMousePosition(sc, e)
{
    // FIXME - figure out how to correctly compute display_id
    this.display_id = 0;
    this.buttons_state = sc.buttons_state;
    if (e)
    {
        var scrollTop = document.body.scrollTop || document.documentElement.scrollTop;
        var scrollLeft = document.body.scrollLeft || document.documentElement.scrollLeft;

        this.x = e.clientX - sc.display.surfaces[sc.display.primary_surface].canvas.offsetLeft + scrollLeft;
        this.y = e.clientY - sc.display.surfaces[sc.display.primary_surface].canvas.offsetTop + scrollTop;
        sc.mousex = this.x;
        sc.mousey = this.y; 
    }
    else
    {
        this.x = this.y = this.buttons_state = 0;
    }
}

SpiceMsgcMousePosition.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.x, true); at += 4;
        dv.setUint32(at, this.y, true); at += 4;
        dv.setUint16(at, this.buttons_state, true); at += 2;
        dv.setUint8(at, this.display_id, true); at += 1;
        return at;
    },
    buffer_size: function()
    {
        return 11;
    }
}

function SpiceMsgcMouseMotion(sc, e)
{
    // FIXME - figure out how to correctly compute display_id
    this.display_id = 0;
    this.buttons_state = sc.buttons_state;
    if (e)
    {
        this.x = e.clientX - sc.display.surfaces[sc.display.primary_surface].canvas.offsetLeft;
        this.y = e.clientY - sc.display.surfaces[sc.display.primary_surface].canvas.offsetTop;

        if (sc.mousex !== undefined)
        {
            this.x -= sc.mousex;
            this.y -= sc.mousey;
        }
        sc.mousex = e.clientX - sc.display.surfaces[sc.display.primary_surface].canvas.offsetLeft;
        sc.mousey = e.clientY - sc.display.surfaces[sc.display.primary_surface].canvas.offsetTop;
    }
    else
    {
        this.x = this.y = this.buttons_state = 0;
    }
}

/* Use the same functions as for MousePosition */
SpiceMsgcMouseMotion.prototype.to_buffer = SpiceMsgcMousePosition.prototype.to_buffer;
SpiceMsgcMouseMotion.prototype.buffer_size = SpiceMsgcMousePosition.prototype.buffer_size;

function SpiceMsgcMousePress(sc, e)
{
    if (e)
    {
        this.button = e.button + 1;
        this.buttons_state = 1 << e.button;
        sc.buttons_state = this.buttons_state;
    }
    else
    {
        this.button = SPICE_MOUSE_BUTTON_LEFT;
        this.buttons_state = SPICE_MOUSE_BUTTON_MASK_LEFT;
    }
}

SpiceMsgcMousePress.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint8(at, this.button, true); at ++;
        dv.setUint16(at, this.buttons_state, true); at += 2;
        return at;
    },
    buffer_size: function()
    {
        return 3;
    }
}

function SpiceMsgcMouseRelease(sc, e)
{
    if (e)
    {
        this.button = e.button + 1;
        this.buttons_state = 0;
        sc.buttons_state = this.buttons_state;
    }
    else
    {
        this.button = SPICE_MOUSE_BUTTON_LEFT;
        this.buttons_state = 0;
    }
}

/* Use the same functions as for MousePress */
SpiceMsgcMouseRelease.prototype.to_buffer = SpiceMsgcMousePress.prototype.to_buffer;
SpiceMsgcMouseRelease.prototype.buffer_size = SpiceMsgcMousePress.prototype.buffer_size;


function SpiceMsgcKeyDown(e)
{
    if (e)
    {
        this.code = keycode_to_start_scan(e.keyCode);
    }
    else
    {
        this.code = 0;
    }
}

SpiceMsgcKeyDown.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint32(at, this.code, true); at += 4;
        return at;
    },
    buffer_size: function()
    {
        return 4;
    }
}

function SpiceMsgcKeyUp(e)
{
    if (e)
    {
        this.code = keycode_to_end_scan(e.keyCode);
    }
    else
    {
        this.code = 0;
    }
}

/* Use the same functions as for KeyDown */
SpiceMsgcKeyUp.prototype.to_buffer = SpiceMsgcKeyDown.prototype.to_buffer;
SpiceMsgcKeyUp.prototype.buffer_size = SpiceMsgcKeyDown.prototype.buffer_size;

function SpiceMsgDisplayStreamCreate(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgDisplayStreamCreate.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.surface_id = dv.getUint32(at, true); at += 4;
        this.id = dv.getUint32(at, true); at += 4;
        this.flags = dv.getUint8(at, true); at += 1;
        this.codec_type = dv.getUint8(at, true); at += 1;
        this.stamp = dv.getUint64(at, true); at += 8;
        this.stream_width = dv.getUint32(at, true); at += 4;
        this.stream_height = dv.getUint32(at, true); at += 4;
        this.src_width = dv.getUint32(at, true); at += 4;
        this.src_height = dv.getUint32(at, true); at += 4;

        this.dest = new SpiceRect;
        at = this.dest.from_dv(dv, at, a);
        this.clip = new SpiceClip;
        this.clip.from_dv(dv, at, a);
    },
}

function SpiceStreamDataHeader(a, at)
{
}

SpiceStreamDataHeader.prototype =
{
    from_dv : function(dv, at, mb)
    {
        this.id = dv.getUint32(at, true); at += 4;
        this.multi_media_time = dv.getUint32(at, true); at += 4;
        return at;
    },
}

function SpiceMsgDisplayStreamData(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgDisplayStreamData.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.base = new SpiceStreamDataHeader;
        at = this.base.from_dv(dv, at, a);
        this.data_size = dv.getUint32(at, true); at += 4;
        this.data = dv.u8.subarray(at, at + this.data_size);
    },
}

function SpiceMsgDisplayStreamClip(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgDisplayStreamClip.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.id = dv.getUint32(at, true); at += 4;
        this.clip = new SpiceClip;
        this.clip.from_dv(dv, at, a);
    },
}

function SpiceMsgDisplayStreamDestroy(a, at)
{
    this.from_buffer(a, at);
}

SpiceMsgDisplayStreamDestroy.prototype =
{
    from_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.id = dv.getUint32(at, true); at += 4;
    },
}

function SpiceMsgDisplayInvalList(a, at)
{
    this.count = 0;
    this.resources = [];
    this.from_buffer(a,at);
}

SpiceMsgDisplayInvalList.prototype =
{
    from_buffer: function (a, at)
    {
        var i;
        at = at || 0;
        var dv = new SpiceDataView(a);
        this.count = dv.getUint16(at, true); at += 2;
        for (i = 0; i < this.count; i++)
        {
            this.resources[i] = {};
            this.resources[i].type = dv.getUint8(at, true); at++;
            this.resources[i].id = dv.getUint64(at, true); at += 8;
        }
    },
}
;
define("spice-spicemsg", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*--------------------------------------------------------------------------------------
**  SpiceWireReader
**      This class will receive messages from a WebSocket and relay it to a given
**  callback.  It will optionally save and pass along a header, useful in processing
**  the mini message format.
**--------------------------------------------------------------------------------------*/
function SpiceWireReader(sc, callback)
{
    this.sc = sc;
    this.callback = callback;
    this.needed = 0;

    this.buffers = [];

    this.sc.ws.wire_reader = this;
    this.sc.ws.binaryType = "arraybuffer";
    this.sc.ws.addEventListener('message', wire_blob_catcher);
}

SpiceWireReader.prototype =
{

    /*------------------------------------------------------------------------
    **  Process messages coming in from our WebSocket
    **----------------------------------------------------------------------*/
    inbound: function (mb)
    {
        var at;

        /* Just buffer if we don't need anything yet */
        if (this.needed == 0)
        {
            this.buffers.push(mb);
            return;
        }

        /* Optimization - if we have just one inbound block, and it's
            suitable for our needs, just use it.  */
        if (this.buffers.length == 0 && mb.byteLength >= this.needed)
        {
            if (mb.byteLength > this.needed)
            {
                this.buffers.push(mb.slice(this.needed));
                mb = mb.slice(0, this.needed);
            }
            this.callback.call(this.sc, mb,
                        this.saved_msg_header || undefined);
        }
        else
        {
            this.buffers.push(mb);
        }


        /* If we have fragments that add up to what we need, combine them */
        /*  FIXME - it would be faster to revise the processing code to handle
        **          multiple fragments directly.  Essentially, we should be
        **          able to do this without any slice() or combine_array_buffers() calls */
        while (this.buffers.length > 1 && this.buffers[0].byteLength < this.needed)
        {
            var mb1 = this.buffers.shift();
            var mb2 = this.buffers.shift();

            this.buffers.unshift(combine_array_buffers(mb1, mb2));
        }


        while (this.buffers.length > 0 && this.buffers[0].byteLength >= this.needed)
        {
            mb = this.buffers.shift();
            if (mb.byteLength > this.needed)
            {
                this.buffers.unshift(mb.slice(this.needed));
                mb = mb.slice(0, this.needed);
            }
            this.callback.call(this.sc, mb,
                        this.saved_msg_header || undefined);
        }
        
    },

    request: function(n)
    {
        this.needed = n;
    },

    save_header: function(h)
    {
        this.saved_msg_header = h;
    },

    clear_header: function()
    {
        this.saved_msg_header = undefined;
    },
}

function wire_blob_catcher(e)
{
    DEBUG > 1 && console.log(">> WebSockets.onmessage");
    DEBUG > 1 && console.log("id " + this.wire_reader.sc.connection_id +"; type " + this.wire_reader.sc.type);
    SpiceWireReader.prototype.inbound.call(this.wire_reader, e.data);
}
;
define("spice-wire", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/


/*----------------------------------------------------------------------------
**  FIXME: putImageData  does not support Alpha blending
**           or compositing.  So if we have data in an ImageData
**           format, we have to draw it onto a context,
**           and then use drawImage to put it onto the target,
**           as drawImage does alpha.
**--------------------------------------------------------------------------*/
function putImageDataWithAlpha(context, d, x, y)
{
    var c = document.createElement("canvas");
    var t = c.getContext("2d");
    c.setAttribute('width', d.width);
    c.setAttribute('height', d.height);
    t.putImageData(d, 0, 0);
    context.drawImage(c, x, y, d.width, d.height);
}

/*----------------------------------------------------------------------------
**  FIXME: Spice will send an image with '0' alpha when it is intended to
**           go on a surface w/no alpha.  So in that case, we have to strip
**           out the alpha.  The test case for this was flux box; in a Xspice
**           server, right click on the desktop to get the menu; the top bar
**           doesn't paint/highlight correctly w/out this change.
**--------------------------------------------------------------------------*/
function stripAlpha(d)
{
    var i;
    for (i = 0; i < (d.width * d.height * 4); i += 4)
        d.data[i + 3] = 255;
}

/*----------------------------------------------------------------------------
**  SpiceDisplayConn
**      Drive the Spice Display Channel
**--------------------------------------------------------------------------*/
function SpiceDisplayConn()
{
    SpiceConn.apply(this, arguments);
}

SpiceDisplayConn.prototype = Object.create(SpiceConn.prototype);
SpiceDisplayConn.prototype.process_channel_message = function(msg)
{
    if (msg.type == SPICE_MSG_DISPLAY_MARK)
    {
        // FIXME - DISPLAY_MARK not implemented (may be hard or impossible)
        this.known_unimplemented(msg.type, "Display Mark");
        return true;
    }

    if (msg.type == SPICE_MSG_DISPLAY_RESET)
    {
        DEBUG > 2 && console.log("Display reset");
        this.surfaces[this.primary_surface].canvas.context.restore();
        return true;
    }

    if (msg.type == SPICE_MSG_DISPLAY_DRAW_COPY)
    {
        var draw_copy = new SpiceMsgDisplayDrawCopy(msg.data);

        DEBUG > 1 && this.log_draw("DrawCopy", draw_copy);

        if (! draw_copy.base.box.is_same_size(draw_copy.data.src_area))
            this.log_warn("FIXME: DrawCopy src_area is a different size than base.box; we do not handle that yet.");
        if (draw_copy.base.clip.type != SPICE_CLIP_TYPE_NONE)
            this.log_warn("FIXME: DrawCopy we don't handle clipping yet");
        if (draw_copy.data.rop_descriptor != SPICE_ROPD_OP_PUT)
            this.log_warn("FIXME: DrawCopy we don't handle ropd type: " + draw_copy.data.rop_descriptor);
        if (draw_copy.data.mask.flags)
            this.log_warn("FIXME: DrawCopy we don't handle mask flag: " + draw_copy.data.mask.flags);
        if (draw_copy.data.mask.bitmap)
            this.log_warn("FIXME: DrawCopy we don't handle mask");

        if (draw_copy.data && draw_copy.data.src_bitmap)
        {
            if (draw_copy.data.src_bitmap.descriptor.flags &&
                draw_copy.data.src_bitmap.descriptor.flags != SPICE_IMAGE_FLAGS_CACHE_ME &&
                draw_copy.data.src_bitmap.descriptor.flags != SPICE_IMAGE_FLAGS_HIGH_BITS_SET)
            {
                this.log_warn("FIXME: DrawCopy unhandled image flags: " + draw_copy.data.src_bitmap.descriptor.flags);
                DEBUG <= 1 && this.log_draw("DrawCopy", draw_copy);
            }

            if (draw_copy.data.src_bitmap.descriptor.type == SPICE_IMAGE_TYPE_QUIC)
            {
                var canvas = this.surfaces[draw_copy.base.surface_id].canvas;
                if (! draw_copy.data.src_bitmap.quic)
                {
                    this.log_warn("FIXME: DrawCopy could not handle this QUIC file.");
                    return false;
                }
                var source_img = convert_spice_quic_to_web(canvas.context,
                                        draw_copy.data.src_bitmap.quic);

                return this.draw_copy_helper(
                    { base: draw_copy.base,
                      src_area: draw_copy.data.src_area,
                      image_data: source_img,
                      tag: "copyquic." + draw_copy.data.src_bitmap.quic.type,
                      has_alpha: (draw_copy.data.src_bitmap.quic.type == QUIC_IMAGE_TYPE_RGBA ? true : false) ,
                      descriptor : draw_copy.data.src_bitmap.descriptor
                    });
            }
            else if (draw_copy.data.src_bitmap.descriptor.type == SPICE_IMAGE_TYPE_FROM_CACHE ||
                    draw_copy.data.src_bitmap.descriptor.type == SPICE_IMAGE_TYPE_FROM_CACHE_LOSSLESS)
            {
                if (! this.cache || ! this.cache[draw_copy.data.src_bitmap.descriptor.id])
                {
                    this.log_warn("FIXME: DrawCopy did not find image id " + draw_copy.data.src_bitmap.descriptor.id + " in cache.");
                    return false;
                }

                return this.draw_copy_helper(
                    { base: draw_copy.base,
                      src_area: draw_copy.data.src_area,
                      image_data: this.cache[draw_copy.data.src_bitmap.descriptor.id],
                      tag: "copycache." + draw_copy.data.src_bitmap.descriptor.id, 
                      has_alpha: true, /* FIXME - may want this to be false... */
                      descriptor : draw_copy.data.src_bitmap.descriptor
                    });

                /* FIXME - LOSSLESS CACHE ramifications not understood or handled */
            }
            else if (draw_copy.data.src_bitmap.descriptor.type == SPICE_IMAGE_TYPE_SURFACE)
            {
                var source_context = this.surfaces[draw_copy.data.src_bitmap.surface_id].canvas.context;
                var target_context = this.surfaces[draw_copy.base.surface_id].canvas.context;

                var source_img = source_context.getImageData(
                        draw_copy.data.src_area.left, draw_copy.data.src_area.top,
                        draw_copy.data.src_area.right - draw_copy.data.src_area.left,
                        draw_copy.data.src_area.bottom - draw_copy.data.src_area.top);
                var computed_src_area = new SpiceRect;
                computed_src_area.top = computed_src_area.left = 0;
                computed_src_area.right = source_img.width;
                computed_src_area.bottom = source_img.height;

                /* FIXME - there is a potential optimization here.
                           That is, if the surface is from 0,0, and
                           both surfaces are alpha surfaces, you should
                           be able to just do a drawImage, which should
                           save time.  */

                return this.draw_copy_helper(
                    { base: draw_copy.base,
                      src_area: computed_src_area,
                      image_data: source_img,
                      tag: "copysurf." + draw_copy.data.src_bitmap.surface_id,
                      has_alpha: this.surfaces[draw_copy.data.src_bitmap.surface_id].format == SPICE_SURFACE_FMT_32_xRGB ? false : true,
                      descriptor : draw_copy.data.src_bitmap.descriptor
                    });

                return true;
            }
            else if (draw_copy.data.src_bitmap.descriptor.type == SPICE_IMAGE_TYPE_JPEG)
            {
                if (! draw_copy.data.src_bitmap.jpeg)
                {
                    this.log_warn("FIXME: DrawCopy could not handle this JPEG file.");
                    return false;
                }

                // FIXME - how lame is this.  Be have it in binary format, and we have
                //         to put it into string to get it back into jpeg.  Blech.
                var tmpstr = "data:image/jpeg,";
                var img = new Image;
                var i;
                var qdv = new Uint8Array(draw_copy.data.src_bitmap.jpeg.data);
                for (i = 0; i < qdv.length; i++)
                {
                    tmpstr +=  '%';
                    if (qdv[i] < 16)
                        tmpstr += '0';
                    tmpstr += qdv[i].toString(16);
                }

                img.o = 
                    { base: draw_copy.base,
                      tag: "jpeg." + draw_copy.data.src_bitmap.surface_id,
                      descriptor : draw_copy.data.src_bitmap.descriptor,
                      sc : this,
                    };
                img.onload = handle_draw_jpeg_onload;
                img.src = tmpstr;

                return true;
            }
            else if (draw_copy.data.src_bitmap.descriptor.type == SPICE_IMAGE_TYPE_JPEG_ALPHA)
            {
                if (! draw_copy.data.src_bitmap.jpeg_alpha)
                {
                    this.log_warn("FIXME: DrawCopy could not handle this JPEG ALPHA file.");
                    return false;
                }

                // FIXME - how lame is this.  Be have it in binary format, and we have
                //         to put it into string to get it back into jpeg.  Blech.
                var tmpstr = "data:image/jpeg,";
                var img = new Image;
                var i;
                var qdv = new Uint8Array(draw_copy.data.src_bitmap.jpeg_alpha.data);
                for (i = 0; i < qdv.length; i++)
                {
                    tmpstr +=  '%';
                    if (qdv[i] < 16)
                        tmpstr += '0';
                    tmpstr += qdv[i].toString(16);
                }

                img.o = 
                    { base: draw_copy.base,
                      tag: "jpeg." + draw_copy.data.src_bitmap.surface_id,
                      descriptor : draw_copy.data.src_bitmap.descriptor,
                      sc : this,
                    };

                if (this.surfaces[draw_copy.base.surface_id].format == SPICE_SURFACE_FMT_32_ARGB)
                {

                    var canvas = this.surfaces[draw_copy.base.surface_id].canvas;
                    img.alpha_img = convert_spice_lz_to_web(canvas.context,
                                            draw_copy.data.src_bitmap.jpeg_alpha.alpha);
                }
                img.onload = handle_draw_jpeg_onload;
                img.src = tmpstr;

                return true;
            }
            else if (draw_copy.data.src_bitmap.descriptor.type == SPICE_IMAGE_TYPE_BITMAP)
            {
                var canvas = this.surfaces[draw_copy.base.surface_id].canvas;
                if (! draw_copy.data.src_bitmap.bitmap)
                {
                    this.log_err("null bitmap");
                    return false;
                }

                var source_img = convert_spice_bitmap_to_web(canvas.context,
                                        draw_copy.data.src_bitmap.bitmap);
                if (! source_img)
                {
                    this.log_warn("FIXME: Unable to interpret bitmap of format: " + 
                        draw_copy.data.src_bitmap.bitmap.format);
                    return false;
                }

                return this.draw_copy_helper(
                    { base: draw_copy.base,
                      src_area: draw_copy.data.src_area,
                      image_data: source_img,
                      tag: "bitmap." + draw_copy.data.src_bitmap.bitmap.format,
                      has_alpha: draw_copy.data.src_bitmap.bitmap == SPICE_BITMAP_FMT_32BIT ? false : true,
                      descriptor : draw_copy.data.src_bitmap.descriptor
                    });
            }
            else if (draw_copy.data.src_bitmap.descriptor.type == SPICE_IMAGE_TYPE_LZ_RGB)
            {
                var canvas = this.surfaces[draw_copy.base.surface_id].canvas;
                if (! draw_copy.data.src_bitmap.lz_rgb)
                {
                    this.log_err("null lz_rgb ");
                    return false;
                }

                if (draw_copy.data.src_bitmap.lz_rgb.top_down != 1)
                    this.log_warn("FIXME: Implement non top down support for lz_rgb");

                var source_img = convert_spice_lz_to_web(canvas.context,
                                            draw_copy.data.src_bitmap.lz_rgb);
                if (! source_img)
                {
                    this.log_warn("FIXME: Unable to interpret bitmap of type: " + 
                        draw_copy.data.src_bitmap.lz_rgb.type);
                    return false;
                }

                return this.draw_copy_helper(
                    { base: draw_copy.base,
                      src_area: draw_copy.data.src_area,
                      image_data: source_img,
                      tag: "lz_rgb." + draw_copy.data.src_bitmap.lz_rgb.type,
                      has_alpha: draw_copy.data.src_bitmap.lz_rgb.type == LZ_IMAGE_TYPE_RGBA ? true : false ,
                      descriptor : draw_copy.data.src_bitmap.descriptor
                    });
            }
            else
            {
                this.log_warn("FIXME: DrawCopy unhandled image type: " + draw_copy.data.src_bitmap.descriptor.type);
                this.log_draw("DrawCopy", draw_copy);
                return false;
            }
        }

        this.log_warn("FIXME: DrawCopy no src_bitmap.");
        return false;
    }

    if (msg.type == SPICE_MSG_DISPLAY_DRAW_FILL)
    {
        var draw_fill = new SpiceMsgDisplayDrawFill(msg.data);

        DEBUG > 1 && this.log_draw("DrawFill", draw_fill);

        if (draw_fill.data.rop_descriptor != SPICE_ROPD_OP_PUT)
            this.log_warn("FIXME: DrawFill we don't handle ropd type: " + draw_fill.data.rop_descriptor);
        if (draw_fill.data.mask.flags)
            this.log_warn("FIXME: DrawFill we don't handle mask flag: " + draw_fill.data.mask.flags);
        if (draw_fill.data.mask.bitmap)
            this.log_warn("FIXME: DrawFill we don't handle mask");

        if (draw_fill.data.brush.type == SPICE_BRUSH_TYPE_SOLID)
        {
            // FIXME - do brushes ever have alpha?
            var color = draw_fill.data.brush.color & 0xffffff;
            var color_str = "rgb(" + (color >> 16) + ", " + ((color >> 8) & 0xff) + ", " + (color & 0xff) + ")";
            this.surfaces[draw_fill.base.surface_id].canvas.context.fillStyle = color_str;

            this.surfaces[draw_fill.base.surface_id].canvas.context.fillRect(
                draw_fill.base.box.left, draw_fill.base.box.top,
                draw_fill.base.box.right - draw_fill.base.box.left,
                draw_fill.base.box.bottom - draw_fill.base.box.top);

            if (DUMP_DRAWS && this.parent.dump_id)
            {
                var debug_canvas = document.createElement("canvas");
                debug_canvas.setAttribute('width', this.surfaces[draw_fill.base.surface_id].canvas.width);
                debug_canvas.setAttribute('height', this.surfaces[draw_fill.base.surface_id].canvas.height);
                debug_canvas.setAttribute('id', "fillbrush." + draw_fill.base.surface_id + "." + this.surfaces[draw_fill.base.surface_id].draw_count);
                debug_canvas.getContext("2d").fillStyle = color_str;
                debug_canvas.getContext("2d").fillRect(
                    draw_fill.base.box.left, draw_fill.base.box.top,
                    draw_fill.base.box.right - draw_fill.base.box.left,
                    draw_fill.base.box.bottom - draw_fill.base.box.top);
                document.getElementById(this.parent.dump_id).appendChild(debug_canvas);
            }
                
            this.surfaces[draw_fill.base.surface_id].draw_count++;

        }
        else
        {
            this.log_warn("FIXME: DrawFill can't handle brush type: " + draw_fill.data.brush.type);
        }
        return true;
    }

    if (msg.type == SPICE_MSG_DISPLAY_COPY_BITS)
    {
        var copy_bits = new SpiceMsgDisplayCopyBits(msg.data);

        DEBUG > 1 && this.log_draw("CopyBits", copy_bits);

        var source_canvas = this.surfaces[copy_bits.base.surface_id].canvas;
        var source_context = source_canvas.context;

        var width = source_canvas.width - copy_bits.src_pos.x;
        var height = source_canvas.height - copy_bits.src_pos.y;
        if (width > (copy_bits.base.box.right - copy_bits.base.box.left))
            width = copy_bits.base.box.right - copy_bits.base.box.left;
        if (height > (copy_bits.base.box.bottom - copy_bits.base.box.top))
            height = copy_bits.base.box.bottom - copy_bits.base.box.top;

        var source_img = source_context.getImageData(
                copy_bits.src_pos.x, copy_bits.src_pos.y, width, height);
        //source_context.putImageData(source_img, copy_bits.base.box.left, copy_bits.base.box.top);
        putImageDataWithAlpha(source_context, source_img, copy_bits.base.box.left, copy_bits.base.box.top);

        if (DUMP_DRAWS && this.parent.dump_id)
        {
            var debug_canvas = document.createElement("canvas");
            debug_canvas.setAttribute('width', width);
            debug_canvas.setAttribute('height', height);
            debug_canvas.setAttribute('id', "copybits" + copy_bits.base.surface_id + "." + this.surfaces[copy_bits.base.surface_id].draw_count);
            debug_canvas.getContext("2d").putImageData(source_img, 0, 0);
            document.getElementById(this.parent.dump_id).appendChild(debug_canvas);
        }


        this.surfaces[copy_bits.base.surface_id].draw_count++;
        return true;
    }

    if (msg.type == SPICE_MSG_DISPLAY_INVAL_ALL_PALETTES)
    {
        this.known_unimplemented(msg.type, "Inval All Palettes");
        return true;
    }

    if (msg.type == SPICE_MSG_DISPLAY_SURFACE_CREATE)
    {
        if (! ("surfaces" in this))
            this.surfaces = [];

        var m = new SpiceMsgSurfaceCreate(msg.data);
        DEBUG > 1 && console.log(this.type + ": MsgSurfaceCreate id " + m.surface.surface_id 
                                    + "; " + m.surface.width + "x" + m.surface.height
                                    + "; format " + m.surface.format 
                                    + "; flags " + m.surface.flags);
        if (m.surface.format != SPICE_SURFACE_FMT_32_xRGB &&
            m.surface.format != SPICE_SURFACE_FMT_32_ARGB)
        {
            this.log_warn("FIXME: cannot handle surface format " + m.surface.format + " yet.");
            return false;
        }

        var canvas = document.createElement("canvas");
        canvas.setAttribute('width', m.surface.width);
        canvas.setAttribute('height', m.surface.height);
        canvas.setAttribute('id', "spice_surface_" + m.surface.surface_id);
        canvas.setAttribute('tabindex', m.surface.surface_id);
        canvas.context = canvas.getContext("2d");

        if (DUMP_CANVASES && this.parent.dump_id)
            document.getElementById(this.parent.dump_id).appendChild(canvas);

        m.surface.canvas = canvas;
        m.surface.draw_count = 0;
        this.surfaces[m.surface.surface_id] = m.surface;

        if (m.surface.flags & SPICE_SURFACE_FLAGS_PRIMARY)
        {
            this.primary_surface = m.surface.surface_id;

            /* This .save() is done entirely to enable SPICE_MSG_DISPLAY_RESET */
            canvas.context.save();
            document.getElementById(this.parent.screen_id).appendChild(canvas);

            /* We're going to leave width dynamic, but correctly set the height */
            document.getElementById(this.parent.screen_id).style.height = m.surface.height + "px";
            this.hook_events();
        }
        return true;
    }

    if (msg.type == SPICE_MSG_DISPLAY_SURFACE_DESTROY)
    {
        var m = new SpiceMsgSurfaceDestroy(msg.data);
        DEBUG > 1 && console.log(this.type + ": MsgSurfaceDestroy id " + m.surface_id);
        this.delete_surface(m.surface_id);
        return true;
    }

    if (msg.type == SPICE_MSG_DISPLAY_STREAM_CREATE)
    {
        var m = new SpiceMsgDisplayStreamCreate(msg.data);
        DEBUG > 1 && console.log(this.type + ": MsgStreamCreate id" + m.id);
        if (!this.streams)
            this.streams = new Array();
        if (this.streams[m.id])
            console.log("Stream already exists");
        else
            this.streams[m.id] = m;
        if (m.codec_type != SPICE_VIDEO_CODEC_TYPE_MJPEG)
            console.log("Unhandled stream codec: "+m.codec_type);
        return true;
    }

    if (msg.type == SPICE_MSG_DISPLAY_STREAM_DATA)
    {
        var m = new SpiceMsgDisplayStreamData(msg.data);
        if (!this.streams[m.base.id])
        {
            console.log("no stream for data");
            return false;
        }
        if (this.streams[m.base.id].codec_type === SPICE_VIDEO_CODEC_TYPE_MJPEG)
        {
            var tmpstr = "data:image/jpeg,";
            var img = new Image;
            var i;
            for (i = 0; i < m.data.length; i++)
            {
                tmpstr +=  '%';
                if (m.data[i] < 16)
                tmpstr += '0';
                tmpstr += m.data[i].toString(16);
            }
            var strm_base = new SpiceMsgDisplayBase();
            strm_base.surface_id = this.streams[m.base.id].surface_id;
            strm_base.box = this.streams[m.base.id].dest;
            strm_base.clip = this.streams[m.base.id].clip;
            img.o =
                { base: strm_base,
                  tag: "mjpeg." + m.base.id,
                  descriptor: null,
                  sc : this,
                };
            img.onload = handle_draw_jpeg_onload;
            img.src = tmpstr;
        }
        return true;
    }

    if (msg.type == SPICE_MSG_DISPLAY_STREAM_CLIP)
    {
        var m = new SpiceMsgDisplayStreamClip(msg.data);
        DEBUG > 1 && console.log(this.type + ": MsgStreamClip id" + m.id);
        this.streams[m.id].clip = m.clip;
        return true;
    }

    if (msg.type == SPICE_MSG_DISPLAY_STREAM_DESTROY)
    {
        var m = new SpiceMsgDisplayStreamDestroy(msg.data);
        DEBUG > 1 && console.log(this.type + ": MsgStreamDestroy id" + m.id);
        this.streams[m.id] = undefined;
        return true;
    }
    if (msg.type == SPICE_MSG_DISPLAY_INVAL_LIST)
    {
        var m = new SpiceMsgDisplayInvalList(msg.data);
        var i;
        DEBUG > 1 && console.log(this.type + ": MsgInvalList " + m.count + " items");
        for (i = 0; i < m.count; i++)
            if (this.cache[m.resources[i].id] != undefined)
                delete this.cache[m.resources[i].id];
        return true;
    }

    return false;
}

SpiceDisplayConn.prototype.delete_surface = function(surface_id)
{
    var canvas = document.getElementById("spice_surface_" + surface_id);
    if (DUMP_CANVASES && this.parent.dump_id)
        document.getElementById(this.parent.dump_id).removeChild(canvas);
    if (this.primary_surface == surface_id)
    {
        this.unhook_events();
        this.primary_surface = undefined;
        document.getElementById(this.parent.screen_id).removeChild(canvas);
    }

    delete this.surfaces[surface_id];
}


SpiceDisplayConn.prototype.draw_copy_helper = function(o)
{

    var canvas = this.surfaces[o.base.surface_id].canvas;
    if (o.has_alpha)
    {
        /* FIXME - This is based on trial + error, not a serious thoughtful
                   analysis of what Spice requires.  See display.js for more. */
        if (this.surfaces[o.base.surface_id].format == SPICE_SURFACE_FMT_32_xRGB)
        {
            stripAlpha(o.image_data);
            canvas.context.putImageData(o.image_data, o.base.box.left, o.base.box.top);
        }
        else
            putImageDataWithAlpha(canvas.context, o.image_data,
                    o.base.box.left, o.base.box.top);
    }
    else
        canvas.context.putImageData(o.image_data, o.base.box.left, o.base.box.top);

    if (o.src_area.left > 0 || o.src_area.top > 0)
    {
        this.log_warn("FIXME: DrawCopy not shifting draw copies just yet...");
    }

    if (o.descriptor && (o.descriptor.flags & SPICE_IMAGE_FLAGS_CACHE_ME))
    {
        if (! ("cache" in this))
            this.cache = {};
        this.cache[o.descriptor.id] = o.image_data;
    }

    if (DUMP_DRAWS && this.parent.dump_id)
    {
        var debug_canvas = document.createElement("canvas");
        debug_canvas.setAttribute('width', o.image_data.width);
        debug_canvas.setAttribute('height', o.image_data.height);
        debug_canvas.setAttribute('id', o.tag + "." +
            this.surfaces[o.base.surface_id].draw_count + "." +
            o.base.surface_id + "@" + o.base.box.left + "x" +  o.base.box.top);
        debug_canvas.getContext("2d").putImageData(o.image_data, 0, 0);
        document.getElementById(this.parent.dump_id).appendChild(debug_canvas);
    }

    this.surfaces[o.base.surface_id].draw_count++;

    return true;
}


SpiceDisplayConn.prototype.log_draw = function(prefix, draw)
{
    var str = prefix + "." + draw.base.surface_id + "." + this.surfaces[draw.base.surface_id].draw_count + ": ";
    str += "base.box " + draw.base.box.left + ", " + draw.base.box.top + " to " + 
                           draw.base.box.right + ", " + draw.base.box.bottom;
    str += "; clip.type " + draw.base.clip.type;

    if (draw.data)
    {
        if (draw.data.src_area)
            str += "; src_area " + draw.data.src_area.left + ", " + draw.data.src_area.top + " to "
                                 + draw.data.src_area.right + ", " + draw.data.src_area.bottom;

        if (draw.data.src_bitmap && draw.data.src_bitmap != null)
        {
            str += "; src_bitmap id: " + draw.data.src_bitmap.descriptor.id;
            str += "; src_bitmap width " + draw.data.src_bitmap.descriptor.width + ", height " + draw.data.src_bitmap.descriptor.height;
            str += "; src_bitmap type " + draw.data.src_bitmap.descriptor.type + ", flags " + draw.data.src_bitmap.descriptor.flags;
            if (draw.data.src_bitmap.surface_id !== undefined)
                str += "; src_bitmap surface_id " + draw.data.src_bitmap.surface_id;
            if (draw.data.src_bitmap.quic)
                str += "; QUIC type " + draw.data.src_bitmap.quic.type + 
                        "; width " + draw.data.src_bitmap.quic.width + 
                        "; height " + draw.data.src_bitmap.quic.height ;
            if (draw.data.src_bitmap.lz_rgb)
                str += "; LZ_RGB length " + draw.data.src_bitmap.lz_rgb.length +
                       "; magic " + draw.data.src_bitmap.lz_rgb.magic + 
                       "; version 0x" + draw.data.src_bitmap.lz_rgb.version.toString(16) +
                       "; type " + draw.data.src_bitmap.lz_rgb.type +
                       "; width " + draw.data.src_bitmap.lz_rgb.width +
                       "; height " + draw.data.src_bitmap.lz_rgb.height +
                       "; stride " + draw.data.src_bitmap.lz_rgb.stride +
                       "; top down " + draw.data.src_bitmap.lz_rgb.top_down;
        }
        else
            str += "; src_bitmap is null";

        if (draw.data.brush)
        {
            if (draw.data.brush.type == SPICE_BRUSH_TYPE_SOLID)
                str += "; brush.color 0x" + draw.data.brush.color.toString(16);
            if (draw.data.brush.type == SPICE_BRUSH_TYPE_PATTERN)
            {
                str += "; brush.pat ";
                if (draw.data.brush.pattern.pat != null)
                    str += "[SpiceImage]";
                else
                    str += "[null]";
                str += " at " + draw.data.brush.pattern.pos.x + ", " + draw.data.brush.pattern.pos.y;
            }
        }

        str += "; rop_descriptor " + draw.data.rop_descriptor;
        if (draw.data.scale_mode !== undefined)
            str += "; scale_mode " + draw.data.scale_mode;
        str += "; mask.flags " + draw.data.mask.flags;
        str += "; mask.pos " + draw.data.mask.pos.x + ", " + draw.data.mask.pos.y;
        if (draw.data.mask.bitmap != null)
        {
            str += "; mask.bitmap width " + draw.data.mask.bitmap.descriptor.width + ", height " + draw.data.mask.bitmap.descriptor.height;
            str += "; mask.bitmap type " + draw.data.mask.bitmap.descriptor.type + ", flags " + draw.data.mask.bitmap.descriptor.flags;
        }
        else
            str += "; mask.bitmap is null";
    }

    console.log(str);
}

SpiceDisplayConn.prototype.hook_events = function()
{
    if (this.primary_surface !== undefined)
    {
        var canvas = this.surfaces[this.primary_surface].canvas;
        canvas.sc = this.parent;
        canvas.addEventListener('mousemove', handle_mousemove);
        canvas.addEventListener('mousedown', handle_mousedown);
        canvas.addEventListener('contextmenu', handle_contextmenu);
        canvas.addEventListener('mouseup', handle_mouseup);
        canvas.addEventListener('keydown', handle_keydown);
        canvas.addEventListener('keyup', handle_keyup);
        canvas.addEventListener('mouseout', handle_mouseout);
        canvas.addEventListener('mouseover', handle_mouseover);
        canvas.addEventListener('wheel', handle_mousewheel);
        canvas.focus();
    }
}

SpiceDisplayConn.prototype.unhook_events = function()
{
    if (this.primary_surface !== undefined)
    {
        var canvas = this.surfaces[this.primary_surface].canvas;
        canvas.removeEventListener('mousemove', handle_mousemove);
        canvas.removeEventListener('mousedown', handle_mousedown);
        canvas.removeEventListener('contextmenu', handle_contextmenu);
        canvas.removeEventListener('mouseup', handle_mouseup);
        canvas.removeEventListener('keydown', handle_keydown);
        canvas.removeEventListener('keyup', handle_keyup);
        canvas.removeEventListener('mouseout', handle_mouseout);
        canvas.removeEventListener('mouseover', handle_mouseover);
        canvas.removeEventListener('wheel', handle_mousewheel);
    }
}


SpiceDisplayConn.prototype.destroy_surfaces = function()
{
    for (var s in this.surfaces)
    {
        this.delete_surface(this.surfaces[s].surface_id);
    }

    this.surfaces = undefined;
}


function handle_mouseover(e)
{
    this.focus();
}

function handle_mouseout(e)
{
    if (this.sc && this.sc.cursor && this.sc.cursor.spice_simulated_cursor)
        this.sc.cursor.spice_simulated_cursor.style.display = 'none';
    this.blur();
}

function handle_draw_jpeg_onload()
{
    var temp_canvas = null;
    var context;

    /*------------------------------------------------------------
    ** FIXME:
    **  The helper should be extended to be able to handle actual HtmlImageElements
    **  ...and the cache should be modified to do so as well
    **----------------------------------------------------------*/
    if (this.o.sc.surfaces[this.o.base.surface_id] === undefined)
    {
        // This can happen; if the jpeg image loads after our surface
        //  has been destroyed (e.g. open a menu, close it quickly),
        //  we'll find we have no surface.  
        DEBUG > 2 && this.o.sc.log_info("Discarding jpeg; presumed lost surface " + this.o.base.surface_id);
        temp_canvas = document.createElement("canvas");
        temp_canvas.setAttribute('width', this.o.base.box.right);
        temp_canvas.setAttribute('height', this.o.base.box.bottom);
        context = temp_canvas.getContext("2d");
    }
    else
        context = this.o.sc.surfaces[this.o.base.surface_id].canvas.context;

    if (this.alpha_img)
    {
        var c = document.createElement("canvas");
        var t = c.getContext("2d");
        c.setAttribute('width', this.alpha_img.width);
        c.setAttribute('height', this.alpha_img.height);
        t.putImageData(this.alpha_img, 0, 0);
        t.globalCompositeOperation = 'source-in';
        t.drawImage(this, 0, 0);
     
        context.drawImage(c, this.o.base.box.left, this.o.base.box.top);

        if (this.o.descriptor && 
            (this.o.descriptor.flags & SPICE_IMAGE_FLAGS_CACHE_ME))
        {
            if (! ("cache" in this.o.sc))
                this.o.sc.cache = {};

            this.o.sc.cache[this.o.descriptor.id] = 
                t.getImageData(0, 0,
                    this.alpha_img.width,
                    this.alpha_img.height);
        }
    }
    else
    {
        context.drawImage(this, this.o.base.box.left, this.o.base.box.top);

        // Give the Garbage collector a clue to recycle this; avoids
        //  fairly massive memory leaks during video playback
        this.src = null;

        if (this.o.descriptor && 
            (this.o.descriptor.flags & SPICE_IMAGE_FLAGS_CACHE_ME))
        {
            if (! ("cache" in this.o.sc))
                this.o.sc.cache = {};

            this.o.sc.cache[this.o.descriptor.id] = 
                context.getImageData(this.o.base.box.left, this.o.base.box.top,
                    this.o.base.box.right - this.o.base.box.left,
                    this.o.base.box.bottom - this.o.base.box.top);
        }
    }

    if (temp_canvas == null)
    {
        if (DUMP_DRAWS && this.o.sc.parent.dump_id)
        {
            var debug_canvas = document.createElement("canvas");
            debug_canvas.setAttribute('id', this.o.tag + "." +
                this.o.sc.surfaces[this.o.base.surface_id].draw_count + "." +
                this.o.base.surface_id + "@" + this.o.base.box.left + "x" +  this.o.base.box.top);
            debug_canvas.getContext("2d").drawImage(this, 0, 0);
            document.getElementById(this.o.sc.parent.dump_id).appendChild(debug_canvas);
        }

        this.o.sc.surfaces[this.o.base.surface_id].draw_count++;
    }
}
;
define("spice-display", ["spice-spiceconn"], function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*----------------------------------------------------------------------------
 ** Modifier Keystates
 **     These need to be tracked because focus in and out can get the keyboard
 **     out of sync.
 **------------------------------------------------------------------------*/
var Shift_state = -1;
var Ctrl_state = -1;
var Alt_state = -1;
var Meta_state = -1;

/*----------------------------------------------------------------------------
**  SpiceInputsConn
**      Drive the Spice Inputs channel (e.g. mouse + keyboard)
**--------------------------------------------------------------------------*/
function SpiceInputsConn()
{
    SpiceConn.apply(this, arguments);

    this.mousex = undefined;
    this.mousey = undefined;
    this.button_state = 0;
    this.waiting_for_ack = 0;
}

SpiceInputsConn.prototype = Object.create(SpiceConn.prototype);
SpiceInputsConn.prototype.process_channel_message = function(msg)
{
    if (msg.type == SPICE_MSG_INPUTS_INIT)
    {
        var inputs_init = new SpiceMsgInputsInit(msg.data);
        this.keyboard_modifiers = inputs_init.keyboard_modifiers;
        DEBUG > 1 && console.log("MsgInputsInit - modifier " + this.keyboard_modifiers);
        // FIXME - We don't do anything with the keyboard modifiers...
        return true;
    }
    if (msg.type == SPICE_MSG_INPUTS_KEY_MODIFIERS)
    {
        var key = new SpiceMsgInputsKeyModifiers(msg.data);
        this.keyboard_modifiers = key.keyboard_modifiers;
        DEBUG > 1 && console.log("MsgInputsKeyModifiers - modifier " + this.keyboard_modifiers);
        // FIXME - We don't do anything with the keyboard modifiers...
        return true;
    }
    if (msg.type == SPICE_MSG_INPUTS_MOUSE_MOTION_ACK)
    {
        DEBUG > 1 && console.log("mouse motion ack");
        this.waiting_for_ack -= SPICE_INPUT_MOTION_ACK_BUNCH;
        return true;
    }
    return false;
}



function handle_mousemove(e)
{
    var msg = new SpiceMiniData();
    var move;
    if (this.sc.mouse_mode == SPICE_MOUSE_MODE_CLIENT)
    {
        move = new SpiceMsgcMousePosition(this.sc, e)
        msg.build_msg(SPICE_MSGC_INPUTS_MOUSE_POSITION, move);
    }
    else
    {
        move = new SpiceMsgcMouseMotion(this.sc, e)
        msg.build_msg(SPICE_MSGC_INPUTS_MOUSE_MOTION, move);
    }
    if (this.sc && this.sc.inputs && this.sc.inputs.state === "ready")
    {
        if (this.sc.inputs.waiting_for_ack < (2 * SPICE_INPUT_MOTION_ACK_BUNCH))
        {
            this.sc.inputs.send_msg(msg);
            this.sc.inputs.waiting_for_ack++;
        }
        else
        {
            DEBUG > 0 && this.sc.log_info("Discarding mouse motion");
        }
    }

    if (this.sc && this.sc.cursor && this.sc.cursor.spice_simulated_cursor)
    {
        this.sc.cursor.spice_simulated_cursor.style.display = 'block';
        this.sc.cursor.spice_simulated_cursor.style.left = e.pageX - this.sc.cursor.spice_simulated_cursor.spice_hot_x + 'px';
        this.sc.cursor.spice_simulated_cursor.style.top = e.pageY - this.sc.cursor.spice_simulated_cursor.spice_hot_y + 'px';
        e.preventDefault();
    }

}

function handle_mousedown(e)
{
    var press = new SpiceMsgcMousePress(this.sc, e)
    var msg = new SpiceMiniData();
    msg.build_msg(SPICE_MSGC_INPUTS_MOUSE_PRESS, press);
    if (this.sc && this.sc.inputs && this.sc.inputs.state === "ready")
        this.sc.inputs.send_msg(msg);

    e.preventDefault();
}

function handle_contextmenu(e)
{
    e.preventDefault();
    return false;
}

function handle_mouseup(e)
{
    var release = new SpiceMsgcMouseRelease(this.sc, e)
    var msg = new SpiceMiniData();
    msg.build_msg(SPICE_MSGC_INPUTS_MOUSE_RELEASE, release);
    if (this.sc && this.sc.inputs && this.sc.inputs.state === "ready")
        this.sc.inputs.send_msg(msg);

    e.preventDefault();
}

function handle_mousewheel(e)
{
    var press = new SpiceMsgcMousePress;
    var release = new SpiceMsgcMouseRelease;
    if (e.deltaY < 0)
        press.button = release.button = SPICE_MOUSE_BUTTON_UP;
    else
        press.button = release.button = SPICE_MOUSE_BUTTON_DOWN;
    press.buttons_state = 0;
    release.buttons_state = 0;

    var msg = new SpiceMiniData();
    msg.build_msg(SPICE_MSGC_INPUTS_MOUSE_PRESS, press);
    if (this.sc && this.sc.inputs && this.sc.inputs.state === "ready")
        this.sc.inputs.send_msg(msg);

    msg.build_msg(SPICE_MSGC_INPUTS_MOUSE_RELEASE, release);
    if (this.sc && this.sc.inputs && this.sc.inputs.state === "ready")
        this.sc.inputs.send_msg(msg);

    e.preventDefault();
}

function handle_keydown(e)
{
    var key = new SpiceMsgcKeyDown(e)
    var msg = new SpiceMiniData();
    check_and_update_modifiers(e, key.code, this.sc);
    msg.build_msg(SPICE_MSGC_INPUTS_KEY_DOWN, key);
    if (this.sc && this.sc.inputs && this.sc.inputs.state === "ready")
        this.sc.inputs.send_msg(msg);

    e.preventDefault();
}

function handle_keyup(e)
{
    var key = new SpiceMsgcKeyUp(e)
    var msg = new SpiceMiniData();
    check_and_update_modifiers(e, key.code, this.sc);
    msg.build_msg(SPICE_MSGC_INPUTS_KEY_UP, key);
    if (this.sc && this.sc.inputs && this.sc.inputs.state === "ready")
        this.sc.inputs.send_msg(msg);

    e.preventDefault();
}

function sendCtrlAltDel()
{
    if (sc && sc.inputs && sc.inputs.state === "ready"){
        var key = new SpiceMsgcKeyDown();
        var msg = new SpiceMiniData();

        update_modifier(true, KEY_LCtrl, sc);
        update_modifier(true, KEY_Alt, sc);

        key.code = KEY_KP_Decimal;
        msg.build_msg(SPICE_MSGC_INPUTS_KEY_DOWN, key);
        sc.inputs.send_msg(msg);
        msg.build_msg(SPICE_MSGC_INPUTS_KEY_UP, key);
        sc.inputs.send_msg(msg);

        if(Ctrl_state == false) update_modifier(false, KEY_LCtrl, sc);
        if(Alt_state == false) update_modifier(false, KEY_Alt, sc);
    }
}

function update_modifier(state, code, sc)
{
    var msg = new SpiceMiniData();
    if (!state)
    {
        var key = new SpiceMsgcKeyUp()
        key.code =(0x80|code);
        msg.build_msg(SPICE_MSGC_INPUTS_KEY_UP, key);
    }
    else
    {
        var key = new SpiceMsgcKeyDown()
        key.code = code;
        msg.build_msg(SPICE_MSGC_INPUTS_KEY_DOWN, key);
    }

    sc.inputs.send_msg(msg);
}

function check_and_update_modifiers(e, code, sc)
{
    if (Shift_state === -1)
    {
        Shift_state = e.shiftKey;
        Ctrl_state = e.ctrlKey;
        Alt_state = e.altKey;
        Meta_state = e.metaKey;
    }

    if (code === KEY_ShiftL)
        Shift_state = true;
    else if (code === KEY_Alt)
        Alt_state = true;
    else if (code === KEY_LCtrl)
        Ctrl_state = true;
    else if (code === 0xE0B5)
        Meta_state = true;
    else if (code === (0x80|KEY_ShiftL))
        Shift_state = false;
    else if (code === (0x80|KEY_Alt))
        Alt_state = false;
    else if (code === (0x80|KEY_LCtrl))
        Ctrl_state = false;
    else if (code === (0x80|0xE0B5))
        Meta_state = false;

    if (sc && sc.inputs && sc.inputs.state === "ready")
    {
        if (Shift_state != e.shiftKey)
        {
            console.log("Shift state out of sync");
            update_modifier(e.shiftKey, KEY_ShiftL, sc);
            Shift_state = e.shiftKey;
        }
        if (Alt_state != e.altKey)
        {
            console.log("Alt state out of sync");
            update_modifier(e.altKey, KEY_Alt, sc);
            Alt_state = e.altKey;
        }
        if (Ctrl_state != e.ctrlKey)
        {
            console.log("Ctrl state out of sync");
            update_modifier(e.ctrlKey, KEY_LCtrl, sc);
            Ctrl_state = e.ctrlKey;
        }
        if (Meta_state != e.metaKey)
        {
            console.log("Meta state out of sync");
            update_modifier(e.metaKey, 0xE0B5, sc);
            Meta_state = e.metaKey;
        }
    }
}
;
define("spice-inputs", ["spice-spiceconn"], function(){});


/*
   Copyright (C) 2014 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/


/*----------------------------------------------------------------------------
**  EBML identifiers
**--------------------------------------------------------------------------*/
var EBML_HEADER =                           [ 0x1a, 0x45, 0xdf, 0xa3 ];
var EBML_HEADER_VERSION =                   [ 0x42, 0x86 ];
var EBML_HEADER_READ_VERSION =              [ 0x42, 0xf7 ];
var EBML_HEADER_MAX_ID_LENGTH =             [ 0x42, 0xf2 ];
var EBML_HEADER_MAX_SIZE_LENGTH =           [ 0x42, 0xf3 ];
var EBML_HEADER_DOC_TYPE =                  [ 0x42, 0x82 ];
var EBML_HEADER_DOC_TYPE_VERSION =          [ 0x42, 0x87 ];
var EBML_HEADER_DOC_TYPE_READ_VERSION =     [ 0x42, 0x85 ];

var WEBM_SEGMENT_HEADER =                   [ 0x18, 0x53, 0x80, 0x67 ];
var WEBM_SEGMENT_INFORMATION =              [ 0x15, 0x49, 0xA9, 0x66 ];

var WEBM_TIMECODE_SCALE =                   [ 0x2A, 0xD7, 0xB1 ];
var WEBM_MUXING_APP =                       [ 0x4D, 0x80 ];
var WEBM_WRITING_APP =                      [ 0x57, 0x41 ];

var WEBM_SEEK_HEAD =                        [ 0x11, 0x4D, 0x9B, 0x74 ];
var WEBM_SEEK =                             [ 0x4D, 0xBB ];
var WEBM_SEEK_ID =                          [ 0x53, 0xAB ];
var WEBM_SEEK_POSITION =                    [ 0x53, 0xAC ];

var WEBM_TRACKS =                           [ 0x16, 0x54, 0xAE, 0x6B ];
var WEBM_TRACK_ENTRY =                      [ 0xAE ];
var WEBM_TRACK_NUMBER =                     [ 0xD7 ];
var WEBM_TRACK_UID =                        [ 0x73, 0xC5 ];
var WEBM_TRACK_TYPE =                       [ 0x83 ];
var WEBM_FLAG_ENABLED =                     [ 0xB9 ];
var WEBM_FLAG_DEFAULT =                     [ 0x88 ];
var WEBM_FLAG_FORCED =                      [ 0x55, 0xAA ];
var WEBM_FLAG_LACING =                      [ 0x9C ];
var WEBM_MIN_CACHE =                        [ 0x6D, 0xE7 ];

var WEBM_MAX_BLOCK_ADDITION_ID =            [ 0x55, 0xEE ];
var WEBM_CODEC_DECODE_ALL =                 [ 0xAA ];
var WEBM_SEEK_PRE_ROLL =                    [ 0x56, 0xBB ];
var WEBM_CODEC_DELAY =                      [ 0x56, 0xAA ];
var WEBM_CODEC_PRIVATE =                    [ 0x63, 0xA2 ];
var WEBM_CODEC_ID =                         [ 0x86 ];

var WEBM_AUDIO =                            [ 0xE1 ] ;
var WEBM_SAMPLING_FREQUENCY =               [ 0xB5 ] ;
var WEBM_CHANNELS =                         [ 0x9F ] ;

var WEBM_CLUSTER =                          [ 0x1F, 0x43, 0xB6, 0x75 ];
var WEBM_TIME_CODE =                        [ 0xE7 ] ;
var WEBM_SIMPLE_BLOCK =                     [ 0xA3 ] ;

/*----------------------------------------------------------------------------
**  Various OPUS / Webm constants
**--------------------------------------------------------------------------*/
var CLUSTER_SIMPLEBLOCK_FLAG_KEYFRAME       = 1 << 7;

var OPUS_FREQUENCY                          = 48000;
var OPUS_CHANNELS                           = 2;

var SPICE_PLAYBACK_CODEC                    = 'audio/webm; codecs="opus"';
var MAX_CLUSTER_TIME                        = 1000;

var GAP_DETECTION_THRESHOLD                 = 50;

/*----------------------------------------------------------------------------
**  EBML utility functions
**      These classes can create the binary representation of a webm file
**--------------------------------------------------------------------------*/
function EBML_write_u1_data_len(len, dv, at)
{
    var b = 0x80 | len;
    dv.setUint8(at, b);
    return at + 1;
}

function EBML_write_u8_value(id, val, dv, at)
{
    at = EBML_write_array(id, dv, at);
    at = EBML_write_u1_data_len(1, dv, at);
    dv.setUint8(at, val);
    return at + 1;
}

function EBML_write_u32_value(id, val, dv, at)
{
    at = EBML_write_array(id, dv, at);
    at = EBML_write_u1_data_len(4, dv, at);
    dv.setUint32(at, val);
    return at + 4;
}

function EBML_write_u16_value(id, val, dv, at)
{
    at = EBML_write_array(id, dv, at);
    at = EBML_write_u1_data_len(2, dv, at);
    dv.setUint16(at, val);
    return at + 2;
}

function EBML_write_float_value(id, val, dv, at)
{
    at = EBML_write_array(id, dv, at);
    at = EBML_write_u1_data_len(4, dv, at);
    dv.setFloat32(at, val);
    return at + 4;
}



function EBML_write_u64_data_len(len, dv, at)
{
    /* Javascript doesn't do 64 bit ints, so this cheats and
        just has a max of 32 bits.  Fine for our purposes */
    dv.setUint8(at++, 0x01);
    dv.setUint8(at++, 0x00);
    dv.setUint8(at++, 0x00);
    dv.setUint8(at++, 0x00);
    var val = len & 0xFFFFFFFF;
    for (var shift = 24; shift >= 0; shift -= 8)
        dv.setUint8(at++, val >> shift);
    return at;
}

function EBML_write_array(arr, dv, at)
{
    for (var i = 0; i < arr.length; i++)
        dv.setUint8(at + i, arr[i]);
    return at + arr.length;
}

function EBML_write_string(str, dv, at)
{
    for (var i = 0; i < str.length; i++)
        dv.setUint8(at + i, str.charCodeAt(i));
    return at + str.length;
}

function EBML_write_data(id, data, dv, at)
{
    at = EBML_write_array(id, dv, at);
    if (data.length < 127)
        at = EBML_write_u1_data_len(data.length, dv, at);
    else
        at = EBML_write_u64_data_len(data.length, dv, at);
    if ((typeof data) == "string")
        at = EBML_write_string(data, dv, at);
    else
        at = EBML_write_array(data, dv, at);
    return at;
}

/*----------------------------------------------------------------------------
**  Webm objects
**      These classes can create the binary representation of a webm file
**--------------------------------------------------------------------------*/
function EBMLHeader()
{
    this.id = EBML_HEADER;
    this.Version = 1;
    this.ReadVersion = 1;
    this.MaxIDLength = 4;
    this.MaxSizeLength = 8;
    this.DocType = "webm";
    this.DocTypeVersion = 2;  /* Not well specified by the WebM guys, but functionally required for Firefox */
    this.DocTypeReadVersion = 2;
}

EBMLHeader.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new DataView(a);

        at = EBML_write_array(this.id, dv, at);
        at = EBML_write_u64_data_len(0x1f, dv, at);
        at = EBML_write_u8_value(EBML_HEADER_VERSION, this.Version, dv, at);
        at = EBML_write_u8_value(EBML_HEADER_READ_VERSION, this.ReadVersion, dv, at);
        at = EBML_write_u8_value(EBML_HEADER_MAX_ID_LENGTH, this.MaxIDLength, dv, at);
        at = EBML_write_u8_value(EBML_HEADER_MAX_SIZE_LENGTH, this.MaxSizeLength, dv, at);
        at = EBML_write_data(EBML_HEADER_DOC_TYPE, this.DocType, dv, at);
        at = EBML_write_u8_value(EBML_HEADER_DOC_TYPE_VERSION, this.DocTypeVersion, dv, at);
        at = EBML_write_u8_value(EBML_HEADER_DOC_TYPE_READ_VERSION, this.DocTypeReadVersion, dv, at);

        return at;
    },
    buffer_size: function()
    {
        return 0x1f + 8 + this.id.length;
    },
}

function webm_Segment()
{
    this.id = WEBM_SEGMENT_HEADER;
}

webm_Segment.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new DataView(a);

        at = EBML_write_array(this.id, dv, at);
        dv.setUint8(at++, 0xff);
        return at;
    },
    buffer_size: function()
    {
        return this.id.length + 1;
    },
}

function webm_SegmentInformation()
{
    this.id = WEBM_SEGMENT_INFORMATION;
    this.timecode_scale = 1000000; /* 1 ms */
    this.muxing_app = "spice";
    this.writing_app = "spice-html5";

}

webm_SegmentInformation.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new DataView(a);

        at = EBML_write_array(this.id, dv, at);
        at = EBML_write_u64_data_len(this.buffer_size() - 8 - this.id.length, dv, at);
        at = EBML_write_u32_value(WEBM_TIMECODE_SCALE, this.timecode_scale, dv, at);
        at = EBML_write_data(WEBM_MUXING_APP, this.muxing_app, dv, at);
        at = EBML_write_data(WEBM_WRITING_APP, this.writing_app, dv, at);
        return at;
    },
    buffer_size: function()
    {
        return this.id.length + 8 +
                 WEBM_TIMECODE_SCALE.length + 1 + 4 +
                 WEBM_MUXING_APP.length + 1 + this.muxing_app.length +
                 WEBM_WRITING_APP.length + 1 + this.writing_app.length;
    },
}

function webm_Audio(frequency)
{
    this.id = WEBM_AUDIO;
    this.sampling_frequency = frequency;
    this.channels = OPUS_CHANNELS;
}

webm_Audio.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new DataView(a);
        at = EBML_write_array(this.id, dv, at);
        at = EBML_write_u64_data_len(this.buffer_size() - 8 - this.id.length, dv, at);
        at = EBML_write_u8_value(WEBM_CHANNELS, this.channels, dv, at);
        at = EBML_write_float_value(WEBM_SAMPLING_FREQUENCY, this.sampling_frequency, dv, at);
        return at;
    },
    buffer_size: function()
    {
        return this.id.length + 8 +
            WEBM_SAMPLING_FREQUENCY.length + 1 + 4 +
            WEBM_CHANNELS.length + 1 + 1;
    },
}


/* ---------------------------
   SeekHead not currently used.  Hopefully not needed.
*/
function webm_Seek(seekid, pos)
{
    this.id = WEBM_SEEK;
    this.pos = pos;
    this.seekid = seekid;
}

webm_Seek.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new DataView(a);
        at = EBML_write_array(this.id, dv, at);
        at = EBML_write_u1_data_len(this.buffer_size() - 1 - this.id.length, dv, at);

        at = EBML_write_data(WEBM_SEEK_ID, this.seekid, dv, at)
        at = EBML_write_u16_value(WEBM_SEEK_POSITION, this.pos, dv, at)

        return at;
    },
    buffer_size: function()
    {
        return this.id.length + 1 +
                WEBM_SEEK_ID.length + 1 + this.seekid.length +
                WEBM_SEEK_POSITION.length + 1 + 2;
    },
}
function webm_SeekHead(info_pos, track_pos)
{
    this.id = WEBM_SEEK_HEAD;
    this.info = new webm_Seek(WEBM_SEGMENT_INFORMATION, info_pos);
    this.track = new webm_Seek(WEBM_TRACKS, track_pos);
}

webm_SeekHead.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new DataView(a);
        at = EBML_write_array(this.id, dv, at);
        at = EBML_write_u64_data_len(this.buffer_size() - 8 - this.id.length, dv, at);

        at = this.info.to_buffer(a, at);
        at = this.track.to_buffer(a, at);

        return at;
    },
    buffer_size: function()
    {
        return this.id.length + 8 +
                this.info.buffer_size() +
                this.track.buffer_size();
    },
}

/* -------------------------------
   End of Seek Head
*/

function webm_TrackEntry()
{
    this.id = WEBM_TRACK_ENTRY;
    this.number = 1;
    this.uid = 1;
    this.type = 2; // Audio
    this.flag_enabled = 1;
    this.flag_default = 1;
    this.flag_forced = 1;
    this.flag_lacing = 0;
    this.min_cache = 0; // fixme - check
    this.max_block_addition_id = 0;
    this.codec_decode_all = 0; // fixme - check
    this.seek_pre_roll = 0; // 80000000; // fixme - check
    this.codec_delay =   80000000; // Must match codec_private.preskip
    this.codec_id = "A_OPUS";
    this.audio = new webm_Audio(OPUS_FREQUENCY);

    // See:  http://tools.ietf.org/html/draft-terriberry-oggopus-01
    this.codec_private = [ 0x4f, 0x70, 0x75, 0x73, 0x48, 0x65, 0x61, 0x64,  // OpusHead
                           0x01, // Version
                           OPUS_CHANNELS,
                           0x00, 0x0F, // Preskip - 3840 samples - should be 8ms at 48kHz
                           0x80, 0xbb, 0x00, 0x00,  // 48000
                           0x00, 0x00, // Output gain
                           0x00  // Channel mapping family
                           ];
}

webm_TrackEntry.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new DataView(a);
        at = EBML_write_array(this.id, dv, at);
        at = EBML_write_u64_data_len(this.buffer_size() - 8 - this.id.length, dv, at);
        at = EBML_write_u8_value(WEBM_TRACK_NUMBER, this.number, dv, at);
        at = EBML_write_u8_value(WEBM_TRACK_UID, this.uid, dv, at);
        at = EBML_write_u8_value(WEBM_FLAG_ENABLED, this.flag_enabled, dv, at);
        at = EBML_write_u8_value(WEBM_FLAG_DEFAULT, this.flag_default, dv, at);
        at = EBML_write_u8_value(WEBM_FLAG_FORCED, this.flag_forced, dv, at);
        at = EBML_write_u8_value(WEBM_FLAG_LACING, this.flag_lacing, dv, at);
        at = EBML_write_data(WEBM_CODEC_ID, this.codec_id, dv, at);
        at = EBML_write_u8_value(WEBM_MIN_CACHE, this.min_cache, dv, at);
        at = EBML_write_u8_value(WEBM_MAX_BLOCK_ADDITION_ID, this.max_block_addition_id, dv, at);
        at = EBML_write_u8_value(WEBM_CODEC_DECODE_ALL, this.codec_decode_all, dv, at);
        at = EBML_write_u32_value(WEBM_CODEC_DELAY, this.codec_delay, dv, at);
        at = EBML_write_u32_value(WEBM_SEEK_PRE_ROLL, this.seek_pre_roll, dv, at);
        at = EBML_write_u8_value(WEBM_TRACK_TYPE, this.type, dv, at);
        at = EBML_write_data(WEBM_CODEC_PRIVATE, this.codec_private, dv, at);

        at = this.audio.to_buffer(a, at);
        return at;
    },
    buffer_size: function()
    {
        return this.id.length + 8 +
            WEBM_TRACK_NUMBER.length + 1 + 1 +
            WEBM_TRACK_UID.length + 1 + 1 +
            WEBM_TRACK_TYPE.length + 1 + 1 +
            WEBM_FLAG_ENABLED.length + 1 + 1 +
            WEBM_FLAG_DEFAULT.length + 1 + 1 +
            WEBM_FLAG_FORCED.length + 1 + 1 +
            WEBM_FLAG_LACING.length + 1 + 1 +
            WEBM_MIN_CACHE.length + 1 + 1 +
            WEBM_MAX_BLOCK_ADDITION_ID.length + 1 + 1 +
            WEBM_CODEC_DECODE_ALL.length + 1 + 1 +
            WEBM_SEEK_PRE_ROLL.length + 1 + 4 +
            WEBM_CODEC_DELAY.length + 1 + 4 +
            WEBM_CODEC_ID.length + this.codec_id.length + 1 +
            WEBM_CODEC_PRIVATE.length + 1 + this.codec_private.length +
            this.audio.buffer_size();
    },
}
function webm_Tracks(entry)
{
    this.id = WEBM_TRACKS;
    this.track_entry = entry;
}

webm_Tracks.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new DataView(a);
        at = EBML_write_array(this.id, dv, at);
        at = EBML_write_u64_data_len(this.buffer_size() - 8 - this.id.length, dv, at);
        at = this.track_entry.to_buffer(a, at);
        return at;
    },
    buffer_size: function()
    {
        return this.id.length + 8 +
                 this.track_entry.buffer_size();
    },
}

function webm_Cluster(timecode, data)
{
    this.id = WEBM_CLUSTER;
    this.timecode = timecode;
    this.data = data;
}

webm_Cluster.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new DataView(a);
        at = EBML_write_array(this.id, dv, at);
        dv.setUint8(at++, 0xff);
        at = EBML_write_u32_value(WEBM_TIME_CODE, this.timecode, dv, at);
        return at;
    },
    buffer_size: function()
    {
        return this.id.length + 1 +
                 WEBM_TIME_CODE.length + 1 + 4;
    },
}

function webm_SimpleBlock(timecode, data, keyframe)
{
    this.id = WEBM_SIMPLE_BLOCK;
    this.timecode = timecode;
    this.data = data;
    this.keyframe = keyframe;
}

webm_SimpleBlock.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new DataView(a);
        at = EBML_write_array(this.id, dv, at);
        at = EBML_write_u64_data_len(this.data.byteLength + 4, dv, at);
        at = EBML_write_u1_data_len(1, dv, at); // Track #
        dv.setUint16(at, this.timecode); at += 2; // timecode - relative to cluster
        dv.setUint8(at, this.keyframe ? CLUSTER_SIMPLEBLOCK_FLAG_KEYFRAME : 0); at += 1;  // flags

        // FIXME - There should be a better way to copy
        var u8 = new Uint8Array(this.data);
        for (var i = 0; i < this.data.byteLength; i++)
            dv.setUint8(at++, u8[i]);

        return at;
    },
    buffer_size: function()
    {
        return this.id.length + 8 +
                 1 + 2 + 1 +
                 this.data.byteLength;
    },
}

function webm_Header()
{
    this.ebml = new EBMLHeader;
    this.segment = new webm_Segment;
    this.seek_head = new webm_SeekHead(0, 0);

    this.seek_head.info.pos = this.segment.buffer_size() + this.seek_head.buffer_size();

    this.info = new webm_SegmentInformation;

    this.seek_head.track.pos = this.seek_head.info.pos + this.info.buffer_size();

    this.track_entry = new webm_TrackEntry;
    this.tracks = new webm_Tracks(this.track_entry);
}

webm_Header.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        at = this.ebml.to_buffer(a, at);
        at = this.segment.to_buffer(a, at);
        at = this.info.to_buffer(a, at);
        at = this.tracks.to_buffer(a, at);

        return at;
    },
    buffer_size: function()
    {
        return this.ebml.buffer_size() +
               this.segment.buffer_size() +
               this.info.buffer_size() +
               this.tracks.buffer_size();
    },
}
;
define("spice-webm", function(){});


/*
   Copyright (C) 2014 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*----------------------------------------------------------------------------
**  SpicePlaybackConn
**      Drive the Spice Playback channel (sound out)
**--------------------------------------------------------------------------*/
function SpicePlaybackConn()
{
    SpiceConn.apply(this, arguments);

    this.queue = new Array();
    this.append_okay = false;
    this.start_time = 0;
    this.skip_until = 0;
    this.gap_time = 0;
}

SpicePlaybackConn.prototype = Object.create(SpiceConn.prototype);
SpicePlaybackConn.prototype.process_channel_message = function(msg)
{
    if (!!!window.MediaSource)
    {
        this.log_err('MediaSource API is not available');
        return false;
    }

    if (msg.type == SPICE_MSG_PLAYBACK_START)
    {
        var start = new SpiceMsgPlaybackStart(msg.data);

        DEBUG > 0 && console.log("PlaybackStart; frequency " + start.frequency);

        if (start.frequency != OPUS_FREQUENCY)
        {
            this.log_err('This player cannot handle frequency ' + start.frequency);
            return false;
        }

        if (start.channels != OPUS_CHANNELS)
        {
            this.log_err('This player cannot handle ' + start.channels + ' channels');
            return false;
        }

        if (start.format != SPICE_AUDIO_FMT_S16)
        {
            this.log_err('This player cannot format ' + start.format);
            return false;
        }

        if (! this.source_buffer)
        {
            this.media_source = new MediaSource();
            this.media_source.spiceconn = this;

            this.audio = document.createElement("audio");
            this.audio.setAttribute('autoplay', true);
            this.audio.src = window.URL.createObjectURL(this.media_source);
            document.getElementById(this.parent.screen_id).appendChild(this.audio);

            this.media_source.addEventListener('sourceopen', handle_source_open, false);
            this.media_source.addEventListener('sourceended', handle_source_ended, false);
            this.media_source.addEventListener('sourceclosed', handle_source_closed, false);

            this.bytes_written = 0;

            return true;
        }
    }

    if (msg.type == SPICE_MSG_PLAYBACK_DATA)
    {
        var data = new SpiceMsgPlaybackData(msg.data);

        // If this packet has the same time as the last, just bump up by one.
        if (this.last_data_time && data.time <= this.last_data_time)
        {
            // FIXME - this is arguably wrong.  But delaying the transmission was worse,
            //          in initial testing.  Could use more research.
            DEBUG > 1 && console.log("Hacking time of " + data.time + " to " + this.last_data_time + 1);
            data.time = this.last_data_time + 1;
        }

        /* Gap detection:  If there has been a delay since our last packet, then audio must
             have paused.  Handling that gets tricky.  In Chrome, you can seek forward,
             but you cannot in Firefox.  And seeking forward in Chrome is nice, as it keeps
             Chrome from being overly cautious in it's buffer strategy.

             So we do two things.  First, we seek forward.  Second, we compute how much of a gap
             there would have been, and essentially eliminate it.
        */
        if (this.last_data_time && data.time >= (this.last_data_time + GAP_DETECTION_THRESHOLD))
        {
            this.skip_until = data.time;
            this.gap_time = (data.time - this.start_time) - 
              (this.source_buffer.buffered.end(this.source_buffer.buffered.end.length - 1) * 1000.0).toFixed(0);
        }

        this.last_data_time = data.time;


        DEBUG > 1 && console.log("PlaybackData; time " + data.time + "; length " + data.data.byteLength);

        if (! this.source_buffer)
            return true;

        if (this.start_time == 0)
            this.start_playback(data);

        else if (data.time - this.cluster_time >= MAX_CLUSTER_TIME || this.skip_until > 0)
            this.new_cluster(data);

        else
            this.simple_block(data, false);

        if (this.skip_until > 0)
        {
            this.audio.currentTime = (this.skip_until - this.start_time - this.gap_time) / 1000.0;
            this.skip_until = 0;
        }

        if (this.audio.paused)
            this.audio.play();

        return true;
    }

    if (msg.type == SPICE_MSG_PLAYBACK_MODE)
    {
        var mode = new SpiceMsgPlaybackMode(msg.data);
        if (mode.mode != SPICE_AUDIO_DATA_MODE_OPUS)
        {
            this.log_err('This player cannot handle mode ' + mode.mode);
            delete this.source_buffer;
        }
        return true;
    }

    if (msg.type == SPICE_MSG_PLAYBACK_STOP)
    {
        return true;
    }

    return false;
}

SpicePlaybackConn.prototype.start_playback = function(data)
{
    this.start_time = data.time;

    var h = new webm_Header();

    var mb = new ArrayBuffer(h.buffer_size())

    this.bytes_written = h.to_buffer(mb);

    this.source_buffer.addEventListener('error', handle_sourcebuffer_error, false);
    this.source_buffer.addEventListener('updateend', handle_append_buffer_done, false);
    playback_append_buffer(this, mb);

    this.new_cluster(data);
}

SpicePlaybackConn.prototype.new_cluster = function(data)
{
    this.cluster_time = data.time;

    var c = new webm_Cluster(data.time - this.start_time - this.gap_time);

    var mb = new ArrayBuffer(c.buffer_size());
    this.bytes_written += c.to_buffer(mb);

    if (this.append_okay)
        playback_append_buffer(this, mb);
    else
        this.queue.push(mb);

    this.simple_block(data, true);
}

SpicePlaybackConn.prototype.simple_block = function(data, keyframe)
{
    var sb = new webm_SimpleBlock(data.time - this.cluster_time, data.data, keyframe);
    var mb = new ArrayBuffer(sb.buffer_size());

    this.bytes_written += sb.to_buffer(mb);

    if (this.append_okay)
        playback_append_buffer(this, mb);
    else
        this.queue.push(mb);
}

function handle_source_open(e)
{
    var p = this.spiceconn;

    if (p.source_buffer)
        return;

    p.source_buffer = this.addSourceBuffer(SPICE_PLAYBACK_CODEC);
    if (! p.source_buffer)
    {
        p.log_err('Codec ' + SPICE_PLAYBACK_CODEC + ' not available.');
        return;
    }
    p.source_buffer.spiceconn = p;
    p.source_buffer.mode = "segments";

    // FIXME - Experimentation with segments and sequences was unsatisfying.
    //         Switching to sequence did not solve our gap problem,
    //         but the browsers didn't fully support the time seek capability
    //         we would expect to gain from 'segments'.
    //         Segments worked at the time of this patch, so segments it is for now.

}

function handle_source_ended(e)
{
    var p = this.spiceconn;
    p.log_err('Audio source unexpectedly ended.');
}

function handle_source_closed(e)
{
    var p = this.spiceconn;
    p.log_err('Audio source unexpectedly closed.');
}

function handle_append_buffer_done(b)
{
    var p = this.spiceconn;
    if (p.queue.length > 0)
    {
        var mb = p.queue.shift();
        playback_append_buffer(p, mb);
    }
    else
        p.append_okay = true;

}

function handle_sourcebuffer_error(e)
{
    var p = this.spiceconn;
    p.log_err('source_buffer error ' + e.message);
}

function playback_append_buffer(p, b)
{
    try
    {
        p.source_buffer.appendBuffer(b);
        p.append_okay = false;
    }
    catch (e)
    {
        p.log_err("Error invoking appendBuffer: " + e.message);
    }
}
;
define("spice-playback", ["spice-spiceconn"], function(){});


/*
   Copyright (C) 2013 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*----------------------------------------------------------------------------
**  SpiceSimulateCursor
**      Internet Explorer 10 does not support data uri's in cursor assignment.
**  This file provides a number of gimmicks to compensate.  First, if there
**  is a preloaded cursor available, we will use that.  Failing that, we will
**  simulate a cursor using an image that is moved around the screen.
**--------------------------------------------------------------------------*/
var SpiceSimulateCursor = {

cursors : new Array(),
unknown_cursors : new Array(),
warned: false,

add_cursor: function(sha1, value)
{
    SpiceSimulateCursor.cursors[sha1] = value;
},

unknown_cursor: function(sha1, curdata)
{
    if (! SpiceSimulateCursor.warned)
    {
        SpiceSimulateCursor.warned = true;
        alert("Internet Explorer does not support dynamic cursors.  " +
              "This page will now simulate cursors with images, " +
              "which will be imperfect.  We recommend using Chrome or Firefox instead.  " +
              "\n\nIf you need to use Internet Explorer, you can create a static cursor " +
              "file for each cursor your application uses.  " +
              "View the console log for more information on creating static cursors for your environment.");
    }

    if (! SpiceSimulateCursor.unknown_cursors[sha1])
    {
        SpiceSimulateCursor.unknown_cursors[sha1] = curdata;
        console.log('Unknown cursor.  Simulation required.  To avoid simulation for this cursor, create and include a custom javascript file, and add the following line:');
        console.log('SpiceCursorSimulator.add_cursor("' + sha1 + '"), "<your filename here>.cur");');
        console.log('And then run following command, redirecting output into <your filename here>.cur:');
        console.log('php -r "echo urldecode(\'' + curdata + '\');"');
    }
},

simulate_cursor: function (spicecursor, cursor, screen, pngstr)
{
    var cursor_sha = hex_sha1(pngstr + ' ' + cursor.header.hot_spot_x + ' ' + cursor.header.hot_spot_y);
    if (typeof SpiceSimulateCursor.cursors != 'undefined')
        if (typeof SpiceSimulateCursor.cursors[cursor_sha] != 'undefined')
        {
            var curstr = 'url(' + SpiceSimulateCursor.cursors[cursor_sha] + '), default';
            screen.style.cursor = curstr;
        }

    if (window.getComputedStyle(screen, null).cursor == 'auto')
    {
        SpiceSimulateCursor.unknown_cursor(cursor_sha, 
            SpiceSimulateCursor.create_icondir(cursor.header.width, cursor.header.height,
            cursor.data.byteLength, cursor.header.hot_spot_x, cursor.header.hot_spot_y) + pngstr);

        document.getElementById(spicecursor.parent.screen_id).style.cursor = 'none';
        if (! spicecursor.spice_simulated_cursor)
        {
            spicecursor.spice_simulated_cursor = document.createElement('img');

            spicecursor.spice_simulated_cursor.style.position = 'absolute';
            spicecursor.spice_simulated_cursor.style.display = 'none';
            spicecursor.spice_simulated_cursor.style.overflow = 'hidden';

            spicecursor.spice_simulated_cursor.spice_screen = document.getElementById(spicecursor.parent.screen_id);

            spicecursor.spice_simulated_cursor.addEventListener('mousemove', SpiceSimulateCursor.handle_sim_mousemove);

            spicecursor.spice_simulated_cursor.spice_screen.appendChild(spicecursor.spice_simulated_cursor);
        }

        spicecursor.spice_simulated_cursor.src = 'data:image/png,' + pngstr;

        spicecursor.spice_simulated_cursor.spice_hot_x = cursor.header.hot_spot_x;
        spicecursor.spice_simulated_cursor.spice_hot_y = cursor.header.hot_spot_y;

        spicecursor.spice_simulated_cursor.style.pointerEvents = "none";
    }
    else
    { 
        if (spicecursor.spice_simulated_cursor)
        {
            spicecursor.spice_simulated_cursor.spice_screen.removeChild(spicecursor.spice_simulated_cursor);
            delete spicecursor.spice_simulated_cursor;
        }
    }
},

handle_sim_mousemove: function(e)
{
    var retval;
    var f = SpiceSimulateCursor.duplicate_mouse_event(e, this.spice_screen);
    return this.spice_screen.dispatchEvent(f);
},

duplicate_mouse_event: function(e, target)
{
    var evt = document.createEvent("mouseevent");
    evt.initMouseEvent(e.type, true, true, e.view, e.detail,
        e.screenX, e.screenY, e.clientX, e.clientY,
        e.ctrlKey, e.altKey, e.shiftKey, e.metaKey, e.button, e.relatedTarget);
    return evt;
},

ICONDIR: function ()
{
},

ICONDIRENTRY: function(width, height, bytes, hot_x, hot_y)
{
    this.width = width;
    this.height = height;
    this.bytes = bytes;
    this.hot_x = hot_x;
    this.hot_y = hot_y;
},


create_icondir: function (width, height, bytes, hot_x, hot_y)
{
    var i;
    var header = new SpiceSimulateCursor.ICONDIR();
    var entry = new SpiceSimulateCursor.ICONDIRENTRY(width, height, bytes, hot_x, hot_y);

    var mb = new ArrayBuffer(header.buffer_size() + entry.buffer_size());
    var at = header.to_buffer(mb);
    at = entry.to_buffer(mb, at);

    var u8 = new Uint8Array(mb);
    var str = "";
    for (i = 0; i < at; i++)
    {
        str += "%";
        if (u8[i] < 16)
            str += "0";
        str += u8[i].toString(16);
    }
    return str;
},

};

SpiceSimulateCursor.ICONDIR.prototype = 
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint16(at, 0, true); at += 2;
        dv.setUint16(at, 2, true); at += 2;
        dv.setUint16(at, 1, true); at += 2;
        return at;
    },
    buffer_size: function()
    {
        return 6;
    }
};

SpiceSimulateCursor.ICONDIRENTRY.prototype =
{
    to_buffer: function(a, at)
    {
        at = at || 0;
        var dv = new SpiceDataView(a);
        dv.setUint8(at, this.width); at++;
        dv.setUint8(at, this.height); at++;
        dv.setUint8(at, 0); at++;  /* color palette count, unused */
        dv.setUint8(at, 0); at++;  /* reserved */
        dv.setUint16(at, this.hot_x, true); at += 2;
        dv.setUint16(at, this.hot_y, true); at += 2;
        dv.setUint32(at, this.bytes, true); at += 4;
        dv.setUint32(at, at + 4, true); at += 4;  /* Offset to bytes */
        return at;
    },
    buffer_size: function()
    {
        return 16;
    }
};

define("spice-simulatecursor", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/


/*----------------------------------------------------------------------------
**  SpiceCursorConn
**      Drive the Spice Cursor Channel
**--------------------------------------------------------------------------*/
function SpiceCursorConn()
{
    SpiceConn.apply(this, arguments);
}

SpiceCursorConn.prototype = Object.create(SpiceConn.prototype);
SpiceCursorConn.prototype.process_channel_message = function(msg)
{
    if (msg.type == SPICE_MSG_CURSOR_INIT)
    {
        var cursor_init = new SpiceMsgCursorInit(msg.data);
        DEBUG > 1 && console.log("SpiceMsgCursorInit");
        if (this.parent && this.parent.inputs &&
            this.parent.inputs.mouse_mode == SPICE_MOUSE_MODE_SERVER)
        {
            // FIXME - this imagines that the server actually
            //          provides the current cursor position,
            //          instead of 0,0.  As of May 11, 2012,
            //          that assumption was false :-(.
            this.parent.inputs.mousex = cursor_init.position.x;
            this.parent.inputs.mousey = cursor_init.position.y;
        }
        // FIXME - We don't handle most of the parameters here...
        return true;
    }

    if (msg.type == SPICE_MSG_CURSOR_SET)
    {
        var cursor_set = new SpiceMsgCursorSet(msg.data);
        DEBUG > 1 && console.log("SpiceMsgCursorSet");
        if (cursor_set.flags & SPICE_CURSOR_FLAGS_NONE)
        {
            document.getElementById(this.parent.screen_id).style.cursor = "none";
            return true;
        }

        if (cursor_set.flags > 0)
            this.log_warn("FIXME: No support for cursor flags " + cursor_set.flags);

        if (cursor_set.cursor.header.type != SPICE_CURSOR_TYPE_ALPHA)
        {
            this.log_warn("FIXME: No support for cursor type " + cursor_set.cursor.header.type);
            return false;
        }

        this.set_cursor(cursor_set.cursor);

        return true;
    }

    if (msg.type == SPICE_MSG_CURSOR_HIDE)
    {
        DEBUG > 1 && console.log("SpiceMsgCursorHide");
        document.getElementById(this.parent.screen_id).style.cursor = "none";
        return true;
    }

    if (msg.type == SPICE_MSG_CURSOR_RESET)
    {
        DEBUG > 1 && console.log("SpiceMsgCursorReset");
        document.getElementById(this.parent.screen_id).style.cursor = "auto";
        return true;
    }

    if (msg.type == SPICE_MSG_CURSOR_INVAL_ALL)
    {
        DEBUG > 1 && console.log("SpiceMsgCursorInvalAll");
        // FIXME - There may be something useful to do here...
        return true;
    }

    return false;
}

SpiceCursorConn.prototype.set_cursor = function(cursor)
{
    var pngstr = create_rgba_png(cursor.header.height, cursor.header.width, cursor.data);
    var curstr = 'url(data:image/png,' + pngstr + ') ' + 
        cursor.header.hot_spot_x + ' ' + cursor.header.hot_spot_y + ", default";
    var screen = document.getElementById(this.parent.screen_id);
    screen.style.cursor = 'auto';
    screen.style.cursor = curstr;
    if (window.getComputedStyle(screen, null).cursor == 'auto')
        SpiceSimulateCursor.simulate_cursor(this, cursor, screen, pngstr);
}
;
define("spice-cursor", ["spice-spiceconn"], function(){});

// Downloaded from http://www-cs-students.stanford.edu/~tjw/jsbn/ by Jeremy White on 6/1/2012

/*
 * Copyright (c) 2003-2005  Tom Wu
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL TOM WU BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
 * THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * In addition, the following condition applies:
 *
 * All redistributions must retain an intact copy of this copyright notice
 * and disclaimer.
 */


// Basic JavaScript BN library - subset useful for RSA encryption.

// Bits per digit
var dbits;

// JavaScript engine analysis
var canary = 0xdeadbeefcafe;
var j_lm = ((canary&0xffffff)==0xefcafe);

// (public) Constructor
function BigInteger(a,b,c) {
  if(a != null)
    if("number" == typeof a) this.fromNumber(a,b,c);
    else if(b == null && "string" != typeof a) this.fromString(a,256);
    else this.fromString(a,b);
}

// return new, unset BigInteger
function nbi() { return new BigInteger(null); }

// am: Compute w_j += (x*this_i), propagate carries,
// c is initial carry, returns final carry.
// c < 3*dvalue, x < 2*dvalue, this_i < dvalue
// We need to select the fastest one that works in this environment.

// am1: use a single mult and divide to get the high bits,
// max digit bits should be 26 because
// max internal value = 2*dvalue^2-2*dvalue (< 2^53)
function am1(i,x,w,j,c,n) {
  while(--n >= 0) {
    var v = x*this[i++]+w[j]+c;
    c = Math.floor(v/0x4000000);
    w[j++] = v&0x3ffffff;
  }
  return c;
}
// am2 avoids a big mult-and-extract completely.
// Max digit bits should be <= 30 because we do bitwise ops
// on values up to 2*hdvalue^2-hdvalue-1 (< 2^31)
function am2(i,x,w,j,c,n) {
  var xl = x&0x7fff, xh = x>>15;
  while(--n >= 0) {
    var l = this[i]&0x7fff;
    var h = this[i++]>>15;
    var m = xh*l+h*xl;
    l = xl*l+((m&0x7fff)<<15)+w[j]+(c&0x3fffffff);
    c = (l>>>30)+(m>>>15)+xh*h+(c>>>30);
    w[j++] = l&0x3fffffff;
  }
  return c;
}
// Alternately, set max digit bits to 28 since some
// browsers slow down when dealing with 32-bit numbers.
function am3(i,x,w,j,c,n) {
  var xl = x&0x3fff, xh = x>>14;
  while(--n >= 0) {
    var l = this[i]&0x3fff;
    var h = this[i++]>>14;
    var m = xh*l+h*xl;
    l = xl*l+((m&0x3fff)<<14)+w[j]+c;
    c = (l>>28)+(m>>14)+xh*h;
    w[j++] = l&0xfffffff;
  }
  return c;
}
if(j_lm && (navigator.appName == "Microsoft Internet Explorer")) {
  BigInteger.prototype.am = am2;
  dbits = 30;
}
else if(j_lm && (navigator.appName != "Netscape")) {
  BigInteger.prototype.am = am1;
  dbits = 26;
}
else { // Mozilla/Netscape seems to prefer am3
  BigInteger.prototype.am = am3;
  dbits = 28;
}

BigInteger.prototype.DB = dbits;
BigInteger.prototype.DM = ((1<<dbits)-1);
BigInteger.prototype.DV = (1<<dbits);

var BI_FP = 52;
BigInteger.prototype.FV = Math.pow(2,BI_FP);
BigInteger.prototype.F1 = BI_FP-dbits;
BigInteger.prototype.F2 = 2*dbits-BI_FP;

// Digit conversions
var BI_RM = "0123456789abcdefghijklmnopqrstuvwxyz";
var BI_RC = new Array();
var rr,vv;
rr = "0".charCodeAt(0);
for(vv = 0; vv <= 9; ++vv) BI_RC[rr++] = vv;
rr = "a".charCodeAt(0);
for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;
rr = "A".charCodeAt(0);
for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;

function int2char(n) { return BI_RM.charAt(n); }
function intAt(s,i) {
  var c = BI_RC[s.charCodeAt(i)];
  return (c==null)?-1:c;
}

// (protected) copy this to r
function bnpCopyTo(r) {
  for(var i = this.t-1; i >= 0; --i) r[i] = this[i];
  r.t = this.t;
  r.s = this.s;
}

// (protected) set from integer value x, -DV <= x < DV
function bnpFromInt(x) {
  this.t = 1;
  this.s = (x<0)?-1:0;
  if(x > 0) this[0] = x;
  else if(x < -1) this[0] = x+DV;
  else this.t = 0;
}

// return bigint initialized to value
function nbv(i) { var r = nbi(); r.fromInt(i); return r; }

// (protected) set from string and radix
function bnpFromString(s,b) {
  var k;
  if(b == 16) k = 4;
  else if(b == 8) k = 3;
  else if(b == 256) k = 8; // byte array
  else if(b == 2) k = 1;
  else if(b == 32) k = 5;
  else if(b == 4) k = 2;
  else { this.fromRadix(s,b); return; }
  this.t = 0;
  this.s = 0;
  var i = s.length, mi = false, sh = 0;
  while(--i >= 0) {
    var x = (k==8)?s[i]&0xff:intAt(s,i);
    if(x < 0) {
      if(s.charAt(i) == "-") mi = true;
      continue;
    }
    mi = false;
    if(sh == 0)
      this[this.t++] = x;
    else if(sh+k > this.DB) {
      this[this.t-1] |= (x&((1<<(this.DB-sh))-1))<<sh;
      this[this.t++] = (x>>(this.DB-sh));
    }
    else
      this[this.t-1] |= x<<sh;
    sh += k;
    if(sh >= this.DB) sh -= this.DB;
  }
  if(k == 8 && (s[0]&0x80) != 0) {
    this.s = -1;
    if(sh > 0) this[this.t-1] |= ((1<<(this.DB-sh))-1)<<sh;
  }
  this.clamp();
  if(mi) BigInteger.ZERO.subTo(this,this);
}

// (protected) clamp off excess high words
function bnpClamp() {
  var c = this.s&this.DM;
  while(this.t > 0 && this[this.t-1] == c) --this.t;
}

// (public) return string representation in given radix
function bnToString(b) {
  if(this.s < 0) return "-"+this.negate().toString(b);
  var k;
  if(b == 16) k = 4;
  else if(b == 8) k = 3;
  else if(b == 2) k = 1;
  else if(b == 32) k = 5;
  else if(b == 4) k = 2;
  else return this.toRadix(b);
  var km = (1<<k)-1, d, m = false, r = "", i = this.t;
  var p = this.DB-(i*this.DB)%k;
  if(i-- > 0) {
    if(p < this.DB && (d = this[i]>>p) > 0) { m = true; r = int2char(d); }
    while(i >= 0) {
      if(p < k) {
        d = (this[i]&((1<<p)-1))<<(k-p);
        d |= this[--i]>>(p+=this.DB-k);
      }
      else {
        d = (this[i]>>(p-=k))&km;
        if(p <= 0) { p += this.DB; --i; }
      }
      if(d > 0) m = true;
      if(m) r += int2char(d);
    }
  }
  return m?r:"0";
}

// (public) -this
function bnNegate() { var r = nbi(); BigInteger.ZERO.subTo(this,r); return r; }

// (public) |this|
function bnAbs() { return (this.s<0)?this.negate():this; }

// (public) return + if this > a, - if this < a, 0 if equal
function bnCompareTo(a) {
  var r = this.s-a.s;
  if(r != 0) return r;
  var i = this.t;
  r = i-a.t;
  if(r != 0) return r;
  while(--i >= 0) if((r=this[i]-a[i]) != 0) return r;
  return 0;
}

// returns bit length of the integer x
function nbits(x) {
  var r = 1, t;
  if((t=x>>>16) != 0) { x = t; r += 16; }
  if((t=x>>8) != 0) { x = t; r += 8; }
  if((t=x>>4) != 0) { x = t; r += 4; }
  if((t=x>>2) != 0) { x = t; r += 2; }
  if((t=x>>1) != 0) { x = t; r += 1; }
  return r;
}

// (public) return the number of bits in "this"
function bnBitLength() {
  if(this.t <= 0) return 0;
  return this.DB*(this.t-1)+nbits(this[this.t-1]^(this.s&this.DM));
}

// (protected) r = this << n*DB
function bnpDLShiftTo(n,r) {
  var i;
  for(i = this.t-1; i >= 0; --i) r[i+n] = this[i];
  for(i = n-1; i >= 0; --i) r[i] = 0;
  r.t = this.t+n;
  r.s = this.s;
}

// (protected) r = this >> n*DB
function bnpDRShiftTo(n,r) {
  for(var i = n; i < this.t; ++i) r[i-n] = this[i];
  r.t = Math.max(this.t-n,0);
  r.s = this.s;
}

// (protected) r = this << n
function bnpLShiftTo(n,r) {
  var bs = n%this.DB;
  var cbs = this.DB-bs;
  var bm = (1<<cbs)-1;
  var ds = Math.floor(n/this.DB), c = (this.s<<bs)&this.DM, i;
  for(i = this.t-1; i >= 0; --i) {
    r[i+ds+1] = (this[i]>>cbs)|c;
    c = (this[i]&bm)<<bs;
  }
  for(i = ds-1; i >= 0; --i) r[i] = 0;
  r[ds] = c;
  r.t = this.t+ds+1;
  r.s = this.s;
  r.clamp();
}

// (protected) r = this >> n
function bnpRShiftTo(n,r) {
  r.s = this.s;
  var ds = Math.floor(n/this.DB);
  if(ds >= this.t) { r.t = 0; return; }
  var bs = n%this.DB;
  var cbs = this.DB-bs;
  var bm = (1<<bs)-1;
  r[0] = this[ds]>>bs;
  for(var i = ds+1; i < this.t; ++i) {
    r[i-ds-1] |= (this[i]&bm)<<cbs;
    r[i-ds] = this[i]>>bs;
  }
  if(bs > 0) r[this.t-ds-1] |= (this.s&bm)<<cbs;
  r.t = this.t-ds;
  r.clamp();
}

// (protected) r = this - a
function bnpSubTo(a,r) {
  var i = 0, c = 0, m = Math.min(a.t,this.t);
  while(i < m) {
    c += this[i]-a[i];
    r[i++] = c&this.DM;
    c >>= this.DB;
  }
  if(a.t < this.t) {
    c -= a.s;
    while(i < this.t) {
      c += this[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    c += this.s;
  }
  else {
    c += this.s;
    while(i < a.t) {
      c -= a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    c -= a.s;
  }
  r.s = (c<0)?-1:0;
  if(c < -1) r[i++] = this.DV+c;
  else if(c > 0) r[i++] = c;
  r.t = i;
  r.clamp();
}

// (protected) r = this * a, r != this,a (HAC 14.12)
// "this" should be the larger one if appropriate.
function bnpMultiplyTo(a,r) {
  var x = this.abs(), y = a.abs();
  var i = x.t;
  r.t = i+y.t;
  while(--i >= 0) r[i] = 0;
  for(i = 0; i < y.t; ++i) r[i+x.t] = x.am(0,y[i],r,i,0,x.t);
  r.s = 0;
  r.clamp();
  if(this.s != a.s) BigInteger.ZERO.subTo(r,r);
}

// (protected) r = this^2, r != this (HAC 14.16)
function bnpSquareTo(r) {
  var x = this.abs();
  var i = r.t = 2*x.t;
  while(--i >= 0) r[i] = 0;
  for(i = 0; i < x.t-1; ++i) {
    var c = x.am(i,x[i],r,2*i,0,1);
    if((r[i+x.t]+=x.am(i+1,2*x[i],r,2*i+1,c,x.t-i-1)) >= x.DV) {
      r[i+x.t] -= x.DV;
      r[i+x.t+1] = 1;
    }
  }
  if(r.t > 0) r[r.t-1] += x.am(i,x[i],r,2*i,0,1);
  r.s = 0;
  r.clamp();
}

// (protected) divide this by m, quotient and remainder to q, r (HAC 14.20)
// r != q, this != m.  q or r may be null.
function bnpDivRemTo(m,q,r) {
  var pm = m.abs();
  if(pm.t <= 0) return;
  var pt = this.abs();
  if(pt.t < pm.t) {
    if(q != null) q.fromInt(0);
    if(r != null) this.copyTo(r);
    return;
  }
  if(r == null) r = nbi();
  var y = nbi(), ts = this.s, ms = m.s;
  var nsh = this.DB-nbits(pm[pm.t-1]);	// normalize modulus
  if(nsh > 0) { pm.lShiftTo(nsh,y); pt.lShiftTo(nsh,r); }
  else { pm.copyTo(y); pt.copyTo(r); }
  var ys = y.t;
  var y0 = y[ys-1];
  if(y0 == 0) return;
  var yt = y0*(1<<this.F1)+((ys>1)?y[ys-2]>>this.F2:0);
  var d1 = this.FV/yt, d2 = (1<<this.F1)/yt, e = 1<<this.F2;
  var i = r.t, j = i-ys, t = (q==null)?nbi():q;
  y.dlShiftTo(j,t);
  if(r.compareTo(t) >= 0) {
    r[r.t++] = 1;
    r.subTo(t,r);
  }
  BigInteger.ONE.dlShiftTo(ys,t);
  t.subTo(y,y);	// "negative" y so we can replace sub with am later
  while(y.t < ys) y[y.t++] = 0;
  while(--j >= 0) {
    // Estimate quotient digit
    var qd = (r[--i]==y0)?this.DM:Math.floor(r[i]*d1+(r[i-1]+e)*d2);
    if((r[i]+=y.am(0,qd,r,j,0,ys)) < qd) {	// Try it out
      y.dlShiftTo(j,t);
      r.subTo(t,r);
      while(r[i] < --qd) r.subTo(t,r);
    }
  }
  if(q != null) {
    r.drShiftTo(ys,q);
    if(ts != ms) BigInteger.ZERO.subTo(q,q);
  }
  r.t = ys;
  r.clamp();
  if(nsh > 0) r.rShiftTo(nsh,r);	// Denormalize remainder
  if(ts < 0) BigInteger.ZERO.subTo(r,r);
}

// (public) this mod a
function bnMod(a) {
  var r = nbi();
  this.abs().divRemTo(a,null,r);
  if(this.s < 0 && r.compareTo(BigInteger.ZERO) > 0) a.subTo(r,r);
  return r;
}

// Modular reduction using "classic" algorithm
function Classic(m) { this.m = m; }
function cConvert(x) {
  if(x.s < 0 || x.compareTo(this.m) >= 0) return x.mod(this.m);
  else return x;
}
function cRevert(x) { return x; }
function cReduce(x) { x.divRemTo(this.m,null,x); }
function cMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }
function cSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

Classic.prototype.convert = cConvert;
Classic.prototype.revert = cRevert;
Classic.prototype.reduce = cReduce;
Classic.prototype.mulTo = cMulTo;
Classic.prototype.sqrTo = cSqrTo;

// (protected) return "-1/this % 2^DB"; useful for Mont. reduction
// justification:
//         xy == 1 (mod m)
//         xy =  1+km
//   xy(2-xy) = (1+km)(1-km)
// x[y(2-xy)] = 1-k^2m^2
// x[y(2-xy)] == 1 (mod m^2)
// if y is 1/x mod m, then y(2-xy) is 1/x mod m^2
// should reduce x and y(2-xy) by m^2 at each step to keep size bounded.
// JS multiply "overflows" differently from C/C++, so care is needed here.
function bnpInvDigit() {
  if(this.t < 1) return 0;
  var x = this[0];
  if((x&1) == 0) return 0;
  var y = x&3;		// y == 1/x mod 2^2
  y = (y*(2-(x&0xf)*y))&0xf;	// y == 1/x mod 2^4
  y = (y*(2-(x&0xff)*y))&0xff;	// y == 1/x mod 2^8
  y = (y*(2-(((x&0xffff)*y)&0xffff)))&0xffff;	// y == 1/x mod 2^16
  // last step - calculate inverse mod DV directly;
  // assumes 16 < DB <= 32 and assumes ability to handle 48-bit ints
  y = (y*(2-x*y%this.DV))%this.DV;		// y == 1/x mod 2^dbits
  // we really want the negative inverse, and -DV < y < DV
  return (y>0)?this.DV-y:-y;
}

// Montgomery reduction
function Montgomery(m) {
  this.m = m;
  this.mp = m.invDigit();
  this.mpl = this.mp&0x7fff;
  this.mph = this.mp>>15;
  this.um = (1<<(m.DB-15))-1;
  this.mt2 = 2*m.t;
}

// xR mod m
function montConvert(x) {
  var r = nbi();
  x.abs().dlShiftTo(this.m.t,r);
  r.divRemTo(this.m,null,r);
  if(x.s < 0 && r.compareTo(BigInteger.ZERO) > 0) this.m.subTo(r,r);
  return r;
}

// x/R mod m
function montRevert(x) {
  var r = nbi();
  x.copyTo(r);
  this.reduce(r);
  return r;
}

// x = x/R mod m (HAC 14.32)
function montReduce(x) {
  while(x.t <= this.mt2)	// pad x so am has enough room later
    x[x.t++] = 0;
  for(var i = 0; i < this.m.t; ++i) {
    // faster way of calculating u0 = x[i]*mp mod DV
    var j = x[i]&0x7fff;
    var u0 = (j*this.mpl+(((j*this.mph+(x[i]>>15)*this.mpl)&this.um)<<15))&x.DM;
    // use am to combine the multiply-shift-add into one call
    j = i+this.m.t;
    x[j] += this.m.am(0,u0,x,i,0,this.m.t);
    // propagate carry
    while(x[j] >= x.DV) { x[j] -= x.DV; x[++j]++; }
  }
  x.clamp();
  x.drShiftTo(this.m.t,x);
  if(x.compareTo(this.m) >= 0) x.subTo(this.m,x);
}

// r = "x^2/R mod m"; x != r
function montSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

// r = "xy/R mod m"; x,y != r
function montMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }

Montgomery.prototype.convert = montConvert;
Montgomery.prototype.revert = montRevert;
Montgomery.prototype.reduce = montReduce;
Montgomery.prototype.mulTo = montMulTo;
Montgomery.prototype.sqrTo = montSqrTo;

// (protected) true iff this is even
function bnpIsEven() { return ((this.t>0)?(this[0]&1):this.s) == 0; }

// (protected) this^e, e < 2^32, doing sqr and mul with "r" (HAC 14.79)
function bnpExp(e,z) {
  if(e > 0xffffffff || e < 1) return BigInteger.ONE;
  var r = nbi(), r2 = nbi(), g = z.convert(this), i = nbits(e)-1;
  g.copyTo(r);
  while(--i >= 0) {
    z.sqrTo(r,r2);
    if((e&(1<<i)) > 0) z.mulTo(r2,g,r);
    else { var t = r; r = r2; r2 = t; }
  }
  return z.revert(r);
}

// (public) this^e % m, 0 <= e < 2^32
function bnModPowInt(e,m) {
  var z;
  if(e < 256 || m.isEven()) z = new Classic(m); else z = new Montgomery(m);
  return this.exp(e,z);
}

// protected
BigInteger.prototype.copyTo = bnpCopyTo;
BigInteger.prototype.fromInt = bnpFromInt;
BigInteger.prototype.fromString = bnpFromString;
BigInteger.prototype.clamp = bnpClamp;
BigInteger.prototype.dlShiftTo = bnpDLShiftTo;
BigInteger.prototype.drShiftTo = bnpDRShiftTo;
BigInteger.prototype.lShiftTo = bnpLShiftTo;
BigInteger.prototype.rShiftTo = bnpRShiftTo;
BigInteger.prototype.subTo = bnpSubTo;
BigInteger.prototype.multiplyTo = bnpMultiplyTo;
BigInteger.prototype.squareTo = bnpSquareTo;
BigInteger.prototype.divRemTo = bnpDivRemTo;
BigInteger.prototype.invDigit = bnpInvDigit;
BigInteger.prototype.isEven = bnpIsEven;
BigInteger.prototype.exp = bnpExp;

// public
BigInteger.prototype.toString = bnToString;
BigInteger.prototype.negate = bnNegate;
BigInteger.prototype.abs = bnAbs;
BigInteger.prototype.compareTo = bnCompareTo;
BigInteger.prototype.bitLength = bnBitLength;
BigInteger.prototype.mod = bnMod;
BigInteger.prototype.modPowInt = bnModPowInt;

// "constants"
BigInteger.ZERO = nbv(0);
BigInteger.ONE = nbv(1);

define("spice-jsbn", function(){});

// Downloaded from http://www-cs-students.stanford.edu/~tjw/jsbn/ by Jeremy White on 6/1/2012

/*
 * Copyright (c) 2003-2005  Tom Wu
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL TOM WU BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
 * THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * In addition, the following condition applies:
 *
 * All redistributions must retain an intact copy of this copyright notice
 * and disclaimer.
 */


// Depends on jsbn.js and rng.js

// Version 1.1: support utf-8 encoding in pkcs1pad2

// convert a (hex) string to a bignum object
function parseBigInt(str,r) {
  return new BigInteger(str,r);
}

function linebrk(s,n) {
  var ret = "";
  var i = 0;
  while(i + n < s.length) {
    ret += s.substring(i,i+n) + "\n";
    i += n;
  }
  return ret + s.substring(i,s.length);
}

function byte2Hex(b) {
  if(b < 0x10)
    return "0" + b.toString(16);
  else
    return b.toString(16);
}

// PKCS#1 (type 2, random) pad input string s to n bytes, and return a bigint
function pkcs1pad2(s,n) {
  if(n < s.length + 11) { // TODO: fix for utf-8
    alert("Message too long for RSA");
    return null;
  }
  var ba = new Array();
  var i = s.length - 1;
  while(i >= 0 && n > 0) {
    var c = s.charCodeAt(i--);
    if(c < 128) { // encode using utf-8
      ba[--n] = c;
    }
    else if((c > 127) && (c < 2048)) {
      ba[--n] = (c & 63) | 128;
      ba[--n] = (c >> 6) | 192;
    }
    else {
      ba[--n] = (c & 63) | 128;
      ba[--n] = ((c >> 6) & 63) | 128;
      ba[--n] = (c >> 12) | 224;
    }
  }
  ba[--n] = 0;
  var rng = new SecureRandom();
  var x = new Array();
  while(n > 2) { // random non-zero pad
    x[0] = 0;
    while(x[0] == 0) rng.nextBytes(x);
    ba[--n] = x[0];
  }
  ba[--n] = 2;
  ba[--n] = 0;
  return new BigInteger(ba);
}

// "empty" RSA key constructor
function RSAKey() {
  this.n = null;
  this.e = 0;
  this.d = null;
  this.p = null;
  this.q = null;
  this.dmp1 = null;
  this.dmq1 = null;
  this.coeff = null;
}

// Set the public key fields N and e from hex strings
function RSASetPublic(N,E) {
  if(N != null && E != null && N.length > 0 && E.length > 0) {
    this.n = parseBigInt(N,16);
    this.e = parseInt(E,16);
  }
  else
    alert("Invalid RSA public key");
}

// Perform raw public operation on "x": return x^e (mod n)
function RSADoPublic(x) {
  return x.modPowInt(this.e, this.n);
}

// Return the PKCS#1 RSA encryption of "text" as an even-length hex string
function RSAEncrypt(text) {
  var m = pkcs1pad2(text,(this.n.bitLength()+7)>>3);
  if(m == null) return null;
  var c = this.doPublic(m);
  if(c == null) return null;
  var h = c.toString(16);
  if((h.length & 1) == 0) return h; else return "0" + h;
}

// Return the PKCS#1 RSA encryption of "text" as a Base64-encoded string
//function RSAEncryptB64(text) {
//  var h = this.encrypt(text);
//  if(h) return hex2b64(h); else return null;
//}

// protected
RSAKey.prototype.doPublic = RSADoPublic;

// public
RSAKey.prototype.setPublic = RSASetPublic;
RSAKey.prototype.encrypt = RSAEncrypt;
//RSAKey.prototype.encrypt_b64 = RSAEncryptB64;

define("spice-rsa", function(){});

// Downloaded from http://www-cs-students.stanford.edu/~tjw/jsbn/ by Jeremy White on 6/1/2012

/*
 * Copyright (c) 2003-2005  Tom Wu
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL TOM WU BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
 * THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * In addition, the following condition applies:
 *
 * All redistributions must retain an intact copy of this copyright notice
 * and disclaimer.
 */


// prng4.js - uses Arcfour as a PRNG

function Arcfour() {
  this.i = 0;
  this.j = 0;
  this.S = new Array();
}

// Initialize arcfour context from key, an array of ints, each from [0..255]
function ARC4init(key) {
  var i, j, t;
  for(i = 0; i < 256; ++i)
    this.S[i] = i;
  j = 0;
  for(i = 0; i < 256; ++i) {
    j = (j + this.S[i] + key[i % key.length]) & 255;
    t = this.S[i];
    this.S[i] = this.S[j];
    this.S[j] = t;
  }
  this.i = 0;
  this.j = 0;
}

function ARC4next() {
  var t;
  this.i = (this.i + 1) & 255;
  this.j = (this.j + this.S[this.i]) & 255;
  t = this.S[this.i];
  this.S[this.i] = this.S[this.j];
  this.S[this.j] = t;
  return this.S[(t + this.S[this.i]) & 255];
}

Arcfour.prototype.init = ARC4init;
Arcfour.prototype.next = ARC4next;

// Plug in your RNG constructor here
function prng_newstate() {
  return new Arcfour();
}

// Pool size must be a multiple of 4 and greater than 32.
// An array of bytes the size of the pool will be passed to init()
var rng_psize = 256;

define("spice-prng4", function(){});

// Downloaded from http://www-cs-students.stanford.edu/~tjw/jsbn/ by Jeremy White on 6/1/2012

/*
 * Copyright (c) 2003-2005  Tom Wu
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL TOM WU BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
 * THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * In addition, the following condition applies:
 *
 * All redistributions must retain an intact copy of this copyright notice
 * and disclaimer.
 */


// Random number generator - requires a PRNG backend, e.g. prng4.js

// For best results, put code like
// <body onClick='rng_seed_time();' onKeyPress='rng_seed_time();'>
// in your main HTML document.

var rng_state;
var rng_pool;
var rng_pptr;

// Mix in a 32-bit integer into the pool
function rng_seed_int(x) {
  rng_pool[rng_pptr++] ^= x & 255;
  rng_pool[rng_pptr++] ^= (x >> 8) & 255;
  rng_pool[rng_pptr++] ^= (x >> 16) & 255;
  rng_pool[rng_pptr++] ^= (x >> 24) & 255;
  if(rng_pptr >= rng_psize) rng_pptr -= rng_psize;
}

// Mix in the current time (w/milliseconds) into the pool
function rng_seed_time() {
  rng_seed_int(new Date().getTime());
}

// Initialize the pool with junk if needed.
if(rng_pool == null) {
  rng_pool = new Array();
  rng_pptr = 0;
  var t;
  if(navigator.appName == "Netscape" && navigator.appVersion < "5" && window.crypto) {
    // Extract entropy (256 bits) from NS4 RNG if available
    var z = window.crypto.random(32);
    for(t = 0; t < z.length; ++t)
      rng_pool[rng_pptr++] = z.charCodeAt(t) & 255;
  }  
  while(rng_pptr < rng_psize) {  // extract some randomness from Math.random()
    t = Math.floor(65536 * Math.random());
    rng_pool[rng_pptr++] = t >>> 8;
    rng_pool[rng_pptr++] = t & 255;
  }
  rng_pptr = 0;
  rng_seed_time();
  //rng_seed_int(window.screenX);
  //rng_seed_int(window.screenY);
}

function rng_get_byte() {
  if(rng_state == null) {
    rng_seed_time();
    rng_state = prng_newstate();
    rng_state.init(rng_pool);
    for(rng_pptr = 0; rng_pptr < rng_pool.length; ++rng_pptr)
      rng_pool[rng_pptr] = 0;
    rng_pptr = 0;
    //rng_pool = null;
  }
  // TODO: allow reseeding after first request
  return rng_state.next();
}

function rng_get_bytes(ba) {
  var i;
  for(i = 0; i < ba.length; ++i) ba[i] = rng_get_byte();
}

function SecureRandom() {}

SecureRandom.prototype.nextBytes = rng_get_bytes;

define("spice-rng", ["spice-prng4"], function(){});

/*
 * A JavaScript implementation of the Secure Hash Algorithm, SHA-1, as defined
 * in FIPS 180-1
 * Version 2.2 Copyright Paul Johnston 2000 - 2009.
 * Other contributors: Greg Holt, Andrew Kepert, Ydnar, Lostinet
 * Distributed under the BSD License
 * See http://pajhome.org.uk/crypt/md5 for details.
 */

 /* Downloaded 6/1/2012 from the above address by Jeremy White.
    License reproduce here for completeness:

Copyright (c) 1998 - 2009, Paul Johnston & Contributors
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

Neither the name of the author nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */

/*
 * Configurable variables. You may need to tweak these to be compatible with
 * the server-side, but the defaults work in most cases.
 */
var hexcase = 0;  /* hex output format. 0 - lowercase; 1 - uppercase        */
var b64pad  = ""; /* base-64 pad character. "=" for strict RFC compliance   */

/*
 * These are the functions you'll usually want to call
 * They take string arguments and return either hex or base-64 encoded strings
 */
function hex_sha1(s)    { return rstr2hex(rstr_sha1(str2rstr_utf8(s))); }
function b64_sha1(s)    { return rstr2b64(rstr_sha1(str2rstr_utf8(s))); }
function any_sha1(s, e) { return rstr2any(rstr_sha1(str2rstr_utf8(s)), e); }
function hex_hmac_sha1(k, d)
  { return rstr2hex(rstr_hmac_sha1(str2rstr_utf8(k), str2rstr_utf8(d))); }
function b64_hmac_sha1(k, d)
  { return rstr2b64(rstr_hmac_sha1(str2rstr_utf8(k), str2rstr_utf8(d))); }
function any_hmac_sha1(k, d, e)
  { return rstr2any(rstr_hmac_sha1(str2rstr_utf8(k), str2rstr_utf8(d)), e); }

/*
 * Perform a simple self-test to see if the VM is working
 */
function sha1_vm_test()
{
  return hex_sha1("abc").toLowerCase() == "a9993e364706816aba3e25717850c26c9cd0d89d";
}

/*
 * Calculate the SHA1 of a raw string
 */
function rstr_sha1(s)
{
  return binb2rstr(binb_sha1(rstr2binb(s), s.length * 8));
}

/*
 * Calculate the HMAC-SHA1 of a key and some data (raw strings)
 */
function rstr_hmac_sha1(key, data)
{
  var bkey = rstr2binb(key);
  if(bkey.length > 16) bkey = binb_sha1(bkey, key.length * 8);

  var ipad = Array(16), opad = Array(16);
  for(var i = 0; i < 16; i++)
  {
    ipad[i] = bkey[i] ^ 0x36363636;
    opad[i] = bkey[i] ^ 0x5C5C5C5C;
  }

  var hash = binb_sha1(ipad.concat(rstr2binb(data)), 512 + data.length * 8);
  return binb2rstr(binb_sha1(opad.concat(hash), 512 + 160));
}

/*
 * Convert a raw string to a hex string
 */
function rstr2hex(input)
{
  try { hexcase } catch(e) { hexcase=0; }
  var hex_tab = hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
  var output = "";
  var x;
  for(var i = 0; i < input.length; i++)
  {
    x = input.charCodeAt(i);
    output += hex_tab.charAt((x >>> 4) & 0x0F)
           +  hex_tab.charAt( x        & 0x0F);
  }
  return output;
}

/*
 * Convert a raw string to a base-64 string
 */
function rstr2b64(input)
{
  try { b64pad } catch(e) { b64pad=''; }
  var tab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  var output = "";
  var len = input.length;
  for(var i = 0; i < len; i += 3)
  {
    var triplet = (input.charCodeAt(i) << 16)
                | (i + 1 < len ? input.charCodeAt(i+1) << 8 : 0)
                | (i + 2 < len ? input.charCodeAt(i+2)      : 0);
    for(var j = 0; j < 4; j++)
    {
      if(i * 8 + j * 6 > input.length * 8) output += b64pad;
      else output += tab.charAt((triplet >>> 6*(3-j)) & 0x3F);
    }
  }
  return output;
}

/*
 * Convert a raw string to an arbitrary string encoding
 */
function rstr2any(input, encoding)
{
  var divisor = encoding.length;
  var remainders = Array();
  var i, q, x, quotient;

  /* Convert to an array of 16-bit big-endian values, forming the dividend */
  var dividend = Array(Math.ceil(input.length / 2));
  for(i = 0; i < dividend.length; i++)
  {
    dividend[i] = (input.charCodeAt(i * 2) << 8) | input.charCodeAt(i * 2 + 1);
  }

  /*
   * Repeatedly perform a long division. The binary array forms the dividend,
   * the length of the encoding is the divisor. Once computed, the quotient
   * forms the dividend for the next step. We stop when the dividend is zero.
   * All remainders are stored for later use.
   */
  while(dividend.length > 0)
  {
    quotient = Array();
    x = 0;
    for(i = 0; i < dividend.length; i++)
    {
      x = (x << 16) + dividend[i];
      q = Math.floor(x / divisor);
      x -= q * divisor;
      if(quotient.length > 0 || q > 0)
        quotient[quotient.length] = q;
    }
    remainders[remainders.length] = x;
    dividend = quotient;
  }

  /* Convert the remainders to the output string */
  var output = "";
  for(i = remainders.length - 1; i >= 0; i--)
    output += encoding.charAt(remainders[i]);

  /* Append leading zero equivalents */
  var full_length = Math.ceil(input.length * 8 /
                                    (Math.log(encoding.length) / Math.log(2)))
  for(i = output.length; i < full_length; i++)
    output = encoding[0] + output;

  return output;
}

/*
 * Encode a string as utf-8.
 * For efficiency, this assumes the input is valid utf-16.
 */
function str2rstr_utf8(input)
{
  var output = "";
  var i = -1;
  var x, y;

  while(++i < input.length)
  {
    /* Decode utf-16 surrogate pairs */
    x = input.charCodeAt(i);
    y = i + 1 < input.length ? input.charCodeAt(i + 1) : 0;
    if(0xD800 <= x && x <= 0xDBFF && 0xDC00 <= y && y <= 0xDFFF)
    {
      x = 0x10000 + ((x & 0x03FF) << 10) + (y & 0x03FF);
      i++;
    }

    /* Encode output as utf-8 */
    if(x <= 0x7F)
      output += String.fromCharCode(x);
    else if(x <= 0x7FF)
      output += String.fromCharCode(0xC0 | ((x >>> 6 ) & 0x1F),
                                    0x80 | ( x         & 0x3F));
    else if(x <= 0xFFFF)
      output += String.fromCharCode(0xE0 | ((x >>> 12) & 0x0F),
                                    0x80 | ((x >>> 6 ) & 0x3F),
                                    0x80 | ( x         & 0x3F));
    else if(x <= 0x1FFFFF)
      output += String.fromCharCode(0xF0 | ((x >>> 18) & 0x07),
                                    0x80 | ((x >>> 12) & 0x3F),
                                    0x80 | ((x >>> 6 ) & 0x3F),
                                    0x80 | ( x         & 0x3F));
  }
  return output;
}

/*
 * Encode a string as utf-16
 */
function str2rstr_utf16le(input)
{
  var output = "";
  for(var i = 0; i < input.length; i++)
    output += String.fromCharCode( input.charCodeAt(i)        & 0xFF,
                                  (input.charCodeAt(i) >>> 8) & 0xFF);
  return output;
}

function str2rstr_utf16be(input)
{
  var output = "";
  for(var i = 0; i < input.length; i++)
    output += String.fromCharCode((input.charCodeAt(i) >>> 8) & 0xFF,
                                   input.charCodeAt(i)        & 0xFF);
  return output;
}

/*
 * Convert a raw string to an array of big-endian words
 * Characters >255 have their high-byte silently ignored.
 */
function rstr2binb(input)
{
  var output = Array(input.length >> 2);
  for(var i = 0; i < output.length; i++)
    output[i] = 0;
  for(var i = 0; i < input.length * 8; i += 8)
    output[i>>5] |= (input.charCodeAt(i / 8) & 0xFF) << (24 - i % 32);
  return output;
}

/*
 * Convert an array of big-endian words to a string
 */
function binb2rstr(input)
{
  var output = "";
  for(var i = 0; i < input.length * 32; i += 8)
    output += String.fromCharCode((input[i>>5] >>> (24 - i % 32)) & 0xFF);
  return output;
}

/*
 * Calculate the SHA-1 of an array of big-endian words, and a bit length
 */
function binb_sha1(x, len)
{
  /* append padding */
  x[len >> 5] |= 0x80 << (24 - len % 32);
  x[((len + 64 >> 9) << 4) + 15] = len;

  var w = Array(80);
  var a =  1732584193;
  var b = -271733879;
  var c = -1732584194;
  var d =  271733878;
  var e = -1009589776;

  for(var i = 0; i < x.length; i += 16)
  {
    var olda = a;
    var oldb = b;
    var oldc = c;
    var oldd = d;
    var olde = e;

    for(var j = 0; j < 80; j++)
    {
      if(j < 16) w[j] = x[i + j];
      else w[j] = bit_rol(w[j-3] ^ w[j-8] ^ w[j-14] ^ w[j-16], 1);
      var t = safe_add(safe_add(bit_rol(a, 5), sha1_ft(j, b, c, d)),
                       safe_add(safe_add(e, w[j]), sha1_kt(j)));
      e = d;
      d = c;
      c = bit_rol(b, 30);
      b = a;
      a = t;
    }

    a = safe_add(a, olda);
    b = safe_add(b, oldb);
    c = safe_add(c, oldc);
    d = safe_add(d, oldd);
    e = safe_add(e, olde);
  }
  return Array(a, b, c, d, e);

}

/*
 * Perform the appropriate triplet combination function for the current
 * iteration
 */
function sha1_ft(t, b, c, d)
{
  if(t < 20) return (b & c) | ((~b) & d);
  if(t < 40) return b ^ c ^ d;
  if(t < 60) return (b & c) | (b & d) | (c & d);
  return b ^ c ^ d;
}

/*
 * Determine the appropriate additive constant for the current iteration
 */
function sha1_kt(t)
{
  return (t < 20) ?  1518500249 : (t < 40) ?  1859775393 :
         (t < 60) ? -1894007588 : -899497514;
}

/*
 * Add integers, wrapping at 2^32. This uses 16-bit operations internally
 * to work around bugs in some JS interpreters.
 */
function safe_add(x, y)
{
  var lsw = (x & 0xFFFF) + (y & 0xFFFF);
  var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
  return (msw << 16) | (lsw & 0xFFFF);
}

/*
 * Bitwise rotate a 32-bit number to the left.
 */
function bit_rol(num, cnt)
{
  return (num << cnt) | (num >>> (32 - cnt));
}
;
define("spice-sha1", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

var SHA_DIGEST_LENGTH = 20;

/*----------------------------------------------------------------------------
**  General ticket RSA encryption functions - just good enough to
**      support what we need to send back an encrypted ticket.
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**  OAEP padding functions.  Inspired by the OpenSSL implementation.
**--------------------------------------------------------------------------*/
function MGF1(mask, seed)
{
    var i, j, outlen;
    for (i = 0, outlen = 0; outlen < mask.length; i++)
    {
        var combo_buf = new String;

        for (j = 0; j < seed.length; j++)
            combo_buf += String.fromCharCode(seed[j]);
        combo_buf += String.fromCharCode((i >> 24) & 255);
        combo_buf += String.fromCharCode((i >> 16) & 255);
        combo_buf += String.fromCharCode((i >> 8)  & 255);
        combo_buf += String.fromCharCode((i)       & 255);

        var combo_hash = rstr_sha1(combo_buf);
        for (j = 0; j < combo_hash.length && outlen < mask.length; j++, outlen++)
        {
            mask[outlen] = combo_hash.charCodeAt(j);
        }
    }
}


function RSA_padding_add_PKCS1_OAEP(tolen, from, param)
{
    var seed = new Array(SHA_DIGEST_LENGTH);
    var rand = new SecureRandom();
    rand.nextBytes(seed);

    var dblen = tolen - 1 - seed.length;
    var db = new Array(dblen);
    var padlen = dblen - from.length - 1;
    var i;

    if (param === undefined)
        param = "";

    if (padlen < SHA_DIGEST_LENGTH)
    {
        console.log("Error - data too large for key size.");
        return null;
    }

    for (i = 0; i < padlen; i++)
        db[i] = 0;

    var param_hash = rstr_sha1(param);
    for (i = 0; i < param_hash.length; i++)
        db[i] = param_hash.charCodeAt(i);

    db[padlen] = 1;
    for (i = 0; i < from.length; i++)
        db[i + padlen + 1] = from.charCodeAt(i);

    var dbmask = new Array(dblen);
    if (MGF1(dbmask, seed) < 0)
            return null;

    for (i = 0; i < dbmask.length; i++)
        db[i] ^= dbmask[i];


    var seedmask = Array(SHA_DIGEST_LENGTH);
    if (MGF1(seedmask, db) < 0)
            return null;

    for (i = 0; i < seedmask.length; i++)
        seed[i] ^= seedmask[i];

    var ret = new String;
    ret += String.fromCharCode(0);
    for (i = 0; i < seed.length; i++)
        ret += String.fromCharCode(seed[i]);
    for (i = 0; i < db.length; i++)
        ret += String.fromCharCode(db[i]);
    return ret;
}


function asn_get_length(u8, at)
{
    var len = u8[at++];
    if (len > 0x80)
    {
        if (len != 0x81)
        {
            console.log("Error:  we lazily don't support keys bigger than 255 bytes.  It'd be easy to fix.");
            return null;
        }
        len = u8[at++];
    }

    return [ at, len];
}

function find_sequence(u8, at)
{
    var lenblock;
    at = at || 0;
    if (u8[at++] != 0x30)
    {
        console.log("Error:  public key should start with a sequence flag.");
        return null;
    }

    lenblock = asn_get_length(u8, at);
    if (! lenblock)
        return null;
    return lenblock;
}

/*----------------------------------------------------------------------------
**  Extract an RSA key from a memory buffer
**--------------------------------------------------------------------------*/
function create_rsa_from_mb(mb, at)
{
    var u8 = new Uint8Array(mb);
    var lenblock;
    var seq;
    var ba;
    var i;
    var ret;

    /* We have a sequence which contains a sequence followed by a bit string */
    seq = find_sequence(u8, at);
    if (! seq)
        return null;

    at = seq[0];
    seq = find_sequence(u8, at);
    if (! seq)
        return null;

    /* Skip over the contained sequence */
    at = seq[0] + seq[1];
    if (u8[at++] != 0x3)
    {
        console.log("Error: expecting bit string next.");
        return null;
    }

    /* Get the bit string, which is *itself* a sequence.  Having fun yet? */
    lenblock = asn_get_length(u8, at);
    if (! lenblock)
        return null;

    at = lenblock[0];
    if (u8[at] != 0 && u8[at + 1] != 0x30)
    {
        console.log("Error: unexpected values in bit string.");
        return null;
    }

    /* Okay, now we have a sequence of two binary values, we hope. */
    seq = find_sequence(u8, at + 1);
    if (! seq)
        return null;

    at = seq[0];
    if (u8[at++] != 0x02)
    {
        console.log("Error: expecting integer n next.");
        return null;
    }
    lenblock = asn_get_length(u8, at);
    if (! lenblock)
        return null;
    at = lenblock[0];

    ba = new Array(lenblock[1]);
    for (i = 0; i < lenblock[1]; i++)
        ba[i] = u8[at + i];

    ret = new RSAKey();
    ret.n = new BigInteger(ba);

    at += lenblock[1];

    if (u8[at++] != 0x02)
    {
        console.log("Error: expecting integer e next.");
        return null;
    }
    lenblock = asn_get_length(u8, at);
    if (! lenblock)
        return null;
    at = lenblock[0];

    ret.e = u8[at++];
    for (i = 1; i < lenblock[1]; i++)
    {
        ret.e <<= 8;
        ret.e |= u8[at++];
    }

    return ret;
}

function rsa_encrypt(rsa, str)
{
    var i;
    var ret = [];
    var oaep = RSA_padding_add_PKCS1_OAEP((rsa.n.bitLength()+7)>>3, str);
    if (! oaep)
        return null;

    var ba = new Array(oaep.length);

    for (i = 0; i < oaep.length; i++)
        ba[i] = oaep.charCodeAt(i);
    var bigint = new BigInteger(ba);
    var enc = rsa.doPublic(bigint);
    var h = enc.toString(16);
    if ((h.length & 1) != 0)
        h = "0" + h;
    for (i = 0; i < h.length; i += 2)
        ret[i / 2] = parseInt(h.substring(i, i + 2), 16);
    return ret;
}
;
define("spice-ticket", function(){});


/*
   Copyright (C) 2014 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*----------------------------------------------------------------------------
**  resize.js
**      This bit of Javascript is a set of logic to help with window
**  resizing, using the agent channel to request screen resizes.
**
**  It's a bit tricky, as we want to wait for resizing to settle down
**  before sending a size.  Further, while horizontal resizing to use the whole
**  browser width is fairly easy to arrange with css, resizing an element to use
**  the whole vertical space (or to force a middle div to consume the bulk of the browser
**  window size) is tricky, and the consensus seems to be that Javascript is
**  the only right way to do it.
**--------------------------------------------------------------------------*/
function resize_helper(sc)
{
    var w = document.getElementById(sc.screen_id).clientWidth;
    var h = document.getElementById(sc.screen_id).clientHeight;

    var m = document.getElementById(sc.message_id);

    /* Resize vertically; basically we leave a 20 pixel margin
         at the bottom, and use the position of the message window
         to figure out how to resize */
    var hd = window.innerHeight - m.offsetHeight - m.offsetTop - 20;

    /* Xorg requires height be a multiple of 8; round up */
    h = h + hd;
    if (h % 8 > 0)
        h += (8 - (h % 8));

    /* Xorg requires width be a multiple of 8; round up */
    if (w % 8 > 0)
        w += (8 - (w % 8));


    sc.resize_window(0, w, h, 32, 0, 0);
    sc.spice_resize_timer = undefined;
}

function handle_resize(e)
{
    var sc = window.spice_connection;

    if (sc && sc.spice_resize_timer)
    {
        window.clearTimeout(sc.spice_resize_timer);
        sc.spice_resize_timer = undefined;
    }

    sc.spice_resize_timer = window.setTimeout(resize_helper, 200, sc);
}
;
define("spice-resize", function(){});


/*
   Copyright (C) 2014 Red Hat, Inc.

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

function SpiceFileXferTask(id, file)
{
    this.id = id;
    this.file = file;
}

SpiceFileXferTask.prototype.create_progressbar = function()
{
    var _this = this;
    var cancel = document.createElement("input");
    this.progressbar_container = document.createElement("div");
    this.progressbar = document.createElement("progress");

    cancel.type = 'button';
    cancel.value = 'Cancel';
    cancel.style.float = 'right';
    cancel.onclick = function()
    {
        _this.cancelled = true;
        _this.remove_progressbar();
    };

    this.progressbar.setAttribute('max', this.file.size);
    this.progressbar.setAttribute('value', 0);
    this.progressbar.style.width = '100%';
    this.progressbar.style.margin = '4px auto';
    this.progressbar.style.display = 'inline-block';
    this.progressbar_container.style.width = '90%';
    this.progressbar_container.style.margin = 'auto';
    this.progressbar_container.style.padding = '4px';
    this.progressbar_container.textContent = this.file.name;
    this.progressbar_container.appendChild(cancel);
    this.progressbar_container.appendChild(this.progressbar);
    document.getElementById('spice-xfer-area').appendChild(this.progressbar_container);
}

SpiceFileXferTask.prototype.update_progressbar = function(value)
{
    this.progressbar.setAttribute('value', value);
}

SpiceFileXferTask.prototype.remove_progressbar = function()
{
    if (this.progressbar_container && this.progressbar_container.parentNode)
        this.progressbar_container.parentNode.removeChild(this.progressbar_container);
}

function handle_file_dragover(e)
{
    e.stopPropagation();
    e.preventDefault();
    e.dataTransfer.dropEffect = 'copy';
}

function handle_file_drop(e)
{
    var sc = window.spice_connection;
    var files = e.dataTransfer.files;

    e.stopPropagation();
    e.preventDefault();
    for (var i = files.length - 1; i >= 0; i--)
    {
        if (files[i].type); // do not copy a directory
            sc.file_xfer_start(files[i]);
    }

}
;
define("spice-filexfer", function(){});


/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This file is part of spice-html5.

   spice-html5 is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   spice-html5 is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with spice-html5.  If not, see <http://www.gnu.org/licenses/>.
*/

/*----------------------------------------------------------------------------
**  SpiceMainConn
**      This is the master Javascript class for establishing and
**  managing a connection to a Spice Server.
**  
**      Invocation:  You must pass an object with properties as follows:
**          uri         (required)  Uri of a WebSocket listener that is
**                                  connected to a spice server.
**          password    (required)  Password to send to the spice server
**          message_id  (optional)  Identifier of an element in the DOM
**                                  where SpiceConn will write messages.
**                                  It will use classes spice-messages-x,
**                                  where x is one of info, warning, or error.
**          screen_id   (optional)  Identifier of an element in the DOM
**                                  where SpiceConn will create any new
**                                  client screens.  This is the main UI.
**          dump_id     (optional)  If given, an element to use for
**                                  dumping every single image + canvas drawn.
**                                  Sometimes useful for debugging.
**          onerror     (optional)  If given, a function to receive async
**                                  errors.  Note that you should also catch
**                                  errors for ones that occur inline
**          onagent     (optional)  If given, a function to be called when
**                                  a VD agent is connected; a good opportunity
**                                  to request a resize
**
**  Throws error if there are troubles.  Requires a modern (by 2012 standards)
**      browser, including WebSocket and WebSocket.binaryType == arraybuffer
**
**--------------------------------------------------------------------------*/
function SpiceMainConn()
{
    if (typeof WebSocket === "undefined")
        throw new Error("WebSocket unavailable.  You need to use a different browser.");

    SpiceConn.apply(this, arguments);

    this.agent_msg_queue = [];
    this.file_xfer_tasks = {};
    this.file_xfer_task_id = 0;
    this.file_xfer_read_queue = [];
}

SpiceMainConn.prototype = Object.create(SpiceConn.prototype);
SpiceMainConn.prototype.process_channel_message = function(msg)
{
    if (msg.type == SPICE_MSG_MAIN_INIT)
    {
        this.log_info("Connected to " + this.ws.url);
        this.report_success("Connected")
        this.main_init = new SpiceMsgMainInit(msg.data);
        this.connection_id = this.main_init.session_id;
        this.agent_tokens = this.main_init.agent_tokens;

        if (DEBUG > 0)
        {
            // FIXME - there is a lot here we don't handle; mouse modes, agent,
            //          ram_hint, multi_media_time
            this.log_info("session id "                 + this.main_init.session_id +
                          " ; display_channels_hint "   + this.main_init.display_channels_hint +
                          " ; supported_mouse_modes "   + this.main_init.supported_mouse_modes +
                          " ; current_mouse_mode "      + this.main_init.current_mouse_mode +
                          " ; agent_connected "         + this.main_init.agent_connected +
                          " ; agent_tokens "            + this.main_init.agent_tokens +
                          " ; multi_media_time "        + this.main_init.multi_media_time +
                          " ; ram_hint "                + this.main_init.ram_hint);
        }

        this.handle_mouse_mode(this.main_init.current_mouse_mode,
                               this.main_init.supported_mouse_modes);

        if (this.main_init.agent_connected)
            this.connect_agent();

        var attach = new SpiceMiniData;
        attach.type = SPICE_MSGC_MAIN_ATTACH_CHANNELS;
        attach.size = attach.buffer_size();
        this.send_msg(attach);
        return true;
    }

    if (msg.type == SPICE_MSG_MAIN_MOUSE_MODE)
    {
        var mode = new SpiceMsgMainMouseMode(msg.data);
        DEBUG > 0 && this.log_info("Mouse supported modes " + mode.supported_modes + "; current " + mode.current_mode);
        this.handle_mouse_mode(mode.current_mode, mode.supported_modes);
        return true;
    }

    if (msg.type == SPICE_MSG_MAIN_CHANNELS_LIST)
    {
        var i;
        var chans;
        DEBUG > 0 && console.log("channels");
        chans = new SpiceMsgChannels(msg.data);
        for (i = 0; i < chans.channels.length; i++)
        {
            var conn = {
                        uri: this.ws.url,
                        parent: this,
                        connection_id : this.connection_id,
                        type : chans.channels[i].type,
                        chan_id : chans.channels[i].id
                    };
            if (chans.channels[i].type == SPICE_CHANNEL_DISPLAY)
                this.display = new SpiceDisplayConn(conn);
            else if (chans.channels[i].type == SPICE_CHANNEL_INPUTS)
            {
                this.inputs = new SpiceInputsConn(conn);
                this.inputs.mouse_mode = this.mouse_mode;
            }
            else if (chans.channels[i].type == SPICE_CHANNEL_CURSOR)
                this.cursor = new SpiceCursorConn(conn);
            else if (chans.channels[i].type == SPICE_CHANNEL_PLAYBACK)
                this.cursor = new SpicePlaybackConn(conn);
            else
            {
                this.log_err("Channel type " + chans.channels[i].type + " unknown.");
                if (! ("extra_channels" in this))
                    this.extra_channels = [];
                this.extra_channels[i] = new SpiceConn(conn);
            }

        }

        return true;
    }

    if (msg.type == SPICE_MSG_MAIN_AGENT_CONNECTED)
    {
        this.connect_agent();
        return true;
    }

    if (msg.type == SPICE_MSG_MAIN_AGENT_CONNECTED_TOKENS)
    {
        var connected_tokens = new SpiceMsgMainAgentTokens(msg.data);
        this.agent_tokens = connected_tokens.num_tokens;
        this.connect_agent();
        return true;
    }

    if (msg.type == SPICE_MSG_MAIN_AGENT_TOKEN)
    {
        var remaining_tokens, tokens = new SpiceMsgMainAgentTokens(msg.data);
        this.agent_tokens += tokens.num_tokens;
        this.send_agent_message_queue();

        remaining_tokens = this.agent_tokens;
        while (remaining_tokens > 0 && this.file_xfer_read_queue.length > 0)
        {
            var xfer_task = this.file_xfer_read_queue.shift();
            this.file_xfer_read(xfer_task, xfer_task.read_bytes);
            remaining_tokens--;
        }
        return true;
    }

    if (msg.type == SPICE_MSG_MAIN_AGENT_DISCONNECTED)
    {
        this.agent_connected = false;
        return true;
    }

    if (msg.type == SPICE_MSG_MAIN_AGENT_DATA)
    {
        var agent_data = new SpiceMsgMainAgentData(msg.data);
        if (agent_data.type == VD_AGENT_ANNOUNCE_CAPABILITIES)
        {
            var agent_caps = new VDAgentAnnounceCapabilities(agent_data.data);
            if (agent_caps.request)
                this.announce_agent_capabilities(0);
            return true;
        }
        else if (agent_data.type == VD_AGENT_FILE_XFER_STATUS)
        {
            this.handle_file_xfer_status(new VDAgentFileXferStatusMessage(agent_data.data));
            return true;
        }

        return false;
    }

    return false;
}

SpiceMainConn.prototype.stop = function(msg)
{
    this.state = "closing";

    if (this.inputs)
    {
        this.inputs.cleanup();
        this.inputs = undefined;
    }

    if (this.cursor)
    {
        this.cursor.cleanup();
        this.cursor = undefined;
    }

    if (this.display)
    {
        this.display.cleanup();
        this.display.destroy_surfaces();
        this.display = undefined;
    }

    this.cleanup();

    if ("extra_channels" in this)
        for (var e in this.extra_channels)
            this.extra_channels[e].cleanup();
    this.extra_channels = undefined;
}

SpiceMainConn.prototype.send_agent_message_queue = function(message)
{
    if (!this.agent_connected)
        return;

    if (message)
        this.agent_msg_queue.push(message);

    while (this.agent_tokens > 0 && this.agent_msg_queue.length > 0)
    {
        var mr = this.agent_msg_queue.shift();
        this.send_msg(mr);
        this.agent_tokens--;
    }
}

SpiceMainConn.prototype.send_agent_message = function(type, message)
{
    var agent_data = new SpiceMsgcMainAgentData(type, message);
    var sb = 0, maxsize = VD_AGENT_MAX_DATA_SIZE - SpiceMiniData.prototype.buffer_size();
    var data = new ArrayBuffer(agent_data.buffer_size());
    agent_data.to_buffer(data);
    while (sb < agent_data.buffer_size())
    {
        var eb = Math.min(sb + maxsize, agent_data.buffer_size());
        var mr = new SpiceMiniData();
        mr.type = SPICE_MSGC_MAIN_AGENT_DATA;
        mr.size = eb - sb;
        mr.data = data.slice(sb, eb);
        this.send_agent_message_queue(mr);
        sb = eb;
    }
}

SpiceMainConn.prototype.announce_agent_capabilities = function(request)
{
    var caps = new VDAgentAnnounceCapabilities(request, (1 << VD_AGENT_CAP_MOUSE_STATE) |
                                                        (1 << VD_AGENT_CAP_MONITORS_CONFIG) |
                                                        (1 << VD_AGENT_CAP_REPLY));
    this.send_agent_message(VD_AGENT_ANNOUNCE_CAPABILITIES, caps);
}

SpiceMainConn.prototype.resize_window = function(flags, width, height, depth, x, y)
{
    var monitors_config = new VDAgentMonitorsConfig(flags, width, height, depth, x, y);
    this.send_agent_message(VD_AGENT_MONITORS_CONFIG, monitors_config);
}

SpiceMainConn.prototype.file_xfer_start = function(file)
{
    var task_id, xfer_start, task;

    task_id = this.file_xfer_task_id++;
    task = new SpiceFileXferTask(task_id, file);
    task.create_progressbar();
    this.file_xfer_tasks[task_id] = task;
    xfer_start = new VDAgentFileXferStartMessage(task_id, file.name, file.size);
    this.send_agent_message(VD_AGENT_FILE_XFER_START, xfer_start);
}

SpiceMainConn.prototype.handle_file_xfer_status = function(file_xfer_status)
{
    var xfer_error, xfer_task;
    if (!this.file_xfer_tasks[file_xfer_status.id])
    {
        return;
    }
    xfer_task = this.file_xfer_tasks[file_xfer_status.id];
    switch (file_xfer_status.result)
    {
        case VD_AGENT_FILE_XFER_STATUS_CAN_SEND_DATA:
            this.file_xfer_read(xfer_task);
            return;
        case VD_AGENT_FILE_XFER_STATUS_CANCELLED:
            xfer_error = "transfer is cancelled by spice agent";
            break;
        case VD_AGENT_FILE_XFER_STATUS_ERROR:
            xfer_error = "some errors occurred in the spice agent";
            break;
        case VD_AGENT_FILE_XFER_STATUS_SUCCESS:
            break;
        default:
            xfer_error = "unhandled status type: " + file_xfer_status.result;
            break;
    }

    this.file_xfer_completed(xfer_task, xfer_error)
}

SpiceMainConn.prototype.file_xfer_read = function(file_xfer_task, start_byte)
{
    var FILE_XFER_CHUNK_SIZE = 32 * VD_AGENT_MAX_DATA_SIZE;
    var _this = this;
    var sb, eb;
    var slice, reader;

    if (!file_xfer_task ||
        !this.file_xfer_tasks[file_xfer_task.id] ||
        (start_byte > 0 && start_byte == file_xfer_task.file.size))
    {
        return;
    }

    if (file_xfer_task.cancelled)
    {
        var xfer_status = new VDAgentFileXferStatusMessage(file_xfer_task.id,
                                                           VD_AGENT_FILE_XFER_STATUS_CANCELLED);
        this.send_agent_message(VD_AGENT_FILE_XFER_STATUS, xfer_status);
        delete this.file_xfer_tasks[file_xfer_task.id];
        return;
    }

    sb = start_byte || 0,
    eb = Math.min(sb + FILE_XFER_CHUNK_SIZE, file_xfer_task.file.size);

    if (!this.agent_tokens)
    {
        file_xfer_task.read_bytes = sb;
        this.file_xfer_read_queue.push(file_xfer_task);
        return;
    }

    reader = new FileReader();
    reader.onload = function(e)
    {
        var xfer_data = new VDAgentFileXferDataMessage(file_xfer_task.id,
                                                       e.target.result.byteLength,
                                                       e.target.result);
        _this.send_agent_message(VD_AGENT_FILE_XFER_DATA, xfer_data);
        _this.file_xfer_read(file_xfer_task, eb);
        file_xfer_task.update_progressbar(eb);
    };

    slice = file_xfer_task.file.slice(sb, eb);
    reader.readAsArrayBuffer(slice);
}

SpiceMainConn.prototype.file_xfer_completed = function(file_xfer_task, error)
{
    if (error)
        this.log_err(error);
    else
        this.log_info("transfer of '" + file_xfer_task.file.name +"' was successful");

    file_xfer_task.remove_progressbar();

    delete this.file_xfer_tasks[file_xfer_task.id];
}

SpiceMainConn.prototype.connect_agent = function()
{
    this.agent_connected = true;

    var agent_start = new SpiceMsgcMainAgentStart(~0);
    var mr = new SpiceMiniData();
    mr.build_msg(SPICE_MSGC_MAIN_AGENT_START, agent_start);
    this.send_msg(mr);

    this.announce_agent_capabilities(1);

    if (this.onagent !== undefined)
        this.onagent(this);

}

SpiceMainConn.prototype.handle_mouse_mode = function(current, supported)
{
    this.mouse_mode = current;
    if (current != SPICE_MOUSE_MODE_CLIENT && (supported & SPICE_MOUSE_MODE_CLIENT))
    {
        var mode_request = new SpiceMsgcMainMouseModeRequest(SPICE_MOUSE_MODE_CLIENT);
        var mr = new SpiceMiniData();
        mr.build_msg(SPICE_MSGC_MAIN_MOUSE_MODE_REQUEST, mode_request);
        this.send_msg(mr);
    }

    if (this.inputs)
        this.inputs.mouse_mode = current;
}

;
define("spice-main", ["spice-spiceconn","spice-spicearraybuffer","spice-enums","spice-atKeynames","spice-utils","spice-png","spice-lz","spice-quic","spice-bitmap","spice-spicedataview","spice-spicetype","spice-spicemsg","spice-wire","spice-display","spice-inputs","spice-webm","spice-playback","spice-simulatecursor","spice-cursor","spice-jsbn","spice-rsa","spice-prng4","spice-rng","spice-sha1","spice-ticket","spice-resize","spice-filexfer"], (function (global) {
    return function () {
        var ret, fn;
        return ret || global.SpiceMainConn;
    };
}(this)));

define('utils/spice',['require','spice-main','sunstone-config','utils/notifier'],function(require) {
  require('spice-main');
  var Config = require('sunstone-config');
  var Notifier = require('utils/notifier');

  var _lock = false;
  var _sc;

  return {
    'lockStatus': lockStatus,
    'lock': lock,
    'unlock': unlock,
    'spiceCallback': spiceCallback,
    'disconnect': disconnect
  }

  function lockStatus() {
    return _lock;
  }

  function lock() {
    _lock = true;
  }

  function unlock() {
    _lock = false;
  }

  function spice_error(e) {
    disconnect();
  }

  function disconnect() {
    if (_sc) {
      _sc.stop();
    }
  }

  function agent_connected(sc) {
    window.addEventListener('resize', handle_resize);
    window.spice_connection = this;

    resize_helper(this);
  }

  function spiceCallback(response) {
    var host, port, password, scheme = "ws://", uri, token, vm_name;

    if (Config.vncWSS == "yes") {
      scheme = "wss://";
    }

    host = window.location.hostname;
    port = Config.vncProxyPort;
    password = response["password"];
    token = response["token"];
    vm_name = response["vm_name"];

    if ((!host) || (!port)) {
      console.log("must specify host and port in URL");
      return;
    }

    if (_sc) {
      _sc.stop();
    }

    uri = scheme + host + ":" + port + "?token=" + token;

    try {
      _sc = new SpiceMainConn({uri: uri, screen_id: "spice-screen", dump_id: "debug-div",
                  message_id: "message-div", password: password, onerror: spice_error, onagent: agent_connected});
    }
    catch (e) {
      spice_error(e)
    }

    var url = "spice?";
    url += "host=" + host;
    url += "&port=" + port;
    url += "&token=" + token;
    url += "&password=" + password;
    url += "&encrypt=" + config['user_config']['vnc_wss'];
    url += "&title=" + vm_name;

    $("#open_in_a_new_window_spice").attr('href', url);
  }
});

define('tabs/vms-tab/dialogs/deploy/dialogId',['require'],function(require){
  return 'deployVMDialog';
});
define('tabs/vms-tab/dialogs/migrate/dialogId',['require'],function(require){
  return 'migrateVMDialog';
});
define('tabs/vms-tab/dialogs/vnc/dialogId',['require'],function(require) {
  return 'vncVMDialog';
});

define('tabs/vms-tab/dialogs/spice/dialogId',['require'],function(require) {
  return 'spiceVMDialog';
});

define('tabs/vms-tab/dialogs/saveas-template/dialogId',['require'],function(require) {
  return 'saveAsTemplate';
})
;
define('tabs/vms-tab/actions',['require','sunstone','utils/notifier','utils/locale','opennebula/vm','utils/common-actions','utils/vnc','utils/spice','./tabId','./form-panels/create/formPanelId','./dialogs/deploy/dialogId','./dialogs/migrate/dialogId','./dialogs/vnc/dialogId','./dialogs/spice/dialogId','./dialogs/saveas-template/dialogId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var OpenNebulaVM = require('opennebula/vm');
  var CommonActions = require('utils/common-actions');
  var Vnc = require('utils/vnc');
  var Spice = require('utils/spice');

  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID           = require('./form-panels/create/formPanelId');
  var DEPLOY_DIALOG_ID           = require('./dialogs/deploy/dialogId');
  var MIGRATE_DIALOG_ID          = require('./dialogs/migrate/dialogId');
  var VNC_DIALOG_ID              = require('./dialogs/vnc/dialogId');
  var SPICE_DIALOG_ID            = require('./dialogs/spice/dialogId');
  var SAVE_AS_TEMPLATE_DIALOG_ID = require('./dialogs/saveas-template/dialogId');

  var XML_ROOT = "VM";
  var RESOURCE = "VM";

  var _commonActions = new CommonActions(OpenNebulaVM, RESOURCE, TAB_ID);

  var _actions = {
    "VM.list":    _commonActions.list(),
    "VM.show":    _commonActions.show(),
    "VM.refresh": _commonActions.refresh(),
    "VM.delete":  _commonActions.del(),
    "VM.chown": _commonActions.multipleAction('chown'),
    "VM.chgrp": _commonActions.multipleAction('chgrp'),

    "VM.hold":    _commonActions.multipleAction('hold'),
    "VM.release": _commonActions.multipleAction('release'),
    "VM.suspend": _commonActions.multipleAction('suspend'),
    "VM.resume": _commonActions.multipleAction('resume'),
    "VM.stop": _commonActions.multipleAction('stop'),
    "VM.reboot_hard": _commonActions.multipleAction('reset'),
    "VM.delete_recreate": _commonActions.multipleAction('resubmit'),
    "VM.reboot": _commonActions.multipleAction('reboot'),
    "VM.poweroff": _commonActions.multipleAction('poweroff'),
    "VM.poweroff_hard": _commonActions.multipleAction('poweroff_hard'),
    "VM.undeploy": _commonActions.multipleAction('undeploy'),
    "VM.undeploy_hard": _commonActions.multipleAction('undeploy_hard'),
    "VM.shutdown": _commonActions.multipleAction('shutdown'),
    "VM.shutdown_hard": _commonActions.multipleAction('shutdown_hard'),
    "VM.recover": _commonActions.multipleAction('recover'),
    "VM.resched": _commonActions.multipleAction('resched'),
    "VM.unresched": _commonActions.multipleAction('unresched'),

    "VM.chmod": _commonActions.singleAction('chmod'),
    "VM.rename": _commonActions.singleAction('rename'),
    "VM.update_template": _commonActions.updateTemplate(),
    "VM.deploy_action": _commonActions.singleAction('deploy'),
    "VM.migrate_action": _commonActions.singleAction('migrate'),
    "VM.migrate_live_action": _commonActions.singleAction('livemigrate'),
    "VM.attachdisk": _commonActions.singleAction('attachdisk'),
    "VM.detachdisk": _commonActions.singleAction('detachdisk'),
    "VM.attachnic": _commonActions.singleAction('attachnic'),
    "VM.detachnic": _commonActions.singleAction('detachnic'),
    "VM.resize": _commonActions.singleAction('resize'),
    "VM.snapshot_create": _commonActions.singleAction('snapshot_create'),
    "VM.snapshot_revert": _commonActions.singleAction('snapshot_revert'),
    "VM.snapshot_delete": _commonActions.singleAction('snapshot_delete'),
    "VM.disk_snapshot_create": _commonActions.singleAction('disk_snapshot_create'),
    "VM.disk_snapshot_revert": _commonActions.singleAction('disk_snapshot_revert'),
    "VM.disk_snapshot_delete": _commonActions.singleAction('disk_snapshot_delete'),
    "VM.disk_saveas" : _commonActions.singleAction('disk_saveas'),

    "VM.create_dialog" : {
      type: "custom",
      call: function() {
        Sunstone.showFormPanel(TAB_ID, CREATE_DIALOG_ID, "create");
      }
    },
    "VM.create" : {
      type: "custom",
      call: function(id, name) {
        Sunstone.runAction("Template.instantiate", [id], name);
        Sunstone.runAction("VM.refresh");
      },
      callback: function(request, response) {
        Sunstone.getDataTable(TAB_ID).addElement(request, response);
      },
      error: Notifier.onError
    },
    "VM.deploy" : {
      type: "custom",
      call: function() {
        Sunstone.getDialog(DEPLOY_DIALOG_ID).show();
      }
    },
    "VM.silent_deploy_action" : {
      type: "single",
      call: OpenNebulaVM.deploy,
      error: Notifier.onError
    },
    "VM.migrate" : {
      type: "custom",
      call: function() {
       var dialog = Sunstone.getDialog(MIGRATE_DIALOG_ID);
       dialog.setLive(false);
       dialog.show();
     }
    },
    "VM.migrate_live" : {
      type: "custom",
      call: function() {
       var dialog = Sunstone.getDialog(MIGRATE_DIALOG_ID);
       dialog.setLive(true);
       dialog.show();
     }
    },
    "VM.startvnc" : {
      type: "custom",
      call: function() {
        $.each(Sunstone.getDataTable(TAB_ID).elements(), function(index, elem) {
          if (!Vnc.lockStatus()) {
            Vnc.lock();
            Sunstone.runAction("VM.startvnc_action", elem);
          } else {
            Notifier.notifyError(Locale.tr("VNC Connection in progress"))
            return false;
          }
        });
      }
    },
    "VM.startvnc_action" : {
      type: "single",
      call: OpenNebulaVM.vnc,
      callback: function(request, response) {
       var dialog = Sunstone.getDialog(VNC_DIALOG_ID);
       dialog.setElement(response);
       dialog.show();
      },
      error: function(req, resp) {
        Notifier.onError(req, resp);
        Vnc.unlock();
      },
      notify: true
    },
    "VM.startspice" : {
      type: "custom",
      call: function() {
        $.each(Sunstone.getDataTable(TAB_ID).elements(), function(index, elem) {
          if (!Spice.lockStatus()) {
            Spice.lock();
            Sunstone.runAction("VM.startspice_action", elem);
          } else {
            Notifier.notifyError(Locale.tr("SPICE Connection in progress"))
            return false;
          }
        });
      }
    },
    "VM.startspice_action" : {
      type: "single",
      call: OpenNebulaVM.vnc,
      callback: function(request, response) {
       var dialog = Sunstone.getDialog(SPICE_DIALOG_ID);
       dialog.setElement(response);
       dialog.show();
      },
      error: function(req, resp) {
        Notifier.onError(req, resp);
        Spice.unlock();
      },
      notify: true
    },
    "VM.saveas_template" : {
      type: "single",
      call: function() {
       var dialog = Sunstone.getDialog(SAVE_AS_TEMPLATE_DIALOG_ID);
       dialog.show();
       },
      error: function(req, resp) {
        Notifier.onError(req, resp);
      },
      notify: false
    },
  };

  return _actions;
});

define('tabs/vms-tab/utils/datatable-common',['require','opennebula/vm','utils/locale','utils/humanize','utils/template-utils'],function(require) {

  var OpenNebulaVM = require('opennebula/vm');
  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var TemplateUtils = require('utils/template-utils');

  var RESOURCE = "VM";
  var XML_ROOT = "VM";

  var _columns = [
    Locale.tr("ID") ,
    Locale.tr("Owner") ,
    Locale.tr("Group"),
    Locale.tr("Name"),
    Locale.tr("Status"),
    Locale.tr("Used CPU"),
    Locale.tr("Used Memory"),
    Locale.tr("Host"),
    Locale.tr("IPs"),
    Locale.tr("Start Time"),
    "",
    Locale.tr("Hidden Template")
  ];

  return {
    'elementArray': _elementArray,
    'emptyElementArray': _emptyElementArray,
    'columns': _columns
  };

  function _elementArray(element_json) {
    var element = element_json[XML_ROOT];

    var state;

    if (element.STATE == OpenNebulaVM.STATES.ACTIVE) {
      state = OpenNebulaVM.shortLcmStateStr(element.LCM_STATE);
    } else {
      state = OpenNebulaVM.stateStr(element.STATE);
    }

    // VNC icon
    var vncIcon;
    if (OpenNebulaVM.isVNCSupported(element)) {
      vncIcon = '<a class="vnc" href="#" vm_id="' + element.ID + '"><i class="fa fa-desktop"/></a>';
    } else if (OpenNebulaVM.isSPICESupported(element)) {
      vncIcon = '<a class="spice" href="#" vm_id="' + element.ID + '"><i class="fa fa-desktop"/></a>';
    } else {
      vncIcon = '';
    }

    var cpuMonitoring = 0;
    var memoryMonitoring = 0;
    if (element.MONITORING) {
      if (element.MONITORING.CPU) {
        cpuMonitoring = element.MONITORING.CPU
      }

      if (element.MONITORING.MEMORY) {
        memoryMonitoring = element.MONITORING.MEMORY
      }
    }
    
    return [
      '<input class="check_item" '+
        'type="checkbox" '+
        'id="' + RESOURCE.toLowerCase() + '_' + element.ID + '" '+
        'name="selected_items" '+
        'value="' + element.ID + '" '+
        'state="'+element.STATE+'" lcm_state="'+element.LCM_STATE+'"/>',
       element.ID,
       element.UNAME,
       element.GNAME,
       element.NAME,
       state,
       cpuMonitoring,
       Humanize.size(memoryMonitoring),
       OpenNebulaVM.hostnameStr(element),
       OpenNebulaVM.ipsStr(element),
       Humanize.prettyTime(element.STIME),
       vncIcon,
       TemplateUtils.templateToString(element)
    ];
  }

  function _emptyElementArray(vmId) {
    return [
      '<input class="check_item" type="checkbox" id="' + RESOURCE.toLowerCase() + '_' +
                             vmId + '" name="selected_items" value="' +
                             vmId + '"/>',
       vmId,
       "",
       "",
       "",
       "",
       "",
       "",
       "",
       "",
       "",
       "",
       ""
    ];
  }
});

define('tabs/vms-tab/utils/state-actions',['require','opennebula/vm'],function(require) {
  var OpenNebulaVM = require('opennebula/vm');

  var STATE_ACTIONS = {};

  STATE_ACTIONS[OpenNebulaVM.STATES.INIT] =
    ["VM.delete", "VM.delete_recreate", "VM.resize"];

  STATE_ACTIONS[OpenNebulaVM.STATES.PENDING] =
    ["VM.delete", "VM.delete_recreate", "VM.hold", "VM.deploy"];

  STATE_ACTIONS[OpenNebulaVM.STATES.HOLD] =
    ["VM.delete", "VM.delete_recreate", "VM.release", "VM.deploy"];

  STATE_ACTIONS[OpenNebulaVM.STATES.ACTIVE] =
    ["VM.delete", "VM.delete_recreate", "VM.recover"];

  STATE_ACTIONS[OpenNebulaVM.STATES.STOPPED] =
    ["VM.delete", "VM.delete_recreate", "VM.resume", "VM.deploy"];

  STATE_ACTIONS[OpenNebulaVM.STATES.SUSPENDED] =
    ["VM.delete", "VM.resume", "VM.disk_saveas", "VM.disk_snapshot_create", "VM.disk_snapshot_revert", "VM.disk_snapshot_delete", "VM.stop", "VM.shutdown_hard"];

  STATE_ACTIONS[OpenNebulaVM.STATES.DONE] =
    [];

  STATE_ACTIONS[OpenNebulaVM.STATES.FAILED] =
    ["VM.delete", "VM.delete_recreate", "VM.resize"];

  STATE_ACTIONS[OpenNebulaVM.STATES.POWEROFF] =
    ["VM.delete", "VM.resume", "VM.resize", "VM.attachdisk", "VM.detachdisk", "VM.attachnic", "VM.detachnic", "VM.disk_saveas", "VM.disk_snapshot_create", "VM.disk_snapshot_revert", "VM.disk_snapshot_delete", "VM.migrate", "VM.undeploy", "VM.undeploy_hard", "VM.shutdown_hard", "VM.saveas_template"];

  STATE_ACTIONS[OpenNebulaVM.STATES.UNDEPLOYED] =
    ["VM.delete", "VM.delete_recreate", "VM.resume", "VM.resize", "VM.deploy"];


  var LCM_STATE_ACTIONS = {};
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.LCM_INIT ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.PROLOG ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.BOOT ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.RUNNING ] =
    ["VM.shutdown", "VM.shutdown_hard", "VM.stop", "VM.suspend", "VM.reboot", "VM.reboot_hard", "VM.resched", "VM.unresched", "VM.poweroff", "VM.poweroff_hard", "VM.undeploy", "VM.undeploy_hard", "VM.migrate", "VM.migrate_live", "VM.attachdisk", "VM.detachdisk", "VM.attachnic", "VM.detachnic", "VM.disk_saveas", "VM.disk_snapshot_create", "VM.disk_snapshot_revert", "VM.disk_snapshot_delete"];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.MIGRATE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.SAVE_STOP ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.SAVE_SUSPEND ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.SAVE_MIGRATE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.PROLOG_RESUME ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.EPILOG_STOP ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.EPILOG ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.SHUTDOWN ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.CANCEL ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.CLEANUP_RESUBMIT ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.UNKNOWN ] =
    ["VM.shutdown", "VM.shutdown_hard", "VM.resched", "VM.unresched", "VM.poweroff", "VM.poweroff_hard", "VM.undeploy", "VM.undeploy_hard", "VM.migrate", "VM.migrate_live", "VM.resume"];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.HOTPLUG ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.SHUTDOWN_POWEROFF ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.BOOT_UNKNOWN ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.BOOT_POWEROFF ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.BOOT_SUSPENDED ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.BOOT_STOPPED ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.CLEANUP_DELETE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.HOTPLUG_SNAPSHOT ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.HOTPLUG_NIC ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.HOTPLUG_SAVEAS ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.HOTPLUG_SAVEAS_POWEROFF ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.HOTPLUG_SAVEAS_SUSPENDED ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.SHUTDOWN_UNDEPLOY ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.EPILOG_UNDEPLOY ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.PROLOG_UNDEPLOY ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.BOOT_UNDEPLOY ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.HOTPLUG_PROLOG_POWEROFF ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.HOTPLUG_EPILOG_POWEROFF ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.BOOT_MIGRATE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.BOOT_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.BOOT_MIGRATE_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.PROLOG_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.EPILOG_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.EPILOG_STOP_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.EPILOG_UNDEPLOY_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_POWEROFF ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_POWEROFF_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_SUSPEND ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_SUSPEND_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.BOOT_UNDEPLOY_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.BOOT_STOPPED_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.PROLOG_RESUME_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.PROLOG_UNDEPLOY_FAILURE ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_POWEROFF ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_REVERT_POWEROFF ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_DELETE_POWEROFF ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_SUSPENDED ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_REVERT_SUSPENDED ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_DELETE_SUSPENDED ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_REVERT ] = [];
  LCM_STATE_ACTIONS[ OpenNebulaVM.LCM_STATES.DISK_SNAPSHOT_DELETE ] = [];

  return {
    'disableAllStateActions': disableAllStateActions,
    'resetStateButtons': resetStateButtons,
    'enableStateButton': enableStateButton,
    'enableStateActions': enableStateActions,
    'enabledStateAction': enabledStateAction
  };

  function disableAllStateActions() {
    $(".state-dependent").prop("disabled", true).
        removeClass("vm-action-enabled").
        addClass("vm-action-disabled").
        on("click.stateaction", function(e) { return false; });
  }

  function resetStateButtons() {
    $(".state-dependent").
        addClass("vm-action-enabled").
        removeClass("vm-action-disabled").
        off("click.stateaction");
  }

  function enableStateButton(button_action) {
    $(".state-dependent[href='" + button_action + "']").removeAttr("disabled").
        addClass("vm-action-enabled").
        removeClass("vm-action-disabled").
        off("click.stateaction");
  }

  // state and lcm_state are numeric
  function enableStateActions(state, lcm_state) {
    var state = parseInt(state);
    var lcm_state = parseInt(lcm_state);

    $.each(STATE_ACTIONS[state], function(i, action) {
      enableStateButton(action);
    });

    if (state == OpenNebulaVM.STATES.ACTIVE) {
      $.each(LCM_STATE_ACTIONS[lcm_state], function(i, action) {
        enableStateButton(action);
      });
    }
  }

  // Returns true if the action is enabled for the given state
  // action is "VM.action", state and lcm_state are numeric
  function enabledStateAction(action, state, lcm_state) {
    var state = parseInt(state);
    var lcm_state = parseInt(lcm_state);

    return (STATE_ACTIONS[state].indexOf(action) != -1 ||
             (state == OpenNebulaVM.STATES.ACTIVE &&
                LCM_STATE_ACTIONS[lcm_state].indexOf(action) != -1));
  }
});

define('tabs/vms-tab/datatable',['require','utils/tab-datatable','./utils/datatable-common','opennebula/vm','sunstone-config','utils/locale','./utils/state-actions','sunstone','utils/vnc','utils/spice','utils/notifier','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var VMsTableUtils = require('./utils/datatable-common');
  var OpenNebulaVM = require('opennebula/vm');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');
  var StateActions = require('./utils/state-actions');
  var Sunstone = require('sunstone');
  var Vnc = require('utils/vnc');
  var Spice = require('utils/spice');
  var Notifier = require('utils/notifier');

  /*
    CONSTANTS
   */

  var RESOURCE = "VM";
  var XML_ROOT = "VM";
  var TAB_NAME = require('./tabId');


  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check", 6, 7, 11]},
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    }

    this.columns = VMsTableUtils.columns;

    this.selectOptions = {
      "id_index": 1,
      "name_index": 4,
      "select_resource": Locale.tr("Please select a VM from the list"),
      "you_selected": Locale.tr("You selected the following VM:"),
      "select_resource_multiple": Locale.tr("Please select one or more VMs from the list"),
      "you_selected_multiple": Locale.tr("You selected the following VMs:")
    };

    this.totalVms = 0;
    this.activeVms = 0;
    this.pendingVms = 0;
    this.failedVms = 0;
    this.offVms = 0;

    TabDataTable.call(this);
  };

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;
  Table.prototype.initialize = _initialize;
  Table.prototype.preUpdateView = _preUpdateView;
  Table.prototype.postUpdateView = _postUpdateView;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json[XML_ROOT];
    var state = parseInt(element.STATE);

    this.totalVms++;
    switch (state) {
      case OpenNebulaVM.STATES.INIT:
      case OpenNebulaVM.STATES.PENDING:
      case OpenNebulaVM.STATES.HOLD:
        this.pendingVms++;
        break;
      case OpenNebulaVM.STATES.ACTIVE:
        if (OpenNebulaVM.isFailureState(element.LCM_STATE)) {
          this.failedVms++;
        } else {
          this.activeVms++;
        }
        break;
      case OpenNebulaVM.STATES.STOPPED:
      case OpenNebulaVM.STATES.SUSPENDED:
      case OpenNebulaVM.STATES.POWEROFF:
        this.offVms++;
        break;
      default:
        break;
    }

    return VMsTableUtils.elementArray(element_json);
  }

  function _preUpdateView() {
    StateActions.disableAllStateActions();

    this.totalVms = 0;
    this.activeVms = 0;
    this.pendingVms = 0;
    this.failedVms = 0;
    this.offVms = 0;
  }

  function _postUpdateView() {
    $(".total_vms").text(this.totalVms);
    $(".active_vms").text(this.activeVms);
    $(".pending_vms").text(this.pendingVms);
    $(".failed_vms").text(this.failedVms);
    $(".off_vms").text(this.offVms);
  }

  function _initialize(opts) {
    var that = this;

    TabDataTable.prototype.initialize.call(this, opts);

    $('#' + this.dataTableId).on("click", '.vnc', function() {
      var vmId = $(this).attr('vm_id');

      if (!Vnc.lockStatus()) {
        Vnc.lock();
        Sunstone.runAction("VM.startvnc_action", vmId);
      } else {
        Notifier.notifyError(Locale.tr("VNC Connection in progress"))
      }

      return false;
    });

    $('#' + this.dataTableId).on("click", '.spice', function() {
      var vmId = $(this).attr('vm_id');

      if (!Spice.lockStatus()) {
        Spice.lock();
        Sunstone.runAction("VM.startspice_action", vmId);
      } else {
        Notifier.notifyError(Locale.tr("SPICE Connection in progress"))
      }

      return false;
    });

    $('#' + this.dataTableId).on("change", 'tbody input.check_item', function() {
      if ($(this).is(":checked")){
        StateActions.enableStateActions($(this).attr("state"), $(this).attr("lcm_state"));
      } else {
        // First disable all actions
        StateActions.disableAllStateActions();

        // Enable actions available to any of the selected VMs
        var nodes = $('tr', that.dataTable); //visible nodes only
        $.each($('input.check_item:checked', nodes), function(){
          StateActions.enableStateActions($(this).attr("state"), $(this).attr("lcm_state"));
        });

      }

      return true;
    });

  }
});


/* START_TEMPLATE */
define('hbs!tabs/vms-tab/dialogs/deploy/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/advancedSection'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "        <div class=\"row\">\n          <div class=\"large-6 columns\">\n            <input type=\"checkbox\" name=\"enforce\" id=\"enforce\"/>\n            <label for=\"enforce\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Enforce", {"name":"tr","hash":{},"data":data})))
    + "\n              <span class=\"tip\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "If it is set to true, the host capacity will be checked. This will only affect oneadmin requests, regular users resize requests will always be enforced", {"name":"tr","hash":{},"data":data})))
    + "\n              </span>\n            </label>\n          </div>\n        </div>\n        <br>\n        <fieldset>\n          <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select a datastore", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n          ";
  stack1 = ((helper = (helper = helpers.datastoresTableHTML || (depth0 != null ? depth0.datastoresTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"datastoresTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n        </fieldset>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <h3 id=\"deploy_vm_header\" class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Deploy Virtual Machine", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n    </div>\n  </div>\n  <div class=\"reveal-body\">\n    <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\" action=\"\">\n      <div class=\"row\">\n        <fieldset>\n          <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select a Host", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n          ";
  stack1 = ((helper = (helper = helpers.hostsTableHTML || (depth0 != null ? depth0.hostsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"hostsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </fieldset>\n      </div>\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced Options", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "      <div class=\"form_buttons reveal-footer\">\n        <div class=\"form_buttons\">\n          <button class=\"button radius right success\" id=\"deploy_vm_proceed\" value=\"VM.deploy\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Deploy", {"name":"tr","hash":{},"data":data})))
    + "</button>\n        </div>\n      </div>\n      <a class=\"close-reveal-modal\">&#215;</a>\n    </form>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/dialogs/deploy/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vms-tab/dialogs/deploy',['require','utils/dialogs/dialog','hbs!./deploy/html','sunstone','tabs/datastores-tab/datatable','tabs/hosts-tab/datatable','utils/notifier','utils/tips','./deploy/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./deploy/html');
  var Sunstone = require('sunstone');
  var DatastoresTable = require('tabs/datastores-tab/datatable');
  var HostsTable = require('tabs/hosts-tab/datatable');
  var Notifier = require('utils/notifier');
  var Tips = require('utils/tips');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./deploy/dialogId');
  var TAB_ID = require('../tabId')

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    this.hostsTable = new HostsTable('deploy_vm', {'select': true});
    this.datastoresTable = new DatastoresTable('deploy_vm_ds', {
      'select': true,
      'selectOptions': {
        'filter_fn': function(ds) { return ds.TYPE == 1; } // Show system DS only
      }
    });

    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId,
      'hostsTableHTML': this.hostsTable.dataTableHTML,
      'datastoresTableHTML': this.datastoresTable.dataTableHTML
    });
  }

  function _setup(context) {
    var that = this;

    that.hostsTable.initialize();
    that.datastoresTable.initialize();

    Tips.setup(context);

    $('#' + DIALOG_ID + 'Form', context).submit(function() {
      var extra_info = {};

      if ($("#selected_resource_id_deploy_vm", context).val()) {
          extra_info['host_id'] = $("#selected_resource_id_deploy_vm", context).val();
      } else {
          Notifier.notifyError(Locale.tr("You have not selected a host"));
          return false;
      }

      extra_info['ds_id'] = $("#selected_resource_id_deploy_vm_ds", context).val() || -1
      extra_info['enforce'] = $("#enforce", this).is(":checked") ? true : false

      $.each(Sunstone.getDataTable(TAB_ID).elements(), function(index, elem) {
          Sunstone.runAction("VM.deploy_action", elem, extra_info);
      });

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      return false;
    });

    return false;
  }

  function _onShow(dialog) {
    this.datastoresTable.resetResourceTableSelect();
    this.hostsTable.resetResourceTableSelect();

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vms-tab/dialogs/migrate/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/advancedSection'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "        <div class=\"row\">\n          <div class=\"large-6 columns\">\n            <input type=\"checkbox\" name=\"enforce\" id=\"enforce\"/>\n            <label for=\"vm_id\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Enforce", {"name":"tr","hash":{},"data":data})))
    + "\n              <span class=\"tip\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "If it is set to true, the host capacity will be checked. This will only affect oneadmin requests, regular users resize requests will always be enforced", {"name":"tr","hash":{},"data":data})))
    + "\n              </span>\n            </label>\n          </div>\n        </div>\n        <br>\n        <fieldset class=\"migrate_vm_ds_selection\">\n          <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select a datastore", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n          ";
  stack1 = ((helper = (helper = helpers.datastoresTableHTML || (depth0 != null ? depth0.datastoresTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"datastoresTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n        </fieldset>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <h3 id=\"migrate_vm_header\" class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Migrate Virtual Machine", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n    </div>\n  </div>\n  <div class=\"reveal-body\">\n    <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\" action=\"\">\n      <div id=\"current_hosts_of_vms\" class=\"row\"></div>\n      <br>\n      <br>\n      <div class=\"row\">\n        <fieldset>\n          <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select a Host", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n          ";
  stack1 = ((helper = (helper = helpers.hostsTableHTML || (depth0 != null ? depth0.hostsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"hostsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </fieldset>\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced Options", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "        <div class=\"form_buttons reveal-footer\">\n          <div class=\"form_buttons\">\n            <button class=\"button radius right success\" id=\"migrate_vm_proceed\" value=\"VM.migrate\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Migrate", {"name":"tr","hash":{},"data":data})))
    + "</button>\n          </div>\n        </div>\n        <a class=\"close-reveal-modal\">&#215;</a>\n      </form>\n    </div>\n  </div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/dialogs/migrate/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vms-tab/dialogs/migrate',['require','utils/dialogs/dialog','hbs!./migrate/html','sunstone','tabs/datastores-tab/datatable','tabs/hosts-tab/datatable','utils/notifier','utils/tips','utils/locale','opennebula/vm','./migrate/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./migrate/html');
  var Sunstone = require('sunstone');
  var DatastoresTable = require('tabs/datastores-tab/datatable');
  var HostsTable = require('tabs/hosts-tab/datatable');
  var Notifier = require('utils/notifier');
  var Tips = require('utils/tips');
  var Locale = require('utils/locale');
  var OpenNebulaVM = require('opennebula/vm');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./migrate/dialogId');
  var TAB_ID = require('../tabId')

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    this.hostsTable = new HostsTable('migrate_vm', {'select': true});
    this.datastoresTable = new DatastoresTable('migrate_vm_ds', {
      'select': true,
      'selectOptions': {
        'filter_fn': function(ds) { return ds.TYPE == 1; } // Show system DS only
      }
    });

    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setLive = _setLive;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId,
      'hostsTableHTML': this.hostsTable.dataTableHTML,
      'datastoresTableHTML': this.datastoresTable.dataTableHTML
    });
  }

  function _setup(context) {
    var that = this;

    that.hostsTable.initialize();
    that.datastoresTable.initialize();

    Tips.setup(context);

    $('#' + DIALOG_ID + 'Form', context).submit(function() {
      var extra_info = {};

      if ($("#selected_resource_id_migrate_vm", context).val()) {
          extra_info['host_id'] = $("#selected_resource_id_migrate_vm", context).val();
      } else {
          notifyError(Locale.tr("You have not selected a host"));
          return false;
      }

      extra_info['ds_id'] = $("#selected_resource_id_migrate_vm_ds", context).val() || -1
      extra_info['enforce'] = $("#enforce", context).is(":checked");

      $.each(Sunstone.getDataTable(TAB_ID).elements(), function(index, elem) {
        if (that.live) {
          Sunstone.runAction("VM.migrate_live_action", elem, extra_info);
        } else {
          Sunstone.runAction("VM.migrate_action", elem, extra_info);
        }
      });

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      return false;
    });

    return false;
  }

  function _onShow(context) {
    this.datastoresTable.resetResourceTableSelect();
    this.hostsTable.resetResourceTableSelect();

    if (this.live) {
      $(".migrate_vm_ds_selection", context).hide();
    }

    $.each(Sunstone.getDataTable(TAB_ID).elements(), function() {
      var vm_id = "" + this;

      OpenNebulaVM.show({
        data : {
          id: vm_id
        },
        timeout: true,
        success: function (request, vm_json) {
          var element = vm_json.VM;
          var hostname = OpenNebulaVM.hostnameStr(element);

          $("#current_hosts_of_vms").append(
              '<span class="radius secondary label">' +
              Locale.tr("VM") + ' [' + element.ID + '] ' +
              Locale.tr("is currently running on Host") +
              ' [' + hostname + ']</span><br>'
          );
        }
      });
    });
    return false;
  }

  // @param [Boolean] live Set migrate live or migrate
  function _setLive(live) {
    this.live = live;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vms-tab/dialogs/resize/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large\" role=\"dialog\" data-reveal >\n<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <h3 class=\"subheader\" id=\"\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Resize VM capacity", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n  </div>\n</div>\n<div class=\"reveal-body\">\n  <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\" action=\"\">\n    <div class=\"row centered\">\n      <div class=\"large-6 columns\">\n        <label for=\"vm_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Machine ID", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n        <label style=\"border-style: inset; background-color: lightgrey\" type=\"text\" name=\"vm_id\" id=\"vm_id\" disabled/>\n      </div>\n      <div class=\"large-6 columns\">\n        <input type=\"checkbox\" name=\"enforce\" id=\"enforce\"/>\n        <label class=\"inline\" for=\"vm_id\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Enforce", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "If it is set to true, the host capacity will be checked. This will only affect oneadmin requests, regular users resize requests will always be enforced", {"name":"tr","hash":{},"data":data})))
    + "\n          </span>\n        </label>\n      </div>\n    </div>\n    ";
  stack1 = ((helper = (helper = helpers.capacityInputsHTML || (depth0 != null ? depth0.capacityInputsHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"capacityInputsHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    <div class=\"reveal-footer\">\n      <div class=\"form_buttons\">\n        <button class=\"button radius right success\" id=\"resize_capacity_button\" type=\"submit\" value=\"VM.resize\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Resize", {"name":"tr","hash":{},"data":data})))
    + "</button>\n      </div>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/dialogs/resize/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/general/capacity-inputs/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/tip','templates/helpers/advancedSection'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <div class=\"large-12 columns\">\n      <label class=\"\" for=\"VCPU\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VCPU", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of virtual cpus. This value is optional, the default hypervisor behavior is used, usually one virtual CPU.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      </label>\n    </div>\n    <div class=\"large-10 medium-10 columns\">\n      <div id=\"vcpu_slider\" class=\"range-slider radius\" data-slider data-options=\"start: 100; end: 1600; step: 100;\">\n        <span class=\"range-slider-handle\"></span>\n        <span class=\"range-slider-active-segment\"></span>\n        <input type=\"hidden\">\n      </div>\n    </div>\n    <div class=\"large-2 medium-2 columns\">\n      <input type=\"text\" wizard_field=\"VCPU\" id=\"VCPU\" name=\"vcpu\"/>\n    </div>\n  </div>\n</div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <div class=\"\">\n      <input type=\"hidden\" wizard_field=\"MEMORY\" id=\"MEMORY\" name=\"memory\" />\n    </div>\n    <div class=\"large-12 columns\">\n      <label class=\"\" for=\"MEMORY\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Memory", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Amount of RAM required for the VM, in Megabytes.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n    </div>\n    <div class=\"medium-8 columns memory_slider_container\">\n      <div id=\"memory_slider\" class=\"range-slider radius\" data-slider data-options=\"start: 0; end: 409600; step: 12800;\">\n        <span class=\"range-slider-handle\"></span>\n        <span class=\"range-slider-active-segment\"></span>\n        <input type=\"hidden\">\n      </div>\n    </div>\n    <div class=\"medium-2 small-6 columns\">\n      <input type=\"text\" id=\"MEMORY_TMP\" name=\"memory_tmp\" size=\"4\" />\n    </div>\n    <div class=\"medium-2 small-6 columns\">\n      <select id=\"memory_unit\" name=\"MEMORY_UNIT\">\n        <option value=\"MB\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "MB", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"GB\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "GB", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <div class=\"large-12 columns\">\n      <label class=\"\" for=\"CPU\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Percentage of CPU divided by 100 required for the Virtual Machine. Half a processor is written 0.5.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n    </div>\n    <div class=\"large-10 medium-10 columns\">\n      <div id=\"cpu_slider\" class=\"range-slider radius\" data-slider data-options=\"start: 0; end: 1600; step: 50;\">\n        <span class=\"range-slider-handle\"></span>\n        <span class=\"range-slider-active-segment\"></span>\n        <input type=\"hidden\">\n      </div>\n    </div>\n    <div class=\"large-2 medium-2 columns\">\n      <input type=\"text\" wizard_field=\"CPU\" id=\"CPU\" name=\"cpu\"/>\n    </div>\n  </div>\n</div>\n<br>\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced Capacity", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/general/capacity-inputs/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/create/wizard-tabs/general/capacity-inputs',['require','foundation.slider','utils/locale','utils/tips','utils/wizard-fields','hbs!./capacity-inputs/html'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.slider');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var WizardFields = require('utils/wizard-fields');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./capacity-inputs/html');

  /*
    CONSTRUCTOR
   */

  return {
    'html': _html,
    'setup': _setup,
    'fill': _fill,
    'retrieve': _retrieve,
    'retrieveResize': _retrieveResize
  };

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML();
  }

  function _setup(context) {
    // Define the cpu slider
    var cpu_input = $("#CPU", context);
    var cpu_slider = $("#cpu_slider", context)

    context.foundation('slider', 'reflow');

    //cpu_slider.attr('data-options', 'start: 0; end: 1600; step: 50;');

    cpu_slider.on('change.fndtn.slider', function(){
      if ($(this).attr('data-slider') >= 0) {
        cpu_input.val($(this).attr('data-slider') / 100);
      }
    });

    cpu_input.on('change', function() {
      if (this.value && this.value >= 0) {
        cpu_slider.foundation('slider', 'set_value', this.value * 100);
      } else {
        cpu_slider.foundation('slider', 'set_value', -1);
      }
    });

    cpu_slider.foundation('slider', 'set_value', 100);

    // Define the memory slider

    var final_memory_input = $("#MEMORY", context);
    var memory_input = $("#MEMORY_TMP", context);
    var memory_unit  = $("#memory_unit", context);

    var current_memory_unit = memory_unit.val();

    var update_final_memory_input = function() {
      if (current_memory_unit == 'MB') {
        final_memory_input.val(Math.floor(memory_input.val()));
      } else {
        final_memory_input.val(Math.floor(memory_input.val() * 1024));
      }
    }

    memory_input.on('change', function() {
      if (this.value && this.value >= 0) {
        $("#memory_slider", context).foundation('slider', 'set_value', this.value * 100);
        update_final_memory_input();
      } else {
        $("#memory_slider", context).foundation('slider', 'set_value', -1);
        final_memory_input.val("");
      }
    });

    final_memory_input.on('change', function() {
      if (this.value && this.value >= 0) {
        $("#memory_slider", context).foundation('slider', 'set_value', this.value * 100);
        memory_input.val(Math.floor(this.value));
      } else {
        $("#memory_slider", context).foundation('slider', 'set_value', -1);
        memory_input.val("");
      }
    });

    $("#memory_slider", context).on('change.fndtn.slider', function() {
      if ($(this).attr('data-slider') >= 0) {
        memory_input.val($(this).attr('data-slider') / 100);
        update_final_memory_input();
      }
    });

    memory_unit.on('change', function() {
      var memory_unit_val = $('#memory_unit :selected', context).val();

      if (current_memory_unit != memory_unit_val) {
        current_memory_unit = memory_unit_val
        var new_val;
        if (memory_unit_val == 'GB') {
          $("#memory_slider", context).detach();
          $(".memory_slider_container", context).html(
            '<div id="memory_slider" class="range-slider radius" data-slider data-options="start: 0; end: 1600; step: 50;">'+
              '<span class="range-slider-handle"></span>'+
              '<span class="range-slider-active-segment"></span>'+
              '<input type="hidden">'+
            '</div>');

          new_val = memory_input.val() / 1024;
        } else if (memory_unit_val == 'MB') {
          $("#memory_slider", context).detach();
          $(".memory_slider_container", context).html(
            '<div id="memory_slider" class="range-slider radius" data-slider data-options="start: 0; end: 409600; step: 12800;">'+
              '<span class="range-slider-handle"></span>'+
              '<span class="range-slider-active-segment"></span>'+
              '<input type="hidden">'+
            '</div>');

          new_val = Math.floor(memory_input.val() * 1024);
        }

        context.foundation('slider', 'reflow');
        memory_input.val(new_val);
        $("#memory_slider", context).foundation('slider', 'set_value', new_val * 100);
        $("#memory_slider", context).on('change.fndtn.slider', function() {
          if ($(this).attr('data-slider') >= 0) {
            memory_input.val($(this).attr('data-slider') / 100);
            update_final_memory_input();
          }
        });

        update_final_memory_input();
      }
    });

    // init::start is ignored for some reason
    $("#memory_slider", context).foundation('slider', 'set_value', 51200);

    // Define the vcpu slider

    var vcpu_input = $("#VCPU", context);
    var vcpu_slider = $("#vcpu_slider", context)

    //vcpu_slider.attr('data-options', 'start: 0; end: 1600; step: 50;');
    vcpu_slider.on('change.fndtn.slider', function(){
      if ($(this).attr('data-slider') > 0) {
        vcpu_input.val($(this).attr('data-slider') / 100);
      }
    });

    vcpu_input.on('change', function() {
      if (this.value && this.value > 0) {
        vcpu_slider.foundation('slider', 'set_value', this.value * 100);
      } else {
        vcpu_slider.foundation('slider', 'set_value', -1);
      }
    });

    vcpu_slider.foundation('slider', 'set_value', 0);
  }

  /**
   * Fills the capacity inputs
   * @param  {Object} context  JQuery selector
   * @param  {Object} template VM or VMTemplate object
   */
  function _fill(context, element) {
    var fields = $('[wizard_field]', context);

    fields.each(function() {
      var field_name = $(this).attr('wizard_field');
      $(this).data("original_value", element.TEMPLATE[field_name]);
    });

    WizardFields.fill(context, element.TEMPLATE);
  }

  /**
   * Retrieves the input values
   * @param  {Object} context  JQuery selector
   * @return {Object}         If the input is not empty, returns:
   *                                  - CPU
   *                                  - MEMORY
   *                                  - VCPU
   */
  function _retrieve(context) {
    return WizardFields.retrieve(context);
  }

  /**
   * Retrieves the input values, but only if the value has changed from the
   * original set in fill()
   * @param  {Object} context  JQuery selector
   * @return {Object}         If the input has changed, returns:
   *                                  - CPU
   *                                  - MEMORY
   *                                  - VCPU
   */
  function _retrieveResize(context) {
    var templateJSON = WizardFields.retrieve(context);

    var fields = $('[wizard_field]', context);

    fields.each(function() {
      var field_name = $(this).attr('wizard_field');
      if (templateJSON[field_name] == $(this).data("original_value")){
        delete templateJSON[field_name];
      }
    });

    return templateJSON;
  }
});

define('tabs/vms-tab/dialogs/resize/dialogId',['require'],function(require) {
  return 'resizeVMDialog';
})
;
define('tabs/vms-tab/dialogs/resize',['require','utils/dialogs/dialog','hbs!./resize/html','sunstone','utils/notifier','utils/tips','tabs/templates-tab/form-panels/create/wizard-tabs/general/capacity-inputs','utils/wizard-fields','./resize/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./resize/html');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Tips = require('utils/tips');
  var CapacityInputs = require('tabs/templates-tab/form-panels/create/wizard-tabs/general/capacity-inputs');
  var WizardFields = require('utils/wizard-fields');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./resize/dialogId');
  var TAB_ID = require('../tabId')

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setElement = _setElement;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId,
      'capacityInputsHTML': CapacityInputs.html()
    });
  }

  function _setup(context) {
    var that = this;
    CapacityInputs.setup(context);

    Tips.setup(context);

    $('#' + DIALOG_ID + 'Form', context).submit(function() {
      var templateJSON = CapacityInputs.retrieveResize(context);

      var enforce = $("#enforce", this).is(":checked");

      var obj = {
        "vm_template": templateJSON,
        "enforce": enforce,
      }

      Sunstone.runAction('VM.resize', that.element.ID, obj);

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      return false;
    });

    return false;
  }

  function _onShow(context) {
    var that = this;
    $('#vm_id', context).text(that.element.ID);
    CapacityInputs.fill(context, that.element);

    context.foundation('slider', 'reflow');
    
    return false;
  }

  function _setElement(element) {
    this.element = element
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vms-tab/dialogs/attach-disk/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <h3 class=\"subheader\" id=\"\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Attach new disk", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n    </div>\n  </div>\n  <div class=\"reveal-body\">\n    <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\" action=\"\">\n      <div class=\"row\">\n        <div class=\"large-6 columns\">\n          <label for=\"vm_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Machine ID", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input style=\"border-style: inset; background-color: lightgrey\" type=\"text\" name=\"vm_id\" id=\"vm_id\" disabled/>\n        </div>\n      </div>\n      ";
  stack1 = ((helper = (helper = helpers.diskTabHTML || (depth0 != null ? depth0.diskTabHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"diskTabHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      <div class=\"reveal-footer\">\n        <div class=\"form_buttons\">\n          <button class=\"button radius right success\" id=\"attach_disk_button\" type=\"submit\" value=\"VM.attachdisk\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Attach", {"name":"tr","hash":{},"data":data})))
    + "</button>\n        </div>\n      </div>\n      <a class=\"close-reveal-modal\">&#215;</a>\n    </form>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/dialogs/attach-disk/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/images-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','utils/humanize','utils/notifier','opennebula/image','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var Notifier = require('utils/notifier');
  var OpenNebulaImage = require('opennebula/image');

  /*
    CONSTANTS
   */

  var RESOURCE = "Image"
  var XML_ROOT = "IMAGE"
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"]},
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    }

    this.columns = [
      Locale.tr("ID"),
      Locale.tr("Owner"),
      Locale.tr("Group"),
      Locale.tr("Name"),
      Locale.tr("Datastore"),
      Locale.tr("Size"),
      Locale.tr("Type"),
      Locale.tr("Registration time"),
      Locale.tr("Persistent"),
      Locale.tr("Status"),
      Locale.tr("#VMS"),
      Locale.tr("Target"),
    ]

    this.selectOptions = {
      "id_index": 1,
      "name_index": 4,
      "uname_index": 2,
      "select_resource": Locale.tr("Please select an image from the list"),
      "you_selected": Locale.tr("You selected the following image:"),
      "select_resource_multiple": Locale.tr("Please select one or more images from the list"),
      "you_selected_multiple": Locale.tr("You selected the following images:")
    };

    this.totalImages = 0;
    this.sizeImages = 0;

    TabDataTable.call(this);
  };

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;
  Table.prototype.preUpdateView = _preUpdateView;
  Table.prototype.postUpdateView = _postUpdateView;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json.IMAGE;

    if (element.TYPE == OpenNebulaImage.TYPES.KERNEL ||
        element.TYPE == OpenNebulaImage.TYPES.RAMDISK ||
        element.TYPE == OpenNebulaImage.TYPES.CONTEXT) {
      return false;
    }

    this.sizeImages = this.sizeImages + parseInt(element.SIZE);
    this.totalImages++;

    return [
      '<input class="check_item" type="checkbox" id="image_' + element.ID + '" name="selected_items" value="' + element.ID + '"/>',
      element.ID,
      element.UNAME,
      element.GNAME,
      element.NAME,
      element.DATASTORE,
      element.SIZE,
      OpenNebulaImage.typeStr(element.TYPE),
      Humanize.prettyTime(element.REGTIME),
      parseInt(element.PERSISTENT) ? "yes" : "no",
      OpenNebulaImage.stateStr(element.STATE),
      element.RUNNING_VMS,
      element.TEMPLATE.TARGET ? element.TEMPLATE.TARGET : '--'
    ];
  }

  function _preUpdateView() {
    this.totalImages = 0;
    this.sizeImages = 0;
  }

  function _postUpdateView() {
    var size = Humanize.sizeFromMB(this.sizeImages);

    $(".total_images").text(this.totalImages);
    $(".size_images").text(size);
  }
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/storage/disk-tab/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/advancedSection','templates/helpers/tip'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "  <fieldset>\n    <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Image", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <label for=\"IMAGE_ID\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Image ID to be used in the Virtual Image disk.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"IMAGE_ID\" id=\"IMAGE_ID\" name=\"IMAGE_ID\"/>\n      </div>\n      <div class=\"medium-6 columns\">\n        <label for=\"IMAGE\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name of the image to be used in the Virtual Image disk.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"IMAGE\" id=\"IMAGE\" name=\"IMAGE\"/>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <label for=\"IMAGE_UID\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User ID", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Identifier of the user owner of the image to reduce ambiguity.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"IMAGE_UID\" id=\"IMAGE_UID\" name=\"IMAGE_UID\"/>\n      </div>\n      <div class=\"medium-6 columns\">\n        <label for=\"IMAGE_UNAME\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User Name", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name of the user owner of the image to reduce ambiguity.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"IMAGE_UNAME\" id=\"IMAGE_UNAME\" name=\"IMAGE_UNAME\"/>\n      </div>\n    </div>\n  </fieldset>\n  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <label for=\"TARGET\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Target", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Device to map image disk. If set, it will overwrite the default device mapping. Xen: Optional. KVM: Optional. VMWare: Optional", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input type=\"text\" wizard_field=\"TARGET\" id=\"TARGET\" name=\"target\"/>\n    </div>\n    <div class=\"medium-6 columns hypervisor only_xen only_kvm\">\n      <label for=\"DRIVER\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Driver", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Specific image mapping driver. Xen: Optional (tap:aio:, file:). KVM: Optional (raw, qcow2). VMWare: Not supported", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input type=\"text\" wizard_field=\"DRIVER\" id=\"DRIVER\" name=\"driver\" />\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <label for=\"DEV_PREFIX\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Device Prefix", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Prefix for the emulated device this image will be mounted at. For instance, hd, sd, or vd for KVM virtio. If omitted, the dev_prefix attribute of the Image will be used. Xen: Optional. KVM: Optional. VMWare: Optional", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input type=\"text\" wizard_field=\"DEV_PREFIX\" id=\"DEV_PREFIX\" name=\"DEV_PREFIX\"/>\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"READONLY\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Read Only", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Set how the image is exposed by the hypervisor. Xen: Optional. KVM: Optional. VMWare: Optional", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <select wizard_field=\"READONLY\" id=\"READONLY\" name=\"READONLY\">\n        <option value=\"\"></option>\n        <option value=\"yes\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "yes", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"no\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "no", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n  </div>\n  <div class=\"row vm_param\">\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"CACHE\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cache", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Selects the cache mechanism for the disk.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <select wizard_field=\"CACHE\" id=\"CACHE\" name=\"CACHE\">\n        <option value=\"\"></option>\n        <option value=\"default\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "default", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"none\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "none", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"writethrough\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "writethrough", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"writeback\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "writeback", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"directsync\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "directsync", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"unsafe\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "unsafe", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"IO\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Set IO policy.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <select wizard_field=\"IO\" id=\"IO\" name=\"IO\">\n        <option value=\"\"></option>\n        <option value=\"threads\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "threads", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"native\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "native", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"DISCARD\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Discard", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Set DISCARD feature.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <select wizard_field=\"DISCARD\" id=\"DISCARD\" name=\"DISCARD\">\n        <option value=\"\"></option>\n        <option value=\"ignore\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ignore", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"unmap\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "unmap", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"SIZE\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Size (MB)", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "The size of the disk will be modified to match this size when the template is instantiated", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"SIZE\" type=\"text\" id=\"SIZE\" name=\"SIZE\" />\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"TOTAL_BYTES_SEC\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Total Bytes Sec", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO throttling attributes for the disk. They are specified in bytes or IOPS (IO Operations) and can be specified for the total (read+write) or specific for read or write. Total and read or write can not be used at the same time. By default these parameters are only allowed to be used by oneadmin.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"TOTAL_BYTES_SEC\" type=\"text\" id=\"TOTAL_BYTES_SEC\" name=\"TOTAL_BYTES_SEC\" />\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"READ_BYTES_SEC\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Read Bytes Sec", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO throttling attributes for the disk. They are specified in bytes or IOPS (IO Operations) and can be specified for the total (read+write) or specific for read or write. Total and read or write can not be used at the same time. By default these parameters are only allowed to be used by oneadmin.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"READ_BYTES_SEC\" type=\"text\" id=\"READ_BYTES_SEC\" name=\"READ_BYTES_SEC\" />\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"WRITE_BYTES_SEC\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Write Bytes Sec", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO throttling attributes for the disk. They are specified in bytes or IOPS (IO Operations) and can be specified for the total (read+write) or specific for read or write. Total and read or write can not be used at the same time. By default these parameters are only allowed to be used by oneadmin.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"WRITE_BYTES_SEC\" type=\"text\" id=\"WRITE_BYTES_SEC\" name=\"WRITE_BYTES_SEC\" />\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"TOTAL_IOPS_SEC\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Total IOPS Sec", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO throttling attributes for the disk. They are specified in bytes or IOPS (IO Operations) and can be specified for the total (read+write) or specific for read or write. Total and read or write can not be used at the same time. By default these parameters are only allowed to be used by oneadmin.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"TOTAL_IOPS_SEC\" type=\"text\" id=\"TOTAL_IOPS_SEC\" name=\"TOTAL_IOPS_SEC\" />\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"READ_IOPS_SEC\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Read IOPS Sec", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO throttling attributes for the disk. They are specified in bytes or IOPS (IO Operations) and can be specified for the total (read+write) or specific for read or write. Total and read or write can not be used at the same time. By default these parameters are only allowed to be used by oneadmin.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"READ_IOPS_SEC\" type=\"text\" id=\"READ_IOPS_SEC\" name=\"READ_IOPS_SEC\" />\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"WRITE_IOPS_SEC\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Write IOPS Sec", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO throttling attributes for the disk. They are specified in bytes or IOPS (IO Operations) and can be specified for the total (read+write) or specific for read or write. Total and read or write can not be used at the same time. By default these parameters are only allowed to be used by oneadmin.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      </label>\n      <input wizard_field=\"WRITE_IOPS_SEC\" type=\"text\" id=\"WRITE_IOPS_SEC\" name=\"WRITE_IOPS_SEC\" />\n    </div>\n  </div>\n";
},"3":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <label for=\"TARGET\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Target", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Device to map image disk. If set, it will overwrite the default device mapping. Xen: Optional. KVM: Optional. VMWare: Optional", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"TARGET\" type=\"text\"  id=\"TARGET\" name=\"target\"/>\n    </div>\n    <div class=\"medium-6 columns hypervisor only_xen only_kvm\">\n      <label for=\"DRIVER\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Driver", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Specific image mapping driver. Xen: Optional (tap:aio:, file:). KVM: Optional (raw, qcow2). VMWare: Not supported", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"DRIVER\" type=\"text\" id=\"DRIVER\" name=\"driver\" />\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <label for=\"DEV_PREFIX\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Device Prefix", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Prefix for the emulated device this image will be mounted at. For instance, hd, sd, or vd for KVM virtio. If omitted, the dev_prefix attribute of the Image will be used. Xen: Optional. KVM: Optional. VMWare: Optional", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"DEV_PREFIX\" type=\"text\" id=\"DEV_PREFIX\" name=\"DEV_PREFIX\"/>\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"READONLY\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Read Only", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Set how the image is exposed by the hypervisor. Xen: Optional. KVM: Optional. VMWare: Optional", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <select wizard_field=\"READONLY\" id=\"READONLY\" name=\"READONLY\">\n        <option value=\"\"></option>\n        <option value=\"yes\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "yes", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"no\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "no", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n  </div>\n  <div class=\"row  vm_param\">\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"CACHE\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cache", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Selects the cache mechanism for the disk.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <select wizard_field=\"CACHE\" id=\"CACHE\" name=\"CACHE\">\n        <option value=\"\"></option>\n        <option value=\"default\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "default", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"none\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "none", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"writethrough\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "writethrough", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"writeback\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "writeback", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"directsync\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "directsync", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"unsafe\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "unsafe", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"IO\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Set IO policy.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <select wizard_field=\"IO\" id=\"IO\" name=\"IO\">\n        <option value=\"\"></option>\n        <option value=\"threads\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "threads", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"native\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "native", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"TOTAL_BYTES_SEC\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Total Bytes Sec", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO throttling attributes for the disk. They are specified in bytes or IOPS (IO Operations) and can be specified for the total (read+write) or specific for read or write. Total and read or write can not be used at the same time. By default these parameters are only allowed to be used by oneadmin.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"TOTAL_BYTES_SEC\" type=\"text\" id=\"TOTAL_BYTES_SEC\" name=\"TOTAL_BYTES_SEC\" />\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"READ_BYTES_SEC\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Read Bytes Sec", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO throttling attributes for the disk. They are specified in bytes or IOPS (IO Operations) and can be specified for the total (read+write) or specific for read or write. Total and read or write can not be used at the same time. By default these parameters are only allowed to be used by oneadmin.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"READ_BYTES_SEC\" type=\"text\" id=\"READ_BYTES_SEC\" name=\"READ_BYTES_SEC\" />\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"WRITE_BYTES_SEC\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Write Bytes Sec", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO throttling attributes for the disk. They are specified in bytes or IOPS (IO Operations) and can be specified for the total (read+write) or specific for read or write. Total and read or write can not be used at the same time. By default these parameters are only allowed to be used by oneadmin.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"WRITE_BYTES_SEC\" type=\"text\" id=\"WRITE_BYTES_SEC\" name=\"WRITE_BYTES_SEC\" />\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"TOTAL_IOPS_SEC\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Total IOPS Sec", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO throttling attributes for the disk. They are specified in bytes or IOPS (IO Operations) and can be specified for the total (read+write) or specific for read or write. Total and read or write can not be used at the same time. By default these parameters are only allowed to be used by oneadmin.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"TOTAL_IOPS_SEC\" type=\"text\" id=\"TOTAL_IOPS_SEC\" name=\"TOTAL_IOPS_SEC\" />\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"READ_IOPS_SEC\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Read IOPS Sec", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO throttling attributes for the disk. They are specified in bytes or IOPS (IO Operations) and can be specified for the total (read+write) or specific for read or write. Total and read or write can not be used at the same time. By default these parameters are only allowed to be used by oneadmin.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"READ_IOPS_SEC\" type=\"text\" id=\"READ_IOPS_SEC\" name=\"READ_IOPS_SEC\" />\n    </div>\n    <div class=\"medium-6 columns hypervisor only_kvm\">\n      <label for=\"WRITE_IOPS_SEC\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Write IOPS Sec", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IO throttling attributes for the disk. They are specified in bytes or IOPS (IO Operations) and can be specified for the total (read+write) or specific for read or write. Total and read or write can not be used at the same time. By default these parameters are only allowed to be used by oneadmin.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      </label>\n      <input wizard_field=\"WRITE_IOPS_SEC\" type=\"text\" id=\"WRITE_IOPS_SEC\" name=\"WRITE_IOPS_SEC\" />\n    </div>\n  </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <input id=\""
    + escapeExpression(((helper = (helper = helpers.diskTabId || (depth0 != null ? depth0.diskTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"diskTabId","hash":{},"data":data}) : helper)))
    + "radioImage\" type=\"radio\" name=\""
    + escapeExpression(((helper = (helper = helpers.diskTabId || (depth0 != null ? depth0.diskTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"diskTabId","hash":{},"data":data}) : helper)))
    + "\" value=\"image\" checked>\n    <label for=\""
    + escapeExpression(((helper = (helper = helpers.diskTabId || (depth0 != null ? depth0.diskTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"diskTabId","hash":{},"data":data}) : helper)))
    + "radioImage\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Image", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input id=\""
    + escapeExpression(((helper = (helper = helpers.diskTabId || (depth0 != null ? depth0.diskTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"diskTabId","hash":{},"data":data}) : helper)))
    + "radioVolatile\" type=\"radio\" name=\""
    + escapeExpression(((helper = (helper = helpers.diskTabId || (depth0 != null ? depth0.diskTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"diskTabId","hash":{},"data":data}) : helper)))
    + "\" value=\"volatile\">\n    <label for=\""
    + escapeExpression(((helper = (helper = helpers.diskTabId || (depth0 != null ? depth0.diskTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"diskTabId","hash":{},"data":data}) : helper)))
    + "radioVolatile\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Volatile Disk", {"name":"tr","hash":{},"data":data})))
    + "</label>\n  </div>\n</div>\n<div id=\"disk_type\" class=\"image\">\n  ";
  stack1 = ((helper = (helper = helpers.imageTableSelectHTML || (depth0 != null ? depth0.imageTableSelectHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"imageTableSelectHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n  <br>\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced Options", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n<div id=\"disk_type\" class=\"volatile\" style=\"display: none;\">\n  <div class=\"row\">\n    <div class=\"medium-6 columns left\">\n      <label for=\"SIZE\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Size", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Size of the new disk in MB", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"SIZE\" type=\"text\" id=\"SIZE\" name=\"SIZE\"/>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <label for=\"TYPE\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk type", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <select wizard_field=\"TYPE\" id=\"TYPE\" name=\"type\">\n        <option value=\"fs\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "FS", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"swap\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Swap", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"FORMAT\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Format", {"name":"tr","hash":{},"data":data})))
    + "\n        ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filesystem type for the fs images", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </label>\n      <input wizard_field=\"FORMAT\" type=\"text\" id=\"FORMAT\" name=\"format\" />\n    </div>\n  </div>\n  <br>\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced Options", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/storage/disk-tab/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/create/wizard-tabs/storage/disk-tab',['require','sunstone-config','utils/locale','utils/tips','tabs/images-tab/datatable','utils/wizard-fields','hbs!./disk-tab/html'],function(require) {
  /*
    DEPENDENCIES
   */

  var Config = require('sunstone-config');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var ImageTable = require('tabs/images-tab/datatable')
  var WizardFields = require('utils/wizard-fields');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./disk-tab/html');

  /*
    CONSTANTS
   */

  /*
    CONSTRUCTOR
   */

  function DiskTab(diskTabId) {
    this.diskTabId = 'diskTab' + diskTabId;

    this.imageTable = new ImageTable(this.diskTabId + 'Table', {'select': true});
  }

  DiskTab.prototype.constructor = DiskTab;
  DiskTab.prototype.html = _html;
  DiskTab.prototype.setup = _setup;
  DiskTab.prototype.onShow = _onShow;
  DiskTab.prototype.retrieve = _retrieve;
  DiskTab.prototype.fill = _fill;

  return DiskTab;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'diskTabId': this.diskTabId,
      'imageTableSelectHTML': this.imageTable.dataTableHTML
    });
  }

  function _onShow(context, panelForm) {
    this.imageTable.refreshResourceTableSelect();
  }

  function _setup(context) {
    var that = this;
    that.imageTable.initialize({
      'selectOptions': {
        'select_callback': function(aData, options) {
          // If the image is selected by Id, avoid overwriting it with name+uname
          if ($('#IMAGE_ID', context).val() != aData[options.id_index]) {
            $('#IMAGE_ID', context).val("");
            $('#IMAGE', context).val(aData[options.name_index]);
            $('#IMAGE_UNAME', context).val(aData[options.uname_index]);
            $('#IMAGE_UID', context).val("");
          }
        }
      }
    });
    that.imageTable.refreshResourceTableSelect();

    // Select Image or Volatile disk. The div is hidden depending on the selection, and the
    // vm_param class is included to be computed when the template is generated.
    var imageContext = $("div.image",  context);
    var volatileContext = $("div.volatile",  context);
    $("input[name='" + that.diskTabId + "']", context).change(function() {
      if ($("input[name='" + that.diskTabId + "']:checked", context).val() == "image") {
        imageContext.toggle();
        volatileContext.hide();
        $("[wizard_field]", imageContext).prop('wizard_field_disabled', false);
        $("[wizard_field]", volatileContext).prop('wizard_field_disabled', true);
      } else {
        imageContext.hide();
        volatileContext.toggle();
        $("[wizard_field]", volatileContext).prop('wizard_field_disabled', false);
        $("[wizard_field]", imageContext).prop('wizard_field_disabled', true);
      }
    });

    $("[wizard_field]", imageContext).prop('wizard_field_disabled', false);
    $("[wizard_field]", volatileContext).prop('wizard_field_disabled', true);
  }

  function _retrieve(context) {
    return WizardFields.retrieve(context);
  }

  function _fill(context, templateJSON) {
    if (templateJSON.IMAGE_ID || templateJSON.IMAGE) {
      $('input#' + this.diskTabId + 'radioImage', context).click();

      if (templateJSON.IMAGE_ID != undefined) {
        var selectedResources = {
          ids : templateJSON.IMAGE_ID
        }

        this.imageTable.selectResourceTableSelect(selectedResources);
      } else if (templateJSON.IMAGE != undefined && templateJSON.IMAGE_UNAME != undefined) {
        var selectedResources = {
          names : {
            name: templateJSON.IMAGE,
            uname: templateJSON.IMAGE_UNAME
          }
        }

        this.imageTable.selectResourceTableSelect(selectedResources);
      }

      WizardFields.fill($(".image", context), templateJSON);
    } else {
      $('input#' + this.diskTabId + 'radioVolatile', context).click();
      WizardFields.fill($(".volatile", context), templateJSON);
    }

  }
});

define('tabs/vms-tab/dialogs/attach-disk/dialogId',['require'],function(require) {
  return 'attachDiskVMDialog';
})
;
define('tabs/vms-tab/dialogs/attach-disk',['require','utils/dialogs/dialog','hbs!./attach-disk/html','sunstone','utils/notifier','utils/tips','tabs/templates-tab/form-panels/create/wizard-tabs/storage/disk-tab','utils/wizard-fields','./attach-disk/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./attach-disk/html');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Tips = require('utils/tips');
  var DiskTab = require('tabs/templates-tab/form-panels/create/wizard-tabs/storage/disk-tab');
  var WizardFields = require('utils/wizard-fields');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./attach-disk/dialogId');
  var TAB_ID = require('../tabId')

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    this.diskTab = new DiskTab(DIALOG_ID + 'DiskTab');

    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setElement = _setElement;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId,
      'diskTabHTML': this.diskTab.html()
    });
  }

  function _setup(context) {
    var that = this;
    that.diskTab.setup(context);

    Tips.setup(context);

    $('#' + DIALOG_ID + 'Form', context).submit(function() {
      var templateJSON = that.diskTab.retrieve(context);
      var obj = {
        "DISK": templateJSON
      }

      Sunstone.runAction('VM.attachdisk', that.element.ID, obj);

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      return false;
    });

    return false;
  }

  function _onShow(context) {
    this.diskTab.onShow(context);
    $("#vm_id", context).val(this.element.ID);
    return false;
  }

  function _setElement(element) {
    this.element = element
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vms-tab/dialogs/disk-snapshot/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal small\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <h3 class=\"subheader\" id=\"\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk Snapshot", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n    </div>\n  </div>\n  <div class=\"reveal-body\">\n    <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\" action=\"\">\n      <div class=\"row\">\n        <div class=\"large-6 columns\">\n          <label for=\"vm_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Machine ID", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input style=\"border-style: inset; background-color: lightgrey\" type=\"text\" name=\"vm_id\" id=\"vm_id\" disabled/>\n        </div>\n        <div class=\"large-6 columns\">\n          <label for=\"disk_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk ID", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input style=\"border-style: inset; background-color: lightgrey\" type=\"text\" name=\"disk_id\" id=\"disk_id\" disabled/>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label for=\"snapshot_name\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input type=\"text\" name=\"snapshot_name\" id=\"snapshot_name\" />\n        </div>\n      </div>\n      <div class=\"form_buttons\">\n        <button class=\"button radius right success\" type=\"submit\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Take snapshot", {"name":"tr","hash":{},"data":data})))
    + "</button>\n      </div>\n      <a class=\"close-reveal-modal\">&#215;</a>\n    </form>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/dialogs/disk-snapshot/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vms-tab/dialogs/disk-snapshot/dialogId',['require'],function(require) {
  return 'diskSnapshotVMDialog';
});

define('tabs/vms-tab/dialogs/disk-snapshot',['require','utils/dialogs/dialog','hbs!./disk-snapshot/html','sunstone','utils/tips','./disk-snapshot/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./disk-snapshot/html');
  var Sunstone = require('sunstone');
  var Tips = require('utils/tips');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./disk-snapshot/dialogId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setParams = _setParams;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId
    });
  }

  function _setup(context) {
    var that = this;

    Tips.setup(context);

    $('#' + DIALOG_ID + 'Form', context).submit(function() {
      var snapshot_name = $('#snapshot_name', this).val();
      var obj = {
        "disk_id" : that.diskId,
        "snapshot_name": snapshot_name
      };

      Sunstone.runAction('VM.disk_snapshot_create', that.element.ID, obj);

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      return false;
    });

    return false;
  }

  function _onShow(context) {
    $("#vm_id", context).val(this.element.ID);
    $("#disk_id", context).val(this.diskId);
    $("#snapshot_name", context).focus();
    return false;
  }

  /**
   * @param {object} params
   *        - params.element : VM element
   *        - params.diskId : Disk ID to snapshot
   */
  function _setParams(params) {
    this.element = params.element;
    this.diskId = params.diskId;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vms-tab/dialogs/disk-saveas/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal small\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <h3 class=\"subheader\" id=\"\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk Saveas", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n    </div>\n  </div>\n  <div class=\"reveal-body\">\n    <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\" action=\"\">\n      <div class=\"row\">\n        <div class=\"large-6 columns\">\n          <label for=\"vm_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Machine ID", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input style=\"border-style: inset; background-color: lightgrey\" type=\"text\" name=\"vm_id\" id=\"vm_id\" disabled/>\n        </div>\n        <div class=\"large-6 columns\">\n          <label for=\"disk_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk ID", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input style=\"border-style: inset; background-color: lightgrey\" type=\"text\" name=\"disk_id\" id=\"disk_id\" disabled/>\n        </div>\n      </div>\n      <div class=\"row snapshot_id_row\">\n        <div class=\"large-6 columns\">\n          <label for=\"snapshot_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Snapshot ID", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input style=\"border-style: inset; background-color: lightgrey\" type=\"text\" name=\"snapshot_id\" id=\"snapshot_id\" disabled/>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label for=\"image_name\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "New Image name", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input type=\"text\" name=\"image_name\" id=\"image_name\" />\n        </div>\n      </div>\n      <div class=\"form_buttons\">\n        <button class=\"button radius right success\" type=\"submit\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Save as", {"name":"tr","hash":{},"data":data})))
    + "</button>\n      </div>\n      <a class=\"close-reveal-modal\">&#215;</a>\n    </form>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/dialogs/disk-saveas/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vms-tab/dialogs/disk-saveas/dialogId',['require'],function(require) {
  return 'diskSaveasVMDialog';
});

define('tabs/vms-tab/dialogs/disk-saveas',['require','utils/dialogs/dialog','hbs!./disk-saveas/html','sunstone','utils/tips','./disk-saveas/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./disk-saveas/html');
  var Sunstone = require('sunstone');
  var Tips = require('utils/tips');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./disk-saveas/dialogId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setParams = _setParams;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId
    });
  }

  function _setup(context) {
    var that = this;

    if (that.snapshotId == -1){
      //$(".snapshot_id_row", context).hide();
    } else {
      $(".snapshot_id_row", context).show();
    }

    Tips.setup(context);

    $('#' + DIALOG_ID + 'Form', context).submit(function() {
      var image_name = $('#image_name', this).val();
      var obj = {
        "disk_id" : that.diskId,
        "image_name": image_name,
        "type": "",
        "snapshot_id": that.snapshotId,
      };

      Sunstone.runAction('VM.disk_saveas', that.element.ID, obj);

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      return false;
    });

    return false;
  }

  function _onShow(context) {
    $("#vm_id", context).val(this.element.ID);
    $("#disk_id", context).val(this.diskId);
    $("#snapshot_id", context).val(this.snapshotId);
    $("#image_name", context).focus();
    return false;
  }

  /**
   * @param {object} params
   *        - params.element : VM element
   *        - params.diskId : Disk ID to save as
   *        - params.snapshotId : Disk snapshot ID to save as. Can be undefined
   */
  function _setParams(params) {
    this.element = params.element;
    this.diskId = params.diskId;
    this.snapshotId = -1;

    if(params.snapshotId){
      this.snapshotId = params.snapshotId;
    }
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vms-tab/dialogs/attach-nic/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <h3 class=\"subheader\" id=\"\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Attach new nic", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n    </div>\n  </div>\n  <div class=\"reveal-body\">\n    <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\" action=\"\">\n      <div class=\"row\">\n        <div class=\"large-6 columns\">\n          <label for=\"vm_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Machine ID", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input style=\"border-style: inset; background-color: lightgrey\" type=\"text\" name=\"vm_id\" id=\"vm_id\" disabled/>\n        </div>\n      </div>\n      ";
  stack1 = ((helper = (helper = helpers.nicTabHTML || (depth0 != null ? depth0.nicTabHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"nicTabHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      <div class=\"reveal-footer\">\n        <div class=\"form_buttons\">\n          <button class=\"button radius right success\" id=\"attach_nic_button\" type=\"submit\" value=\"VM.attachdisk\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Attach", {"name":"tr","hash":{},"data":data})))
    + "</button>\n        </div>\n      </div>\n      <a class=\"close-reveal-modal\">&#215;</a>\n    </form>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/dialogs/attach-nic/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/secgroups-tab/tabId',['require'],function(require){
  return 'secgroups-tab';
});
define('tabs/secgroups-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');

  /*
    CONSTANTS
   */

  var RESOURCE = "SecurityGroup";
  var XML_ROOT = "SECURITY_GROUP";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"]},
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    };

    this.columns = [
      Locale.tr("ID"),
      Locale.tr("Owner"),
      Locale.tr("Group"),
      Locale.tr("Name")
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 4,
      "select_resource": Locale.tr("Please select a security group from the list"),
      "you_selected": Locale.tr("You selected the following security group:"),
      "select_resource_multiple": Locale.tr("Please select one or more security groups from the list"),
      "you_selected_multiple": Locale.tr("You selected the following security groups:")
    };

    TabDataTable.call(this);
  }

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json[XML_ROOT];

    return [
        '<input class="check_item" type="checkbox" id="'+RESOURCE.toLowerCase()+'_' +
                             element.ID + '" name="selected_items" value="' +
                             element.ID + '"/>',
        element.ID,
        element.UNAME,
        element.GNAME,
        element.NAME
    ];
  }
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/network/nic-tab/html',['hbs','hbs/handlebars','templates/helpers/advancedSection','templates/helpers/tr','templates/helpers/tip'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "  <fieldset>\n    <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Choose Network", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <label for=\"NETWORK_ID\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Identifier of the virtual network from which to lease an IP and MAC address to this Virtual Machine network interface.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"NETWORK_ID\" id=\"NETWORK_ID\" name=\"NETWORK_ID\"/>\n      </div>\n      <div class=\"medium-6 columns\">\n        <label for=\"NETWORK\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name of the virtual network from which to lease an IP and MAC address to this Virtual Machine network interface.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"NETWORK\" id=\"NETWORK\" name=\"NETWORK\" />\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <label for=\"NETWORK_UID\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User ID", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Identifier of the user owner of the virtual network from which to lease an IP and MAC address to this Virtual Machine network interface.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"NETWORK_UID\" id=\"NETWORK_UID\" name=\"NETWORK_UID\"/>\n      </div>\n      <div class=\"medium-6 columns\">\n        <label for=\"NETWORK_UNAME\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User Name", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name of the user owner of the virtual network from which to lease an IP and MAC address to this Virtual Machine network interface.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"NETWORK_UNAME\" id=\"NETWORK_UNAME\" name=\"NETWORK_UNAME\"/>\n      </div>\n    </div>\n  </fieldset>\n  <fieldset>\n    <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Override Network Values IPv4", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <label for=\"IP\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IP", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Request an specific IP from the Network", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"IP\" id=\"IP\" name=\"IP\" size=\"3\" />\n      </div>\n      <div class=\"medium-6 columns\">\n        <label for=\"MAC\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "MAC", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Set a specific MAC to the NIC", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"MAC\" id=\"MAC\" name=\"MAC\"/>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <label for=\"NETWORK_MASK\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network Mask", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Override the network mask of the network. For example, 255.255.255.0", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"NETWORK_MASK\" id=\"NETWORK_MASK\" name=\"NETWORK_MASK\"/>\n      </div>\n      <div class=\"medium-6 columns\">\n        <label for=\"NETWORK_ADDRESS\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network Address", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Override the base network address. For example, 192.168.1.0", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"NETWORK_ADDRESS\" id=\"NETWORK_ADDRESS\" name=\"NETWORK_ADDRESS\"/>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <label for=\"GATEWAY\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Gateway", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Override the Router of the network.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"GATEWAY\" id=\"GATEWAY\" name=\"GATEWAY\"/>\n      </div>\n      <div class=\"medium-6 columns\">\n        <label for=\"DNS\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "DNS", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Override the specific DNS for this network", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"DNS\" id=\"DNS\" name=\"DNS\"/>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <label for=\"SEARCH_DOMAIN\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Search Domain", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Override the Search Domain for this network.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"SEARCH_DOMAIN\" id=\"SEARCH_DOMAIN\" name=\"SEARCH_DOMAIN\"/>\n      </div>\n    </div>\n  </fieldset>\n  <fieldset>\n    <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Override Network Values IPv6", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <label for=\"IP6_GLOBAL\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IP6 Global", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Request an specific IP from the IPv6 Network", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"IP6_GLOBAL\" id=\"IP6_GLOBAL\" name=\"IP6_GLOBAL\" size=\"3\" />\n      </div>\n      <div class=\"medium-6 columns\">\n        <label for=\"GATEWAY6\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "GATEWAY6", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Override the Router of the IPv6 network.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"GATEWAY6\" id=\"GATEWAY6\" name=\"GATEWAY6\"/>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <label for=\"CONTEXT_FORCE_IPV4\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network Mask", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Override force IPv4 for this IPv6 network. Values: Yes or No.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"CONTEXT_FORCE_IPV4\" id=\"CONTEXT_FORCE_IPV4\" name=\"CONTEXT_FORCE_IPV4\"/>\n      </div>\n    </div>\n  </fieldset>\n  <fieldset>\n    <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Harware", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <label for=\"MODEL\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Model", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hardware that will emulate this network interface. With Xen this is the type attribute of the vif.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"MODEL\" id=\"MODEL\" name=\"MODEL\" />\n      </div>\n    </div>\n  </fieldset>\n  <br>\n  <fieldset>\n    <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Security Groups", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n    <div name=\"str_nic_tab_id\" str_nic_tab_id=\""
    + escapeExpression(((helper = (helper = helpers.nicTabId || (depth0 != null ? depth0.nicTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"nicTabId","hash":{},"data":data}) : helper)))
    + "\">\n      ";
  stack1 = ((helper = (helper = helpers.secgroupsTableSelectHTML || (depth0 != null ? depth0.secgroupsTableSelectHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"secgroupsTableSelectHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n    </div>\n  </fieldset>\n  <div style=\"display: none;\">\n    <fieldset>\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "TCP Firewall", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"large-12 columns text-center\">\n          <input type=\"radio\" class=\"tcp_type\" name=\"tcp_type\" id=\""
    + escapeExpression(((helper = (helper = helpers.nicTabId || (depth0 != null ? depth0.nicTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"nicTabId","hash":{},"data":data}) : helper)))
    + "white_tcp_type\" value=\"WHITE_PORTS_TCP\">\n          <label for=\""
    + escapeExpression(((helper = (helper = helpers.nicTabId || (depth0 != null ? depth0.nicTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"nicTabId","hash":{},"data":data}) : helper)))
    + "white_tcp_type\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Whitelist", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <input type=\"radio\" class=\"tcp_type\" name=\"tcp_type\" id=\""
    + escapeExpression(((helper = (helper = helpers.nicTabId || (depth0 != null ? depth0.nicTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"nicTabId","hash":{},"data":data}) : helper)))
    + "black_tcp_type\" value=\"BLACK_PORTS_TCP\">\n          <label for=\""
    + escapeExpression(((helper = (helper = helpers.nicTabId || (depth0 != null ? depth0.nicTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"nicTabId","hash":{},"data":data}) : helper)))
    + "black_tcp_type\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Blacklist", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label for=\"TCP_PORTS\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PORTS", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "A list of ports separated by commas or a ranges separated by semicolons, e.g.: 22,80,5900:6000", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" id=\"TCP_PORTS\" name=\"ports\" />\n        </div>\n      </div>\n    </fieldset>\n    <fieldset>\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "UDP Firewall", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"large-12 columns text-center\">\n          <input type=\"radio\" class=\"udp_type\" name=\"udp_type\" id=\""
    + escapeExpression(((helper = (helper = helpers.nicTabId || (depth0 != null ? depth0.nicTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"nicTabId","hash":{},"data":data}) : helper)))
    + "white_udp_type\" value=\"WHITE_PORTS_UDP\">\n          <label for=\""
    + escapeExpression(((helper = (helper = helpers.nicTabId || (depth0 != null ? depth0.nicTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"nicTabId","hash":{},"data":data}) : helper)))
    + "white_udp_type\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Whitelist", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <input type=\"radio\" class=\"udp_type\" name=\"udp_type\" id=\""
    + escapeExpression(((helper = (helper = helpers.nicTabId || (depth0 != null ? depth0.nicTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"nicTabId","hash":{},"data":data}) : helper)))
    + "black_udp_type\" value=\"BLACK_PORTS_UDP\">\n          <label for=\""
    + escapeExpression(((helper = (helper = helpers.nicTabId || (depth0 != null ? depth0.nicTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"nicTabId","hash":{},"data":data}) : helper)))
    + "black_udp_type\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Blacklist", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label for=\"UDP_PORTS\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PORTS", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "A list of ports separated by commas or a ranges separated by semicolons, e.g.: 22,80,5900:6000", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n          </label>\n          <input type=\"text\" id=\"UDP_PORTS\" name=\"ports\" />\n        </div>\n      </div>\n    </fieldset>\n    <fieldset>\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ICMP", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label>\n            <input type=\"checkbox\" name=\"icmp_type\" value=\"ICMP\" id=\"icmp_type\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Drop", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        </div>\n      </div>\n    </fieldset>\n  </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, buffer = "<div class=\"row\">\n  ";
  stack1 = ((helper = (helper = helpers.vnetsTableSelectHTML || (depth0 != null ? depth0.vnetsTableSelectHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"vnetsTableSelectHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n  <br>\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced Options", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/network/nic-tab/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/create/wizard-tabs/network/nic-tab',['require','sunstone-config','utils/locale','utils/tips','tabs/vnets-tab/datatable','tabs/secgroups-tab/datatable','utils/wizard-fields','hbs!./nic-tab/html'],function(require) {
  /*
    DEPENDENCIES
   */

  var Config = require('sunstone-config');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var VNetsTable = require('tabs/vnets-tab/datatable');
  var SecgroupsTable = require('tabs/secgroups-tab/datatable');
  var WizardFields = require('utils/wizard-fields');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./nic-tab/html');

  /*
    CONSTANTS
   */

  /*
    CONSTRUCTOR
   */

  function DiskTab(nicTabId) {
    this.nicTabId = 'nicTab' + nicTabId;

    this.vnetsTable = new VNetsTable(this.nicTabId + 'Table', {'select': true});

    var secgroupSelectOptions = {
      'select': true,
      'selectOptions': {
        "multiple_choice": true
      }
    }
    this.secgroupsTable = new SecgroupsTable(this.nicTabId + 'SGTable', secgroupSelectOptions);
  }

  DiskTab.prototype.constructor = DiskTab;
  DiskTab.prototype.html = _html;
  DiskTab.prototype.setup = _setup;
  DiskTab.prototype.onShow = _onShow;
  DiskTab.prototype.retrieve = _retrieve;
  DiskTab.prototype.fill = _fill;

  return DiskTab;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'nicTabId': this.nicTabId,
      'vnetsTableSelectHTML': this.vnetsTable.dataTableHTML,
      'secgroupsTableSelectHTML': this.secgroupsTable.dataTableHTML
    });
  }

  function _onShow(context, panelForm) {
    this.vnetsTable.refreshResourceTableSelect();
  }

  function _setup(context) {
    var that = this;
    that.vnetsTable.initialize({
      'selectOptions': {
        'select_callback': function(aData, options) {
          // If the net is selected by Id, avoid overwriting it with name+uname
          if ($('#NETWORK_ID', context).val() != aData[options.id_index]) {
            $('#NETWORK_ID', context).val("");
            $('#NETWORK', context).val(aData[options.name_index]);
            $('#NETWORK_UNAME', context).val(aData[options.uname_index]);
            $('#NETWORK_UID', context).val("");
          }
        }
      }
    });
    that.vnetsTable.refreshResourceTableSelect();

    that.secgroupsTable.initialize();
    that.secgroupsTable.refreshResourceTableSelect();
  }

  function _retrieve(context) {
    var nicJSON = WizardFields.retrieve(context);

    var tcp = $("input.tcp_type:checked", context).val();
    if (tcp) {
      nicJSON[tcp] = $("#TCP_PORTS", context).val();
    }

    var udp = $("input.udp_type:checked", context).val();
    if (udp) {
      nicJSON[udp] = $("#UDP_PORTS", context).val();
    }

    if ($("#icmp_type", context).is(":checked")) {
      nicJSON["ICMP"] = "drop"
    }

    var secgroups = this.secgroupsTable.retrieveResourceTableSelect();
    if (secgroups != undefined && secgroups.length != 0) {
      nicJSON["SECURITY_GROUPS"] = secgroups.join(",");
    }

    return nicJSON;
  }

  function _fill(context, templateJSON) {
    if (templateJSON.NETWORK_ID != undefined) {
      var selectedResources = {
          ids : templateJSON.NETWORK_ID
        }

      this.vnetsTable.selectResourceTableSelect(selectedResources);
    } else if (templateJSON.NETWORK != undefined && templateJSON.NETWORK_UNAME != undefined) {
      var selectedResources = {
          names : {
            name: templateJSON.NETWORK,
            uname: templateJSON.NETWORK_UNAME
          }
        }

      this.vnetsTable.selectResourceTableSelect(selectedResources);
    }

    if (templateJSON["WHITE_PORTS_TCP"]) {
      var field = $("input.tcp_type[value='WHITE_PORTS_TCP']", context);
      field.click();

      $("#TCP_PORTS", context).val(templateJSON["WHITE_PORTS_TCP"]);
    } else if (templateJSON["BLACK_PORTS_TCP"]) {
      var field = $("input.tcp_type[value='BLACK_PORTS_TCP']", context);
      field.click();

      $("#TCP_PORTS", context).val(templateJSON["BLACK_PORTS_TCP"]);
    }

    if (templateJSON["WHITE_PORTS_UDP"]) {
      var field = $("input.udp_type[value='WHITE_PORTS_UDP']", context);
      field.click();

      $("#UDP_PORTS", context).val(templateJSON["WHITE_PORTS_UDP"]);
    } else if (templateJSON["BLACK_PORTS_UDP"]) {
      var field = $("input.udp_type[value='BLACK_PORTS_UDP']", context);
      field.click();

      $("#UDP_PORTS", context).val(templateJSON["BLACK_PORTS_UDP"]);
    }

    if (templateJSON["ICMP"]) {
      var field = $("#icmp_type", context);
      $("#icmp_type", context).attr('checked', 'checked');
    }

    if (templateJSON["SECURITY_GROUPS"] != undefined &&
        templateJSON["SECURITY_GROUPS"].length != 0) {

      var selectedResources = {ids: templateJSON["SECURITY_GROUPS"].split(",")};
      this.secgroupsTable.selectResourceTableSelect(selectedResources);
    } else {
      this.secgroupsTable.refreshResourceTableSelect();
    }

    WizardFields.fill(context, templateJSON);
  }
});

define('tabs/vms-tab/dialogs/attach-nic/dialogId',['require'],function(require) {
  return 'attachNICVMDialog';
});

define('tabs/vms-tab/dialogs/attach-nic',['require','utils/dialogs/dialog','hbs!./attach-nic/html','sunstone','utils/notifier','utils/tips','tabs/templates-tab/form-panels/create/wizard-tabs/network/nic-tab','utils/wizard-fields','./attach-nic/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./attach-nic/html');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Tips = require('utils/tips');
  var NicTab = require('tabs/templates-tab/form-panels/create/wizard-tabs/network/nic-tab');
  var WizardFields = require('utils/wizard-fields');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./attach-nic/dialogId');
  var TAB_ID = require('../tabId')

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    this.nicTab = new NicTab(DIALOG_ID + 'NickTab');

    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setElement = _setElement;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId,
      'nicTabHTML': this.nicTab.html()
    });
  }

  function _setup(context) {
    var that = this;
    that.nicTab.setup(context);

    Tips.setup(context);

    $('#' + DIALOG_ID + 'Form', context).submit(function() {
      var templateJSON = that.nicTab.retrieve(context);
      var obj = {
        "NIC": templateJSON
      }

      Sunstone.runAction('VM.attachnic', that.element.ID, obj);

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      return false;
    });

    return false;
  }

  function _onShow(context) {
    this.nicTab.onShow(context);
    $("#vm_id", context).val(this.element.ID);
    return false;
  }

  function _setElement(element) {
    this.element = element
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vms-tab/dialogs/snapshot/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal small\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <h3 class=\"subheader\" id=\"\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Snapshot", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n    </div>\n  </div>\n  <div class=\"reveal-body\">\n    <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\" action=\"\">\n      <div class=\"row\">\n        <div class=\"large-6 columns\">\n          <label for=\"vm_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Machine ID", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input style=\"border-style: inset; background-color: lightgrey\" type=\"text\" name=\"vm_id\" id=\"vm_id\" disabled/>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label for=\"snapshot_name\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Snapshot name", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input type=\"text\" name=\"snapshot_name\" id=\"snapshot_name\" />\n        </div>\n      </div>\n      <div class=\"form_buttons\">\n        <button class=\"button radius right success\" type=\"submit\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Take snapshot", {"name":"tr","hash":{},"data":data})))
    + "</button>\n      </div>\n      <a class=\"close-reveal-modal\">&#215;</a>\n    </form>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/dialogs/snapshot/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vms-tab/dialogs/snapshot/dialogId',['require'],function(require) {
  return 'snapshotVMDialog';
});

define('tabs/vms-tab/dialogs/snapshot',['require','utils/dialogs/dialog','hbs!./snapshot/html','sunstone','utils/tips','./snapshot/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./snapshot/html');
  var Sunstone = require('sunstone');
  var Tips = require('utils/tips');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./snapshot/dialogId');
  var TAB_ID = require('../tabId')

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setElement = _setElement;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId
    });
  }

  function _setup(context) {
    var that = this;

    Tips.setup(context);

    $('#' + DIALOG_ID + 'Form', context).submit(function() {
      var snapshot_name = $('#snapshot_name', this).val();
      var obj = {
        "snapshot_name": snapshot_name
      }

      Sunstone.runAction('VM.snapshot_create', that.element.ID, obj);

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      return false;
    });

    return false;
  }

  function _onShow(context) {
    $("#vm_id", context).val(this.element.ID);
    $("#snapshot_name", context).focus();
    return false;
  }

  function _setElement(element) {
    this.element = element
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vms-tab/dialogs/vnc/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large max-height\" style=\"width:auto; max-width:70%\" data-reveal >\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <h3 class=\"subheader\" id=\"vnc_dialog\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VNC", {"name":"tr","hash":{},"data":data})))
    + "\n        <span id=\"VNC_status\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Loading", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        <a id=\"open_in_a_new_window\" href=\"\" target=\"_blank\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Open in a new window", {"name":"tr","hash":{},"data":data})))
    + "\"> <i class=\"fa fa-external-link detach-vnc-icon\"/></a>\n        <span id=\"VNC_buttons\" class=\"right\">\n          <input type=\"button\" value=\"Send CtrlAltDel\" id=\"sendCtrlAltDelButton\"></span>\n      </h3>\n    </div>\n  </div>\n  <div class=\"reveal-body\" style=\"width:100%; overflow-x:overlay\">\n    <canvas id=\"VNC_canvas\" width=\"640px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Canvas not supported.", {"name":"tr","hash":{},"data":data})))
    + "</canvas>\n    <div id=\"VNC_status_bar\" class=\"VNC_status_bar\"></div>\n  </div>\n  <a class=\"close-reveal-modal\">&#215;</a>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/dialogs/vnc/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vms-tab/dialogs/vnc',['require','utils/dialogs/dialog','hbs!./vnc/html','sunstone','utils/vnc','./vnc/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./vnc/html');
  var Sunstone = require('sunstone');
  var Vnc = require('utils/vnc');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./vnc/dialogId');
  var TAB_ID = require('../tabId')

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.onClose = _onClose;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setElement = _setElement;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId
    });
  }

  function _setup(context) {
    var that = this;

    $("#open_in_a_new_window", context).on("click", function() {
       var dialog = Sunstone.getDialog(DIALOG_ID);
       dialog.hide();
    });

    $('#sendCtrlAltDelButton', context).click(function() {
      Vnc.sendCtrlAltDel();
      return false;
    });

    return false;
  }

  function _onShow(context) {
    Vnc.vncCallback(this.element);
    return false;
  }

  function _onClose(context) {
    Vnc.disconnect();
    Vnc.unlock();
    return false;
  }

  function _setElement(element) {
    this.element = element
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vms-tab/dialogs/spice/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large max-height\" style=\"width:auto; max-width:70%\" data-reveal >\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <h3 class=\"subheader\" id=\"spice_dialog\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "SPICE", {"name":"tr","hash":{},"data":data})))
    + "\n        <span id=\"vnc_buttons\">\n          <a id=\"open_in_a_new_window_spice\" href=\"\" target=\"_blank\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Open in a new window", {"name":"tr","hash":{},"data":data})))
    + "\"> <i class=\"fa fa-external-link detach-spice-icon\"/></a>\n        </span>\n      </h3>\n    </div>\n  </div>\n  <div class=\"reveal-body\" style=\"width:100%; overflow-x:overlay\">\n    <div id=\"spice-area\">\n      <div id=\"spice-screen\" class=\"spice-screen\"></div>\n    </div>\n  </div>\n  <div id=\"message-div\" class=\"spice-message\" hidden></div>\n  <a class=\"close-reveal-modal\">&#215;</a>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/dialogs/spice/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vms-tab/dialogs/spice',['require','utils/dialogs/dialog','hbs!./spice/html','sunstone','utils/spice','./spice/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./spice/html');
  var Sunstone = require('sunstone');
  var Spice = require('utils/spice');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./spice/dialogId');
  var TAB_ID = require('../tabId')

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.onClose = _onClose;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setElement = _setElement;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId
    });
  }

  function _setup(context) {
    var that = this;

    $("#open_in_a_new_window_spice", context).on("click", function() {
      var dialog = Sunstone.getDialog(DIALOG_ID);
      dialog.hide();
    });

    return false;
  }

  function _onShow(context) {
    Spice.spiceCallback(this.element);
    return false;
  }

  function _onClose(context) {
    Spice.disconnect();
    Spice.unlock();
    return false;
  }

  function _setElement(element) {
    this.element = element
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vms-tab/dialogs/saveas-template/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal small\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <h3 class=\"subheader\" id=\"\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Save as Template", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n    </div>\n  </div>\n  <div class=\"reveal-body\">\n    <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\" action=\"\">\n      <div class=\"row\">\n        <div class=\"large-6 columns\">\n          <label for=\"vm_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Machine ID", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input style=\"border-style: inset; background-color: lightgrey\" type=\"text\" name=\"vm_id\" id=\"vm_id\" disabled/>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label for=\"template_name\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Template name", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input type=\"text\" name=\"template_name\" id=\"template_name\" />\n        </div>\n      </div>\n      <div class=\"form_buttons\">\n        <button class=\"button radius right success\" type=\"submit\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Save As Template", {"name":"tr","hash":{},"data":data})))
    + "</button>\n      </div>\n      <a class=\"close-reveal-modal\">&#215;</a>\n    </form>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/dialogs/saveas-template/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vms-tab/dialogs/saveas-template',['require','utils/dialogs/dialog','hbs!./saveas-template/html','sunstone','opennebula','utils/notifier','utils/tips','./saveas-template/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./saveas-template/html');
  var Sunstone = require('sunstone');
  var OpenNebula = require('opennebula');
  var Notifier = require('utils/notifier');
  var Tips = require('utils/tips');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./saveas-template/dialogId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId
    });
  }

  function _setup(context) {
    var that = this;

    Tips.setup(context);


    $('#' + DIALOG_ID + 'Form', context).submit(function() {
      var template_name = $('#template_name', this).val();
      var vm_id = $("#vm_id", context).val();

      OpenNebula.VM.save_as_template({
        data : {
          id: vm_id,
          extra_param: {
            name : template_name
          }
        },
        success: function(request, response){
          OpenNebula.Action.clear_cache("VMTEMPLATE");
          Notifier.notifyMessage(Locale.tr("VM Template") + ' ' + template_name + ' ' + Locale.tr("saved successfully"))
        },
        error: function(request, response){
          Notifier.onError(request, response);
        }
      })

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      return false;
    });

    return false;
  }

  function _onShow(context) {
    $("#vm_id", context).val(Sunstone.getDataTable(TAB_ID).elements());
    $("#template_name", context).focus();
    return false;
  }
});


/* START_TEMPLATE */
define('hbs!utils/panel/permissions-table/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<table  class=\""
    + escapeExpression(((helper = (helper = helpers.resourceType || (depth0 != null ? depth0.resourceType : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"resourceType","hash":{},"data":data}) : helper)))
    + "_permissions_table dataTable extended_table\">\n  ";
  stack1 = ((helper = (helper = helpers.permissionsHTML || (depth0 != null ? depth0.permissionsHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"permissionsHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n  <thead>\n    <tr>\n      <th colspan=\"4\" style=\"width:130px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ownership", {"name":"tr","hash":{},"data":data})))
    + "</th>\n    </tr>\n  </thead>\n  ";
  stack1 = ((helper = (helper = helpers.ownerHTML || (depth0 != null ? depth0.ownerHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"ownerHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n  ";
  stack1 = ((helper = (helper = helpers.groupHTML || (depth0 != null ? depth0.groupHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"groupHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n</table>";
},"useData":true});
Handlebars.registerPartial('utils/panel/permissions-table/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!utils/panel/permissions-table/permissions',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<thead>\n  <tr>\n    <th style=\"width:130px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Permissions", {"name":"tr","hash":{},"data":data})))
    + ":</th>\n    <th style=\"width:40px;text-align:center;\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Use", {"name":"tr","hash":{},"data":data})))
    + "</th>\n    <th style=\"width:40px;text-align:center;\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Manage", {"name":"tr","hash":{},"data":data})))
    + "</th>\n    <th style=\"width:40px;text-align:center;\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Admin", {"name":"tr","hash":{},"data":data})))
    + "</th>\n  </tr>\n</thead>\n<tr>\n  <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Owner", {"name":"tr","hash":{},"data":data})))
    + "</td>\n  <td style=\"text-align:center\">\n    <input type=\"checkbox\" class=\"permission_check owner_u\" />\n  </td>\n  <td style=\"text-align:center\">\n    <input type=\"checkbox\" class=\"permission_check owner_m\" />\n  </td>\n  <td style=\"text-align:center\">\n    <input type=\"checkbox\" class=\"permission_check owner_a\" />\n  </td>\n</tr>\n<tr>\n  <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group", {"name":"tr","hash":{},"data":data})))
    + "</td>\n  <td style=\"text-align:center\">\n    <input type=\"checkbox\" class=\"permission_check group_u\" />\n  </td>\n  <td style=\"text-align:center\">\n    <input type=\"checkbox\" class=\"permission_check group_m\" />\n  </td>\n  <td style=\"text-align:center\">\n    <input type=\"checkbox\" class=\"permission_check group_a\" />\n  </td>\n</tr>\n<tr>\n  <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Other", {"name":"tr","hash":{},"data":data})))
    + "</td>\n  <td style=\"text-align:center\">\n    <input type=\"checkbox\" class=\"permission_check other_u\" />\n  </td>\n  <td style=\"text-align:center\">\n    <input type=\"checkbox\" class=\"permission_check other_m\" />\n  </td>\n  <td style=\"text-align:center\">\n    <input type=\"checkbox\" class=\"permission_check other_a\" />\n  </td>\n</tr>";
},"useData":true});
Handlebars.registerPartial('utils/panel/permissions-table/permissions', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!utils/panel/permissions-table/owner',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/isTabActionEnabled'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  return "    <div id=\"div_edit_chg_owner\">\n      <a id=\"div_edit_chg_owner_link\" class=\"edit_e\" href=\"#\"> <i class=\"fa fa-pencil-square-o right\"/></a>\n    </div>\n";
  },"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, buffer = "<tr>\n  <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Owner", {"name":"tr","hash":{},"data":data})))
    + "</td>\n  <td colspan=\"2\" id=\"value_td_owner\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.UNAME : stack1), depth0))
    + "</td>\n  <td>\n";
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, (depth0 != null ? depth0.tabName : depth0), (depth0 != null ? depth0.action : depth0), {"name":"isTabActionEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  </td>\n</tr>\n";
},"useData":true});
Handlebars.registerPartial('utils/panel/permissions-table/owner', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!utils/panel/permissions-table/group',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/isTabActionEnabled'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  return "    <div id=\"div_edit_chg_group\">\n      <a id=\"div_edit_chg_group_link\" class=\"edit_e\" href=\"#\"> <i class=\"fa fa-pencil-square-o right\"/></a>\n    </div>\n";
  },"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, buffer = "<tr>\n  <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Group", {"name":"tr","hash":{},"data":data})))
    + "</td>\n  <td colspan=\"2\" id=\"value_td_group\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.GNAME : stack1), depth0))
    + "</td>\n  <td>\n";
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, (depth0 != null ? depth0.tabName : depth0), (depth0 != null ? depth0.action : depth0), {"name":"isTabActionEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  </td>\n</tr>\n";
},"useData":true});
Handlebars.registerPartial('utils/panel/permissions-table/group', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/panel/permissions-table',['require','hbs!./permissions-table/html','hbs!./permissions-table/permissions','hbs!./permissions-table/owner','hbs!./permissions-table/group','utils/resource-select','sunstone','sunstone-config'],function(require) {
  /*
    This module insert a row with the name of the resource.
    The row can be edited and a rename action will be sent
   */

  var TemplatePermissionsTable = require('hbs!./permissions-table/html');
  var TemplatePermissions = require('hbs!./permissions-table/permissions');
  var TemplateOwner = require('hbs!./permissions-table/owner');
  var TemplateGroup = require('hbs!./permissions-table/group');
  var ResourceSelect = require('utils/resource-select');
  var Sunstone = require('sunstone');
  var Config = require('sunstone-config');

  /*
    Generate the tr HTML with the name of the resource and an edit icon
    @param {String} tabName
    @param {String} resourceType Resource type (i.e: Zone, Host, Image...)
    @param {Object} element OpenNebula object (i.e: element.ID, element.GNAME)
    @returns {String} HTML row
   */
  var _html = function(tabName, resourceType, element) {
    var permissionsHTML = '';
    if (Config.isTabActionEnabled(tabName, resourceType + '.chmod')) {
      permissionsHTML = TemplatePermissions({'element': element})
    }

    var ownerHTML = TemplateOwner({
      'tabName': tabName,
      'action': resourceType + '.chown',
      'element': element
    });

    var groupHTML = TemplateGroup({
      'tabName': tabName,
      'action': resourceType + '.chgrp',
      'element': element
    })

    var permissionsTableHTML = TemplatePermissionsTable({
      'resourceType': resourceType.toLowerCase(),
      'permissionsHTML': permissionsHTML,
      'ownerHTML': ownerHTML,
      'groupHTML': groupHTML
    })

    return permissionsTableHTML;
  };

  /*
    Initialize the row, clicking the edit icon will add an input to edit the name
    @param {String} tabName
    @param {String} resourceType Resource type (i.e: Zone, Host, Image...)
    @param {Object} element OpenNebula object (i.e: element.ID, element.GNAME)
    @param {jQuery Object} context Selector including the tr
   */
  var _setup = function(tabName, resourceType, element, context) {
    var resourceId = element.ID
    if (Config.isTabActionEnabled(tabName, resourceType + '.chmod')) {
      _setPermissionsTable(element, context);

      context.off('change', ".permission_check");
      context.on('change', ".permission_check", function() {
        var permissionsOctet = {octet : _buildOctet(context)};
        Sunstone.runAction(resourceType + ".chmod", resourceId, permissionsOctet);
      });
    }

    if (Config.isTabActionEnabled(tabName, resourceType + '.chown')) {
      context.off("click", "#div_edit_chg_owner_link");
      context.on("click", "#div_edit_chg_owner_link", function() {
          var tr_context = $(this).parents("tr");
          ResourceSelect.insert("#value_td_owner", context, "User", element.UID, false);
        });

      context.off("change", "#value_td_owner .resource_list_select");
      context.on("change", "#value_td_owner .resource_list_select", function() {
          var newOwnerId = $(this).val();
          if (newOwnerId != "") {
            Sunstone.runAction(resourceType + ".chown", [resourceId], newOwnerId);
          }
        });
    }

    if (Config.isTabActionEnabled(tabName, resourceType + '.chgrp')) {
      context.off("click", "#div_edit_chg_group_link");
      context.on("click", "#div_edit_chg_group_link", function() {
          ResourceSelect.insert("#value_td_group", context, "Group", element.GID, false);
        });

      context.off("change", "#value_td_group .resource_list_select");
      context.on("change", "#value_td_group .resource_list_select", function() {
          var newGroupId = $(this).val();
          if (newGroupId != "") {
            Sunstone.runAction(resourceType + ".chgrp", [resourceId], newGroupId);
          }
        });
    }

    return false;
  }

  //Returns an octet given a permission table with checkboxes
  var _buildOctet = function(context) {
    var owner = 0;
    var group = 0;
    var other = 0;

    if ($('.owner_u', context).is(':checked'))
        owner += 4;
    if ($('.owner_m', context).is(':checked'))
        owner += 2;
    if ($('.owner_a', context).is(':checked'))
        owner += 1;

    if ($('.group_u', context).is(':checked'))
        group += 4;
    if ($('.group_m', context).is(':checked'))
        group += 2;
    if ($('.group_a', context).is(':checked'))
        group += 1;

    if ($('.other_u', context).is(':checked'))
        other += 4;
    if ($('.other_m', context).is(':checked'))
        other += 2;
    if ($('.other_a', context).is(':checked'))
        other += 1;

    return "" + owner + group + other;
  };

  var _ownerUse = function(element) {
    return parseInt(element.PERMISSIONS.OWNER_U);
  };
  var _ownerManage = function(element) {
    return parseInt(element.PERMISSIONS.OWNER_M);
  };
  var _ownerAdmin = function(element) {
    return parseInt(element.PERMISSIONS.OWNER_A);
  };

  var _groupUse = function(element) {
    return parseInt(element.PERMISSIONS.GROUP_U);
  };
  var _groupManage = function(element) {
    return parseInt(element.PERMISSIONS.GROUP_M);
  };
  var _groupAdmin = function(element) {
    return parseInt(element.PERMISSIONS.GROUP_A);
  };

  var _otherUse = function(element) {
    return parseInt(element.PERMISSIONS.OTHER_U);
  };
  var _otherManage = function(element) {
    return parseInt(element.PERMISSIONS.OTHER_M);
  };
  var _otherAdmin = function(element) {
    return parseInt(element.PERMISSIONS.OTHER_A);
  };

  var _ownerPermStr = function(element) {
    var result = "";
    result += _ownerUse(element) ? "u" : "-";
    result += _ownerManage(element) ? "m" : "-";
    result += _ownerAdmin(element) ? "a" : "-";
    return result;
  };

  var _groupPermStr = function(element) {
    var result = "";
    result += _groupUse(element) ? "u" : "-";
    result += _groupManage(element) ? "m" : "-";
    result += _groupAdmin(element) ? "a" : "-";
    return result;
  };

  var _otherPermStr = function(element) {
    var result = "";
    result += _otherUse(element) ? "u" : "-";
    result += _otherManage(element) ? "m" : "-";
    result += _otherAdmin(element) ? "a" : "-";
    return result;
  };

  var _setPermissionsTable = function(element, context) {
    if (_ownerUse(element))
        $('.owner_u', context).attr('checked', 'checked');
    if (_ownerManage(element))
        $('.owner_m', context).attr('checked', 'checked');
    if (_ownerAdmin(element))
        $('.owner_a', context).attr('checked', 'checked');
    if (_groupUse(element))
        $('.group_u', context).attr('checked', 'checked');
    if (_groupManage(element))
        $('.group_m', context).attr('checked', 'checked');
    if (_groupAdmin(element))
        $('.group_a', context).attr('checked', 'checked');
    if (_otherUse(element))
        $('.other_u', context).attr('checked', 'checked');
    if (_otherManage(element))
        $('.other_m', context).attr('checked', 'checked');
    if (_otherAdmin(element))
        $('.other_a', context).attr('checked', 'checked');
  };

  return {
    'html': _html,
    'setup': _setup
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vms-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "<div class=\"row\">\n<div class=\"large-9 columns\">\n  <table class=\"dataTable extended_table\">\n    <thead>\n      <tr>\n        <th colspan=\"2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Monitoring", {"name":"tr","hash":{},"data":data})))
    + "</th>\n      </tr>\n    </thead>\n    <tbody>\n      ";
  stack1 = ((helper = (helper = helpers.monitoringTableContentHTML || (depth0 != null ? depth0.monitoringTableContentHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"monitoringTableContentHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </tbody>\n  </table>\n</div>\n</div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n        </tr>\n        ";
  stack1 = ((helper = (helper = helpers.renameTrHTML || (depth0 != null ? depth0.renameTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"renameTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "State", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.stateStr || (depth0 != null ? depth0.stateStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"stateStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "LCM State", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.lcmStateStr || (depth0 != null ? depth0.lcmStateStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"lcmStateStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Host", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.hostname || (depth0 != null ? depth0.hostname : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"hostname","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Start time", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.prettyStartTime || (depth0 != null ? depth0.prettyStartTime : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"prettyStartTime","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Deploy ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.deployId || (depth0 != null ? depth0.deployId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"deployId","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Reschedule", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.resched || (depth0 != null ? depth0.resched : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"resched","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n      </tr>\n    </tbody>\n  </table>\n</div>\n<div class=\"large-6 columns\">";
  stack1 = ((helper = (helper = helpers.permissionsTableHTML || (depth0 != null ? depth0.permissionsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"permissionsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n</div>\n<div class=\"row\">\n<div class=\"large-9 columns\">";
  stack1 = ((helper = (helper = helpers.templateTableHTML || (depth0 != null ? depth0.templateTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templateTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n</div>\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.monitoringTableContentHTML : depth0), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vms-tab/panels/info/panelId',['require'],function(require){
  return 'vm_info_tab';
});
define('tabs/vms-tab/panels/info',['require','utils/locale','utils/humanize','utils/panel/rename-tr','utils/panel/permissions-table','utils/panel/template-table','opennebula/vm','../utils/state-actions','hbs!./info/html','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var RenameTr = require('utils/panel/rename-tr');
  var PermissionsTable = require('utils/panel/permissions-table');
  var TemplateTable = require('utils/panel/template-table');
  var OpenNebulaVM = require('opennebula/vm');
  var StateActions = require('../utils/state-actions');

  /*
    TEMPLATES
   */

  var TemplateInfo = require('hbs!./info/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "VM"
  var XML_ROOT = "VM"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[XML_ROOT];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var renameTrHTML = RenameTr.html(RESOURCE, this.element.NAME);
    var permissionsTableHTML = PermissionsTable.html(TAB_ID, RESOURCE, this.element);
    var prettyStartTime = Humanize.prettyTime(this.element.STIME);

    var stateStr = OpenNebulaVM.stateStr(this.element.STATE);
    var lcmStateStr = OpenNebulaVM.lcmStateStr(this.element.LCM_STATE);
    var hostname = OpenNebulaVM.hostnameStr(this.element);

    var deployId = (typeof(this.element.DEPLOY_ID) == "object" ? "-" : this.element.DEPLOY_ID);
    var resched = (parseInt(this.element.RESCHED) ? Locale.tr("yes") : Locale.tr("no"))
    var templateTableHTML = TemplateTable.html(this.element.USER_TEMPLATE, RESOURCE, Locale.tr("Attributes"));

    var monitoring = $.extend({}, this.element.MONITORING);
    delete monitoring.CPU;
    delete monitoring.MEMORY;
    delete monitoring.NETTX;
    delete monitoring.NETRX;
    delete monitoring.STATE;
    delete monitoring.DISK_SIZE;
    delete monitoring.SNAPSHOT_SIZE;
    var monitoringTableContentHTML;
    if (!$.isEmptyObject(monitoring)) {
      monitoringTableContentHTML = Humanize.prettyPrintJSON(monitoring);
    }

    return TemplateInfo({
      'element': this.element,
      'renameTrHTML': renameTrHTML,
      'stateStr': stateStr,
      'lcmStateStr': lcmStateStr,
      'hostname': hostname,
      'prettyStartTime': prettyStartTime,
      'deployId': deployId,
      'resched': resched,
      'permissionsTableHTML': permissionsTableHTML,
      'templateTableHTML': templateTableHTML,
      'monitoringTableContentHTML': monitoringTableContentHTML,
      'renameTrHTML': renameTrHTML
    });
  }

  function _setup(context) {
    var state;
    if (this.element.STATE == OpenNebulaVM.STATES.ACTIVE) {
      state = OpenNebulaVM.shortLcmStateStr(this.element.LCM_STATE);
    } else {
      state = OpenNebulaVM.stateStr(this.element.STATE);
    }

    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);
    $('.resource-info-header-small', '#' + TAB_ID).text(state);

    // Enable only action buttons for the current state
    StateActions.disableAllStateActions();
    StateActions.enableStateActions(this.element.STATE, this.element.LCM_STATE);
    // Enable / disable vnc button
    if (OpenNebulaVM.isVNCSupported(this.element)) {
      $(".vnc-right-info").show();
    } else {
      $(".vnc-right-info").hide();
    }

    if (OpenNebulaVM.isSPICESupported(this.element)) {
      $(".spice-right-info").show()
    } else {
      $(".spice-right-info").hide();;
    }

    RenameTr.setup(RESOURCE, this.element.ID, context);
    PermissionsTable.setup(TAB_ID, RESOURCE, this.element, context);

    // Get rid of the unwanted (for show) SCHED_* keys
    var that = this;
    var strippedTemplate = {};
    var unshownValues = {};
    $.each(that.element.USER_TEMPLATE, function(key, value) {
      if (!key.match(/^SCHED_*/)) {
        strippedTemplate[key] = value;
      } else {
        unshownValues[key] = value;
      }
    })

    TemplateTable.setup(strippedTemplate, RESOURCE, this.element.ID, context, unshownValues);
  }
});

define('templates/helpers/humanizeSize',['require','hbs/handlebars','utils/humanize'],function(require) {
  var Handlebars = require('hbs/handlebars');
  var Humanize = require('utils/humanize');

  /**
   * Returns a human readable size in Kilo, Mega, Giga or Tera bytes
   * @param  {string} unit    one of MB, KB, B
   * @param  {integer} value  value
   * @param  {object} options
   * @return {string}         human readable size
   */
  var humanizeSize = function(unit, value, options) {
    switch(unit.toUpperCase()){
      case 'B':
        return Humanize.sizeFromB(value);
      case 'K':
      case 'KB':
        return Humanize.sizeFromKB(value);
      case 'M':
      case 'MB':
        return Humanize.sizeFromMB(value);
      default:
        return value;
    }
  };

  Handlebars.registerHelper('humanizeSize', humanizeSize);

  return humanizeSize;
});

/* START_TEMPLATE */
define('hbs!tabs/vms-tab/panels/capacity/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/valOrDefault','templates/helpers/humanizeSize','templates/helpers/isTabActionEnabled'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, buffer = "";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.resizeStateEnabled : depth0), {"name":"if","hash":{},"fn":this.program(2, data),"inverse":this.program(4, data),"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"2":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "            <button id=\"resize_capacity\" class=\"button tiny success right radius\" >"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Resize", {"name":"tr","hash":{},"data":data})))
    + "</button>\n";
},"4":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "            <button id=\"resize_capacity\" class=\"button tiny success right radius\" disabled=\"disabled\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Resize", {"name":"tr","hash":{},"data":data})))
    + "</button>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row\">\n  <div class=\"large-9 columns\">\n    <table class=\"info_table dataTable extended_table\">\n      <thead>\n        <tr>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VCPU", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "MEMORY", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cost / CPU", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cost / MByte", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th></th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td id=\"cpu_info\">"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.CPU : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td id=\"vcpu_info\">"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.VCPU : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td id=\"memory_info\">"
    + escapeExpression(((helpers.humanizeSize || (depth0 && depth0.humanizeSize) || helperMissing).call(depth0, "MB", ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.MEMORY : stack1), {"name":"humanizeSize","hash":{},"data":data})))
    + "</td>\n          <td id=\"cpu_cost_info\">"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.CPU_COST : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td id=\"memory_cost_info\" >"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.MEMORY_COST : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td>\n";
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "vms-tab", "VM.resize", {"name":"isTabActionEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "          </td>\n        </tr>\n      </tbody>\n    </table>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"medium-6 columns\">\n    <div class=\"row text-center\">\n      <div class=\"large-12 columns\">\n        <h3 class=\"subheader\">\n          <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "REAL CPU", {"name":"tr","hash":{},"data":data})))
    + "</small>\n        </h3>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <div class=\"large-12 columns centered graph vm_cpu_graph text-center\" style=\"height: 100px;\">\n          <span  id=\"provision_dashboard_total\" style=\"font-size:80px\">\n            <i class=\"fa fa-spinner fa-spin\"></i>\n          </span>\n        </div>\n      </div>\n    </div>\n  </div>\n  <div class=\"medium-6 columns\">\n    <div class=\"row text-center\">\n      <h3 class=\"subheader\">\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "REAL MEMORY", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h3>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns centered graph vm_memory_graph text-center\" style=\"height: 100px;\">\n        <span  id=\"provision_dashboard_total\" style=\"font-size:80px\">\n          <i class=\"fa fa-spinner fa-spin\"></i>\n        </span>\n      </div>\n    </div>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/panels/capacity/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vms-tab/panels/capacity/panelId',['require'],function(require){
  return 'vm_capacity_tab';
});
define('tabs/vms-tab/panels/capacity',['require','utils/locale','sunstone-config','sunstone','opennebula/vm','utils/notifier','utils/graphs','hbs!./capacity/html','../tabId','./capacity/panelId','../dialogs/resize/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var Sunstone = require('sunstone');
  var OpenNebulaVM = require('opennebula/vm');
  var Notifier = require('utils/notifier');
  var Graphs = require('utils/graphs');

  /*
    TEMPLATES
   */

  var TemplateInfo = require('hbs!./capacity/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./capacity/panelId');
  var RESIZE_DIALOG_ID = require('../dialogs/resize/dialogId');
  var RESOURCE = "VM"
  var XML_ROOT = "VM"
  var RESIZE_STATES = [
    OpenNebulaVM.STATES.INIT,
    OpenNebulaVM.STATES.PENDING,
    OpenNebulaVM.STATES.HOLD,
    OpenNebulaVM.STATES.POWEROFF,
    OpenNebulaVM.STATES.UNDEPLOYED
  ];

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.panelId = PANEL_ID;
    this.title = Locale.tr("Capacity");
    this.icon = "fa-laptop";

    this.element = info[XML_ROOT];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;
  Panel.prototype.onShow = _onShow;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var resizeStateEnabled =
      (RESIZE_STATES.indexOf(parseInt(this.element.STATE)) > -1);

    return TemplateInfo({
      'element': this.element,
      'resizeStateEnabled': resizeStateEnabled
    });
  }

  function _setup(context) {
    var that = this;
    if (Config.isTabActionEnabled("vms-tab", "VM.resize")) {
      context.off('click', '#resize_capacity');
      context.on('click', '#resize_capacity', function() {
        var dialog = Sunstone.getDialog(RESIZE_DIALOG_ID);
        dialog.setElement(that.element);
        dialog.show();
        return false;
      });
    }
  }

  function _onShow(context) {
    var that = this;
    OpenNebulaVM.monitor({
      data: {
        id: that.element.ID,
        monitor: {
          monitor_resources : "MONITORING/CPU,MONITORING/MEMORY"
        }
      },
      success: function(req, response) {
        var vmGraphs = [
          {
            monitor_resources : "MONITORING/CPU",
            labels : Locale.tr("Real CPU"),
            humanize_figures : false,
            div_graph : $(".vm_cpu_graph")
          },
          {
            monitor_resources : "MONITORING/MEMORY",
            labels : Locale.tr("Real MEM"),
            humanize_figures : true,
            div_graph : $(".vm_memory_graph")
          }
        ];

        for (var i = 0; i < vmGraphs.length; i++) {
          Graphs.plot(response, vmGraphs[i]);
        }
      },
      error: Notifier.onError
    });
  }
});

define('utils/tree',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  /*
    CONSTRUCTOR
   */

  return {
    'html': _html,
    'setup': _setup
  };

  /*
    FUNCTION DEFINITIONS
   */

  /**
   * Returns a <ul> html tree
   * @param  {object} tree Recursive structure of elements with attributes
   *                       htmlStr: html to be inserted in this line,
   *                       subTree: array of tree elements. Must exist, can be empty
   * @return {string}      html string
   */
  function _html(tree){
    var html;

    if (tree.htmlStr && tree.htmlStr != ""){
      html = '<ul class="tree">'+_innerHtml(tree)+'</ul>';
    } else {
      html = '<ul class="tree">';
      $.each(tree.subTree, function(){
        html += _innerHtml(this);
      });
      html += '</ul>';
    }

    return html;
  }

  function _innerHtml(tree){
    var html = '<li>'+tree.htmlStr;

    html += '<ul>';
    $.each(tree.subTree, function(){
      html += _innerHtml(this);
    });
    html += '</ul>';

    return html;
  }

  function _setup(context){
  }
});

/* START_TEMPLATE */
define('hbs!tabs/vms-tab/panels/storage/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/isTabActionEnabled'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "              <span class=\"right\">\n                <button id=\"attach_disk\" class=\"button tiny success right radius\">\n                  "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Attach disk", {"name":"tr","hash":{},"data":data})))
    + "\n                </button>\n              </span>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, lambda=this.lambda, escapeExpression=this.escapeExpression, helperMissing=helpers.helperMissing, buffer = "<form id=\"tab_storage_form\" vmid=\""
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "\" >\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <table class=\"disks_table no-hover info_table dataTable extended_table\">\n        <thead>\n          <tr>\n            <th></th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Target", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Image / Format-Size", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Size", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Persistent", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Actions", {"name":"tr","hash":{},"data":data})))
    + "\n";
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "vms-tab", "VM.attachdisk", {"name":"isTabActionEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "            </th>\n          </tr>\n        </thead>\n        <tbody>\n        </tbody>\n      </table>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/panels/storage/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/vms-tab/panels/storage/disk-details',['hbs','hbs/handlebars','templates/helpers/isTabActionEnabled','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "      <button class=\"disk_snapshot_saveas button secondary small radius\" disabled=\"disabled\">\n        <span class=\"fa fa-save fa-fw\"></span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Save as", {"name":"tr","hash":{},"data":data})))
    + "\n      </button>\n";
},"3":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "      <button class=\"disk_snapshot_revert button secondary small radius\" disabled=\"disabled\">\n        <span class=\"fa fa-reply fa-fw\"></span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Revert", {"name":"tr","hash":{},"data":data})))
    + "\n      </button>\n";
},"5":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "      <button class=\"disk_snapshot_delete button secondary small radius\" disabled=\"disabled\">\n        <span class=\"fa fa-times fa-fw\"></span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Delete", {"name":"tr","hash":{},"data":data})))
    + "\n      </button>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"snapshots large-12 columns\" disk_id=\""
    + escapeExpression(((helper = (helper = helpers.diskId || (depth0 != null ? depth0.diskId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"diskId","hash":{},"data":data}) : helper)))
    + "\" style=\"padding-left: 30px;\">\n  <div class=\"row collapse\">\n    <div class=\"large-12 columns\" style=\"padding-bottom: 1em;\">\n";
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "vms-tab", "VM.disk_saveas", {"name":"isTabActionEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "vms-tab", "VM.disk_snapshot_revert", {"name":"isTabActionEnabled","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "vms-tab", "VM.disk_snapshot_delete", {"name":"isTabActionEnabled","hash":{},"fn":this.program(5, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "    </div>\n  </div>\n  <div class=\"row collapse\">\n    <div class=\"large-12 columns\" style=\"width:800px; overflow-x:auto\">\n      ";
  stack1 = ((helper = (helper = helpers.snapshotsHTML || (depth0 != null ? depth0.snapshotsHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"snapshotsHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </div>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vms-tab/panels/storage/disk-details', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vms-tab/panels/storage/panelId',['require'],function(require){
  return 'vm_hotplugging_tab';
});
define('utils/dialogs/generic-confirm/dialogId',['require'],function(require){
  return 'genericConfirmDialog';
});
define('tabs/vms-tab/panels/storage',['require','utils/locale','sunstone-config','sunstone','utils/humanize','../utils/state-actions','opennebula/vm','utils/tree','hbs!./storage/html','hbs!./storage/disk-details','../tabId','./storage/panelId','../dialogs/attach-disk/dialogId','../dialogs/disk-snapshot/dialogId','../dialogs/disk-saveas/dialogId','utils/dialogs/generic-confirm/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var Sunstone = require('sunstone');
  var Humanize = require('utils/humanize');
  var StateActions = require('../utils/state-actions');
  var OpenNebulaVM = require('opennebula/vm');
  var Tree = require('utils/tree');
  var TemplateHtml = require('hbs!./storage/html');
  var DiskDetailsHtml = require('hbs!./storage/disk-details');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./storage/panelId');
  var ATTACH_DISK_DIALOG_ID = require('../dialogs/attach-disk/dialogId');
  var DISK_SNAPSHOT_DIALOG_ID = require('../dialogs/disk-snapshot/dialogId');
  var DISK_SAVEAS_DIALOG_ID = require('../dialogs/disk-saveas/dialogId');
  var CONFIRM_DIALOG_ID = require('utils/dialogs/generic-confirm/dialogId');
  var RESOURCE = "VM"
  var XML_ROOT = "VM"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.panelId = PANEL_ID;
    this.title = Locale.tr("Storage");
    this.icon = "fa-tasks";

    this.element = info[XML_ROOT];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;
  Panel.prototype.getState = _getState;
  Panel.prototype.setState = _setState;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHtml({element: this.element});
  }

  function _setup(context) {
    var that = this;

    var snapshots = [];
    if ($.isArray(this.element.SNAPSHOTS)){
      snapshots = this.element.SNAPSHOTS;
    } else if (!$.isEmptyObject(this.element.SNAPSHOTS)) {
      snapshots = [this.element.SNAPSHOTS];
    }

    var snapshotsSize = {};
    var monitoringSnapshots = [];
    if ($.isArray(that.element.MONITORING.SNAPSHOT_SIZE))
      monitoringSnapshots = that.element.MONITORING.SNAPSHOT_SIZE;
    else if (!$.isEmptyObject(that.element.MONITORING.SNAPSHOT_SIZE))
      monitoringSnapshots = [that.element.MONITORING.SNAPSHOT_SIZE];

    $.each(monitoringSnapshots, function(index, monitoringSnap){
      if(snapshotsSize[monitoringSnap.DISK_ID] == undefined){
        snapshotsSize[monitoringSnap.DISK_ID] = {};
      }

      snapshotsSize[monitoringSnap.DISK_ID][monitoringSnap.ID] = monitoringSnap.SIZE;
    })

    var snapshotsHtml = {};

    $.each(snapshots, function(){
      var diskId = this.DISK_ID;
      var diskSnapshots = this.SNAPSHOT;

      if (!$.isArray(diskSnapshots)){
        diskSnapshots = [diskSnapshots];
      }

      var indexedSize = snapshotsSize[diskId];

      if(indexedSize == undefined){
        indexedSize = {};
      }

      var treeRoot = {
        htmlStr : '',
        subTree : []
      };

      var indexedSnapshots = {};
      var noParent = [];

      $.each(diskSnapshots, function(){
        indexedSnapshots[this.ID] = this;

        if(this.PARENT == "-1"){
          noParent.push(this.ID);
        }
      });

      $.each(noParent, function(){
        treeRoot.subTree.push(
          _makeTree(that, indexedSnapshots[this], indexedSnapshots, indexedSize)
        );
      });

      snapshotsHtml[diskId] = Tree.html(treeRoot);
    });

    var disks = [];
    if ($.isArray(that.element.TEMPLATE.DISK))
      disks = that.element.TEMPLATE.DISK;
    else if (!$.isEmptyObject(that.element.TEMPLATE.DISK))
      disks = [that.element.TEMPLATE.DISK];

    if (!$.isEmptyObject(that.element.TEMPLATE.CONTEXT)) {
      var context_disk = that.element.TEMPLATE.CONTEXT;

      context_disk["IMAGE"] = Locale.tr("Context");
      context_disk["CONTEXT"] = true;

      disks.push(context_disk);
    }

    var disksSize = {};
    var monitoringDisks = [];
    if ($.isArray(that.element.MONITORING.DISK_SIZE))
      monitoringDisks = that.element.MONITORING.DISK_SIZE;
    else if (!$.isEmptyObject(that.element.MONITORING.DISK_SIZE))
      monitoringDisks = [that.element.MONITORING.DISK_SIZE];

    $.each(monitoringDisks, function(index, monitoringDisk){
      disksSize[monitoringDisk.ID] = monitoringDisk.SIZE;
    })

    var disk_dt_data = [];
    if (disks.length) {

      for (var i = 0; i < disks.length; i++) {
        var disk = disks[i];

        // Save as
        if (
           (
            that.element.STATE == OpenNebulaVM.STATES.ACTIVE) &&
           (
            that.element.LCM_STATE == OpenNebulaVM.LCM_STATES.HOTPLUG_SAVEAS ||
            that.element.LCM_STATE == OpenNebulaVM.LCM_STATES.HOTPLUG_SAVEAS_POWEROFF ||
            that.element.LCM_STATE == OpenNebulaVM.LCM_STATES.HOTPLUG_SAVEAS_SUSPENDED) &&
           (
            disk.HOTPLUG_SAVE_AS_ACTIVE == "YES")
           ) {
          actions = Locale.tr('Save as in progress');
        }
        // Attach / Detach
        else if (
           (
            that.element.STATE == OpenNebulaVM.STATES.ACTIVE) &&
           (
            that.element.LCM_STATE == OpenNebulaVM.LCM_STATES.HOTPLUG) &&
           (
            disk.ATTACH = "YES")
           ) {
          actions = Locale.tr('attach/detach in progress');
        } else {
          actions = '';


          if (Config.isTabActionEnabled("vms-tab", "VM.disk_saveas")) {
            // Check if it's volatile
            if (disk.IMAGE_ID &&
                 StateActions.enabledStateAction("VM.disk_saveas", that.element.STATE, that.element.LCM_STATE)) {
              actions += '<a href="VM.disk_saveas" class="disk_saveas nowrap" >\
              <i class="fa fa-save fa-fw"></i>' + Locale.tr("Save as") + '</a> &emsp;';
            }
          }


          if (Config.isTabActionEnabled("vms-tab", "VM.detachdisk")) {
            if (StateActions.enabledStateAction("VM.detachdisk", that.element.STATE, that.element.LCM_STATE) && !disk.CONTEXT) {
              actions += ('<a href="VM.detachdisk" class="detachdisk nowrap" >\
              <i class="fa fa-times fa-fw"></i>' + Locale.tr("Detach") +
              '</a> &emsp;');
            }
          }

          if (Config.isTabActionEnabled("vms-tab", "VM.disk_snapshot_create")) {
            if (StateActions.enabledStateAction("VM.disk_snapshot_create", that.element.STATE, that.element.LCM_STATE) && disk.IMAGE_ID) {
              actions += ('<a href="VM.disk_snapshot_create" class="disk_snapshot_create nowrap" >\
              <i class="fa fa-camera fa-fw"></i>' + Locale.tr("Snapshot") +
              '</a>');
            }
          }
        }

        var sizeStr = "";
        if (disksSize[disk.DISK_ID]) {
          sizeStr += Humanize.sizeFromMB(disksSize[disk.DISK_ID]);
        } else {
          sizeStr += '-';
        }

        sizeStr += '/';
        if (disk.SIZE) {
          sizeStr += Humanize.sizeFromMB(disk.SIZE);
        } else {
          sizeStr += '-';
        }

        disk_dt_data.push({
          DISK_ID : disk.DISK_ID,
          TARGET : disk.TARGET,
          IMAGE : (disk.IMAGE ? disk.IMAGE : (Humanize.sizeFromMB(disk.SIZE) + (disk.FORMAT ? (' - ' + disk.FORMAT) : ''))),
          SIZE: sizeStr,
          SAVE : ((disk.SAVE && disk.SAVE == 'YES') ? Locale.tr('YES') : Locale.tr('NO')),
          ACTIONS : actions,
          SNAPSHOTS : snapshotsHtml[disk.DISK_ID]
        });
      }
    }

    var disks_table = $("#tab_storage_form .disks_table", context).DataTable({
      "bDeferRender": true,
      "data": disk_dt_data,
      "columns": [
        {
          "class":          'open-control',
          "orderable":      false,
          "data":           null,
          "defaultContent": '<span class="fa fa-fw fa-chevron-down"></span>'
        },
        {"data": "DISK_ID",   "defaultContent": ""},
        {"data": "TARGET",    "defaultContent": ""},
        {"data": "IMAGE",     "defaultContent": "", "orderable": false},
        {"data": "SIZE",      "defaultContent": ""},
        {"data": "SAVE",      "defaultContent": "", "orderable": false},
        {"data": "ACTIONS",   "defaultContent": "", "orderable": false}
      ],

      "fnRowCallback": function(nRow, aData, iDisplayIndex, iDisplayIndexFull) {

        if (aData.SNAPSHOTS == undefined ||
            aData.SNAPSHOTS.length == 0) {

          $("td.open-control", nRow).html("").removeClass('open-control');
        }

        $(nRow).attr("disk_id", aData.DISK_ID);
      }
    });

    $("#tab_storage_form .disks_table", context).dataTable().fnSort([[1, 'asc']]);

    // Add event listener for opening and closing each row details
    context.off('click', '#tab_storage_form .disks_table td.open-control')
    context.on('click', '#tab_storage_form .disks_table td.open-control', function () {
      var row = $(this).closest('table').DataTable().row($(this).closest('tr'));

      if (row.child.isShown()) {
        row.child.hide();
        $(this).children("span").addClass('fa-chevron-down');
        $(this).children("span").removeClass('fa-chevron-up');
      } else {
        var html = DiskDetailsHtml({
          diskId: row.data().DISK_ID,
          snapshotsHTML: row.data().SNAPSHOTS
        });

        row.child(html).show();
        $(this).children("span").removeClass('fa-chevron-down');
        $(this).children("span").addClass('fa-chevron-up');
      }
    });

    if (Config.isTabActionEnabled("vms-tab", "VM.disk_saveas")) {
      context.off('click', '.disk_saveas');
      context.on('click', '.disk_saveas', function() {
        var disk_id = $(this).parents('tr').attr('disk_id');

        var dialog = Sunstone.getDialog(DISK_SAVEAS_DIALOG_ID);
        dialog.setParams(
          { element: that.element,
            diskId: disk_id
          });

        dialog.reset();
        dialog.show();
        return false;
      });
    }

    if (Config.isTabActionEnabled("vms-tab", "VM.attachdisk")) {
      if (!StateActions.enabledStateAction("VM.attachdisk", that.element.STATE, that.element.LCM_STATE)){
        $('#attach_disk', context).attr("disabled", "disabled");
      }

      context.off('click', '#attach_disk');
      context.on('click', '#attach_disk', function() {
        var dialog = Sunstone.getDialog(ATTACH_DISK_DIALOG_ID);
        dialog.setElement(that.element);
        dialog.show();
        return false;
      });
    }

    if (Config.isTabActionEnabled("vms-tab", "VM.detachdisk")) {
      context.off('click', '.detachdisk');
      context.on('click', '.detachdisk', function() {
        var disk_id = $(this).parents('tr').attr('disk_id');

        Sunstone.getDialog(CONFIRM_DIALOG_ID).setParams({
          //header :
          body : Locale.tr("This will detach the disk inmediately"),
          //question :
          submit : function(){
            Sunstone.runAction('VM.detachdisk', that.element.ID, disk_id);
            return false;
          }
        });

        Sunstone.getDialog(CONFIRM_DIALOG_ID).reset();
        Sunstone.getDialog(CONFIRM_DIALOG_ID).show();

        return false;
      });
    }

    if (Config.isTabActionEnabled("vms-tab", "VM.disk_snapshot_create")) {
      context.off('click', '.disk_snapshot_create');
      context.on('click', '.disk_snapshot_create', function() {
        var disk_id = $(this).parents('tr').attr('disk_id');

        var dialog = Sunstone.getDialog(DISK_SNAPSHOT_DIALOG_ID);
        dialog.setParams(
          { element: that.element,
            diskId: disk_id
          });

        dialog.reset();
        dialog.show();
        return false;
      });
    }

    context.off("change", ".snapshot_check_item");
    context.on("change", ".snapshot_check_item", function() {
      var snapshotsSection = $(this).closest('.snapshots');

      // Unselect other check inputs
      var checked = $(this).is(':checked');
      $('.snapshot_check_item:checked', snapshotsSection).prop('checked', false);
      $(this).prop('checked', checked);

      // Enable/disable buttons
      if ($(this).is(":checked")) {
        $(".disk_snapshot_saveas", snapshotsSection).prop('disabled', false);
        $(".disk_snapshot_revert", snapshotsSection).prop('disabled', false);
        $(".disk_snapshot_delete", snapshotsSection).prop('disabled', false);
      } else {
        $(".disk_snapshot_saveas", snapshotsSection).prop('disabled', true);
        $(".disk_snapshot_revert", snapshotsSection).prop('disabled', true);
        $(".disk_snapshot_delete", snapshotsSection).prop('disabled', true);
      }
    });

    if (Config.isTabActionEnabled("vms-tab", "VM.disk_saveas")) {
      context.off('click', '.disk_snapshot_saveas');
      context.on('click', '.disk_snapshot_saveas', function() {
        var snapshotsSection = $(this).closest('.snapshots');

        var disk_id = snapshotsSection.attr('disk_id');
        var snapshot_id = $(".snapshot_check_item:checked", snapshotsSection).attr('snapshot_id');

        var dialog = Sunstone.getDialog(DISK_SAVEAS_DIALOG_ID);
        dialog.setParams(
          { element: that.element,
            diskId: disk_id,
            snapshotId: snapshot_id
          });

        dialog.reset();
        dialog.show();
        return false;
      });
    }

    if (Config.isTabActionEnabled("vms-tab", "VM.disk_snapshot_revert")) {
      context.off('click', '.disk_snapshot_revert');
      context.on('click', '.disk_snapshot_revert', function() {
        var snapshotsSection = $(this).closest('.snapshots');

        var disk_id = snapshotsSection.attr('disk_id');
        var snapshot_id = $(".snapshot_check_item:checked", snapshotsSection).attr('snapshot_id');

        Sunstone.runAction(
          'VM.disk_snapshot_revert',
          that.element.ID,
          { "disk_id": disk_id,
            "snapshot_id": snapshot_id
          });

        return false;
      });
    }

    if (Config.isTabActionEnabled("vms-tab", "VM.disk_snapshot_delete")) {
      context.off('click', '.disk_snapshot_delete');
      context.on('click', '.disk_snapshot_delete', function() {
        var snapshotsSection = $(this).closest('.snapshots');

        var disk_id = snapshotsSection.attr('disk_id');
        var snapshot_id = $(".snapshot_check_item:checked", snapshotsSection).attr('snapshot_id');

        Sunstone.getDialog(CONFIRM_DIALOG_ID).setParams({
          //header :
          body : Locale.tr("This will delete the disk snapshot "+snapshot_id),
          //question :
          submit : function(){
            Sunstone.runAction(
              'VM.disk_snapshot_delete',
              that.element.ID,
              { "disk_id": disk_id,
                "snapshot_id": snapshot_id
              });
          }
        });

        Sunstone.getDialog(CONFIRM_DIALOG_ID).reset();
        Sunstone.getDialog(CONFIRM_DIALOG_ID).show();

        return false;
      });
    }

    Tree.setup(context);
  }

  function _getState(context) {
    var state = {
      openDisksDetails : [],
      checkedSnapshots : []
    };

    $.each($("#tab_storage_form .disks_table .fa-chevron-up", context), function(){
      state.openDisksDetails.push($(this).closest("tr").attr("disk_id"));
    });

    $.each($('#tab_storage_form .disks_table .snapshot_check_item:checked', context), function(){
      state.checkedSnapshots.push({
        snapshot_id : $(this).attr("snapshot_id"),
        disk_id : $(this).closest(".snapshots").attr('disk_id')
      });
    });

    return state;
  }

  function _setState(state, context) {
    var that = this;

    $.each(state["openDisksDetails"], function(){
      $('#tab_storage_form .disks_table tr[disk_id="'+this+'"] td.open-control', context).click();
    });

    $.each(state["checkedSnapshots"], function(){
      $('#tab_storage_form .disks_table .snapshots[disk_id="'+this.disk_id+'"] '+
        '.snapshot_check_item[snapshot_id="'+this.snapshot_id+'"]', context).click();
    });
  }

  function _makeTree(that, snapshot, indexedSnapshots, indexedSize){
    var SPACE = '&nbsp;&nbsp;&nbsp;&nbsp;';

    var subTree = [];

    if (snapshot.CHILDREN){
      $.each(snapshot.CHILDREN.split(","), function(){
        subTree.push(
          _makeTree(that, indexedSnapshots[this], indexedSnapshots, indexedSize)
        );
      });
    }

    var html = '<div class="snapshot_row nowrap">'+
      '<input class="snapshot_check_item" type="checkbox" snapshot_id="'+snapshot.ID+'"/>'+
      SPACE + snapshot.ID + SPACE;

    var active = (snapshot.ACTIVE == "YES");

    if(active){
      html += '<i class="fa fa-play-circle-o fa-lg" title="'+
                Locale.tr("Active")+'"/>' + SPACE;
    }

    var sizeStr = "";
    if (indexedSize[snapshot.ID]) {
      sizeStr += Humanize.sizeFromMB(indexedSize[snapshot.ID]);
    } else {
      sizeStr += '-';
    }

    sizeStr += '/';
    if (snapshot.SIZE) {
      sizeStr += Humanize.sizeFromMB(snapshot.SIZE);
    } else {
      sizeStr += '-';
    }

    html += Humanize.prettyTime(snapshot.DATE) + SPACE + sizeStr + SPACE +
            (snapshot.NAME ? snapshot.NAME + SPACE : '');

    html += '</div>';

    return {
      htmlStr : html,
      subTree : subTree
    };
  }
});

define('tabs/secgroups-tab/utils/common',['require','utils/locale'],function(require) {
  /*
    Common functions for Security Groups
   */

  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');

  /*
  @param {Object} rule Object representing the rule as returned by OpenNebula
  Returns an object with the human readable attributes of the rule. List of attributes:
    PROTOCOL
    RULE_TYPE
    ICMP_TYPE
    RANGE
    NETWORK
  */
  function _sgRuleToSt(rule){
    var text = {};

    if(rule.PROTOCOL != undefined){
      switch(rule.PROTOCOL.toUpperCase()){
      case "TCP":
        text["PROTOCOL"] = Locale.tr("TCP");
        break;
      case "UDP":
        text["PROTOCOL"] = Locale.tr("UDP");
        break;
      case "ICMP":
        text["PROTOCOL"] = Locale.tr("ICMP");
        break;
      case "IPSEC":
        text["PROTOCOL"] = Locale.tr("IPsec");
        break;
      case "ALL":
        text["PROTOCOL"] = Locale.tr("All");
        break;
      default:
        text["PROTOCOL"] = "";
      }
    } else {
      text["PROTOCOL"] = "";
    }

    if(rule.RULE_TYPE != undefined){
      switch(rule.RULE_TYPE.toUpperCase()){
      case "OUTBOUND":
        text["RULE_TYPE"] = Locale.tr("Outbound");
        break;
      case "INBOUND":
        text["RULE_TYPE"] = Locale.tr("Inbound");
        break;
      default:
        text["RULE_TYPE"] = "";
      }
    } else {
      text["RULE_TYPE"] = "";
    }

    if(rule.ICMP_TYPE != undefined){
      text["ICMP_TYPE"] = _icmpToSt(rule.ICMP_TYPE);
    } else {
      text["ICMP_TYPE"] = "";
    }

    if(rule.RANGE != undefined && rule.RANGE != ""){
      text["RANGE"] = rule.RANGE;
    } else {
      text["RANGE"] = Locale.tr("All");
    }

    var network = "";

    if(rule.NETWORK_ID != undefined && rule.NETWORK_ID != ""){
      network += (Locale.tr("Virtual Network") + " " + rule.NETWORK_ID);
    }

    if(rule.SIZE != undefined && rule.SIZE != ""){
      if(network != ""){
        network += ":<br>";
      }

      if(rule.IP != undefined && rule.IP != ""){
        network += Locale.tr("Start") + ": " + rule.IP + ", ";
      } else if(rule.MAC != undefined && rule.MAC != ""){
        network += Locale.tr("Start") + ": " + rule.MAC + ", ";
      }

      network += Locale.tr("Size") + ": " + rule.SIZE;
    }

    if(network == ""){
      network = Locale.tr("Any");
    }

    text["NETWORK"] = network;

    return text;
  }

  function _icmpToSt(icmp_type){
    switch( icmp_type ){
      case "":    return Locale.tr("All");
      case "0":   return "0: Echo Reply";
      case "3":   return "3: Destination Unreachable";
      case "4":   return "4: Source Quench";
      case "5":   return "5: Redirect";
      case "6":   return "6: Alternate Host Address";
      case "8":   return "8: Echo";
      case "9":   return "9: Router Advertisement";
      case "10":  return "10: Router Solicitation";
      case "11":  return "11: Time Exceeded";
      case "12":  return "12: Parameter Problem";
      case "13":  return "13: Timestamp";
      case "14":  return "14: Timestamp Reply";
      case "15":  return "15: Information Request";
      case "16":  return "16: Information Reply";
      case "17":  return "17: Address Mask Request";
      case "18":  return "18: Address Mask Reply";
      case "30":  return "30: Traceroute";
      case "31":  return "31: Datagram Conversion Error";
      case "32":  return "32: Mobile Host Redirect";
      case "33":  return "33: IPv6 Where-Are-You";
      case "34":  return "34: IPv6 I-Am-Here";
      case "35":  return "35: Mobile Registration Request";
      case "36":  return "36: Mobile Registration Reply";
      case "37":  return "37: Domain Name Request";
      case "38":  return "38: Domain Name Reply";
      case "39":  return "39: SKIP";
      case "40":  return "40: Photuris";
      case "41":  return "41: ICMP messages utilized by experimental mobility protocols such as Seamoby";
      case "253": return "253: RFC3692-style Experiment 1";
      case "254": return "254: RFC3692-style Experiment 2";
      default:  return "" + icmp_type;
    }
  }

  /*
    @param {Object} info Object representing the Security Group as returned by OpenNebula
   */
  function _getRules(info){
    var rules = info.TEMPLATE.RULE;

    if (!rules){ //empty
      rules = [];
    } else if (rules.constructor != Array) { //>1 rule
      rules = [rules];
    }

    return rules;
  }

  return {
    'sgRuleToSt': _sgRuleToSt,
    'getRules': _getRules
  };
});

define('tabs/vms-tab/panels/network/panelId',['require'],function(require) {
  return 'vm_network_tab';
});

define('tabs/vms-tab/panels/network',['require','utils/locale','sunstone-config','sunstone','utils/humanize','utils/notifier','utils/graphs','../utils/state-actions','opennebula/vm','tabs/secgroups-tab/utils/common','../tabId','./network/panelId','../dialogs/attach-nic/dialogId','utils/dialogs/generic-confirm/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var Sunstone = require('sunstone');
  var Humanize = require('utils/humanize');
  var Notifier = require('utils/notifier');
  var Graphs = require('utils/graphs');
  var StateActions = require('../utils/state-actions');
  var OpenNebulaVM = require('opennebula/vm');
  var SecGroupsCommon = require('tabs/secgroups-tab/utils/common');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./network/panelId');
  var ATTACH_NIC_DIALOG_ID = require('../dialogs/attach-nic/dialogId');
  var CONFIRM_DIALOG_ID = require('utils/dialogs/generic-confirm/dialogId');
  var RESOURCE = "VM"
  var XML_ROOT = "VM"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.panelId = PANEL_ID;
    this.title = Locale.tr("Network");
    this.icon = "fa-globe";

    this.element = info[XML_ROOT];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;
  Panel.prototype.onShow = _onShow;
  Panel.prototype.getState = _getState;
  Panel.prototype.setState = _setState;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var that = this;
    var html = '<form id="tab_network_form" vmid="' + that.element.ID + '" >\
        <div class="row">\
        <div class="large-12 columns">\
           <table class="nics_table no-hover info_table dataTable extended_table">\
             <thead>\
               <tr>\
                  <th></th>\
                  <th>' + Locale.tr("ID") + '</th>\
                  <th>' + Locale.tr("Network") + '</th>\
                  <th>' + Locale.tr("IP") + '</th>\
                  <th>' + Locale.tr("MAC") + '</th>\
                  <th>' + Locale.tr("IPv6 ULA") + '</th>\
                  <th>' + Locale.tr("IPv6 Global") + '</th>\
                  <th colspan="">' + Locale.tr("Actions") + '</th>\
                  <th>'                 ;

    if (Config.isTabActionEnabled("vms-tab", "VM.attachnic")) {
      if (StateActions.enabledStateAction("VM.attachnic",
            that.element.STATE,
            that.element.LCM_STATE) &&
          OpenNebulaVM.isNICAttachSupported(that.element)) {
        html += '\
             <button id="attach_nic" class="button tiny success right radius" >' + Locale.tr("Attach nic") + '</button>'
      } else {
        html += '\
             <button id="attach_nic" class="button tiny success right radius" disabled="disabled">' + Locale.tr("Attach nic") + '</button>'
      }
    }

    html += '</th>\
                </tr>\
             </thead>\
             <tbody>\
             </tbody>\
            </table>\
          </div>\
        </div>'      ;

    var externalNetworkAttrs = OpenNebulaVM.retrieveExternalNetworkAttrs(that.element);
    if (!$.isEmptyObject(externalNetworkAttrs)) {
      html += '<div class="row">' +
        '<div class="large-12 columns">' +
         '<table class="dataTable extended_table">' +
            '<thead>' +
              '<tr>' +
                 '<th colspan=2>' + Locale.tr("Network Monitoring Attributes") + '</th>' +
              '</tr>' +
            '</thead>' +
            '<tbody>';

      $.each(externalNetworkAttrs, function(key, value) {
        html += '<tr>' +
           '<td>' + key + '</td>' +
           '<td>' + value + '</td>' +
          '</tr>';
      });

      html += '</tbody>' +
            '</table>' +
          '</div>' +
        '</div>';
    }

    // Do not show statistics for not hypervisors that do not gather net data
    if (OpenNebulaVM.isNICGraphsSupported(that.element)) {
      html += '\
          <div class="row">\
              <div class="medium-6 columns">\
                <div class="row text-center">\
                  <h3 class="subheader"><small>' + Locale.tr("NET RX") + '</small></h3>\
                </div>\
                <div class="row">\
                  <div class="large-12 columns centered graph text-center" id="vm_net_rx_graph" style="height: 100px;">\
                    <span  id="provision_dashboard_total" style="font-size:80px">\
                      <i class="fa fa-spinner fa-spin"></i>\
                    </span>\
                  </div>\
                </div>\
                <div class="row graph_legend">\
                  <div class="large-12 columns centered" id="vm_net_rx_legend">\
                  </div>\
                </div>\
              </div>\
              <div class="medium-6 columns">\
                <div class="row text-center">\
                  <h3 class="subheader"><small>' + Locale.tr("NET TX") + '</small></h3>\
                </div>\
                <div class="row">\
                  <div class="large-12 columns centered graph text-center" id="vm_net_tx_graph" style="height: 100px;">\
                    <span  id="provision_dashboard_total" style="font-size:80px">\
                      <i class="fa fa-spinner fa-spin"></i>\
                    </span>\
                  </div>\
                </div>\
                <div class="row graph_legend">\
                  <div class="large-12 columns centered" id="vm_net_tx_legend">\
                  </div>\
                </div>\
              </div>\
              <div class="medium-6 columns">\
                <div class="row text-center">\
                  <h3 class="subheader"><small>' + Locale.tr("NET DOWNLOAD SPEED") + '</small></h3>\
                </div>\
                <div class="row">\
                  <div class="large-12 columns centered graph text-center" id="vm_net_rx_speed_graph" style="height: 100px;">\
                    <span  id="provision_dashboard_total" style="font-size:80px">\
                      <i class="fa fa-spinner fa-spin"></i>\
                    </span>\
                  </div>\
                </div>\
                <div class="row graph_legend">\
                  <div class="large-12 columns centered" id="vm_net_rx_speed_legend">\
                  </div>\
                </div>\
              </div>\
              <div class="medium-6 columns">\
                <div class="row text-center">\
                  <h3 class="subheader"><small>' + Locale.tr("NET UPLOAD SPEED") + '</small></h3>\
                </div>\
                <div class="row">\
                  <div class="large-12 columns centered graph text-center" id="vm_net_tx_speed_graph" style="height: 100px;">\
                    <span  id="provision_dashboard_total" style="font-size:80px">\
                      <i class="fa fa-spinner fa-spin"></i>\
                    </span>\
                  </div>\
                </div>\
                <div class="row graph_legend">\
                  <div class="large-12 columns centered" id="vm_net_tx_speed_legend">\
                  </div>\
                </div>\
              </div>\
          </div>\
        </form>';
    }

    return html;
  }

  function _setup(context) {
    var that = this;

    var nics = []

    if ($.isArray(that.element.TEMPLATE.NIC))
        nics = that.element.TEMPLATE.NIC
    else if (!$.isEmptyObject(that.element.TEMPLATE.NIC))
        nics = [that.element.TEMPLATE.NIC]

    var nic_dt_data = [];
    if (nics.length) {
      var nic_dt_data = [];

      for (var i = 0; i < nics.length; i++) {
        var nic = nics[i];

        var actions;
        // Attach / Detach
        if (
           (
            that.element.STATE == OpenNebulaVM.STATES.ACTIVE) &&
           (
            that.element.LCM_STATE == OpenNebulaVM.LCM_STATES.HOTPLUG_NIC) &&
           (
            nic.ATTACH == "YES")
           ) {
          actions = Locale.tr("attach/detach in progress")
        } else {
          actions = '';

          if (Config.isTabActionEnabled("vms-tab", "VM.detachnic")) {
            if (StateActions.enabledStateAction("VM.detachnic", that.element.STATE, that.element.LCM_STATE)) {
              actions += '<a href="VM.detachnic" class="detachnic" ><i class="fa fa-times"/>' + Locale.tr("Detach") + '</a>'
            }
          }
        }

        var secgroups = [];

        var nic_secgroups = {};
        if (!$.isEmptyObject(nic.SECURITY_GROUPS)) {
          $.each(nic.SECURITY_GROUPS.split(","), function() {
            nic_secgroups[this] = true;
          });
        }

        if (that.element.TEMPLATE.SECURITY_GROUP_RULE != undefined) {
          $.each(that.element.TEMPLATE.SECURITY_GROUP_RULE, function() {
            if (nic_secgroups[this.SECURITY_GROUP_ID]) {
              secgroups.push(this);
            }
          });
        }

        nic_dt_data.push({
          NIC_ID : nic.NIC_ID,
          NETWORK : nic.NETWORK,
          IP : (nic.IP ? nic.IP : "--"),
          MAC : nic.MAC,
          IP6_ULA : (nic.IP6_ULA ? nic.IP6_ULA : "--"),
          IP6_GLOBAL : (nic.IP6_GLOBAL ? nic.IP6_GLOBAL : "--"),
          ACTIONS : actions,
          SECURITY_GROUP_RULES : secgroups
        });
      }
    }

    var nics_table = $("#tab_network_form .nics_table", context).DataTable({
      "bDeferRender": true,
      "data": nic_dt_data,
      "columns": [
        {
          "class":          'open-control',
          "orderable":      false,
          "data":           null,
          "defaultContent": '<span class="fa fa-fw fa-chevron-down"></span>'
        },
        {"data": "NIC_ID",     "defaultContent": ""},
        {"data": "NETWORK",    "defaultContent": ""},
        {"data": "IP",         "defaultContent": ""},
        {"data": "MAC",        "defaultContent": ""},
        {"data": "IP6_ULA",    "defaultContent": ""},
        {"data": "IP6_GLOBAL", "defaultContent": ""},
        {"data": "ACTIONS",    "defaultContent": "", "orderable": false},
        {"defaultContent": "", "orderable": false}
      ],

      "fnRowCallback": function(nRow, aData, iDisplayIndex, iDisplayIndexFull) {

        if (aData.SECURITY_GROUP_RULES == undefined ||
            aData.SECURITY_GROUP_RULES.length == 0) {

          $("td.open-control", nRow).html("").removeClass('open-control');
        }

        $(nRow).attr("nic_id", aData.NIC_ID);
      }
    });

    $("#tab_network_form .nics_table", context).dataTable().fnSort([[1, 'asc']]);

    // Add event listener for opening and closing each NIC row details
    context.off('click', '#tab_network_form .nics_table td.open-control')
    context.on('click', '#tab_network_form .nics_table td.open-control', function () {
      var row = $(this).closest('table').DataTable().row($(this).closest('tr'));

      if (row.child.isShown()) {
        row.child.hide();
        $(this).children("span").addClass('fa-chevron-down');
        $(this).children("span").removeClass('fa-chevron-up');
      } else {
        var html = '<div style="padding-left: 30px;">\
              <table class="extended_table dataTable">\
                <thead>\
                  <tr>\
                    <th colspan="2">' + Locale.tr("Security Group") + '</th>\
                    <th>' + Locale.tr("Protocol") + '</th>\
                    <th>' + Locale.tr("Type") + '</th>\
                    <th>' + Locale.tr("Range") + '</th>\
                    <th>' + Locale.tr("Network") + '</th>\
                    <th>' + Locale.tr("ICMP Type") + '</th>\
                  </tr>\
                <thead>\
                <tbody>'            ;

        $.each(row.data().SECURITY_GROUP_RULES, function(index, elem) {
          var rule_st = SecGroupsCommon.sgRuleToSt(this);

          var new_tr = '<tr>\
                  <td>' + this.SECURITY_GROUP_ID + '</td>\
                  <td>' + this.SECURITY_GROUP_NAME + '</td>\
                  <td>' + rule_st.PROTOCOL + '</td>\
                  <td>' + rule_st.RULE_TYPE + '</td>\
                  <td>' + rule_st.RANGE + '</td>\
                  <td>' + rule_st.NETWORK + '</td>\
                  <td>' + rule_st.ICMP_TYPE + '</td>\
                </tr>'

          html += new_tr;
        });

        row.child(html).show();
        $(this).children("span").removeClass('fa-chevron-down');
        $(this).children("span").addClass('fa-chevron-up');
      }
    });

    if (Config.isTabActionEnabled("vms-tab", "VM.attachnic")) {
      context.off('click', '#attach_nic');
      context.on('click', '#attach_nic', function() {
        var dialog = Sunstone.getDialog(ATTACH_NIC_DIALOG_ID);
        dialog.setElement(that.element);
        dialog.show();
        return false;
      });
    }

    if (Config.isTabActionEnabled("vms-tab", "VM.detachnic")) {
      context.off('click', '.detachnic');
      context.on('click', '.detachnic', function() {
        var nic_id = $(this).parents('tr').attr('nic_id');

        Sunstone.getDialog(CONFIRM_DIALOG_ID).setParams({
          //header :
          body : Locale.tr("This will detach the nic inmediately"),
          //question :
          submit : function(){
            Sunstone.runAction('VM.detachnic', that.element.ID, nic_id);
            return false;
          }
        });

        Sunstone.getDialog(CONFIRM_DIALOG_ID).reset();
        Sunstone.getDialog(CONFIRM_DIALOG_ID).show();

        return false;
      });
    }
  }

  function _onShow(context) {
    var that = this;
    if (OpenNebulaVM.isNICGraphsSupported(that.element)) {
      OpenNebulaVM.monitor({
        data: {
          id: that.element.ID,
          monitor: {
            monitor_resources : "MONITORING/NETTX,MONITORING/NETRX"
          }
        },
        success: function(req, response) {
          var vmGraphs = [
            {
              labels : Locale.tr("Network reception"),
              monitor_resources : "MONITORING/NETRX",
              humanize_figures : true,
              convert_from_bytes : true,
              div_graph : $("#vm_net_rx_graph")
            },
            {
              labels : Locale.tr("Network transmission"),
              monitor_resources : "MONITORING/NETTX",
              humanize_figures : true,
              convert_from_bytes : true,
              div_graph : $("#vm_net_tx_graph")
            },
            {
              labels : Locale.tr("Network reception speed"),
              monitor_resources : "MONITORING/NETRX",
              humanize_figures : true,
              convert_from_bytes : true,
              y_sufix : "B/s",
              derivative : true,
              div_graph : $("#vm_net_rx_speed_graph")
            },
            {
              labels : Locale.tr("Network transmission speed"),
              monitor_resources : "MONITORING/NETTX",
              humanize_figures : true,
              convert_from_bytes : true,
              y_sufix : "B/s",
              derivative : true,
              div_graph : $("#vm_net_tx_speed_graph")
            }
          ];

          for (var i = 0; i < vmGraphs.length; i++) {
            Graphs.plot(response, vmGraphs[i]);
          }
        },
        error: Notifier.onError
      });
    }
  }

  function _getState(context) {
    var state = {
      openNicsDetails : []
    };

    $.each($("#tab_network_form .nics_table .fa-chevron-up", context), function(){
      state.openNicsDetails.push($(this).closest("tr").attr("nic_id"));
    });

    return state;
  }

  function _setState(state, context) {
    var that = this;

    $.each(state["openNicsDetails"], function(){
      $('#tab_network_form .nics_table tr[nic_id="'+this+'"] td.open-control', context).click();
    });
  }
});

define('tabs/vms-tab/panels/snapshots/panelId',['require'],function(require) {
  return 'vm_snapshot_tab';
});

define('tabs/vms-tab/panels/snapshots',['require','utils/locale','sunstone-config','sunstone','utils/humanize','utils/notifier','opennebula/vm','../tabId','./snapshots/panelId','../dialogs/snapshot/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var Sunstone = require('sunstone');
  var Humanize = require('utils/humanize');
  var Notifier = require('utils/notifier');
  var OpenNebulaVM = require('opennebula/vm');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./snapshots/panelId');
  var SNAPSHOT_DIALOG_ID = require('../dialogs/snapshot/dialogId');
  var RESOURCE = "VM"
  var XML_ROOT = "VM"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.panelId = PANEL_ID;
    this.title = Locale.tr("Snapshots");
    this.icon = "fa-laptop";

    this.element = info[XML_ROOT];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var that = this;
    var html = '<form id="snapshot_form" vmid="' + that.element.ID + '" >\
      <div class="row">\
      <div class="large-12 columns">\
         <table class="info_table dataTable extended_table">\
           <thead>\
             <tr>\
                <th>' + Locale.tr("ID") + '</th>\
                <th>' + Locale.tr("Name") + '</th>\
                <th>' + Locale.tr("Timestamp") + '</th>\
                <th>' + Locale.tr("Actions") + '</th>\
                <th>'

    if (Config.isTabActionEnabled("vms-tab", "VM.snapshot_create")) {
      // If VM is not RUNNING, then we forget about the attach disk form.
      if (that.element.STATE == OpenNebulaVM.STATES.ACTIVE && that.element.LCM_STATE == OpenNebulaVM.LCM_STATES.RUNNING) {
        html += '\
           <button id="take_snapshot" class="button tiny success right radius" >' + Locale.tr("Take snapshot") + '</button>'
      } else {
        html += '\
           <button id="take_snapshot" class="button tiny success right radius" disabled="disabled">' + Locale.tr("Take snapshot") + '</button>'
      }
    }

    html +=  '</th>\
              </tr>\
           </thead>\
           <tbody>';

    var snapshots = []
    if ($.isArray(that.element.TEMPLATE.SNAPSHOT))
        snapshots = that.element.TEMPLATE.SNAPSHOT
    else if (!$.isEmptyObject(that.element.TEMPLATE.SNAPSHOT))
        snapshots = [that.element.TEMPLATE.SNAPSHOT]

    if (!snapshots.length) {
      html += '\
          <tr id="no_snapshots_tr">\
            <td colspan="6">'          + Locale.tr("No snapshots to show") + '</td>\
          </tr>'        ;
    } else {

      for (var i = 0; i < snapshots.length; i++) {
        var snapshot = snapshots[i];

        if (
           (
            that.element.STATE == OpenNebulaVM.STATES.ACTIVE) &&
           (
            that.element.LCM_STATE == OpenNebulaVM.LCM_STATES.HOTPLUG_SNAPSHOT)) {
          actions = Locale.tr("snapshot in progress");
        } else {
          actions = '';

          if ((that.element.STATE == OpenNebulaVM.STATES.ACTIVE &&
               that.element.LCM_STATE == OpenNebulaVM.LCM_STATES.RUNNING)) {

            if (Config.isTabActionEnabled("vms-tab", "VM.snapshot_revert")) {
              actions += '<a href="VM.snapshot_revert" class="snapshot_revert" ><i class="fa fa-reply"/>' + Locale.tr("Revert") + '</a> &emsp;'
            }

            if (Config.isTabActionEnabled("vms-tab", "VM.snapshot_delete")) {
              actions += '<a href="VM.snapshot_delete" class="snapshot_delete" ><i class="fa fa-times"/>' + Locale.tr("Delete") + '</a>'
            }
          }
        }

        html += '\
              <tr snapshot_id="' + (snapshot.SNAPSHOT_ID) + '">\
                <td>'            + snapshot.SNAPSHOT_ID + '</td>\
                <td>'            + snapshot.NAME + '</td>\
                <td>'            + Humanize.prettyTime(snapshot.TIME) + '</td>\
                <td>'            + actions + '</td>\
            </tr>'        ;
      }
    }

    html += '\
            </tbody>\
          </table>\
        </div>\
        </div>\
      </form>';

    return html;
  }

  function _setup(context) {
    var that = this;

    if (Config.isTabActionEnabled("vms-tab", "VM.snapshot_create")) {
      context.off('click', '#take_snapshot');
      context.on('click', '#take_snapshot', function() {
        var dialog = Sunstone.getDialog(SNAPSHOT_DIALOG_ID);
        dialog.setElement(that.element);
        dialog.show();
        return false;
      });
    }

    if (Config.isTabActionEnabled("vms-tab", "VM.snapshot_revert")) {
      context.off('click', '.snapshot_revert');
      context.on('click', '.snapshot_revert', function() {
        var snapshot_id = $(this).parents('tr').attr('snapshot_id');
        Sunstone.runAction('VM.snapshot_revert', that.element.ID,  {"snapshot_id": snapshot_id});
        return false;
      });
    }

    if (Config.isTabActionEnabled("vms-tab", "VM.snapshot_delete")) {
      context.off('click', '.snapshot_delete');
      context.on('click', '.snapshot_delete', function() {
        var snapshot_id = $(this).parents('tr').attr('snapshot_id');
        Sunstone.runAction('VM.snapshot_delete', that.element.ID,  {"snapshot_id": snapshot_id});
        return false;
      });
    }
  }
});

define('tabs/vms-tab/panels/placement/panelId',['require'],function(require) {
  return 'vm_placement_tab';
});

define('tabs/vms-tab/panels/placement',['require','utils/locale','utils/humanize','opennebula/vm','../tabId','./placement/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var OpenNebulaVM = require('opennebula/vm');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./placement/panelId');
  var RESOURCE = "VM";
  var XML_ROOT = "VM";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.panelId = PANEL_ID;
    this.title = Locale.tr("Placement");
    this.icon = "fa-sitemap";

    this.element = info[XML_ROOT];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var that = this;
    var html = '<div class="row"><div class="large-12 columns">\
           <table id="vm_history_table" class="extended_table dataTable">\
                    <thead>\
                      <tr>\
                          <th>' + Locale.tr("#") + '</th>\
                          <th>' + Locale.tr("Host") + '</th>\
                          <th>' + Locale.tr("Datastore") + '</th>\
                          <th>' + Locale.tr("Action") + '</th>\
                          <th>' + Locale.tr("Reason") + '</th>\
                          <th>' + Locale.tr("Chg time") + '</th>\
                          <th>' + Locale.tr("Total time") + '</th>\
                          <th colspan="2">' + Locale.tr("Prolog time") + '</th>\
                      </tr>\
                    </thead>\
                    <tbody>'                   ;

    var history = [];
    if (that.element.HISTORY_RECORDS.HISTORY) {
      if ($.isArray(that.element.HISTORY_RECORDS.HISTORY))
          history = that.element.HISTORY_RECORDS.HISTORY;
      else if (that.element.HISTORY_RECORDS.HISTORY.SEQ)
          history = [that.element.HISTORY_RECORDS.HISTORY];
    } else {
      html += '     <tr>\
                <td colspan="8" style="width:5%">' + Locale.tr("No data available in table") + '</td>\
               </tr>'
    }

    var now = Math.round(new Date().getTime() / 1000);

    for (var i = 0; i < history.length; i++) {
      // :TIME time calculations copied from onevm_helper.rb
      var stime = parseInt(history[i].STIME, 10);

      var etime = parseInt(history[i].ETIME, 10)
      etime = etime == 0 ? now : etime;

      var dtime = etime - stime;
      // end :TIME

      //:PTIME
      var stime2 = parseInt(history[i].PSTIME, 10);
      var etime2;
      var ptime2 = parseInt(history[i].PETIME, 10);
      if (stime2 == 0)
          etime2 = 0;
      else
          etime2 = ptime2 == 0 ? now : ptime2;
      var dtime2 = etime2 - stime2;

      //end :PTIME

      html += '     <tr>\
                        <td style="width:5%">' + history[i].SEQ + '</td>\
                        <td style="width:15%">' + history[i].HOSTNAME + '</td>\
                        <td style="width:5%">' + history[i].DS_ID + '</td>\
                        <td style="width:16%">' + OpenNebulaVM.migrateActionStr(parseInt(history[i].ACTION, 10)) + '</td>\
                        <td style="width:10%">' + OpenNebulaVM.migrateReasonStr(parseInt(history[i].REASON, 10)) + '</td>\
                        <td style="width:16%">' + Humanize.prettyTime(history[i].STIME) + '</td>\
                        <td style="width:16%">' + Humanize.prettyDuration(dtime) + '</td>\
                        <td style="width:16%">' + Humanize.prettyDuration(dtime2) + '</td>\
                        <td></td>\
                       </tr>'
    };
    html += '</tbody>\
                 </table>\
           </div>\
         </div>'        ;

    if (that.element.USER_TEMPLATE.SCHED_MESSAGE) {
      html += '<div class="row">\
         <div class="large-12 columns">\
           <table id="vm_ds_placement_table" class="extended_table dataTable">\
                    <thead>\
                      <tr>\
                          <th align="center">' + Locale.tr("Sched Message") + '</th>\
                      </tr>\
                    </thead>\
                    <tbody>\
                       <tr>\
                        <td>' + that.element.USER_TEMPLATE.SCHED_MESSAGE + '</td>\
                      </tr>\
                    </tbody>\
           </table>\
           </div>\
         </div>'      ;
    }

    var requirements_str = that.element.USER_TEMPLATE.SCHED_REQUIREMENTS ? that.element.USER_TEMPLATE.SCHED_REQUIREMENTS : "-";
    var rank_str = that.element.USER_TEMPLATE.SCHED_RANK ? that.element.USER_TEMPLATE.SCHED_RANK : "-";
    var ds_requirements_str = that.element.USER_TEMPLATE.SCHED_DS_REQUIREMENTS ? that.element.USER_TEMPLATE.SCHED_DS_REQUIREMENTS : "-";
    var ds_rank_str = that.element.USER_TEMPLATE.SCHED_DS_RANK ? that.element.USER_TEMPLATE.SCHED_DS_RANK : "-";

    html += '<div class="row">\
       <div class="large-9 columns">\
           <table id="vm_placement_table" class="extended_table dataTable">\
                    <thead>\
                      <tr>\
                          <th colspan="2" align="center">' + Locale.tr("Placement - Host") + '</th>\
                      </tr>\
                    </thead>\
                    <tbody>\
                       <tr>\
                        <td>' + Locale.tr("Requirements") + '</td>\
                        <td>' + requirements_str + '</td>\
                      </tr>\
                       <tr>\
                        <td>' + Locale.tr("Rank") + '</td>\
                        <td>' + rank_str + '</td>\
                      </tr>\
                    </tbody>\
           </table>\
           <table id="vm_ds_placement_table" class="extended_table dataTable">\
                    <thead>\
                      <tr>\
                          <th colspan="2" align="center">' + Locale.tr("Placement - Datastore") + '</th>\
                      </tr>\
                    </thead>\
                    <tbody>\
                       <tr>\
                        <td>' + Locale.tr("DS Requirements") + '</td>\
                        <td>' + ds_requirements_str + '</td>\
                      </tr>\
                       <tr>\
                        <td>' + Locale.tr("DS Rank") + '</td>\
                        <td>' + ds_rank_str + '</td>\
                      </tr>\
                    </tbody>\
           </table>\
           </div>\
         </div>'        ;

    return html;
  }

  function _setup(context) {
  }
});

define('tabs/vms-tab/panels/actions/panelId',['require'],function(require){
  return 'vm_actions_tab';
});
define('tabs/vms-tab/panels/actions',['require','utils/locale','sunstone','utils/humanize','utils/template-utils','../tabId','./actions/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Sunstone = require('sunstone');
  var Humanize = require('utils/humanize');
  var TemplateUtils = require('utils/template-utils');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./actions/panelId');
  var RESOURCE = "VM";
  var XML_ROOT = "VM";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.panelId = PANEL_ID;
    this.title = Locale.tr("Actions");
    this.icon = "fa-calendar";

    this.element = info[XML_ROOT];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var that = this;
    var html = '<div class="row">\
      <div class="large-12 columns">\
        <table id="scheduling_actions_table" class="info_table dataTable extended_table">\
         <thead>\
           <tr>\
              <th>' + Locale.tr("ID") + '</th>\
              <th>' + Locale.tr("ACTION") + '</th>\
              <th>' + Locale.tr("TIME") + '</th>\
              <th>' + Locale.tr("DONE") + '</th>\
              <th>' + Locale.tr("MESSAGE") + '</th>\
              <th colspan="">' + Locale.tr("Actions") + '</th>\
              <th><button id="add_scheduling_action" class="button tiny success right radius" >' + Locale.tr("Add action") + '</button></th>\
           </tr>\
          </thead>' +
            fromJSONtoActionsTable(that.element.USER_TEMPLATE.SCHED_ACTION) +
         '</table>\
        </div>\
      </div>';

    return html;
  }

  function _setup(context) {
    var that = this;
    context.off('click', '#add_scheduling_action');
    context.on('click', '#add_scheduling_action', function() {
      $("#add_scheduling_action", context).attr("disabled", "disabled");
      $("#scheduling_actions_table").append('<tr>\
          <td></td>\
          <td class="columns">\
            <select id="select_new_action" class="select_new_action" name="select_action">\
              <option value="shutdown">' + Locale.tr("shutdown") + '</option>\
              <option value="shutdown-hard">' + Locale.tr("shutdown-hard") + '</option>\
              <option value="hold">' + Locale.tr("hold") + '</option>\
              <option value="release">' + Locale.tr("release") + '</option>\
              <option value="stop">' + Locale.tr("stop") + '</option>\
              <option value="suspend">' + Locale.tr("suspend") + '</option>\
              <option value="resume">' + Locale.tr("resume") + '</option>\
              <option value="delete">' + Locale.tr("delete") + '</option>\
              <option value="delete-recreate">' + Locale.tr("delete-recreate") + '</option>\
              <option value="reboot">' + Locale.tr("reboot") + '</option>\
              <option value="reboot-hard">' + Locale.tr("reboot-hard") + '</option>\
              <option value="poweroff">' + Locale.tr("poweroff") + '</option>\
              <option value="poweroff-hard">' + Locale.tr("poweroff-hard") + '</option>\
              <option value="undeploy">' + Locale.tr("undeploy") + '</option>\
              <option value="undeploy-hard">' + Locale.tr("undeploy-hard") + '</option>\
              <option value="snapshot-create">' + Locale.tr("snapshot-create") + '</option>\
            </select>\
          </td>\
         <td>\
            <input id="date_input" type="date" placeholder="2013/12/30"/>\
            <input id="time_input" type="time" placeholder="12:30"/>\
         </td>\
         <td>\
            <button id="submit_scheduling_action" class="button small secondary radius" >'              + Locale.tr("Add") + '</button>\
         </td>\
         <td colspan=2></td>\
       </tr>');

      return false;
    });

    context.off("click", "#submit_scheduling_action");
    context.on("click", "#submit_scheduling_action", function() {
      var date_input_value = $("#date_input", context).val();
      var time_input_value = $("#time_input", context).val();

      if (date_input_value == "" || time_input_value == "")
        return false;

      var time_value = date_input_value + ' ' + time_input_value

      // Calculate MAX_ID
      var max_id = -1;

      if (that.element.USER_TEMPLATE.SCHED_ACTION) {
        if (!that.element.USER_TEMPLATE.SCHED_ACTION.length) {
          var tmp_element = that.element.USER_TEMPLATE.SCHED_ACTION;
          that.element.USER_TEMPLATE.SCHED_ACTION = new Array();
          that.element.USER_TEMPLATE.SCHED_ACTION.push(tmp_element);
        }

        $.each(that.element.USER_TEMPLATE.SCHED_ACTION, function(i, element) {
          if (max_id < element.ID)
            max_id = element.ID
        })
      } else {
        that.element.USER_TEMPLATE.SCHED_ACTION = new Array();
      }

      var new_action = {};
      new_action.ID  = parseInt(max_id) + 1;
      new_action.ACTION = $("#select_new_action", context).val();
      var epoch_str   = new Date(time_value);

      new_action.TIME = parseInt(epoch_str.getTime()) / 1000;

      that.element.USER_TEMPLATE.SCHED_ACTION.push(new_action);

      // Let OpenNebula know
      var template_str = TemplateUtils.templateToString(that.element.USER_TEMPLATE);
      Sunstone.runAction("VM.update_template", that.element.ID, template_str);

      $("#add_scheduling_action", context).removeAttr("disabled");
      return false;
    });

    // Listener for key,value pair remove action
    context.off("click", ".remove_action_x");
    context.on("click", ".remove_action_x", function() {
      var index = this.id.substring(6, this.id.length);
      var tmp_tmpl = new Array();

      $.each(that.element.USER_TEMPLATE.SCHED_ACTION, function(i, element) {
        if (element.ID != index)
          tmp_tmpl[i] = element
      })

      that.element.USER_TEMPLATE.SCHED_ACTION = tmp_tmpl;
      var template_str = TemplateUtils.templateToString(that.element.USER_TEMPLATE);

      // Let OpenNebula know
      Sunstone.runAction("VM.update_template", that.element.ID, template_str);
    });

  }

  // Returns an HTML string with the json keys and values
  function fromJSONtoActionsTable(actions_array) {
    var str = ""
    var empty = '\
      <tr id="no_actions_tr">\
          <td colspan="6">' + Locale.tr("No actions to show") + '</td>\
      </tr>'    ;

    if (!actions_array) {
      return empty;
    }

    if (!$.isArray(actions_array)) {
      var tmp_array = new Array();
      tmp_array[0]  = actions_array;
      actions_array = tmp_array;
    }

    if (!actions_array.length) {
      return empty;
    }

    $.each(actions_array, function(index, scheduling_action) {
      str += fromJSONtoActionRow(scheduling_action);
    });

    return str;
  }

  // Helper for fromJSONtoHTMLTable function
  function fromJSONtoActionRow(scheduling_action) {
    var done_str    = scheduling_action.DONE ? (Humanize.prettyTime(scheduling_action.DONE)) : "";
    var message_str = scheduling_action.MESSAGE ? scheduling_action.MESSAGE : "";
    var time_str    = Humanize.prettyTime(scheduling_action.TIME);

    var str = "";
    str += '<tr class="tr_action_' + scheduling_action.ID + '">\
       <td class="id_row">' + scheduling_action.ID + '</td>\
       <td class="action_row">' + scheduling_action.ACTION + '</td>\
       <td nowrap class="time_row">' + time_str + '</td>\
       <td class="done_row">' + done_str + '</td>\
       <td class="message_row">' + message_str + '</td>\
       <td>\
         <div>\
           <a id="minus_' + scheduling_action.ID + '" class="remove_action_x" href="#"><i class="fa fa-trash-o"/></a>\
         </div>\
       </td>\
     </tr>';

    return str;
  }
});

define('tabs/vms-tab/panels/template/panelId',['require'],function(require){
  return 'vm_template_tab';
});
define('tabs/vms-tab/panels/template',['require','utils/locale','utils/humanize','../tabId','./template/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./template/panelId');
  var RESOURCE = "VM"
  var XML_ROOT = "VM"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Template");
    this.icon = "fa-file-o";

    this.element = info[XML_ROOT];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return '<div class="row">'+
      '<div class="large-12 columns">'+
        '<table class="dataTable extended_table">'+
          '<thead>'+
            '<tr>'+
              '<th colspan="2">' + Locale.tr("Template") + '</th>'+
            '</tr>'+
          '</thead>'+
          '<tbody>'+
            Humanize.prettyPrintJSON(this.element.TEMPLATE) +
          '</tbody>'+
        '</table>'+
      '</div>'+
    '</div>'
  }

  function _setup(context) {
  }
});

define('tabs/vms-tab/panels/log/panelId',['require'],function(require){
  return 'vm_log_tab';
});
define('tabs/vms-tab/panels/log',['require','utils/locale','utils/notifier','opennebula/vm','../tabId','./log/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Notifier = require('utils/notifier');
  var OpenNebulaVM = require('opennebula/vm');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./log/panelId');
  var RESOURCE = "VM"
  var XML_ROOT = "VM"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.panelId = PANEL_ID;
    this.title = Locale.tr("Log");
    this.icon = "fa-file-text";

    this.element = info[XML_ROOT];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;
  Panel.prototype.onShow = _onShow;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return '<div class="row">' +
      '<div class="large-12 columns vm_log_container">' +
        '<div class="text-center" style="height: 100px;">' +
          '<span id="provision_dashboard_total" style="font-size:80px">' +
            '<i class="fa fa-spinner fa-spin"></i>' +
          '</span>' +
        '</div>' +
      '</div>' +
    '</div>';
  }

  function _setup(context) {
  }

  function _onShow(context) {
    var that = this;
    OpenNebulaVM.log({
      data: {id: that.element.ID},
      success: function(req, response) {
        var log_lines = response['vm_log'].split("\n");
        var colored_log = '';
        for (var i = 0; i < log_lines.length; i++) {
          var line = log_lines[i];
          if (line.match(/\[E\]/)) {
            line = '<span class="vm_log_error">' + line + '</span>';
          }
          colored_log += line + "<br>";
        }

        $('.vm_log_container', context).html(
          '<div class="row">' +
            '<div class="large-11 small-centered columns log-tab">' +
              colored_log +
            '</div>' +
          '</div>')

      },
      error: function(request, error_json) {
        $('.vm_log_container', context).html('');
        Notifier.onError(request, error_json);
      }
    });
  }
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/instantiate/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"large-12 large-centered columns selectTemplateTable\">\n    </div>\n  </div>\n  <div class=\"row nameContainer\">\n    <div class=\"medium-4 columns\">\n      <label for=\"vm_name\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VM Name", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Defaults to template name when emtpy. You can use the wildcard &#37;i. When creating several VMs, &#37;i will be replaced with a different number starting from 0 in each of them", {"name":"tr","hash":{},"data":data})))
    + ".\n        </span>\n      </label>\n      <input type=\"text\" name=\"vm_name\" id=\"vm_name\" />\n    </div> \n    <div class=\"medium-4 columns\">\n      <label for=\"vm_n_times\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of instances", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of Virtual Machines that will be created using this template", {"name":"tr","hash":{},"data":data})))
    + ".\n        </span>\n      </label>\n      <input type=\"text\" name=\"vm_n_times\" id=\"vm_n_times\" value=\"1\">\n    </div>\n    <div class=\"medium-4 columns\">\n      <input type=\"checkbox\" name=\"hold\" id=\"hold\"/>\n      <label for=\"hold\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hold", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Sets the new VM to hold state, instead of pending. The scheduler will not deploy VMs in this state. It can be released later, or deployed manually.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 large-centered columns list_of_templates\">\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/instantiate/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/instantiate/templateRow',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, lambda=this.lambda, escapeExpression=this.escapeExpression, helperMissing=helpers.helperMissing, functionType="function", buffer = "<h3 style=\"border-bottom: 1px solid #efefef; padding-bottom: 10px;\">\n  <span style=\"border-bottom: 2px solid #0098c3; padding: 0px 50px 10px 0px;\">\n    "
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.NAME : stack1), depth0))
    + "\n  </span>\n</h3>\n<div class=\"row\">\n  <div class=\"large-8 columns left capacityContext"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "\">\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <h3 class=\"subheader text-right\">\n          <span class=\"left\">\n            <i class=\"fa fa-laptop fa-lg\"></i>&emsp;"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Capacity", {"name":"tr","hash":{},"data":data})))
    + "\n          </span>\n          <br>\n        </h3>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        ";
  stack1 = ((helper = (helper = helpers.capacityInputsHTML || (depth0 != null ? depth0.capacityInputsHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"capacityInputsHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n        <br>\n      </div>\n    </div>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-8 columns left disksContext"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "\"></div>\n</div>\n<div class=\"row\">\n  <div class=\"large-12 columns nicsContext"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "\">\n    <div class=\"provision_network_selector\">\n    </div>\n    <br>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-12 columns large-centered template_user_inputs"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "\">\n  </div>\n  <br>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/instantiate/templateRow', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/user-inputs',['require','utils/locale','utils/template-utils','tabs/vnets-tab/datatable'],function(require) {
  var Locale = require('utils/locale');
  var TemplateUtils = require('utils/template-utils');
  var VNetsTable = require('tabs/vnets-tab/datatable');

  //==============================================================================
  // VM & Service user inputs
  //==============================================================================

  return {
    'vmTemplateInsert': _generateVMTemplateUserInputs,
    'serviceTemplateInsert': _generateServiceTemplateUserInputs
  }

  // It will replace the div's html with a row for each USER_INPUTS
  // opts.text_header: header text for the text & password inputs
  // opts.network_header: header text for the network inputs
  // returns true if at least one input was inserted
  function _generateVMTemplateUserInputs(div, template_json, opts) {
    return _generateInstantiateUserInputs(
        div, template_json.VMTEMPLATE.TEMPLATE.USER_INPUTS, opts);
  }

  // It will replace the div's html with a row for each USER_INPUTS
  // opts.text_header: header text for the text & password inputs
  // opts.network_header: header text for the network inputs
  // returns true if at least one input was inserted
  function _generateServiceTemplateUserInputs(div, template_json, opts) {
    return _generateInstantiateUserInputs(
        div, template_json.DOCUMENT.TEMPLATE.BODY.custom_attrs, opts);
  }

  // It will replace the div's html with a row for each USER_INPUTS
  // opts.text_header: header text for the text & password inputs
  // opts.network_header: header text for the network inputs
  // returns true if at least one input was inserted
  function _generateInstantiateUserInputs(div, user_inputs, opts) {
    div.empty();

    if (user_inputs == undefined) {
      return false;
    }

    if (opts == undefined) {
      opts = {};
    }

    if (opts.text_header == undefined) {
      opts.text_header = Locale.tr("Custom Attributes");
    }

    if (opts.network_header == undefined) {
      opts.network_header = Locale.tr("Network");
    }

    var network_attrs = [];
    var text_attrs = [];

    $.each(user_inputs, function(key, value) {
      var parts = value.split("|");
      // 0 mandatory; 1 type; 2 desc;
      var attrs = {
        "name": key,
        "mandatory": parts[0],
        "type": parts[1],
        "description": parts[2],
      }

      switch (parts[1]) {
        case "vnet_id":
          network_attrs.push(attrs)
          break;
        case "text":
          text_attrs.push(attrs)
          break;
        case "password":
          text_attrs.push(attrs)
          break;
      }
    });

    if (network_attrs.length > 0) {
      if (opts.network_header.length > 0) {
        div.append(
        '<br>' +
        '<div class="row">' +
          '<div class="large-12 large-centered columns">' +
            '<h3 class="subheader">' +
              opts.network_header +
            '</h3>' +
          '</div>' +
        '</div>');
      }

      div.append('<div class="instantiate_user_inputs"/>');

      var separator = "";

      var vnetsTable;
      $.each(network_attrs, function(index, vnet_attr) {
        var unique_id = "user_input_" + (vnet_attr.name.replace(/ /g, "_"));
        vnetsTable = new VNetsTable(unique_id, {'select': true});

        $(".instantiate_user_inputs", div).append(
          '<div class="row">' +
            '<div class="large-12 large-centered columns">' +
              separator +
              '<h5>' +
                TemplateUtils.htmlDecode(vnet_attr.description) +
              '</h5>' +
              vnetsTable.dataTableHTML +
            '</div>' +
          '</div>');

        separator = "<hr/>";

        vnetsTable.initialize();

        $('#refresh_button_' + unique_id).click();

        vnetsTable.idInput().attr("wizard_field", vnet_attr.name).attr("required", "");
      });
    }

    if (text_attrs.length > 0) {
      if (opts.text_header.length > 0) {
        div.append(
        '<br>' +
        '<div class="row">' +
          '<div class="large-12 large-centered columns">' +
            '<h3 class="subheader">' +
              opts.text_header +
            '</h3>' +
          '</div>' +
        '</div>');
      }

      div.append('<div class="instantiate_user_inputs"/>');

      $.each(text_attrs, function(index, custom_attr) {
            $(".instantiate_user_inputs", div).append(
              '<div class="row">' +
                '<div class="large-12 large-centered columns">' +
                  '<label>' +
                    TemplateUtils.htmlDecode(custom_attr.description) +
                    '<input type="' + custom_attr.type + '" wizard_field="' + custom_attr.name + '" required/>' +
                  '</label>' +
                '</div>' +
              '</div>');
          });
    }

    return (network_attrs.length > 0 || text_attrs.length > 0);
  }
})
;
define('tabs/templates-tab/form-panels/instantiate/formPanelId',['require'],function(require){
  return 'instantiateTemplateDialog';
});
define('tabs/templates-tab/tabId',['require'],function(require){
  return 'templates-tab';
});
define('tabs/templates-tab/form-panels/instantiate',['require','utils/form-panels/form-panel','hbs!./instantiate/html','hbs!./instantiate/templateRow','sunstone','utils/notifier','opennebula/template','utils/locale','utils/tips','utils/user-inputs','utils/wizard-fields','utils/disks-resize','utils/nics-section','tabs/templates-tab/form-panels/create/wizard-tabs/general/capacity-inputs','./instantiate/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseFormPanel = require('utils/form-panels/form-panel');
  var TemplateHTML = require('hbs!./instantiate/html');
  var TemplateRowHTML = require('hbs!./instantiate/templateRow');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var OpenNebulaTemplate = require('opennebula/template');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var UserInputs = require('utils/user-inputs');
  var WizardFields = require('utils/wizard-fields');
  var DisksResize = require('utils/disks-resize');
  var NicsSection = require('utils/nics-section');
  var CapacityInputs = require('tabs/templates-tab/form-panels/create/wizard-tabs/general/capacity-inputs');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./instantiate/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'instantiate': {
        'title': Locale.tr("Instantiate VM Template"),
        'buttonText': Locale.tr("Instantiate"),
        'resetButton': false
      }
    };

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.setTemplateIds = _setTemplateIds;
  FormPanel.prototype.htmlWizard = _html;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'formPanelId': this.formPanelId
    });
  }

  function _setup(context) {
    context.foundation('abide', 'reflow');
  }

  function _submitWizard(context) {
    if (!this.selected_nodes || this.selected_nodes.length == 0) {
      Notifier.notifyError(Locale.tr("No template selected"));
      Sunstone.hideFormPanelLoading(this.tabId);
      return false;
    }

    var vm_name = $('#vm_name', context).val();
    var n_times = $('#vm_n_times', context).val();
    var n_times_int = 1;

    var hold = $('#hold', context).prop("checked");

    $.each(this.selected_nodes, function(index, template_id) {
      if (n_times.length) {
        n_times_int = parseInt(n_times, 10);
      }

      var extra_msg = "";
      if (n_times_int > 1) {
        extra_msg = n_times_int + " times";
      }

      Notifier.notifySubmit("Template.instantiate", template_id, extra_msg);

      var extra_info = {
        'hold': hold
      };

      var tmp_json = WizardFields.retrieve($(".template_user_inputs" + template_id, context));
      var disks = DisksResize.retrieve($(".disksContext"  + template_id, context));
      if (disks.length > 0) {
        tmp_json.DISK = disks;
      }

      var nics = NicsSection.retrieve($(".nicsContext"  + template_id, context));
      if (nics.length > 0) {
        tmp_json.NIC = nics;
      }

      capacityContext = $(".capacityContext"  + template_id, context);
      $.extend(tmp_json, CapacityInputs.retrieveResize(capacityContext));

      extra_info['template'] = tmp_json;

      if (!vm_name.length) { //empty name use OpenNebula core default
        for (var i = 0; i < n_times_int; i++) {
          extra_info['vm_name'] = "";
          Sunstone.runAction("Template.instantiate_quiet", template_id, extra_info);
        }
      } else {
        if (vm_name.indexOf("%i") == -1) {//no wildcard, all with the same name
          extra_info['vm_name'] = vm_name;

          for (var i = 0; i < n_times_int; i++) {
            Sunstone.runAction(
                "Template.instantiate_quiet",
                template_id,
                extra_info);
          }
        } else { //wildcard present: replace wildcard
          for (var i = 0; i < n_times_int; i++) {
            extra_info['vm_name'] = vm_name.replace(/%i/gi, i);

            Sunstone.runAction(
                "Template.instantiate_quiet",
                template_id,
                extra_info);
          }
        }
      }
    });

    return false;
  }

  function _setTemplateIds(context, selected_nodes) {
    this.selected_nodes = selected_nodes;

    var templatesContext = $(".list_of_templates", context);

    var idsLength = this.selected_nodes.length;
    var idsDone = 0;

    $.each(this.selected_nodes, function(index, template_id) {
      OpenNebulaTemplate.show({
        data : {
          id: template_id
        },
        timeout: true,
        success: function (request, template_json) {
          templatesContext.append(
            TemplateRowHTML(
              { element: template_json.VMTEMPLATE,
                capacityInputsHTML: CapacityInputs.html()
              }) );

          DisksResize.insert(template_json, $(".disksContext"  + template_json.VMTEMPLATE.ID, context));
          NicsSection.insert(template_json, $(".nicsContext"  + template_json.VMTEMPLATE.ID, context));

          var inputs_div = $(".template_user_inputs" + template_json.VMTEMPLATE.ID, context);

          UserInputs.vmTemplateInsert(
              inputs_div,
              template_json,
              {text_header: '<i class="fa fa-gears fa-lg"></i>&emsp;'+Locale.tr("Custom Attributes")});

          inputs_div.data("opennebula_id", template_json.VMTEMPLATE.ID);

          capacityContext = $(".capacityContext"  + template_json.VMTEMPLATE.ID, context);
          CapacityInputs.setup(capacityContext);
          CapacityInputs.fill(capacityContext, template_json.VMTEMPLATE);

          if (template_json.VMTEMPLATE.TEMPLATE.SUNSTONE_CAPACITY_SELECT &&
              template_json.VMTEMPLATE.TEMPLATE.SUNSTONE_CAPACITY_SELECT.toUpperCase() == "NO"){

            capacityContext.hide();
          }

          idsDone += 1;
          if (idsLength == idsDone){
            Sunstone.enableFormPanelSubmit(TAB_ID);
          }
        },
        error: function(request, error_json, container) {
          Notifier.onError(request, error_json, container);
          $("#instantiate_vm_user_inputs", context).empty();
        }
      });
    });
  }

  function _onShow(context) {
    Sunstone.disableFormPanelSubmit(TAB_ID);

    var templatesContext = $(".list_of_templates", context);
    templatesContext.html("");

    Tips.setup(context);
    return false;
  }
});

define('tabs/templates-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','utils/humanize','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');

  /*
    CONSTANTS
   */

  var RESOURCE = "Template";
  var XML_ROOT = "VMTEMPLATE";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"]},
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    }

    this.columns = [
      Locale.tr("ID") ,
      Locale.tr("Owner") ,
      Locale.tr("Group"),
      Locale.tr("Name"),
      Locale.tr("Registration time")
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 4,
      "select_resource": Locale.tr("Please select a Template from the list"),
      "you_selected": Locale.tr("You selected the following Template:"),
      "select_resource_multiple": Locale.tr("Please select one or more Templates from the list"),
      "you_selected_multiple": Locale.tr("You selected the following Templates:")
    };

    TabDataTable.call(this);
  };

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json[XML_ROOT];

    return [
        '<input class="check_item" type="checkbox" id="' + RESOURCE.toLowerCase() + '_' +
                             element.ID + '" name="selected_items" value="' +
                             element.ID + '"/>',
        element.ID,
        element.UNAME,
        element.GNAME,
        element.NAME,
        Humanize.prettyTime(element.REGTIME)
    ];
  }
});

define('tabs/vms-tab/form-panels/create',['require','foundation.tab','tabs/templates-tab/form-panels/instantiate','sunstone','utils/locale','utils/tips','utils/notifier','utils/wizard-fields','utils/user-inputs','opennebula/template','tabs/templates-tab/datatable','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var InstantiateTemplateFormPanel = require('tabs/templates-tab/form-panels/instantiate');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var Notifier = require('utils/notifier');
  var WizardFields = require('utils/wizard-fields');
  var UserInputs = require('utils/user-inputs');
  var OpenNebulaTemplate = require('opennebula/template');
  var TemplatesTable = require('tabs/templates-tab/datatable');
  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    InstantiateTemplateFormPanel.call(this);

    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create Virtual Machine"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      }
    };

    this.templatesTable = new TemplatesTable('vm_create', {'select': true});
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(InstantiateTemplateFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */
  function _setup(context) {
    var that = this;
    InstantiateTemplateFormPanel.prototype.setup.call(this, context);

    $(".selectTemplateTable", context).html(
      '<fieldset>' +
        '<legend>' + Locale.tr("Select a template") + '</legend>' +
        this.templatesTable.dataTableHTML +
      '</fieldset>');

    this.templatesTable.initialize();

    $("#selected_resource_id_vm_create", context).on("change", function(){
        $(".nameContainer", context).show();

         var templatesContext = $(".list_of_templates", context);
        templatesContext.html("");
        templatesContext.show();

        var template_id = $(this).val();
        that.setTemplateIds(context, [template_id]);
    });

    Tips.setup(context);
  }

  function _onShow(context) {
    this.templatesTable.resetResourceTableSelect();
    InstantiateTemplateFormPanel.prototype.onShow.call(this, context);

    $(".nameContainer", context).hide();
  }
});

define('tabs/vms-tab',['require','utils/locale','./vms-tab/buttons','./vms-tab/actions','./vms-tab/datatable','utils/vnc','./vms-tab/tabId','./vms-tab/dialogs/deploy','./vms-tab/dialogs/migrate','./vms-tab/dialogs/resize','./vms-tab/dialogs/attach-disk','./vms-tab/dialogs/disk-snapshot','./vms-tab/dialogs/disk-saveas','./vms-tab/dialogs/attach-nic','./vms-tab/dialogs/snapshot','./vms-tab/dialogs/vnc','./vms-tab/dialogs/spice','./vms-tab/dialogs/saveas-template','./vms-tab/panels/info','./vms-tab/panels/capacity','./vms-tab/panels/storage','./vms-tab/panels/network','./vms-tab/panels/snapshots','./vms-tab/panels/placement','./vms-tab/panels/actions','./vms-tab/panels/template','./vms-tab/panels/log','./vms-tab/form-panels/create'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./vms-tab/buttons');
  var Actions = require('./vms-tab/actions');
  var Table = require('./vms-tab/datatable');
  require('utils/vnc');
  var TAB_ID = require('./vms-tab/tabId');
  var DATATABLE_ID = "dataTableVms";

  var _dialogs = [
    require('./vms-tab/dialogs/deploy'),
    require('./vms-tab/dialogs/migrate'),
    require('./vms-tab/dialogs/resize'),
    require('./vms-tab/dialogs/attach-disk'),
    require('./vms-tab/dialogs/disk-snapshot'),
    require('./vms-tab/dialogs/disk-saveas'),
    require('./vms-tab/dialogs/attach-nic'),
    require('./vms-tab/dialogs/snapshot'),
    require('./vms-tab/dialogs/vnc'),
    require('./vms-tab/dialogs/spice'),
    require('./vms-tab/dialogs/saveas-template')
  ];

  var _panels = [
    require('./vms-tab/panels/info'),
    require('./vms-tab/panels/capacity'),
    require('./vms-tab/panels/storage'),
    require('./vms-tab/panels/network'),
    require('./vms-tab/panels/snapshots'),
    require('./vms-tab/panels/placement'),
    require('./vms-tab/panels/actions'),
    require('./vms-tab/panels/template'),
    require('./vms-tab/panels/log')
  ];

  var _formPanels = [
    require('./vms-tab/form-panels/create')
  ]

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("Virtual Machines"),
    tabClass: "subTab",
    parentTab: "vresources-tab",
    listHeader: '<i class="fa fa-fw fa-th"></i>&emsp;' + Locale.tr("Virtual Machines"),
    infoHeader: '<i class="fa fa-fw fa-th"></i>&emsp;' + Locale.tr("VM"),
    subheader: '<span class="total_vms"/> <small>' + Locale.tr("TOTAL") + '</small>&emsp;\
        <span class="active_vms"/> <small>' + Locale.tr("ACTIVE") + '</small>&emsp;\
        <span class="off_vms"/> <small>' + Locale.tr("OFF") + '</small>&emsp;\
        <span class="pending_vms"/> <small>' + Locale.tr("PENDING") + '</small>&emsp;\
        <span class="failed_vms"/> <small>' + Locale.tr("FAILED") + '</small>',
    resource: 'VM',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});

define('tabs/templates-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "Template.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "Template.create_dialog" : {
      type: "create_dialog",
      layout: "create"
    },
    "Template.import_dialog" : {
      type: "create_dialog",
      layout: "create",
      text:  Locale.tr("Import"),
      icon: '<i class="fa fa-download">',
      alwaysActive: true
    },
    "Template.update_dialog" : {
      type: "action",
      layout: "main",
      text: Locale.tr("Update")
    },
    "Template.instantiate_vms" : {
      type: "action",
      layout: "main",
      text: Locale.tr("Instantiate")
    },
    "Template.chown" : {
      type: "confirm_with_select",
      text: Locale.tr("Change owner"),
      layout: "user_select",
      select: "User",
      tip: Locale.tr("Select the new owner") + ":",
    },
    "Template.chgrp" : {
      type: "confirm_with_select",
      text: Locale.tr("Change group"),
      layout: "user_select",
      select: "Group",
      tip: Locale.tr("Select the new group") + ":",
    },
    "Template.clone_dialog" : {
      type: "action",
      layout: "main",
      text: Locale.tr("Clone")
    },
    "Template.delete" : {
      type: "confirm",
      layout: "del"
    }
  }

  return Buttons;
})
;
define('tabs/templates-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createVMTemplateForm';
});
define('tabs/templates-tab/dialogs/clone/dialogId',['require'],function(require){
  return 'cloneTemplateDialog';
});
define('tabs/templates-tab/form-panels/import/formPanelId',['require'],function(require){
  return 'importTemplateForm';
});
define('tabs/templates-tab/actions',['require','sunstone','utils/notifier','utils/locale','opennebula/template','utils/common-actions','opennebula/action','./tabId','./form-panels/create/formPanelId','./dialogs/clone/dialogId','./form-panels/instantiate/formPanelId','./form-panels/import/formPanelId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var OpenNebulaResource = require('opennebula/template');
  var CommonActions = require('utils/common-actions');
  var OpenNebulaAction = require('opennebula/action');

  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');
  var CLONE_DIALOG_ID = require('./dialogs/clone/dialogId');
  var INSTANTIATE_DIALOG_ID = require('./form-panels/instantiate/formPanelId');
  var IMPORT_DIALOG_ID = require('./form-panels/import/formPanelId');

  var XML_ROOT = "VMTEMPLATE"
  var RESOURCE = "Template"

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "Template.list" : _commonActions.list(),
    "Template.show" : _commonActions.show(),
    "Template.refresh" : _commonActions.refresh(),
    "Template.delete" : _commonActions.del(),
    "Template.chown": _commonActions.multipleAction('chown'),
    "Template.chgrp": _commonActions.multipleAction('chgrp'),
    "Template.chmod": _commonActions.singleAction('chmod'),
    "Template.rename": _commonActions.singleAction('rename'),
    "Template.create" : _commonActions.create(CREATE_DIALOG_ID),
    "Template.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID),
    "Template.import_dialog" : _commonActions.showCreate(IMPORT_DIALOG_ID),
    "Template.update" : _commonActions.update(),
    "Template.update_dialog" : _commonActions.checkAndShowUpdate(),
    "Template.show_to_update" : _commonActions.showUpdate(CREATE_DIALOG_ID),
    "Template.instantiate" : {
      type: "multiple",
      call: OpenNebulaResource.instantiate,
      callback: function(req) {
        Sunstone.hideFormPanel(TAB_ID);
        OpenNebulaAction.clear_cache("VM");
      },
      elements: function() {
        return Sunstone.getDataTable(TAB_ID).elements();
      },
      error: Notifier.onError,
      notify: true
    },
    "Template.instantiate_quiet" : {
      type: "single",
      call: OpenNebulaResource.instantiate,
      callback: function(req) {
        Sunstone.hideFormPanel(TAB_ID);
        OpenNebulaAction.clear_cache("VM");
      },
      error: Notifier.onError
    },
    "Template.instantiate_vms" : {
      type: "custom",
      call: function(){
        //Sunstone.resetFormPanel(TAB_ID, INSTANTIATE_DIALOG_ID);
        var selected_nodes = Sunstone.getDataTable(TAB_ID).elements();

        Sunstone.showFormPanel(TAB_ID, INSTANTIATE_DIALOG_ID, "instantiate",
          function(formPanelInstance, context) {
            formPanelInstance.setTemplateIds(context, selected_nodes);
          });
      }
    },
    "Template.clone_dialog" : {
      type: "custom",
      call: function(){
        Sunstone.getDialog(CLONE_DIALOG_ID).show();
      }
    },
    "Template.clone" : {
      type: "single",
      call: OpenNebulaResource.clone,
      error: Notifier.onError,
      notify: true
    }
  };

  return _actions;
})
;

/* START_TEMPLATE */
define('hbs!tabs/templates-tab/dialogs/clone/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <h3 id=\"create_vnet_header\" class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Clone Template", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n  </div>\n  <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\" action=\"\" class=\"custom creation\">\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <div class=\"clone_one\"></div>\n        <div class=\"clone_several\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Several templates are selected, please choose prefix to name the new copies", {"name":"tr","hash":{},"data":data})))
    + "\n          <br></div>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <label class=\"clone_one\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        <label class=\"clone_several\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Prefix", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        <input type=\"text\" name=\"name\"/>\n      </div>\n    </div>\n    <div class=\"form_buttons row\">\n      <button class=\"button radius right\" id=\"template_clone_button\" value=\"Template.clone\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Clone", {"name":"tr","hash":{},"data":data})))
    + "</button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/dialogs/clone/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/dialogs/clone',['require','utils/dialogs/dialog','hbs!./clone/html','sunstone','utils/notifier','opennebula/template','./clone/dialogId','tabs/templates-tab/tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./clone/html');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var OpenNebulaTemplate = require('opennebula/template');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./clone/dialogId');
  var TEMPLATES_TAB_ID = require('tabs/templates-tab/tabId')

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId
    });
  }

  function _setup(context) {
    var that = this;

    $('#' + DIALOG_ID + 'Form', context).submit(function() {
      var extra_info;
      var name = $('input[name="name"]', this).val();
      var sel_elems = Sunstone.getDataTable(TEMPLATES_TAB_ID).elements();

      if (!name || !sel_elems.length)
        Notifier.notifyError('A name or prefix is needed!');

      if (sel_elems.length > 1) {
        for (var i = 0; i < sel_elems.length; i++) {
          //If we are cloning several images we
          //use the name as prefix
          extra_info = name + OpenNebulaTemplate.getName(sel_elems[i]);
          Sunstone.runAction('Template.clone', sel_elems[i], extra_info);
        }
      } else {
        extra_info = name;
        Sunstone.runAction('Template.clone', sel_elems[0], extra_info)
      }

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      setTimeout(function() {
        Sunstone.runAction('Template.refresh');
      }, 1500);
      return false;
    });

    return false;
  }

  function _onShow(context) {
    var sel_elems = Sunstone.getDataTable(TEMPLATES_TAB_ID).elements();
    //show different text depending on how many elements are selected
    if (sel_elems.length > 1) {
      $('.clone_one', context).hide();
      $('.clone_several', context).show();
      $('input[name="name"]',context).val('Copy of ');
    } else {
      $('.clone_one', context).show();
      $('.clone_several', context).hide();
      $('input[name="name"]', context).val('Copy of ' + OpenNebulaTemplate.getName(sel_elems[0]));
    };

    $("input[name='name']", context).focus();

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n        </tr>\n        ";
  stack1 = ((helper = (helper = helpers.renameTrHTML || (depth0 != null ? depth0.renameTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"renameTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Register time", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.prettyRegTime || (depth0 != null ? depth0.prettyRegTime : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"prettyRegTime","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n      </tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\">";
  stack1 = ((helper = (helper = helpers.permissionsTableHTML || (depth0 != null ? depth0.permissionsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"permissionsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/panels/info/panelId',['require'],function(require){
  return 'template_info_tab';
});
define('tabs/templates-tab/panels/info',['require','utils/locale','utils/humanize','utils/panel/rename-tr','utils/panel/permissions-table','hbs!./info/html','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var RenameTr = require('utils/panel/rename-tr');
  var PermissionsTable = require('utils/panel/permissions-table');

  /*
    TEMPLATES
   */

  var TemplateInfo = require('hbs!./info/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "Template"
  var XML_ROOT = "VMTEMPLATE"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[XML_ROOT];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var renameTrHTML = RenameTr.html(RESOURCE, this.element.NAME);
    var permissionsTableHTML = PermissionsTable.html(TAB_ID, RESOURCE, this.element);
    var prettyRegTime = Humanize.prettyTime(this.element.REGTIME);

    return TemplateInfo({
      'element': this.element,
      'renameTrHTML': renameTrHTML,
      'permissionsTableHTML': permissionsTableHTML,
      'prettyRegTime': prettyRegTime,
    });
  }

  function _setup(context) {
    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);
    RenameTr.setup(RESOURCE, this.element.ID, context);
    PermissionsTable.setup(TAB_ID, RESOURCE, this.element, context);
  }
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/panels/template/html',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, buffer = "<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <table  class=\"info_table dataTable\" style=\"width:80%\">";
  stack1 = ((helper = (helper = helpers.prettyTemplateHTML || (depth0 != null ? depth0.prettyTemplateHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"prettyTemplateHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</table>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/panels/template/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/panels/template/panelId',['require'],function(require){
  return 'template_template_tab';
});
define('tabs/templates-tab/panels/template',['require','utils/locale','utils/humanize','hbs!./template/html','../tabId','./template/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');

  /*
    TEMPLATES
   */

  var TemplateInfo = require('hbs!./template/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./template/panelId');
  var RESOURCE = "Template"
  var XML_ROOT = "VMTEMPLATE"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Template");
    this.icon = "fa-file-o";

    this.element = info[XML_ROOT];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var prettyTemplateHTML = Humanize.prettyPrintJSON(this.element.TEMPLATE);

    return TemplateInfo({
      'element': this.element,
      'prettyTemplateHTML': prettyTemplateHTML
    });
  }

  function _setup(context) {
  }
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "      <dd class='"
    + escapeExpression(((helper = (helper = helpers.classes || (depth0 != null ? depth0.classes : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"classes","hash":{},"data":data}) : helper)))
    + "'>\n        <a href='#"
    + escapeExpression(((helper = (helper = helpers.wizardTabId || (depth0 != null ? depth0.wizardTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"wizardTabId","hash":{},"data":data}) : helper)))
    + "'> <i class='fa "
    + escapeExpression(((helper = (helper = helpers.icon || (depth0 != null ? depth0.icon : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"icon","hash":{},"data":data}) : helper)))
    + "'></i>\n          <br>"
    + escapeExpression(((helper = (helper = helpers.title || (depth0 != null ? depth0.title : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"title","hash":{},"data":data}) : helper)))
    + "</a>\n      </dd>\n";
},"3":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "        <div id=\""
    + escapeExpression(((helper = (helper = helpers.wizardTabId || (depth0 != null ? depth0.wizardTabId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"wizardTabId","hash":{},"data":data}) : helper)))
    + "\" class=\"wizard_tab content\">\n          ";
  stack1 = ((helper = (helper = helpers.contentHTML || (depth0 != null ? depth0.contentHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"contentHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n        </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"bordered-tabs\">\n    <dl id=\"template_create_tabs\" class=\"tabs right-info-tabs text-center\" data-tab>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.wizardTabs : depth0), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "    </dl>\n    <div id=\"template_create_tabs_content\" class=\"tabs-content\" style=\"min-height: 300px\">\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.wizardTabs : depth0), {"name":"each","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/advanced',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Advanced\" class=\"custom creation\">\n   <div class=\"row\">\n     <div class=\"large-12 columns\">\n       <p>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Write the Virtual Machine template here", {"name":"tr","hash":{},"data":data})))
    + "</p>\n     </div>\n   </div>\n   <div class=\"row\">\n     <div class=\"large-12 columns\">\n       <textarea id=\"template\" rows=\"15\" required></textarea>\n     </div>\n   </div>\n </form>";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/advanced', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/general/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/tip','templates/helpers/isFeatureEnabled'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "          <option value=\""
    + escapeExpression(((helper = (helper = helpers.path || (depth0 != null ? depth0.path : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"path","hash":{},"data":data}) : helper)))
    + "\">"
    + escapeExpression(((helper = (helper = helpers.title || (depth0 != null ? depth0.title : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"title","hash":{},"data":data}) : helper)))
    + "</option>\n";
},"3":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "    <fieldset>\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cost", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n        <div class=\"medium-6 columns\">\n          <label  for=\"MEMORY_COST\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Memory", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cost of each MB per hour", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"MEMORY_COST\" id=\"MEMORY_COST\" name=\"name\"/>\n        </div>\n        <div class=\"medium-6 columns\">\n          <label  for=\"CPU_COST\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cost of each CPU per hour", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"CPU_COST\" id=\"CPU_COST\" name=\"name\"/>\n        </div>\n        <div class=\"medium-6 columns left\">\n          <label  for=\"DISK_COST\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cost of each MB per hour", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n          </label>\n          <input type=\"text\" wizard_field=\"DISK_COST\" id=\"DISK_COST\" name=\"name\"/>\n        </div>\n    </fieldset>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "<div class=\"row\">\n  <div id=\"template_name_form\"  class=\"medium-6 columns\">\n    <label  for=\"NAME\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "\n      ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name that the VM will get for description purposes.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      <input type=\"text\" wizard_field=\"NAME\" id=\"NAME\" name=\"name\" required/>\n    </label>\n  </div>\n  <div  id=\"template_hypervisor_form\" class=\"medium-6 columns\">\n    <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hypervisor", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input type=\"radio\" wizard_field=\"HYPERVISOR\" name=\"hypervisor\" value=\"kvm\" id=\"kvmRadio\">\n    <label for=\"kvmRadio\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "KVM", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input type=\"radio\" wizard_field=\"HYPERVISOR\" name=\"hypervisor\" value=\"vmware\" id=\"vmwareRadio\">\n    <label for=\"vmwareRadio\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMware", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input type=\"radio\" wizard_field=\"HYPERVISOR\" name=\"hypervisor\" value=\"xen\" id=\"xenRadio\">\n    <label for=\"xenRadio\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Xen", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input type=\"radio\" wizard_field=\"HYPERVISOR\" name=\"hypervisor\" value=\"vcenter\" id=\"vcenterRadio\">\n    <label for=\"vcenterRadio\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "vCenter", {"name":"tr","hash":{},"data":data})))
    + "</label>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"medium-6 columns\">\n    <label  for=\"DESCRIPTION\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description", {"name":"tr","hash":{},"data":data})))
    + "\n      ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description of the template", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n    </label>\n    <textarea type=\"text\" wizard_field=\"DESCRIPTION\" id=\"DESCRIPTION\" name=\"DESCRIPTION\" style=\"height: 70px;\"/>\n  </div>\n  <div class=\"medium-6 columns\">\n    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <label  for=\"LOGO\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Logo", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Logo for the template.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <select wizard_field=\"LOGO\" id=\"LOGO\" name=\"LOGO\">\n          <option value=\"\"></option>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.logos : depth0), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "        </select>\n      </div>\n      <div id=\"template_create_logo\" class=\"text-center medium-6 columns\" style=\"margin-bottom: 15px\"></div>\n      <br></div>\n  </div>\n</div>\n<div class=\"row hypervisor only_vcenter\" style=\"display: none;\">\n  <div class=\"medium-6 columns\">\n    <label  for=\"vcenter_template_uuid\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "vCenter Template UUID", {"name":"tr","hash":{},"data":data})))
    + "\n      <span class=\"tip\"></span>\n    </label>\n    <input type=\"text\" id=\"vcenter_template_uuid\" name=\"name\"/>\n  </div>\n  <div class=\"medium-6 columns\">\n    <div class=\"row\">\n      <div class=\"large-6 columns\">\n        <input type=\"checkbox\" id=\"KEEP_DISKS\" name=\"KEEP_DISKS\"/>\n        <label  for=\"KEEP_DISKS\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Keep VM disks", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "If selected, VM disks won't be deleted after the VM finishes its lifecycle.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n      </div>\n    </div>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-7 columns\">\n    ";
  stack1 = ((helper = (helper = helpers.capacityInputsHTML || (depth0 != null ? depth0.capacityInputsHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"capacityInputsHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n  </div>\n  <div class=\"large-5 columns\">\n";
  stack1 = ((helpers.isFeatureEnabled || (depth0 && depth0.isFeatureEnabled) || helperMissing).call(depth0, "showback", {"name":"isFeatureEnabled","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <input type=\"checkbox\" id=\"sunstone_capacity_select\" name=\"name\"/>\n    <label  for=\"sunstone_capacity_select\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Do not allow to change capacity", {"name":"tr","hash":{},"data":data})))
    + "\n      ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Users using the cloud view will not be able to select a new capacity for this template", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n    </label>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <input type=\"checkbox\" id=\"sunstone_network_select\" name=\"name\"/>\n    <label  for=\"sunstone_network_select\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Do not allow to modify network configuration", {"name":"tr","hash":{},"data":data})))
    + "\n      ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Users using the cloud view will not be able to remove or add new NICs", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </label>\n  </div>\n</div>\n";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/general/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/create/wizard-tabs/general/wizardTabId',['require'],function(require){
  return 'generalTab';
});
define('tabs/templates-tab/form-panels/create/wizard-tabs/general',['require','utils/locale','utils/tips','./general/capacity-inputs','utils/wizard-fields','hbs!./general/html','./general/wizardTabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var CapacityInputs = require('./general/capacity-inputs');
  var WizardFields = require('utils/wizard-fields');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./general/html');

  /*
    CONSTANTS
   */

  var WIZARD_TAB_ID = require('./general/wizardTabId');
  var LOGOS = [
    {'path': "images/logos/arch.png",       'title': Locale.tr("Arch Linux")},
    {'path': "images/logos/centos.png",     'title': Locale.tr("CentOS")},
    {'path': "images/logos/debian.png",     'title': Locale.tr("Debian")},
    {'path': "images/logos/fedora.png",     'title': Locale.tr("Fedora")},
    {'path': "images/logos/linux.png",      'title': Locale.tr("Linux")},
    {'path': "images/logos/redhat.png",     'title': Locale.tr("Redhat")},
    {'path': "images/logos/ubuntu.png",     'title': Locale.tr("Ubuntu")},
    {'path': "images/logos/windowsxp.png",  'title': Locale.tr("Windows XP/2003")},
    {'path': "images/logos/windows8.png",   'title': Locale.tr("Windows 8")}
  ]

  /*
    CONSTRUCTOR
   */

  function WizardTab() {
    this.wizardTabId = WIZARD_TAB_ID;
    this.icon = 'fa-laptop';
    this.title = Locale.tr("General");
  }

  WizardTab.prototype.constructor = WizardTab;
  WizardTab.prototype.html = _html;
  WizardTab.prototype.setup = _setup;
  WizardTab.prototype.onShow = _onShow;
  WizardTab.prototype.retrieve = _retrieve;
  WizardTab.prototype.fill = _fill;

  return WizardTab;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'capacityInputsHTML': CapacityInputs.html(),
      'logos': LOGOS
    });
  }

  function _onShow(context, panelForm) {
    if (panelForm.action == 'create') {
      $('#template_name_form', context).show();
      $('#template_hypervisor_form', context).removeClass("left");
      $('#NAME', context).removeAttr('disabled');
      $('#NAME', context).attr("required", "");
    } else if (panelForm.action == 'update') {
      $('#template_name_form', context).hide();
      $('#template_hypervisor_form', context).addClass("left");
      $('#NAME', context).attr("disabled", "disabled");
      $('#NAME', context).removeAttr("required");
    }

    context.foundation('slider', 'reflow');
  }

  function _setup(context) {
    $(document).on('click', "[href='#" + this.wizardTabId + "']", function(){
      context.foundation('slider', 'reflow');
    });

    context.on("change", "#LOGO", function() {
      $("#template_create_logo", context).show();
      $("#template_create_logo", context).html('<span  class="">' +
          '<img src="' + $(this).val() + '">' +
        '</span>');
    });

    context.on("change", "input[name='hypervisor']", function() {
      // TODO define context (for example: this.closest('form'))
      $(".hypervisor").hide();
      $(".only_" + this.value).show();
    });

    CapacityInputs.setup(context);
  }

  function _retrieve(context) {
    var templateJSON = WizardFields.retrieve(context);

    if (templateJSON["HYPERVISOR"] == 'vcenter') {
      templateJSON["PUBLIC_CLOUD"] = {
        'TYPE': 'vcenter',
        'VM_TEMPLATE': $("#vcenter_template_uuid", context).val()
      }

      templateJSON["KEEP_DISKS_ON_DONE"] = $("#KEEP_DISKS", context).is(':checked')?"YES":"NO"
    }

    if ($('#sunstone_capacity_select:checked', context).length > 0) {
      templateJSON["SUNSTONE_CAPACITY_SELECT"] = "NO"
    }

    if ($('#sunstone_network_select:checked', context).length > 0) {
      templateJSON["SUNSTONE_NETWORK_SELECT"] = "NO"
    }

    return templateJSON;
  }

  function _fill(context, templateJSON) {
    if (templateJSON["SUNSTONE_CAPACITY_SELECT"] &&
          (templateJSON["SUNSTONE_CAPACITY_SELECT"].toUpperCase() == "NO")) {
      $("#sunstone_capacity_select", context).attr("checked", "checked");
      delete templateJSON["SUNSTONE_CAPACITY_SELECT"]
    }

    if (templateJSON["SUNSTONE_NETWORK_SELECT"] &&
          (templateJSON["SUNSTONE_NETWORK_SELECT"].toUpperCase() == "NO")) {
      $("#sunstone_network_select", context).attr("checked", "checked");
      delete templateJSON["SUNSTONE_NETWORK_SELECT"]
    }

    if (templateJSON["HYPERVISOR"] == 'vcenter' &&
        templateJSON["KEEP_DISKS_ON_DONE"].toLowerCase() == "yes" ) {
      $("#KEEP_DISKS", context).attr("checked", "checked");
      delete templateJSON["KEEP_DISKS_ON_DONE"]
    }

    if (templateJSON["HYPERVISOR"]) {
      $("input[name='hypervisor'][value='"+templateJSON["HYPERVISOR"]+"']", context).trigger("click")
      delete templateJSON["HYPERVISOR"];
    }

    WizardFields.fill(context, templateJSON);
  }
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/storage/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<dl class=\"tabs vertical\" id=\""
    + escapeExpression(((helper = (helper = helpers.linksContainerId || (depth0 != null ? depth0.linksContainerId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"linksContainerId","hash":{},"data":data}) : helper)))
    + "\" data-tab>\n  <dt class=\"text-center\">\n    <button href=\"\" type=\"button\" class=\"button tiny radius\" id=\"tf_btn_disks\">\n      <span class=\"fa fa-plus\"></span>\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add another disk", {"name":"tr","hash":{},"data":data})))
    + "\n    </button>\n  </dt>\n</dl>\n<div class=\"tabs-content vertical\" id=\""
    + escapeExpression(((helper = (helper = helpers.contentsContainerId || (depth0 != null ? depth0.contentsContainerId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"contentsContainerId","hash":{},"data":data}) : helper)))
    + "\"></div>\n";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/storage/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/create/wizard-tabs/storage/wizardTabId',['require'],function(require){
  return 'storageTab';
});
define('tabs/templates-tab/form-panels/create/wizard-tabs/storage',['require','foundation.tab','sunstone-config','utils/locale','utils/tips','utils/wizard-fields','./storage/disk-tab','hbs!./storage/html','./storage/wizardTabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var Config = require('sunstone-config');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var WizardFields = require('utils/wizard-fields');
  var DiskTab = require('./storage/disk-tab');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./storage/html');

  /*
    CONSTANTS
   */

  var WIZARD_TAB_ID = require('./storage/wizardTabId');
  var LINKS_CONTAINER_ID = 'template_create_storage_tabs';
  var CONTENTS_CONTAINER_ID = 'template_create_storage_tabs_content';

  /*
    CONSTRUCTOR
   */

  function WizardTab() {
    if (!Config.isTemplateCreationTabEnabled('storage')) {
      throw "Wizard Tab not enabled";
    }

    this.wizardTabId = WIZARD_TAB_ID;
    this.icon = 'fa-tasks';
    this.title = Locale.tr("Storage");
    this.classes = "hypervisor only_kvm only_vmware only_xen"
  }

  WizardTab.prototype.constructor = WizardTab;
  WizardTab.prototype.html = _html;
  WizardTab.prototype.setup = _setup;
  WizardTab.prototype.onShow = _onShow;
  WizardTab.prototype.retrieve = _retrieve;
  WizardTab.prototype.fill = _fill;
  WizardTab.prototype.renameTabLinks = _renameTabLinks;
  WizardTab.prototype.addDiskTab = _addDiskTab;

  return WizardTab;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'linksContainerId': LINKS_CONTAINER_ID,
      'contentsContainerId': CONTENTS_CONTAINER_ID
    });
  }

  function _onShow(context, panelForm) {
  }

  function _setup(context) {
    var that = this;

    that.numberOfDisks = 0;
    that.diskTabObjects = {};

    context.foundation('tab', 'reflow');

    // close icon: removing the tab on click
    context.on("click", "i.remove-tab", function() {
      var target = $(this).parent().attr("href");
      var dd = $(this).closest('dd');
      var dl = $(this).closest('dl');
      var content = $(target);

      dd.remove();
      content.remove();

      var diskId = content.attr("diskId");
      delete that.diskTabObjects[diskId];

      if (dd.attr("class") == 'active') {
        $('a', dl.children('dd').last()).click();
      }

      that.renameTabLinks(context);
    });

    context.on("click", "#tf_btn_disks", function() {
      that.addDiskTab(context);
      return false;
    });

    that.addDiskTab(context);
  }

  function _retrieve(context) {
    var templateJSON = {};
    var disksJSON = [];
    var diskJSON;
    $.each(this.diskTabObjects, function(id, diskTab) {
      diskJSON = diskTab.retrieve($('#' + diskTab.diskTabId, context))
      if (!$.isEmptyObject(diskJSON)) {disksJSON.push(diskJSON)};
    })

    if (disksJSON.length > 0) { templateJSON['DISK'] = disksJSON; };

    return templateJSON;
  }

  function _fill(context, templateJSON) {
    var that = this;
    var disks = templateJSON.DISK
    if (disks instanceof Array) {
      $.each(disks, function(diskId, diskJSON) {
        if (diskId > 0) {
          that.addDiskTab(context);
        }

        var diskTab = that.diskTabObjects[that.numberOfDisks];
        var diskContext = $('#' + diskTab.diskTabId, context);
        diskTab.fill(diskContext, diskJSON);
      });
    } else if (disks instanceof Object) {
      var diskTab = that.diskTabObjects[that.numberOfDisks];
      var diskContext = $('#' + diskTab.diskTabId, context);
      diskTab.fill(diskContext, disks);
    }

    if (templateJSON.DISK) {
      delete templateJSON.DISK;
    }
  }

  function _addDiskTab(context) {
    this.numberOfDisks++;
    var diskTab = new DiskTab(this.numberOfDisks);

    var content = $('<div id="' + diskTab.diskTabId + '" class="active disk wizard_internal_tab content">' +
        diskTab.html() +
      '</div>').appendTo($("#" + CONTENTS_CONTAINER_ID, context));

    var a = $("<dd class='active'>" +
       "<a href='#" + diskTab.diskTabId + "'>" + Locale.tr("DISK") + "</a>" +
      "</dd>").appendTo($("#" + LINKS_CONTAINER_ID, context));

    $("a", a).trigger("click");

    diskTab.setup(content);
    content.attr("diskId", this.numberOfDisks);

    this.renameTabLinks(context);
    this.diskTabObjects[this.numberOfDisks] = diskTab;
  }

  function _renameTabLinks(context) {
    $("#" + LINKS_CONTAINER_ID + " dd", context).each(function(index) {
      $("a", this).html(Locale.tr("Disk") + ' ' + index + " <i class='fa fa-times-circle remove-tab'></i>");
    })
  }
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/network/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/tip'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<dl class=\"tabs vertical\" id=\""
    + escapeExpression(((helper = (helper = helpers.linksContainerId || (depth0 != null ? depth0.linksContainerId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"linksContainerId","hash":{},"data":data}) : helper)))
    + "\" data-tab>\n  <dt class=\"text-center\">\n    <button href=\"#\" type=\"button\" class=\"button tiny radius\" id=\"tf_btn_nics\">\n      <span class=\"fa fa-plus\"></span>\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add another interface", {"name":"tr","hash":{},"data":data})))
    + "\n    </button>\n  </dt>\n</dl>\n<div class=\"tabs-content vertical\" id=\""
    + escapeExpression(((helper = (helper = helpers.contentsContainerId || (depth0 != null ? depth0.contentsContainerId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"contentsContainerId","hash":{},"data":data}) : helper)))
    + "\"></div>\n<div class=\"row\">\n  <div class=\"medium-6 columns\">\n    <label for=\"DEFAULT_MODEL\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Default model", {"name":"tr","hash":{},"data":data})))
    + "\n      ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Default value for all NICs. Hardware that will emulate the network interface. With Xen this is the type attribute of the vif.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </label>\n    <input type=\"text\" id=\"DEFAULT_MODEL\" name=\"DEFAULT_MODEL\"/>\n  </div>\n</div>\n";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/network/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/create/wizard-tabs/network/wizardTabId',['require'],function(require){
  return 'networkTab';
});
define('tabs/templates-tab/form-panels/create/wizard-tabs/network',['require','foundation.tab','sunstone-config','utils/locale','utils/tips','utils/wizard-fields','./network/nic-tab','hbs!./network/html','./network/wizardTabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var Config = require('sunstone-config');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var WizardFields = require('utils/wizard-fields');
  var NicTab = require('./network/nic-tab');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./network/html');

  /*
    CONSTANTS
   */

  var WIZARD_TAB_ID = require('./network/wizardTabId');
  var LINKS_CONTAINER_ID = 'template_create_network_tabs';
  var CONTENTS_CONTAINER_ID = 'template_create_network_tabs_content';

  /*
    CONSTRUCTOR
   */

  function WizardTab() {
    if (!Config.isTemplateCreationTabEnabled('network')) {
      throw "Wizard Tab not enabled";
    }

    this.wizardTabId = WIZARD_TAB_ID;
    this.icon = 'fa-globe';
    this.title = Locale.tr("Network");
    this.classes = "hypervisor only_kvm only_vmware only_xen only_vcenter"
  }

  WizardTab.prototype.constructor = WizardTab;
  WizardTab.prototype.html = _html;
  WizardTab.prototype.setup = _setup;
  WizardTab.prototype.onShow = _onShow;
  WizardTab.prototype.retrieve = _retrieve;
  WizardTab.prototype.fill = _fill;
  WizardTab.prototype.renameTabLinks = _renameTabLinks;
  WizardTab.prototype.addNicTab = _addNicTab;

  return WizardTab;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'linksContainerId': LINKS_CONTAINER_ID,
      'contentsContainerId': CONTENTS_CONTAINER_ID
    });
  }

  function _onShow(context, panelForm) {
  }

  function _setup(context) {
    var that = this;

    that.numberOfNics = 0;
    that.nicTabObjects = {};

    context.foundation('tab', 'reflow');

    // close icon: removing the tab on click
    context.on("click", "i.remove-tab", function() {
      var target = $(this).parent().attr("href");
      var dd = $(this).closest('dd');
      var dl = $(this).closest('dl');
      var content = $(target);

      dd.remove();
      content.remove();

      var nicId = content.attr("nicId");
      delete that.nicTabObjects[nicId];

      if (dd.attr("class") == 'active') {
        $('a', dl.children('dd').last()).click();
      }

      that.renameTabLinks(context);
    });

    context.on("click", "#tf_btn_nics", function() {
      that.addNicTab(context);
      return false;
    });

    that.addNicTab(context);
  }

  function _retrieve(context) {
    var templateJSON = {}
    var nicsJSON = [];
    var nicJSON;
    $.each(this.nicTabObjects, function(id, nicTab) {
      nicJSON = nicTab.retrieve($('#' + nicTab.nicTabId, context))
      if (!$.isEmptyObject(nicJSON)) {nicsJSON.push(nicJSON)};
    })

    if (nicsJSON.length > 0) { templateJSON['NIC'] = nicsJSON; };

    var nicDefault = $('#DEFAULT_MODEL', context).val();
    if (nicDefault) {
      templateJSON['NIC_DEFAULT'] = {
        'MODEL': nicDefault
      }
    }

    return templateJSON;
  }

  function _fill(context, templateJSON) {
    var that = this;
    var nics = templateJSON.NIC;
    if (nics instanceof Array) {
      $.each(nics, function(nicId, nicJSON) {
        if (nicId > 0) {
          that.addNicTab(context);
        }

        var nicTab = that.nicTabObjects[that.numberOfNics];
        var nicContext = $('#' + nicTab.nicTabId, context);
        nicTab.fill(nicContext, nicJSON);
      });
    } else if (nics instanceof Object) {
      var nicTab = that.nicTabObjects[that.numberOfNics];
      var nicContext = $('#' + nicTab.nicTabId, context);
      nicTab.fill(nicContext, nics);
    }

    if (templateJSON.NIC) {
      delete templateJSON.NIC;
    }

    var nicDefault = templateJSON.NIC_DEFAULT
    if (nicDefault != undefined) {
      if (nicDefault.MODEL) {
        $('#DEFAULT_MODEL', context).val(nicDefault.MODEL);
      }

      delete templateJSON.NIC_DEFAULT;
    }
  }

  function _addNicTab(context) {
    this.numberOfNics++;
    var nicTab = new NicTab(this.numberOfNics);

    var content = $('<div id="' + nicTab.nicTabId + '" class="active nic wizard_internal_tab content">' +
        nicTab.html() +
      '</div>').appendTo($("#" + CONTENTS_CONTAINER_ID, context));

    var a = $("<dd class='active'>" +
       "<a href='#" + nicTab.nicTabId + "'>" + Locale.tr("NIC") + "</a>" +
      "</dd>").appendTo($("#" + LINKS_CONTAINER_ID, context));

    $("a", a).trigger("click");

    nicTab.setup(content);
    content.attr("nicId", this.numberOfNics);

    this.renameTabLinks(context);
    this.nicTabObjects[this.numberOfNics] = nicTab;
  }

  function _renameTabLinks(context) {
    $("#" + LINKS_CONTAINER_ID + " dd", context).each(function(index) {
      $("a", this).html(Locale.tr("NIC") + ' ' + index + " <i class='fa fa-times-circle remove-tab'></i>");
    })
  }
});

define('tabs/files-tab/tabId',['require'],function(require){
  return 'files-tab';
});
define('tabs/files-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','utils/humanize','utils/notifier','opennebula/image','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var Notifier = require('utils/notifier');
  var OpenNebulaImage = require('opennebula/image');

  /*
    CONSTANTS
   */

  var RESOURCE = "File";
  var XML_ROOT = "IMAGE";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"]},
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    };

    this.columns = [
      Locale.tr("ID"),
      Locale.tr("Owner"),
      Locale.tr("Group"),
      Locale.tr("Name"),
      Locale.tr("Datastore"),
      Locale.tr("Size"),
      Locale.tr("Type"),
      Locale.tr("Registration time"),
      Locale.tr("Persistent"),
      Locale.tr("Status"),
      Locale.tr("#VMS"),
      Locale.tr("Target"),
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 4,
      "uname_index": 2,
      "select_resource": Locale.tr("Please select an file from the list"),
      "you_selected": Locale.tr("You selected the following file:"),
      "select_resource_multiple": Locale.tr("Please select one or more files from the list"),
      "you_selected_multiple": Locale.tr("You selected the following files:")
    };

    TabDataTable.call(this);
  };

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json.IMAGE;

    if (element.TYPE == OpenNebulaImage.TYPES.OS ||
        element.TYPE == OpenNebulaImage.TYPES.CDROM ||
        element.TYPE == OpenNebulaImage.TYPES.DATABLOCK) {
      return false;
    }

    return [
      '<input class="check_item" type="checkbox" id="'+RESOURCE.toLowerCase()+'_' +
                           element.ID + '" name="selected_items" value="' +
                           element.ID + '"/>',
      element.ID,
      element.UNAME,
      element.GNAME,
      element.NAME,
      element.DATASTORE,
      element.SIZE,
      OpenNebulaImage.typeStr(element.TYPE),
      Humanize.prettyTime(element.REGTIME),
      parseInt(element.PERSISTENT) ? "yes" : "no",
      OpenNebulaImage.stateStr(element.STATE),
      element.RUNNING_VMS,
      element.TEMPLATE.TARGET ? element.TEMPLATE.TARGET : '--'
    ];
  }
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/os/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/tip'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var lambda=this.lambda, escapeExpression=this.escapeExpression;
  return "            <option value=\""
    + escapeExpression(lambda(depth0, depth0))
    + "\">"
    + escapeExpression(lambda(depth0, depth0))
    + "</option>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "<div id=\"tabs-bootos\">\n  <dl class=\"tabs vertical\" data-tab>\n    <dd class=\"active\">\n      <a href=\"#bootTab\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Boot", {"name":"tr","hash":{},"data":data})))
    + "</a>\n    </dd>\n    <dd>\n      <a href=\"#kernelTab\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Kernel", {"name":"tr","hash":{},"data":data})))
    + "</a>\n    </dd>\n    <dd>\n      <a href=\"#ramdiskTab\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ramdisk", {"name":"tr","hash":{},"data":data})))
    + "</a>\n    </dd>\n    <dd>\n      <a href=\"#featuresTab\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Features", {"name":"tr","hash":{},"data":data})))
    + "</a>\n    </dd>\n  </dl>\n  <div class=\"tabs-content vertical\">\n    <div class=\"wizard_internal_tab active content\" id=\"bootTab\">\n      <div class=\"row\">\n        <div class=\"medium-4 columns\">\n          <label for=\"ARCH\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Arch", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU architecture to virtualization", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <select wizard_field=\"ARCH\" id=\"ARCH\" name=\"arch\">\n            <option id=\"no_arch\" name=\"no_arch\" value=\"\"></option>\n            <option value=\"i686\">i686</option>\n            <option value=\"x86_64\">x86_64</option>\n          </select>\n        </div>\n        <div class=\"medium-8 columns hypervisor only_kvm\">\n          <label for=\"MACHINE\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Machine type", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "libvirt machine type, only for KVM", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"MACHINE\" id=\"MACHINE\" name=\"machine\" />\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"medium-4 columns\">\n          <label for=\"ROOT\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Root", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Device to be mounted as root", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"ROOT\" id=\"ROOT\" name=\"root\"/>\n        </div>\n        <div class=\"medium-8 columns hypervisor only_vmware\">\n          <label for=\"GUESTOS\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Guest OS", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Set the OS of the VM, only for VMware", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <select wizard_field=\"GUESTOS\" id=\"GUESTOS\" name=\"GUESTOS\">\n            <option id=\"no_guestos\" name=\"no_guestos\" value=\"\"></option>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.guestOS : depth0), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "          </select>\n        </div>\n      </div>\n      <br>\n      <div class=\"row\">\n        <div class=\"medium-4 columns\">\n          <label for=\"BOOT_0\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "1st Boot", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "1st Boot device type", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <select id=\"BOOT_0\" name=\"boot\">\n            <option id=\"no_boot\" name=\"no_boot\" value=\"\"></option>\n            <option value=\"hd\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "HD", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"fd\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "FD", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"cdrom\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CDROM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"network\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "NETWORK", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"medium-4 columns\">\n          <label for=\"BOOT_1\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "2nd Boot", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "2nd Boot device type", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <select id=\"BOOT_1\" name=\"boot\">\n            <option id=\"no_boot\" name=\"no_boot\" value=\"\"></option>\n            <option value=\"hd\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "HD", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"fd\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "FD", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"cdrom\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CDROM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"network\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "NETWORK", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"medium-4 columns\">\n          <label for=\"BOOT_2\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "3rd Boot", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "3rd Boot device type", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <select id=\"BOOT_2\" name=\"boot\">\n            <option id=\"no_boot\" name=\"no_boot\" value=\"\"></option>\n            <option value=\"hd\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "HD", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"fd\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "FD", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"cdrom\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CDROM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"network\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "NETWORK", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label for=\"KERNEL_CMD\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Kernel cmd", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Arguments for the booting kernel", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"KERNEL_CMD\" id=\"KERNEL_CMD\" name=\"kernel_cmd\" />\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label for=\"BOOTLOADER\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Bootloader", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Path to the bootloader executable", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"BOOTLOADER\" id=\"BOOTLOADER\" name=\"bootloader\" />\n        </div>\n      </div>\n    </div>\n    <div id=\"kernelTab\" class=\"wizard_internal_tab content\">\n      <div class=\"row\">\n        <div class=\"large-12 columns text-center\">\n          <input id=\"radioKernelDs\" type=\"radio\" name=\"kernel_type\" value=\"kernel_ds\" checked/>\n          <label for=\"radioKernelDs\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Registered Image", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <input id=\"radioKernelPath\" type=\"radio\" name=\"kernel_type\" value=\"kernel_path\"/>\n          <label for=\"radioKernelPath\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Remote PATH", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        </div>\n      </div>\n      <br>\n      <div class=\"kernel_ds\">\n        ";
  stack1 = ((helper = (helper = helpers.kernelFilesTableHTML || (depth0 != null ? depth0.kernelFilesTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"kernelFilesTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        <div id=\"kernel_ds_inputs\" class=\"row\">\n          <div class=\"large-12 columns\">\n            <label for=\"KERNEL_DS\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "KERNEL_DS", {"name":"tr","hash":{},"data":data})))
    + "</label>\n            <input type=\"text\" wizard_field=\"KERNEL_DS\" id=\"KERNEL_DS\" name=\"KERNEL_DS\"/>\n          </div>\n        </div>\n      </div>\n      <div id=\"kernel_path_inputs\" class=\"kernel_path hidden row\">\n        <div class=\"large-12 columns\">\n          <label for=\"KERNEL\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PATH", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Path to the OS kernel to boot the image", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"KERNEL\" id=\"KERNEL\" name=\"kernel\" />\n        </div>\n      </div>\n    </div>\n    <div id=\"ramdiskTab\" class=\"wizard_internal_tab content\">\n      <div class=\"row\">\n        <div class=\"large-12 columns text-center\">\n          <input id=\"radioInintrdDs\" type=\"radio\" name=\"initrd_type\" value=\"initrd_ds\" checked>\n          <label for=\"radioInintrdDs\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Registered Image ", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <input id=\"radioInitrdPath\" type=\"radio\" name=\"initrd_type\" value=\"initrd_path\">\n          <label for=\"radioInitrdPath\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Remote PATH", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        </div>\n      </div>\n      <br>\n      <div class=\"initrd_ds\">\n        ";
  stack1 = ((helper = (helper = helpers.initrdFilesTableHTML || (depth0 != null ? depth0.initrdFilesTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"initrdFilesTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <label for=\"INITRD_DS\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "INITRD_DS", {"name":"tr","hash":{},"data":data})))
    + "</label>\n            <input type=\"text\" wizard_field=\"INITRD_DS\" id=\"INITRD_DS\" name=\"initrd_id\"/>\n          </div>\n        </div>\n      </div>\n      <div id=\"initrd_path_inputs\" class=\"initrd_path hidden\">\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <label for=\"INITRD\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PATH", {"name":"tr","hash":{},"data":data})))
    + "\n              ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Path to the initrd image", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n            </label>\n            <input type=\"text\" wizard_field=\"INITRD\" id=\"INITRD\" name=\"initrd\"/>\n          </div>\n        </div>\n      </div>\n    </div>\n    <div class=\"wizard_internal_tab content\" id=\"featuresTab\">\n      <div class=\"row\">\n        <div class=\"medium-6 columns\">\n          <label for=\"ACPI\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ACPI", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add support in the VM for Advanced Configuration and Power Interface (ACPI)", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <select wizard_field=\"ACPI\" id=\"ACPI\" name=\"acpi\">\n            <option id=\"no_apci\" name=\"no_apci\" value=\"\"></option>\n            <option value=\"yes\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Yes", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"no\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "No", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"medium-6 columns\">\n          <label for=\"PAE\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PAE", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add support in the VM for Physical Address Extension (PAE)", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <select wizard_field=\"PAE\" id=\"PAE\" name=\"pae\">\n            <option id=\"no_pae\" name=\"no_pae\" value=\"\"></option>\n            <option value=\"yes\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Yes", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"no\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "No", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"medium-6 columns\">\n          <label for=\"APIC\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "APIC", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Enables the advanced programmable IRQ management.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <select wizard_field=\"APIC\" id=\"APIC\" name=\"apic\">\n            <option id=\"no_apic\" name=\"no_apic\" value=\"\"></option>\n            <option value=\"yes\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Yes", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"no\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "No", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"medium-6 columns\">\n          <label for=\"HYPERV\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "HYPERV", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add support in the VM for hyper-v features (HYPERV)", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <select wizard_field=\"HYPERV\" id=\"HYPERV\" name=\"hyperv\">\n            <option id=\"no_hyperv\" name=\"no_hyperv\" value=\"\"></option>\n            <option value=\"yes\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Yes", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"no\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "No", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"medium-6 columns\">\n          <label for=\"LOCALTIME\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Localtime", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "The guest clock will be synchronized to the hosts configured timezone when booted.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <select wizard_field=\"LOCALTIME\" id=\"LOCALTIME\" name=\"localtime\">\n            <option id=\"no_localtime\" name=\"no_localtime\" value=\"\"></option>\n            <option value=\"yes\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Yes", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"no\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "No", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"medium-6 columns hypervisor only_xen\">\n          <label for=\"DEVICE_MODEL\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Device model", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Used to change the IO emulator in Xen HVM. Only XEN.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"DEVICE_MODEL\" id=\"DEVICE_MODEL\" name=\"device_model\"/>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"medium-6 columns hypervisor only_vmware\">\n          <label for=\"PCIBRIDGE\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PCI BRIDGE", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, " Adds a PCI Controller that provides bridge-to-bridge capability, only for VMware.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n          </label>\n          <select wizard_field=\"PCIBRIDGE\" id=\"PCIBRIDGE\" name=\"PCIBRIDGE\">\n            <option id=\"no_pcibridge\" name=\"no_pcibridge\" value=\"\"></option>\n            <option value=\"0\">0</option>\n            <option value=\"1\">1</option>\n            <option value=\"2\">2</option>\n            <option value=\"3\">3</option>\n            <option value=\"4\">4</option>\n            <option value=\"5\">5</option>\n            <option value=\"6\">6</option>\n            <option value=\"7\">7</option>\n            <option value=\"8\">8</option>\n            <option value=\"9\">9</option>\n            <option value=\"10\">10</option>\n          </select>\n        </div>\n      </div>\n    </div>\n  </div>\n</div>\n";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/os/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/create/wizard-tabs/os/wizardTabId',['require'],function(require){
  return 'osTab';
});
define('tabs/templates-tab/form-panels/create/wizard-tabs/os',['require','foundation.tab','sunstone-config','utils/locale','utils/tips','utils/wizard-fields','tabs/files-tab/datatable','hbs!./os/html','./os/wizardTabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var Config = require('sunstone-config');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var WizardFields = require('utils/wizard-fields');
  var FilesTable = require('tabs/files-tab/datatable');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./os/html');

  /*
    CONSTANTS
   */

  var WIZARD_TAB_ID = require('./os/wizardTabId');
  var GUESTOS = [
    'asianux3_64Guest',
    'asianux3Guest',
    'asianux4_64Guest',
    'asianux4Guest',
    'centos64Guest',
    'centosGuest',
    'darwin64Guest',
    'darwinGuest',
    'debian4_64Guest',
    'debian4Guest',
    'debian5_64Guest',
    'debian5Guest',
    'dosGuest',
    'eComStationGuest',
    'freebsd64Guest',
    'freebsdGuest',
    'mandriva64Guest',
    'mandrivaGuest',
    'netware4Guest',
    'netware5Guest',
    'netware6Guest',
    'nld9Guest',
    'oesGuest',
    'openServer5Guest',
    'openServer6Guest',
    'oracleLinux64Guest',
    'oracleLinuxGuest',
    'os2Guest',
    'other24xLinux64Guest',
    'other24xLinuxGuest',
    'other26xLinux64Guest',
    'other26xLinuxGuest',
    'otherGuest',
    'otherGuest64',
    'otherLinux64Guest',
    'otherLinuxGuest',
    'redhatGuest',
    'rhel2Guest',
    'rhel3_64Guest',
    'rhel3Guest',
    'rhel4_64Guest',
    'rhel4Guest',
    'rhel5_64Guest',
    'rhel5Guest',
    'rhel6_64Guest',
    'rhel6Guest',
    'sjdsGuest',
    'sles10_64Guest',
    'sles10Guest',
    'sles11_64Guest',
    'sles11Guest',
    'sles64Guest',
    'slesGuest',
    'solaris10_64Guest',
    'solaris10Guest',
    'solaris6Guest',
    'solaris7Guest',
    'solaris8Guest',
    'solaris9Guest',
    'suse64Guest',
    'suseGuest',
    'turboLinux64Guest',
    'turboLinuxGuest',
    'ubuntu64Guest',
    'ubuntuGuest',
    'unixWare7Guest',
    'win2000AdvServGuest',
    'win2000ProGuest',
    'win2000ServGuest',
    'win31Guest',
    'win95Guest',
    'win98Guest',
    'windows7_64Guest',
    'windows7Guest',
    'windows7Server64Guest',
    'winLonghorn64Guest',
    'winLonghornGuest',
    'winMeGuest',
    'winNetBusinessGuest',
    'winNetDatacenter64Guest',
    'winNetDatacenterGuest',
    'winNetEnterprise64Guest',
    'winNetEnterpriseGuest',
    'winNetStandard64Guest',
    'winNetStandardGuest',
    'winNetWebGuest',
    'winNTGuest',
    'winVista64Guest',
    'winVistaGuest',
    'winXPHomeGuest',
    'winXPPro64Guest',
    'winXPProGues'
  ]

  /*
    CONSTRUCTOR
   */

  function WizardTab() {
    if (!Config.isTemplateCreationTabEnabled('os_booting')) {
      throw "Wizard Tab not enabled";
    }

    this.wizardTabId = WIZARD_TAB_ID;
    this.icon = 'fa-power-off';
    this.title = Locale.tr("OS Booting");
    this.classes = "hypervisor only_kvm only_vmware only_xen"

    this.kernelFilesTable = new FilesTable(this.wizardTabId + 'KernelTable', {
      'select': true,
      'selectOptions': {
        "filter_fn": function(file) { return file.TYPE == 3; } // KERNEL
      }
    });
    this.initrdFilesTable = new FilesTable(this.wizardTabId + 'InitrdTable', {
      'select': true,
      'selectOptions': {
        "filter_fn": function(file) { return file.TYPE == 4; } // RAMDISK
      }
    });
  }

  WizardTab.prototype.constructor = WizardTab;
  WizardTab.prototype.html = _html;
  WizardTab.prototype.setup = _setup;
  WizardTab.prototype.onShow = _onShow;
  WizardTab.prototype.retrieve = _retrieve;
  WizardTab.prototype.fill = _fill;

  return WizardTab;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'guestOS': GUESTOS,
      'kernelFilesTableHTML': this.kernelFilesTable.dataTableHTML,
      'initrdFilesTableHTML': this.initrdFilesTable.dataTableHTML
    });
  }

  function _onShow(context, panelForm) {
  }

  function _setup(context) {
    var that = this;
    context.foundation('tab', 'reflow');

    var kernelDSContext = $(".kernel_ds",  context);
    var kernelDSInputsContext = $("#kernel_path_inputs",  context);
    $("input[name='kernel_type']", context).change(function() {
      if ($("input[name='kernel_type']:checked", context).val() == "kernel_ds") {
        kernelDSContext.toggle();
        kernelDSInputsContext.hide();
        $("[wizard_field]", kernelDSContext).prop('wizard_field_disabled', false);
        $("[wizard_field]", kernelDSInputsContext).prop('wizard_field_disabled', true);
      } else {
        kernelDSContext.hide();
        kernelDSInputsContext.toggle();
        $("[wizard_field]", kernelDSInputsContext).prop('wizard_field_disabled', false);
        $("[wizard_field]", kernelDSContext).prop('wizard_field_disabled', true);
      }
    });

    var initrdDSContext = $(".initrd_ds",  context);
    var initrdDSInputsContext = $("#initrd_path_inputs",  context);
    $("input[name='initrd_type']", context).change(function() {
      if ($("input[name='initrd_type']:checked", context).val() == "initrd_ds") {
        initrdDSContext.toggle();
        initrdDSInputsContext.hide();
        $("[wizard_field]", initrdDSContext).prop('wizard_field_disabled', false);
        $("[wizard_field]", initrdDSInputsContext).prop('wizard_field_disabled', true);
      } else {
        initrdDSContext.hide();
        initrdDSInputsContext.toggle();
        $("[wizard_field]", initrdDSInputsContext).prop('wizard_field_disabled', false);
        $("[wizard_field]", initrdDSContext).prop('wizard_field_disabled', true);
      }
    });

    that.kernelFilesTable.initialize({
      'selectOptions': {
        'select_callback': function(aData, options) {
          $('#KERNEL_DS', context).val("$FILE[IMAGE_ID="+ aData[options.id_index] +"]");
        }
      }
    });
    that.kernelFilesTable.refreshResourceTableSelect();


    that.initrdFilesTable.initialize({
      'selectOptions': {
        'select_callback': function(aData, options) {
          $('#INITRD_DS', context).val("$FILE[IMAGE_ID="+ aData[options.id_index] +"]");
        }
      }
    });
    that.initrdFilesTable.refreshResourceTableSelect();
  }

  function _retrieve(context) {
    var templateJSON = {};
    var osJSON = {}
    $.extend(osJSON, WizardFields.retrieve('#bootTab', context));
    $.extend(osJSON, WizardFields.retrieve('#kernelTab', context));
    $.extend(osJSON, WizardFields.retrieve('#ramdiskTab', context));

    var boot = "";
    var val;
    for (var i = 0; i < 3; i++) {
      val = $('#BOOT_' + i, context).val();
      if (val != undefined && val.length > 0) {
        if (boot.length > 0) {boot += ","}
        boot += val;
      }
    }

    if (boot.length > 0) {
      osJSON["BOOT"] = boot;
    }

    if (!$.isEmptyObject(osJSON)) { templateJSON['OS'] = osJSON; };

    var featuresJSON = WizardFields.retrieve('#featuresTab', context)
    if (!$.isEmptyObject(featuresJSON)) { templateJSON['FEATURES'] = featuresJSON; };

    return templateJSON;
  }

  function _fill(context, templateJSON) {
    var osJSON = templateJSON['OS'];
    if (osJSON) {
      WizardFields.fill(context, osJSON);

      if (osJSON && osJSON['BOOT']) {
        var boot_vals = osJSON['BOOT'].split(",");

        for (var i = 0; i < 3 && i < boot_vals.length; i++) {
          $('#BOOT_' + i, context).val(boot_vals[i]);
        }
      }

      delete templateJSON['OS'];
    }

    var featuresJSON = templateJSON['FEATURES'];
    if (featuresJSON) {
      WizardFields.fill(context, featuresJSON);
      delete templateJSON['FEATURES'];
    }
  }
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/io/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/tip'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row\">\n  <div class=\"medium-6 columns graphics\">\n    <fieldset>\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Graphics", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"large-12 columns text-center\">\n          <input type=\"radio\" name=\"graphics_type\" ID=\"radioVncType\" value=\"VNC\">\n          <label for=\"radioVncType\">VNC</label>\n          <input type=\"radio\" name=\"graphics_type\" ID=\"radioSdlType\" value=\"SDL\" class=\"hypervisor only_kvm only_vmware only_xen\" >\n          <label class=\"hypervisor only_kvm only_vmware only_xen\"  for=\"radioSdlType\">SDL</label>\n          <input type=\"radio\" name=\"graphics_type\" ID=\"radioSpiceType\" value=\"SPICE\" class=\"hypervisor only_kvm only_vmware only_xen\" >\n          <label  class=\"hypervisor only_kvm only_vmware only_xen\" for=\"radioSpiceType\">SPICE</label>\n        </div>\n      </div>\n      <br>\n      <div class=\"row\">\n        <input type=\"hidden\" wizard_field=\"TYPE\" name=\"graphics_type\" ID=\"TYPE\">\n        <div class=\"large-12 columns\">\n          <label for=\"LISTEN\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Listen IP", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IP to listen on", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"LISTEN\" id=\"LISTEN\" name=\"graphics_ip\" />\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"medium-6 columns\">\n          <label for=\"PORT\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Port", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Port for the VNC/SPICE server", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"PORT\" id=\"PORT\" name=\"port\" />\n        </div>\n        <div class=\"medium-6 columns\">\n          <label for=\"KEYMAP\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Keymap", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Keyboard configuration locale to use in the VNC/SPICE display", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"KEYMAP\" id=\"KEYMAP\" name=\"keymap\" />\n        </div>\n      </div>\n      <div class=\"row hypervisor only_kvm only_vmware only_xen\">\n        <div class=\"large-12 columns\">\n          <label for=\"PASSWD\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Password", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Password for the VNC/SPICE server", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"PASSWD\" id=\"PASSWD\" name=\"graphics_pw\" />\n        </div>\n      </div>\n      <div class=\"row hypervisor only_kvm only_vmware only_xen\">\n        <div class=\"columns large-12\">\n          <input type=\"checkbox\" name=\"RANDOM_PASSWD\" id=\"RANDOM_PASSWD\">\n          <label for=\"RANDOM_PASSWD\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Generate Random Password", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "A random password will be generated for each VM, and will be included in the VM information", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n          </label>\n        </div>\n      </div>\n    </fieldset>\n  </div>\n  <div class=\"medium-6 columns inputs hypervisor only_kvm only_vmware only_xen\">\n    <fieldset>\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Inputs", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"medium-5 columns\">\n          <select id=\"INPUT_TYPE\" name=\"input_type\">\n            <option id=\"no_type\" name=\"no_type\" value=\"\"></option>\n            <option value=\"mouse\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Mouse", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"tablet\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Tablet", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"medium-4 columns\">\n          <select id=\"INPUT_BUS\" name=\"input_bus\">\n            <option id=\"no_input\" name=\"no_input\" value=\"\"></option>\n            <option value=\"usb\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "USB", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"ps2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PS2", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"xen\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "XEN", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"medium-3 columns\">\n          <a href=\"#\" class=\"button small radius secondary\" id=\"add_input\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add", {"name":"tr","hash":{},"data":data})))
    + "</a>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <table id=\"input_table\" class=\"dataTable policies_table\">\n            <thead>\n              <tr>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "TYPE", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "BUS", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                <th></th>\n              </tr>\n            </thead>\n            <tbody id=\"tbodyinput\">\n              <tr></tr>\n              <tr></tr>\n            </tbody>\n          </table>\n        </div>\n      </div>\n    </fieldset>\n  </div>\n</div>\n";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/io/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/create/wizard-tabs/io/wizardTabId',['require'],function(require){
  return 'ioTab';
});
define('tabs/templates-tab/form-panels/create/wizard-tabs/io',['require','sunstone-config','utils/locale','utils/tips','utils/wizard-fields','utils/template-utils','hbs!./io/html','./io/wizardTabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Config = require('sunstone-config');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var WizardFields = require('utils/wizard-fields');
  var TemplateUtils = require('utils/template-utils');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./io/html');

  /*
    CONSTANTS
   */

  var WIZARD_TAB_ID = require('./io/wizardTabId');

  /*
    CONSTRUCTOR
   */

  function WizardTab() {
    if (!Config.isTemplateCreationTabEnabled('input_output')) {
      throw "Wizard Tab not enabled";
    }

    this.wizardTabId = WIZARD_TAB_ID;
    this.icon = 'fa-exchange';
    this.title = Locale.tr("Input/Output");
  }

  WizardTab.prototype.constructor = WizardTab;
  WizardTab.prototype.html = _html;
  WizardTab.prototype.setup = _setup;
  WizardTab.prototype.onShow = _onShow;
  WizardTab.prototype.retrieve = _retrieve;
  WizardTab.prototype.fill = _fill;

  return WizardTab;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML();
  }

  function _onShow(context, panelForm) {
  }

  function _setup(context) {
    $("input[name='graphics_type']", context).change(function() {
      $("#TYPE", context).val($(this).attr("value"))
      $("#LISTEN", context).val("0.0.0.0")
    });

    context.off("click", '#add_input');
    context.on("click", '#add_input', function() {
      var table = $('#input_table', context)[0];
      var rowCount = table.rows.length;
      var row = table.insertRow(-1);
      $(row).addClass("vm_param");

      var cell1 = row.insertCell(0);
      var element1 = document.createElement("input");
      element1.id = "INPUT_TYPE"
      element1.type = "text";
      element1.value = $('select#INPUT_TYPE', context).val()
      cell1.appendChild(element1);

      var cell2 = row.insertCell(1);
      var element2 = document.createElement("input");
      element2.id = "INPUT_BUS"
      element2.type = "text";
      element2.value = $('select#INPUT_BUS', context).val()
      cell2.appendChild(element2);

      var cell3 = row.insertCell(2);
      cell3.innerHTML = "<i class='fa fa-times-circle fa fa-lg remove-tab'></i>";
    });

    context.off('click', "i.remove-tab");
    context.on('click', "i.remove-tab", function() {
      $(this).closest("tr").remove()
    });
  }

  function _retrieve(context) {
    var templateJSON = {};
    var graphicsJSON = WizardFields.retrieve(context);

    if (!$.isEmptyObject(graphicsJSON) && $("#RANDOM_PASSWD:checked", context).length > 0) {
      graphicsJSON["RANDOM_PASSWD"] = "YES";
    }

    if (!$.isEmptyObject(graphicsJSON)) { templateJSON['GRAPHICS'] = graphicsJSON; };

    var inputsJSON = [];
    $('#input_table tr', context).each(function() {
      if ($('#INPUT_TYPE', $(this)).val()) {
        inputsJSON.push({
          'TYPE': $('#INPUT_TYPE', $(this)).val(),
          'BUS': $('#INPUT_BUS', $(this)).val()
        });
      }
    });

    if (!$.isEmptyObject(inputsJSON)) { templateJSON['INPUTS'] = inputsJSON; };

    return templateJSON;
  }

  function _fill(context, templateJSON) {
    var graphicsJSON = templateJSON['GRAPHICS'];
    if (graphicsJSON) {
      var type = graphicsJSON.TYPE;
      if (type) {
        $("input[value='" + type.toUpperCase() + "']").click();
      }

      if (graphicsJSON["RANDOM_PASSWD"] == "YES") {
        $("#RANDOM_PASSWD", context).attr("checked", "checked");
      }

      WizardFields.fill(context, graphicsJSON);
      delete templateJSON['GRAPHICS']
    }

    var inputsJSON = templateJSON['INPUTS'];
    if (inputsJSON) {
      if (!(inputsJSON instanceof Array)) {
        inputsJSON = [inputsJSON];
      }

      $.each(inputsJSON, function() {
        var table = $('#input_table', context)[0];
        var rowCount = table.rows.length;
        var row = table.insertRow(rowCount);

        var cell1 = row.insertCell(0);
        var element1 = document.createElement("input");
        element1.id = "INPUT_TYPE";
        element1.type = "text";
        element1.value = TemplateUtils.htmlDecode(this.TYPE);
        cell1.appendChild(element1);

        var cell2 = row.insertCell(1);
        var element2 = document.createElement("input");
        element2.id = "INPUT_BUS";
        element2.type = "text";
        element2.value = TemplateUtils.htmlDecode(this.BUS);
        cell2.appendChild(element2);

        var cell3 = row.insertCell(2);
        cell3.innerHTML = "<i class='fa fa-times-circle fa fa-lg remove-tab'></i>";
      });

      delete templateJSON['INPUTS']
    }
  }
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/context/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/tip'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "<dl id=\"context_tabs\" class=\"tabs vertical\" data-tab>\n  <dd class=\"active\">\n    <a href=\"#netsshTab\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Configuration", {"name":"tr","hash":{},"data":data})))
    + "</a>\n  </dd>\n  <dd>\n    <a href=\"#filesTab\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Files", {"name":"tr","hash":{},"data":data})))
    + "</a>\n  </dd>\n  <dd>\n    <a href=\"#zcustomTab\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom vars", {"name":"tr","hash":{},"data":data})))
    + "</a>\n  </dd>\n</dl>\n<div class=\"tabs-content vertical\">\n  <div class=\"wizard_internal_tab active content\" id=\"netsshTab\">\n    <div class=\"row\">\n      <div class=\"columns medium-6\">\n        <div class=\"row\">\n          <div class=\"columns large-12\">\n            <input type=\"checkbox\" name=\"ssh_context\" id=\"ssh_context\" checked>\n            <label for=\"ssh_context\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "  Add SSH contextualization", {"name":"tr","hash":{},"data":data})))
    + "\n              ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add an ssh public key to the context. If the Public Key textarea is empty then the user variable SSH_PUBLIC_KEY will be used.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n            </label>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <label for=\"ssh_public_key\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Public Key", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n            <textarea rows=\"2\" type=\"text\" id=\"ssh_public_key\" name=\"ssh_public_key\" />\n          </div>\n        </div>\n      </div>\n      <div class=\"columns medium-6\">\n        <div class=\"row\">\n          <div class=\"columns large-12\">\n            <input type=\"checkbox\" name=\"network_context\" id=\"network_context\" checked>\n            <label for=\"network_context\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "  Add Network contextualization", {"name":"tr","hash":{},"data":data})))
    + "\n              ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add network contextualization parameters. For each NIC defined in the NETWORK section, ETH$i_IP, ETH$i_NETWORK... parameters will be included in the CONTEXT section and will be available in the Virtual Machine", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n            </label>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"columns large-12\">\n            <input type=\"checkbox\" name=\"token_context\" id=\"token_context\">\n            <label for=\"token_context\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "  Add OneGate token", {"name":"tr","hash":{},"data":data})))
    + "\n              ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add a file (token.txt) to the context contaning the token to push custom metrics to the VirtualMachine through OneGate", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n            </label>\n          </div>\n        </div>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <label for=\"START_SCRIPT\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Start Script", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Text of the script executed when the machine starts up. It can contain shebang in case it is not shell script. For example 'yum upgrade'", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <textarea rows=\"4\" type=\"text\" id=\"START_SCRIPT\" name=\"START_SCRIPT\" />\n        <input type=\"checkbox\" name=\"ENCODE_START_SCRIPT\" id=\"ENCODE_START_SCRIPT\">\n        <label for=\"ENCODE_START_SCRIPT\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "  Encode Script in Base64", {"name":"tr","hash":{},"data":data})))
    + "\n        </label>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <table class=\"service_custom_attrs policies_table dataTable\">\n          <thead>\n            <tr>\n              <th colspan=\"4\" style=\"font-size: 16px !important\"> <i class=\"fa fa-lg fa-fw fa-cogs off-color\"/>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User Inputs", {"name":"tr","hash":{},"data":data})))
    + "\n                ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "These attributes must be provided by the user when a new VM is intantatiated using each template. They will be included in the VM context", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n              </th>\n            </tr>\n          </thead>\n          <thead>\n            <tr>\n              <th style=\"width:30%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th style=\"width:20%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th style=\"width:50%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th style=\"width:3%\"></th>\n            </tr>\n          </thead>\n          <tbody></tbody>\n          <tfoot>\n            <tr>\n              <td colspan=\"4\">\n                <a href=\"#\" class=\"add_service_custom_attr button small small-12 secondary radius\">\n                  <i class=\"fa fa-plus\"></i>\n                  "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add another attribute", {"name":"tr","hash":{},"data":data})))
    + "\n                </a>\n              </td>\n            </tr>\n          </tfoot>\n        </table>\n      </div>\n    </div>\n  </div>\n  <div class=\"wizard_internal_tab content\" id=\"filesTab\">\n    ";
  stack1 = ((helper = (helper = helpers.contextFilesTableHTML || (depth0 != null ? depth0.contextFilesTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"contextFilesTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <label for=\"FILES_DS\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "FILES_DS", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Raw String for the FILE_DS attribute of the VM template, representing files that will be included in the contextualization image. Each file must be stored in a FILE_DS Datastore and must be of type CONTEXT", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"FILES_DS\" id=\"FILES_DS\" name=\"FILES_DS\" />\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <label for=\"INIT_SCRIPTS\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Init scripts", {"name":"tr","hash":{},"data":data})))
    + "\n          ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "If the VM uses the OpenNebula contextualization package the init.sh file is executed by default. When the init script added is not called init.sh or more than one init script is added, this list contains the scripts to run and the order. Ex. “init.sh users.sh mysql.sh”", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </label>\n        <input type=\"text\" wizard_field=\"FILES_DS\" id=\"INIT_SCRIPTS\" name=\"INIT_SCRIPTS\" />\n      </div>\n    </div>\n  </div>\n  <div class=\"wizard_internal_tab content\" id=\"zcustomTab\">\n    ";
  stack1 = ((helper = (helper = helpers.customTagsTableHTML || (depth0 != null ? depth0.customTagsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customTagsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n  </div>\n</div>\n";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/context/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/create/wizard-tabs/context/wizardTabId',['require'],function(require){
  return 'contextTab';
});
define('tabs/templates-tab/form-panels/create/wizard-tabs/context',['require','sunstone-config','utils/locale','utils/tips','utils/wizard-fields','utils/template-utils','utils/custom-tags-table','tabs/files-tab/datatable','hbs!./context/html','./context/wizardTabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Config = require('sunstone-config');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var WizardFields = require('utils/wizard-fields');
  var TemplateUtils = require('utils/template-utils');
  var CustomTagsTable = require('utils/custom-tags-table');
  var FilesTable = require('tabs/files-tab/datatable')

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./context/html');

  /*
    CONSTANTS
   */

  var WIZARD_TAB_ID = require('./context/wizardTabId');

  /*
    CONSTRUCTOR
   */

  function WizardTab() {
    if (!Config.isTemplateCreationTabEnabled('context')) {
      throw "Wizard Tab not enabled";
    }

    this.wizardTabId = WIZARD_TAB_ID;
    this.icon = 'fa-folder';
    this.title = Locale.tr("Context");
    this.classes = "hypervisor only_kvm only_vmware only_xen only_vcenter";

    this.contextFilesTable = new FilesTable(this.wizardTabId + 'ContextTable', {
      'select': true,
      'selectOptions': {
        'multiple_choice': true,
        "filter_fn": function(file) { return file.TYPE == 5; } // CONTEXT
      }});
  }

  WizardTab.prototype.constructor = WizardTab;
  WizardTab.prototype.html = _html;
  WizardTab.prototype.setup = _setup;
  WizardTab.prototype.onShow = _onShow;
  WizardTab.prototype.retrieve = _retrieve;
  WizardTab.prototype.fill = _fill;
  WizardTab.prototype.generateContextFiles = _generateContextFiles;

  return WizardTab;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'customTagsTableHTML': CustomTagsTable.html(),
      'contextFilesTableHTML': this.contextFilesTable.dataTableHTML
    });
  }

  function _onShow(context, panelForm) {
  }

  function _setup(context) {
    var that = this;

    CustomTagsTable.setup(context);

    var selectOptions = {
      'selectOptions': {
        'select_callback': function(aData, options) {
          that.generateContextFiles(context)
        },
        'unselect_callback': function(aData, options) {
          that.generateContextFiles(context)
        }
      }
    }

    that.contextFilesTable.initialize(selectOptions);
    that.contextFilesTable.refreshResourceTableSelect();

    context.on("click", ".add_service_custom_attr", function() {
      $(".service_custom_attrs tbody").append(
        '<tr>' +
          '<td>' +
            '<input class="user_input_name" type="text" pattern="[\\w]+"/>' +
            '<small class="error">' + Locale.tr("Only word characters are allowed") + '</small>' +
          '</td>' +
          '<td>' +
            '<select class="user_input_type" >' +
              '<option value="text">' + Locale.tr("text") + '</option>' +
              '<option value="password">' + Locale.tr("password") + '</option>' +
            '</select>' +
          '</td>' +
          '<td>' +
            '<textarea class="user_input_description"/>' +
          '</td>' +
          '<td>' +
            '<a href="#"><i class="fa fa-times-circle remove-tab"></i></a>' +
          '</td>' +
        '</tr>');
    })

    context.on("click", ".service_custom_attrs i.remove-tab", function() {
      var tr = $(this).closest('tr');
      tr.remove();
    });
  }

  function _retrieve(context) {
    var templateJSON = {};
    var contextJSON = WizardFields.retrieve(context);
    $.extend(contextJSON, CustomTagsTable.retrieve(context));

    if ($("#ssh_context", context).is(":checked")) {
      var public_key = $("#ssh_public_key", context).val();
      if (public_key) {
        contextJSON["SSH_PUBLIC_KEY"] = TemplateUtils.escapeDoubleQuotes(public_key);
      } else {
        contextJSON["SSH_PUBLIC_KEY"] = '$USER[SSH_PUBLIC_KEY]';
      }
    };

    if ($("#network_context", context).is(":checked")) {
      contextJSON["NETWORK"] = "YES";
    };

    if ($("#token_context", context).is(":checked")) {
      contextJSON["TOKEN"] = "YES";
    };

    var userInputsJSON = {};
    $(".service_custom_attrs tbody tr", context).each(function() {
      if ($(".user_input_name", $(this)).val()) {
        var attr_name = $(".user_input_name", $(this)).val();
        var attr_type = $(".user_input_type", $(this)).val();
        var attr_desc = $(".user_input_description", $(this)).val();
        userInputsJSON[attr_name] = "M|" + attr_type + "|" + attr_desc;
        contextJSON[attr_name] = "$" + attr_name.toUpperCase();
      }
    });

    var start_script = $("#START_SCRIPT", context).val();
    if (start_script != "") {
      if ($("#ENCODE_START_SCRIPT", context).is(":checked")) {
        contextJSON["START_SCRIPT_BASE64"] = btoa(start_script);
      } else {
        contextJSON["START_SCRIPT"] = start_script;
      }
    }

    if (!$.isEmptyObject(contextJSON)) { templateJSON['CONTEXT'] = contextJSON; };
    if (!$.isEmptyObject(userInputsJSON)) { templateJSON['USER_INPUTS'] = userInputsJSON; };

    return templateJSON;
  }

  function _fill(context, templateJSON) {
    var that = this;
    $("#ssh_context", context).removeAttr('checked');
    $("#network_context", context).removeAttr('checked');

    var contextJSON = templateJSON['CONTEXT'];
    var userInputsJSON = templateJSON['USER_INPUTS'];
    if (userInputsJSON) {
      $.each(userInputsJSON, function(key, value) {
        $(".add_service_custom_attr", context).trigger("click");

        var context = $(".service_custom_attrs tbody tr", context).last();
        var parts = value.split("|");
        $(".user_input_name", context).val(key);
        $(".user_input_type", context).val(parts[1]);
        $(".user_input_description", context).val(TemplateUtils.escapeDoubleQuotes(TemplateUtils.htmlDecode(parts[2])));

        if (contextJSON) {
          delete contextJSON[key];
        }
      });

      delete templateJSON['USER_INPUTS'];
    }

    if (contextJSON) {
      var file_ds_regexp = /\$FILE\[IMAGE_ID=([0-9]+)+/g;
      var net_regexp = /^NETWORK$/;;
      var ssh_regexp = /^SSH_PUBLIC_KEY$/;
      var token_regexp = /^TOKEN$/;
      var publickey_regexp = /\$USER\[SSH_PUBLIC_KEY\]/;

      var net_flag = false;
      var files = [];

      var customTagsJSON = {};
      $.each(contextJSON, function(key, value) {
        if (ssh_regexp.test(key)) {
          $("#ssh_context", context).prop('checked', 'checked');

          if (!publickey_regexp.test(value)) {
            $("#ssh_public_key", context).val(TemplateUtils.htmlDecode(value));
          }
        } else if (token_regexp.test(key)) {
          $("#token_context", context).prop('checked', 'checked');
        } else if (net_regexp.test(key)) {
          $("#network_context", context).prop('checked', 'checked');
        } else if ("INIT_SCRIPTS" == key) {
          $("input#INIT_SCRIPTS").val(TemplateUtils.htmlDecode(value));
        } else if ("FILES_DS" == key) {
          $('#FILES_DS', context).val(TemplateUtils.escapeDoubleQuotes(TemplateUtils.htmlDecode(contextJSON["FILES_DS"])))
          var files = [];
          while (match = file_ds_regexp.exec(value)) {
            files.push(match[1])
          }

          var selectedResources = {
              ids : files
            }
          that.contextFilesTable.selectResourceTableSelect(selectedResources);
        } else if ("START_SCRIPT_BASE64" == key) {
          $("#ENCODE_START_SCRIPT", context).prop('checked', 'checked');
          $("#START_SCRIPT", context).val(atob(value));
        } else if ("START_SCRIPT" ==  key) {
          $("#START_SCRIPT", context).val(value);
        } else {
          customTagsJSON[key] = value;
        }
      });

      CustomTagsTable.fill(context, customTagsJSON);

      delete templateJSON['CONTEXT'];
    }
  }

  function _generateContextFiles(context) {
    var req_string=[];
    var selected_files = this.contextFilesTable.retrieveResourceTableSelect();

    $.each(selected_files, function(index, fileId) {
      req_string.push("$FILE[IMAGE_ID="+ fileId +"]");
    });

    $('#FILES_DS', context).val(req_string.join(" "));
  };
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/scheduling/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/tip'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "<dl class=\"tabs vertical\" data-tab>\n  <dd class=\"active\">\n    <a href=\"#placementTab\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Placement", {"name":"tr","hash":{},"data":data})))
    + "</a>\n  </dd>\n  <dd>\n    <a href=\"#policyTab\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Policy", {"name":"tr","hash":{},"data":data})))
    + "</a>\n  </dd>\n</dl>\n<div class=\"tabs-content vertical\">\n  <div class=\"requirements wizard_internal_tab active content\" id=\"placementTab\">\n    <fieldset>\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Host Requirements", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"large-12 columns text-center\">\n          <input type=\"radio\" id=\"hosts_req\" name=\"req_select\" value=\"host_select\">\n          <label for=\"hosts_req\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select Hosts ", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <input type=\"radio\" id=\"clusters_req\"  name=\"req_select\" value=\"cluster_select\">\n          <label for=\"clusters_req\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select Clusters ", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        </div>\n      </div>\n      <br>\n      <div id=\"req_type\" class=\"host_select\" hidden>\n      ";
  stack1 = ((helper = (helper = helpers.hostsTableSelectHTML || (depth0 != null ? depth0.hostsTableSelectHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"hostsTableSelectHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </div>\n      <div id=\"req_type\" class=\"cluster_select hidden\">\n      ";
  stack1 = ((helper = (helper = helpers.clustersTableSelectHTML || (depth0 != null ? depth0.clustersTableSelectHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"clustersTableSelectHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </div>\n      <br>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label for=\"SCHED_REQUIREMENTS\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Expression", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Boolean expression that rules out provisioning hosts from list of machines suitable to run this VM", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"SCHED_REQUIREMENTS\" id=\"SCHED_REQUIREMENTS\" name=\"requirements\" />\n        </div>\n      </div>\n    </fieldset>\n    <fieldset class=\"hypervisor only_kvm only_vmware only_xen\">\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastore Requirements", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label for=\"SCHED_DS_REQUIREMENTS\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Expression", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Boolean expression that rules out entries from the pool of datastores suitable to run this VM.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"text\" wizard_field=\"SCHED_DS_REQUIREMENTS\" id=\"SCHED_DS_REQUIREMENTS\" name=\"requirements\" />\n        </div>\n      </div>\n    </fieldset>\n  </div>\n  <div id=\"policyTab\" class=\"wizard_internal_tab content\">\n    <fieldset class=\"host_rank\">\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Host Rank", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"large-12 columns text-center\">\n          <input type=\"radio\" id=\"packingRadio\" name=\"rank_select\" value=\"RUNNING_VMS\">\n          <label for=\"packingRadio\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Packing", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Pack the VMs in the cluster nodes to reduce VM fragmentation", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"radio\"  id=\"stripingRadio\" name=\"rank_select\" value=\"-RUNNING_VMS\">\n          <label for=\"stripingRadio\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Stripping", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Spread the VMs in the cluster nodes", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"radio\"  id=\"loadawareRadio\" name=\"rank_select\" value=\"FREE_CPU\">\n          <label for=\"loadawareRadio\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Load-aware", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Maximize the resources available to VMs in a node", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n        </div>\n      </div>\n      <br>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label for=\"SCHED_RANK\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Expression", {"name":"tr","hash":{},"data":data})))
    + "\n            <span class=\"tip\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "This field sets which attribute will be used to sort the suitable hosts for this VM", {"name":"tr","hash":{},"data":data})))
    + ".\n            </span>\n          </label>\n          <input type=\"text\" wizard_field=\"SCHED_RANK\" id=\"SCHED_RANK\" name=\"RANK\" />\n        </div>\n      </div>\n    </fieldset>\n    <fieldset class=\"ds_rank hypervisor only_kvm only_vmware only_xen\">\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastore Rank", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"large-12 columns text-center\">\n          <input type=\"radio\" id=\"packingDSRadio\" name=\"ds_rank_select\" value=\"-FREE_MB\">\n          <label for=\"packingDSRadio\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Packing", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Tries to optimize storage usage by selecting the DS with less free space", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <input type=\"radio\"  id=\"stripingDSRadio\" name=\"ds_rank_select\" value=\"FREE_MB\">\n          <label for=\"stripingDSRadio\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Stripping", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Striping. Tries to optimize I/O by distributing the VMs across datastores.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n        </div>\n      </div>\n      <br>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <label for=\"SCHED_DS_RANK\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Expression", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "This field sets which attribute will be used to sort the suitable datastores for this VM", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n          </label>\n          <input type=\"text\" wizard_field=\"SCHED_DS_RANK\" id=\"SCHED_DS_RANK\" name=\"RANK\" />\n        </div>\n      </div>\n    </fieldset>\n  </div>\n</div>\n";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/scheduling/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/create/wizard-tabs/scheduling/wizardTabId',['require'],function(require){
  return 'schedulingTab';
});
define('tabs/templates-tab/form-panels/create/wizard-tabs/scheduling',['require','foundation.tab','sunstone-config','utils/locale','utils/tips','utils/wizard-fields','utils/template-utils','tabs/hosts-tab/datatable','tabs/clusters-tab/datatable','hbs!./scheduling/html','./scheduling/wizardTabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var Config = require('sunstone-config');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var WizardFields = require('utils/wizard-fields');
  var TemplateUtils = require('utils/template-utils');
  var HostsTable = require('tabs/hosts-tab/datatable');
  var ClustersTable = require('tabs/clusters-tab/datatable');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./scheduling/html');

  /*
    CONSTANTS
   */

  var WIZARD_TAB_ID = require('./scheduling/wizardTabId');

  /*
    CONSTRUCTOR
   */

  function WizardTab() {
    if (!Config.isTemplateCreationTabEnabled('scheduling')) {
      throw "Wizard Tab not enabled";
    }

    this.wizardTabId = WIZARD_TAB_ID;
    this.icon = 'fa-sitemap';
    this.title = Locale.tr("Scheduling");

    var options = {
      'select': true,
      'selectOptions': {
        'multiple_choice': true
      }
    }
    this.hostsTable = new HostsTable(this.wizardTabId + 'HostsTable', options);
    this.clustersTable = new ClustersTable(this.wizardTabId + 'ClustersTable', options);
  }

  WizardTab.prototype.constructor = WizardTab;
  WizardTab.prototype.html = _html;
  WizardTab.prototype.setup = _setup;
  WizardTab.prototype.onShow = _onShow;
  WizardTab.prototype.retrieve = _retrieve;
  WizardTab.prototype.fill = _fill;
  WizardTab.prototype.generateRequirements = _generateRequirements;

  return WizardTab;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'hostsTableSelectHTML': this.hostsTable.dataTableHTML,
      'clustersTableSelectHTML': this.clustersTable.dataTableHTML
    });
  }

  function _onShow(context, panelForm) {
  }

  function _setup(context) {
    var  that = this;
    context.foundation('tab', 'reflow');

    context.on("change", "input[name='req_select']", function() {
      if ($("input[name='req_select']:checked").val() == "host_select") {
        $("div.host_select",    context).toggle();
        $("div.cluster_select", context).hide();
      } else {
        $("div.host_select",    context).hide();
        $("div.cluster_select", context).toggle();
      }
    });

    context.on("change", "input[name='rank_select']", function() {
      $("#SCHED_RANK", context).val(this.value);
    });

    context.on("change", "input[name='ds_rank_select']", function() {
      $("#SCHED_DS_RANK", context).val(this.value);
    });

    var selectOptions = {
      'selectOptions': {
        'select_callback': function(aData, options) {
          that.generateRequirements(context)
        },
        'unselect_callback': function(aData, options) {
          that.generateRequirements(context)
        }
      }
    }

    that.hostsTable.initialize(selectOptions);
    that.hostsTable.refreshResourceTableSelect();
    that.clustersTable.initialize(selectOptions);
    that.clustersTable.refreshResourceTableSelect();
  }

  function _retrieve(context) {
    return WizardFields.retrieve(context);
  }

  function _fill(context, templateJSON) {
    var reqJSON = templateJSON['SCHED_REQUIREMENTS'];
    if (reqJSON) {
      var req = TemplateUtils.escapeDoubleQuotes(reqJSON);

      var host_id_regexp = /(\s|\||\b)ID=\\"([0-9]+)\\"/g;
      var cluster_id_regexp = /CLUSTER_ID=\\"([0-9]+)\\"/g;

      var hosts = [];
      while (match = host_id_regexp.exec(req)) {
          hosts.push(match[2])
      }

      var clusters = [];
      while (match = cluster_id_regexp.exec(req)) {
          clusters.push(match[1])
      }

      var selectedResources = {
          ids : hosts
        }

      this.hostsTable.selectResourceTableSelect(selectedResources);


      var selectedResources = {
          ids : clusters
        }

      this.clustersTable.selectResourceTableSelect(selectedResources);

      $('input#SCHED_REQUIREMENTS', context).val(TemplateUtils.htmlDecode(req));
      delete templateJSON['SCHED_REQUIREMENTS'];
    }

    var dsReqJSON = templateJSON['SCHED_DS_REQUIREMENTS'];
    if (dsReqJSON) {
      var dsReq = TemplateUtils.escapeDoubleQuotes(dsReqJSON);
      $('input#SCHED_DS_REQUIREMENTS', context).val(TemplateUtils.htmlDecode(dsReq));
      delete templateJSON['SCHED_DS_REQUIREMENTS'];
    }


    var rankJSON = templateJSON["SCHED_RANK"];
    if (rankJSON) {
        var striping_regexp = /^-RUNNING_VMS$/;
        var packing_regexp = /^RUNNING_VMS$/;
        var loadaware_regexp = /^FREE_CPU$/;

        if (striping_regexp.test(rankJSON)) {
            $('input[name="rank_select"]#stripingRadio', context).click()
        }
        else if (packing_regexp.test(rankJSON)) {
            $('input[name="rank_select"]#packingRadio', context).click()
        }
        else if (loadaware_regexp.test(rankJSON)) {
            $('input[name="rank_select"]#loadawareRadio', context).click()
        }

        $('input#SCHED_RANK', context).val(TemplateUtils.htmlDecode(rankJSON));

        delete templateJSON["SCHED_RANK"];
    }

    var dsRankJSON = templateJSON["SCHED_DS_RANK"];
    if (dsRankJSON) {
        var striping_regexp = /^FREE_MB$/;
        var packing_regexp = /^-FREE_MB$/;

        if (striping_regexp.test(dsRankJSON)) {
            $('input[name="ds_rank_select"]#stripingDSRadio', context).click()
        }
        else if (packing_regexp.test(dsRankJSON)) {
            $('input[name="ds_rank_select"]#packingDSRadio', context).click()
        }

        $('input#SCHED_DS_RANK', context).val(TemplateUtils.htmlDecode(dsRankJSON));

        delete templateJSON["SCHED_DS_RANK"];
    }
  }

  function _generateRequirements(context) {
      var req_string=[];
      var selected_hosts = this.hostsTable.retrieveResourceTableSelect();
      var selected_clusters = this.clustersTable.retrieveResourceTableSelect();

      $.each(selected_hosts, function(index, hostId) {
        req_string.push('ID=\\"'+hostId+'\\"');
      });

      $.each(selected_clusters, function(index, clusterId) {
        req_string.push('CLUSTER_ID=\\"'+clusterId+'\\"');
      });

      $('#SCHED_REQUIREMENTS', context).val(req_string.join(" | "));
  };
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/hybrid/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<dl class=\"tabs vertical\" id=\"template_create_hybrid_tabs\" data-tab>\n  <dt class=\"text-center\">\n    <a href=\"\" class=\"button tiny radius\" id=\"tf_btn_hybrid\">\n      <span class=\"fa fa-plus\"></span>\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add another provider", {"name":"tr","hash":{},"data":data})))
    + "\n    </a>\n  </dt>\n</dl>\n<div class=\"tabs-content vertical\" id=\"template_create_hybrid_tabs_content\"></div>";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/hybrid/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/create/wizard-tabs/hybrid/wizardTabId',['require'],function(require){
  return 'hybridTab';
});
define('tabs/templates-tab/form-panels/create/wizard-tabs/hybrid',['require','foundation.tab','sunstone-config','utils/locale','utils/tips','utils/wizard-fields','hbs!./hybrid/html','./hybrid/wizardTabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var Config = require('sunstone-config');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var WizardFields = require('utils/wizard-fields');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./hybrid/html');

  /*
    CONSTANTS
   */

  var WIZARD_TAB_ID = require('./hybrid/wizardTabId');
  var HYBRID_INPUTS = {
    ec2 : [
      {
        name: "AKI",
        label: Locale.tr("AKI"),
        tooltip: Locale.tr("The ID of the kernel with which to launch the instance.")
      },
      {
        name: "AMI",
        label: Locale.tr("AMI"),
        tooltip: Locale.tr("Unique ID of a machine image, returned by a call to ec2-describe-images."),
        required: true
      },
      {
        name: "AVAILABILITYZONE",
        label: Locale.tr("Availability Zone"),
        tooltip: Locale.tr("The Availability Zone in which to run the instance.")
      },
      {
        name: "BLOCKDEVICEMAPPING",
        label: Locale.tr("Block Device Mapping"),
        tooltip: Locale.tr("The block device mapping for the instance. More than one can be specified in a space-separated list. Check the –block-device-mapping option of the EC2 CLI Reference for the syntax")
      },
      {
        name: "CLIENTTOKEN",
        label: Locale.tr("Client Token"),
        tooltip: Locale.tr("Unique, case-sensitive identifier you provide to ensure idempotency of the request.")
      },
      {
        name: "EBS_OPTIMIZED",
        label: Locale.tr("EBS Optimized"),
        tooltip: Locale.tr("Obtain a better I/O throughput for VMs with EBS provisioned volumes")
      },
      {
        name: "ELASTICIP",
        label: Locale.tr("Elastic IP"),
        tooltip: Locale.tr("EC2 Elastic IP address to assign to the instance. This parameter is passed to the command ec2-associate-address -i i-0041230 elasticip.")
      },
      {
        name: "HOST",
        label: Locale.tr("OpenNebula Host"),
        tooltip: Locale.tr("Defines which OpenNebula host will use this template")
      },
      {
        name: "INSTANCETYPE",
        label: Locale.tr("Instance Type"),
        tooltip: Locale.tr("Specifies the instance type."),
        required: true
      },
      {
        name: "KEYPAIR",
        label: Locale.tr("Keypair"),
        tooltip: Locale.tr("The name of the key pair, later will be used to execute commands like ssh -i id_keypair or scp -i id_keypair")
      },
      {
        name: "LICENSEPOOL",
        label: Locale.tr("License Pool"),
        tooltip: Locale.tr("Name of the license pool.")
      },
      {
        name: "PLACEMENTGROUP",
        label: Locale.tr("Placement Group"),
        tooltip: Locale.tr("Name of the placement group.")
      },
      {
        name: "PRIVATEIP",
        label: Locale.tr("Private IP"),
        tooltip: Locale.tr("If you’re using Amazon Virtual Private Cloud, you can optionally use this parameter to assign the instance a specific available IP address from the subnet.")
      },
      {
        name: "RAMDISK",
        label: Locale.tr("Ramdisk"),
        tooltip: Locale.tr("The ID of the RAM disk to select.")
      },
      {
        name: "SECURITYGROUPS",
        label: Locale.tr("Security Groups"),
        tooltip: Locale.tr("Name of the security group. You can specify more than one security group (comma separated).")
      },
      {
        name: "SUBNETID",
        label: Locale.tr("Subnet ID"),
        tooltip: Locale.tr("If you’re using Amazon Virtual Private Cloud, this specifies the ID of the subnet you want to launch the instance into. This parameter is also passed to the command ec2-associate-address -i i-0041230 -a elasticip.")
      },
      {
        name: "TAGS",
        label: Locale.tr("Tags"),
        tooltip: Locale.tr("Key and optional value of the tag, separated by an equals sign ( = ).You can specify more than one tag (comma separated).")
      },
      {
        name: "TENANCY",
        label: Locale.tr("Tenancy"),
        tooltip: Locale.tr("The tenancy of the instance you want to launch.")
      },
      {
        name: "USERDATA",
        label: Locale.tr("User Data"),
        tooltip: Locale.tr("Specifies Base64-encoded MIME user data to be made available to the instance(s) in this reservation.")
      }
    ],
    softlayer: [
      {
        name: "BLOCKDEVICETEMPLATE",
        label: Locale.tr("Block Device Template"),
        tooltip: Locale.tr("A global identifier for the template to be used to provision the computing instance")
      },
      {
        name: "BLOCKDEVICE",
        label: Locale.tr("Block Device Size"),
        tooltip: Locale.tr("Size of the block device size to be presented to the VM")
      },
      {
        name: "DATACENTER",
        label: Locale.tr("Datacenter"),
        tooltip: Locale.tr("Specifies which datacenter the instance is to be provisioned in")
      },
      {
        name: "DEDICATEDHOST",
        label: Locale.tr("Dedicated Host"),
        tooltip: Locale.tr("Specifies whether or not the instance must only run on hosts with instances from the same account")
      },
      {
        name: "DOMAIN",
        label: Locale.tr("Domain"),
        tooltip: Locale.tr("Domain for the computing instance"),
        required: true
      },
      {
        name: "HOSTNAME",
        label: Locale.tr("Hostname"),
        tooltip: Locale.tr("Hostname for the computing instance"),
        required: true
      },
      {
        name: "HOURLYBILLING",
        label: Locale.tr("Hourly Billing"),
        tooltip: Locale.tr("Specifies the billing type for the instance . When true the computing instance will be billed on hourly usage, otherwise it will be billed on a monthly basis"),
        required: true
      },
      {
        name: "INSTANCE_TYPE",
        label: Locale.tr("Instance Type"),
        tooltip: Locale.tr("Specifies the capacity of the VM in terms of CPU and memory. If both STARTCPUS and MAXMEMORY are used, then this parameter is disregarded"),
        required: true
      },
      {
        name: "LOCALDISK",
        label: Locale.tr("Local Disk"),
        tooltip: Locale.tr("Name of the placement group. When true the disks for the computing instance will be provisioned on the host which it runs, otherwise SAN disks will be provisioned"),
        required: true
      },
      {
        name: "MAXMEMORY",
        label: Locale.tr("Max Memory"),
        tooltip: Locale.tr("The amount of memory to allocate in megabytes")
      },
      {
        name: "NETWORKCOMPONENTSMAXSPEED",
        label: Locale.tr("Network Components Max Speed"),
        tooltip: Locale.tr("Specifies the connection speed for the instance's network components")
      },
      {
        name: "OPERATINGSYSTEM",
        label: Locale.tr("Operating System"),
        tooltip: Locale.tr("An identifier for the operating system to provision the computing instance with. A non exhaustive list of identifiers can be found here"),
        required: true
      },
      {
        name: "POSTSCRIPT",
        label: Locale.tr("Postscript"),
        tooltip: Locale.tr("Specifies the uri location of the script to be downloaded and run after installation is complete")
      },
      {
        name: "PRIVATENETWORKONLY",
        label: Locale.tr("Private Netwrok Only"),
        tooltip: Locale.tr("Specifies whether or not the instance only has access to the private network  (ie, if it is going to have a public IP interface or not)")
      },
      {
        name: "PRIMARYNETWORKVLAN",
        label: Locale.tr("Primary Network VLAN"),
        tooltip: Locale.tr("Specifies the network vlan which is to be used for the frontend interface of the computing instance")
      },
      {
        name: "PRIMARYBACKENDNETWORKVLAN",
        label: Locale.tr("Primary Backed Network VLAN"),
        tooltip: Locale.tr("Specifies the network vlan which is to be used for the backend interface of the computing instance")
      },
      {
        name: "SSHKEYS",
        label: Locale.tr("SSH Keys"),
        tooltip: Locale.tr("SSH keys to install on the computing instance upon provisioning")
      },
      {
        name: "STARTCPUS",
        label: Locale.tr("Start CPUs"),
        tooltip: Locale.tr("The number of CPU cores to allocate to the VM")
      },
      {
        name: "USERDATA",
        label: Locale.tr("User Data"),
        tooltip: Locale.tr("Arbitrary data to be made available to the computing instance")
      }
    ],
    azure: [
      {
        name: "AFFINITY_GROUP",
        label: Locale.tr("Affinity Group"),
        tooltip: Locale.tr("Affinity groups allow you to group your Azure services to optimize performance. All services and VMs within an affinity group will be located in the same region")
      },
      {
        name: "AVAILABILITY_SET",
        label: Locale.tr("Availability Set"),
        tooltip: Locale.tr("Name of the availability set to which this VM will belong")
      },
      {
        name: "CLOUD_SERVICE",
        label: Locale.tr("Cloud Service"),
        tooltip: Locale.tr("Specifies the name of the cloud service where this VM will be linked. Defaults to 'OpennebulaDefaultCloudServiceName'")
      },
      {
        name: "IMAGE",
        label: Locale.tr("Image"),
        tooltip: Locale.tr("Specifies the base OS of the VM."),
        required: true
      },
      {
        name: "INSTANCE_TYPE",
        label: Locale.tr("Instance Type"),
        tooltip: Locale.tr("Specifies the capacity of the VM in terms of CPU and memory"),
        required: true
      },
      {
        name: "LOCATION",
        label: Locale.tr("Location"),
        tooltip: Locale.tr("Azure datacenter where the VM will be sent. See /etc/one/az_driver.conf for possible values (under region_name)"),
        required: true
      },
      {
        name: "SSHPORT",
        label: Locale.tr("SSH Port"),
        tooltip: Locale.tr("Port where the VMs ssh server will listen on")
      },
      {
        name: "STORAGE_ACCOUNT",
        label: Locale.tr("Storage Account"),
        tooltip: Locale.tr("Specify the storage account where this VM will belong")
      },
      {
        name: "SUBNET",
        label: Locale.tr("Subnet"),
        tooltip: Locale.tr("Name of the particular Subnet where this VM will be connected to")
      },
      {
        name: "TCP_ENDPOINTS",
        label: Locale.tr("TCP Endpoints"),
        tooltip: Locale.tr("Comma-separated list of TCP ports to be accesible from the public internet to this VM")
      },
      {
        name: "VIRTUAL_NETWORK_NAME",
        label: Locale.tr("Virtual Network Name"),
        tooltip: Locale.tr("Name of the virtual network to which this VM will be connected")
      },
      {
        name: "VM_USER",
        label: Locale.tr("VM User"),
        tooltip: Locale.tr("If the selected IMAGE is prepared for Azure provisioning, a username can be specified here to access the VM once booted"),
        required: true
      },
      {
        name: "VM_PASSWORD",
        label: Locale.tr("VM Password"),
        tooltip: Locale.tr("Password for VM_USER"),
        required: true
      },
      {
        name: "WIN_RM",
        label: Locale.tr("Win RM"),
        tooltip: Locale.tr("Comma-separated list of possible protocols to access this Windows VM")
      }
    ]
  }

  /*
    CONSTRUCTOR
   */

  function WizardTab() {
    if (!Config.isTemplateCreationTabEnabled('hybrid')) {
      throw "Wizard Tab not enabled";
    }

    this.wizardTabId = WIZARD_TAB_ID;
    this.icon = 'fa-cloud';
    this.title = Locale.tr("Hybrid");
  }

  WizardTab.prototype.constructor = WizardTab;
  WizardTab.prototype.html = _html;
  WizardTab.prototype.setup = _setup;
  WizardTab.prototype.onShow = _onShow;
  WizardTab.prototype.retrieve = _retrieve;
  WizardTab.prototype.fill = _fill;
  WizardTab.prototype.addProviderTab = _addProviderTab;
  WizardTab.prototype.fillProviderTab = _fillProviderTab;

  return WizardTab;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML();
  }

  function _onShow(context, panelForm) {
  }

  function _setup(context) {
    var that = this;
    context.foundation('tab', 'reflow');
    that.numberOfProviders = 0;

    // close icon: removing the tab on click
    context.on("click", "i.remove-tab", function() {
      var target = $(this).parent().attr("href");
      var dd = $(this).closest('dd');
      var dl = $(this).closest('dl');
      var content = $(target);

      dd.remove();
      content.remove();

      if (dd.attr("class") == 'active') {
        $('a', dl.children('dd').last()).click();
      }

      $("dl#template_create_hybrid_tabs dd", context).each(function(index) {
          $("a", this).html(Locale.tr("Provider") + ' ' + index + " <i class='fa fa-times-circle remove-tab'></i>");
        })
    });

    context.on("click", "#tf_btn_hybrid", function() {
      that.addProviderTab(that.numberOfProviders, context);
      that.numberOfProviders++;
      return false;
    });

    $("#tf_btn_hybrid", context).trigger("click");
  }

  function _retrieve(context) {
    var templateJSON = {};
    var publicCloudJSON = [];
    var ec2JSON = [];

    /* TODO Check if vCenter is defined
    if ($.isEmptyObject(vm_json["PUBLIC_CLOUD"])) {
      vm_json["PUBLIC_CLOUD"] = [];
    }*/

    if ($("[wizard_field='HYPERVISOR']:checked").val() == 'vcenter') {
      publicCloudJSON.push({
        'TYPE': 'vcenter',
        'VM_TEMPLATE': $("#vcenter_template_uuid").val()
      });
    }

    $('.provider', context).each(function() {
      var hash  = WizardFields.retrieve(this);
      if (!$.isEmptyObject(hash)) {
        var hybrid = $("input.hybridRadio:checked", this).val();
        switch (hybrid) {
          case 'ec2':
            ec2JSON.push(hash);
            break;
          case 'softlayer':
            hash["TYPE"] = hybrid.toUpperCase();
            publicCloudJSON.push(hash);
            break;
          case 'azure':
            hash["TYPE"] = hybrid.toUpperCase();
            publicCloudJSON.push(hash);
            break;
        }
      };
    });

    if (!$.isEmptyObject(publicCloudJSON)) { templateJSON['PUBLIC_CLOUD'] = publicCloudJSON; };
    if (!$.isEmptyObject(ec2JSON)) { templateJSON['EC2'] = ec2JSON; };

    return templateJSON;
  }

  function _fill(context, templateJSON) {
    var that = this;
    var clickButton = false;
    if (templateJSON.PUBLIC_CLOUD) {
      var providers = templateJSON.PUBLIC_CLOUD

      if (providers instanceof Array) {
        $.each(providers, function(index, provider) {
          clickButton = index > 0;
          that.fillProviderTab(context, provider, provider.TYPE.toLowerCase(), clickButton);
        });
      } else if (providers instanceof Object) {
        that.fillProviderTab(context, providers, providers.TYPE.toLowerCase(), clickButton);
        clickButton = true;
      }

      delete templateJSON.PUBLIC_CLOUD
    }

    if (templateJSON.EC2) {
      var providers = templateJSON.EC2

      if (providers instanceof Array) {
        $.each(providers, function(index, provider) {
          clickButton = clickButton || index > 0;
          that.fillProviderTab(context, provider, "ec2", clickButton);
        });
      } else if (providers instanceof Object) {
        that.fillProviderTab(context, providers, "ec2", clickButton);
      }

      delete templateJSON.EC2
    }
  }

  function _addProviderTab(provider_id, context) {
    var htmlId  = 'provider' + provider_id;

    // Append the new div containing the tab and add the tab to the list
    var html_tab_content = '<div id="' + htmlId + 'Tab" class="provider wizard_internal_tab content">' +
      '<div class="row">' +
        '<div class="large-12 columns">' +
          '<label>' + Locale.tr("Hybrid Cloud") + '</label>' +
          '<input type="radio" class="hybridRadio" name="hybrid' + htmlId + '" value="ec2" id="amazonRadio' + htmlId + '"><label for="amazonRadio' + htmlId + '">Amazon EC2</label>' +
          '<input type="radio" class="hybridRadio" name="hybrid' + htmlId + '" value="softlayer" id="softlayerRadio' + htmlId + '"><label for="softlayerRadio' + htmlId + '">IBM Softlayer</label>' +
          '<input type="radio" class="hybridRadio" name="hybrid' + htmlId + '" value="azure" id="azureRadio' + htmlId + '"><label for="azureRadio' + htmlId + '">Microsoft Azure</label>' +
        '</div>' +
      '</div>' +
      '<div class="row hybrid_inputs vm_param">' +
      '</div>' +
    '</div>'
    $(html_tab_content).appendTo($("#template_create_hybrid_tabs_content", context));

    var a = $("<dd>\
        <a id='provider_tab" + htmlId + "' href='#" + htmlId + "Tab'>" + Locale.tr("PROVIDER") + "</a>\
      </dd>").appendTo($("dl#template_create_hybrid_tabs", context));

    $("dl#template_create_hybrid_tabs dd", context).each(function(index) {
        $("a", this).html(Locale.tr("Provider") + ' ' + index + " <i class='fa fa-times-circle remove-tab'></i>");
      })

    $("a", a).trigger("click");

    var providerSection = $('#' + htmlId + 'Tab', context);

    providerSection.on("change", "input.hybridRadio", function() {
      $(".hybrid_inputs", providerSection).html("");

      var required_str = "";
      var not_required_str = "";

      $.each(HYBRID_INPUTS[this.value], function(index, obj) {
        if (obj.required) {
          required_str += '<div class="large-6 columns">' +
            '<label>' +
              obj.label + ' ' +
              Tips.html(obj.tooltip) +
            '</label>' +
            '<input wizard_field="' + obj.name + '" type="text" id="' + obj.name + '">' +
          '</div>'
        } else {
          not_required_str += '<div class="large-6 columns">' +
            '<label>' +
              obj.label + ' ' +
              Tips.html(obj.tooltip) +
            '</label>' +
            '<input wizard_field="' + obj.name + '" type="text" id="' + obj.name + '">' +
          '</div>'
        }
      });

      $(".hybrid_inputs", providerSection).append(
        required_str +
        '<br><hr><br>' +
        not_required_str);
    })
  }

  function _fillProviderTab(context, provider, providerType, clickButton) {
    var that = this;
    if (providerType == "vcenter") {
      $("#vcenter_template_uuid").val(provider["VM_TEMPLATE"])
    } else {
      if (clickButton) {
        $("#tf_btn_hybrid", context).trigger("click");
      }

      var providerContext = $(".provider", context).last();
      $("input.hybridRadio[value='" + providerType + "']", providerContext).trigger("click");
      WizardFields.fill(providerContext, provider);
    }
  }
});


/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/other/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/tip'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <fieldset class=\"hypervisor only_xen only_kvm only_vmware\">\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "RAW data", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"medium-4 columns\">\n          <label for=\"raw_type\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "TYPE", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <select id=\"raw_type\" name=\"raw_type\">\n            <option value=\"\"></option>\n            <option value=\"kvm\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "kvm", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"xen\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "xen", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"vmware\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "vmware", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"medium-8 columns\">\n          <label class=\"\" for=\"raw_data\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "DATA", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Raw data to be passed directly to the hypervisor", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <textarea rows=\"2\" type=\"text\" id=\"raw_data\" name=\"raw_data\" />\n        </div>\n      </div>\n      <div id=\"data_vmx_div\" class=\"row hidden\">\n        <div class=\"medium-4 columns\"></div>\n        <div class=\"medium-8 columns\">\n          <label class=\"\" for=\"raw_data_vmx\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "DATA_VMX", {"name":"tr","hash":{},"data":data})))
    + "\n            ";
  stack1 = ((helpers.tip || (depth0 && depth0.tip) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Raw data to be added directly to the .vmx file.", {"name":"tr","hash":{},"data":data})), {"name":"tip","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n          </label>\n          <textarea rows=\"2\" type=\"text\" id=\"raw_data_vmx\" name=\"raw_data_vmx\" />\n        </div>\n      </div>\n    </fieldset>\n    <fieldset class=\"hypervisor only_kvm\">\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PCI Devices", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <table class=\"dataTable pci_devices\">\n            <thead>\n              <tr>\n                <th style=\"width:50%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Device name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Vendor", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Device", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Class", {"name":"tr","hash":{},"data":data})))
    + "</th>\n                <th></th>\n              </tr>\n            </thead>\n            <tbody>\n            </tbody>\n            <tfoot>\n              <tr>\n                <td colspan=\"5\">\n                  <a type=\"button\" class=\"add_pci button small small-12 secondary radius\">\n                    <i class=\"fa fa-plus\"></i> "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add PCI device", {"name":"tr","hash":{},"data":data})))
    + "\n                  </a>\n                </td>\n              </tr>\n            </tfoot>\n          </table>\n        </div>\n      </div>\n    </fieldset>\n    <fieldset>\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom Tags", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      ";
  stack1 = ((helper = (helper = helpers.customTagsTableHTML || (depth0 != null ? depth0.customTagsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customTagsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </fieldset>\n  </div>\n</div>\n";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/other/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/create/wizard-tabs/other/pciRow',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<tr>\n  <td class=\"device_name\">\n    <i class=\"fa fa-spinner fa-spin\"></i>\n  </td>\n  <td>\n    <input type=\"text\" maxlength=\"4\" wizard_field=\"VENDOR\" value=\""
    + escapeExpression(((helper = (helper = helpers.vendor || (depth0 != null ? depth0.vendor : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"vendor","hash":{},"data":data}) : helper)))
    + "\">\n  </td>\n  <td>\n    <input type=\"text\" maxlength=\"4\" wizard_field=\"DEVICE\" value=\""
    + escapeExpression(((helper = (helper = helpers.device || (depth0 != null ? depth0.device : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"device","hash":{},"data":data}) : helper)))
    + "\">\n  </td>\n  <td>\n    <input type=\"text\" maxlength=\"4\" wizard_field=\"CLASS\" value=\""
    + escapeExpression(((helper = (helper = helpers['class'] || (depth0 != null ? depth0['class'] : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"class","hash":{},"data":data}) : helper)))
    + "\">\n  </td>\n  <td>\n    <a href=\"#\"><i class=\"fa fa-times-circle remove-tab\"></i></a>\n  </td>\n</tr>";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/create/wizard-tabs/other/pciRow', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/create/wizard-tabs/other/wizardTabId',['require'],function(require){
  return 'otherTab';
});
define('tabs/templates-tab/form-panels/create/wizard-tabs/other',['require','sunstone-config','utils/locale','utils/tips','utils/wizard-fields','utils/template-utils','utils/custom-tags-table','opennebula/host','hbs!./other/html','hbs!./other/pciRow','./other/wizardTabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Config = require('sunstone-config');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var WizardFields = require('utils/wizard-fields');
  var TemplateUtils = require('utils/template-utils');
  var CustomTagsTable = require('utils/custom-tags-table');
  var OpenNebulaHost = require('opennebula/host');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./other/html');
  var RowTemplateHTML = require('hbs!./other/pciRow');

  /*
    CONSTANTS
   */

  var WIZARD_TAB_ID = require('./other/wizardTabId');

  /*
    CONSTRUCTOR
   */

  function WizardTab() {
    if (!Config.isTemplateCreationTabEnabled('other')) {
      throw "Wizard Tab not enabled";
    }

    this.wizardTabId = WIZARD_TAB_ID;
    this.icon = 'fa-ellipsis-h';
    this.title = Locale.tr("Other");
  }

  WizardTab.prototype.constructor = WizardTab;
  WizardTab.prototype.html = _html;
  WizardTab.prototype.setup = _setup;
  WizardTab.prototype.onShow = _onShow;
  WizardTab.prototype.retrieve = _retrieve;
  WizardTab.prototype.fill = _fill;

  return WizardTab;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'customTagsTableHTML': CustomTagsTable.html()
    });
  }

  function _onShow(context, panelForm) {
  }

  function _setup(context) {
    CustomTagsTable.setup(context);

    context.on("change", "#raw_type", function() {
      var choice_str = $(this).val();
      switch (choice_str) {
      case 'vmware':
        $("#data_vmx_div", context).show();
        break;
      default:
        $("#data_vmx_div", context).hide();
      }
    });

    context.off("click", ".add_pci");
    context.on("click", ".add_pci", function(){
      var tr = $(RowTemplateHTML()).appendTo( $(".pci_devices tbody", context) );

      OpenNebulaHost.pciDevices({
        data : {},
        timeout: true,
        success: function (request, pciDevices){
          var html = "<select>";

          html += '<option device="" class="" vendor="">'+Locale.tr("Please select")+'</option>';

          $.each(pciDevices, function(i,pci){
            html += '<option device="'+pci['device']+'" '+
                            'class="'+pci['class']+'" '+
                            'vendor="'+pci['vendor']+'">'+
                                pci.device_name+
                    '</option>';
          });

          html += '</select>';

          $(".device_name", tr).html(html);

          $("input", tr).trigger("change");
        },
        error: function(request, error_json){
          console.error("There was an error requesting the PCI devices: "+
                        error_json.error.message);

          $(".device_name", tr).html("");
        }
      });
    });

    context.on("change", ".pci_devices td.device_name select", function(){
      var tr = $(this).closest('tr');

      var option = $("option:selected", this);

      $('input[wizard_field="DEVICE"]', tr).val( option.attr("device") );
      $('input[wizard_field="CLASS"]',  tr).val( option.attr("class") );
      $('input[wizard_field="VENDOR"]', tr).val( option.attr("vendor") );
    });

    context.on("change", ".pci_devices tbody input", function(){
      var tr = $(this).closest('tr');

      var opt =
        $('option'+
          '[device="'+$('input[wizard_field="DEVICE"]', tr).val()+'"]'+
          '[class="'+$('input[wizard_field="CLASS"]',  tr).val()+'"]'+
          '[vendor="'+$('input[wizard_field="VENDOR"]', tr).val()+'"]', tr);

        opt.attr('selected', 'selected');
    });

    context.on("click", ".pci_devices i.remove-tab", function(){
      var tr = $(this).closest('tr');
      tr.remove();
    });
  }

  function _retrieve(context) {
    var templateJSON = CustomTagsTable.retrieve(context);

    var rawJSON = {}
    t = $('#raw_type', context).val();
    if (t) { rawJSON['TYPE'] = t; }
    t = TemplateUtils.escapeDoubleQuotes($('#raw_data', context).val());
    if (t) { rawJSON['DATA'] = t; }
    t = TemplateUtils.escapeDoubleQuotes($('#raw_data_vmx', context).val());
    if (t) { rawJSON['DATA_VMX'] = t; }

    if (!$.isEmptyObject(rawJSON)) { templateJSON['RAW'] = rawJSON; };

    $('.pci_devices tbody tr', context).each(function(i,row){
      var pci = WizardFields.retrieve(row);

      if (!$.isEmptyObject(pci)){
        if (templateJSON['PCI'] == undefined){
          templateJSON['PCI'] = [];
        }

        templateJSON['PCI'].push(pci);
      }
    });

    return templateJSON;
  }

  function _fill(context, templateJSON) {
    var rawJSON = templateJSON.RAW;
    if (rawJSON) {
      $('#raw_type', context).val(rawJSON['TYPE']);
      $('#raw_type', context).change();
      $('#raw_data', context).val(TemplateUtils.htmlDecode(rawJSON['DATA']));
      $('#raw_data_vmx', context).val(TemplateUtils.htmlDecode(rawJSON['DATA_VMX']));

      delete templateJSON.RAW
    }

    $(".pci_devices i.remove-tab", context).trigger("click");

    var pcis = templateJSON['PCI'];

    if (pcis == undefined){
      pcis = [];
    }

    if (!$.isArray(pcis)){ // If only 1 convert to array
      pcis = [pcis];
    }

    $.each(pcis, function(i, pci){
      $(".add_pci", context).trigger("click");
      var tr = $('.pci_devices tbody tr', context).last();

      WizardFields.fill(tr, pci);
    });

    delete templateJSON.PCI;

    CustomTagsTable.fill(context, templateJSON);
  }
});

define('tabs/templates-tab/form-panels/create',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','utils/tips','utils/template-utils','hbs!./create/wizard','hbs!./create/advanced','./create/formPanelId','../tabId','./create/wizard-tabs/general','./create/wizard-tabs/storage','./create/wizard-tabs/network','./create/wizard-tabs/os','./create/wizard-tabs/io','./create/wizard-tabs/context','./create/wizard-tabs/scheduling','./create/wizard-tabs/hybrid','./create/wizard-tabs/other'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var TemplateUtils = require('utils/template-utils');

  /*
    TEMPLATES
   */

  var TemplateWizardHTML = require('hbs!./create/wizard');
  var TemplateAdvancedHTML = require('hbs!./create/advanced');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');
  var WIZARD_TABS = [
    require('./create/wizard-tabs/general'),
    require('./create/wizard-tabs/storage'),
    require('./create/wizard-tabs/network'),
    require('./create/wizard-tabs/os'),
    require('./create/wizard-tabs/io'),
    require('./create/wizard-tabs/context'),
    require('./create/wizard-tabs/scheduling'),
    require('./create/wizard-tabs/hybrid'),
    require('./create/wizard-tabs/other')
  ]

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create VM Template"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      },
      'update': {
        'title': Locale.tr("Update VM Template"),
        'buttonText': Locale.tr("Update"),
        'resetButton': false
      }
    };

    var that = this;
    that.wizardTabs = [];
    var wizardTabInstance;
    $.each(WIZARD_TABS, function(index, wizardTab) {
      try {
        wizardTabInstance = new wizardTab();
        wizardTabInstance.contentHTML = wizardTabInstance.html();
        that.wizardTabs.push(wizardTabInstance);
      } catch (err) {
        console.log(err);
      }
    })

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.htmlAdvanced = _htmlAdvanced;
  FormPanel.prototype.setup = _setup;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.submitAdvanced = _submitAdvanced;
  FormPanel.prototype.fill = _fill;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {

    return TemplateWizardHTML({
      'formPanelId': this.formPanelId,
      'wizardTabs': this.wizardTabs
    });
  }

  function _htmlAdvanced() {
    return TemplateAdvancedHTML({formPanelId: this.formPanelId});
  }

  function _setup(context) {
    $.each(this.wizardTabs, function(index, wizardTab) {
      wizardTab.setup($('#' + wizardTab.wizardTabId, context));
    });

    context.foundation('tab', 'reflow');
  }

  function _onShow(context) {
    var that = this;
    $('a[href="#'+ that.wizardTabs[0].wizardTabId +'"]', context).trigger("click");

    $.each(that.wizardTabs, function(index, wizardTab) {
      wizardTab.onShow($('#' + wizardTab.wizardTabId, context), that);
    });
  }

  function _submitWizard(context) {
    var templateJSON = {}
    $.each(this.wizardTabs, function(index, wizardTab) {
      $.extend(true, templateJSON, wizardTab.retrieve($('#' + wizardTab.wizardTabId, context)));
    });

    if (this.action == "create") {
      Sunstone.runAction("Template.create",
                          {'vmtemplate': templateJSON});
      return false;
    } else if (this.action == "update") {
      Sunstone.runAction("Template.update",
                          this.resourceId,
                          JSON.stringify({'vmtemplate': templateJSON}));
      return false;
    }
  }

  function _submitAdvanced(context) {
    var template = $('textarea#template', context).val();
    if (this.action == "create") {
      Sunstone.runAction("Template.create",
                          {"vmtemplate": {"template_raw": template}});
      return false;

    } else if (this.action == "update") {
      Sunstone.runAction("Template.update",
                          this.resourceId,
                          JSON.stringify({"vmtemplate": {"template_raw": template}}));
      return false;
    }
  }

  function _fill(context, element) {
    if (this.action != "update") {return;}
    this.resourceId = element.ID;

    var templateJSON = element.TEMPLATE;

    // Populates the Avanced mode Tab
    $('#template', context).val(
      TemplateUtils.templateToString(templateJSON).replace(/^[\r\n]+$/g, ""));

    $.each(this.wizardTabs, function(index, wizardTab) {
      wizardTab.fill($('#' + wizardTab.wizardTabId, context), templateJSON);
    });
  }
});


/* START_TEMPLATE */
define('hbs!utils/vcenter/templates/html',['hbs','hbs/handlebars','templates/helpers/advancedSection','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  return "<span class=\"fa-stack fa-2x\" style=\"color: #dfdfdf\">\n  <i class=\"fa fa-cloud fa-stack-2x\"></i>\n  <i class=\"fa  fa-spinner fa-spin fa-stack-1x fa-inverse\"></i>\n</span>\n";
  },"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, buffer = "";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Templates", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"useData":true});
Handlebars.registerPartial('utils/vcenter/templates/html', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/vcenter/templates',['require','utils/locale','opennebula/template','opennebula/error','hbs!./templates/html'],function(require) {
  // Dependencies
  var Locale = require('utils/locale');
  var OpenNebulaTemplate = require('opennebula/template');
  var OpenNebulaError = require('opennebula/error');

  var TemplateHTML = require('hbs!./templates/html');

  function VCenterTemplates() {
    return this;
  }

  VCenterTemplates.prototype = {
    'html': _html,
    'insert': _fillVCenterTemplates,
    'import': _import
  };
  VCenterTemplates.prototype.constructor = VCenterTemplates;

  return VCenterTemplates;

  function _html() {
    return '<div class="vcenter_templates hidden"></div>';
  }

  /*
    Retrieve the list of templates from vCenter and fill the container with them

    opts = {
      datacenter: "Datacenter Name",
      cluster: "Cluster Name",
      container: Jquery div to inject the html,
      vcenter_user: vCenter Username,
      vcenter_password: vCenter Password,
      vcenter_host: vCenter Host
    }
   */
  function _fillVCenterTemplates(opts) {
    var path = '/vcenter/templates';

    var context = $(".vcenter_templates", opts.container);

    context.html( TemplateHTML({}) );

    context.show();

    $(".accordion_advanced_toggle", context).trigger("click");

    $.ajax({
      url: path,
      type: "GET",
      data: {timeout: false},
      dataType: "json",
      headers: {
        "X_VCENTER_USER": opts.vcenter_user,
        "X_VCENTER_PASSWORD": opts.vcenter_password,
        "X_VCENTER_HOST": opts.vcenter_host
      },
      success: function(response){
        $(".content", context).html("");

        $('<div class="row">' +
            '<div class="large-12 columns">' +
              '<p style="color: #999">' + Locale.tr("Please select the vCenter Templates to be imported to OpenNebula.") + '</p>' +
            '</div>' +
          '</div>').appendTo($(".content", context))

        $.each(response, function(datacenter_name, templates){
          $('<div class="row">' +
              '<div class="large-12 columns">' +
                '<h5>' +
                  datacenter_name + ' ' + Locale.tr("DataCenter") +
                '</h5>' +
              '</div>' +
            '</div>').appendTo($(".content", context))

          if (templates.length == 0) {
              $('<div class="row">' +
                  '<div class="large-12 columns">' +
                    '<label>' +
                      Locale.tr("No new templates found in this DataCenter") +
                    '</label>' +
                  '</div>' +
                '</div>').appendTo($(".content", context))
          } else {
            $.each(templates, function(id, template){
              var trow = $('<div class="vcenter_template">' +
                  '<div class="row">' +
                    '<div class="large-10 columns">' +
                      '<label>' +
                        '<input type="checkbox" class="template_name" checked/> ' +
                        template.name + '&emsp;<span style="color: #999">' + template.host + '</span>' +
                      '</label>' +
                      '<div class="large-12 columns vcenter_template_response">'+
                      '</div>'+
                    '</div>' +
                    '<div class="large-2 columns vcenter_template_result">'+
                    '</div>'+
                  '</div>'+
                '</div>').appendTo($(".content", context))

              $(".template_name", trow).data("template_name", template.name)
              $(".template_name", trow).data("one_template", template.one)
            });
          };
        });
      },
      error: function(response){
        context.hide();
        Notifier.onError({}, OpenNebulaError(response));
      }
    });
  }

  function _import(context) {
    $.each($(".template_name:checked", context), function() {
      var template_context = $(this).closest(".vcenter_template");

      $(".vcenter_template_result:not(.success)", template_context).html(
          '<span class="fa-stack fa-2x" style="color: #dfdfdf">' +
            '<i class="fa fa-cloud fa-stack-2x"></i>' +
            '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>' +
          '</span>');

      var template_json = {
        "vmtemplate": {
          "template_raw": $(this).data("one_template")
        }
      };

      OpenNebulaTemplate.create({
        timeout: true,
        data: template_json,
        success: function(request, response) {
          $(".vcenter_template_result", template_context).addClass("success").html(
              '<span class="fa-stack fa-2x" style="color: #dfdfdf">' +
                '<i class="fa fa-cloud fa-stack-2x"></i>' +
                '<i class="fa  fa-check fa-stack-1x fa-inverse"></i>' +
              '</span>');

          $(".vcenter_template_response", template_context).html('<p style="font-size:12px" class="running-color">' +
                Locale.tr("Template created successfully") + ' ID:' + response.VMTEMPLATE.ID +
              '</p>');
        },
        error: function (request, error_json) {
          $(".vcenter_template_result", template_context).html('<span class="fa-stack fa-2x" style="color: #dfdfdf">' +
                '<i class="fa fa-cloud fa-stack-2x"></i>' +
                '<i class="fa  fa-warning fa-stack-1x fa-inverse"></i>' +
              '</span>');

          $(".vcenter_template_response", template_context).html('<p style="font-size:12px" class="error-color">' +
                (error_json.error.message || Locale.tr("Cannot contact server: is it running and reachable?")) +
              '</p>');
        }
      });
    });
  }
});

/* START_TEMPLATE */
define('hbs!tabs/templates-tab/form-panels/import/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row collapse vcenter_credentials\">\n    <div class=\"row\">\n      <div class=\"large-6 columns\">\n        <label for=\"vcenter_user\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        <input type=\"text\" name=\"vcenter_user\" id=\"vcenter_user\" />\n      </div>\n      <div class=\"large-6 columns\">\n        <label for=\"vcenter_host\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hostname", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        <input type=\"text\" name=\"vcenter_host\" id=\"vcenter_host\" />\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-6 columns\">\n        <label for=\"vcenter_password\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Password", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        <input type=\"password\" name=\"vcenter_password\" id=\"vcenter_password\" />\n      </div>\n      <div class=\"large-6 columns\">\n        <br>\n        <a class=\"button radius small right\" id=\"get_vcenter_templates\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Get VM Templates", {"name":"tr","hash":{},"data":data})))
    + "\n        </a>\n      </div>\n    </div>\n  </div>\n  <div class=\"row collapse\">\n    ";
  stack1 = ((helper = (helper = helpers.vCenterTemplatesHTML || (depth0 != null ? depth0.vCenterTemplatesHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"vCenterTemplatesHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/templates-tab/form-panels/import/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/templates-tab/form-panels/import',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','utils/vcenter/templates','hbs!./import/html','./import/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var VCenterTemplates = require('utils/vcenter/templates');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./import/html');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./import/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'import': {
        'title': Locale.tr("Import vCenter VM Templates"),
        'buttonText': Locale.tr("Import"),
        'resetButton': true
      }
    };

    this.vCenterTemplates = new VCenterTemplates();

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    return TemplateHTML({
      'formPanelId': this.formPanelId,
      'vCenterTemplatesHTML': this.vCenterTemplates.html()
    });
  }

  function _setup(context) {
    var that = this;

    Sunstone.disableFormPanelSubmit(TAB_ID);

    $("#get_vcenter_templates", context).on("click", function(){
      Sunstone.enableFormPanelSubmit(TAB_ID);

      var vcenter_user = $("#vcenter_user", context).val();
      var vcenter_password = $("#vcenter_password", context).val();
      var vcenter_host = $("#vcenter_host", context).val();

      that.vCenterTemplates.insert({
        container: context,
        vcenter_user: vcenter_user,
        vcenter_password: vcenter_password,
        vcenter_host: vcenter_host
      });

      return false;
    });

    return false;
  }

  function _submitWizard(context) {
    var that = this;

    Sunstone.hideFormPanelLoading(TAB_ID);
    Sunstone.disableFormPanelSubmit(TAB_ID);

    this.vCenterTemplates.import();

    return false;
  }

  function _onShow(context) {
  }
});

define('tabs/templates-tab',['require','utils/locale','./templates-tab/buttons','./templates-tab/actions','./templates-tab/datatable','./templates-tab/tabId','./templates-tab/dialogs/clone','./templates-tab/panels/info','./templates-tab/panels/template','./templates-tab/form-panels/create','./templates-tab/form-panels/import','./templates-tab/form-panels/instantiate'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./templates-tab/buttons');
  var Actions = require('./templates-tab/actions');
  var Table = require('./templates-tab/datatable');

  var TAB_ID = require('./templates-tab/tabId');
  var DATATABLE_ID = "dataTableTemplates";

  var _dialogs = [
    require('./templates-tab/dialogs/clone')
  ];

  var _panels = [
    require('./templates-tab/panels/info'),
    require('./templates-tab/panels/template')
  ];

  var _formPanels = [
    require('./templates-tab/form-panels/create'),
    require('./templates-tab/form-panels/import'),
    require('./templates-tab/form-panels/instantiate')
  ];

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("Templates"),
    tabClass: "subTab",
    parentTab: "vresources-tab",
    listHeader: '<i class="fa fa-fw fa-file-o"></i>&emsp;' + Locale.tr("Templates"),
    infoHeader: '<i class="fa fa-fw fa-file-o"></i>&emsp;' + Locale.tr("Template"),
    subheader: '',
    resource: 'Template',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});

define('tabs/images-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "Image.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "Image.create_dialog" : {
      type: "create_dialog",
      layout: "create"
    },
    "Image.chown" : {
      type: "confirm_with_select",
      text: Locale.tr("Change owner"),
      layout: "user_select",
      select: "User",
      tip: Locale.tr("Select the new owner") + ":"
    },
    "Image.chgrp" : {
      type: "confirm_with_select",
      text: Locale.tr("Change group"),
      layout: "user_select",
      select: "Group",
      tip: Locale.tr("Select the new group") + ":"
    },
    "Image.enable" : {
      type: "action",
      layout: "more_select",
      text: Locale.tr("Enable")
    },
    "Image.disable" : {
      type: "action",
      layout: "more_select",
      text: Locale.tr("Disable")
    },
    "Image.persistent" : {
      type: "action",
      layout: "more_select",
      text: Locale.tr("Make persistent")
    },
    "Image.nonpersistent" : {
      type: "action",
      layout: "more_select",
      text: Locale.tr("Make non persistent")
    },
    "Image.clone_dialog" : {
      type: "action",
      layout: "main",
      text: Locale.tr("Clone")
    },
    "Image.delete" : {
      type: "confirm",
      layout: "del",
      text: Locale.tr("Delete")
    },
  }

  return Buttons;
});

define('tabs/images-tab/dialogs/clone/dialogId',['require'],function(require){
  return 'cloneImageDialog';
});
define('tabs/images-tab/actions',['require','sunstone','utils/notifier','utils/locale','opennebula/image','utils/common-actions','./tabId','./form-panels/create/formPanelId','./dialogs/clone/dialogId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var OpenNebulaResource = require('opennebula/image');
  var CommonActions = require('utils/common-actions');

  var RESOURCE = "Image";
  var XML_ROOT = "IMAGE";
  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');
  var CLONE_DIALOG_ID = require('./dialogs/clone/dialogId');

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "Image.create" : _commonActions.create(CREATE_DIALOG_ID),
    "Image.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID),
    "Image.list" : _commonActions.list(),
    "Image.show" : _commonActions.show(),
    "Image.refresh" : _commonActions.refresh(),
    "Image.delete" : _commonActions.del(),
    "Image.update_template" : _commonActions.updateTemplate(),
    "Image.chown": _commonActions.multipleAction('chown'),
    "Image.chgrp": _commonActions.multipleAction('chgrp'),
    "Image.chmod": _commonActions.singleAction('chmod'),
    "Image.rename": _commonActions.singleAction('rename'),
    "Image.enable": _commonActions.multipleAction('enable'),
    "Image.disable": _commonActions.multipleAction('disable'),
    "Image.persistent": _commonActions.multipleAction('persistent'),
    "Image.nonpersistent": _commonActions.multipleAction('nonpersistent'),
    "Image.chtype": _commonActions.singleAction('chtype'),
    "Image.snapshot_flatten": _commonActions.singleAction("snapshot_flatten"),
    "Image.snapshot_revert": _commonActions.singleAction("snapshot_revert"),
    "Image.snapshot_delete": _commonActions.singleAction("snapshot_delete"),
    
    "Image.clone_dialog" : {
      type: "custom",
      call: function(){
        Sunstone.getDialog(CLONE_DIALOG_ID).show();
      }
    },
    "Image.clone" : {
      type: "single",
      call: OpenNebulaResource.clone,
      error: Notifier.onError,
      notify: true
    }
  };

  return _actions;
});


/* START_TEMPLATE */
define('hbs!tabs/images-tab/dialogs/clone/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <h3 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Clone Image", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n  </div>\n  <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\" action=\"\" class=\"custom creation\">\n    <div class=\"row\">\n      <div class=\"columns large-12\">\n        <label class=\"clone_one\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n        <label class=\"clone_several\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Several images are selected, please choose a prefix to name the new copies", {"name":"tr","hash":{},"data":data})))
    + ":\n        </label>\n        <input type=\"text\" name=\"image_clone_name\"/>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <dl class=\"accordion\" id=\"image_clone_advanced_toggle\" data-accordion>\n          <dd class=\"accordion-navigation\">\n            <a href=\"#image_clone_advanced\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced options", {"name":"tr","hash":{},"data":data})))
    + "</a>\n          </dd>\n        </dl>\n        <div id=\"image_clone_advanced\" class=\"row collapse content\">\n          <br>\n          <div class=\"large-12 columns\">\n            <span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "You can select a different target datastore", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            <br/>\n            <br/>\n          </div>\n          ";
  stack1 = ((helper = (helper = helpers.datastoreTableSelectHTML || (depth0 != null ? depth0.datastoreTableSelectHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"datastoreTableSelectHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n        </div>\n      </div>\n    </div>\n    <div class=\"form_buttons row\">\n      <button class=\"button radius right\" id=\"image_clone_button\" value=\"Image.clone\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Clone", {"name":"tr","hash":{},"data":data})))
    + "</button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/images-tab/dialogs/clone/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/images-tab/dialogs/clone',['require','utils/dialogs/dialog','hbs!./clone/html','sunstone','tabs/datastores-tab/datatable','utils/notifier','opennebula/image','opennebula/datastore','./clone/dialogId','tabs/images-tab/tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./clone/html');
  var Sunstone = require('sunstone');
  var DatastoreTable = require('tabs/datastores-tab/datatable')
  var Notifier = require('utils/notifier');
  var OpenNebulaImage = require('opennebula/image');
  var OpenNebulaDatastore = require('opennebula/datastore');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./clone/dialogId');
  var IMAGES_TAB_ID = require('tabs/images-tab/tabId')

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    this.datastoreTable = new DatastoreTable('image_clone', {
      'select': true,
      'selectOptions': {
        'filter_fn': function(ds) {
          return ds.TYPE == OpenNebulaDatastore.TYPES.IMAGE_DS;
        }
      }
    });

    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId,
      'datastoreTableSelectHTML': this.datastoreTable.dataTableHTML
    });
  }

  function _setup(dialog) {
    var that = this;
    // TODO: Show DS with the same ds mad only
    that.datastoreTable.initialize();

    $('#image_clone_advanced_toggle', dialog).click(function() {
      $('#image_clone_advanced', dialog).toggle();
      return false;
    });

    $('#' + DIALOG_ID + 'Form', dialog).submit(function() {
      var name = $('input[name="image_clone_name"]', this).val();
      var sel_elems = Sunstone.getDataTable(IMAGES_TAB_ID).elements();

      if (!name || !sel_elems.length)
        Notifier.notifyError('A name or prefix is needed!');

      var extra_info = {};

      var targeDS = that.datastoreTable.retrieveResourceTableSelect();
      if (targeDS) {
        extra_info['target_ds'] = targeDS;
      }

      if (sel_elems.length > 1) {
        for (var i = 0; i < sel_elems.length; i++) {
          //If we are cloning several images we
          //use the name as prefix
          extra_info['name'] = name + OpenNebulaImage.getName(sel_elems[i]);
          Sunstone.runAction('Image.clone', sel_elems[i], extra_info);
        }
      } else {
        extra_info['name'] = name;
        Sunstone.runAction('Image.clone', sel_elems[0], extra_info)
      }

      Sunstone.getDialog(DIALOG_ID).hide();
      Sunstone.getDialog(DIALOG_ID).reset();
      setTimeout(function() {
        Sunstone.runAction('Image.refresh');
      }, 1500);
      return false;
    });

    return false;
  }

  function _onShow(dialog) {
    var sel_elems = Sunstone.getDataTable(IMAGES_TAB_ID).elements();
    //show different text depending on how many elements are selected
    if (sel_elems.length > 1) {
      $('.clone_one', dialog).hide();
      $('.clone_several', dialog).show();
      $('input[name="image_clone_name"]', dialog).val('Copy of ');
    } else {
      $('.clone_one', dialog).show();
      $('.clone_several', dialog).hide();
      $('input[name="image_clone_name"]', dialog).val('Copy of ' + OpenNebulaImage.getName(sel_elems[0]));
    };

    $('#image_clone_advanced', dialog).hide();
    this.datastoreTable.resetResourceTableSelect();

    $("input[name='image_clone_name']", dialog).focus();

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/images-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  return "              <a id=\"div_edit_persistency_link\" class=\"edit_e\" href=\"#\"> <i class=\"fa fa-pencil-square-o right\"/></a>\n";
  },"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n        </tr>\n        ";
  stack1 = ((helper = (helper = helpers.renameTrHTML || (depth0 != null ? depth0.renameTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"renameTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastore", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.DATASTORE : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Register time", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.prettyRegTime || (depth0 != null ? depth0.prettyRegTime : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"prettyRegTime","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td_type\">"
    + escapeExpression(((helper = (helper = helpers.typeStr || (depth0 != null ? depth0.typeStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"typeStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td>\n            <div id=\"div_edit_chg_type\">\n              <a id=\"div_edit_chg_type_link\" class=\"edit_e\" href=\"#\"> <i class=\"fa fa-pencil-square-o right\"/></a>\n            </div>\n          </td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Persistent", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td_persistency\">"
    + escapeExpression(((helper = (helper = helpers.persistentStr || (depth0 != null ? depth0.persistentStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"persistentStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td>\n            <div id=\"div_edit_persistency\">\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.persistentActionEnabled : depth0), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "            </div>\n          </td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filesystem type", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.fsTypeStr || (depth0 != null ? depth0.fsTypeStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"fsTypeStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Size", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.sizeStr || (depth0 != null ? depth0.sizeStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"sizeStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "State", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.stateStr || (depth0 != null ? depth0.stateStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"stateStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Running VMS", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.RUNNING_VMS : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n      </tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\">";
  stack1 = ((helper = (helper = helpers.permissionsTableHTML || (depth0 != null ? depth0.permissionsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"permissionsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n</div>\n<div class=\"row\">\n  <div class=\"large-9 columns\">";
  stack1 = ((helper = (helper = helpers.templateTableHTML || (depth0 != null ? depth0.templateTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templateTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/images-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/images-tab/panels/info/panelId',['require'],function(require){
  return 'image_info_tab';
});
define('tabs/images-tab/panels/info',['require','utils/locale','utils/humanize','utils/panel/rename-tr','utils/panel/template-table','utils/panel/permissions-table','opennebula/image','sunstone-config','sunstone','hbs!./info/html','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var RenameTr = require('utils/panel/rename-tr');
  var TemplateTable = require('utils/panel/template-table');
  var PermissionsTable = require('utils/panel/permissions-table');
  var OpenNebulaImage = require('opennebula/image');
  var Config = require('sunstone-config');
  var Sunstone = require('sunstone');

  /*
    TEMPLATES
   */

  var TemplateInfo = require('hbs!./info/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "Image"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[RESOURCE.toUpperCase()];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var renameTrHTML = RenameTr.html(RESOURCE, this.element.NAME);
    var templateTableHTML = TemplateTable.html(this.element.TEMPLATE, RESOURCE, Locale.tr("Attributes"));
    var permissionsTableHTML = PermissionsTable.html(TAB_ID, RESOURCE, this.element);
    var prettyRegTime = Humanize.prettyTime(this.element.REGTIME);
    var fsTypeStr = this.element.FS_TYPE != undefined ? this.element.FS_TYPE : '-';
    var sizeStr = Humanize.sizeFromMB(this.element.SIZE);
    var persistentStr = parseInt(this.element.PERSISTENT) ? Locale.tr("yes") : Locale.tr("no");

    return TemplateInfo({
      'element': this.element,
      'renameTrHTML': renameTrHTML,
      'templateTableHTML': templateTableHTML,
      'permissionsTableHTML': permissionsTableHTML,
      'stateStr': OpenNebulaImage.stateStr(this.element.STATE),
      'prettyRegTime': prettyRegTime,
      'fsTypeStr': fsTypeStr,
      'persistentActionEnabled': Config.isTabActionEnabled('images-tab', "Image.persistent"),
      'persistentStr': persistentStr,
      'typeStr': OpenNebulaImage.typeStr(this.element.TYPE),
      'sizeStr': sizeStr
    });
  }

  function _setup(context) {
    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);
    RenameTr.setup(RESOURCE, this.element.ID, context);
    TemplateTable.setup(this.element.TEMPLATE, RESOURCE, this.element.ID, context);
    PermissionsTable.setup(TAB_ID, RESOURCE, this.element, context);

    // Listener for edit link for type change
    var that = this;
    context.off("click", "#div_edit_chg_type_link")
    context.on("click", "#div_edit_chg_type_link", function() {
      $(".value_td_type", context).html(
                '<select id="chg_type_select">\
                      <option value="OS">'+Locale.tr("OS")+'</option>\
                      <option value="CDROM">'+Locale.tr("CDROM")+'</option>\
                      <option value="DATABLOCK">'+Locale.tr("DATABLOCK")+'</option>\
                  </select>');

      var currentVal = "";
      switch(parseInt(that.element.TYPE)){
        case OpenNebulaImage.TYPES.OS:
          currentVal = "OS";
          break;
        case OpenNebulaImage.TYPES.CDROM:
          currentVal = "CDROM";
          break;
        case OpenNebulaImage.TYPES.DATABLOCK:
          currentVal = "DATABLOCK";
          break;
      }

      $('#chg_type_select', context).val(currentVal);
    });

    context.off("change", "#chg_type_select");
    context.on("change", "#chg_type_select", function() {
      var new_value = $(this).val();
      Sunstone.runAction("Image.chtype", that.element.ID, new_value);
    });

    // Listener for edit link for persistency change
    context.off("click", "#div_edit_persistency")
    context.on("click", "#div_edit_persistency", function() {
      $(".value_td_persistency", context).html(
                '<select id="persistency_select">\
                      <option value="yes">' + Locale.tr("yes") + '</option>\
                      <option value="no">' + Locale.tr("no") + '</option>\
                  </select>');

      $('#persistency_select', context).val(parseInt(that.element.PERSISTENT) ? "yes" : "no");
    });

    context.off("change", "#persistency_select")
    context.on("change", "#persistency_select", function() {
      var new_value = $(this).val();

      if (new_value == "yes")
          Sunstone.runAction("Image.persistent", [that.element.ID]);
      else
          Sunstone.runAction("Image.nonpersistent", [that.element.ID]);

    });

    return false;
  }
});

define('tabs/images-tab/panels/vms/panelId',['require'],function(require){
  return 'image_vms_tab';
});
define('tabs/images-tab/panels/vms',['require','utils/locale','tabs/vms-tab/datatable','./vms/panelId'],function(require){
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var VMsTable = require('tabs/vms-tab/datatable');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./vms/panelId');
  var VMS_TABLE_ID = PANEL_ID + "VMsTable";
  var RESOURCE = "Image";
  var XML_ROOT = "IMAGE";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("VMs");
    this.icon = "fa-cloud";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var vms = [];

    if (this.element.VMS.ID != undefined){
      vms = this.element.VMS.ID;

      if (!$.isArray(vms)){
        vms = [vms];
      }
    }

    var opts = {
      info: true,
      select: true,
      selectOptions: {
        read_only: true,
        fixed_ids: vms
      }
    };

    this.vmsTable = new VMsTable(VMS_TABLE_ID, opts);

    return this.vmsTable.dataTableHTML;
  }

  function _setup(context) {
    this.vmsTable.initialize();
    this.vmsTable.refreshResourceTableSelect();

    return false;
  }
});

/* START_TEMPLATE */
define('hbs!tabs/images-tab/panels/snapshots/html',['hbs','hbs/handlebars','templates/helpers/isTabActionEnabled','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "        <button class=\"button secondary small radius\" id=\"snapshot_flatten\" disabled=\"disabled\">\n          <span class=\"fa fa-sort-amount-asc\"></span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Flatten", {"name":"tr","hash":{},"data":data})))
    + "\n        </button>\n";
},"3":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "        <button class=\"button secondary small radius\" id=\"snapshot_revert\" disabled=\"disabled\">\n          <span class=\"fa fa-reply\"></span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Revert", {"name":"tr","hash":{},"data":data})))
    + "\n        </button>\n";
},"5":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "        <button class=\"button secondary small radius\" id=\"snapshot_delete\" disabled=\"disabled\">\n          <span class=\"fa fa-times\"></span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Delete", {"name":"tr","hash":{},"data":data})))
    + "\n        </button>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, functionType="function", buffer = "<div class=\"row collapse\">\n  <div class=\"large-12 columns\">\n    <span class=\"right\">\n";
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "images-tab", "Image.snapshot_flatten", {"name":"isTabActionEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "images-tab", "Image.snapshot_revert", {"name":"isTabActionEnabled","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "images-tab", "Image.snapshot_delete", {"name":"isTabActionEnabled","hash":{},"fn":this.program(5, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "    </span>\n  </div>\n</div>\n<div class=\"row collapse\">\n  <div class=\"large-12 columns\" style=\"overflow-x:auto\">\n    ";
  stack1 = ((helper = (helper = helpers.snapshotsHTML || (depth0 != null ? depth0.snapshotsHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"snapshotsHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/images-tab/panels/snapshots/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/images-tab/panels/snapshots/panelId',['require'],function(require){
  return 'image_snapshots_tab';
});
define('tabs/images-tab/panels/snapshots',['require','utils/locale','utils/humanize','sunstone-config','sunstone','utils/tree','hbs!./snapshots/html','hbs!utils/tab-datatable/empty-table','utils/dialogs/generic-confirm/dialogId','./snapshots/panelId'],function(require){
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var Config = require('sunstone-config');
  var Sunstone = require('sunstone');
  var Tree = require('utils/tree');
  var TemplateHtml = require('hbs!./snapshots/html');
  var TemplateEmptyTable = require('hbs!utils/tab-datatable/empty-table');

  var CONFIRM_DIALOG_ID = require('utils/dialogs/generic-confirm/dialogId');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./snapshots/panelId');
  var RESOURCE = "Image";
  var XML_ROOT = "IMAGE";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Snapshots");
    this.icon = "fa-camera";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {

    var snapshotsHTML = TemplateEmptyTable();

    if (!$.isEmptyObject(this.element.SNAPSHOTS)){
      var snapshots = this.element.SNAPSHOTS.SNAPSHOT;

      if (!$.isArray(snapshots)){
        snapshots = [snapshots];
      }

      var treeRoot = {
        htmlStr : '',
        subTree : []
      };

      var indexedSnapshots = {};
      var noParent = [];

      $.each(snapshots, function(){
        indexedSnapshots[this.ID] = this;

        if(this.PARENT == "-1"){
          noParent.push(this.ID);
        }
      });

      $.each(noParent, function(){
        treeRoot.subTree.push(
          _makeTree(indexedSnapshots[this], indexedSnapshots)
        );
      });

      snapshotsHTML = Tree.html(treeRoot);
    }

    return TemplateHtml({snapshotsHTML: snapshotsHTML});
  }

  function _setup(context) {
    var that = this;

    $(".snapshot_check_item", context).on("change", function() {
      // Unselect other check inputs
      var checked = $(this).is(':checked');
      $('.snapshot_check_item:checked', context).prop('checked', false);
      $(this).prop('checked', checked);

      // Enable/disable buttons
      if ($(this).is(":checked")) {
        $("#snapshot_flatten", context).prop('disabled', false);
        $("#snapshot_revert", context).prop('disabled', false);
        $("#snapshot_delete", context).prop('disabled', false);
      } else {
        $("#snapshot_flatten", context).prop('disabled', true);
        $("#snapshot_revert", context).prop('disabled', true);
        $("#snapshot_delete", context).prop('disabled', true);
      }
    });

    if (Config.isTabActionEnabled("images-tab", "Image.snapshot_flatten")) {
      $("#snapshot_flatten", context).on('click', function() {
        var snapshot_id = $(".snapshot_check_item:checked", context).attr('snapshot_id');

        Sunstone.getDialog(CONFIRM_DIALOG_ID).setParams({
          //header :
          body : Locale.tr("This will delete all the other image snapshots"),
          //question :
          submit : function(){
            Sunstone.runAction('Image.snapshot_flatten', that.element.ID,
              { "snapshot_id": snapshot_id});
          }
        });

        Sunstone.getDialog(CONFIRM_DIALOG_ID).reset();
        Sunstone.getDialog(CONFIRM_DIALOG_ID).show();

        return false;
      });
    }

    if (Config.isTabActionEnabled("images-tab", "Image.snapshot_revert")) {
      $("#snapshot_revert", context).on('click', function() {
        var snapshot_id = $(".snapshot_check_item:checked", context).attr('snapshot_id');

        Sunstone.runAction('Image.snapshot_revert', that.element.ID,
          { "snapshot_id": snapshot_id});

        return false;
      });
    }

    if (Config.isTabActionEnabled("images-tab", "Image.snapshot_delete")) {
      $("#snapshot_delete", context).on('click', function() {
        var snapshot_id = $(".snapshot_check_item:checked", context).attr('snapshot_id');

        Sunstone.getDialog(CONFIRM_DIALOG_ID).setParams({
          //header :
          body : Locale.tr("This will delete the image snapshot "+snapshot_id),
          //question :
          submit : function(){
            Sunstone.runAction('Image.snapshot_delete', that.element.ID,
              { "snapshot_id": snapshot_id});
          }
        });

        Sunstone.getDialog(CONFIRM_DIALOG_ID).reset();
        Sunstone.getDialog(CONFIRM_DIALOG_ID).show();

        return false;
      });
    }
    return false;
  }

  function _makeTree(snapshot, indexedSnapshots){
    var SPACE = '&nbsp;&nbsp;&nbsp;&nbsp;';

    var subTree = [];

    if (snapshot.CHILDREN){
      $.each(snapshot.CHILDREN.split(","), function(){
        subTree.push(
          _makeTree(indexedSnapshots[this], indexedSnapshots)
        );
      });
    }

    var html = '<div class="snapshot_row nowrap">'+
      '<input class="snapshot_check_item" type="checkbox" snapshot_id="'+snapshot.ID+'"/>'+
      SPACE + snapshot.ID + SPACE;

    var active = (snapshot.ACTIVE == "YES");

    if(active){
      html += '<i class="fa fa-play-circle-o fa-lg" title="'+
              Locale.tr("Active")+'"/>' + SPACE;
    }

    html += Humanize.prettyTime(snapshot.DATE) + SPACE +
            Humanize.sizeFromMB(snapshot.SIZE) + SPACE +
            (snapshot.NAME ? snapshot.NAME + SPACE : '');

    html += '</div>';

    return {
      htmlStr : html,
      subTree : subTree
    };
  }
});
/*
* MIT Licensed
* http://www.23developer.com/opensource
* http://github.com/23/resumable.js
* Steffen Tiedemann Christensen, steffen@23company.com
*/

(function(){
"use strict";

  var Resumable = function(opts){
    if ( !(this instanceof Resumable) ) {
      return new Resumable(opts);
    }
    this.version = 1.0;
    // SUPPORTED BY BROWSER?
    // Check if these features are support by the browser:
    // - File object type
    // - Blob object type
    // - FileList object type
    // - slicing files
    this.support = (
                   (typeof(File)!=='undefined')
                   &&
                   (typeof(Blob)!=='undefined')
                   &&
                   (typeof(FileList)!=='undefined')
                   &&
                   (!!Blob.prototype.webkitSlice||!!Blob.prototype.mozSlice||!!Blob.prototype.slice||false)
                   );
    if(!this.support) return(false);


    // PROPERTIES
    var $ = this;
    $.files = [];
    $.defaults = {
      chunkSize:1*1024*1024,
      forceChunkSize:false,
      simultaneousUploads:3,
      fileParameterName:'file',
      throttleProgressCallbacks:0.5,
      query:{},
      headers:{},
      preprocess:null,
      method:'multipart',
      prioritizeFirstAndLastChunk:false,
      target:'/',
      testChunks:true,
      generateUniqueIdentifier:null,
      maxChunkRetries:undefined,
      chunkRetryInterval:undefined,
      permanentErrors:[400, 404, 415, 500, 501],
      maxFiles:undefined,
      withCredentials:false,
      xhrTimeout:0,
      maxFilesErrorCallback:function (files, errorCount) {
        var maxFiles = $.getOpt('maxFiles');
        alert('Please upload ' + maxFiles + ' file' + (maxFiles === 1 ? '' : 's') + ' at a time.');
      },
      minFileSize:1,
      minFileSizeErrorCallback:function(file, errorCount) {
        alert(file.fileName||file.name +' is too small, please upload files larger than ' + $h.formatSize($.getOpt('minFileSize')) + '.');
      },
      maxFileSize:undefined,
      maxFileSizeErrorCallback:function(file, errorCount) {
        alert(file.fileName||file.name +' is too large, please upload files less than ' + $h.formatSize($.getOpt('maxFileSize')) + '.');
      },
      fileType: [],
      fileTypeErrorCallback: function(file, errorCount) {
        alert(file.fileName||file.name +' has type not allowed, please upload files of type ' + $.getOpt('fileType') + '.');
      }
    };
    $.opts = opts||{};
    $.getOpt = function(o) {
      var $opt = this;
      // Get multiple option if passed an array
      if(o instanceof Array) {
        var options = {};
        $h.each(o, function(option){
          options[option] = $opt.getOpt(option);
        });
        return options;
      }
      // Otherwise, just return a simple option
      if ($opt instanceof ResumableChunk) {
        if (typeof $opt.opts[o] !== 'undefined') { return $opt.opts[o]; }
        else { $opt = $opt.fileObj; }
      }
      if ($opt instanceof ResumableFile) {
        if (typeof $opt.opts[o] !== 'undefined') { return $opt.opts[o]; }
        else { $opt = $opt.resumableObj; }
      }
      if ($opt instanceof Resumable) {
        if (typeof $opt.opts[o] !== 'undefined') { return $opt.opts[o]; }
        else { return $opt.defaults[o]; }
      }
    };
    
    // EVENTS
    // catchAll(event, ...)
    // fileSuccess(file), fileProgress(file), fileAdded(file, event), fileRetry(file), fileError(file, message),
    // complete(), progress(), error(message, file), pause()
    $.events = [];
    $.on = function(event,callback){
      $.events.push(event.toLowerCase(), callback);
    };
    $.fire = function(){
      // `arguments` is an object, not array, in FF, so:
      var args = [];
      for (var i=0; i<arguments.length; i++) args.push(arguments[i]);
      // Find event listeners, and support pseudo-event `catchAll`
      var event = args[0].toLowerCase();
      for (var i=0; i<=$.events.length; i+=2) {
        if($.events[i]==event) $.events[i+1].apply($,args.slice(1));
        if($.events[i]=='catchall') $.events[i+1].apply(null,args);
      }
      if(event=='fileerror') $.fire('error', args[2], args[1]);
      if(event=='fileprogress') $.fire('progress');
    };
    
    
    // INTERNAL HELPER METHODS (handy, but ultimately not part of uploading)
    var $h = {
      stopEvent: function(e){
        e.stopPropagation();
        e.preventDefault();
      },
      each: function(o,callback){
        if(typeof(o.length)!=='undefined') {
          for (var i=0; i<o.length; i++) {
            // Array or FileList
            if(callback(o[i])===false) return;
          }
        } else {
          for (i in o) {
            // Object
            if(callback(i,o[i])===false) return;
          }
        }
      },
      generateUniqueIdentifier:function(file){
        var custom = $.getOpt('generateUniqueIdentifier');
        if(typeof custom === 'function') {
          return custom(file);
        }
        var relativePath = file.webkitRelativePath||file.fileName||file.name; // Some confusion in different versions of Firefox
        var size = file.size;
        return(size + '-' + relativePath.replace(/[^0-9a-zA-Z_-]/img, ''));
      },
      contains:function(array,test) {
        var result = false;

        $h.each(array, function(value) {
          if (value == test) {
            result = true;
            return false;
          }
          return true;
        });

        return result;
      },
      formatSize:function(size){
        if(size<1024) {
          return size + ' bytes';
        } else if(size<1024*1024) {
          return (size/1024.0).toFixed(0) + ' KB';
        } else if(size<1024*1024*1024) {
          return (size/1024.0/1024.0).toFixed(1) + ' MB';
        } else {
          return (size/1024.0/1024.0/1024.0).toFixed(1) + ' GB';
        }
      },
      getTarget:function(params){
        var target = $.getOpt('target');
        if(target.indexOf('?') < 0) {
          target += '?';
        } else {
          target += '&';
        }
        return target + params.join('&');
      }
    };

    var onDrop = function(event){
      $h.stopEvent(event);

      //handle dropped things as items if we can (this lets us deal with folders nicer in some cases)
      if (event.dataTransfer && event.dataTransfer.items) {
        loadFiles(event.dataTransfer.items, event);
      }
      //else handle them as files
      else if (event.dataTransfer && event.dataTransfer.files) {
        loadFiles(event.dataTransfer.files, event);
      }
    };
    var onDragOver = function(e) {
      e.preventDefault();
    };

    // INTERNAL METHODS (both handy and responsible for the heavy load)
    /**
     * @summary This function loops over the files passed in from a drag and drop operation and gets them ready for appendFilesFromFileList
     *            It attempts to use FileSystem API calls to extract files and subfolders if the dropped items include folders
     *            That capability is only currently available in Chrome, but if it isn't available it will just pass the items along to 
     *            appendFilesFromFileList (via enqueueFileAddition to help with asynchronous processing.)
     * @param files {Array} - the File or Entry objects to be processed depending on your browser support
     * @param event {Object} - the drop event object
     * @param [queue] {Object} - an object to keep track of our progress processing the dropped items
     * @param [path] {String} - the relative path from the originally selected folder to the current files if extracting files from subfolders
     */
    var loadFiles = function (files, event, queue, path){
      //initialize the queue object if it doesn't exist
      if (!queue) {
        queue = {
          total: 0,
          files: [],
          event: event
        };
      }

      //update the total number of things we plan to process
      updateQueueTotal(files.length, queue);

      //loop over all the passed in objects checking if they are files or folders
      for (var i = 0; i < files.length; i++) {
        var file = files[i];
        var entry, reader;

        if (file.isFile || file.isDirectory) {
          //this is an object we can handle below with no extra work needed up front
          entry = file;
        }
        else if (file.getAsEntry) {
          //get the file as an entry object if we can using the proposed HTML5 api (unlikely to get implemented by anyone)
          entry = file.getAsEntry();
        }
        else if (file.webkitGetAsEntry) {
          //get the file as an entry object if we can using the Chrome specific webkit implementation
          entry = file.webkitGetAsEntry();
        }
        else if (typeof file.getAsFile === 'function') {
          //if this is still a DataTransferItem object, get it as a file object
          enqueueFileAddition(file.getAsFile(), queue, path);
          //we just added this file object to the queue so we can go to the next object in the loop and skip the processing below
          continue;
        }
        else if (File && file instanceof File) {
          //this is already a file object so just queue it up and move on
          enqueueFileAddition(file, queue, path);
          //we just added this file object to the queue so we can go to the next object in the loop and skip the processing below
          continue;
        }
        else {
          //we can't do anything with this object, decrement the expected total and skip the processing below
          updateQueueTotal(-1, queue);
          continue;
        }

        if (!entry) {
          //there isn't anything we can do with this so decrement the total expected
          updateQueueTotal(-1, queue);
        }
        else if (entry.isFile) {
          //this is handling to read an entry object representing a file, parsing the file object is asynchronous which is why we need the queue
          //currently entry objects will only exist in this flow for Chrome
          entry.file(function(file) {
            enqueueFileAddition(file, queue, path);
          }, function(err) {
            console.warn(err);
          });
        }
        else if (entry.isDirectory) {
          //this is handling to read an entry object representing a folder, parsing the directory object is asynchronous which is why we need the queue
          //currently entry objects will only exist in this flow for Chrome
          reader = entry.createReader();

          //wrap the callback in another function so we can store the path in a closure
          var readDir = function(path){
            return function(entries){
              //process each thing in this directory recursively
              loadFiles(entries, event, queue, path);
              //this was a directory rather than a file so decrement the expected file count
              updateQueueTotal(-1, queue);
            }
          };

          reader.readEntries(readDir(entry.fullPath), function(err) {
            console.warn(err);
          });
        }
      }
    };

    /**
     * @summary Adjust the total number of files we are expecting to process
     *          if decrementing and the new expected total is equal to the number processed, flush the queue
     * @param addition {Number} - the number of additional files we expect to process (may be negative)
     * @param queue {Object} - an object to keep track of our progress processing the dropped items
     */
    var updateQueueTotal = function(addition, queue){
      queue.total += addition;
      
      // If all the files we expect have shown up, then flush the queue.
      if (queue.files.length === queue.total) {
        appendFilesFromFileList(queue.files, queue.event);
      }
    };

    /**
     * @summary Add a file to the queue of processed files, if it brings the total up to the expected total, flush the queue
     * @param file {Object} - File object to be passed along to appendFilesFromFileList eventually
     * @param queue {Object} - an object to keep track of our progress processing the dropped items
     * @param [path] {String} - the file's relative path from the originally dropped folder if we are parsing folder content (Chrome only for now)
     */
    var enqueueFileAddition = function(file, queue, path) {
      //store the path to this file if it came in as part of a folder
      if (path) file.relativePath = path + '/' + file.name;
      queue.files.push(file);

      // If all the files we expect have shown up, then flush the queue.
      if (queue.files.length === queue.total) {
        appendFilesFromFileList(queue.files, queue.event);
      }
    };

    var appendFilesFromFileList = function(fileList, event){
      // check for uploading too many files
      var errorCount = 0;
      var o = $.getOpt(['maxFiles', 'minFileSize', 'maxFileSize', 'maxFilesErrorCallback', 'minFileSizeErrorCallback', 'maxFileSizeErrorCallback', 'fileType', 'fileTypeErrorCallback']);
      if (typeof(o.maxFiles)!=='undefined' && o.maxFiles<(fileList.length+$.files.length)) {
        // if single-file upload, file is already added, and trying to add 1 new file, simply replace the already-added file 
        if (o.maxFiles===1 && $.files.length===1 && fileList.length===1) {
          $.removeFile($.files[0]);
        } else {
          o.maxFilesErrorCallback(fileList, errorCount++);
          return false;
        }
      }
      var files = [];
      $h.each(fileList, function(file){
        var fileName = file.name.split('.');
        var fileType = fileName[fileName.length-1].toLowerCase();
        
        if (o.fileType.length > 0 && !$h.contains(o.fileType, fileType)) {
          o.fileTypeErrorCallback(file, errorCount++);
          return false;
        }

        if (typeof(o.minFileSize)!=='undefined' && file.size<o.minFileSize) {
          o.minFileSizeErrorCallback(file, errorCount++);
          return false;
        }
        if (typeof(o.maxFileSize)!=='undefined' && file.size>o.maxFileSize) {
          o.maxFileSizeErrorCallback(file, errorCount++);
          return false;
        }

        // directories have size == 0
        if (!$.getFromUniqueIdentifier($h.generateUniqueIdentifier(file))) {(function(){
          var f = new ResumableFile($, file);
          window.setTimeout(function(){
            $.files.push(f);
            files.push(f);
            f.container = (typeof event != 'undefined' ? event.srcElement : null);
            $.fire('fileAdded', f, event)
          },0);
        })()};
      });
      window.setTimeout(function(){
        $.fire('filesAdded', files)
      },0);
    };

    // INTERNAL OBJECT TYPES
    function ResumableFile(resumableObj, file){
      var $ = this;
      $.opts = {};
      $.getOpt = resumableObj.getOpt;
      $._prevProgress = 0;
      $.resumableObj = resumableObj;
      $.file = file;
      $.fileName = file.fileName||file.name; // Some confusion in different versions of Firefox
      $.size = file.size;
      $.relativePath = file.webkitRelativePath || file.relativePath || $.fileName;
      $.uniqueIdentifier = $h.generateUniqueIdentifier(file);
      $._pause = false;
      $.container = '';
      var _error = false;

      // Callback when something happens within the chunk
      var chunkEvent = function(event, message){
        // event can be 'progress', 'success', 'error' or 'retry'
        switch(event){
        case 'progress':
          $.resumableObj.fire('fileProgress', $);
          break;
        case 'error':
          $.abort();
          _error = true;
          $.chunks = [];
          $.resumableObj.fire('fileError', $, message);
          break;
        case 'success':
          if(_error) return;
          $.resumableObj.fire('fileProgress', $); // it's at least progress
          if($.isComplete()) {
            $.resumableObj.fire('fileSuccess', $, message);
          }
          break;
        case 'retry':
          $.resumableObj.fire('fileRetry', $);
          break;
        }
      };

      // Main code to set up a file object with chunks,
      // packaged to be able to handle retries if needed.
      $.chunks = [];
      $.abort = function(){
        // Stop current uploads
        var abortCount = 0;
        $h.each($.chunks, function(c){
          if(c.status()=='uploading') {
            c.abort();
            abortCount++;
          }
        });
        if(abortCount>0) $.resumableObj.fire('fileProgress', $);
      };
      $.cancel = function(){
        // Reset this file to be void
        var _chunks = $.chunks;
        $.chunks = [];
        // Stop current uploads
        $h.each(_chunks, function(c){
          if(c.status()=='uploading')  {
            c.abort();
            $.resumableObj.uploadNextChunk();
          }
        });
        $.resumableObj.removeFile($);
        $.resumableObj.fire('fileProgress', $);
      };
      $.retry = function(){
        $.bootstrap();
        var firedRetry = false;
        $.resumableObj.on('chunkingComplete', function(){
          if(!firedRetry) $.resumableObj.upload();
          firedRetry = true;
        });
      };
      $.bootstrap = function(){
        $.abort();
        _error = false;
        // Rebuild stack of chunks from file
        $.chunks = [];
        $._prevProgress = 0;
        var round = $.getOpt('forceChunkSize') ? Math.ceil : Math.floor;
        var maxOffset = Math.max(round($.file.size/$.getOpt('chunkSize')),1);
        for (var offset=0; offset<maxOffset; offset++) {(function(offset){
            window.setTimeout(function(){
                $.chunks.push(new ResumableChunk($.resumableObj, $, offset, chunkEvent));
                $.resumableObj.fire('chunkingProgress',$,offset/maxOffset);
            },0);
        })(offset)}
        window.setTimeout(function(){
            $.resumableObj.fire('chunkingComplete',$);
        },0);
      };
      $.progress = function(){
        if(_error) return(1);
        // Sum up progress across everything
        var ret = 0;
        var error = false;
        $h.each($.chunks, function(c){
          if(c.status()=='error') error = true;
          ret += c.progress(true); // get chunk progress relative to entire file
        });
        ret = (error ? 1 : (ret>0.99999 ? 1 : ret));
        ret = Math.max($._prevProgress, ret); // We don't want to lose percentages when an upload is paused
        $._prevProgress = ret;
        return(ret);
      };
      $.isUploading = function(){
        var uploading = false;
        $h.each($.chunks, function(chunk){
          if(chunk.status()=='uploading') {
            uploading = true;
            return(false);
          }
        });
        return(uploading);
      };    
      $.isComplete = function(){
        var outstanding = false;
        $h.each($.chunks, function(chunk){
          var status = chunk.status();
          if(status=='pending' || status=='uploading' || chunk.preprocessState === 1) {
            outstanding = true;
            return(false);
          }
        });
        return(!outstanding);
      };
      $.pause = function(pause){
          if(typeof(pause)==='undefined'){
              $._pause = ($._pause ? false : true);
          }else{
              $._pause = pause;
          }
      };
      $.isPaused = function() {
        return $._pause;
      };


      // Bootstrap and return
      $.resumableObj.fire('chunkingStart', $);
      $.bootstrap();
      return(this);
    }

    function ResumableChunk(resumableObj, fileObj, offset, callback){
      var $ = this;
      $.opts = {};
      $.getOpt = resumableObj.getOpt;
      $.resumableObj = resumableObj;
      $.fileObj = fileObj;
      $.fileObjSize = fileObj.size;
      $.fileObjType = fileObj.file.type;
      $.offset = offset;
      $.callback = callback;
      $.lastProgressCallback = (new Date);
      $.tested = false;
      $.retries = 0;
      $.pendingRetry = false;
      $.preprocessState = 0; // 0 = unprocessed, 1 = processing, 2 = finished

      // Computed properties
      var chunkSize = $.getOpt('chunkSize');
      $.loaded = 0;
      $.startByte = $.offset*chunkSize;
      $.endByte = Math.min($.fileObjSize, ($.offset+1)*chunkSize);
      if ($.fileObjSize-$.endByte < chunkSize && !$.getOpt('forceChunkSize')) {
        // The last chunk will be bigger than the chunk size, but less than 2*chunkSize
        $.endByte = $.fileObjSize;
      }
      $.xhr = null;

      // test() makes a GET request without any data to see if the chunk has already been uploaded in a previous session
      $.test = function(){
        // Set up request and listen for event
        $.xhr = new XMLHttpRequest();

        var testHandler = function(e){
          $.tested = true;
          var status = $.status();
          if(status=='success') {
            $.callback(status, $.message());
            $.resumableObj.uploadNextChunk();
          } else {
            $.send();
          }
        };
        $.xhr.addEventListener('load', testHandler, false);
        $.xhr.addEventListener('error', testHandler, false);
        $.xhr.addEventListener('timeout', testHandler, false);

        // Add data from the query options
        var params = [];
        var customQuery = $.getOpt('query'); 
        if(typeof customQuery == 'function') customQuery = customQuery($.fileObj, $);
        $h.each(customQuery, function(k,v){
          params.push([encodeURIComponent(k), encodeURIComponent(v)].join('='));
        });
        // Add extra data to identify chunk
        params.push(['resumableChunkNumber', encodeURIComponent($.offset+1)].join('='));
        params.push(['resumableChunkSize', encodeURIComponent($.getOpt('chunkSize'))].join('='));
        params.push(['resumableCurrentChunkSize', encodeURIComponent($.endByte - $.startByte)].join('='));
        params.push(['resumableTotalSize', encodeURIComponent($.fileObjSize)].join('='));
        params.push(['resumableType', encodeURIComponent($.fileObjType)].join('='));
        params.push(['resumableIdentifier', encodeURIComponent($.fileObj.uniqueIdentifier)].join('='));
        params.push(['resumableFilename', encodeURIComponent($.fileObj.fileName)].join('='));
        params.push(['resumableRelativePath', encodeURIComponent($.fileObj.relativePath)].join('='));
        params.push(['resumableTotalChunks', encodeURIComponent($.fileObj.chunks.length)].join('='));
        // Append the relevant chunk and send it
        $.xhr.open('GET', $h.getTarget(params));
        $.xhr.timeout = $.getOpt('xhrTimeout');
        $.xhr.withCredentials = $.getOpt('withCredentials');
        // Add data from header options
        $h.each($.getOpt('headers'), function(k,v) {
          $.xhr.setRequestHeader(k, v);
        });
        $.xhr.send(null);
      };

      $.preprocessFinished = function(){
        $.preprocessState = 2;
        $.send();
      };

      // send() uploads the actual data in a POST call
      $.send = function(){
        var preprocess = $.getOpt('preprocess');
        if(typeof preprocess === 'function') {
          switch($.preprocessState) {
          case 0: preprocess($); $.preprocessState = 1; return;
          case 1: return;
          case 2: break;
          }
        }
        if($.getOpt('testChunks') && !$.tested) {
          $.test();
          return;
        }

        // Set up request and listen for event
        $.xhr = new XMLHttpRequest();

        // Progress
        $.xhr.upload.addEventListener('progress', function(e){
          if( (new Date) - $.lastProgressCallback > $.getOpt('throttleProgressCallbacks') * 1000 ) {
            $.callback('progress');
            $.lastProgressCallback = (new Date);
          }
          $.loaded=e.loaded||0;
        }, false);
        $.loaded = 0;
        $.pendingRetry = false;
        $.callback('progress');

        // Done (either done, failed or retry)
        var doneHandler = function(e){
          var status = $.status();
          if(status=='success'||status=='error') {
            $.callback(status, $.message());
            $.resumableObj.uploadNextChunk();
          } else {
            $.callback('retry', $.message());
            $.abort();
            $.retries++;
            var retryInterval = $.getOpt('chunkRetryInterval');          
            if(retryInterval !== undefined) {
              $.pendingRetry = true;
              setTimeout($.send, retryInterval);
            } else {
              $.send();
            }
          }
        };
        $.xhr.addEventListener('load', doneHandler, false);
        $.xhr.addEventListener('error', doneHandler, false);
        $.xhr.addEventListener('timeout', doneHandler, false);

        // Set up the basic query data from Resumable
        var query = {
          resumableChunkNumber: $.offset+1,
          resumableChunkSize: $.getOpt('chunkSize'),
          resumableCurrentChunkSize: $.endByte - $.startByte,
          resumableTotalSize: $.fileObjSize,
          resumableType: $.fileObjType,
          resumableIdentifier: $.fileObj.uniqueIdentifier,
          resumableFilename: $.fileObj.fileName,
          resumableRelativePath: $.fileObj.relativePath,
          resumableTotalChunks: $.fileObj.chunks.length
        };
        // Mix in custom data
        var customQuery = $.getOpt('query');
        if(typeof customQuery == 'function') customQuery = customQuery($.fileObj, $);
        $h.each(customQuery, function(k,v){
          query[k] = v;
        });

        var func   = ($.fileObj.file.slice ? 'slice' : ($.fileObj.file.mozSlice ? 'mozSlice' : ($.fileObj.file.webkitSlice ? 'webkitSlice' : 'slice'))),
        bytes  = $.fileObj.file[func]($.startByte,$.endByte), 
        data   = null,
        target = $.getOpt('target');
        
        if ($.getOpt('method') === 'octet') {
          // Add data from the query options
          data = bytes;
          var params = [];
          $h.each(query, function(k,v){
            params.push([encodeURIComponent(k), encodeURIComponent(v)].join('='));
          });
          target = $h.getTarget(params);
        } else {
          // Add data from the query options
          data = new FormData();
          $h.each(query, function(k,v){
            data.append(k,v);
          });
          data.append($.getOpt('fileParameterName'), bytes);
        }
        
        $.xhr.open('POST', target);
        $.xhr.timeout = $.getOpt('xhrTimeout');
        $.xhr.withCredentials = $.getOpt('withCredentials');
        // Add data from header options
        $h.each($.getOpt('headers'), function(k,v) {
          $.xhr.setRequestHeader(k, v);
        });
        $.xhr.send(data);
      };
      $.abort = function(){
        // Abort and reset
        if($.xhr) $.xhr.abort();
        $.xhr = null;
      };
      $.status = function(){
        // Returns: 'pending', 'uploading', 'success', 'error'
        if($.pendingRetry) {
          // if pending retry then that's effectively the same as actively uploading,
          // there might just be a slight delay before the retry starts
          return('uploading');
        } else if(!$.xhr) {
          return('pending');
        } else if($.xhr.readyState<4) {
          // Status is really 'OPENED', 'HEADERS_RECEIVED' or 'LOADING' - meaning that stuff is happening
          return('uploading');
        } else {
          if($.xhr.status==200) {
            // HTTP 200, perfect
            return('success');
          } else if($h.contains($.getOpt('permanentErrors'), $.xhr.status) || $.retries >= $.getOpt('maxChunkRetries')) {
            // HTTP 415/500/501, permanent error
            return('error');
          } else {
            // this should never happen, but we'll reset and queue a retry
            // a likely case for this would be 503 service unavailable
            $.abort();
            return('pending');
          }
        }
      };
      $.message = function(){
        return($.xhr ? $.xhr.responseText : '');
      };
      $.progress = function(relative){
        if(typeof(relative)==='undefined') relative = false;
        var factor = (relative ? ($.endByte-$.startByte)/$.fileObjSize : 1);
        if($.pendingRetry) return(0);
        var s = $.status();
        switch(s){
        case 'success':
        case 'error':
          return(1*factor);
        case 'pending':
          return(0*factor);
        default:
          return($.loaded/($.endByte-$.startByte)*factor);
        }
      };
      return(this);
    }

    // QUEUE
    $.uploadNextChunk = function(){
      var found = false;

      // In some cases (such as videos) it's really handy to upload the first
      // and last chunk of a file quickly; this let's the server check the file's
      // metadata and determine if there's even a point in continuing.
      if ($.getOpt('prioritizeFirstAndLastChunk')) {
        $h.each($.files, function(file){
          if(file.chunks.length && file.chunks[0].status()=='pending' && file.chunks[0].preprocessState === 0) {
            file.chunks[0].send();
            found = true;
            return(false);
          }
          if(file.chunks.length>1 && file.chunks[file.chunks.length-1].status()=='pending' && file.chunks[file.chunks.length-1].preprocessState === 0) {
            file.chunks[file.chunks.length-1].send();
            found = true;
            return(false);
          }
        });
        if(found) return(true);
      }

      // Now, simply look for the next, best thing to upload
      $h.each($.files, function(file){
        if(file.isPaused()===false){
         $h.each(file.chunks, function(chunk){
           if(chunk.status()=='pending' && chunk.preprocessState === 0) {
             chunk.send();
             found = true;
             return(false);
           }
          });
        }
        if(found) return(false);
      });
      if(found) return(true);

      // The are no more outstanding chunks to upload, check is everything is done
      var outstanding = false;
      $h.each($.files, function(file){
        if(!file.isComplete()) {
          outstanding = true;
          return(false);
        }
      });
      if(!outstanding) {
        // All chunks have been uploaded, complete
        $.fire('complete');
      }
      return(false);
    };


    // PUBLIC METHODS FOR RESUMABLE.JS
    $.assignBrowse = function(domNodes, isDirectory){
      if(typeof(domNodes.length)=='undefined') domNodes = [domNodes];

      $h.each(domNodes, function(domNode) {
        var input;
        if(domNode.tagName==='INPUT' && domNode.type==='file'){
          input = domNode;
        } else {
          input = document.createElement('input');
          input.setAttribute('type', 'file');
          input.style.display = 'none';
          domNode.addEventListener('click', function(){
            input.style.opacity = 0;
            input.style.display='block';
            input.focus();
            input.click();
            input.style.display='none';
          }, false);
          domNode.appendChild(input);
        }
        var maxFiles = $.getOpt('maxFiles');
        if (typeof(maxFiles)==='undefined'||maxFiles!=1){
          input.setAttribute('multiple', 'multiple');
        } else {
          input.removeAttribute('multiple');
        }
        if(isDirectory){
          input.setAttribute('webkitdirectory', 'webkitdirectory');
        } else {
          input.removeAttribute('webkitdirectory');
        }
        // When new files are added, simply append them to the overall list
        input.addEventListener('change', function(e){
          appendFilesFromFileList(e.target.files,e);
          e.target.value = '';
        }, false);
      });
    };
    $.assignDrop = function(domNodes){
      if(typeof(domNodes.length)=='undefined') domNodes = [domNodes];

      $h.each(domNodes, function(domNode) {
        domNode.addEventListener('dragover', onDragOver, false);
        domNode.addEventListener('drop', onDrop, false);
      });
    };
    $.unAssignDrop = function(domNodes) {
      if (typeof(domNodes.length) == 'undefined') domNodes = [domNodes];

      $h.each(domNodes, function(domNode) {
        domNode.removeEventListener('dragover', onDragOver);
        domNode.removeEventListener('drop', onDrop);
      });
    };
    $.isUploading = function(){
      var uploading = false;
      $h.each($.files, function(file){
        if (file.isUploading()) {
          uploading = true;
          return(false);
        }
      });
      return(uploading);
    };
    $.upload = function(){
      // Make sure we don't start too many uploads at once
      if($.isUploading()) return;
      // Kick off the queue
      $.fire('uploadStart');
      for (var num=1; num<=$.getOpt('simultaneousUploads'); num++) {
        $.uploadNextChunk();
      }
    };
    $.pause = function(){
      // Resume all chunks currently being uploaded
      $h.each($.files, function(file){
        file.abort();
      });
      $.fire('pause');
    };
    $.cancel = function(){
      for(var i = $.files.length - 1; i >= 0; i--) {
        $.files[i].cancel();
      }
      $.fire('cancel');
    };
    $.progress = function(){
      var totalDone = 0;
      var totalSize = 0;
      // Resume all chunks currently being uploaded
      $h.each($.files, function(file){
        totalDone += file.progress()*file.size;
        totalSize += file.size;
      });
      return(totalSize>0 ? totalDone/totalSize : 0);
    };
    $.addFile = function(file, event){
      appendFilesFromFileList([file], event);
    };
    $.removeFile = function(file){
      for(var i = $.files.length - 1; i >= 0; i--) {
        if($.files[i] === file) {
          $.files.splice(i, 1);
        }
      }
    };
    $.getFromUniqueIdentifier = function(uniqueIdentifier){
      var ret = false;
      $h.each($.files, function(f){
        if(f.uniqueIdentifier==uniqueIdentifier) ret = f;
      });
      return(ret);
    };
    $.getSize = function(){
      var totalSize = 0;
      $h.each($.files, function(file){
        totalSize += file.size;
      });
      return(totalSize);
    };

    return(this);
  };


  // Node.js-style export for Node and Component
  if (typeof module != 'undefined') {
    module.exports = Resumable;
  } else if (typeof define === "function" && define.amd) {
    // AMD/requirejs: Define the module
    define('resumable',[],function(){
      return Resumable;
    });
  } else {
    // Browser: Expose to window
    window.Resumable = Resumable;
  }

})();

define('utils/browser-info',['require'],function(require) {

  /**
   * Returns the version of Internet Explorer or a -1
   * (indicating the use of another browser).
   * @return {integer} version
   */
  function _getInternetExplorerVersion() {
    var rv = -1; // Return value assumes failure.
    if (navigator.appName == 'Microsoft Internet Explorer') {
      var ua = navigator.userAgent;
      var re  = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
      if (re.exec(ua) != null)
          rv = parseFloat(RegExp.$1);
    }
    return rv;
  }

  return {
    'getInternetExplorerVersion': _getInternetExplorerVersion
  };
});



/* START_TEMPLATE */
define('hbs!tabs/images-tab/form-panels/create/wizard',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/isTabActionEnabled','templates/helpers/advancedSection'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "    <div class=\"row\">\n      <div class=\"large-4 columns\">\n        <input type=\"checkbox\" id=\"img_persistent\" name=\"img_persistent\" value=\"YES\" />\n        <label for=\"img_persistent\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Persistent", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Persistence of the image", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </label>\n      </div>\n    </div>\n";
},"3":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "    <div class=\"row\">\n      <div class=\"medium-6 columns\">\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <label for=\"img_dev_prefix\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Device prefix", {"name":"tr","hash":{},"data":data})))
    + "\n              <span class=\"tip\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Prefix for the emulated device this image will be mounted at. For instance, “hd”, “sd”. If omitted, the default value is the one defined in oned.conf (installation default is “hd”).", {"name":"tr","hash":{},"data":data})))
    + "\n              </span>\n            </label>\n            <input type=\"text\" name=\"img_dev_prefix\" id=\"img_dev_prefix\" />\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <label for=\"img_driver\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Driver", {"name":"tr","hash":{},"data":data})))
    + "\n              <span class=\"tip\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Specific image mapping driver. KVM: raw, qcow2. XEN: tap:aio, file:", {"name":"tr","hash":{},"data":data})))
    + "\n              </span>\n            </label>\n            <input type=\"text\" name=\"img_driver\" id=\"img_driver\" />\n          </div>\n        </div>\n      </div>\n      <div class=\"medium-6 columns\">\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <label for=\"img_target\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Target", {"name":"tr","hash":{},"data":data})))
    + "\n              <span class=\"tip\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Target on which the image will be mounted at. For example: hda, sdb...", {"name":"tr","hash":{},"data":data})))
    + "\n              </span>\n            </label>\n            <input type=\"text\" name=\"img_target\" id=\"img_target\" />\n          </div>\n        </div>\n      </div>\n    </div>\n    <br>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom attributes", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        <br>\n        <br>\n      </div>\n    </div>\n    ";
  stack1 = ((helper = (helper = helpers.customTagsHTML || (depth0 != null ? depth0.customTagsHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customTagsHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"medium-4 columns\">\n      <label for=\"img_name\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name that the Image will get. Every image must have a unique name.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input required type=\"text\" name=\"img_name\" id=\"img_name\" />\n    </div>\n    <div class=\"medium-8 columns\">\n      <label for=\"img_desc\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Human readable description of the image for other users.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <textarea name=\"img_desc\" id=\"img_desc\" rows=\"4\"></textarea>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"medium-4 columns\">\n      <label for=\"img_type\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type of the image.", {"name":"tr","hash":{},"data":data})))
    + "\n          <br/>\n          <br/>\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "OS images contain a working operative system.", {"name":"tr","hash":{},"data":data})))
    + "\n          <br/>\n          <br/>\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CDROM images are readonly data.", {"name":"tr","hash":{},"data":data})))
    + "\n          <br/>\n          <br/>\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "DATABLOCK images are a storage for data. They can be created from previous existing data, or as an empty drive.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <select name=\"img_type\" id=\"img_type\">\n        <option value=\"OS\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "OS", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"CDROM\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CDROM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"DATABLOCK\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "DATABLOCK", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n    <div class=\"medium-8 columns\">\n      <label for=\"img_datastore\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastore", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select the datastore for this image", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </label>\n      <div id=\"img_datastore\" name=\"img_datastore\"></div>\n    </div>\n  </div>\n";
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "images-tab", "Image.persistent", {"name":"isTabActionEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "  <br>\n  <fieldset>\n    <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Image location", {"name":"tr","hash":{},"data":data})))
    + ":</legend>\n    <div class=\"row\" id=\"src_path_select\">\n      <div class=\"large-12 columns text-center\">\n        <input type=\"radio\" name=\"src_path\" id=\"path_image\" value=\"path\">\n        <label for=\"path_image\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Provide a path", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        <input type=\"radio\" name=\"src_path\" id=\"upload_image\" value=\"upload\">\n        <label for=\"upload_image\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Upload", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        <input type=\"radio\" name=\"src_path\" id=\"datablock_img\" value=\"datablock\" disabled>\n        <label for=\"datablock_img\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Empty datablock", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      </div>\n    </div>\n    <br>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <label for=\"img_path\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Path", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Path to the original file that will be copied to the image repository. If not specified for a DATABLOCK type image, an empty image will be created.", {"name":"tr","hash":{},"data":data})))
    + "\n          </span>\n        </label>\n        <input type=\"text\" name=\"img_path\" id=\"img_path\" />\n      </div>\n    </div>\n    <div class=\"row\">\n      <div id=\"file-uploader\" class=\"large-12 columns text-center\">\n        <label id=\"file-uploader-label\" for=\"file-uploader-input\"></label>\n        <input id=\"file-uploader-input\" type=\"file\"/>\n      </div>\n    </div>\n    <div class=\"img_size row\">\n      <div class=\"medium-6 columns\">\n        <label for=\"img_size\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Size", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Size of the datablock in MB.", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </label>\n        <input type=\"text\" name=\"img_size\" id=\"img_size\" />\n      </div>\n      <div class=\"medium-6 columns\">\n        <label for=\"img_fstype\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "FS type", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type of file system to be built.", {"name":"tr","hash":{},"data":data})))
    + "\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Plain. When the disk image is used directly by the hypervisor we can format the image, and so it is ready to be used by the guest OS. Values: ext2, ext3, ext4, ntfs, reiserfs, jfs, swap. Any other fs supported by mkfs will work if no special option is needed.", {"name":"tr","hash":{},"data":data})))
    + "\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Formatted. The disk image is stored in a hypervisor specific format VMDK or Qcow2. Then we cannot really make a filesystem on the image, just create the device and let the guest OS format the disk. Use raw to not to format the new image. Values: raw, qcow2, vmdk_*.", {"name":"tr","hash":{},"data":data})))
    + "\n          </span>\n        </label>\n        <input type=\"text\" name=\"img_fstype\" id=\"img_fstype\" />\n      </div>\n    </div>\n  </fieldset>\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced Options", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</form>";
},"useData":true});
Handlebars.registerPartial('tabs/images-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/images-tab/form-panels/create/advanced',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Advanced\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"columns large-12\">\n      <label for=\"img_datastores_raw\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastore", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n      <div id=\"img_datastore_raw\" name=\"img_datastore_raw\"></div>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <p>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Write the Image template here", {"name":"tr","hash":{},"data":data})))
    + "</p>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <textarea id=\"template\" rows=\"15\" required></textarea>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/images-tab/form-panels/create/advanced', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/images-tab/form-panels/create',['require','utils/form-panels/form-panel','resumable','sunstone','opennebula/error','utils/notifier','utils/locale','utils/tips','utils/resource-select','utils/custom-tags-table','utils/browser-info','hbs!./create/wizard','hbs!./create/advanced','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Resumable = require('resumable');
  var Sunstone = require('sunstone');
  var OpenNebulaError = require('opennebula/error');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var ResourceSelect = require('utils/resource-select');
  var CustomTagsTable = require('utils/custom-tags-table');
  var BrowserInfo = require('utils/browser-info');

  var TemplateWizardHTML = require('hbs!./create/wizard');
  var TemplateAdvancedHTML = require('hbs!./create/advanced');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create Image"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      }
    };

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.htmlAdvanced = _htmlAdvanced;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.submitAdvanced = _submitAdvanced;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.setup = _setup;

  return FormPanel;


  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    return TemplateWizardHTML({
      'formPanelId': this.formPanelId,
      'customTagsHTML': CustomTagsTable.html(),
    });
  }

  function _htmlAdvanced() {
    return TemplateAdvancedHTML({formPanelId: this.formPanelId});
  }

  function _onShow(context) {
    $("#img_name", context).focus();

    var ds_id = $('#img_datastore .resource_list_select', context).val();
    var ds_id_raw = $('#img_datastore_raw .resource_list_select', context).val();

    // Filter out DS with type system (1) or file (2)
    var filter_att = ["TYPE", "TYPE"];
    var filter_val = ["1", "2"];

    ResourceSelect.insert('div#img_datastore', context, "Datastore",
                        ds_id, false, null, filter_att, filter_val);

    ResourceSelect.insert('div#img_datastore_raw', context, "Datastore",
                        ds_id_raw, false, null, filter_att, filter_val);

    return false;
  }

  function _setup(context) {
    var that = this;
    Tips.setup(context);

    $('select#img_type', context).change(function() {
      var value = $(this).val();
      switch (value){
      case "DATABLOCK":
        $('#datablock_img', context).removeAttr("disabled");
        break;
      default:
        $('#datablock_img', context).attr('disabled', 'disabled');
        $('#path_image', context).click();

      }
    });

    $('#img_path,#img_fstype,#img_size,#file-uploader', context).closest('.row').hide();

    $("input[name='src_path']", context).change(function() {
      var value = $(this).val();
      switch (value){
      case "path":
        $('#img_fstype,#img_size,#file-uploader', context).closest('.row').hide();
        $('#img_path', context).closest('.row').show();

        $('#img_path', context).attr('required', '');
        $('#img_size', context).removeAttr('required');
        break;
      case "datablock":
        $('#img_path,#file-uploader', context).closest('.row').hide();
        $('#img_fstype,#img_size', context).closest('.row').show();

        $('#img_path', context).removeAttr('required');
        $('#img_size', context).attr('required', '');
        break;
      case "upload":
        $('#img_path,#img_fstype,#img_size', context).closest('.row').hide();
        $('#file-uploader', context).closest('.row').show();

        $('#img_path', context).removeAttr('required');
        $('#img_size', context).removeAttr('required');
        break;
      }
    });

    $('#path_image', context).click();

    CustomTagsTable.setup(context);

    if (BrowserInfo.getInternetExplorerVersion() > -1) {
      $("#upload_image").attr("disabled", "disabled");
    } else {
      that.uploader = new Resumable({
        target: 'upload_chunk',
        chunkSize: 10 * 1024 * 1024,
        maxFiles: 1,
        testChunks: false,
        query: {
          csrftoken: csrftoken
        }
      });

      that.uploader.assignBrowse($('#file-uploader-input', context));

      var fileName = '';
      var file_input = false;

      that.uploader.on('fileAdded', function(file) {
        fileName = file.fileName;
        file_input = fileName;

        $('#file-uploader-input', context).hide()
        $("#file-uploader-label", context).html(file.fileName);
      });

      that.uploader.on('uploadStart', function() {
        $('#upload_progress_bars').append('<div id="' + fileName + 'progressBar" class="row" style="margin-bottom:10px">\
            <div id="' + fileName + '-info" class="large-2 columns dataTables_info">\
              ' + Locale.tr("Uploading...") + '\
            </div>\
            <div class="large-10 columns">\
              <div id="upload_progress_container" class="progress nine radius" style="height:25px !important">\
                <span class="meter" style="width:0%"></span>\
              </div>\
              <div class="progress-text" style="margin-left:15px">' + fileName + '</div>\
            </div>\
          </div>');
      });

      that.uploader.on('progress', function() {
        $('span.meter', $('div[id="' + fileName + 'progressBar"]')).css('width', this.progress() * 100.0 + '%')
      });
    }

    return false;
  }

  function _submitWizard(context) {
    var that = this;
    var upload = false;

    var ds_id = $('#img_datastore .resource_list_select', context).val();
    if (!ds_id) {
      Sunstone.hideFormPanelLoading(TAB_ID);
      Notifier.notifyError(Locale.tr("Please select a datastore for this image"));
      return false;
    }

    var img_json = {};

    var name = $('#img_name', context).val();
    img_json["NAME"] = name;

    var desc = $('#img_desc', context).val();
    if (desc.length) {
      img_json["DESCRIPTION"] = desc;
    }

    var type = $('#img_type', context).val();
    img_json["TYPE"] = type;

    img_json["PERSISTENT"] = $('#img_persistent:checked', context).length ? "YES" : "NO";

    var dev_prefix = $('#img_dev_prefix', context).val();
    if (dev_prefix.length) {
      img_json["DEV_PREFIX"] = dev_prefix;
    }

    var driver = $('#img_driver', context).val();
    if (driver.length)
        img_json["DRIVER"] = driver;

    var target = $('#img_target', context).val();
    if (target)
        img_json["TARGET"] = target;

    switch ($('#src_path_select input:checked', context).val()){
    case "path":
      path = $('#img_path', context).val();
      if (path) img_json["PATH"] = path;
      break;
    case "datablock":
      size = $('#img_size', context).val();
      fstype = $('#img_fstype', context).val();
      if (size) img_json["SIZE"] = size;
      if (fstype) img_json["FSTYPE"] = fstype;
      break;
    case "upload":
      upload = true;
      break;
    }

    $.extend(img_json, CustomTagsTable.retrieve(context));

    var img_obj = {
      "image" : img_json,
      "ds_id" : ds_id
    };

    //this is an image upload we trigger FileUploader
    //to start the upload
    if (upload) {
      if (that.uploader.files.length == 0) {
        Sunstone.hideFormPanelLoading(TAB_ID);
        Notifier.notifyError(Locale.tr("Please select a file to upload"));
        return false;
      }

      Sunstone.resetFormPanel(that.tabId, that.formPanelId);
      Sunstone.hideFormPanel(that.tabId);

      that.uploader.on('fileSuccess', function(file) {
        $('div[id="' + file.fileName + '-info"]').text(Locale.tr("Registering in OpenNebula"));
        $.ajax({
          url: 'upload',
          type: "POST",
          data: {
            csrftoken: csrftoken,
            img : JSON.stringify(img_obj),
            file: file.fileName,
            tempfile: file.uniqueIdentifier
          },
          success: function() {
            Notifier.notifyMessage("Image uploaded correctly");
            $('div[id="' + file.fileName + 'progressBar"]').remove();
            Sunstone.runAction("Image.refresh");
          },
          error: function(response) {
            Notifier.onError({}, OpenNebulaError(response));
            $('div[id="' + file.fileName + 'progressBar"]').remove();
          }
        });
      });

      that.uploader.upload();
    } else {
      Sunstone.runAction("Image.create", img_obj);
    }

    return false;
  }

  function _submitAdvanced(context) {
    var template = $('#template', context).val();
    var ds_id = $('#img_datastore_raw .resource_list_select', context).val();

    if (!ds_id) {
      Notifier.notifyError(Locale.tr("Please select a datastore for this image"));
      return false;
    }

    var img_obj = {
      "image" : {
        "image_raw" : template
      },
      "ds_id" : ds_id
    };

    Sunstone.runAction("Image.create", img_obj);

    return false;
  }
});

define('tabs/images-tab',['require','utils/locale','./images-tab/buttons','./images-tab/actions','./images-tab/datatable','./images-tab/tabId','./images-tab/dialogs/clone','./images-tab/panels/info','./images-tab/panels/vms','./images-tab/panels/snapshots','./images-tab/form-panels/create'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./images-tab/buttons');
  var Actions = require('./images-tab/actions');
  var Table = require('./images-tab/datatable');

  var TAB_ID = require('./images-tab/tabId');
  var DATATABLE_ID = "dataTableImages";

  var _dialogs = [
    require('./images-tab/dialogs/clone')
  ];

  var _panels = [
    require('./images-tab/panels/info'),
    require('./images-tab/panels/vms'),
    require('./images-tab/panels/snapshots')
  ];

  var _formPanels = [
    require('./images-tab/form-panels/create')
  ];

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("Images"),
    tabClass: "subTab",
    parentTab: "vresources-tab",
    listHeader: '<i class="fa fa-fw fa-upload"></i>&emsp;' + Locale.tr("Images"),
    infoHeader: '<i class="fa fa-fw fa-upload"></i>&emsp;' + Locale.tr("Image"),
    subheader: '',
    resource: 'Image',
    content: '<div id="upload_progress_bars" class="large-12 columns"></div>',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});

define('tabs/files-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "File.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "File.create_dialog" : {
      type: "create_dialog",
      layout: "create"
    },
    "File.chown" : {
      type: "confirm_with_select",
      text: Locale.tr("Change owner"),
      layout: "user_select",
      select: "User",
      tip: Locale.tr("Select the new owner") + ":"
    },
    "File.chgrp" : {
      type: "confirm_with_select",
      text: Locale.tr("Change group"),
      layout: "user_select",
      select: "Group",
      tip: Locale.tr("Select the new group") + ":"
    },
    "File.enable" : {
      type: "action",
      layout: "more_select",
      text: Locale.tr("Enable")
    },
    "File.disable" : {
      type: "action",
      layout: "more_select",
      text: Locale.tr("Disable")
    },
    "File.delete" : {
      type: "confirm",
      layout: "del",
      text: Locale.tr("Delete")
    },
  };

  return Buttons;
});

define('tabs/files-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createFileForm';
});
define('tabs/files-tab/actions',['require','sunstone','utils/notifier','utils/locale','opennebula/image','utils/common-actions','./tabId','./form-panels/create/formPanelId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var OpenNebulaResource = require('opennebula/image');
  var CommonActions = require('utils/common-actions');

  var RESOURCE = "File";
  var XML_ROOT = "IMAGE";
  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "File.create" : _commonActions.create(CREATE_DIALOG_ID),
    "File.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID),
    "File.list" : _commonActions.list(),
    "File.show" : _commonActions.show(),
    "File.refresh" : _commonActions.refresh(),
    "File.delete" : _commonActions.del(),
    "File.update_template" : _commonActions.updateTemplate(),
    "File.chown": _commonActions.multipleAction('chown'),
    "File.chgrp": _commonActions.multipleAction('chgrp'),
    "File.chmod": _commonActions.singleAction('chmod'),
    "File.rename": _commonActions.singleAction('rename'),
    "File.enable": _commonActions.multipleAction('enable'),
    "File.disable": _commonActions.multipleAction('disable'),
    "File.chtype": _commonActions.singleAction('chtype')
  };

  return _actions;
});


/* START_TEMPLATE */
define('hbs!tabs/files-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n        </tr>\n        ";
  stack1 = ((helper = (helper = helpers.renameTrHTML || (depth0 != null ? depth0.renameTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"renameTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastore", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.DATASTORE : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Register time", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.prettyRegTime || (depth0 != null ? depth0.prettyRegTime : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"prettyRegTime","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td_type\">"
    + escapeExpression(((helper = (helper = helpers.typeStr || (depth0 != null ? depth0.typeStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"typeStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td>\n            <div id=\"div_edit_chg_type\">\n              <a id=\"div_edit_chg_type_link\" class=\"edit_e\" href=\"#\"> <i class=\"fa fa-pencil-square-o right\"/></a>\n            </div>\n          </td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filesystem type", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.fsTypeStr || (depth0 != null ? depth0.fsTypeStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"fsTypeStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Size", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.sizeStr || (depth0 != null ? depth0.sizeStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"sizeStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "State", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.stateStr || (depth0 != null ? depth0.stateStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"stateStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Running VMS", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.RUNNING_VMS : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n      </tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\">";
  stack1 = ((helper = (helper = helpers.permissionsTableHTML || (depth0 != null ? depth0.permissionsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"permissionsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "</div>\n</div>\n<div class=\"row\">\n  <div class=\"large-9 columns\">";
  stack1 = ((helper = (helper = helpers.templateTableHTML || (depth0 != null ? depth0.templateTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templateTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/files-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/files-tab/panels/info/panelId',['require'],function(require){
  return 'file_info_tab';
});
define('tabs/files-tab/panels/info',['require','utils/locale','utils/humanize','utils/panel/rename-tr','utils/panel/template-table','utils/panel/permissions-table','opennebula/image','sunstone-config','sunstone','hbs!./info/html','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var RenameTr = require('utils/panel/rename-tr');
  var TemplateTable = require('utils/panel/template-table');
  var PermissionsTable = require('utils/panel/permissions-table');
  var OpenNebulaImage = require('opennebula/image');
  var Config = require('sunstone-config');
  var Sunstone = require('sunstone');

  /*
    TEMPLATES
   */

  var TemplateInfo = require('hbs!./info/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "File";
  var XML_ROOT = "IMAGE";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var renameTrHTML = RenameTr.html(RESOURCE, this.element.NAME);
    var templateTableHTML = TemplateTable.html(this.element.TEMPLATE, RESOURCE, Locale.tr("Attributes"));
    var permissionsTableHTML = PermissionsTable.html(TAB_ID, RESOURCE, this.element);
    var stateStr = Locale.tr(OpenNebulaImage.stateStr(this.element.STATE));
    var prettyRegTime = Humanize.prettyTime(this.element.REGTIME);
    var fsTypeStr = this.element.FS_TYPE != undefined ? this.element.FS_TYPE : '-';
    var sizeStr = Humanize.sizeFromMB(this.element.SIZE);

    return TemplateInfo({
      'element': this.element,
      'renameTrHTML': renameTrHTML,
      'templateTableHTML': templateTableHTML,
      'permissionsTableHTML': permissionsTableHTML,
      'stateStr': stateStr,
      'prettyRegTime': prettyRegTime,
      'fsTypeStr': fsTypeStr,
      'typeStr': OpenNebulaImage.typeStr(this.element.TYPE),
      'sizeStr': sizeStr
    });
  }

  function _setup(context) {
    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);
    RenameTr.setup(RESOURCE, this.element.ID, context);
    TemplateTable.setup(this.element.TEMPLATE, RESOURCE, this.element.ID, context);
    PermissionsTable.setup(TAB_ID, RESOURCE, this.element, context);

    // Listener for edit link for type change
    var that = this;
    context.off("click", "#div_edit_chg_type_link")
    context.on("click", "#div_edit_chg_type_link", function() {
      $(".value_td_type", context).html(
                '<select id="chg_type_select">\
                      <option value="KERNEL">'+Locale.tr("KERNEL")+'</option>\
                      <option value="RAMDISK">'+Locale.tr("RAMDISK")+'</option>\
                      <option value="CONTEXT">'+Locale.tr("CONTEXT")+'</option>\
                  </select>');

      var currentVal = "";
      switch(parseInt(that.element.TYPE)){
        case OpenNebulaImage.TYPES.KERNEL:
          currentVal = "KERNEL";
          break;
        case OpenNebulaImage.TYPES.RAMDISK:
          currentVal = "RAMDISK";
          break;
        case OpenNebulaImage.TYPES.CONTEXT:
          currentVal = "CONTEXT";
          break;
      }

      $('#chg_type_select', context).val(currentVal);
    });

    context.off("change", "#chg_type_select");
    context.on("change", "#chg_type_select", function() {
      var new_value = $(this).val();
      Sunstone.runAction("File.chtype", that.element.ID, new_value);
    });

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/files-tab/form-panels/create/wizard',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"medium-4 columns\">\n      <label for=\"file_name\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name that the File will get. Every file must have a unique name.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input required type=\"text\" name=\"file_name\" id=\"file_name\" />\n    </div>\n    <div class=\"medium-8 columns\">\n      <label for=\"file_desc\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Human readable description of the file for other users.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <textarea name=\"file_desc\" id=\"file_desc\" rows=\"4\"></textarea>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"medium-4 columns\">\n      <label for=\"file_type\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type of the file.", {"name":"tr","hash":{},"data":data})))
    + "\n          <br/>\n          <br/>\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, " KERNEL & RAMDISK files can be used in the OS Booting section of the Template wizard.", {"name":"tr","hash":{},"data":data})))
    + "\n          <br/>\n          <br/>\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, " CONTEXT files can be included in the context CD-ROM, from the Context/Files section of the Template wizard.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <select name=\"file_type\" id=\"file_type\">\n        <option value=\"CONTEXT\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Context", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"KERNEL\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Kernel", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"RAMDISK\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ramdisk", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n    <div class=\"medium-8 columns\">\n      <label for=\"file_datastore\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastore", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select the datastore for this file", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </label>\n      <div id=\"file_datastore\" name=\"file_datastore\"></div>\n    </div>\n  </div>\n  <br>\n  <fieldset>\n    <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Image location", {"name":"tr","hash":{},"data":data})))
    + ":</legend>\n    <div class=\"row\" id=\"src_path_select\">\n      <div class=\"large-12 columns text-center\">\n        <input type=\"radio\" name=\"src_path\" id=\"path_file\" value=\"path\">\n          <label for=\"path_file\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Provide a path", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        </input>\n        <input type=\"radio\" name=\"src_path\" id=\"upload_file\" value=\"upload\">\n          <label for=\"upload_file\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Upload", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        </input>\n      </div>\n    </div>\n    <br>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <label for=\"file_path\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Path", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Path to the original file that will be copied to the file repository.", {"name":"tr","hash":{},"data":data})))
    + "\n          </span>\n        </label>\n        <input type=\"text\" name=\"file_path\" id=\"file_path\" />\n      </div>\n    </div>\n    <div class=\"row\">\n      <div id=\"files_file-uploader\" class=\"large-12 columns text-center\">\n        <label id=\"files_file-uploader-label\" for=\"files_file-uploader-input\"></label>\n        <input id=\"files_file-uploader-input\" type=\"file\"/>\n      </div>\n    </div>\n  </fieldset>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/files-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/files-tab/form-panels/create/advanced',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Advanced\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"columns large-12\">\n      <label for=\"file_datastores_raw\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastore", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n      <div id=\"file_datastore_raw\" name=\"file_datastore_raw\"></div>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <p>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Write the Image template here", {"name":"tr","hash":{},"data":data})))
    + "</p>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <textarea id=\"template\" rows=\"15\" required></textarea>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/files-tab/form-panels/create/advanced', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/files-tab/form-panels/create',['require','utils/form-panels/form-panel','resumable','sunstone','opennebula/error','utils/notifier','utils/locale','utils/tips','utils/resource-select','utils/browser-info','hbs!./create/wizard','hbs!./create/advanced','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Resumable = require('resumable');
  var Sunstone = require('sunstone');
  var OpenNebulaError = require('opennebula/error');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var ResourceSelect = require('utils/resource-select');
  var BrowserInfo = require('utils/browser-info');

  var TemplateWizardHTML = require('hbs!./create/wizard');
  var TemplateAdvancedHTML = require('hbs!./create/advanced');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create File"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      }
    };

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.htmlAdvanced = _htmlAdvanced;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.submitAdvanced = _submitAdvanced;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.setup = _setup;

  return FormPanel;


  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    return TemplateWizardHTML({
      'formPanelId': this.formPanelId
    });
  }

  function _htmlAdvanced() {
    return TemplateAdvancedHTML({
      'formPanelId': this.formPanelId
    });
  }

  function _onShow(context) {
    $("#file_name", context).focus();

    var ds_id = $('#file_datastore .resource_list_select', context).val();
    var ds_id_raw = $('#file_datastore_raw .resource_list_select', context).val();

    // Filter out DS with type image (0) or system (1)
    var filter_att = ["TYPE", "TYPE"];
    var filter_val = ["0", "1"];

    ResourceSelect.insert('div#file_datastore', context, "Datastore",
                        ds_id, false, null, filter_att, filter_val);

    ResourceSelect.insert('div#file_datastore_raw', context, "Datastore",
                        ds_id_raw, false, null, filter_att, filter_val);

    return false;
  }

  function _setup(context) {
    var that = this;
    Tips.setup(context);

    $('#file_path,#files_file-uploader', context).closest('.row').hide();

    $("input[name='src_path']", context).change(function() {
      var value = $(this).val();
      switch (value){
      case "path":
        $('#files_file-uploader', context).closest('.row').hide();
        $('#file_path', context).closest('.row').show();

        $('#file_path', context).attr('required', '');
        break;
      case "upload":
        $('#file_path', context).closest('.row').hide();
        $('#files_file-uploader', context).closest('.row').show();

        $('#file_path', context).removeAttr('required');
        break;
      }
    });

    $('#path_file', context).click();

    if (BrowserInfo.getInternetExplorerVersion() > -1) {
      $("#upload_file").attr("disabled", "disabled");
    } else {
      that.uploader = new Resumable({
        target: 'upload_chunk',
        chunkSize: 10 * 1024 * 1024,
        maxFiles: 1,
        testChunks: false,
        query: {
          csrftoken: csrftoken
        }
      });

      that.uploader.assignBrowse($('#files_file-uploader-input', context));

      var fileName = '';
      var file_input = false;

      that.uploader.on('fileAdded', function(file) {
        fileName = file.fileName;
        file_input = fileName;

        $('#files_file-uploader-input', context).hide()
        $("#files_file-uploader-label", context).html(file.fileName);
      });

      that.uploader.on('uploadStart', function() {
        $('#files_upload_progress_bars').append('<div id="files-' + fileName + 'progressBar" class="row" style="margin-bottom:10px">\
            <div id="files-' + fileName + '-info" class="large-2 columns dataTables_info">\
              ' + Locale.tr("Uploading...") + '\
            </div>\
            <div class="large-10 columns">\
              <div id="upload_progress_container" class="progress nine radius" style="height:25px !important">\
                <span class="meter" style="width:0%"></span>\
              </div>\
              <div class="progress-text" style="margin-left:15px">' + fileName + '</div>\
            </div>\
          </div>');
      });

      that.uploader.on('progress', function() {
        $('span.meter', $('div[id="files-' + fileName + 'progressBar"]')).css('width', this.progress() * 100.0 + '%')
      });
    }

    return false;
  }

  function _submitWizard(context) {
    var that = this;
    var upload = false;

    var ds_id = $('#file_datastore .resource_list_select', context).val();
    if (!ds_id) {
      Sunstone.hideFormPanelLoading(TAB_ID);
      Notifier.notifyError(Locale.tr("Please select a datastore for this file"));
      return false;
    }

    var img_json = {};

    var name = $('#file_name', context).val();
    img_json["NAME"] = name;

    var desc = $('#file_desc', context).val();
    if (desc.length) {
      img_json["DESCRIPTION"] = desc;
    }

    var type = $('#file_type', context).val();
    img_json["TYPE"] = type;

    switch ($('#src_path_select input:checked', context).val()){
    case "path":
      path = $('#file_path', context).val();
      if (path) img_json["PATH"] = path;
      break;
    case "upload":
      upload = true;
      break;
    }

    var img_obj = {
      "image" : img_json,
      "ds_id" : ds_id
    };

    //this is an image upload we trigger FileUploader
    //to start the upload
    if (upload) {
      if (that.uploader.files.length == 0) {
        Sunstone.hideFormPanelLoading(TAB_ID);
        Notifier.notifyError(Locale.tr("Please select a file to upload"));
        return false;
      }

      Sunstone.resetFormPanel(that.tabId, that.formPanelId);
      Sunstone.hideFormPanel(that.tabId);

      that.uploader.on('fileSuccess', function(file) {
        $('div[id="files-' + file.fileName + '-info"]').text(Locale.tr("Registering in OpenNebula"));
        $.ajax({
          url: 'upload',
          type: "POST",
          data: {
            csrftoken: csrftoken,
            img : JSON.stringify(img_obj),
            file: file.fileName,
            tempfile: file.uniqueIdentifier
          },
          success: function() {
            Notifier.notifyMessage("Image uploaded correctly");
            $('div[id="files-' + file.fileName + 'progressBar"]').remove();
            Sunstone.runAction("File.refresh");
          },
          error: function(response) {
            Notifier.onError({}, OpenNebulaError(response));
            $('div[id="files-' + file.fileName + 'progressBar"]').remove();
          }
        });
      });

      that.uploader.upload();
    } else {
      Sunstone.runAction("File.create", img_obj);
    }

    return false;
  }

  function _submitAdvanced(context) {
    var template = $('#template', context).val();
    var ds_id = $('#file_datastore_raw .resource_list_select', context).val();

    if (!ds_id) {
      Notifier.notifyError(Locale.tr("Please select a datastore for this file"));
      return false;
    }

    var img_obj = {
      "image" : {
        "image_raw" : template
      },
      "ds_id" : ds_id
    };

    Sunstone.runAction("File.create", img_obj);

    return false;
  }
});

define('tabs/files-tab',['require','utils/locale','./files-tab/buttons','./files-tab/actions','./files-tab/datatable','./files-tab/tabId','./files-tab/panels/info','./files-tab/form-panels/create'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./files-tab/buttons');
  var Actions = require('./files-tab/actions');
  var Table = require('./files-tab/datatable');

  var TAB_ID = require('./files-tab/tabId');
  var DATATABLE_ID = "dataTableFiles";

  var _dialogs = [];

  var _panels = [
    require('./files-tab/panels/info')
  ];

  var _formPanels = [
    require('./files-tab/form-panels/create')
  ];

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("Files & Kernels"),
    tabClass: "subTab",
    parentTab: "vresources-tab",
    listHeader: '<i class="fa fa-fw fa-folder-open"></i>&emsp;' + Locale.tr("Files & Kernels"),
    infoHeader: '<i class="fa fa-fw fa-folder-open"></i>&emsp;' + Locale.tr("File"),
    subheader: '',
    resource: 'File',
    content: '<div class="large-12 columns">\
                <div id="files_upload_progress_bars"></div>\
              </div>',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});

define('tabs/infra-tab',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');
  var TAB_ID = 'infra-tab';

  var InfraTab = {
    tabId: TAB_ID,
    title: '<i class="fa fa-lg fa-fw fa-sitemap"></i>&emsp;' + Locale.tr("Infrastructure"),
    no_content: true
  }

  return InfraTab;
});

define('tabs/clusters-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "Cluster.refresh" : {
        type: "action",
        layout: "refresh",
        alwaysActive: true
    },
    "Cluster.create_dialog" : {
        type: "create_dialog",
        layout: "create"
    },
    "Cluster.update_dialog" : {
        type : "action",
        layout: "main",
        text : Locale.tr("Update")
    },
    "Cluster.delete" : {
        type: "confirm",
        layout: "del",
        text: Locale.tr("Delete")
    }
  };

  return Buttons;
});

define('tabs/clusters-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createClusterForm';
});
define('tabs/clusters-tab/actions',['require','sunstone','utils/notifier','utils/locale','./datatable','opennebula/cluster','opennebula/action','utils/common-actions','./tabId','./form-panels/create/formPanelId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var DataTable = require('./datatable');
  var OpenNebulaResource = require('opennebula/cluster');
  var OpenNebulaAction = require('opennebula/action');
  var CommonActions = require('utils/common-actions');

  var RESOURCE = "Cluster";
  var XML_ROOT = "CLUSTER";
  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "Cluster.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID),
    "Cluster.list" : _commonActions.list(),
    "Cluster.show" : _commonActions.show(),
    "Cluster.refresh" : _commonActions.refresh(),
    "Cluster.delete" : _commonActions.del(),
    "Cluster.update_template" : _commonActions.updateTemplate(),
    "Cluster.update_dialog" : _commonActions.checkAndShowUpdate(),
    "Cluster.show_to_update" : _commonActions.showUpdate(CREATE_DIALOG_ID),
    "Cluster.rename": _commonActions.singleAction('rename'),

    "Cluster.create" : {
      type: "create",
      call: OpenNebulaResource.create,
      callback: function(request, response){
        Sunstone.resetFormPanel(TAB_ID, CREATE_DIALOG_ID);
        Sunstone.hideFormPanel(TAB_ID);
        Sunstone.getDataTable(TAB_ID).addElement(request, response);

        for (var host in request.request.data[0].cluster.hosts)
          if (request.request.data[0].cluster.hosts[host])
            Sunstone.runAction("Cluster.addhost",response[XML_ROOT].ID,host);
        for (var vnet in request.request.data[0].cluster.vnets)
          if (request.request.data[0].cluster.vnets[vnet])
            Sunstone.runAction("Cluster.addvnet",response[XML_ROOT].ID,vnet);
        for (var datastore in request.request.data[0].cluster.datastores)
          if (request.request.data[0].cluster.datastores[datastore])
            Sunstone.runAction("Cluster.adddatastore",response[XML_ROOT].ID,datastore);

        Notifier.notifyCustom(Locale.tr("Cluster created"), " ID: " + response[XML_ROOT].ID, false);
      },
      error: Notifier.onError
    },

    "Cluster.addhost" : {
      type: "single",
      call : OpenNebulaResource.addhost,
      callback : function (req) {
        OpenNebulaAction.clear_cache("HOST");
        Sunstone.runAction('Cluster.show',req.request.data[0][0]);
      },
      error : Notifier.onError
    },

    "Cluster.delhost" : {
      type: "single",
      call : OpenNebulaResource.delhost,
      callback : function (req) {
        OpenNebulaAction.clear_cache("HOST");
        Sunstone.runAction('Cluster.show',req.request.data[0][0]);
      },
      error : Notifier.onError
    },

    "Cluster.adddatastore" : {
      type: "single",
      call : OpenNebulaResource.adddatastore,
      callback : function (req) {
        OpenNebulaAction.clear_cache("DATASTORE");
        Sunstone.runAction('Cluster.show',req.request.data[0][0]);
      },
      error : Notifier.onError
    },

    "Cluster.deldatastore" : {
      type: "single",
      call : OpenNebulaResource.deldatastore,
      callback : function (req) {
        OpenNebulaAction.clear_cache("DATASTORE");
        Sunstone.runAction('Cluster.show',req.request.data[0][0]);
      },
      error : Notifier.onError
    },

    "Cluster.addvnet" : {
      type: "single",
      call : OpenNebulaResource.addvnet,
      callback : function (req) {
        OpenNebulaAction.clear_cache("VNET");
        Sunstone.runAction('Cluster.show',req.request.data[0][0]);
      },
      error : Notifier.onError
    },

    "Cluster.delvnet" : {
      type: "single",
      call : OpenNebulaResource.delvnet,
      callback : function (req) {
        OpenNebulaAction.clear_cache("VNET");
        Sunstone.runAction('Cluster.show',req.request.data[0][0]);
      },
      error : Notifier.onError
    }
  };

  return _actions;
});


/* START_TEMPLATE */
define('hbs!tabs/clusters-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n       <tr>\n        <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n        <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n        <td></td>\n      </tr>\n      ";
  stack1 = ((helper = (helper = helpers.renameTrHTML || (depth0 != null ? depth0.renameTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"renameTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n    </tbody>\n  </table>\n</div>\n</div>\n<div class=\"row\">\n  <div class=\"large-9 columns\">\n    ";
  stack1 = ((helper = (helper = helpers.templateTableHTML || (depth0 != null ? depth0.templateTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templateTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/clusters-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/clusters-tab/panels/info/panelId',['require'],function(require){
  return 'cluster_info_tab';
});
define('tabs/clusters-tab/panels/info',['require','hbs!./info/html','utils/locale','utils/panel/rename-tr','utils/panel/template-table','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TemplateHTML = require('hbs!./info/html');
  var Locale = require('utils/locale');
  var RenameTr = require('utils/panel/rename-tr');
  var TemplateTable = require('utils/panel/template-table');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "Cluster";
  var XML_ROOT = "CLUSTER";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var renameTrHTML = RenameTr.html(RESOURCE, this.element.NAME);
    var templateTableHTML = TemplateTable.html(this.element.TEMPLATE, RESOURCE,
                                              Locale.tr("Attributes"));

    return TemplateHTML({
      'element': this.element,
      'renameTrHTML': renameTrHTML,
      'templateTableHTML': templateTableHTML
    });
  }

  function _setup(context) {
    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);
    RenameTr.setup(RESOURCE, this.element.ID, context);

    TemplateTable.setup(this.element.TEMPLATE, RESOURCE, this.element.ID, context);
  }
});

define('tabs/clusters-tab/panels/hosts/panelId',['require'],function(require){
  return 'cluster_host_tab';
});
define('tabs/clusters-tab/panels/hosts',['require','utils/locale','tabs/hosts-tab/datatable','./hosts/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var HostsTable = require('tabs/hosts-tab/datatable');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./hosts/panelId');
  var HOSTS_TABLE_ID = PANEL_ID + "HostsTable";
  var RESOURCE = "Cluster";
  var XML_ROOT = "CLUSTER";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Hosts");
    this.icon = "fa-hdd-o";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var hosts = [];

    if (this.element.HOSTS.ID != undefined){
      hosts = this.element.HOSTS.ID;

      if (!$.isArray(hosts)){
        hosts = [hosts];
      }
    }

    var opts = {
      info: true,
      select: true,
      selectOptions: {
        read_only: true,
        fixed_ids: hosts
      }
    };

    this.HostsTable = new HostsTable(HOSTS_TABLE_ID, opts);

    return this.HostsTable.dataTableHTML;
  }

  function _setup(context) {
    this.HostsTable.initialize();
    this.HostsTable.refreshResourceTableSelect();

    return false;
  }
});

define('tabs/clusters-tab/panels/vnets/panelId',['require'],function(require){
  return 'cluster_vnet_tab';
});
define('tabs/clusters-tab/panels/vnets',['require','utils/locale','tabs/vnets-tab/datatable','./vnets/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var VNetsTable = require('tabs/vnets-tab/datatable');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./vnets/panelId');
  var VNETS_TABLE_ID = PANEL_ID + "VNetsTable";
  var RESOURCE = "Cluster";
  var XML_ROOT = "CLUSTER";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("VNets");
    this.icon = "fa-globe";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var vnets = [];

    if (this.element.VNETS.ID != undefined){
      vnets = this.element.VNETS.ID;

      if (!$.isArray(vnets)){
        vnets = [vnets];
      }
    }

    var opts = {
      info: true,
      select: true,
      selectOptions: {
        read_only: true,
        fixed_ids: vnets
      }
    };

    this.VNetsTable = new VNetsTable(VNETS_TABLE_ID, opts);

    return this.VNetsTable.dataTableHTML;
  }

  function _setup(context) {
    this.VNetsTable.initialize();
    this.VNetsTable.refreshResourceTableSelect();

    return false;
  }
});

define('tabs/clusters-tab/panels/datastores/panelId',['require'],function(require){
  return 'cluster_datastore_tab';
});
define('tabs/clusters-tab/panels/datastores',['require','utils/locale','tabs/datastores-tab/datatable','./datastores/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var DatastoresTable = require('tabs/datastores-tab/datatable');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./datastores/panelId');
  var DATASTORES_TABLE_ID = PANEL_ID + "DatastoresTable";
  var RESOURCE = "Cluster";
  var XML_ROOT = "CLUSTER";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Datastores");
    this.icon = "fa-folder-open";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var datastores = [];

    if (this.element.DATASTORES.ID != undefined){
      datastores = this.element.DATASTORES.ID;

      if (!$.isArray(datastores)){
        datastores = [datastores];
      }
    }

    var opts = {
      info: true,
      select: true,
      selectOptions: {
        read_only: true,
        fixed_ids: datastores
      }
    };

    this.DatastoresTable = new DatastoresTable(DATASTORES_TABLE_ID, opts);

    return this.DatastoresTable.dataTableHTML;
  }

  function _setup(context) {
    this.DatastoresTable.initialize();
    this.DatastoresTable.refreshResourceTableSelect();

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/clusters-tab/form-panels/create/wizard',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <label for=\"name\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      <input type=\"text\" required name=\"name\" id=\"name\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <dl class=\"tabs right-info-tabs text-center right\" data-tab>\n        <dd class=\"active\">\n          <a href=\"#tab-hostsTab\">\n            <i class=\"fa fa-hdd-o\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hosts", {"name":"tr","hash":{},"data":data})))
    + "\n          </a>\n        </dd>\n        <dd>\n          <a href=\"#tab-vnetsTab\">\n            <i class=\"fa fa-globe\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VNets", {"name":"tr","hash":{},"data":data})))
    + "\n          </a>\n        </dd>\n        <dd>\n          <a href=\"#tab-datastoresTab\">\n            <i class=\"fa fa-folder-open\"></i><br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastores", {"name":"tr","hash":{},"data":data})))
    + "\n          </a>\n        </dd>\n      </dl>\n    </div>\n  </div>\n  <div class=\"tabs-content\">\n    <div id=\"tab-hostsTab\" class=\"active content\">\n      ";
  stack1 = ((helper = (helper = helpers.hostsTableHTML || (depth0 != null ? depth0.hostsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"hostsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n    </div>\n    <div id=\"tab-vnetsTab\" class=\"content\">\n      ";
  stack1 = ((helper = (helper = helpers.vnetsTableHTML || (depth0 != null ? depth0.vnetsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"vnetsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n    </div>\n    <div id=\"tab-datastoresTab\" class=\"content\">\n      ";
  stack1 = ((helper = (helper = helpers.datastoresTableHTML || (depth0 != null ? depth0.datastoresTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"datastoresTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/clusters-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/clusters-tab/form-panels/create',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','tabs/hosts-tab/datatable','tabs/vnets-tab/datatable','tabs/datastores-tab/datatable','hbs!./create/wizard','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var HostsTable = require('tabs/hosts-tab/datatable');
  var VNetsTable = require('tabs/vnets-tab/datatable');
  var DatastoresTable = require('tabs/datastores-tab/datatable');

  /*
    TEMPLATES
   */

  var TemplateWizardHTML = require('hbs!./create/wizard');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create Cluster"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      },
      'update': {
        'title': Locale.tr("Update Cluster"),
        'buttonText': Locale.tr("Update"),
        'resetButton': false
      }
    };

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.fill = _fill;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    var opts = {
      info: false,
      select: true,
      selectOptions: {"multiple_choice": true}
    };

    this.hostsTable = new HostsTable("cluster_wizard_hosts", opts);
    this.vnetsTable = new VNetsTable("cluster_wizard_vnets", opts);
    this.datastoresTable = new DatastoresTable("cluster_wizard_datastores", opts);

    return TemplateWizardHTML({
      'formPanelId': this.formPanelId,
      'hostsTableHTML': this.hostsTable.dataTableHTML,
      'vnetsTableHTML': this.vnetsTable.dataTableHTML,
      'datastoresTableHTML': this.datastoresTable.dataTableHTML

    });
  }

  function _setup(context) {
    context.foundation('tab', 'reflow');
    this.hostsTable.initialize();
    this.vnetsTable.initialize();
    this.datastoresTable.initialize();
  }

  function _submitWizard(context) {
    var that = this;

    var selectedHostsList      = that.hostsTable.retrieveResourceTableSelect();
    var selectedVNetsList      = that.vnetsTable.retrieveResourceTableSelect();
    var selectedDatastoresList = that.datastoresTable.retrieveResourceTableSelect();

    if (that.action == "create") {
      var selectedHosts = {};
      $.each(selectedHostsList, function(i,e){
        selectedHosts[e] = 1;
      });

      var selectedVNets = {};
      $.each(selectedVNetsList, function(i,e){
        selectedVNets[e] = 1;
      });

      var selectedDatastores = {};
      $.each(selectedDatastoresList, function(i,e){
        selectedDatastores[e] = 1;
      });

      var cluster_json = {
        "cluster": {
          "name": $('#name',context).val(),
          "hosts": selectedHosts,
          "vnets": selectedVNets,
          "datastores": selectedDatastores
        }
      };

      Sunstone.runAction("Cluster.create",cluster_json);
      return false;
    } else if (that.action == "update") {

      // find out which ones are in and out
      $.each(selectedHostsList, function(i,hostId){
        if (that.originalHostsList.indexOf(hostId) == -1){
          Sunstone.runAction("Cluster.addhost",that.clusterUpdateId,hostId);
        }
      });

      $.each(that.originalHostsList, function(i,hostId){
        if (selectedHostsList.indexOf(hostId) == -1){
          Sunstone.runAction("Cluster.delhost",that.clusterUpdateId,hostId);
        }
      });

      $.each(selectedVNetsList, function(i,vnetId){
        if (that.originalVNetsList.indexOf(vnetId) == -1){
          Sunstone.runAction("Cluster.addvnet",that.clusterUpdateId,vnetId);
        }
      });

      $.each(that.originalVNetsList, function(i,vnetId){
        if (selectedVNetsList.indexOf(vnetId) == -1){
          Sunstone.runAction("Cluster.delvnet",that.clusterUpdateId,vnetId);
        }
      });

      $.each(selectedDatastoresList, function(i,datastoreId){
        if (that.originalDatastoresList.indexOf(datastoreId) == -1){
          Sunstone.runAction("Cluster.adddatastore",that.clusterUpdateId,datastoreId);
        }
      });

      $.each(that.originalDatastoresList, function(i,datastoreId){
        if (selectedDatastoresList.indexOf(datastoreId) == -1){
          Sunstone.runAction("Cluster.deldatastore",that.clusterUpdateId,datastoreId);
        }
      });

      Sunstone.hideFormPanel(TAB_ID);

      return false;
    }
  }

  function _onShow(context) {
    // TODO bug, does not work until the input is visible
    $("input#name", context).focus();

    this.hostsTable.refreshResourceTableSelect();
    this.vnetsTable.refreshResourceTableSelect();
    this.datastoresTable.refreshResourceTableSelect();
  }

  function _fill(context, element) {

    var name    = element.NAME;
    var hostIds = element.HOSTS.ID;
    var vnetIds = element.VNETS.ID;
    var dsIds   = element.DATASTORES.ID;

    if (typeof hostIds == 'string'){
      hostIds = [hostIds];
    }

    if (typeof vnetIds == 'string'){
      vnetIds = [vnetIds];
    }

    if (typeof dsIds == 'string'){
      dsIds = [dsIds];
    }

    $('#name',context).val(name);
    $('#name',context).attr("disabled", "disabled");

    this.originalHostsList = [];

    // Select hosts belonging to the cluster
    if (hostIds){
      this.originalHostsList = hostIds;

      this.hostsTable.selectResourceTableSelect({ids: hostIds});
    }

    this.originalVNetsList = [];

    // Select vnets belonging to the cluster
    if (vnetIds){
      this.originalVNetsList = vnetIds;

      this.vnetsTable.selectResourceTableSelect({ids: vnetIds});
    }

    this.originalDatastoresList = [];

    // Select datastores belonging to the cluster
    if (dsIds){
      this.originalDatastoresList = dsIds;

      this.datastoresTable.selectResourceTableSelect({ids: dsIds});
    }

    this.clusterUpdateId = element.ID;
  }
});

define('tabs/clusters-tab',['require','utils/locale','./clusters-tab/buttons','./clusters-tab/actions','./clusters-tab/datatable','./clusters-tab/tabId','./clusters-tab/panels/info','./clusters-tab/panels/hosts','./clusters-tab/panels/vnets','./clusters-tab/panels/datastores','./clusters-tab/form-panels/create'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./clusters-tab/buttons');
  var Actions = require('./clusters-tab/actions');
  var Table = require('./clusters-tab/datatable');

  var TAB_ID = require('./clusters-tab/tabId');
  var DATATABLE_ID = "dataTableClusters";

  var _dialogs = [
  ];

  var _panels = [
    require('./clusters-tab/panels/info'),
    require('./clusters-tab/panels/hosts'),
    require('./clusters-tab/panels/vnets'),
    require('./clusters-tab/panels/datastores')
  ];

  var _formPanels = [
    require('./clusters-tab/form-panels/create')
  ];

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("Clusters"),
    tabClass: "subTab",
    parentTab: "infra-tab",
    listHeader: '<i class="fa fa-fw fa-th"></i>&emsp;'+Locale.tr("Clusters"),
    infoHeader: '<i class="fa fa-fw fa-th"></i>&emsp;'+Locale.tr("Cluster"),
    subheader: '<span/> <small></small>&emsp;',
    resource: 'Cluster',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});

define('tabs/hosts-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "Host.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "Host.create_dialog" : {
      type: "create_dialog",
      layout: "create"
    },
    "Host.addtocluster" : {
      type: "confirm_with_select",
      text: Locale.tr("Select cluster"),
      select: "Cluster",
      tip: Locale.tr("Select the destination cluster:"),
      layout: "main"
    },
    "Host.enable" : {
      type: "action",
      text: Locale.tr("Enable"),
      layout: "main"
    },
    "Host.disable" : {
      type: "action",
      text: Locale.tr("Disable"),
      layout: "main"
    },
    "Host.delete" : {
      type: "confirm",
      text: Locale.tr("Delete host"),
      layout: "del"
    }
  };

  return Buttons;
})
;
define('tabs/hosts-tab/actions',['require','sunstone','utils/notifier','utils/locale','./datatable','opennebula/host','opennebula/cluster','opennebula/action','utils/common-actions','./tabId','./form-panels/create/formPanelId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var DataTable = require('./datatable');
  var OpenNebulaResource = require('opennebula/host');
  var OpenNebulaCluster = require('opennebula/cluster');
  var OpenNebulaAction = require('opennebula/action');
  var CommonActions = require('utils/common-actions');

  var TAB_ID = require('./tabId');
  var XML_ROOT = "HOST"
  var RESOURCE = "Host"
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "Host.create" : _commonActions.create(CREATE_DIALOG_ID),
    "Host.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID),
    "Host.list" : _commonActions.list(),
    "Host.show" : _commonActions.show(),
    "Host.refresh" : _commonActions.refresh(),
    "Host.delete" : _commonActions.del(),
    "Host.update_template" : _commonActions.updateTemplate(),
    "Host.enable": _commonActions.multipleAction('enable'),
    "Host.disable": _commonActions.multipleAction('disable'),
    "Host.rename": _commonActions.singleAction('rename'),

    "Host.addtocluster" : {
      type: "multiple",
      call: function(params){
        var cluster = params.data.extra_param;
        var host = params.data.id;

        if (cluster == -1){
          OpenNebulaResource.show({
            data : {
              id: host
            },
            success: function (request, info){
              var element = info.HOST;

              var current_cluster = element.CLUSTER_ID;

              if(current_cluster != -1){
                OpenNebulaCluster.delhost({
                  data: {
                    id: current_cluster,
                    extra_param: host
                  },
                  success: function(){
                    OpenNebulaAction.clear_cache("HOST");
                    Sunstone.runAction('Host.show',host);
                  },
                  error: Notifier.onError
                });
              } else {
                OpenNebulaAction.clear_cache("HOST");
                Sunstone.runAction('Host.show',host);
              }
            },
            error: Notifier.onError
          });
        } else {
          OpenNebulaCluster.addhost({
            data: {
              id: cluster,
              extra_param: host
            },
            success: function(){
              OpenNebulaAction.clear_cache("HOST");
              Sunstone.runAction('Host.show',host);
            },
            error: Notifier.onError
          });
        }
      },
      elements: function() {
        return Sunstone.getDataTable(TAB_ID).elements();
      }
    }
  };

  return _actions;
})
;

/* START_TEMPLATE */
define('hbs!utils/panel/cluster-tr/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function";
  return "<tr>\n  <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cluster", {"name":"tr","hash":{},"data":data})))
    + "</td>\n  <td class=\"value_td_cluster\">"
    + escapeExpression(((helper = (helper = helpers.clusterName || (depth0 != null ? depth0.clusterName : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"clusterName","hash":{},"data":data}) : helper)))
    + "</td>\n  <td>\n    <div id=\"div_edit_chg_cluster\">\n      <a id=\"div_edit_chg_cluster_link\" class=\"edit_e\" href=\"#\"> <i class=\"fa fa-pencil-square-o right\"/></a>\n    </div>\n  </td>\n</tr>";
},"useData":true});
Handlebars.registerPartial('utils/panel/cluster-tr/html', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/panel/cluster-tr',['require','hbs!./cluster-tr/html','utils/resource-select','sunstone'],function(require) {
  /*
    This module insert a row with the name of the resource.
    The row can be edited and a rename action will be sent
   */

  var TemplateClusterTr = require('hbs!./cluster-tr/html');
  var ResourceSelect = require('utils/resource-select');
  var Sunstone = require('sunstone');

  /*
    Generate the tr HTML with the name of the resource and an edit icon
    @param {String} resourceType Resource type (i.e: Zone, Host, Image...)
    @param {String} resourceName Name of the resource
    @returns {String} HTML row
   */
  var _html = function(clusterName) {
    var renameTrHTML = TemplateClusterTr({
      'clusterName': (clusterName.length ? clusterName : '-')
    })

    return renameTrHTML;
  };

  /*
    Initialize the row, clicking the edit icon will add an input to edit the name
    @param {String} resourceType Resource type (i.e: Zone, Host, Image...)
    @param {String} resourceId ID of the resource
    @param {jQuery Object} context Selector including the tr
   */
  var _setup = function(resourceType, resourceId, clusterId, context) {
    context.off("click", "#div_edit_chg_cluster_link");
    context.on("click", "#div_edit_chg_cluster_link", function() {
      var tr_context = $(this).parents("tr");
      ResourceSelect.insert(".value_td_cluster", context, "Cluster", clusterId, false);
    });

    context.off("change", ".value_td_cluster .resource_list_select");
    context.on("change", ".value_td_cluster .resource_list_select", function() {
      var newClusterId = $(this).val();
      if (newClusterId != "") {
        Sunstone.runAction(resourceType + ".addtocluster", [resourceId], newClusterId);
      }
    });

    return false;
  }

  return {
    'html': _html,
    'setup': _setup
  }
});


/* START_TEMPLATE */
define('hbs!tabs/hosts-tab/utils/datastore-capacity-table/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "    <tr>\n      <td class=\"key_td\">"
    + escapeExpression(((helper = (helper = helpers.datastoreId || (depth0 != null ? depth0.datastoreId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"datastoreId","hash":{},"data":data}) : helper)))
    + "</td>\n      <td class=\"value_td\" colspan=\"2\">";
  stack1 = ((helper = (helper = helpers.datastoreBar || (depth0 != null ? depth0.datastoreBar : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"datastoreBar","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</td>\n    </tr>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<table class=\"dataTable extended_table\">\n  <thead>\n    <tr>\n      <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastore ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n      <th style=\"width:70%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Capacity", {"name":"tr","hash":{},"data":data})))
    + "</th>\n    </tr>\n  </thead>\n  <tbody>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.datastoreBars : depth0), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  </tbody>\n</table>";
},"useData":true});
Handlebars.registerPartial('tabs/hosts-tab/utils/datastore-capacity-table/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/hosts-tab/utils/datastores-capacity-table',['require','tabs/datastores-tab/utils/datastore-capacity-bar','hbs!./datastore-capacity-table/html'],function(require){
  /*
    DEPENDENCIES
   */

  var DatastoreCapacityBar = require('tabs/datastores-tab/utils/datastore-capacity-bar');

  /*
    TEMPLATES
   */

  var TemplateDatastoresCapacityTable = require('hbs!./datastore-capacity-table/html');

  /*
    CONSTRUCTOR
   */

  return {
    'html': _html
  }

  /*
    FUNCTION DEFINITIONS
   */

  function _html(element) {
    var hostShare = element.HOST_SHARE;

    var datastores = []
    if ($.isArray(hostShare.DATASTORES.DS))
      datastores = hostShare.DATASTORES.DS
    else if (!$.isEmptyObject(hostShare.DATASTORES.DS))
      datastores = [hostShare.DATASTORES.DS]
    else
      return "";

    var datastoreBars = [];
    $.each(datastores, function(index, value){
      datastoreBars.push({
        'datastoreId': value.ID,
        'datastoreBar': DatastoreCapacityBar.html(value)
      })
    });

    return TemplateDatastoresCapacityTable({'datastoreBars': datastoreBars});
  }
});
define('tabs/hosts-tab/utils/can-import-wilds',['require'],function(require){
  /*
    CONSTRUCTOR
   */

  return _canImportWilds;

  /*
    FUNCTION DEFINITIONS
   */

  /* Check if any of the existing VMs in the Host define the IMPORT_TEMPLATE
      attribute to be imported into OpenNebula.
    @param {Object} element Host element as returned by OpenNebula
    @result {Boolean}
  */
  function _canImportWilds(element) {
    var canImportWilds = false;
    if (element.TEMPLATE.VM) {
      var vms = element.TEMPLATE.VM;
      if (!$.isArray(vms)) { // If only 1 VM convert to array
        vms = [vms];
      }
      $.each(vms, function() {
        if (this.IMPORT_TEMPLATE) {
          canImportWilds = true;
          return false;
        }
      });
    }
    return canImportWilds;
  }
});

/* START_TEMPLATE */
define('hbs!tabs/hosts-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n        </tr>\n        ";
  stack1 = ((helper = (helper = helpers.renameTrHTML || (depth0 != null ? depth0.renameTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"renameTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        ";
  stack1 = ((helper = (helper = helpers.clusterTrHTML || (depth0 != null ? depth0.clusterTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"clusterTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "State", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.stateStr || (depth0 != null ? depth0.stateStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"stateStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IM MAD", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.IM_MAD : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VM MAD", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.VM_MAD : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VN MAD", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.VN_MAD : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n      </tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Capacity", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Allocated Memory", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\" style=\"width:50%;\">";
  stack1 = lambda(((stack1 = (depth0 != null ? depth0.memoryBars : depth0)) != null ? stack1.allocated : stack1), depth0);
  if (stack1 != null) { buffer += stack1; }
  buffer += "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Allocated CPU", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\" style=\"width:50%;\">";
  stack1 = lambda(((stack1 = (depth0 != null ? depth0.cpuBars : depth0)) != null ? stack1.allocated : stack1), depth0);
  if (stack1 != null) { buffer += stack1; }
  buffer += "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Real Memory", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\" style=\"width:50%;\">";
  stack1 = lambda(((stack1 = (depth0 != null ? depth0.memoryBars : depth0)) != null ? stack1.real : stack1), depth0);
  if (stack1 != null) { buffer += stack1; }
  buffer += "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Real CPU", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\" style=\"width:50%;\">";
  stack1 = lambda(((stack1 = (depth0 != null ? depth0.cpuBars : depth0)) != null ? stack1.real : stack1), depth0);
  if (stack1 != null) { buffer += stack1; }
  buffer += "</td>\n        </tr>\n      </tbody>\n    </table>\n    ";
  stack1 = ((helper = (helper = helpers.datastoresCapacityTableHTML || (depth0 != null ? depth0.datastoresCapacityTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"datastoresCapacityTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-9 columns\">";
  stack1 = ((helper = (helper = helpers.templateTableHTML || (depth0 != null ? depth0.templateTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templateTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/hosts-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/hosts-tab/panels/info/panelId',['require'],function(require) {
  return 'host_info_tab';
})
;
define('tabs/hosts-tab/panels/info',['require','utils/locale','utils/humanize','utils/panel/rename-tr','utils/panel/template-table','utils/panel/permissions-table','utils/panel/cluster-tr','opennebula/host','../utils/cpu-bars','../utils/memory-bars','../utils/datastores-capacity-table','../utils/can-import-wilds','hbs!./info/html','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var RenameTr = require('utils/panel/rename-tr');
  var TemplateTable = require('utils/panel/template-table');
  var PermissionsTable = require('utils/panel/permissions-table');
  var ClusterTr = require('utils/panel/cluster-tr');
  var OpenNebulaHost = require('opennebula/host');
  var CPUBars = require('../utils/cpu-bars');
  var MemoryBars = require('../utils/memory-bars');
  var DatastoresCapacityTable = require('../utils/datastores-capacity-table');
  var CanImportWilds = require('../utils/can-import-wilds');

  /*
    TEMPLATES
   */

  var TemplateInfo = require('hbs!./info/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "Host"
  var XML_ROOT = "HOST"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    var that = this;
    that.title = Locale.tr("Info");
    that.icon = "fa-info-circle";

    that.element = info[XML_ROOT];

    that.canImportWilds = CanImportWilds(that.element);

    // Hide information of the Wild VMs of the Host and the ESX Hosts
    //  in the template table. Unshow values are stored in the unshownTemplate
    //  object to be used when the host info is updated.
    that.unshownTemplate = {};
    that.strippedTemplate = {};
    var unshownKeys = ['HOST', 'VM', 'WILDS'];
    $.each(that.element.TEMPLATE, function(key, value) {
      if ($.inArray(key, unshownKeys) > -1) {
        that.unshownTemplate[key] = value;
      } else {
        that.strippedTemplate[key] = value;
      }
    });

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var templateTableHTML = TemplateTable.html(
                                      this.strippedTemplate,
                                      RESOURCE,
                                      Locale.tr("Attributes"));

    var renameTrHTML = RenameTr.html(RESOURCE, this.element.NAME);
    var clusterTrHTML = ClusterTr.html(this.element.CLUSTER);
    var permissionsTableHTML = PermissionsTable.html(TAB_ID, RESOURCE, this.element);
    var cpuBars = CPUBars.html(this.element);
    var memoryBars = MemoryBars.html(this.element);
    var datastoresCapacityTableHTML = DatastoresCapacityTable.html(this.element);

    return TemplateInfo({
      'element': this.element,
      'renameTrHTML': renameTrHTML,
      'clusterTrHTML': clusterTrHTML,
      'templateTableHTML': templateTableHTML,
      'permissionsTableHTML': permissionsTableHTML,
      'cpuBars': cpuBars,
      'memoryBars': memoryBars,
      'stateStr': OpenNebulaHost.stateStr(this.element.STATE),
      'datastoresCapacityTableHTML': datastoresCapacityTableHTML
    });
  }

  function _setup(context) {
    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);
    RenameTr.setup(RESOURCE, this.element.ID, context);
    ClusterTr.setup(RESOURCE, this.element.ID, this.element.CLUSTER_ID, context);
    TemplateTable.setup(this.strippedTemplate, RESOURCE, this.element.ID, context, this.unshownTemplate);
    PermissionsTable.setup(TAB_ID, RESOURCE, this.element, context);
    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/hosts-tab/panels/monitor/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"\">\n  <div class=\"large-6 columns\">\n    <div class=\"row graph_legend\">\n      <h3 class=\"subheader\">\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CPU", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h3>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-10 columns centered graph text-center\" id=\"host_cpu_graph\" style=\"height: 100px;\">\n        <span  id=\"provision_dashboard_total\" style=\"font-size:80px\">\n          <i class=\"fa fa-spinner fa-spin\"></i>\n        </span>\n      </div>\n    </div>\n    <div class=\"row graph_legend\">\n      <div class=\"large-10 columns centered\" id=\"host_cpu_legend\"></div>\n    </div>\n  </div>\n  <div class=\"large-6 columns\">\n    <div class=\"row graph_legend\">\n      <h3 class=\"subheader\">\n        <small>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "MEMORY", {"name":"tr","hash":{},"data":data})))
    + "</small>\n      </h3>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-10 columns centered graph text-center\" id=\"host_mem_graph\" style=\"height: 100px;\">\n        <span  id=\"provision_dashboard_total\" style=\"font-size:80px\">\n          <i class=\"fa fa-spinner fa-spin\"></i>\n        </span>\n      </div>\n    </div>\n    <div class=\"row graph_legend\">\n      <div class=\"large-10 columns centered\" id=\"host_mem_legend\"></div>\n    </div>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/hosts-tab/panels/monitor/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/hosts-tab/panels/monitor/panelId',['require'],function(require) {
  return 'host_monitoring_tab';
})
;
define('tabs/hosts-tab/panels/monitor',['require','opennebula/host','utils/notifier','utils/locale','utils/graphs','hbs!./monitor/html','../tabId','./monitor/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var OpenNebulaHost = require('opennebula/host');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var Graphs = require('utils/graphs');

  /*
    TEMPLATES
   */

  var TemplateInfo = require('hbs!./monitor/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./monitor/panelId');
  var RESOURCE = "Host"
  var XML_ROOT = "HOST"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.panelId = PANEL_ID;
    this.title = Locale.tr("Graphs");
    this.icon = "fa-bar-chart-o";
    this.element = info[XML_ROOT];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;
  Panel.prototype.onShow = _onShow;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateInfo();
  }

  function _setup(context) {
    return false;
  }

  function _onShow(context) {
    OpenNebulaHost.monitor({
      data: {
        id: this.element.ID,
        monitor: {
          monitor_resources : "HOST_SHARE/CPU_USAGE,HOST_SHARE/USED_CPU,HOST_SHARE/MAX_CPU,HOST_SHARE/MEM_USAGE,HOST_SHARE/USED_MEM,HOST_SHARE/MAX_MEM"
        }
      },
      success: function(req, response) {
        var host_graphs = [
            {
              monitor_resources : "HOST_SHARE/CPU_USAGE,HOST_SHARE/USED_CPU,HOST_SHARE/MAX_CPU",
              labels : Locale.tr("Allocated") + "," + Locale.tr("Real") + "," + Locale.tr("Total"),
              humanize_figures : false,
              div_graph : $("#host_cpu_graph"),
              div_legend : $("#host_cpu_legend")
            },
            {
              monitor_resources : "HOST_SHARE/MEM_USAGE,HOST_SHARE/USED_MEM,HOST_SHARE/MAX_MEM",
              labels : Locale.tr("Allocated") + "," + Locale.tr("Real") + "," + Locale.tr("Total"),
              humanize_figures : true,
              div_graph : $("#host_mem_graph"),
              div_legend : $("#host_mem_legend")
            }
            ];

        for (var i = 0; i < host_graphs.length; i++) {
          Graphs.plot(response, host_graphs[i]);
        }
      },
      error: Notifier.onError
    });

    return false;
  }
});

define('tabs/hosts-tab/panels/vms/panelId',['require'],function(require){
  return 'host_vms_tab';
});
define('tabs/hosts-tab/panels/vms',['require','utils/locale','tabs/vms-tab/datatable','./vms/panelId'],function(require){
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var VMsTable = require('tabs/vms-tab/datatable');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./vms/panelId');
  var VMS_TABLE_ID = PANEL_ID + "VMsTable";
  var RESOURCE = "Host";
  var XML_ROOT = "HOST";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("VMs");
    this.icon = "fa-cloud";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var vms = [];

    if (this.element.VMS.ID != undefined){
      vms = this.element.VMS.ID;

      if (!$.isArray(vms)){
        vms = [vms];
      }
    }

    var opts = {
      info: true,
      select: true,
      selectOptions: {
        read_only: true,
        fixed_ids: vms
      }
    };

    this.vmsTable = new VMsTable(VMS_TABLE_ID, opts);

    return this.vmsTable.dataTableHTML;
  }

  function _setup(context) {
    this.vmsTable.initialize();
    this.vmsTable.refreshResourceTableSelect();

    return false;
  }
});

/* START_TEMPLATE */
define('hbs!tabs/hosts-tab/panels/wilds/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <button id=\"import_wilds\" class=\"button tiny success right radius\" >"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Import Wilds", {"name":"tr","hash":{},"data":data})))
    + "</button>\n    <table id=\"datatable_host_wilds\" class=\"datatable twelve\">\n      <thead>\n        <tr>\n          <th></th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VM name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Remote ID", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody id=\"tbody_host_wilds\"></tbody>\n    </table>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/hosts-tab/panels/wilds/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/hosts-tab/panels/wilds/panelId',['require'],function(require) {
  return 'host_wilds_tab';
})
;
define('tabs/hosts-tab/panels/wilds',['require','foundation-datatables','utils/locale','../utils/can-import-wilds','opennebula/vm','opennebula/action','sunstone','utils/notifier','hbs!./wilds/html','./wilds/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation-datatables');
  var Locale = require('utils/locale');
  var CanImportWilds = require('../utils/can-import-wilds');
  var OpenNebulaVM = require('opennebula/vm');
  var OpenNebulaAction = require('opennebula/action');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');

  /*
    TEMPLATES
   */

  var TemplateWilds = require('hbs!./wilds/html');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./wilds/panelId');
  var RESOURCE = "Host"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Wilds");
    this.icon = "fa-hdd-o";

    this.element = info[RESOURCE.toUpperCase()];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateWilds();
  }

  function _setup(context) {
    var that = this;

    // Hide the import button if the Wilds cannot be imported
    if (!CanImportWilds(this.element)) {
      $("#import_wilds").hide();
    }

    that.dataTableWildHosts = $("#datatable_host_wilds", context).dataTable({
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": [0]},
          {"sWidth": "35px", "aTargets": [0]}
      ]
    });

    if (that.element.TEMPLATE.VM) {
      wilds = that.element.TEMPLATE.VM;

      if (!$.isArray(wilds)) { // If only 1 VM convert to array
        wilds = [wilds];
      }

      i = 0;

      $.each(wilds, function() {
        var name      = this.VM_NAME;
        var safe_name = i;
        i += 1;
        var deploy_id = this.DEPLOY_ID;

        var wilds_list_array = [
          [
            '<input type="checkbox" class="import_wild_checker import_' + safe_name + '" unchecked/>',
            name,
            deploy_id
          ]
        ];

        that.dataTableWildHosts.fnAddData(wilds_list_array);

        $(".import_" + safe_name, that.dataTableWildHosts).data("wild_template", atob(this.IMPORT_TEMPLATE));
      });
    }

    delete that.element.TEMPLATE.WILDS;
    delete that.element.TEMPLATE.VM;

    // Enable the import button when at least a VM is selected
    $("#import_wilds", context).attr("disabled", "disabled").on("click.disable", function(e) { return false; });

    $(".import_wild_checker", context).off("change");
    $(".import_wild_checker", context).on("change", function(){
      if ($(".import_wild_checker:checked", context).length == 0){
        $("#import_wilds", context).attr("disabled", "disabled").on("click.disable", function(e) { return false; });
      } else {
        $("#import_wilds", context).removeAttr("disabled").off("click.disable");
      }
    });

    // Add event listener for importing WILDS
    context.off("click", '#import_wilds');
    context.on("click", '#import_wilds', function () {
      $("#import_wilds", context).attr("disabled", "disabled").on("click.disable", function(e) { return false; });
      $("#import_wilds", context).html('<i class="fa fa-spinner fa-spin"></i>');

      $(".import_wild_checker:checked", "#datatable_host_wilds").each(function() {
        var vm_json = {
          "vm": {
            "vm_raw": $(this).data("wild_template")
          }
        };

        var import_host_id = that.element.ID;
        var wild_row       = $(this).closest('tr');

        // Create the VM in OpenNebula
        OpenNebulaVM.create({
          timeout: true,
          data: vm_json,
          success: function(request, response) {
            OpenNebulaAction.clear_cache("VM");

            var extra_info = {};

            extra_info['host_id'] = import_host_id;
            extra_info['ds_id']   = -1;
            extra_info['enforce'] = false;

            // Deploy the VM
            Sunstone.runAction("VM.silent_deploy_action",
                               response.VM.ID,
                               extra_info);

            // Notify
            Notifier.notifyCustom(Locale.tr("VM imported"), " ID: " + response.VM.ID, false);

            // Delete row (shouldn't be there in next monitorization)
            that.dataTableWildHosts.fnDeleteRow(wild_row);

            $("#import_wilds", context).removeAttr("disabled").off("click.disable");
            $("#import_wilds", context).html(Locale.tr("Import Wilds"));
          },
          error: function (request, error_json) {
            var msg;
            if (error_json.error.message){
              msg = error_json.error.message;
            } else {
              msg = Locale.tr("Cannot contact server: is it running and reachable?");
            }

            Notifier.notifyError(msg);

            $("#import_wilds", context).removeAttr("disabled").off("click.disable");
            $("#import_wilds", context).html(Locale.tr("Import Wilds"));
          }
        });
      });
    });

    return false;
  }
});

/* START_TEMPLATE */
define('hbs!tabs/hosts-tab/panels/esx/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\"datatable_host_esx_info_div\" class=\"row\">\n  <div class=\"large-12 columns\">\n    <table id=\"datatable_host_esx\" class=\"datatable twelve\">\n      <thead>\n        <tr>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hostname", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Status", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Real CPU", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Real Memory", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody id=\"tbody_host_esx\"></tbody>\n    </table>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/hosts-tab/panels/esx/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/hosts-tab/panels/esx/panelId',['require'],function(require) {
  return 'host_esx_tab';
})
;
define('tabs/hosts-tab/panels/esx',['require','foundation-datatables','utils/locale','../utils/cpu-bars','../utils/memory-bars','hbs!./esx/html','./esx/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation-datatables');
  var Locale = require('utils/locale');
  var CPUBars = require('../utils/cpu-bars');
  var MemoryBars = require('../utils/memory-bars');

  /*
    TEMPLATES
   */

  var TemplateESX = require('hbs!./esx/html');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./esx/panelId');
  var RESOURCE = "Host"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("ESX");
    this.icon = "fa-hdd-o";

    this.element = info[RESOURCE.toUpperCase()];

    // Do not create an instance of this panel if no vcenter hypervisor
    if (this.element.TEMPLATE.HYPERVISOR != "vcenter") {
      throw "Panel not available for this element";
    }

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateESX();
  }

  function _setup(context) {
    var that = this;
    var dataTableESX = $("#datatable_host_esx", context).dataTable({
          "bSortClasses" : false,
          "bDeferRender": true
    });

    var hostListArray = [];

    if (that.element.TEMPLATE.HOST) {
      if (!(that.element.TEMPLATE.HOST instanceof Array)) {
        that.element.TEMPLATE.HOST = [that.element.TEMPLATE.HOST];
      }

      if (that.element.TEMPLATE.HOST instanceof Array) {
        $.each(that.element.TEMPLATE.HOST, function(){
          var cpuBars = CPUBars.html(this, true);
          var memoryBars = MemoryBars.html(this, true);

          hostListArray.push([
              this.HOSTNAME,
              this.STATE,
              cpuBars.real,
              memoryBars.real
          ]);
        });
      }

      dataTableESX.fnAddData(hostListArray);
      delete that.element.TEMPLATE.HOST;
    }
  }
})
;
define('utils/dom-datatable',['require','utils/tab-datatable','sunstone-config','utils/locale'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');

  /*
    CONSTRUCTOR
   */

  /**
   * Generic datatable that uses the columns and row data from the html dom,
   * instead of .list and elementArray methods. Offers the same row check
   * and row info mechanism as TabDataTable
   * @param {string} dataTableId dataTable ID
   * @param {object} conf        Same as tab-datatable.js, plus:
   *                             - dataTableOptions: replaces the default dataTableOptions
   */
  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.dataTableId = dataTableId;

    this.dataTableOptions = conf.dataTableOptions || {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"]}
      ]
    };

    TabDataTable.call(this);
  }

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;

  return Table;
});


/* START_TEMPLATE */
define('hbs!tabs/hosts-tab/panels/pci/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "          <tr>\n            <td>"
    + escapeExpression(((helper = (helper = helpers.VMID || (depth0 != null ? depth0.VMID : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"VMID","hash":{},"data":data}) : helper)))
    + "</td>\n            <td>"
    + escapeExpression(((helper = (helper = helpers.SHORT_ADDRESS || (depth0 != null ? depth0.SHORT_ADDRESS : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"SHORT_ADDRESS","hash":{},"data":data}) : helper)))
    + "</td>\n            <td>"
    + escapeExpression(((helper = (helper = helpers.TYPE || (depth0 != null ? depth0.TYPE : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"TYPE","hash":{},"data":data}) : helper)))
    + "</td>\n            <td>"
    + escapeExpression(((helper = (helper = helpers.DEVICE_NAME || (depth0 != null ? depth0.DEVICE_NAME : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"DEVICE_NAME","hash":{},"data":data}) : helper)))
    + "</td>\n          </tr>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <table id=\"datatable_pci_"
    + escapeExpression(((helper = (helper = helpers.panelId || (depth0 != null ? depth0.panelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"panelId","hash":{},"data":data}) : helper)))
    + "\" class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VM", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PCI Address", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.pcis : depth0), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "      </tbody>\n    </table>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/hosts-tab/panels/pci/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/hosts-tab/panels/pci/panelId',['require'],function(require) {
  return 'host_pci_tab';
})
;
define('tabs/hosts-tab/panels/pci',['require','utils/locale','utils/dom-datatable','hbs!./pci/html','../tabId','./pci/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var DomDataTable = require('utils/dom-datatable');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./pci/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./pci/panelId');
  var RESOURCE = "Host"
  var XML_ROOT = "HOST"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("PCI");
    this.icon = "fa-list-ul";

    this.element = info[XML_ROOT];

    this.panelId = PANEL_ID;

    // Do not create an instance of this panel if no pci devices are found
    if (this.element.HOST_SHARE.PCI_DEVICES.PCI == undefined) {
      throw "Panel not available for this element";
    }

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var pcis = [];

    if(this.element.HOST_SHARE &&
       this.element.HOST_SHARE.PCI_DEVICES &&
       this.element.HOST_SHARE.PCI_DEVICES.PCI){

      pcis = this.element.HOST_SHARE.PCI_DEVICES.PCI;

      if (!$.isArray(pcis)){ // If only 1 convert to array
        pcis = [pcis];
      }

      $.each(pcis, function(){
        if(this.VMID == "-1"){
          this.VMID = "";
        }
      });
    }

    return TemplateHTML({
      'pcis': pcis,
      'panelId': this.panelId
    });
  }

  function _setup(context) {
    this.pciDataTable = new DomDataTable(
      'datatable_pci_'+this.panelId,
      {
        actions: false,
        info: false,
        dataTableOptions: {
          "bAutoWidth": false,
          "bSortClasses" : false,
          "bDeferRender": true,
          "ordering": false,
          "aoColumnDefs": [
          ]
        }
      });

    this.pciDataTable.initialize();

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!utils/vcenter/clusters/html',['hbs','hbs/handlebars','templates/helpers/advancedSection','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  return "<span class=\"fa-stack fa-2x\" style=\"color: #dfdfdf\">\n  <i class=\"fa fa-cloud fa-stack-2x\"></i>\n  <i class=\"fa  fa-spinner fa-spin fa-stack-1x fa-inverse\"></i>\n</span>\n";
  },"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, buffer = "";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Clusters", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"useData":true});
Handlebars.registerPartial('utils/vcenter/clusters/html', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/vcenter/clusters',['require','utils/locale','sunstone','opennebula/host','opennebula/error','hbs!./clusters/html'],function(require) {
  // Dependencies
  var Locale = require('utils/locale');
  var Sunstone = require('sunstone');
  var OpenNebulaHost = require('opennebula/host');
  var OpenNebulaError = require('opennebula/error');

  var TemplateHTML = require('hbs!./clusters/html');

  function VCenterClusters() {
    return this;
  }

  VCenterClusters.prototype = {
    'html': _html,
    'insert': _fillVCenterClusters,
    'import': _import
  };
  VCenterClusters.prototype.constructor = VCenterClusters;

  return VCenterClusters;

  function _html() {
    return '<div class="vcenter_clusters hidden"></div>';
  }

  /*
    opts = {
      container: Jquery div to inject the html,
      vcenter_user: vCenter Username,
      vcenter_password: vCenter Password,
      vcenter_host: vCenter Host
      success: function to call after a success
    }
   */
  function _fillVCenterClusters(opts) {
    this.opts = opts;

    var path = '/vcenter';

    var context = $(".vcenter_clusters", opts.container);

    context.html( TemplateHTML({}) );

    context.show();

    $(".accordion_advanced_toggle", context).trigger("click");

    $.ajax({
      url: path,
      type: "GET",
      data: {timeout: false},
      dataType: "json",
      headers: {
        "X_VCENTER_USER": opts.vcenter_user,
        "X_VCENTER_PASSWORD": opts.vcenter_password,
        "X_VCENTER_HOST": opts.vcenter_host
      },
      success: function(response){
        $(".content", context).html("");

        $('<div class="row">' +
            '<div class="large-12 columns">' +
              '<p style="color: #999">' + Locale.tr("Please select the vCenter Clusters to be imported to OpenNebula. Each vCenter Cluster will be included as a new OpenNebula Host") + '</p>' +
            '</div>' +
          '</div>').appendTo($(".content", context));

        $.each(response, function(datacenter_name, clusters) {
          $('<div class="row">' +
              '<div class="large-12 columns">' +
                '<h5>' +
                  datacenter_name + ' ' + Locale.tr("Datacenter") +
                '</h5>' +
              '</div>' +
            '</div>').appendTo($(".content", context));

          if (clusters.length == 0) {
            $('<div class="row">' +
                '<div class="large-12 columns">' +
                  '<label>' +
                    Locale.tr("No clusters found in this DataCenter") +
                  '</label>' +
                '</div>' +
              '</div>').appendTo($(".content", context));
          } else {
            $.each(clusters, function(id, cluster_name) {
              var row = $('<div class="vcenter_cluster">' +
                  '<div class="row">' +
                    '<div class="large-10 columns">' +
                      '<label>' +
                        '<input type="checkbox" class="cluster_name"/> ' +
                        cluster_name +
                      '</label>' +
                      '<div class="large-12 columns vcenter_host_response">' +
                      '</div>' +
                    '</div>' +
                    '<div class="large-2 columns vcenter_host_result">' +
                    '</div>' +
                  '</div>' +
                '</div>').appendTo($(".content", context));

              $(".cluster_name", row).data("cluster_name", cluster_name);
              //$(".cluster_name", row).data("datacenter_name", datacenter_name);
            });
          }
        });

        if(opts.success){
          opts.success();
        }
      },
      error: function(response){
        context.hide();
        Notifier.onError({}, OpenNebulaError(response));
      }
    });
  }

  function _import(context, cluster_id) {
    var that = this;

    $.each($(".cluster_name:checked", context), function() {
      var cluster_context = $(this).closest(".vcenter_cluster");
      $(".vcenter_host_result:not(.success)", cluster_context).html('<span class="fa-stack fa-2x" style="color: #dfdfdf">' +
            '<i class="fa fa-cloud fa-stack-2x"></i>' +
            '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>' +
          '</span>');

      var host_json = {
        "host": {
          "name": $(this).data("cluster_name"),
          "vm_mad": "vcenter",
          "vnm_mad": "dummy",
          "im_mad": "vcenter",
          "cluster_id": cluster_id
        }
      };

      OpenNebulaHost.create({
        timeout: true,
        data: host_json,
        success: function(request, response) {
          $(".vcenter_host_result", cluster_context).addClass("success").html(
              '<span class="fa-stack fa-2x" style="color: #dfdfdf">' +
                '<i class="fa fa-cloud fa-stack-2x"></i>' +
                '<i class="fa  fa-check fa-stack-1x fa-inverse"></i>' +
              '</span>');

          $(".vcenter_host_response", cluster_context).html('<p style="font-size:12px" class="running-color">' +
                Locale.tr("Host created successfully") + ' ID:' + response.HOST.ID +
              '</p>');

          var template_raw =
            "VCENTER_USER=\"" + that.opts.vcenter_user + "\"\n" +
            "VCENTER_PASSWORD=\"" + that.opts.vcenter_password + "\"\n" +
            "VCENTER_HOST=\"" + that.opts.vcenter_host + "\"\n";

          Sunstone.runAction("Host.update_template", response.HOST.ID, template_raw);
        },
        error: function (request, error_json) {
          $(".vcenter_host_result", cluster_context).html('<span class="fa-stack fa-2x" style="color: #dfdfdf">' +
                '<i class="fa fa-cloud fa-stack-2x"></i>' +
                '<i class="fa  fa-warning fa-stack-1x fa-inverse"></i>' +
              '</span>');

          $(".vcenter_host_response", cluster_context).html('<p style="font-size:12px" class="error-color">' +
                (error_json.error.message || Locale.tr("Cannot contact server: is it running and reachable?")) +
              '</p>');
        }
      });
    });
  }
});

/* START_TEMPLATE */
define('hbs!utils/vcenter/networks/html',['hbs','hbs/handlebars','templates/helpers/advancedSection','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  return "<span class=\"fa-stack fa-2x\" style=\"color: #dfdfdf\">\n  <i class=\"fa fa-cloud fa-stack-2x\"></i>\n  <i class=\"fa  fa-spinner fa-spin fa-stack-1x fa-inverse\"></i>\n</span>\n";
  },"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, buffer = "";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Networks", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"useData":true});
Handlebars.registerPartial('utils/vcenter/networks/html', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/vcenter/networks',['require','utils/locale','opennebula/network','opennebula/error','hbs!./networks/html'],function(require) {
  // Dependencies
  var Locale = require('utils/locale');
  var OpenNebulaNetwork = require('opennebula/network');
  var OpenNebulaError = require('opennebula/error');

  var TemplateHTML = require('hbs!./networks/html');

  function VCenterNetworks() {
    return this;
  }

  VCenterNetworks.prototype = {
    'html': _html,
    'insert': _fillVCenterNetworks,
    'import': _import
  };
  VCenterNetworks.prototype.constructor = VCenterNetworks;

  return VCenterNetworks;

  function _html() {
    return '<div class="vcenter_networks hidden"></div>';
  }

  /*
    Retrieve the list of networks from vCenter and fill the container with them

    opts = {
      datacenter: "Datacenter Name",
      cluster: "Cluster Name",
      container: Jquery div to inject the html,
      vcenter_user: vCenter Username,
      vcenter_password: vCenter Password,
      vcenter_host: vCenter Host
    }
   */
  function _fillVCenterNetworks(opts) {
    var path = '/vcenter/networks';

    var context = $(".vcenter_networks", opts.container);

    context.html( TemplateHTML({}) );

    context.show();

    $(".accordion_advanced_toggle", context).trigger("click");

    $.ajax({
      url: path,
      type: "GET",
      data: {timeout: false},
      dataType: "json",
      headers: {
        "X_VCENTER_USER": opts.vcenter_user,
        "X_VCENTER_PASSWORD": opts.vcenter_password,
        "X_VCENTER_HOST": opts.vcenter_host
      },
      success: function(response){
        $(".content", context).html("");

        $('<div class="row">' +
            '<div class="large-12 columns">' +
              '<p style="color: #999">' + Locale.tr("Please select the vCenter Networks to be imported to OpenNebula.") + '</p>' +
            '</div>' +
          '</div>').appendTo($(".content", context))

        $.each(response, function(datacenter_name, networks){
          $('<div class="row">' +
              '<div class="large-12 columns">' +
                '<h5>' +
                  datacenter_name + ' ' + Locale.tr("DataCenter") +
                '</h5>' +
              '</div>' +
            '</div>').appendTo($(".content", context))

          if (networks.length == 0) {
            $('<div class="row">' +
                '<div class="large-12 columns">' +
                  '<label>' +
                    Locale.tr("No new networks found in this DataCenter") +
                  '</label>' +
                '</div>' +
              '</div>').appendTo($(".content", context))
          } else {
            $.each(networks, function(id, network){
              var netname   = network.name.replace(" ","_");
              var vlan_info = ""

              if (network.vlan) {
                var vlan_info = '<div class="vlan_info">' +
                      '<div class="large-4 columns">'+
                        '<label>' + Locale.tr("VLAN") +
                           '<input type="text" class="vlaninfo" value="'+network.vlan+'" disabled/>' +
                        '</label>'+
                      '</div>'+
                    '</div>';
              }

              var trow = $('<div class="vcenter_network">' +
                  '<div class="row">' +
                    '<div class="large-10 columns">' +
                      '<div class="large-12 columns">' +
                        '<label>' +
                          '<input type="checkbox" class="network_name" checked/> ' +
                          network.name + '&emsp;<span style="color: #999">' + network.type + '</span>' +
                        '</label>' +
                      '</div>'+
                      '<div class="large-2 columns">'+
                        '<label>' + Locale.tr("SIZE") +
                          '<input type="text" class="netsize" value="255"/>' +
                        '</label>' +
                      '</div>'+
                      '<div class="large-2 columns">'+
                        '<label>' + Locale.tr("TYPE") +
                          '<select class="type_select">'+
                            '<option value="ETHER">eth</option>' +
                            '<option value="IP4">ipv4</option>'+
                            '<option value="IP6">ipv6</option>' +
                          '</select>' +
                        '</label>' +
                      '</div>'+
                      '<div class="net_options">' +
                        '<div class="large-4 columns">'+
                          '<label>' + Locale.tr("MAC") +
                            '<input type="text" class="eth_mac_net" placeholder="'+Locale.tr("Optional")+'"/>' +
                          '</label>'+
                        '</div>'+
                      '</div>'+
                      vlan_info +
                      '<div class="large-12 columns vcenter_network_response">'+
                      '</div>'+
                    '</div>' +
                    '<div class="large-2 columns vcenter_network_result">'+
                    '</div>'+
                  '</div>'+
                '</div>').appendTo($(".content", context))


              $('.type_select', trow).on("change",function(){
                var network_context = $(this).closest(".vcenter_network");
                var type = $(this).val();

                var net_form_str = ''

                switch(type) {
                  case 'ETHER':
                    net_form_str =
                      '<div class="large-4 columns">'+
                        '<label>' + Locale.tr("MAC") +
                          '<input type="text" class="eth_mac_net" placeholder="'+Locale.tr("Optional")+'"/>' +
                        '</label>'+
                      '</div>';
                    break;
                  case 'IP4':
                    net_form_str =
                      '<div class="large-4 columns">'+
                        '<label>' + Locale.tr("IP START") +
                          '<input type="text" class="four_ip_net"/>' +
                        '</label>'+
                      '</div>'+
                      '<div class="large-4 columns">'+
                        '<label>' + Locale.tr("MAC") +
                          '<input type="text" class="eth_mac_net" placeholder="'+Locale.tr("Optional")+'"/>' +
                        '</label>'+
                      '</div>';
                    break;
                  case 'IP6':
                    net_form_str =
                      '<div class="large-4 columns">'+
                        '<label>' + Locale.tr("MAC") +
                          '<input type="text" class="eth_mac_net"/>' +
                        '</label>'+
                      '</div>'+
                      '<div class="large-6 columns">'+
                        '<label>' + Locale.tr("GLOBAL PREFIX") +
                          '<input type="text" class="six_global_net" placeholder="'+Locale.tr("Optional")+'"/>' +
                        '</label>'+
                      '</div>'+
                      '<div class="large-6 columns">'+
                        '<label>' + Locale.tr("ULA_PREFIX") +
                          '<input type="text" class="six_ula_net" placeholder="'+Locale.tr("Optional")+'"/>' +
                        '</label>'+
                      '</div>';
                    break;
                }

                $('.net_options', network_context).html(net_form_str);
              });

              $(".network_name", trow).data("network_name", netname)
              $(".network_name", trow).data("one_network", network.one)
            });
          };
        });
      },
      error: function(response){
        context.hide();
        Notifier.onError({}, OpenNebulaError(response));
      }
    });
  }

  function _import(context) {
    $.each($(".network_name:checked", context), function() {
      var network_context = $(this).closest(".vcenter_network");

      $(".vcenter_network_result:not(.success)", network_context).html(
          '<span class="fa-stack fa-2x" style="color: #dfdfdf">' +
            '<i class="fa fa-cloud fa-stack-2x"></i>' +
            '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>' +
          '</span>');

      var network_size = $(".netsize", network_context).val();
      var network_tmpl = $(this).data("one_network");
      var netname      = $(this).data("network_name");
      var type         = $('.type_select', network_context).val();

      var ar_array = [];
      ar_array.push("TYPE=" + type);
      ar_array.push("SIZE=" + network_size);

      switch (type) {
        case 'ETHER':
          var mac = $('.eth_mac_net', network_context).val();

          if (mac) {
            ar_array.push("MAC=" + mac);
          }

          break;
        case 'IP4':
          var mac = $('.four_mac_net', network_context).val();
          var ip = $('.four_ip_net', network_context).val();

          if (mac) {
            ar_array.push("MAC=" + mac);
          }
          if (ip) {
            ar_array.push("IP=" + ip);
          }

          break;
        case 'IP6':
          var mac = $('.six_mac_net', network_context).val();
          var gp = $('.six_global_net', network_context).val();
          var ula = $('.six_mac_net', network_context).val();

          if (mac) {
            ar_array.push("MAC=" + mac);
          }
          if (gp) {
            ar_array.push("GLOBAL_PREFIX=" + gp);
          }
          if (ula) {
            ar_array.push("ULA_PREFIX=" + ula);
          }

          break;
      }

      network_tmpl += "\nAR=["
      network_tmpl += ar_array.join(",\n")
      network_tmpl += "]"

      if ($(".vlaninfo", network_context)) {
        network_tmpl += "VLAN=\"YES\"\n";
        network_tmpl += "VLAN_ID=" + $(".vlaninfo", network_context).val() + "\n";
      }

      var vnet_json = {
        "vnet": {
          "vnet_raw": network_tmpl
        }
      };

      OpenNebulaNetwork.create({
        timeout: true,
        data: vnet_json,
        success: function(request, response) {
          $(".vcenter_network_result", network_context).addClass("success").html(
              '<span class="fa-stack fa-2x" style="color: #dfdfdf">' +
                '<i class="fa fa-cloud fa-stack-2x"></i>' +
                '<i class="fa  fa-check fa-stack-1x fa-inverse"></i>' +
              '</span>');

          $(".vcenter_network_response", network_context).html('<p style="font-size:12px" class="running-color">' +
                Locale.tr("Virtual Network created successfully") + ' ID:' + response.VNET.ID +
              '</p>');
        },
        error: function (request, error_json) {
          $(".vcenter_network_result", network_context).html('<span class="fa-stack fa-2x" style="color: #dfdfdf">' +
                '<i class="fa fa-cloud fa-stack-2x"></i>' +
                '<i class="fa  fa-warning fa-stack-1x fa-inverse"></i>' +
              '</span>');

          $(".vcenter_network_response", network_context).html('<p style="font-size:12px" class="error-color">' +
                (error_json.error.message || Locale.tr("Cannot contact server: is it running and reachable?")) +
              '</p>');
        }
      });
    });

  }
});

/* START_TEMPLATE */
define('hbs!tabs/hosts-tab/form-panels/create/wizard',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <label for=\"host_type\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      <select id=\"host_type_mad\" name=\"host_type\">\n        <option value=\"kvm\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "KVM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"xen\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "XEN", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"vmware\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMware", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"vcenter\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "vCenter", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"az\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Microsoft Azure", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"ec2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Amazon EC2", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"sl\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IBM Softlayer", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"dummy\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Dummy", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"custom\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n    <div class=\"medium-6 columns\" id=\"cluster_select\">\n      <label for=\"host_cluster_id\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cluster", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      <div id=\"host_cluster_id\" name=\"host_cluster_id\"></div>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"medium-6 columns\" id=\"name_container\">\n      <label for=\"name\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hostname", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      <input type=\"text\" name=\"name\" id=\"name\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <div class=\"manager clear row\" id=\"vnm_mads\">\n        <div class=\"medium-12 columns\">\n          <label for=\"vn\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Networking", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <select id=\"vnm_mad\" name=\"vn\">\n            <option value=\"dummy\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Default (dummy)", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"fw\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Firewall", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"802.1Q\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "802.1Q", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"vxlan\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VXLAN", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"ebtables\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ebtables", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"ovswitch\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Open vSwitch", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"vmware\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMware", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"custom\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"large-12 columns\">\n          <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom VNM_MAD", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <input type=\"text\" name=\"custom_vnm_mad\" />\n        </div>\n      </div>\n    </div>\n  </div>\n  <div class=\"drivers\">\n    <fieldset>\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Drivers", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"medium-6 columns\">\n          <div class=\"manager clear row\" id=\"vmm_mads\">\n            <div class=\"large-12 columns\">\n              <label for=\"vmm\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtualization", {"name":"tr","hash":{},"data":data})))
    + "</label>\n              <select id=\"vmm_mad\" name=\"vmm\">\n                <option value=\"kvm\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "KVM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"xen\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "XEN", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"vmware\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMware", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"vcenter\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "vCenter", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"az\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Microsoft Azure", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"ec2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Amazon EC2", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"sl\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IBM Softlayer", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"dummy\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Dummy", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"custom\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom", {"name":"tr","hash":{},"data":data})))
    + "</option>\n              </select>\n            </div>\n            <div class=\"large-12 columns\">\n              <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom VMM_MAD", {"name":"tr","hash":{},"data":data})))
    + "</label>\n              <input type=\"text\" name=\"custom_vmm_mad\" />\n            </div>\n          </div>\n        </div>\n        <div class=\"medium-6 columns\">\n          <div class=\"manager clear row\" id=\"im_mads\">\n            <div class=\"large-12 columns\">\n              <label for=\"im\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</label>\n              <select id=\"im_mad\" name=\"im\">\n                <option value=\"kvm\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "KVM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"xen\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "XEN", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"vmware\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMware", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"vcenter\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "vCenter", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"az\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Microsoft Azure", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"ec2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Amazon EC2", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"sl\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IBM Softlayer", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"dummy\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Dummy", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                <option value=\"custom\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom", {"name":"tr","hash":{},"data":data})))
    + "</option>\n              </select>\n            </div>\n            <div class=\"large-12 columns\">\n              <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom IM_MAD", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n              <input type=\"text\" name=\"custom_im_mad\" />\n            </div>\n          </div>\n        </div>\n      </div>\n    </fieldset>\n  </div>\n  <div class=\"row vcenter_credentials hidden\">\n    <fieldset>\n      <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "vCenter", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n      <div class=\"row\">\n        <div class=\"medium-6 columns\">\n          <label for=\"vcenter_user\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <input type=\"text\" name=\"vcenter_user\" id=\"vcenter_user\" />\n        </div>\n        <div class=\"medium-6 columns\">\n          <label for=\"vcenter_host\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hostname", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <input type=\"text\" name=\"vcenter_host\" id=\"vcenter_host\" />\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"medium-6 columns\">\n          <label for=\"vcenter_password\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Password", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <input type=\"password\" name=\"vcenter_password\" id=\"vcenter_password\" />\n        </div>\n        <div class=\"medium-6 columns\">\n          <br>\n          <a class=\"button radius small right\" id=\"get_vcenter_clusters\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Get vCenter Clusters", {"name":"tr","hash":{},"data":data})))
    + "</a>\n        </div>\n      </div>\n      <div>\n        ";
  stack1 = ((helper = (helper = helpers.vCenterClustersHTML || (depth0 != null ? depth0.vCenterClustersHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"vCenterClustersHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </div>\n      <br>\n      <div>\n        ";
  stack1 = ((helper = (helper = helpers.vCenterTemplatesHTML || (depth0 != null ? depth0.vCenterTemplatesHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"vCenterTemplatesHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </div>\n      <br>\n      <br>\n      <div class=\"vcenter_vms\"></div>\n      <br>\n      <div>\n        ";
  stack1 = ((helper = (helper = helpers.vCenterNetworksHTML || (depth0 != null ? depth0.vCenterNetworksHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"vCenterNetworksHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      </div>\n      <div class=\"row import_vcenter_clusters_div hidden\">\n        <div class=\"large-12 columns\">\n          <br>\n          <a class=\"button radius small right success\" id=\"import_vcenter_clusters\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Import", {"name":"tr","hash":{},"data":data})))
    + "</a>\n        </div>\n      </div>\n    </fieldset>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/hosts-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/hosts-tab/form-panels/create',['require','utils/form-panels/form-panel','sunstone','utils/locale','utils/tips','utils/notifier','utils/resource-select','utils/vcenter/clusters','utils/vcenter/templates','utils/vcenter/networks','hbs!./create/wizard','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var Notifier = require('utils/notifier');
  var ResourceSelect = require('utils/resource-select');
  var VCenterClusters = require('utils/vcenter/clusters');
  var VCenterTemplates = require('utils/vcenter/templates');
  var VCenterNetworks = require('utils/vcenter/networks');

  /*
    TEMPLATES
   */

  var TemplateWizardHTML = require('hbs!./create/wizard');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create Host"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      }
    }

    this.vCenterClusters = new VCenterClusters();
    this.vCenterNetworks = new VCenterNetworks();
    this.vCenterTemplates = new VCenterTemplates();

    BaseFormPanel.call(this);
  };

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    return TemplateWizardHTML({
      'formPanelId': this.formPanelId,
      'vCenterClustersHTML': this.vCenterClusters.html(),
      'vCenterTemplatesHTML': this.vCenterTemplates.html(),
      'vCenterNetworksHTML': this.vCenterNetworks.html()
    });
  }

  function _setup(context) {
    var that = this;

    $(".drivers", context).hide();

    $("#host_type_mad", context).on("change", function() {
      $("#vmm_mad", context).val(this.value).change();
      $("#im_mad", context).val(this.value).change();

      if (this.value == "custom") {
        $(".vcenter_credentials", context).hide();
        $("#vnm_mads", context).show();
        $("#name_container", context).show();
        Sunstone.showFormPanelSubmit(TAB_ID);
        $(".drivers", context).show();
      } else if (this.value == "vcenter") {
        $("#vnm_mads", context).hide();
        $("#name_container", context).hide();
        $(".vcenter_credentials", context).show();
        Sunstone.hideFormPanelSubmit(TAB_ID);
        $(".drivers", context).hide();
      } else {
        $(".vcenter_credentials", context).hide();
        $("#vnm_mads", context).show();
        $("#name_container", context).show();
        Sunstone.showFormPanelSubmit(TAB_ID);
        $(".drivers", context).hide();
      }
    });

    $("#host_type_mad", context).change();

    $("#get_vcenter_clusters", context).on("click", function() {
      // TODO notify if credentials empty

      var vcenter_user = $("#vcenter_user", context).val();
      var vcenter_password = $("#vcenter_password", context).val();
      var vcenter_host = $("#vcenter_host", context).val();

      that.vCenterClusters.insert({
        container: context,
        vcenter_user: vcenter_user,
        vcenter_password: vcenter_password,
        vcenter_host: vcenter_host,
        success: function(){
          $("#vcenter_user", context).attr("disabled", "disabled");
          $("#vcenter_password", context).attr("disabled", "disabled");
          $("#vcenter_host", context).attr("disabled", "disabled");
          $("#get_vcenter_clusters", context).hide();
          $(".import_vcenter_clusters_div", context).show();

          that.vCenterTemplates.insert({
            container: context,
            vcenter_user: vcenter_user,
            vcenter_password: vcenter_password,
            vcenter_host: vcenter_host
          });

          that.vCenterNetworks.insert({
            container: context,
            vcenter_user: vcenter_user,
            vcenter_password: vcenter_password,
            vcenter_host: vcenter_host
          });
        }
      });

      return false;
    });

    $("#import_vcenter_clusters", context).on("click", function() {
      $(this).hide();

      var cluster_id = $('#host_cluster_id .resource_list_select', context).val();
      if (!cluster_id) cluster_id = "-1";

      that.vCenterClusters.import(context, cluster_id);
      that.vCenterTemplates.import(context);
      that.vCenterNetworks.import(context);

      return false;
    });

    // Show custom driver input only when custom is selected in selects
    $('input[name="custom_vmm_mad"],' +
       'input[name="custom_im_mad"],' +
       'input[name="custom_vnm_mad"]', context).parent().hide();

    $('select#vmm_mad', context).change(function() {
      if ($(this).val() == "custom")
          $('input[name="custom_vmm_mad"]').parent().show();
      else
          $('input[name="custom_vmm_mad"]').parent().hide();
    });

    $('select#im_mad', context).change(function() {
      if ($(this).val() == "custom")
          $('input[name="custom_im_mad"]').parent().show();
      else
          $('input[name="custom_im_mad"]').parent().hide();
    });

    $('select#vnm_mad', context).change(function() {
      if ($(this).val() == "custom")
          $('input[name="custom_vnm_mad"]').parent().show();
      else
          $('input[name="custom_vnm_mad"]').parent().hide();
    });

    $('#create_host_form').on("keyup keypress", function(e) {
          var code = e.keyCode || e.which;
          if (code  == 13) {
            e.preventDefault();
            return false;
          }
        });

    Tips.setup();
    return false;
  }

  function _submitWizard(context) {
    var name = $('#name', context).val();
    if (!name) {
      Sunstone.hideFormPanelLoading(this.tabId);
      Notifier.notifyError(Locale.tr("Host name missing!"));
      return false;
    }

    var cluster_id = $('#host_cluster_id .resource_list_select', context).val();
    if (!cluster_id) cluster_id = "-1";

    var vmm_mad = $('select#vmm_mad', context).val();
    vmm_mad = vmm_mad == "custom" ? $('input[name="custom_vmm_mad"]').val() : vmm_mad;
    var im_mad = $('select#im_mad', context).val();
    im_mad = im_mad == "custom" ? $('input[name="custom_im_mad"]').val() : im_mad;
    var vnm_mad = $('select#vnm_mad', context).val();
    vnm_mad = vnm_mad == "custom" ? $('input[name="custom_vnm_mad"]').val() : vnm_mad;

    var host_json = {
      "host": {
        "name": name,
        "vm_mad": vmm_mad,
        "vnm_mad": vnm_mad,
        "im_mad": im_mad,
        "cluster_id": cluster_id
      }
    };

    //Create the OpenNebula.Host.
    //If it is successfull we refresh the list.
    Sunstone.runAction("Host.create", host_json);
    return false;
  }

  function _onShow(context) {
    $("#name", context).focus();

    var cluster_id = $("#host_cluster_id .resource_list_select", context).val();
    if (!cluster_id) cluster_id = "-1";

    ResourceSelect.insert('#host_cluster_id', context, "Cluster", cluster_id, false);

    $("#host_type_mad", context).change();

    return false;
  }
});

define('tabs/hosts-tab',['require','utils/locale','./hosts-tab/buttons','./hosts-tab/actions','./hosts-tab/datatable','./hosts-tab/tabId','./hosts-tab/panels/info','./hosts-tab/panels/monitor','./hosts-tab/panels/vms','./hosts-tab/panels/wilds','./hosts-tab/panels/esx','./hosts-tab/panels/pci','./hosts-tab/form-panels/create'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./hosts-tab/buttons');
  var Actions = require('./hosts-tab/actions');
  var Table = require('./hosts-tab/datatable');

  var TAB_ID = require('./hosts-tab/tabId');
  var DATATABLE_ID = "dataTableHosts";

  //var _dialogs = [
  //  require('./hosts-tab/dialogs/create'),
  //  require('./hosts-tab/dialogs/clone')
  //];

  var _panels = [
    require('./hosts-tab/panels/info'),
    require('./hosts-tab/panels/monitor'),
    require('./hosts-tab/panels/vms'),
    require('./hosts-tab/panels/wilds'),
    require('./hosts-tab/panels/esx'),
    require('./hosts-tab/panels/pci')
  ];

  var _formPanels = [
    require('./hosts-tab/form-panels/create')
  ]

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("Hosts"),
    tabClass: "subTab",
    parentTab: "infra-tab",
    listHeader: '<i class="fa fa-fw fa-hdd-o"></i>&emsp;' + Locale.tr("Hosts"),
    infoHeader: '<i class="fa fa-fw fa-hdd-o"></i>&emsp;' + Locale.tr("Host"),
    subheader: '<span class="total_hosts"/> <small>' + Locale.tr("TOTAL") + '</small>&emsp;\
        <span class="on_hosts"/> <small>' + Locale.tr("ON") + '</small>&emsp;\
        <span class="off_hosts"/> <small>' + Locale.tr("OFF") + '</small>&emsp;\
        <span class="error_hosts"/> <small>' + Locale.tr("ERROR") + '</small>',
    resource: 'Host',
    content: '',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    //dialogs: _dialogs
  };

  return Tab;
});

define('tabs/datastores-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var DatastoreButtons = {
    "Datastore.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "Datastore.create_dialog" : {
      type: "create_dialog",
      layout: "create"
    },
    "Datastore.addtocluster" : {
      type: "confirm_with_select",
      text: Locale.tr("Select cluster"),
      select: "Cluster",
      layout: "main",
      tip: Locale.tr("Select the destination cluster:")
    },
    "Datastore.chown" : {
      type: "confirm_with_select",
      text: Locale.tr("Change owner"),
      select: "User",
      layout: "user_select",
      tip: Locale.tr("Select the new owner") + ":"
    },
    "Datastore.chgrp" : {
      type: "confirm_with_select",
      text: Locale.tr("Change group"),
      select: "Group",
      layout: "user_select",
      tip: Locale.tr("Select the new group") + ":"
    },
    "Datastore.delete" : {
      type: "confirm",
      text: Locale.tr("Delete"),
      layout: "del"
    },
    "Datastore.enable" : {
      type: "action",
      layout: "more_select",
      text: Locale.tr("Enable")
    },
    "Datastore.disable" : {
      type: "action",
      layout: "more_select",
      text: Locale.tr("Disable")
    }
  };

  return DatastoreButtons;
})
;
define('tabs/datastores-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createDatastoreForm';
});
define('tabs/datastores-tab/actions',['require','sunstone','utils/notifier','utils/locale','opennebula/datastore','opennebula/cluster','opennebula/action','utils/common-actions','./tabId','./form-panels/create/formPanelId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var OpenNebulaResource = require('opennebula/datastore');
  var OpenNebulaCluster = require('opennebula/cluster');
  var OpenNebulaAction = require('opennebula/action');
  var CommonActions = require('utils/common-actions');

  var RESOURCE = "Datastore";
  var XML_ROOT = "DATASTORE";
  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "Datastore.create" : _commonActions.create(CREATE_DIALOG_ID),
    "Datastore.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID),
    "Datastore.list" : _commonActions.list(),
    "Datastore.show" : _commonActions.show(),
    "Datastore.refresh" : _commonActions.refresh(),
    "Datastore.delete" : _commonActions.del(),
    "Datastore.chown": _commonActions.multipleAction('chown'),
    "Datastore.chgrp": _commonActions.multipleAction('chgrp'),
    "Datastore.chmod": _commonActions.singleAction('chmod'),
    "Datastore.update" : _commonActions.updateTemplate(),
    "Datastore.update_template" : _commonActions.updateTemplate(),
    "Datastore.rename": _commonActions.singleAction('rename'),
    "Datastore.enable": _commonActions.multipleAction('enable'),
    "Datastore.disable": _commonActions.multipleAction('disable'),

    "Datastore.addtocluster" : {
      type: "multiple",
      call: function(params, success) {
        var cluster = params.data.extra_param;
        var ds = params.data.id;

        if (cluster == -1) {
          OpenNebulaResource.show({
            data : {
              id: ds
            },
            success: function (request, ds_info) {
              var current_cluster = ds_info.DATASTORE.CLUSTER_ID;

              if (current_cluster != -1) {
                OpenNebulaCluster.deldatastore({
                  data: {
                    id: current_cluster,
                    extra_param: ds
                  },
                  success: function() {
                    OpenNebulaAction.clear_cache("DATASTORE");
                    Sunstone.runAction('Datastore.show', ds);
                  },
                  error: Notifier.onError
                });
              } else {
                OpenNebulaAction.clear_cache("DATASTORE");
                Sunstone.runAction('Datastore.show', ds);
              }
            },
            error: Notifier.onError
          });
        } else {
          OpenNebulaCluster.adddatastore({
            data: {
              id: cluster,
              extra_param: ds
            },
            success: function() {
              OpenNebulaAction.clear_cache("DATASTORE");
              Sunstone.runAction('Datastore.show', ds);
            },
            error: Notifier.onError
          });
        }
      },
      elements: function() {
        return Sunstone.getDataTable(TAB_ID).elements();
      }
    }
  };

  return _actions;
});


/* START_TEMPLATE */
define('hbs!tabs/datastores-tab/form-panels/create/wizard',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <label for=\"name\" >"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n      <input required type=\"text\" name=\"name\" id=\"name\"/>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <label for=\"presets\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Presets", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      <select id=\"presets\" name=\"presets\">\n        <option value=\"fs\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filesystem", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"vmware_vmfs\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMware VMFS", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"block_lvm\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Block LVM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"fs_lvm\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "FS LVM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"ceph\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ceph", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"gluster\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Gluster", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"dev\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Devices", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"custom\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"cluster\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cluster", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      <div id=\"cluster_id\" name=\"cluster_id\"></div>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <fieldset>\n        <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n        <div class=\"large-12 columns text-center\">\n          <input id=\"image_ds_type\" type=\"radio\" name=\"ds_type\" value=\"IMAGE_DS\" checked/>\n          <label for=\"image_ds_type\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Images", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <input id=\"system_ds_type\" type=\"radio\" name=\"ds_type\" value=\"SYSTEM_DS\" />\n          <label for=\"system_ds_type\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "System", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <input id=\"file_ds_type\" type=\"radio\" name=\"ds_type\" value=\"FILE_DS\" />\n          <label for=\"file_ds_type\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Files", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        </div>\n      </fieldset>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <fieldset>\n        <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Managers", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n        <div class=\"medium-6 columns\">\n          <label for=\"ds_mad\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Datastore", {"name":"tr","hash":{},"data":data})))
    + "</label>\n          <select id=\"ds_mad\" name=\"ds_mad\">\n            <option value=\"fs\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filesystem", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"vmware\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMware", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"lvm\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "LVM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"vmfs\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMFS", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"ceph\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ceph", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"dev\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Devices", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"custom\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n          <div>\n            <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom DS_MAD", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n            <input type=\"text\" name=\"ds_tab_custom_ds_mad\" />\n          </div>\n        </div>\n        <div class=\"medium-6 columns\">\n          <label for=\"tm_mad\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Transfer", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <select id=\"tm_mad\" name=\"tm_mad\">\n            <option value=\"shared\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shared", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"ssh\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "SSH", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"qcow2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "qcow2", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"lvm\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "LVM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"fs_lvm\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "FS LVM", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"vmfs\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMFS", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"ceph\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ceph", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"dev\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Devices", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"custom\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n          <div>\n            <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom TM_MAD", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n            <input type=\"text\" name=\"ds_tab_custom_tm_mad\" />\n          </div>\n        </div>\n      </fieldset>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <label for=\"disk_type\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Disk type", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n      <select id=\"disk_type\" name=\"disk_type\">\n        <option value=\"file\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "File", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"block\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Block", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"RBD\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "RBD", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"gluster\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Gluster", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <label for=\"safe_dirs\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Safe Directories", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "If you need to un-block a directory under one of the RESTRICTED_DIRS", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"safe_dirs\" id=\"safe_dirs\" />\n    </div>\n    <div class=\"large-12 columns\">\n      <label for=\"restricted_dirs\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Restricted Directories", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Paths that can not be used to register images. A space separated list of paths. This will prevent users registering important files as VM images and accessing them thourgh their VMs. OpenNebula will automatically add its configuration directories: /var/lib/one, /etc/one and oneadmin's home ($HOME).", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"restricted_dirs\" id=\"restricted_dirs\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"base_path\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Base Path", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "When needed, the front-end will access the datastores using BASE_PATH (defaults to /var/lib/one/datastores).", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"base_path\" id=\"base_path\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"limit_transfer_bw\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Transfer BW Limit", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Specify the maximum transfer rate in bytes/second when downloading images from a http/https URL. Suffixes K, M or G can be used.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"limit_transfer_bw\" id=\"limit_transfer_bw\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"limit_mb\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Limit", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Optional limit, in MB. If set, OpenNebula will only use this amount of storage, instead of the whole free disk.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"limit_mb\" id=\"limit_mb\" />\n    </div>\n    <div class=\"large-12 columns\">\n      <input id=\"no_decompress\" type=\"checkbox\" name=\"no_decompress\" value=\"YES\" />\n      <label for=\"no_decompress\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Do not try to untar or decompress", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    </div>\n    <div class=\"large-12 columns\">\n      <input id=\"datastore_capacity_check\" type=\"checkbox\" name=\"datastore_capacity_check\" value=\"YES\" />\n      <label for=\"datastore_capacity_check\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Check available capacity of the datastore before creating a new image", {"name":"tr","hash":{},"data":data})))
    + "\n      </label>\n    </div>\n    <div class=\"large-12 columns\">\n      <label for=\"bridge_list\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Host Bridge List", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Space separated list of Server names or IPs where OpenNebula will be staging the new images into. This server will act as the entry point for new inmages in the datastore.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"bridge_list\" id=\"bridge_list\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"ds_tmp_dir\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "DS Tmp Dir", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Path in the OpenNebula front-end to be used as a buffer to stage in files in vmfs datastores.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"ds_tmp_dir\" id=\"ds_tmp_dir\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"vg_name\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Volume Group Name", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      <input type=\"text\" name=\"vg_name\" id=\"vg_name\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"gluster_host\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Gluster Host", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Host and port of one (and only one) Gluster server (host:port)", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"gluster_host\" id=\"gluster_host\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"gluster_volume\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Gluster Volume", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Gluster volume to use for the datastore", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </label>\n      <input type=\"text\" name=\"gluster_volume\" id=\"gluster_volume\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"pool_name\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Pool Name", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "The OpenNebula Ceph pool name. Defaults to 'one' (this pool must exist before using the drivers).", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"pool_name\" id=\"pool_name\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"ceph_host\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ceph Host", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Space-separated list of Ceph monitors. Example: host1 host2:port2 host3 host4:port4 (if no port is specified, the default one is chosen) (Required for Libvirt 1.x when cephx is enabled).", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"ceph_host\" id=\"ceph_host\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"ceph_user\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ceph User", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "The OpenNebula Ceph user name. If set it is used by RBD commands. This ceph user must exist before using the drivers. Required for Libvirt 1.x when cephx is enabled .", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"ceph_user\" id=\"ceph_user\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"ceph_secret\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ceph Secret", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "A generated UUID for a LibVirt secret (to hold the CephX authentication key in Libvirt on each hypervisor). This should be generated when creating the Ceph datastore in OpenNebula. (Required for Libvirt 1.x when cephx is enabled).", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"ceph_secret\" id=\"ceph_secret\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label class=\"fs\" for=\"staging_dir\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Staging Dir", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "FS: Default path where images will be temporarily copied to in the host carrying out the registration operation (chosen from the bridge list). If empty, defaults to /var/tmp.", {"name":"tr","hash":{},"data":data})))
    + "\n          <br>\n          <br>\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ceph: Default path for image operations in the OpenNebula Ceph frontend.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"staging_dir\" id=\"staging_dir\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"rbd_format\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "RBD Format", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "By default RBD Format 2 will be used. If RBD_FORMAT=2 is specified then when instantiating non-persistent images the Ceph driver will perform rbd snap instead of rbd copy.", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </label>\n      <input type=\"text\" name=\"rbd_format\" id=\"rbd_format\" />\n    </div>\n  </div>\n</form>\n";
},"useData":true});
Handlebars.registerPartial('tabs/datastores-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/datastores-tab/form-panels/create/advanced',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Advanced\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"columns large-6\">\n      <label for=\"datastore_cluster_raw\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cluster", {"name":"tr","hash":{},"data":data})))
    + "</label>\n      <div id=\"datastore_cluster_raw\" name=\"datastore_cluster_raw\"></div>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <p>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Write the Virtual Network template here", {"name":"tr","hash":{},"data":data})))
    + "</p>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <textarea id=\"template\" rows=\"15\" required></textarea>\n    </div>\n  </div>\n</form>\n\n\n\n\n";
},"useData":true});
Handlebars.registerPartial('tabs/datastores-tab/form-panels/create/advanced', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/datastores-tab/form-panels/create',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','utils/notifier','utils/tips','utils/resource-select','hbs!./create/wizard','hbs!./create/advanced','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var Notifier = require('utils/notifier');
  var Tips = require('utils/tips');
  var ResourceSelect = require('utils/resource-select');

  /*
    TEMPLATES
   */

  var TemplateWizardHTML = require('hbs!./create/wizard');
  var TemplateAdvancedHTML = require('hbs!./create/advanced');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create Datastore"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      }
    };

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.htmlAdvanced = _htmlAdvanced;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.submitAdvanced = _submitAdvanced;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    return TemplateWizardHTML({
      'formPanelId': this.formPanelId,
    });
  }

  function _htmlAdvanced() {
    return TemplateAdvancedHTML({formPanelId: this.formPanelId});
  }

  function _onShow(dialog) {
    $("#name", dialog).focus();

    var cluster_id = $("div#cluster_id .resource_list_select", dialog).val();
    if (!cluster_id) cluster_id = "-1";

    var cluster_id_raw = $("div#datastore_cluster_raw .resource_list_select", dialog).val();
    if (!cluster_id_raw) cluster_id_raw = "-1";

    ResourceSelect.insert('div#cluster_id', dialog, "Cluster", cluster_id, false);
    ResourceSelect.insert('div#datastore_cluster_raw', dialog, "Cluster", cluster_id_raw, false);

    return false;
  }

  // Set up the create datastore dialog
  function _setup(dialog) {
    Tips.setup(dialog);

    // Show custom driver input only when custom is selected in selects
    $('input[name="ds_tab_custom_ds_mad"],' +
      'input[name="ds_tab_custom_tm_mad"]', dialog).parent().hide();

    $('select#ds_mad', dialog).change(function() {
      if ($(this).val() == "custom")
          $('input[name="ds_tab_custom_ds_mad"]').parent().show();
      else
          $('input[name="ds_tab_custom_ds_mad"]').parent().hide();
    });

    $('select#tm_mad', dialog).change(function() {
      if ($(this).val() == "custom")
          $('input[name="ds_tab_custom_tm_mad"]').parent().show();
      else
          $('input[name="ds_tab_custom_tm_mad"]').parent().hide();
    });

    $('#presets').change(function() {
      _hideAll(dialog);
      var choice_str = $(this).val();
      switch (choice_str)
      {
        case 'fs':
          _selectFilesystem();
          break;
        case 'vmware_vmfs':
          _selectVmwareVmfs();
          break;
        case 'block_lvm':
          _selectBlockLvm();
          break;
        case 'fs_lvm':
          _selectFsLvm();
          break;
        case 'ceph':
          _selectCeph();
          break;
        case 'gluster':
          _selectGluster();
          break;
        case 'dev':
          _selectDevices();
          break;
        case 'custom':
          _selectCustom();
          break;
      }
    });

    // Hide disk_type
    $('select#disk_type').parent().hide();

    _hideAll(dialog);
    _selectFilesystem();
  }


  function _submitWizard(dialog) {
    var name            = $('#name', dialog).val();
    var cluster_id      = $(".resource_list_select", $('#cluster_id', dialog)).val();
    var ds_type         = $('input[name=ds_type]:checked', dialog).val();
    var ds_mad          = $('#ds_mad', dialog).val();
    ds_mad              = ds_mad == "custom" ? $('input[name="ds_tab_custom_ds_mad"]').val() : ds_mad;
    var tm_mad          = $('#tm_mad', dialog).val();
    tm_mad              = tm_mad == "custom" ? $('input[name="ds_tab_custom_tm_mad"]').val() : tm_mad;
    var type            = $('#disk_type', dialog).val();

    var safe_dirs       = $('#safe_dirs', dialog).val();
    var base_path       = $('#base_path', dialog).val();
    var restricted_dirs = $('#restricted_dirs', dialog).val();
    var limit_transfer_bw = $('#limit_transfer_bw', dialog).val();
    var datastore_capacity_check = $('#datastore_capacity_check', dialog).is(':checked');
    var no_decompress   = $('#no_decompress', dialog).is(':checked');

    var bridge_list     = $('#bridge_list', dialog).val();
    var ds_tmp_dir     = $('#ds_tmp_dir', dialog).val();
    var vg_name         = $('#vg_name', dialog).val();
    var limit_mb        = $('#limit_mb', dialog).val();
    var gluster_host    = $('#gluster_host', dialog).val();
    var gluster_volume  = $('#gluster_volume', dialog).val();
    var pool_name       = $('#pool_name', dialog).val();
    var ceph_host       = $('#ceph_host', dialog).val();
    var ceph_secret     = $('#ceph_secret', dialog).val();
    var ceph_user       = $('#ceph_user', dialog).val();
    var rbd_format      = $('#rbd_format', dialog).val();
    var staging_dir     = $('#staging_dir', dialog).val();

    var ds_obj = {
      "datastore" : {
        "name" : name,
        "tm_mad" : tm_mad,
        "disk_type" : type,
        "type" : ds_type
      },
      "cluster_id" : cluster_id
    };

    // If we are adding a system datastore then
    // we do not use ds_mad
    if (ds_type != "SYSTEM_DS")
        ds_obj.datastore.ds_mad = ds_mad;

    if (base_path)
        ds_obj.datastore.base_path = base_path;

    if (safe_dirs)
        ds_obj.datastore.safe_dirs = safe_dirs;

    if (restricted_dirs)
        ds_obj.datastore.restricted_dirs = restricted_dirs;

    if (limit_transfer_bw)
        ds_obj.datastore.limit_transfer_bw = limit_transfer_bw;

    if (no_decompress)
        ds_obj.datastore.no_decompress = "YES";

    if (datastore_capacity_check)
        ds_obj.datastore.datastore_capacity_check = "YES";

    if (bridge_list)
        ds_obj.datastore.bridge_list = bridge_list;

    if (ds_tmp_dir)
        ds_obj.datastore.ds_tmp_dir = ds_tmp_dir;

    if (vg_name)
        ds_obj.datastore.vg_name = vg_name;

    if (limit_mb)
        ds_obj.datastore.limit_mb = limit_mb;

    if (gluster_host)
        ds_obj.datastore.gluster_host = gluster_host;

    if (gluster_volume)
        ds_obj.datastore.gluster_volume = gluster_volume;

    if (pool_name)
        ds_obj.datastore.pool_name = pool_name;

    if (ceph_host)
        ds_obj.datastore.ceph_host = ceph_host;

    if (ceph_secret)
        ds_obj.datastore.ceph_secret = ceph_secret;

    if (ceph_user)
        ds_obj.datastore.ceph_user = ceph_user;

    if (rbd_format)
        ds_obj.datastore.rbd_format = rbd_format;

    if (staging_dir)
        ds_obj.datastore.staging_dir = staging_dir;

    Sunstone.runAction("Datastore.create", ds_obj);
    return false;
  }

  function _submitAdvanced(dialog) {
    var template   = $('#template', dialog).val();
    var cluster_id = $(".resource_list_select", $('#datastore_cluster_raw', dialog)).val();

    if (!cluster_id) {
      Notifier.notifyError(Locale.tr("Please select a cluster for this datastore"));
      return false;
    }

    var ds_obj = {
      "datastore" : {
        "datastore_raw" : template
      },
      "cluster_id" : cluster_id
    };

    Sunstone.runAction("Datastore.create", ds_obj);
    return false;
  }

  function _hideAll(dialog) {
    // Hide all the options that depends on datastore type
    // and reset the selects

    $('input#image_ds_type').attr('checked', 'true');
    $('input[name=ds_type]').removeAttr('disabled', 'disabled');

    $('label[for="bridge_list"],input#bridge_list', dialog).parent().hide();
    $('label[for="ds_tmp_dir"],input#ds_tmp_dir', dialog).parent().hide();
    $('label[for="vg_name"],input#vg_name', dialog).hide();
    $('label[for="gluster_host"],input#gluster_host', dialog).parent().hide();
    $('label[for="gluster_volume"],input#gluster_volume', dialog).parent().hide();
    $('label[for="pool_name"],input#pool_name', dialog).parent().hide();
    $('label[for="ceph_host"],input#ceph_host', dialog).parent().hide();
    $('label[for="ceph_secret"],input#ceph_secret', dialog).parent().hide();
    $('label[for="ceph_user"],input#ceph_user', dialog).parent().hide();
    $('label[for="rbd_format"],input#rbd_format', dialog).parent().hide();
    $('label[for="staging_dir"],input#staging_dir', dialog).parent().hide();
    $('label[for="limit_transfer_bw"],input#limit_transfer_bw', dialog).parent().hide();
    $('label[for="no_decompress"],input#no_decompress', dialog).parent().hide();
    $('select#ds_mad').removeAttr('disabled');
    $('select#tm_mad').removeAttr('disabled');
    $('select#tm_mad').children('option').each(function() {
        $(this).removeAttr('disabled');
      });
    $('select#disk_type').removeAttr('disabled');
    $('select#disk_type').children('option').each(function() {
        $(this).removeAttr('disabled');
      });

    $('input[name="ds_tab_custom_ds_mad"]', dialog).parent().hide();
    $('input[name="ds_tab_custom_tm_mad"]', dialog).parent().hide();
  }

  function _selectFilesystem(dialog) {
    $('select#ds_mad').val('fs');
    $('select#tm_mad').val('shared');
    $('select#ds_mad').attr('disabled', 'disabled');
    $('select#tm_mad').children('option').each(function() {
        var value_str = $(this).val();
        $(this).attr('disabled', 'disabled');
        if (value_str == "qcow2"  ||
            value_str == "shared" ||
            value_str == "ssh") {
          $(this).removeAttr('disabled');
        }
      });
    $('select#disk_type').val('file');
    $('select#disk_type').attr('disabled', 'disabled');
    $('label[for="limit_transfer_bw"],input#limit_transfer_bw').parent().fadeIn();
    $('label[for="no_decompress"],input#no_decompress').parent().fadeIn();
    $('label[for="datastore_capacity_check"],input#datastore_capacity_check').parent().fadeIn();
    $('input#safe_dirs').removeAttr('disabled');
    $('select#disk_type').removeAttr('disabled');
    $('input#base_path').removeAttr('disabled');
    $('input#limit_mb').removeAttr('disabled');
    $('input#restricted_dirs').removeAttr('disabled');
    $('label[for="bridge_list"],input#bridge_list').parent().fadeIn();
    $('label[for="staging_dir"],input#staging_dir').parent().fadeIn();
  }

  function _selectVmwareVmfs(dialog) {
    $('label[for="bridge_list"],input#bridge_list').parent().fadeIn();
    $('label[for="ds_tmp_dir"],input#ds_tmp_dir').parent().fadeIn();
    $('select#ds_mad').val('vmfs');
    $('select#ds_mad').attr('disabled', 'disabled');
    $('select#tm_mad').val('vmfs');
    $('select#tm_mad').attr('disabled', 'disabled');
    $('label[for="limit_transfer_bw"],input#limit_transfer_bw').parent().fadeIn();
    $('label[for="no_decompress"],input#no_decompress').parent().fadeIn();
    $('label[for="datastore_capacity_check"],input#datastore_capacity_check').parent().fadeIn();
    $('select#disk_type').val('file');
    $('select#disk_type').attr('disabled', 'disabled');
    $('input#safe_dirs').removeAttr('disabled');
    $('input#base_path').removeAttr('disabled');
    $('input#limit_mb').removeAttr('disabled');
    $('input#restricted_dirs').removeAttr('disabled');
  }

  function _selectCeph(dialog) {
    $('input#image_ds_type').attr('checked', 'true');
    $('input[name=ds_type]').attr('disabled', 'disabled');
    $('select#ds_mad').val('ceph');
    $('select#ds_mad').attr('disabled', 'disabled');
    $('select#tm_mad').val('ceph');
    $('select#tm_mad').attr('disabled', 'disabled');
    $('label[for="bridge_list"],input#bridge_list').parent().fadeIn();
    $('label[for="pool_name"],input#pool_name').parent().fadeIn();
    $('label[for="ceph_host"],input#ceph_host').parent().fadeIn();
    $('label[for="ceph_secret"],input#ceph_secret').parent().fadeIn();
    $('label[for="ceph_user"],input#ceph_user').parent().fadeIn();
    $('label[for="rbd_format"],input#rbd_format').parent().fadeIn();
    $('label[for="staging_dir"],input#staging_dir').parent().fadeIn();
    $('label[for="limit_transfer_bw"],input#limit_transfer_bw').parent().fadeIn();
    $('label[for="no_decompress"],input#no_decompress').parent().fadeIn();
    $('label[for="datastore_capacity_check"],input#datastore_capacity_check').parent().fadeIn();
    $('select#disk_type').val('RBD');
    $('select#disk_type').attr('disabled', 'disabled');
    $('input#safe_dirs').removeAttr('disabled');
    $('input#base_path').removeAttr('disabled');
    $('input#limit_mb').removeAttr('disabled');
    $('input#restricted_dirs').removeAttr('disabled');
  }

  function _selectBlockLvm(dialog) {
    $('select#ds_mad').val('lvm');
    $('select#ds_mad').attr('disabled', 'disabled');
    $('select#tm_mad').val('lvm');
    $('select#tm_mad').attr('disabled', 'disabled');
    $('input#image_ds_type').attr('checked', 'true');
    $('input[name=ds_type]').attr('disabled', 'disabled');
    $('label[for="bridge_list"],input#bridge_list').parent().fadeIn();
    $('label[for="vg_name"],input#vg_name').fadeIn();
    $('label[for="limit_transfer_bw"],input#limit_transfer_bw').parent().fadeIn();
    $('label[for="no_decompress"],input#no_decompress').parent().fadeIn();
    $('label[for="datastore_capacity_check"],input#datastore_capacity_check').parent().fadeIn();
    $('select#disk_type').val('block');
    $('select#disk_type').attr('disabled', 'disabled');
    $('input#safe_dirs').removeAttr('disabled');
    $('input#base_path').removeAttr('disabled');
    $('input#limit_mb').removeAttr('disabled');
    $('input#restricted_dirs').removeAttr('disabled');
  }

  function _selectFsLvm(dialog) {
    $('select#ds_mad').val('fs');
    $('select#ds_mad').attr('disabled', 'disabled');
    $('select#tm_mad').val('fs_lvm');
    $('select#tm_mad').attr('disabled', 'disabled');
    $('input#image_ds_type').attr('checked', 'true');
    $('input[name=ds_type]').attr('disabled', 'disabled');
    $('label[for="limit_transfer_bw"],input#limit_transfer_bw').parent().fadeIn();
    $('label[for="no_decompress"],input#no_decompress').parent().fadeIn();
    $('label[for="datastore_capacity_check"],input#datastore_capacity_check').parent().fadeIn();
    $('select#disk_type').val('block');
    $('select#disk_type').attr('disabled', 'disabled');
    $('input#safe_dirs').removeAttr('disabled');
    $('input#base_path').removeAttr('disabled');
    $('input#limit_mb').removeAttr('disabled');
    $('input#restricted_dirs').removeAttr('disabled');
  }

  function _selectGluster(dialog) {
    $('select#ds_mad').val('fs');
    $('select#ds_mad').attr('disabled', 'disabled');
    $('select#tm_mad').val('shared');
    $('select#tm_mad').children('option').each(function() {
        var value_str = $(this).val();
        $(this).attr('disabled', 'disabled');
        if (value_str == "shared"  ||
            value_str == "ssh") {
          $(this).removeAttr('disabled');
        }
      });
    $('input#image_ds_type').attr('checked', 'true');
    $('input[name=ds_type]').attr('disabled', 'disabled');
    $('select#disk_type').val('gluster');
    $('select#disk_type').attr('disabled', 'disabled');
    $('label[for="gluster_host"],input#gluster_host').parent().fadeIn();
    $('label[for="gluster_volume"],input#gluster_volume').parent().fadeIn();
    $('label[for="limit_transfer_bw"],input#limit_transfer_bw').parent().fadeIn();
    $('label[for="no_decompress"],input#no_decompress').parent().fadeIn();
    $('label[for="datastore_capacity_check"],input#datastore_capacity_check').parent().fadeIn();
    $('input#safe_dirs').removeAttr('disabled');
    $('input#base_path').removeAttr('disabled');
    $('input#limit_mb').removeAttr('disabled');
    $('input#restricted_dirs').removeAttr('disabled');
  }

  function _selectDevices(dialog) {
    $('select#ds_mad').val('dev');
    $('select#ds_mad').attr('disabled', 'disabled');
    $('select#tm_mad').val('dev');
    $('select#tm_mad').attr('disabled', 'disabled');
    $('input#image_ds_type').attr('checked', 'true');
    $('input[name=ds_type]').attr('disabled', 'disabled');
    $('select#disk_type').val('block');
    $('select#disk_type').attr('disabled', 'disabled');
    $('label[for="limit_transfer_bw"],input#limit_transfer_bw').parent().hide();
    $('label[for="no_decompress"],input#no_decompress').parent().hide();
    $('label[for="datastore_capacity_check"],input#datastore_capacity_check').parent().hide();
    $('input#safe_dirs').attr('disabled', 'disabled');
    $('input#base_path').attr('disabled', 'disabled');
    $('input#limit_mb').attr('disabled', 'disabled');
    $('input#restricted_dirs').attr('disabled', 'disabled');
  }

  function _selectCustom(dialog) {
    _hideAll(dialog);
    $('select#ds_mad').val('fs');
    $('select#tm_mad').val('shared');
    $('input#safe_dirs').removeAttr('disabled');
    $('select#disk_type').removeAttr('disabled');
    $('input#base_path').removeAttr('disabled');
    $('input#limit_mb').removeAttr('disabled');
    $('input#restricted_dirs').removeAttr('disabled');
    $('label[for="limit_transfer_bw"],input#limit_transfer_bw').parent().fadeIn();
    $('label[for="no_decompress"],input#no_decompress').parent().fadeIn();
    $('label[for="datastore_capacity_check"],input#datastore_capacity_check').parent().fadeIn();
  }
});


/* START_TEMPLATE */
define('hbs!tabs/datastores-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n        </tr>\n        ";
  stack1 = ((helper = (helper = helpers.renameTrHTML || (depth0 != null ? depth0.renameTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"renameTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        ";
  stack1 = ((helper = (helper = helpers.clusterTrHTML || (depth0 != null ? depth0.clusterTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"clusterTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "State", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.stateStr || (depth0 != null ? depth0.stateStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"stateStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.typeStr || (depth0 != null ? depth0.typeStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"typeStr","hash":{},"data":data}) : helper)))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Base path", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.BASE_PATH : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Capacity", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">";
  stack1 = ((helper = (helper = helpers.capacityBar || (depth0 != null ? depth0.capacityBar : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"capacityBar","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Limit", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(((helper = (helper = helpers.limitStr || (depth0 != null ? depth0.limitStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"limitStr","hash":{},"data":data}) : helper)))
    + "</td>\n        </tr>\n      </tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\">\n    ";
  stack1 = ((helper = (helper = helpers.permissionsTableHTML || (depth0 != null ? depth0.permissionsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"permissionsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-9 columns\">";
  stack1 = ((helper = (helper = helpers.templateTableHTML || (depth0 != null ? depth0.templateTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templateTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/datastores-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/datastores-tab/panels/info/panelId',['require'],function(require){
  return 'datastore_info_tab';
});
define('tabs/datastores-tab/panels/info',['require','utils/locale','utils/humanize','utils/panel/rename-tr','utils/panel/template-table','utils/panel/permissions-table','utils/panel/cluster-tr','opennebula/datastore','../utils/datastore-capacity-bar','hbs!./info/html','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var RenameTr = require('utils/panel/rename-tr');
  var TemplateTable = require('utils/panel/template-table');
  var PermissionsTable = require('utils/panel/permissions-table');
  var ClusterTr = require('utils/panel/cluster-tr');
  var OpenNebulaDatastore = require('opennebula/datastore');
  var DatastoreCapacityBar = require('../utils/datastore-capacity-bar');

  /*
    TEMPLATES
   */

  var TemplateInfo = require('hbs!./info/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "Datastore"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[RESOURCE.toUpperCase()];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */


  function _html() {
    var renameTrHTML = RenameTr.html(RESOURCE, this.element.NAME);
    var clusterTrHTML = ClusterTr.html(this.element.CLUSTER);
    var templateTableHTML = TemplateTable.html(
                                      this.element.TEMPLATE, RESOURCE,
                                      Locale.tr("Attributes"));
    var permissionsTableHTML = PermissionsTable.html(TAB_ID, RESOURCE, this.element);
    var capacityBar = DatastoreCapacityBar.html(this.element);
    var stateStr = OpenNebulaDatastore.stateStr(this.element.STATE);
    var typeStr = OpenNebulaDatastore.typeStr(this.element.TYPE);

    var limitStr = '-';
    if (this.element.TEMPLATE.SHARED == "NO" || this.element.TEMPLATE.LIMIT_MB != undefined) {
      limitStr = Humanize.sizeFromMB(this.element.TEMPLATE.LIMIT_MB)
    }

    return TemplateInfo({
      'element': this.element,
      'renameTrHTML': renameTrHTML,
      'clusterTrHTML': clusterTrHTML,
      'templateTableHTML': templateTableHTML,
      'permissionsTableHTML': permissionsTableHTML,
      'capacityBar': capacityBar,
      'stateStr': stateStr,
      'typeStr': typeStr,
      'limitStr': limitStr
    });
  }

  function _setup(context) {
    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);
    RenameTr.setup(RESOURCE, this.element.ID, context);
    ClusterTr.setup(RESOURCE, this.element.ID, this.element.CLUSTER_ID, context);
    TemplateTable.setup(this.element.TEMPLATE, RESOURCE, this.element.ID, context);
    PermissionsTable.setup(TAB_ID, RESOURCE, this.element, context);
    return false;
  }
});

define('tabs/datastores-tab/panels/images/panelId',['require'],function(require){
  return 'datastore_image_tab';
});
define('tabs/datastores-tab/panels/images',['require','utils/locale','opennebula/datastore','tabs/images-tab/datatable','tabs/files-tab/datatable','./images/panelId'],function(require){
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var OpenNebulaDatastore = require('opennebula/datastore');
  var ImagesTable = require('tabs/images-tab/datatable');
  var FilesTable = require('tabs/files-tab/datatable');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./images/panelId');
  var IMAGES_TABLE_ID = PANEL_ID + "ImagesTable"
  var RESOURCE = "Datastore"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Images");
    this.icon = "fa-upload";

    this.element = info[RESOURCE.toUpperCase()];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var imgs = [];

    if (this.element.IMAGES.ID != undefined){
      imgs = this.element.IMAGES.ID;

      if (!$.isArray(imgs)){
        imgs = [imgs];
      }
    }

    var opts = {
      info: true,
      select: true,
      selectOptions: {
        read_only: true,
        fixed_ids: imgs
      }
    };

    if (this.element.TYPE == OpenNebulaDatastore.TYPES.FILE_DS){
      this.imagesDataTable = new FilesTable(IMAGES_TABLE_ID, opts);
    } else {
      this.imagesDataTable = new ImagesTable(IMAGES_TABLE_ID, opts);
    }

    return this.imagesDataTable.dataTableHTML;
  }

  function _setup(context) {
    this.imagesDataTable.initialize();
    this.imagesDataTable.refreshResourceTableSelect();

    return false;
  }
});
define('tabs/datastores-tab',['require','utils/locale','./datastores-tab/buttons','./datastores-tab/actions','./datastores-tab/datatable','./datastores-tab/tabId','./datastores-tab/form-panels/create','./datastores-tab/panels/info','./datastores-tab/panels/images'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./datastores-tab/buttons');
  var Actions = require('./datastores-tab/actions');
  var Table = require('./datastores-tab/datatable');

  var TAB_ID = require('./datastores-tab/tabId');
  var DATATABLE_ID = "dataTableDatastores";

  var _dialogs = [
  ];

  var _formPanels = [
    require('./datastores-tab/form-panels/create')
  ];

  var _panels = [
    require('./datastores-tab/panels/info'),
    require('./datastores-tab/panels/images')
  ];

  var DatastoresTab = {
    tabId: TAB_ID,
    title: Locale.tr("Datastores"),
    tabClass: "subTab",
    parentTab: "infra-tab",
    listHeader: '<i class="fa fa-fw fa-folder-open"></i>&emsp;' + Locale.tr("Datastores"),
    infoHeader: '<i class="fa fa-fw fa-folder-open"></i>&emsp;' + Locale.tr("Datastore"),
    subheader: '',
    resource: 'Datastore',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return DatastoresTab;
});
define('tabs/vnets-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var VNetButtons = {
    "Network.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "Network.create_dialog" : {
      type: "create_dialog",
      layout: "create"
    },
    "Network.import_dialog" : {
      type: "create_dialog",
      layout: "create",
      text:  Locale.tr("Import"),
      icon: '<i class="fa fa-download">',
      alwaysActive: true
    },
    "Network.update_dialog" : {
      type: "action",
      layout: "main",
      text: Locale.tr("Update")
    },
    "Network.reserve_dialog" : {
      type: "action",
      layout: "main",
      text: Locale.tr("Reserve"),
      custom_classes: "only-right-info reserve-right-info",
    },
    "Network.addtocluster" : {
      type: "confirm_with_select",
      text: Locale.tr("Select cluster"),
      layout: "main",
      select: "Cluster",
      tip: Locale.tr("Select the destination cluster:")
    },
    "Network.chown" : {
      type: "confirm_with_select",
      text: Locale.tr("Change owner"),
      layout: "user_select",
      select: "User",
      tip: Locale.tr("Select the new owner")+":"
    },

    "Network.chgrp" : {
      type: "confirm_with_select",
      text: Locale.tr("Change group"),
      layout: "user_select",
      select: "Group",
      tip: Locale.tr("Select the new group")+":"
    },

    "Network.delete" : {
      type: "confirm",
      layout: "del",
      text: Locale.tr("Delete")
    }
  };

  return VNetButtons;
})
;
define('tabs/vnets-tab/dialogs/add-ar/dialogId',['require'],function(require){
  return 'add_ar_dialog';
});
define('tabs/vnets-tab/dialogs/update-ar/dialogId',['require'],function(require){
  return 'update_ar_dialog';
});
define('tabs/vnets-tab/dialogs/reserve/dialogId',['require'],function(require){
  return 'reserve_dialog';
});
define('tabs/vnets-tab/form-panels/import/formPanelId',['require'],function(require){
  return 'importVNetForm';
});
define('tabs/vnets-tab/actions',['require','sunstone','utils/notifier','utils/locale','./datatable','opennebula/network','opennebula/cluster','opennebula/action','utils/common-actions','./tabId','./form-panels/create/formPanelId','./dialogs/add-ar/dialogId','./dialogs/update-ar/dialogId','./dialogs/reserve/dialogId','./form-panels/import/formPanelId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var DataTable = require('./datatable');
  var OpenNebulaResource = require('opennebula/network');
  var OpenNebulaCluster = require('opennebula/cluster');
  var OpenNebulaAction = require('opennebula/action');
  var CommonActions = require('utils/common-actions');

  var RESOURCE = "Network";
  var XML_ROOT = "VNET";
  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');
  var ADD_AR_DIALOG_ID = require('./dialogs/add-ar/dialogId');
  var UPDATE_AR_DIALOG_ID = require('./dialogs/update-ar/dialogId');
  var RESERVE_DIALOG_ID = require('./dialogs/reserve/dialogId');
  var IMPORT_DIALOG_ID = require('./form-panels/import/formPanelId');

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "Network.create" : _commonActions.create(CREATE_DIALOG_ID),
    "Network.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID),
    "Network.import_dialog" :  _commonActions.showCreate(IMPORT_DIALOG_ID),
    "Network.list" : _commonActions.list(),
    "Network.show" : _commonActions.show(),
    "Network.refresh" : _commonActions.refresh(),
    "Network.delete" : _commonActions.del(),
    "Network.hold": _commonActions.singleAction('hold'),
    "Network.release": _commonActions.singleAction('release'),
    "Network.chown": _commonActions.multipleAction('chown'),
    "Network.chgrp": _commonActions.multipleAction('chgrp'),
    "Network.chmod": _commonActions.singleAction('chmod'),
    "Network.rename": _commonActions.singleAction('rename'),
    "Network.update" : _commonActions.update(),
    "Network.update_template" : _commonActions.updateTemplate(),
    "Network.update_dialog" : _commonActions.checkAndShowUpdate(),
    "Network.show_to_update" : _commonActions.showUpdate(CREATE_DIALOG_ID),

    "Network.add_ar" : {
      type: "single",
      call: OpenNebulaResource.add_ar,
      callback: function(req) {
        // Reset the wizard
        Sunstone.getDialog(ADD_AR_DIALOG_ID).hide();
        Sunstone.getDialog(ADD_AR_DIALOG_ID).reset();

        Sunstone.runAction("Network.show",req.request.data[0][0]);
      },
      error: Notifier.onError
    },

    "Network.rm_ar" : {
      type: "single",
      call: OpenNebulaResource.rm_ar,
      callback: function(req) {
        OpenNebulaAction.clear_cache("VNET");
        Sunstone.runAction("Network.show",req.request.data[0][0]);
      },
      error: Notifier.onError
    },

    "Network.update_ar" : {
      type: "single",
      call: OpenNebulaResource.update_ar,
      callback: function(req) {
        // Reset the wizard
        Sunstone.getDialog(UPDATE_AR_DIALOG_ID).hide();
        Sunstone.getDialog(UPDATE_AR_DIALOG_ID).reset();

        Sunstone.runAction("Network.show",req.request.data[0][0]);
      },
      error: Notifier.onError
    },

    "Network.reserve_dialog" : {
      type: "custom",
      call: function() {
        var selected_nodes = Sunstone.getDataTable(TAB_ID).elements();
        if (selected_nodes.length != 1) {
          Notifier.notifyMessage("Please select one (and just one) Virtual Network.");
          return false;
        }

        var resource_id = "" + selected_nodes[0];

        Sunstone.getDialog(RESERVE_DIALOG_ID).setParams({vnetId: resource_id});
        Sunstone.getDialog(RESERVE_DIALOG_ID).reset();
        Sunstone.getDialog(RESERVE_DIALOG_ID).show();
      }
    },

    "Network.reserve" : {
      type: "single",
      call: OpenNebulaResource.reserve,
      callback: function(req) {
        // Reset the wizard
        Sunstone.getDialog(RESERVE_DIALOG_ID).hide();
        Sunstone.getDialog(RESERVE_DIALOG_ID).reset();

        OpenNebulaAction.clear_cache("VNET");
        Sunstone.runAction("Network.show",req.request.data[0][0]);
      },
      error: Notifier.onError
    },

    "Network.addtocluster" : {
      type: "multiple",
      call: function(params){
        var cluster = params.data.extra_param;
        var vnet = params.data.id;

        if (cluster == -1){
          OpenNebulaResource.show({
            data : {
              id: vnet
            },
            success: function (request, vn){
              var vn_info = vn.VNET;

              var current_cluster = vn_info.CLUSTER_ID;

              if(current_cluster != -1){
                OpenNebulaCluster.delvnet({
                  data: {
                    id: current_cluster,
                    extra_param: vnet
                  },
                  success: function(){
                    OpenNebulaAction.clear_cache("VNET");
                    Sunstone.runAction('Network.show',vnet);
                  },
                  error: Notifier.onError
                });
              } else {
                OpenNebulaAction.clear_cache("VNET");
                Sunstone.runAction('Network.show',vnet);
              }
            },
            error: Notifier.onError
          });
        } else {
          OpenNebulaCluster.addvnet({
            data: {
              id: cluster,
              extra_param: vnet
            },
            success: function(){
              OpenNebulaAction.clear_cache("VNET");
              Sunstone.runAction('Network.show',vnet);
            },
            error: Notifier.onError
          });
        }
      },
      elements: function() {
        return Sunstone.getDataTable(TAB_ID).elements();
      }
    }
  };

  return _actions;
});


/* START_TEMPLATE */
define('hbs!tabs/vnets-tab/dialogs/add-ar/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large max-height\" data-reveal >\n  <div class=\"reveal-body\">\n    <form data-abide=\"ajax\" id=\"add_ar_form\" action=\"\">\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <h3 class=\"subheader\" id=\"\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Network", {"name":"tr","hash":{},"data":data})))
    + " <span id=\"vnet_id\"/>\n            <br>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "New Address Range", {"name":"tr","hash":{},"data":data})))
    + "\n          </h3>\n        </div>\n      </div>\n      ";
  stack1 = ((helper = (helper = helpers.arTabHTML || (depth0 != null ? depth0.arTabHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"arTabHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      <div class=\"reveal-footer\">\n        <div class=\"form_buttons\">\n          <button class=\"button radius right success\" id=\"submit_ar_button\" type=\"submit\" value=\"Network.add_ar\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add", {"name":"tr","hash":{},"data":data})))
    + "\n          </button>\n          <button id=\"submit_ar_reset_button\" class=\"button secondary radius\" type=\"reset\" value=\"reset\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Reset", {"name":"tr","hash":{},"data":data})))
    + "\n          </button>\n        </div>\n      </div>\n      <a class=\"close-reveal-modal\">&#215;</a>\n    </form>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vnets-tab/dialogs/add-ar/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/vnets-tab/utils/ar-tab/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/advancedSection'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom attributes", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      <br>\n      <br>\n    </div>\n  </div>\n  <div class=\"row\" id=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_custom_tags\">\n    <div class=\"12 columns\">\n      ";
  stack1 = ((helper = (helper = helpers.customTagsHTML || (depth0 != null ? depth0.customTagsHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customTagsHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n    </div>\n  </div>\n  <div class=\"row\">\n    <br>\n    <br>\n    <hr>\n    <div class=\"large-12 columns\">\n      <span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Security Groups", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      <br>\n      <br>\n    </div>\n  </div>\n  <div class=\"row\" id=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_security_groups\">\n    ";
  stack1 = ((helper = (helper = helpers.securityGroupsTableHTML || (depth0 != null ? depth0.securityGroupsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"securityGroupsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n  </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row\" name=\"str_ar_tab_id\" str_ar_tab_id=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "\">\n  <div class=\"large-12 columns\">\n    <input wizard_field=\"TYPE\" type=\"radio\" name=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ar_type\" id=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ar_type_ip4\" value=\"IP4\"/><label for=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ar_type_ip4\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPv4", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input wizard_field=\"TYPE\" type=\"radio\" name=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ar_type\" id=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ar_type_ip4_6\" value=\"IP4_6\"/><label for=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ar_type_ip4_6\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPv4/6", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input wizard_field=\"TYPE\" type=\"radio\" name=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ar_type\" id=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ar_type_ip6\" value=\"IP6\"/><label for=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ar_type_ip6\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPv6", {"name":"tr","hash":{},"data":data})))
    + "</label>\n    <input wizard_field=\"TYPE\" type=\"radio\" name=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ar_type\" id=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ar_type_ether\" value=\"ETHER\"/><label for=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ar_type_ether\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ethernet", {"name":"tr","hash":{},"data":data})))
    + "</label>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"medium-6 columns\">\n    <div class=\"row collapse ar_input type_ip4 type_ip4_6\">\n      <label for=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ip_start\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IP Start", {"name":"tr","hash":{},"data":data})))
    + ":\n        <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "First IP address", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </label>\n      <input wizard_field=\"IP\" type=\"text\" name=\"IP\" id=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ip_start\"/>\n    </div>\n    <div class=\"row collapse ar_input type_ip4 type_ip4_6 type_ip6 type_ether\">\n      <label for=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_mac_start\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "MAC Start", {"name":"tr","hash":{},"data":data})))
    + ":\n        <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "First MAC address", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </label>\n      <input wizard_field=\"MAC\" type=\"text\" name=\"MAC\" id=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_mac_start\" />\n    </div>\n  </div>\n  <div class=\"medium-6 columns ar_input type_ip4 type_ip4_6 type_ip6 type_ether\">\n    <label for=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_size\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Size", {"name":"tr","hash":{},"data":data})))
    + ":\n      <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of addresses in the range", {"name":"tr","hash":{},"data":data})))
    + "</span>\n    </label>\n    <input wizard_field=\"SIZE\" required type=\"text\" name=\"SIZE\" id=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_size\" />\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"medium-6 columns ar_input type_ip4_6 type_ip6\">\n    <label for=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_global_prefix\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Global prefix", {"name":"tr","hash":{},"data":data})))
    + ":\n      <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPv6 global address prefix to create leases", {"name":"tr","hash":{},"data":data})))
    + "</span>\n    </label>\n    <input wizard_field=\"GLOBAL_PREFIX\" type=\"text\" name=\"GLOBAL_PREFIX\" id=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_global_prefix\"/>\n  </div>\n  <div class=\"medium-6 columns ar_input type_ip4_6 type_ip6\">\n    <label for=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ula_prefix\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ULA prefix", {"name":"tr","hash":{},"data":data})))
    + ":\n      <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPv6 unique local address (ULA) prefix to create leases", {"name":"tr","hash":{},"data":data})))
    + "</span>\n    </label>\n    <input wizard_field=\"ULA_PREFIX\" type=\"text\" name=\"ULA_PREFIX\" id=\""
    + escapeExpression(((helper = (helper = helpers.str_ar_tab_id || (depth0 != null ? depth0.str_ar_tab_id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"str_ar_tab_id","hash":{},"data":data}) : helper)))
    + "_ula_prefix\"/>\n  </div>\n</div>\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced Options", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"useData":true});
Handlebars.registerPartial('tabs/vnets-tab/utils/ar-tab/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vnets-tab/utils/ar-tab',['require','utils/locale','utils/tips','utils/custom-tags-table','utils/wizard-fields','tabs/secgroups-tab/datatable','hbs!./ar-tab/html'],function(require) {
  // Dependencies
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var CustomTagsTable = require('utils/custom-tags-table');
  var WizardFields = require('utils/wizard-fields');
  var SecurityGroupsTable = require('tabs/secgroups-tab/datatable');

  var TemplateHTML = require('hbs!./ar-tab/html');

  function ArTab() {
    return this;
  }

  ArTab.prototype = {
    'html': _generate_ar_tab_content,
    'setup': _setup_ar_tab_content,
    'onShow': _onShow,
    'fill': _fill_ar_tab_data,
    'retrieve': _retrieve_ar_tab_data
  };
  ArTab.prototype.constructor = ArTab;

  return ArTab;

  function _generate_ar_tab_content(str_ar_tab_id){
    var opts = {
      info: false,
      select: true,
      selectOptions: {"multiple_choice": true}
    };

    this.securityGroupsTable = new SecurityGroupsTable(str_ar_tab_id, opts);

    return TemplateHTML({
          'str_ar_tab_id': str_ar_tab_id,
          'customTagsHTML': CustomTagsTable.html(),
          'securityGroupsTableHTML': this.securityGroupsTable.dataTableHTML
        });
  }

  function _setup_ar_tab_content(ar_section, str_ar_tab_id) {

    this.ar_section = ar_section;

    $('input[name$="ar_type"]',ar_section).change(function(){
      $('div.ar_input', ar_section).hide();

      $('input[wizard_field="IP"]',ar_section).removeAttr('required');

      switch($(this).val()){
      case "IP4":
        $('div.type_ip4', ar_section).show();
        $('input[wizard_field="IP"]',ar_section).attr('required', '');

        break;
      case "IP4_6":
        $('div.type_ip4_6', ar_section).show();
        $('input[wizard_field="IP"]',ar_section).attr('required', '');

        break;
      case "IP6":
        $('div.type_ip6', ar_section).show();
        break;
      case "ETHER":
        $('div.type_ether', ar_section).show();
        break;
      }
    });

    $('input#'+str_ar_tab_id+'_ar_type_ip4',ar_section).prop('checked', true);
    $('input#'+str_ar_tab_id+'_ar_type_ip4',ar_section).change();

    CustomTagsTable.setup($('#'+str_ar_tab_id+'_custom_tags',ar_section));

    this.securityGroupsTable.initialize();

    Tips.setup(ar_section);
  }

  function _onShow(){
    this.securityGroupsTable.refreshResourceTableSelect();
  }

  function _retrieve_ar_tab_data(){
    var data  = {};

    var ar_type = $('input[name$="ar_type"]:checked',this.ar_section).val();

    var fields = [];

    switch(ar_type){
      case "IP4":
        fields = $('div.type_ip4', this.ar_section).children("input");
        break;
      case "IP4_6":
        fields = $('div.type_ip4_6', this.ar_section).children("input");
        break;
      case "IP6":
        fields = $('div.type_ip6', this.ar_section).children("input");
        break;
      case "ETHER":
        fields = $('div.type_ether', this.ar_section).children("input");
        break;
    }

    fields.each(function(){
      var field=$(this);

      if (field.val() != null && field.val().length){ //if has a length
        data[field.attr('name')] = field.val();
      }

    });

    if (!$.isEmptyObject(data)) {
      data["TYPE"] = ar_type;
    }

    $.extend(data, CustomTagsTable.retrieve(this.ar_section));

    var str_ar_tab_id = $('div[name="str_ar_tab_id"]', this.ar_section).attr("str_ar_tab_id");

    var secgroups = this.securityGroupsTable.retrieveResourceTableSelect();
    if (secgroups != undefined && secgroups.length != 0){
      data["SECURITY_GROUPS"] = secgroups.join(",");
    }

    return data;
  }

  function _fill_ar_tab_data(ar_json){
    WizardFields.fill(this.ar_section, ar_json);

    var fields = $('[wizard_field]',this.ar_section);

    fields.each(function(){
      var field = $(this);
      var field_name = field.attr('wizard_field');

      // Delete so these attributes don't end in the custom tags table also
      delete ar_json[field_name];
    });

    delete ar_json["AR_ID"];
    delete ar_json["USED_LEASES"];
    delete ar_json["LEASES"];
    delete ar_json["MAC_END"];
    delete ar_json["IP_END"];
    delete ar_json["IP6_ULA"];
    delete ar_json["IP6_ULA_END"];
    delete ar_json["IP6_GLOBAL"];
    delete ar_json["IP6_GLOBAL_END"];

    if (ar_json["SECURITY_GROUPS"] != undefined &&
        ar_json["SECURITY_GROUPS"].length != 0){

      var secgroups = ar_json["SECURITY_GROUPS"].split(",");

      this.securityGroupsTable.selectResourceTableSelect({ids: secgroups});
    }

    delete ar_json["SECURITY_GROUPS"];

    CustomTagsTable.fill(this.ar_section, ar_json);

    $('input[name$="ar_type"]',this.ar_section).prop("disabled", true);
    $('input[wizard_field="IP"]',this.ar_section).prop("disabled", true);
    $('input[wizard_field="MAC"]',this.ar_section).prop("disabled", true);
  }
});
define('tabs/vnets-tab/dialogs/add-ar',['require','utils/dialogs/dialog','hbs!./add-ar/html','tabs/vnets-tab/utils/ar-tab','sunstone','utils/notifier','utils/locale','./add-ar/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./add-ar/html');
  var ArTab = require('tabs/vnets-tab/utils/ar-tab');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./add-ar/dialogId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    this.arTab = new ArTab();

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setId = _setId;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'arTabHTML': this.arTab.html("add_ar")
    });
  }

  function _setup(context) {
    var that = this;
    context.foundation('abide', 'reflow');
    
    that.arTab.setup(context, "add_ar");

    $('#submit_ar_reset_button', context).click(function(){
      Sunstone.getDialog(DIALOG_ID).reset();
      Sunstone.getDialog(DIALOG_ID).show();
    });

    $('#add_ar_form',context).on('invalid.fndtn.abide', function () {
        Notifier.notifyError(Locale.tr("One or more required fields are missing."));
    }).on('valid.fndtn.abide', function () {
        var data = that.arTab.retrieve();

        var obj = {AR: data};
        Sunstone.runAction('Network.add_ar', that.vnetId, obj);

        return false;
    });
  }

  function _onShow(context) {
    this.arTab.onShow();
    $('#vnet_id', this.dialogElement).text(this.vnetId);
  }

  function _setId(id) {
    this.vnetId = id;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vnets-tab/dialogs/update-ar/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large max-height\" data-reveal >\n  <div class=\"reveal-body\">\n    <form data-abide=\"ajax\" id=\"update_ar_form\" action=\"\">\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <h3 class=\"subheader\" id=\"\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Network", {"name":"tr","hash":{},"data":data})))
    + " <span id=\"vnet_id\"/>\n            <br>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Edit Address Range", {"name":"tr","hash":{},"data":data})))
    + " <span id=\"ar_id\"/>\n          </h3>\n        </div>\n      </div>\n      ";
  stack1 = ((helper = (helper = helpers.arTabHTML || (depth0 != null ? depth0.arTabHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"arTabHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      <div class=\"reveal-footer\">\n        <div class=\"form_buttons\">\n          <button class=\"button radius right success\" id=\"submit_ar_button\" type=\"submit\" value=\"Network.update_ar\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update", {"name":"tr","hash":{},"data":data})))
    + "\n          </button>\n        </div>\n      </div>\n      <a class=\"close-reveal-modal\">&#215;</a>\n    </form>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vnets-tab/dialogs/update-ar/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vnets-tab/dialogs/update-ar',['require','utils/dialogs/dialog','hbs!./update-ar/html','tabs/vnets-tab/utils/ar-tab','sunstone','utils/notifier','utils/locale','./update-ar/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./update-ar/html');
  var ArTab = require('tabs/vnets-tab/utils/ar-tab');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./update-ar/dialogId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    this.arTab = new ArTab();

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setParams = _setParams;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'arTabHTML': this.arTab.html("update_ar")
    });
  }

  function _setup(context) {
    var that = this;
    context.foundation('abide', 'reflow');
    
    that.arTab.setup(context, "update_ar");

    $('#update_ar_form',context).on('invalid.fndtn.abide', function () {
        Notifier.notifyError(Locale.tr("One or more required fields are missing."));
    }).on('valid.fndtn.abide', function () {
        var data = that.arTab.retrieve();

        data['AR_ID'] = that.arId;

        var obj = {AR: data};
        Sunstone.runAction('Network.update_ar', that.vnetId, obj);

        return false;
    });
  }

  function _onShow(context) {
    this.arTab.onShow();
  }

  function _setParams(params) {
    this.vnetId = params.vnetId;
    this.arId = params.arId;

    $('#vnet_id', this.dialogElement).text(params.vnetId);
    $('#ar_id', this.dialogElement).text(params.arId);
    this.arTab.fill(params.arData);
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vnets-tab/dialogs/reserve/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/advancedSection'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "        <div class=\"content\" id=\"advanced_reserve\">\n          <div class=\"row\">\n            <div class=\"large-12 columns\">\n              <p>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "You can select the addresses from an specific Address Range", {"name":"tr","hash":{},"data":data})))
    + "</p>\n            </div>\n          </div>\n          <div class=\"row\">\n            <div class=\"large-12 columns\">\n              ";
  stack1 = ((helper = (helper = helpers.arsTableHTML || (depth0 != null ? depth0.arsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"arsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n            </div>\n          </div>\n          <br>\n          <div class=\"row\">\n            <div class=\"large-6 columns\">\n              <label for=\"reserve_addr\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "First address", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n              <input wizard_field=\"addr\" type=\"text\" id=\"reserve_addr\" placeholder=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPv4 or MAC", {"name":"tr","hash":{},"data":data})))
    + "\"/>\n            </div>\n          </div>\n        </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal large max-height\" data-reveal >\n  <div class=\"reveal-body\">\n    <form id=\"reserve_form\" action=\"\">\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <h3 class=\"subheader\" id=\"\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Reservation from Virtual Network", {"name":"tr","hash":{},"data":data})))
    + " <span id=\"vnet_id\"/>\n          </h3>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"large-6 columns\">\n          <label for=\"reserve_size\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of addresses", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n          <input wizard_field=\"size\" type=\"text\" id=\"reserve_size\"/>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"large-12 columns\">\n          <input type=\"radio\" name=\"reserve_target\" id=\"reserve_new\" value=\"NEW\"/>\n          <label for=\"reserve_new\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add to a new Virtual Network", {"name":"tr","hash":{},"data":data})))
    + "\n          </label>\n          <input type=\"radio\" name=\"reserve_target\" id=\"reserve_add\" value=\"ADD\"/>\n          <label for=\"reserve_add\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add to an existing Reservation", {"name":"tr","hash":{},"data":data})))
    + "\n          </label>\n        </div>\n      </div>\n      <div id=\"reserve_new_body\">\n        <div class=\"row\">\n          <div class=\"large-6 columns\">\n            <label for=\"reserve_name\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Network Name", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n            <input wizard_field=\"name\" type=\"text\" id=\"reserve_name\"/>\n          </div>\n        </div>\n      </div>\n      <div id=\"reserve_add_body\">\n        ";
  stack1 = ((helper = (helper = helpers.vnetsTableHTML || (depth0 != null ? depth0.vnetsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"vnetsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </div>\n      <br>\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced Options", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "      <div class=\"reveal-footer\">\n        <div class=\"form_buttons\">\n          <button class=\"button radius right success\" id=\"submit_reserve_button\" type=\"submit\" value=\"Network.reserve\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Reserve", {"name":"tr","hash":{},"data":data})))
    + "\n          </button>\n        </div>\n      </div>\n      <a class=\"close-reveal-modal\">&#215;</a>\n    </form>\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/vnets-tab/dialogs/reserve/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vnets-tab/dialogs/reserve/ar-datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','opennebula/network','tabs/vnets-tab/utils/common','utils/notifier','utils/progress-bar'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');
  var OpenNebulaNetwork = require('opennebula/network');
  var Utils = require('tabs/vnets-tab/utils/common');
  var Notifier = require('utils/notifier');
  var ProgressBar = require('utils/progress-bar');

  /*
    CONSTRUCTOR
   */

  /**
   * Constructor
   * @param {string} dataTableId
   * @param {object} conf        Same options as TabDatatable constructor, plus
   *                             conf.vnetId
   */
  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.dataTableId = dataTableId;

    //this.tabId = TAB_NAME;
    //this.resource = RESOURCE;
    //this.xmlRoot = XML_ROOT;

    this.vnetId = conf.vnetId;

    this.dataTableOptions = {
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
        //{ "bSortable": false, "aTargets": [3,4] },
      ]
    };

    this.columns = [
      Locale.tr("Address Range"),
      Locale.tr("Type"),
      Locale.tr("Start"),
      Locale.tr("IPv6 Prefix"),
      Locale.tr("Leases")
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 1,
      "select_resource": Locale.tr("Please select an Address Range from the list"),
      "you_selected": Locale.tr("You selected the following Address Range:")
    };

    TabDataTable.call(this);
  }

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;
  Table.prototype.updateFn = _updateFn;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _updateFn() {
    var that = this;

    OpenNebulaNetwork.show({
      data : {
        id: that.vnetId
      },
      timeout: true,
      success: function (request, vn){
        var ar_list_array = [];

        var ar_list = Utils.getARList(vn.VNET);

        $.each(ar_list, function(){
          var ar = this;
          var id = ar.AR_ID;

          var start;

          if(ar.TYPE == "IP4" || ar.TYPE == "IP4_6"){
            start = (ar.IP ? ar.IP : "--");
          } else {
            start = (ar.MAC ? ar.MAC : "--");
          }

          var prefix = "";

          if(ar.GLOBAL_PREFIX && ar.ULA_PREFIX){
            prefix += ar.GLOBAL_PREFIX + "<br>" + ar.ULA_PREFIX;
          } else if (ar.GLOBAL_PREFIX){
            prefix += ar.GLOBAL_PREFIX;
          } else if (ar.ULA_PREFIX){
            prefix += ar.ULA_PREFIX;
          } else {
            prefix = "--";
          }

          ar_list_array.push([
            null,
            id,
            (ar.TYPE ? ar.TYPE : "--"),
            start,
            prefix,
            ProgressBar.html(ar.USED_LEASES, ar.SIZE)
            ]);
        });

        that.updateView(null, ar_list_array, true);
      },
      error: Notifier.onError
    });
  }

  function _elementArray(element_json) {
    return [];
  }
});

define('tabs/vnets-tab/dialogs/reserve',['require','utils/dialogs/dialog','hbs!./reserve/html','sunstone','utils/locale','utils/tips','utils/wizard-fields','tabs/vnets-tab/datatable','./reserve/ar-datatable','./reserve/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./reserve/html');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var WizardFields = require('utils/wizard-fields');
  var VNetsTable = require('tabs/vnets-tab/datatable');
  var ARsTable = require('./reserve/ar-datatable');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./reserve/dialogId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    var that = this;

    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setParams = _setParams;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var that = this;

    var opts = {
      info: false,
      select: true,
      selectOptions: {
        filter_fn: function(vnet){
          return (vnet['PARENT_NETWORK_ID'] == that.vnetId);
        }
      }
    };

    this.vnetsTable = new VNetsTable("reserve", opts);

    var arOpts = {
      info: false,
      select: true,
      vnetId: this.vnetId
    };

    this.arsTable = new ARsTable("ar_reserve", arOpts);

    return TemplateHTML({
      'vnetsTableHTML': this.vnetsTable.dataTableHTML,
      'arsTableHTML': this.arsTable.dataTableHTML
    });
  }

  function _setup(context) {
    var that = this;

    $('input[name="reserve_target"]',context).change(function(){
      $('div#reserve_new_body', context).hide();
      $('div#reserve_add_body', context).hide();

      $('input', $('div#reserve_new_body', context)).prop('wizard_field_disabled', true);
      $('input', $('div#reserve_add_body', context)).prop('wizard_field_disabled', true);

      switch($(this).val()){
        case "NEW":
          $('div#reserve_new_body', context).show();
          $('input', $('div#reserve_new_body', context)).prop('wizard_field_disabled', false);
          break;
        case "ADD":
          $('div#reserve_add_body', context).show();
          $('input', $('div#reserve_add_body', context)).prop('wizard_field_disabled', false);
          break;
      }
    });

    $('input#reserve_new', context).prop('checked', true);
    $('input#reserve_new', context).change();

    that.vnetsTable.idInput().attr("wizard_field", "vnet");
    that.arsTable.idInput().attr("wizard_field", "ar_id");

    this.vnetsTable.initialize();
    this.arsTable.initialize();

    Tips.setup(context);

    $('#reserve_form',context).submit(function(){
      var data = WizardFields.retrieve(context);

      Sunstone.runAction('Network.reserve', that.vnetId, data);

      return false;
    });
  }

  function _onShow(context) {
    this.vnetsTable.refreshResourceTableSelect();
    this.arsTable.refreshResourceTableSelect();
  }

  /**
   * [_setParams description]
   * @param {object} params
   *        - params.vnetId : Virtual Network id
   */
  function _setParams(params) {
    this.vnetId = params.vnetId;

    $('#vnet_id', this.dialogElement).text(params.vnetId);
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vnets-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table id=\"info_vnet_table\" class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n          <td></td>\n        </tr>\n        ";
  stack1 = ((helper = (helper = helpers.renameTrHTML || (depth0 != null ? depth0.renameTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"renameTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        <tr>\n          ";
  stack1 = ((helper = (helper = helpers.clusterTrHTML || (depth0 != null ? depth0.clusterTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"clusterTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        </tr>\n        ";
  stack1 = ((helper = (helper = helpers.reservationTrHTML || (depth0 != null ? depth0.reservationTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"reservationTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      <tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\">\n    ";
  stack1 = ((helper = (helper = helpers.permissionsTableHTML || (depth0 != null ? depth0.permissionsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"permissionsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-9 columns\">\n    ";
  stack1 = ((helper = (helper = helpers.templateTableHTML || (depth0 != null ? depth0.templateTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templateTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n  </div>\n</div>\n";
},"useData":true});
Handlebars.registerPartial('tabs/vnets-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vnets-tab/panels/info/panelId',['require'],function(require){
  return 'vnet_info_tab';
});
define('tabs/vnets-tab/panels/info',['require','hbs!./info/html','utils/locale','utils/panel/permissions-table','utils/panel/cluster-tr','utils/panel/rename-tr','opennebula/network','utils/panel/template-table','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TemplateInfo = require('hbs!./info/html');
  var Locale = require('utils/locale');
  var PermissionsTable = require('utils/panel/permissions-table');
  var ClusterTr = require('utils/panel/cluster-tr');
  var RenameTr = require('utils/panel/rename-tr');
  var OpenNebulaNetwork = require('opennebula/network');

  /*
    TEMPLATES
   */

  var TemplateTable = require('utils/panel/template-table');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "Network";
  var XML_ROOT = "VNET";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var renameTrHTML = RenameTr.html(RESOURCE, this.element.NAME);
    var clusterTrHTML = ClusterTr.html(this.element.CLUSTER);
    var permissionsTableHTML = PermissionsTable.html(TAB_ID, RESOURCE, this.element);

    // TODO: simplify interface?
    var strippedTemplate = $.extend({}, this.element.TEMPLATE);
    delete strippedTemplate["SECURITY_GROUPS"];

    var templateTableHTML = TemplateTable.html(strippedTemplate, RESOURCE,
                                              Locale.tr("Attributes"));
    //====

    // TODO: move to util?
    var reservationTrHTML = '';

    if(this.element.PARENT_NETWORK_ID.length > 0){
      reservationTrHTML =
        '<tr>\
          <td class="key_td">'+Locale.tr("Reservation parent")+'</td>\
          <td class="value_td">'+this.element.PARENT_NETWORK_ID+'</td>\
          <td></td>\
        </tr>';

      $(".reserve-right-info").prop("disabled", true);
      $(".reserve-right-info").addClass("has-tip");
      $(".reserve-right-info").attr("title", Locale.tr("This Network is already a reservation"));
    } else{
      $(".reserve-right-info").prop("disabled", false);
      $(".reserve-right-info").removeClass("has-tip");
      $(".reserve-right-info").removeAttr("title");
    }
    //====

    return TemplateInfo({
      'element': this.element,
      'renameTrHTML': renameTrHTML,
      'clusterTrHTML': clusterTrHTML,
      'reservationTrHTML': reservationTrHTML,
      'permissionsTableHTML': permissionsTableHTML,
      'templateTableHTML': templateTableHTML
    });
  }

  function _setup(context) {
    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);
    RenameTr.setup(RESOURCE, this.element.ID, context);
    ClusterTr.setup(RESOURCE, this.element.ID, this.element.CLUSTER_ID, context);
    PermissionsTable.setup(TAB_ID, RESOURCE, this.element, context);

    // TODO: simplify interface?
    var strippedTemplate = $.extend({}, this.element.TEMPLATE);
    delete strippedTemplate["SECURITY_GROUPS"];

    var hiddenValues = {};

    if (this.element.TEMPLATE.SECURITY_GROUPS != undefined){
        hiddenValues.SECURITY_GROUPS = this.element.TEMPLATE.SECURITY_GROUPS;
    }

    TemplateTable.setup(strippedTemplate, RESOURCE, this.element.ID, context, hiddenValues);
    //===

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vnets-tab/panels/ar/html',['hbs','hbs/handlebars','templates/helpers/isTabActionEnabled','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "        <button class=\"button success small radius\" id=\"add_ar_button\">\n          <span class=\"fa fa-plus\"></span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add", {"name":"tr","hash":{},"data":data})))
    + "\n        </button>\n";
},"3":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "          <button class=\"button secondary small radius\" id=\"update_ar_button\" ar_id=\"\" disabled=\"disabled\">\n            <span class=\"fa fa-pencil-square-o\"></span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Update", {"name":"tr","hash":{},"data":data})))
    + "\n          </button>\n";
},"5":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "          <button class=\"button secondary small radius\" id=\"rm_ar_button\" ar_id=\"\" disabled=\"disabled\">\n            <span class=\"fa fa-trash-o\"></span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Remove", {"name":"tr","hash":{},"data":data})))
    + "\n          </button>\n";
},"7":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "            <tr ar=\""
    + escapeExpression(((helper = (helper = helpers.id || (depth0 != null ? depth0.id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"id","hash":{},"data":data}) : helper)))
    + "\">\n              <td style=\"white-space: nowrap\" class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.id || (depth0 != null ? depth0.id : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"id","hash":{},"data":data}) : helper)))
    + "</td>\n              <td style=\"white-space: nowrap\" class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.type || (depth0 != null ? depth0.type : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"type","hash":{},"data":data}) : helper)))
    + "</td>\n              <td style=\"white-space: nowrap\" class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.start || (depth0 != null ? depth0.start : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"start","hash":{},"data":data}) : helper)))
    + "</td>\n              <td style=\"white-space: nowrap\" class=\"value_td\">";
  stack1 = ((helper = (helper = helpers.prefixHTML || (depth0 != null ? depth0.prefixHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"prefixHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "</td>\n              <td style=\"white-space: nowrap\" class=\"value_td\">";
  stack1 = ((helper = (helper = helpers.leasesHTML || (depth0 != null ? depth0.leasesHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"leasesHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</td>\n            </tr>\n";
},"9":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "      <div class=\"text-center\">\n        <span class=\"fa-stack fa-5x\" style=\"color: #dfdfdf\">\n          <i class=\"fa fa-cloud fa-stack-2x\"></i>\n          <i class=\"fa fa-info-circle fa-stack-1x fa-inverse\"></i>\n        </span>\n        <br>\n        <br>\n        <span style=\"font-size: 18px; color: #999\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select an Address Range to see more information", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form id=\"ar_list_form\">\n  <div class=\"row collapse\">\n    <div class=\"large-12 columns\">\n";
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "vnets-tab", "Network.add_ar", {"name":"isTabActionEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "      <span class=\"right\">\n";
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "vnets-tab", "Network.update_ar", {"name":"isTabActionEnabled","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "vnets-tab", "Network.remove_ar", {"name":"isTabActionEnabled","hash":{},"fn":this.program(5, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "      </span>\n    </div>\n  </div>\n  <div class=\"row collapse\">\n    <div class=\"large-12 columns\" style=\"overflow:auto\">\n      <table id=\"ar_list_datatable\" class=\"datatable twelve\">\n        <thead>\n          <tr>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Address Range", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Start", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPv6 Prefix", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Leases", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          </tr>\n        </thead>\n        <tbody>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.arList : depth0), {"name":"each","hash":{},"fn":this.program(7, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "        </tbody>\n      </table>\n    </div>\n  </div>\n  <div id=\"ar_show_info\">\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.arList : depth0), {"name":"if","hash":{},"fn":this.program(9, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/vnets-tab/panels/ar/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/vnets-tab/panels/ar/arInfo',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, buffer = "";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.value : depth0), {"name":"if","hash":{},"fn":this.program(2, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"2":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "            <tr>\n              <td class=\"key_td\">"
    + escapeExpression(((helper = (helper = helpers.key || (depth0 != null ? depth0.key : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"key","hash":{},"data":data}) : helper)))
    + "</td>\n              <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.value || (depth0 != null ? depth0.value : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"value","hash":{},"data":data}) : helper)))
    + "</td>\n            </tr>\n";
},"4":function(depth0,helpers,partials,data) {
  var lambda=this.lambda, escapeExpression=this.escapeExpression;
  return "          <tr>\n           <td class=\"key_td\">"
    + escapeExpression(lambda((data && data.key), depth0))
    + "</td>\n           <td class=\"value_td\">"
    + escapeExpression(lambda(depth0, depth0))
    + "</td>\n         </tr>\n";
},"6":function(depth0,helpers,partials,data) {
  var helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function";
  return "          <tr>\n            <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IP", {"name":"tr","hash":{},"data":data})))
    + "</td>\n            <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.first_ip || (depth0 != null ? depth0.first_ip : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"first_ip","hash":{},"data":data}) : helper)))
    + "</td>\n            <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.last_ip || (depth0 != null ? depth0.last_ip : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"last_ip","hash":{},"data":data}) : helper)))
    + "</td>\n          </tr>\n";
},"8":function(depth0,helpers,partials,data) {
  var helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function";
  return "          <tr>\n            <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IP6_GLOBAL", {"name":"tr","hash":{},"data":data})))
    + "</td>\n            <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.first_ip6_global || (depth0 != null ? depth0.first_ip6_global : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"first_ip6_global","hash":{},"data":data}) : helper)))
    + "</td>\n            <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.last_ip6_global || (depth0 != null ? depth0.last_ip6_global : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"last_ip6_global","hash":{},"data":data}) : helper)))
    + "</td>\n          </tr>\n";
},"10":function(depth0,helpers,partials,data) {
  var helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function";
  return "          <tr>\n            <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IP6_ULA", {"name":"tr","hash":{},"data":data})))
    + "</td>\n            <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.first_ip6_ula || (depth0 != null ? depth0.first_ip6_ula : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"first_ip6_ula","hash":{},"data":data}) : helper)))
    + "</td>\n            <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.last_ip6_ula || (depth0 != null ? depth0.last_ip6_ula : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"last_ip6_ula","hash":{},"data":data}) : helper)))
    + "</td>\n          </tr>\n";
},"12":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "  <div class=\"row collapse\">\n    <div class=\"large-12 columns\">\n      <table class=\"dataTable extended_table\">\n        <thead>\n          <tr><th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Security Groups", {"name":"tr","hash":{},"data":data})))
    + "</th></tr>\n        </thead>\n        <tbody/>\n      </table>\n    </div>\n    <div class=\"large-12 columns\">\n      ";
  stack1 = ((helper = (helper = helpers.secgroupTableHTML || (depth0 != null ? depth0.secgroupTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"secgroupTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </div>\n  </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "<div class=\"row collapse\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr><th colspan=\"2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Address Range", {"name":"tr","hash":{},"data":data})))
    + " "
    + escapeExpression(((helper = (helper = helpers.arId || (depth0 != null ? depth0.arId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"arId","hash":{},"data":data}) : helper)))
    + "</th></tr>\n      </thead>\n      <tbody>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.arKnownAttr : depth0), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "      </tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr><th colspan=\"2\">&nbsp;</th></tr>\n      </thead>\n      <tbody>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.arExtraAttr : depth0), {"name":"each","hash":{},"fn":this.program(4, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "      </tbody>\n    </table>\n  </div>\n</div>\n<div class=\"row collapse\">\n  <div class=\"large-12 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Range", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "First", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Last", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "MAC", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.first_mac || (depth0 != null ? depth0.first_mac : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"first_mac","hash":{},"data":data}) : helper)))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.last_mac || (depth0 != null ? depth0.last_mac : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"last_mac","hash":{},"data":data}) : helper)))
    + "</td>\n        </tr>\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.first_ip : depth0), {"name":"if","hash":{},"fn":this.program(6, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.first_ip6_global : depth0), {"name":"if","hash":{},"fn":this.program(8, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.first_ip6_ula : depth0), {"name":"if","hash":{},"fn":this.program(10, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "      </tbody>\n    </table>\n  </div>\n</div>\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.secgroupTableHTML : depth0), {"name":"if","hash":{},"fn":this.program(12, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"useData":true});
Handlebars.registerPartial('tabs/vnets-tab/panels/ar/arInfo', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vnets-tab/panels/ar/panelId',['require'],function(require){
  return 'vnet_ar_list_tab';
});
define('tabs/vnets-tab/panels/ar',['require','hbs!./ar/html','hbs!./ar/arInfo','utils/locale','utils/progress-bar','sunstone-config','../utils/common','tabs/secgroups-tab/datatable','sunstone','opennebula/network','utils/notifier','../tabId','./ar/panelId','../dialogs/add-ar/dialogId','../dialogs/update-ar/dialogId','utils/dialogs/generic-confirm/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TemplateAR = require('hbs!./ar/html');
  var TemplateARInfo = require('hbs!./ar/arInfo');
  var Locale = require('utils/locale');
  var ProgressBar = require('utils/progress-bar');
  var Config = require('sunstone-config');
  var Utils = require('../utils/common');
  var SecurityGroupsTable = require('tabs/secgroups-tab/datatable');
  var Sunstone = require('sunstone');
  var OpenNebulaNetwork = require('opennebula/network');
  var Notifier = require('utils/notifier');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./ar/panelId');
  var SG_TABLE_ID = PANEL_ID + "SecurityGroupsTable";
  var RESOURCE = "Network";
  var XML_ROOT = "VNET";

  var ADD_AR_DIALOG_ID = require('../dialogs/add-ar/dialogId');
  var UPDATE_AR_DIALOG_ID = require('../dialogs/update-ar/dialogId');
  var CONFIRM_DIALOG_ID = require('utils/dialogs/generic-confirm/dialogId');

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Addresses");
    this.icon = "fa-align-justify";

    this.element = info[XML_ROOT];

    this.last_selected_row_ar = "";

    this.secgroupTable = undefined;

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;
  Panel.prototype.getState = _getState;
  Panel.prototype.setState = _setState;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var arList = Utils.getARList(this.element);
    var processedARList = [];

    for (var i=0; i<arList.length; i++){
      var ar = arList[i];
      var id = ar.AR_ID;

      var type = (ar.TYPE ? ar.TYPE : "--");

      var start = "";

      if(ar.TYPE == "IP4" || ar.TYPE == "IP4_6"){
        start = (ar.IP ? ar.IP : "--");
      } else {
        start = (ar.MAC ? ar.MAC : "--");
      }

      var prefix = "";

      if(ar.GLOBAL_PREFIX && ar.ULA_PREFIX){
        prefix += ar.GLOBAL_PREFIX + "<br>" + ar.ULA_PREFIX;
      } else if (ar.GLOBAL_PREFIX){
        prefix += ar.GLOBAL_PREFIX;
      } else if (ar.ULA_PREFIX){
        prefix += ar.ULA_PREFIX;
      } else {
        prefix = "--";
      }

      var leases = ProgressBar.html(ar.USED_LEASES, ar.SIZE);

      processedARList.push({
        "id" : id,
        "type" : type,
        "start" : start,
        "prefixHTML" : prefix,
        "leasesHTML" : leases
      });
    }

    return TemplateAR({
      'element': this.element,
      'arList' : processedARList
    });
  }

  function _setup(context) {

    var that = this;

    var ar_list_dataTable = $("#ar_list_datatable", context).dataTable({
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
        //{ "bSortable": false, "aTargets": [3,4] },
      ]
    });

    // TODO: should be a method for sunstone-config?
    ar_list_dataTable.fnSort( [ [0,config['user_config']['table_order']] ] );

    ar_list_dataTable.off("click", 'tbody tr');
    ar_list_dataTable.on("click", 'tbody tr', function(e){
      var aData = ar_list_dataTable.fnGetData(this);
      if (!aData) return true;
      var id = aData[0];
      if (!id) return true;

      if(that.last_selected_row_ar) {
        that.last_selected_row_ar.children().each(function(){
          $(this).removeClass('markrowchecked');
        });
      }

      that.last_selected_row_ar = $(this);
      $(this).children().each(function(){
        $(this).addClass('markrowchecked');
      });

      $("#update_ar_button", context).attr("ar_id", id);
      $("#update_ar_button", context).prop("disabled", false);

      $("#rm_ar_button", context).attr("ar_id", id).removeAttr('disabled');

      $("#ar_show_info", context).html(_arHTML(that.element, id));

      _arSetup($("#ar_show_info", context), that.element, id);

      return false;
    });


    if (Config.isTabActionEnabled("vnets-tab", "Network.remove_ar")) {
      context.off("click", 'button#rm_ar_button');
      context.on("click", 'button#rm_ar_button', function(){
        var ar_id = $(this).attr('ar_id');

        Sunstone.getDialog(CONFIRM_DIALOG_ID).setParams({
          //header :
          body : Locale.tr("This will delete all the addresses in this range"),
          //question :
          submit : function(){
            var obj = {ar_id: ar_id};
            Sunstone.runAction('Network.rm_ar',that.element.ID,obj);

            return false;
          }
        });

        Sunstone.getDialog(CONFIRM_DIALOG_ID).reset();
        Sunstone.getDialog(CONFIRM_DIALOG_ID).show();

        return false;
      });
    }

    if (Config.isTabActionEnabled("vnets-tab", "Network.add_ar")) {
      context.off("click", 'button#add_ar_button');
      context.on("click", 'button#add_ar_button', function(){
        var id = that.element.ID;

        Sunstone.getDialog(ADD_AR_DIALOG_ID).setId(id);
        Sunstone.getDialog(ADD_AR_DIALOG_ID).show();

        return false;
      });
    }

    if (Config.isTabActionEnabled("vnets-tab", "Network.update_ar")) {
      context.off("click", 'button#update_ar_button');
      context.on("click", 'button#update_ar_button', function(){
        var id = that.element.ID;
        var ar_id = $(this).attr('ar_id');

        OpenNebulaNetwork.show({
          data : {
            id: id
          },
          timeout: true,
          success: function (request, vn){
            var vn_info = vn.VNET;

            var ar = getAR(vn_info, ar_id);

            if(ar != undefined){
              Sunstone.getDialog(UPDATE_AR_DIALOG_ID).reset();

              Sunstone.getDialog(UPDATE_AR_DIALOG_ID).setParams({
                'vnetId': id,
                'arId': ar_id,
                'arData': $.extend({}, ar)
              });

              Sunstone.getDialog(UPDATE_AR_DIALOG_ID).show();

            } else {
              Notifier.notifyError(Locale.tr("The Adress Range was not found"));
              Sunstone.runAction("Network.show", id);
            }
          },
          error: Notifier.onError
        });

        return false;
      });
    }

    return false;
  }

  function _getState(context) {
    var state = {};

    if(this.last_selected_row_ar){
      state["ar"] = this.last_selected_row_ar.attr("ar");
    }

    return state;
  }

  function _setState(state, context) {
    var that = this;

    if(state.ar){
      $('#ar_list_datatable tr[ar="'+state.ar+'"]',context).click();
    }
  }

  //============================================================================
  //============================================================================

  // TODO move to util?
  function getAR(vn_info, arId){
    var ar_list = Utils.getARList(vn_info);
    var ar = undefined;

    for (var i=0; i<ar_list.length; i++){
      if (arId == ar_list[i].AR_ID){
        ar = $.extend({}, ar_list[i]);
        break;
      }
    }

    return ar;
  }
  //====


  // TODO: move to its own file?

  function _arHTML(vn_info, arId){
    var ar = getAR(vn_info, arId);

    if(ar == undefined){
        return "";
    }

    var first_mac       = ar.MAC;
    var last_mac        = ar.MAC_END;
    var first_ip        = ar.IP;
    var last_ip         = ar.IP_END;
    var first_ip6_global= ar.IP6_GLOBAL;
    var last_ip6_global = ar.IP6_GLOBAL_END;
    var first_ip6_ula   = ar.IP6_ULA;
    var last_ip6_ula    = ar.IP6_ULA_END;

    var arKnownAttr = [
      {key: Locale.tr("Type"),         value: ar.TYPE},
      {key: Locale.tr("MAC Start"),    value: ar.MAC},
      {key: Locale.tr("IP Start"),     value: ar.IP},
      {key: Locale.tr("Global prefix"),value: ar.GLOBAL_PREFIX},
      {key: Locale.tr("ULA prefix"),   value: ar.ULA_PREFIX},
      {key: Locale.tr("Size"),         value: ar.SIZE},
      {key: Locale.tr("Used leases"),  value: ar.USED_LEASES},
      {key: Locale.tr("Reservation parent AR"),  value: ar.PARENT_NETWORK_AR_ID}
    ];

    delete ar["MAC_END"];
    delete ar["IP_END"];
    delete ar["IP6_ULA"];
    delete ar["IP6_ULA_END"];
    delete ar["IP6_GLOBAL"];
    delete ar["IP6_GLOBAL_END"];
    delete ar["AR_ID"];
    delete ar["TYPE"];
    delete ar["MAC"];
    delete ar["IP"];
    delete ar["GLOBAL_PREFIX"];
    delete ar["ULA_PREFIX"];
    delete ar["SIZE"];
    delete ar["USED_LEASES"];
    delete ar["LEASES"];
    delete ar["PARENT_NETWORK_AR_ID"];

    this.secgroupTable = undefined;
    var secgroupTableHTML = undefined;

    if (ar.SECURITY_GROUPS != undefined &&
        ar.SECURITY_GROUPS.length != 0){

      var secgroups = ar.SECURITY_GROUPS.split(",");

      var opts = {
        info: true,
        select: true,
        selectOptions: {
          read_only: true,
          fixed_ids: secgroups
        }
      };

      this.secgroupTable = new SecurityGroupsTable(SG_TABLE_ID, opts);
      secgroupTableHTML = this.secgroupTable.dataTableHTML;
    }

    delete ar["SECURITY_GROUPS"];

    var arExtraAttr = ar;

    return TemplateARInfo({
      'arId': arId,
      'arKnownAttr': arKnownAttr,
      'arExtraAttr': arExtraAttr,
      'first_mac': first_mac,
      'last_mac': last_mac,
      'first_ip': first_ip,
      'last_ip': last_ip,
      'first_ip6_global': first_ip6_global,
      'last_ip6_global': last_ip6_global,
      'first_ip6_ula': first_ip6_ula,
      'last_ip6_ula': last_ip6_ula,
      'secgroupTableHTML': secgroupTableHTML
    });
  }

  function _arSetup(section, vn_info, ar_id){
    var ar = getAR(vn_info, ar_id);

    if(ar == undefined){
        return;
    }

    if (this.secgroupTable != undefined){
      this.secgroupTable.initialize();
      this.secgroupTable.refreshResourceTableSelect();
    }
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vnets-tab/panels/leases/html',['hbs','hbs/handlebars','templates/helpers/isTabActionEnabled','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "    <div class=\"row collapse\">\n      <div class=\"large-4 columns\">\n        <input type=\"text\" id=\"panel_hold_lease\"/>\n      </div>\n      <div class=\"large-2 columns end\">\n        <button class=\"button small secondary radius\" id=\"panel_hold_lease_button\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hold IP", {"name":"tr","hash":{},"data":data})))
    + "\n        </button>\n      </div>\n    </div>\n";
},"3":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "            <tr ip=\""
    + escapeExpression(((helper = (helper = helpers.IP || (depth0 != null ? depth0.IP : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"IP","hash":{},"data":data}) : helper)))
    + "\" mac=\""
    + escapeExpression(((helper = (helper = helpers.MAC || (depth0 != null ? depth0.MAC : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"MAC","hash":{},"data":data}) : helper)))
    + "\">\n              <td class=\"key_td\">";
  stack1 = ((helper = (helper = helpers.col0HTML || (depth0 != null ? depth0.col0HTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"col0HTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "</td>\n              <td>";
  stack1 = ((helper = (helper = helpers.col1HTML || (depth0 != null ? depth0.col1HTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"col1HTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</td>\n              <td style=\"white-space: nowrap\" class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.IP || (depth0 != null ? depth0.IP : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"IP","hash":{},"data":data}) : helper)))
    + "</td>\n              <td style=\"white-space: nowrap\" class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.MAC || (depth0 != null ? depth0.MAC : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"MAC","hash":{},"data":data}) : helper)))
    + "</td>\n              <td style=\"white-space: nowrap\" class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.IP6_LINK || (depth0 != null ? depth0.IP6_LINK : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"IP6_LINK","hash":{},"data":data}) : helper)))
    + "</td>\n              <td style=\"white-space: nowrap\" class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.IP6_ULA || (depth0 != null ? depth0.IP6_ULA : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"IP6_ULA","hash":{},"data":data}) : helper)))
    + "</td>\n              <td style=\"white-space: nowrap\" class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.IP6_GLOBAL || (depth0 != null ? depth0.IP6_GLOBAL : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"IP6_GLOBAL","hash":{},"data":data}) : helper)))
    + "</td>\n              <td style=\"white-space: nowrap\" class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.AR_ID || (depth0 != null ? depth0.AR_ID : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"AR_ID","hash":{},"data":data}) : helper)))
    + "</td>\n            </tr>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form id=\"leases_form\">\n";
  stack1 = ((helpers.isTabActionEnabled || (depth0 && depth0.isTabActionEnabled) || helperMissing).call(depth0, "vnets-tab", "Network.hold_lease", {"name":"isTabActionEnabled","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "  <div class=\"row collapse\">\n    <div class=\"large-12 columns\" style=\"overflow:auto\">\n      <table id=\"leases_datatable\" class=\"datatable twelve\">\n        <thead>\n          <tr>\n            <th></th>\n            <th></th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IP", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "MAC", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPv6 Link", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPv6 ULA", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPv6 Global", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Address Range", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          </tr>\n        </thead>\n        <tbody>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.leases : depth0), {"name":"each","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "        </tbody>\n      </table>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/vnets-tab/panels/leases/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vnets-tab/panels/leases/panelId',['require'],function(require){
  return 'vnet_leases_tab';
});
define('tabs/vnets-tab/panels/leases',['require','hbs!./leases/html','utils/locale','sunstone-config','sunstone','../utils/common','../tabId','./leases/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TemplateLeases = require('hbs!./leases/html');
  var Locale = require('utils/locale');
  var Config = require('sunstone-config');
  var Sunstone = require('sunstone');
  var Utils = require('../utils/common');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./leases/panelId');
  var RESOURCE = "Network";
  var XML_ROOT = "VNET";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Leases");
    this.icon = "fa-list-ul";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var arList = Utils.getARList(this.element);
    var processedLeases = [];

    for (var i=0; i<arList.length; i++){
      var ar = arList[i];
      var id = ar.AR_ID;

      var leases = ar.LEASES.LEASE;

      if (!leases) { //empty
        continue;
      } else if (leases.constructor != Array) { //>1 lease
        leases = [leases];
      }

      for (var j=0; j<leases.length; j++){
        var lease = leases[j];

        var col0HTML = "";
        var col1HTML = "";

        if (lease.VM == "-1") { //hold
          col0HTML = '<span type="text" class="alert radius label"></span>';

          if (Config.isTabActionEnabled("vnets-tab", "Network.release_lease")) {
            col1HTML = '<a class="release_lease" href="#"><i class="fa fa-play"/></a>';
          }
        } else if (lease.VM != undefined) { //used by a VM
          col0HTML = '<span type="text" class="radius label "></span>';
          col1HTML = Locale.tr("VM:") + lease.VM;
        } else if (lease.VNET != undefined) { //used by a VNET
          col0HTML = '<span type="text" class="radius label "></span>';
          col1HTML = Locale.tr("NET:") + lease.VNET;
        } else {
          col0HTML = '<span type="text" class="radius label "></span>';
          col1HTML = '--';
        }

        processedLeases.push({
          "col0HTML"  : col0HTML,
          "col1HTML"  : col1HTML,
          "IP"        : (lease.IP ? lease.IP : "--"),
          "MAC"       : (lease.MAC ? lease.MAC : "--"),
          "IP6_LINK"  : (lease.IP6_LINK ? lease.IP6_LINK : "--"),
          "IP6_ULA"   : (lease.IP6_ULA ? lease.IP6_ULA : "--"),
          "IP6_GLOBAL": (lease.IP6_GLOBAL ? lease.IP6_GLOBAL : "--"),
          "AR_ID"     : id
        });
      }
    }

    return TemplateLeases({
      'element': this.element,
      'leases' : processedLeases
    });
  }

  function _setup(context) {
    var that = this;

    var leases_dataTable = $("#leases_datatable", context).dataTable({
      "bSortClasses" : false,
      "bDeferRender": true,
      // "sScrollX": "100%",
      "aoColumnDefs": [
        { "bSortable": false, "aTargets": [0,1] },
      ]
    });

    if (Config.isTabActionEnabled("vnets-tab", "Network.hold_lease")) {
      context.off("click", 'button#panel_hold_lease_button');
      context.on("click", 'button#panel_hold_lease_button', function(){
        // TODO: context for selector
        var lease = $('input#panel_hold_lease').val();

        if (lease.length){
            var obj = {ip: lease};
            Sunstone.runAction('Network.hold',that.element.ID,obj);
        }

        return false;
      });

      context.off("click", 'a.hold_lease');
      context.on("click", 'a.hold_lease', function(){
        var lease = $(this).parents('tr').attr('ip');
        var obj = { ip: lease};
        Sunstone.runAction('Network.hold',that.element.ID,obj);
        // TODO: spinner is a global var set in sunstone.js
        //Set spinner
        $(this).parents('tr').html('<td class="key_td">'+spinner+'</td><td class="value_td"></td>');
        return false;
      });
    }

    if (Config.isTabActionEnabled("vnets-tab", "Network.release_lease")) {
      context.off("click", 'a.release_lease');
      context.on("click", 'a.release_lease', function(){
        var lease = $(this).parents('tr').attr('ip');
        if (lease == "undefined"){
          lease = $(this).parents('tr').attr('mac');
        }

        var obj = { ip: lease};
        Sunstone.runAction('Network.release',that.element.ID,obj);
        // TODO: spinner is a global var set in sunstone.js
        //Set spinner
        $(this).parents('tr').html('<td class="key_td">'+spinner+'</td><td class="value_td"></td>');
        return false;
      });
    }

    return false;
  }
});

define('tabs/vnets-tab/panels/secgroups/panelId',['require'],function(require){
  return 'vnet_sg_list_tab';
});
define('tabs/vnets-tab/panels/secgroups',['require','utils/locale','tabs/secgroups-tab/datatable','./secgroups/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var SecurityGroupsTable = require('tabs/secgroups-tab/datatable');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./secgroups/panelId');
  var SG_TABLE_ID = PANEL_ID + "SecurityGroupsTable";
  var RESOURCE = "Network";
  var XML_ROOT = "VNET";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Security");
    this.icon = "fa-shield";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var secgroups = [];

    if (this.element.TEMPLATE.SECURITY_GROUPS != undefined &&
        this.element.TEMPLATE.SECURITY_GROUPS.length != 0){

        secgroups = this.element.TEMPLATE.SECURITY_GROUPS.split(",");
    }

    var opts = {
      info: true,
      select: true,
      selectOptions: {
        read_only: true,
        fixed_ids: secgroups
      }
    };

    this.secgroupTable = new SecurityGroupsTable(SG_TABLE_ID, opts);

    return this.secgroupTable.dataTableHTML;
  }

  function _setup(context) {
    this.secgroupTable.initialize();
    this.secgroupTable.refreshResourceTableSelect();

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vnets-tab/form-panels/create/wizard',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div>\n    <dl id=\"vnet_create_tabs\" class=\"tabs right-info-tabs text-center\" data-tab>\n      <dd class=\"active\">\n        <a href=\"#vnetCreateGeneralTab\"> <i class=\"fa fa-globe\"></i>\n          <br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "General", {"name":"tr","hash":{},"data":data})))
    + "</a>\n      </dd>\n      <dd>\n        <a href=\"#vnetCreateBridgeTab\"> <i class=\"fa fa-cog\"></i>\n          <br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Conf", {"name":"tr","hash":{},"data":data})))
    + "</a>\n      </dd>\n      <dd>\n        <a href=\"#vnetCreateARTab\">\n          <i class=\"fa fa-align-justify\"></i>\n          <br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Addresses", {"name":"tr","hash":{},"data":data})))
    + "</a>\n      </dd>\n      <dd>\n        <a href=\"#vnetCreateSecurityTab\">\n          <i class=\"fa fa-shield\"></i>\n          <br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Security", {"name":"tr","hash":{},"data":data})))
    + "</a>\n      </dd>\n      <dd>\n        <a href=\"#vnetCreateContextTab\">\n          <i class=\"fa fa-folder\"></i>\n          <br>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Context", {"name":"tr","hash":{},"data":data})))
    + "</a>\n      </dd>\n    </dl>\n    <div id=\"vnet_create_tabs_content\" class=\"tabs-content\">\n      <div class=\"content active\" id=\"vnetCreateGeneralTab\">\n        <div class=\"row\">\n          <div class=\"medium-6 columns\">\n            <label for=\"name\" >\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name that the Virtual Network will get for description purposes.", {"name":"tr","hash":{},"data":data})))
    + "\n              </span>\n            </label>\n            <input type=\"text\" wizard_field=\"NAME\" required name=\"name\" id=\"name\"/>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"medium-6 columns\">\n            <label for=\"DESCRIPTION\" >\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description of the Virtual Network", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n            <textarea type=\"text\" wizard_field=\"DESCRIPTION\" id=\"DESCRIPTION\" name=\"DESCRIPTION\"/>\n          </div>\n        </div>\n      </div>\n      <div class=\"content\" id=\"vnetCreateBridgeTab\">\n        <div class=\"row\">\n          <div class=\"medium-6 columns\">\n            <label for=\"bridge\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Bridge", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name of the physical bridge in the physical host where the VM should connect its network interface", {"name":"tr","hash":{},"data":data})))
    + "\n              </span>\n            </label>\n            <input type=\"text\" wizard_field=\"BRIDGE\" name=\"bridge\" id=\"bridge\" />\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"medium-6 columns\">\n            <label for=\"network_mode\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network model", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Choose the same networking model you chose for the hosts that will use this newtork", {"name":"tr","hash":{},"data":data})))
    + "\n              </span>\n            </label>\n            <select name=\"network_mode\" id=\"network_mode\">\n              <option value=\"default\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Default", {"name":"tr","hash":{},"data":data})))
    + "</option>\n              <option value=\"802.1Q\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "802.1Q", {"name":"tr","hash":{},"data":data})))
    + "</option>\n              <option value=\"vxlan\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VXLAN", {"name":"tr","hash":{},"data":data})))
    + "</option>\n              <option value=\"ebtables\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ebtables", {"name":"tr","hash":{},"data":data})))
    + "</option>\n              <option value=\"openvswitch\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Open vSwitch", {"name":"tr","hash":{},"data":data})))
    + "</option>\n              <option value=\"vmware\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMware", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            </select>\n          </div>\n          <div class=\"large-12 columns\">\n            <div class=\"network_mode_description\" value=\"default\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Default: dummy driver that doesn’t perform any network operation. Firewalling rules are also ignored.", {"name":"tr","hash":{},"data":data})))
    + "\n            </div>\n            <div class=\"network_mode_description\" value=\"802.1Q\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "802.1Q: restrict network access through VLAN tagging, which also requires support from the hardware switches.", {"name":"tr","hash":{},"data":data})))
    + "\n            </div>\n            <div class=\"network_mode_description\" value=\"vxlan\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VXLAN: creates a L2 network overlay based on the VXLAN protocol, each VLAN has associated a multicast address in the 239.0.0.0/8 range.", {"name":"tr","hash":{},"data":data})))
    + "\n            </div>\n            <div class=\"network_mode_description\" value=\"ebtables\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ebtables: restrict network access through Ebtables rules. No special hardware configuration required.", {"name":"tr","hash":{},"data":data})))
    + "\n            </div>\n            <div class=\"network_mode_description\" value=\"openvswitch\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Open vSwitch: restrict network access with Open vSwitch Virtual Switch.", {"name":"tr","hash":{},"data":data})))
    + "\n            </div>\n            <div class=\"network_mode_description\" value=\"vmware\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMware: uses the VMware networking infrastructure to provide an isolated and 802.1Q compatible network for VMs launched with the VMware hypervisor.", {"name":"tr","hash":{},"data":data})))
    + "\n            </div>\n          </div>\n        </div>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-6 columns\">\n            <label for=\"mac_spoofing\">\n              <input type=\"checkbox\" wizard_field=\"FILTER_MAC_SPOOFING\" value=\"YES\" name=\"mac_spoofing\" id=\"mac_spoofing\" />\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filter MAC spoofing", {"name":"tr","hash":{},"data":data})))
    + "\n              <span class=\"tip\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Activate the filter to prevent mac spoofing. Only works with FW, 802.1Q, VXLAN and Ebtables network drivers.", {"name":"tr","hash":{},"data":data})))
    + "\n              </span>\n            </label>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"large-6 columns\">\n            <label for=\"ip_spoofing\">\n              <input type=\"checkbox\" wizard_field=\"FILTER_IP_SPOOFING\" value=\"YES\" name=\"ip_spoofing\" id=\"ip_spoofing\" />\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Filter IP spoofing", {"name":"tr","hash":{},"data":data})))
    + "\n              <span class=\"tip\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Activate the filter to prevent IP spoofing. Only works with FW, 802.1Q, VXLAN and Ebtables network drivers.", {"name":"tr","hash":{},"data":data})))
    + "\n              </span>\n            </label>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"medium-6 columns\">\n            <div class=\"row\">\n              <div class=\"medium-6 columns\">\n                <label for=\"vlan\">\n                  "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VLAN", {"name":"tr","hash":{},"data":data})))
    + ":\n                  <span class=\"tip\">\n                    "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Whether or not to isolate this virtual network using the Virtual Network Manager drivers", {"name":"tr","hash":{},"data":data})))
    + "\n                  </span>\n                </label>\n                <select wizard_field=\"VLAN\" name=\"vlan\" id=\"vlan\">\n                  <option value=\"YES\" selected=\"selected\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Yes", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                  <option value=\"NO\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "No", {"name":"tr","hash":{},"data":data})))
    + "</option>\n                </select>\n              </div>\n              <div class=\"medium-6 columns\">\n                <label for=\"vlan_id\">\n                  "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VLAN ID", {"name":"tr","hash":{},"data":data})))
    + ":\n                  <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Optional: Set a specific VLAN id", {"name":"tr","hash":{},"data":data})))
    + "</span>\n                </label>\n                <input type=\"text\" wizard_field=\"VLAN_ID\" name=\"vlan_id\" id=\"vlan_id\" />\n              </div>\n            </div>\n          </div>\n          <div class=\"medium-6 columns\">\n            <div class=\"row\">\n              <div class=\"large-12 columns\">\n                <label for=\"phydev\">\n                  "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Physical device", {"name":"tr","hash":{},"data":data})))
    + ":\n                  <span class=\"tip\">\n                    "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name of the physical network device that will be attached to the bridge", {"name":"tr","hash":{},"data":data})))
    + "\n                  </span>\n                </label>\n                <input type=\"text\" wizard_field=\"PHYDEV\" name=\"phydev\" id=\"phydev\" />\n              </div>\n            </div>\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"medium-3 columns\">\n            <label for=\"mtu\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "MTU", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Set the MTU for the tagged interface. This MTU will be then inherited by the bridge and by the tagged interface.", {"name":"tr","hash":{},"data":data})))
    + "\n              </span>\n            </label>\n            <input type=\"text\" wizard_field=\"MTU\" name=\"mtu\" id=\"mtu\" />\n          </div>\n        </div>\n      </div>\n      <div class=\"content\" id=\"vnetCreateARTab\">\n        <div class=\"row\" id=\"vnetCreateARTabCreate\">\n          <div class=\"large-12 columns\">\n            <div class=\"row\">\n              <dl class=\"tabs vertical\" id=\"vnet_wizard_ar_tabs\" data-tab>\n                <dt class=\"text-center\">\n                  <button type=\"button\" class=\"button tiny radius\" id=\"vnet_wizard_ar_btn\">\n                    <span class=\"fa fa-plus\"></span>\n                    "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add another Address Range", {"name":"tr","hash":{},"data":data})))
    + "\n                  </button>\n                </dt>\n              </dl>\n              <div class=\"tabs-content vertical\" id=\"vnet_wizard_ar_tabs_content\"></div>\n            </div>\n          </div>\n        </div>\n        <div class=\"row\" id=\"vnetCreateARTabUpdate\">\n          <div class=\"large-12 columns\">\n            <p>\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Address Ranges need to be managed in the individual Virtual Network panel", {"name":"tr","hash":{},"data":data})))
    + "\n            </p>\n          </div>\n        </div>\n      </div>\n      <div class=\"content\" id=\"vnetCreateSecurityTab\">\n        ";
  stack1 = ((helper = (helper = helpers.securityGroupsTableHTML || (depth0 != null ? depth0.securityGroupsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"securityGroupsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n        <div class=\"row\" id=\"default_sg_warning\">\n          <div class=\"large-12 columns\">\n            <span class=\"radius secondary label\">\n              <i class=\"fa fa-warning\"/>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "The default Security Group 0 is automatically added to new Virtual Networks", {"name":"tr","hash":{},"data":data})))
    + "\n            </span>\n          </div>\n        </div>\n      </div>\n      <div class=\"content\" id=\"vnetCreateContextTab\">\n        <div class=\"row\">\n          <div class=\"medium-6 columns\">\n            <label for=\"net_address\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network address", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Base network address. For example, 192.168.1.0", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n            <input type=\"text\" wizard_field=\"NETWORK_ADDRESS\" name=\"net_address\" id=\"net_address\" />\n          </div>\n          <div class=\"medium-6 columns\">\n            <label for=\"net_mask\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network mask", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network mask. For example, 255.255.255.0", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n            <input type=\"text\" wizard_field=\"NETWORK_MASK\" name=\"net_mask\" id=\"net_mask\" />\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"medium-6 columns\">\n            <label for=\"net_gateway\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Gateway", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Router for this network. Leave empty if the network is not routable", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n            <input type=\"text\" wizard_field=\"GATEWAY\" name=\"net_gateway\" id=\"net_gateway\" />\n          </div>\n          <div class=\"medium-6 columns\">\n            <label for=\"net_gateway6\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPv6 Gateway", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPv6 Router for this network", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n            <input type=\"text\" wizard_field=\"GATEWAY6\" name=\"net_gateway6\" id=\"net_gateway6\" />\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"medium-6 columns\">\n            <label for=\"net_dns\">\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "DNS", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Specific DNS for this network", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n            <input type=\"text\" wizard_field=\"DNS\" name=\"net_dns\" id=\"net_dns\" />\n          </div>\n        </div>\n        <br>\n        <div class=\"row\">\n          <div class=\"large-12 columns\">\n            <span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Custom attributes", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            <br>\n            <br>\n          </div>\n        </div>\n        ";
  stack1 = ((helper = (helper = helpers.customTagsHTML || (depth0 != null ? depth0.customTagsHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"customTagsHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n      </div>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/vnets-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/vnets-tab/form-panels/create/advanced',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Advanced\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <p>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Write the Virtual Network template here", {"name":"tr","hash":{},"data":data})))
    + "</p>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <textarea id=\"template\" rows=\"15\" required></textarea>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/vnets-tab/form-panels/create/advanced', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vnets-tab/form-panels/create',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','utils/tips','utils/custom-tags-table','tabs/vnets-tab/utils/ar-tab','tabs/secgroups-tab/datatable','utils/template-utils','utils/wizard-fields','hbs!./create/wizard','hbs!./create/advanced','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var CustomTagsTable = require('utils/custom-tags-table');
  var ArTab = require('tabs/vnets-tab/utils/ar-tab');
  var SecurityGroupsTable = require('tabs/secgroups-tab/datatable');
  var TemplateUtils = require('utils/template-utils');
  var WizardFields = require('utils/wizard-fields');

  /*
    TEMPLATES
   */

  var TemplateWizardHTML = require('hbs!./create/wizard');
  var TemplateAdvancedHTML = require('hbs!./create/advanced');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create Virtual Network"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      },
      'update': {
        'title': Locale.tr("Update Virtual Network"),
        'buttonText': Locale.tr("Update"),
        'resetButton': false
      }
    };

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.htmlAdvanced = _htmlAdvanced;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.submitAdvanced = _submitAdvanced;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.fill = _fill;
  FormPanel.prototype.setup = _setup;
  FormPanel.prototype.addARTab = _add_ar_tab;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    var opts = {
      info: false,
      select: true,
      selectOptions: {"multiple_choice": true}
    };

    this.securityGroupsTable = new SecurityGroupsTable("vnet_create", opts);

    return TemplateWizardHTML({
      'formPanelId': this.formPanelId,
      'customTagsHTML': CustomTagsTable.html(),
      'securityGroupsTableHTML': this.securityGroupsTable.dataTableHTML
    });
  }

  function _htmlAdvanced() {
    return TemplateAdvancedHTML({formPanelId: this.formPanelId});
  }

  function _setup(context) {
    this.arTabObjects = {};
    var that = this;

    var number_of_ar = 0;

    // add new ar tabs
    $("#vnet_wizard_ar_btn", context).bind("click", function() {
      that.addARTab(number_of_ar, context);
      number_of_ar++;

      context.foundation();

      return false;
    });

    // close icon: removing the tab on click
    $("#vnetCreateARTab", context).on("click", "i.remove-tab", function() {
      var target = $(this).parent().attr("href");
      var dd = $(this).closest('dd');
      var dl = $(this).closest('dl');
      var content = $(target);

      var ar_id = content.attr("ar_id");

      dd.remove();
      content.remove();

      if (dd.attr("class") == 'active') {
        $('a', dl.children('dd').last()).click();
      }

      delete that.arTabObjects[ar_id];

      return false;
    });

    $("#vnetCreateARTab #vnetCreateARTabUpdate", context).hide();

    $('#network_mode', context).change(function() {
      $('input,select#vlan,label[for!="network_mode"]', $(this).parent()).hide();
      $('input', $(this).parent()).val("");
      switch ($(this).val()) {
      case "default":
        $('input#bridge,label[for="bridge"]', context).show().prop('wizard_field_disabled', false);
        $('input#phydev,label[for="phydev"]', context).hide().prop('wizard_field_disabled', true);
        $('select#vlan,label[for="vlan"]', context).hide().prop('wizard_field_disabled', true);
        $('input#vlan_id,label[for="vlan_id"]', context).hide().prop('wizard_field_disabled', true);
        $('input#ip_spoofing,label[for="ip_spoofing"]', context).show().prop('wizard_field_disabled', false);
        $('input#mac_spoofing,label[for="mac_spoofing"]', context).show().prop('wizard_field_disabled', false);
        $('input#mtu,label[for="mtu"]', context).hide().prop('wizard_field_disabled', false);

        $('input#phydev', context).removeAttr('required');
        $('input#bridge', context).attr('required', '');
        break;
      case "802.1Q":
        $('input#bridge,label[for="bridge"]', context).show().prop('wizard_field_disabled', false);
        $('input#phydev,label[for="phydev"]', context).show().prop('wizard_field_disabled', false);
        $('select#vlan,label[for="vlan"]', context).show().prop('wizard_field_disabled', false);
        $('input#vlan_id,label[for="vlan_id"]', context).show().prop('wizard_field_disabled', false);
        $('input#ip_spoofing,label[for="ip_spoofing"]', context).show().prop('wizard_field_disabled', false);
        $('input#mac_spoofing,label[for="mac_spoofing"]', context).show().prop('wizard_field_disabled', false);
        $('input#mtu,label[for="mtu"]', context).show().prop('wizard_field_disabled', false);

        $('input#phydev', context).removeAttr('required');
        $('input#bridge', context).removeAttr('required');
        break;
      case "vxlan":
        $('input#bridge,label[for="bridge"]', context).show().prop('wizard_field_disabled', false);
        $('input#phydev,label[for="phydev"]', context).show().prop('wizard_field_disabled', false);
        $('select#vlan,label[for="vlan"]', context).show().prop('wizard_field_disabled', false);
        $('input#vlan_id,label[for="vlan_id"]', context).show().prop('wizard_field_disabled', false);
        $('input#ip_spoofing,label[for="ip_spoofing"]', context).show().prop('wizard_field_disabled', false);
        $('input#mac_spoofing,label[for="mac_spoofing"]', context).show().prop('wizard_field_disabled', false);
        $('input#mtu,label[for="mtu"]', context).show().prop('wizard_field_disabled', false);

        $('input#phydev', context).removeAttr('required');
        $('input#bridge', context).removeAttr('required');
        break;
      case "ebtables":
        $('input#bridge,label[for="bridge"]', context).show().prop('wizard_field_disabled', false);
        $('input#phydev,label[for="phydev"]', context).hide().prop('wizard_field_disabled', true);
        $('select#vlan,label[for="vlan"]', context).show().prop('wizard_field_disabled', false);
        $('input#vlan_id,label[for="vlan_id"]', context).hide().prop('wizard_field_disabled', true);
        $('input#ip_spoofing,label[for="ip_spoofing"]', context).show().prop('wizard_field_disabled', false);
        $('input#mac_spoofing,label[for="mac_spoofing"]', context).show().prop('wizard_field_disabled', false);
        $('input#mtu,label[for="mtu"]', context).hide().prop('wizard_field_disabled', false);

        $('input#phydev', context).removeAttr('required');
        $('input#bridge', context).attr('required', '');
        break;
      case "openvswitch":
        $('input#bridge,label[for="bridge"]', context).show().prop('wizard_field_disabled', false);
        $('input#phydev,label[for="phydev"]', context).hide().prop('wizard_field_disabled', true);
        $('select#vlan,label[for="vlan"]', context).show().prop('wizard_field_disabled', false);
        $('input#vlan_id,label[for="vlan_id"]', context).show().prop('wizard_field_disabled', false);
        $('input#ip_spoofing,label[for="ip_spoofing"]', context).hide().prop('wizard_field_disabled', true);
        $('input#mac_spoofing,label[for="mac_spoofing"]', context).show().prop('wizard_field_disabled', false);
        $('input#mtu,label[for="mtu"]', context).hide().prop('wizard_field_disabled', false);

        $('input#phydev', context).removeAttr('required');
        $('input#bridge', context).attr('required', '');
        break;
      case "vmware":
        $('input#bridge,label[for="bridge"]', context).show();
        $('input#phydev,label[for="phydev"]', context).hide();
        $('select#vlan,label[for="vlan"]', context).show();
        $('input#vlan_id,label[for="vlan_id"]', context).show();
        $('input#ip_spoofing,label[for="ip_spoofing"]', context).hide().prop('wizard_field_disabled', true);
        $('input#mac_spoofing,label[for="mac_spoofing"]', context).hide().prop('wizard_field_disabled', true);
        $('input#mtu,label[for="mtu"]', context).hide().prop('wizard_field_disabled', false);

        $('input#phydev', context).removeAttr('required');
        $('input#bridge', context).attr('required', '');
        break;
      }

      $("div.network_mode_description").hide();
      $('div.network_mode_description[value="' + $(this).val() + '"]').show();
    });

    //Initialize shown options
    $('#network_mode', context).trigger("change");

    this.securityGroupsTable.initialize();

    CustomTagsTable.setup($("#vnetCreateContextTab", context));

    context.foundation('tab', 'reflow');

    // Add first AR
    $("#vnet_wizard_ar_btn", context).trigger("click");

    Tips.setup();
    return false;
  }

  function _add_ar_tab(ar_id, context) {
    var str_ar_tab_id  = 'ar' + ar_id;

    var ar_tab = new ArTab();
    this.arTabObjects[ar_id] = ar_tab;

    var html_tab_content =
      '<div id="' + str_ar_tab_id + 'Tab" class="ar_tab content" ar_id="' + ar_id + '">' +
        ar_tab.html(str_ar_tab_id) +
      '</div>';

    // Append the new div containing the tab and add the tab to the list
    var a = $("<dd><a id='ar_tab" + str_ar_tab_id + "' href='#" + str_ar_tab_id + "Tab'>" +
        Locale.tr("Address Range") + " <i class='fa fa-times-circle remove-tab'></i></a></dd>"
        ).appendTo($("dl#vnet_wizard_ar_tabs", context));

    $(html_tab_content).appendTo($("#vnet_wizard_ar_tabs_content", context));

    $("a", a).trigger("click");

    var ar_section = $('#' + str_ar_tab_id + 'Tab', context);
    ar_tab.setup(ar_section, str_ar_tab_id);
    ar_tab.onShow();
  }

  function _submitWizard(context) {
    var that = this;

    //Fetch values
    var network_json = {};

    $.extend(network_json, WizardFields.retrieve($("#vnetCreateGeneralTab", context)));
    $.extend(network_json, WizardFields.retrieve($("#vnetCreateBridgeTab", context)));
    $.extend(network_json, WizardFields.retrieve($("#vnetCreateContextTab", context)));

    var secgroups = this.securityGroupsTable.retrieveResourceTableSelect();
    if (secgroups != undefined && secgroups.length != 0) {
      network_json["SECURITY_GROUPS"] = secgroups.join(",");
    }

    $.extend(network_json, CustomTagsTable.retrieve($("#vnetCreateContextTab", context)));

    $('.ar_tab', context).each(function() {
      var ar_id = $(this).attr("ar_id");
      var hash = that.arTabObjects[ar_id].retrieve();

      if (!$.isEmptyObject(hash)) {
        if (!network_json["AR"])
            network_json["AR"] = [];

        network_json["AR"].push(hash);
      }
    });

    if (this.action == "create") {
      network_json = {
        "vnet" : network_json
      };

      Sunstone.runAction("Network.create", network_json);
      return false;
    } else if (this.action == "update") {
      Sunstone.runAction("Network.update", this.resourceId, TemplateUtils.templateToString(network_json));
      return false;
    }
  }

  function _submitAdvanced(context) {
    if (this.action == "create") {
      var template = $('textarea#template', context).val();
      var vnet_json = {vnet: {vnet_raw: template}};
      Sunstone.runAction("Network.create", vnet_json);
      return false;

    } else if (this.action == "update") {
      var template_raw = $('textarea#template', context).val();
      Sunstone.runAction("Network.update", this.resourceId, template_raw);
      return false;
    }
  }

  function _onShow(context) {
    var that = this;

    this.securityGroupsTable.refreshResourceTableSelect();

    $('.ar_tab', context).each(function() {
      var ar_id = $(this).attr("ar_id");
      that.arTabObjects[ar_id].onShow();
    });
  }

  function _fill(context, element) {
    if (this.action != "update") {return;}
    this.resourceId = element.ID;

    $("#default_sg_warning").hide();
    // Populates the Avanced mode Tab
    $('#template', context).val(TemplateUtils.templateToString(element.TEMPLATE).replace(/^[\r\n]+$/g, ""));

    $('[wizard_field="NAME"]', context).val(
        TemplateUtils.escapeDoubleQuotes( TemplateUtils.htmlDecode(element.NAME) )).
        prop("disabled", true).
        prop('wizard_field_disabled', true);

    WizardFields.fill($("#vnetCreateGeneralTab", context), element.TEMPLATE);
    WizardFields.fill($("#vnetCreateBridgeTab", context), element.TEMPLATE);
    WizardFields.fill($("#vnetCreateContextTab", context), element.TEMPLATE);

    // Show all network mode inputs, and make them not required. This will change
    // if a different network model is selected
    $('input#bridge,label[for="bridge"]', context).show().prop('wizard_field_disabled', false).removeAttr('required');
    $('input#phydev,label[for="phydev"]', context).show().prop('wizard_field_disabled', false).removeAttr('required');
    $('select#vlan,label[for="vlan"]', context).show().prop('wizard_field_disabled', false).removeAttr('required');
    $('input#vlan_id,label[for="vlan_id"]', context).show().prop('wizard_field_disabled', false).removeAttr('required');
    $('input#ip_spoofing,label[for="ip_spoofing"]', context).show().prop('wizard_field_disabled', false);
    $('input#mac_spoofing,label[for="mac_spoofing"]', context).show().prop('wizard_field_disabled', false);

    if (element.TEMPLATE["SECURITY_GROUPS"] != undefined &&
        element.TEMPLATE["SECURITY_GROUPS"].length != 0) {

      var secgroups = element.TEMPLATE["SECURITY_GROUPS"].split(",");

      this.securityGroupsTable.selectResourceTableSelect({ids : secgroups});
    } else {
      this.securityGroupsTable.refreshResourceTableSelect();
    }

    // Delete so these attributes don't end in the custom tags table also
    delete element.TEMPLATE["SECURITY_GROUPS"];

    var fields = $('[wizard_field]', context);

    fields.each(function() {
      var field = $(this);
      var field_name = field.attr('wizard_field');

      delete element.TEMPLATE[field_name];
    });

    CustomTagsTable.fill($("#vnetCreateContextTab", context), element.TEMPLATE);

    // Remove the first AR added in initialize_
    $("#vnetCreateARTab i.remove-tab", context).trigger("click");
    $("#vnetCreateARTab #vnetCreateARTabUpdate", context).show();
    $("#vnetCreateARTab #vnetCreateARTabCreate", context).hide();
  }
});


/* START_TEMPLATE */
define('hbs!tabs/vnets-tab/form-panels/import/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row collapse vcenter_credentials\">\n    <div class=\"row\">\n      <div class=\"large-6 columns\">\n        <label for=\"vcenter_user\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "User", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        <input type=\"text\" name=\"vcenter_user\" id=\"vcenter_user\" />\n      </div>\n      <div class=\"large-6 columns\">\n        <label for=\"vcenter_host\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hostname", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        <input type=\"text\" name=\"vcenter_host\" id=\"vcenter_host\" />\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-6 columns\">\n        <label for=\"vcenter_password\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Password", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        <input type=\"password\" name=\"vcenter_password\" id=\"vcenter_password\" />\n      </div>\n      <div class=\"large-6 columns\">\n        <br>\n        <a class=\"button radius small right\" id=\"get_vcenter_networks\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Get Networks", {"name":"tr","hash":{},"data":data})))
    + "\n        </a>\n      </div>\n    </div>\n  </div>\n  <div class=\"row collapse\">\n    ";
  stack1 = ((helper = (helper = helpers.vCenterNetworksHTML || (depth0 != null ? depth0.vCenterNetworksHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"vCenterNetworksHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/vnets-tab/form-panels/import/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/vnets-tab/form-panels/import',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','utils/vcenter/networks','hbs!./import/html','./import/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var VCenterNetworks = require('utils/vcenter/networks');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./import/html');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./import/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'import': {
        'title': Locale.tr("Import vCenter Networks"),
        'buttonText': Locale.tr("Import"),
        'resetButton': true
      }
    };

    this.vCenterNetworks = new VCenterNetworks();

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    return TemplateHTML({
      'formPanelId': this.formPanelId,
      'vCenterNetworksHTML': this.vCenterNetworks.html()
    });
  }

  function _setup(context) {
    var that = this;

    Sunstone.disableFormPanelSubmit(TAB_ID);

    $("#get_vcenter_networks", context).on("click", function(){
      Sunstone.enableFormPanelSubmit(TAB_ID);

      var vcenter_user = $("#vcenter_user", context).val();
      var vcenter_password = $("#vcenter_password", context).val();
      var vcenter_host = $("#vcenter_host", context).val();

      that.vCenterNetworks.insert({
        container: context,
        vcenter_user: vcenter_user,
        vcenter_password: vcenter_password,
        vcenter_host: vcenter_host
      });

      return false;
    });

    return false;
  }

  function _submitWizard(context) {
    var that = this;

    Sunstone.hideFormPanelLoading(TAB_ID);
    Sunstone.disableFormPanelSubmit(TAB_ID);

    this.vCenterNetworks.import();

    return false;
  }

  function _onShow(context) {
  }
});

define('tabs/vnets-tab',['require','utils/locale','./vnets-tab/buttons','./vnets-tab/actions','./vnets-tab/datatable','./vnets-tab/tabId','./vnets-tab/dialogs/add-ar','./vnets-tab/dialogs/update-ar','./vnets-tab/dialogs/reserve','./vnets-tab/panels/info','./vnets-tab/panels/ar','./vnets-tab/panels/leases','./vnets-tab/panels/secgroups','./vnets-tab/form-panels/create','./vnets-tab/form-panels/import'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./vnets-tab/buttons');
  var Actions = require('./vnets-tab/actions');
  var Table = require('./vnets-tab/datatable');

  var TAB_ID = require('./vnets-tab/tabId');
  var DATATABLE_ID = "dataTableVNets";

  var _dialogs = [
    require('./vnets-tab/dialogs/add-ar'),
    require('./vnets-tab/dialogs/update-ar'),
    require('./vnets-tab/dialogs/reserve')
  ];

  var _panels = [
    require('./vnets-tab/panels/info'),
    require('./vnets-tab/panels/ar'),
    require('./vnets-tab/panels/leases'),
    require('./vnets-tab/panels/secgroups')
  ];

  var _formPanels = [
    require('./vnets-tab/form-panels/create'),
    require('./vnets-tab/form-panels/import')
  ];

  var VNetsTab = {
    tabId: TAB_ID,
    title: Locale.tr("Virtual Networks"),
    tabClass: "subTab",
    parentTab: "infra-tab",
    listHeader: '<i class="fa fa-fw fa-globe"></i>&emsp;' + Locale.tr("Virtual Networks"),
    infoHeader: '<i class="fa fa-fw fa-globe"></i>&emsp;' + Locale.tr("Virtual Network"),
    subheader: '<span class="total_vnets"/> <small>'+Locale.tr("TOTAL")+'</small>&emsp;\
        <span class="addresses_vnets"/> <small>'+Locale.tr("USED IPs")+'</small>',
    resource: 'Network',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return VNetsTab;
});

define('tabs/secgroups-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "SecurityGroup.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "SecurityGroup.create_dialog" : {
      type: "create_dialog",
      layout: "create"
    },
    "SecurityGroup.update_dialog" : {
      type: "action",
      layout: "main",
      text: Locale.tr("Update")
    },
    "SecurityGroup.chown" : {
      type: "confirm_with_select",
      text: Locale.tr("Change owner"),
      layout: "user_select",
      select: "User",
      tip: Locale.tr("Select the new owner")+":"
    },
    "SecurityGroup.chgrp" : {
      type: "confirm_with_select",
      text: Locale.tr("Change group"),
      layout: "user_select",
      select: "Group",
      tip: Locale.tr("Select the new group")+":"
    },
    "SecurityGroup.clone_dialog" : {
      type: "action",
      layout: "main",
      text: Locale.tr("Clone")
    },
    "SecurityGroup.delete" : {
      type: "confirm",
      layout: "del",
      text: Locale.tr("Delete")
    }
  };

  return Buttons;
})
;
define('tabs/secgroups-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createSecurityGroupForm';
});
define('tabs/secgroups-tab/dialogs/clone/dialogId',['require'],function(require){
  return 'cloneSecurityGroupDialog';
});
define('tabs/secgroups-tab/actions',['require','sunstone','utils/notifier','utils/locale','./datatable','opennebula/securitygroup','utils/common-actions','./tabId','./form-panels/create/formPanelId','./dialogs/clone/dialogId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var DataTable = require('./datatable');
  var OpenNebulaResource = require('opennebula/securitygroup');
  var CommonActions = require('utils/common-actions');

  var RESOURCE = "SecurityGroup";
  var XML_ROOT = "SECURITY_GROUP";
  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');
  var CLONE_DIALOG_ID = require('./dialogs/clone/dialogId');

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "SecurityGroup.create" : _commonActions.create(CREATE_DIALOG_ID),
    "SecurityGroup.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID),
    "SecurityGroup.list" : _commonActions.list(),
    "SecurityGroup.show" : _commonActions.show(),
    "SecurityGroup.refresh" : _commonActions.refresh(),
    "SecurityGroup.delete" : _commonActions.del(),
    "SecurityGroup.chown": _commonActions.multipleAction('chown'),
    "SecurityGroup.chgrp": _commonActions.multipleAction('chgrp'),
    "SecurityGroup.chmod": _commonActions.singleAction('chmod'),
    "SecurityGroup.rename": _commonActions.singleAction('rename'),
    "SecurityGroup.update" : _commonActions.update(),
    "SecurityGroup.update_template" : _commonActions.updateTemplate(),
    "SecurityGroup.update_dialog" : _commonActions.checkAndShowUpdate(),
    "SecurityGroup.show_to_update" : _commonActions.showUpdate(CREATE_DIALOG_ID),

    "SecurityGroup.clone_dialog" : {
      type: "custom",
      call: function(){
        Sunstone.getDialog(CLONE_DIALOG_ID).show();
      }
    },

    "SecurityGroup.clone" : {
      type: "single",
      call: OpenNebulaResource.clone,
      callback: function(request, response) {
        Sunstone.getDialog(CLONE_DIALOG_ID).hide();
        Sunstone.getDialog(CLONE_DIALOG_ID).reset();
        Sunstone.runAction('SecurityGroup.refresh');
      },
      error: Notifier.onError,
      notify: true
    }
  };

  return _actions;
})
;

/* START_TEMPLATE */
define('hbs!tabs/secgroups-tab/dialogs/clone/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <h3 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Clone Security Group", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n  </div>\n  <form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\">\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <div class=\"clone_one\"></div>\n        <div class=\"clone_several\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Several security groups are selected, please choose a prefix to name the new copies", {"name":"tr","hash":{},"data":data})))
    + "\n          <br>\n        </div>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <label class=\"clone_one\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        <label class=\"clone_several\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Prefix", {"name":"tr","hash":{},"data":data})))
    + "</label>\n        <input required type=\"text\" name=\"name\"></input>\n      </div>\n    </div>\n    <div class=\"form_buttons row\">\n      <button type=\"submit\" class=\"button radius right\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Clone", {"name":"tr","hash":{},"data":data})))
    + "\n      </button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/secgroups-tab/dialogs/clone/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/secgroups-tab/dialogs/clone',['require','utils/dialogs/dialog','hbs!./clone/html','sunstone','utils/notifier','utils/locale','opennebula/securitygroup','./clone/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./clone/html');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var OpenNebulaSecurityGroup = require('opennebula/securitygroup');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./clone/dialogId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId
    });
  }

  function _setup(context) {
    var that = this;

    context.foundation('abide', 'reflow');
    context.off('invalid.fndtn.abide', '#' + DIALOG_ID + 'Form');
    context.off('valid.fndtn.abide', '#' + DIALOG_ID + 'Form');

    context.on('invalid.fndtn.abide', '#' + DIALOG_ID + 'Form', function(e) {
      Notifier.notifyError(Locale.tr("One or more required fields are missing or malformed."));
    }).on('valid.fndtn.abide', '#' + DIALOG_ID + 'Form', function(e) {
      var name = $('input', this).val();
      var sel_elems = Sunstone.getDataTable(TAB_ID).elements();

      if (sel_elems.length > 1){
        for (var i=0; i< sel_elems.length; i++)
          //use name as prefix if several items selected
          Sunstone.runAction('SecurityGroup.clone',
            sel_elems[i],
            name + OpenNebulaSecurityGroup.getName(sel_elems[i]));
      } else {
        Sunstone.runAction('SecurityGroup.clone',sel_elems[0],name);
      }

      return false;
    });

    return false;
  }

  function _onShow(context) {
    var sel_elems = Sunstone.getDataTable(TAB_ID).elements();

    //show different text depending on how many elements are selected
    if (sel_elems.length > 1) {
      $('.clone_one', context).hide();
      $('.clone_several', context).show();
      $('input',context).val('Copy of ');
    } else {
      $('.clone_one', context).show();
      $('.clone_several', context).hide();

      $('input',context).val('Copy of ' + OpenNebulaSecurityGroup.getName(sel_elems[0]));
    }

    $("input[name='name']",context).focus();

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/secgroups-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "          <tr>\n            <td>"
    + escapeExpression(((helper = (helper = helpers.PROTOCOL || (depth0 != null ? depth0.PROTOCOL : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"PROTOCOL","hash":{},"data":data}) : helper)))
    + "</td>\n            <td>"
    + escapeExpression(((helper = (helper = helpers.RULE_TYPE || (depth0 != null ? depth0.RULE_TYPE : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"RULE_TYPE","hash":{},"data":data}) : helper)))
    + "</td>\n            <td>"
    + escapeExpression(((helper = (helper = helpers.RANGE || (depth0 != null ? depth0.RANGE : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"RANGE","hash":{},"data":data}) : helper)))
    + "</td>\n            <td>"
    + escapeExpression(((helper = (helper = helpers.NETWORK || (depth0 != null ? depth0.NETWORK : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"NETWORK","hash":{},"data":data}) : helper)))
    + "</td>\n            <td>"
    + escapeExpression(((helper = (helper = helpers.ICMP_TYPE || (depth0 != null ? depth0.ICMP_TYPE : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"ICMP_TYPE","hash":{},"data":data}) : helper)))
    + "</td>\n          </tr>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table id=\"info_security_group_table\" class=\"dataTable extended_table\">\n      <thead>\n         <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n        </tr>\n        ";
  stack1 = ((helper = (helper = helpers.renameTrHTML || (depth0 != null ? depth0.renameTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"renameTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\">\n    ";
  stack1 = ((helper = (helper = helpers.permissionsTableHTML || (depth0 != null ? depth0.permissionsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"permissionsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-9 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Rules", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n    </table>\n    <table class=\"policies_table dataTable\">\n      <thead>\n        <tr>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Protocol", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Port Range", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ICMP Type", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.ruleList : depth0), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "      </tbody>\n    </table>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-9 columns\">\n    ";
  stack1 = ((helper = (helper = helpers.templateTableHTML || (depth0 != null ? depth0.templateTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templateTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/secgroups-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/secgroups-tab/panels/info/panelId',['require'],function(require){
  return 'security_group_info_tab';
});
define('tabs/secgroups-tab/panels/info',['require','hbs!./info/html','utils/locale','utils/panel/permissions-table','utils/panel/rename-tr','opennebula/securitygroup','../utils/common','utils/panel/template-table','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TemplateInfo = require('hbs!./info/html');
  var Locale = require('utils/locale');
  var PermissionsTable = require('utils/panel/permissions-table');
  var RenameTr = require('utils/panel/rename-tr');
  var OpenNebulaSecurityGroup = require('opennebula/securitygroup');
  var Utils = require('../utils/common');

  /*
    TEMPLATES
   */

  var TemplateTable = require('utils/panel/template-table');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "SecurityGroup";
  var XML_ROOT = "SECURITY_GROUP";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var renameTrHTML = RenameTr.html(RESOURCE, this.element.NAME);
    var permissionsTableHTML = PermissionsTable.html(TAB_ID, RESOURCE, this.element);

    var rules = Utils.getRules(this.element);

    var ruleTextList = [];

    $.each(rules, function(){
      ruleTextList.push(Utils.sgRuleToSt(this));
    });

    // TODO
    //$(".resource-info-header", $("#secgroups-tab")).html(this.element.NAME);

    // TODO: simplify interface?
    var strippedTemplate = $.extend({}, this.element.TEMPLATE);
    delete strippedTemplate["RULE"];

    var templateTableHTML = TemplateTable.html(strippedTemplate, RESOURCE,
                                              Locale.tr("Attributes"));
    //====

    return TemplateInfo({
      'element': this.element,
      'renameTrHTML': renameTrHTML,
      'permissionsTableHTML': permissionsTableHTML,
      'ruleList': ruleTextList,
      'templateTableHTML': templateTableHTML
    });
  }

  function _setup(context) {
    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);
    RenameTr.setup(RESOURCE, this.element.ID, context);
    PermissionsTable.setup(TAB_ID, RESOURCE, this.element, context);

    // TODO: simplify interface?
    var strippedTemplate = $.extend({}, this.element.TEMPLATE);
    delete strippedTemplate["RULE"];

    var hiddenValues = {RULE: this.element.TEMPLATE.RULE};

    TemplateTable.setup(strippedTemplate, RESOURCE, this.element.ID, context, hiddenValues);
    //===

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/secgroups-tab/form-panels/create/wizard',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div id=\"create_security_group_form_wrapper\">\n  <form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n    <div class=\"row\">\n      <div class=\"medium-4 columns\">\n        <label for=\"security_group_name\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Security Group Name", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n        <input required type=\"text\" name=\"security_group_name\" id=\"security_group_name\"/>\n      </div>\n      <div class=\"medium-8 columns\">\n        <label for=\"security_group_description\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description for the Security Group", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </label>\n        <textarea type=\"text\" id=\"security_group_description\" name=\"security_group_description\" style=\"height: 70px;\"/>\n      </div>\n    </div>\n    <hr/>\n  </form>\n  <form data-abide=\"ajax\" id=\"rules_form_wizard\" action=\"\">\n    <div class=\"row collapse\" id=\"new_rule_wizard\">\n      <div class=\"row\">\n        <div class=\"medium-4 columns\">\n          <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "\n            <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Defines the direction of the rule.", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </label>\n          <select class=\"security_group_rule_type\">\n            <option value=\"inbound\" selected=\"selected\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Inbound", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"outbound\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Outbound", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"medium-4 columns\">\n          <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Protocol", {"name":"tr","hash":{},"data":data})))
    + "\n            <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Defines the protocol of the rule.", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </label>\n          <select class=\"security_group_rule_protocol\">\n            <option value=\"TCP\" selected=\"selected\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "TCP", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"UDP\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "UDP", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"ICMP\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ICMP", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"IPSEC\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IPsec", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"ALL\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "All", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"medium-4 columns icmp_type_wrapper\">\n          <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ICMP Type", {"name":"tr","hash":{},"data":data})))
    + "\n            <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Specific ICMP type of the rule. If a type has multiple codes, it includes all the codes within.", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </label>\n          <select class=\"security_group_rule_icmp_type\">\n            <option value=\"\" selected=\"selected\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "All", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value = \"0\">0/0: Echo Reply (pong)</option>\n            <option value = \"3\">3: Destination Unreachable</option>\n            <option value = \"4\">4/0: Source Quench</option>\n            <option value = \"5\">5: Redirect</option>\n            <option value = \"8\">8/0: Echo Request (ping)</option>\n            <option value = \"9\">9/0: Router Advertisement</option>\n            <option value = \"10\">10/0: Router Solicitation</option>\n            <option value = \"11\">11: Time Exceeded</option>\n            <option value = \"12\">12: Parameter Problem</option>\n            <option value = \"13\">13/0: Timestamp Request</option>\n            <option value = \"14\">14/0: Timestamp Reply</option>\n            <option value = \"17\">17/0: Address Mask Request</option>\n            <option value = \"18\">18/0: Address Mask Reply</option>\n          </select>\n        </div>\n      </div>\n      <div class=\"row range_row\">\n        <div class=\"medium-4 columns\">\n          <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Port range", {"name":"tr","hash":{},"data":data})))
    + "\n            <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Selects whether to apply this TCP or UDP rule to all the ports or just a specific set.", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </label>\n          <select class=\"security_group_rule_range_sel\">\n            <option value=\"ALL\" selected=\"selected\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "All", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"RANGE\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Port range", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"medium-4 columns end security_group_rule_range\">\n          <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Iptables range", {"name":"tr","hash":{},"data":data})))
    + "\n            <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "A Port Range to filter specific ports. Uses the iptables port syntax: port[,port|,port:port]...", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </label>\n          <input type=\"text\" placeholder=\"22,53,80:90,110,1024:65535\"/>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"medium-4 columns\">\n          <label>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network", {"name":"tr","hash":{},"data":data})))
    + "\n            <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Any: apply this rule to any connection regardless of its source or destination. Network: Apply this rule only to a custom defined network range. Virtual Network: Apply this rule only to connections with origin or destination in one of the already defined VNETs.", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </label>\n          <select class=\"security_group_rule_network_sel\">\n            <option value=\"ANY\" selected=\"selected\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Any", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"NETWORK\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network", {"name":"tr","hash":{},"data":data})))
    + "</option>\n            <option value=\"VNET\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Network", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          </select>\n        </div>\n        <div class=\"medium-4 columns security_group_rule_network\">\n          <label for=\"security_group_rule_first_ip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "IP Start", {"name":"tr","hash":{},"data":data})))
    + ":\n            <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "First IP address", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </label>\n          <input id=\"security_group_rule_first_ip\" type=\"text\" placeholder=\"192.168.10.1\"/>\n        </div>\n        <div class=\"medium-4 columns security_group_rule_network\">\n          <label for=\"security_group_rule_size\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Size", {"name":"tr","hash":{},"data":data})))
    + ":\n            <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of addresses in the range", {"name":"tr","hash":{},"data":data})))
    + "</span>\n          </label>\n          <input id=\"security_group_rule_size\" type=\"text\" placeholder=\"254\"/>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"small-12 columns vnet_select\">\n          ";
  stack1 = ((helper = (helper = helpers.vnetsTableHTML || (depth0 != null ? depth0.vnetsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"vnetsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n          <br/>\n        </div>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"medium-8 small-centered columns\">\n        <button type=\"submit\" class=\"add_security_group_rule button small small-12 radius\">\n          <i class=\"fa fa-angle-double-down\"></i> "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add Rule", {"name":"tr","hash":{},"data":data})))
    + "\n        </button>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <table class=\"security_group_rules policies_table dataTable\">\n          <thead>\n            <tr>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Protocol", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Port Range", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ICMP Type", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th style=\"width:3%\"></th>\n            </tr>\n          </thead>\n          <tbody>\n          </tbody>\n        </table>\n      </div>\n    </div>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/secgroups-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/secgroups-tab/form-panels/create/advanced',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Advanced\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <p>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Write the Security Group template here", {"name":"tr","hash":{},"data":data})))
    + "</p>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <textarea id=\"template\" rows=\"15\" required></textarea>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/secgroups-tab/form-panels/create/advanced', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/secgroups-tab/form-panels/create',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','utils/tips','utils/template-utils','utils/wizard-fields','tabs/vnets-tab/datatable','../utils/common','hbs!./create/wizard','hbs!./create/advanced','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var TemplateUtils = require('utils/template-utils');
  var WizardFields = require('utils/wizard-fields');
  var VNetsTable = require('tabs/vnets-tab/datatable');
  var Utils = require('../utils/common');

  /*
    TEMPLATES
   */

  var TemplateWizardHTML = require('hbs!./create/wizard');
  var TemplateAdvancedHTML = require('hbs!./create/advanced');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create Security Group"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      },
      'update': {
        'title': Locale.tr("Update Security Group"),
        'buttonText': Locale.tr("Update"),
        'resetButton': false
      }
    };

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.htmlAdvanced = _htmlAdvanced;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.submitAdvanced = _submitAdvanced;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.fill = _fill;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {

    var opts = {
      info: false,
      select: true
    };

    this.vnetsTable = new VNetsTable("new_sg_rule", opts);

    return TemplateWizardHTML({
      'formPanelId': this.formPanelId,
      'vnetsTableHTML': this.vnetsTable.dataTableHTML
    });
  }

  function _htmlAdvanced() {
    return TemplateAdvancedHTML({formPanelId: this.formPanelId});
  }

  function _setup(context) {
    var that = this;
    context.foundation('abide', 'reflow');

    context.off("change", '.security_group_rule_protocol');
    context.on("change", '.security_group_rule_protocol', function(){
      $('.range_row', context).hide();
      $('.range_row input', context).removeAttr('required');

      $('.icmp_type_wrapper', context).hide();

      switch ($(this).val()) {
      case "TCP":
      case "UDP":
        $('.range_row', context).show();
        $(".range_row select", context).trigger("change");
        break;
      case "ICMP":
        $('.icmp_type_wrapper', context).show();
        break;
      case "IPSEC":
      case "ALL":
        break;
      }
    });

    context.off("change", '.security_group_rule_network_sel');
    context.on("change", '.security_group_rule_network_sel', function(){
      $('.security_group_rule_network',context).hide();
      $('div.security_group_rule_network input',context).removeAttr('required');
      that.vnetsTable.idInput().removeAttr("required");

      $('.vnet_select',context).hide();

      switch ($(this).val()) {
      case "ANY":
        break;
      case "NETWORK":
        $('.security_group_rule_network',context).show();
        $('div.security_group_rule_network input',context).attr('required', '');
        break;
      case "VNET":
        $('.vnet_select',context).show();
        that.vnetsTable.idInput().attr("required", "");

        that.vnetsTable.refreshResourceTableSelect();

        break;
      }
    });

    context.off("change", '.security_group_rule_range_sel');
    context.on("change", '.security_group_rule_range_sel', function(){
      switch ($(this).val()) {
      case "ALL":
        $('.security_group_rule_range', context).hide();
        $(".security_group_rule_range input", context).removeAttr('required');
        break;
      case "RANGE":
        $('.security_group_rule_range', context).show();
        $(".security_group_rule_range input", context).attr('required', '');
        break;
      }
    });

    $('#rules_form_wizard',context).off('invalid.fndtn.abide');
    $('#rules_form_wizard',context).off('valid.fndtn.abide');

    $('#rules_form_wizard',context).on('invalid.fndtn.abide', function () {

    }).on('valid.fndtn.abide', function() {
      var rule = {};

      rule["PROTOCOL"] = $(".security_group_rule_protocol", context).val();
      rule["RULE_TYPE"] = $(".security_group_rule_type", context).val();

      switch ($('.security_group_rule_range_sel', context).val()) {
      case "ALL":
        break;
      case "RANGE":
        rule["RANGE"] = $(".security_group_rule_range input", context).val();
        break;
      }

      switch ($('.security_group_rule_network_sel', context).val()) {
      case "ANY":
        break;
      case "NETWORK":
        rule["IP"] = $('#security_group_rule_first_ip', context).val();
        rule["SIZE"] = $('#security_group_rule_size', context).val();
        break;
      case "VNET":
        rule["NETWORK_ID"] = that.vnetsTable.retrieveResourceTableSelect();
        break;
      }

      if (rule["PROTOCOL"] == "ICMP" ){
        var icmp_type_val = $(".security_group_rule_icmp_type", context).val();

        if (icmp_type_val != ""){
          rule["ICMP_TYPE"] = icmp_type_val;
        }
      }

      var text = Utils.sgRuleToSt(rule);

      $(".security_group_rules tbody", context).append(
          '<tr>\
            <td>'+text.PROTOCOL+'</td>\
            <td>'+text.RULE_TYPE+'</td>\
            <td>'+text.RANGE+'</td>\
            <td>'+text.NETWORK+'</td>\
            <td>'+text.ICMP_TYPE+'</td>\
            <td>\
              <a href="#"><i class="fa fa-times-circle remove-tab"></i></a>\
            </td>\
          </tr>');

      // Add data to tr element
      $(".security_group_rules tbody", context).children("tr").last().data("rule", rule);

      // Reset new rule fields
      $('#new_rule_wizard select option', context).prop('selected', function() {
        return this.defaultSelected;
      });

      $('#new_rule_wizard select', context).trigger("change");

      $('#new_rule_wizard input', context).val("");

      that.vnetsTable.resetResourceTableSelect();
    });

    context.off("click", ".security_group_rules i.remove-tab");
    context.on("click", ".security_group_rules i.remove-tab", function(){
      var tr = $(this).closest('tr');
      tr.remove();
    });

    context.foundation();

    this.vnetsTable.initialize();

    Tips.setup();

    $('#new_rule_wizard select', context).trigger("change");

    return false;
  }

  function _submitWizard(context) {

    var name = $('#security_group_name', context).val();
    var description = $('#security_group_description', context).val();

    var rules =  [];

    $(".security_group_rules tbody tr").each(function(){
      rules.push($(this).data("rule"));
    });

    var security_group_json = {
      "NAME" : name,
      "DESCRIPTION": description,
      "RULE" : rules
    };

    if (this.action == "create") {
      security_group_json = {
        "security_group" : security_group_json
      };

      Sunstone.runAction("SecurityGroup.create",security_group_json);
      return false;
    } else if (this.action == "update") {
      delete security_group_json["NAME"];

      Sunstone.runAction(
        "SecurityGroup.update",
        this.resourceId,
        TemplateUtils.templateToString(security_group_json));

      return false;
    }
  }

  function _submitAdvanced(context) {
    if (this.action == "create") {
      var template = $('textarea#template', context).val();
      var security_group_json = {security_group: {security_group_raw: template}};
      Sunstone.runAction("SecurityGroup.create",security_group_json);
      return false;
    } else if (this.action == "update") {
      var template_raw = $('textarea#template', context).val();
      Sunstone.runAction("SecurityGroup.update", this.resourceId, template_raw);
      return false;
    }
  }

  function _onShow(context) {
    this.vnetsTable.refreshResourceTableSelect();
  }

  function _fill(context, element) {
    var that = this;

    this.resourceId = element.ID;

    // Populates the Avanced mode Tab
    $('#template', context).val(TemplateUtils.templateToString(element.TEMPLATE).replace(/^[\r\n]+$/g, ""));

    $('#security_group_name',context).val(
      TemplateUtils.escapeDoubleQuotes(TemplateUtils.htmlDecode( element.NAME ))).
      prop("disabled", true);

    $('#security_group_description', context).val(
      TemplateUtils.escapeDoubleQuotes(TemplateUtils.htmlDecode( element.TEMPLATE.DESCRIPTION )) );

    var rules = element.TEMPLATE.RULE;

    if (!rules) { //empty
      rules = [];
    }
    else if (rules.constructor != Array) { //>1 rule
      rules = [rules];
    }

    $.each(rules, function(){
      var text = Utils.sgRuleToSt(this);

      $(".security_group_rules tbody", context).append(
        '<tr>\
        <td>'+text.PROTOCOL+'</td>\
        <td>'+text.RULE_TYPE+'</td>\
        <td>'+text.RANGE+'</td>\
        <td>'+text.NETWORK+'</td>\
        <td>'+text.ICMP_TYPE+'</td>\
        <td>\
        <a href="#"><i class="fa fa-times-circle remove-tab"></i></a>\
        </td>\
        </tr>');

      $(".security_group_rules tbody", context).children("tr").last().data("rule", this);
    });
  }
});

define('tabs/secgroups-tab',['require','utils/locale','./secgroups-tab/buttons','./secgroups-tab/actions','./secgroups-tab/datatable','./secgroups-tab/tabId','./secgroups-tab/dialogs/clone','./secgroups-tab/panels/info','./secgroups-tab/form-panels/create'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./secgroups-tab/buttons');
  var Actions = require('./secgroups-tab/actions');
  var Table = require('./secgroups-tab/datatable');

  var TAB_ID = require('./secgroups-tab/tabId');
  var DATATABLE_ID = "dataTableSecurityGroups";

  var _dialogs = [
    require('./secgroups-tab/dialogs/clone')
  ];

  var _panels = [
    require('./secgroups-tab/panels/info')
  ];

  var _formPanels = [
    require('./secgroups-tab/form-panels/create')
  ];

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("Security Groups"),
    tabClass: "subTab",
    parentTab: "infra-tab",
    listHeader: '<i class="fa fa-fw fa-shield"></i>&emsp;'+Locale.tr("Security Groups"),
    infoHeader: '<i class="fa fa-fw fa-shield"></i>&emsp;'+Locale.tr("Security Group"),
    subheader: '<span/> <small></small>&emsp;',
    resource: 'SecurityGroup',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});

define('tabs/zones-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var ZoneButtons = {
    "Zone.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "Zone.create_dialog" : {
      type: "create_dialog",
      layout: "create"
    },
    "Zone.delete" : {
      type: "confirm",
      layout: "del",
      text: Locale.tr("Delete")
    }
  };

  return ZoneButtons;
})
;
define('tabs/zones-tab/tabId',['require'],function(require){
  return 'zones-tab';
});
define('tabs/zones-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');

  /*
    CONSTANTS
   */

  var RESOURCE = "Zone";
  var XML_ROOT = "ZONE";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"]},
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    }

    this.columns = [
      Locale.tr("ID"),
      Locale.tr("Name"),
      Locale.tr("Endpoint"),
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 2,
      "select_resource": Locale.tr("Please select a Zone from the list"),
      "you_selected": Locale.tr("You selected the following Zone:"),
      "select_resource_multiple": Locale.tr("Please select one or more Zones from the list"),
      "you_selected_multiple": Locale.tr("You selected the following Zones:")
    };

    TabDataTable.call(this);
  };

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json.ZONE;

    return [
        '<input class="check_item" type="checkbox" id="' + RESOURCE.toLowerCase() + '_' +
                             element.ID + '" name="selected_items" value="' +
                             element.ID + '"/>',
        element.ID,
        element.NAME,
        element.TEMPLATE.ENDPOINT
    ];
  }
});

define('tabs/zones-tab/dialogs/create/dialogId',['require'],function(require){
  return 'createZoneDialog';
});
define('tabs/zones-tab/actions',['require','sunstone','utils/notifier','utils/locale','./datatable','opennebula/zone','utils/common-actions','./tabId','./dialogs/create/dialogId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var DataTable = require('./datatable');
  var OpenNebulaResource = require('opennebula/zone');
  var CommonActions = require('utils/common-actions');

  var XML_ROOT = "ZONE"
  var RESOURCE = "Zone"
  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./dialogs/create/dialogId');

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "Zone.create" : {
      type: "create",
      call: OpenNebulaResource.create,
      callback: function(request, response) {
        Sunstone.getDialog(CREATE_DIALOG_ID).hide();
        Sunstone.getDialog(CREATE_DIALOG_ID).reset();
        Sunstone.getDataTable(TAB_ID).addElement(request, response);
      },
      error: Notifier.onError,
      notify: true
    },
    
    "Zone.create_dialog" : {
      type: "custom",
      call: function() {
        Sunstone.getDialog(CREATE_DIALOG_ID).show();
      }
    },
    "Zone.list" : _commonActions.list(),
    "Zone.show" : _commonActions.show(),
    "Zone.refresh" : _commonActions.refresh(),
    "Zone.delete" : _commonActions.del(),
    "Zone.update_template" : _commonActions.updateTemplate(),
    "Zone.rename": _commonActions.singleAction('rename'),

    "Zone.show_to_update" : {
      type: "single",
      call: OpenNebulaResource.show,
      // TODO callback: fillPopPup,
      error: Notifier.onError
    }
  };

  return _actions;
})
;

/* START_TEMPLATE */
define('hbs!tabs/zones-tab/dialogs/create/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"row reveal-modal\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <h3 id=\"create_zone_header\" class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Create Zone", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n    </div>\n  </div>\n  <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\" action=\"\">\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <label for=\"zonename\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Zone Name", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n        <input type=\"text\" name=\"zonename\" id=\"zonename\" />\n      </div>\n    </div>\n    <div class=\"row centered\">\n      <div class=\"large-12 columns\">\n        <label for=\"endpoint\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Endpoint", {"name":"tr","hash":{},"data":data})))
    + ":</label>\n        <input type=\"text\" name=\"endpoint\" id=\"endpoint\" />\n      </div>\n    </div>\n    <div class=\"form_buttons\">\n      <button class=\"button radius right success\" id=\"create_zone_submit\" value=\"zone/create\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Create", {"name":"tr","hash":{},"data":data})))
    + "</button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/zones-tab/dialogs/create/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/zones-tab/dialogs/create',['require','utils/dialogs/dialog','hbs!./create/html','sunstone','./create/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./create/html');
  var Sunstone = require('sunstone');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./create/dialogId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;
    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({dialogId: this.dialogId});
  }

  function _setup(dialog) {
    $('#'+DIALOG_ID+'Form', dialog).submit(_submit);
    return false;
  }

  function _submit() {
    var name = $('#zonename', this).val();
    var endpoint = $("#endpoint", this).val();
    var zoneJSON = {"zone" : {"name" : name, "endpoint" : endpoint}};
    Sunstone.runAction("Zone.create", zoneJSON);
    return false;
  }

  function _onShow(dialog) {
    $("#zonename", dialog).focus();
    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/zones-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table id=\"info_zone_table\" class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"3\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n        </tr>\n        ";
  stack1 = ((helper = (helper = helpers.renameTrHTML || (depth0 != null ? depth0.renameTrHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"renameTrHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n      </tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\"></div>\n</div>\n<div class=\"row\">\n  <div class=\"large-9 columns\">\n    ";
  stack1 = ((helper = (helper = helpers.templateTableHTML || (depth0 != null ? depth0.templateTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templateTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/zones-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/zones-tab/panels/info/panelId',['require'],function(require){
  return 'zone_info_tab';
});
define('tabs/zones-tab/panels/info',['require','hbs!./info/html','utils/locale','utils/panel/rename-tr','utils/panel/template-table','./info/panelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TemplateInfo = require('hbs!./info/html');
  var Locale = require('utils/locale');
  var RenameTr = require('utils/panel/rename-tr');

  /*
    TEMPLATES
   */

  var TemplateTable = require('utils/panel/template-table');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./info/panelId');
  var TAB_ID = require('../tabId');
  var RESOURCE = "Zone"

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[RESOURCE.toUpperCase()];

    return this;
  };

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var renameTrHTML = RenameTr.html(RESOURCE, this.element.NAME);
    var templateTableHTML = TemplateTable.html(this.element.TEMPLATE, RESOURCE,
                                      Locale.tr("Attributes"));

    return TemplateInfo({
      'element': this.element,
      'renameTrHTML': renameTrHTML,
      'templateTableHTML': templateTableHTML
    });
  }

  function _setup(context) {
    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);
    RenameTr.setup(RESOURCE, this.element.ID, context);
    TemplateTable.setup(this.element.TEMPLATE, RESOURCE, this.element.ID, context);
    return false;
  }
});

define('tabs/zones-tab',['require','utils/locale','./zones-tab/buttons','./zones-tab/actions','./zones-tab/datatable','./zones-tab/tabId','./zones-tab/dialogs/create','./zones-tab/panels/info'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./zones-tab/buttons');
  var Actions = require('./zones-tab/actions');
  var Table = require('./zones-tab/datatable');

  var TAB_ID = require('./zones-tab/tabId');
  var DATATABLE_ID = "dataTableZones";

  var _dialogs = [
    require('./zones-tab/dialogs/create')
  ];

  var _panels = [
    require('./zones-tab/panels/info')
  ];

  var ZonesTab = {
    tabId: TAB_ID,
    title: Locale.tr("Zones"),
    tabClass: "subTab",
    parentTab: "infra-tab",
    listHeader: '<i class="fa fa-fw fa-files-o"></i>&emsp;' + Locale.tr("Zones"),
    infoHeader: '<i class="fa fa-fw fa-files-o"></i>&emsp;' + Locale.tr("Zone"),
    subheader: '',
    resource: 'Zone',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    dialogs: _dialogs
  };

  return ZonesTab;
});

define('tabs/marketplace-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "Marketplace.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "Marketplace.import" : {
      type: "action",
      layout: "main",
      text: Locale.tr('Import')
    }
  };

  return Buttons;
});
define('tabs/marketplace-tab/tabId',['require'],function(require){
  return 'marketplace-tab';
});
define('tabs/marketplace-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');

  /*
    CONSTANTS
   */

  var RESOURCE = "Marketplace";
  //var XML_ROOT = "";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    //this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"]},
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    };

    this.columns = [
      Locale.tr("ID"),
      Locale.tr("Name"),
      Locale.tr("Publisher"),
      Locale.tr("Hypervisor"),
      Locale.tr("Arch"),
      Locale.tr("Format"),
      Locale.tr("Tags")
    ];
    /*
    this.selectOptions = {
    };
    */
    TabDataTable.call(this);
  }

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element) {
    var publisher   = '-';
    var hypervisor  = '-';
    var arch        = '-';
    var format      = '-';
    var tags        = '-';

    if(element.publisher != undefined){
      publisher = element.publisher;
    }

    if(element.files != undefined && element.files.length > 0){
      if(element.files[0]["hypervisor"] != undefined){
        hypervisor = element.files[0]["hypervisor"];
      }

      if(element.files[0]["os-arch"] != undefined){
        arch = element.files[0]["os-arch"];
      }

      if(element.files[0]["format"] != undefined){
        format = element.files[0]["format"];
      }
    }

    if(element.tags != undefined){
      tags = element.tags;
    }

    return [
        '<input class="check_item" type="checkbox" id="'+
                    RESOURCE.toLowerCase()+'_' +
                    element._id.$oid + '" name="selected_items" value="' +
                    element._id.$oid + '"/>',
        element._id.$oid,
        element.name,
        publisher,
        hypervisor,
        arch,
        format,
        tags
    ];
  }
});

define('tabs/marketplace-tab/dialogs/import/dialogId',['require'],function(require){
  return 'importMarketplaceDialog';
});
define('tabs/marketplace-tab/actions',['require','sunstone','utils/notifier','utils/locale','./datatable','opennebula/marketplace','./tabId','./dialogs/import/dialogId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var DataTable = require('./datatable');
  var OpenNebulaResource = require('opennebula/marketplace');

  var RESOURCE = "Marketplace";
  var TAB_ID = require('./tabId');
  var IMPORT_DIALOG_ID = require('./dialogs/import/dialogId');

  var _actions = {
    "Marketplace.list" : {
      type: "list",
      call: OpenNebulaResource.list,
      callback: function(request, response) {
        Sunstone.getDataTable(TAB_ID).updateView(request, response.appliances);
      }
    },

    "Marketplace.refresh" : {
      type: "custom",
      call: function() {
        var tab = $('#' + TAB_ID);
        if (Sunstone.rightInfoVisible(tab)) {
          Sunstone.runAction(RESOURCE+".show", Sunstone.rightInfoResourceId(tab));
        } else {
          Sunstone.getDataTable(TAB_ID).waitingNodes();
          Sunstone.runAction(RESOURCE+".list", {force: true});
        }
      },
    },

    "Marketplace.import" : {
      type: "multiple",
      call: OpenNebulaResource.show,
      callback: function(request, response) {
        if (response['status'] && response['status'] != 'ready') {
            Notifier.notifyError(Locale.tr("The appliance is not ready"));
            return;
        }

        Sunstone.getDialog(IMPORT_DIALOG_ID).setParams({element: response});
        Sunstone.getDialog(IMPORT_DIALOG_ID).reset();
        Sunstone.getDialog(IMPORT_DIALOG_ID).show();
      },
      elements: function() {
        return Sunstone.getDataTable(TAB_ID).elements();
      },
      error: Notifier.onError
    },

    "Marketplace.show" : {
      type: "single",
      call: OpenNebulaResource.show,
      callback: function(request, response) {
        if (Sunstone.rightInfoVisible($('#'+TAB_ID))) {
          Sunstone.insertPanels(TAB_ID, response);
        }
      },
      error: Notifier.onError
    }
  };

  return _actions;
});


/* START_TEMPLATE */
define('hbs!tabs/marketplace-tab/dialogs/import/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal medium\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <h3 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Import Appliance", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n  </div>\n  <form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\">\n    <div id=\"market_import_dialog_content\">\n    </div>\n    <div class=\"form_buttons row\">\n      <button type=\"submit\" class=\"button radius right\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Import", {"name":"tr","hash":{},"data":data})))
    + "\n      </button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/marketplace-tab/dialogs/import/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/marketplace-tab/dialogs/import/content',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/humanizeSize'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, lambda=this.lambda, escapeExpression=this.escapeExpression, helperMissing=helpers.helperMissing, buffer = "<div class=\"row\" id=\"market_import_file_"
    + escapeExpression(lambda((data && data.index), depth0))
    + "\">\n  <div class=\"large-10 large-centered columns\">\n    <div class=\"large-10 columns\">\n      <label>\n        <i class=\"fa fa-fw fa-download\"/>\n        &emsp;"
    + escapeExpression(lambda((data && data.index), depth0))
    + " - "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Image Name", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"right\">"
    + escapeExpression(((helpers.humanizeSize || (depth0 && depth0.humanizeSize) || helperMissing).call(depth0, "B", (depth0 != null ? depth0.size : depth0), {"name":"humanizeSize","hash":{},"data":data})))
    + "</span>\n      </label>\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.name : depth0), {"name":"if","hash":{},"fn":this.program(2, data),"inverse":this.program(4, data),"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "    </div>\n    <div class=\"large-2 columns market_image_result\">\n    </div>\n  </div>\n  <div class=\"large-10 columns market_image_response\">\n  </div>\n</div>\n";
},"2":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "      <input type=\"text\" class=\"name\" value=\""
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "\"/>\n";
},"4":function(depth0,helpers,partials,data) {
  var stack1, lambda=this.lambda, escapeExpression=this.escapeExpression;
  return "      <input type=\"text\" class=\"name\" value=\""
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.name : stack1), depth0))
    + "\"/>\n";
},"6":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<br>\n<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <p style=\"font-size:14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "The following template will be created in OpenNebula and the previous images will be referenced in the disks", {"name":"tr","hash":{},"data":data})))
    + " "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "If you want to edit parameters of the template you can do it later in the templates tab", {"name":"tr","hash":{},"data":data})))
    + " </p>\n  </div>\n</div>\n<div class=\"row\" id=\"market_import_file_template\">\n  <div class=\"large-10 large-centered columns\">\n    <div class=\"large-10 columns\">\n      <label>\n        <i class=\"fa fa-fw fa-file-text-o\"/>&emsp;"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Template Name", {"name":"tr","hash":{},"data":data})))
    + "\n      </label>\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.NAME : depth0), {"name":"if","hash":{},"fn":this.program(7, data),"inverse":this.program(4, data),"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "    </div>\n    <div class=\"large-2 columns market_template_result\">\n    </div>\n  </div>\n  <div class=\"large-10 columns market_template_response\">\n  </div>\n</div>\n";
},"7":function(depth0,helpers,partials,data) {
  return "      <input type=\"text\" class=\"name\" value=\"NAME\"/>\n";
  },"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<div class=\"row\">\n  <div class=\"large-12 columns\">\n    <p style=\"font-size:14px\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "The following images will be created in OpenNebula.", {"name":"tr","hash":{},"data":data})))
    + " "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "If you want to edit parameters of the image you can do it later in the images tab", {"name":"tr","hash":{},"data":data})))
    + " </p>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-10 large-centered columns\">\n    <div class=\"large-10 columns\">\n      <label for=\"market_img_datastore\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select the datastore for the images", {"name":"tr","hash":{},"data":data})))
    + "\n      </label>\n      <div id=\"market_img_datastore\" name=\"market_img_datastore\">\n      </div>\n    </div>\n    <div class=\"large-2 columns\">\n    </div>\n  </div>\n</div>\n";
  stack1 = helpers.each.call(depth0, ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.files : stack1), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.opennebula_template : stack1), {"name":"if","hash":{},"fn":this.program(6, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"useData":true});
Handlebars.registerPartial('tabs/marketplace-tab/dialogs/import/content', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/marketplace-tab/dialogs/import',['require','utils/dialogs/dialog','hbs!./import/html','hbs!./import/content','sunstone','utils/notifier','utils/locale','utils/resource-select','opennebula/image','opennebula/template','./import/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./import/html');
  var ContentHTML = require('hbs!./import/content');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var ResourceSelect = require('utils/resource-select');
  var OpenNebulaImage = require('opennebula/image');
  var OpenNebulaTemplate = require('opennebula/template');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./import/dialogId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setParams = _setParams;
  Dialog.prototype.try_to_create_template = _try_to_create_template;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId
    });
  }

  function _setup(context) {
    var that = this;

    context.foundation('abide', 'reflow');
    $("#market_import_dialog_content", context).html(
      ContentHTML({'element': this.element})
    );

    // Filter out DS with type system (1) or file (2)
    var filter_att = ["TYPE", "TYPE"];
    var filter_val = ["1", "2"];

    ResourceSelect.insert('div#market_img_datastore', context, "Datastore",
                        null, false, null, filter_att, filter_val);

    context.off('invalid.fndtn.abide', '#' + DIALOG_ID + 'Form');
    context.off('valid.fndtn.abide', '#' + DIALOG_ID + 'Form');

    context.on('invalid.fndtn.abide', '#' + DIALOG_ID + 'Form', function(e) {
      Notifier.notifyError(Locale.tr("One or more required fields are missing or malformed."));
    }).on('valid.fndtn.abide', '#' + DIALOG_ID + 'Form', function(e) {
      that.number_of_files = that.element['files'].length;
      that.template_created = false;
      that.images_information = [];


      $("input, button", context).attr("disabled", "disabled");
      $(".market_image_result:not(.success)",  context).html(
        '<span class="fa-stack fa-2x" style="color: #dfdfdf">'+
          '<i class="fa fa-cloud fa-stack-2x"></i>'+
          '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>'+
        '</span>');
      $(".market_template_result",  context).html(
        '<span class="fa-stack fa-2x" style="color: #dfdfdf">'+
          '<i class="fa fa-cloud fa-stack-2x"></i>'+
          '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>'+
        '</span>');

      var template_context = $("#market_import_file_template", context);

      $.each(that.element['files'], function(index, value){
        var local_context = $("#market_import_file_"+index,  local_context);

        if ($(".market_image_result:not(.success)", local_context).length > 0) {
          img_obj = {
            "image" : {
              "NAME": $("input.name",local_context).val(),
              "PATH": that.element['links']['download']['href']+'/'+index,
              "TYPE": value['type'],
              "MD5": value['md5'],
              "SHA1": value['sha1'],
              "DRIVER": value['driver'],
              "DEV_PREFIX": value['dev_prefix'],
              "FROM_APP": that.element['_id']["$oid"],
              "FROM_APP_NAME": that.element['name'],
              "FROM_APP_FILE": index
            },
            "ds_id" : $("#market_img_datastore select", context).val()
          };

          OpenNebulaImage.create({
            timeout: true,
            data: img_obj,
            success: function (file_index, file_context){
              return function(request, response) {
                $(".market_image_result", file_context).addClass("success").html(
                  '<span class="fa-stack fa-2x" style="color: #dfdfdf">'+
                    '<i class="fa fa-cloud fa-stack-2x"></i>'+
                    '<i class="fa  fa-check fa-stack-1x fa-inverse"></i>'+
                  '</span>');

                $(".market_image_response", file_context).html(
                  '<p style="font-size:12px" class="running-color">'+
                  Locale.tr("Image created successfully")+' ID:'+response.IMAGE.ID+
                  '</p>');

                that.images_information[file_index] = response;

                that.try_to_create_template(context);
              };
            }(index, local_context),
            error: function (request, error_json){
              $(".market_template_result", template_context).html('');

              $(".market_image_result", local_context).html(
                '<span class="fa-stack fa-2x" style="color: #dfdfdf">'+
                  '<i class="fa fa-cloud fa-stack-2x"></i>'+
                  '<i class="fa  fa-warning fa-stack-1x fa-inverse"></i>'+
                '</span>');

              $(".market_image_response", local_context).html(
                '<p style="font-size:12px" class="error-color">'+
                (error_json.error.message || Locale.tr("Cannot contact server: is it running and reachable?"))+
                '</p>');

              $("input", template_context).removeAttr("disabled");
              $("input", local_context).removeAttr("disabled");
              $("button", context).removeAttr("disabled");
            }
          });
        }
      });

      that.try_to_create_template(context);

      return false;
    });

    return false;
  }

  function _onShow(context) {

    return false;
  }

  /**
   * @param {object} params
   *        - params.element : Marketplace appliance as returned by a .show call
   */
  function _setParams(params) {
    this.element = params.element;
  }


  function _try_to_create_template(context){
    var that = this;

    var template_context = $("#market_import_file_template",  context);

    var images_created = $(".market_image_result.success", context).length;

    if ((images_created == this.number_of_files) && !that.template_created) {
      that.template_created = true;

      if (that.element['opennebula_template'] && that.element['opennebula_template'] !== "CPU=1") {
        var vm_template;
        try {
          vm_template = JSON.parse(that.element['opennebula_template']);
        } catch (error) {
          $(".market_template_result", template_context).html(
            '<span class="fa-stack fa-2x" style="color: #dfdfdf">'+
              '<i class="fa fa-cloud fa-stack-2x"></i>'+
              '<i class="fa  fa-warning fa-stack-1x fa-inverse"></i>'+
            '</span>');

          $(".market_template_response", template_context).html(
            '<p style="font-size:12px" class="error-color">'+
            (error.message || Locale.tr("Cannot contact server: is it running and reachable?"))+
            '</p>');

          $("input", template_context).removeAttr("disabled");
          $("button", context).removeAttr("disabled");
          that.template_created = false;
          return;
        }

        if ($.isEmptyObject(vm_template.DISK)){
          vm_template.DISK = [];
        } else if (!$.isArray(vm_template.DISK)){
          vm_template.DISK = [vm_template.DISK];
        }

        vm_template.NAME = $("input", template_context).val();
        if (!vm_template.CPU){
          vm_template.CPU = "1";
        }
        if (!vm_template.MEMORY){
          vm_template.MEMORY = "1024";
        }

        $.each(that.images_information, function(image_index, image_info){
          if (!vm_template.DISK[image_index]) {
            vm_template.DISK[image_index] = {};
          }

          vm_template.DISK[image_index].IMAGE = image_info.IMAGE.NAME;
          vm_template.DISK[image_index].IMAGE_UNAME = image_info.IMAGE.UNAME;
        });

        vm_template.FROM_APP = that.element['_id']["$oid"];
        vm_template.FROM_APP_NAME = that.element['name'];

        OpenNebulaTemplate.create({
          timeout: true,
          data: {vmtemplate: vm_template},
          success: function (request, response){
            $(".market_template_result", template_context).addClass("success").html(
              '<span class="fa-stack fa-2x" style="color: #dfdfdf">'+
                '<i class="fa fa-cloud fa-stack-2x"></i>'+
                '<i class="fa  fa-check fa-stack-1x fa-inverse"></i>'+
              '</span>');

            $(".market_template_response", template_context).html(
              '<p style="font-size:12px" class="running-color">'+
              Locale.tr("Template created successfully")+' ID:'+response.VMTEMPLATE.ID+
              '</p>');

            $("button", context).hide();
          },
          error: function (request, error_json){
            $(".market_template_result", template_context).html(
              '<span class="fa-stack fa-2x" style="color: #dfdfdf">'+
                '<i class="fa fa-cloud fa-stack-2x"></i>'+
                '<i class="fa  fa-warning fa-stack-1x fa-inverse"></i>'+
              '</span>');

            $(".market_template_response", template_context).html(
              '<p style="font-size:12px" class="error-color">'+
              (error_json.error.message || Locale.tr("Cannot contact server: is it running and reachable?"))+
              '</p>');

            $("input", template_context).removeAttr("disabled");
            $("button", context).removeAttr("disabled");
            that.template_created = false;
          }
        });
      } else {
        $("button", context).hide();
      }
    }
  }

});


/* START_TEMPLATE */
define('hbs!tabs/marketplace-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/humanizeSize'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda;
  return "        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Status", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.status : stack1), depth0))
    + "</td>\n        </tr>\n";
},"3":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda;
  return "        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Tags", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.tags : stack1), depth0))
    + "</td>\n        </tr>\n";
},"5":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda;
  return "        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Catalog", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.catalog : stack1), depth0))
    + "</td>\n        </tr>\n";
},"7":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Size", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helpers.humanizeSize || (depth0 && depth0.humanizeSize) || helperMissing).call(depth0, "B", ((stack1 = ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.files : stack1)) != null ? stack1['0'] : stack1)) != null ? stack1.size : stack1), {"name":"humanizeSize","hash":{},"data":data})))
    + "</td>\n        </tr>\n";
},"9":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "    <table class=\"dataTable\">\n      <thead>\n        <tr>\n          <th colspan=\"2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Short Description", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"value_td\">";
  stack1 = ((helper = (helper = helpers.short_description || (depth0 != null ? depth0.short_description : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"short_description","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</td>\n        </tr>\n      </tbody>\n    </table>\n";
},"11":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "        <tr>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helpers.humanizeSize || (depth0 && depth0.humanizeSize) || helperMissing).call(depth0, "B", (depth0 != null ? depth0.size : depth0), {"name":"humanizeSize","hash":{},"data":data})))
    + "</td>\n        </tr>\n";
},"13":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "        <tr>\n          <td colspan=\"2\" class=\"value_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "No Images defined", {"name":"tr","hash":{},"data":data})))
    + "</td>\n        </tr>\n";
},"15":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda;
  return "    <table class=\"dataTable\">\n      <thead>\n        <tr>\n          <th colspan=\"2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "OpenNebula Template", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"value_td\"><pre>"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.opennebula_template : stack1), depth0))
    + "</pre></td>\n        </tr>\n      </tbody>\n    </table>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable\">\n      <thead>\n        <tr><th colspan=\"2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th></tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1._id : stack1)) != null ? stack1.$oid : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.name : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "URL", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">\n            <a href=\""
    + escapeExpression(((helper = (helper = helpers.url || (depth0 != null ? depth0.url : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"url","hash":{},"data":data}) : helper)))
    + "\" target=\"_blank\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "link", {"name":"tr","hash":{},"data":data})))
    + "</a>\n          </td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Publisher", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.publisher : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Downloads", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.downloads : stack1), depth0))
    + "</td>\n        </tr>\n";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.status : stack1), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.tags : stack1), {"name":"if","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.catalog : stack1), {"name":"if","hash":{},"fn":this.program(5, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "OS", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1['os-id'] : stack1), depth0))
    + " "
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1['os-release'] : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Arch", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1['os-arch'] : stack1), depth0))
    + "</td>\n        </tr>\n";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.files : stack1), {"name":"if","hash":{},"fn":this.program(7, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Hypervisor", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.hypervisor : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Format", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.format : stack1), depth0))
    + "</td>\n        </tr>\n      </tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\">\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.short_description : depth0), {"name":"if","hash":{},"fn":this.program(9, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "    <table class=\"dataTable\">\n      <thead>\n        <tr>\n          <th colspan=\"2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"value_td\">";
  stack1 = ((helper = (helper = helpers.description || (depth0 != null ? depth0.description : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"description","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "</td>\n        </tr>\n      </tbody>\n    </table>\n    <table class=\"dataTable\">\n      <thead>\n        <tr>\n          <th colspan=\"2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Images", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n";
  stack1 = helpers.each.call(depth0, ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.files : stack1), {"name":"each","hash":{},"fn":this.program(11, data),"inverse":this.program(13, data),"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "      </tbody>\n    </table>\n";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.opennebula_template : stack1), {"name":"if","hash":{},"fn":this.program(15, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/marketplace-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/marketplace-tab/panels/info/panelId',['require'],function(require){
  return 'marketplace_info_tab';
});
define('tabs/marketplace-tab/panels/info',['require','hbs!./info/html','utils/locale','utils/humanize','utils/template-utils','utils/panel/template-table','./info/panelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TemplateInfo = require('hbs!./info/html');
  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');
  var TemplateUtils = require('utils/template-utils');

  /*
    TEMPLATES
   */

  var TemplateTable = require('utils/panel/template-table');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "Marketplace";
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info;

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var url = this.element.links.download.href;
    url = url.replace(/\/download$/, '');

    var short_description = "";

    if(this.element.short_description){
      short_description = TemplateUtils.htmlDecode(this.element.short_description).replace(/\n/g, "<br/>");
    }

    var description = TemplateUtils.htmlDecode(this.element.description).replace(/\n/g, "<br/>");

    return TemplateInfo({
      'element': this.element,
      'url': url,
      'short_description': short_description,
      'description': description
    });
  }

  function _setup(context) {
    $('.resource-id', '#' + TAB_ID).hide();
    $('.resource-info-header', '#' + TAB_ID).text(this.element.name);
    return false;
  }
});

define('tabs/marketplace-tab',['require','utils/locale','./marketplace-tab/buttons','./marketplace-tab/actions','./marketplace-tab/datatable','./marketplace-tab/tabId','./marketplace-tab/dialogs/import','./marketplace-tab/panels/info'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./marketplace-tab/buttons');
  var Actions = require('./marketplace-tab/actions');
  var Table = require('./marketplace-tab/datatable');

  var TAB_ID = require('./marketplace-tab/tabId');
  var DATATABLE_ID = "dataTableMarketplace";

  var _dialogs = [
    require('./marketplace-tab/dialogs/import')
  ];

  var _panels = [
    require('./marketplace-tab/panels/info')
  ];

  var _formPanels = [
  ];

  var Tab = {
    tabId: TAB_ID,
    title: '<i class="fa fa-lg fa-fw fa-shopping-cart"></i>&emsp;' + Locale.tr("Marketplace"),
    listHeader: '<i class="fa fa-fw fa-shopping-cart"></i>&emsp;'+Locale.tr("OpenNebula Marketplace"),
    infoHeader: '<i class="fa fa-fw fa-shopping-cart"></i>&emsp;'+Locale.tr("Appliance"),
    subheader: '<span/> <small></small>&emsp;',
    resource: 'Marketplace',
    content: '<div class="row marketplace_error_message" hidden>\
        <div class="small-6 columns small-centered text-center">\
            <div class="alert-box alert radius">'+Locale.tr("Cannot connect to OpenNebula Marketplace")+'</div>\
        </div>\
    </div>',
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true, oneSelection: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});

define('tabs/oneflow-dashboard',['require'],function(require) {
  var TAB_ID = 'oneflow-dashboard';

  var Tab = {
    tabId: TAB_ID,
    title: '<i class="fa fa-lg fa-fw fa-cubes"></i>&emsp;OneFlow',
    no_content: true
  };

  return Tab;
});

define('tabs/oneflow-services-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "Service.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "Service.chown" : {
      type: "confirm_with_select",
      text: Locale.tr("Change owner"),
      select: "User",
      tip: Locale.tr("Select the new owner")+":",
      layout: "user_select"
    },
    "Service.chgrp" : {
      type: "confirm_with_select",
      text: Locale.tr("Change group"),
      select: "Group",
      tip: Locale.tr("Select the new group")+":",
      layout: "user_select"
    },
    "Service.shutdown" : {
      type: "action",
      layout: "main",
      text: Locale.tr("Shutdown")
    },
    "Service.recover" : {
      type: "action",
      layout: "main",
      text: Locale.tr("Recover")
    },
    "Service.delete" : {
      type: "confirm",
      text: Locale.tr("Delete"),
      layout: "del",
      tip: Locale.tr("This will delete the selected services")
    }
  };

  return Buttons;
});
define('tabs/oneflow-services-tab/dialogs/scale/dialogId',['require'],function(require){
  return 'scaleServiceRoleDialog';
});
define('tabs/oneflow-services-tab/actions',['require','sunstone','utils/notifier','utils/locale','opennebula/service','opennebula/role','opennebula/vm','utils/common-actions','./tabId','./panels/roles/panelId','./dialogs/scale/dialogId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var OpenNebulaResource = require('opennebula/service');
  var OpenNebulaRole = require('opennebula/role');
  var OpenNebulaVM = require('opennebula/vm');
  var CommonActions = require('utils/common-actions');

  var TAB_ID = require('./tabId');
  var XML_ROOT = "DOCUMENT";
  var RESOURCE = "Service";

  var ROLES_PANEL_ID = require('./panels/roles/panelId');
  var SCALE_DIALOG_ID = require('./dialogs/scale/dialogId');

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  function roleElements() {
    var selected_nodes = [];

    var dataTable = $('table[id^=datatable_roles]', '#'+TAB_ID+' #'+ROLES_PANEL_ID);
    var nodes = $('tbody input.check_item:checked', dataTable);
    $.each(nodes, function() {
      selected_nodes.push($(this).val());
    });

    return selected_nodes;
  }

  function roleVMElements() {
    var selected_nodes = [];

    var dataTable = $('table[id^=datatable_vms]', '#'+TAB_ID+' #'+ROLES_PANEL_ID);
    var nodes = $('tbody input.check_item:checked', dataTable);
    $.each(nodes, function() {
      selected_nodes.push($(this).val());
    });

    return selected_nodes;
  }

  function roleCallback() {
    return Sunstone.runAction('Service.refresh');
  }

  var _actions = {
    "Service.show" : _commonActions.show(),
    "Service.refresh" : _commonActions.refresh(),
    "Service.delete" : _commonActions.del(),
    "Service.chown": _commonActions.multipleAction('chown'),
    "Service.chgrp": _commonActions.multipleAction('chgrp'),
    "Service.chmod": _commonActions.singleAction('chmod'),
    "Service.shutdown":    _commonActions.multipleAction('shutdown'),
    "Service.recover":    _commonActions.multipleAction('recover'),

    "Service.list" : {
      type: "list",
      call: OpenNebulaResource.list,
      callback: function(request, response) {
        $(".oneflow_services_error_message").hide();
        Sunstone.getDataTable(TAB_ID).updateView(request, response);
      },
      error: function(request, error_json) {
        Notifier.onError(request, error_json, $(".oneflow_services_error_message"));
      }
    },

    //--------------------------------------------------------------------------

    "Role.scale_dialog" : {
      type: "custom",
      call: function(){
        selected = roleElements();

        if(selected.lenght == 0){
          return;
        }

        Sunstone.getDialog(SCALE_DIALOG_ID).setParams({roleIds: selected});
        Sunstone.getDialog(SCALE_DIALOG_ID).reset();
        Sunstone.getDialog(SCALE_DIALOG_ID).show();
      }
    },

    "Role.update" : {
      type: "multiple",
      call: OpenNebulaRole.update,
      callback: function() {
        Sunstone.getDialog(SCALE_DIALOG_ID).hide();
        roleCallback();
      },
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.hold" : {
      type: "multiple",
      call: OpenNebulaRole.hold,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.release" : {
      type: "multiple",
      call: OpenNebulaRole.release,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.suspend" : {
      type: "multiple",
      call: OpenNebulaRole.suspend,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.resume" : {
      type: "multiple",
      call: OpenNebulaRole.resume,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.stop" : {
      type: "multiple",
      call: OpenNebulaRole.stop,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.reboot_hard" : {
      type: "multiple",
      call: OpenNebulaRole.reboot_hard,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.delete_recreate" : {
      type: "multiple",
      call: OpenNebulaRole.delete_recreate,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.reboot" : {
      type: "multiple",
      call: OpenNebulaRole.reboot,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.poweroff" : {
      type: "multiple",
      call: OpenNebulaRole.poweroff,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.poweroff_hard" : {
      type: "multiple",
      call: OpenNebulaRole.poweroff_hard,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.undeploy" : {
      type: "multiple",
      call: OpenNebulaRole.undeploy,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.undeploy_hard" : {
      type: "multiple",
      call: OpenNebulaRole.undeploy_hard,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.snapshot_create" : {
      type: "single",
      call: OpenNebulaRole.snapshot_create,
      callback: roleCallback,
      error: Notifier.onError,
      notify: true
    },

    "Role.shutdown" : {
      type: "multiple",
      call: OpenNebulaRole.shutdown,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.shutdown_hard" : {
      type: "multiple",
      call: OpenNebulaRole.shutdown_hard,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.delete" : {
      type: "multiple",
      call: OpenNebulaRole.del,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    "Role.recover" : {
      type: "multiple",
      call: OpenNebulaRole.recover,
      callback: roleCallback,
      elements: roleElements,
      error: Notifier.onError,
      notify: true
    },

    //--------------------------------------------------------------------------

    "RoleVM.deploy" : {
      type: "multiple",
      call: OpenNebulaVM.deploy,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.migrate" : {
      type: "multiple",
      call: OpenNebulaVM.migrate,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.migrate_live" : {
      type: "multiple",
      call: OpenNebulaVM.livemigrate,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.hold" : {
      type: "multiple",
      call: OpenNebulaVM.hold,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.release" : {
      type: "multiple",
      call: OpenNebulaVM.release,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.suspend" : {
      type: "multiple",
      call: OpenNebulaVM.suspend,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.resume" : {
      type: "multiple",
      call: OpenNebulaVM.resume,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.stop" : {
      type: "multiple",
      call: OpenNebulaVM.stop,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.reboot_hard" : {
      type: "multiple",
      call: OpenNebulaVM.reset,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.delete_recreate" : {
      type: "multiple",
      call: OpenNebulaVM.resubmit,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.reboot" : {
      type: "multiple",
      call: OpenNebulaVM.reboot,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.poweroff" : {
      type: "multiple",
      call: OpenNebulaVM.poweroff,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.poweroff_hard" : {
      type: "multiple",
      call: OpenNebulaVM.poweroff_hard,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.undeploy" : {
      type: "multiple",
      call: OpenNebulaVM.undeploy,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.undeploy_hard" : {
      type: "multiple",
      call: OpenNebulaVM.undeploy_hard,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.shutdown" : {
      type: "multiple",
      call: OpenNebulaVM.shutdown,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.shutdown_hard" : {
      type: "multiple",
      call: OpenNebulaVM.shutdown_hard,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.delete" : {
      type: "multiple",
      call: OpenNebulaVM.del,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.recover" : {
      type: "multiple",
      call: OpenNebulaVM.recover,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.resched" : {
      type: "multiple",
      call: OpenNebulaVM.resched,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.unresched" : {
      type: "multiple",
      call: OpenNebulaVM.unresched,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },

    "RoleVM.chown" : {
      type: "multiple",
      call: OpenNebulaVM.chown,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    },
    "RoleVM.chgrp" : {
      type: "multiple",
      call: OpenNebulaVM.chgrp,
      callback: roleCallback,
      elements: roleVMElements,
      error: Notifier.onError,
      notify: true
    }


  };

  return _actions;
});

define('tabs/oneflow-services-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','opennebula/service','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');
  var OpenNebulaService = require('opennebula/service');

  /*
    CONSTANTS
   */

  var RESOURCE = "Service";
  var XML_ROOT = "DOCUMENT";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"]},
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    };

    this.columns = [
      Locale.tr("ID") ,
      Locale.tr("Owner") ,
      Locale.tr("Group"),
      Locale.tr("Name"),
      Locale.tr("State")
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 4,
      "select_resource": Locale.tr("Please select a Service from the list"),
      "you_selected": Locale.tr("You selected the following Service:"),
      "select_resource_multiple": Locale.tr("Please select one or more Services from the list"),
      "you_selected_multiple": Locale.tr("You selected the following Services:")
    };

    TabDataTable.call(this);
  }

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json[XML_ROOT];

    return [
        '<input class="check_item" type="checkbox" id="' + RESOURCE.toLowerCase() + '_' +
                             element.ID + '" name="selected_items" value="' +
                             element.ID + '"/>',
        element.ID,
        element.UNAME,
        element.GNAME,
        element.NAME,
        OpenNebulaService.stateStr(element.TEMPLATE.BODY.state)
    ];
  }
});


/* START_TEMPLATE */
define('hbs!tabs/oneflow-services-tab/dialogs/scale/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <h3 class=\"subheader\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Scale", {"name":"tr","hash":{},"data":data})))
    + "</h3>\n  </div>\n  <form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\">\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <label for=\"cardinality\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cardinality", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of VMs to instantiate with this role", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </label>\n        <input type=\"text\" name=\"cardinality\" id=\"cardinality\"/>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <input type=\"checkbox\" name=\"force\" id=\"force\"/>\n        <label class=\"inline\" for=\"force\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Force", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Force the new cardinality even if it is outside the limits", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </label>\n      </div>\n    </div>\n    <div class=\"form_buttons row\">\n      <button type=\"submit\" class=\"button radius right\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Scale", {"name":"tr","hash":{},"data":data})))
    + "\n      </button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/oneflow-services-tab/dialogs/scale/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/oneflow-services-tab/dialogs/scale',['require','utils/dialogs/dialog','hbs!./scale/html','sunstone','utils/notifier','utils/locale','utils/tips','./scale/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./scale/html');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./scale/dialogId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setParams = _setParams;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId
    });
  }

  function _onShow(context) {
  }

  function _setup(context) {
    var that = this;

    context.foundation('abide', 'reflow');
    context.off('invalid.fndtn.abide', '#' + DIALOG_ID + 'Form');
    context.off('valid.fndtn.abide', '#' + DIALOG_ID + 'Form');

    context.on('invalid.fndtn.abide', '#' + DIALOG_ID + 'Form', function(e) {
      Notifier.notifyError(Locale.tr("One or more required fields are missing or malformed."));
    }).on('valid.fndtn.abide', '#' + DIALOG_ID + 'Form', function(e) {
      var force = false;
      if ($("#force", context).is(":checked")) {
        force = true;
      }

      var obj = {
        "force": force,
        "cardinality": $("#cardinality", context).val(),
      };

      Sunstone.runAction('Role.update', that.roleIds, obj);

      return false;
    });

    Tips.setup(context);

    return false;
  }

  /**
   * @param {object} params
   *        - params.roleIds : Array of selected role IDs
   */
  function _setParams(params) {
    this.roleIds = params.roleIds;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/oneflow-services-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/valOrDefault'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "yes", {"name":"tr","hash":{},"data":data})))
    + "\n";
},"3":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "no", {"name":"tr","hash":{},"data":data})))
    + "\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.NAME : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Strategy", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.BODY : stack1)) != null ? stack1.deployment : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shutdown action", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">\n            "
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.BODY : stack1)) != null ? stack1.shutdown_action : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "\n          </td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "State", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.stateStr || (depth0 != null ? depth0.stateStr : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"stateStr","hash":{},"data":data}) : helper)))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ready Status Gate", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">\n";
  stack1 = helpers['if'].call(depth0, ((stack1 = ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.BODY : stack1)) != null ? stack1.ready_status_gate : stack1), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.program(3, data),"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "          </td>\n        </tr>\n      </tbody>\n    </table>\n  </div>\n  <div class=\"large-6 columns\">";
  stack1 = ((helper = (helper = helpers.permissionsTableHTML || (depth0 != null ? depth0.permissionsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"permissionsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/oneflow-services-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/oneflow-services-tab/panels/info/panelId',['require'],function(require){
  return 'service_info_tab';
});
define('tabs/oneflow-services-tab/panels/info',['require','utils/locale','utils/panel/permissions-table','opennebula/service','hbs!./info/html','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var PermissionsTable = require('utils/panel/permissions-table');
  var OpenNebulaService = require('opennebula/service');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./info/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var XML_ROOT = "DOCUMENT";
  var RESOURCE = "Service";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var permissionsTableHTML = PermissionsTable.html(TAB_ID, RESOURCE, this.element);

    return TemplateHTML({
      'element': this.element,
      'permissionsTableHTML': permissionsTableHTML,
      'stateStr': OpenNebulaService.stateStr(this.element.TEMPLATE.BODY.state)
    });
  }

  function _setup(context) {
    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);
    PermissionsTable.setup(TAB_ID, RESOURCE, this.element, context);
  }
});

define('tabs/oneflow-services-tab/panels/roles/roles-buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "Role.scale_dialog" : {
      type: "action",
      text: Locale.tr("Scale"),
      layout: "create"
    },
    "Role.hold" : {
      type: "action",
      text: Locale.tr("Hold"),
      tip: Locale.tr("This will hold selected pending VMs from being deployed"),
      layout: "vmsplanification_buttons"
    },
    "Role.release" : {
      type: "action",
      text: Locale.tr("Release"),
      layout: "vmsplanification_buttons",
      tip: Locale.tr("This will release held machines")
    },
    "Role.suspend" : {
      type: "action",
      text: Locale.tr("Suspend"),
      layout: "vmspause_buttons",
      tip: Locale.tr("This will suspend selected machines")
    },
    "Role.resume" : {
      type: "action",
      text: '<i class="fa fa-play"/>',
      layout: "vmsplay_buttons",
      tip: Locale.tr("This will resume selected VMs")
    },
    "Role.stop" : {
      type: "action",
      text: Locale.tr("Stop"),
      layout: "vmsstop_buttons",
      tip: Locale.tr("This will stop selected VMs")
    },
    "Role.reboot" : {
      type: "action",
      text: Locale.tr("Reboot"),
      layout: "vmsrepeat_buttons",
      tip: Locale.tr("This will send a reboot action to running VMs")
    },
    "Role.reboot_hard" : {
      type: "action",
      text: Locale.tr("Reboot") + ' <span class="label secondary radius">hard</span>',
      layout: "vmsrepeat_buttons",
      tip: Locale.tr("This will perform a hard reboot on selected VMs")
    },
    "Role.poweroff" : {
      type: "action",
      text: Locale.tr("Power Off"),
      layout: "vmspause_buttons",
      tip: Locale.tr("This will send a power off signal to running VMs. They can be resumed later.")
    },
    "Role.poweroff_hard" : {
      type: "action",
      text: Locale.tr("Power Off") + ' <span class="label secondary radius">hard</span>',
      layout: "vmspause_buttons",
      tip: Locale.tr("This will send a forced power off signal to running VMs. They can be resumed later.")
    },
    "Role.undeploy" : {
      type: "action",
      text: Locale.tr("Undeploy"),
      layout: "vmsstop_buttons",
      tip: Locale.tr("Shuts down the given VM. The VM is saved in the system Datastore.")
    },
    "Role.undeploy_hard" : {
      type: "action",
      text: Locale.tr("Undeploy") + ' <span class="label secondary radius">hard</span>',
      layout: "vmsstop_buttons",
      tip: Locale.tr("Shuts down the given VM. The VM is saved in the system Datastore.")
    },
    "Role.shutdown" : {
      type: "action",
      text: Locale.tr("Shutdown"),
      layout: "vmsdelete_buttons",
      tip: Locale.tr("This will initiate the shutdown process in the selected VMs")
    },
    "Role.shutdown_hard" : {
      type: "action",
      text: Locale.tr("Shutdown") + ' <span class="label secondary radius">hard</span>',
      layout: "vmsdelete_buttons",
      tip: Locale.tr("This will initiate the shutdown-hard (forced) process in the selected VMs")
    },
    "Role.delete" : {
      type: "action",
      text: Locale.tr("Delete"),
      layout: "vmsdelete_buttons",
      tip: Locale.tr("This will delete the selected VMs from the database")
    },
    "Role.delete_recreate" : {
      type: "action",
      text: Locale.tr("Delete") + ' <span class="label secondary radius">recreate</span>',
      layout: "vmsrepeat_buttons",
      tip: Locale.tr("This will delete and recreate VMs to PENDING state")
    }
  };

  return Buttons;
});
define('tabs/oneflow-services-tab/panels/roles/roles-vm-buttons',['require','utils/locale','tabs/vms-tab/buttons'],function(require) {
  var Locale = require('utils/locale');
  var VMButtons = require('tabs/vms-tab/buttons');

  var Buttons = {
    "RoleVM.hold" : VMButtons["VM.hold"],
    "RoleVM.release" : VMButtons["VM.release"],
    "RoleVM.suspend" : VMButtons["VM.suspend"],
    "RoleVM.resume" : VMButtons["VM.resume"],
    "RoleVM.stop" : VMButtons["VM.stop"],
    "RoleVM.reboot" : VMButtons["VM.reboot"],
    "RoleVM.reboot_hard" : VMButtons["VM.reboot_hard"],
    "RoleVM.poweroff" : VMButtons["VM.poweroff"],
    "RoleVM.poweroff_hard" : VMButtons["VM.poweroff_hard"],
    "RoleVM.undeploy" : VMButtons["VM.undeploy"],
    "RoleVM.undeploy_hard" : VMButtons["VM.undeploy_hard"],
    "RoleVM.shutdown" : VMButtons["VM.shutdown"],
    "RoleVM.shutdown_hard" : VMButtons["VM.shutdown_hard"],
    "RoleVM.delete" : VMButtons["VM.delete"],
    "RoleVM.delete_recreate" : VMButtons["VM.delete_recreate"],
    "RoleVM.resched" : VMButtons["VM.resched"],
    "RoleVM.unresched" : VMButtons["VM.unresched"]
  };

  return Buttons;
});

/* START_TEMPLATE */
define('hbs!tabs/oneflow-services-tab/panels/roles/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "    <div id=\"role_actions\">\n      <div class=\"columns large-4 medium-6 right\">\n        <div class=\"row\">\n          <div class=\"small-6 columns\">\n            <label for=\"batch_action_period\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Period", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Seconds between each group of actions", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n            <input type=\"text\" id=\"batch_action_period\" name=\"batch_action_period\"/>\n          </div>\n          <div class=\"small-6 columns\">\n            <label for=\"batch_action_number\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number", {"name":"tr","hash":{},"data":data})))
    + ":\n              <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of VMs to apply the action to each period", {"name":"tr","hash":{},"data":data})))
    + "</span>\n            </label>\n            <input type=\"text\" id=\"batch_action_number\" name=\"batch_action_number\"/>\n          </div>\n        </div>\n      </div>\n      <div class=\"action_blocks columns large-12\">\n      </div>\n    </div>\n";
},"3":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "State", {"name":"tr","hash":{},"data":data})))
    + "</th>\n";
},"5":function(depth0,helpers,partials,data,depths) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, buffer = "          <tr>\n            <td class=\"hidden\">\n              <input class=\"check_item\" type=\"checkbox\" id=\"role_"
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "\" name=\"selected_items\" value=\""
    + escapeExpression(lambda(((stack1 = (depths[1] != null ? depths[1].element : depths[1])) != null ? stack1.ID : stack1), depth0))
    + "/role/"
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "\"/>\n            </td>\n            <td>"
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "</td>\n";
  stack1 = helpers['if'].call(depth0, (depths[1] != null ? depths[1].servicePanel : depths[1]), {"name":"if","hash":{},"fn":this.program(6, data, depths),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "            <td>"
    + escapeExpression(((helper = (helper = helpers.cardinality || (depth0 != null ? depth0.cardinality : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"cardinality","hash":{},"data":data}) : helper)))
    + "</td>\n            <td>"
    + escapeExpression(((helper = (helper = helpers.vm_template || (depth0 != null ? depth0.vm_template : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"vm_template","hash":{},"data":data}) : helper)))
    + "</td>\n            <td>"
    + escapeExpression(((helper = (helper = helpers.parents || (depth0 != null ? depth0.parents : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"parents","hash":{},"data":data}) : helper)))
    + "</td>\n          </tr>\n";
},"6":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "            <td>"
    + escapeExpression(((helper = (helper = helpers.state || (depth0 != null ? depth0.state : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"state","hash":{},"data":data}) : helper)))
    + "</td>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data,depths) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form class=\"custom\" id=\"roles_form\" action=\"\">\n  <div class=\"row\">\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.servicePanel : depth0), {"name":"if","hash":{},"fn":this.program(1, data, depths),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "    <div id=\"roles_info\" class=\"columns large-12\">\n      <table id=\"datatable_roles_"
    + escapeExpression(((helper = (helper = helpers.panelId || (depth0 != null ? depth0.panelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"panelId","hash":{},"data":data}) : helper)))
    + "\" class=\"dataTable twelve\">\n        <thead>\n          <tr>\n            <th class=\"check hidden\">\n              <input type=\"checkbox\" class=\"check_all\" value=\"\"></input>\n            </th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.servicePanel : depth0), {"name":"if","hash":{},"fn":this.program(3, data, depths),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cardinality", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VM Template", {"name":"tr","hash":{},"data":data})))
    + "</th>\n            <th>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Parents", {"name":"tr","hash":{},"data":data})))
    + "</th>\n          </tr>\n        </thead>\n        <tbody>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.roleList : depth0), {"name":"each","hash":{},"fn":this.program(5, data, depths),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "        </tbody>\n      </table>\n    </div>\n    <div id=\"roles_extended_info\" class=\"columns twelve\">\n      <div class=\"text-center\">\n        <span class=\"fa-stack fa-5x\" style=\"color: #dfdfdf\">\n          <i class=\"fa fa-cloud fa-stack-2x\"></i>\n          <i class=\"fa fa-info-circle fa-stack-1x fa-inverse\"></i>\n        </span>\n        <br>\n        <br>\n        <span style=\"font-size: 18px; color: #999\">\n          "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Select a role in the table for more information", {"name":"tr","hash":{},"data":data})))
    + "\n        </span>\n      </div>\n    </div>\n  </div>\n</form>";
},"useData":true,"useDepths":true});
Handlebars.registerPartial('tabs/oneflow-services-tab/panels/roles/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/oneflow-services-tab/panels/roles/roleInfo',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/valOrDefault'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", lambda=this.lambda, buffer = "  <fieldset>\n    <legend>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Virtual Machines", {"name":"tr","hash":{},"data":data})))
    + "</legend>\n    <div id=\"role_vms_actions\">\n      <div class=\"action_blocks large-12 columns\">\n      </div>\n    </div>\n    <div class=\"large-12 columns\">\n      <br>\n      <table id=\"datatable_vms_"
    + escapeExpression(((helper = (helper = helpers.panelId || (depth0 != null ? depth0.panelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"panelId","hash":{},"data":data}) : helper)))
    + "_"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.role : depth0)) != null ? stack1.name : stack1), depth0))
    + "\" class=\"dataTable twelve \">\n        <thead>\n          <tr>\n            <th></th>\n            <th></th>\n            <th class=\"check\">\n              <input type=\"checkbox\" class=\"check_all\" value=\"\"></input>\n            </th>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.vmsTableColumns : depth0), {"name":"each","hash":{},"fn":this.program(2, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "          </tr>\n        </thead>\n        <tbody>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.vms : depth0), {"name":"each","hash":{},"fn":this.program(4, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "        </tbody>\n      </table>\n    </div>\n  </fieldset>\n";
},"2":function(depth0,helpers,partials,data) {
  var lambda=this.lambda, escapeExpression=this.escapeExpression;
  return "            <th>"
    + escapeExpression(lambda(depth0, depth0))
    + "</th>\n";
},"4":function(depth0,helpers,partials,data) {
  var stack1, buffer = "          <tr>\n";
  stack1 = helpers.each.call(depth0, depth0, {"name":"each","hash":{},"fn":this.program(5, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "          </tr>\n";
},"5":function(depth0,helpers,partials,data) {
  var stack1, lambda=this.lambda, buffer = "            <td>";
  stack1 = lambda(depth0, depth0);
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</td>\n";
},"7":function(depth0,helpers,partials,data,depths) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "    <table class=\"dataTable extended_table policies_table\">\n      <thead style=\"background:#dfdfdf\">\n        <tr>\n          <th colspan=\"7\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Elasticity policies", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <thead>\n        <tr>\n          <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type of adjustment.", {"name":"tr","hash":{},"data":data})))
    + "<br><br>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CHANGE: Add/substract the given number of VMs.", {"name":"tr","hash":{},"data":data})))
    + "<br>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CARDINALITY: Set the cardinality to the given number.", {"name":"tr","hash":{},"data":data})))
    + "<br>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PERCENTAGE_CHANGE: Add/substract the given percentage to the current cardinality.", {"name":"tr","hash":{},"data":data})))
    + "\n            \" style=\"width:14%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n          <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Positive or negative adjustment. Its meaning depends on 'type'", {"name":"tr","hash":{},"data":data})))
    + "<br><br>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CHANGE: -2, will substract 2 VMs from the role", {"name":"tr","hash":{},"data":data})))
    + "<br>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CARDINALITY: 8, will set carditanilty to 8", {"name":"tr","hash":{},"data":data})))
    + "<br>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PERCENTAGE_CHANGE: 20, will increment cardinality by 20%", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:12%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Adjust", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n          <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Optional parameter for PERCENTAGE_CHANGE adjustment type.", {"name":"tr","hash":{},"data":data})))
    + "<br>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, " If present, the policy will change the cardinality by at least the number of VMs set in this attribute.", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:9%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Min", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n          <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Expression to trigger the elasticity", {"name":"tr","hash":{},"data":data})))
    + "<br><br>\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Example: ATT < 20", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:30%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Expression", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n          <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of periods that the expression must be true before the elasticity is triggered", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:8%\">#\n          </th>\n          <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Duration, in seconds, of each period in '# Periods'", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:9%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Period", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n          <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cooldown period duration after a scale operation, in seconds", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:15%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cooldown", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n          <th style=\"width:3%\"></th>\n        </tr>\n      </thead>\n      <tbody>\n";
  stack1 = helpers.each.call(depth0, ((stack1 = (depth0 != null ? depth0.role : depth0)) != null ? stack1.elasticity_policies : stack1), {"name":"each","hash":{},"fn":this.program(8, data, depths),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "      </tbody>\n    </table>\n";
},"8":function(depth0,helpers,partials,data,depths) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "        <tr>\n          <td>"
    + escapeExpression(((helper = (helper = helpers.type || (depth0 != null ? depth0.type : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"type","hash":{},"data":data}) : helper)))
    + "</td>\n          <td>"
    + escapeExpression(((helper = (helper = helpers.adjust || (depth0 != null ? depth0.adjust : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"adjust","hash":{},"data":data}) : helper)))
    + "</td>\n          <td>"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, (depth0 != null ? depth0.min_adjust_step : depth0), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td>"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, (depth0 != null ? depth0.expression_evaluated : depth0), (depth0 != null ? depth0.expression : depth0), {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td>\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.period_number : depth0), {"name":"if","hash":{},"fn":this.program(9, data, depths),"inverse":this.program(14, data, depths),"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "          </td>\n          <td>"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, (depth0 != null ? depth0.period : depth0), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td>"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, (depth0 != null ? depth0.cooldown : depth0), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n        </tr>\n";
},"9":function(depth0,helpers,partials,data,depths) {
  var stack1, buffer = "";
  stack1 = helpers['if'].call(depth0, (depths[2] != null ? depths[2].servicePanel : depths[2]), {"name":"if","hash":{},"fn":this.program(10, data, depths),"inverse":this.program(12, data, depths),"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"10":function(depth0,helpers,partials,data) {
  var helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function";
  return "            "
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, (depth0 != null ? depth0.true_evals : depth0), "0", {"name":"valOrDefault","hash":{},"data":data})))
    + "/"
    + escapeExpression(((helper = (helper = helpers.period_number || (depth0 != null ? depth0.period_number : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"period_number","hash":{},"data":data}) : helper)))
    + "\n";
},"12":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "            "
    + escapeExpression(((helper = (helper = helpers.period_number || (depth0 != null ? depth0.period_number : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"period_number","hash":{},"data":data}) : helper)))
    + "\n";
},"14":function(depth0,helpers,partials,data) {
  return "            -\n";
  },"16":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "    <table class=\"dataTable extended_table policies_table\">\n      <thead style=\"background:#dfdfdf\">\n        <tr>\n          <th colspan=\"5\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Scheduled policies", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <thead>\n        <tr>\n          <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type of adjustment.", {"name":"tr","hash":{},"data":data})))
    + "\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CHANGE: Add/substract the given number of VMs.", {"name":"tr","hash":{},"data":data})))
    + "\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CARDINALITY: Set the cardinality to the given number.", {"name":"tr","hash":{},"data":data})))
    + "\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PERCENTAGE_CHANGE: Add/substract the given percentage to the current cardinality.", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:14%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n          <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Positive or negative adjustment. Its meaning depends on 'type'", {"name":"tr","hash":{},"data":data})))
    + "\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CHANGE: -2, will substract 2 VMs from the role", {"name":"tr","hash":{},"data":data})))
    + "\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CARDINALITY: 8, will set carditanilty to 8", {"name":"tr","hash":{},"data":data})))
    + "\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PERCENTAGE_CHANGE: 20, will increment cardinality by 20%", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:12%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Adjust", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n          <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Optional parameter for PERCENTAGE_CHANGE adjustment type. If present, the policy will change the cardinality by at least the number of VMs set in this attribute.", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:9%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Min", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n          <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Recurrence: Time for recurring adjustements. Time is specified with the Unix cron syntax", {"name":"tr","hash":{},"data":data})))
    + "\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Start time: Exact time for the adjustement", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:28%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Time format", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n          <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Time expression depends on the the time formar selected", {"name":"tr","hash":{},"data":data})))
    + "\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Recurrence: Time for recurring adjustements. Time is specified with the Unix cron syntax", {"name":"tr","hash":{},"data":data})))
    + "\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Start time: Exact time for the adjustement", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:33%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Time expression", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n          <th style=\"width:3%\"></th>\n        </tr>\n      </thead>\n      <tbody>\n";
  stack1 = helpers.each.call(depth0, ((stack1 = (depth0 != null ? depth0.role : depth0)) != null ? stack1.scheduled_policies : stack1), {"name":"each","hash":{},"fn":this.program(17, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "      </tbody>\n    </table>\n";
},"17":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "        <tr>\n          <td>"
    + escapeExpression(((helper = (helper = helpers.type || (depth0 != null ? depth0.type : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"type","hash":{},"data":data}) : helper)))
    + "</td>\n          <td>"
    + escapeExpression(((helper = (helper = helpers.adjust || (depth0 != null ? depth0.adjust : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"adjust","hash":{},"data":data}) : helper)))
    + "</td>\n          <td>"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, (depth0 != null ? depth0.min_adjust_step : depth0), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.start_time : depth0), {"name":"if","hash":{},"fn":this.program(18, data),"inverse":this.program(20, data),"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "        </tr>\n";
},"18":function(depth0,helpers,partials,data) {
  var helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function";
  return "          <td>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Start time", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td>"
    + escapeExpression(((helper = (helper = helpers.start_time || (depth0 != null ? depth0.start_time : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"start_time","hash":{},"data":data}) : helper)))
    + "</td>\n";
},"20":function(depth0,helpers,partials,data) {
  var stack1, buffer = "";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.recurrence : depth0), {"name":"if","hash":{},"fn":this.program(21, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"21":function(depth0,helpers,partials,data) {
  var helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function";
  return "          <td>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Recurrence", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td>"
    + escapeExpression(((helper = (helper = helpers.recurrence || (depth0 != null ? depth0.recurrence : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"recurrence","hash":{},"data":data}) : helper)))
    + "</td>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data,depths) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, buffer = "<form>\n  <h4>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Role", {"name":"tr","hash":{},"data":data})))
    + " - "
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.role : depth0)) != null ? stack1.name : stack1), depth0))
    + "</h4>\n  <div class='large-12 columns'>\n    <table class='dataTable extended_table policies_table'>\n      <thead>\n        <tr>\n          <th colspan='8'>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class='key_td'>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shutdown action", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class='value_td'>"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = (depth0 != null ? depth0.role : depth0)) != null ? stack1.shutdown_action : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td class='key_td'>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cooldown", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class='value_td'>"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = (depth0 != null ? depth0.role : depth0)) != null ? stack1.cooldown : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td class='key_td'>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Min VMs", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class='value_td'>"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = (depth0 != null ? depth0.role : depth0)) != null ? stack1.min_vms : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n          <td class='key_td'>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Max VMs", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class='value_td'>"
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = (depth0 != null ? depth0.role : depth0)) != null ? stack1.max_vms : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "</td>\n        </tr>\n      </tbody>\n    </table>\n    <br>\n  </div>\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.servicePanel : depth0), {"name":"if","hash":{},"fn":this.program(1, data, depths),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "  <div class='large-12 columns'>\n    <br>\n";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.role : depth0)) != null ? stack1.elasticity_policies : stack1), {"name":"if","hash":{},"fn":this.program(7, data, depths),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "    <br>\n";
  stack1 = helpers['if'].call(depth0, ((stack1 = (depth0 != null ? depth0.role : depth0)) != null ? stack1.scheduled_policies : stack1), {"name":"if","hash":{},"fn":this.program(16, data, depths),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  </div>\n</form>";
},"useData":true,"useDepths":true});
Handlebars.registerPartial('tabs/oneflow-services-tab/panels/roles/roleInfo', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/oneflow-services-tab/panels/roles',['require','utils/locale','utils/tips','opennebula/role','./roles/roles-buttons','./roles/roles-vm-buttons','sunstone','utils/dom-datatable','tabs/vms-tab/utils/datatable-common','sunstone-config','utils/vnc','utils/spice','utils/notifier','tabs/vms-tab/tabId','hbs!./roles/html','hbs!./roles/roleInfo','../tabId','./roles/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var OpenNebulaRole = require('opennebula/role');
  var roles_buttons = require('./roles/roles-buttons');
  var roles_vm_buttons = require('./roles/roles-vm-buttons');
  var Sunstone = require('sunstone');
  var DomDataTable = require('utils/dom-datatable');
  var VMsTableUtils = require('tabs/vms-tab/utils/datatable-common');
  var SunstoneConfig = require('sunstone-config');
  var Vnc = require('utils/vnc');
  var Spice = require('utils/spice');
  var Notifier = require('utils/notifier');

  var VMS_TAB_ID = require('tabs/vms-tab/tabId');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./roles/html');
  var TemplateRoleInfo = require('hbs!./roles/roleInfo');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./roles/panelId');
  var XML_ROOT = "DOCUMENT";
  var RESOURCE = "Service";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Roles");
    this.icon = "fa-wrench";

    this.element = info[XML_ROOT];

    this.selected_row_role_id = undefined;

    // Controls visibility of buttons only available to OneFlow services. This
    // panel is also used by the OneFlow templates
    this.servicePanel = true;

    this.panelId = PANEL_ID;

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;
  Panel.prototype.getState = _getState;
  Panel.prototype.setState = _setState;
  Panel.prototype.roleHTML = _roleHTML;
  Panel.prototype.roleSetup = _roleSetup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var roleList = [];

    var roles = this.element.TEMPLATE.BODY.roles;
    if (roles && roles.length) {
      $.each(roles, function(){
        roleList.push(
          {
            'name': this.name,
            'state': OpenNebulaRole.state(this.state),
            'cardinality': this.cardinality,
            'vm_template': this.vm_template,
            'parents': this.parents ? this.parents.join(', ') : '-'
          });
      });
    }

    return TemplateHTML({
      'element': this.element,
      'panelId': this.panelId,
      'servicePanel': this.servicePanel,
      'roleList': roleList
    });
  }

  function _getState(context) {
    var state = {};

    if (this.servicerolesDataTable){
      var selectedCheck = $(".check_item:checked", this.servicerolesDataTable.dataTable);

      if (selectedCheck.length > 0){
        state["selectedRole"] = selectedCheck.attr("id");

        if(this.serviceroleVMsDataTable){
          var selectedVMs = [];

          $.each($(".check_item:checked", this.serviceroleVMsDataTable.dataTable), function(){
            selectedVMs.push($(this).attr("id"));
          });

          if (selectedVMs.length > 0){
            state["selectedVMs"] = selectedVMs;
          }
        }
      }
    }

    return state;
  }

  function _setState(state, context) {
    var that = this;

    if (this.servicerolesDataTable && state["selectedRole"]){
      $('.check_item[id="'+state["selectedRole"]+'"]', this.servicerolesDataTable.dataTable).closest('tr').click();
    }

    if (this.serviceroleVMsDataTable && state["selectedVMs"]){
      $.each(state["selectedVMs"], function(){
        $('.check_item[id="'+this+'"]', that.serviceroleVMsDataTable.dataTable).closest('tr').click();
      });
    }
  }

  function _setup(context) {
    var that = this;

    Tips.setup(context);

    that.last_selected_row_role = undefined;

    var roles = this.element.TEMPLATE.BODY.roles;
    if (roles && roles.length) {
      this.servicerolesDataTable = new DomDataTable(
        'datatable_roles_'+this.panelId,
        {
          actions: true,
          info: false,
          oneSelection: true,
          customTabContext: $('#role_actions', context),
          customTrListener: function(tableObj, tr){
            var aData = tableObj.dataTable.fnGetData(tr);
            var role_name = $(aData[0]).val();

            var role_index = tableObj.dataTable.fnGetPosition(tr);

            $("#roles_extended_info", context).html(that.roleHTML(role_index));
            that.roleSetup($("#roles_extended_info", context), role_index);

            // The info listener is triggered instead of
            // the row selection. So we click the check input to select
            // the row also
            var check = $('.check_item', tr);
            if (!check.is(":checked")) {
              check.trigger('click');
            }
          }
        });

      this.servicerolesDataTable.initialize();

      Sunstone.insertButtonsInTab("oneflow-services", "service_roles_tab", roles_buttons, $('#role_actions', context));
    }
  }


  function _roleHTML(role_index) {
    var that = this;

    var role = this.element.TEMPLATE.BODY.roles[role_index];

    var vms = [];

    if (role.nodes) {
      $.each(role.nodes, function(){
        var vm_info = this.vm_info;

        var info = [];
        if (this.scale_up) {
          info.push("<i class='fa fa-arrow-up'/>");
        } else if (this.disposed) {
          info.push("<i class='fa fa-arrow-down'/>");
        } else {
          info.push("");
        }

        if (that.element.TEMPLATE.BODY.ready_status_gate) {
          if (vm_info.VM.USER_TEMPLATE.READY == "YES") {
            info.push('<span class="has-tip" title="'+Locale.tr("The VM is ready")+'"><i class="fa fa-check"/></span>');

          } else {
            info.push('<span class="has-tip" title="'+Locale.tr("Waiting for the VM to be ready")+'"><i class="fa fa-clock-o"/></span>');
          }
        } else {
          info.push("");
        }

        if (vm_info) {
          vms.push(info.concat(VMsTableUtils.elementArray(vm_info)));
        } else {
          vms.push(info.concat(VMsTableUtils.emptyElementArray(this.deploy_id)));
        }
      });
    }

    return TemplateRoleInfo({
      'role': role,
      'servicePanel': this.servicePanel,
      'panelId': this.panelId,
      'vmsTableColumns': VMsTableUtils.columns,
      'vms': vms
    });
  }

  function _roleSetup(context, role_index) {
    if(this.servicePanel) {
      var role = this.element.TEMPLATE.BODY.roles[role_index];

      $(".vnc", context).off("click");
      $(".vnc", context).on("click", function() {
        var vmId = $(this).attr('vm_id');

        if (!Vnc.lockStatus()) {
          Vnc.lock();
          Sunstone.runAction("VM.startvnc_action", vmId);
        } else {
          Notifier.notifyError(Locale.tr("VNC Connection in progress"));
        }

        return false;
      });

      $(".spice", context).off("click");
      $(".spice", context).on("click", function() {
        var vmId = $(this).attr('vm_id');

        if (!Spice.lockStatus()) {
          Spice.lock();
          Sunstone.runAction("VM.startspice_action", vmId);
        } else {
          Notifier.notifyError(Locale.tr("SPICE Connection in progress"));
        }

        return false;
      });

      // This table has 2 more columns to the left compared to the normal VM table
      // The visibility index array needs to be adjusted
      var visibleColumns = [0,1].concat(
        SunstoneConfig.tabTableColumns(VMS_TAB_ID).map(function(n){
          return n+2;
        }));

      this.serviceroleVMsDataTable = new DomDataTable(
        'datatable_vms_'+this.panelId+'_'+role.name,
        {
          actions: true,
          info: false,
          customTabContext: $('#role_vms_actions', context),
          dataTableOptions: {
            "bAutoWidth": false,
            "bSortClasses" : false,
            "bDeferRender": true,
            "aoColumnDefs": [
              {"bSortable": false, "aTargets": [0,1,"check"]},
              {"bVisible": true, "aTargets": visibleColumns},
              {"bVisible": false, "aTargets": ['_all']}
            ]
          }
        });

      this.serviceroleVMsDataTable.initialize();
      Sunstone.insertButtonsInTab(
        "oneflow-services",
        "service_roles_tab",
        roles_vm_buttons,
        $('div#role_vms_actions', context));
    }

    Tips.setup(context);
  }
});

define('tabs/oneflow-services-tab/panels/log/panelId',['require'],function(require){
  return 'service_log_tab';
});
define('tabs/oneflow-services-tab/panels/log',['require','utils/locale','utils/humanize','../tabId','./log/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Humanize = require('utils/humanize');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./log/panelId');
  var XML_ROOT = "DOCUMENT";
  var RESOURCE = "Service";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Log");
    this.icon = "fa-file-text";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var logs = this.element.TEMPLATE.BODY.log;
    var log_info = '';
    if (logs) {
      log_info += '<div class="row"><div class="large-12 columns log-tab">';

      for (var i = 0; i < logs.length; i++) {
        var line = Humanize.prettyTime(logs[i].timestamp)+' ['+logs[i].severity + '] ' + logs[i].message+ '<br>';

        if (logs[i].severity == 'E'){
          line = '<span class="vm_log_error">'+line+'</span>';
        }

        log_info += line;
      }

      log_info += '</div></div>';
    }

    return log_info;
  }

  function _setup(context) {
  }
});

define('tabs/oneflow-services-tab',['require','utils/locale','./oneflow-services-tab/buttons','./oneflow-services-tab/actions','./oneflow-services-tab/datatable','./oneflow-services-tab/tabId','./oneflow-services-tab/dialogs/scale','./oneflow-services-tab/panels/info','./oneflow-services-tab/panels/roles','./oneflow-services-tab/panels/log'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./oneflow-services-tab/buttons');
  var Actions = require('./oneflow-services-tab/actions');
  var Table = require('./oneflow-services-tab/datatable');

  var TAB_ID = require('./oneflow-services-tab/tabId');
  var DATATABLE_ID = "dataTableService";
  var RESOURCE = "Service";

  var _dialogs = [
    require('./oneflow-services-tab/dialogs/scale'),
  ];

  var _panels = [
    require('./oneflow-services-tab/panels/info'),
    require('./oneflow-services-tab/panels/roles'),
    require('./oneflow-services-tab/panels/log')
  ];

  var _formPanels = [
  ];

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("Services"),
    tabClass: "subTab",
    parentTab: "oneflow-dashboard",
    listHeader: '<i class="fa fa-fw fa-cubes"></i>&emsp;'+Locale.tr("OneFlow - Services"),
    infoHeader: '<i class="fa fa-fw fa-cubes"></i>&emsp;'+Locale.tr("OneFlow - Service"),
    subheader: '',
    content: '<div class="row oneflow_services_error_message" hidden>\
        <div class="small-6 columns small-centered text-center">\
            <div class="alert-box alert radius">'+Locale.tr("Cannot connect to OneFlow server")+'</div>\
        </div>\
    </div>',
    resource: RESOURCE,
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});

define('tabs/oneflow-templates-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "ServiceTemplate.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "ServiceTemplate.create_dialog" : {
      type: "create_dialog",
      layout: "create"
    },
    "ServiceTemplate.instantiate_dialog" : {
      type: "action",
      layout: "main",
      text: Locale.tr("Instantiate")
    },
    "ServiceTemplate.update_dialog" : {
      type: "action",
      layout: "main",
      text: Locale.tr("Update")
    },
    "ServiceTemplate.chown" : {
      type: "confirm_with_select",
      text: Locale.tr("Change owner"),
      select: "User",
      layout: "user_select",
      tip: Locale.tr("Select the new owner")+":"
    },
    "ServiceTemplate.chgrp" : {
      type: "confirm_with_select",
      text: Locale.tr("Change group"),
      select: "Group",
      layout: "user_select",
      tip: Locale.tr("Select the new group")+":"
    },
    "ServiceTemplate.delete" : {
      type: "confirm",
      text: Locale.tr("Delete"),
      layout: "del",
      tip: Locale.tr("This will delete the selected templates")
    }
  };

  return Buttons;
});
define('tabs/oneflow-templates-tab/tabId',['require'],function(require){
  return 'oneflow-templates';
});
define('tabs/oneflow-templates-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createServiceTemplateForm';
});
define('tabs/oneflow-templates-tab/form-panels/instantiate/formPanelId',['require'],function(require){
  return 'instantiateServiceTemplateForm';
});
define('tabs/oneflow-templates-tab/actions',['require','sunstone','utils/notifier','utils/locale','opennebula/servicetemplate','utils/common-actions','opennebula/action','./tabId','./form-panels/create/formPanelId','./form-panels/instantiate/formPanelId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var Locale = require('utils/locale');
  var OpenNebulaResource = require('opennebula/servicetemplate');
  var CommonActions = require('utils/common-actions');
  var OpenNebulaAction = require('opennebula/action');

  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');
  var INSTANTIATE_DIALOG_ID = require('./form-panels/instantiate/formPanelId');
  var XML_ROOT = "DOCUMENT";
  var RESOURCE = "ServiceTemplate";

  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);

  var _actions = {
    "ServiceTemplate.create" : _commonActions.create(CREATE_DIALOG_ID),
    "ServiceTemplate.create_dialog" : _commonActions.showCreate(CREATE_DIALOG_ID),
    "ServiceTemplate.show" : _commonActions.show(),
    "ServiceTemplate.refresh" : _commonActions.refresh(),
    "ServiceTemplate.delete" : _commonActions.del(),
    "ServiceTemplate.chown": _commonActions.multipleAction('chown'),
    "ServiceTemplate.chgrp": _commonActions.multipleAction('chgrp'),
    "ServiceTemplate.chmod": _commonActions.singleAction('chmod'),
    "ServiceTemplate.update" : _commonActions.update(),
    "ServiceTemplate.update_dialog" : _commonActions.checkAndShowUpdate(),
    "ServiceTemplate.show_to_update" : _commonActions.showUpdate(CREATE_DIALOG_ID),

    "ServiceTemplate.list" : {
      type: "list",
      call: OpenNebulaResource.list,
      callback: function(request, response) {
        $(".oneflow_templates_error_message").hide();
        Sunstone.getDataTable(TAB_ID).updateView(request, response);
      },
      error: function(request, error_json) {
        Notifier.onError(request, error_json, $(".oneflow_templates_error_message"));
      }
    },


    "ServiceTemplate.instantiate" : {
      type: "single",
      call: OpenNebulaResource.instantiate,
      callback: function(request, response){
        Sunstone.hideFormPanel(TAB_ID);
        OpenNebulaAction.clear_cache("SERVICE");
      },
      elements: function() {
        return Sunstone.getDataTable(TAB_ID).elements();
      },
      error: function(request, response){
        Sunstone.hideFormPanelLoading(TAB_ID);
        Notifier.onError(request, response);
      },
      notify: true
    },

    "ServiceTemplate.instantiate_dialog" : {
      type: "custom",
      call: function() {
        var selected_nodes = Sunstone.getDataTable(TAB_ID).elements();
        if (selected_nodes.length != 1) {
          Notifier.notifyMessage("Please select one (and just one) template to instantiate.");
          return false;
        }

        var templateId = "" + selected_nodes[0];

        Sunstone.resetFormPanel(TAB_ID, INSTANTIATE_DIALOG_ID);
        Sunstone.showFormPanel(TAB_ID, INSTANTIATE_DIALOG_ID, "instantiate",
          function(formPanelInstance, context) {
            formPanelInstance.setTemplateId(context, templateId);
          });
      }
    },

  };

  return _actions;
});
define('tabs/oneflow-templates-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');

  /*
    CONSTANTS
   */

  var RESOURCE = "ServiceTemplate";
  var XML_ROOT = "DOCUMENT";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"]},
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    };

    this.columns = [
      Locale.tr("ID") ,
      Locale.tr("Owner") ,
      Locale.tr("Group"),
      Locale.tr("Name")
    ];

    this.selectOptions = {
      "id_index": 1,
      "name_index": 4,
      "select_resource": Locale.tr("Please select a Template from the list"),
      "you_selected": Locale.tr("You selected the following Template:"),
      "select_resource_multiple": Locale.tr("Please select one or more Templates from the list"),
      "you_selected_multiple": Locale.tr("You selected the following Templates:")
    };

    TabDataTable.call(this);
  }

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element_json) {
    var element = element_json[XML_ROOT];

    return [
        '<input class="check_item" type="checkbox" id="' + RESOURCE.toLowerCase() + '_' +
                             element.ID + '" name="selected_items" value="' +
                             element.ID + '"/>',
        element.ID,
        element.UNAME,
        element.GNAME,
        element.NAME
    ];
  }
});


/* START_TEMPLATE */
define('hbs!tabs/oneflow-templates-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/valOrDefault'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "yes", {"name":"tr","hash":{},"data":data})))
    + "\n";
},"3":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "no", {"name":"tr","hash":{},"data":data})))
    + "\n";
},"5":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "    <table id=\"info_template_table\" class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network Configuration", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.customAttrs : depth0), {"name":"each","hash":{},"fn":this.program(6, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "    </table>\n";
},"6":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "      <tr>\n        <td class=\"key_td\">"
    + escapeExpression(((helper = (helper = helpers.name || (depth0 != null ? depth0.name : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"name","hash":{},"data":data}) : helper)))
    + "</td>\n        <td class=\"value_td\">"
    + escapeExpression(((helper = (helper = helpers.description || (depth0 != null ? depth0.description : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"description","hash":{},"data":data}) : helper)))
    + "</td>\n      </tr>\n      <tr>\n        <td class=\"key_td\"></td>\n        <td class=\"value_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Roles", {"name":"tr","hash":{},"data":data})))
    + ": "
    + escapeExpression(((helper = (helper = helpers.roles || (depth0 != null ? depth0.roles : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"roles","hash":{},"data":data}) : helper)))
    + "</td>\n      </tr>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, lambda=this.lambda, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"large-6 columns\">\n    <table class=\"dataTable extended_table\">\n      <thead>\n        <tr>\n          <th colspan=\"2\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Information", {"name":"tr","hash":{},"data":data})))
    + "</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "ID", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.ID : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.NAME : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\" colspan=\"2\">"
    + escapeExpression(lambda(((stack1 = ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.BODY : stack1)) != null ? stack1.description : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Strategy", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.BODY : stack1)) != null ? stack1.deployment : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shutdown action", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">\n            "
    + escapeExpression(((helpers.valOrDefault || (depth0 && depth0.valOrDefault) || helperMissing).call(depth0, ((stack1 = ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.BODY : stack1)) != null ? stack1.shutdown_action : stack1), "-", {"name":"valOrDefault","hash":{},"data":data})))
    + "\n          </td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Ready Status Gate", {"name":"tr","hash":{},"data":data})))
    + "</td>\n          <td class=\"value_td\">\n";
  stack1 = helpers['if'].call(depth0, ((stack1 = ((stack1 = ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.TEMPLATE : stack1)) != null ? stack1.BODY : stack1)) != null ? stack1.ready_status_gate : stack1), {"name":"if","hash":{},"fn":this.program(1, data),"inverse":this.program(3, data),"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "          </td>\n        </tr>\n      </tbody>\n    </table>\n";
  stack1 = helpers['if'].call(depth0, (depth0 != null ? depth0.customAttrs : depth0), {"name":"if","hash":{},"fn":this.program(5, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  buffer += "  </div>\n  <div class=\"large-6 columns\">";
  stack1 = ((helper = (helper = helpers.permissionsTableHTML || (depth0 != null ? depth0.permissionsTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"permissionsTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "</div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/oneflow-templates-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/oneflow-templates-tab/panels/info/panelId',['require'],function(require){
  return 'service_template_info_tab';
});
define('tabs/oneflow-templates-tab/panels/info',['require','utils/locale','utils/panel/permissions-table','utils/template-utils','hbs!./info/html','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var PermissionsTable = require('utils/panel/permissions-table');
  var TemplateUtils = require('utils/template-utils');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./info/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var XML_ROOT = "DOCUMENT";
  var RESOURCE = "ServiceTemplate";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = Locale.tr("Info");
    this.icon = "fa-info-circle";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var that = this;

    var permissionsTableHTML = PermissionsTable.html(TAB_ID, RESOURCE, this.element);

    var customAttrs = [];

    if ( ! $.isEmptyObject( this.element.TEMPLATE.BODY['custom_attrs'] ) ) {
      $.each(this.element.TEMPLATE.BODY['custom_attrs'], function(key, attr){
        var parts = attr.split("|");
        // 0 mandatory; 1 type; 2 desc;

        var roles_using_net = [];

        switch (parts[1]) {
          case "vnet_id":
            $.each(that.element.TEMPLATE.BODY.roles, function(index, value){
              if (value.vm_template_contents){
                var reg = new RegExp("\\$"+TemplateUtils.htmlDecode(key)+"\\b");

                if(reg.exec(value.vm_template_contents) != null){
                  roles_using_net.push(value.name);
                }
              }
            });

            break;
        }

        customAttrs.push({
          "name": key,
          "mandatory": parts[0],
          "type": parts[1],
          "description": parts[2],
          "roles": roles_using_net.join(", ")
        });
      });
    }

    return TemplateHTML({
      'element': this.element,
      'permissionsTableHTML': permissionsTableHTML,
      'customAttrs': customAttrs
    });
  }

  function _setup(context) {
    $('.resource-info-header', '#' + TAB_ID).text(this.element.NAME);
    PermissionsTable.setup(TAB_ID, RESOURCE, this.element, context);
  }
});

define('tabs/oneflow-templates-tab/panels/roles/panelId',['require'],function(require){
  return 'service_template_roles_tab';
});
define('tabs/oneflow-templates-tab/panels/roles',['require','tabs/oneflow-services-tab/panels/roles','./roles/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BasePanel = require('tabs/oneflow-services-tab/panels/roles');

  /*
    CONSTANTS
   */

  var PANEL_ID = require('./roles/panelId');

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    BasePanel.call(this, info);

    this.servicePanel = false;
    this.panelId = PANEL_ID;

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype = Object.create(BasePanel.prototype);
  Panel.prototype.constructor = Panel;

  return Panel;
});


/* START_TEMPLATE */
define('hbs!tabs/oneflow-templates-tab/utils/role-tab/html',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/advancedSection'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "    <div class=\"row\">\n      <div class=\"medium-4 columns\">\n        <label for=\"min_vms\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Min VMs", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Minimum number of VMs for elasticity adjustments", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </label>\n        <input type=\"text\" id=\"min_vms\" name=\"min_vms\" value=\"\" />\n      </div>\n      <div class=\"medium-4 columns\">\n        <label for=\"max_vms\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Max VMs", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Maximum number of VMs for elasticity adjustments", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </label>\n        <input type=\"text\" id=\"max_vms\" name=\"max_vms\" value=\"\" />\n      </div>\n      <div class=\"service_template_param service_role medium-4 columns\">\n        <label for=\"cooldown\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cooldown", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cooldown time after an elasticity operation (secs)", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </label>\n        <input type=\"text\" id=\"cooldown\" name=\"cooldown\" value=\"\" />\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <table id=\"elasticity_policies_table\" class=\"policies_table dataTable\">\n          <thead style=\"background:#dfdfdf\">\n            <tr>\n              <th colspan=\"8\" style=\"font-size: 16px !important\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Elasticity policies", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n            </tr>\n          </thead>\n          <thead>\n            <tr>\n              <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type of adjustment.", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CHANGE: Add/substract the given number of VMs.", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CARDINALITY: Set the cardinality to the given number.", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PERCENTAGE_CHANGE: Add/substract the given percentage to the current cardinality.", {"name":"tr","hash":{},"data":data})))
    + "\n                \" style=\"width:14%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n              <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Positive or negative adjustment. Its meaning depends on 'type'", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CHANGE: -2, will substract 2 VMs from the role", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CARDINALITY: 8, will set carditanilty to 8", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PERCENTAGE_CHANGE: 20, will increment cardinality by 20%", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:12%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Adjust", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n              <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Optional parameter for PERCENTAGE_CHANGE adjustment type.", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, " If present, the policy will change the cardinality by at least the number of VMs set in this attribute.", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:9%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Min", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n              <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Expression to trigger the elasticity", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Example: ATT < 20", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:30%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Expression", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n              <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of periods that the expression must be true before the elasticity is triggered", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:8%\">#\n              </th>\n              <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Duration, in seconds, of each period in '# Periods'", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:9%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Period", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n              <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cooldown period duration after a scale operation, in seconds", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:15%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cooldown", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n              <th style=\"width:3%\"></th>\n            </tr>\n          </thead>\n          <tbody id=\"elasticity_policies_tbody\">\n          </tbody>\n          <tfoot>\n            <tr>\n              <td colspan=\"8\">\n                <a type=\"button\" class=\"button small radius secondary small-12\" id=\"tf_btn_elas_policies\"><i class=\"fa fa-plus\"></i> "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add another policy", {"name":"tr","hash":{},"data":data})))
    + "</a>\n              </td>\n            </tr>\n          </tfoot>\n        </table>\n      </div>\n    </div>\n    <br>\n    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <table id=\"scheduled_policies_table\" class=\"policies_table dataTable\">\n          <thead style=\"background:#dfdfdf\">\n            <tr>\n              <th colspan=\"6\" style=\"font-size: 16px !important\">\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Scheduled policies", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n            </tr>\n          </thead>\n          <thead>\n            <tr>\n              <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type of adjustment.", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CHANGE: Add/substract the given number of VMs.", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CARDINALITY: Set the cardinality to the given number.", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PERCENTAGE_CHANGE: Add/substract the given percentage to the current cardinality.", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:14%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Type", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n              <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Positive or negative adjustment. Its meaning depends on 'type'", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CHANGE: -2, will substract 2 VMs from the role", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "CARDINALITY: 8, will set carditanilty to 8", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "PERCENTAGE_CHANGE: 20, will increment cardinality by 20%", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:12%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Adjust", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n              <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Optional parameter for PERCENTAGE_CHANGE adjustment type. If present, the policy will change the cardinality by at least the number of VMs set in this attribute.", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:9%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Min", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n              <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Recurrence: Time for recurring adjustements. Time is specified with the Unix cron syntax", {"name":"tr","hash":{},"data":data})))
    + "\n              "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Start time: Exact time for the adjustement", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:28%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Time format", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n              <th class=\"has-tip\" title=\""
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Time expression depends on the the time formar selected", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Recurrence: Time for recurring adjustements. Time is specified with the Unix cron syntax", {"name":"tr","hash":{},"data":data})))
    + "\n                "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Start time: Exact time for the adjustement", {"name":"tr","hash":{},"data":data})))
    + "\" style=\"width:33%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Time expression", {"name":"tr","hash":{},"data":data})))
    + "\n              </th>\n              <th style=\"width:3%\"></th>\n            </tr>\n          </thead>\n          <tbody id=\"scheduled_policies_tbody\">\n          </tbody>\n          <tfoot>\n            <tr>\n              <td colspan=\"6\">\n                <a type=\"button\" class=\"button small radius secondary small-12\" id=\"tf_btn_sche_policies\"><i class=\"fa fa-plus\"></i> "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add another policy", {"name":"tr","hash":{},"data":data})))
    + "</a>\n              </td>\n            </tr>\n          </tfoot>\n        </table>\n      </div>\n    </div>\n";
},"3":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "    <div class=\"row\">\n      <div class=\"service_template_param service_role medium-6 columns\">\n        <label for=\"shutdown_action_role\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shutdown action", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VM shutdown action: 'shutdown' or 'shutdown-hard'. If it is not set, the one set for the Service will be used", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </label>\n        <select name=\"shutdown_action_role\">\n          <option value=\"\"></option>\n          <option value=\"shutdown\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shutdown", {"name":"tr","hash":{},"data":data})))
    + "</option>\n          <option value=\"shutdown-hard\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shutdown hard", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        </select>\n      </div>\n      <div class=\"medium-6 columns\">\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"service_template_param st_man large-12 columns\">\n        <label  for=\"vm_template_contents\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VM Template Content", {"name":"tr","hash":{},"data":data})))
    + "\n          <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "This information will be merged with the original Virtual Machine template. Configuration attributes and network interfaces will be replaced by those provided by the user when the template is instantiated", {"name":"tr","hash":{},"data":data})))
    + "</span>\n        </label>\n        <textarea type=\"text\" class=\"vm_template_contents\" name=\"vm_template_contents\"/>\n      </div>\n    </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"service_template_param service_role st_man medium-6 columns\">\n    <label for=\"name\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Role Name", {"name":"tr","hash":{},"data":data})))
    + "\n      <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name of the role", {"name":"tr","hash":{},"data":data})))
    + "</span>\n    </label>\n    <input type=\"text\" id=\"role_name\" name=\"name\" required/>\n  </div>\n  <div class=\"service_template_param service_role medium-2 columns end\">\n    <label for=\"cardinality\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VMs", {"name":"tr","hash":{},"data":data})))
    + "\n      <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of VMs to instantiate with this role", {"name":"tr","hash":{},"data":data})))
    + "</span>\n    </label>\n    <input type=\"text\" id=\"cardinality\" name=\"cardinality\" value=\"1\"/>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"service_template_param service_role small-12 columns\">\n    <label for=\"vm_template\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VM template", {"name":"tr","hash":{},"data":data})))
    + "\n      <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Template associated to this role", {"name":"tr","hash":{},"data":data})))
    + "</span>\n    </label>\n    ";
  stack1 = ((helper = (helper = helpers.templatesTableHTML || (depth0 != null ? depth0.templatesTableHTML : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"templatesTableHTML","hash":{},"data":data}) : helper));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n    <div id=\"vm_template\">\n    </div>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"service_template_param service_role medium-6 columns\">\n    <table class=\"networks_role extended_table dataTable\">\n      <thead>\n        <tr>\n          <th colspan=\"2\">\n            <i class=\"fa fa-lg fa-fw fa-globe off-color\"/>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network Interfaces", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n        </tr>\n      </thead>\n      <tbody class=\"networks_role_body\">\n      </tbody>\n    </table>\n  </div>\n  <div class=\"service_template_param service_role medium-6 columns\">\n    <table class=\"parent_roles extended_table dataTable\">\n      <thead>\n        <tr>\n          <th colspan=\"2\">\n            "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Parent roles", {"name":"tr","hash":{},"data":data})))
    + "\n          </th>\n        </tr>\n      </thead>\n      <tbody class=\"parent_roles_body\">\n      </tbody>\n    </table>\n  </div>\n</div>\n<br>\n<div class=\"row\">\n  <div class=\"large-12 columns elasticity_accordion\">\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Role Elasticity", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "  </div>\n</div>\n<br>\n<div class=\"row\">\n  <div class=\"large-12 columns advanced_role_accordion\">\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced Role Parameters", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  </div>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/oneflow-templates-tab/utils/role-tab/html', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/oneflow-templates-tab/utils/role-tab/elasticity-row',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<tr>\n  <td>\n    <select id=\"type\" name=\"type\">\n      <option value=\"\"></option>\n      <option value=\"CHANGE\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Change", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"CARDINALITY\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cardinality", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"PERCENTAGE_CHANGE\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Percentage", {"name":"tr","hash":{},"data":data})))
    + "</option>\n    </select>\n  </td>\n  <td>\n    <input type=\"text\" id=\"adjust\" name=\"adjust\"/>\n  </td>\n  <td id=\"min_adjust_step_td\">\n  </td>\n  <td>\n    <input type=\"text\" id=\"expression\" name=\"expression\"/>\n  </td>\n  <td>\n    <input type=\"text\" id=\"period_number\" name=\"period_number\"/>\n  </td>\n  <td>\n    <input type=\"text\" id=\"period\" name=\"period\"/>\n  </td>\n  <td>\n    <input type=\"text\" id=\"cooldown\" name=\"cooldown\"/>\n  </td>\n  <td>\n    <a href=\"#\"><i class=\"fa fa-times-circle remove-tab\"></i></a>\n  </td>\n</tr>";
},"useData":true});
Handlebars.registerPartial('tabs/oneflow-templates-tab/utils/role-tab/elasticity-row', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/oneflow-templates-tab/utils/role-tab/sche-row',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<tr>\n  <td>\n    <select id=\"type\" name=\"type\">\n      <option value=\"\"></option>\n      <option value=\"CHANGE\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Change", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"CARDINALITY\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Cardinality", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"PERCENTAGE_CHANGE\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Percentage", {"name":"tr","hash":{},"data":data})))
    + "</option>\n    </select>\n  </td>\n  <td>\n    <input type=\"text\" id=\"adjust\" name=\"adjust\"/>\n  </td>\n  <td id=\"min_adjust_step_td\">\n  </td>\n  <td>\n    <select id=\"time_format\" name=\"time_format\">\n      <option value=\"start_time\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Start time", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      <option value=\"recurrence\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Recurrence", {"name":"tr","hash":{},"data":data})))
    + "</option>\n    </select>\n  </td>\n  <td>\n    <input type=\"text\" id=\"time\" name=\"time\"/>\n  </td>\n  <td>\n    <a href=\"#\"><i class=\"fa fa-times-circle remove-tab\"></i></a>\n  </td>\n</tr>";
},"useData":true});
Handlebars.registerPartial('tabs/oneflow-templates-tab/utils/role-tab/sche-row', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/oneflow-templates-tab/utils/role-tab',['require','utils/locale','utils/tips','tabs/templates-tab/datatable','utils/template-utils','hbs!./role-tab/html','hbs!./role-tab/elasticity-row','hbs!./role-tab/sche-row'],function(require) {
  // Dependencies
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var TemplatesTable = require('tabs/templates-tab/datatable');
  var TemplateUtils = require('utils/template-utils');

  var TemplateHTML = require('hbs!./role-tab/html');
  var TemplateElasticityRowHTML = require('hbs!./role-tab/elasticity-row');
  var TemplateScheRowHTML = require('hbs!./role-tab/sche-row');

  function RoleTab(html_role_id) {
    this.html_role_id = html_role_id;

    return this;
  }

  RoleTab.prototype = {
    'html': _role_tab_content,
    'setup': _setup_role_tab_content,
    'onShow': _onShow,
    'retrieve': _retrieve,
    'fill': _fill
  };
  RoleTab.prototype.constructor = RoleTab;

  return RoleTab;

  function _role_tab_content(){
    var opts = {
      info: false,
      select: true
    };

    this.templatesTable = new TemplatesTable("roleTabTemplates"+this.html_role_id, opts);

    return TemplateHTML({
      'templatesTableHTML': this.templatesTable.dataTableHTML
    });
  }

  function _setup_role_tab_content(role_section) {
    var that = this;

    Tips.setup(role_section);

    this.templatesTable.initialize();
    this.templatesTable.idInput().attr("required", "");

    role_section.on("change", "#role_name", function(){
      $("#" + that.html_role_id +" #role_name_text").html($(this).val());
    });

    role_section.on("change", "select#type", function(){
      var new_tr = $(this).closest('tr');
      if ($(this).val() == "PERCENTAGE_CHANGE") {
        $("#min_adjust_step_td", new_tr).html('<input type="text" id="min_adjust_step" name="min_adjust_step"/>');
      } else {
        $("#min_adjust_step_td", new_tr).empty();
      }
    });

    $("#tf_btn_elas_policies", role_section).bind("click", function(){
      $( TemplateElasticityRowHTML({}) ).appendTo($("#elasticity_policies_tbody", role_section));
    });

    role_section.on("click", "#elasticity_policies_table i.remove-tab", function() {
      var tr = $(this).closest('tr');
      tr.remove();
    });

    $("#tf_btn_sche_policies", role_section).bind("click", function(){
      $( TemplateScheRowHTML({}) ).appendTo($("#scheduled_policies_tbody", role_section));
    });

    role_section.on("click", "#scheduled_policies_table i.remove-tab", function() {
      var tr = $(this).closest('tr');
      tr.remove();
    });

    $("#tf_btn_elas_policies", role_section).trigger("click");
    $("#tf_btn_sche_policies", role_section).trigger("click");

    role_section.on("change", ".service_network_checkbox", function(){
      var vm_template_contents = "";
      $(".service_network_checkbox:checked", role_section).each(function(){
        vm_template_contents += "NIC=[NETWORK_ID=\"$"+$(this).val()+"\"]\n";
      });

      $(".vm_template_contents", role_section).val(vm_template_contents);
    });
  }

  function _onShow(){
    this.templatesTable.refreshResourceTableSelect();
  }

  function _retrieve(context){
    var role = {};
    role['name'] = $('input[name="name"]', context).val();
    role['cardinality'] = $('input[name="cardinality"]', context).val();
    role['vm_template'] = this.templatesTable.retrieveResourceTableSelect();
    role['shutdown_action'] = $('select[name="shutdown_action_role"]', context).val();
    role['parents'] = [];
    role['vm_template_contents'] = $(".vm_template_contents", context).val();

    $('.parent_roles_body input.check_item:checked', context).each(function(){
      role['parents'].push($(this).val());
    });

    var shutdown_action = $('select[name="shutdown_action_role"]', context).val();
    if (shutdown_action) {
      role['shutdown_action'] = shutdown_action;
    }

    var min_vms = $('input[name="min_vms"]', context).val();
    if (min_vms) {
      role['min_vms'] = min_vms;
    }

    var max_vms = $('input[name="max_vms"]', context).val();
    if (max_vms) {
      role['max_vms'] = max_vms;
    }

    var cooldown = $('input[name="cooldown"]', context).val();
    if (cooldown) {
      role['cooldown'] = cooldown;
    }

    role = _removeEmptyObjects(role);
    role['elasticity_policies'] = [];
    $("#elasticity_policies_tbody tr", context).each(function(){
      if ($("#type" ,this).val()) {
        var policy = {};
        policy['type'] = $("#type" ,this).val();
        policy['adjust']  = $("#adjust" ,this).val();
        policy['min_adjust_step']  = $("#min_adjust_step" ,this).val();
        policy['expression']  = $("#expression" ,this).val();
        policy['period_number']  = $("#period_number" ,this).val();
        policy['period']  = $("#period" ,this).val();
        policy['cooldown']  = $("#cooldown" ,this).val();

        // TODO remove empty policies
        role['elasticity_policies'].push(_removeEmptyObjects(policy));
      }
    });

    role['scheduled_policies'] = [];
    $("#scheduled_policies_tbody tr", context).each(function(){
      if ($("#type" ,this).val()) {
        var policy = {};
        policy['type'] = $("#type" ,this).val();
        policy['adjust']  = $("#adjust" ,this).val();
        policy['min_adjust_step']  = $("#min_adjust_step" ,this).val();

        var time_format = $("#time_format" ,this).val();
        policy[time_format] = $("#time" ,this).val();

        // TODO remove empty policies
        role['scheduled_policies'].push(_removeEmptyObjects(policy));
      }
    });

    return role;
  }

  function _fill(context, value, network_names) {
    $("#role_name", context).val(TemplateUtils.htmlDecode(value.name));
    $("#role_name", context).change();

    $("#cardinality", context).val(TemplateUtils.htmlDecode(value.cardinality));

    this.templatesTable.selectResourceTableSelect({ids : value.vm_template});

    if (value.vm_template_contents){
      $(network_names).each(function(){
        var reg = new RegExp("\\$"+this+"\\b");

        if(reg.exec(value.vm_template_contents) != null){
          $(".service_network_checkbox[value='"+this+"']", context).attr('checked', true).change();
        }
      });

      $(".vm_template_contents", context).val(TemplateUtils.htmlDecode(value.vm_template_contents));
    }

    $("select[name='shutdown_action_role']", context).val(value.shutdown_action);

    $("#min_vms", context).val(TemplateUtils.htmlDecode(value.min_vms));
    $("#max_vms", context).val(TemplateUtils.htmlDecode(value.max_vms));
    $("#cooldown", context).val(TemplateUtils.htmlDecode(value.cooldown));

    if (value['elasticity_policies'].length > 0 ||
        value['scheduled_policies'].length > 0) {
      $("div.elasticity_accordion a.accordion_advanced_toggle", context).trigger("click");
    }

    $("#elasticity_policies_table i.remove-tab", context).trigger("click");
    $("#scheduled_policies_table i.remove-tab", context).trigger("click");

    if (value['elasticity_policies']) {
      $.each(value['elasticity_policies'], function(){
        $("#tf_btn_elas_policies", context).click();
        var td = $("#elasticity_policies_tbody tr", context).last();
        $("#type" ,td).val(TemplateUtils.htmlDecode(this['type']));
        $("#type" ,td).change();
        $("#adjust" ,td).val(TemplateUtils.htmlDecode(this['adjust'] ));
        $("#min_adjust_step" ,td).val(TemplateUtils.htmlDecode(this['min_adjust_step'] || ""));
        $("#expression" ,td).val(TemplateUtils.htmlDecode(this.expression));
        $("#period_number" ,td).val(TemplateUtils.htmlDecode(this['period_number'] || ""));
        $("#period" ,td).val(TemplateUtils.htmlDecode(this['period'] || "" ));
        $("#cooldown" ,td).val(TemplateUtils.htmlDecode(this['cooldown'] || "" ));
      });
    }

    if (value['scheduled_policies']) {
      $.each(value['scheduled_policies'], function(){
        $("#tf_btn_sche_policies", context).click();
        var td = $("#scheduled_policies_tbody tr", context).last();
        $("#type", td).val(TemplateUtils.htmlDecode(this['type']));
        $("#type" ,td).change();
        $("#adjust", td).val(TemplateUtils.htmlDecode(this['adjust'] ));
        $("#min_adjust_step", td).val(TemplateUtils.htmlDecode(this['min_adjust_step']  || ""));

        if (this['start_time']) {
          $("#time_format", td).val('start_time');
          $("#time", td).val(TemplateUtils.htmlDecode(this['start_time']));
        } else if (this['recurrence']) {
          $("#time_format", td).val('recurrence');
          $("#time", td).val(TemplateUtils.htmlDecode(this['recurrence']));
        }
      });
    }
  }

  //----------------------------------------------------------------------------

  function _removeEmptyObjects(obj){
    for (var elem in obj){
      var remove = false;
      var value = obj[elem];
      if (value instanceof Array){
        if (value.length == 0)
          remove = true;
        else if (value.length > 0){
          value = jQuery.grep(value, function (n) {
            var obj_length = 0;
            for (e in n)
              obj_length += 1;

            if (obj_length == 0)
              return false;

            return true;
          });

          if (value.length == 0)
            remove = true;
        }
      }
      else if (value instanceof Object){
        var obj_length = 0;
        for (e in value)
          obj_length += 1;
        if (obj_length == 0)
          remove = true;
      }else{
        value = String(value);
        if (value.length == 0)
          remove = true;
      }

      if (remove)
        delete obj[elem];
    }

    return obj;
  }
});

/* START_TEMPLATE */
define('hbs!tabs/oneflow-templates-tab/form-panels/create/wizard',['hbs','hbs/handlebars','templates/helpers/tr','templates/helpers/advancedSection'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "    <div class=\"row\">\n      <div class=\"large-12 columns\">\n        <table class=\"service_networks policies_table dataTable\">\n          <thead>\n            <tr>\n              <th style=\"width:30%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th style=\"width:70%\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description", {"name":"tr","hash":{},"data":data})))
    + "</th>\n              <th style=\"width:3%\"></th>\n            </tr>\n          </thead>\n          <tbody>\n          </tbody>\n          <tfoot>\n            <tr>\n              <td colspan=\"3\">\n                <a type=\"button\" class=\"add_service_network button small small-12 secondary radius\"><i class=\"fa fa-plus\"></i>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add another Network", {"name":"tr","hash":{},"data":data})))
    + "</a>\n              </td>\n            </tr>\n          </tfoot>\n        </table>\n      </div>\n    </div>\n";
},"3":function(depth0,helpers,partials,data) {
  var helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "  <div class=\"row\">\n    <div class=\"service_template_param st_man medium-6 columns\">\n      <label for=\"deployment\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Strategy", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Straight strategy will instantiate each role in order: parents role will be deployed before their children. None strategy will instantiate the roles regardless of their relationships.", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </label>\n      <select name=\"deployment\">\n        <option value=\"straight\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Straight", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"none\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "None", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n    <div class=\"service_template_param st_man medium-6 columns\">\n      <label for=\"shutdown_action_service\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shutdown action", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "VM shutdown action: 'shutdown' or 'shutdown-hard'.", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </label>\n      <select name=\"shutdown_action_service\">\n        <option value=\"\"></option>\n        <option value=\"shutdown\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shutdown", {"name":"tr","hash":{},"data":data})))
    + "</option>\n        <option value=\"shutdown-hard\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Shutdown hard", {"name":"tr","hash":{},"data":data})))
    + "</option>\n      </select>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"service_template_param st_man medium-6 columns\">\n      <input type=\"checkbox\" name=\"ready_status_gate\" id=\"ready_status_gate\"/>\n      <label for=\"ready_status_gate\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Wait for VMs to report that they are READY", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Before deploying any child roles, wait for all VMs of the parent roles to report via OneGate that they are READY=YES", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </label>\n    </div>\n  </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression, buffer = "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"service_template_param st_man medium-6 columns\">\n      <label for=\"service_name\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Name for this template", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </label>\n      <input type=\"text\" id=\"service_name\" name=\"service_name\" required/>\n    </div>\n    <div class=\"service_template_param st_man medium-6 columns\">\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"service_template_param st_man large-12 columns\">\n      <label  for=\"description\">\n        "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Description of the service", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      </label>\n      <textarea type=\"text\" id=\"description\" name=\"description\"/>\n    </div>\n  </div>\n  <br>\n  <div id=\"network_configuration\">\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Network Configuration", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "  </div>\n  <br>\n";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, ((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Advanced Service Parameters", {"name":"tr","hash":{},"data":data})), {"name":"advancedSection","hash":{},"fn":this.program(3, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "  <br>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <h4>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Roles", {"name":"tr","hash":{},"data":data})))
    + "</h4>\n    </div>\n  </div>\n  <br>\n  <div class=\"row\">\n    <div id=\"new_role\" class=\"bordered-tabs large-12 columns\">\n      <dl class=\"tabs\" id=\"roles_tabs\" data-tab>\n        <a class=\"button small right radius\" id=\"tf_btn_roles\">\n          <span class=\"fa fa-plus\"></span>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Add another role", {"name":"tr","hash":{},"data":data})))
    + "\n        </a>\n      </dl>\n      <div class=\"tabs-content\" id=\"roles_tabs_content\">\n      </div>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/oneflow-templates-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/oneflow-templates-tab/form-panels/create/advanced',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Advanced\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <p>"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Write the Service template here", {"name":"tr","hash":{},"data":data})))
    + "</p>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <textarea id=\"template\" rows=\"15\" required></textarea>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/oneflow-templates-tab/form-panels/create/advanced', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/oneflow-templates-tab/form-panels/create',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','utils/tips','tabs/oneflow-templates-tab/utils/role-tab','utils/template-utils','hbs!./create/wizard','hbs!./create/advanced','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var RoleTab = require('tabs/oneflow-templates-tab/utils/role-tab');
  var TemplateUtils = require('utils/template-utils');

  /*
    TEMPLATES
   */

  var TemplateWizardHTML = require('hbs!./create/wizard');
  var TemplateAdvancedHTML = require('hbs!./create/advanced');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': Locale.tr("Create Service Template"),
        'buttonText': Locale.tr("Create"),
        'resetButton': true
      },
      'update': {
        'title': Locale.tr("Update Service Template"),
        'buttonText': Locale.tr("Update"),
        'resetButton': false
      }
    };

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.htmlAdvanced = _htmlAdvanced;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.submitAdvanced = _submitAdvanced;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.fill = _fill;
  FormPanel.prototype.setup = _setup;
  FormPanel.prototype.addRoleTab = _add_role_tab;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    return TemplateWizardHTML({
      'formPanelId': this.formPanelId
    });
  }

  function _htmlAdvanced() {
    return TemplateAdvancedHTML({
      'formPanelId': this.formPanelId
    });
  }

  function _setup(context) {
    this.roleTabObjects = {};
    var that = this;

    var roles_index = 0;

    $(".add_service_network", context).on("click", function(){
      $(".service_networks tbody").append(
        '<tr>\
          <td>\
            <input class="service_network_name" type="text" pattern="[\\w]+"/>\
            <small class="error">'+Locale.tr("Only word characters are allowed")+'</small>\
          </td>\
          <td>\
            <textarea class="service_network_description"/>\
          </td>\
          <td>\
            <a href="#"><i class="fa fa-times-circle remove-tab"></i></a>\
          </td>\
        </tr>');
    });

    $(".add_service_network", context).trigger("click");

    context.on("change", ".service_network_name", function(){
      _redo_service_networks_selector(context);
    });

    context.on("click", ".service_networks i.remove-tab", function(){
      var tr = $(this).closest('tr');
      tr.remove();

      _redo_service_networks_selector(context);
    });

    $("#tf_btn_roles", context).bind("click", function(){
      that.addRoleTab(roles_index, context);
      roles_index++;

      context.foundation();

      return false;
    });

    // close icon: removing the tab on click
    $("#roles_tabs", context).on("click", "i.remove-tab", function() {
      var target = $(this).parent().attr("href");
      var dd = $(this).closest('dd');
      var dl = $(this).closest('dl');
      var content = $(target);

      var role_id = content.attr("role_id");

      dd.remove();
      content.remove();

      if (dd.attr("class") == 'active') {
        $('a', dl.children('dd').last()).click();
      }

      delete that.roleTabObjects[role_id];

      return false;
    });

    // Fill parents table
    // Each time a tab is clicked the table is filled with existing tabs (roles)
    // Selected roles are kept
    // TODO If the name of a role is changed and is selected, selection will be lost
    $("#roles_tabs", context).on("click", "a", function() {
      var tab_id = "#"+this.id+"Tab";
      var str = "";

      $(tab_id+" .parent_roles").hide();
      var parent_role_available = false;

      $("#roles_tabs_content #role_name", context).each(function(){
        if ($(this).val() && ($(this).val() != $(tab_id+" #role_name", context).val())) {
          parent_role_available = true;
          str += "<tr>\
            <td style='width:10%'>\
              <input class='check_item' type='checkbox' value='"+$(this).val()+"' id='"+$(this).val()+"'/>\
            </td>\
            <td>"+$(this).val()+"</td>\
          </tr>";
        }
      });

      if (parent_role_available) {
        $(tab_id+" .parent_roles", context).show();
      }

      var selected_parents = [];
      $(tab_id+" .parent_roles_body input:checked", context).each(function(){
        selected_parents.push($(this).val());
      });

      $(tab_id+" .parent_roles_body", context).html(str);

      $.each(selected_parents, function(){
        $(tab_id+" .parent_roles_body #"+this, context).attr('checked', true);
      });
    });


    $(document).foundation('tab', 'reflow');
    
    // Add first role
    $("#tf_btn_roles", context).trigger("click");

    Tips.setup(context);
    return false;
  }

  function _submitWizard(context) {
    var that = this;

    var name = $('input[name="service_name"]', context).val();
    var description = $('#description', context).val();
    var deployment = $('select[name="deployment"]', context).val();
    var shutdown_action_service = $('select[name="shutdown_action_service"]', context).val();
    var ready_status_gate = $('input[name="ready_status_gate"]', context).prop("checked");

    var custom_attrs =  {};

    $(".service_networks tbody tr").each(function(){
      if ($(".service_network_name", $(this)).val()) {
        var attr_name = $(".service_network_name", $(this)).val();
        var attr_type = "vnet_id";
        var attr_desc = $(".service_network_description", $(this)).val();
        custom_attrs[attr_name] = "M|" + attr_type + "|" + attr_desc;
      }
    });

    var roles = [];

    $('.role_content', context).each(function() {
      var role_id = $(this).attr("role_id");

      roles.push( that.roleTabObjects[role_id].retrieve($(this)) );
    });

    var json_template = {
      name: name,
      deployment: deployment,
      description: description,
      roles: roles
    };

    if (!$.isEmptyObject(custom_attrs)){
      json_template['custom_attrs'] = custom_attrs;
    }

    if (shutdown_action_service){
      json_template['shutdown_action'] = shutdown_action_service;
    }

    json_template['ready_status_gate'] = ready_status_gate;

    if (this.action == "create") {
      Sunstone.runAction("ServiceTemplate.create", json_template );
      return false;
    } else if (this.action == "update") {
      Sunstone.runAction("ServiceTemplate.update",this.resourceId, JSON.stringify(json_template));
      return false;
    }
  }

  function _submitAdvanced(context) {
    var json_template = $('textarea#template', context).val();

    if (this.action == "create") {
      Sunstone.runAction("ServiceTemplate.create", JSON.parse(json_template) );
      return false;
    } else if (this.action == "update") {
      Sunstone.runAction("ServiceTemplate.update", this.resourceId, json_template);
      return false;
    }
  }

  function _onShow(context) {
    var that = this;

    $('.role_content', context).each(function() {
      var role_id = $(this).attr("role_id");
      that.roleTabObjects[role_id].onShow();
    });
  }

  function _fill(context, element) {
    var that = this;

    if (this.action != "update") {return;}
    this.resourceId = element.ID;

    // Populates the Avanced mode Tab
    $('#template', context).val(TemplateUtils.htmlDecode(
                          JSON.stringify(element.TEMPLATE.BODY, null, "  ")));


    $("#service_name", context).attr("disabled", "disabled");
    $("#service_name", context).val(TemplateUtils.htmlDecode(element.NAME));

    $("#description", context).val(TemplateUtils.htmlDecode(element.TEMPLATE.BODY.description));

    $('select[name="deployment"]', context).val(element.TEMPLATE.BODY.deployment);
    $("select[name='shutdown_action_service']", context).val(element.TEMPLATE.BODY.shutdown_action);
    $("input[name='ready_status_gate']", context).prop("checked",element.TEMPLATE.BODY.ready_status_gate || false);


    $(".service_networks i.remove-tab", context).trigger("click");

    if ( ! $.isEmptyObject( element.TEMPLATE.BODY['custom_attrs'] ) ) {
      $("div#network_configuration a.accordion_advanced_toggle", context).trigger("click");

      $.each(element.TEMPLATE.BODY['custom_attrs'], function(key, attr){
        var parts = attr.split("|");
        // 0 mandatory; 1 type; 2 desc;
        var attrs = {
          "name": key,
          "mandatory": parts[0],
          "type": parts[1],
          "description": parts[2],
        };

        switch (parts[1]) {
          case "vnet_id":
            $(".add_service_network", context).trigger("click");

            var tr = $(".service_networks tbody tr", context).last();
            $(".service_network_name", tr).val(TemplateUtils.htmlDecode(attrs.name)).change();
            $(".service_network_description", tr).val(TemplateUtils.htmlDecode(attrs.description));

            break;
        }
      });
    }

    $("#roles_tabs i.remove-tab", context).trigger("click");

    var network_names = [];

    $(".service_networks .service_network_name", context).each(function(){
      if ($(this).val()) {
        network_names.push($(this).val());
      }
    });

    var roles_names = [];
    $.each(element.TEMPLATE.BODY.roles, function(index, value){
      roles_names.push(value.name);

      $("#tf_btn_roles", context).click();

      var role_context = $('.role_content', context).last();
      var role_id = $(role_context).attr("role_id");

      that.roleTabObjects[role_id].fill(role_context, value, network_names);
    });

    $.each(element.TEMPLATE.BODY.roles, function(index, value){
        var role_context = $('.role_content', context)[index];
        var str = "";

        $.each(roles_names, function(){
          if (this != value.name) {
            str += "<tr>\
              <td style='width:10%'>\
                <input class='check_item' type='checkbox' value='"+this+"' id='"+this+"'/>\
              </td>\
              <td>"+this+"</td>\
            </tr>";
          }
        });

        $(".parent_roles_body", role_context).html(str);

        if (value.parents) {
          $.each(value.parents, function(index, value){
            $(".parent_roles_body #"+this, role_context).attr('checked', true);
          });
        }
    });
  }

  //----------------------------------------------------------------------------

  function _redo_service_networks_selector(dialog){
    $('#roles_tabs_content .role_content', dialog).each(function(){
      var role_section = this;
      _redo_service_networks_selector_role(dialog, role_section);
    });
  }

  function _redo_service_networks_selector_role(dialog, role_section){
    $('#roles_tabs_content .role_content', dialog).each(function(){
      var role_section = this;

      var selected_networks = [];
      $(".service_network_checkbox:checked", role_section).each(function(){
        selected_networks.push($(this).val());
      });

      $(".networks_role", role_section).hide();
      var service_networks = false;

      var role_tab_id = $(role_section).attr('id');

      var str = "";
      $(".service_networks .service_network_name", dialog).each(function(){
        if ($(this).val()) {
          service_networks = true;
          str += "<tr>\
            <td style='width:10%'>\
              <input class='service_network_checkbox check_item' type='checkbox' value='"+$(this).val()+"' id='"+role_tab_id+"_"+$(this).val()+"'/>\
            </td>\
            <td>\
              <label for='"+role_tab_id+"_"+$(this).val()+"'>"+$(this).val()+"</label>\
            </td>\
          </tr>";
        }
      });

      $(".networks_role_body", role_section).html(str);

      if (service_networks) {
        $(".networks_role", role_section).show();
      }

      $(".vm_template_contents", role_section).val("");

      $.each(selected_networks, function(){
        $(".service_network_checkbox[value='"+this+"']", role_section).attr('checked', true).change();
      });
    });
  }

  function _add_role_tab(role_id, dialog) {
    var html_role_id  = 'role' + role_id;

    var role_tab = new RoleTab(html_role_id);
    this.roleTabObjects[role_id] = role_tab;

    // Append the new div containing the tab and add the tab to the list
    var role_section = $('<div id="'+html_role_id+'Tab" class="content role_content wizard_internal_tab" role_id="'+role_id+'">'+
        role_tab.html() +
    '</div>').appendTo($("#roles_tabs_content", dialog));

    _redo_service_networks_selector_role(dialog, role_section);

    var a = $("<dd>\
      <a class='text-center' id='"+html_role_id+"' href='#"+html_role_id+"Tab'>\
        <span>\
          <i class='off-color fa fa-cube fa-3x'/>\
          <br>\
          <span id='role_name_text'>"+Locale.tr("Role ")+role_id+"</span>\
        </span>\
        <i class='fa fa-times-circle remove-tab'></i>\
      </a>\
    </dd>").appendTo($("dl#roles_tabs", dialog));

    $("a", a).trigger("click");

    role_tab.setup(role_section);
    role_tab.onShow();
  }

});


/* START_TEMPLATE */
define('hbs!tabs/oneflow-templates-tab/form-panels/instantiate/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <label for=\"service_name\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Service Name", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Defaults to template name when emtpy. You can use the wildcard &#37;i. When creating several Services, &#37;i will be replaced with a different number starting from 0 in each of them", {"name":"tr","hash":{},"data":data})))
    + ".</span>\n      </label>\n      <input type=\"text\" name=\"service_name\" id=\"service_name\" />\n    </div>\n    <div class=\"medium-6 columns\">\n      <label for=\"service_n_times\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of instances", {"name":"tr","hash":{},"data":data})))
    + "\n        <span class=\"tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Number of Services that will be created using this template", {"name":"tr","hash":{},"data":data})))
    + ".</span>\n      </label>\n      <input type=\"text\" name=\"service_n_times\" id=\"service_n_times\" value=\"1\">\n    </div>\n  </div>\n  <div id=\"instantiate_service_user_inputs\">\n    <i class=\"fa fa-spinner fa-spin\"></i>\n  </div>\n  <div class=\"row\" id=\"instantiate_service_role_user_inputs\">\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/oneflow-templates-tab/form-panels/instantiate/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/oneflow-templates-tab/form-panels/instantiate',['require','foundation.tab','utils/form-panels/form-panel','sunstone','utils/locale','utils/tips','opennebula/servicetemplate','opennebula/template','utils/notifier','utils/wizard-fields','utils/template-utils','utils/user-inputs','hbs!./instantiate/html','./instantiate/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var Tips = require('utils/tips');
  var OpenNebulaServiceTemplate = require('opennebula/servicetemplate');
  var OpenNebulaTemplate = require('opennebula/template');
  var Notifier = require('utils/notifier');
  var WizardFields = require('utils/wizard-fields');
  var TemplateUtils = require('utils/template-utils');
  var UserInputs = require('utils/user-inputs');

  /*
    TEMPLATES
   */

  var TemplateHTML = require('hbs!./instantiate/html');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./instantiate/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'instantiate': {
        'title': Locale.tr("Instantiate Service Template"),
        'buttonText': Locale.tr("Instantiate"),
        'resetButton': false
      }
    };

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.setTemplateId = _setTemplateId;
  FormPanel.prototype.htmlWizard = _html;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'formPanelId': this.formPanelId
    });
  }

  function _setup(context) {
    var that = this;

    Tips.setup(context);
    return false;
  }

  function _onShow(context) {
  }

  function _setTemplateId(context, templateId) {
    var that = this;

    this.templateId = templateId;

    this.service_template_json = {};

    OpenNebulaServiceTemplate.show({
      data : {
        id: templateId
      },
      timeout: true,
      success: function (request, template_json){

        that.service_template_json = template_json;

        $("#instantiate_service_user_inputs", context).empty();

        UserInputs.serviceTemplateInsert(
          $("#instantiate_service_user_inputs", context),
          template_json);

        n_roles = template_json.DOCUMENT.TEMPLATE.BODY.roles.length;
        n_roles_done = 0;

        $.each(template_json.DOCUMENT.TEMPLATE.BODY.roles, function(index, role){
          var div_id = "user_input_role_"+index;

          $("#instantiate_service_role_user_inputs", context).append(
            '<div id="'+div_id+'" class="large-6 columns">\
            </div>'
            );

          OpenNebulaTemplate.show({
            data : {
              id: role.vm_template
            },
            timeout: true,
            success: function (request, vm_template_json){

              $("#"+div_id, context).empty();

              UserInputs.vmTemplateInsert(
                $("#"+div_id, context),
                vm_template_json,
                {
                  text_header: Locale.tr("Role") + " " + role.name
                }
              );

              n_roles_done += 1;

              if(n_roles_done == n_roles){
                Sunstone.enableFormPanelSubmit(TAB_ID);
              }
            },
            error: function(request,error_json, container){
              Notifier.onError(request,error_json, container);
              $("#instantiate_vm_user_inputs", context).empty();
            }
          });
        });
      },
      error: function(request,error_json, container){
        Notifier.onError(request,error_json, container);
        $("#instantiate_service_user_inputs", context).empty();
      }
    });
  }

  function _submitWizard(context) {
    var that = this;

    var service_name = $('#service_name',context).val();
    var n_times = $('#service_n_times',context).val();
    var n_times_int=1;

    if (n_times.length){
      n_times_int=parseInt(n_times,10);
    }

    var extra_msg = "";
    if (n_times_int > 1) {
      extra_msg = n_times_int+" times";
    }

    var extra_info = {
      'merge_template': {}
    };

    var tmp_json = WizardFields.retrieve($("#instantiate_service_user_inputs", context));

    extra_info.merge_template.custom_attrs_values = tmp_json;

    extra_info.merge_template.roles = [];

    $.each(that.service_template_json.DOCUMENT.TEMPLATE.BODY.roles, function(index, role){
      var div_id = "user_input_role_"+index;

      tmp_json = {};

      $.extend( tmp_json, WizardFields.retrieve($("#"+div_id, context)) );

      $.each(role.elasticity_policies, function(i, pol){
        pol.expression = TemplateUtils.htmlDecode(pol.expression);
      });

      role.user_inputs_values = tmp_json;

      extra_info.merge_template.roles.push(role);
    });

    if (!service_name.length){ //empty name
      for (var i=0; i< n_times_int; i++){
        Sunstone.runAction("ServiceTemplate.instantiate", that.templateId, extra_info);
      }
    } else {
      if (service_name.indexOf("%i") == -1){//no wildcard, all with the same name
        extra_info['merge_template']['name'] = service_name;

        for (var i=0; i< n_times_int; i++){
          Sunstone.runAction(
              "ServiceTemplate.instantiate",
              that.templateId, extra_info);
        }
      } else { //wildcard present: replace wildcard
        for (var i=0; i< n_times_int; i++){
          extra_info['merge_template']['name'] = service_name.replace(/%i/gi,i);

          Sunstone.runAction(
              "ServiceTemplate.instantiate",
              that.templateId, extra_info);
        }
      }
    }

    return false;
  }

});

define('tabs/oneflow-templates-tab',['require','utils/locale','./oneflow-templates-tab/buttons','./oneflow-templates-tab/actions','./oneflow-templates-tab/datatable','./oneflow-templates-tab/tabId','./oneflow-templates-tab/panels/info','./oneflow-templates-tab/panels/roles','./oneflow-templates-tab/form-panels/create','./oneflow-templates-tab/form-panels/instantiate'],function(require) {
  var Locale = require('utils/locale');
  var Buttons = require('./oneflow-templates-tab/buttons');
  var Actions = require('./oneflow-templates-tab/actions');
  var Table = require('./oneflow-templates-tab/datatable');

  var TAB_ID = require('./oneflow-templates-tab/tabId');
  var DATATABLE_ID = "dataTableServiceTemplates";
  var RESOURCE = "ServiceTemplate";

  var _dialogs = [
  ];

  var _panels = [
    require('./oneflow-templates-tab/panels/info'),
    require('./oneflow-templates-tab/panels/roles')
  ];

  var _formPanels = [
    require('./oneflow-templates-tab/form-panels/create'),
    require('./oneflow-templates-tab/form-panels/instantiate')
  ];

  var Tab = {
    tabId: TAB_ID,
    title: Locale.tr("Templates"),
    tabClass: "subTab",
    parentTab: "oneflow-dashboard",
    listHeader: '<i class="fa fa-fw fa-file-o"></i>&emsp;'+Locale.tr("OneFlow - Templates"),
    infoHeader: '<i class="fa fa-fw fa-file-o"></i>&emsp;'+Locale.tr("OneFlow - Template"),
    subheader: '',
    content: '<div class="row oneflow_templates_error_message" hidden>\
        <div class="small-6 columns small-centered text-center">\
            <div class="alert-box alert radius">'+Locale.tr("Cannot connect to OneFlow server")+'</div>\
        </div>\
    </div>',
    resource: RESOURCE,
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs
  };

  return Tab;
});

define('tabs/support-tab/buttons',['require','utils/locale'],function(require) {
  var Locale = require('utils/locale');

  var Buttons = {
    "Support.refresh" : {
      type: "action",
      layout: "refresh",
      alwaysActive: true
    },
    "Support.upload" : {
      type: "action",
      layout: "main",
      text: '<i class="fa fa-cloud-upload" style="color: rgb(111, 111, 111)"/> '+Locale.tr("Upload a file"),
      custom_classes: "only-right-info"
    },
    "Support.signout" : {
      type: "action",
      layout: "main",
      text: '<i class="fa fa-sign-out fa fa-lg">',
      tip: "Sign out of Commercial Support",
      alwaysActive: true
    },
    "Support.create_dialog" : {
      type: "create_dialog",
      layout: "create",
      text: "Submit a Request"
    }
  };

  return Buttons;
});

define('tabs/support-tab/tabId',['require'],function(require) {
  return 'support-tab';
});

define('tabs/support-tab/utils/common',['require','sunstone','../tabId'],function(require) {
  /*
    Common functions for the support tab
   */

  var Sunstone = require('sunstone');

  var TAB_ID = require('../tabId');

  var support_interval_function;

  function _show_support_connect() {
    $(".support_info").hide();
    $("#"+Sunstone.getDataTable(TAB_ID).dataTableId+"Container", "#"+TAB_ID).hide();
    $(".support_connect").show();
    $(".actions_row", "#"+TAB_ID).hide();
  }

  function _show_support_list() {
    $(".support_info").show();
    $(".support_connect").hide();
    $(".actions_row", "#"+TAB_ID).show();
    $("#"+Sunstone.getDataTable(TAB_ID).dataTableId+"Container", "#"+TAB_ID).show();
  }

  function _startIntervalRefresh() {
    Sunstone.runAction('Support.list');

    support_interval_function = setInterval(function(){
      Sunstone.runAction('Support.list');
    }, Sunstone.TOP_INTERVAL);
  }

  function _stopIntervalRefresh() {
    clearInterval(support_interval_function);
  }

  return {
    'showSupportConnect': _show_support_connect,
    'showSupportList': _show_support_list,
    'startIntervalRefresh': _startIntervalRefresh,
    'stopIntervalRefresh': _stopIntervalRefresh,
  };
});
define('tabs/support-tab/form-panels/create/formPanelId',['require'],function(require){
  return 'createSupportRequestForm';
});
define('tabs/support-tab/dialogs/upload/dialogId',['require'],function(require){
  return 'uploadSupportRequestDialog';
});
define('tabs/support-tab/actions',['require','sunstone','utils/notifier','opennebula/support','./utils/common','./tabId','./form-panels/create/formPanelId','./dialogs/upload/dialogId'],function(require) {
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var OpenNebulaSupport = require('opennebula/support');
  var SupportUtils = require('./utils/common');

  var RESOURCE = "Support";
  var TAB_ID = require('./tabId');
  var CREATE_DIALOG_ID = require('./form-panels/create/formPanelId');
  var UPLOAD_DIALOG_ID = require('./dialogs/upload/dialogId');

  var _actions = {
    "Support.list" : {
      type: "list",
      call: OpenNebulaSupport.list,
      callback: function(req, list, res){
        SupportUtils.showSupportList();
        $(".support_open_value").text(res.open_requests);
        $(".support_pending_value").text(res.pending_requests);

        var elements = [];
        if(res.REQUEST_POOL.REQUEST){
          elements = res.REQUEST_POOL.REQUEST;
        }

        Sunstone.getDataTable(TAB_ID).updateView(req, elements);
      },
      error: function(request, error_json) {
        if (error_json.error.http_status=="401") {
          SupportUtils.stopIntervalRefresh();
        }

        SupportUtils.showSupportConnect();
      }
    },
    "Support.refresh" : {
      type: "custom",
      call: function() {
        var tab = $('#' + TAB_ID);
        if (Sunstone.rightInfoVisible(tab)) {
          Sunstone.runAction(RESOURCE+".show", Sunstone.rightInfoResourceId(tab));
        } else {
          Sunstone.getDataTable(TAB_ID).waitingNodes();
          Sunstone.runAction(RESOURCE+".list", {force: true});
        }
      },
      error: function(request, error_json) {
        SupportUtils.showSupportConnect();
      }
    },
    "Support.show" : {
      type: "single",
      call: OpenNebulaSupport.show,
      callback: function(request, response) {
        //Sunstone.getDataTable(TAB_ID).updateElement(request, response);
        if (Sunstone.rightInfoVisible($('#'+TAB_ID))) {
          Sunstone.insertPanels(TAB_ID, response);
        }
      },
      error: function(request, error_json) {
        SupportUtils.showSupportConnect();
      }
    },
    "Support.create" : {
      type: "create",
      call: OpenNebulaSupport.create,
      callback: function(request, response){
        Sunstone.resetFormPanel(TAB_ID, CREATE_DIALOG_ID);
        Sunstone.hideFormPanel(TAB_ID);
        Sunstone.runAction("Support.refresh");
      },
      error: function(request, error_json){
        if (error_json.error.http_status=="403") {
          Sunstone.hideFormPanelLoading(TAB_ID);
          Notifier.notifyError(error_json.error.message);
        } else {
          Sunstone.hideFormPanel(TAB_ID);
          SupportUtils.showSupportConnect();
        }
      }
    },
    "Support.create_dialog" : {
      type: "custom",
      call: function(){
        Sunstone.showFormPanel(TAB_ID, CREATE_DIALOG_ID, "create");
      }
    },
    "Support.update" : {
      type: "single",
      call: OpenNebulaSupport.update,
      callback: function(request, response){
        Sunstone.runAction("Support.refresh");
        Notifier.notifyMessage("Comment added correctly");
      },
      error: function(request, response){
        Sunstone.runAction("Support.refresh");
        //Notifier.onError(request, response);
        Notifier.notifyError("Comment failed to be added");
      }
    },
    "Support.signout" : {
      type: "single",
      call: function() {
        $.ajax({
          url: 'support/credentials',
          type: "DELETE",
          dataType: "text",
          success: function(){
            SupportUtils.showSupportConnect();
            Sunstone.runAction("Support.refresh");
          },
          error: function(response){
            if (response.status=="401") {
              Notifier.notifyError("Support credentials are incorrect");
            } else {
              Notifier.notifyError(response.responseText);
            }
          }
        });
      }
    },
    "Support.upload" : {
      type: "single",
      call: function() {
        var selected_nodes = Sunstone.getDataTable(TAB_ID).elements();
        var resource_id = "" + selected_nodes[0];

        Sunstone.getDialog(UPLOAD_DIALOG_ID).setParams({requestId: resource_id});
        Sunstone.getDialog(UPLOAD_DIALOG_ID).reset();
        Sunstone.getDialog(UPLOAD_DIALOG_ID).show();
      }
    }
  };

  return _actions;
});
define('tabs/support-tab/datatable',['require','utils/tab-datatable','sunstone-config','utils/locale','./tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var TabDataTable = require('utils/tab-datatable');
  var SunstoneConfig = require('sunstone-config');
  var Locale = require('utils/locale');

  /*
    CONSTANTS
   */

  var RESOURCE = "Support";
  var XML_ROOT = "REQUEST";
  var TAB_NAME = require('./tabId');

  /*
    CONSTRUCTOR
   */

  function Table(dataTableId, conf) {
    this.conf = conf || {};
    this.tabId = TAB_NAME;
    this.dataTableId = dataTableId;
    this.resource = RESOURCE;
    this.xmlRoot = XML_ROOT;

    this.dataTableOptions = {
      "bAutoWidth": false,
      "bSortClasses" : false,
      "bDeferRender": true,
      "aoColumnDefs": [
          {"bSortable": false, "aTargets": ["check"]},
          {"sWidth": "35px", "aTargets": [0]},
          {"bVisible": true, "aTargets": SunstoneConfig.tabTableColumns(TAB_NAME)},
          {"bVisible": false, "aTargets": ['_all']}
      ]
    };

    this.columns = [
      "ID",
      "Subject",
      "Requested",
      "Status"
    ];


    TabDataTable.call(this);
  }

  Table.prototype = Object.create(TabDataTable.prototype);
  Table.prototype.constructor = Table;
  Table.prototype.elementArray = _elementArray;

  return Table;

  /*
    FUNCTION DEFINITIONS
   */

  function _elementArray(element) {
    return [
        '<input class="check_item" type="checkbox" id="'+RESOURCE.toLowerCase()+'_' +
                             element.id + '" name="selected_items" value="' +
                             element.id + '"/>',
        element.id,
        element.subject,
        element.created_at,
        element.status
    ];
  }
});


/* START_TEMPLATE */
define('hbs!tabs/support-tab/title',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  return "<span class=\"support_title\"><i class=\"fa fa-lg fa-fw fa-support\"></i> Support</span>\n<br>\n<div class=\"support_info\" style=\"display: none;\">\n  <span class=\"support_open\">Open</span><span class=\"label secondary right support_open_value\">-</span>\n  <br>\n  <span class=\"support_pending\">Pending</span><span class=\"label right support_pending_value\">-</span>\n  <br>\n  <button class=\"button tiny success radius support_button\">Submit a Request</button>\n</div>\n<div class=\"support_connect\" style=\"display: none;\">\n  <span class=\"\">Not connected</span>\n  <br>\n  <button class=\"button tiny success radius support_connect_button\">Sign in</button>\n</div>";
  },"useData":true});
Handlebars.registerPartial('tabs/support-tab/title', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/support-tab/subheader',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  return "<div class=\"row text-left support_connect\">\n  <div class=\"medium-6 columns\" style=\"font-\">\n    <p>The Support Subscription provides expert integration and production support on supported platforms and include:</p>\n    <ul class=\"fa-ul\" style=\"font-size: 14px;\">\n      <li><i class=\"fa-li fa fa-check\"></i>Problem diagnosis, resolution and bug fixing</li>\n      <li><i class=\"fa-li fa fa-check\"></i>Solving unexpected problems when using, installing or configuring the software</li>\n      <li><i class=\"fa-li fa fa-check\"></i>Guidance about tuning for optimal and scalable performance in your environment</li>\n      <li><i class=\"fa-li fa fa-check\"></i>Answering “how to” questions related to standard and intended product usage</li>\n      <li><i class=\"fa-li fa fa-check\"></i>Offering hints about how to go around missing features</li>\n      <li><i class=\"fa-li fa fa-check\"></i>Answering questions about product adaptation and integration</li>\n    </ul>\n    <p>For more info on support subcriptions, <a href=\"http://opennebula.systems/support/\" target=\"_blank\">click here</a></p>\n  </div>\n  <div class=\"medium-6 columns\" style=\"padding: 0px 50px;\">\n    <fieldset>\n      <legend>Commercial Support</legend>\n      <form id=\"support_credentials_form\">\n        <div class=\"large-12 columns\">\n          <label for=\"support_email\">Email</label>\n          <input id=\"support_email\" type=\"text\"></input>\n        </div>\n        <div class=\"large-12 columns\">\n          <label for=\"support_password\">Password</label>\n          <input id=\"support_password\" type=\"password\"></input>\n        </div>\n        <div class=\"large-12 columns\">\n          <button class=\"button right radius success submit_support_credentials_button\" type=\"submit\">Sign in</button>\n        </div>\n        <div class=\"large-12 columns text-center\">\n          <p>or</p>\n        </div>\n        <div class=\"large-12 columns\">\n          <a  href=\"http://opennebula.systems/buy/\" target=\"_blank\" class=\"button large-12 radius\" style=\"color: #fff !important\">Get an account</a>\n        </div>\n      </form>\n    </fieldset>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"large-12 columns text-left\">\n    <h3 class=\"subheader\"><i class=\"fa fa-fw fa-info-circle\"></i> Additional Help Resources</h3>\n  </div>\n</div>\n<br>\n<div class=\"row\">\n  <div class=\"medium-6 columns\">\n    <a href=\"http://docs.opennebula.org/4.12/\" target=\"_blank\">\n      <span class=\"fa-stack fa-2x\" style=\"color: #cfcfcf;\">\n       <i class=\"fa fa-circle fa-stack-2x\"></i>\n       <i class=\"fa fa-book fa-stack-1x fa-inverse\"></i>\n     </span>\n     <br>\n     Documentation\n   </a>\n </div>\n <div class=\"medium-6 columns\">\n  <a href=\"http://opennebula.org/support/community/\" target=\"_blank\">\n    <span class=\"fa-stack fa-2x\" style=\"color: #cfcfcf;\">\n     <i class=\"fa fa-circle fa-stack-2x\"></i>\n     <i class=\"fa fa-comments fa-stack-1x fa-inverse\"></i>\n   </span>\n   <br>\n   Community\n </a>\n</div>\n</div>\n<br>\n<p class=\"text-center\" style=\"font-size: 14px; color: #999;\">This tab can be disabled in the sunstone views configuration files</p>";
  },"useData":true});
Handlebars.registerPartial('tabs/support-tab/subheader', t);
return t;
});
/* END_TEMPLATE */
;

/* START_TEMPLATE */
define('hbs!tabs/support-tab/dialogs/upload/html',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal\" role=\"dialog\" data-reveal >\n  <div class=\"row\">\n    <h3 class=\"subheader\">Upload File</h3>\n  </div>\n  <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\">\n    <div class=\"row\">\n      <div id=\"support_file-uploader\" class=\"large-12 columns text-center\">\n        <label id=\"support_file-uploader-label\" for=\"support_file-uploader-input\"></label>\n        <input id=\"support_file-uploader-input\" type=\"file\"/>\n      </div>\n    </div>\n    <div class=\"form_buttons row\">\n      <button class=\"button right radius success upload_support_file_form_button\" type=\"submit\" disabled>\n        Upload\n      </button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('tabs/support-tab/dialogs/upload/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/support-tab/dialogs/upload',['require','utils/dialogs/dialog','hbs!./upload/html','resumable','sunstone','utils/notifier','opennebula/error','utils/browser-info','./upload/dialogId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./upload/html');
  var Resumable = require('resumable');
  var Sunstone = require('sunstone');
  var Notifier = require('utils/notifier');
  var OpenNebulaError = require('opennebula/error');
  var BrowserInfo = require('utils/browser-info');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./upload/dialogId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;

    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setParams = _setParams;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({
      'dialogId': this.dialogId
    });
  }

  /**
   * @param {object} params
   *        - params.requestId : Support Request id
   */
  function _setParams(params) {
    this.requestId = params.requestId;
  }

  function _setup(context) {
    var that = this;

    var tabContext = $("#"+TAB_ID);

    if (BrowserInfo.getInternetExplorerVersion() > -1) {
      $(".upload_support_file_form_button", context).text("Uploading files through IE is not supported");
      $(".upload_support_file_form_button", context).attr("disabled", "disabled");
    } else {
      var uploader = new Resumable({
        target: 'upload_chunk',
        chunkSize: 10*1024*1024,
        maxFiles: 1,
        testChunks: false,
        query: {
          csrftoken: csrftoken
        }
      });

      uploader.assignBrowse($('#support_file-uploader-input', context));

      var fileName = '';
      var file_input = false;

      uploader.on('fileAdded', function(file){
        $(".upload_support_file_form_button", context).removeAttr("disabled");
        fileName = file.fileName;
        file_input = fileName;

        $('#support_file-uploader-input', context).hide();
        $("#support_file-uploader-label", context).html(file.fileName);
      });

      uploader.on('uploadStart', function() {
        $(".upload_support_file_form_button", context).attr("disabled", "disabled");
        $('.support_upload_progress_bars', tabContext).append(
          '<div id="'+fileName+'progressBar" class="row" style="margin-bottom:10px">\
            <div id="'+fileName+'-info" class="large-2 columns dataTables_info">\
              Uploading...\
            </div>\
            <div class="large-10 columns">\
              <div id="upload_progress_container" class="progress nine radius" style="height:25px !important">\
                <span class="meter" style="width:0%"></span>\
              </div>\
              <div class="progress-text" style="margin-left:15px">'+fileName+'</div>\
            </div>\
          </div>');
      });

      uploader.on('progress', function() {
        $('span.meter', $('div[id="'+fileName+'progressBar"]', tabContext)).css('width', this.progress()*100.0+'%')
      });

      uploader.on('fileSuccess', function(file) {
        $('div[id="'+fileName+'-info"]', tabContext).text('Registering in OpenNebula');
        $.ajax({
          url: 'support/request/' + that.requestId + '/upload',
          type: "POST",
          data: {
            csrftoken: csrftoken,
            file: fileName,
            tempfile: file.uniqueIdentifier
          },
          success: function(){
            Notifier.notifyMessage("File uploaded correctly");
            $('div[id="'+fileName+'progressBar"]', tabContext).remove();
            Sunstone.runAction("Support.refresh");

            Sunstone.getDialog(DIALOG_ID).hide();
          },
          error: function(response){
            Notifier.onError({}, OpenNebulaError(response));
            $('div[id="'+fileName+'progressBar"]', tabContext).remove();
          }
        });
      });

      $('#' + DIALOG_ID + 'Form', context).on("submit", function(){
        uploader.upload();
        Sunstone.getDialog(DIALOG_ID).hide();
        return false;
      });
    }
  }

  function _onShow(context) {
    return false;
  }
});

define('templates/helpers/htmlDecode',['require','hbs/handlebars','utils/template-utils'],function(require) {
  /**
   * Decodes an escaped html string back to html. For example,
   * "&lt;p&gt;This is a test&lt;/p&gt;" -->
   * "<p>This is a test</p>"
   */

  var Handlebars = require('hbs/handlebars');
  var TemplateUtils = require('utils/template-utils');

  var htmlDecode = function(value, options) {
    return TemplateUtils.htmlDecode(value);
  };

  Handlebars.registerHelper('htmlDecode', htmlDecode);

  return htmlDecode;
});

/* START_TEMPLATE */
define('hbs!tabs/support-tab/panels/info/html',['hbs','hbs/handlebars','templates/helpers/htmlDecode','templates/helpers/advancedSection'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"1":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, buffer = "";
  stack1 = ((helpers.advancedSection || (depth0 && depth0.advancedSection) || helperMissing).call(depth0, (depth0 != null ? depth0.title : depth0), {"name":"advancedSection","hash":{},"fn":this.program(2, data),"inverse":this.noop,"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer;
},"2":function(depth0,helpers,partials,data) {
  var stack1, helperMissing=helpers.helperMissing, buffer = "  <div class=\"row\">\n    <div class=\"large-12 columns comment\" style=\"font-size: 14px !important;\">\n      ";
  stack1 = ((helpers.htmlDecode || (depth0 && depth0.htmlDecode) || helperMissing).call(depth0, (depth0 != null ? depth0.html_body : depth0), {"name":"htmlDecode","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  return buffer + "\n    </div>\n  </div>\n";
},"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var stack1, helper, lambda=this.lambda, escapeExpression=this.escapeExpression, helperMissing=helpers.helperMissing, functionType="function", buffer = "<div class=\"row\">\n  <div class=\"medium-6 columns\">\n    <h5>"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.subject : stack1), depth0))
    + "</h5>\n    <p class=\"subheader\" style=\"font-size: 14px;\">\n      ";
  stack1 = ((helpers.htmlDecode || (depth0 && depth0.htmlDecode) || helperMissing).call(depth0, ((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.html_description : stack1), {"name":"htmlDecode","hash":{},"data":data}));
  if (stack1 != null) { buffer += stack1; }
  buffer += "\n    </p>\n  </div>\n  <div class=\"medium-6 columns\">\n    <table id=\"info_marketplace_table\" class=\"dataTable\">\n      <thead>\n        <tr>\n          <th colspan=\"2\">Information</th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr>\n          <td class=\"key_td\">Requested</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.created_at : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td class=\"key_td\">Status</td>\n          <td class=\"value_td\">"
    + escapeExpression(lambda(((stack1 = (depth0 != null ? depth0.element : depth0)) != null ? stack1.status : stack1), depth0))
    + "</td>\n        </tr>\n        <tr>\n          <td colspan=\"2\"><span class=\"large-12 label secondary radius\">"
    + escapeExpression(((helper = (helper = helpers.message || (depth0 != null ? depth0.message : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"message","hash":{},"data":data}) : helper)))
    + "</span></td>\n        </tr>\n      </tbody>\n    </table>\n  </div>\n</div>\n<br>\n";
  stack1 = helpers.each.call(depth0, (depth0 != null ? depth0.comments : depth0), {"name":"each","hash":{},"fn":this.program(1, data),"inverse":this.noop,"data":data});
  if (stack1 != null) { buffer += stack1; }
  return buffer + "<div class=\"row\">\n  <div class=\"large-12 columns support_upload_progress_bars\">\n  </div>\n</div>\n<form id=\"submit_support_comment\">\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <textarea class=\"comment\" placeholder=\"Add a comment to this request\" rows=\"4\"/>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <input id=\"solved\" type=\"checkbox\">\n      <label for=\"solved\">Please consider this request resolved</label>\n      <button class=\"button right radius success\" type=\"submit\">Submit</button>\n    </div>\n  </div>\n</form>\n";
},"useData":true});
Handlebars.registerPartial('tabs/support-tab/panels/info/html', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/support-tab/panels/info/panelId',['require'],function(require){
  return 'support_info_tab';
});
define('tabs/support-tab/panels/info',['require','utils/locale','sunstone','hbs!./info/html','../tabId','./info/panelId'],function(require) {
  /*
    DEPENDENCIES
   */

  var Locale = require('utils/locale');
  var Sunstone = require('sunstone');

  var TemplateInfo = require('hbs!./info/html');

  /*
    CONSTANTS
   */

  var TAB_ID = require('../tabId');
  var PANEL_ID = require('./info/panelId');
  var RESOURCE = "Support";
  var XML_ROOT = "REQUEST";

  /*
    CONSTRUCTOR
   */

  function Panel(info) {
    this.title = "Info";
    this.icon = "fa-info-circle";

    this.element = info[XML_ROOT];

    return this;
  }

  Panel.PANEL_ID = PANEL_ID;
  Panel.prototype.html = _html;
  Panel.prototype.setup = _setup;

  return Panel;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    var message;
    if (this.element["status"] == "open") {
      message = "This request is currently being processed by our staff";
    } else if (this.element["status"] == "pending") {
      message = "This request is awaiting your response";
    }

    var comments = [];
    if (this.element["comments"]) {
      $.each(this.element["comments"], function(index, comment){
        var author = (comment["author_id"] == 21231023 ? "OpenNebula Support Team" : 'Me');

        comments.push({
          title: '<span style="width: 100%;">'+author+' <span style="color: #999;"> - '+comment["created_at"]+'</span></span>',
          html_body: comment["html_body"]
        });
      });
    }

    return TemplateInfo({
      'element': this.element,
      'message': message,
      'comments': comments
    });
  }

  function _setup(context) {
    var that = this;

    $("#submit_support_comment").on("submit", function(){
      $("button[type=submit]", context).attr("disabled", "disabled");
      $("button[type=submit]", context).html('<i class="fa fa-spinner fa-spin"></i>');

      var request_id = that.element.id;
      var request_json = {
        "comment" : {
          "value" : $(".comment", this).val()
        },
        "solved" : $("#solved:checked", this).length > 0 ? true : false
      };

      Sunstone.runAction("Support.update", request_id, request_json);
      return false;
    });

    $(".accordion_advanced_toggle", context).trigger("click");
    $("dl.right-info-tabs", context).hide();

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!tabs/support-tab/form-panels/create/wizard',['hbs','hbs/handlebars'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<form data-abide=\"ajax\" id=\""
    + escapeExpression(((helper = (helper = helpers.formPanelId || (depth0 != null ? depth0.formPanelId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"formPanelId","hash":{},"data":data}) : helper)))
    + "Wizard\" class=\"custom creation\">\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <label for=\"subject\">Subject</label>\n      <input id=\"subject\" type=\"text\" required></input>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"medium-6 columns\">\n      <label for=\"opennebula_version\">OpenNebula Version</label>\n      <input id=\"opennebula_version\" type=\"text\" required></input>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <label for=\"description\">Description</label>\n      <textarea id=\"description\" rows=\"5\" placeholder=\"Please enter the details of your request. A member of our support staff will respond as soon as possible.\" required></textarea>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"large-12 columns\">\n      <label for=\"severity\">Subject</label>\n      <select id=\"severity\" name=\"severity\">\n        <option value=\"severity_1\">Severity 1. Product Error: Catastrophic problem in running production systems</option>\n        <option value=\"severity_2\">Severity 2. Product Error: High-impact problem in running production systems</option>\n        <option value=\"severity_3\">Severity 3. Product Error: Low impact problem on a running production system </option>\n        <option value=\"severity_4\" selected>Severity 4. Usage, Design, Configuration, or Integration Question</option>\n      </select>\n    </div>\n  </div>\n</form>";
},"useData":true});
Handlebars.registerPartial('tabs/support-tab/form-panels/create/wizard', t);
return t;
});
/* END_TEMPLATE */
;
define('tabs/support-tab/form-panels/create',['require','foundation.tab','utils/form-panels/form-panel','sunstone','hbs!./create/wizard','./create/formPanelId','../tabId'],function(require) {
  /*
    DEPENDENCIES
   */

  require('foundation.tab');
  var BaseFormPanel = require('utils/form-panels/form-panel');
  var Sunstone = require('sunstone');

  /*
    TEMPLATES
   */

  var TemplateWizardHTML = require('hbs!./create/wizard');

  /*
    CONSTANTS
   */

  var FORM_PANEL_ID = require('./create/formPanelId');
  var TAB_ID = require('../tabId');

  /*
    CONSTRUCTOR
   */

  function FormPanel() {
    this.formPanelId = FORM_PANEL_ID;
    this.tabId = TAB_ID;
    this.actions = {
      'create': {
        'title': "Submit a Request",
        'buttonText': "Submit",
        'resetButton': true
      }
    };

    BaseFormPanel.call(this);
  }

  FormPanel.FORM_PANEL_ID = FORM_PANEL_ID;
  FormPanel.prototype = Object.create(BaseFormPanel.prototype);
  FormPanel.prototype.constructor = FormPanel;
  FormPanel.prototype.htmlWizard = _htmlWizard;
  FormPanel.prototype.submitWizard = _submitWizard;
  FormPanel.prototype.onShow = _onShow;
  FormPanel.prototype.setup = _setup;

  return FormPanel;

  /*
    FUNCTION DEFINITIONS
   */

  function _htmlWizard() {
    return TemplateWizardHTML({
      'formPanelId': this.formPanelId
    });
  }

  function _onShow(context) {
  }

  function _setup(context) {
    return false;
  }

  function _submitWizard(context) {
    var template = {
      "subject" : $('#subject', context).val(),
      "description" : $('#description', context).val(),
      "opennebula_version" : $('#opennebula_version', context).val(),
      "severity" : $('#severity', context).val(),
    };

    Sunstone.runAction("Support.create", template);
    return false;
  }
});

define('tabs/support-tab',['require','./support-tab/buttons','./support-tab/actions','./support-tab/datatable','utils/notifier','./support-tab/utils/common','sunstone','hbs!./support-tab/title','hbs!./support-tab/subheader','./support-tab/tabId','./support-tab/dialogs/upload','./support-tab/panels/info','./support-tab/form-panels/create'],function(require) {
  var Buttons = require('./support-tab/buttons');
  var Actions = require('./support-tab/actions');
  var Table = require('./support-tab/datatable');
  var Notifier = require('utils/notifier');
  var SupportUtils = require('./support-tab/utils/common');
  var Sunstone = require('sunstone');

  var TemplateTitle = require('hbs!./support-tab/title');
  var TemplateSubheader = require('hbs!./support-tab/subheader');

  var TAB_ID = require('./support-tab/tabId');
  var DATATABLE_ID = "dataTableSupport";

  var _dialogs = [
    require('./support-tab/dialogs/upload')
  ];

  var _panels = [
    require('./support-tab/panels/info')
  ];

  var _formPanels = [
    require('./support-tab/form-panels/create')
  ];

  var Tab = {
    tabId: TAB_ID,
    resource: 'Support',
    title: TemplateTitle(),
    listHeader: '<i class="fa fa-fw fa-support"></i> Commercial Support Requests',
    infoHeader: '<i class="fa fa-fw fa-support"></i> Commercial Support Request',
    subheader: TemplateSubheader(),
    buttons: Buttons,
    actions: Actions,
    dataTable: new Table(DATATABLE_ID, {actions: true, info: true, oneSelection: true}),
    panels: _panels,
    formPanels: _formPanels,
    dialogs: _dialogs,
    setup: _setup
  };

  return Tab;

  function _setup(context) {

    SupportUtils.showSupportConnect();
    SupportUtils.startIntervalRefresh();

    $(".support_button").on("click", function(){
      $("#li_support-tab > a").trigger("click");
      $(".create_dialog_button", "#support-tab").trigger("click");
      return false;
    });

    $("#support_credentials_form", context).on("submit", function(){
      $(".submit_support_credentials_button", context).attr("disabled", "disabled");
      $(".submit_support_credentials_button", context).html('<i class="fa fa-spinner fa-spin"></i>');

      var data = {
        email : $("#support_email", this).val(),
        password : $("#support_password", this).val()
      };

      $.ajax({
        url: 'support/credentials',
        type: "POST",
        data: JSON.stringify(data),
        success: function(){
          $(".submit_support_credentials_button", context).removeAttr("disabled");
          $(".submit_support_credentials_button", context).html('Sign in');

          Sunstone.runAction("Support.refresh");

          SupportUtils.showSupportList();
          SupportUtils.startIntervalRefresh();
        },
        error: function(response){
          if (response.status=="401") {
            Notifier.notifyError("Support credentials are incorrect");
          } else {
            Notifier.notifyError(response.responseText);
          }

          $(".submit_support_credentials_button", context).removeAttr("disabled");
          $(".submit_support_credentials_button", context).html('Sign in');
        }
      });

      return false;
    });
  }

});


/* START_TEMPLATE */
define('hbs!utils/dialogs/confirm/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal\" data-reveal>\n  <div class=\"row\">\n    <h3 class=\"subheader\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Confirm", {"name":"tr","hash":{},"data":data})))
    + "\n      <br>\n      &emsp;\n      <small class=\"confirm_action\"></small>\n    </h3>\n  </div>\n  <form action=\"\">\n    <div id=\"confirm_tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "You have to confirm this action.", {"name":"tr","hash":{},"data":data})))
    + "</div>\n    <br />\n    <div id=\"question\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Do you want to proceed?", {"name":"tr","hash":{},"data":data})))
    + "</div>\n    <br />\n    <div class=\"form_buttons\">\n      <button id=\"confirm_proceed\" class=\"action_button radius button right\" value=\"\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "OK", {"name":"tr","hash":{},"data":data})))
    + "</button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('utils/dialogs/confirm/html', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/dialogs/confirm',['require','utils/dialogs/dialog','hbs!./confirm/html','sunstone','utils/locale','./confirm/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./confirm/html');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./confirm/dialogId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;
    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({dialogId: this.dialogId});
  }

  function _setup(dialog) {
    // Submit action is configured in sunstone.js since it's an action_button
    return false;
  }

  function _onShow(dialog) {
    var actionId = dialog.data('buttonAction');
    var tabId = dialog.data('buttonTab');
    var button = Sunstone.getButton(tabId, actionId);

    var tip = Locale.tr("You have to confirm this action");
    if (button && button.tip) {
      tip = button.tip
    }

    $('#confirm_proceed', dialog).val(actionId);
    $('#confirm_tip', dialog).text(tip);

    var action = Sunstone.getAction(actionId);
    var elements = action.elements();
    if (elements) {
      var str = actionId.split('.');
      $(".confirm_action", dialog).html(str[1] + ' ' + str[0] + ': ' + elements.join(', '))
    }

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!utils/dialogs/confirm-with-select/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal\" data-reveal>\n  <div class=\"row\">\n    <h3 class=\"subheader\">\n      "
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Confirm", {"name":"tr","hash":{},"data":data})))
    + "\n      <br>\n      &emsp;\n      <small class=\"confirm_action\"></small>\n    </h3>\n  </div>\n  <form action=\"\">\n    <div class=\"row\">\n      <div id=\"confirm_with_select_tip\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "You need to select something.", {"name":"tr","hash":{},"data":data})))
    + "</div>\n    </div>\n    <br />\n    <div class=\"row\">\n      <div class=\"large-12 columns\" id=\"confirm_select\"></div>\n    </div>\n    <br />\n    <br />\n    <div class=\"form_buttons\">\n      <button id=\"confirm_with_select_proceed\" class=\"action_button radius button right\" value=\"\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "OK", {"name":"tr","hash":{},"data":data})))
    + "</button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('utils/dialogs/confirm-with-select/html', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/dialogs/confirm-with-select',['require','utils/dialogs/dialog','hbs!./confirm-with-select/html','sunstone','utils/locale','utils/notifier','utils/resource-select','./confirm-with-select/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./confirm-with-select/html');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');
  var Notifier = require('utils/notifier');
  var ResourceSelect = require('utils/resource-select');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./confirm-with-select/dialogId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;
    BaseDialog.call(this);
  };

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */


  function _html() {
    return TemplateHTML({dialogId: this.dialogId});
  }

  function _setup(dialog) {
    //when we proceed with a "confirm with select" we need to
    //find out if we are running an action with a parametre on a datatable
    //items or if its just an action
    $('#confirm_with_select_proceed', dialog).click(function() {
      var actionId = dialog.data('buttonAction');
      var action = Sunstone.getAction(actionId);
      var param = $('.resource_list_select', dialog).val();

      if (!param.length) {
        Notifier.notifyError("You must select a value");
        return false;
      };

      if (!action) {
        Notifier.notifyError("Action " + action + " not defined.");
        return false;
      };

      var error;
      switch (action.type){
      case "multiple":
        error = Sunstone.runAction(actionId, action.elements(), param);
        break;
      default:
        error = Sunstone.runAction(actionId, param);
        break;
      }

      if (!error) {
        dialog.foundation('reveal', 'close');
      }

      return false;
    });

    return false;
  }

  function _onShow(dialog) {
    var actionId = dialog.data('buttonAction');
    var tabId = dialog.data('buttonTab');
    var button = Sunstone.getButton(tabId, actionId);

    var tip = Locale.tr("You have to confirm this action");
    if (button.tip) {
      tip = button.tip
    }

    if (button.custom_select) {
      $('div#confirm_select', dialog).html(button.custom_select);
    } else {
      ResourceSelect.insert('#confirm_select', dialog, button.select, null, true);
    }

    $('#confirm_with_select_tip', dialog).text(tip);

    var action = Sunstone.getAction(actionId);
    var elements = action.elements();
    if (elements) {
      var str = actionId.split('.');
      $(".confirm_action", dialog).html(str[1] + ' ' + str[0] + ': ' + elements.join(', '))
    }

    return false;
  }
});


/* START_TEMPLATE */
define('hbs!utils/dialogs/generic-confirm/html',['hbs','hbs/handlebars','templates/helpers/tr'], function( hbs, Handlebars ){ 
var t = Handlebars.template({"compiler":[6,">= 2.0.0-beta.1"],"main":function(depth0,helpers,partials,data) {
  var helper, functionType="function", helperMissing=helpers.helperMissing, escapeExpression=this.escapeExpression;
  return "<div id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "\" class=\"reveal-modal\" data-reveal>\n  <div class=\"row\">\n    <h3 class=\"subheader\">\n      <span id=\"header\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Confirm", {"name":"tr","hash":{},"data":data})))
    + "</span>\n      <br>\n    </h3>\n  </div>\n  <form id=\""
    + escapeExpression(((helper = (helper = helpers.dialogId || (depth0 != null ? depth0.dialogId : depth0)) != null ? helper : helperMissing),(typeof helper === functionType ? helper.call(depth0, {"name":"dialogId","hash":{},"data":data}) : helper)))
    + "Form\">\n    <div id=\"body\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "You have to confirm this action.", {"name":"tr","hash":{},"data":data})))
    + "</div>\n    <br/>\n    <div id=\"question\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "Do you want to proceed?", {"name":"tr","hash":{},"data":data})))
    + "</div>\n    <br/>\n    <div class=\"form_buttons\">\n      <button id=\"generic_confirm_proceed\" class=\"radius button right\" value=\"\">"
    + escapeExpression(((helpers.tr || (depth0 && depth0.tr) || helperMissing).call(depth0, "OK", {"name":"tr","hash":{},"data":data})))
    + "</button>\n    </div>\n    <a class=\"close-reveal-modal\">&#215;</a>\n  </form>\n</div>";
},"useData":true});
Handlebars.registerPartial('utils/dialogs/generic-confirm/html', t);
return t;
});
/* END_TEMPLATE */
;
define('utils/dialogs/generic-confirm',['require','utils/dialogs/dialog','hbs!./generic-confirm/html','sunstone','utils/locale','./generic-confirm/dialogId'],function(require) {
  /*
    DEPENDENCIES
   */

  var BaseDialog = require('utils/dialogs/dialog');
  var TemplateHTML = require('hbs!./generic-confirm/html');
  var Sunstone = require('sunstone');
  var Locale = require('utils/locale');

  /*
    CONSTANTS
   */

  var DIALOG_ID = require('./generic-confirm/dialogId');

  /*
    CONSTRUCTOR
   */

  function Dialog() {
    this.dialogId = DIALOG_ID;
    BaseDialog.call(this);
  }

  Dialog.DIALOG_ID = DIALOG_ID;
  Dialog.prototype = Object.create(BaseDialog.prototype);
  Dialog.prototype.constructor = Dialog;
  Dialog.prototype.html = _html;
  Dialog.prototype.onShow = _onShow;
  Dialog.prototype.setup = _setup;
  Dialog.prototype.setParams = _setParams;

  return Dialog;

  /*
    FUNCTION DEFINITIONS
   */

  function _html() {
    return TemplateHTML({dialogId: this.dialogId});
  }

  /**
   * @param {object} params.
   *        - params.header : Optional, html string
   *        - params.body : Optional, html string
   *        - params.question : Optional, html string
   *        - params.submit : Mandatory, function to call if user confirms
   */
  function _setParams(params) {
    this.params = params;
  }

  function _setup(context) {
    var that = this;

    $('#' + DIALOG_ID + 'Form', context).submit(function() {
      Sunstone.getDialog(DIALOG_ID).hide();

      if (that.params.submit){
        that.params.submit(this);
      }

      return false;
    });

    return false;
  }

  function _onShow(context) {
    if (this.params.header){
      $("#header", context).html(this.params.header);
    }

    if (this.params.body){
      $("#body", context).html(this.params.body);
    }

    if (this.params.question){
      $("#question", context).html(this.params.question);
    }

    return false;
  }
});

define('app',['require','jquery','foundation.dropdown','tabs/dashboard-tab/tabId','tabs/settings-tab/tabId','tabs/provision-tab/tabId','sunstone','sunstone-config','opennebula','utils/notifier','utils/menu','tabs/provision-tab','tabs/users-tab','tabs/settings-tab','tabs/dashboard-tab','tabs/system-tab','tabs/users-tab','tabs/groups-tab','tabs/vdcs-tab','tabs/acls-tab','tabs/vresources-tab','tabs/vms-tab','tabs/templates-tab','tabs/images-tab','tabs/files-tab','tabs/infra-tab','tabs/clusters-tab','tabs/hosts-tab','tabs/datastores-tab','tabs/vnets-tab','tabs/secgroups-tab','tabs/zones-tab','tabs/marketplace-tab','tabs/oneflow-dashboard','tabs/oneflow-services-tab','tabs/oneflow-templates-tab','tabs/settings-tab','tabs/support-tab','utils/dialogs/confirm','utils/dialogs/confirm-with-select','utils/dialogs/generic-confirm'],function(require) {
  require('jquery');
  require('foundation.dropdown');

  var DASHBOARD_TAB_ID = require('tabs/dashboard-tab/tabId');
  var SETTINGS_TAB_ID = require('tabs/settings-tab/tabId');
  var PROVISION_TAB_ID = require('tabs/provision-tab/tabId');
  var Sunstone = require('sunstone');
  var Config = require('sunstone-config');
  var OpenNebula = require('opennebula');
  var Notifier = require('utils/notifier');
  var Menu = require('utils/menu');

  var _tabs;
  if (Config.isTabEnabled(PROVISION_TAB_ID)) {
    _tabs = [
      require('tabs/provision-tab'),
      require('tabs/users-tab'),
      require('tabs/settings-tab')
    ];
  } else {
    _tabs = [
      require('tabs/dashboard-tab'),
      require('tabs/system-tab'),
      require('tabs/users-tab'),
      require('tabs/groups-tab'),
      require('tabs/vdcs-tab'),
      require('tabs/acls-tab'),
      require('tabs/vresources-tab'),
      require('tabs/vms-tab'),
      require('tabs/templates-tab'),
      require('tabs/images-tab'),
      require('tabs/files-tab'),
      require('tabs/infra-tab'),
      require('tabs/clusters-tab'),
      require('tabs/hosts-tab'),
      require('tabs/datastores-tab'),
      require('tabs/vnets-tab'),
      require('tabs/secgroups-tab'),
      require('tabs/zones-tab'),
      require('tabs/marketplace-tab'),
      require('tabs/oneflow-dashboard'),
      require('tabs/oneflow-services-tab'),
      require('tabs/oneflow-templates-tab'),
      require('tabs/settings-tab'),
      require('tabs/support-tab')
    ];
  }

  var _commonDialogs = [
    require('utils/dialogs/confirm'),
    require('utils/dialogs/confirm-with-select'),
    require('utils/dialogs/generic-confirm')
  ]

  Sunstone.addDialogs(_commonDialogs);

  $.each(_tabs, function(index, tab) {
    Sunstone.addMainTab(tab);
  });

  //$(window).load(function() {
  //   $('#loading').hide();
  //});

  $(document).ready(function() {
    Sunstone.insertTabs();

    if (Config.isTabEnabled(PROVISION_TAB_ID)){
      Menu.insertProvision();
    }else{
      Menu.insert();
    }

    _setupAccordion();
    _setupCloseDropdownsOnClick();
    _insertUserAndZoneSelector();

    if (Config.isTabEnabled(PROVISION_TAB_ID)) {
      Sunstone.showTab(PROVISION_TAB_ID);
      $('#loading').hide();
    } else if (Config.isTabEnabled(DASHBOARD_TAB_ID)) {
      Sunstone.showTab(DASHBOARD_TAB_ID);
      $('#loading').hide();
    }
  });

  function _setupCloseDropdownsOnClick() {
    $(document).on("click", '[data-dropdown-content] a', function() {
      var id = $(this).closest('ul').attr('id');
      $('[data-dropdown=' + id + ']').trigger('click');
    });
  }

  function _setupAccordion() {
    $(document).on("click", ".accordion_advanced > a", function() {
      if ($(this).hasClass("active")) {
        $(this).removeClass("active");
      } else {
        $(this).addClass("active");
      }

      $(this).closest(".accordion_advanced").children(".content").toggle();

      return false;
    })
  }

  function _insertUserAndZoneSelector() {
    var user_login_content =  '<button href="#" data-dropdown="userSelectDropdown" class="button small radius secondary dropdown" id="logout">\
      <i class="fa fa-user fa-lg fa-fw header-icon"></i> ' + config['display_name'] + '</button>\
      <ul id="userSelectDropdown" data-dropdown-content class="f-dropdown">\
        <li><a href="#" class="configuration"><i class="fa fa-cog"></i> Settings</a></li>\
        <li><a href="#" class="logout"><i class="fa fa-power-off"></i> Sign Out</a></li>\
      </ul>\
    <button href="#" data-dropdown="drop2" class="button small radius secondary dropdown" id="zonelector">\
      <i class="fa fa-home fa-lg fa-fw header-icon"></i> ' + config['zone_name'] + '</button>\
      <ul id="drop2" data-dropdown-content class="zone-ul f-dropdown"></ul>';

    $(".user-zone-info").html(user_login_content);

    function zoneRefresh() {
      // Populate Zones dropdown
      OpenNebula.Zone.list({
        timeout: true,
        success: function (request, obj_list) {
          $('.zone-ul').empty();
          $.each(obj_list, function() {
            $('.zone-ul').append('<li><a id="' + this.ZONE.NAME + '" class="zone-choice">' + this.ZONE.NAME + '</a></li>');
          });
        },
        error: Notifier.onError
      });
    }

    $('#zonelector').on("click", function() {
      zoneRefresh();
    });

    $('a.zone-choice').on("click", function() {
       $.ajax({
         url: 'config',
         type: "GET",
         headers: {
           "ZONE_NAME" : this.id
         },
         dataType: "json",
         success: function() {
           window.location.href = ".";
         },
         error: function(response) {
           Notifier.onError(null, OpenNebula.Error(response))
         }
       });
     });

    $("a.logout", $(".user-zone-info ")).click(function() {
      OpenNebula.Auth.logout({
          success: function() {
            window.location.href = "login";
          },
          error: Notifier.onError
        });

      return false;
    });

    $(".user-zone-info a.configuration").click(function() {
      $(document).foundation('dropdown', 'closeall');
      Sunstone.showTab(SETTINGS_TAB_ID);
    });
  }
});

require.config({
  paths: {
    /* Almond */
    'almond': '../bower_components/almond/almond',

    /* jQuery */
    'jquery': '../bower_components/jquery/dist/jquery',

    /* DataTables */
    'datatables': '../bower_components/datatables/media/js/jquery.dataTables',
    'foundation-datatables': '../bower_components/foundation-datatables/integration/foundation/dataTables.foundation',

    /* DataTables */
    'jgrowl': '../bower_components/jgrowl/jquery.jgrowl',

    /* Foundation */
    'foundation.core': '../bower_components/foundation/js/foundation/foundation',
    'foundation.abide': '../bower_components/foundation/js/foundation/foundation.abide',
    'foundation.accordion': '../bower_components/foundation/js/foundation/foundation.accordion',
    'foundation.alert': '../bower_components/foundation/js/foundation/foundation.alert',
    'foundation.clearing': '../bower_components/foundation/js/foundation/foundation.clearing',
    'foundation.dropdown': '../bower_components/foundation/js/foundation/foundation.dropdown',
    'foundation.equalizer': '../bower_components/foundation/js/foundation/foundation.equalizer',
    'foundation.interchange': '../bower_components/foundation/js/foundation/foundation.interchange',
    'foundation.joyride': '../bower_components/foundation/js/foundation/foundation.joyride',
    'foundation.magellan': '../bower_components/foundation/js/foundation/foundation.magellan',
    'foundation.offcanvas': '../bower_components/foundation/js/foundation/foundation.offcanvas',
    'foundation.orbit': '../bower_components/foundation/js/foundation/foundation.orbit',
    'foundation.reveal': '../bower_components/foundation/js/foundation/foundation.reveal',
    'foundation.slider': '../bower_components/foundation/js/foundation/foundation.slider',
    'foundation.tab': '../bower_components/foundation/js/foundation/foundation.tab',
    'foundation.tooltip': '../bower_components/foundation/js/foundation/foundation.tooltip',
    'foundation.topbar': '../bower_components/foundation/js/foundation/foundation.topbar',

    /* Handlebars */
    'hbs': '../bower_components/require-handlebars-plugin/hbs',

    /* Vendor Scripts */
    'jquery.cookie': '../bower_components/foundation/js/vendor/jquery.cookie',
    'fastclick': '../bower_components/foundation/js/vendor/fastclick',
    'modernizr': '../bower_components/foundation/js/vendor/modernizr',
    'placeholder': '../bower_components/foundation/js/vendor/placeholder',

    /* Resumable */
    'resumable': '../bower_components/resumablejs/resumable',

    /* Flot Graphs */
    'flot': '../bower_components/flot/jquery.flot',
    'flot.stack': '../bower_components/flot/jquery.flot.stack',
    'flot.resize': '../bower_components/flot/jquery.flot.resize',
    'flot.time': '../bower_components/flot/jquery.flot.time',
    'flot.tooltip': '../bower_components/flot.tooltip/js/jquery.flot.tooltip',

    /* VNC */
    'vnc-util': '../bower_components/no-vnc/include/util',
    'vnc-webutil': '../bower_components/no-vnc/include/webutil',
    'vnc-base64': '../bower_components/no-vnc/include/base64',
    'vnc-websock': '../bower_components/no-vnc/include/websock',
    'vnc-des': '../bower_components/no-vnc/include/des',
    'vnc-keysymdef': '../bower_components/no-vnc/include/keysymdef',
    'vnc-keyboard': '../bower_components/no-vnc/include/keyboard',
    'vnc-input': '../bower_components/no-vnc/include/input',
    'vnc-display': '../bower_components/no-vnc/include/display',
    'vnc-jsunzip': '../bower_components/no-vnc/include/jsunzip',
    'vnc-rfb': '../bower_components/no-vnc/include/rfb',
    'vnc-keysym': '../bower_components/no-vnc/include/keysym',

    /* Spice */
    'spice-main': '../bower_components/spice-html5/main',
    'spice-spicearraybuffer': '../bower_components/spice-html5/spicearraybuffer',
    'spice-enums': '../bower_components/spice-html5/enums',
    'spice-atKeynames': '../bower_components/spice-html5/atKeynames',
    'spice-utils': '../bower_components/spice-html5/utils',
    'spice-png': '../bower_components/spice-html5/png',
    'spice-lz': '../bower_components/spice-html5/lz',
    'spice-quic': '../bower_components/spice-html5/quic',
    'spice-bitmap': '../bower_components/spice-html5/bitmap',
    'spice-spicedataview': '../bower_components/spice-html5/spicedataview',
    'spice-spicetype': '../bower_components/spice-html5/spicetype',
    'spice-spicemsg': '../bower_components/spice-html5/spicemsg',
    'spice-wire': '../bower_components/spice-html5/wire',
    'spice-spiceconn': '../bower_components/spice-html5/spiceconn',
    'spice-display': '../bower_components/spice-html5/display',
    'spice-inputs': '../bower_components/spice-html5/inputs',
    'spice-webm': '../bower_components/spice-html5/webm',
    'spice-playback': '../bower_components/spice-html5/playback',
    'spice-simulatecursor': '../bower_components/spice-html5/simulatecursor',
    'spice-cursor': '../bower_components/spice-html5/cursor',
    'spice-jsbn': '../bower_components/spice-html5/thirdparty/jsbn',
    'spice-rsa': '../bower_components/spice-html5/thirdparty/rsa',
    'spice-prng4': '../bower_components/spice-html5/thirdparty/prng4',
    'spice-rng': '../bower_components/spice-html5/thirdparty/rng',
    'spice-sha1': '../bower_components/spice-html5/thirdparty/sha1',
    'spice-ticket': '../bower_components/spice-html5/ticket',
    'spice-resize': '../bower_components/spice-html5/resize',
    'spice-filexfer': '../bower_components/spice-html5/filexfer'
  },
  shim: {
    /* jQuery */
    'jquery': {
      exports: '$'
    },

    /* jGrowl */
    'jgrowl': {
      deps: ['jquery']
    },

    /* dataTables */
    'foundation-datatables': {
      deps: ['jquery', 'datatables']
    },

    /* Foundation */
    'foundation.core': {
      deps: ['jquery', 'modernizr'],
      exports: 'Foundation'
    },
    'foundation.abide': {
      deps: ['foundation.core']
    },
    'foundation.accordion': {
      deps: ['foundation.core']
    },
    'foundation.alert': {
      deps: ['foundation.core']
    },
    'foundation.clearing': {
      deps: ['foundation.core']
    },
    'foundation.dropdown': {
      deps: ['foundation.core']
    },
    'foundation.equalizer': {
      deps: ['foundation.core']
    },
    'foundation.interchange': {
      deps: ['foundation.core']
    },
    'foundation.joyride': {
      deps: ['foundation.core', 'jquery.cookie']
    },
    'foundation.magellan': {
      deps: ['foundation.core']
    },
    'foundation.offcanvas': {
      deps: ['foundation.core']
    },
    'foundation.orbit': {
      deps: ['foundation.core']
    },
    'foundation.reveal': {
      deps: ['foundation.core']
    },
    'foundation.slider': {
      deps: ['foundation.core']
    },
    'foundation.tab': {
      deps: ['foundation.core']
    },
    'foundation.tooltip': {
      deps: ['foundation.core']
    },
    'foundation.topbar': {
      deps: ['foundation.core']
    },

    /* Vendor Scripts */
    'jquery.cookie': {
      deps: ['jquery']
    },
    'fastclick': {
      exports: 'FastClick'
    },
    'modernizr': {
      exports: 'Modernizr'
    },
    'placeholder': {
      exports: 'Placeholders'
    },

    /* Flot Graphs */
    'flot': {
      deps: ['jquery']
    },
    'flot.stack': {
      deps: ['flot']
    },
    'flot.resize': {
      deps: ['flot']
    },
    'flot.time': {
      deps: ['flot']
    },
    'flot.tooltip': {
      deps: ['flot']
    },

    /* VNC */
    'vnc-util': {
      exports: 'Util'
    },
    'vnc-webutil': {
      deps: ["vnc-util"]
    },
    'vnc-base64': {
      deps: ["vnc-util"]
    },
    'vnc-websock': {
      deps: ["vnc-util"]
    },
    'vnc-des': {
      deps: ["vnc-util"]
    },
    'vnc-keysymdef': {
      deps: ["vnc-util"]
    },
    'vnc-keyboard': {
      deps: ["vnc-util"]
    },
    'vnc-input': {
      deps: ["vnc-util"]
    },
    'vnc-display': {
      deps: ["vnc-util"]
    },
    'vnc-jsunzip': {
      deps: ["vnc-util"]
    },
    'vnc-rfb': {
      deps: ["vnc-util"]
    },
    'vnc-keysym': {
      deps: ["vnc-util"]
    },

    'spice-main': {
      exports: 'SpiceMainConn',
      deps: [
        'spice-spiceconn',
        'spice-spicearraybuffer',
        'spice-enums',
        'spice-atKeynames',
        'spice-utils',
        'spice-png',
        'spice-lz',
        'spice-quic',
        'spice-bitmap',
        'spice-spicedataview',
        'spice-spicetype',
        'spice-spicemsg',
        'spice-wire',
        'spice-display',
        'spice-inputs',
        'spice-webm',
        'spice-playback',
        'spice-simulatecursor',
        'spice-cursor',
        'spice-jsbn',
        'spice-rsa',
        'spice-prng4',
        'spice-rng',
        'spice-sha1',
        'spice-ticket',
        'spice-resize',
        'spice-filexfer'
      ]
    },

    'spice-rng': {
      deps: [
        'spice-prng4'
      ]
    },

    'spice-display': {
      deps: [
        'spice-spiceconn'
      ]
    },

    'spice-inputs': {
      deps: [
        'spice-spiceconn'
      ]
    },

    'spice-playback': {
      deps: [
        'spice-spiceconn'
      ]
    },

    'spice-cursor': {
      deps: [
        'spice-spiceconn'
      ]
    }
  }
});

require(['app'], function(App) {});

define("main", function(){});


//# sourceMappingURL=main.js.map